@@ -43,6 +43,7 @@ int main(int argc, char ** argv)
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sqlext.h>
 
 #define ERR_INVALID_OPTION "Invalid option"
 #define ERR_MISSING_DRIVERNAME "Missing drivername"
@@ -58,9 +59,9 @@ int main(int argc, char ** argv)
 #define WARN_VERSION_NOT_FOUND "Can't get version info of ODBC Driver"
 #define WARN_DRIVER_NOT_EXIST  "The specified driver does not exist"
 
-#define DEFAULT_DRIVER_NAME_80 "SAP DB 8.0"
-#define DEFAULT_DRIVER_NAME_76 "SAP DB 7.6"
-#define DEFAULT_DRIVER_NAME_75 "SAP DB 7.5"
+#define DEFAULT_DRIVER_NAME_80 "MaxDB"
+#define DEFAULT_DRIVER_NAME_76 "MaxDB"
+#define DEFAULT_DRIVER_NAME_75 "MaxDB"
 #define DEFAULT_DRIVER_NAME_74 "SAP DB 7.4"
 #define DEFAULT_DRIVER_NAME_73 "SAP DB 7.3"
 #define DEFAULT_DRIVER_NAME_72 "LiveCache 7.2"
@@ -75,6 +76,8 @@ int main(int argc, char ** argv)
 #define ENTRY_SETUP                      "Setup"
 #define ENTRY_CPTIMEOUT                  "CPTimeout"
 
+#define KEY_MDAC_VERSION                 "Software\\Microsoft\\DataAccess"
+
 #define DRIVER_NAME_DEFAULT "sqlod32.dll"
 #define SETUP_NAME_DEFAULT  "sqlsp32.dll"
 
@@ -116,6 +119,7 @@ int main(int argc, char ** argv)
     int   i;
     BOOL  install=FALSE;
     BOOL  remove=FALSE;
+    BOOL  test=FALSE;
     BOOL  show=TRUE;
     char  szDriverName[MAX_PATH]="";
     char  szDriver[MAX_PATH]=DRIVER_NAME_DEFAULT;
@@ -153,6 +157,12 @@ int main(int argc, char ** argv)
                                 return(t02PrintUsage());
                                 break;
                             }
+                            case ('t'): {
+                                test = TRUE;
+                                show = install = FALSE;
+                                i++;
+                                break;
+                            }
                             case ('i'): {
                                 install = TRUE;
                                 show = FALSE;
@@ -213,6 +223,7 @@ int main(int argc, char ** argv)
                 } /* if ( opt ) */
             }
         }
+
     if (install) {
         if (szDriver[0] == 0)
             return(t02PrintError(ERR_MISSING_DRIVER));
@@ -351,6 +362,59 @@ int main(int argc, char ** argv)
         else
             printf("ODBC Driver registration for '%s' not found.\n", szDriverName);
     }
+
+    if (test) {
+      SQLHENV henv;
+      SQLHDBC hdbc = SQL_NULL_HANDLE;
+      char    connectString[512];
+      SQLRETURN retcode;
+      SQLSMALLINT length;
+      char buffer[64];
+
+      /* show MDAC-version first */
+      if ( RegOpenKeyEx(HKEY_LOCAL_MACHINE, KEY_MDAC_VERSION, 0, KEY_READ, &hkResult) == ERROR_SUCCESS ) {
+        char fullInstallVer[512];
+        char Version[512];
+        DWORD cbfIV, cbVersion;
+        DWORD cbType;
+
+        fullInstallVer[0] = Version[0] = '\0';
+        cbfIV = sizeof (fullInstallVer);
+        cbVersion = sizeof (Version);
+
+        ret = RegQueryValueEx( hkResult, "FullInstallVer", 0, &cbType, fullInstallVer, &cbfIV);
+        ret = RegQueryValueEx( hkResult, "Version", 0, &cbType, Version, &cbVersion);
+
+        if (strncmp (fullInstallVer, Version, cbfIV) == 0)
+          printf ("MDAC version: %s\n\n", Version);
+        else
+          printf ("MDAC version:\n\tFullInstallVersion:\t%s\n\tVersion:\t\t%s\n\n", fullInstallVer, Version);
+      } else
+        printf ("no MDAC found\n\n");
+
+      retcode = SQLAllocHandle (SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
+      if (retcode == SQL_SUCCESS)
+        retcode = SQLSetEnvAttr (henv, SQL_ATTR_ODBC_VERSION, (void*) SQL_OV_ODBC3, 0);
+      if (retcode == SQL_SUCCESS)
+        retcode = SQLAllocHandle( SQL_HANDLE_DBC, henv, &hdbc );
+
+      sprintf (connectString, "driver={%s}", szDriverName);
+      if (retcode == SQL_SUCCESS) {
+        printf("\nODBC Driver test for '%s'.\n", szDriverName);
+        retcode = SQLDriverConnect (hdbc, NULL, connectString, SQL_NTS, NULL, 0, NULL, SQL_DRIVER_COMPLETE_REQUIRED);
+      }
+
+      if (retcode == SQL_SUCCESS) {
+        SQLGetInfo (hdbc, SQL_DRIVER_VER, buffer, sizeof(buffer), &length);
+        printf("Driver version %s.\n", buffer);
+      } else {
+        printf("not connected.\n", szDriverName);
+      }
+
+      if (hdbc != SQL_NULL_HANDLE)
+        SQLDisconnect (hdbc);
+    }
+    printf("\r\n");
     return(0);
 }
 
@@ -502,22 +566,23 @@ int t02PrintWarning(char * szString)
 
 int t02PrintUsage()
 {
-    printf("Usage : odbcreg [Drivername] [ -i[d:s:p:] | -r | -d ].\n\n"\
+    printf("\nUsage : odbcreg [\\\"][Drivername][\\\"] [ -i[d:s:p:] | -r | -d ].\n\n"\
+           "Note:           if the driver name contains spaces, the name must be qouted\n\n"\
            "Options:\n"\
            "                -i Installs a driver\n"\
            "                -r Removes a driver\n"\
            "                -p <full path including drive letter>   \n"\
            "                -d <odbc dll>  default 'sqlod32.dll'\n"\
            "                -s <setup dll> default 'sqlsp32.dll'\n"\
+           "                -t test, whether the driver can be loaded and MDAC version\n"\
            "\nDefault drivernames:\n"\
            "                8.0 = %s\n"\
+           "                7.6 = %s\n"\
+           "                7.5 = %s\n"\
            "                7.4 = %s\n"\
            "                7.3 = %s\n"\
            "                7.2 = %s\n"\
-           "                7.1 = %s\n"\
-           "                6.2 = %s\n"\
-           "                6.1 = %s\n"\
-           , DEFAULT_DRIVER_NAME_80, DEFAULT_DRIVER_NAME_74, DEFAULT_DRIVER_NAME_73, DEFAULT_DRIVER_NAME_72, DEFAULT_DRIVER_NAME_71, DEFAULT_DRIVER_NAME_62, DEFAULT_DRIVER_NAME_61);
+           , DEFAULT_DRIVER_NAME_80, DEFAULT_DRIVER_NAME_76, DEFAULT_DRIVER_NAME_75, DEFAULT_DRIVER_NAME_74, DEFAULT_DRIVER_NAME_73, DEFAULT_DRIVER_NAME_72);
     return(-1);
 }
 #endif
