@@ -48,8 +48,6 @@
 #include "Pager/Pager_IRestart.hpp"
 #include "Pager/Pager_ISavepoint.hpp"
 #include "Logging/Log_Types.hpp"
-#include "Logging/Log_Savepoint.hpp"
-#include "KernelCommon/Kernel_IAdminInfo.hpp"
 #include "KernelCommon/Kernel_IAdminRestartShutdown.hpp"
 #include "KernelCommon/ParameterNames/KernelParam_Converter.hpp"
 #include "RunTime/RTE_Message.hpp"
@@ -204,7 +202,6 @@ Converter_Converter::Converter_Converter( SAPDBMem_IRawAllocator  &allocator )
         m_Index (m_Allocator),
         m_Active( false ),
         m_InMaintenanceMode( false ),
-        m_SavepointIsRequested( false ),
         m_DBFullWaitQueue( CONV_SUSPEND_HANDLE_DB_FULL ),
         m_FreePageNoWaitQueue( CONV_SUSPEND_FREE_PAGENO_WAIT_DURING_SVP ),
         m_ExpandConverterWaitQueue( CONV_SUSPEND_EXPAND_WAIT_DURING_SVP ),
@@ -355,7 +352,7 @@ Converter_Converter::Restart(
 void
 Converter_Converter::ReadPages(
     const tsp00_TaskId  taskId,
-    const SAPDB_UInt    pagerIndex,
+    SAPDB_UInt          pagerIndex,
     SAPDB_UInt          &totalPages,
     SAPDB_UInt          &totalIO )
 {
@@ -364,105 +361,103 @@ Converter_Converter::ReadPages(
 
     SAPDBERR_ASSERT_STATE (NULL != m_VolumeList);
 
-    // check if Pager has something to do ..
-    if ((*m_VolumeList).GetSize() <= pagerIndex){
-        return;
-    }
-
     Converter_Pager pager( taskId, m_SnapShotVersion );
 
     // loop over all converter leaf pages located in one data volume. This
     // will speed up the restart time of the converter. The leaf pages located
     // in the other data volumes are read by another pager task.
 
-    Converter_Index::BlockVector::Iterator        iter    = (*m_VolumeList)[pagerIndex].Begin();
-    const Converter_Index::BlockVector::Iterator  iterEnd = (*m_VolumeList)[pagerIndex].End();
+    while (pagerIndex < ((*m_VolumeList).GetSize())){
+        Converter_Index::BlockVector::Iterator        iter    = (*m_VolumeList)[pagerIndex].Begin();
+        const Converter_Index::BlockVector::Iterator  iterEnd = (*m_VolumeList)[pagerIndex].End();
 
-    for( ; iter < iterEnd; iter++ )
-    {
-        SAPDB_UInt NumPermPagesUpdated        = 0;
-        SAPDB_UInt NumPendingUpdatedPermPages = 0;
-
-        Converter_LeafPage  leafPage;
-
-        pager.Read( iter->GetBlockAddress(), leafPage );
-
-        totalPages++;
-        totalIO++;
-
-        const Data_AddressingMode addrMode = leafPage.ReadPageRecoveryMode().GetAddressingMode();
-
-        SAPDBERR_ASSERT_STATE (leafPage.ReadPageRecoveryMode().PageIsRecoverable());
-
-        // write the address of the parent index page on the leaf page according the
-        // iterator. This is needed for converter leaf page i/o to be able to update
-        // the block address of the leaf in the corresponding converter index page.
-
-        leafPage.WriteParentReference (iter->GetParentReference());
-
-        SAPDB_UInt FreeEntryCount = 0;
-
-        // mark all used blocks referenced by the current converter leaf pages as
-        // occupied in the FBM. Moreover all reserved converter leaf page entries
-        // are set to free.
-
-        Converter_PageEntryIterator         pageEntryIter    = leafPage.Begin();
-        const Converter_PageEntryIterator   pageEntryEndIter = leafPage.End();
-
-        for( ; pageEntryIter != pageEntryEndIter; ++pageEntryIter )
+        for( ; iter < iterEnd; iter++ )
         {
-            if( ! pageEntryIter.EntryInUse())
+            SAPDB_UInt NumPermPagesUpdated        = 0;
+            SAPDB_UInt NumPendingUpdatedPermPages = 0;
+
+            Converter_LeafPage  leafPage;
+
+            pager.Read( iter->GetBlockAddress(), leafPage );
+
+            totalPages++;
+            totalIO++;
+
+            const Data_AddressingMode addrMode = leafPage.ReadPageRecoveryMode().GetAddressingMode();
+
+            SAPDBERR_ASSERT_STATE (leafPage.ReadPageRecoveryMode().PageIsRecoverable());
+
+            // write the address of the parent index page on the leaf page according the
+            // iterator. This is needed for converter leaf page i/o to be able to update
+            // the block address of the leaf in the corresponding converter index page.
+
+            leafPage.WriteParentReference (iter->GetParentReference());
+
+            SAPDB_UInt FreeEntryCount = 0;
+
+            // mark all used blocks referenced by the current converter leaf pages as
+            // occupied in the FBM. Moreover all reserved converter leaf page entries
+            // are set to free.
+
+            Converter_PageEntryIterator         pageEntryIter    = leafPage.Begin();
+            const Converter_PageEntryIterator   pageEntryEndIter = leafPage.End();
+
+            for( ; pageEntryIter != pageEntryEndIter; ++pageEntryIter )
             {
-                ++FreeEntryCount;
-                continue;
+                if( ! pageEntryIter.EntryInUse())
+                {
+                    ++FreeEntryCount;
+                    continue;
+                }
+
+                const Converter_PageEntryNo entryPos = pageEntryIter.GetPosition();
+
+                if( pageEntryIter.GetPackedBlockAddress().IsReserved())
+                {
+                    leafPage.ReleaseEntry( entryPos );
+                    ++FreeEntryCount;
+                    continue;
+                }
+
+                m_NumPermPagesUsed.Increment();
+
+                CheckFillingOfDataBase();
+
+                // Update the number of changed data pages since the last save data
+
+                if( leafPage.SavePagesPending( entryPos ) && !m_LastSaveDataSuccessful)
+                    ++ NumPendingUpdatedPermPages;
+
+                if( leafPage.SavePagesRequested( entryPos ))
+                    ++ NumPermPagesUpdated;
             }
 
-            const Converter_PageEntryNo entryPos = pageEntryIter.GetPosition();
+            Converter_BlockAddressIterator elements = leafPage.elements();
 
-            if( pageEntryIter.GetPackedBlockAddress().IsReserved())
+            if( SAPDB_TRUE != FBM_IManager::Instance().SetBlockStateToOccupied(
+                        taskId, elements))
             {
-                leafPage.ReleaseEntry( entryPos );
-                ++FreeEntryCount;
-                continue;
+
+                Converter_Exception errMsg( __CONTEXT__, CONVERTER_BAD_ENTRY_STATE,
+                        (( Data_Dynamic == leafPage.ReadPageRecoveryMode().GetAddressingMode())
+                        ? "permanent" : "static"),
+                        SAPDB_ToString( leafPage.GetPageNo( elements.GetPosition()), _T_d ));
+
+                RTE_Crash( errMsg );
             }
 
-            m_NumPermPagesUsed.Increment();
-
-            CheckFillingOfDataBase();
-
-            // Update the number of changed data pages since the last save data
-
-            if( leafPage.SavePagesPending( entryPos ) && !m_LastSaveDataSuccessful)
-                ++ NumPendingUpdatedPermPages;
-
-            if( leafPage.SavePagesRequested( entryPos ))
-                ++ NumPermPagesUpdated;
-        }
-
-        Converter_BlockAddressIterator elements = leafPage.elements();
-
-        if( SAPDB_TRUE != FBM_IManager::Instance().SetBlockStateToOccupied(
-                    taskId, elements))
-        {
-
-            Converter_Exception errMsg( __CONTEXT__, CONVERTER_BAD_ENTRY_STATE,
-                    (( Data_Dynamic == leafPage.ReadPageRecoveryMode().GetAddressingMode())
-                    ? "permanent" : "static"),
-                    SAPDB_ToString( leafPage.GetPageNo( elements.GetPosition()), _T_d ));
-
-            RTE_Crash( errMsg );
-        }
-
-        if( Data_Static == addrMode )
-        {
-            m_PermStaticMapControl.IntegratePage( taskId, leafPage, FreeEntryCount, NumPermPagesUpdated,
-                                                  NumPendingUpdatedPermPages );
-        }
-        else // Data_Dynamic
-        {
-            m_PermMapControl.IntegratePage( taskId, leafPage, FreeEntryCount, NumPermPagesUpdated,
-                                            NumPendingUpdatedPermPages );
+            if( Data_Static == addrMode )
+            {
+                m_PermStaticMapControl.IntegratePage( taskId, leafPage, FreeEntryCount, NumPermPagesUpdated,
+                                                      NumPendingUpdatedPermPages );
+            }
+            else // Data_Dynamic
+            {
+                m_PermMapControl.IntegratePage( taskId, leafPage, FreeEntryCount, NumPermPagesUpdated,
+                                                NumPendingUpdatedPermPages );
+            }
         }
+        pagerIndex += g01maxdatawriter();
     }
 }
 
@@ -549,7 +544,9 @@ Converter_Converter::ReadSnapShotPages(
                                 "Permanent" : "Static") << " pageNo: " << pageNo << " on block: " <<
                                 newBlock.GetDeviceNo() << "/" << newBlock.GetBlockNo() << " marked as used ");
 
-            if( FBM_IManager::Instance().SetBlockStateToOccupied( taskId, newBlock )){
+            if( FBM_IManager::Instance().SetBlockStateToOccupied( taskId, newBlock ))
+            {
+                m_NumPermPagesUsed.Increment();
                 continue;
             }
 
@@ -1028,8 +1025,6 @@ Converter_Converter::EndSavepoint(
     m_PermStaticMapControl.ReSetSavepointActiveFlag( taskId );
 
     CheckDataAreaPeakFilling();
-
-    m_SavepointIsRequested = false;
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1055,6 +1050,11 @@ Converter_Converter::FreePageNosAfterSavepoint( const tsp00_TaskId taskId )
     // resume all tasks suspended because a FBM overflow occured
     if( ! m_DBFullWaitQueue.IsEmpty() )
         m_DBFullWaitQueue.ResumeAllWaitingTasks( taskId );
+
+    // At this time all relevant FBM blocks are set to free and therefore the 
+    // database filling could be changed although no pages are requested or released.
+    CheckFillingOfDataBase(); 
+
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1114,7 +1114,12 @@ Converter_Converter::GetPositionForWriting(
     }
 
     if( Converter_Ok == retCode )
+    {
+        // At this time all relevant FBM blocks are set to free and therefore the 
+        // database filling could be changed although no pages are requested or released.
+        CheckFillingOfDataBase(); 
         return;
+    }
 
     Converter_MapError mapError( __CONTEXT__ , pageRecoveryMode, pageNo, retCode );
     RTE_Crash( mapError );
@@ -1181,6 +1186,10 @@ Converter_Converter::SetPositionForWriting(
     }
     else
         m_TempMapControl.SetPositionForWriting ( taskId, pageConverterVersion, pageNo, block );
+
+    // At this time all relevant FBM blocks are set to free and therefore the 
+    // database filling could be changed although no pages are requested or released
+    CheckFillingOfDataBase(); 
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1284,18 +1293,6 @@ Converter_Converter::HandleDBFull (
         if( blocksExhausted && SavepointIsPossible() && FBM_IManager::Instance().FreeAfterSVPExist() )
             StartSavepoint( Trans, numChangedPages ); // Do not check necessity of SVP
 
-        if( Kernel_IAdminInfo::Instance().KernelStateIsRestart() )
-        {
-            if((  pageNosExhausted ) || ( blocksExhausted && ! m_SavepointIsRequested ))
-            {
-                Converter_Exception errMsg( __CONTEXT__, CONVERTER_DB_FULL_DURING_RESTART );
-                RTE_Message( errMsg );
-
-                trError = e_no_more_perm_space; // PTS 1111138 TS 2001-07-27
-                return;
-            }
-        }
-
         if(( m_IsLiveCacheInstance ) && ( ! SavepointNeeded() ))
             bd91StartOMSGarbageCollection( Trans, false );
 
@@ -1622,7 +1619,6 @@ Converter_Converter::Dump(
     CM.dmpRestorePagesIsActive            = m_RestorePagesActive;
     CM.dmpColdVerifyIsActive              = m_ColdVerifyIsActive;
     CM.dmpLastSaveDataSuccessful          = m_LastSaveDataSuccessful;
-    CM.dmpSavepointIsRequested            = m_SavepointIsRequested;
 
     if( m_RootBlockAddress.IsValid() )
     {
@@ -1707,14 +1703,17 @@ SAPDB_Bool
 Converter_Converter::DeleteBlockAddress(
     const tsp00_TaskId          taskId,
     const Data_PageNo           &pageNo,
-    const Data_AddressingMode   &addrMode )
+    const Data_PageRecoveryMode &recMode )
 {
     SAPDBERR_ASSERT_STATE( m_Active );
 
-    IOMan_BlockAddress      block;
-    Converter_ReturnCode    retCode;
-
+    IOMan_BlockAddress          block;
+    Converter_ReturnCode        retCode;
+    const Data_AddressingMode   addrMode = recMode.GetAddressingMode();
 
+    if( ! recMode.PageIsRecoverable())
+        return( SAPDB_TRUE );
+    
     if( Data_Dynamic == addrMode ){
         retCode = m_PermMapControl.GetBlockAddress( taskId, pageNo, block );
     }
@@ -1725,24 +1724,17 @@ Converter_Converter::DeleteBlockAddress(
         retCode = Converter_NoConverterEntry;
     }
 
-    if( Converter_Ok == retCode )
-    {
-        Converter_Version  pageConverterVersion; // dummy version, not used in this case
-
-        pageConverterVersion.SetToInitialVersion(); // set to a defined value, which will not influence the execution
-
-        if( Data_Dynamic == addrMode )
-            return( FreePermDynamicPageNo( taskId, pageNo, pageConverterVersion ));
-        else
-            return( FreePermStaticPageNo( taskId, pageNo, pageConverterVersion ));
-    }
-
-    Converter_Exception noConvEntry(__CONTEXT__,CONVERTER_NOENTRY,
-                                    (( Data_Dynamic == addrMode )? "permanent" : "static" ),
-                                    SAPDB_ToString( pageNo, _T_d ));
-    RTE_Message( noConvEntry );
-
-    return( SAPDB_FALSE );
+    if( Converter_Ok != retCode )
+        return( SAPDB_FALSE );
+    
+    Converter_Version  pageConverterVersion; // dummy version, not used in this case
+    
+    pageConverterVersion.SetToInitialVersion(); // set to a defined value, which will not influence the execution
+    
+    if( Data_Dynamic == addrMode )
+        return( FreePermDynamicPageNo( taskId, pageNo, pageConverterVersion ));
+    else
+        return( FreePermStaticPageNo( taskId, pageNo, pageConverterVersion ));
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1751,15 +1743,20 @@ SAPDB_Bool
 Converter_Converter::GetBlockAddress(
     const tsp00_TaskId          taskId,
     const Data_PageNo           &pageNo,
-    const Data_AddressingMode   &addrMode,
+    const Data_PageRecoveryMode &recMode,
     IOMan_BlockAddress          &block )
 {
     SAPDBERR_ASSERT_STATE( m_Active );
 
-    Converter_ReturnCode    retCode;
+    Converter_ReturnCode        retCode;
+    const Data_AddressingMode   addrMode = recMode.GetAddressingMode();
 
-    if( Data_Dynamic == addrMode ){
-        retCode = m_PermMapControl.GetBlockAddress( taskId, pageNo, block );
+    if( Data_Dynamic == addrMode )
+    {
+        if( recMode.PageIsRecoverable())
+            retCode = m_PermMapControl.GetBlockAddress( taskId, pageNo, block );
+        else
+            retCode = m_TempMapControl.GetBlockAddress( taskId, pageNo, block );
     }
     else if( Data_Static == addrMode ){
         retCode = m_PermStaticMapControl.GetBlockAddress( taskId, pageNo, block );
@@ -1771,11 +1768,6 @@ Converter_Converter::GetBlockAddress(
     if( Converter_Ok == retCode )
         return( SAPDB_TRUE );
 
-    Converter_Exception noConvEntry(__CONTEXT__,CONVERTER_NOENTRY,
-                                    (( Data_Dynamic == addrMode )? "permanent" : "static" ),
-                                    SAPDB_ToString( pageNo, _T_d ));
-    RTE_Message( noConvEntry );
-
     block.Invalidate();
 
     return( SAPDB_FALSE );
@@ -1787,18 +1779,21 @@ SAPDB_Bool
 Converter_Converter::SetNewBlockAddress(
     const tsp00_TaskId          taskId,
     const Data_PageNo           &pageNo,
-    const Data_AddressingMode   &addrMode,
+    const Data_PageRecoveryMode &recMode,
     const IOMan_BlockAddress    &blockAddr )
 {
     SAPDBERR_ASSERT_STATE( m_Active );
-
-    if( Data_Dynamic == addrMode )
-        m_PermMapControl.SetBlockAddressOfRestoredPage(
-            taskId, pageNo, m_SnapShotVersion, blockAddr, m_RestoreDataActive );
-    else if( Data_Static == addrMode )
+    
+    if( ! recMode.PageIsRecoverable())
+        return( SAPDB_TRUE );
+    
+    if( recMode.UseStaticPageAddressing() )
         m_PermStaticMapControl.SetBlockAddressOfRestoredPage(
-            taskId, pageNo, m_SnapShotVersion, blockAddr, m_RestoreDataActive );
-
+        taskId, pageNo, m_SnapShotVersion, blockAddr, m_RestoreDataActive );
+    else
+        m_PermMapControl.SetBlockAddressOfRestoredPage(
+        taskId, pageNo, m_SnapShotVersion, blockAddr, m_RestoreDataActive );
+    
     return( SAPDB_TRUE );
 }
 
@@ -1829,7 +1824,6 @@ Converter_Converter::InitializeBase( const SAPDB_UInt NumAllPages )
     m_SecuritySpaceLimit               = ( m_NumAllPages - SAPDB_MIN( m_NumAllPages/25, 4000));
     m_Active                           = true;
     m_InMaintenanceMode                = false;
-    m_SavepointIsRequested             = false;
     m_SavePagesIsActiv                 = false;
     m_SaveDataIsActiv                  = false;
     m_RestoreDataActive                = false;
@@ -1910,6 +1904,7 @@ Converter_Converter::FreePermPageNo(
 
     SAPDB_Bool  bWaitUntilSavepointFinished  = false;
     SAPDB_Bool  bOldBlockIsUsableImmediately = false;
+    SAPDB_Bool  bDoDecrementPermUseCount     = true;
 
     do
     {
@@ -1917,7 +1912,7 @@ Converter_Converter::FreePermPageNo(
 
         mapControl.FreePageNo( taskId, pageConverterVersion, m_SnapShotVersion, pageNo,
                                waitContext, m_FreePageNoWaitQueue, bWaitUntilSavepointFinished,
-                               bOldBlockIsUsableImmediately );
+                               bOldBlockIsUsableImmediately, bDoDecrementPermUseCount );
 
         if( bWaitUntilSavepointFinished )
         {
@@ -1930,7 +1925,8 @@ Converter_Converter::FreePermPageNo(
     }
     while( bWaitUntilSavepointFinished );
 
-    m_NumPermPagesUsed.Decrement();
+    if( bDoDecrementPermUseCount )
+        m_NumPermPagesUsed.Decrement();
 
     SAPDBTRACE_WRITELN( Converter_Trace, 7, "PermPagesUsed: " << m_NumPermPagesUsed );
 
@@ -1955,8 +1951,6 @@ Converter_Converter::StartSavepoint(
                                 SAPDB_ToString( FBM_IManager::Instance().GetNumberOfFreeBlocks(), _T_d ));
     RTE_Message( errMsg );
 
-    m_SavepointIsRequested = true;
-
     Log_SavepointManager.StartSavepoint( trans, Log_SVPReasonConverter );
 }
 
