@@ -70,6 +70,7 @@
 #include "KernelCommon/Kernel_DateTime.hpp"
 
 #include "SAPDBCommon/MemoryManagement/SAPDBMem_IRawAllocator.hpp"
+#include "Logging/Log_Savepoint.hpp"
 
 
 /*===========================================================================*
@@ -611,19 +612,19 @@ public:
     virtual SAPDB_Bool GetBlockAddress(
         const tsp00_TaskId          taskId,
         const Data_PageNo           &pageNo,
-        const Data_AddressingMode   &addrMode,
+        const Data_PageRecoveryMode &recMode,
         IOMan_BlockAddress          &block );
 
     virtual SAPDB_Bool SetNewBlockAddress(
         const tsp00_TaskId          taskId,
         const Data_PageNo           &pageNo,
-        const Data_AddressingMode   &addrMode,
+        const Data_PageRecoveryMode &recMode,
         const IOMan_BlockAddress    &blockAddr );
 
     virtual SAPDB_Bool DeleteBlockAddress(
         const tsp00_TaskId          taskId,
         const Data_PageNo           &pageNo,
-        const Data_AddressingMode   &addrMode );
+        const Data_PageRecoveryMode &recMode );
 
     virtual SAPDB_UInt GetVolumeNoBitCount() const{
         return( m_VolumeNoBitCount );
@@ -677,6 +678,10 @@ public:
         m_NumSnapShotPages.Increment();
     }
 
+    /// Increment number of used permanent (dynamic and static) pages
+    void IncrementNumUsedPermPages(){
+        m_NumPermPagesUsed.Increment();
+    }
 private:
 
     /// Is a list of data volume specific block addresses. Each item
@@ -747,8 +752,10 @@ private:
        @return         none
      */
 
-    void CheckFillingOfDataBase(){
-        m_DBUsageMonitor.Watch (GetNumberOfUsedPages());
+    void CheckFillingOfDataBase()
+    {
+        const SAPDB_UInt usedBlocks = FBM_IManager::Instance().GetNumberOfUsedBlocks();
+        m_DBUsageMonitor.Watch( SAPDB_MAX( usedBlocks, GetNumberOfUsedPages()));
     }
 
     /*!
@@ -766,7 +773,7 @@ private:
         if( m_InMaintenanceMode ) // converter is in special mode
             return( false );
 
-        if( m_SavepointIsRequested ) // SVP is still requested
+        if( Log_SavepointManager.IsActive() ) // SVP is still requested
             return( false );
 
         return( true );
