@@ -780,7 +780,7 @@ Converter_Map::RestoreUsedPageNos(
         // ---------------------------------------------------------------------
         if (mustrelease != 0)
         {
-            const SAPDB_Bool bIsDynamic = section.GetRecoveryMode().UseDynamicPageAddressing();
+            const SAPDB_Bool bIsPerm = section.GetRecoveryMode().UseDynamicPageAddressing();
 
             for (SAPDB_UInt i=0; i<CONVERTER_MAP_BITBLOCKSIZE; ++i)
             {
@@ -802,7 +802,7 @@ Converter_Map::RestoreUsedPageNos(
 
                         SAPDBTRACE_WRITELN( Converter_Trace, 7, "Snapshot: Block: " <<
                                             block.GetDeviceNo() << "/" << block.GetBlockNo() << " of " <<
-                                            (( bIsDynamic ) ? "permanent pageNo " : "static pageNo " ) <<
+                                            (( bIsPerm ) ? "permanent pageNo " : "static pageNo " ) <<
                                             basepageentryno + i << " is not marked as free");
 
                     }
@@ -810,7 +810,7 @@ Converter_Map::RestoreUsedPageNos(
                     {
                         SAPDBTRACE_WRITELN( Converter_Trace, 7, "Block: " <<
                                             block.GetDeviceNo() << "/" << block.GetBlockNo() << " of " <<
-                                            (( bIsDynamic ) ? "permanent pageNo " : "static pageNo " ) <<
+                                            (( bIsPerm ) ? "permanent pageNo " : "static pageNo " ) <<
                                             basepageentryno + i << " is marked as free");
 
                         FBM_IManager::Instance().SetBlockStateToFree( block );
@@ -928,184 +928,53 @@ Converter_Map::FinishSectionRestoreUsedPageNos(
     Converter_PermMapSection    &section,
     const Converter_Version     &snapShotVersion )
 {
-    // -------------------------------------------------------------------------
     // check completely filled pages:
-    // -------------------------------------------------------------------------
+
     Converter_MapSection::FullEntryUsedIterator       fullIter    = section.FullUsedEntriesBegin();
     const Converter_MapSection::FullEntryUsedIterator fullEndIter = section.FullUsedEntriesEnd();
 
-    while (fullIter != fullEndIter)
+    SAPDBTRACE_WRITELN( Converter_Trace, 7,
+                        "Delete converter leaves from section: " << section.GetSectionNo());
+
+    while( fullIter != fullEndIter )
     {
         const Converter_MapEntryNo entryNo = fullIter.GetPosition();
 
-        if ( false == m_IsPageUsedAfterRestorePages[ entryNo ] )
+        if ( m_IsPageUsedAfterRestorePages[ entryNo ] )
         {
-            // During the foregoing restore pages nobody looks at this leaf
-            // page therefore it could be removed and the corresponding data
-            // pages could be set to free.
-
-            Converter_MapEntry          &entry        = GetEntry( entryNo );
-            Converter_LeafPage          page          = entry.GetPage();
-            Converter_PageEntryCount    releasedcount = 0;
-            const SAPDB_Bool            bIsDynamic    = section.GetRecoveryMode().UseDynamicPageAddressing();
-
-            for( Converter_PageEntryNo i = 0; i < page.GetEntryCount(); ++i )
-            {
-                // must release leaf page entry and block addr: note that
-
-                // all entries are in use, because page is in full chain
-                IOMan_BlockAddress block;
-
-                page.ReadBlockAddress( i, block );
-                page.ReleaseEntry( i );
-
-                if( snapShotVersion.IsValid())
-                {
-                    // without reading the converter version of the page it's not possible
-                    // to decide whether the old block is needed for the frozen version,
-                    // therefore the old block is not set to free. If the block is not
-                    // needed and therefore not part of the frozen converter, the block
-                    // is set to free in the FBM at the next restart!
-
-                    SAPDBTRACE_WRITELN( Converter_Trace, 7, "Snapshot: Block: " <<
-                                        block.GetDeviceNo() << "/" << block.GetBlockNo() << " of " <<
-                                        (( bIsDynamic ) ? "permanent pageNo " : "static pageNo " ) <<
-                                        page.ReadBasePageNo() + i << " is not marked as free");
-                }
-                else
-                {
-                    SAPDBTRACE_WRITELN( Converter_Trace, 7, "Block: " <<
-                                        block.GetDeviceNo() << "/" << block.GetBlockNo() << " of " <<
-                                        (( bIsDynamic ) ? "permanent pageNo " : "static pageNo " ) <<
-                                        page.ReadBasePageNo() + i << " is marked as free");
-
-                    FBM_IManager::Instance().SetBlockStateToFree( block );
-                }
-                ++releasedcount;
-            }
-
-            section.RechainEntryFromFullToPartiallyUsed( entryNo );
-            section.RechainEntryFromPartiallyUsedToUnused( entryNo );
-
-            // -------------------------------------------------------------------------
-            // add parentReference for removal of index entry during flush/savepoint
-            // -------------------------------------------------------------------------
-            const Converter_ParentReference             parentReference = page.ReadParentReference();
-            Container_Vector<Converter_ParentReference> &parentRefs     = section.GetParentReferences();
-
-            if( parentReference.GetIndexEntryNo().IsValid() && parentReference.GetPageEntryNo().IsValid())
-            {
-                if( parentRefs.IsFull()){
-                    parentRefs.Reserve( parentRefs.GetCapacity() + 5 );
-                }
-                parentRefs.InsertEnd( parentReference );
-
-                // is old block address needed for frozen converter version
-                if(( ! snapShotVersion.IsValid()) || ( page.ReadVersion() > snapShotVersion )){
-                    FBM_IManager::Instance().SetBlockStateToFree( page.ReadBlockAddress() );
-                }
-            }
-
-            DeletePage( entryNo );
-            // we don't want to flush this empty LeafPage
-            section.MarkPageAsUnChanged( entryNo );
-
-            // PTS 1116251 TS 2002-07-09
-            entry.SetFreePageEntryCount( 0 );
-
-            section.DecrementNumUsedDataPages( ( SAPDB_UInt) releasedcount );
+            ++fullIter;
+            continue;
         }
+
+        RemoveUnusedPageAfterRestore( entryNo, section, snapShotVersion );
         ++fullIter;
+
+        // +++ rechain modifies partiallyUsedEnty chain!!! +++
+
+        section.RechainEntryFromFullToPartiallyUsed( entryNo );
+        section.RechainEntryFromPartiallyUsedToUnused( entryNo );
     }
 
-    // -------------------------------------------------------------------------
     // check partially used pages:
-    // -------------------------------------------------------------------------
+
     Converter_MapSection::PartiallyUsedEntryIterator       partiallyIter    = section.PartiallyUsedEntriesBegin();
     const Converter_MapSection::PartiallyUsedEntryIterator partiallyEndIter = section.PartiallyUsedEntriesEnd();
 
-    while (partiallyIter != partiallyEndIter)
+    while( partiallyIter != partiallyEndIter )
     {
         const Converter_MapEntryNo entryNo = partiallyIter.GetPosition();
 
-        if (false == m_IsPageUsedAfterRestorePages[ entryNo ] )
+        if( m_IsPageUsedAfterRestorePages[ entryNo ] )
         {
-            // During the foregoing restore pages nobody looks at this leaf
-            // page therefore it could be removed and the corresponding data
-            // pages could be set to free.
-
-            Converter_MapEntry          &entry        = GetEntry( entryNo );
-            Converter_LeafPage          page          = entry.GetPage();
-            Converter_PageEntryCount    releasedcount = 0;
-            const SAPDB_Bool            bIsDynamic    = section.GetRecoveryMode().UseDynamicPageAddressing();
-
-            for (Converter_PageEntryNo i = 0; i < page.GetEntryCount(); ++i)
-            {
-                if( page.EntryInUse( i ))
-                {
-                    // must release leaf page entry and block blockAddr:
-                    IOMan_BlockAddress block;
-
-                    page.ReadBlockAddress( i, block );
-                    page.ReleaseEntry( i );
-
-                    if( snapShotVersion.IsValid())
-                    {
-                        // without reading the converter version of the page it's not possible
-                        // to decide whether the old block is needed for the frozen version,
-                        // therefore the old block is not set to free. If the block is not
-                        // needed and therefore not part of the frozen converter, the block
-                        // is set to free in the FBM at the next restart!
-
-                        SAPDBTRACE_WRITELN( Converter_Trace, 7, "Snapshot: Block: " <<
-                                            block.GetDeviceNo() << "/" << block.GetBlockNo() << " of " <<
-                                            (( bIsDynamic ) ? "permanent pageNo " : "static pageNo " ) <<
-                                            page.ReadBasePageNo() + i << " is not marked as free");
-                    }
-                    else
-                    {
-                        SAPDBTRACE_WRITELN( Converter_Trace, 7, "Block: " <<
-                                            block.GetDeviceNo() << "/" << block.GetBlockNo() << " of " <<
-                                            (( bIsDynamic ) ? "permanent pageNo " : "static pageNo " ) <<
-                                            page.ReadBasePageNo() + i << " is marked as free");
-
-                        FBM_IManager::Instance().SetBlockStateToFree( block );
-                    }
-                    ++releasedcount;
-                }
-            }
-
-            section.RechainEntryFromPartiallyUsedToUnused( entryNo );
-
-            // -------------------------------------------------------------------------
-            // add parentReference for removal of index entry during flush/savepoint
-            // -------------------------------------------------------------------------
-            const Converter_ParentReference             parentReference = page.ReadParentReference();
-            Container_Vector<Converter_ParentReference> &parentRefs     = section.GetParentReferences();
-
-            if( parentReference.GetIndexEntryNo().IsValid() && parentReference.GetPageEntryNo().IsValid())
-            {
-                if( parentRefs.IsFull()){
-                    parentRefs.Reserve( parentRefs.GetCapacity() + 5 );
-                }
-                parentRefs.InsertEnd( parentReference );
-
-                // is old block address needed for frozen converter version
-                if(( ! snapShotVersion.IsValid()) || ( page.ReadVersion() > snapShotVersion )){
-                    FBM_IManager::Instance().SetBlockStateToFree( page.ReadBlockAddress() );
-                }
-            }
-
-            DeletePage( entryNo );
-            // we don't want to flush this empty LeafPage
-            section.MarkPageAsUnChanged(entryNo );
-
-            // PTS 1116251 TS 2002-07-09
-            entry.SetFreePageEntryCount( 0 );
-
-            section.DecrementNumUsedDataPages( ( SAPDB_UInt) releasedcount );
+            ++partiallyIter;
+            continue;
         }
+
+        RemoveUnusedPageAfterRestore( entryNo, section, snapShotVersion );
         ++partiallyIter;
+
+        // +++ rechain modifies partiallyUsedEnty chain!!! +++
+        section.RechainEntryFromPartiallyUsedToUnused( entryNo );
     }
 }
 
@@ -1305,6 +1174,94 @@ Converter_Map::UseUnusedEntryAndGetPage(
     return( SAPDB_TRUE );
 }
 
+// -----------------------------------------------------------------------------
+
+void
+Converter_Map::RemoveUnusedPageAfterRestore(  // PTS 1132116 TS 2004-10-25
+    const Converter_MapEntryNo  entryNo,
+    Converter_PermMapSection&   section,
+    const Converter_Version&    snapShotVersion )
+{
+
+    // During the foregoing restore pages nobody looks at this leaf
+    // page therefore it could be removed and the corresponding data
+    // pages could be set to free.
+
+    SAPDBTRACE_WRITELN( Converter_Trace, 7, "Delete leaf of entryNo: " << entryNo );
+
+    Converter_MapEntry&         entry         = GetEntry( entryNo );
+    Converter_LeafPage          page          = entry.GetPage();
+    Converter_PageEntryCount    releasedCount = 0;
+
+    const SAPDB_Bool bIsPerm = section.GetRecoveryMode().UseDynamicPageAddressing();
+    const Converter_PageEntryNo maxPageEntries = page.GetEntryCount();
+
+    for( Converter_PageEntryNo pageEntry = 0; pageEntry < maxPageEntries; ++pageEntry)
+    {
+        if( ! page.EntryInUse( pageEntry )){
+            continue;
+        }
+
+        ++releasedCount;
+
+        IOMan_BlockAddress block;
+
+        page.ReadBlockAddress( pageEntry, block);
+        page.ReleaseEntry( pageEntry );
+
+        // release unused data page
+
+        if( snapShotVersion.IsValid())
+        {
+            // without reading the converter version of the page it's not possible
+            // to decide whether the old block is needed for the frozen version,
+            // therefore the old block is not set to free. If the block is not
+            // needed and therefore not part of the frozen converter, the block
+            // is set to free in the FBM at the next restart!
+
+            SAPDBTRACE_WRITELN( Converter_Trace, 7, "Snapshot: Block: " <<
+                                block.GetDeviceNo() << "/" << block.GetBlockNo() << " of " <<
+                                (( bIsPerm ) ? "permanent pageNo " : "static pageNo " ) <<
+                                page.ReadBasePageNo() + pageEntry << " is not marked as free");
+        }
+        else
+        {
+            SAPDBTRACE_WRITELN( Converter_Trace, 7, "Block: " <<
+                                block.GetDeviceNo() << "/" << block.GetBlockNo() << " of " <<
+                                (( bIsPerm ) ? "permanent pageNo " : "static pageNo " ) <<
+                                page.ReadBasePageNo() + pageEntry << " is marked as free");
+
+            FBM_IManager::Instance().SetBlockStateToFree( block );
+        }
+    }
+
+    // add parentReference for removal of index entry during flush/savepoint
+
+    const Converter_ParentReference             parentReference = page.ReadParentReference();
+    Container_Vector<Converter_ParentReference> &parentRefs     = section.GetParentReferences();
+
+    if( parentReference.GetIndexEntryNo().IsValid() && parentReference.GetPageEntryNo().IsValid())
+    {
+        if( parentRefs.IsFull()){
+            parentRefs.Reserve( parentRefs.GetCapacity() + 5 );
+        }
+        parentRefs.InsertEnd( parentReference );
+
+        // release unused converter leaf page
+        // is old block address needed for frozen converter version
+        if(( ! snapShotVersion.IsValid()) || ( page.ReadVersion() > snapShotVersion )){
+            FBM_IManager::Instance().SetBlockStateToFree( page.ReadBlockAddress() );
+        }
+    }
+
+    entry.SetFreePageEntryCount( 0 );
+
+    DeletePage( entryNo );
+    // we don't want to flush this empty LeafPage
+    section.MarkPageAsUnChanged( entryNo );
+
+    section.DecrementNumUsedDataPages( ( SAPDB_UInt) releasedCount );
+}
 
 /*===========================================================================*
  *  END OF CODE                                                              *
