@@ -452,14 +452,14 @@ public:
     /*! @name SnapShot                                                        */
     /* -----------------------------------------------------------------------*/
 
-     /*!
-       @brief   Checks if the converter leaf page specified by the given
-                basePageNo, has the same block address as the given block
-                address.
-       @param   basePageNo [in] data base page number
-       @param   block      [in] block address of the converter leaf page
-       @return  SAPDB_Bool
-     */
+    /*!
+      @brief   Checks if the converter leaf page specified by the given
+               basePageNo, has the same block address as the given block
+               address.
+      @param   basePageNo [in] data base page number
+      @param   block      [in] block address of the converter leaf page
+      @return  SAPDB_Bool
+    */
 
     SAPDB_Bool IsMember(
         const Data_PageNo           basePageNo,
@@ -540,6 +540,11 @@ private:
         IOMan_BlockAddress          &oldaddr,
         const Converter_PageEntryNo pageEntryNo );
 
+    void RemoveUnusedPageAfterRestore( // PTS 1132116 TS 2004-10-25
+        const Converter_MapEntryNo  entryNo,
+        Converter_PermMapSection&   section,
+        const Converter_Version&    snapShotVersion );
+
 private:
 
     /// buffer containing the map entries
