@@ -605,6 +605,7 @@ Converter_PermMapControl::RemovePageNoFromBackUpPageNoList(
     if( !bPnoFound )
     {
         Converter_Exception errMsg(__CONTEXT__, CONVERTER_PNO_NOT_IN_BACKUP_PNO_LIST,
+                                   ((GetRecoveryMode().UseDynamicPageAddressing()) ? "Permanent" : "Static" ),
                                    SAPDB_ToString( pageNo, _T_d));
         RTE_Message( errMsg );
     }
@@ -1054,7 +1055,8 @@ Converter_PermMapControl::FreePageNo(
     Converter_WaitQueue::WaitContext    &waitContext,
     Converter_WaitQueue                 &freePageNoWaitQueue,
     SAPDB_Bool                          &bWaitUntilSavepointFinished,
-    SAPDB_Bool                          &bOldBlockIsUsableImmediately )
+    SAPDB_Bool                          &bOldBlockIsUsableImmediately,
+    SAPDB_Bool                          &bDoDecrementPermUseCount )
 {
     SAPDBTRACE_ROUTINE_DEBUG( "PermMapControl::FreePageNo", Converter_Trace, 5 );
 
@@ -1063,6 +1065,8 @@ Converter_PermMapControl::FreePageNo(
 
     bWaitUntilSavepointFinished  = false;
     bOldBlockIsUsableImmediately = true;
+    bDoDecrementPermUseCount     = true;
+
 
     {
         Converter_PermMapSection section( taskId, m_SectionData[ sectionNo ], m_SectionPermData[ sectionNo ]);
@@ -1122,6 +1126,8 @@ Converter_PermMapControl::FreePageNo(
                              "permanent pageNo " : "static pageNo " ) << pageNo <<
                             " is not marked as free");
 
+        bDoDecrementPermUseCount = false;
+
         return; // oldBlock is needed for frozen version
     }
 
