@@ -562,6 +562,9 @@ public:
                                                    due to a currently active savepoint
        @param   bOldBlockIsUsableImmediately [out] true if block is set to free and usable
                                                    during the current savepoint cycle by another page
+       @param   bDoDecrementPermUseCount [out] true if page number is not part of the snapshot 
+                                               and therefore the perm used statistic count could
+                                               be decreased.
        @return  none
      */
 
@@ -573,7 +576,8 @@ public:
         Converter_WaitQueue::WaitContext    &waitContext,
         Converter_WaitQueue                 &freePageNoWaitQueue,
         SAPDB_Bool                          &bWaitUntilSavepointFinished,
-        SAPDB_Bool                          &bOldBlockIsUsableImmediately );
+        SAPDB_Bool                          &bOldBlockIsUsableImmediately,
+        SAPDB_Bool                          &bDoDecrementPermUseCount );
 
     /* -----------------------------------------------------------------------*/
     /*! @name ColdVerify/Check data with update                               */
