@@ -887,45 +887,6 @@ bd10CheckAndStartSavepoint(tgg00_TransContext & Trans )
 
 /*---------------------------------------------------------------------------*/
 
-/*!
-   @brief          Returns actual i/o address of data page with given page number
-   @param          Trans [in] current tranaction context
-   @param          PageNo [in] page number for which the corresponding block address is requested
-   @param          bIsDynamic [in] addressing mode of the page for which a deviced position is wanted
-   @param          DevNo [out] device number of requested page
-   @param          BlockNo [out] device offset of requested page
-   @return         none
-
-   - The actual i/o address of a given page number is read from
-     the converter and returned.
-   - If the corresponding converter entry contains an invalid
-     i/o address or if 'id' denotes a free pageid, the program
-     aborts.
- */
-externC void
-bd10GetBlockAddress (
-    tgg00_TransContext & trans,
-    tsp00_PageNo         pageNo,
-    pasbool              bIsDynamic,
-    tsp00_Int2         & devNo,
-    tsp00_Int4         & blockNo )
-{
-    const Data_AddressingMode  addrMode = ( bIsDynamic ? Data_Dynamic : Data_Static );
-
-    IOMan_BlockAddress  block;
-
-    if( SAPDB_TRUE == Converter_IDiagnose::Instance().GetBlockAddress(
-                trans.trTaskId_gg00, pageNo, addrMode, block ))
-    {
-        devNo   = block.GetDeviceNo();
-        blockNo = block.GetBlockNo();
-    }
-    else
-        trans.trError_gg00 = e_no_converter_entry;
-}
-
-/*---------------------------------------------------------------------------*/
-
 /*!
     @brief   Checks if the current back up finished successfully. Checks if 
              all pages marked as to be included in the back up were fetched 
