@@ -60,6 +60,7 @@
 #include "hsp81.h"
 
 #include "livecachetypes.h"  /* no check */
+#include "SAPDBCommon/SAPDB_MemCopyMove.hpp"
 
 #if defined (EXT_CLIENT_SQL)
 #include "ggg00.h"
@@ -359,7 +360,7 @@ int SQL_ColumnDesc::getParmLongDescFromBuffer( const tsp00_Byte * dataPtr,
         vfi.ind = -1;
       }
       else {
-        memcpy( stmt.getLongDescPtr(long_idx), dataPtr + sfi.sp1i_bufpos,
+        SAPDB_MemCopyNoCheck(stmt.getLongDescPtr(long_idx), dataPtr + sfi.sp1i_bufpos,
                 sfi.sp1i_length );
       }
       break;
@@ -423,7 +424,7 @@ int SQL_ColumnDesc::getOpenLongDataFromBuffer( const tsp00_Byte * dataPtr,
           }
           break;
         default :
-          memcpy( (char*)vfi.vaddr + longDescPtr->varPos,
+          SAPDB_MemCopyNoCheck( (char*)vfi.vaddr + longDescPtr->varPos,
                   dataPtr + longDescPtr->dsc.ld_valpos() - 1,
                   longDescPtr->dsc.ld_vallen() );
           longDescPtr->varPos = longDescPtr->varPos + longDescPtr->dsc.ld_vallen();
@@ -731,11 +732,11 @@ int SQL_ColumnDesc::addParmVChar( PIn_Part * part,
               for ( int j = 0; j < len ; j++ ) {
                 if ( *((char*)vfi.vaddr + j) & 0x80 ) {
                   rtError = e_nonascii_char;
-                  rc = 0;
+                  rc = -(j + 1); /* PTS 1130628 */
                   break;
                 }
               }
-              if (!rc) break;
+              if (rc < 0) break; /* PTS 1130628 */
             }
 # endif
           case SqlCol::VBYTE_DB :  /* PTS 1124672 */
@@ -1036,7 +1037,7 @@ int SQL_ColumnDesc::addParmVChar( PIn_Part * part,
     }
   }
 
-  if (0 == rc) {
+  if (0 >= rc) { /* PTS 1130628 */
     sprintf( buf1, "Prm:%d", idx );
     sprintf( buf2, "VT:%x", vfi.vtype );                  /* PTS 1111983 %d -> %x */
     sprintf( buf3, "DBT:%x", (int)(sfi.sp1i_data_type) ); /* PTS 1111983 %d -> %x */
@@ -1380,11 +1381,13 @@ int SQL_ColumnDesc::getParmVChar( const teo00_Byte * dataPtr,
                   for ( int j = 0; j < len ; j++ ) {
                     if ( *((char*)dataPtr + sfi.sp1i_bufpos + j) & 0x80 ) {
                       rtError = e_nonascii_char;
-                      rc = 0;
+                      rc = -(j + 1); /* PTS 1130628 */
                       break;
                     }
                   }
-                  if (!rc) break;
+                  if ( (rc < 0) && (-rc - 1 >= vfi.vlen) ) {
+                    rc = 0;
+                  }
                 }
 # endif
               case SqlCol::VCHAR :
@@ -1393,7 +1396,7 @@ int SQL_ColumnDesc::getParmVChar( const teo00_Byte * dataPtr,
                   vfi.ind = len;
                   len = vfi.vlen;
                 }
-                memcpy( vfi.vaddr, dataPtr + sfi.sp1i_bufpos, len );
+                SAPDB_MemCopyNoCheck( vfi.vaddr, dataPtr + sfi.sp1i_bufpos, len );
                 if (len < vfi.vlen) {
                   memset( (char*)vfi.vaddr + len, defChar, 
                           vfi.vlen - len );
@@ -1405,7 +1408,7 @@ int SQL_ColumnDesc::getParmVChar( const teo00_Byte * dataPtr,
                   vfi.ind = len;
                   len = vfi.vlen - 1;
                 }
-                memcpy( vfi.vaddr, dataPtr + sfi.sp1i_bufpos, len );
+                SAPDB_MemCopyNoCheck( vfi.vaddr, dataPtr + sfi.sp1i_bufpos, len );
                 *((char*)vfi.vaddr + len) = '\0';
                 break;
             }
@@ -1462,7 +1465,7 @@ int SQL_ColumnDesc::getParmVChar( const teo00_Byte * dataPtr,
       else {
         SQL_LongDesc * longDescPtr = stmt.getLongDescPtr(long_idx);
         char colIdx = longDescPtr->dsc.ld_used_in_ak()[0];
-        memcpy( &longDescPtr->dsc, dataPtr + longDescPtr->bufPos,
+        SAPDB_MemCopyNoCheck( &longDescPtr->dsc, dataPtr + longDescPtr->bufPos,
                 sizeof(longDescPtr->dsc) );
         longDescPtr->dsc.ld_used_in_ak()[0] = colIdx;
         if ( (vm_datapart == longDescPtr->dsc.ld_valmode()) ||
@@ -1504,7 +1507,7 @@ int SQL_ColumnDesc::getParmVChar( const teo00_Byte * dataPtr,
               }
               break;
             default :
-              memcpy( (char*)vfi.vaddr + longDescPtr->varPos,
+              SAPDB_MemCopyNoCheck( (char*)vfi.vaddr + longDescPtr->varPos,
                       dataPtr + longDescPtr->dsc.ld_valpos() - 1,
                       longDescPtr->dsc.ld_vallen() );
               longDescPtr->varPos = longDescPtr->varPos + 
@@ -1536,7 +1539,7 @@ int SQL_ColumnDesc::getParmVChar( const teo00_Byte * dataPtr,
         unsigned int converted_len;
         SQL_LongDesc * longDescPtr = stmt.getLongDescPtr(long_idx);
         char colIdx = longDescPtr->dsc.ld_used_in_ak()[0];
-        memcpy( &longDescPtr->dsc, dataPtr + longDescPtr->bufPos,
+        SAPDB_MemCopyNoCheck( &longDescPtr->dsc, dataPtr + longDescPtr->bufPos,
                 sizeof(longDescPtr->dsc));
         longDescPtr->dsc.ld_used_in_ak()[0] = colIdx;
         if ( (vm_datapart == longDescPtr->dsc.ld_valmode()) ||
@@ -1680,7 +1683,7 @@ int SQL_ColumnDesc::getParmVChar( const teo00_Byte * dataPtr,
       break;
   }
 
-  if (rc == 0) {
+  if (rc <= 0) { /* PTS 1130628 */
     sprintf( buf1, "Prm:%d", idx );
     sprintf( buf2, "VT:%x", vfi.vtype );                  /* PTS 1111983 %d -> %x */
     sprintf( buf3, "DBT:%x", (int)(sfi.sp1i_data_type) ); /* PTS 1111983 %d -> %x */
