@@ -79,6 +79,12 @@
 
 #include "hsp81.h"
 #include "hsp100.h"    /* PTS 1112481 */
+#include "SAPDBCommon/SAPDB_MemCopyMove.hpp"
+
+/* PTS 1130628 */
+#if !defined (EXT_CLIENT_SQL)
+#include "hsp78_0.h"
+#endif
 
 
 /************************************************************************/
@@ -170,6 +176,7 @@ SQL_SessionContext::SQL_SessionContext(SqlHandle & pHandle) :
   SAPDBMem_IRawAllocator * m_alloc;
   m_pSink->GetSqlClsAllocator(m_alloc); /* PTS 1122076 */
   m_curr_method = (tsp00_KnlIdentifier*)m_alloc->Allocate(sizeof(tsp00_KnlIdentifier));
+  m_hasUnicodeCatalog = OMS_Globals::KernelInterfaceInstance->IsUnicodeInstance(); /* PTS 1130628 */
   short error = 0;                                          /* PTS 1121342 */
   m_pSink->IsUnicodeInstance( m_isUnicodeInstance, error ); /* PTS 1121342 */
 # else
@@ -177,6 +184,9 @@ SQL_SessionContext::SQL_SessionContext(SqlHandle & pHandle) :
 {
 # endif
   m_sqlhandle.m_session_context = this; /* PTS 1122076 */
+# if !defined(EXT_CLIENT_SQL)
+    m_sqlhandle.dbpAdviseLcSink(m_pSink); /* PTS 1130628 */
+# endif
   m_sqlmode.becomes(sp1sm_internal);
 }
 
@@ -333,7 +343,7 @@ int SQL_SessionContext::connect( SQL_Statement * pSqlStmt,
     part->AddParameterArg( stCrypt, 1, sizeof(stCrypt), 50, 'x' );
     tsp00_TermId termid;
     // sqltermid (termid); later: use module var initialized by environment
-    memcpy( termid, "<unknown><unknown>", sizeof (termid) );
+    SAPDB_MemCopyNoCheck( termid, "<unknown><unknown>", sizeof (termid) );
     part->AddParameterArg( termid, 51, sizeof(termid),
                            sizeof(termid) + 1, ' ');
     reqWriter.Close();
@@ -858,10 +868,10 @@ int SQL_SessionContext::executeDBRequest( IliveCacheSink * pILCSink,
 
   m_sqlca.sqlcode   = m_reply_packet->sp1_segm().sp1r_returncode();
   m_sqlca.sqlerrpos = m_reply_packet->sp1_segm().sp1r_errorpos();
-  memcpy( m_sqlca.sqlstate, 
+  SAPDB_MemCopyNoCheck( m_sqlca.sqlstate, 
           &m_reply_packet->sp1_segm().sp1r_sqlstate(),
           sizeof(tsp00_SqlState) );
-  memcpy( &warnset,
+  SAPDB_MemCopyNoCheck( &warnset,
           &m_reply_packet->sp1_segm().sp1s_segm_header().sp1r_extern_warning(),
           sizeof(warnset) );
   if (warnset[0] != 0) {
@@ -1138,7 +1148,7 @@ void SQL_SessionContext::setRtError( tgg00_BasisError e, const char * p1,
   const char * msg = getErrorDesc(e);
   m_sqlca.sqlcode = e;
   m_sqlca.sqlerrml  = (short)strlen(msg);
-  memcpy( m_sqlca.sqlerrmc, msg, m_sqlca.sqlerrml );
+  SAPDB_MemCopyNoCheck( m_sqlca.sqlerrmc, msg, m_sqlca.sqlerrml );
 
   if (p1) {
     if ((m_sqlca.sqlerrml + 1 + strlen(p1)) < sizeof(m_sqlca.sqlerrmc)) {
@@ -1172,7 +1182,7 @@ void SQL_SessionContext::setRtError( tgg00_BasisError e, const char * p1,
 void SQL_SessionContext::setError( tgg00_BasisError e, const char * msg ) {
   m_sqlca.sqlcode = e;
   m_sqlca.sqlerrml = (short)strlen(msg);
-  memcpy( m_sqlca.sqlerrmc, msg, m_sqlca.sqlerrml );
+  SAPDB_MemCopyNoCheck( m_sqlca.sqlerrmc, msg, m_sqlca.sqlerrml );
 }
 
 /*----------------------------------------------------------------------*/
@@ -1189,7 +1199,7 @@ void SQL_SessionContext::setErrortext( int argCount, int len,
     if (sizeof(m_sqlca.sqlerrmc) <= len) {
       len = sizeof(m_sqlca.sqlerrmc) - 2;
     }
-    memcpy( m_sqlca.sqlerrmuc, data, len );
+    SAPDB_MemCopyNoCheck( m_sqlca.sqlerrmuc, data, len );
     m_sqlca.sqlerrml  = len / 2;
     sp81UCS2toASCII( (tsp00_Byte*)(m_sqlca.sqlerrmc),
                      sizeof(m_sqlca.sqlerrmc)-1, &converted,
@@ -1200,7 +1210,7 @@ void SQL_SessionContext::setErrortext( int argCount, int len,
       len = sizeof(m_sqlca.sqlerrmc) - 1;
     }
     m_sqlca.sqlerrml = len;
-    memcpy( m_sqlca.sqlerrmc, data, len );
+    SAPDB_MemCopyNoCheck( m_sqlca.sqlerrmc, data, len );
     sp81ASCIItoUCS2( (tsp81_UCS2Char*)(m_sqlca.sqlerrmuc), len , UCS2Swap,
                      &converted, (const unsigned char*)data, len );
   }
@@ -1428,7 +1438,7 @@ void SQL_SessionContext::resetMethodCallEpilog() {
 /*----- PTS 1120794 ----------------------------------------------------*/
 
 void SQL_SessionContext::setCurrMethod(unsigned char * name) {
-  memcpy( &m_curr_method[0], name, sizeof(tsp00_KnlIdentifier) );
+  SAPDB_MemCopyNoCheck( &m_curr_method[0], name, sizeof(tsp00_KnlIdentifier) );
 }
 
 /*----- PTS 1120794 ----------------------------------------------------*/
@@ -1442,7 +1452,7 @@ void SQL_SessionContext::resetCurrMethod() {
 bool SQL_SessionContext::getMethodCallMonitorInfo(SqlMonitorData & info) const {
   if (m_inMethodCallEpilog) {
     m_monitor.GetCallStatistics(info);
-    memcpy( &info.m_MethodName[0], m_curr_method[0], sizeof(info.m_MethodName) );
+    SAPDB_MemCopyNoCheck( &info.m_MethodName[0], m_curr_method[0], sizeof(info.m_MethodName) );
     return true;
   }
   return false;
@@ -1462,3 +1472,69 @@ int SQL_SessionContext::getRequestPacketSize() {
   return m_request_packet_size;
 }
 
+/*----- PTS 1130628 ----------------------------------------------------*/
+
+const SQL_Statement * SQL_SessionContext::getSqlStmtFromDirectory(const tin02_ParseId * parseId) {
+  SQL_Statement * stmt = NULL;
+  const tin02_ParseId * parseId2 = NULL;
+  int len = 0;
+
+  for ( cgg251dclIterator<SQL_Statement*,SessionHeap> iter = m_CSqlStmtDir.begin();
+        iter; ++iter ) {
+    stmt = *iter();
+    len = stmt->getParsid(&parseId2);
+    if (memcmp( parseId, parseId2, len )) {
+      break;
+    }
+  }
+
+  return stmt;
+}
+
+#if !defined (EXT_CLIENT_SQL)
+
+/*----- PTS 1130628 ----------------------------------------------------*/
+
+int SQL_SessionContext::getCurrMethodAscii( char * name, int & len ) {
+  int rc = 0;
+  char * name2 = name;
+  int len2 = 0;
+
+  if (m_hasUnicodeCatalog) {
+    tsp00_Uint4           destBytesWritten;
+    tsp00_Uint4           srcBytesParsed;
+    tsp78ConversionResult error;
+    error = sp78convertString( sp77encodingAscii,            /* destEndcoding           */
+                               name,                         /* destBuffer              */
+                               len,                          /* destBufferLengthInBytes */
+                               &destBytesWritten,            /* destBytesWritten        */
+                               false,                        /* addZeroTerminator       */
+                               sp77encodingUCS2,             /* srcEncoding             */
+                               &m_curr_method[0],            /* srcBuffer               */
+                               sizeof(tsp00_KnlIdentifier),  /* srcBufferLengthInBytes  */
+                               &srcBytesParsed );            /* srcBytesParsed          */
+    if (sp78_Ok == error) {
+      len = 0;
+      len2 = destBytesWritten;
+      for ( int i = 0; i < len2; i++ ) {
+        if (' ' == *name2) {
+          break;
+        }
+        else {
+          len++;
+          name2++;
+        }
+      }
+      rc = 1;
+    }
+  }
+  else {
+    rc = 1;
+  }
+
+  return rc;
+
+}
+
+# endif
+
