@@ -69,12 +69,18 @@
 #endif
 
 #include "hsp81.h"
+#include "SAPDBCommon/SAPDB_MemCopyMove.hpp"
 
 /* PTS 1119960 */
 #ifndef HSP41_H
 #include "hsp41.h"
 #endif
 
+/* PTS 1130628 */
+#ifndef HSP77_H
+#include "hsp77.h"
+#endif
+
 
 /************************************************************************/
 /* Auto pointer                                                         */
@@ -519,7 +525,7 @@ int SQL_Statement::addResCntParm( PIn_Part * part, int swi, int cnt ) {
 
 int SQL_Statement::addInputParms(PIn_Part * part) {
   int rc = 1;
-  int i;
+  int i, prmNo;
 
   m_sqlda.fix_buf_len = 0;
   if (m_sqlda.has_long_in_col) {
@@ -547,14 +553,23 @@ int SQL_Statement::addInputParms(PIn_Part * part) {
     }
   }
 
-  for ( i = 0; (i < m_sqlda.sqln) && rc; i++ ) {
+  /* PTS 1130628, writing extended error info in case of VCHAR_ASCII violation */
+  for ( i = 0; (i < m_sqlda.sqln) && (rc > 0); i++ ) {
+    prmNo = i + 1;
     SqlCol * var = m_sqlda.sqlvar + i;
     if ( var->sqlInOut() == sp1io_input ||
          var->sqlInOut() == sp1io_inout ) {
       rc = var->m_desc->addParmToBuffer( part, m_session_context,
-                                         *this, i + 1 );
+                                         *this, prmNo );
     }
   }
+# if !defined (EXT_CLIENT_SQL)
+    if (rc < 0) {
+      dumpBadAsciidataInfo( prmNo, 0, -rc );
+      rc = 0;
+    }
+# endif
+
   return rc;
 }
 
@@ -1161,6 +1176,8 @@ int SQL_Statement::getOutputParms( int argCount,
                                    const teo00_Byte * dataPtr ) {
   int rc = 1;
   SQL_LongDesc * longDescPtr = NULL;
+  int i, prmNo;
+
 // BEGIN: MassLong
 //  // PTS 1119960
 //  if (m_mass_cmd || 
@@ -1169,10 +1186,12 @@ int SQL_Statement::getOutputParms( int argCount,
 //    m_fetch_count = 1;
 //    m_max_fetch_count = argCount;
 //    m_data_ptr = (teo00_Byte*)m_session_context.allocate(dataLen);  //malloc(dataLen);
-//    memcpy( m_data_ptr, dataPtr, dataLen );
+//    SAPDB_MemCopyNoCheck( m_data_ptr, dataPtr, dataLen );
 //  }
 // END: MassLong
-  for ( int i = 0; (i < m_sqlda.sqln) && rc; i++) {
+  /* PTS 1130628, writing extended error info in case of VCHAR_ASCII violation */
+  for ( i = 0; (i < m_sqlda.sqln) && (rc > 0); i++) {
+    prmNo = i + 1;
     SqlCol * var = m_sqlda.sqlvar + i;
     switch (var->m_desc->sfi.sp1i_data_type) {
       case dstra :
@@ -1190,9 +1209,16 @@ int SQL_Statement::getOutputParms( int argCount,
     if ( var->sqlInOut() == sp1io_output ||
          var->sqlInOut() == sp1io_inout ) {
       rc = var->m_desc->getParmFromBuffer( dataPtr, m_session_context,
-                                           *this, i + 1 );
+                                           *this, prmNo );
     }
   }
+# if !defined (EXT_CLIENT_SQL)
+    if (rc < 0) {
+      dumpBadAsciidataInfo( prmNo, 0, -rc );
+      rc = 0;
+    }
+# endif
+
   return rc;
 }
 
@@ -1208,7 +1234,7 @@ int SQL_Statement::getOutputParmsMass( int argCount, int dataLen,
   if (NULL == m_data_ptr) {
     m_data_ptr = (teo00_Byte*)m_session_context.allocate(m_session_context.getRequestPacketSize());
   }
-  memcpy( m_data_ptr, dataPtr, dataLen );
+  SAPDB_MemCopyNoCheck( m_data_ptr, dataPtr, dataLen );
 
   if (m_sqlda.has_long_out_col) { // (hasLongOutput()) {
     m_longdesc_ptr = 
@@ -1246,7 +1272,7 @@ int SQL_Statement::getOutputParmsMass( int argCount, int dataLen,
       for ( j = 0; j < m_sqlda.has_long_out_col; j++ ) {
         offset = i*m_sqlda.has_long_out_col + j;
         longDescPtr = m_longdesc_ptr + offset;
-        memcpy( &(longDescPtr->dsc),
+        SAPDB_MemCopyNoCheck( &(longDescPtr->dsc),
                 m_data_ptr + *(longDescPos + j) + i*m_sqlda.orig_fix_buf_len,
                 sizeof(tsp00_LongDescriptor) );
         if ((char)csp_undef_byte == 
@@ -1402,7 +1428,7 @@ int SQL_Statement::getNextLongOutputParmsMass() {
       longDescSrcPtr = (tsp00_LongDescriptor*)(m_longdata_ptr + descPos);
       if (longDescSrcPtr->ld_valind() < (m_fetch_count+1) * m_sqlda.has_long_out_col) {
         longDescDestPtr = m_longdesc_ptr + longDescSrcPtr->ld_valind();
-        memcpy( &longDescDestPtr->dsc, longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
+        SAPDB_MemCopyNoCheck( &longDescDestPtr->dsc, longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
         SqlCol * var = m_sqlda.sqlvar + longDescDestPtr->colIdx;
         if ( (vm_datapart == longDescDestPtr->dsc.ld_valmode()) ||
              (vm_alldata  == longDescDestPtr->dsc.ld_valmode()) ||
@@ -1469,21 +1495,21 @@ int SQL_Statement::prepare(const pasbool parsingAgain) {
 /*----------------------------------------------------------------------*/
 
 void SQL_Statement::setParsid( int len, const tin02_ParseId * data ) {
-  memcpy( &m_parseid[0], data, sizeof(m_parseid) );
+  SAPDB_MemCopyNoCheck( &m_parseid[0], data, sizeof(m_parseid) );
   m_parseid_len = len;
 }
 
 /*----------------------------------------------------------------------*/
 
 void SQL_Statement::setParsidOfSelect( int len, const tin02_ParseId * data ) {
-  memcpy( &m_parseid_of_select[0], data, sizeof(m_parseid_of_select) );
+  SAPDB_MemCopyNoCheck( &m_parseid_of_select[0], data, sizeof(m_parseid_of_select) );
   m_parseid_len = len;
 }
 
 /*----------------------------------------------------------------------*/
 
 void SQL_Statement::setResultTablename( int len, const char * data ) {
-  memcpy( &m_res_tabname[0], data, len );
+  SAPDB_MemCopyNoCheck( &m_res_tabname[0], data, len );
   m_res_tabname[len] = '\0';
 }
 
@@ -1501,7 +1527,7 @@ void SQL_Statement::setSqlDA_sqln(int argCount) {
 void SQL_Statement::setSqlDA_sfi( int i, const tsp1_param_info * sfi ) {
   SqlCol * var = m_sqlda.sqlvar + i;
   if (var->m_desc) {
-    memcpy( &var->m_desc->sfi, sfi, sizeof(*sfi) );
+    SAPDB_MemCopyNoCheck( &var->m_desc->sfi, sfi, sizeof(*sfi) );
     switch (var->m_desc->sfi.sp1i_data_type) {
       case dstra :
       case dstrb :
@@ -1533,7 +1559,7 @@ void SQL_Statement::setSqlDA_sfi( short i, int jx, int jlen,
                                   const tsp1_param_info * sfi ) {
   SqlCol * var = m_sqlda.sqlvar + jx + i;
     if (var->m_desc) {
-      memcpy( &(var->m_desc->sfi), sfi, sizeof(*sfi) );
+      SAPDB_MemCopyNoCheck( &(var->m_desc->sfi), sfi, sizeof(*sfi) );
       var->m_desc->sfi.sp1i_bufpos += jlen;
       switch (var->m_desc->sfi.sp1i_data_type) {
         case dstra :
@@ -1697,7 +1723,7 @@ int SQL_Statement::getLongDesc( int argCount, int dataLen,
     longDescSrcPtr = (unsigned char *)(dataPtr + i*(sizeof(tsp00_LongDescriptor)+1) + 1);
     longDescDestPtr = m_sqlda.longDesc + *(longDescSrcPtr +
                                            OFFSET(tsp00_LongDescriptor,ld_used_in_ak()));
-    memcpy( &(longDescDestPtr->dsc), longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
+    SAPDB_MemCopyNoCheck( &(longDescDestPtr->dsc), longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
   }
   m_sqlda.fix_buf_len = argCount * (1 + sizeof(tsp00_LongDescriptor));
   return 1;
@@ -1715,7 +1741,7 @@ int SQL_Statement::getLongDescMass( int argCount, int dataLen,
   for (int i = 0; i < argCount; i++) {
     longDescSrcPtr = (unsigned char *)(dataPtr + i * (sizeof(tsp00_LongDescriptor) + 1) + 1);
     longDescDestPtr = m_sqlda.longDesc + *( longDescSrcPtr +  OFFSET(tsp00_LongDescriptor,ld_used_in_ak()) );
-    memcpy( &longDescDestPtr->dsc, longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
+    SAPDB_MemCopyNoCheck( &longDescDestPtr->dsc, longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
   }
   m_sqlda.fix_buf_len = argCount * (1 + sizeof(tsp00_LongDescriptor));
 
@@ -1736,7 +1762,7 @@ int SQL_Statement::getLongDescAndValue( int argCount, int dataLen,
     longDescSrcPtr  = (unsigned char *)(dataPtr + descPos);
     longDescDestPtr = m_sqlda.longDesc +
                       *(longDescSrcPtr + OFFSET(tsp00_LongDescriptor,ld_used_in_ak()));
-    memcpy( &(longDescDestPtr->dsc), longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
+    SAPDB_MemCopyNoCheck( &(longDescDestPtr->dsc), longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
     SqlCol * var = m_sqlda.sqlvar + longDescDestPtr->colIdx;
     if ( (vm_datapart == longDescDestPtr->dsc.ld_valmode()) ||
          (vm_alldata  == longDescDestPtr->dsc.ld_valmode()) ||
@@ -1769,7 +1795,7 @@ int SQL_Statement::getLongDescAndValueMass( int argCount, int dataLen,
     longDescSrcPtr = (tsp00_LongDescriptor*)(dataPtr + descPos);
     if (longDescSrcPtr->ld_valind() < (m_fetch_count+1) * m_sqlda.has_long_out_col) {
       longDescDestPtr = m_longdesc_ptr + longDescSrcPtr->ld_valind();
-      memcpy( &(longDescDestPtr->dsc), longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
+      SAPDB_MemCopyNoCheck( &(longDescDestPtr->dsc), longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
       SqlCol * var = m_sqlda.sqlvar + longDescDestPtr->colIdx;
       if ( (vm_datapart == longDescDestPtr->dsc.ld_valmode()) ||
            (vm_alldata  == longDescDestPtr->dsc.ld_valmode()) ||
@@ -1794,7 +1820,7 @@ int SQL_Statement::getLongDescAndValueMass( int argCount, int dataLen,
     if (NULL == m_longdata_ptr) {
       m_longdata_ptr = (teo00_Byte*)m_session_context.allocate(m_session_context.getRequestPacketSize());
     }
-    memcpy( m_longdata_ptr, dataPtr, dataLen );
+    SAPDB_MemCopyNoCheck( m_longdata_ptr, dataPtr, dataLen );
     m_longdata_buf  = true;
     m_longdesc_pos  = descPos;
     m_longdesc_cnt  = argCount;
@@ -2000,6 +2026,97 @@ int SQL_Statement::release() {
   }
 }
 
+/*----- PTS 1130628 ----------------------------------------------------*/
+
+# if !defined (EXT_CLIENT_SQL)
+
+void SQL_Statement::dumpBadAsciidataInfo( int prmNo, int offset, int pos ) {
+  const char * stmtText = NULL;
+  int stmtLen      = 0;
+  int remLen       = 0;
+  const int outLen = 29;
+  char name[32];
+  int nameLen = sizeof(name);
+  char buffer[256];
+  int bufLen = sizeof(buffer);
+
+  if (m_unicode) {
+    return;
+  }
+  if (!(m_session_context.getSqlHandle())) {
+    return;
+  }
+  if (!(m_session_context.getSqlHandle()->getSinkPtr())) {
+    return;
+  }
+
+  memset( buffer, '\0', sizeof(buffer) );
+  sp77sprintf( buffer, bufLen, "DbpError %d", m_session_context.sqlCode() );
+  m_session_context.getSqlHandle()->dbpOpError(buffer);
+
+  memset( buffer, '\0', sizeof(buffer) );
+  strcpy( buffer, "-- " );
+  strcat( buffer, m_session_context.getErrorDesc(e_nonascii_char) );
+  strcat( buffer, " --" );
+  m_session_context.getSqlHandle()->dbpOpError(buffer);
+
+  memset( buffer, '\0', sizeof(buffer) );
+  memset( name, '\0', nameLen );
+  m_session_context.getCurrMethodAscii( name, nameLen );
+  strcat( buffer, "in " );
+  strncat( buffer, name, nameLen );
+  m_session_context.getSqlHandle()->dbpOpError(buffer);
+
+  if ( getParsidInfo1() == 42 || getParsidInfo1() == 43 ) {
+    const tin02_ParseId * parseId;
+    getParsid(&parseId);
+    const SQL_Statement * selStmt = m_session_context.getSqlStmtFromDirectory(parseId);
+    stmtText = selStmt->m_statement.c;
+  }
+  else {
+    stmtText = m_statement.c;
+  }
+  stmtLen = strlen(stmtText);
+  memset( buffer, '\0', sizeof(buffer) );
+  strcat( buffer, "Statement: " );
+  if ( stmtLen < outLen  ) {
+    strncat( buffer, stmtText, stmtLen );
+    m_session_context.getSqlHandle()->dbpOpError(buffer);
+  }
+  else {
+    strncat( buffer, stmtText, outLen );
+    m_session_context.getSqlHandle()->dbpOpError(buffer);
+    remLen = stmtLen - outLen;
+    while ( remLen > 0 ) {
+      memset( buffer, '\0', sizeof(buffer) );
+      strcat( buffer, "           " );
+      if ( remLen < outLen ) {
+        strncat( buffer, stmtText+(stmtLen-remLen), remLen );
+      }
+      else {
+        strncat( buffer, stmtText+(stmtLen-remLen), outLen );
+      }
+      m_session_context.getSqlHandle()->dbpOpError(buffer);
+      remLen = remLen - outLen;
+    }
+  }
+
+  memset( buffer, '\0', sizeof(buffer) );
+  sp77sprintf( buffer, bufLen, "Parameter: %d", prmNo );
+  m_session_context.getSqlHandle()->dbpOpError(buffer);
+
+  memset( buffer, '\0', sizeof(buffer) );
+  sp77sprintf( buffer, bufLen, "Position : %d", pos );
+  m_session_context.getSqlHandle()->dbpOpError(buffer);
+
+  memset( buffer, '\0', sizeof(buffer) );
+  unsigned char c = *((char*)((m_sqlda.sqlvar + offset + prmNo - 1)->m_desc->vfi.vaddr) + pos - 1);
+  sp77sprintf( buffer, bufLen, "Value (x): %x", c );
+  m_session_context.getSqlHandle()->dbpOpError(buffer);
+}
+
+# endif
+
 /*----------------------------------------------------------------------*/
 
 SqlCol & SQL_Statement::operator[] (int i) {
