@@ -1024,7 +1024,7 @@ void DBMCli_Buffer< TYPE >::Resize( int nSize )
     
     // this will eventually make a shallow copy, but this ok since
     // the old buffer will be deleted
-    memcpy( pNewData, m_pData, sizeof( TYPE ) * m_nSize );
+    SAPDB_memcpy( pNewData, m_pData, sizeof( TYPE ) * m_nSize );
 
     // delete old buffer (no destructors called)
     delete [ ] ( BUFFER_BYTE* ) m_pData;
@@ -1049,7 +1049,7 @@ void DBMCli_Buffer< TYPE >::ForceResize( int nSize )
     
     // this will eventually make a shallow copy, but this ok since
     // the old buffer will be deleted
-    memcpy( pNewData, m_pData, sizeof( TYPE ) * m_nSize );
+    SAPDB_memcpy( pNewData, m_pData, sizeof( TYPE ) * m_nSize );
 
     // delete old buffer (no destructors called)
     delete [ ] ( BUFFER_BYTE* ) m_pData;
@@ -1063,7 +1063,7 @@ void DBMCli_Buffer< TYPE >::ForceResize( int nSize )
     
     // this will eventually make a shallow copy, but this ok since
     // the old buffer will be deleted
-    memcpy( pNewData, m_pData, sizeof( TYPE ) * nSize );
+    SAPDB_memcpy( pNewData, m_pData, sizeof( TYPE ) * nSize );
 
     // destruct unnecessary elements
     DestructElements( m_pData + nSize, m_nSize - nSize );
@@ -1081,34 +1081,34 @@ void DBMCli_Buffer< TYPE >::ForceResize( int nSize )
 // Better versions for intrinsics
 
 inline void CopyElements( char* pDest, const char* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( char ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( char ) ); }
 inline void CopyElements( short* pDest, const short* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( short ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( short ) ); }
 inline void CopyElements( int* pDest, const int* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( int ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( int ) ); }
 inline void CopyElements( long* pDest, const long* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( long ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( long ) ); }
 inline void CopyElements( float* pDest, const float* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( float ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( float ) ); }
 inline void CopyElements( double* pDest, const double* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( double ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( double ) ); }
 inline void CopyConstructElements( char* pDest, const char* pSrc, int nCount )
-  {  memcpy( pDest, pSrc, nCount * sizeof( char ) ); }
+  {  SAPDB_memcpy( pDest, pSrc, nCount * sizeof( char ) ); }
 inline void CopyConstructElements( short* pDest, const short* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( short ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( short ) ); }
 inline void CopyConstructElements( int* pDest, const int* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( int ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( int ) ); }
 inline void CopyConstructElements( long* pDest, const long* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( long ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( long ) ); }
 inline void CopyConstructElements( float* pDest, const float* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( float ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( float ) ); }
 inline void CopyConstructElements( double* pDest, const double* pSrc, int nCount )
-  { memcpy( pDest, pSrc, nCount * sizeof( double ) ); }
+  { SAPDB_memcpy( pDest, pSrc, nCount * sizeof( double ) ); }
 
 inline DBMCli_Buffer< char >& DBMCli_Buffer< char >::operator=( const DBMCli_Buffer< char >& rOther )
 {
   Realloc( rOther.m_nSize );
-  memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( char ) );
+  SAPDB_memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( char ) );
 
   return *this;
 }
@@ -1116,7 +1116,7 @@ inline DBMCli_Buffer< char >& DBMCli_Buffer< char >::operator=( const DBMCli_Buf
 inline DBMCli_Buffer< short >& DBMCli_Buffer< short >::operator=( const DBMCli_Buffer< short >& rOther )
 {
   Realloc( rOther.m_nSize );
-  memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( char ) );
+  SAPDB_memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( char ) );
 
   return *this;
 }
@@ -1124,7 +1124,7 @@ inline DBMCli_Buffer< short >& DBMCli_Buffer< short >::operator=( const DBMCli_B
 inline DBMCli_Buffer< int >& DBMCli_Buffer< int >::operator=( const DBMCli_Buffer< int >& rOther )
 {
   Realloc( rOther.m_nSize );
-  memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( int ) );
+  SAPDB_memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( int ) );
 
   return *this;
 }
@@ -1132,7 +1132,7 @@ inline DBMCli_Buffer< int >& DBMCli_Buffer< int >::operator=( const DBMCli_Buffe
 inline DBMCli_Buffer< long >& DBMCli_Buffer< long >::operator=( const DBMCli_Buffer< long >& rOther )
 {
   Realloc( rOther.m_nSize );
-  memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( long ) );
+  SAPDB_memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( long ) );
 
   return *this;
 }
@@ -1140,7 +1140,7 @@ inline DBMCli_Buffer< long >& DBMCli_Buffer< long >::operator=( const DBMCli_Buf
 inline DBMCli_Buffer< float >& DBMCli_Buffer< float >::operator=( const DBMCli_Buffer< float >& rOther )
 {
   Realloc( rOther.m_nSize );
-  memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( float ) );
+  SAPDB_memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( float ) );
 
   return *this;
 }
@@ -1148,7 +1148,7 @@ inline DBMCli_Buffer< float >& DBMCli_Buffer< float >::operator=( const DBMCli_B
 inline DBMCli_Buffer< double >& DBMCli_Buffer< double >::operator=( const DBMCli_Buffer< double >& rOther )
 {
   Realloc( rOther.m_nSize );
-  memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( double ) );
+  SAPDB_memcpy( m_pData, rOther.m_pData, rOther.m_nSize * sizeof( double ) );
 
   return *this;
 }
@@ -1579,8 +1579,8 @@ void DBMCli_Array< TYPE, ARG_TYPE >::InsertAt( int nIndex, ARG_TYPE newElement,
     DestructElements( ( TYPE* ) m_data + m_nSize, nCount );
 
     // We can make shallow copies, because the old elements will be overwritten.
-    // The memory blocks may overlap so use memmove!
-    memmove( ( TYPE* ) m_data + nIndex + nCount, ( TYPE* ) m_data + nIndex, ( m_nSize - nIndex ) * sizeof( TYPE ) );  
+    // The memory blocks may overlap so use SAPDB_memmove!
+    SAPDB_memmove( ( TYPE* ) m_data + nIndex + nCount, ( TYPE* ) m_data + nIndex, ( m_nSize - nIndex ) * sizeof( TYPE ) );  
 
     // Copyconstruct the new elements
     CopyConstructElements( ( TYPE* ) m_data + nIndex, newElement, nCount );
@@ -1606,8 +1606,8 @@ void DBMCli_Array< TYPE, ARG_TYPE >::InsertAt( int nStartIndex, DBMCli_Array* pN
     DestructElements( ( TYPE* ) m_data + m_nSize, pNewArray->m_nSize );
 
     // We can make shallow copies, because the old elements will be overwritten.
-    // The memory blocks may overlap so use memmove!
-    memmove( ( TYPE* ) m_data + nStartIndex + pNewArray->m_nSize, ( TYPE* ) m_data + nStartIndex, ( m_nSize - nStartIndex ) * sizeof( TYPE ) );  
+    // The memory blocks may overlap so use SAPDB_memmove!
+    SAPDB_memmove( ( TYPE* ) m_data + nStartIndex + pNewArray->m_nSize, ( TYPE* ) m_data + nStartIndex, ( m_nSize - nStartIndex ) * sizeof( TYPE ) );  
 
     // Copyconstruct the new elements
     CopyConstructElements( ( TYPE* ) m_data + nStartIndex, ( TYPE* ) pNewArray->m_data, pNewArray->m_nSize );
@@ -1627,8 +1627,8 @@ void DBMCli_Array< TYPE, ARG_TYPE >::RemoveAt( int nIndex, int nCount )
   DestructElements( ( TYPE* ) m_data + nIndex, nCount );
 
   // We can make shallow copies, because the old elements will no longer be used.
-  // The memory blocks may overlap so use memmove!
-  memmove( ( TYPE* ) m_data + nIndex, ( TYPE* ) m_data + nIndex + nCount, ( m_nSize - nIndex - 1 ) * sizeof( TYPE ) );
+  // The memory blocks may overlap so use SAPDB_memmove!
+  SAPDB_memmove( ( TYPE* ) m_data + nIndex, ( TYPE* ) m_data + nIndex + nCount, ( m_nSize - nIndex - 1 ) * sizeof( TYPE ) );
 
   // Reinit unused elements
   ConstructElements( ( TYPE* ) m_data + m_nSize - nCount, nCount );
