@@ -309,9 +309,6 @@ SAPDB_Bool DBMCli_User :: Put ( const DBMCli_String & sDisabled,
     sCmd += " ";
   }
 
-  sCmd += PROP_DBMUSR;
-  sCmd += "=YES ";
-
   if( !sDisabled.IsEmpty( ) ) {
     sCmd += PROP_DISABL;
     sCmd += "=" + sDisabled; 
