@@ -415,8 +415,8 @@ int DBMSrvBHist_Backup::HasContentOlderThan(const char * Date, const char *Time)
     char ThisDate[20];
     char ThisTime[20];
 
-    strcpy(ThisDate, "00000101");
-    strcpy(ThisTime, "000000");
+    SAPDB_strcpy(ThisDate, "00000101");
+    SAPDB_strcpy(ThisTime, "000000");
 
     if(IsAData() || IsAPages())   
         source=ColumnArray[DBStamp1Column];
@@ -597,7 +597,7 @@ bool DBMSrvBHist_Backup::PrintBackupInto(char                                *&
         for(h=Columns.begin(); h!=Columns.end(); ++h)
             AppendFilledUpPart(Buffer, ColumnArray[(*h)], ColumnWidths.GiveWidthOf(*h), MaxUsableBufferSize, DBMSrvBHist_Backup_ColumnAlignment[(*h)]);
 
-        strcpy(Buffer, LINE_SEPSTRING_CN00);
+        SAPDB_strcpy(Buffer, LINE_SEPSTRING_CN00);
         Buffer+=strlen(LINE_SEPSTRING_CN00);
         MaxUsableBufferSize-=strlen(LINE_SEPSTRING_CN00);
     }
