@@ -182,7 +182,7 @@ tcn00_Error DBMSrvBHist_History::BackupHistoryOpen(VControlDataT * vcontrol,
     // look for switch -e
     Param[0]='\0';
 
-    while(cn90GetToken(command->args, Param, NParam) && !WithEBIDs)
+    while(cn90GetToken(command->args, Param, NParam, 256) && !WithEBIDs)
     {
         if(0==cn90Stricmp(Param, DBMSrvBHist_History_LISTPARAM_EBIDS))  //-e
             CheckEBIDs=true;
@@ -257,7 +257,7 @@ tcn00_Error DBMSrvBHist_History::BackupHistoryList(VControlDataT * vcontrol,
         ParamString[0]='\0';
 
         // read the list parameters
-        while(OK_CN00==rc && cn90GetToken(command->args, ParamString, CurParam))
+        while(OK_CN00==rc && cn90GetToken(command->args, ParamString, CurParam, 256))
         {
             if('\0'!=ParamString[0])
             {
@@ -521,7 +521,7 @@ tcn00_Error DBMSrvBHist_History::AnalyzeLogBackup(tcn003_BackupResult * Result)
         // compute backup timestamp
         if(20>strlen(Result->szDBStamp1Date)+strlen(Result->szDBStamp1Time+2))
         {
-            strcpy(szStamp, Result->szDBStamp1Date);
+            SAPDB_strcpy(szStamp, Result->szDBStamp1Date);
             strcat(szStamp, Result->szDBStamp1Time+2);
         }
         else
