@@ -391,47 +391,47 @@ void DBMSrvBHist_Part::UnsetAdded()
     PartAdded=0;
 }
 
-int DBMSrvBHist_Part::IsALog()
+int DBMSrvBHist_Part::IsALog() const
 {
     return (0==strncmp(DBMSrvBHist_Part_LBLLOG, Label, strlen(DBMSrvBHist_Part_LBLLOG)));
 }
 
-int DBMSrvBHist_Part::IsAReplace()
+int DBMSrvBHist_Part::IsAReplace() const
 {
     return (0==strcmp(MediumKind, DBMSrvBHist_Part_MEDIUMKIND_REPLACE));
 }
 
-int DBMSrvBHist_Part::IsNewFormat()
+int DBMSrvBHist_Part::IsNewFormat() const
 {
     return ('\0'!=MediumLocation[0]); //parts in new format must have a non empty location 
 }
 
-int DBMSrvBHist_Part::IsAdded()
+int DBMSrvBHist_Part::IsAdded() const
 {
     return PartAdded;
 }
 
-int DBMSrvBHist_Part::HasSameKeyAndLabelAs(const DBMSrvBHist_Part &OtherPart)
+int DBMSrvBHist_Part::HasSameKeyAndLabelAs(const DBMSrvBHist_Part &OtherPart) const
 {
     return (0==strcmp(Key, OtherPart.Key) && 0==strcmp(Label, OtherPart.Label));
 }
 
-int DBMSrvBHist_Part::HasSameEBIDAs(const DBMSrvBHist_Part &OtherPart)
+int DBMSrvBHist_Part::HasSameEBIDAs(const DBMSrvBHist_Part &OtherPart) const
 {
     return (0==strcmp(EBID, OtherPart.EBID));
 }
 
-const char * DBMSrvBHist_Part::GiveKey()
+const char * DBMSrvBHist_Part::GiveKey() const
 {
     return Key;
 }
 
-const char * DBMSrvBHist_Part::GiveLabel()
+const char * DBMSrvBHist_Part::GiveLabel() const
 {
     return Label;
 }
 
-tcn35_BackupToolConnectorEnum DBMSrvBHist_Part::GiveUsedBackupTool()
+tcn35_BackupToolConnectorEnum DBMSrvBHist_Part::GiveUsedBackupTool() const
 {
     tcn35_BackupToolConnectorEnum rc;
 
@@ -443,22 +443,22 @@ tcn35_BackupToolConnectorEnum DBMSrvBHist_Part::GiveUsedBackupTool()
     return rc;
 }
 
-int DBMSrvBHist_Part::GivePartNumber()
+int DBMSrvBHist_Part::GivePartNumber() const
 {
     return PartNumber;
 }
 
-int DBMSrvBHist_Part::GiveDBMReturnCode()
+int DBMSrvBHist_Part::GiveDBMReturnCode() const
 {
     return DBMReturnCode;
 }
 
-const char * DBMSrvBHist_Part::GiveDBMReturnText()
+const char * DBMSrvBHist_Part::GiveDBMReturnText() const
 {
     return DBMReturnText;
 }
 
-const char * DBMSrvBHist_Part::GiveEBID()
+const char * DBMSrvBHist_Part::GiveEBID() const
 {
     return EBID;
 }
@@ -468,6 +468,46 @@ const char * DBMSrvBHist_Part::GiveMediumName() const
     return MediumName;
 }
 
+const char * DBMSrvBHist_Part::getBackupDateTime() const
+{
+    return BackupDateTime;
+}
+
+const char * DBMSrvBHist_Part::getBackupType() const
+{
+    return BackupType;
+}
+
+const char * DBMSrvBHist_Part::getMediumBlockSize() const
+{
+    return MediumBlockSize;
+}
+
+const char * DBMSrvBHist_Part::getMediumKind() const
+{
+    return MediumKind;
+}
+
+const char * DBMSrvBHist_Part::getMediumLocation() const
+{
+    return MediumLocation;
+}
+
+const char * DBMSrvBHist_Part::getMediumOverwrite() const
+{
+    return MediumOverwrite;
+}
+
+const char * DBMSrvBHist_Part::getMediumSize() const
+{
+    return MediumSize;
+}
+
+const char * DBMSrvBHist_Part::getMediumType() const
+{
+    return MediumType;
+}
+
 bool DBMSrvBHist_Part::PrintMediumInto(char                                    *& Buffer,
                                        size_t                                   & MaxUsableBufferSize,
                                        const DBMSrvBHist_PartMediaColumnWidths  & ColumnWidths)
@@ -490,7 +530,7 @@ bool DBMSrvBHist_Part::PrintMediumInto(char                                    *
     {
         Full=false;
 
-        strcpy(Buffer, "M:");
+        SAPDB_strcpy(Buffer, "M:");
         MaxUsableBufferSize-=strlen(Buffer);
         Buffer+=strlen(Buffer);
 
@@ -502,7 +542,7 @@ bool DBMSrvBHist_Part::PrintMediumInto(char                                    *
         AppendFilledUpPart(Buffer, MediumKind,      ColumnWidths.GiveWidthOf(MediumKindColumn),      MaxUsableBufferSize, DBMSrvBHist_Part_ColumnAlignment[MediumKindColumn]);
         AppendFilledUpPart(Buffer, MediumLocation,  ColumnWidths.GiveWidthOf(MediumLocationColumn),  MaxUsableBufferSize, DBMSrvBHist_Part_ColumnAlignment[MediumLocationColumn]);
 
-        strcpy(Buffer, LINE_SEPSTRING_CN00);
+        SAPDB_strcpy(Buffer, LINE_SEPSTRING_CN00);
         Buffer+=strlen(LINE_SEPSTRING_CN00);
         MaxUsableBufferSize-=strlen(LINE_SEPSTRING_CN00);
     }
@@ -553,7 +593,7 @@ bool DBMSrvBHist_Part::PrintEBIDInto(char *& Buffer, size_t & MaxUsableBufferSiz
         if(WithExtendedInfo)
             Buffer[0]='\0';         //make sure the strcat's work
         else
-            strcpy(Buffer, "E:");   // "E:"
+            SAPDB_strcpy(Buffer, "E:");   // "E:"
 
         switch(Status)              // <status>
         {
