@@ -68,12 +68,12 @@ tcn00_Error DBMSrvBHist_RestorePlanner::AutomaticRecovery(VControlDataT * vcontr
                 FirstLogPage=DBMSrvBHist_History_NOLOG;
     int         Restartable=0;
 
-    UntilDate[0]='\0';//strcpy(UntilDate, "99991231"); //init to maximal until date
-    UntilTime[0]='\0';//strcpy(UntilTime, "235959");   //init to maximal hour
+    UntilDate[0]='\0';//SAPDB_strcpy(UntilDate, "99991231"); //init to maximal until date
+    UntilTime[0]='\0';//SAPDB_strcpy(UntilTime, "235959");   //init to maximal hour
     IBID[0]='\0';
 
     //analyse command parameters ([LOG], [UNTIL <date> <time>], [IBID <int_backup_ID>])
-    while(OK_CN00==rc && cn90GetToken(command->args, szParam, nParam))
+    while(OK_CN00==rc && cn90GetToken(command->args, szParam, nParam, 255))
     {
         if(0==cn90Stricmp(szParam, "LOG"))
             LogOnly=1;
@@ -81,8 +81,8 @@ tcn00_Error DBMSrvBHist_RestorePlanner::AutomaticRecovery(VControlDataT * vcontr
             if(0==cn90Stricmp(szParam, BA_KEY_UNTIL_CN003))
             {
                 // until time is only valid for log restore
-                if(cn90GetToken(command->args, UntilDate, nParam+1) &&
-                   cn90GetToken(command->args, UntilTime, nParam+2)   )
+                if(cn90GetToken(command->args, UntilDate, nParam+1, 255) &&
+                   cn90GetToken(command->args, UntilTime, nParam+2, 255)   )
                 {
                     nParam+=2;
                 }
@@ -92,7 +92,7 @@ tcn00_Error DBMSrvBHist_RestorePlanner::AutomaticRecovery(VControlDataT * vcontr
             else
                 if(0==cn90Stricmp(szParam, BA_KEY_IBID_CN003))
                 {
-                    if(cn90GetToken(command->args, IBID, nParam+1))
+                    if(cn90GetToken(command->args, IBID, nParam+1, 255))
                         ++nParam;
                     else
                         rc=cn90AnswerIError(replyData, replyLen, ERR_PARAM_CN00);
@@ -100,12 +100,12 @@ tcn00_Error DBMSrvBHist_RestorePlanner::AutomaticRecovery(VControlDataT * vcontr
                 else
                     if(0==cn90Stricmp(szParam, BA_KEY_KEY_CN003))
                     {
-                        if(cn90GetToken(command->args, Key, nParam+1) &&
-                           cn90GetToken(command->args, szParam, nParam+2) &&
+                        if(cn90GetToken(command->args, Key, nParam+1, 255) &&
+                           cn90GetToken(command->args, szParam, nParam+2,255) &&
                            0==cn90Stricmp(szParam, BA_KEY_LABEL_CN003) &&
-                           cn90GetToken(command->args, Label, nParam+3)      )
+                           cn90GetToken(command->args, Label, nParam+3, 255)      )
                         {
-                            strcpy(IBID, Key);
+                            SAPDB_strcpy(IBID, Key);
                             strcat(IBID, "|");
                             strcat(IBID, Label);
 
@@ -504,7 +504,7 @@ tcn00_Error DBMSrvBHist_RestorePlanner::RestoreABackup(const DBMSrvBHist_Restore
         {
             if(0!=cn36_StrUprCmp(Backup->getBackup()->GiveBackupType(), LastBackupType))
             {
-                strcpy(LastBackupType, Backup->getBackup()->GiveBackupType());
+                SAPDB_strcpy(LastBackupType, Backup->getBackup()->GiveBackupType());
 
                 rc=cn31BackupRestoreRequest(vcontrol,
                                             replyData,
@@ -952,7 +952,7 @@ tcn00_Error DBMSrvBHist_RestorePlanner::DBOnline(VControlDataT * vcontrol,
             rc=cn90AnswerIError(replyData, replyLen, ERR_MEM_CN00);
         else
         {
-            strcpy(Arguments, "-u ");
+            SAPDB_strcpy(Arguments, "-u ");
             strcat(Arguments, UntilDate);
             strcat(Arguments, " ");
             strcat(Arguments, UntilTime);
