@@ -8,7 +8,7 @@
 \if EMIT_LICENCE
 
     ========== licence begin  GPL
-    Copyright (C) 2002-2004 SAP AG
+    Copyright (c) 2002-2004 SAP AG
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -32,6 +32,7 @@
 //-----------------------------------------------------------------------------
 // includes
 //-----------------------------------------------------------------------------
+#include <errno.h>
 
 #include "SAPDB/DBM/Srv/DBMSrv_Reply.hpp"
 #include "hcn36.h"
@@ -45,25 +46,53 @@ const size_t DummyBufferSize_DBMSrv_Reply=1204;
 char         DummyBuffer_DBMSrv_Reply[DummyBufferSize_DBMSrv_Reply]; //<! a small dummy buffer, that is always available
 int          DummyBufferLength;
 
-
 //-----------------------------------------------------------------------------
 // members of class DBMSrv_Reply
 //-----------------------------------------------------------------------------
 
+//-----------------------------------------------------------------------------
+// public static members
+//-----------------------------------------------------------------------------
+const char * DBMSrv_Reply::STRING_OK              = ANSWER_OK_CN00;
+const char * DBMSrv_Reply::STRING_ERR             = ANSWER_ERR_CN00;
+const char * DBMSrv_Reply::STRING_SEPARATOR_LINE  = LINE_SEPSTRING_CN00;
+const char * DBMSrv_Reply::STRING_SEPARATOR_TOKEN = TOKEN_SEPSTRING_CN00;
+const char * DBMSrv_Reply::STRING_CONTINUE        = "CONTINUE";
+const char * DBMSrv_Reply::STRING_END             = "END";
+
+//-----------------------------------------------------------------------------
+// public constructor
+//-----------------------------------------------------------------------------
 DBMSrv_Reply::DBMSrv_Reply()
-  :replyData(DummyBuffer_DBMSrv_Reply),
-   replyLength(&DummyBufferLength),
-   maximalReplyLength(DummyBufferSize_DBMSrv_Reply)
+  :replyData(defaultBuffer),
+   constReplyData(defaultBuffer),
+   replyLength(&defaultLength),
+   maximalReplyLength(1), //space for terminating zero byte
+   defaultLength(0)
 {
+    defaultBuffer[0]='\0';
 }
 
 DBMSrv_Reply::DBMSrv_Reply(char * ReplyData,
                            int  * ReplyLength,
                            int    ReplyLenMax)
   :replyData(ReplyData),
+   constReplyData(ReplyData),
    replyLength(ReplyLength),
-   maximalReplyLength(ReplyLenMax)
+   maximalReplyLength(ReplyLenMax),
+   defaultLength(0)
 {
+    defaultBuffer[0]='\0';
+}
+
+DBMSrv_Reply::DBMSrv_Reply(const char * constantReplyData)
+    :replyData(0),                          //by seting replyData to 0, only 'const' functions will work properly
+     constReplyData(constantReplyData),
+     replyLength(&defaultLength),
+     maximalReplyLength(1),                 //to achieve an available space of 0 and thereby avoid any changes to the buffer
+     defaultLength(0)
+{
+    defaultBuffer[0]='\0';
 }
 
 void DBMSrv_Reply::setTo(char * ReplyData,
@@ -71,6 +100,7 @@ void DBMSrv_Reply::setTo(char * ReplyData,
                          int    ReplyLenMax)
 {
     replyData=ReplyData;
+    constReplyData=ReplyData;
     replyLength=ReplyLength;
     maximalReplyLength=ReplyLenMax;
 }
@@ -82,7 +112,7 @@ char * DBMSrv_Reply::giveData()
 
 const char * DBMSrv_Reply::giveData() const
 {
-    return replyData;
+    return constReplyData;
 }
 
 int * DBMSrv_Reply::giveLength()
@@ -92,7 +122,7 @@ int * DBMSrv_Reply::giveLength()
 
 void DBMSrv_Reply::adaptLength()
 {
-    *replyLength=(int)strlen(replyData);
+    *replyLength=(int)strlen(constReplyData);
 }
 
 int DBMSrv_Reply::giveMaximalLength() const
@@ -105,15 +135,56 @@ int DBMSrv_Reply::giveAvailableSpace() const
     return maximalReplyLength-1-(*replyLength);
 }
 
+DBMSrv_Reply::Position DBMSrv_Reply::getEndOfFirstLine() const
+{
+    return getEndOfLine(constReplyData);
+}
+
+DBMSrv_Reply::Position DBMSrv_Reply::getEndOfLine(Position positionInLine) const
+{
+    Position end=strstr(positionInLine, DBMSrv_Reply::STRING_SEPARATOR_LINE);
+
+    return (0==end?positionInLine+strlen(end):end);
+}
+
+DBMSrv_Reply::Position DBMSrv_Reply::getStartOfNextLine(Position positionInLine) const
+{
+    Position pos=getEndOfLine(positionInLine);
+
+    if('\0'!=(*pos))    //if there is a line break
+        pos+=strlen(DBMSrv_Reply::STRING_SEPARATOR_LINE);
+    
+    return pos;
+}
+
+void DBMSrv_Reply::getStartAndEndOfLine(unsigned int lineNumber, Position & start, Position & end) const
+{
+    start=constReplyData; //start at the beginning of the buffer
+
+    while(0<lineNumber) //skip all previous lines
+    {
+        start=getStartOfNextLine(start);
+        lineNumber--;
+    }
+
+    end=getEndOfLine(start);    //determine end of current line
+}
+
+void DBMSrv_Reply::getStartAndEndOfNextLine(Position positionInLine, Position & start, Position & end) const
+{
+    start=getStartOfNextLine(positionInLine);
+    end=getEndOfLine(start);
+}
+
 void DBMSrv_Reply::startWithOK()
 {
-    startWithLine(ANSWER_OK_CN00);
+    startWithLine(DBMSrv_Reply::STRING_OK);
 }
 
 void DBMSrv_Reply::startWithOKIfRCIsOk(tcn00_Error rc)
 {
     if(OK_CN00==rc)
-        startWithLine(ANSWER_OK_CN00);
+        startWithLine(DBMSrv_Reply::STRING_OK);
 }
 
 tcn00_Error DBMSrv_Reply::startWithError(const tcn00_Error Error,
@@ -137,32 +208,32 @@ tcn00_Error DBMSrv_Reply::startWithError(const tcn00_Error Error,
         i++;
 
     //write the error text to the reply buffer
-    startWithLine((Error == OK_CN00) ? ANSWER_OK_CN00 : ANSWER_ERR_CN00);
+    startWithLine((Error == OK_CN00) ? DBMSrv_Reply::STRING_OK : DBMSrv_Reply::STRING_ERR);
     sprintf(NumberBuf, "%d", (int)Error); //50 chars should be sufficient
-    appendStrings(NumberBuf, TOKEN_SEPSTRING_CN00);
+    appendStrings(NumberBuf, DBMSrv_Reply::STRING_SEPARATOR_TOKEN);
     appendLine(errorArray[i].Text);
 
     if(0!=ExtErrorText && 0<ExtErrorTextLength)
     {
         sprintf(NumberBuf, "%d", (int)ExtError); //50 chars should be sufficient
-        appendStrings(NumberBuf, TOKEN_SEPSTRING_CN00);
+        appendStrings(NumberBuf, DBMSrv_Reply::STRING_SEPARATOR_TOKEN);
         appendLineN(ExtErrorText, ExtErrorTextLength);
     }
 
     if(0!=ExtError2Text)
     {
         sprintf(NumberBuf, "%d", (int)ExtError2); //50 chars should be sufficient
-        appendStrings(NumberBuf, TOKEN_SEPSTRING_CN00);
+        appendStrings(NumberBuf, DBMSrv_Reply::STRING_SEPARATOR_TOKEN);
         appendLine(ExtError2Text);
     }
 
     return Error;
 }
 
-tcn00_Error DBMSrv_Reply::startWithSQLError(    tcn00_Error  Error,
-                                                const char*  SQLMessage,
-                                                const int    SQLError) {
-    
+tcn00_Error DBMSrv_Reply::startWithSQLError(tcn00_Error  Error,
+                                            const char*  SQLMessage,
+                                            const int    SQLError)
+{    
     return startWithError(Error, SQLError, SQLMessage, strlen(SQLMessage));
 }
 
@@ -177,17 +248,105 @@ tcn00_Error DBMSrv_Reply::startWithRTEError(tcn00_Error  Error,
     return startWithError(Error, RTEError, RTEMessage, MessageLength);
 }
 
+//-----------------------------------------------------------------------------
+// public member function DBMSrv_Reply::startWithErrno
+//-----------------------------------------------------------------------------
+tcn00_Error DBMSrv_Reply::startWithErrno (  tcn00_Error  Error,
+                                            const int    nErrno,
+                                            const int    nExtError,  
+                                            const char * szExtError )
+{
+  // the errno array
+  // The available errnos are different on different platforms. Therefore
+  // exists a ifdef confidition for every errno.
+  struct {
+    int          nValue;
+    const char * szText;
+  }               errnoArray[] = {
+#ifdef E2BIG
+  { E2BIG     , "E2BIG : argument list to long"             },
+#endif
+#ifdef EACCES
+  { EACCES    , "EACCES : permission denied"                },
+#endif
+#ifdef EAGAIN
+  { EAGAIN    , "EAGAIN : No more processes or not enough memory or maximum nesting level reached" },
+#endif
+#ifdef EBADF
+  { EBADF     , "EBADF : bad file number"                   },
+#endif
+#ifdef ECHILD
+  { ECHILD    , "ECHILD : no spawned processes"             },
+#endif
+#ifdef EDEADLOCK
+  { EDEADLOCK , "EDEADLOCK : resource deadlock would occur" },
+#endif
+#ifdef EDOM
+  { EDOM      , "EDOM : math argument"                      },
+#endif
+#ifdef EEXIST
+  { EEXIST    , "EEXIST : file exist"                       },
+#endif
+#ifdef EINVAL
+  { EINVAL    , "EIVAL : invalid argument"                  },
+#endif
+#ifdef EMFILE
+  { EMFILE    , "EMFILE : too many open files"              },
+#endif
+#ifdef ENOENT
+  { ENOENT    , "ENOENT : no such file or directory"        },
+#endif
+#ifdef ENOEXEC
+  { ENOEXEC   , "ENOEXEC : exec format error"               },
+#endif
+#ifdef ENOMEM
+  { ENOMEM    , "ENOMEM : not enough memory"                },
+#endif
+#ifdef ENOSPC
+  { ENOSPC    , "ENOSPC : no space left on device"          },
+#endif
+#ifdef ERANGE
+  { ERANGE    , "ERANGE : result too large"                 },
+#endif
+#ifdef EXDEV
+  { EXDEV     , "EXDEV : cross-device link"                 },
+#endif
+  { 0 , NULL       },
+  { 0 , "unknown errno" }};
+
+  char NumberBuf[50];
+
+  // DBMServer error
+  this->startWithError(Error);
+
+  // errno error
+  int i;
+  for (i = 0; (errnoArray[i].szText != NULL) && (nErrno != errnoArray[i].nValue); ++i);
+  i = (errnoArray[i].szText == NULL) ? i + 1 : i;
+  sprintf(NumberBuf, "%d", (int)nErrno); //50 chars should be sufficient
+  appendStringsAndLine(NumberBuf, DBMSrv_Reply::STRING_SEPARATOR_TOKEN, errnoArray[i].szText);
+  
+  // addtional error
+  if(szExtError != NULL)
+  {
+    sprintf(NumberBuf, "%d", (int)nExtError); //50 chars should be sufficient
+    appendStringsAndLine(NumberBuf, DBMSrv_Reply::STRING_SEPARATOR_TOKEN, szExtError);
+  }
+
+  return Error;
+} // end DBMSrv_Reply::startWithErrno
+
 tcn00_Error DBMSrv_Reply::startWithEventList(const teo200_EventList & EventList)
 {
     const teo200_EventList * Event=&EventList;
     char NumberBuf[50];
   
-    startWithLine(EventList.eo200_EventID()==cn00_1_ErrId(OK_CN00_1) ? ANSWER_OK_CN00 : ANSWER_ERR_CN00);
+    startWithLine(EventList.eo200_EventID()==cn00_1_ErrId(OK_CN00_1) ? DBMSrv_Reply::STRING_OK : DBMSrv_Reply::STRING_ERR);
 
     while(0!=Event)
     {
         sprintf(NumberBuf, "%d", (int)Event->eo200_EventID()); //50 chars should be sufficient
-        appendStrings(NumberBuf, TOKEN_SEPSTRING_CN00);    
+        appendStrings(NumberBuf, DBMSrv_Reply::STRING_SEPARATOR_TOKEN);    
         appendLine(Event->eo200_EventMsg());
 
         Event=Event->eo200_NextEvent();
@@ -201,12 +360,12 @@ tcn00_Error DBMSrv_Reply::startWithMessageList(const SAPDBErr_MessageList &MsgLi
     const SAPDBErr_MessageList* Msg=&MsgList;
     char NumberBuf[50];
   
-    startWithLine(Msg->ID()==cn00_1_ErrId(OK_CN00_1) ? ANSWER_OK_CN00 : ANSWER_ERR_CN00);
+    startWithLine(Msg->ID()==cn00_1_ErrId(OK_CN00_1) ? DBMSrv_Reply::STRING_OK : DBMSrv_Reply::STRING_ERR);
 
     while(0!=Msg)
     {
         sprintf(NumberBuf, "%d", (int)Msg->ID()); //50 chars should be sufficient
-        appendStrings(NumberBuf, TOKEN_SEPSTRING_CN00);    
+        appendStrings(NumberBuf, DBMSrv_Reply::STRING_SEPARATOR_TOKEN);    
         appendLine(Msg->Message());
 
         Msg=Msg->NextMessage();
@@ -223,19 +382,19 @@ void DBMSrv_Reply::startWithLine(const char * Line)
 
 void DBMSrv_Reply::appendLine()
 {
-    appendString(LINE_SEPSTRING_CN00);
+    appendString(DBMSrv_Reply::STRING_SEPARATOR_LINE);
 }
 
 void DBMSrv_Reply::appendLine(const char * Line)
 {
     appendString(Line);
-    appendString(LINE_SEPSTRING_CN00);
+    appendString(DBMSrv_Reply::STRING_SEPARATOR_LINE);
 }
 
 void DBMSrv_Reply::appendLineN(const char * Line, size_t LineSize)
 {
     appendStringN(Line, LineSize);
-    appendString(LINE_SEPSTRING_CN00);
+    appendString(DBMSrv_Reply::STRING_SEPARATOR_LINE);
 }
 
 void DBMSrv_Reply::appendLines(const char * Line1, const char * Line2)
@@ -324,6 +483,15 @@ void DBMSrv_Reply::appendIndentedWrappedLineN(const char   * LineStart,
     }
 }
 
+//-----------------------------------------------------------------------------
+// public member function DBMSrv_Reply::startWithString
+//-----------------------------------------------------------------------------
+void DBMSrv_Reply::startWithString ( const char * String )
+{
+    *replyLength=0;
+    appendString(String);
+} // end DBMSrv_Reply::startWithString
+
 void DBMSrv_Reply::appendString(const char *String)
 {
     if( (maximalReplyLength-1) > (*replyLength) ) //is there space to add something and a terminating zero?
@@ -383,6 +551,14 @@ void DBMSrv_Reply::appendStringsAndLine(const char *String1, const char *String2
     appendLine(Line3);
 }
 
+void DBMSrv_Reply::appendStringsAndLine(const char * string1, const char * string2, const char * string3, const char * line)
+{
+    appendString(string1);
+    appendString(string2);
+    appendString(string3);
+    appendLine(line);
+}
+
 void DBMSrv_Reply::appendStringsStringNAndLine(const char   * String1,
                                                const char   * String2,
                                                const char   * String3,
@@ -417,8 +593,11 @@ void DBMSrv_Reply::appendStringNWithMinWidth(const char *String, size_t StringLe
 
 void DBMSrv_Reply::appendChar(const char Character)
 {
-    replyData[(*replyLength)++]=Character;
-    replyData[*replyLength]='\0';
+    if( (maximalReplyLength-1) > (*replyLength) ) //is there space to add something and a terminating zero?
+    {
+        replyData[(*replyLength)++]=Character;
+        replyData[*replyLength]='\0';
+    }
 }
 
 void DBMSrv_Reply::appendCharacterRepeatedly(const char Character, size_t NumberOfAppends)
@@ -426,10 +605,18 @@ void DBMSrv_Reply::appendCharacterRepeatedly(const char Character, size_t Number
     if(NumberOfAppends+(*replyLength) >= (size_t)maximalReplyLength)
        NumberOfAppends=maximalReplyLength-1-(*replyLength);
 
-    memset(replyData+(*replyLength), Character, NumberOfAppends);
+    if(0<NumberOfAppends)
+    {
+        memset(replyData+(*replyLength), Character, NumberOfAppends);
 
-    (*replyLength)+=(int)NumberOfAppends;
-    replyData[*replyLength]='\0';
+        (*replyLength)+=(int)NumberOfAppends;
+        replyData[*replyLength]='\0';
+    }
+}
+
+DBMSrv_Reply::Position DBMSrv_Reply::giveCurrentEnd() const
+{
+    return constReplyData+strlen(constReplyData);
 }
 
 char * DBMSrv_Reply::giveCurrentEnd()
@@ -441,9 +628,11 @@ void DBMSrv_Reply::skipAt(Position SkipPosition)
 {
     size_t SkipLength=SkipPosition-replyData;
 
-    replyData[SkipLength]='\0';
-
-    (*replyLength)=(int)strlen(replyData);
+    if(SkipLength<(size_t)maximalReplyLength && 0!=replyData)
+    {
+        replyData[SkipLength]='\0';
+        (*replyLength)=(int)strlen(replyData);
+    }
 }
 
 void DBMSrv_Reply::insertString(Position InsertPosition, const char * String)
@@ -452,7 +641,8 @@ void DBMSrv_Reply::insertString(Position InsertPosition, const char * String)
     size_t StringLength=strlen(String);
     
     if( size_t(maximalReplyLength) > InsPos+StringLength && //is there enough space to insert the string and a terminating zero?
-        size_t(*replyLength) > InsPos                      )//inserting in front of the terminating zero?
+        size_t(*replyLength) > InsPos                    && //inserting in front of the terminating zero?
+        0!=replyData )                                      //insert only into changeable strings
     {
         strncpy(replyData+InsPos, String, StringLength); //copy the string without the terminating zero
 
@@ -464,6 +654,54 @@ void DBMSrv_Reply::insertString(Position InsertPosition, const char * String)
         //else nothing to do, old reply from InsertPosition was already longer than inserted string
     }
 }
+
+void DBMSrv_Reply::appendStartOfHSSNodeAnswerFor(const char * standbyName)
+{
+    appendStringAndLine("node: ", standbyName);
+}
+
+void DBMSrv_Reply::appendMessageListForHSSNode(const SAPDBErr_MessageList & messageList)
+{
+    const SAPDBErr_MessageList* message=&messageList;
+
+    char numberBuf[50];
+
+    while(0!=message)
+    {
+        sprintf(numberBuf, "%d", (int)message->ID()); //50 chars should be sufficient
+        appendStringsAndLine("    ", numberBuf, DBMSrv_Reply::STRING_SEPARATOR_TOKEN, message->Message());    
+
+        message=message->NextMessage();
+    }
+}
+
+void DBMSrv_Reply::appendOKForHSSNode()
+{
+    appendStringAndLine("    ", DBMSrv_Reply::STRING_OK);
+}
+
+bool DBMSrv_Reply::startsWithOKAndContinueFlag() const
+{
+    bool rc=false;
+
+    const char * firstLineStart=constReplyData;
+    const char * firstLineEnd=getEndOfFirstLine();
+
+    if(firstLineEnd-firstLineStart==strlen(STRING_OK) && 0==strncmp(DBMSrv_Reply::STRING_OK, firstLineStart, strlen(STRING_OK)))
+    {
+        const char * secondLineStart=getStartOfNextLine(firstLineEnd);
+        const char * secondLineEnd  =getEndOfLine(secondLineStart);
+
+        if(secondLineEnd-secondLineStart==strlen(DBMSrv_Reply::STRING_CONTINUE) &&
+           strncmp(DBMSrv_Reply::STRING_CONTINUE, secondLineStart, strlen(DBMSrv_Reply::STRING_CONTINUE)))
+        {
+            rc=true;
+        }
+    }
+
+    return rc;
+}
+
 size_t DBMSrv_Reply::calculateLengthOfNextWord(const char *WordStart, size_t MaxWordLength, const char * LineBreakString)
 {
     size_t rc=cn36_FirstWhiteSpaceOf(WordStart, MaxWordLength)-WordStart;
