@@ -8,7 +8,7 @@
 \if EMIT_LICENCE
 
     ========== licence begin  GPL
-    Copyright (C) 2002-2004 SAP AG
+    Copyright (c) 2002-2004 SAP AG
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -48,23 +48,33 @@ class DBMSrv_Reply
 {
   public:
 
-    typedef const char * Position;
+    typedef const char * Position; //!< a data type for storing positions within the reply
     /*! constructor
 
         initialising a reply object without a real reply buffer, before using
-        such an object SetTo() must be called */
+        such an object it is useful to call SetTo() */
     DBMSrv_Reply();
 
     /*! constructor
 
         initialising a reply object with the needed information
-        \param ReplyData   [IN] a pointer to the replay buffer that shall contain the reply
+        \param ReplyData   [IN] a pointer to the reply buffer that shall contain the reply
         \param ReplyLength [IN] a pointer to the length of the current reply
         \param ReplyLenMax [IN] the maximal allowed reply length */
     DBMSrv_Reply(char * ReplyData,
                  int  * ReplyLength,
                  int    ReplyLenMax);
 
+    /*! constructor
+
+        initialising a readonly reply object with the needed information, only const functions
+        can be used with this object
+        \param constantReplyData  [IN] a pointer to the existing constant reply buffer */
+    DBMSrv_Reply(const char * constantReplyData);
+
+    /*! destructor */
+    virtual ~DBMSrv_Reply() {}
+
     /*! function for changing the managed reply buffer
         \param ReplyData   [IN] a pointer to the new replay buffer that shall contain the reply
         \param ReplyLength [IN] a pointer to the new length of the current reply
@@ -73,6 +83,12 @@ class DBMSrv_Reply
                int  * ReplyLength,
                int    ReplyLenMax);
 
+    /*! function for changing the managed reply buffer to a global available dummy
+
+        the result is undefined, if more then one reply object is using the global dummy buffer
+        at the same time */
+    void setToGlobalDummy();
+
     /*! function for obtaining the reply buffer
 
         This function should be avoided whenever possible, because it's the long term goal of this class
@@ -99,9 +115,9 @@ class DBMSrv_Reply
         This function determines the string length of the current zero terminated
         reply in the buffer and sets the reply length to that value.
         This function must be used whenever the reply buffer is manipulated
-        directly with the help of the pointer returned by the Data() function.
+        directly with the help of the pointer returned by the 'char * giveData()' function.
         When all operations on the reply buffers are capsulated in this class,
-        the AdaptLength() function is no longer needed.
+        the adaptLength() function is no longer needed.
         \todo eliminate all occurences of this function when Data() is no longer used either*/
     void adaptLength();
 
@@ -113,6 +129,32 @@ class DBMSrv_Reply
         \return number of unused characters of the reply buffer*/
     int giveAvailableSpace() const;
 
+    /*! function for obtaining the end of the first line of the reply
+        \return the position of the first character behind the first line (pointer to line separator or terminating zero) */
+    Position getEndOfFirstLine() const;
+
+    /*! function for obtaining the end of the line containing a certain Position
+        \param  positionInLine [IN] the position in the reply buffer to start searching from
+        \return the position of the first character behind the line (pointer to line separator or terminating zero) */
+    Position getEndOfLine(Position positionInLine) const;
+
+    /*! function for obtaining the first character of the next line
+        \param  positionInLine [IN] the position in the reply buffer to start searching from for end of line and next line
+        \return the position of the first character of the next line, or 0 pointer if there is no next line */
+    Position getStartOfNextLine(Position positionInLine) const;
+
+    /*! function for obtaining the start and end of a certain line of the reply
+        \param lineNumber [IN]  number of the line to search for
+        \param start      [OUT] first character of the line, equals end, if line is empty
+        \param end        [OUT] first character behind the line (pointer to line separator or terminating zero) */        
+    void getStartAndEndOfLine(unsigned int lineNumber, Position & start, Position & end) const;
+
+    /*! function for obtaining the start and end of the next complete line following a certain position in the reply
+        \param positionInLine [IN]  the position in the reply buffer to start searching from for end of line and next line
+        \param start          [OUT] first character of the line, equals end, if line is empty
+        \param end            [OUT] first character behind the line (pointer to line separator or terminating zero) */        
+    void getStartAndEndOfNextLine(Position positionInLine, Position & start, Position & end) const;
+
     /*! function for obtaining the ReturnCode
         \return the current return code of the DBMServer command*/
     //tcn00_Error   ReturnCode() const;
@@ -130,7 +172,12 @@ class DBMSrv_Reply
     void startWithOKIfRCIsOk(tcn00_Error rc);
 
     /*! function for reporting a tcn00_Error as reply
-        \param Error [IN] DBMServer error to report
+        \param Error              [IN] DBMServer error to report
+        \param ExtError           [IN] the number of a second error to report
+        \param ExtErrorText       [IN] text of the second error to report
+        \param ExtErrorTextLength [IN] length of the text of the second error
+        \param ExtError2          [IN] the number of a third error to report
+        \param ExtError2Text      [IN] zero terminated text of the third error to report
         \return copy of Error */
     tcn00_Error startWithError(const tcn00_Error Error,
                                int               ExtError          =0,
@@ -152,9 +199,20 @@ class DBMSrv_Reply
                                   int          RTEMessageLength,
                                   int          RTEError);
 
+    /*! function for reporting a errno as reply
+        \param Error            [IN] DBMServer error to report
+        \param nErrno           [IN] errno to report
+        \param nExtError        [IN] additional error code
+        \param szExtError       [IN] additional error text
+        \return copy of Error */
+    tcn00_Error startWithErrno (  tcn00_Error  Error,
+                                  const int    nErrno,
+                                  const int    nExtError  = 0,  
+                                  const char * szExtError = NULL );
+
     /*! function for reporting an SQLError as reply
         \param Error            [IN] DBMServer error to report
-        \param SQLMessage       [IN] SQLMessage to report (must contain trailing '\0')
+        \param SQLMessage       [IN] SQLMessage to report (must contain trailing '\\0')
         \param SQLError         [IN] SQL error code
         \return copy of Error
         \todo check maximal allowed reply length (!) */
@@ -208,7 +266,7 @@ class DBMSrv_Reply
         output is indented and a maximal line size is forcing
         additional line breaks at word boundaries, if need.
         \param LineStart  [IN] start of the string, containing the words
-        \param LineLength [IN] length of tha string
+        \param LineLength [IN] length of the string
         \param Indent     [IN] number of spaces used for indent
         \param WrapAt     [IN] number of characters were a line break should happen
         \param PositionInOutputLine [IN] starting position within the first line of output*/
@@ -223,11 +281,11 @@ class DBMSrv_Reply
         output is indented and a maximal line size and a special
         line break string is forcing additional line breaks at
         word boundaries
-        \param LineStart       [IN] start of the string, containing the words
-        \param LineLength      [IN] length of tha string
-        \param Indent          [IN] number of spaces used for indent
-        \param WrapAt          [IN] number of characters were a line break should happen
-        \param LineBreakString [IN] special string for forcing line breaks
+        \param LineStart            [IN] start of the string, containing the words
+        \param LineLength           [IN] length of the string
+        \param Indent               [IN] number of spaces used for indent
+        \param WrapAt               [IN] number of characters were a line break should happen
+        \param LineBreakString      [IN] special string for forcing line breaks
         \param PositionInOutputLine [IN] starting position within the first line of output*/
     void appendIndentedWrappedLineN(const char   * LineStart,
                                     const size_t   LineLength,
@@ -236,6 +294,10 @@ class DBMSrv_Reply
                                     const char   * LineBreakString,
                                     size_t         PositionInOutputLine);
 
+    /*! function for writing the first string of a reply
+        \param String [IN] the string to start the reply with */
+    void startWithString(const char * String);
+
     /*! function for appending a string to the reply
         \param String [IN] string to add to the reply */
     void appendString(const char *String);
@@ -266,7 +328,7 @@ class DBMSrv_Reply
     /*! function for appending two strings (the last one terminated via
         length option), followed by a line break to the reply
         \param String1     [IN] first string to add to the reply
-        \param Line2       [IN] the second string to add to the reply
+        \param Line2       [IN] the second string to add to the reply, a line break will be added behind that string
         \param Line2Length [IN] the length of the second string */
     void appendStringAndLineN(const char *String1, const char *Line2, size_t Line2Length);
 
@@ -274,16 +336,24 @@ class DBMSrv_Reply
         line break to the reply
         \param String1 [IN] first string to add to the reply
         \param String2 [IN] second string to add to the reply
-        \param Line3   [IN] the third string to add to the reply */
+        \param Line3   [IN] the third string to add to the reply, a line break will be added behind that string */
     void appendStringsAndLine(const char *String1, const char *String2, const char *Line3);
 
+    /*! function for appending four strings, followed by a
+        line break to the reply
+        \param string1 [IN] first string to add to the reply
+        \param string2 [IN] second string to add to the reply
+        \param string3 [IN] third string to add to the reply
+        \param line    [IN] the fourth string to add to the reply, a line break will be added behind that string */
+    void appendStringsAndLine(const char * string1, const char * string2, const char * string3, const char * line);
+
     /*! function for appending three strings a not zero terminated string
         another string and a line break to the reply
         \param String1       [IN] first string to add to the reply
         \param String2       [IN] second string to add to the reply
         \param String3       [IN] third string to be added to the reply
         \param String4       [IN] fourth not zero terminated string to be added to the reply
-        \param Stirng4Length [IN] the length of the fourth string
+        \param String4Length [IN] the length of the fourth string
         \param Line          [IN] the fifth an last string to be added */
     void appendStringsStringNAndLine(const char   * String1,
                                      const char   * String2,
@@ -328,7 +398,7 @@ class DBMSrv_Reply
     char * giveCurrentEnd();
 
     /*! function terminating the string at Position
-        \param the position in the reply buffer, where to end the reply at*/
+        \param SkipPosition [IN] the position in the reply buffer, where to end the reply at */
     void skipAt(Position SkipPosition);
 
     /*! function for inserting a string into existing reply
@@ -336,15 +406,52 @@ class DBMSrv_Reply
         \param String         [IN] string to add to the reply */
     void insertString(Position InsertPosition, const char * String);
 
+    /*! function for starting a new "sub"-reply of another HSS node within the
+        current reply.
+
+        The "sub"-reply will be preceeded by the line "node: \<node\>", where \<node\> is substituted
+        by the string supplied as name of the HSS mode. The "sub"-reply itself will be indented
+        with 4 spaces.
+        \param standbyName [IN] name of the HSS node */
+    void appendStartOfHSSNodeAnswerFor(const char * standbyName);
+
+    /*! append a message list as "sub"-reply for a HSS node
+    
+        The "sub"-reply should have been started with appendStartOfHSSNodeAnswerFor()
+        \see appendStartOfHSSNodeAnswerFor 
+        \param messageList [IN] the message list will be appended using an indent of 4 spaces*/
+    void appendMessageListForHSSNode(const SAPDBErr_MessageList & messageList);
+
+    /*! append an "OK" indented by 4 spaces
+        The HSS node "sub"-reply should have been started with appendStartOfHSSNodeAnswerFor()
+        \see appendStartOfHSSNodeAnswerFor */
+    void appendOKForHSSNode();
+
+    /*! function for checking a reply for the continue flag
+        \return true if reply starts with a line containing OK followed
+                by a line containing CONTINUE, false otherwise */
+    bool startsWithOKAndContinueFlag() const;
+
   private:
     char        * replyData;
+    const char  * constReplyData;
     int         * replyLength;
     int           maximalReplyLength;
 
+    char defaultBuffer[1];
+    int  defaultLength;
+
     size_t calculateLengthOfNextWord(const char *WordStart, size_t MaxWordLength, const char * LineBreakString);
 
     size_t CStringLengthFromPascalString(const char * SourcePascalString,
                                          const int    PascalStringMaximalLength);
+
+    static const char * STRING_OK;
+    static const char * STRING_ERR;
+    static const char * STRING_SEPARATOR_LINE;
+    static const char * STRING_SEPARATOR_TOKEN;
+    static const char * STRING_CONTINUE;
+    static const char * STRING_END;
 };
 
 #endif
