@@ -521,12 +521,16 @@ bool DBMWeb_DBMWeb :: GetParameterValue(const char * pName,
   bool              bReturn = false;
   sapdbwa_StringSeq values; 
 
+//  sValue="";
+
   if (request.GetParameterValues(pName, values) == sapdbwa_True) {
     if (values.GetNumElem() > 0) {
-      sValue = values[0];
-      bReturn = true;
-    } 
-  }
+      if (strlen(values[0]) < 512) {
+        sValue = values[0];
+        bReturn = true;
+      } // end if
+    } // end if
+  } // end if
   return bReturn;
 } // DBMWeb_DBMWeb :: GetParameterValue
 
