@@ -454,7 +454,7 @@ Tools_DynamicUTF8String DBMWeb_TemplateBackup :: askForValue ( const Tools_Dynam
 
   pReturn = sValue;
 
-  return pReturn;
+  return _Tools_UTF8Ptr(pReturn);
 } // end DBMWeb_TemplateBackup :: askForValue
 
 /*
