@@ -302,11 +302,15 @@ SAPDB_Int2 DBMWeb_TemplateWizard :: askForWriteCountStep2 ( const Tools_DynamicU
   } else if( ISOBJ( OBJ_FORM2 ) ) {
     nReturn = 1;
   } else if( ISOBJ( OBJ_ROWPAIR ) ) {
-    nReturn = (m_Wizard->GetNode( ).EnumInstallationArray( ).GetSize( )  / 2) + 
-              (m_Wizard->GetNode( ).EnumInstallationArray( ).GetSize( ) % 2); 
+    if( m_Wizard != NULL ) {
+      nReturn = (m_Wizard->GetNode( ).EnumInstallationArray( ).GetSize( )  / 2) + 
+                (m_Wizard->GetNode( ).EnumInstallationArray( ).GetSize( ) % 2); 
+    }
   } else if (( ISOBJ( OBJ_EVENROW ) ) || 
              ( ISOBJ( OBJ_ODDROW ) )     ) {
-    nReturn = (m_nItem < m_Wizard->GetNode( ).EnumInstallationArray( ).GetSize( )) ? 1 : 0;
+    if( m_Wizard != NULL ) {
+      nReturn = (m_nItem < m_Wizard->GetNode( ).EnumInstallationArray( ).GetSize( )) ? 1 : 0;
+    }
     m_nItem++;
   }
 
@@ -343,47 +347,65 @@ SAPDB_Int2 DBMWeb_TemplateWizard :: askForWriteCountStep4 ( const Tools_DynamicU
   } else if( ISOBJ( OBJ_FORM4 ) ) {
     nReturn = 1;
   } else if( ISOBJ( OBJ_IMCUR ) ) {
-    nReturn = (m_Wizard->GetCreatedFlag( )) ? 0 : 1;
+    if( m_Wizard != NULL ) {
+      nReturn = (m_Wizard->GetCreatedFlag( )) ? 0 : 1;
+    }
   } else if( ISOBJ( OBJ_IMCURDIS ) ) {
-    nReturn = (m_Wizard->GetCreatedFlag( )) ? 1 : 0;
+    if( m_Wizard != NULL ) {
+      nReturn = (m_Wizard->GetCreatedFlag( )) ? 1 : 0;
+    }
   } else if( ISOBJ( OBJ_INITMODE1 ) ) {
-    nReturn = (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_DEF) ? 1 : 0; 
+    if( m_Wizard != NULL ) {
+      nReturn = (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_DEF) ? 1 : 0; 
+    }
   } else if( ISOBJ( OBJ_INITMODE2 ) ) {
-    nReturn = (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_CUR) ? 1 : 0; 
+    if( m_Wizard != NULL ) {
+      nReturn = (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_CUR) ? 1 : 0; 
+    }
   } else if( ISOBJ( OBJ_INITMODE3 ) ) {
-    nReturn = (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_CPY) ? 1 : 0; 
+    if( m_Wizard != NULL ) {
+      nReturn = (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_CPY) ? 1 : 0; 
+    }
   } else if( ISOBJ( OBJ_INITMODE4 ) ) {
-    nReturn = (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_RES) ? 1 : 0; 
+    if( m_Wizard != NULL ) {
+      nReturn = (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_RES) ? 1 : 0; 
+    }
   } else {
-    if (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_CPY) {
-      if( ISOBJ( OBJ_ROWPAIR ) ) {
-        nReturn = (m_Wizard->GetNode( ).EnumDatabaseArray( ).GetSize( )  / 2) + 
-                  (m_Wizard->GetNode( ).EnumDatabaseArray( ).GetSize( ) % 2); 
-      } else if (( ISOBJ( OBJ_EVENROW ) ) || 
-                 ( ISOBJ( OBJ_ODDROW ) )     ) {
-        nReturn = (m_nItem < m_Wizard->GetNode( ).EnumDatabaseArray( ).GetSize( )) ? 1 : 0;
-        m_nItem++;
-      }
-    } else if (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_RES) {
-      if( ISOBJ( OBJ_ROWPAIR ) ) {
-        nReturn = (m_Database->GetMedia( ).MediumArray( ).GetSize( ) / 2) + 
-                  (m_Database->GetMedia( ).MediumArray( ).GetSize( ) % 2); 
-      } else if (( ISOBJ( OBJ_EVENROW ) ) || 
-                 ( ISOBJ( OBJ_ODDROW ) )     ) {
-        nReturn = (m_nItem < m_Database->GetMedia( ).MediumArray( ).GetSize( )) ? 1 : 0;
-        m_nItem++;
-      } else if( ISOBJ( OBJ_TABSNG ) ) {
-        nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_SINGLE) ? 1 : 0;
-      } else if( ISOBJ( OBJ_TABPAR ) ) {
-        nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_PARALLEL) ? 1 : 0;
-      } else if( ISOBJ( OBJ_TABLESNG ) ) {
-        nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_SINGLE) ? 1 : 0;
-      } else if( ISOBJ( OBJ_TABLEPAR ) ) {
-        nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_PARALLEL) ? 1 : 0;
-      } else if( ISOBJ( OBJ_BTNADDSNG ) ) {
-        nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_SINGLE) ? 1 : 0;
-      } else if( ISOBJ( OBJ_BTNADDPAR ) ) {
-        nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_PARALLEL) ? 1 : 0;
+    if( m_Wizard != NULL ) {
+      if (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_CPY) {
+        if( ISOBJ( OBJ_ROWPAIR ) ) {
+          nReturn = (m_Wizard->GetNode( ).EnumDatabaseArray( ).GetSize( )  / 2) + 
+                    (m_Wizard->GetNode( ).EnumDatabaseArray( ).GetSize( ) % 2); 
+        } else if (( ISOBJ( OBJ_EVENROW ) ) || 
+                  ( ISOBJ( OBJ_ODDROW ) )     ) {
+          nReturn = (m_nItem < m_Wizard->GetNode( ).EnumDatabaseArray( ).GetSize( )) ? 1 : 0;
+          m_nItem++;
+        }
+      } else if (m_Wizard->GetInitMode( ) == DBMCLI_WIZARD_INITMODE_RES) {
+        if( ISOBJ( OBJ_ROWPAIR ) ) {
+          if( m_Database != NULL ) {
+            nReturn = (m_Database->GetMedia( ).MediumArray( ).GetSize( ) / 2) + 
+                      (m_Database->GetMedia( ).MediumArray( ).GetSize( ) % 2); 
+          }
+        } else if (( ISOBJ( OBJ_EVENROW ) ) || 
+                  ( ISOBJ( OBJ_ODDROW ) )     ) {
+          if( m_Database != NULL ) {
+            nReturn = (m_nItem < m_Database->GetMedia( ).MediumArray( ).GetSize( )) ? 1 : 0;
+          }
+          m_nItem++;
+        } else if( ISOBJ( OBJ_TABSNG ) ) {
+          nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_SINGLE) ? 1 : 0;
+        } else if( ISOBJ( OBJ_TABPAR ) ) {
+          nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_PARALLEL) ? 1 : 0;
+        } else if( ISOBJ( OBJ_TABLESNG ) ) {
+          nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_SINGLE) ? 1 : 0;
+        } else if( ISOBJ( OBJ_TABLEPAR ) ) {
+          nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_PARALLEL) ? 1 : 0;
+        } else if( ISOBJ( OBJ_BTNADDSNG ) ) {
+          nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_SINGLE) ? 1 : 0;
+        } else if( ISOBJ( OBJ_BTNADDPAR ) ) {
+          nReturn = (m_nMediumType == DBMCLI_MEDIUMTYPE_PARALLEL) ? 1 : 0;
+        }
       }
     }
   }
@@ -414,8 +436,10 @@ SAPDB_Int2 DBMWeb_TemplateWizard :: askForWriteCountStep5 ( const Tools_DynamicU
   } else if( ISOBJ( OBJ_TABLEHEADER ) ) {
     nReturn = 1;
   } else if( ISOBJ( OBJ_ROWPAIR ) ) {
-    DBMCli_Parameters & oParams = m_Database->GetParameters( ); 
-    nReturn = (m_nItem < oParams.ParameterArray( ).GetSize( )) ? -1 : 0; 
+    if( m_Database != NULL ) {
+      DBMCli_Parameters & oParams = m_Database->GetParameters( ); 
+      nReturn = (m_nItem < oParams.ParameterArray( ).GetSize( )) ? -1 : 0; 
+    }
   } else if (( ISOBJ( OBJ_EVENROW ) ) || 
              ( ISOBJ( OBJ_ODDROW ) )     ) {
     nReturn = (FindNextParam( ) == true) ? 1 : 0;
@@ -450,8 +474,10 @@ SAPDB_Int2 DBMWeb_TemplateWizard :: askForWriteCountStep6 ( const Tools_DynamicU
   } else if( ISOBJ( OBJ_TABLEHEADER ) ) {
     nReturn = 1;
   } else if( ISOBJ( OBJ_ROWPAIR ) ) {
-    DBMCli_Devspaces & oDevspaces = m_Database->GetDevspaces( ); 
-    nReturn = (m_nItem < oDevspaces.DevspaceArray( ).GetSize( )) ? -1 : 0; 
+    if( m_Database != NULL ) {
+      DBMCli_Devspaces & oDevspaces = m_Database->GetDevspaces( ); 
+      nReturn = (m_nItem < oDevspaces.DevspaceArray( ).GetSize( )) ? -1 : 0; 
+    }
   } else if (( ISOBJ( OBJ_EVENROW ) ) || 
              ( ISOBJ( OBJ_ODDROW ) )     ) {
     nReturn = (FindNextDevspace( ) == true) ? 1 : 0;
@@ -562,7 +588,7 @@ Tools_DynamicUTF8String DBMWeb_TemplateWizard :: askForValue ( const Tools_Dynam
   }
 
   pReturn = m_sValue;
-  return pReturn;
+  return _Tools_UTF8Ptr(pReturn);
 } 
 
 // Databasename
@@ -869,10 +895,12 @@ bool DBMWeb_TemplateWizard :: askForContinueParams ( const Tools_DynamicUTF8Stri
 {
   bool bRC = false;
   
-  DBMCli_ParameterArray & aParams = m_Database->GetParameters( ).ParameterArray( );
+  if( m_Database != NULL ) {
+    DBMCli_ParameterArray & aParams = m_Database->GetParameters( ).ParameterArray( );
 
-  if( ISOBJ(OBJ_ROWPAIR ) ) {
-    bRC = ( m_nItem < aParams.GetSize( ) );
+    if( ISOBJ(OBJ_ROWPAIR ) ) {
+      bRC = ( m_nItem < aParams.GetSize( ) );
+    }
   }
 
   return bRC;
@@ -882,13 +910,15 @@ bool DBMWeb_TemplateWizard  :: FindNextParam ( )
 {
   bool bFound = false;
 
-  DBMCli_ParameterArray & aParams = m_Database->GetParameters( ).ParameterArray( );
+  if( m_Database != NULL ) {
+    DBMCli_ParameterArray & aParams = m_Database->GetParameters( ).ParameterArray( );
 
-  while( ( m_nItem < aParams.GetSize( ) ) && ( bFound == false ) ) {
-    if( aParams[m_nItem].Group( ) == m_nParamGroup ) { 
-      bFound = true;
-    } else {
-      m_nItem++;
+    while( ( m_nItem < aParams.GetSize( ) ) && ( bFound == false ) ) {
+      if( aParams[m_nItem].Group( ) == m_nParamGroup ) { 
+        bFound = true;
+      } else {
+        m_nItem++;
+      }
     }
   }
 
@@ -899,10 +929,12 @@ bool DBMWeb_TemplateWizard :: askForContinueDevspaces ( const Tools_DynamicUTF8S
 {
   bool bRC = false;
   
-  DBMCli_DevspaceArray & aDevspaces = m_Database->GetDevspaces( ).DevspaceArray( );
+  if( m_Database != NULL ) {
+    DBMCli_DevspaceArray & aDevspaces = m_Database->GetDevspaces( ).DevspaceArray( );
 
-  if( ISOBJ( OBJ_ROWPAIR ) ) {
-    bRC = ( m_nItem< aDevspaces.GetSize( ) );
+    if( ISOBJ( OBJ_ROWPAIR ) ) {
+      bRC = ( m_nItem< aDevspaces.GetSize( ) );
+    }
   }
 
   return bRC;
@@ -912,13 +944,15 @@ bool DBMWeb_TemplateWizard :: FindNextDevspace ( )
 {
   bool bFound = false;
 
-  DBMCli_DevspaceArray & aDevspaces = m_Database->GetDevspaces( ).DevspaceArray( );
+  if( m_Database != NULL) {
+    DBMCli_DevspaceArray & aDevspaces = m_Database->GetDevspaces( ).DevspaceArray( );
 
-  while( ( m_nItem < aDevspaces.GetSize( ) ) && ( bFound == false ) ) {
-    if( aDevspaces[m_nItem].Class( ) == m_nDevspaceClass ) {
-      bFound = true;
-    } else {
-      m_nItem++;
+    while( ( m_nItem < aDevspaces.GetSize( ) ) && ( bFound == false ) ) {
+      if( aDevspaces[m_nItem].Class( ) == m_nDevspaceClass ) {
+        bFound = true;
+      } else {
+        m_nItem++;
+      }
     }
   }
 
@@ -929,14 +963,16 @@ bool DBMWeb_TemplateWizard :: FindSystemDevspace ( )
 {
   bool        bFound = false;
 
-  DBMCli_DevspaceArray & aDevspaces = m_Database->GetDevspaces( ).DevspaceArray( );
+  if( m_Database != NULL ) {
+    DBMCli_DevspaceArray & aDevspaces = m_Database->GetDevspaces( ).DevspaceArray( );
 
-  for (SAPDB_Int2  nDevspace = 0; nDevspace < aDevspaces.GetSize( ); ++nDevspace) {
-    if( aDevspaces[nDevspace].Class( ) == DBMCLI_DEVSPACECLASS_SYS ) {
-      bFound = true;
-      break;
-    } // end if
-  }  // end if
+    for (SAPDB_Int2  nDevspace = 0; nDevspace < aDevspaces.GetSize( ); ++nDevspace) {
+      if( aDevspaces[nDevspace].Class( ) == DBMCLI_DEVSPACECLASS_SYS ) {
+        bFound = true;
+        break;
+      } 
+    }  
+  }
 
   return bFound;
-} // end DBMWeb_TemplateWizard :: FindSystemDevspace
+} 
