@@ -8,7 +8,7 @@
   version:      7.4 and higher 
   -----------------------------------------------------------------------------
 
-  copyright: (c) 2002-2004 SAP AG
+  Copyright (c) 2002-2004 SAP AG
 
 
     ========== licence begin  GPL
@@ -54,11 +54,13 @@ const char * const DBA_config_Version::MinVersion = "0.0.0.0";
 
 DBA_configuration::DBA_configuration () 
                   : m_oValueProvider(m_lParameters, m_lArrays),
-                    m_oDBVersion(DBA_config_Version::MinVersion)
+                    m_oDBVersion(DBA_config_Version::MinVersion),
+                    m_nInstance(DBA_INSTANCETYPE_OLTP)
 {
   m_sFilename = "";
   m_hFile = NULL;
   m_nLine = 0;
+  m_sDate = "";
 }
 
 DBA_configuration::~DBA_configuration () {
@@ -89,7 +91,8 @@ DBA_bool DBA_configuration::Open
     DBA_int            nMajor,
     DBA_int            nMinor,
     DBA_int            nLevel,
-    DBA_int            nBuild )
+    DBA_int            nBuild,
+    DBA_int            nInstance )
 {
   DBA_bool   bRC = false;
 
@@ -97,6 +100,7 @@ DBA_bool DBA_configuration::Open
   m_sFilename = sFilename;
 
   m_oDBVersion.setVersion(nMajor, nMinor, nLevel, nBuild);
+  m_nInstance = nInstance;
 
   if (m_sFilename.Empty()) {
     DBA_string sDir = ToolsSys_getcwd(true);
@@ -290,33 +294,25 @@ DBA_string DBA_configuration::GetSetting(const DBA_string & sName)
 //
 // create the necessary views for the dbanalyzer
 //
-DBA_bool DBA_configuration::InitViews(DBA_database * pDatabase,
-                                      DBA_int        nInstanceType)
+DBA_bool DBA_configuration::InitViews(DBA_database * pDatabase)
 {
   DBA_config_view_list::iterator iter = m_lViews.begin();
   while (iter != m_lViews.end()) {
-    // check the instance type and version
-    if ((((*iter)->nInstanceType & nInstanceType) == nInstanceType) &&
-        ((*iter)->oMinVersion                     <= m_oDBVersion ) &&
-        ((*iter)->oMaxVersion                     >= m_oDBVersion )    ) {
+    // drop view
+    DBA_string sStmt = "DROP VIEW ";
+    sStmt.Append((*iter)->sName);
+    pDatabase->Execute(sStmt, NULL);
 
+    // build the statement
+    sStmt = "CREATE OR REPLACE VIEW ";
+    sStmt.Append((*iter)->sName);
+    sStmt.Append(" AS ");
+    sStmt.Append((*iter)->sQuery);
+    sStmt.Append(" INTERNAL ");
 
-      // drop view
-      DBA_string sStmt = "DROP VIEW ";
-      sStmt.Append((*iter)->sName);
-      pDatabase->Execute(sStmt, NULL);
-
-      // build the statement
-      sStmt = "CREATE OR REPLACE VIEW ";
-      sStmt.Append((*iter)->sName);
-      sStmt.Append(" AS ");
-      sStmt.Append((*iter)->sQuery);
-      sStmt.Append(" INTERNAL ");
-
-      // create view
-      if (!pDatabase->Execute(sStmt, NULL)) {
-        return false;
-      }
+    // create view
+    if (!pDatabase->Execute(sStmt, NULL)) {
+      return false;
     }
     ++iter;
   }
@@ -331,35 +327,30 @@ DBA_bool DBA_configuration::InitViews(DBA_database * pDatabase,
 // from the database or colculate the expression
 // for an environment parameter
 //
-DBA_bool DBA_configuration::InitEnvironment(DBA_database * pDatabase,
-                                            DBA_int        nInstanceType)
+DBA_bool DBA_configuration::InitEnvironment(DBA_database * pDatabase)
 {
   DBA_config_parameter_list::iterator iter = m_lParameters.begin();
   while (iter != m_lParameters.end()) {
     if ((*iter)->nType == DBA_CNFPTYPE_ENVIRONMENT) {
-      // check the instance type  and version
-      if ((((*iter)->nInstanceType & nInstanceType) == nInstanceType) &&
-          ((*iter)->oMinVersion                     <= m_oDBVersion ) &&
-          ((*iter)->oMaxVersion                     >= m_oDBVersion )    ) {
+      if ((*iter)->sFormula.Empty()) {
+        DBA_string sStmt = "SELECT ";
+        sStmt.Append((*iter)->sColumn);
+        sStmt.Append(" FROM ");
+        sStmt.Append((*iter)->sTable);
+        if (!(*iter)->sConditionColumn.Empty()) {
+          sStmt.Append(" WHERE ")
+               .Append((*iter)->sConditionColumn)
+               .Append(" = ")
+               .Append((*iter)->sConditionValue);
+        }
 
-        if ((*iter)->sFormula.Empty()) {
-          DBA_string sStmt = "SELECT ";
-          sStmt.Append((*iter)->sColumn);
-          sStmt.Append(" FROM ");
-          sStmt.Append((*iter)->sTable);
-          if (!(*iter)->sCondition.Empty()) {
-            sStmt.Append(" WHERE ");
-            sStmt.Append((*iter)->sCondition);
-          }
-
-          DBA_snapshot oSnapshot;
-          if (pDatabase->Execute(sStmt, &oSnapshot)) {
-            if (!oSnapshot.Empty()) {
-              (*iter)->SetNewValue(oSnapshot.GetField(0,0));
-            }
-          } else {
-            return false;
+        DBA_snapshot oSnapshot;
+        if (pDatabase->Execute(sStmt, &oSnapshot)) {
+          if (!oSnapshot.Empty()) {
+            (*iter)->SetNewValue(oSnapshot.GetField(0,0));
           }
+        } else {
+          return false;
         }
       }
     }
@@ -372,13 +363,12 @@ DBA_bool DBA_configuration::InitEnvironment(DBA_database * pDatabase,
 //
 // InitProtocolFiles
 //
-DBA_bool DBA_configuration::InitProtocolFiles(DBA_protocol * pProtocol,
-                                              DBA_int        nInstanceType)
+DBA_bool DBA_configuration::InitProtocolFiles(DBA_protocol * pProtocol)
 {
   DBA_config_class_list::iterator iterClass = m_lClasses.begin();
   while (iterClass != m_lClasses.end()) {
 
-    if (((*iterClass)->nInstanceType & nInstanceType) == nInstanceType) {
+    if (((*iterClass)->nInstanceType & m_nInstance) == m_nInstance) {
       if (!pProtocol->AddParameterClass((*iterClass)->sName, (*iterClass)->nFileIndex)) {
         return false;
       }
@@ -386,31 +376,25 @@ DBA_bool DBA_configuration::InitProtocolFiles(DBA_protocol * pProtocol,
       DBA_config_parameter_list::iterator iterParameter = (*iterClass)->lParameters.begin();
       DBA_string                          sType;
       while (iterParameter != (*iterClass)->lParameters.end()) {
-        // check the instance type and version
-        if ( (((*iterParameter)->nInstanceType & nInstanceType) == nInstanceType) &&
-             ((*iterParameter)->oMinVersion                     <= m_oDBVersion ) &&
-             ((*iterParameter)->oMaxVersion                     >= m_oDBVersion )    ) {
-
-          // add parameter name to header
-          if ((*iterParameter)->bProtocol) {
-            switch ((*iterParameter)->GetType()) {
-            case Tools_ExpressionValue::ExTypeNumeric:
-              sType = DBA_PRT_TYPENUMRIC;
-              break;
-            case Tools_ExpressionValue::ExTypeString:
-              sType = DBA_PRT_TYPESTRING;
-              break;
-            default:
-              sType = DBA_PRT_TYPENUMRIC;
-            } // end switch
-            if ((*iterParameter)->bImportant) {
-              sType.Append(DBA_PRT_TYPEIMPORTANT);
-            } else if ((*iterParameter)->sProtocolInfo.Length() > 0) {
-              sType.Append(" ");
-            } // end if
-            sType.Append((*iterParameter)->sProtocolInfo);
-            pProtocol->AddParameterHeader((*iterClass)->nFileIndex, (*iterParameter)->sID, sType, (*iterParameter)->sLabel);
-          }
+        // add parameter name to header
+        if ((*iterParameter)->bProtocol) {
+          switch ((*iterParameter)->GetType()) {
+          case Tools_ExpressionValue::ExTypeNumeric:
+            sType = DBA_PRT_TYPENUMRIC;
+            break;
+          case Tools_ExpressionValue::ExTypeString:
+            sType = DBA_PRT_TYPESTRING;
+            break;
+          default:
+            sType = DBA_PRT_TYPENUMRIC;
+          } // end switch
+          if ((*iterParameter)->bImportant) {
+            sType.Append(DBA_PRT_TYPEIMPORTANT);
+          } else if ((*iterParameter)->sProtocolInfo.Length() > 0) {
+            sType.Append(" ");
+          } // end if
+          sType.Append((*iterParameter)->sProtocolInfo);
+          pProtocol->AddParameterHeader((*iterClass)->nFileIndex, (*iterParameter)->sID, sType, (*iterParameter)->sLabel);
         }
         ++iterParameter;
       }
@@ -426,20 +410,28 @@ DBA_bool DBA_configuration::InitProtocolFiles(DBA_protocol * pProtocol,
 //
 DBA_bool DBA_configuration::GetSnapshot
   (DBA_database * pDatabase,
-   const DBA_bool bStartup)
+   time_t       & tSnapshot,
+   DBA_bool     & bStartup)
 {
   DBA_string     sStmt;
   DBA_string     sValue;
   DBA_snapshot * pSnapshot = NULL;
   
+  if (!pDatabase->GetTimestamp(tSnapshot)) {
+    return false;
+  } else {
+    char szDate[11];
+    strftime(szDate, 10, "%Y%m%d", localtime(&tSnapshot));
+    bStartup = (m_sDate.Compare(szDate) != 0);
+    m_sDate = szDate;
+  } // end if
+
   // select the snapshots from the database
   DBA_config_table_list::iterator iterTable = m_lTables.begin();
   while (iterTable != m_lTables.end()) {
 
-    if ( (((*iterTable)->nInstanceType & pDatabase->GetInstanceType()) == pDatabase->GetInstanceType()) &&
-         (((*iterTable)->bStartup && bStartup) || !(*iterTable)->bStartup                             ) &&
-         ((*iterTable)->oMinVersion                                    <= m_oDBVersion                ) &&
-         ((*iterTable)->oMaxVersion                                    >= m_oDBVersion                )    ) {
+    if (((((*iterTable)->nInstanceType & m_nInstance)) ==  m_nInstance    ) &&
+         (((*iterTable)->bStartup && bStartup) || !(*iterTable)->bStartup )    ) {
       sStmt = "SELECT * FROM ";
       sStmt.Append((*iterTable)->sName);
 
@@ -451,7 +443,7 @@ DBA_bool DBA_configuration::GetSnapshot
         }
       }
 
-      if (pDatabase->Execute(sStmt, (*iterTable)->pSnapshot)) {
+      if (pDatabase->Execute(sStmt, (*iterTable)->pSnapshot, (*iterTable)->pPreparedQuery)) {
         if (gError.GetReconnect() || gError.GetIgnore() ) {
           return true;
         }
@@ -466,12 +458,7 @@ DBA_bool DBA_configuration::GetSnapshot
   // assign values from snapshot to parameters
   DBA_config_parameter_list::iterator iterParameter = m_lParameters.begin();
   while (iterParameter != m_lParameters.end()) {
-    if ( (((*iterParameter)->nInstanceType & pDatabase->GetInstanceType()) == pDatabase->GetInstanceType()) &&
-          (((*iterParameter)->bStartup && bStartup) || !(*iterParameter)->bStartup                        ) &&
-          ((*iterParameter)->oMinVersion                                   <= m_oDBVersion                ) &&
-          ((*iterParameter)->oMaxVersion                                   >= m_oDBVersion                )    ) {
-
-
+    if (((*iterParameter)->bStartup && bStartup) || !(*iterParameter)->bStartup) {
 
       if ((*iterParameter)->pTable) {
         if ((*iterParameter)->pTable->pSnapshot) {
@@ -504,11 +491,7 @@ DBA_bool DBA_configuration::GetSnapshot
   DBA_string                             sKey;
   
   while (iterArray != m_lArrays.end()) {
-    if ( (((*iterArray)->nInstanceType & pDatabase->GetInstanceType()) == pDatabase->GetInstanceType()) &&
-         (((*iterArray)->bStartup && bStartup) || !(*iterArray)->bStartup                             ) &&
-         ((*iterArray)->oMinVersion                                    <= m_oDBVersion                ) &&
-         ((*iterArray)->oMaxVersion                                    >= m_oDBVersion                )    ) {
-
+    if (((*iterArray)->bStartup && bStartup) || !(*iterArray)->bStartup) {
 
       if ((*iterArray)->pTable) {
         if ((*iterArray)->pTable->pSnapshot) {
@@ -571,18 +554,14 @@ DBA_bool DBA_configuration::GetSnapshot
 // CalculateParameters
 //
 DBA_bool DBA_configuration::CalculateParameters
-  (const DBA_int nInstanceType,
-   const DBA_bool bStartup)
+  (const DBA_bool bStartup)
 {
   Tools_ExpressionValue   oValue;
 
   DBA_config_parameter_list::iterator iterParameter = m_lParameters.begin();
   while (iterParameter != m_lParameters.end()) {
     // check the instance type
-    if ( (((*iterParameter)->nInstanceType & nInstanceType) == nInstanceType ) &&
-         (((*iterParameter)->bStartup && bStartup) || !(*iterParameter)->bStartup) &&
-         ((*iterParameter)->oMinVersion                     <= m_oDBVersion  ) &&
-         ((*iterParameter)->oMaxVersion                     >= m_oDBVersion  )    ) {
+    if (((*iterParameter)->bStartup && bStartup) || !(*iterParameter)->bStartup) {
 
 
       if (!(*iterParameter)->sFormula.Empty()) {
@@ -626,8 +605,7 @@ DBA_bool DBA_configuration::CalculateParameters
 // CalculateArrays
 //
 DBA_bool DBA_configuration::CalculateArrays
-  (const DBA_int nInstanceType,
-   const DBA_bool bStartup)
+  (const DBA_bool bStartup)
 {
 
   // iterate through arrays
@@ -638,11 +616,7 @@ DBA_bool DBA_configuration::CalculateArrays
   Tools_ExpressionValue                   oValue;
 
   while (iterArr != m_lArrays.end()) {
-    // check the instance type
-    if ( (((*iterArr)->nInstanceType & nInstanceType) == nInstanceType ) &&
-         (((*iterArr)->bStartup && bStartup) || !(*iterArr)->bStartup  ) &&
-         ((*iterArr)->oMinVersion                     <= m_oDBVersion  ) &&
-         ((*iterArr)->oMaxVersion                     >= m_oDBVersion  )    ) {
+    if (((*iterArr)->bStartup && bStartup) || !(*iterArr)->bStartup) {
 
       if (!(*iterArr)->sFormula.Empty()) {
 
@@ -762,8 +736,7 @@ DBA_bool DBA_configuration::CalculateMonitorExpression
 // CalculateMonitors
 //
 DBA_bool DBA_configuration::CalculateMonitors
-  (const DBA_int  nInstanceType,
-   const DBA_bool bStartup)
+  (const DBA_bool bStartup)
 {
 
   // iterate through monitors
@@ -773,11 +746,7 @@ DBA_bool DBA_configuration::CalculateMonitors
   Tools_ExpressionValueArray                         * pArray;
   while (iterMon != m_lMonitors.end()) {
     // check the instance type
-    if (( ((*iterMon)->nInstanceType & nInstanceType) == nInstanceType ) &&
-        ( (*iterMon)->bStartup                        == bStartup      ) &&
-        ( (*iterMon)->oMinVersion                     <= m_oDBVersion  ) &&
-        ( (*iterMon)->oMaxVersion                     >= m_oDBVersion  )    ) {
-
+    if (((*iterMon)->bStartup && bStartup) || !(*iterMon)->bStartup) {
 
       // find Index
       if (!(*iterMon)->sForEach.Empty()) {
@@ -907,28 +876,33 @@ DBA_bool DBA_configuration::CalculateMonitors
 // ProtocolParameters
 //
 DBA_bool DBA_configuration::ProtocolParameters(DBA_protocol * pProtocol,
-                                               DBA_int        nInstanceType,
                                                DBA_ulong      nCount,
                                                time_t         tAct,
-                                               time_t         tLast)
+                                               time_t         tLast,
+                                               time_t         tNow)
 {
-  pProtocol->SetParameterStatus(nCount, tAct, tLast);
+  pProtocol->SetParameterStatus(nCount, tAct, tLast, tNow);
 
   DBA_config_class_list::iterator iterClass = m_lClasses.begin();
   while (iterClass != m_lClasses.end()) {
-    if (((*iterClass)->nInstanceType & nInstanceType) == nInstanceType) {
+    if (((*iterClass)->nInstanceType & m_nInstance) == m_nInstance) {
 
       pProtocol->ClearParameterValues((*iterClass)->nFileIndex);  
       DBA_config_parameter_list::iterator iterParameter = (*iterClass)->lParameters.begin();
       while (iterParameter != (*iterClass)->lParameters.end()) {
-        // !=
-        if ( (((*iterParameter)->nInstanceType & nInstanceType) == nInstanceType ) &&
-             ((*iterParameter)->oMinVersion                     <= m_oDBVersion  ) &&
-             ((*iterParameter)->oMaxVersion                     >= m_oDBVersion  )    ) {
-          // add the parameter
-          if ((*iterParameter)->bProtocol) {
+        // add the parameter
+        if ((*iterParameter)->bProtocol) {
+          if ((*iterParameter)->IsDefined()) {
             pProtocol->AddParameterValue((*iterClass)->nFileIndex, (*iterParameter)->GetStringValue());  
-          }
+          } else {
+            if ((*iterParameter)->GetType() == Tools_ExpressionValue::ExTypeNumeric) {
+              (*iterParameter)->SetNewValue(0, (*iterParameter)->GetType());
+            } else if ((*iterParameter)->GetType() == Tools_ExpressionValue::ExTypeString) {
+              (*iterParameter)->SetNewValue("", (*iterParameter)->GetType());
+            } // end if
+            pProtocol->AddParameterValue((*iterClass)->nFileIndex, (*iterParameter)->GetStringValue());  
+            (*iterParameter)->SetUndefined();
+          } // end if
         }
         ++iterParameter;
       }
@@ -949,7 +923,6 @@ DBA_bool DBA_configuration::ProtocolParameters(DBA_protocol * pProtocol,
 // ProtocolHistory
 //
 DBA_bool DBA_configuration::ProtocolHistory(DBA_protocol * pProtocol,
-                                            DBA_int        nInstanceType,
                                             DBA_ulong      nCount,
                                             time_t         tAct,
                                             DBA_bool       bStartup)
@@ -964,10 +937,7 @@ DBA_bool DBA_configuration::ProtocolHistory(DBA_protocol * pProtocol,
 
   while (iterMonitor != m_lMonitors.end()) {
     // check the instance type
-    if (( ((*iterMonitor)->nInstanceType & nInstanceType) == nInstanceType ) &&
-        ( (*iterMonitor)->bStartup                        == bStartup      ) &&
-        ( (*iterMonitor)->oMinVersion                     <= m_oDBVersion  ) &&
-        ( (*iterMonitor)->oMaxVersion                     >= m_oDBVersion  )    ) {
+    if ((*iterMonitor)->bStartup == bStartup) {
 
       for (nIndex = 0; nIndex < (*iterMonitor)->oValues.size() ; ++nIndex) {
 
@@ -1278,15 +1248,11 @@ DBA_bool DBA_configuration::ReadConstants()
 
         if (GetConfigKey(sName, nKey)) {
           switch (nKey) {
-            case DBA_CNFKEY_ID:  
-              if (GetParameter(sValue) != NULL) {
-                gError.SetError(DBA_ERR_DUPLICATE, sValue, GetLineNumStr());
-                return false;
-              } else {
-                pConstant->sID           = sValue; 
-              } // end if
-              break;
-            case DBA_CNFKEY_VAL: pConstant->SetNewValue(sValue);    break;
+            case DBA_CNFKEY_ID:  pConstant->sID         = sValue; break;
+            case DBA_CNFKEY_TYP: pConstant->SetNewValue("", GetType(sValue)); break;
+            case DBA_CNFKEY_VAL: pConstant->SetNewValue(sValue);  break;
+            case DBA_CNFKEY_MIV: pConstant->oMinVersion = sValue; break;
+            case DBA_CNFKEY_MAV: pConstant->oMaxVersion = sValue; break;
             case DBA_CNFKEY_ITY: 
               pConstant->nInstanceType  = GetInstanceType(sValue); 
               break;
@@ -1309,14 +1275,29 @@ DBA_bool DBA_configuration::ReadConstants()
 
   if (!bRC && feof(m_hFile)) {
     bRC = true;
-  }
+  } // end if
 
-  if (bRC && pConstant) {
-    m_lParameters.push_back(pConstant);
-  }
+  // ready handling
+    if (bRC && pConstant) {
+    // check version
+    if (((pConstant->nInstanceType & m_nInstance) == m_nInstance ) &&
+        (pConstant->oMinVersion                   <= m_oDBVersion) &&
+        (pConstant->oMaxVersion                   >= m_oDBVersion)    ) {
+      // check for duplicates
+      if (GetParameter(pConstant->sID) != NULL) {
+        gError.SetError(DBA_ERR_DUPLICATE, pConstant->sID, GetLineNumStr());
+        delete pConstant;
+        bRC = false;
+      } else {
+        m_lParameters.push_back(pConstant);
+      } // end if
+    } else {
+      delete pConstant;
+    } // end if
+  } // end if
 
   return bRC;
-}
+} // end DBA_configuration::ReadConstants
 
 //
 // ReadEnvironment
@@ -1340,11 +1321,6 @@ DBA_bool DBA_configuration::ReadEnvironment()
   while (!feof(m_hFile)) {
     if (GetNextLine(sLine)) {
       if (IsSectionBegin(sLine)) {
-        // object ready handling
-        if ((pEnvironment->oClasses[0] != NULL) && pEnvironment->bProtocol) {
-          pEnvironment->oClasses[0]->nInstanceType = pEnvironment->oClasses[0]->nInstanceType | pEnvironment->nInstanceType;
-        } // end if
-
         fsetpos(m_hFile, &nPos);
         m_nLine = nLine;
         bRC = true;
@@ -1366,14 +1342,7 @@ DBA_bool DBA_configuration::ReadEnvironment()
 
         if (GetConfigKey(sName, nKey)) {
           switch (nKey) {
-            case DBA_CNFKEY_ID:  
-              if (GetParameter(sValue) != NULL) {
-                gError.SetError(DBA_ERR_DUPLICATE, sValue, GetLineNumStr());
-                return false;
-              } else {
-                pEnvironment->sID            = sValue; 
-              } // end if
-              break;
+            case DBA_CNFKEY_ID:  pEnvironment->sID            = sValue; break;
             case DBA_CNFKEY_LBL: pEnvironment->sLabel         = sValue; break;
             case DBA_CNFKEY_CLS: 
               sValue.ToUpper();
@@ -1382,7 +1351,43 @@ DBA_bool DBA_configuration::ReadEnvironment()
               break;
             case DBA_CNFKEY_TAB: pEnvironment->sTable         = sValue; break;
             case DBA_CNFKEY_COL: pEnvironment->sColumn        = sValue; break;
-            case DBA_CNFKEY_CND: pEnvironment->sCondition     = sValue; break;
+            case DBA_CNFKEY_CND: {
+              DBA_string sConditionValue;
+              DBA_bool bString = false;
+              SplitNameValue(sValue, pEnvironment->sConditionColumn, sConditionValue);
+              if (!sConditionValue.Empty()) {
+                if ((sConditionValue[0] == '\'') &&
+                    (sConditionValue[sConditionValue.Length()-1] == '\'')) {
+                  sConditionValue = sConditionValue.SubStr(1, sConditionValue.Length()-2);
+                  bString = true;
+                } // end if
+
+                if (sConditionValue[0] == '$') {
+                  if (sConditionValue[1] != '$') {
+                    DBA_config_parameter * pCondition = GetParameter(sConditionValue.SubStr(1, sConditionValue.Length()-1));
+                    if (pCondition != NULL) {
+                      sConditionValue = pCondition->GetStringValue();
+                      bString = bString || (pCondition->GetType() == DBA_config_parameter::ExTypeString);
+                    } else {
+                      gError.SetError(DBA_ERR_UNKNOWNVAL, sConditionValue.SubStr(1, sConditionValue.Length()-1), pEnvironment->sID);
+                      return false;
+                    } // end if
+                  } else {
+                    sConditionValue = sConditionValue.SubStr(1, sConditionValue.Length()-1);
+                  } // end if
+                } // end if
+
+                if (bString) {
+                  pEnvironment->sConditionValue.Assign("'")
+                                               .Append(sConditionValue)
+                                               .Append("'");
+                } else {
+                  pEnvironment->sConditionValue = sConditionValue;
+                } // end if
+
+              } // end if
+            } // end case
+            break;
             case DBA_CNFKEY_DES: pEnvironment->sDescription   = sValue; break;
             case DBA_CNFKEY_FOR: pEnvironment->sFormula       = sValue; break;
             case DBA_CNFKEY_UNI: pEnvironment->sUnit          = sValue; break;
@@ -1415,16 +1420,34 @@ DBA_bool DBA_configuration::ReadEnvironment()
   }
 
   if (bRC && pEnvironment) {
-    if (pEnvironment->oClasses[0]) {
-      if (pEnvironment->bProtocol) {
-        pEnvironment->oClasses[0]->lParameters.push_back(pEnvironment);
-      }
-    }
-    m_lParameters.push_back(pEnvironment);
-  }
+    // check version
+    if (((pEnvironment->nInstanceType & m_nInstance) == m_nInstance ) &&
+        (pEnvironment->oMinVersion                   <= m_oDBVersion) &&
+        (pEnvironment->oMaxVersion                   >= m_oDBVersion)    ) {
+      // check for duplicates
+      if (GetParameter(pEnvironment->sID) != NULL) {
+        gError.SetError(DBA_ERR_DUPLICATE, pEnvironment->sID, GetLineNumStr());
+        delete pEnvironment;
+        bRC = false;
+      } else {
+        // success
+        if ((pEnvironment->oClasses[0] != NULL) && pEnvironment->bProtocol) {
+          pEnvironment->oClasses[0]->nInstanceType = pEnvironment->oClasses[0]->nInstanceType | pEnvironment->nInstanceType;
+        } // end if
+        if (pEnvironment->oClasses[0]) {
+          if (pEnvironment->bProtocol) {
+            pEnvironment->oClasses[0]->lParameters.push_back(pEnvironment);
+          }
+        }
+        m_lParameters.push_back(pEnvironment);
+      } // end if
+    } else {
+      delete pEnvironment;
+    } // end if
+  } // end if
 
   return bRC;
-}
+} // end DBA_configuration::ReadEnvironment
 
 //
 // ReadMonitor
@@ -1469,14 +1492,7 @@ DBA_bool DBA_configuration::ReadMonitor()
 
         if (GetConfigKey(sName, nKey)) {
           switch (nKey) {
-            case DBA_CNFKEY_ID:  
-              if (GetMonitor(sValue) != NULL) {
-                gError.SetError(DBA_ERR_DUPLICATE, sValue, GetLineNumStr());
-                return false;
-              } else {
-                pMonitor->sID           = sValue; 
-              } // end if
-              break;
+            case DBA_CNFKEY_ID:  pMonitor->sID            = sValue; break;
             case DBA_CNFKEY_LBL: pMonitor->sLabel         = sValue; break;
             case DBA_CNFKEY_EAC: pMonitor->sForEach       = sValue; break;
             case DBA_CNFKEY_CLS: pMonitor->sClass         = sValue; break;
@@ -1510,18 +1526,32 @@ DBA_bool DBA_configuration::ReadMonitor()
         }
       }
     }
-  }  
+  } // end while
 
   if (!bRC && feof(m_hFile)) {
     bRC = true;
   }
 
   if (bRC && pMonitor) {
-    m_lMonitors.push_back(pMonitor);
-  }
+    // check version
+    if (((pMonitor->nInstanceType & m_nInstance) == m_nInstance ) &&
+        (pMonitor->oMinVersion                   <= m_oDBVersion) &&
+        (pMonitor->oMaxVersion                   >= m_oDBVersion)    ) {
+      // check for duplicates
+      if (GetMonitor(pMonitor->sID) != NULL) {
+        gError.SetError(DBA_ERR_DUPLICATE, pMonitor->sID, GetLineNumStr());
+        delete pMonitor;
+        bRC = false;
+      } else {
+        m_lMonitors.push_back(pMonitor);
+      } // end if
+    } else {
+      delete pMonitor;
+    } // end if
+  } // end if
 
   return bRC;
-}
+} // end DBA_configuration::ReadMonitor
 
 //
 // ReadParameter
@@ -1548,19 +1578,8 @@ DBA_bool DBA_configuration::ReadParameter()
 
   while (!feof(m_hFile)) {
     if (GetNextLine(sLine)) {
+      // parameter ready handling
       if (IsSectionBegin(sLine)) {
-        // parameter ready handling
-        if (pParameter->pTable != NULL) {
-          pParameter->pTable->nInstanceType = pParameter->pTable->nInstanceType | pParameter->nInstanceType;
-          pParameter->pTable->bStartup      = pParameter->pTable->bStartup     && pParameter->bStartup;
-          pParameter->pTable->oMinVersion   = DBA_config_Version::MINVersion(pParameter->pTable->oMinVersion, pParameter->oMinVersion);;
-          pParameter->pTable->oMaxVersion   = DBA_config_Version::MAXVersion(pParameter->pTable->oMaxVersion, pParameter->oMaxVersion);;
-        } // end if
-        if ((pParameter->oClasses.size() > 0) && pParameter->bProtocol) {
-          for (nIndex = 0; nIndex < pParameter->oClasses.size(); ++nIndex) {
-            pParameter->oClasses[nIndex]->nInstanceType = pParameter->oClasses[nIndex]->nInstanceType | pParameter->nInstanceType;
-          } // end if
-        } // end if
         // reset file and return
         fsetpos(m_hFile, &nPos);
         m_nLine = nLine;
@@ -1583,14 +1602,7 @@ DBA_bool DBA_configuration::ReadParameter()
 
         if (GetConfigKey(sName, nKey)) {
           switch (nKey) {
-            case DBA_CNFKEY_ID:  
-               if (GetParameter(sValue) != NULL) {
-                gError.SetError(DBA_ERR_DUPLICATE, sValue, GetLineNumStr());
-                return false;
-              } else {
-                pParameter->sID = sValue; 
-              } // end if
-              break;
+            case DBA_CNFKEY_ID:  pParameter->sID          = sValue; break;
             case DBA_CNFKEY_LBL: pParameter->sLabel       = sValue; break;
             case DBA_CNFKEY_CLS: 
               sValue.ToUpper();
@@ -1618,16 +1630,41 @@ DBA_bool DBA_configuration::ReadParameter()
               break;
             case DBA_CNFKEY_COL: pParameter->sColumn        = sValue; break;
             case DBA_CNFKEY_CND: {
-              pParameter->sCondition = sValue;
               DBA_string sConditionValue;
+              DBA_bool bString = false;
               SplitNameValue(sValue, pParameter->sConditionColumn, sConditionValue);
               if (!sConditionValue.Empty()) {
                 if ((sConditionValue[0] == '\'') &&
                     (sConditionValue[sConditionValue.Length()-1] == '\'')) {
-                  pParameter->sConditionValue = sConditionValue.SubStr(1, sConditionValue.Length()-2);
-                }
-              }
-            }
+                  sConditionValue = sConditionValue.SubStr(1, sConditionValue.Length()-2);
+                  bString = true;
+                } // end if
+
+                if (sConditionValue[0] == '$') {
+                  if (sConditionValue[1] != '$') {
+                    DBA_config_parameter * pCondition = GetParameter(sConditionValue.SubStr(1, sConditionValue.Length()-1));
+                    if (pCondition != NULL) {
+                      sConditionValue = pCondition->GetStringValue();
+                      bString = bString || (pCondition->GetType() == DBA_config_parameter::ExTypeString);
+                    } else {
+                      gError.SetError(DBA_ERR_UNKNOWNVAL, sConditionValue.SubStr(1, sConditionValue.Length()-1), pParameter->sID);
+                      return false;
+                    } // end if
+                  } else {
+                    sConditionValue = sConditionValue.SubStr(1, sConditionValue.Length()-1);
+                  } // end if
+                } // end if
+
+                if (bString) {
+                  pParameter->sConditionValue.Assign("'")
+                                             .Append(sConditionValue)
+                                             .Append("'");
+                } else {
+                  pParameter->sConditionValue = sConditionValue;
+                } // end if
+
+              } // end if
+            } // end case
             break;
 
             case DBA_CNFKEY_DEF: pParameter->sDefault       = sValue; break;
@@ -1672,20 +1709,45 @@ DBA_bool DBA_configuration::ReadParameter()
       }
     }
 
-  }  
+  } // end while
 
   if (!bRC && feof(m_hFile)) {
     bRC = true;
-  }
+  } // end if
 
   if (bRC && pParameter) {
-    if ((pParameter->oClasses.size() > 0) && pParameter->bProtocol) {
-      for (nIndex = 0; nIndex < pParameter->oClasses.size(); ++nIndex) {
-        pParameter->oClasses[nIndex]->lParameters.push_back(pParameter);
+    // check version
+    if (((pParameter->nInstanceType & m_nInstance) == m_nInstance ) &&
+        (pParameter->oMinVersion                   <= m_oDBVersion) &&
+        (pParameter->oMaxVersion                   >= m_oDBVersion)    ) {
+      // check for duplicates
+      if (GetParameter(pParameter->sID) != NULL) {
+        gError.SetError(DBA_ERR_DUPLICATE, pParameter->sID, GetLineNumStr());
+        delete pParameter;
+        bRC = false;
+      } else {
+        if (pParameter->pTable != NULL) {
+          pParameter->pTable->nInstanceType = pParameter->pTable->nInstanceType | pParameter->nInstanceType;
+          pParameter->pTable->bStartup      = pParameter->pTable->bStartup     && pParameter->bStartup;
+          pParameter->pTable->oMinVersion   = DBA_config_Version::MINVersion(pParameter->pTable->oMinVersion, pParameter->oMinVersion);;
+          pParameter->pTable->oMaxVersion   = DBA_config_Version::MAXVersion(pParameter->pTable->oMaxVersion, pParameter->oMaxVersion);;
+        } // end if
+        if ((pParameter->oClasses.size() > 0) && pParameter->bProtocol) {
+          for (nIndex = 0; nIndex < pParameter->oClasses.size(); ++nIndex) {
+            pParameter->oClasses[nIndex]->nInstanceType = pParameter->oClasses[nIndex]->nInstanceType | pParameter->nInstanceType;
+          } // end if
+        } // end if
+        if ((pParameter->oClasses.size() > 0) && pParameter->bProtocol) {
+          for (nIndex = 0; nIndex < pParameter->oClasses.size(); ++nIndex) {
+            pParameter->oClasses[nIndex]->lParameters.push_back(pParameter);
+          } // end if
+        } // end if
+        m_lParameters.push_back(pParameter);
       } // end if
+    } else {
+      delete pParameter;
     } // end if
-    m_lParameters.push_back(pParameter);
-  }
+  } // end if
 
   return bRC;
 } // DBA_configuration::ReadParameter
@@ -1712,13 +1774,6 @@ DBA_bool DBA_configuration::ReadArray()
   while (!feof(m_hFile)) {
     if (GetNextLine(sLine)) {
       if (IsSectionBegin(sLine)) {
-        // parameter ready handling
-        if (pArray->pTable != NULL) {
-          pArray->pTable->nInstanceType = pArray->pTable->nInstanceType | pArray->nInstanceType;
-          pArray->pTable->bStartup      = pArray->pTable->bStartup     && pArray->bStartup;
-          pArray->pTable->oMinVersion   = DBA_config_Version::MINVersion(pArray->pTable->oMinVersion, pArray->oMinVersion);;
-          pArray->pTable->oMaxVersion   = DBA_config_Version::MAXVersion(pArray->pTable->oMaxVersion, pArray->oMaxVersion);;
-        } // end if
         // reset file and return
         fsetpos(m_hFile, &nPos);
         m_nLine = nLine;
@@ -1750,14 +1805,6 @@ DBA_bool DBA_configuration::ReadArray()
                 AddTable(sValue,&(pArray->pTable));
               } // end if
               break;
-            case DBA_CNFKEY_ID:  
-               if (m_oValueProvider.AskForArray(sValue) != NULL) {
-                gError.SetError(DBA_ERR_DUPLICATE, sValue, GetLineNumStr());
-                return false;
-              } else {
-                pArray->sID = sValue; 
-              } // end if
-              break;
             case DBA_CNFKEY_FOR: 
               if (!pArray->sTable.Empty()) {
                 gError.SetError(DBA_ERR_FORMTAB, pArray->sID, GetLineNumStr());
@@ -1766,6 +1813,7 @@ DBA_bool DBA_configuration::ReadArray()
                 pArray->sFormula       = sValue; 
               } // end if
               break;
+            case DBA_CNFKEY_ID:  pArray->sID            = sValue;                        break;
             case DBA_CNFKEY_EAC: pArray->sForEach       = sValue;                        break;
             case DBA_CNFKEY_LBL: pArray->sLabel         = sValue;                        break;
             case DBA_CNFKEY_COL: pArray->sColumn        = sValue;                        break;
@@ -1806,7 +1854,28 @@ DBA_bool DBA_configuration::ReadArray()
   }
 
   if (bRC && pArray) {
-    m_lArrays.push_back(pArray);
+    // check version
+    if (((pArray->nInstanceType & m_nInstance) == m_nInstance ) &&
+        (pArray->oMinVersion                   <= m_oDBVersion) &&
+        (pArray->oMaxVersion                   >= m_oDBVersion)    ) {
+      // check for duplicates
+      if (m_oValueProvider.AskForArray(pArray->sID) != NULL) {
+        gError.SetError(DBA_ERR_DUPLICATE, pArray->sID, GetLineNumStr());
+        delete pArray;
+        bRC = false;
+      } else {
+        // parameter ready handling
+        if (pArray->pTable != NULL) {
+          pArray->pTable->nInstanceType = pArray->pTable->nInstanceType | pArray->nInstanceType;
+          pArray->pTable->bStartup      = pArray->pTable->bStartup     && pArray->bStartup;
+          pArray->pTable->oMinVersion   = DBA_config_Version::MINVersion(pArray->pTable->oMinVersion, pArray->oMinVersion);;
+          pArray->pTable->oMaxVersion   = DBA_config_Version::MAXVersion(pArray->pTable->oMaxVersion, pArray->oMaxVersion);;
+        } // end if
+        m_lArrays.push_back(pArray);
+      } // end if
+    } else {
+      delete pArray;
+    } // end if
   } // end if
 
   return bRC;
@@ -1935,11 +2004,18 @@ DBA_bool DBA_configuration::ReadView()
   }
 
   if (bRC && pView) {
-    m_lViews.push_back(pView);
-  }
+    // check version
+    if (((pView->nInstanceType & m_nInstance) == m_nInstance ) &&
+        (pView->oMinVersion                   <= m_oDBVersion) &&
+        (pView->oMaxVersion                   >= m_oDBVersion)    ) {
+      m_lViews.push_back(pView);
+    } else {
+      delete pView;
+    } // end if
+  } // end if
 
   return bRC;
-}
+} // end DBA_configuration::ReadView
 
 //
 // IsSectionBegin
