@@ -8,7 +8,7 @@
   version:      7.4 and higher 
   -----------------------------------------------------------------------------
 
-  copyright: (c) 2002-2004 SAP AG
+  Copyright (c) 2002-2004 SAP AG
 
 
     ========== licence begin  GPL
@@ -415,12 +415,20 @@ public:
      : oMinVersion(DBA_config_Version::MaxVersion),
        oMaxVersion(DBA_config_Version::MinVersion)
   {
-    nInstanceType = 0;
-    bStartup      = true;
+    nInstanceType  = 0;
+    bStartup       = true;
+    pPreparedQuery = NULL;
+  }
+  ~DBA_config_table () 
+  {
+    if (pPreparedQuery != NULL) {
+      delete pPreparedQuery;
+    } // end if
   }
 
   DBA_string                  sName;
   DBA_snapshot              * pSnapshot;
+  StudioOAL_WResult         * pPreparedQuery;
   DBA_int                     nInstanceType;
 
   DBA_bool                    bStartup;
@@ -496,7 +504,6 @@ public:
   DBA_string sClass;
   DBA_string sTable;
   DBA_string sColumn;
-  DBA_string sCondition;
   DBA_string sConditionColumn;
   DBA_string sConditionValue;
   DBA_string sFormula;
@@ -592,7 +599,8 @@ class DBA_configuration {
                   DBA_int            nMajor,
                   DBA_int            nMinor,
                   DBA_int            nLevel,
-                  DBA_int            nBuild );
+                  DBA_int            nBuild,
+                  DBA_int            nInstance );
 
   DBA_bool SetParameterValue(const DBA_string & sName, 
                              const DBA_string & sValue);
@@ -606,30 +614,24 @@ class DBA_configuration {
 
   DBA_string GetSetting(const DBA_string & sName);
 
-  DBA_bool InitViews(DBA_database * pDatabase,
-                     DBA_int        nInstanceType);
-  DBA_bool InitEnvironment(DBA_database * pDatabase,
-                           DBA_int        nInstanceType);
-  DBA_bool InitProtocolFiles(DBA_protocol * pProtocol,
-                             DBA_int        nInstanceType);
+  DBA_bool InitViews(DBA_database * pDatabase);
+  DBA_bool InitEnvironment(DBA_database * pDatabase);
+  DBA_bool InitProtocolFiles(DBA_protocol * pProtocol);
   DBA_bool GetSnapshot(DBA_database * pDatabase,
-                       const DBA_bool bStartup);
+                       time_t       & tSnapshot,
+                       DBA_bool     & bStartup);
 
-  DBA_bool CalculateParameters (const DBA_int nInstanceType,
-                                const DBA_bool bStartup);
-  DBA_bool CalculateArrays     (const DBA_int  nInstanceType,
-                                const DBA_bool bStartup);
-  DBA_bool CalculateMonitors   (const DBA_int  nInstanceType,
-                                const DBA_bool bStartup);
+  DBA_bool CalculateParameters (const DBA_bool bStartup);
+  DBA_bool CalculateArrays     (const DBA_bool bStartup);
+  DBA_bool CalculateMonitors   (const DBA_bool bStartup);
   
   DBA_bool ProtocolParameters(DBA_protocol * pProtocol,
-                              DBA_int        nInstanceType,
                               DBA_ulong      nCount,
                               time_t         tAct,
-                              time_t         tLast);
+                              time_t         tLast,
+                              time_t         tNow);
   DBA_bool ParseExpressions();
   DBA_bool ProtocolHistory(DBA_protocol * pProtocol,
-                           DBA_int        nInstanceType,
                            DBA_ulong      nCount,
                            time_t         tAct,
                            DBA_bool       bStartup);
@@ -685,7 +687,9 @@ class DBA_configuration {
   DBA_config_ValueProvider    m_oValueProvider;
 
   DBA_config_Version          m_oDBVersion;
+  DBA_int                     m_nInstance;
 
+  DBA_string                  m_sDate;
 }; /* EndClass: DBA_configuration */
 
 #endif
