@@ -8,7 +8,7 @@
   version:      7.4 and higher 
   -----------------------------------------------------------------------------
 
-  copyright: (c) 2002-2004 SAP AG
+  Copyright (c) 2002-2004 SAP AG
 
 
     ========== licence begin  GPL
@@ -49,6 +49,7 @@ extern DBA_error gError;
 #define DBA_SQL_INFO_SESSION        "SELECT SESSION FROM CONNECTPARAMETERS"
 #define DBA_SQL_INFO_PID            "SELECT T.APPLPROCESS FROM TRANSACTIONS T,CONNECTPARAMETERS C WHERE T.SESSION = C.SESSION"
 #define DBA_SQL_CHECK_ANALYZER      "SELECT T.APPLPROCESS FROM TRANSACTIONS T,CONNECTPARAMETERS C WHERE T.SESSION = %u AND T.APPLPROCESS = %u"
+#define DBA_SQL_INFO_DATETIME       "SELECT YEAR(NOW()),MONTH(NOW()),DAY(NOW()),HOUR(NOW()),MINUTE(NOW()),SECOND(NOW()) FROM DUAL"
 
 class DBA_handles {
 private:
@@ -61,6 +62,9 @@ private:
   friend class DBA_database; 
 }; // end class DBA_handles
 
+// ========================================================
+// DBA_database::DBA_database
+// ========================================================
 DBA_database::DBA_database (const DBA_string & sServer,
                             const DBA_string & sDatabase,
                             const DBA_string & sUser,
@@ -82,13 +86,19 @@ DBA_database::DBA_database (const DBA_string & sServer,
   m_sRundirectory  = "";
   m_nSid           = 0;
   m_nPid           = 0;
-}
+} // end DBA_database::DBA_database
 
+// ========================================================
+// DBA_database::~DBA_database
+// ========================================================
 DBA_database::~DBA_database () 
 {
   delete m_pHandles;
-}
+} // end DBA_database::~DBA_database
 
+// ========================================================
+// DBA_database::Connect
+// ========================================================
 DBA_bool DBA_database::Connect()
 {
   gError.Clear(); 
@@ -117,6 +127,14 @@ DBA_bool DBA_database::Connect()
     return false;
   } // end if
 
+  if (SQLSetConnectAttr( m_pHandles->hDbc, SQL_TXN_ISOLATION, (SQLPOINTER) SQL_TXN_READ_UNCOMMITTED, 0 ) != SQL_SUCCESS) {
+    SQLFreeHandle(SQL_HANDLE_DBC, m_pHandles->hDbc);
+    SQLFreeHandle(SQL_HANDLE_ENV, m_pHandles->hEnv);
+    m_pHandles->hEnv = NULL;
+    m_pHandles->hDbc = NULL;
+    return false;
+  } // end if
+  
   if (SQLSetConnectAttr( m_pHandles->hDbc, SQL_PRODUCER, (SQLPOINTER) sp1pr_internal_cmd, 0 ) != SQL_SUCCESS) {
     SQLFreeHandle(SQL_HANDLE_DBC, m_pHandles->hDbc);
     SQLFreeHandle(SQL_HANDLE_ENV, m_pHandles->hEnv);
@@ -148,16 +166,34 @@ DBA_bool DBA_database::Connect()
     m_sPassword.ToUpper();
   } // end if
 
-  sprintf((char *)sConnect, "DRIVER=SAP DB;SERVERDB=%s;SERVERNODE=%s;UID=%s;PWD=%s",
+  m_sDriver = "SAP DB <MAJOR_VERSION>.<MINOR_VERSION>";
+  sprintf((char *)sConnect, "DRIVER=%s;SERVERDB=%s;SERVERNODE=%s;UID=%s;PWD=%s",
+          m_sDriver.CharPtr(),
           m_sDatabase.CharPtr(),
           m_sServer.CharPtr(),
           m_sUser.CharPtr(),
           m_sPassword.CharPtr());
+  
 
   nRC = SQLDriverConnect(m_pHandles->hDbc, (SQLHWND)NULL, sConnect, SQL_NTS, sOut, 
                          sizeof(sOut), &nOut, SQL_DRIVER_NOPROMPT);
 
-  if (!m_pError->checkSQLReturnCode(nRC, NULL)) {
+  bool bError =  (!m_pError->checkSQLReturnCode(nRC, NULL));
+
+  if (bError && (m_pError->getSQLStateErrorNumber() == "IM002")) {
+    m_sDriver = "SAP DB";
+    sprintf((char *)sConnect, "DRIVER=%s;SERVERDB=%s;SERVERNODE=%s;UID=%s;PWD=%s",
+            m_sDriver.CharPtr(),
+            m_sDatabase.CharPtr(),
+            m_sServer.CharPtr(),
+            m_sUser.CharPtr(),
+            m_sPassword.CharPtr());
+    nRC = SQLDriverConnect(m_pHandles->hDbc, (SQLHWND)NULL, sConnect, SQL_NTS, sOut, 
+                          sizeof(sOut), &nOut, SQL_DRIVER_NOPROMPT);
+    bError = (!m_pError->checkSQLReturnCode(nRC, NULL));
+  } // end if
+
+  if (bError) {
     CheckReconnectError(m_pError->getNativeError());
     if (!m_bReconnectEnabled || !m_bReconnect) {
       SQLFreeHandle(SQL_HANDLE_DBC, m_pHandles->hDbc);
@@ -178,14 +214,51 @@ DBA_bool DBA_database::Connect()
   }
 
   return true;
-}
+} // DBA_database::Connect
 
-//
-// Execute
-// 
+// ========================================================
+// DBA_database::Execute
+// ========================================================
 DBA_bool DBA_database::Execute(const DBA_string & sSQL, DBA_snapshot * pSnapshot) 
 {
+  StudioOAL_WResult   oResult(m_pHandles->hDbc, m_pHandles->hEnv);
+  StudioOAL_WResult * pResult = &oResult;
+  if (!PrepareQuery(sSQL, pResult)) {
+    return false;
+  } // end if
+  return Execute(sSQL, pSnapshot, pResult);
+} // end DBA_database::Execute
 
+// ========================================================
+// DBA_database::PrepareQuery
+// ========================================================
+DBA_bool DBA_database::PrepareQuery (const DBA_string & sSQL, StudioOAL_WResult * & pPreparedQuery)
+{
+  gError.Clear(); 
+
+  if (pPreparedQuery == NULL) {
+    pPreparedQuery = new StudioOAL_WResult(m_pHandles->hDbc, m_pHandles->hEnv);
+    if (pPreparedQuery == NULL) {
+      gError.SetError(DBA_ERR_NOMEM);
+      return false;
+     } // end if
+  } // end if
+
+  if (!pPreparedQuery->openResultPrepare (sSQL)) {
+    gError.SetError(DBA_ERR_PREPARE);
+    DBA_string sError = pPreparedQuery->getErrorText();
+    gError.SetExtText(sError.Append("\n").Append(sSQL));
+    return false;
+  } // end if
+
+  return true;
+} // end DBA_database::PrepareQuery
+
+// ========================================================
+// DBA_database::Execute
+// ========================================================
+DBA_bool DBA_database::Execute (const DBA_string & sSQL, DBA_snapshot * pSnapshot, StudioOAL_WResult * & pPreparedQuery)
+{
   gError.Clear(); 
 
   if (m_bReconnectEnabled && m_bReconnect) {
@@ -197,28 +270,45 @@ DBA_bool DBA_database::Execute(const DBA_string & sSQL, DBA_snapshot * pSnapshot
         return true;
       } else {
         m_bReconnect = false;
-      }
-    }
-  }
-
-  StudioOAL_WResult oResult(m_pHandles->hDbc, m_pHandles->hEnv);
+        if (!PrepareQuery(sSQL, pPreparedQuery)) {
+          return false;
+        } // end if
+      } // end if
+    } // end if
+  } // end if
 
   gError.SetIgnore(false);
 
-  if (!oResult.openResult(sSQL)) {
-    CheckReconnectError(oResult.getNativeError());
+  if (pPreparedQuery == NULL) {
+    if (!PrepareQuery(sSQL, pPreparedQuery)) {
+      return false;
+    } // end if
+  } // end if
+
+  if (!pPreparedQuery->executePreparedStmt()) {
+    CheckReconnectError(pPreparedQuery->getNativeError());
     if (!m_bReconnectEnabled || !m_bReconnect) {
-      if (CheckIgnoreError(oResult.getNativeError())) {
+      if (CheckIgnoreError(pPreparedQuery->getNativeError())) {
         gError.SetError(DBA_ERR_EXECUTE);
-        DBA_string sError = oResult.getErrorText();
+        DBA_string sError = pPreparedQuery->getErrorText();
         gError.SetExtText(sError.Append("\n").Append(sSQL));
         gError.SetIgnore(true);
         return true;
       } else {
         gError.SetError(DBA_ERR_EXECUTE);
-        DBA_string sError = oResult.getErrorText();
+        DBA_string sError = pPreparedQuery->getErrorText();
         gError.SetExtText(sError.Append("\n").Append(sSQL));
-        return false;
+        // prepare again
+        if (!PrepareQuery(sSQL, pPreparedQuery)) {
+          return false;
+        } // end if
+        // excute again
+        if (!pPreparedQuery->executePreparedStmt()) {
+          gError.SetError(DBA_ERR_EXECUTE);
+          DBA_string sError = pPreparedQuery->getErrorText();
+          gError.SetExtText(sError.Append("\n").Append(sSQL));
+          return false;
+        } // end if
       } // end if
     } else {
       gError.SetReconnect(true);
@@ -229,12 +319,12 @@ DBA_bool DBA_database::Execute(const DBA_string & sSQL, DBA_snapshot * pSnapshot
   if (pSnapshot) {
     pSnapshot->Clear();  
 
-    DBA_int nCols = oResult.getColCount();
+    DBA_int nCols = pPreparedQuery->getColCount();
     DBA_int nCol;
     
     for (nCol = 0; nCol < nCols; nCol++) {
       DBA_string sName;
-      oResult.getColumnName(nCol+1, sName);
+      pPreparedQuery->getColumnName(nCol+1, sName);
       if (!pSnapshot->AddColumn(sName)) {
         return false;
       }
@@ -243,25 +333,36 @@ DBA_bool DBA_database::Execute(const DBA_string & sSQL, DBA_snapshot * pSnapshot
     DBA_bool bFetch = true;
   
     while (bFetch) {
-      bFetch = oResult.fetch();
+      bFetch = pPreparedQuery->fetch();
       if(!bFetch) {
-        CheckReconnectError(oResult.getNativeError());
+        CheckReconnectError(pPreparedQuery->getNativeError());
         if (!m_bReconnectEnabled || !m_bReconnect) {
-          gError.SetError(DBA_ERR_EXECUTE);
-          gError.SetExtText(oResult.getErrorText());
-          return false;
+          if (CheckIgnoreError(pPreparedQuery->getNativeError())) {
+            gError.SetError(DBA_ERR_EXECUTE);
+            DBA_string sError = pPreparedQuery->getErrorText();
+            gError.SetExtText(sError.Append("\n").Append(sSQL));
+            gError.SetIgnore(true);
+            return true;
+          } else {
+            gError.SetError(DBA_ERR_EXECUTE);
+            DBA_string sError = pPreparedQuery->getErrorText();
+            gError.SetExtText(sError.Append("\n").Append(sSQL));
+            return false;
+          } // end if
         } else {
           gError.SetReconnect(true);
+          pPreparedQuery->closeCursor ();
           return true;
         }
       } else {
-        if (oResult.noRows()) {
+        if (pPreparedQuery->noRows()) {
+          pPreparedQuery->closeCursor ();
           return true;
         } else {
           if (pSnapshot->AddRow()) {
             for (nCol = 0; nCol < nCols; nCol++) {
               DBA_string sValue;
-              oResult.getColumnDataAsString(nCol+1, sValue);
+              pPreparedQuery->getColumnDataAsString(nCol+1, sValue);
               if (!pSnapshot->AddField(sValue)) {
                 return false;
               }
@@ -273,12 +374,13 @@ DBA_bool DBA_database::Execute(const DBA_string & sSQL, DBA_snapshot * pSnapshot
 
   } 
 
+  pPreparedQuery->closeCursor ();
   return true;
-}
+} // end DBA_database::Execute
 
-//
-// Disconnect
-//
+// ========================================================
+// DBA_database::Disconnect
+// ========================================================
 DBA_bool DBA_database::Disconnect ()
 {
   SQLDisconnect(m_pHandles->hDbc);
@@ -294,11 +396,61 @@ DBA_bool DBA_database::Disconnect ()
   }
 
   return true;
-} 
+} // end DBA_database::Disconnect
 
-//
-// GetInfos
-//
+// ========================================================
+// DBA_database::GetTimestamp
+// ========================================================
+DBA_bool DBA_database::GetTimestamp(time_t & tTime)
+{
+  StudioOAL_WResult oResult(m_pHandles->hDbc, m_pHandles->hEnv);
+  gError.Clear(); 
+
+  // select date and time
+  if (!oResult.openResult(DBA_SQL_INFO_DATETIME)) {
+    gError.SetError(DBA_ERR_EXECUTE);
+    DBA_string sError = oResult.getErrorText();
+    gError.SetExtText(sError.Append("\n").Append(DBA_SQL_INFO_DATETIME));
+    return false;
+  } // end if
+
+  if (!oResult.fetch()) {
+    gError.SetError(DBA_ERR_EXECUTE);
+    gError.SetExtText(oResult.getErrorText());
+    return false;
+  } // end if
+  
+  if (!oResult.noRows()) {
+    struct tm oTime;
+    memset(&oTime, 0, sizeof(oTime));
+    DBA_string sColumn = "";
+    oResult.getColumnDataAsString(1, sColumn);
+    oTime.tm_year = atoi(sColumn.CharPtr()) - 1900;
+    oResult.getColumnDataAsString(2, sColumn);
+    oTime.tm_mon  = atoi(sColumn.CharPtr()) - 1;
+    oResult.getColumnDataAsString(3, sColumn);
+    oTime.tm_mday = atoi(sColumn.CharPtr());
+    oResult.getColumnDataAsString(4, sColumn);
+    oTime.tm_hour = atoi(sColumn.CharPtr());
+    oResult.getColumnDataAsString(5, sColumn);
+    oTime.tm_min  = atoi(sColumn.CharPtr());
+    oResult.getColumnDataAsString(6, sColumn);
+    oTime.tm_sec  = atoi(sColumn.CharPtr());
+    oTime.tm_isdst = -1;
+    tTime = mktime(&oTime);
+  } else {
+    gError.SetError(DBA_ERR_EXECUTE);
+    gError.SetExtText(DBA_SQL_INFO_DATETIME);
+    tTime = 0;
+    return false;
+  } // end if
+
+  return true;
+} // end DBA_database::GetTimestamp
+
+// ========================================================
+// DBA_database::GetInfos
+// ========================================================
 // get some infos from the database
 //
 // INSTANCE_TYPE
