@@ -8,7 +8,7 @@
   version:      7.4 and higher 
   -----------------------------------------------------------------------------
 
-  copyright: (c) 2002-2004 SAP AG
+  Copyright (c) 2002-2004 SAP AG
 
 
     ========== licence begin  GPL
@@ -115,10 +115,13 @@ int main (int argcIn, char *argvIn[])
   DBA_string sDuration;
   time_t     tStart          = 0;
   time_t     tLast           = 0;
+  time_t     tSnapshot       = 0;
+  time_t     tLastSnapshot   = 0;
   time_t     tWakeup         = 0;
   time_t     tAct            = 0;
   DBA_uint   nConsole        = 0;
   DBA_bool   bDisconnected   = false;
+  DBA_bool   bStartup        = false;
 
   // read arguments from a pipe
   nRC = Tools_PipeCall::ReadArguments(argcIn, argvIn, argc, argv, Tools_PipeCall::ReadXUser, "w");
@@ -198,7 +201,8 @@ int main (int argcIn, char *argvIn[])
 
   // create the protocol object
   sOutputDir = ToolsSys_fullpath(sOutputDir.CharPtr());
-  pProtocol = new DBA_protocol(sOutputDir, nConsole, pArguments->GetIgnore());
+  pDatabase->GetTimestamp(tAct);
+  pProtocol = new DBA_protocol(sOutputDir, nConsole, pArguments->GetIgnore(), tAct);
   if (!pProtocol) {
     gError.SetError(DBA_ERR_NOMEM);
     gError.ShowErrorMsg(pProtocol);
@@ -239,6 +243,9 @@ int main (int argcIn, char *argvIn[])
   while ((nFind = sEvent.Find(":")) != DBA_string::NPos) {
     sEvent.Replace(nFind, 1, "x");
   } // end while
+  DBA_string sTemp = sEvent;
+  sEvent = "Global\\";
+  sEvent.Append(sTemp);
 
   // stops the dbanalyzer
   if (pArguments->GetStop()) {
@@ -251,7 +258,7 @@ int main (int argcIn, char *argvIn[])
       sEvent.Append(sSid);
       sEvent.Append(sPid);
 
-      HANDLE hEvent = OpenEvent(EVENT_ALL_ACCESS, FALSE, sEvent.CharPtr());
+      HANDLE hEvent = OpenEvent(EVENT_MODIFY_STATE, FALSE, sEvent.CharPtr());
       if (hEvent != 0) {
         if (!SetEvent(hEvent)) {
           gError.SetError(DBA_ERR_KILL);
@@ -290,7 +297,8 @@ int main (int argcIn, char *argvIn[])
                             pDatabase->GetMajor(),
                             pDatabase->GetMinor(),
                             pDatabase->GetLevel(),
-                            pDatabase->GetBuild())) {
+                            pDatabase->GetBuild(),
+                            pDatabase->GetInstanceType())) {
     gError.ShowErrorMsg(pProtocol);
     disconnect(DO_NOT_REMOVE_SESSION_DATA); return 1;
   } else {
@@ -301,7 +309,7 @@ int main (int argcIn, char *argvIn[])
   } // end if
 
   // init protocol files
-  if (!pConfiguration->InitProtocolFiles(pProtocol, pDatabase->GetInstanceType())) {
+  if (!pConfiguration->InitProtocolFiles(pProtocol)) {
     gError.ShowErrorMsg(pProtocol);
     disconnect(DO_NOT_REMOVE_SESSION_DATA); return 1;
   }
@@ -321,25 +329,6 @@ int main (int argcIn, char *argvIn[])
     disconnect(DO_NOT_REMOVE_SESSION_DATA); return 1;
   } // end if
 
-  // check and save the analyzer information
-  if (!pProtocol->CheckAndSaveInfo(pArguments->GetServer(),
-                                   pArguments->GetDatabase(),
-                                   pDatabase->GetRundirectory(),
-                                   pConfiguration->GetConfigFile(),
-                                   nConfigInterval,
-                                   pArguments->GetReconnect(),
-                                   pDatabase->GetSid(),
-                                   pDatabase->GetPid())) {
-    gError.ShowErrorMsg(pProtocol);
-    disconnect(DO_NOT_REMOVE_SESSION_DATA); return 1;
-  } // end if
-
-  // parse the expressions in the configuration file
-  if (!pConfiguration->ParseExpressions()) {
-    gError.ShowErrorMsg(pProtocol);
-    disconnect(REMOVE_SESSION_DATA); return 1;
-  } // end if
-
   // overwrite interval constant 
   if (!pArguments->GetInterval().Empty()) {
     if (!pConfiguration->SetParameterValue("INTERVAL", pArguments->GetInterval())) {
@@ -352,50 +341,67 @@ int main (int argcIn, char *argvIn[])
   nCount          = 0;
   nConfigInterval = atoi(pConfiguration->GetParameterValue("INTERVAL").CharPtr());
 
+  // check and save the analyzer information
+  if (!pProtocol->CheckAndSaveInfo(pArguments->GetServer(),
+                                   pArguments->GetDatabase(),
+                                   pDatabase->GetRundirectory(),
+                                   pConfiguration->GetConfigFile(),
+                                   nConfigInterval,
+                                   pArguments->GetReconnect(),
+                                   pDatabase->GetSid(),
+                                   pDatabase->GetPid(),
+                                   pDatabase->GetDriver())) {
+    gError.ShowErrorMsg(pProtocol);
+    disconnect(DO_NOT_REMOVE_SESSION_DATA); return 1;
+  } // end if
+
+  // parse the expressions in the configuration file
+  if (!pConfiguration->ParseExpressions()) {
+    gError.ShowErrorMsg(pProtocol);
+    disconnect(REMOVE_SESSION_DATA); return 1;
+  } // end if
+
   // create specified views 
-  if (!pConfiguration->InitViews(pDatabase, pDatabase->GetInstanceType())) {
+  if (!pConfiguration->InitViews(pDatabase)) {
     gError.ShowErrorMsg(pProtocol);
     disconnect(REMOVE_SESSION_DATA); return 1;
   }
 
   // get environment values 
-  if (!pConfiguration->InitEnvironment(pDatabase, pDatabase->GetInstanceType())) {
+  if (!pConfiguration->InitEnvironment(pDatabase)) {
     gError.ShowErrorMsg(pProtocol);
     disconnect(REMOVE_SESSION_DATA); return 1;
   }
 
-  // get the start time
-  tStart = time(&tStart);
-
   // get the first snapshot
-  if (!pConfiguration->GetSnapshot(pDatabase, true)) {
+  if (!pConfiguration->GetSnapshot(pDatabase, tSnapshot, bStartup)) {
     gError.ShowErrorMsg(pProtocol);
     disconnect(REMOVE_SESSION_DATA); return 1;
   } // end if
 
   // get the start time
   tStart = time(&tStart);
-  
+    
   // make the first calculation of parameters
-  if (!pConfiguration->CalculateParameters(pDatabase->GetInstanceType(), true)) {
+  if (!pConfiguration->CalculateParameters(true)) {
     gError.ShowErrorMsg(pProtocol);
     disconnect(REMOVE_SESSION_DATA); return 1;
   } // end if
 
   // make the first calculation of arrays
-  if (!pConfiguration->CalculateArrays(pDatabase->GetInstanceType(), true)) {
+  if (!pConfiguration->CalculateArrays(true)) {
     gError.ShowErrorMsg(pProtocol);
     disconnect(REMOVE_SESSION_DATA); return 1;
   } // end if
 
   // make the first calculation of monitors ("startup")
-  if (!pConfiguration->CalculateMonitors(pDatabase->GetInstanceType(), true)) {
+  if (!pConfiguration->CalculateMonitors(true)) {
     gError.ShowErrorMsg(pProtocol);
     disconnect(REMOVE_SESSION_DATA); return 1;
   } // end if
 
   // write the monitors to the protocol files
-  if (!pConfiguration->ProtocolHistory(pProtocol, pDatabase->GetInstanceType(), nCount, tStart, true)) {
+  if (!pConfiguration->ProtocolHistory(pProtocol, nCount, tSnapshot, true)) {
     gError.ShowErrorMsg(pProtocol);
     disconnect(REMOVE_SESSION_DATA); return 1;
   } // end if
@@ -414,7 +420,21 @@ int main (int argcIn, char *argvIn[])
   sEvent.Append(sSid);
   sEvent.Append(sPid);
 
-  HANDLE hEvent = CreateEvent(NULL, FALSE, FALSE, sEvent.CharPtr());
+  SECURITY_DESCRIPTOR oSD;
+  SECURITY_ATTRIBUTES oSA;
+  HANDLE              hEvent;
+  if (InitializeSecurityDescriptor(&oSD, SECURITY_DESCRIPTOR_REVISION))  {  
+    if (SetSecurityDescriptorDacl(&oSD, TRUE, NULL, FALSE)) {  
+      oSA.nLength = sizeof (SECURITY_ATTRIBUTES);
+      oSA.lpSecurityDescriptor = &oSD;
+      oSA.bInheritHandle = FALSE;
+      hEvent = CreateEvent(&oSA, FALSE, FALSE, sEvent.CharPtr());
+    } else {
+      hEvent = CreateEvent(NULL, FALSE, FALSE, sEvent.CharPtr());
+    } // end if
+  } else {
+    hEvent = CreateEvent(NULL, FALSE, FALSE, sEvent.CharPtr());
+  } // end if
 #endif
 
   // the looooooooooop
@@ -461,7 +481,8 @@ int main (int argcIn, char *argvIn[])
     signal(SIGABRT,  SIG_IGN);
 
     // get the next snapshot
-    if (!pConfiguration->GetSnapshot(pDatabase, false)) {
+    tLastSnapshot = tSnapshot;
+    if (!pConfiguration->GetSnapshot(pDatabase, tSnapshot, bStartup)) {
       gError.ShowErrorMsg(pProtocol);
       disconnect(REMOVE_SESSION_DATA); return 1;
     } // end if
@@ -485,38 +506,44 @@ int main (int argcIn, char *argvIn[])
       pConfiguration->SetParameterValue("INTERVAL", sDuration);
 
       // calculate parameters 
-      if (!pConfiguration->CalculateParameters(pDatabase->GetInstanceType(), false)) {
+      if (!pConfiguration->CalculateParameters(bStartup)) {
         gError.ShowErrorMsg(pProtocol);
         disconnect(REMOVE_SESSION_DATA); return 1;
       } // end if
 
       // calculate arrays
-      if (!pConfiguration->CalculateArrays(pDatabase->GetInstanceType(), false)) {
+      if (!pConfiguration->CalculateArrays(bStartup)) {
         gError.ShowErrorMsg(pProtocol);
         disconnect(REMOVE_SESSION_DATA); return 1;
       } // end if
 
       // calculate monitors
-      if (!pConfiguration->CalculateMonitors(pDatabase->GetInstanceType(), false)) {
+      if (!pConfiguration->CalculateMonitors(bStartup)) {
         gError.ShowErrorMsg(pProtocol);
         disconnect(REMOVE_SESSION_DATA); return 1;
       } // end if
 
       // write the parameters to the protocol files
-      if (!pConfiguration->ProtocolParameters(pProtocol, pDatabase->GetInstanceType(), 
-                                              nCount, tStart, tLast)) {
+      pDatabase->GetTimestamp(tAct);
+      if (!pConfiguration->ProtocolParameters(pProtocol, nCount, tSnapshot, tLastSnapshot, tAct)) {
         gError.ShowErrorMsg(pProtocol);
         disconnect(REMOVE_SESSION_DATA); return 1;
       } // end if
 
       // write the monitors to the protocol files
-      if (!pConfiguration->ProtocolHistory(pProtocol, pDatabase->GetInstanceType(), nCount, tStart, false)) {
+      if (!pConfiguration->ProtocolHistory(pProtocol, nCount, tSnapshot, bStartup)) {
         gError.ShowErrorMsg(pProtocol);
         disconnect(REMOVE_SESSION_DATA); return 1;
       } // end if
+      if (bStartup) {
+        if (!pConfiguration->ProtocolHistory(pProtocol, nCount, tSnapshot, false)) {
+          gError.ShowErrorMsg(pProtocol);
+          disconnect(REMOVE_SESSION_DATA); return 1;
+        } // end if
+      } // end if
     
     } else {
-      pProtocol->SetHistoryStatus(nCount, tStart); 
+      pProtocol->SetHistoryStatus(nCount, tSnapshot); 
       if (gError.GetReconnect()) {
         pProtocol->SetHistoryNotConnected();
         pProtocol->WriteHistory(); 
