@@ -8,7 +8,7 @@
   version:      7.4 and higher 
   -----------------------------------------------------------------------------
 
-  copyright: (c) 2002-2004 SAP AG
+  Copyright (c) 2002-2004 SAP AG
 
 
     ========== licence begin  GPL
@@ -47,14 +47,15 @@ extern DBA_error gError;
 
 DBA_protocol::DBA_protocol (const DBA_string & sOutputDir,
                             const DBA_uint     nConsoleLevel,
-                            const DBA_bool     bIgnore) 
+                            const DBA_bool     bIgnore,
+                            const time_t       tNow) 
              : m_sOutputDir(sOutputDir),
                m_nConsoleLevel(nConsoleLevel),
                m_bIgnore(bIgnore)
 {
   m_nParameterFiles = 0;
   
-  m_tAct = 0;
+  m_tSnapshot = tNow;
 
   m_sParameterStatus = "";
 
@@ -65,7 +66,6 @@ DBA_protocol::DBA_protocol (const DBA_string & sOutputDir,
   m_sConsoleText  = "";
 
   if (m_bIgnore) {
-    m_tAct = time(&m_tAct);
     DBA_string sAct = GetActDir();
     DBA_string sDir = ToolsSys_joinpath(m_sOutputDir.CharPtr(), sAct.CharPtr());
     DBA_string sFilename = ToolsSys_joinpath(sDir.CharPtr(),m_sHistoryFile.CharPtr());
@@ -112,7 +112,6 @@ DBA_bool DBA_protocol::AddParameterClass (const DBA_string & sClass, DBA_int & n
   nIndex = m_nParameterFiles;
 
   if (m_bIgnore) {
-    m_tAct = time(&m_tAct);
     DBA_string sAct = GetActDir();
     DBA_string sDir = ToolsSys_joinpath(m_sOutputDir.CharPtr(), sAct.CharPtr());
     DBA_string sFilename = ToolsSys_joinpath(sDir.CharPtr(), m_aParameterFiles[nIndex-1]->sFile.CharPtr());
@@ -169,29 +168,26 @@ DBA_void DBA_protocol::AddParameterValue (const DBA_int nIndex, const DBA_string
 //
 // SetParameterStatus
 //
-DBA_void DBA_protocol::SetParameterStatus(DBA_ulong nCount, time_t tAct, time_t tLast)
+DBA_void DBA_protocol::SetParameterStatus(DBA_ulong nCount, time_t tSnapshot, time_t tLastSnapshot, time_t tNow)
 {
-  m_tAct  = tAct;
+  m_tSnapshot  = tSnapshot;
 
-  struct tm *tmAct;
-  tmAct = localtime( &tAct );
+  struct tm *tmSnapshot;
+  tmSnapshot = localtime( &tSnapshot );
   
-  time_t tNow = 0;
-  time(&tNow);
-
   DBA_char szBuffer[1024];
 
   sprintf(szBuffer,
           "%d;%04d%02d%02d;%02d%02d%02d;%d;%d;", 
-          nCount,                   // count
-          tmAct->tm_year+1900,      // year
-          tmAct->tm_mon+1,          // month
-          tmAct->tm_mday,           // day
-          tmAct->tm_hour,           // hour
-          tmAct->tm_min,            // minute
-          tmAct->tm_sec,            // second
-          tNow - tAct,              // duration  
-          tAct - tLast              // delta
+          nCount,                        // count
+          tmSnapshot->tm_year+1900,      // year
+          tmSnapshot->tm_mon+1,          // month
+          tmSnapshot->tm_mday,           // day
+          tmSnapshot->tm_hour,           // hour
+          tmSnapshot->tm_min,            // minute
+          tmSnapshot->tm_sec,            // second
+          tNow - tSnapshot,              // duration  
+          tSnapshot - tLastSnapshot      // delta
          );   
 
   m_sParameterStatus = DBA_string(szBuffer);
@@ -247,24 +243,24 @@ DBA_void DBA_protocol::ClearHistoryText()
 //
 // SetHistoryStatus
 //
-DBA_void DBA_protocol::SetHistoryStatus(DBA_ulong nCount, time_t tAct)
+DBA_void DBA_protocol::SetHistoryStatus(DBA_ulong nCount, time_t tSnapshot)
 {
-  m_tAct  = tAct;
+  m_tSnapshot  = tSnapshot;
 
-  struct tm *tmAct;
-  tmAct = localtime( &tAct );
+  struct tm *tmSnapshot;
+  tmSnapshot = localtime( &tSnapshot );
 
   DBA_char szBuffer[1024];
 
   sprintf(szBuffer,
   "===== #%-10d at %04d-%02d-%02d %02d:%02d:%02d\n", 
-           nCount,                   // count
-           tmAct->tm_year+1900,      // year
-           tmAct->tm_mon+1,          // month
-           tmAct->tm_mday,           // day
-           tmAct->tm_hour,           // hour
-           tmAct->tm_min,            // minute
-           tmAct->tm_sec             // second
+           nCount,                        // count
+           tmSnapshot->tm_year+1900,      // year
+           tmSnapshot->tm_mon+1,          // month
+           tmSnapshot->tm_mday,           // day
+           tmSnapshot->tm_hour,           // hour
+           tmSnapshot->tm_min,            // minute
+           tmSnapshot->tm_sec             // second
           );   
 
   m_sHistoryStatus = DBA_string(szBuffer);
@@ -493,7 +489,7 @@ DBA_string DBA_protocol::GetActDir()
   DBA_char szDir[10];
   
   struct tm *tmAct;
-  tmAct = localtime( &m_tAct );
+  tmAct = localtime( &m_tSnapshot );
 
   sprintf(szDir,"%04d%02d%02d", tmAct->tm_year+1900, tmAct->tm_mon+1, tmAct->tm_mday);   
 
@@ -620,7 +616,8 @@ DBA_bool DBA_protocol::CheckAndSaveInfo
         DBA_int     nInterval,
         DBA_bool    bReconnect,
         DBA_uint    nSid,
-        DBA_uint    nPid)
+        DBA_uint    nPid,
+        DBA_string  sDriver)
 {
 
   DBA_file hFile;
@@ -673,6 +670,7 @@ Interval          = %d\n\
 Reconnect         = %s\n\
 ProcessID         = %u\n\
 SessionID         = %u\n\
+Driver            = %s\n\
 Started           = %04d-%02d-%02d %02d:%02d:%02d\n", 
             sCheckDatabase.CharPtr(), 
             sNewDatabase.CharPtr(),
@@ -684,6 +682,7 @@ Started           = %04d-%02d-%02d %02d:%02d:%02d\n",
             bReconnect ? "true" : "false",
             (unsigned int) nPid,
             (unsigned int) nSid,
+            sDriver.CharPtr(), 
             tmAct->tm_year+1900,      // year
             tmAct->tm_mon+1,          // month
             tmAct->tm_mday,           // day
