@@ -45,12 +45,6 @@
 #ifndef DATA_BASEPAGE_HPP
 #define DATA_BASEPAGE_HPP
 
-
-
-/*===========================================================================*
- *  INCLUDES                                                                 *
- *===========================================================================*/
-
 #include "gbd00.h"
 
 #include "Converter/Converter_Version.hpp"
@@ -59,19 +53,7 @@
 #include "KernelCommon/Kernel_Common.hpp"
 #include "SAPDBCommon/SAPDB_PersistentTypes.hpp"
 
-/*===========================================================================*
- *  DEFINES                                                                  *
- *===========================================================================*/
-
-
-/*===========================================================================*
- *  MACROS                                                                   *
- *===========================================================================*/
-
-
-/*===========================================================================*
- *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
- *===========================================================================*/
+class Data_PageAccessManager;
 
 /*!*****************************************************************************
 
@@ -88,8 +70,9 @@
 
 class Data_BasePage : public Kernel_Page72
 {
+/// The page access manager is a friend of this class.
+friend class Data_PageAccessManager;
 public:
-
     /* -----------------------------------------------------------------------*/
     /*! chapter: Type definitions                                             */
     /* -----------------------------------------------------------------------*/
@@ -118,7 +101,12 @@ public:
     return value: none
     --------------------------------------------------------------------------*/
 
-    Data_BasePage(){}
+    Data_BasePage ()
+    : m_AccessMode     (Data_ForRead),
+      m_ReleaseMode    (Data_Unchanged),
+      m_LruChainInfo   (Data_LruNormal),
+      m_pAccessManager (NULL)
+    {}
 
     /*!-------------------------------------------------------------------------
     function:     Data_BasePage()
@@ -130,7 +118,13 @@ public:
     return value: none
     --------------------------------------------------------------------------*/
 
-    Data_BasePage( const PageFrame &frame ):Kernel_Page72( frame ){}
+    Data_BasePage( const PageFrame &frame )
+    : Kernel_Page72    (frame),
+      m_AccessMode     (Data_ForRead),
+      m_ReleaseMode    (Data_Unchanged),
+      m_LruChainInfo   (Data_LruNormal),
+      m_pAccessManager (NULL)
+    {}
 
     /*!-------------------------------------------------------------------------
     function:     Data_BasePage()
@@ -146,7 +140,11 @@ public:
     Data_BasePage(
         const PageFrame             &frame,
         const Data_PageRecoveryMode &recoveryMode)
-    : Kernel_Page72( frame )
+    : Kernel_Page72    (frame),
+      m_AccessMode     (Data_ForRead),
+      m_ReleaseMode    (Data_Unchanged),
+      m_LruChainInfo   (Data_LruNormal),
+      m_pAccessManager (NULL)
     {
         SetRecoveryMode( recoveryMode );
     }
@@ -162,9 +160,12 @@ public:
     return value: none
     --------------------------------------------------------------------------*/
 
-    Data_BasePage(
-        tbd_nodeptr pNode,
-        void *pCblock = NULL )
+    Data_BasePage( tbd_nodeptr  pNode,
+                   void        *pCblock = NULL )
+    : m_AccessMode     (Data_ForRead),
+      m_ReleaseMode    (Data_Unchanged),
+      m_LruChainInfo   (Data_LruNormal),
+      m_pAccessManager (NULL)
     {
         m_Frame.Assign( pNode, sizeof( *pNode ));
         SetHint( pCblock);
@@ -189,6 +190,10 @@ public:
         SAPDB_Bool  bIsDynamic,
         SAPDB_Bool  bIsTemp,
         void *pCblock = NULL )
+    : m_AccessMode     (Data_ForRead),
+      m_ReleaseMode    (Data_Unchanged),
+      m_LruChainInfo   (Data_LruNormal),
+      m_pAccessManager (NULL)
     {
         m_Frame.Assign( pNode, sizeof( *pNode ));
         SetHint( pCblock);
@@ -255,6 +260,16 @@ public:
             SetRecoveryMode( Data_PageRecoveryMode( Data_Dynamic, Data_Recoverable ));
     }
 
+    /// This initializes all node header fields used here.
+    virtual void InitializeFrame ()
+    {
+        // many parts of the Kernel72_Page frame were initialized in vbd13.cpp
+        /*
+        NodePtr()->nd_id()     = SAPDB_MAX_INT4;
+        NodePtr()->nd_root()   = SAPDB_MAX_INT4;
+        NodePtr()->nd_bottom() = MinFreeOffset();
+        */
+    }
     /* -----------------------------------------------------------------------*/
     /*! endchapter: Constructors and initialization                           */
     /* -----------------------------------------------------------------------*/
@@ -442,6 +457,73 @@ protected:
         return( PersistentHeaderPtr()->m_RemainingNodeHeader.m_RecordCount );
     }
 
+protected:
+    /// This returns a pointer to the page frame.
+    SAPDB_Byte * FramePtr()
+    {
+        return m_Frame.GetDataPtr();
+    }
+    /// This returns a const pointer to the page frame.
+    const SAPDB_Byte * FramePtr() const
+    {
+        return m_Frame.GetDataPtr();
+    }
+    /// This returns a formatted pointer to the frame.
+    tbd_node* NodePtr()
+    {
+        return REINTERPRET_CAST (tbd_node*, FramePtr());
+    }
+    /// This returns a formatted const pointer to the frame.
+    const tbd_node* NodePtr() const
+    {
+        return REINTERPRET_CAST (const tbd_node*, FramePtr());
+    }
+    /// This copies a new node to the current frame.
+    void SetNodePtr(tbd_node* pNode)
+    {
+        m_Frame.Assign (pNode, sizeof(tbd_node));
+    }
+
+public:
+
+    /// PageId becomes the id from the page frame.
+    void PageId (Data_PageId& PageId) const
+    {
+        SAPDBERR_ASSERT_STATE(IsAssigned());
+        PageId.SetPageNo          (NodePtr()->nd_id());
+        PageId.SetPageRecoveryMode( RecoveryMode() );
+    }
+    /// Give a hint to Release that there had been no changes to the page frame.
+    void SetUnChanged()
+    {
+        m_ReleaseMode = Data_Unchanged;
+    }
+    /// Give a hint to Release that there had been changes to the page frame.
+    void SetChanged()
+    {
+        m_ReleaseMode = Data_Changed;
+    }
+    /// Set a hint for the datacache.
+    void SetLruChainInfo( const Data_LRUChainInfo   lruChainInfo )
+    {
+        m_LruChainInfo = lruChainInfo;
+    }
+    /// return the set release mode
+    Data_ReleaseMode ReleaseMode () const
+    {
+        return m_ReleaseMode;
+    }
+    /// return the set access mode
+    Data_AccessMode AccessMode () const
+    {
+        return m_AccessMode;
+    }
+    /// set the access mode
+    void SetAccessMode (Data_AccessMode AccessMode)
+    {
+        m_AccessMode = AccessMode;
+    }
+
 protected:
 
     /*-----------------------------------------------------------------------------
@@ -453,6 +535,15 @@ protected:
 
     Data_PageRecoveryMode   m_RecoveryMode;
 
+    /// the access mode
+    Data_AccessMode m_AccessMode;
+    /// the release mode
+    Data_ReleaseMode m_ReleaseMode;
+    /// the data cache hint
+    Data_LRUChainInfo m_LruChainInfo;
+    /// the used data access manager
+    Data_PageAccessManager* m_pAccessManager;
+
 #if defined(AIX) || defined(HPUX) || defined(SUN)
 public:
 #else
