@@ -35,7 +35,7 @@
 #include "DataAccess/Data_Exceptions.hpp"
 #include "DataAccess/Data_Messages.hpp"
 #include "DataAccess/Data_ChainPage.hpp"
-#include "DataAccess/Data_IPageAccessManager.hpp"
+#include "DataAccess/Data_PageAccessManager.hpp"
 
 /*!
    @interface      Data_IBreakable
@@ -65,7 +65,7 @@ public:
        It points to the root of the given chain.
        If the chain is not created, the iterator is invalid.
      */
-    Data_ChainIterator<T> (Data_IPageAccessManager& PAM)
+    Data_ChainIterator<T> (Data_PageAccessManager& PAM)
     : m_PAM(PAM)
     {}
     /*!
@@ -201,7 +201,7 @@ private:
     /// This identifier is the main entry point to a persistent page chain.
     T m_Page;
     /// This is used to access pages.
-    Data_IPageAccessManager& m_PAM;
+    Data_PageAccessManager& m_PAM;
 };
 
 /// A persistent page chain handler, based on data pages. It is a double linked chain.
@@ -215,7 +215,7 @@ public:
        
        The RootId is saved internally for accessing the chain.
      */
-    Data_Chain (Data_IPageAccessManager& PAM,
+    Data_Chain (Data_PageAccessManager& PAM,
                 Data_PageId&             RootId)
     : m_PAM    (PAM),
       m_RootId (RootId)
@@ -228,7 +228,7 @@ public:
        
        The RootId is saved internally for accessing the chain.
      */
-    Data_Chain (Data_IPageAccessManager &PAM,
+    Data_Chain (Data_PageAccessManager &PAM,
                 Data_PageId             &RootId,
                 Data_PageNo              LastPageNo)
     : m_PAM        (PAM),
@@ -474,7 +474,7 @@ public:
         ReleasePage (rootIter);
     }
     /// This returns  the internally used page access manager handler.
-    Data_IPageAccessManager& GetPageAccessManager()
+    Data_PageAccessManager& GetPageAccessManager()
     {
         return m_PAM;
     }
@@ -714,7 +714,7 @@ protected:
     /// This Id is stored within the first page of the chain.
     Data_PageNo m_LastPageNo;
     /// This is used to access pages.
-    Data_IPageAccessManager& m_PAM;
+    Data_PageAccessManager& m_PAM;
     /// This is used to realize a lock protocol.
     T m_RootPage;
 };
