@@ -60,7 +60,7 @@ public:
     friend class Data_ChainFixSizeSpace<PAGE>;
     public:
         /// The iterater is not initialized and at first invalid.
-        Iterator (Data_IPageAccessManager& PAM)
+        Iterator (Data_PageAccessManager& PAM)
         : m_PAM        (PAM),
           m_PageIter   (PAM)
         {}
@@ -261,7 +261,7 @@ public:
         }
     private:
         /// the page access manager handle
-        Data_IPageAccessManager& m_PAM;
+        Data_PageAccessManager& m_PAM;
         /// internal record iterator
         RecordIterator m_RecordIter;
         /// internal page iterator
@@ -294,7 +294,7 @@ public:
        @param          FixRecordLength [in] 
        @brief          if the rootid is valid, the given fix-record-size is checked.
      */
-    Data_ChainFixSizeSpace (Data_IPageAccessManager &PAM,
+    Data_ChainFixSizeSpace (Data_PageAccessManager &PAM,
                             Data_PageId             &RootId,
                             Data_RecordLength        FixRecordLength)
     : Data_Chain<PAGE>  (PAM, RootId),
@@ -311,7 +311,7 @@ public:
        @param          LastPageNo [in] this id defines the ending of the page chain
        @param          FixRecordLength [in] 
      */
-    Data_ChainFixSizeSpace (Data_IPageAccessManager &PAM,
+    Data_ChainFixSizeSpace (Data_PageAccessManager &PAM,
                             Data_PageId             &RootId,
                             Data_PageNo              LastPageNo,
                             Data_RecordLength        FixRecordLength)
