@@ -64,7 +64,7 @@ public:
         /*!
            @brief The iterater is not initialized and at first invalid.
          */
-        Iterator (Data_IPageAccessManager& PAM,
+        Iterator (Data_PageAccessManager& PAM,
                   SAPDBMem_IRawAllocator&  Allocator)
         : m_PAM                   (PAM),
           m_RecordIterator        (),
@@ -424,7 +424,7 @@ public:
         }
     private:
         /// page access manager
-        Data_IPageAccessManager& m_PAM;
+        Data_PageAccessManager& m_PAM;
         /// internally used record iterator
         Data_SplitSpaceBackwardReadIterator m_RecordIterator;
         /// This refers to the space of the current record.
@@ -457,7 +457,7 @@ public:
         @param PAM    [in/out] this is the page access manager
         @param RootId [in/out] this specifies the first page of the page chain
      */
-    Data_ChainSplitSpaceBackwardRead (Data_IPageAccessManager &PAM,
+    Data_ChainSplitSpaceBackwardRead (Data_PageAccessManager &PAM,
                                       Data_PageId             &RootId)
     : Data_Chain<PAGE> (PAM, RootId)
     {}
@@ -474,7 +474,7 @@ public:
         @param RootId [in/out] this specifies the first page of the page chain
         @param LastPageNo [in] this specifies the last page of the page chain
      */
-    Data_ChainSplitSpaceBackwardRead (Data_IPageAccessManager &PAM,
+    Data_ChainSplitSpaceBackwardRead (Data_PageAccessManager &PAM,
                                       Data_PageId             &RootId,
                                       Data_PageNo              LastPageNo)
     : Data_Chain<PAGE> (PAM, RootId, LastPageNo)
@@ -602,7 +602,6 @@ public:
                     
                     CurrentPage.Append (NextPage, this->LastPageNo());
                     NextPage.End(RecordIterator.m_RecordIterator);
-                    ++numberOfAddedPages; // PTS 1121659 UH 2003-04-30
                     
                     SAPDBERR_ASSERT_STATE( this->LastPageNo() == NextPage.PageNo() );
                     SAPDBERR_ASSERT_STATE( CurrentSpace != NULL );
@@ -615,18 +614,14 @@ public:
                 {
                     // the space was not big enough for FirstPartMiniumSize
                     PAGE& CurrentPage = Space.GetCurrentPage();
-                    PAGE  OldPage;
+                    // PTS 1133123 UH 2004-12-22 keep old last page
+                    PAGE& OldPage = RecordIterator.GetRecordSpace().GetPrevPage();
 
                     OldPage.Assign (CurrentPage);
                     OldPage.Append (CurrentPage, this->LastPageNo());
                     CurrentPage.End(RecordIterator.m_RecordIterator);
-                    SAPDBERR_ASSERT_STATE( this->LastPageNo() == CurrentPage.PageNo() );
-                    //#ifdef SAPDB_QUICK
-                    if ( CurrentPage.RootPageNo() != OldPage.RootPageNo() )
-                        RTE_Crash(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"INVALID ROOT2"));
-                    //#endif
-                    ++numberOfAddedPages; // PTS 1121659 UH 2003-04-30
                 }
+                ++numberOfAddedPages; // PTS 1121659 UH 2003-04-30
             }
         }
         while ( RestWantedSize > 0 );        
@@ -641,13 +636,12 @@ public:
        @brief All changes done be ReserveSpace() are undone.
        @param RecordIterator [in] The RecordIterator used by ReserveSpace
      */
-    void UndoReserveSpace (Iterator& RecordIterator)
+    void UndoReserveSpace (const RecordSpace &space)
     {
 		// PTS 1117126 UH 2002-08-07 new
+        // PTS 1133123 UH 2004-12-22 rewritten, input of iterator changed to space
         SAPDBTRACE_METHOD_DEBUG ("Data_ChainSplitSpaceBackwardRead::UndoReserveSpace", DataChain_Trace, 5);
-        SAPDBERR_ASSERT_STATE ( RecordIterator.IsValid() );
-
-        this->LastPageNo() = (*RecordIterator).GetPage(0).PageNo();
+        this->LastPageNo() = space.GetFirstPageNo();
     }
     /// The RecordIterator is set to the last record in the chain.
     void End (Iterator&       RecordIterator,
