@@ -65,7 +65,7 @@ public:
             @param          PAM [in] The AccessManager to use by the Iterator.
             @param          Allocator [in] Is used for the Split space.
          */
-        Iterator (Data_IPageAccessManager& PAM,
+        Iterator (Data_PageAccessManager& PAM,
                   SAPDBMem_IRawAllocator&  Allocator)
         : m_PAM                   (PAM),
           m_RecordIterator        (),
@@ -373,7 +373,7 @@ public:
         }
     private:
         /// The used page access manager
-        Data_IPageAccessManager& m_PAM;
+        Data_PageAccessManager& m_PAM;
         /// The internally used record iterator.
         Data_SplitSpaceForwardReadIterator m_RecordIterator;
         /// This refers to the space of the current record.
@@ -406,7 +406,7 @@ public:
        @param PAM [in] the page access manager which must be used
        @param RootId [in] this id defines the beginning of the page chain
      */
-    Data_ChainSplitSpaceForwardRead (Data_IPageAccessManager &PAM,
+    Data_ChainSplitSpaceForwardRead (Data_PageAccessManager &PAM,
                                       Data_PageId             &RootId)
     : Data_Chain<PAGE> (PAM, RootId)
     {}
@@ -416,7 +416,7 @@ public:
        @param RootId [in] this id defines the beginning of the page chain
        @param LastPageNo [in] this id defines the known last page no.
      */
-    Data_ChainSplitSpaceForwardRead (Data_IPageAccessManager &PAM,
+    Data_ChainSplitSpaceForwardRead (Data_PageAccessManager &PAM,
                                      Data_PageId             &RootId,
                                      Data_PageNo              LastPageNo)
     : Data_Chain<PAGE> (PAM, RootId, LastPageNo)
@@ -464,8 +464,7 @@ public:
         
         SAPDBERR_ASSERT_STATE( WantedSize >= FirstPartMiniumSize );
 
-        SAPDBTRACE_WRITELN (DataChain_Trace, 6, "Wanted: " << WantedSize << \
-                                           ", FirstPartMin: " << FirstPartMiniumSize);
+        SAPDBTRACE_WRITELN (DataChain_Trace, 6, "Wanted: " << WantedSize << ", FirstPartMin: " << FirstPartMiniumSize);
 
         // PTS 1115593 UH 2002-05-07
 
@@ -550,8 +549,10 @@ public:
                 else
                 {
                     // the space was not big enough for FirstPartMiniumSize
+                    // This is only entered at the first time once
                     PAGE& CurrentPage = RecordIterator.GetRecordSpace().GetCurrentPage();
-                    PAGE  OldPage;
+                    // PTS 1133123 UH 2004-12-22 keep old last page
+                    PAGE& OldPage = RecordIterator.GetRecordSpace().GetPrevPage();
                     
                     OldPage.Assign (CurrentPage);
                     OldPage.Append (CurrentPage, this->LastPageNo());
@@ -570,13 +571,12 @@ public:
         return true;
     }
     /// All changes done be ReserveSpace() are undone.
-    void UndoReserveSpace (Iterator& RecordIterator)
+    void UndoReserveSpace (const RecordSpace &space)
     {
 		// PTS 1117126 UH 2002-08-07 new
+        // PTS 1133123 UH 2004-12-22 rewritten, input of iterator changed to space
         SAPDBTRACE_METHOD_DEBUG ("Data_ChainSplitSpaceForwardRead::UndoReserveSpace", DataChain_Trace, 5);
-        SAPDBERR_ASSERT_STATE ( RecordIterator.IsValid() );
-
-        this->LastPageNo() = (*RecordIterator).GetPage(0).PageNo();
+        this->LastPageNo() = space.GetFirstPageNo();
     }
     /// The RecordIterator is set to the first record in the chain. The internal RecordSpace is assigned.
     void Begin (Iterator&       RecordIterator,
