@@ -45,8 +45,6 @@
 #include "KernelCommon/Kernel_VTrace.hpp"
 #include "KernelCommon/Kernel_IAdminConfig.hpp"
 #include "KernelCommon/Kernel_Migration.hpp"
-#include "Converter/Converter_IPageNoManager.hpp"
-#include "Converter/Converter_IVerify.hpp"
 #include "DataAccess/Data_PageAccessManager.hpp"
 #include "DataAccess/Data_Exceptions.hpp"
 #include "DataAccess/Data_Messages.hpp"
@@ -68,7 +66,7 @@
  *  LOCAL FUNCTIONS                                                          *
  *===========================================================================*/
 
-inline static tbd_node_request_Param
+inline tbd_node_request_Param
 ConvertToRequestMode (Data_AccessMode Mode)
 {
     switch (Mode)
@@ -86,7 +84,7 @@ ConvertToRequestMode (Data_AccessMode Mode)
 
 /* --------------------------------------------------------------------------- */
 
-inline static tbd_lru_info_Param
+inline tbd_lru_info_Param
 ConvertToChainInfo( const Data_LRUChainInfo chainInfo )
 {
     switch( chainInfo )
@@ -107,9 +105,10 @@ ConvertToChainInfo( const Data_LRUChainInfo chainInfo )
 
 /* --------------------------------------------------------------------------- */
 Data_PageAccessManager::Data_PageAccessManager
-    ( tgg00_TransContext& TransContext,
-      Data_FileType       FileType,
-      Data_PageNo         RootPageNo)
+    ( tgg00_TransContext&           TransContext,
+      const Data_FileType           FileType,
+      const Data_PageRecoveryMode&  RecoveryMode,
+      const Data_PageNo             RootPageNo)
 {
     SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::Data_PageAccessManager", DataPam_Trace, 5);
 
@@ -126,6 +125,7 @@ Data_PageAccessManager::Data_PageAccessManager
          SearchForChangeInfo("historyRootIdCheck").IsEnabled() ) // PTS 1114877 UH 2002-03-18
         FileId.fileRoot_gg00() = RootPageNo; // PTS 1114728 UH 2002-03-11
 
+ 
     switch (FileType)
     {
     case Data_UnknownFile:
@@ -164,11 +164,23 @@ Data_PageAccessManager::Data_PageAccessManager
         FileId.fileType_gg00().addElement (ftsPerm_egg00);
         break;
     case Data_TableFile:
-        FileId.fileTfn_gg00().becomes(tfnTable_egg00);
-        FileId.fileType_gg00().clear();
-        FileId.fileType_gg00().addElement (ftsConcurrent_egg00);
-        FileId.fileType_gg00().addElement (ftsPerm_egg00);
-        break;
+        {
+            if( RecoveryMode.PageIsRecoverable())
+            {
+                FileId.fileTfn_gg00().becomes(tfnTable_egg00);
+                FileId.fileType_gg00().clear();
+                FileId.fileType_gg00().addElement (ftsConcurrent_egg00);
+                FileId.fileType_gg00().addElement (ftsPerm_egg00);
+            }
+            else
+            {
+                FileId.fileTfn_gg00().becomes(tfnTemp_egg00);
+                FileId.fileType_gg00().clear();
+                FileId.fileType_gg00().addElement (ftsConcurrent_egg00);
+                FileId.fileType_gg00().addElement (ftsTemp_egg00);
+            }
+            break;
+        }
     case Data_IndexFile:
         RTE_Crash( SAPDBErr_Exception(__CONTEXT__, SAPDBERR_ASSERT_STATE_FAILED, "invalid filetype") );
         break;
@@ -179,46 +191,8 @@ Data_PageAccessManager::Data_PageAccessManager
         RTE_Crash( SAPDBErr_Exception(__CONTEXT__, SAPDBERR_ASSERT_STATE_FAILED, "invalid filetype") );
     }
 }
-
 /* --------------------------------------------------------------------------- */
-Data_PageAccessManager::Data_PageAccessManager (Data_PageAccessManager  &Pam,
-                                                Data_FileType            FileType,
-                                                Data_PageNo              RootPageNo)
-{
-    SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::Data_PageAccessManager(pam)", DataPam_Trace, 5);
-    *this = Data_PageAccessManager ( *(Pam.m_Current.curr_trans), FileType, RootPageNo);
-}
-
-/* --------------------------------------------------------------------------- */
-void Data_PageAccessManager::RegisterToPage (Data_Page&                   Page,
-        tbd_node_ptrs&               Nptrs,
-        const Data_PageRecoveryMode& RecoveryMode,
-        Data_AccessMode              AccessMode)
-{
-    SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::RegisterToPage", DataPam_Trace, 5);
-
-    Page.m_pAccessManager = this;
-    Page.m_AccessMode     = AccessMode;
-    Page.SetRecoveryMode(RecoveryMode);
-    Page.SetHint(Nptrs.np_cbptr());
-    Page.SetNodePtr(Nptrs.np_ptr());
-
-    if ( Data_ForUpdate          == Page.m_AccessMode ||
-            Data_ForStructureChange == Page.m_AccessMode )
-        Page.m_ReleaseMode = Data_Changed;
-    else
-        Page.m_ReleaseMode = Data_Unchanged;
-
-    if ( DataPam_Trace.TracesLevel(6) )
-    {
-        Kernel_VTrace() << "registered: Pno:" << Page.PageNo()
-        << ", AccessMode: "   << Page.m_AccessMode
-        << ", ReleaseMode: "  << Page.m_ReleaseMode;
-    }
-}
-
-/* --------------------------------------------------------------------------- */
-bool Data_PageAccessManager::GetPage (Data_Page                   &Page,
+bool Data_PageAccessManager::GetPage (Data_BasePage                   &Page,
                                       const Data_PageNo           &PageNo,
                                       const Data_PageRecoveryMode &RecoveryMode,
                                       Data_AccessMode              AccessMode)
@@ -270,26 +244,7 @@ bool Data_PageAccessManager::GetPage (Data_Page                   &Page,
 }
 
 /* --------------------------------------------------------------------------- */
-bool Data_PageAccessManager::GetPage (Data_Page                   &Page,
-                                      const Data_PageNo           &PageNo,
-                                      Data_AccessMode              AccessMode)
-{
-    SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::GetPage2", DataPam_Trace, 5);
-
-    return GetPage (Page, PageNo, Page.RecoveryMode(), AccessMode);
-}
-
-/* --------------------------------------------------------------------------- */
-bool Data_PageAccessManager::GetPage (Data_Page                   &Page,
-                                      const Data_PageNo           &PageNo)
-{
-    SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::GetPage3", DataPam_Trace, 5);
-
-    return GetPage (Page, PageNo, Page.RecoveryMode(), Page.AccessMode());
-}
-
-/* --------------------------------------------------------------------------- */
-void Data_PageAccessManager::ReleasePage (Data_Page &Page)
+void Data_PageAccessManager::ReleasePage (Data_BasePage &Page)
 {
     SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::ReleasePage", DataPam_Trace, 5);
 
@@ -309,24 +264,6 @@ void Data_PageAccessManager::ReleasePage (Data_Page &Page)
         tgg00_BasisError AuxError = m_Current.curr_trans->trError_gg00;
         m_Current.curr_trans->trError_gg00 = e_ok;
 
-        if( g01glob.bd_lock_check )
-        {   // TEST BEGIN PTS 1128448 TS 2004-03-18
-            // Dieses Stck Code wird nur in diesem Release temporre 
-            // eingebaut und verschwindet in den nchsten Tagen wieder!
-            const tbd_nodeptr   pRoot  = m_Current.currRootNptrs_bd00.np_ptr();
-            const tbd_nodeptr   pIndex = m_Current.currIndexNptrs_bd00.np_ptr();
-            
-            if( ( AuxPageNo != m_Current.curr_tree_id.fileRoot_gg00()) &&
-                (
-                (( NULL != pRoot  ) && ( pRoot == Nptrs.np_ptr())) ||
-                (( NULL != pIndex ) && ( pIndex == Nptrs.np_ptr()))
-                )
-                )
-            {
-                RTE_Crash( SAPDBErr_Exception(__CONTEXT__, SAPDBERR_ASSERT_STATE_FAILED, "corrupted PAM current") );
-            }
-        } // TEMPORARY TEST END
-
         if ( Data_Changed == Page.m_ReleaseMode )
             b13w_release_node (Nptrs, m_Current);
         else
@@ -362,7 +299,7 @@ void Data_PageAccessManager::ReleasePage (Data_Page &Page)
 }
 
 /* --------------------------------------------------------------------------- */
-void Data_PageAccessManager::NewPage (Data_Page&            Page,
+void Data_PageAccessManager::NewPage (Data_BasePage&            Page,
                                       Data_PageRecoveryMode Mode)
 {
     SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::NewPage", DataPam_Trace, 5);
@@ -412,7 +349,7 @@ void Data_PageAccessManager::NewPage (Data_Page&            Page,
 }
 
 /* --------------------------------------------------------------------------- */
-void Data_PageAccessManager::FreePage (Data_Page& Page)
+void Data_PageAccessManager::FreePage (Data_BasePage& Page)
 {
     SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::FreePage", DataPam_Trace, 5);
 
@@ -492,54 +429,4 @@ void Data_PageAccessManager::FreePageNo (
                                   SAPDB_ToString(m_Current.curr_trans->trError_gg00,_T_d) ) );
 
     m_Current.curr_trans->trError_gg00 = AuxError;
-}
-
-/* --------------------------------------------------------------------------- */
-void Data_PageAccessManager::Invalidate()
-{
-    m_Current.curr_tree_id.fileRoot_gg00() = NIL_PAGE_NO_GG00;
-}
-
-/* --------------------------------------------------------------------------- */
-bool Data_PageAccessManager::IsValid() const
-{
-    return Data_PageNo(m_Current.curr_tree_id.fileRoot_gg00()).IsValid();
-}
-
-/* --------------------------------------------------------------------------- */
-bool Data_PageAccessManager::CheckSpace( SAPDB_UInt pagecount ) const
-{
-    // PTS 1115170 UH 2002-04-09 new
-    SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::CheckSpace", DataPam_Trace, 5);
-
-    Converter_IPageNoManager::Instance().HandleDBFull( *(m_Current.curr_trans), pagecount);
-
-    return e_ok == m_Current.curr_trans->trError_gg00;
-}
-
-/* --------------------------------------------------------------------------- */
-void Data_PageAccessManager::MarkPageIdAsUsed (const Data_PageId &pageid)
-{
-    // 2003-05-09 UH added
-    SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::MarkPageIdAsUsed", DataPam_Trace, 5);
-
-    Converter_IVerify::Instance().MarkPageNoAsUsed ( m_Current.curr_trans->trTaskId_gg00,
-                                                     pageid.PageNo(),
-                                                     pageid.PageRecoveryMode().GetAddressingMode() );
-}
-
-/* --------------------------------------------------------------------------- */
-SAPDB_Int Data_PageAccessManager::GetLastError () const
-{
-    // 2003-05-09 UH added
-    SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::GetLastError", DataPam_Trace, 5);
-    return m_Current.curr_trans->trError_gg00;
-}
-
-/* --------------------------------------------------------------------------- */
-void Data_PageAccessManager::ResetLastError()
-{
-    // 2003-05-09 UH added
-    SAPDBTRACE_METHOD_DEBUG ("Data_PageAccessManager::ResetLastError", DataPam_Trace, 5);
-    m_Current.curr_trans->trError_gg00 = e_ok;
-}
+}
\ No newline at end of file
