@@ -1,22 +1,10 @@
-/****************************************************************************
-
-  module      : Data_RandomTreeAccess.cpp
-
-  -------------------------------------------------------------------------
-
-  responsible : TorstenS
-
-  special area:
-  description : 
-
-  last changed: 2002-04-18
-  see also    :
-
-  -------------------------------------------------------------------------
-
-  copyright:    (c) 2001-2004 SAP AG
+/*****************************************************************************/
+/*!
 
+  @file   Data_RandomTreeAccess.cpp
+  @author TorstenS
 
+\if EMIT_LICENCE
     ========== licence begin  GPL
     Copyright (c) 2001-2004 SAP AG
 
@@ -34,12 +22,9 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
-*****************************************************************************/
-
-
+\endif
+*/
+/*****************************************************************************/
 
 /*===========================================================================*
  *  INCLUDES                                                                 *
@@ -70,8 +55,6 @@
  *  GLOBAL VARIABLES                                                         *
  *===========================================================================*/
 
-Container_Vector< BitMapSlot >  *Data_IndexNode::m_pBitMap = NULL;
-
 BitMapSlot BIT_MAP_SLOT_MASK[] = { 0x80, 0x40, 0x20, 0x10, 0x8, 0x4, 0x2, 0x1 };
 
 // bitNo 0: 0x80 == 10000000
@@ -160,7 +143,6 @@ Data_RandomTreeAccess::Data_RandomTreeAccess(
         m_IndexNodes( *( reinterpret_cast< SAPDBMem_IRawAllocator* >( trans.trAllocator_gg00 )) ),
         m_HashAnchors( *( reinterpret_cast< SAPDBMem_IRawAllocator* >( trans.trAllocator_gg00 )) )
 {
-    Data_IndexNode::SetBitMap( m_BitMap );
 }
 
 /*---------------------------------------------------------------------------*/
@@ -402,7 +384,7 @@ Data_RandomTreeAccess::AddIndexNode(
         return( SAPDB_TRUE ); //already inserted
 
     HashLink        &anchor = GetHashAnchor( pageNo );
-    Data_IndexNode  node( pageNo, recordCount, level );
+    Data_IndexNode  node( pageNo, recordCount, level, m_BitMap );
 
     if( ! node.Initialize())
     {
