@@ -65,9 +65,7 @@ inline void Check (const char * Message,
                    bool         CheckValue)
 {
     if ( ! CheckValue )
-        RTE_Crash( SAPDBErr_Exception(__FILE__, __LINE__,
-                                      SAPDBERR_ASSERT_STATE_FAILED,
-                                      Message) );
+        RTE_Crash(SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,Message));
 }
 
 /* --------------------------------------------------------------------------- */
@@ -83,7 +81,8 @@ inline void ChainTest (tgg00_TransContext& Trans)
 
     Data_PageId InvalidRootId;
 
-    Data_PageAccessManager PAM (Trans, Data_UnknownFile, Data_PageNo());
+    Data_PageAccessManager PAM (Trans, Data_UnknownFile, 
+        Data_PageRecoveryMode(Data_Dynamic,Data_Recoverable), Data_PageNo());
     
     Data_Chain<Data_ChainPage> Chain (PAM, InvalidRootId);
 
@@ -267,7 +266,9 @@ inline void TestBackwardChainSplitSpaceNegative
     SAPDB_UInt  AddedPages;
     if ( ! Chain.ReserveSpace (sizeof(Block), 10, Iter, AddedPages) )
         RTE_Crash(Data_Exception(__CONTEXT__,DATA_COMPONENT_TEST_FAILED,"ReserveSpace"));
-    Data_SplitSpaceWriter Writer (*Iter);
+        
+    BackwardChainType::RecordSpace &space = *Iter;
+    Data_SplitSpaceWriter Writer (space);
     Writer.Write (Block, sizeof(Block), WriteResult);
     
     Iter.WriteToTrace ("TestBackwardChainSplitSpaceNegative2");
@@ -276,10 +277,8 @@ inline void TestBackwardChainSplitSpaceNegative
         RTE_Crash( Data_Exception(__CONTEXT__,DATA_COMPONENT_TEST_FAILED,
                                   "TestReserveWriteReadNegative::Write") );
 
-    Data_PageNo testPageNoAfter = Chain.LastPageNo();
-    
-    Chain.UndoReserveSpace (Iter);
-    Iter.Invalidate(false);    
+    Chain.UndoReserveSpace(space);
+	Iter.Invalidate(false);
 
     Iter.WriteToTrace ("TestBackwardChainSplitSpaceNegative3");
 
@@ -380,7 +379,8 @@ inline void StartTestBackwardChainSplitSpace (tgg00_TransContext     &Trans,
     SAPDBTRACE_ROUTINE_DEBUG ("Data_Test::StartTestBackwardChainSplitSpace", DataChain_Trace, 5);
 
     Data_PageId                 InvalidRootId;
-    Data_PageAccessManager      PAM   (Trans, Data_UndoFile, Data_PageNo());
+    Data_PageAccessManager      PAM   (Trans, Data_UndoFile, 
+                                       Data_PageRecoveryMode(Data_Dynamic,Data_Recoverable), Data_PageNo());
     BackwardChainType           Chain (PAM, InvalidRootId);
     BackwardChainType::Iterator Iter  (PAM, Allocator);
     
@@ -402,7 +402,8 @@ inline void StartTestForwardChainSplitSpace (tgg00_TransContext     &Trans,
     SAPDBTRACE_ROUTINE_DEBUG ("Data_Test::StartTestForwardChainSplitSpace", DataChain_Trace, 5);
 
     Data_PageId                InvalidRootId;
-    Data_PageAccessManager     PAM   (Trans, Data_RedoFile, Data_PageNo());
+    Data_PageAccessManager     PAM   (Trans, Data_RedoFile, 
+                                      Data_PageRecoveryMode(Data_Dynamic,Data_Recoverable), Data_PageNo());
     ForwardChainType           Chain (PAM, InvalidRootId);
     ForwardChainType::Iterator Iter  (PAM, Allocator);
     
@@ -422,7 +423,8 @@ inline void StartTestFixSizeChain (tgg00_TransContext &Trans)
     SAPDBTRACE_ROUTINE_DEBUG ("Data_Test::StartTestFixSizeChain", DataChain_Trace, 5);
 
     Data_PageId                InvalidRootId;
-    Data_PageAccessManager     PAM    (Trans, Data_OpenTransFile, Data_PageNo());
+    Data_PageAccessManager     PAM    (Trans, Data_OpenTransFile, 
+                                       Data_PageRecoveryMode(Data_Dynamic,Data_Recoverable), Data_PageNo());
     FixSizeChain               Chain  (PAM, InvalidRootId, 2000);
     FixSizeChain::Iterator     Iter   (PAM);
     
@@ -514,5 +516,5 @@ externCpp void Data_Test (tgg00_TransContext& Trans)
     // fast compiler and linker test of new primary file implementation
     Data_PrimaryFile file (Trans);
     
-    TestTempFile (Trans, Allocator);
+    // TestTempFile (Trans, Allocator);
 }
