@@ -635,11 +635,12 @@ public:
                 be checked. In case of a fatal errror during the read operation
                 caused by the IO subsystem the error is logged in the knldiag 
                 file and the return code is false.
-       @param   taskId     [in] Identification of the calling task.
-       @param   pFrame     [in] Memory where the read page will be stored.
-       @param   blockNo    [in] Position of the wanted page within the device.
-       @param   blockCount [in] Number of pages to be read in one step
-                                Default is one page
+       @param   taskId       [in] Identification of the calling task.
+       @param   pFrame       [in] Memory where the read page will be stored.
+       @param   blockNo      [in] Position of the wanted page within the device.
+       @param   bFromPrimary [in] Read from primary log volume of the device
+       @param   blockCount   [in] Number of pages to be read in one step
+                                  Default is one page
        @return  true if the read operation is successful; else false.
      */
     /* -----------------------------------------------------------------------*/
@@ -648,9 +649,13 @@ public:
         const tsp00_TaskId      taskId,
         const tsp00_PageAddr    pFrame,
         const IOMan_BlockNo     blockNo,
+        const SAPDB_Bool        bFromPrimary,
         const IOMan_BlockCount  blockCount = 1 )
     {
-        return( m_PrimaryLogVolume.ReadSimple( taskId, pFrame, blockNo, blockCount ));
+        if( bFromPrimary )
+            return( m_PrimaryLogVolume.ReadSimple( taskId, pFrame, blockNo, blockCount ));
+        else
+            return( m_SecondaryLogVolume.ReadSimple( taskId, pFrame, blockNo, blockCount ));
     }
 
     /* -----------------------------------------------------------------------*/
@@ -704,9 +709,10 @@ public:
                 fatal errror during the write operation caused by the IO subsystem
                 the error is logged in the knldiag file and the return code is 
                 false. 
-       @param   taskId  [in] Identification of the calling task.
-       @param   pFrame  [in] Pointer to the frame
-       @param   blockNo [in] Position for the page within the device.
+       @param   taskId     [in] Identification of the calling task.
+       @param   pFrame     [in] Pointer to the frame
+       @param   blockNo    [in] Position for the page within the device.
+       @param   bToPrimary [in] Write to primary log volume of the device
        @return  true if the write operation is successful; else false.
 
      */
@@ -715,9 +721,13 @@ public:
     virtual SAPDB_Bool WriteSimple(
         const tsp00_TaskId      taskId,
         const tsp00_PageAddr    pFrame,
-        const IOMan_BlockNo     blockNo )
+        const IOMan_BlockNo     blockNo,
+        const SAPDB_Bool        bToPrimary )
     {
-        return( m_PrimaryLogVolume.WriteSimple( taskId, pFrame, blockNo ));
+        if( bToPrimary )
+            return( m_PrimaryLogVolume.WriteSimple( taskId, pFrame, blockNo ));
+        else
+            return( m_SecondaryLogVolume.WriteSimple( taskId, pFrame, blockNo ));
     }
 
     /* -----------------------------------------------------------------------*/
