@@ -289,13 +289,16 @@ IOMan_LogArea::OpenAll(
 SAPDB_Bool
 IOMan_LogArea::GetVolumeNo(
     tsp00_VFilename &devName,
-    IOMan_DeviceNo  &devNo )
+    IOMan_DeviceNo  &devNo,
+    SAPDB_Bool      &bIsPrimary)
 {
     SAPDBTRACE_ROUTINE_DEBUG( "IOMan_LogArea::GetVolumeNo", IOMan_Trace, 6 );
 
     tsp00_VFilename         auxName;
     IOMan_LogDeviceIterator iter( m_LogDevices );
 
+    bIsPrimary = SAPDB_TRUE;
+
     for( iter.Begin(); !iter.End(); ++iter )
     {
         if(( *iter ).GetName( auxName ))
@@ -313,6 +316,7 @@ IOMan_LogArea::GetVolumeNo(
                 if( 0 == memcmp( devName, auxName, sizeof( devName )))
                 {
                     devNo = (*iter).GetLogicalDevNo();
+                    bIsPrimary = SAPDB_FALSE;
                     return( true );
                 }
             }
@@ -474,8 +478,9 @@ IOMan_LogArea::ReIntegrateBadLogVolume(
         return( SAPDB_FALSE );
 
     IOMan_DeviceNo  devNo;
+    SAPDB_Bool      bDummyPrimary;
 
-    if( ! GetVolumeNo( badDevName, devNo ))
+    if( ! GetVolumeNo( badDevName, devNo, bDummyPrimary ))
     {
         SAPDB_Char volName[ sizeof( tsp00_VFilename ) ];
         memcpy( volName, badDevName, sizeof( tsp00_VFilename ));
