@@ -336,16 +336,20 @@ public:
     /*!
        @brief   This method is used to determine the logical identifier for
                 a given log devices name. It is not needed that the log 
-                devices are in mode open.
-       @param   devName [in] 
-       @param   devNo [out] 
+                devices are in mode open. In case of a mirrored 
+                log, it is returned whether the given log volume name identifies 
+                the primary volume or the secondary of the log device.
+       @param   devName     [in] Name of a log volume
+       @param   devNo      [out] Logical volume identifier
+       @param   bIsPrimary [out] Is primary log volume
        @return  true means devices was found; else false
      */
     /* -----------------------------------------------------------------------*/
 
     SAPDB_Bool GetVolumeNo(
         tsp00_VFilename &devName,
-        IOMan_DeviceNo  &devNo );
+        IOMan_DeviceNo  &devNo,
+        SAPDB_Bool      &bIsPrimary );
 
     /* -----------------------------------------------------------------------*/
     /*!
@@ -401,6 +405,17 @@ public:
         return( m_TotalLogPages );
     }
 
+    /*!
+       @brief   This method returns the number of log devices which are 
+                currently in mode open. Usualy all configured log devices 
+                are still in mode open.
+       @return  (SAPDB_Int)
+     */
+
+    SAPDB_Int UsedVolumes() const{
+        return( m_UsedDevices );
+    }
+
     /* -----------------------------------------------------------------------*/
     /*!
        @brief   This method checks the validity of the given log device number
