@@ -482,10 +482,11 @@ public:
                 be checked. In case of a fatal errror during the read operation
                 caused by the IO subsystem the error is logged in the knldiag 
                 file and the return code is false. The device is marked as bad.
-       @param   taskId     [in] Identification of the calling task.
-       @param   pFrame     [in] Memory where the read page will be stored.
-       @param   blockNo    [in] Position of the wanted page within the device.
-       @param   blockCount [in] Number of pages to be read in one step
+       @param   taskId       [in] Identification of the calling task.
+       @param   pFrame       [in] Memory where the read page will be stored.
+       @param   blockNo      [in] Position of the wanted page within the device.
+       @param   bFromPrimary [in] Read from primary log volume of the device
+       @param   blockCount   [in] Number of pages to be read in one step
                                 Default is one page
        @return  true if the read operation is successful; else false.
      */
@@ -495,6 +496,7 @@ public:
         const tsp00_TaskId      taskId,
         const tsp00_PageAddr    pFrame,
         const IOMan_BlockNo     blockNo,
+        const SAPDB_Bool        bFromPrimary,
         const IOMan_BlockCount  blockCount = 1 ) = 0;
 
     /* -----------------------------------------------------------------------*/
@@ -548,9 +550,10 @@ public:
                 fatal errror during the write operation caused by the IO subsystem
                 the error is logged in the knldiag file and the return code is 
                 false. The device is marked as bad.
-       @param   taskId  [in] Identification of the calling task.
-       @param   pFrame  [in] Pointer to the frame
-       @param   blockNo [in] Position for the page within the device.
+       @param   taskId     [in] Identification of the calling task.
+       @param   pFrame     [in] Pointer to the frame
+       @param   blockNo    [in] Position for the page within the device.
+       @param   bToPrimary [in] Write to primary log volume of the device
        @return  true if the write operation is successful; else false.
      */
     /* -----------------------------------------------------------------------*/
@@ -558,7 +561,8 @@ public:
     virtual SAPDB_Bool WriteSimple(
         const tsp00_TaskId      taskId,
         const tsp00_PageAddr    pFrame,
-        const IOMan_BlockNo     blockNo ) = 0;
+        const IOMan_BlockNo     blockNo,
+        const SAPDB_Bool        bToPrimary ) = 0;
 
     /* -----------------------------------------------------------------------*/
     /*@}*/
