@@ -386,22 +386,23 @@ IOMan_Manager::OpenOneLogVolume(
 SAPDB_Bool
 IOMan_Manager::AddDataVolume(
     const tsp00_TaskId      taskId,
-    const SAPDB_Bool        bIsCold,
+    const SAPDB_Bool        bIsCold, // obsolete
     tsp00_VFilename         &devName,
     const IOMan_BlockCount  devSize,
     const IOMan_DeviceNo    devNo )
 {
     SAPDBTRACE_ROUTINE_DEBUG( "IOMan_Manager::AddDataVolume", IOMan_Trace, 5 );
 
-    bool retCode = SAPDB_TRUE;
+    bool        retCode            = SAPDB_TRUE;
+    const bool  bDataAreaRestarted = m_DataArea.ConfiguredVolumes() == m_DataArea.UsedVolumes();
 
-    if( bIsCold )
+    if( ! bDataAreaRestarted )
         retCode = m_DataArea.OpenAll( taskId );
 
     if( retCode )
-        retCode = m_DataArea.AddVolume( taskId, bIsCold, devName, devSize, devNo );
+        retCode = m_DataArea.AddVolume( taskId, ! bDataAreaRestarted, devName, devSize, devNo );
 
-    if( bIsCold )
+    if( ! bDataAreaRestarted )
         m_DataArea.CloseAll( taskId );
 
     return( retCode );
@@ -412,7 +413,7 @@ IOMan_Manager::AddDataVolume(
 SAPDB_Bool
 IOMan_Manager::AddLogVolume(
     const tsp00_TaskId          taskId,
-    const SAPDB_Bool            bIsCold,
+    const SAPDB_Bool            bIsCold, // obsolete
     tsp00_VFilename             &devName,
     tsp00_VFilename             &mirrDevName,
     const IOMan_BlockCount      devSize,
@@ -422,9 +423,11 @@ IOMan_Manager::AddLogVolume(
 {
     SAPDBTRACE_ROUTINE_DEBUG( "IOMan_Manager::AddLogVolume", IOMan_Trace, 5 );
 
-    bool retCode = SAPDB_TRUE;
+    bool        retCode           = SAPDB_TRUE;
+    const bool  bLogAreaRestarted = m_LogArea.ConfiguredVolumes() == m_LogArea.UsedVolumes();
 
-    if( bIsCold )
+
+   if( ! bLogAreaRestarted )
         retCode = m_LogArea.OpenAll( taskId, SAPDB_TRUE );
 
     if( retCode )
@@ -436,7 +439,7 @@ IOMan_Manager::AddLogVolume(
             newLogSize = m_LogArea.TotalUsableLogPages();
     }
 
-    if( bIsCold )
+   if( ! bLogAreaRestarted )
         m_LogArea.CloseAll( taskId ); // PTS 1112474 TS 2001-11-07
 
     return( retCode );
@@ -622,9 +625,8 @@ void IOMan_Manager::ReadLogPageWithoutCheck(
 
     const IOMan_BlockAddress  block = m_LogArea.GetBlockAddress( address.GetOffset() );
 
-    if (!m_LogArea.Get( block.GetDeviceNo() ).ReadSimple( taskId,
-            tsp00_PageAddr( page.GetFramePtr()),
-            block.GetBlockNo() ))
+    if(! m_LogArea.Get( block.GetDeviceNo() ).ReadSimple( taskId,
+            tsp00_PageAddr( page.GetFramePtr()), block.GetBlockNo(), false ))
     {
         Kernel_FileIO   dumpFile( m_LogArea.Get( block.GetDeviceNo() ).GetTypeName(),
                                   KERNEL_BAD_FILE_EXTENSION,
@@ -1056,29 +1058,6 @@ IOMan_Manager::WriteSnapShotRestartPage(
 
 /*---------------------------------------------------------------------------*/
 
-bool
-IOMan_Manager::EnableBadVolume(
-    const tsp00_TaskId  taskId,
-    tsp00_VFilename     &devName )
-{
-    SAPDBTRACE_ROUTINE_DEBUG( "IOMan_Manager::EnableBadVolume", IOMan_Trace, 5 );
-
-    IOMan_DeviceNo  searchedDevNo;
-
-    if( GetDataVolumeNo( devName, searchedDevNo ))
-    {
-        return( IOMan_Okay == m_DataArea.Get( searchedDevNo ).UnSetBad( taskId ));
-    }
-    else if( GetLogVolumeNo( devName, searchedDevNo ))
-    {
-        return( IOMan_Okay == m_LogArea.Get( searchedDevNo ).UnSetBad( taskId ));
-    }
-    else
-        return( false );
-}
-
-/*---------------------------------------------------------------------------*/
-
 bool
 IOMan_Manager::ReadPageDirectFromDataVolume(
     const tsp00_TaskId          taskId,
@@ -1117,7 +1096,8 @@ bool
 IOMan_Manager::WritePageDirectToLogVolume(
     const tsp00_TaskId          taskId,
     const IOMan_BlockAddress    &block,
-    const tsp00_PageAddr        pPage )
+    const tsp00_PageAddr        pPage,
+    const SAPDB_Bool            bToPrimary )
 {
     SAPDBTRACE_ROUTINE_DEBUG( "IOMan_Manager::WritePageDirectToLogVolume", IOMan_Trace, 5 );
 
@@ -1125,7 +1105,7 @@ IOMan_Manager::WritePageDirectToLogVolume(
                         block.GetDeviceNo() << "/" << block.GetBlockNo());
 
     return( m_LogArea.Get( block.GetDeviceNo() ).WriteSimple(
-                taskId, pPage, block.GetBlockNo() ));
+                taskId, pPage, block.GetBlockNo(), bToPrimary ));
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1134,7 +1114,8 @@ bool
 IOMan_Manager::ReadPageDirectFromLogVolume(
     const tsp00_TaskId          taskId,
     const IOMan_BlockAddress    &block,
-    const tsp00_PageAddr        pPage )
+    const tsp00_PageAddr        pPage,
+    const SAPDB_Bool            bFromPrimary )
 {
     SAPDBTRACE_ROUTINE_DEBUG( "IOMan_Manager::ReadPageDirectFromLogVolume", IOMan_Trace, 5 );
 
@@ -1142,7 +1123,7 @@ IOMan_Manager::ReadPageDirectFromLogVolume(
                         block.GetDeviceNo() << "/" << block.GetBlockNo());
 
     return( m_LogArea.Get( block.GetDeviceNo() ).ReadSimple(
-                taskId, pPage, block.GetBlockNo() ));
+                taskId, pPage, block.GetBlockNo(), bFromPrimary ));
 }
 
 /*---------------------------------------------------------------------------*/
