@@ -220,9 +220,10 @@ public:
 
     virtual bool GetLogVolumeNo(
         tsp00_VFilename     &devName,
-        IOMan_DeviceNo      &devNo )
+        IOMan_DeviceNo      &devNo,
+        SAPDB_Bool          &bIsPrimary )
     {
-        return( m_LogArea.GetVolumeNo( devName, devNo ));
+        return( m_LogArea.GetVolumeNo( devName, devNo, bIsPrimary ));
     }
 
     virtual bool GetLogVolumeName(
@@ -372,10 +373,6 @@ public:
         Kernel_RestartPage  &page,
         IOMan_BlockAddress  &block );
 
-    virtual bool EnableBadVolume(
-        const tsp00_TaskId  taskId,
-        tsp00_VFilename     &devName );
-
     virtual bool ReadPageDirectFromDataVolume(
         const tsp00_TaskId          taskId,
         const IOMan_BlockAddress    &block,
@@ -389,12 +386,14 @@ public:
     virtual bool ReadPageDirectFromLogVolume(
         const tsp00_TaskId          taskId,
         const IOMan_BlockAddress    &block,
-        const tsp00_PageAddr        pPage );
+        const tsp00_PageAddr        pPage,
+        const SAPDB_Bool            bFromPrimary );
 
     virtual bool WritePageDirectToLogVolume(
         const tsp00_TaskId          taskId,
         const IOMan_BlockAddress    &block,
-        const tsp00_PageAddr        pPage );
+        const tsp00_PageAddr        pPage,
+        const SAPDB_Bool            bToPrimary );
 
     virtual IOMan_ReturnCode MigrateDataBase( const tsp00_TaskId  taskId );
 
