@@ -568,11 +568,12 @@ public:
                 be checked. In case of a fatal errror during the read operation
                 caused by the IO subsystem the error is logged in the knldiag 
                 file and the return code is false.
-       @param   taskId     [in] Identification of the calling task.
-       @param   pFrame     [in] Memory where the read page will be stored.
-       @param   blockNo    [in] Position of the wanted page within the device.
-       @param   blockCount [in] Number of pages to be read in one step
-                                Default is one page
+       @param   taskId       [in] Identification of the calling task.
+       @param   pFrame       [in] Memory where the read page will be stored.
+       @param   blockNo      [in] Position of the wanted page within the device.
+       @param   bFromPrimary [in] obsolete; not used for non mirrored log devices
+       @param   blockCount   [in] Number of pages to be read in one step
+                                  Default is one page
        @return  true if the read operation is successful; else false.
      */
     /* -----------------------------------------------------------------------*/
@@ -582,6 +583,7 @@ public:
         const tsp00_TaskId      taskId,
         const tsp00_PageAddr    pFrame,
         const IOMan_BlockNo     blockNo,
+        const SAPDB_Bool        bFromPrimary,
         const IOMan_BlockCount  blockCount = 1 )
     {
         return( m_LogVolume.ReadSimple( taskId, pFrame, blockNo, blockCount ));
@@ -644,9 +646,10 @@ public:
                 fatal errror during the write operation caused by the IO subsystem
                 the error is logged in the knldiag file and the return code is 
                 false.
-       @param   taskId  [in] Identification of the calling task.
-       @param   pFrame  [in] pointer to the frame
-       @param   blockNo [in] Position for the page within the device.
+       @param   taskId     [in] Identification of the calling task.
+       @param   pFrame     [in] pointer to the frame
+       @param   blockNo    [in] Position for the page within the device.
+       @param   bToPrimary [in] obsolete; not used for non mirrored log devices
        @return  true if the write operation is successful; else false.
      */
     /* -----------------------------------------------------------------------*/
@@ -654,7 +657,8 @@ public:
     virtual SAPDB_Bool WriteSimple(
         const tsp00_TaskId      taskId,
         const tsp00_PageAddr    pFrame,
-        const IOMan_BlockNo     blockNo )
+        const IOMan_BlockNo     blockNo,
+        const SAPDB_Bool        bToPrimary )
     {
         return( m_LogVolume.WriteSimple( taskId, pFrame, blockNo ));
     }
