@@ -62,14 +62,13 @@ IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateInput);
-    char putdata[2];
-    putdata[0]=0;
+    char putdata[1];
     if(data) {
-        putdata[1]=1;
+        putdata[0]=1;
     } else {
-        putdata[1]=0;
+        putdata[0]=0;
     }
-    datapart.addData(putdata, m_shortinfo.bufpos, m_shortinfo.iolength);
+    datapart.addBinaryParameter(putdata, 1, m_shortinfo);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -81,14 +80,13 @@ IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateInput);
-    char putdata[2];
-    putdata[0]=0;
+    char putdata[1];
     if(data) {
-        putdata[1]=1;
+        putdata[0]=1;
     } else {
-        putdata[1]=0;
+        putdata[0]=0;
     }
-    datapart.addData(putdata, m_shortinfo.bufpos, m_shortinfo.iolength);
+    datapart.addBinaryParameter(putdata, 1, m_shortinfo);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -101,14 +99,13 @@ IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateInput);
-    char putdata[2];
-    putdata[0]=0;
+    char putdata[1];
     if(data) {
-        putdata[1]=1;
+        putdata[0]=1;
     } else {
-        putdata[1]=0;
+        putdata[0]=0;
     }
-    datapart.addData(putdata, m_shortinfo.bufpos, m_shortinfo.iolength);
+    datapart.addBinaryParameter(putdata, 1, m_shortinfo);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -121,14 +118,13 @@ IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateInput);
-    char putdata[2];
-    putdata[0]=0;
+    char putdata[1];
     if(data) {
-        putdata[1]=1;
+        putdata[0]=1;
     } else {
-        putdata[1]=0;
+        putdata[0]=0;
     }
-    datapart.addData(putdata, m_shortinfo.bufpos, m_shortinfo.iolength);
+    datapart.addBinaryParameter(putdata, 1, m_shortinfo);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -140,14 +136,13 @@ IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateInput);
-    char putdata[2];
-    putdata[0]=0;
+    char putdata[1];
     if(data) {
-        putdata[1]=1;
+        putdata[0]=1;
     } else {
-        putdata[1]=0;
+        putdata[0]=0;
     }
-    datapart.addData(putdata, m_shortinfo.bufpos, m_shortinfo.iolength);
+    datapart.addBinaryParameter(putdata, 1, m_shortinfo);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -159,14 +154,13 @@ IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateInput);
-    char putdata[2];
-    putdata[0]=0;
+    char putdata[1];
     if(data) {
-        putdata[1]=1;
+        putdata[0]=1;
     } else {
-        putdata[1]=0;
+        putdata[0]=0;
     }
-    datapart.addData(putdata, m_shortinfo.bufpos, m_shortinfo.iolength);
+    datapart.addBinaryParameter(putdata, 1, m_shortinfo);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -178,14 +172,13 @@ IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateInput);
-    char putdata[2];
-    putdata[0]=0;
+    char putdata[1];
     if(data) {
-        putdata[1]=1;
+        putdata[0]=1;
     } else {
-        putdata[1]=0;
+        putdata[0]=0;
     }
-    datapart.addData(putdata, m_shortinfo.bufpos, m_shortinfo.iolength);
+    datapart.addBinaryParameter(putdata, 1, m_shortinfo);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -197,14 +190,13 @@ IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateInput);
-    char putdata[2];
-    putdata[0]=0;
+    char putdata[1];
     if(data) {
-        putdata[1]=1;
+        putdata[0]=1;
     } else {
-        putdata[1]=0;
+        putdata[0]=0;
     }
-    datapart.addData(putdata, m_shortinfo.bufpos, m_shortinfo.iolength);
+    datapart.addBinaryParameter(putdata, 1, m_shortinfo);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -216,14 +208,13 @@ IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateInput);
-    char putdata[2];
-    putdata[0]=0;
+    char putdata[1];
     if(data != 0.0) {
-        putdata[1]=1;
+        putdata[0]=1;
     } else {
-        putdata[1]=0;
+        putdata[0]=0;
     }
-    datapart.addData(putdata, m_shortinfo.bufpos, m_shortinfo.iolength);
+    datapart.addBinaryParameter(putdata, 1, m_shortinfo);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -235,14 +226,13 @@ IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem &clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateInput);
-    char putdata[2];
-    putdata[0]=0;
-    if(data != 0.0) {
-        putdata[1]=1;
+    char putdata[1];
+    if(data != 0.0f) {
+        putdata[0]=1;
     } else {
-        putdata[1]=0;
+        putdata[0]=0;
     }
-    datapart.addData(putdata, m_shortinfo.bufpos, m_shortinfo.iolength);
+    datapart.addBinaryParameter(putdata, 1, m_shortinfo);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -255,11 +245,14 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
-    if(((unsigned char *)datapart.getParameterData(m_shortinfo.bufpos))[0]) {
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
         data=1;
     } else {
         data=0;
     }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(IFR_Int1);
+    }
     DBUG_RETURN(IFR_OK);
 }
 
@@ -271,11 +264,14 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
-    if(((unsigned char *)datapart.getParameterData(m_shortinfo.bufpos))[0]) {
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
         data=1;
     } else {
         data=0;
     }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(IFR_Int2);
+    }
     DBUG_RETURN(IFR_OK);
 }
 
@@ -287,11 +283,14 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
-    if(((unsigned char *)datapart.getParameterData(m_shortinfo.bufpos))[0]) {
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
         data=1;
     } else {
         data=0;
     }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(IFR_Int4);
+    }
     DBUG_RETURN(IFR_OK);
 }
 
@@ -303,11 +302,14 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
-    if(((unsigned char *)datapart.getParameterData(m_shortinfo.bufpos))[0]) {
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
         data=1;
     } else {
         data=0;
     }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(IFR_Int8);
+    }
     DBUG_RETURN(IFR_OK);
 }
 
@@ -320,11 +322,14 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
-    if(((unsigned char *)datapart.getParameterData(m_shortinfo.bufpos))[0]) {
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
         data=1;
     } else {
         data=0;
     }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(IFR_UInt1);
+    }
     DBUG_RETURN(IFR_OK);
 }
 
@@ -336,11 +341,14 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
-    if(((unsigned char *)datapart.getParameterData(m_shortinfo.bufpos))[0]) {
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
         data=1;
     } else {
         data=0;
     }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(IFR_UInt2);
+    }
     DBUG_RETURN(IFR_OK);
 }
 
@@ -352,11 +360,14 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
-    if(((unsigned char *)datapart.getParameterData(m_shortinfo.bufpos))[0]) {
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
         data=1;
     } else {
         data=0;
     }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(IFR_UInt4);
+    }
     DBUG_RETURN(IFR_OK);
 }
 
@@ -368,11 +379,14 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
-    if(((unsigned char *)datapart.getParameterData(m_shortinfo.bufpos))[0]) {
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
         data=1;
     } else {
         data=0;
     }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(IFR_UInt8);
+    }
     DBUG_RETURN(IFR_OK);
 }
 
@@ -385,11 +399,14 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
-    if(((unsigned char *)datapart.getParameterData(m_shortinfo.bufpos))[0]) {
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
         data=1.0;
     } else {
         data=0.0;
     }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(double);
+    }
     DBUG_RETURN(IFR_OK);
 }
 
@@ -401,14 +418,183 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem & clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
-    if(((unsigned char *)datapart.getParameterData(m_shortinfo.bufpos))[0]) {
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
         data=1.0f;
     } else {
         data=0.0f;
     }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(float);
+    }
     DBUG_RETURN(IFR_OK);
 }
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& datapart,
+                                                SQL_NUMERIC_STRUCT&   data,
+                                                IFR_Length*           lengthindicator,
+                                                IFR_ConnectionItem & clink)
+{
+    DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateOutput);
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
+        data.precision = 1;
+        data.scale     = 0;
+        data.sign      = 1;
+        memset(data.val, 0, SQL_MAX_NUMERIC_LENGTH);
+        data.val[0] = 1;
+    } else {
+        data.precision = 1;
+        data.scale     = 0;
+        data.sign      = 1;
+        memset(data.val, 0, SQL_MAX_NUMERIC_LENGTH);
+    }
+    if(lengthindicator) {
+        *lengthindicator = sizeof(SQL_NUMERIC_STRUCT);
+    }
+    DBUG_RETURN(IFR_OK);
+}
+
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRConversion_BooleanConverter::translateAsciiOutput(IFRPacket_DataPart& datapart,
+                                                     char           *data,
+                                                     IFR_Length        datalength,
+                                                     IFR_Length*           lengthindicator,
+                                                     IFR_Bool        terminate,
+                                                     IFR_ConnectionItem &clink)
+{
+    DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateAsciiOutput);
+    IFR_Retcode rc = IFR_OK;
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
+        // TRUE
+        if(terminate) {
+            if(datalength < 5) {
+                memcpy(data, "TRUE", datalength);
+                data[datalength -1] = '\0';
+                rc = IFR_DATA_TRUNC;
+            } else {
+                memcpy(data, "TRUE", 5);
+            }
+        } else {
+            if(datalength < 4) {
+                memcpy(data, "TRUE", datalength);
+                rc = IFR_DATA_TRUNC;
+            } else {
+                memcpy(data, "TRUE", 4);
+            }
+        }
+        if(lengthindicator) {
+            *lengthindicator = 4;
+        }
+    } else {
+        // FALSE
+        if(terminate) {
+            if(datalength < 6) {
+                memcpy(data, "FALSE", datalength);
+                data[datalength -1] = '\0';
+                rc = IFR_DATA_TRUNC;
+            } else {
+                memcpy(data, "FALSE", 6);
+            }
+        } else {
+            if(datalength < 5) {
+                memcpy(data, "FALSE", datalength);
+                rc = IFR_DATA_TRUNC;
+            } else {
+                memcpy(data, "FALSE", 5);
+            }
+        }
+        if(lengthindicator) {
+            *lengthindicator = 5;
+        }
+    }
+    DBUG_RETURN(rc);
+}
+
+IFR_Retcode 
+IFRConversion_BooleanConverter::translateUCS2Output(IFRPacket_DataPart& datapart,
+                                                    char           *data,
+                                                    IFR_Bool        swapped,
+                                                    IFR_Length        datalength,
+                                                    IFR_Length*       lengthindicator,
+                                                    IFR_Bool        terminate,
+                                                    IFR_ConnectionItem &clink)
+{
+    DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateUCS2Output);
+    char true_ucs2 [] = { 'T', '\0',
+                          'R', '\0',
+                          'U', '\0',
+                          'E', '\0'
+    };
+    char true_ucs2_swapped [] = { '\0', 'T',
+                                  '\0', 'R',
+                                  '\0', 'U',
+                                  '\0', 'E'
+    };
+    char false_ucs2 [] = { 'F', '\0',
+                           'A', '\0',
+                           'L', '\0',
+                           'S', '\0',
+                           'E', '\0'
+    };
+    char false_ucs2_swapped [] = { '\0', 'F',
+                                   '\0', 'A',
+                                   '\0', 'L',
+                                   '\0', 'S',
+                                   '\0', 'E'
+    };
+    char *resultdata;
+    IFR_Length resultlength;
+    if(((unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos))[0]) {
+        resultlength = 4;
+        resultdata = swapped ? true_ucs2_swapped : true_ucs2;
+    } else {
+        resultlength = 5;
+        resultdata = swapped ? false_ucs2_swapped : false_ucs2;
+    }
+    IFR_Retcode rc = IFR_OK;
+    if(terminate) {
+        if(datalength < (resultlength * 2) + 2) {
+            memcpy(data, resultdata, (datalength / 2) * 2);
+            data[((datalength / 2) * 2) - 2 ] = 0;
+            data[((datalength / 2) * 2) - 1 ] = 0;
+            rc = IFR_DATA_TRUNC;
+        } else {
+            memcpy(data, resultdata, resultlength * 2);
+            data[ (resultlength * 2)    ] = 0;
+            data[ (resultlength * 2) + 1] = 0;
+        }
+    } else {
+        if(datalength < (resultlength * 2)) {
+            memcpy(data, resultdata, (datalength / 2) * 2);
+            rc = IFR_DATA_TRUNC;
+        } else {
+            memcpy(data, resultdata, resultlength * 2);
+        }
+    }
+    if(lengthindicator) {
+        *lengthindicator = (resultlength * 2);
+    }
+    DBUG_RETURN(rc);
+}
+
+
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRConversion_BooleanConverter::translateUTF8Output(IFRPacket_DataPart& datapart,
+                                                    char           *data,
+                                                    IFR_Length        datalength,
+                                                    IFR_Length*           lengthindicator,
+                                                    IFR_Bool        terminate,
+                                                    IFR_ConnectionItem &clink)
+{
+    DBUG_CLINK_METHOD_ENTER(IFRConversion_BooleanConverter, translateUTF8Output);
+    DBUG_RETURN(translateAsciiOutput(datapart, data, datalength, lengthindicator, terminate, clink));
+}
+
+
+
 #if IFR_NEED_VIRTUAL_DELEGATION
 
 #define TRANSLATE_METHOD_IMPL(xxtype)                                                               \
@@ -432,7 +618,14 @@ IFRConversion_BooleanConverter::translateOutput(IFRPacket_DataPart& part,
 TRANSLATE_METHOD_IMPL(SQL_TIME_STRUCT)
 TRANSLATE_METHOD_IMPL(SQL_DATE_STRUCT)
 TRANSLATE_METHOD_IMPL(SQL_TIMESTAMP_STRUCT)
-TRANSLATE_METHOD_IMPL(SQL_NUMERIC_STRUCT)
 TRANSLATE_METHOD_IMPL(GUID)
 
+IFR_Retcode                                                                                         
+IFRConversion_BooleanConverter::translateInput(IFRPacket_DataPart& part,                            
+                                            SQL_NUMERIC_STRUCT& data,                                           
+                                            IFR_Length* lengthindicator,                            
+                                            IFR_ConnectionItem& clink)                              
+{                                                                                                   
+    return IFRConversion_Converter::translateInput(part, data, lengthindicator, clink);             
+}                                                                                                   
 #endif
