@@ -2,8 +2,8 @@
   @file           IFRConversion_ByteCharDataConverter.h
   @author         D039759
   @ingroup        IFR_DataConv
-  @brief          
-  @see            
+  @brief
+  @see
 
 \if EMIT_LICENCE
 
@@ -32,41 +32,62 @@
 #define IFRCONVERSION_BYTECHARDATACONVERTER_H
 
 #include "SAPDB/Interfaces/Runtime/Conversion/IFRConversion_Converter.h"
+#include "SAPDB/Interfaces/Runtime/IFR_Connection.h"
 
 /**
  * Class that handles the conversion from and into data types that represent
- * byte oriented data. 
+ * byte oriented data. There are two kinds of character oriented data in
+ * MaxDB - BYTE and UCS2 - and this one is for BYTE only. This is used
+ * for the data types:
+ *
+ * - CHAR BYTE
+ * - VARCHAR BYTE
+ * - CHAR ASCII
+ * - VARCHAR ASCII
+ * - DATE, TIME, TIMESTAMP (in ASCII mode)
+ *
+ * Take note that the encoding at side of the database is <i>always</i>
+ * ASCII (resp. plain binary). For any type that uses an UCS2 layout
+ *  at the database @c IFRConversion_UCS2CharDataConverter exists.
+ *
+ * @todo
+ * There is a member @c m_partencoding that represents the encoding in 
+ * the part. For performance reasons this should be removed and code
+ * in @c IFRConversion_UCS2CharDataConverter should be added 
+ * appropriately.
+ * 
  */
-class IFRConversion_ByteCharDataConverter 
-    : public IFRConversion_Converter 
+class IFRConversion_ByteCharDataConverter
+    : public IFRConversion_Converter
 {
-public:    
+public:
     /**
      * Constructor.
      * @param shortinfo The short field infos of the parameter.
      * @param allocator The allocator used.
-     * @param istrimming Flag, whether this instance will trim blanks on output.
-     * @param integerconversion Flag, whether integer values are converted and 
-     *        inserted, or not supported
+     * @param istrimming Flag, whether this instance will trim
+     *        the blank character on output (CHARACTER VARYING).
+     * @param integerconversion Flag, whether conversion to/from numbers
+     *        is supported.
      */
     IFRConversion_ByteCharDataConverter(IFR_ShortInfo& shortinfo,
                                         SAPDBMem_IRawAllocator& allocator,
                                         IFR_Bool istrimming,
-                                        IFR_Bool integerconversion=false);   
-
+                                        IFR_Bool integerconversion);
+    
     /**
      * Destructor.
      */
     virtual ~IFRConversion_ByteCharDataConverter();
-    
-    /** 
+
+    /**
      * This method will copy the data into <code>data</code>. The number of
      * bytes that shall be copied is computed as follows:
      * <ol>
      *   <li>If <code>lengthindicator</code> is 0, <code>datalength</code>
      *       is used.
      *   </li>
-     *   <li>If <code>*lengthindicator</code> is <code>IFR_NTS</code>, 
+     *   <li>If <code>*lengthindicator</code> is <code>IFR_NTS</code>,
      *       <code>min(datalength, strlen(data))</code> is used.
      *   </li>
      *   <li>Otherwise, <code>min(datalength, *lengthindicator)</code>
@@ -92,14 +113,14 @@ public:
                                              IFR_Length            datalength,
                                              IFR_Length*           lengthindicator,
                                              IFR_ConnectionItem &clink);
-    /** 
+    /**
      * This method will copy the data into <code>data</code>. The number of
      * bytes that shall be copied is computed as follows:
      * <ol>
      *   <li>If <code>lengthindicator</code> is 0, <code>datalength</code>
      *       is used.
      *   </li>
-     *   <li>If <code>*lengthindicator</code> is <code>IFR_NTS</code>, 
+     *   <li>If <code>*lengthindicator</code> is <code>IFR_NTS</code>,
      *       <code>min(datalength, strlen(data))</code> is used.
      *   </li>
      *   <li>Otherwise, <code>min(datalength, *lengthindicator)</code>
@@ -114,12 +135,12 @@ public:
      * @param datalength      The maximum byte length of the data.
      * @param lengthindicator The actual length of the data, or a special indicator value.
      * @param terminate Specifies that the output buffer should be finished
-     *        with a C style zero terminator. The <code>Terminated</code> flag 
+     *        with a C style zero terminator. The <code>Terminated</code> flag
      *        works only for the hostvar type character (ASCII, UCS2 or UTF8).
      * @param clink           Link to connection for error reporting.
      * @param offset          The offset where to start the insertion.
      * @return <code>IFR_NOT_OK</code> if the conversion of this
-     *   parameter did not succeed.  If that computed length plus the given offset 
+     *   parameter did not succeed.  If that computed length plus the given offset
      *   is greater
      *   than the length which is supported by the data type,
      *   <code>IFR_DATA_TRUNC</code> is returned.  <code>IFR_OK</code>
@@ -135,14 +156,14 @@ public:
                                           IFR_Length&           offset,
                                           IFRConversion_Putval *putval);
 
-    
+
     virtual IFR_Retcode translateAsciiInput(IFRPacket_DataPart& datapart,
                                             char               *data,
                                             IFR_Length          datalength,
                                             IFR_Length*           lengthindicator,
                                             IFR_ConnectionItem &clink);
-    
-    
+
+
     virtual IFR_Retcode appendAsciiInput(IFRPacket_DataPart& datapart,
                                          char               *data,
                                          IFR_Length            datalength,
@@ -157,7 +178,7 @@ public:
                                            IFR_Length            datalength,
                                            IFR_Length*           lengthindicator,
                                            IFR_ConnectionItem &clink);
-    
+
     virtual IFR_Retcode appendUCS2Input(IFRPacket_DataPart&  datapart,
                                         char                *data,
                                         IFR_Bool            swapped,
@@ -172,7 +193,7 @@ public:
                                            IFR_Length            datalength,
                                            IFR_Length*           lengthindicator,
                                            IFR_ConnectionItem &clink);
-    
+
     virtual IFR_Retcode appendUTF8Input(IFRPacket_DataPart& datapart,
                                         char               *data,
                                         IFR_Length            datalength,
@@ -190,12 +211,12 @@ public:
                                        IFR_Int2&           data,
                                        IFR_Length*           lengthindicator,
                                        IFR_ConnectionItem& clink);
-    
+
     virtual IFR_Retcode translateInput(IFRPacket_DataPart& datapart,
                                        IFR_Int4&           data,
                                        IFR_Length*           lengthindicator,
                                        IFR_ConnectionItem& clink);
-    
+
     virtual IFR_Retcode translateInput(IFRPacket_DataPart& datapart,
                                        IFR_Int8&           data,
                                        IFR_Length*           lengthindicator,
@@ -214,7 +235,7 @@ public:
                                        IFR_UInt4&          data,
                                        IFR_Length*           lengthindicator,
                                        IFR_ConnectionItem& clink);
-    
+
     virtual IFR_Retcode translateInput(IFRPacket_DataPart& datapart,
                                        IFR_UInt8&          data,
                                        IFR_Length*           lengthindicator,
@@ -224,12 +245,12 @@ public:
                                        double&             data,
                                        IFR_Length*           lengthindicator,
                                        IFR_ConnectionItem& clink);
-    
+
     virtual IFR_Retcode translateInput(IFRPacket_DataPart& datapart,
                                        float&              data,
                                        IFR_Length*           lengthindicator,
                                        IFR_ConnectionItem& clink);
-    
+
     virtual IFR_Retcode translateInput(IFRPacket_DataPart& datapart,
                                        SQL_DATE_STRUCT&    data,
                                        IFR_Length*           lengthindicator,
@@ -285,7 +306,7 @@ public:
                                           IFR_Length&           dataoffset,
                                           IFR_Length&           offset,
                                           IFRConversion_Getval *getval);
-    
+
     virtual IFR_Retcode translateUCS2Output(IFRPacket_DataPart& datapart,
                                             char           *data,
                                             IFR_Bool            swapped,
@@ -356,7 +377,7 @@ public:
                                         IFR_UInt4&             data,
                                         IFR_Length*             lengthindicator,
                                         IFR_ConnectionItem&   clink);
-    
+
     virtual IFR_Retcode translateOutput(IFRPacket_DataPart&   datapart,
                                         IFR_UInt8&             data,
                                         IFR_Length*             lengthindicator,
@@ -397,6 +418,15 @@ public:
                                         IFR_Length*             lengthindicator,
                                         IFR_ConnectionItem&   clink);
 
+protected:
+    struct ConversionFlags {
+        unsigned trimming                  : 1;
+        unsigned numbers                   : 1;
+        unsigned binary_as_encoded         : 1;
+    };
+
+    ConversionFlags     m_flags;
+    IFR_StringEncoding  m_partencoding;
 private:
     /**
      * Moves data to the buffer, and signals if the data was truncated
@@ -408,24 +438,23 @@ private:
      * @return <code>IFR_OK</code> on success, <code>IFR_DATA_TRUNC</code>
      *   if the data was not copied completely or <code>IFR_NOT_OK</code>
      *   if a conversion error occured.
-     *   
+     *
      */
     IFR_Retcode moveDataToPart(IFRPacket_DataPart& datapart,
                                char *buffer,
                                IFR_size_t bufferlength,
                                IFR_ErrorHndl& error);
-    
+
     /**
-     * Moves data from the data part, hereby converting it to ASCII 
+     * Moves data from the data part, hereby converting it to ASCII
      * characters.
      * @param datapart The source data part.
      * @param buffer The target buffer.
      */
-    void moveDataToBuffer(IFRPacket_DataPart& datapart, 
+    void moveDataToBuffer(IFRPacket_DataPart& datapart,
                           char *buffer);
 
-    IFR_Bool   m_istrimming;        //!< Stores whether the data type is 'trimming' (<code>VARCHAR</code>).
-    IFR_Bool   m_integerconversion; //!< Whether this instance performs integer conversion.
+
 };
 
-#endif 
+#endif
