@@ -1,9 +1,10 @@
+
 /*!
   @file           IFRConversion_Converter.h
   @author         D030044
   @ingroup        IFR_DataConv
   @brief          Conversion of input and output data
-  @see            
+  @see
 
 \if EMIT_LICENCE
 
@@ -43,6 +44,10 @@
 #include "SAPDB/Interfaces/Runtime/Conversion/IFRConversion_Putval.h"
 #include "SAPDB/Interfaces/Runtime/Conversion/IFRConversion_Getval.h"
 
+#include "SAPDB/SAPDBCommon/Algorithms/SAPDBAlgo_QuickSort.hpp"
+
+#include <string.h>
+
 /**
  * @ingroup IFR_DataConv
  * @brief Base class for data conversion.
@@ -55,15 +60,15 @@ class IFRConversion_Converter
 {
 public:
     /**
-     * Constructor. 
+     * Constructor.
      * @param shortinfo Short field info of the parameter.
      * @param allocator The allocator to be used.
      * @param paramindex Index of the parameter.
      */
-    IFRConversion_Converter(IFR_ShortInfo& shortinfo, 
+    IFRConversion_Converter(IFR_ShortInfo& shortinfo,
                             SAPDBMem_IRawAllocator& allocator,
                             IFR_Int4 paramindex=-1);
-    
+
     /**
      * Destructor.
      */
@@ -71,7 +76,7 @@ public:
 
     /**
      * Translates an input parameter from a host type into the database's
-     * native format, and puts it at the appropriate position in the 
+     * native format, and puts it at the appropriate position in the
      * data part.
      * @param part       The data part to fill.
      * @param parameter  The parameter to insert.
@@ -83,18 +88,12 @@ public:
      *     <code>IFR_OK</code> if everything was ok. In case of <code>IFR_NOT_OK</code>, the
      *     appropriate error code is stored in <code>clink</code>.
      */
-    IFR_Retcode translateInput(IFRPacket_DataPart& part, 
-                               IFR_Parameter& parameter, 
+    IFR_Retcode translateInput(IFRPacket_DataPart& part,
+                               IFR_Parameter& parameter,
                                IFR_ConnectionItem& clink,
                                IFR_Int4 dataoffset=0,
                                IFR_size_t rowsize=0);
-    
-    IFR_Retcode _translateInput(IFRPacket_DataPart& part, 
-                               IFR_Parameter& parameter, 
-                               IFR_ConnectionItem& clink,
-                               IFR_Int4 dataoffset=0,
-                               IFR_size_t rowsize=0);
-    
+
     /**
      * Sets the input parameter to the NULL value.
      * @param datapart destination data part
@@ -106,7 +105,7 @@ public:
                                            IFR_Int4 dataoffset,
                                            IFR_size_t rowsize);
 
-    
+
     /**
      * Sets the input parameter to the DEFAULT value.
      * @param datapart destination data part
@@ -117,7 +116,7 @@ public:
                                               IFR_ConnectionItem& clink,
                                               IFR_Int4 dataoffset,
                                               IFR_size_t rowsize);
-    
+
     /**
      * Translates binary input data.
      * @param datapart        The data part to fill.
@@ -188,7 +187,7 @@ public:
                                            IFR_Length*         lengthindicator,
                                            IFR_ConnectionItem& clink);
 
-    
+
     virtual IFR_Retcode appendUCS2Input(IFRPacket_DataPart& datapart,
                                         char               *data,
                                         IFR_Bool            swapped,
@@ -215,8 +214,8 @@ public:
                                            IFR_Length            datalength,
                                            IFR_Length*           lengthindicator,
                                            IFR_ConnectionItem& clink);
-    
-    
+
+
     virtual IFR_Retcode appendUTF8Input(IFRPacket_DataPart& datapart,
                                         char               *data,
                                         IFR_Length            datalength,
@@ -256,7 +255,7 @@ public:
                                        IFR_Int2&           data,
                                        IFR_Length*           lengthindicator,
                                        IFR_ConnectionItem& clink);
-    
+
     /**
      * Translates a 4-byte integer.
      * @param datapart        The destination data part.
@@ -272,7 +271,7 @@ public:
                                        IFR_Int4&           data,
                                        IFR_Length*         lengthindicator,
                                        IFR_ConnectionItem& clink);
-    
+
     /**
      * Translates a 8-byte integer.
      * @param datapart        The destination data part.
@@ -336,7 +335,7 @@ public:
                                        IFR_UInt4&          data,
                                        IFR_Length*           lengthindicator,
                                        IFR_ConnectionItem& clink);
-    
+
     /**
      * Translates an unsigned 8-byte integer.
      * @param datapart        The destination data part.
@@ -368,7 +367,7 @@ public:
                                        double&             data,
                                        IFR_Length*           lengthindicator,
                                        IFR_ConnectionItem& clink);
-    
+
     /**
      * Translates a float value.
      * @param datapart        The destination data part.
@@ -384,7 +383,7 @@ public:
                                        float&              data,
                                        IFR_Length*           lengthindicator,
                                        IFR_ConnectionItem& clink);
-    
+
     /**
      * Translates a <code>SQL_DATE_STRUCT</code> value.
      * @param datapart        The destination data part.
@@ -468,14 +467,18 @@ public:
                                 IFR_Parameter& parameter,
                                 IFR_ConnectionItem& clink,
                                 IFR_Int4 dataoffset=0,
-                                IFR_size_t rowsize=0);
-private:
-    IFR_Retcode _translateOutput(IFRPacket_DataPart& part,
-                                IFR_Parameter& parameter,
-                                IFR_ConnectionItem& clink,
-                                IFR_Int4 dataoffset=0,
-                                IFR_size_t rowsize=0);
-public:
+                                IFR_size_t rowsize=0,
+                                IFR_Bool ignoreUnsetParameter = false);
+
+
+    IFR_Retcode translateRawHexOutput(IFRPacket_DataPart& datapart,
+                                      char               *data,
+                                      IFR_Length          datalength,
+                                      IFR_Length*         lengthindicator,
+                                      IFR_Bool            terminate,
+                                      IFR_ConnectionItem &clink);
+
+
     virtual IFR_Retcode translateBinaryOutput(IFRPacket_DataPart& datapart,
                                               char           *data,
                                               IFR_Length        datalength,
@@ -489,8 +492,8 @@ public:
      * @param datalength The total data length.
      * @param lengthindicator Indicator.
      * @param clink Link to connection.
-     * @param dataoffset Offset for output. Data is actually appended at 
-     *      <code>(data + dataoffset)</code>. @c dataoffset is increased by the number of 
+     * @param dataoffset Offset for output. Data is actually appended at
+     *      <code>(data + dataoffset)</code>. @c dataoffset is increased by the number of
      *      bytes transferred by this method.
      * @param offset Offset in the data that is read. This is updated by the number of bytes
      *   in the data that have been read.
@@ -504,7 +507,7 @@ public:
                                            IFR_Length         &dataoffset,
                                            IFR_Length         &offset,
                                            IFRConversion_Getval *getval);
-    /** 
+    /**
      * This method will convert the data from the given datapart to an ASCII
      * character buffer pointed to <code>data</code>.
      * @param datapart        The source data part.
@@ -512,7 +515,7 @@ public:
      * @param datalength      The maximum byte length of the data.
      * @param lengthindicator [out] The number of bytes without the possible
      *        termination bytes copied buffer pointed data. For termination
-     *        of character data see the <code>terminate</code> argument. 
+     *        of character data see the <code>terminate</code> argument.
      * @param terminate Specifies that the output buffer should be finished
      *        with a C style zero terminator.
      * @param clink           Link to connection for error reporting.
@@ -531,7 +534,7 @@ public:
                                              IFR_Length*           lengthindicator,
                                              IFR_Bool        terminate,
                                              IFR_ConnectionItem &clink);
-    
+
     virtual IFR_Retcode appendAsciiOutput(IFRPacket_DataPart& datapart,
                                           char               *data,
                                           IFR_Length            datalength,
@@ -541,8 +544,8 @@ public:
                                           IFR_Length         &dataoffset,
                                           IFR_Length         &offset,
                                           IFRConversion_Getval *getval);
-    
-    /** 
+
+    /**
      * This method will convert the data from the given datapart to an UCS2
      * character buffer pointed to <code>data</code>.
      * @param datapart        The source data part.
@@ -551,7 +554,7 @@ public:
      * @param datalength      The maximum byte length of the data.
      * @param lengthindicator [out] The number of bytes without the possible
      *        termination bytes copied buffer pointed data. For termination
-     *        of character data see the <code>terminate</code> argument. 
+     *        of character data see the <code>terminate</code> argument.
      * @param terminate Specifies that the output buffer should be finished
      *        with a C style zero terminator.
      * @param clink           Link to connection for error reporting.
@@ -583,7 +586,7 @@ public:
                                          IFR_Length     &offset,
                                          IFRConversion_Getval *getval);
 
-    /** 
+    /**
      * This method will convert the data from the given datapart to an UTF8
      * character buffer pointed to <code>data</code>.
      * @param datapart        The source data part.
@@ -591,9 +594,9 @@ public:
      * @param datalength      The maximum byte length of the data.
      * @param lengthindicator [out] The number of bytes without the possible
      *        termination bytes copied buffer pointed data. For termination
-     *        of character data see the <code>terminate</code> argument. 
+     *        of character data see the <code>terminate</code> argument.
      * @param terminate Specifies that the output buffer should be finished
-     *        with a C style zero terminator. For UCS2 data a double zero is 
+     *        with a C style zero terminator. For UCS2 data a double zero is
      *        append to the data.
      * @param clink           Link to connection for error reporting.
      * @return <code>IFR_NOT_OK</code> if the conversion of this
@@ -699,7 +702,7 @@ public:
     /**
      * Puts input data that is delivered for late binding.
      * There are three different cases:
-     * - integral data types (numbers, time, date, timestamp) must be delivered in 
+     * - integral data types (numbers, time, date, timestamp) must be delivered in
      *   one piece.
      * - string data (binary, ASCII, UCS2) can be delivered in pieces, as long as the
      *   host type is not an integral data type (a number or structure).
@@ -716,7 +719,8 @@ public:
      * @param clink Link to connection for error reporting.
      * @param putval The PUTVAL object that is associated with this operation.
      * @return The status of the conversion and insertion, which may be <code>IFR_OK</code>,
-     *         <code>IFR_NOT_OK</code>, <code>IFR_OVERFLOW</code> or <code>IFR_DATA_TRUNC</code>.
+     *         <code>IFR_NOT_OK</code>, <code>IFR_OVERFLOW</code> or <code>IFR_DATA_TRUNC</code>,
+     *         <code>IFR_NO_DATA_FOUND</code> if there was no data supplied.
      */
     IFR_Retcode putData(IFRPacket_DataPart& datapart,
                         void *data,
@@ -727,15 +731,6 @@ public:
                         IFR_Bool &integral,
                         IFR_ConnectionItem& clink,
                         IFRConversion_Putval *putval);
-    IFR_Retcode _putData(IFRPacket_DataPart& datapart,
-                        void *data,
-                        IFR_Length* lengthindicator,
-                        IFR_Parameter& parameter,
-                        IFR_Length& offset,
-                        IFR_Bool &nullordefault,
-                        IFR_Bool &integral,
-                        IFR_ConnectionItem& clink,
-                        IFRConversion_Putval *putval);
 
     /**
      * Gets long data piecewise from long column.
@@ -743,9 +738,9 @@ public:
      * @param data     The buffer the data has to be delivered.
      * @param lengthindicator The length or indicator value.
      * @param parameter The parameter to set.
-     * @param offset     The offset (bytes) of data that has already been read.
+     * @param offset     The offset (bytes) of data that has already been read. 
      * @param dataoffset The offset (bytes) of the data already have been written. The value
-     *                   is updated by this method 
+     *                   is updated by this method
      * @param nullordefault Whether a NULL or DEFAULT value is inserted by this method.
      * @param integral Whether the inserted value is integral (i.e. an append is not possible).
      * @param clink Link to connection for error reporting.
@@ -762,32 +757,18 @@ public:
                         IFR_ConnectionItem& clink,
                         IFR_Length&         datalength,
                         IFRConversion_Getval *getval);
-private:
-    /**
-     * @internal Method called by @c getData after performing SQL trace for doing the real 
-     *   work.
-     */
-    IFR_Retcode _getData(IFRPacket_DataPart& datapart,
-                         void *data,
-                         IFR_Length* lengthindicator,
-                         IFR_Parameter& parameter,
-                         IFR_Length& dataoffset,
-                         IFR_Length& offset,
-                         IFR_ConnectionItem& clink,
-                         IFR_Length&  datalength,
-                         IFRConversion_Getval *getval);
-public:
+
     /**
      * Gets the index of this parameter.
      * @return the parameter index, or -1 if this was not set.
      */
-    inline IFR_Int4 getIndex() 
+    inline IFR_Int4 getIndex()
     {
         return m_index;
     }
 
     /**
-     * Sets the parameter index. 
+     * Sets the parameter index.
      * @param index the parameter index
      */
     inline void setIndex(IFR_Int4 index)
@@ -798,24 +779,24 @@ public:
     /**
      * Sets the parameter name.
      * @param name The name of the parameter (e.g. from column names)
-     * @param ok   Set to false if the assignment fails due to 
+     * @param ok   Set to false if the assignment fails due to
      *             memory allocation problems.
      */
     inline void setName(const IFR_String& name,IFR_Bool& ok)
     {
         m_name.assign(name, ok);
-    }  
+    }
 
     /**
      * @brief Get the parameter/column name.
      * @param buffer the buffer where the column name should be copied into
      * @param encoding the encoding of the column name that is requested
      * @param bufferSize The size in bytes of the target buffer.
-     * @param bufferLength [out] The number of bytes copied to the buffer, 
-     *        except the number of bytes necessary for the zero terminator. 
+     * @param bufferLength [out] The number of bytes copied to the buffer,
+     *        except the number of bytes necessary for the zero terminator.
      *        If the source string exceeds the <code>bufferSize</code>.
      *        IFR_DATA_TRUNC will returned and the <code>bufferLength</code>
-     *        is set to the number of bytes (except the terminator bytes) 
+     *        is set to the number of bytes (except the terminator bytes)
      *        needed to copy without truncation.
      * @return <code>IFR_OK</code> on success, <code>IFR_DATA_TRUNC</code> if the buffer is too small.
      */
@@ -823,7 +804,7 @@ public:
                         IFR_StringEncoding encoding,
                         IFR_Length bufferSize,
                         IFR_Length *bufferLength) const;
-    
+
     inline const IFR_String& getName() const
     {
         return m_name;
@@ -853,7 +834,7 @@ public:
     {
         return m_shortinfo.frac;
     }
-    
+
     /**
      * Return whether this column is nullable.
      * @return <code>true</code> if the parameter accepts NULL values,
@@ -869,9 +850,9 @@ public:
      */
     inline IFR_UInt4 getBufpos() const
     {
-        return m_shortinfo.bufpos;
+        return m_shortinfo.pos.bufpos;
     }
-    
+
     /**
      * Returns true if this is an input parameter.
      */
@@ -887,7 +868,7 @@ public:
     {
         return m_shortinfo.isOutput();
     }
-    
+
     /**
      * Checks whether this value is <code>NULL</code>.
      * @param datapart The datapart which contains the value.
@@ -905,12 +886,12 @@ public:
      * @param clink      The link to the connection.
      * @param dataoffset The array offset of the data stored in the part, default 0.
      * @param rowsize    The row size of one row of host variables for row-wise binding, default 0.
-     * 
-     * @return The newly created PUTVAL object, or 0 if that 
+     *
+     * @return The newly created PUTVAL object, or 0 if that
      *         operation is not applicable.
      */
-    virtual IFRConversion_Putval *createPutval(IFRPacket_DataPart& part, 
-                                               IFR_Parameter& parameter, 
+    virtual IFRConversion_Putval *createPutval(IFRPacket_DataPart& part,
+                                               IFR_Parameter& parameter,
                                                IFR_ConnectionItem& clink,
                                                IFR_Int4 dataoffset=0,
                                                IFR_size_t rowsize=0);
@@ -923,12 +904,12 @@ public:
      * @param clink      The link to the connection.
      * @param dataoffset The array offset of the data stored in the part, default 0.
      * @param rowsize    The row size of one row of host variables for row-wise binding, default 0.
-     * 
-     * @return The newly created PUTVAL object, or 0 if that 
+     *
+     * @return The newly created PUTVAL object, or 0 if that
      *         operation is not applicable.
      */
-    virtual IFRConversion_Getval *createGetval (IFRPacket_DataPart& part, 
-                                                IFR_Parameter& parameter, 
+    virtual IFRConversion_Getval *createGetval (IFRPacket_DataPart& part,
+                                                IFR_Parameter* parameter,
                                                 IFR_ConnectionItem& clink,
                                                 IFR_Int4 dataoffset=0,
                                                 IFR_size_t rowsize=0);
@@ -946,20 +927,29 @@ public:
                                                IFR_Length*         lengthindicator,
                                                IFR_Int4            dataoffset,
                                                IFR_ConnectionItem& clink);
-    
+
     virtual IFR_Retcode translateUCS2LOBInput(IFRPacket_DataPart& datapart,
                                               IFR_LOBData           &lob,
                                               IFR_Bool            swapped,
                                               IFR_Length*         lengthindicator,
                                               IFR_Int4            dataoffset,
                                               IFR_ConnectionItem& clink);
-    
+
     virtual IFR_Retcode translateUTF8LOBInput(IFRPacket_DataPart& datapart,
                                               IFR_LOBData           &lob,
                                               IFR_Length*         lengthindicator,
                                               IFR_Int4            dataoffset,
                                               IFR_ConnectionItem& clink);
 
+
+    virtual IFR_Retcode translateABAPStreamInput(IFRPacket_DataPart& datapart,
+                                                 void               *streamDescriptor,
+                                                 IFR_Length           datalength,
+                                                 IFR_Length         *lengthindicator,
+                                                 IFR_Int4            dataoffset,
+                                                 IFR_ConnectionItem& clink);
+
+
     virtual IFR_Retcode translateBinaryLOBOutput(IFRPacket_DataPart& datapart,
                                                  IFR_LOBData        &lob,
                                                  IFR_Length*         lengthindicator,
@@ -984,7 +974,36 @@ public:
                                                IFR_Length*         lengthindicator,
                                                IFR_Int4            dataoffset,
                                                IFR_ConnectionItem& clink);
-    
+
+    /**
+     * Puts an ABAP stream.
+     * @param parameter The stream parameter.
+     * @param datapart The data part to fill.
+     * @param clink The link to the connection.
+     */
+    virtual IFR_Retcode putABAPStream(IFR_Parameter& parameter,
+                                      IFRPacket_DataPart& datapart,
+                                      void               *maskptr,
+                                      IFR_Int4            masklength,
+                                      IFR_Int4            requiredrows,
+                                      IFR_Bool &          last,
+                                      IFR_ConnectionItem& clink);
+
+    /**
+     * Puts an ABAP stream.
+     * @param parameter The stream parameter.
+     * @param datapart The data part to fill.
+     * @param maskptr The exclusion mask ptr, as delivered by the stream.
+     * @param abap_rc The return code from the callback function.
+     * @param clink The link to the connection.
+     */
+    virtual IFR_Retcode getABAPStream(IFR_Parameter& parameter,
+                                      IFRPacket_DataPart& datapart,
+                                      IFR_Int4 masklength,
+                                      void *maskptr,
+                                      IFR_Int4& abap_rc,
+                                      IFR_ConnectionItem& clink);
+
     /**
      * @brief Get whether this data item is LONG data.
      * @return <code>true</code> if long data present,
@@ -999,7 +1018,7 @@ public:
      * Returns <i>true</i> if this value
      * represents an integral data type, not a character or
      * byte sequence.
-     * @return <code>true</code> if this data type is not 
+     * @return <code>true</code> if this data type is not
      *   a CHAR, VARCHAR or LONG.
      */
     inline IFR_Bool isIntegral() const
@@ -1008,7 +1027,16 @@ public:
     }
 
     /**
-     * Gets the 'defined byte' for the data type stored in 
+     * Get whether the parameter is an ABAP/OMS stream.
+     * @return @c true if the parameter is an ABAP/OMS stream.
+     */
+    inline IFR_Bool isABAPStream() const
+    {
+        return m_shortinfo.isABAPStream();
+    }
+
+    /**
+     * Gets the 'defined byte' for the data type stored in
      * this short info object.
      * @param forUnicode whether this is queried for an unicode database.
      * @return the value of the defined byte for the datatype.
@@ -1043,6 +1071,181 @@ protected:
 
 };
 
+/**
+ * List of converters.
+ * The methods of the vector are not exposed, only some are available.
+ */
+class IFRConversion_ConverterList : protected IFRUtil_Vector<IFRConversion_Converter*>
+{
+public:
+    IFRConversion_ConverterList(SAPDBMem_IRawAllocator& allocator)
+    :IFRUtil_Vector< IFRConversion_Converter* >(allocator),
+     m_reference(false),
+     m_posvector(0),
+     m_indirectpos(false)
+    {}
+
+    ~IFRConversion_ConverterList()
+    {
+        clear();
+    }
+
+    void clear()
+    {
+        if(!m_reference) {
+            IFRConversion_Converter **p=Data();
+            SAPDBMem_IRawAllocator& alloc = allocator();
+            IFR_UInt4 sz = GetSize();
+            for(IFR_UInt4 i=0; i < sz; ++i) {
+                IFRUtil_Delete(p[i], alloc);
+            }
+        }
+        if(m_posvector) {
+            allocator().Deallocate(m_posvector);
+            m_posvector = 0;
+        }
+        Clear();
+    }
+
+    inline void add(IFRConversion_Converter* c, IFR_Bool& memory_ok)
+    {
+        InsertEnd(c, memory_ok);
+        m_reference = false;
+    }
+
+    inline void buildIndex(IFR_Bool& memory_ok)
+    {
+        if(memory_ok) {
+            m_indirectpos = true;
+            if(m_posvector) {
+                allocator().Deallocate(m_posvector);
+            }
+            IFR_UInt4 sz = size();
+            m_posvector = (IFR_Int2 *) allocator().Allocate(sz * sizeof(IFR_Int2));
+            if(m_posvector == 0) {
+                memory_ok = false;
+                return;
+            }
+            for(IFR_UInt4 i=0; i<sz; ++i) {
+                m_posvector[i] = i;
+            }
+            SAPDBAlgo_QuickSort(m_posvector, sz, *this);
+        }
+    }
+
+    inline void swap(IFRConversion_ConverterList& l, IFR_Bool& memory_ok)
+    {
+        Swap(l, memory_ok);
+        IFRUtil_Swap(m_reference, l.m_reference);
+        IFRUtil_Swap(m_indirectpos, l.m_indirectpos);
+        IFRUtil_Swap(m_posvector, l.m_posvector);
+    }
+
+    inline void append(IFRConversion_ConverterList& l, IFR_Bool& memory_ok)
+    {
+        if(memory_ok) {
+            Iterator e = l.End();
+            for(Iterator i=l.Begin(); i!=e; ++i) {
+                add(*i, memory_ok);
+            }
+            l.Clear();
+        }
+    }
+
+    inline void copyref(IFRConversion_ConverterList& l, IFR_Bool& memory_ok)
+    {
+        if(memory_ok) {
+            clear();
+            IFR_UInt4 sz = l.size();
+            Reserve(sz, memory_ok);
+            if(memory_ok) {
+                memcpy(Data(), l.Data(), sizeof(IFRConversion_Converter*) * sz);
+                setSize(sz);
+                m_reference = true;
+                m_indirectpos = l.m_indirectpos;
+                if(m_indirectpos) {
+                    m_posvector = (IFR_Int2 *)allocator().Allocate(sz * sizeof(IFR_Int2));
+                    if(m_posvector == 0) {
+                        memory_ok = false;
+                        return;
+                    }
+                    memcpy(m_posvector, l.m_posvector, sz);
+                }
+            }
+        }
+    }
+
+    inline void copyref(IFRConversion_Converter** l, IFR_UInt4 count, IFR_Bool indirectpos, IFR_Bool& memory_ok)
+    {
+        if(memory_ok) {
+            clear();
+            IFR_UInt4 sz = count;
+            Reserve(sz, memory_ok);
+            if(memory_ok) {
+                memcpy(Data(), l, sizeof(IFRConversion_Converter*) * sz);
+                setSize(sz);
+                m_reference = true;
+                if(indirectpos) {
+                    buildIndex(memory_ok);
+                }
+            }
+        }
+    }
+
+    inline IFR_size_t size()
+    {
+        return GetSize();
+    }
+
+    inline IFR_size_t size() const
+    {
+        return GetSize();
+    }
+
+    inline IFRConversion_Converter*& operator[] (IFR_Int4 index)
+    {
+        return IFRUtil_Vector<IFRConversion_Converter*>::operator [](index);
+    }
+    
+    inline IFR_Int4 parameterForInputPosition(IFR_Int4 index)
+    {
+        if(m_indirectpos) {
+            return m_posvector[index];
+        } else {
+            return index;
+        }
+    }
+    
+    IFR_Bool IsPredecessor(IFR_UInt2 a, IFR_UInt2 b)
+    {
+        IFR_ShortInfo *shortinfo_a = &(IFRUtil_Vector<IFRConversion_Converter*>::operator [](m_posvector[a])->shortinfo());
+        IFR_ShortInfo *shortinfo_b = &(IFRUtil_Vector<IFRConversion_Converter*>::operator [](m_posvector[b])->shortinfo());
+        if(shortinfo_a->isOutput()) {
+            return a < b;
+        } else {
+            if(shortinfo_b->isOutput()) {
+                return a < b;
+            } else {
+                return shortinfo_a->pos.newpos.position < shortinfo_b->pos.newpos.position;
+            }
+        }
+    }
+
+    inline IFR_Bool hasVariableInput() const
+    {
+        return m_indirectpos;
+    }
+
+private:
+
+    IFRConversion_ConverterList& operator = (const IFRConversion_ConverterList&);
+    IFRConversion_ConverterList(const IFRConversion_ConverterList&);
+
+    IFR_Bool   m_reference;
+    IFR_Int2  *m_posvector;
+    IFR_Bool   m_indirectpos;
+};
+
 
 /**
  * @ingroup IFR_Trace
@@ -1051,4 +1254,230 @@ protected:
 IFR_TraceStream&
 operator << (IFR_TraceStream& s,const IFRConversion_Converter& c);
 
+inline IFR_Length IFRConversion_Strnlen(char *p, IFR_Length sz)
+{
+    register char *pf = (char *) memchr(p, 0, sz);
+    if(pf) {
+        return pf - p;
+    } else {
+        return sz;
+    }
+}
+
+/**
+ * Length in bytes for UCS2.
+ */
+inline IFR_Length IFRConversion_StrnlenUCS2(char *p, IFR_Length sz)
+{
+    register char *pp = p;
+    IFR_Length all = 0;
+    while(sz >= 2) {
+        if(*p == 0) {
+            ++p;
+            if(*p == 0) {
+                return p - pp - 1;
+            }
+             ++p;
+        } else {
+            p += 2;
+        }
+        sz -= 2;
+    }
+    return all; // returns an invalid UCS bytecount which is handled later.
+}
+
+
+/**
+ * Common method for determining the length of binary/ASCII/UTF8 input.
+ * @param datalength      The data length specified for the parameter.
+ * @param lengthindicator The length/indicator value.
+ * @param data            The data.
+ * @param byteslength     The length of the data in bytes (output).
+ * @return @c 0 if the computation succeeds, and @c byteslength is filled,
+ *    a value not equal to @c 0 otherwise.
+ */
+inline IFR_Int4 IFRConversion_InputDataLength(IFR_Length datalength,
+                                              IFR_Length *lengthindicator,
+                                              char       *data,
+                                              IFR_Length& byteslength)
+{
+    if(lengthindicator) {
+        register IFR_Length l = *lengthindicator;
+        if(l < 0) {
+            if(l == IFR_NTS) {
+                if(datalength == 0) {
+                    byteslength = strlen(data);
+                } else {
+                    byteslength = IFRConversion_Strnlen(data, l);
+                }
+                return 0;
+            }
+            return 1;
+        } else {
+            if(datalength != 0
+               && datalength < l) {
+                byteslength = datalength;
+                return 0;
+            } else {
+                byteslength = l;
+                return 0;
+            }
+        }
+    } else {
+        if(datalength == 0) {
+            byteslength = strlen(data);
+        } else {
+            byteslength = IFRConversion_Strnlen(data, datalength);
+        }
+        return 0;
+    }
+}
+
+/**
+ * Input data length for UCS2 strings.
+ */
+inline IFR_Int4 IFRConversion_InputDataLengthUCS2(IFR_Length datalength,
+                                                  IFR_Length *lengthindicator,
+                                                  char       *data,
+                                                  IFR_Length& byteslength)
+{
+    if(lengthindicator) {
+        register IFR_Length l = *lengthindicator;
+        if(l < 0) {
+            if(l == IFR_NTS) {
+                if(datalength == 0) {
+                    byteslength = IFRConversion_StrnlenUCS2(data, MAX_IFR_INT4);
+                } else {
+                    byteslength = IFRConversion_StrnlenUCS2(data, datalength);
+                }
+                return 0;
+            }
+            return 1;
+        } else {
+            if(datalength != 0 && datalength < l) {
+                byteslength = datalength;
+                return 0;
+            } else {
+                byteslength = l;
+                return 0;
+            }
+        }
+    } else {
+        if(datalength == 0) {
+            byteslength = IFRConversion_StrnlenUCS2(data, MAX_IFR_INT4);
+        } else {
+             byteslength = IFRConversion_StrnlenUCS2(data, datalength);
+        }
+        return 0;
+    }
+}
+
+
+
+/**
+ * Count ASCII padding characters.
+ */
+inline IFR_Length IFRConversion_StringPadLength(char *data, IFR_Length byteslength, char padchar)
+{
+    register char *end = (--data) + byteslength;
+    while(end != data) {
+        if(*end != padchar) {
+            return end - data;
+        }
+        --end;
+    }
+    return 0;
+}
+
+/**
+ * Count UCS2 padding characters. The length of the string in bytes is
+ * returned, assuming the string is padded at the end with the padding
+ * character.
+ * @param data        The string to be checked.
+ * @param byteslength The maximum string length in bytes.
+ * @param padchar     The padding character.
+ * @param swapped     Whether the string is swapped (little-endian) or not.
+ */
+inline IFR_Length IFRConversion_StringPadLengthUCS2(char *data,
+                                                    IFR_Length byteslength,
+                                                    IFR_Int4 padchar,
+                                                    IFR_Bool swapped)
+{
+    register char *p = (--data) + byteslength;
+    register IFR_Int4 first;
+    register IFR_Int4 second;
+    if(swapped) {
+        first  = padchar & 0xFF;
+        second = (padchar >> 8) & 0xFF;
+    } else {
+        second = padchar & 0xFF;
+        first  = (padchar >> 8) & 0xFF;
+    }
+    // 1st byte is 'second', as we move backwards
+    while(p > data) {
+        if(*p != second) {
+            return p - data;
+        }
+        --p;
+        if(p > data) {
+            if(*p != first) {
+                return (p - data) + 1;
+            }
+        }
+        --p;
+    }
+    return 0;
+}
+
+/**
+ * Determine the length of an UCS2 string as UTF8. 
+ */
+inline IFR_Length 
+IFRConversion_StringUCS2LengthAsUTF8(unsigned char *buffer,
+                                     IFR_Length bytelength,
+                                     IFR_Bool swapped)
+{
+    IFR_Length utf8length=0;
+    if(swapped) {
+        for(IFR_Length i=0; i<bytelength-1; i+=2) {
+            IFR_UInt4 c = buffer[i] + (((IFR_UInt4)buffer[i+1]) << 8);
+            if(c < 0x80) {
+                utf8length += 1;
+            } else if(c < 0x800) {
+                utf8length += 2;
+            } else {
+                utf8length += 3;
+            }
+        }
+    } else {
+        for(IFR_Length i=0; i<bytelength-1; i+=2) {
+            IFR_UInt4 c = buffer[i+1] + (((IFR_UInt4)buffer[i]) << 8);
+            if(c < 0x80) {
+                utf8length += 1;
+            } else if(c < 0x800) {
+                utf8length += 2;
+            } else {
+                utf8length += 3;
+            }
+        }
+    }
+    return utf8length;
+}
+
+inline IFR_Length 
+IFRConversion_StringAsciiLengthAsUTF8(unsigned char *buffer,
+                                      IFR_Length bytelength)
+{
+    IFR_Length utf8length = 0;
+    for(IFR_Length i=0; i<bytelength; ++i) {
+        if(buffer[i] < 0x80) {
+            ++utf8length;
+        } else {
+            utf8length+=2;
+        }
+    }
+    return utf8length;
+}
+                                                
+
 #endif //IFRCONVERSION_CONVERTER_H
