@@ -86,11 +86,9 @@ IFRConversion_Getval::~IFRConversion_Getval()
 
 //----------------------------------------------------------------------
 void
-IFRConversion_Getval::putDescriptor(IFRPacket_DataPart& datapart, 
-                                    IFR_Int4 position)
+IFRConversion_Getval::putDescriptor(IFRPacket_DataPart& datapart)
 {
     DBUG_CONTEXT_METHOD_ENTER(IFRConversion_Getval, putDescriptor, m_clink);
-    DBUG_PRINT(position);
     
     m_lastinternpos = m_longdesc.internpos;
 
@@ -102,13 +100,7 @@ IFRConversion_Getval::putDescriptor(IFRPacket_DataPart& datapart,
         m_longdesc.valmode = IFRPacket_LongDescriptor::DataPart_C;
     }
     
-    datapart.addParameter (&m_longdesc, 
-                           position,
-                           m_length,
-                           m_iolength,
-                           m_definedbyte);
-    
-    datapart.GetRawPart()->sp1p_part_header().sp1p_arg_count++;
+    datapart.addDescriptor(m_longdesc);
 }
 
 //----------------------------------------------------------------------
@@ -123,7 +115,7 @@ IFRConversion_Getval::updateDescriptor(IFRPacket_DataPart& datapart,
     // set in data part.
     IFR_Int2 valindex = m_longdesc.valind;  
     
-    updateDescriptor(datapart.getParameterData (position));
+    updateDescriptor(datapart.getOutputData(position));
     
     if (m_longdesc.valind == 0) {
         m_longdesc.valind = valindex;
@@ -199,6 +191,14 @@ IFRConversion_Getval::setValIndex(IFR_Int2 valindex)
     return;
 }
 
+//----------------------------------------------------------------------
+IFR_StringEncoding
+IFRConversion_Getval::getEncoding()
+{
+  DBUG_CONTEXT_METHOD_ENTER(IFRConversion_Getval, getEncoding, m_clink);
+  DBUG_RETURN(m_encoding);
+}
+
 //----------------------------------------------------------------------
 IFR_Retcode
 IFRConversion_Getval::transferStream(IFRPacket_DataPart& datapart,        // data part to read from
@@ -300,6 +300,41 @@ IFRConversion_Getval::transferStream(IFRPacket_DataPart& datapart,        // dat
             // OK, there's some work to do ...
             IFR_Retcode rc = IFR_OK;
             IFR_Length  required_datalength = used_datalength - offset; // we have already 'offset' bytes in the data.
+
+            // ... or maybe not?
+            if(required_datalength == 0) {
+                if(lengthindicator) {
+                    *lengthindicator = m_maxlength + 1 - m_readoffset;
+                    switch(encoding) {
+                    case IFR_StringEncodingAscii:
+                        if(m_datatype == IFR_SQLTYPE_STRUNI || m_datatype==IFR_SQLTYPE_LONGUNI) {
+                            *lengthindicator /= 2;
+                        }
+                        break;
+                    case IFR_StringEncodingUCS2:
+                    case IFR_StringEncodingUCS2Swapped:
+                        if(m_datatype != IFR_SQLTYPE_STRUNI && m_datatype != IFR_SQLTYPE_LONGUNI) {
+                            *lengthindicator *= 2;
+                        }
+                        break;
+                    case IFR_StringEncodingUTF8:  // In case of DATA_TRUNC, it will be later corrected
+                        // to IFR_NO_TOTAL
+                        *lengthindicator = 0;
+                        break;
+                    }
+                }
+                
+                if(m_readoffset > m_maxlength) {
+                    DBUG_RETURN(checkStoreLongData(datapart, IFR_OK));
+                } else {
+                    if(lengthindicator && encoding == IFR_StringEncodingUTF8) {
+                        *lengthindicator = IFR_NO_TOTAL;
+                    }
+                    DBUG_RETURN(checkStoreLongData(datapart, IFR_DATA_TRUNC));
+                }
+            }
+
+
             IFR_Length  available_bytes = m_longdesc.vallen - m_readoffset + datastart; // the data length in bytes available
                   // in this chunk in case  the data is not read from the beginning of this long
             
@@ -326,8 +361,9 @@ IFRConversion_Getval::transferStream(IFRPacket_DataPart& datapart,        // dat
             case sp78_SourceExhausted:
             case sp78_Ok: {
                 memset(dest + bytes_written, 0, terminatorSize); 
-
-                if(lengthindicator) {
+                // The length indicator will only be updated if it hasn't been 
+                // set before.
+                if(lengthindicator && *lengthindicator==0) {
                     *lengthindicator = m_maxlength + 1 - m_readoffset;
                     switch(encoding) {
                     case IFR_StringEncodingAscii:
@@ -480,19 +516,22 @@ IFRConversion_Getval::setLongData(IFRPacket_DataPart& longdatapart,
     DBUG_CONTEXT_METHOD_ENTER(IFRConversion_Getval, setLongData, m_clink);
     if(memory_ok) {
         clearLongData();
-        
-        IFR_size_t copysize = sizeof(tsp1_part_header) + longdatapart.Length();
+        IFR_size_t copysize = m_longdesc.vallen + sizeof(tsp1_part_header);
         IFR_Byte  *source   = (IFR_Byte *) longdatapart.GetRawPart();
-        IFR_Byte  *dest     = (IFR_Byte *) allocator.Allocate(copysize);
+        IFR_Byte  *source2  = (IFR_Byte *) longdatapart.GetReadData() + m_longdesc.valpos - 1;
+        IFR_Byte  *dest     = (IFR_Byte *) IFR_ALLOCATOR(allocator).Allocate(copysize);
         if(dest == 0) {
             memory_ok = false;
             return;
         }
-        memcpy(dest, source, copysize);
-        m_longdata = new (allocator) IFRConversion_GetvalDataPart(dest,
-                                                                  allocator);
+        memcpy(dest, source, sizeof(tsp1_part_header));
+        memcpy(dest + sizeof(tsp1_part_header), source2, m_longdesc.vallen);
+        m_longdesc.valpos = 1;
+        m_longdata = new IFR_ALLOCATOR(allocator) IFRConversion_GetvalDataPart(dest,
+                                                                               allocator,
+                                                                               longdatapart.getEncoding());
         if(m_longdata == 0) {
-            allocator.Deallocate(dest);
+            IFR_ALLOCATOR(allocator).Deallocate(dest);
             memory_ok = false;
             return;
         }
