@@ -128,7 +128,7 @@ IFRConversion_NumericConverter::translateAsciiInput(IFRPacket_DataPart& datapart
         DBUG_RETURN(IFR_NOT_OK);
     }
 
-    unsigned char *num=(unsigned char*)datapart.getParameterData(m_shortinfo.bufpos);
+    unsigned char *num=(unsigned char*)datapart.getInputData(m_shortinfo);
     IFR_Bool nan = false;
     IFR_Retcode rc=IFRUtil_VDNNumber::stringToNumber(data,
                                                      used_datalength,
@@ -139,7 +139,7 @@ IFRConversion_NumericConverter::translateAsciiInput(IFRPacket_DataPart& datapart
                                                      m_shortinfo.length,
                                                      m_shortinfo.frac);
     if(rc==IFR_OK) {
-        datapart.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        datapart.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -172,7 +172,7 @@ IFRConversion_NumericConverter::translateUCS2Input(IFRPacket_DataPart& datapart,
         DBUG_RETURN(IFR_NOT_OK);
     }
 
-    unsigned char *num=(unsigned char*)datapart.getParameterData(m_shortinfo.bufpos);
+    unsigned char *num=(unsigned char*)datapart.getInputData(m_shortinfo);
     IFR_Bool nan = false;
     IFR_Retcode rc=IFRUtil_VDNNumber::stringToNumber(data,
                                                      used_datalength,
@@ -183,7 +183,7 @@ IFRConversion_NumericConverter::translateUCS2Input(IFRPacket_DataPart& datapart,
                                                      m_shortinfo.length,
                                                      m_shortinfo.frac);
     if(rc==IFR_OK) {
-        datapart.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        datapart.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -220,31 +220,20 @@ IFRConversion_NumericConverter::translateInput(IFRPacket_DataPart& datapart,
                                                IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateInput_double);
-    unsigned char num[21];
-    num[0]=0;
+    unsigned char num[20];
     IFR_Retcode conversion_rc;
-    if(m_shortinfo.isFloat()) {
-        if(ISNAN(data)) {
-            DBUG_TRACE << "NaN detected." << endl;
-            conversion_rc = IFR_OK;
-            memset(num+1, csp_oflw_byte, 20);
-            num[0] = csp_oflw_byte;
-        } else {
-            conversion_rc=IFRUtil_VDNNumber::doubleToNumber(data,
-                                                            num+1, 
-                                                            m_shortinfo.length, 
-                                                            -1);
-        }
-    } else {
-        conversion_rc=IFRUtil_VDNNumber::doubleToNumber(data,
-                                                        num+1, 
-                                                        m_shortinfo.length, 
-                                                        m_shortinfo.frac);
-    }
+    if(ISNAN(data)) {
+        clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
+        DBUG_RETURN(IFR_NOT_OK);
+    } 
+    conversion_rc=IFRUtil_VDNNumber::doubleToNumber(data,
+                                                    num, 
+                                                    m_shortinfo.length, 
+                                                    m_shortinfo.isFloat()?-1:m_shortinfo.frac);
     if(conversion_rc == IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(conversion_rc==IFR_OK || conversion_rc==IFR_DATA_TRUNC) {
-        datapart.addData(num, m_shortinfo.bufpos, m_shortinfo.iolength);
+        datapart.addBinaryParameter(num, m_shortinfo.iolength-1, m_shortinfo);
     }
     DBUG_RETURN(conversion_rc);
 }
@@ -269,10 +258,10 @@ IFRConversion_NumericConverter::translateInput(IFRPacket_DataPart& part,
                                                IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateInput_Int1);
-    unsigned char *num=(unsigned char*)part.getParameterData(m_shortinfo.bufpos);
+    unsigned char *num=(unsigned char*)part.getInputData(m_shortinfo);
     IFR_Retcode rc=IFRUtil_VDNNumber::int1ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
     if(rc==IFR_OK) {
-        part.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        part.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -289,10 +278,10 @@ IFRConversion_NumericConverter::translateInput(IFRPacket_DataPart& part,
                                                IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateInput_Int2);
-    unsigned char *num=(unsigned char*)part.getParameterData(m_shortinfo.bufpos);
+    unsigned char *num=(unsigned char*)part.getInputData(m_shortinfo);
     IFR_Retcode rc=IFRUtil_VDNNumber::int2ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
     if(rc==IFR_OK) {
-        part.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        part.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -310,10 +299,10 @@ IFRConversion_NumericConverter::translateInput(IFRPacket_DataPart& part,
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateInput_Int4);
     DBUG_PRINT(data);
-    unsigned char *num=(unsigned char*)part.getParameterData(m_shortinfo.bufpos);
+    unsigned char *num=(unsigned char*)part.getInputData(m_shortinfo);
     IFR_Retcode rc=IFRUtil_VDNNumber::int4ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
     if(rc==IFR_OK) {
-        part.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        part.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -330,10 +319,10 @@ IFRConversion_NumericConverter::translateInput(IFRPacket_DataPart& part,
                                                IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateInput_Int8);
-    unsigned char *num=(unsigned char*)part.getParameterData(m_shortinfo.bufpos);
+    unsigned char *num=(unsigned char*)part.getInputData(m_shortinfo);
     IFR_Retcode rc=IFRUtil_VDNNumber::int8ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
     if(rc==IFR_OK) {
-        part.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        part.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -351,10 +340,10 @@ IFRConversion_NumericConverter::translateInput(IFRPacket_DataPart& part,
                                                IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateInput_UInt1);
-    unsigned char *num=(unsigned char*)part.getParameterData(m_shortinfo.bufpos);
-    IFR_Retcode rc=IFRUtil_VDNNumber::int8ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
+    unsigned char *num=(unsigned char*)part.getInputData(m_shortinfo);
+    IFR_Retcode rc=IFRUtil_VDNNumber::uint1ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
     if(rc==IFR_OK) {
-        part.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        part.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -370,10 +359,10 @@ IFRConversion_NumericConverter::translateInput(IFRPacket_DataPart& part,
                                                IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateInput_UInt2);
-    unsigned char *num=(unsigned char*)part.getParameterData(m_shortinfo.bufpos);
-    IFR_Retcode rc=IFRUtil_VDNNumber::int8ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
+    unsigned char *num=(unsigned char*)part.getInputData(m_shortinfo);
+    IFR_Retcode rc=IFRUtil_VDNNumber::uint2ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
     if(rc==IFR_OK) {
-        part.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        part.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -390,10 +379,10 @@ IFRConversion_NumericConverter::translateInput(IFRPacket_DataPart& part,
                                                IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateInput_UInt4);
-    unsigned char *num=(unsigned char*)part.getParameterData(m_shortinfo.bufpos);
-    IFR_Retcode rc=IFRUtil_VDNNumber::int8ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
+    unsigned char *num=(unsigned char*)part.getInputData(m_shortinfo);
+    IFR_Retcode rc=IFRUtil_VDNNumber::uint4ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
     if(rc==IFR_OK) {
-        part.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        part.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -410,10 +399,10 @@ IFRConversion_NumericConverter::translateInput(IFRPacket_DataPart& part,
                                                IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateInput_UInt8);
-    unsigned char *num=(unsigned char*)part.getParameterData(m_shortinfo.bufpos);
+    unsigned char *num=(unsigned char*)part.getInputData(m_shortinfo);
     IFR_Retcode rc=IFRUtil_VDNNumber::uint8ToNumber(data, num, m_shortinfo.length-m_shortinfo.frac);
     if(rc==IFR_OK) {
-        part.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        part.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -431,7 +420,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateOutput_Int1);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);        
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);        
     IFR_Retcode rc=IFRUtil_VDNNumber::numberToInt1(vdn, data, m_shortinfo.iolength);
     if(rc == IFR_NOT_OK) {
         clink.error().setRuntimeError(IFR_ERR_INVALID_NUMERIC_VALUE_I, (IFR_Int4)this->m_index);
@@ -453,7 +442,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateOutput_Int2);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);        
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);        
     IFR_Retcode rc=IFRUtil_VDNNumber::numberToInt2(vdn, data, m_shortinfo.iolength);
     if(rc == IFR_NOT_OK) {
         clink.error().setRuntimeError(IFR_ERR_INVALID_NUMERIC_VALUE_I, (IFR_Int4)this->m_index);
@@ -475,7 +464,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateOutput_Int4);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);        
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);        
     IFR_Retcode rc=IFRUtil_VDNNumber::numberToInt4(vdn, data, m_shortinfo.iolength);
     if(rc == IFR_NOT_OK) {
         clink.error().setRuntimeError(IFR_ERR_INVALID_NUMERIC_VALUE_I, (IFR_Int4)this->m_index);
@@ -497,7 +486,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateOutput_Int8);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);        
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);        
     IFR_Retcode rc=IFRUtil_VDNNumber::numberToInt8(vdn, data, m_shortinfo.iolength);
     if(rc == IFR_NOT_OK) {
         clink.error().setRuntimeError(IFR_ERR_INVALID_NUMERIC_VALUE_I, (IFR_Int4)this->m_index);
@@ -520,7 +509,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateOutput_UInt1);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);        
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);        
     IFR_Retcode rc=IFRUtil_VDNNumber::numberToUInt1(vdn, data, m_shortinfo.iolength);
     if(rc == IFR_NOT_OK) {
         clink.error().setRuntimeError(IFR_ERR_INVALID_NUMERIC_VALUE_I, (IFR_Int4)this->m_index);
@@ -542,7 +531,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateOutput_UInt2);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);        
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);        
     IFR_Retcode rc=IFRUtil_VDNNumber::numberToUInt2(vdn, data, m_shortinfo.iolength);
     if(rc == IFR_NOT_OK) {
         clink.error().setRuntimeError(IFR_ERR_INVALID_NUMERIC_VALUE_I, (IFR_Int4)this->m_index);
@@ -564,7 +553,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateOutput_UInt4);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);        
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);        
     IFR_Retcode rc=IFRUtil_VDNNumber::numberToUInt4(vdn, data, m_shortinfo.iolength);
     if(rc == IFR_NOT_OK) {
         clink.error().setRuntimeError(IFR_ERR_INVALID_NUMERIC_VALUE_I, (IFR_Int4)this->m_index);
@@ -586,7 +575,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateOutput_UInt8);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);        
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);        
     IFR_Retcode rc=IFRUtil_VDNNumber::numberToUInt8(vdn, data, m_shortinfo.iolength);
     if(rc == IFR_NOT_OK) {
         clink.error().setRuntimeError(IFR_ERR_INVALID_NUMERIC_VALUE_I, (IFR_Int4)this->m_index);
@@ -609,7 +598,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart& datapart,
                                                 IFR_ConnectionItem& clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateOutput_double);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);        
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);        
     // Check whether there was a division by zero or similar harmful stuff
     if(*(vdn - 1) == csp_oflw_byte) {
 #if defined(WIN32)
@@ -673,7 +662,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart&   datapart,
 inline IFR_size_t ucsx_bytelen(char *c)
 {
     IFR_size_t result=0;
-    while(*c && *(c+1)) {
+    while(*c || *(c+1)) {
         c+=2;
         result+=2;
     }
@@ -691,7 +680,7 @@ IFRConversion_NumericConverter::translateCharacterOutput(IFRPacket_DataPart& dat
                                                          IFR_Bool terminate)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateAsciiOutput);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);        
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);        
 
     // unfortunately, we must possibly report the original length to the
     // caller. So for now we will copy the result anyway ...
@@ -794,10 +783,10 @@ IFRConversion_NumericConverter::translateInput(IFRPacket_DataPart&   datapart,
                            IFR_ConnectionItem&   clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateInput_NUMERIC);
-    unsigned char *num=(unsigned char*)datapart.getParameterData(m_shortinfo.bufpos);
+    unsigned char *num=(unsigned char*)datapart.getInputData(m_shortinfo);
     IFR_Retcode rc=IFRUtil_SQLNumeric::numericToNumber(data, num, m_shortinfo.length);
     if(rc==IFR_OK) {
-        datapart.addData((IFR_Int1)0, m_shortinfo.bufpos, m_shortinfo.iolength);
+        datapart.finishData(m_shortinfo.iolength, m_shortinfo);
     } else if(rc==IFR_OVERFLOW) {
         clink.error().setRuntimeError(IFR_ERR_NUMERIC_OVERFLOW_I, (IFR_Int4)this->m_index);
     } else if(rc==IFR_NOT_OK) {
@@ -815,7 +804,7 @@ IFRConversion_NumericConverter::translateOutput(IFRPacket_DataPart&   datapart,
                                                IFR_ConnectionItem&    clink)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_NumericConverter, translateOutput_NUMERIC);
-    unsigned char *vdn=(unsigned char *)datapart.getParameterData(m_shortinfo.bufpos);
+    unsigned char *vdn=(unsigned char *)datapart.getOutputData(m_shortinfo.pos.bufpos);
     if(lengthindicator) {
         *lengthindicator=sizeof(SQL_NUMERIC_STRUCT);
     }
