@@ -38,9 +38,12 @@
 
 //----------------------------------------------------------------------
 IFRConversion_StreamConverter::IFRConversion_StreamConverter(IFR_ShortInfo &shortinfo,
-                                                             SAPDBMem_IRawAllocator& allocator)
+                                                             SAPDBMem_IRawAllocator& allocator,
+                                                             IFR_Connection& connection)
 :IFRConversion_Converter(shortinfo, allocator)
-{}
+{
+    m_binary_as_encoded = connection.translateBinaryAsEncoded() ? true : false;
+}
 
 //----------------------------------------------------------------------
 IFRConversion_StreamConverter::~IFRConversion_StreamConverter()
@@ -64,20 +67,16 @@ IFRConversion_StreamConverter::createPutval(IFRPacket_DataPart& part,
       *param_lengthindicator == IFR_DEFAULT_PARAM )) {
       DBUG_RETURN((IFRConversion_Putval*)0);
   }
-  IFRConversion_Putval *p=new (m_name.getAllocator()) IFRConversion_Putval(getIndex(),
-                                                                           dataoffset + 1,
-                                                                           getLength(),
-                                                                           getIOLength(),
-                                                                           getDefinedByte(),
-                                                                           getBufpos(),
-                                                                           getSQLType(),
-                                                                           parameter.getHostType(),
-                                                                           parameter.asChar(dataoffset, rowsize),
-                                                                           parameter.getBytesLength(),
-                                                                           parameter.getLengthIndicator(dataoffset),
-                                                                           clink);
+  IFRConversion_Putval *p=new IFR_ALLOCATOR(m_name.getAllocator()) IFRConversion_Putval(getIndex(),
+                                                                                        dataoffset + 1,
+                                                                                        m_shortinfo,
+                                                                                        parameter.getHostType(),
+                                                                                        parameter.asChar(dataoffset, rowsize),
+                                                                                        parameter.getBytesLength(),
+                                                                                        parameter.getLengthIndicator(dataoffset),
+                                                                                        clink);
   if(p) {
-      p->putDescriptor (part, getBufpos());
+      p->putDescriptor (part);
   } else {
       clink.error().setMemoryAllocationFailed();
   }
@@ -87,14 +86,14 @@ IFRConversion_StreamConverter::createPutval(IFRPacket_DataPart& part,
 //----------------------------------------------------------------------
 IFRConversion_Getval *
 IFRConversion_StreamConverter::createGetval (IFRPacket_DataPart& part, 
-                                             IFR_Parameter& parameter, 
+                                             IFR_Parameter* parameter, 
                                              IFR_ConnectionItem& clink,
                                              IFR_Int4 dataoffset,
                                              IFR_size_t rowsize)
 {
     DBUG_CLINK_METHOD_ENTER(IFRConversion_StreamConverter, createGetval);
     
-    IFR_Length *param_lengthindicator=parameter.getLengthIndicator (dataoffset);
+    IFR_Length *param_lengthindicator=parameter ? parameter->getLengthIndicator (dataoffset) : 0;
   
     // look if it is null ...
     if(isNull(part, clink)) {
@@ -105,13 +104,13 @@ IFRConversion_StreamConverter::createGetval (IFRPacket_DataPart& part,
     }
     
     IFRConversion_Getval *p = 
-        new (clink.getConnection()->allocator) IFRConversion_Getval (getIndex(),
-                                                                     dataoffset + 1,
-                                                                     getLength(),
-                                                                     getIOLength(),
-                                                                     getDefinedByte(),
-                                                                     getSQLType(),
-                                                                     clink);
+        new IFR_ALLOCATOR(clink.getConnection()->allocator) IFRConversion_Getval (getIndex(),
+                                                                                  dataoffset + 1,
+                                                                                  getLength(),
+                                                                                  getIOLength(),
+                                                                                  getDefinedByte(),
+                                                                                  getSQLType(),
+                                                                                  clink);
     
     if(p == 0) {
         clink.error().setMemoryAllocationFailed();
@@ -131,6 +130,10 @@ IFRConversion_StreamConverter::translateBinaryInput(IFRPacket_DataPart& datapart
                                                     IFR_ConnectionItem & clink)
 {
   DBUG_CLINK_METHOD_ENTER(IFRConversion_StreamConverter, translateBinaryInput);   
+  if(!m_shortinfo.isBinary() && !m_binary_as_encoded) {
+      clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+      DBUG_RETURN(IFR_NOT_OK);
+  }
   DBUG_RETURN(IFR_OK);
 }
 
@@ -143,8 +146,12 @@ IFRConversion_StreamConverter
                       IFR_Length*           lengthindicator,
                       IFR_ConnectionItem &clink)
 {
-  DBUG_CLINK_METHOD_ENTER(IFRConversion_StreamConverter, translateAsciiInput);
-  DBUG_RETURN(IFR_OK);
+    DBUG_CLINK_METHOD_ENTER(IFRConversion_StreamConverter, translateAsciiInput);
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    DBUG_RETURN(IFR_OK);
 }
 
 //----------------------------------------------------------------------
@@ -156,8 +163,12 @@ IFRConversion_StreamConverter::translateUCS2Input(IFRPacket_DataPart& datapart,
                                                   IFR_Length*           lengthindicator,
                                                   IFR_ConnectionItem &clink)
 {
-  DBUG_CLINK_METHOD_ENTER(IFRConversion_StreamConverter, translateUCS2Input);
-  DBUG_RETURN(IFR_OK);
+    DBUG_CLINK_METHOD_ENTER(IFRConversion_StreamConverter, translateUCS2Input);
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    DBUG_RETURN(IFR_OK);
 }
 
 //----------------------------------------------------------------------
@@ -174,7 +185,10 @@ IFRConversion_StreamConverter::appendBinaryInput(IFRPacket_DataPart& datapart,
   DBUG_PRINT(datalength);
   DBUG_PRINT(lengthindicator);
   DBUG_PRINT(offset);
-  
+  if(!m_shortinfo.isBinary() && !m_binary_as_encoded) {
+      clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+      DBUG_RETURN(IFR_NOT_OK);
+  }
   DBUG_RETURN(putval->transferStream(datapart, data, datalength, lengthindicator, offset));
 }
                                                  
@@ -192,7 +206,10 @@ IFRConversion_StreamConverter::appendAsciiInput(IFRPacket_DataPart& datapart,
   DBUG_PRINT(datalength);
   DBUG_PRINT(lengthindicator);
   DBUG_PRINT(offset);
-  
+  if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+      clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+      DBUG_RETURN(IFR_NOT_OK);
+  }
   DBUG_RETURN(putval->transferStream(datapart, data, datalength, lengthindicator, offset));
 }
 
@@ -211,7 +228,10 @@ IFRConversion_StreamConverter::appendUCS2Input(IFRPacket_DataPart&  datapart,
   DBUG_PRINT(datalength);
   DBUG_PRINT(lengthindicator);
   DBUG_PRINT(offset);
-  
+  if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+      clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+      DBUG_RETURN(IFR_NOT_OK);
+  }
   DBUG_RETURN(putval->transferStream(datapart, data, datalength, lengthindicator, offset));
 }
                                                  
@@ -225,6 +245,10 @@ IFRConversion_StreamConverter::translateAsciiOutput(IFRPacket_DataPart& datapart
 						    IFR_ConnectionItem &clink)
 {
   DBUG_CLINK_METHOD_ENTER(IFRConversion_StreamConverter, translateAsciiOutput);
+  if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+      clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+      DBUG_RETURN(IFR_NOT_OK);
+  }
   DBUG_RETURN(IFR_OK);
 }
 
@@ -245,7 +269,12 @@ IFRConversion_StreamConverter::appendAsciiOutput(IFRPacket_DataPart& datapart,
     DBUG_PRINT(lengthindicator);
     DBUG_PRINT(dataoffset);
     DBUG_PRINT(offset);
-    
+
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+  
     if(offset) {
         getval->setReadOffset(offset);
     }
@@ -273,6 +302,10 @@ IFRConversion_StreamConverter::translateBinaryOutput(IFRPacket_DataPart& datapar
 						     IFR_ConnectionItem &clink)
 {
   DBUG_CLINK_METHOD_ENTER(IFRConversion_StreamConverter, translateBinaryOutput);
+  if(!m_shortinfo.isBinary() && !m_binary_as_encoded) {
+      clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+      DBUG_RETURN(IFR_NOT_OK);
+  }
   DBUG_RETURN(IFR_OK);
 }
 
@@ -292,12 +325,17 @@ IFRConversion_StreamConverter::appendBinaryOutput(IFRPacket_DataPart& datapart,
     DBUG_PRINT(lengthindicator);
     DBUG_PRINT(dataoffset);
     DBUG_PRINT(offset);
-    
+    if(!m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+  
     getval->setNoClose();
     if(offset) {
         getval->setReadOffset(offset);
     }
-    IFR_Retcode rc = getval->transferStream(datapart, data, datalength, lengthindicator, dataoffset, IFR_StringEncodingAscii, false);
+    IFR_Retcode rc = getval->transferStream(datapart, data, datalength, lengthindicator, 
+					    dataoffset, getval->getEncoding(), false);
     if(rc == IFR_OK || rc==IFR_DATA_TRUNC || rc == IFR_NO_DATA_FOUND ||  rc==IFR_NEED_DATA) {
         offset = getval->getReadOffset();
     } else {
@@ -317,6 +355,10 @@ IFRConversion_StreamConverter::translateUCS2Output(IFRPacket_DataPart& datapart,
                                                    IFR_ConnectionItem &clink)
 {
   DBUG_CLINK_METHOD_ENTER(IFRConversion_StreamConverter, translateUCS2Output);
+  if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+      clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+      DBUG_RETURN(IFR_NOT_OK);
+  }
   DBUG_RETURN(IFR_OK);  
 }
 
@@ -338,7 +380,10 @@ IFRConversion_StreamConverter::appendUCS2Output(IFRPacket_DataPart& datapart,
     DBUG_PRINT(lengthindicator);
     DBUG_PRINT(offset);
     DBUG_PRINT(dataoffset);
-  
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
     getval->setNoClose ();
     if(offset) {
         getval->setReadOffset(offset);
@@ -370,7 +415,10 @@ IFRConversion_StreamConverter::appendUTF8Output(IFRPacket_DataPart& datapart,
     DBUG_PRINT(lengthindicator);
     DBUG_PRINT(dataoffset);
     DBUG_PRINT(offset);
-    
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
     getval->setNoClose();
     if(offset) {
         getval->setReadOffset(offset);
@@ -394,6 +442,10 @@ IFRConversion_StreamConverter::translateBinaryLOBInput(IFRPacket_DataPart& datap
                                                        IFR_ConnectionItem& clink)
 {
     DBUG_CONTEXT_METHOD_ENTER(IFRConversion_Converter, translateInput, &clink);
+    if(!m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
     if(lengthindicator && 
        (*lengthindicator == IFR_NULL_DATA ||
         *lengthindicator == IFR_DEFAULT_PARAM)) {
@@ -402,11 +454,11 @@ IFRConversion_StreamConverter::translateBinaryLOBInput(IFRPacket_DataPart& datap
         data.connectionitem = &clink;
 
     } else {
-        data.lobdata = new (m_name.getAllocator()) IFR_LOB(getIndex(),
-                                                           dataoffset + 1, 
-                                                           IFR_HOSTTYPE_BLOB,
-                                                           clink,
-                                                           true);
+        data.lobdata = new IFR_ALLOCATOR(m_name.getAllocator()) IFR_LOB(getIndex(),
+                                                                        dataoffset + 1, 
+                                                                        IFR_HOSTTYPE_BLOB,
+                                                                        clink,
+                                                                        true);
         data.connectionitem = &clink;
         if(data.lobdata == 0) {
             clink.error().setMemoryAllocationFailed();
@@ -432,6 +484,10 @@ IFRConversion_StreamConverter::translateAsciiLOBInput(IFRPacket_DataPart& datapa
                                                       IFR_ConnectionItem& clink)
 {
     DBUG_CONTEXT_METHOD_ENTER(IFRConversion_Converter, translateInput, &clink);
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
     if(lengthindicator && 
        (*lengthindicator == IFR_NULL_DATA ||
         *lengthindicator == IFR_DEFAULT_PARAM)) {
@@ -440,11 +496,11 @@ IFRConversion_StreamConverter::translateAsciiLOBInput(IFRPacket_DataPart& datapa
         data.connectionitem = &clink;
 
     } else {
-        data.lobdata = new (m_name.getAllocator()) IFR_LOB(getIndex(),
-                                                           dataoffset + 1,
-                                                           IFR_HOSTTYPE_ASCII_LOB,
-                                                           clink,
-                                                           true);
+        data.lobdata = new IFR_ALLOCATOR(m_name.getAllocator()) IFR_LOB(getIndex(),
+                                                                        dataoffset + 1,
+                                                                        IFR_HOSTTYPE_ASCII_LOB,
+                                                                        clink,
+                                                                        true);
         data.connectionitem = &clink;
         if(data.lobdata == 0) {
             clink.error().setMemoryAllocationFailed();
@@ -471,6 +527,10 @@ IFRConversion_StreamConverter::translateUCS2LOBInput(IFRPacket_DataPart& datapar
                                                      IFR_ConnectionItem& clink)
 {
     DBUG_CONTEXT_METHOD_ENTER(IFRConversion_Converter, translateInput, &clink);
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
     if(lengthindicator && 
        (*lengthindicator == IFR_NULL_DATA ||
         *lengthindicator == IFR_DEFAULT_PARAM)) {
@@ -480,11 +540,11 @@ IFRConversion_StreamConverter::translateUCS2LOBInput(IFRPacket_DataPart& datapar
 
     } else {
         
-        data.lobdata = new (m_name.getAllocator()) IFR_LOB(getIndex(),
-                                                           dataoffset + 1,
-                                                           swapped?IFR_HOSTTYPE_UCS2_SWAPPED_LOB:IFR_HOSTTYPE_UCS2_LOB,
-                                                           clink, 
-                                                           true);
+        data.lobdata = new IFR_ALLOCATOR(m_name.getAllocator()) IFR_LOB(getIndex(),
+                                                                        dataoffset + 1,
+                                                                        swapped?IFR_HOSTTYPE_UCS2_SWAPPED_LOB:IFR_HOSTTYPE_UCS2_LOB,
+                                                                        clink, 
+                                                                        true);
         data.connectionitem = &clink;
         if(data.lobdata == 0) {
             clink.error().setMemoryAllocationFailed();
@@ -510,6 +570,10 @@ IFRConversion_StreamConverter::translateUTF8LOBInput(IFRPacket_DataPart& datapar
                                                      IFR_ConnectionItem& clink)
 {
     DBUG_CONTEXT_METHOD_ENTER(IFRConversion_StreamConverter, translateUTF8LOBInput, &clink);
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
     if(lengthindicator && 
        (*lengthindicator == IFR_NULL_DATA ||
         *lengthindicator == IFR_DEFAULT_PARAM)) {
@@ -518,11 +582,11 @@ IFRConversion_StreamConverter::translateUTF8LOBInput(IFRPacket_DataPart& datapar
         data.connectionitem = &clink;
 
     } else {
-        data.lobdata = new (m_name.getAllocator()) IFR_LOB(getIndex(),
-                                                           dataoffset + 1,
-                                                           IFR_HOSTTYPE_UTF8_LOB,
-                                                           clink,
-                                                           true);
+        data.lobdata = new IFR_ALLOCATOR(m_name.getAllocator()) IFR_LOB(getIndex(),
+                                                                        dataoffset + 1,
+                                                                        IFR_HOSTTYPE_UTF8_LOB,
+                                                                        clink,
+                                                                        true);
         data.connectionitem = &clink;
         if(data.lobdata == 0) {
             clink.error().setMemoryAllocationFailed();
@@ -548,11 +612,15 @@ IFRConversion_StreamConverter::translateBinaryLOBOutput(IFRPacket_DataPart& data
                                                         IFR_ConnectionItem& clink)
 {
     DBUG_CONTEXT_METHOD_ENTER(IFRConversion_StreamConverter, translateBinaryLOBOutput, &clink);
-    data.lobdata = new (m_name.getAllocator()) IFR_LOB(getIndex(),
-                                                       dataoffset + 1,
-                                                       IFR_HOSTTYPE_BLOB,
-                                                       clink,
-                                                       false);
+    if(!m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    data.lobdata = new IFR_ALLOCATOR(m_name.getAllocator()) IFR_LOB(getIndex(),
+                                                                    dataoffset + 1,
+                                                                    IFR_HOSTTYPE_BLOB,
+                                                                    clink,
+                                                                    false);
     data.connectionitem = &clink;
     if(data.lobdata == 0) {
         clink.error().setMemoryAllocationFailed();
@@ -578,11 +646,15 @@ IFRConversion_StreamConverter::translateAsciiLOBOutput(IFRPacket_DataPart& datap
                                                 IFR_ConnectionItem& clink)
 {
     DBUG_CONTEXT_METHOD_ENTER(IFRConversion_StreamConverter, translateAsciiLOBOutput, &clink);
-    data.lobdata = new (m_name.getAllocator()) IFR_LOB(getIndex(),
-                                                       dataoffset + 1,
-                                                       IFR_HOSTTYPE_ASCII_LOB,
-                                                       clink,
-                                                       false);
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    data.lobdata = new IFR_ALLOCATOR(m_name.getAllocator()) IFR_LOB(getIndex(),
+                                                                    dataoffset + 1,
+                                                                    IFR_HOSTTYPE_ASCII_LOB,
+                                                                    clink,
+                                                                    false);
     data.connectionitem = &clink;
     if(data.lobdata == 0) {
         clink.error().setMemoryAllocationFailed();
@@ -608,11 +680,15 @@ IFRConversion_StreamConverter::translateUCS2LOBOutput(IFRPacket_DataPart& datapa
                                                       IFR_ConnectionItem& clink)
 {
     DBUG_CONTEXT_METHOD_ENTER(IFRConversion_StreamConverter, translateUCS2LOBOutput, &clink);
-    data.lobdata = new (m_name.getAllocator()) IFR_LOB(getIndex(),
-                                                       dataoffset + 1,
-                                                       swapped?IFR_HOSTTYPE_UCS2_SWAPPED_LOB:IFR_HOSTTYPE_UCS2_LOB,
-                                                       clink,
-                                                       false);
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    data.lobdata = new IFR_ALLOCATOR(m_name.getAllocator()) IFR_LOB(getIndex(),
+                                                                    dataoffset + 1,
+                                                                    swapped?IFR_HOSTTYPE_UCS2_SWAPPED_LOB:IFR_HOSTTYPE_UCS2_LOB,
+                                                                    clink,
+                                                                    false);
     data.connectionitem = &clink;
     if(data.lobdata == 0) {
         clink.error().setMemoryAllocationFailed();
@@ -637,11 +713,15 @@ IFRConversion_StreamConverter::translateUTF8LOBOutput(IFRPacket_DataPart& datapa
                                                       IFR_ConnectionItem& clink)
 {
     DBUG_CONTEXT_METHOD_ENTER(IFRConversion_StreamConverter, translateUTF8LOBOutput, &clink);
-    data.lobdata = new (m_name.getAllocator()) IFR_LOB(getIndex(),
-                                                       dataoffset + 1,
-                                                       IFR_HOSTTYPE_UTF8_LOB,
-                                                       clink,
-                                                       false);
+    if(m_shortinfo.isBinary() && !m_binary_as_encoded) {
+        clink.error().setRuntimeError(IFR_ERR_BINARY_TO_ENCODED_UNSUPPORTED_I, (IFR_Int4)getIndex());
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    data.lobdata = new IFR_ALLOCATOR(m_name.getAllocator()) IFR_LOB(getIndex(),
+                                                                    dataoffset + 1,
+                                                                    IFR_HOSTTYPE_UTF8_LOB,
+                                                                    clink,
+                                                                    false);
     data.connectionitem = &clink;
     if(data.lobdata == 0) {
         clink.error().setMemoryAllocationFailed();
@@ -675,11 +755,11 @@ IFRConversion_StreamConverter::translateNullInput(IFR_Parameter& parameter,
 
     if(parameter.data() && parameter.isLOB()) {
         IFR_LOBData& data = parameter.asLOBData(dataoffset, rowsize);
-        data.lobdata = new (m_name.getAllocator()) IFR_LOB(getIndex(),
-                                                           dataoffset + 1, 
-                                                           parameter.getHostType(),
-                                                           clink,
-                                                           true);
+        data.lobdata = new IFR_ALLOCATOR(m_name.getAllocator()) IFR_LOB(getIndex(),
+                                                                        dataoffset + 1, 
+                                                                        parameter.getHostType(),
+                                                                        clink,
+                                                                        true);
         data.connectionitem = &clink;
         if(data.lobdata == 0) {
             clink.error().setMemoryAllocationFailed();
@@ -713,11 +793,11 @@ IFRConversion_StreamConverter::translateDefaultInput(IFR_Parameter& parameter,
     }
     if(parameter.data() && parameter.isLOB()) {
         IFR_LOBData& data = parameter.asLOBData(dataoffset, rowsize);
-        data.lobdata = new (m_name.getAllocator()) IFR_LOB(getIndex(),
-                                                           dataoffset + 1,
-                                                           parameter.getHostType(),
-                                                           clink,
-                                                           true);
+        data.lobdata = new IFR_ALLOCATOR(m_name.getAllocator()) IFR_LOB(getIndex(),
+                                                                        dataoffset + 1,
+                                                                        parameter.getHostType(),
+                                                                        clink,
+                                                                        true);
         data.connectionitem = &clink;
         if(data.lobdata == 0) {
             clink.error().setMemoryAllocationFailed();
