@@ -31,55 +31,49 @@
 #ifdef HPUX
 #include <sys/param.h>
 #endif
+
+#include "SAPDB/Interfaces/Runtime/IFR_Common.h"
 #include "SAPDB/Interfaces/Runtime/IFR_FetchInfo.h"
 #include "SAPDB/Interfaces/Runtime/IFR_Statement.h"
 #include "SAPDB/Interfaces/Runtime/Packet/IFRPacket_ReplySegment.h"
 #include "SAPDB/Interfaces/Runtime/Packet/IFRPacket_PartEnum.h"
 #include "SAPDB/Interfaces/Runtime/Packet/IFRPacket_Part.h"
+#include "SAPDB/Interfaces/Runtime/Util/IFRUtil_StackAllocator.h"
 
 IFR_FetchInfo::IFR_FetchInfo (IFR_Statement& statement,
-                              IFRConversion_Converter **info,
+                              IFRConversion_ConverterList& info,
 		                      IFRUtil_Vector<IFR_String>* colName,
                               IFR_Bool& memory_ok)
 :IFRUtil_RuntimeItem(statement),
  m_Statement(statement), 
  m_CursorName(statement.getCursorName(), memory_ok),
- m_FetchParamString ("", IFR_StringEncodingAscii, allocator, memory_ok), 
- m_ConverterLength (0),
- m_ColumnInfo(0),
- m_columnInfoIsReference(false)
+ m_columninfo(allocator)
 {
     DBUG_METHOD_ENTER(IFR_FetchInfo, IFR_FetchInfo);
     if(memory_ok) {
-        if(info!=0 && colName!=0) {
+        if(info.size() !=0 && colName!=0) {
             this->setMetaData(info, colName);
-        }
+        } 
     }
 }
 
 //----------------------------------------------------------------------
 IFR_FetchInfo::~IFR_FetchInfo ()
 {
-    DBUG_METHOD_ENTER(IFR_FetchInfo, ~IFR_FetchInfo);
-    
-    if (!m_columnInfoIsReference) {
-        for (IFR_Int4 i = 0; i < m_ConverterLength; i++) {
-            IFRUtil_Delete(m_ColumnInfo[i], allocator);
-        }  
-        IFRUtil_DeleteArray(m_ColumnInfo, m_ConverterLength, allocator);
-    }
 }
 
 IFR_Retcode IFR_FetchInfo::describe () 
 {
     DBUG_METHOD_ENTER(IFR_FetchInfo, describe);
     // return if we have already all information
-    if (this->m_ColumnInfo != 0)
+    if (this->m_columninfo.size() != 0) {
         DBUG_RETURN(IFR_OK);
+    }
     IFR_Bool memory_ok=true;
     IFR_Connection *connection = m_Statement.getConnection();
     IFRPacket_RequestPacket requestPacket(*this);
     IFR_Retcode rc=connection->getRequestPacket (requestPacket);
+    IFRUtil_StackAllocator<256> describe_allocator;
     if(rc != IFR_OK) {
         error() = connection->error();
         DBUG_RETURN(rc);
@@ -87,7 +81,7 @@ IFR_Retcode IFR_FetchInfo::describe ()
 
     requestPacket.setEncoding(m_Statement.getCommandEncoding());
     
-    IFR_String sqlCmd(requestPacket.getEncoding(), allocator);
+    IFR_String sqlCmd(requestPacket.getEncoding(), describe_allocator);
     sqlCmd.append("DESCRIBE \"", IFR_StringEncodingAscii, IFR_NTS, memory_ok);
     sqlCmd.append(this->m_CursorName, memory_ok);
     sqlCmd.append("\"", IFR_StringEncodingAscii, IFR_NTS, memory_ok);
@@ -95,6 +89,8 @@ IFR_Retcode IFR_FetchInfo::describe ()
         error().setMemoryAllocationFailed();
         DBUG_RETURN(IFR_NOT_OK);
     }
+    
+    IFR_SQL_TRACE << "::DESCRIBE CURSOR " << m_CursorName << " " << currenttime << endl;
 
     IFRPacket_RequestSegment segment(requestPacket, IFRPacket_CommandMessageType::Dbs_C);      
     IFRPacket_CommandPart command;
@@ -109,83 +105,97 @@ IFR_Retcode IFR_FetchInfo::describe ()
     segment.close();
     
     IFRPacket_ReplyPacket replyPacket;
-    rc = connection->sqlaexecute (requestPacket, replyPacket, false, error());
+    rc = connection->sqlaexecute (requestPacket, replyPacket, IFR_Connection::AppendNotAllowed_C, error());
     if(rc != IFR_OK) {
         DBUG_RETURN(rc);
     } else if(error()) {
         DBUG_RETURN(IFR_NOT_OK);
     }
     IFRPacket_ReplySegment replysegment(replyPacket);
-    IFRPacket_PartEnum partEnum (&replysegment);
+    IFRPacket_PartEnum partEnum (replysegment);
     
-    IFRConversion_Converter **infos = 0;
+    IFRConversion_ConverterList infos(allocator);
     IFRUtil_Vector<IFR_String> *columnNames = 0;
     
     while (partEnum.hasMoreElements ()) {
-        int knd = partEnum.partKind ();
-        if (knd == IFRPacket_PartKind::Columnnames_C) {
+        int knd = partEnum.getPartKind ();
+        switch(knd) {
+        case IFRPacket_PartKind::Columnnames_C:
             replysegment.parseColumnNames (&columnNames, allocator, memory_ok);
             if(!memory_ok) {
                 error().setMemoryAllocationFailed();
                 DBUG_RETURN(IFR_NOT_OK);
             }
-        } else if (knd == IFRPacket_PartKind::Shortinfo_C) {
-            replysegment.parseShortFields (infos, m_Statement);
-        } else {
+            break;
+        case IFRPacket_PartKind::Shortinfo_C: {
+            IFRPacket_ShortInfoPart part;
+            partEnum.getPart(part);
+            part.parseShortFields (infos, m_Statement);
+            break;
         }
-        partEnum.nextElement ();
+        case IFRPacket_PartKind::VarDataShortInfo_C: {
+            IFRPacket_VarDataShortInfoPart part;
+            partEnum.getPart(part);
+            part.parseShortFields(infos, m_Statement);
+            break;
+        }
+        }
+        partEnum.next();
     }
     this->setMetaData (infos, columnNames);
+    IFRUtil_Delete(columnNames, allocator);
     DBUG_RETURN(IFR_OK);
 }
 
 //----------------------------------------------------------------------
 IFR_Retcode
-IFR_FetchInfo::setMetaData (IFRConversion_Converter **info,
+IFR_FetchInfo::setMetaData (IFRConversion_ConverterList& info,
                             IFRUtil_Vector<IFR_String>* colName)
 {
   DBUG_METHOD_ENTER(IFR_FetchInfo, setMetaData);
   
   IFR_String currentName(allocator);
   int currentFieldEnd;
-
-  this->m_RecordSize = 0;
-  this->m_ColumnInfo = info;
+  
   IFR_Bool memory_ok=true;
-
+  m_RecordSize = 0;
+  m_columninfo.swap(info, memory_ok);
+  
   for (IFR_UInt4 i = 0; i < colName->GetSize (); i++) {
-      info[i]->setName ((*colName)[i], memory_ok);
+      m_columninfo[i]->setName ((*colName)[i], memory_ok);
       if(!memory_ok) {
           error().setMemoryAllocationFailed();
           DBUG_RETURN(IFR_NOT_OK);
       }
-      info[i]->setIndex(i+1);
-      currentFieldEnd = info[i]->getIOLength () + info[i]->getBufpos () - 1;
+      m_columninfo[i]->setIndex(i+1);
+      currentFieldEnd = m_columninfo[i]->getIOLength () + m_columninfo[i]->getBufpos () - 1;
       this->m_RecordSize = MAX (this->m_RecordSize, currentFieldEnd);
   }
-  m_ConverterLength = colName->GetSize ();
-  
   DBUG_RETURN(IFR_NOT_OK);
 }
 
 //----------------------------------------------------------------------
-void 
+IFR_Retcode 
 IFR_FetchInfo::setMetaDataRefFromParseinfo (IFRConversion_Converter **info,
-                                            IFR_Int2 columncount) 
+                                            IFR_UInt4 columncount) 
 {
     DBUG_METHOD_ENTER(IFR_FetchInfo, copyMetaDataFromParseinfo);
     
     int currentFieldEnd;
-    this->m_columnInfoIsReference = true;
-    this->m_ConverterLength = columncount;
-    this->m_RecordSize = 0;
-    this->m_ColumnInfo = info;
-    
-    for (IFR_Int4 i = 0; i < columncount; i++) {
+    m_RecordSize = 0;
+    IFR_Bool memory_ok = true;
+    // no indirect pos as only output parameters are interesting
+    m_columninfo.copyref(info, columncount, false, memory_ok);
+
+    if(!memory_ok) {
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    for (IFR_UInt4 i = 0; i < columncount; i++) {
         info[i]->setIndex(i+1);
         currentFieldEnd = info[i]->getIOLength () + info[i]->getBufpos () - 1;
         this->m_RecordSize = MAX (this->m_RecordSize, currentFieldEnd);
     }
+    DBUG_RETURN(IFR_OK);
 }
 
 //----------------------------------------------------------------------
@@ -210,7 +220,7 @@ IFR_Retcode IFR_FetchInfo::executeFetchAbsolute (IFR_Int4 position,
     cmd.append(" \"",IFR_StringEncodingAscii, IFR_NTS, memory_ok);
     cmd.append(m_CursorName, memory_ok);
     cmd.append("\" INTO ", IFR_StringEncodingAscii, IFR_NTS, memory_ok);
-    cmd.append(getFetchParamString(memory_ok), memory_ok);
+    cmd.append(s_fetchparamstring, IFR_StringEncodingAscii, m_columninfo.size() * 3 - 2, memory_ok);
     if(!memory_ok) {
         error().setMemoryAllocationFailed();
         DBUG_RETURN(IFR_NOT_OK);
@@ -235,7 +245,7 @@ IFR_Retcode IFR_FetchInfo::executeFetchRelative (IFR_Int4 position,
     cmd.append(" \"", IFR_StringEncodingAscii, IFR_NTS, memory_ok);
     cmd.append(m_CursorName, memory_ok);
     cmd.append("\" INTO ", IFR_StringEncodingAscii, IFR_NTS, memory_ok);
-    cmd.append(getFetchParamString(memory_ok), memory_ok);
+    cmd.append(s_fetchparamstring, IFR_StringEncodingAscii, m_columninfo.size() * 3 - 2, memory_ok);
     if(!memory_ok) {
         error().setMemoryAllocationFailed();
         DBUG_RETURN(IFR_NOT_OK);
@@ -257,7 +267,29 @@ IFR_FetchInfo::executeFetchFirst(IFR_Int4 fetchSize,
     }
     cmd.append(this->m_CursorName, memory_ok);
     cmd.append("\" INTO ", IFR_StringEncodingAscii, IFR_NTS, memory_ok);
-    cmd.append(getFetchParamString(memory_ok), memory_ok);
+    cmd.append(s_fetchparamstring, IFR_StringEncodingAscii, m_columninfo.size() * 3 - 2, memory_ok);
+    if(!memory_ok) {
+        error().setMemoryAllocationFailed();
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    DBUG_RETURN(executeFetchCommand (cmd, fetchSize, replyPacket));
+}
+
+//----------------------------------------------------------------------
+IFR_Retcode 
+IFR_FetchInfo::executeFetchNext(IFR_Int4 fetchSize, 
+                                 IFRPacket_ReplyPacket& replyPacket)
+{
+    DBUG_METHOD_ENTER(IFR_FetchInfo, executeFetchNext);
+    IFR_Bool memory_ok=true;
+    IFR_String cmd ("FETCH NEXT \"", IFR_StringEncodingAscii, allocator, memory_ok);
+    if(!memory_ok) {
+        error().setMemoryAllocationFailed();
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    cmd.append(this->m_CursorName, memory_ok);
+    cmd.append("\" INTO ", IFR_StringEncodingAscii, IFR_NTS, memory_ok);
+    cmd.append(s_fetchparamstring, IFR_StringEncodingAscii, m_columninfo.size() * 3 - 2, memory_ok);
     if(!memory_ok) {
         error().setMemoryAllocationFailed();
         DBUG_RETURN(IFR_NOT_OK);
@@ -279,7 +311,7 @@ IFR_FetchInfo::executeFetchLast (IFR_Int4 fetchSize,
     }
     cmd.append(this->m_CursorName, memory_ok);
     cmd.append("\" INTO ", IFR_StringEncodingAscii, IFR_NTS, memory_ok);
-    cmd.append(getFetchParamString(memory_ok), memory_ok);
+    cmd.append(s_fetchparamstring, IFR_StringEncodingAscii, m_columninfo.size() * 3 - 2, memory_ok);
     if(!memory_ok) {
         error().setMemoryAllocationFailed();
         DBUG_RETURN(IFR_NOT_OK);
@@ -289,25 +321,40 @@ IFR_FetchInfo::executeFetchLast (IFR_Int4 fetchSize,
 }
 
 //----------------------------------------------------------------------
-const IFR_String& 
-IFR_FetchInfo::getFetchParamString(IFR_Bool& memory_ok)
-{
-    DBUG_METHOD_ENTER(IFR_FetchInfo, getFetchParamString);
-
-    if(!memory_ok) {
-        DBUG_RETURN(m_FetchParamString);
-    }
-
-    if(m_FetchParamString.getLength()==0) {
-        m_FetchParamString.expand(m_ConverterLength*3-2, memory_ok);
-        m_FetchParamString.setBuffer("?", 1, IFR_StringEncodingAscii, memory_ok);
-        for(int i=1; i<m_ConverterLength;++i) {
-            m_FetchParamString.append(", ?", IFR_StringEncodingAscii, 3, memory_ok);
-        }
-    }
-    
-    DBUG_RETURN(m_FetchParamString);
-}
+const char *
+IFR_FetchInfo::s_fetchparamstring = 
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
+    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ";
 
 //----------------------------------------------------------------------
 IFR_Retcode 
@@ -329,6 +376,18 @@ IFR_FetchInfo::executeFetchCommand (IFR_String& command,
 
     IFR_Retcode rc = IFR_OK;
     IFR_String errText(allocator);
+    IFR_StringEncoding cmdEncoding = m_Statement.getCommandEncoding(); 
+    IFR_String realCommandBuffer(cmdEncoding, allocator);
+    IFR_String *realcommand=&command;
+    IFR_Bool   memory_ok = true;
+    if(command.getEncoding() != cmdEncoding) {
+        realCommandBuffer.append(command, memory_ok);
+        if(!memory_ok) {
+            error().setMemoryAllocationFailed();
+            DBUG_RETURN(IFR_NOT_OK);
+        }
+        realcommand = &realCommandBuffer;
+    }
 
     IFRPacket_RequestPacket request(*this);
     IFR_Bool reset = true;
@@ -339,54 +398,44 @@ IFR_FetchInfo::executeFetchCommand (IFR_String& command,
         return rc;
     }
     int currentSQLMode = request.switchSqlMode (IFR_INTERNAL);
+    request.setEncoding(cmdEncoding);
     IFRPacket_RequestSegment segment (request, sp1m_dbs, reset);
     if(segment.isValid()) {
         IFRPacket_CommandPart commandpart;
-        IFRPacket_LongDemandPart longdemandpart;
         if((rc = segment.addPart(commandpart)) != IFR_OK) {
             error().setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
             request.switchSqlMode(currentSQLMode);
             DBUG_RETURN(rc);
         }
-        if (IFR_OK != commandpart.setText(command, error())) {
+        if (IFR_OK != commandpart.setText(*realcommand, error())) {
           request.switchSqlMode(currentSQLMode);
           DBUG_RETURN(IFR_NOT_OK);
         } 
         segment.closePart ();
         if (m_Statement.m_resultset->hasPositionedRead ()) {
-          if((rc = segment.addPart(longdemandpart)) != IFR_OK) {
-            error().setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
-            request.switchSqlMode(currentSQLMode);
-            DBUG_RETURN(rc);
-          }
-          const IFRUtil_Vector<IFR_Parameter> *paramvec = m_Statement.m_resultset->getParamVector ();
-          for (unsigned int i = 0; i < paramvec->GetSize (); i++) {
-              if (m_ColumnInfo[i]->isLong ()) {
-                  IFR_Parameter param = (*paramvec)[i];
-                  const IFR_Length *pos = param.getPosIndicator ();
-#if 1
-                  if (pos) {
-                      longdemandpart.addPosParams (true, 
-                                                   *pos, 
-                                                   (*paramvec)[i].getBytesLength ());
-                  } else {
-                      longdemandpart.addPosParams (true, 
-                                                   1, 
-                                                   (*paramvec)[i].getBytesLength ());
-                  }
-#else
-                  if (pos) {
-                      longdemandpart.addPosParams (true, 
-                                                   2, 
-                                                   50);
-                  } else {
-                      longdemandpart.addPosParams (true, 
-                                                   3, 
-                                                   60);
-                  }
-#endif
-              }
-          }
+            IFRPacket_LongDemandPart longdemandpart;
+            if((rc = segment.addPart(longdemandpart)) != IFR_OK) {
+                error().setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
+                request.switchSqlMode(currentSQLMode);
+                DBUG_RETURN(rc);
+            }
+            const IFRUtil_Vector<IFR_Parameter> *paramvec = m_Statement.m_resultset->getParamVector ();
+            IFR_size_t paramvec_size = paramvec->GetSize();
+            for (unsigned int i = 0; i < paramvec_size; i++) {
+                if (m_columninfo[i]->isLong ()) {
+                    IFR_Parameter param = (*paramvec)[i];
+                    const IFR_Length *pos = param.getPosIndicator ();
+                    if (pos) {
+                        longdemandpart.addPosParams (true, 
+                                                     *pos, 
+                                                     (*paramvec)[i].getBytesLength ());
+                    } else {
+                        longdemandpart.addPosParams (true, 
+                                                     1, 
+                                                     (*paramvec)[i].getBytesLength ());
+                    }
+                }
+            }
           segment.closePart ();
         }
         error ().clear ();
@@ -398,7 +447,8 @@ IFR_FetchInfo::executeFetchCommand (IFR_String& command,
         segment.addFetchSize(fetchSize);
         segment.close();
         IFR_Retcode rc_sqlaexecute;
-        if((rc_sqlaexecute=connection->sqlaexecute (request, replyPacket, false, error())) != IFR_OK) {
+        if((rc_sqlaexecute=connection->sqlaexecute (request, replyPacket, 
+                                                    IFR_Connection::AppendNotAllowed_C, error())) != IFR_OK) {
             DBUG_RETURN(rc_sqlaexecute);
         } 
         IFRPacket_ReplySegment replysegment(replyPacket);
@@ -417,17 +467,19 @@ IFRConversion_Converter *
 IFR_FetchInfo::findColInfo (IFR_Int2 columnIndex)
 {
     DBUG_METHOD_ENTER(IFR_FetchInfo, findColInfo);
-  if (columnIndex > 0 && columnIndex <= this->m_ConverterLength)
-    return (this->m_ColumnInfo[columnIndex-1]); 
-  else
-    return 0;  
+    if(columnIndex < 0  || (IFR_UInt4)columnIndex > m_columninfo.size()) {
+        return 0;
+    } else {
+        return m_columninfo[columnIndex-1]; 
+    }
 }
+
 //----------------------------------------------------------------------
 IFR_Int2
 IFR_FetchInfo::getColumnCount()
 {
     DBUG_METHOD_ENTER(IFR_FetchInfo, getColumnCount);
-    DBUG_RETURN(this->m_ConverterLength);
+    DBUG_RETURN((IFR_Int2) m_columninfo.size());
 }
 
 
