@@ -48,8 +48,6 @@
  * on the pointer that refers to the data - if bound by address the data
  * pointer points to a pointer to the data, otherwise it points directly
  * to the data. 
- *
- * @todo User-defined parameters.
  */
 class IFR_Parameter 
 
@@ -96,6 +94,18 @@ public:
      m_posindicator(posindicator)
     {}
 
+
+
+    /**
+     * Determine whether a parameter is not set.
+     * @return @c true if the parameter is not set and is here
+     *  only because parameters are stored as a vector.
+     */
+    inline IFR_Bool isNotSet() const 
+    {
+        return m_hosttype == IFR_HOSTTYPE_PARAMETER_NOTSET;
+    }
+
     /**
      * Get a pointer to the data.
      * @return A pointer to the data, which may be <code>0</code>.
@@ -516,7 +526,10 @@ public:
      * @return The precision that is most likely to match 
      *         the parameter format.
      */
-    IFR_Int1 getPreferredPrecision() const;
+    inline IFR_Int1 getPreferredPrecision() const
+    {
+        return 0;
+    }
     
     /**
      * Gets the length that is likely to match the host type.
@@ -536,8 +549,21 @@ public:
      * @return <code>true</code> if the 'DATA AT EXECUTE' mechanism is used, 
      *   <code>false</code> if not.
      */
-    IFR_Bool hasDataAtExecute(IFR_Int4 offset=0, IFR_size_t rowsize=0);
-   
+    inline IFR_Bool hasDataAtExecute(IFR_Int4 offset=0, IFR_size_t rowsize=0)
+    {
+        return hasDataAtExecute(getLengthIndicator(offset, rowsize));
+    }
+    
+    /**
+     * Checks whether the data of this parameter points to the
+     * ABAP table / OMS stream with the given id.
+     * @param abapTabId The table id to look for.
+     * @return @c true if this parameter identifies the requested
+     *   ABAP table or OMS stream.
+     */
+    IFR_Bool isABAPTable(IFR_Int4 abapTabId);
+
+
     /**
      * Check whether a given indicator means that the data is delivered at
      * execute time instead of being delivered at prepare time.
@@ -563,12 +589,49 @@ public:
      * @param hosttype The host type to check.
      */
     static IFR_StringEncoding getEncoding(IFR_HostType hosttype);
+
+    /**
+     * Traces the parameter. The data content is not traced, but the 
+     * pointers that are stored in the parameter.
+     * @param s The output stream.
+     * @param parameterindex The index of the parameter.
+     * @param nodata Do not print The indicator/data pointers
+     * @return @c true if the parameter is really bound, @c false if it is not set.
+     */
+    IFR_Bool sqlTraceParameter(IFR_TraceStream& s, IFR_UInt2 parameterindex, IFR_Bool nodata=false);
+
+    /**
+     * Traces the parameter. The data content is not traced, but the 
+     * pointers that are stored in the parameter.
+     * @param s The output stream.
+     * @param parametertag Tag printed instead of index.
+     * @param nodata Do not print The indicator/data pointers
+     * @return @c true if the parameter is really bound, @c false if it is not set.
+     */
+    IFR_Bool sqlTraceParameter(IFR_TraceStream& s, const char *parametertag, IFR_Bool nodata=false);
     
-  void sqlTrace(IFR_TraceStream& s, IFR_Retcode convrc = IFR_OK, IFR_Int4 offset=0, IFR_size_t rowsize = 0,
-                unsigned char *parameterdata = 0, IFR_Length* lengthindicator = 0 );
-    
-  void sqlTraceData(IFR_TraceStream& s, IFR_Int4 offset=0, IFR_size_t rowsize = 0,
-                unsigned char *parameterdata = 0, IFR_Length* lengthindicator = 0 );
+    /**
+     * Traces the parameter. The data content is not traced, but the 
+     * pointers that are stored in the parameter.
+     * @param s              The output stream.
+     * @param parameterindex The index of the parameter.
+     * @param conversionRC   The return code of the conversion routine that previously did fill
+     *                       the parameter.
+     * @param offset         The offset for getting the parameter data.
+     * @param rowsize        The rowsize for getting the parameter data.
+     * @param parameterdata  Alternative parameter data.
+     * @param lengthindicator Alternative length indicator.
+     * @param input          Tracing is for input data (data is there).
+     * @return @c true if the parameter is really bound, @c false if it is not set.
+     */
+    void sqlTraceParameterData(IFR_TraceStream& s, 
+                               IFR_UInt2        parameterindex, 
+                               IFR_Retcode      conversionRC,
+                               IFR_Int4         offset,
+                               IFR_Int4         rowsize,
+                               unsigned char   *parameterdata,
+                               IFR_Length      *lengthindicator,
+                               IFR_Bool         input);
     
 private:
     /**
