@@ -52,21 +52,23 @@ IFR_Statement::IFR_Statement (IFR_Connection &conn, IFR_Bool& memory_ok)
 :IFRUtil_RuntimeItem(conn),
  IFR_ConnectionItem(&conn),
  m_RowsAffected(-1),
- m_ResultSetType (SCROLL_SENSITIVE),
  m_ResultSetConcurrency (READ_ONLY),
  m_ResultSetHoldability (HOLD_CURSORS_OVER_COMMIT),
  m_FetchSize (SAPDB_MAX_INT2),
  m_MaxRows (0),
  m_QueryTimeout (0),
  m_CursorName(allocator),
+ m_commandInfo(allocator),
  m_TableName(allocator),
  m_resultset(NULL), 
  m_Connection(&conn),
- m_cursorreallyused (false),
+ m_CommandEncoding(IFR_StringEncodingUnknown), 
+ m_cursorstate(CursorStateNotUsed_C),
  m_rowarraysize(1),
- m_rowstatusarray(new (allocator) IFR_Int4[1]),
+ m_rowstatusarray(new IFR_ALLOCATOR(allocator) IFR_Int4[1]),
  m_rowstatussize(1),
- m_batchelements(0)
+ m_batchelements(0),
+ m_lastserial(0)
 {
      DBUG_METHOD_ENTER(IFR_Statement, IFR_Statement);
      conn.nextCursorName(m_CursorName, memory_ok);
@@ -79,13 +81,14 @@ IFR_Statement::IFR_Statement (IFR_Connection &conn, IFR_Bool& memory_ok)
      DBUG_PRINT(&conn);
      DBUG_PRINT(m_CursorName);
      m_rowstatusarray[0]=IFR_EXECUTE_FAILED;
+     m_ResultSetType = (IFR_Statement::ResultSetType) conn.getDefaultResultSetType();
 }
 
 //----------------------------------------------------------------------
 IFR_Statement::~IFR_Statement ()
 {
     DBUG_METHOD_ENTER(IFR_Statement, ~IFR_Statement);
-    if(m_cursorreallyused) {
+    if(m_cursorstate == CursorStateUsed_C) {
         DBUG_PRINT(m_CursorName);
         IFR_Bool memory_ok = true;
         getConnection()->dropCursor(m_CursorName, memory_ok);
@@ -94,8 +97,9 @@ IFR_Statement::~IFR_Statement ()
         }
     }
     IFRUtil_Delete(m_resultset, allocator);
-    IFRUtil_DeleteArray(m_rowstatusarray, m_rowarraysize, allocator);
+    IFRUtil_DeleteArray(m_rowstatusarray, m_rowarraysize, IFR_ALLOCATOR(allocator));
     IFRUtil_Delete(m_batchelements, allocator);
+    IFRUtil_Delete(m_lastserial, allocator);
 }
 
 //----------------------------------------------------------------------
@@ -147,7 +151,7 @@ IFR_Statement::sendCommand (IFRPacket_RequestPacket &requestPacket,
                             IFR_Bool masscommand, 
                             IFR_Bool parseonly,
                             IFR_Bool parseAgain,
-                            IFR_Bool allowAppended,
+                            IFR_Connection::AppendMode appendFlag,
                             IFRPacket_ReplyPacket& replyPacket)
 {
   DBUG_METHOD_ENTER(IFR_Statement, sendCommand);
@@ -155,7 +159,8 @@ IFR_Statement::sendCommand (IFRPacket_RequestPacket &requestPacket,
   DBUG_PRINT(masscommand);
   DBUG_PRINT(parseonly);
   DBUG_PRINT(parseAgain);
-  DBUG_PRINT(allowAppended);
+  DBUG_PRINT(appendFlag);
+
   if (requestPacket.isValid()) {
       /* set encoding of packet */
       if (!setCommandEncoding(sqlCmd.getEncoding(), requestPacket.isSwapped())) {        
@@ -210,11 +215,14 @@ IFR_Statement::sendCommand (IFRPacket_RequestPacket &requestPacket,
       if(this->m_Connection->isWithInfoSet()) {
           requestPacket.setWithInfo();
       }
-
       if(m_Connection->getAutoCommit()) {
           segment.setCommitImmediately();
       }
-
+      /*
+      if(messagetype = IFRPacket_CommandMessageType::Parse_C) {
+          segment.setParseWithPrepare();
+      }
+      */
       // puts the the SQL command
       IFRPacket_CommandPart command;
       if (IFR_OK != segment.addPart(command)) {
@@ -247,10 +255,23 @@ IFR_Statement::sendCommand (IFRPacket_RequestPacket &requestPacket,
             DBUG_RETURN(IFR_NOT_OK);
           }
           segment.closePart();
+          
+          segment.setScrollableOption(getConnection()->getScrollableOption(m_ResultSetType));
+          
+      } else if (m_commandInfo.getLength() !=0) {
+          IFRPacket_CommandInfoPart comInf;
+          if(IFR_OK != segment.addPart(comInf)) {
+            error().setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
+            DBUG_RETURN(IFR_NOT_OK);
+          }
+          if (IFR_OK != comInf.addCommandInfo(this->m_commandInfo, this->m_commandLineNumber, error())) {
+            DBUG_RETURN(IFR_NOT_OK);
+          }
+          segment.closePart();
       }
       segment.close();
 
-      IFR_Retcode rc = m_Connection->sqlaexecute (requestPacket, replyPacket, allowAppended, error());
+      IFR_Retcode rc = m_Connection->sqlaexecute (requestPacket, replyPacket, appendFlag, error());
       DBUG_RETURN(rc);
   } else {
       error().setRuntimeError(IFR_ERR_INVALID_REQUESTPACKET);
@@ -265,14 +286,14 @@ IFR_Statement::sendSQL (const IFR_String &sql,
                         IFR_Bool masscommand, 
                         IFR_Bool parseonly,
                         IFR_Bool parseAgain,
-                        IFR_Bool allowAppended)
+                        IFR_Connection::AppendMode appendFlag)
 {
     DBUG_METHOD_ENTER(IFR_Statement, sendSQL);
     DBUG_PRINT(sql);
     DBUG_PRINT(masscommand);
     DBUG_PRINT(parseonly);
     DBUG_PRINT(parseAgain);
-    DBUG_PRINT(allowAppended);
+    DBUG_PRINT(appendFlag);
     
     IFR_String actualSQL(sql.getAllocator());
     bool touchedCommand = false;
@@ -303,22 +324,16 @@ IFR_Statement::sendSQL (const IFR_String &sql,
         }
     }
     DBUG_PRINT(touchedCommand);
-    if (touchedCommand  &&  this->m_ResultSetType == SCROLL_INSENSITIVE) {
-      // @todo; implement for update and for reuse
-      error().setRuntimeError(IFR_ERR_NOT_IMPLEMENTED("FOR UPDATE/FOR REUSE"));
-      DBUG_RETURN(IFR_NOT_OK);
-    } else {
-        IFRPacket_RequestPacket requestPacket(*this);
-        rc= m_Connection->getRequestPacket (requestPacket);
-        if(rc != IFR_OK) {
-            error() = m_Connection->error();
-        }
-        else {
-          if (touchedCommand)
-            rc = this->sendCommand (requestPacket, actualSQL, masscommand, parseonly, parseAgain, allowAppended, replyPacket);    
-          else
-            rc = this->sendCommand (requestPacket, sql, masscommand, parseonly, parseAgain, allowAppended, replyPacket);    
-        }
+    IFRPacket_RequestPacket requestPacket(*this);
+    rc= m_Connection->getRequestPacket (requestPacket);
+    if(rc != IFR_OK) {
+        error() = m_Connection->error();
+    }
+    else {
+        if (touchedCommand)
+            rc = this->sendCommand (requestPacket, actualSQL, masscommand, parseonly, parseAgain, appendFlag, replyPacket);    
+        else
+            rc = this->sendCommand (requestPacket, sql, masscommand, parseonly, parseAgain, appendFlag, replyPacket);    
     }
     DBUG_RETURN(rc);
 }
@@ -333,7 +348,7 @@ IFR_Statement::parseResult (IFRPacket_ReplyPacket &replyPacket,
   IFR_Bool memory_ok = true;
   bool rowNotFound = false;
   bool isQuery = false;
-  IFRConversion_Converter **infos = 0;
+  IFRConversion_ConverterList infos(allocator);
   IFR_String oldCursorName(allocator);
   // oldCursorName is used as dummy.
   IFRUtil_Vector<IFR_String> columNames(allocator, 0, oldCursorName, memory_ok);
@@ -341,7 +356,7 @@ IFR_Statement::parseResult (IFRPacket_ReplyPacket &replyPacket,
   if (segm.FunctionCode().isQuery()) {
       isQuery = true;
   }
-  if(m_cursorreallyused || isQuery) {
+  if(m_cursorstate == CursorStateUsed_C || isQuery) {
       // need to record old name cause result table part will 
       // overwrite
       oldCursorName.assign(m_CursorName, memory_ok);
@@ -351,37 +366,42 @@ IFR_Statement::parseResult (IFRPacket_ReplyPacket &replyPacket,
       }
   }
   DBUG_PRINT(isQuery);
-  IFRPacket_PartEnum partEnum (&segm);
+  IFR_FetchChunk *select_fetch_chunk = 0;
+  IFR_Bool datapart_found = false;
+  IFRPacket_PartEnum partEnum (segm);
   while (partEnum.hasMoreElements ()) {
-    IFRPacket_PartKind::PartKind PartKind = partEnum.partKind ();
+      IFRPacket_PartKind::PartKind PartKind = partEnum.getPartKind();
     DBUG_PRINT(PartKind);
     switch(PartKind) {
     case (IFRPacket_PartKind::Columnnames_C):{
-      IFRPacket_ColumnNamesPart part = partEnum.getPart();
-      part.getColumnNames(columNames, memory_ok);
-      if(!memory_ok) {
-          error().setMemoryAllocationFailed();
-          DBUG_RETURN(IFR_NOT_OK);
-      }
+        IFRPacket_ColumnNamesPart part;
+        partEnum.getPart(part);
+        part.getColumnNames(columNames, memory_ok);
+        if(!memory_ok) {
+            error().setMemoryAllocationFailed();
+            DBUG_RETURN(IFR_NOT_OK);
+        }
       break;
     }  
     case (IFRPacket_PartKind::Shortinfo_C): {
-      IFRPacket_ShortInfoPart part = partEnum.getPart();
-      part.parseShortFields(infos, *this);
-      break;
+        IFRPacket_ShortInfoPart part;
+        partEnum.getPart(part);
+        part.parseShortFields(infos, *this);
+        break;
     }
     case (IFRPacket_PartKind::Resultcount_C): {
-      IFR_Int4 ResCount;
-      if (segm.getResultCount(ResCount) == IFR_OK)
-        this->m_RowsAffected = ResCount;        
-      DBUG_PRINT(ResCount);
-      //>>> SQL TRACE
-      IFR_SQL_TRACE << "RESULT COUNT: " << ResCount << endl;
-      //<<< SQL TRACE
-      break;
+        IFR_Int4 ResCount;
+        if (segm.getResultCount(ResCount) == IFR_OK)
+            this->m_RowsAffected = ResCount;        
+        DBUG_PRINT(ResCount);
+        //>>> SQL TRACE
+        IFR_SQL_TRACE << "RESULT COUNT: " << ResCount << endl;
+        //<<< SQL TRACE
+        break;
     }
     case (IFRPacket_PartKind::Resulttablename_C): {
-        IFRPacket_TableNamePart part = partEnum.getPart();
+        IFRPacket_TableNamePart part;
+        partEnum.getPart(part);
         part.getResultName(this->m_CursorName, memory_ok);
         // The cursor will be only changed if this is 
         // not a parse that delivered a cursor name of length 
@@ -400,6 +420,14 @@ IFR_Statement::parseResult (IFRPacket_ReplyPacket &replyPacket,
         //<<< SQL TRACE
         break;
     }
+    case (IFRPacket_PartKind::Serial_C): {
+        IFRPacket_DataPart serialData;
+        partEnum.getPart(serialData);
+        if(setSerialPart(serialData) == IFR_NOT_OK) {
+            DBUG_RETURN(IFR_NOT_OK);
+        }
+        break;
+    }
     case (IFRPacket_PartKind::Errortext_C): {
         segm.getSQLError(error(), allocator);
         if (error ().getErrorCode () == 100) {
@@ -416,46 +444,92 @@ IFR_Statement::parseResult (IFRPacket_ReplyPacket &replyPacket,
         break;
     }
     case (IFRPacket_PartKind::Tablename_C):{
-      IFRPacket_TableNamePart part = partEnum.getPart();
-      part.getResultName(this->m_TableName, memory_ok);
+        IFRPacket_TableNamePart part;
+        partEnum.getPart(part);
+        part.getResultName(this->m_TableName, memory_ok);
+        if(!memory_ok) {
+            error().setMemoryAllocationFailed();
+            DBUG_RETURN(IFR_NOT_OK);
+        }
+        DBUG_PRINT(this->m_TableName);
+        //>>> SQL TRACE
+        IFR_SQL_TRACE << "TABLE NAME: " << m_TableName << endl;
+        //<<< SQL TRACE
+        break;
+    }
+    case (IFRPacket_PartKind::ParsidOfSelect_C):{
+      // @todo; Handling for ParsidOfSelect_C
+        error().setRuntimeError(IFR_ERR_NOT_IMPLEMENTED("ParsidOfSelect"));
+        DBUG_RETURN(IFR_NOT_OK);
+        break;
+    }
+    case (IFRPacket_PartKind::Data_C): {
+        datapart_found = true;
+        break;
+    }
+    default: {
+        break;
+    }
+    }
+    partEnum.next();
+  }
+  IFR_Bool closeOnLast = false;
+  if(datapart_found) {
+      select_fetch_chunk = new IFR_ALLOCATOR(allocator)
+          IFR_FetchChunk(IFR_FetchChunk::IFR_TYPE_FIRST,
+                         1,
+                         replyPacket,
+                         -1,
+                         m_MaxRows,
+                         -1, //unknown rows in result
+                         0, // no current chunk
+                         memory_ok);
+      closeOnLast = true;
       if(!memory_ok) {
           error().setMemoryAllocationFailed();
           DBUG_RETURN(IFR_NOT_OK);
       }
-      DBUG_PRINT(this->m_TableName);
-      //>>> SQL TRACE
-      IFR_SQL_TRACE << "TABLE NAME: " << m_TableName << endl;
-      //<<< SQL TRACE
-      break;
-    }
-    case (IFRPacket_PartKind::ParsidOfSelect_C):{
-      // @todo; Handling for ParsidOfSelect_C
-      error().setRuntimeError(IFR_ERR_NOT_IMPLEMENTED("ParsidOfSelect"));
-      DBUG_RETURN(IFR_NOT_OK);
-      break;
-    }
-    default: {
-      break;
-    }
-    }
-    partEnum.nextElement ();
   }
+      
+  if(m_Connection->getSelectFetchStrategy() == IFR_Connection::SelectFetchFetchImmediate_C) {
+      PIn_ReplySegment second_segment = replyPacket.GetNextSegment(segm);
+      if(second_segment.IsValid() && second_segment.FindPart(sp1pk_data).IsValid()) {
+          select_fetch_chunk = new IFR_ALLOCATOR(allocator)
+              IFR_FetchChunk(second_segment,
+                             replyPacket,
+                             m_MaxRows,
+                             memory_ok);
+          if(!memory_ok) {
+              error().setMemoryAllocationFailed();
+              DBUG_RETURN(IFR_NOT_OK);
+          }
+      }
+  }
+     
+
   // we KNOW that this is the last time we need the reply packet.
   DBUG_PRINT(rowNotFound);
+
   replyPacket.releaseLock();
-  IFR_Bool cursorwasused = m_cursorreallyused; // was the cursor used before this statement
+  IFR_Bool cursorwasused = (m_cursorstate == CursorStateUsed_C); // was the cursor used before this statement
   if (((rc == IFR_OK) || (rc == IFR_NO_DATA_FOUND)) && isQuery) {
-      createResultSet (tableName, this->m_CursorName, rowNotFound, infos, &columNames);
+      createResultSet (tableName, this->m_CursorName, rowNotFound, infos, &columNames, 
+                       closeOnLast,
+                       select_fetch_chunk);
+      // on row not found and select-fetch optimize, forget the cursor
+      if(rowNotFound && m_ResultSetType == ::SQLDBC::SQLDBC_Statement::FORWARD_ONLY
+         && m_Connection->getSelectFetchStrategy() == IFR_Connection::SelectFetchScrollable_C) {
+          m_cursorstate = CursorStateUsedClosed_C;
+      }
   }
-  
   // now drop the old cursor, if
-  if((!isQuery && m_cursorreallyused)) { // this is not a query and the cursor was used      
+  if((!isQuery && m_cursorstate == CursorStateUsed_C)) { // this is not a query and the cursor was used      
       getConnection()->dropCursor(oldCursorName, memory_ok);
       if(!memory_ok) {
           error().setMemoryAllocationFailed();
           DBUG_RETURN(IFR_NOT_OK);
       }
-      this->m_cursorreallyused=false;
+      this->m_cursorstate = CursorStateNotUsed_C;
   } else if(isQuery && cursorwasused && IFR_String::compare(this->m_CursorName, oldCursorName, memory_ok)) {
      // or is a query, cursor was used before and name has changed.
       getConnection()->dropCursor(oldCursorName, memory_ok);
@@ -471,8 +545,10 @@ IFR_Statement::parseResult (IFRPacket_ReplyPacket &replyPacket,
 IFR_Retcode IFR_Statement::createResultSet (IFR_String& tableName,
                                             IFR_String& cursorName,
                                             IFR_Bool rowNotFound,
-                                            IFRConversion_Converter **info,
-                                            IFRUtil_Vector<IFR_String>* colName)
+                                            IFRConversion_ConverterList& info,
+                                            IFRUtil_Vector<IFR_String>* colName,
+                                            IFR_Bool closeOnLast,
+                                            IFR_FetchChunk *firstChunk)
 {
   DBUG_METHOD_ENTER(IFR_Statement, createResultSet);
   DBUG_PRINT(tableName);
@@ -481,8 +557,7 @@ IFR_Retcode IFR_Statement::createResultSet (IFR_String& tableName,
   
   IFR_Retcode rc = IFR_OK;
   IFR_Bool    memory_ok = true;
-  IFR_FetchInfo *fetchInfo = new (allocator) IFR_FetchInfo (*this, info, colName, memory_ok);
-  
+  IFR_FetchInfo *fetchInfo = new IFR_ALLOCATOR(allocator) IFR_FetchInfo (*this, info, colName, memory_ok);
   if(fetchInfo == 0) {
       error().setMemoryAllocationFailed();
       this->m_resultset = 0;
@@ -493,8 +568,7 @@ IFR_Retcode IFR_Statement::createResultSet (IFR_String& tableName,
       this->m_resultset = 0;
       DBUG_RETURN(IFR_NOT_OK);
   }
-      
-  if (info == 0 || colName == 0) {
+  if (info.size() == 0 || colName == 0) {
       rc = fetchInfo->describe();
       if(rc != IFR_OK) {
           error() = fetchInfo->error();
@@ -503,10 +577,18 @@ IFR_Retcode IFR_Statement::createResultSet (IFR_String& tableName,
           DBUG_RETURN(IFR_NOT_OK);
       }
   }
-  
-  this->m_cursorreallyused=true;
-  this->m_resultset = new (allocator) IFR_ResultSet (*this->m_Connection, *fetchInfo, *this, this->m_FetchSize, 
-                                                     this->m_MaxRows, m_ResultSetConcurrency, rowNotFound, memory_ok);
+  this->m_cursorstate = CursorStateUsed_C;
+  this->m_resultset = new IFR_ALLOCATOR(allocator) IFR_ResultSet (*this->m_Connection, 
+                                                                  *fetchInfo, 
+                                                                  *this, 
+                                                                  this->m_FetchSize, 
+                                                                  this->m_MaxRows, 
+                                                                  m_ResultSetConcurrency, 
+                                                                  m_ResultSetType, 
+                                                                  rowNotFound, 
+                                                                  firstChunk,
+                                                                  closeOnLast,
+                                                                  memory_ok);
   if(this->m_resultset==0) {
       error().setMemoryAllocationFailed();
       DBUG_RETURN(IFR_NOT_OK);
@@ -576,7 +658,8 @@ IFR_Retcode IFR_Statement::execute(const IFR_String &sql, IFR_Bool recycleObject
   this->resetResults();
   IFR_Retcode rc = IFR_OK;
   IFRPacket_ReplyPacket replyPacket;
-  rc = this->sendSQL(sql, replyPacket, false, false, false, recycleObjects);
+  rc = this->sendSQL(sql, replyPacket, false, false, false, 
+                     recycleObjects?IFR_Connection::AppendAllowed_C:IFR_Connection::AppendNotAllowed_C);
   if (rc == IFR_OK) {
       rc = this->parseResult (replyPacket, false);
   }
@@ -643,7 +726,7 @@ IFR_Statement::setCursorName(const char *_name,
 
     DBUG_PRINT(name);
 
-    if(m_cursorreallyused) {
+    if(m_cursorstate == CursorStateUsed_C) {
         getConnection()->dropCursor(m_CursorName, memory_ok);
         if(!memory_ok) {
             error().setMemoryAllocationFailed();
@@ -656,11 +739,42 @@ IFR_Statement::setCursorName(const char *_name,
         error().setMemoryAllocationFailed();
         DBUG_RETURN(IFR_NOT_OK);
     }
-    m_cursorreallyused = false;
+    m_cursorstate = CursorStateUsed_C;
     clearError();
     DBUG_RETURN(IFR_OK);
 }
 
+//----------------------------------------------------------------------
+#define COMMANDINFO_MAXLEN 40
+IFR_Retcode IFR_Statement::setCommandInfo(const char *_commandInfo, 
+                               SQLDBC_Length bufferLength, 
+                               SQLDBC_Int4 lineNumber)   {
+    DBUG_METHOD_ENTER(IFR_Statement, setCommandInfo);
+    DBUG_PRINT(lineNumber);
+
+    IFR_Bool memory_ok = true;
+    if (bufferLength == SQLDBC_NULL_DATA) {
+      m_commandInfo.clear();
+      m_commandLineNumber = 0;
+      DBUG_RETURN(IFR_OK);
+    }
+    
+    if (bufferLength > COMMANDINFO_MAXLEN){
+        error().setRuntimeError(IFR_ERR_COMMAND_INFO_TOOLONG, bufferLength, COMMANDINFO_MAXLEN);
+        DBUG_RETURN(IFR_DATA_TRUNC);
+    }
+
+    m_commandInfo.setBuffer(_commandInfo, bufferLength, IFR_StringEncodingAscii, memory_ok);
+    if(!memory_ok) {
+        error().setMemoryAllocationFailed();
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+
+    DBUG_PRINT(m_commandInfo);
+    m_commandLineNumber=lineNumber;
+    clearError();
+    DBUG_RETURN(IFR_OK);
+}               
 //----------------------------------------------------------------------
 IFR_Bool
 IFR_Statement::isQuery() const
@@ -778,6 +892,7 @@ IFR_Statement::addApplicationInfo(IFRPacket_RequestSegment& requestsegment)
     DBUG_RETURN(IFR_OK);
 }
 
+//----------------------------------------------------------------------
 IFR_Bool 
 IFR_Statement::setCommandEncoding(IFR_StringEncoding encoding, IFR_Bool swapped)
 {
@@ -824,13 +939,13 @@ IFR_Statement::setRowArraySize(IFR_Length rowarraysize)
         if(usefornew < rowarraysize) {
             usefornew = rowarraysize;
         }
-        IFR_Int4 *tmp = new (allocator) IFR_Int4[usefornew];
+        IFR_Int4 *tmp = new IFR_ALLOCATOR(allocator) IFR_Int4[usefornew];
         // copy only the information
         memcpy(tmp, m_rowstatusarray, sizeof(IFR_Int4) * m_rowarraysize); 
         for(IFR_Length j = m_rowarraysize; j<rowarraysize; ++j) {
             tmp[j]=IFR_EXECUTE_FAILED;
         }
-        IFRUtil_DeleteArray(m_rowstatusarray, m_rowarraysize, allocator);
+        IFRUtil_DeleteArray(m_rowstatusarray, m_rowarraysize, IFR_ALLOCATOR(allocator));
         m_rowstatusarray = tmp;
         m_rowstatussize = usefornew;
         m_rowarraysize = rowarraysize;
@@ -880,10 +995,10 @@ IFR_Statement::addBatch(const char *sql,
     
     if(m_batchelements == 0) {
         IFR_String tmp(allocator);
-        m_batchelements = new (allocator) IFRUtil_Vector<IFR_String>(allocator, 
-                                                                     0,
-                                                                     tmp,
-                                                                     memory_ok);
+        m_batchelements = new IFR_ALLOCATOR(allocator) IFRUtil_Vector<IFR_String>(allocator, 
+                                                                                  0,
+                                                                                  tmp,
+                                                                                  memory_ok);
         if(m_batchelements == 0) {
             error().setMemoryAllocationFailed();
             DBUG_RETURN(IFR_NOT_OK);
@@ -920,6 +1035,97 @@ IFR_Statement::clearBatch()
     DBUG_RETURN(IFR_OK);
 }
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFR_Statement::getLastInsertedKey(IFR_Int4 tag,
+                                  IFR_HostType hosttype,
+                                  void *paramAddr,
+                                  IFR_Length *lengthIndicator,
+                                  IFR_Length size,
+                                  IFR_Bool terminate)
+{
+    DBUG_METHOD_ENTER(IFR_Statement, getLastInsertedKey);
+    //>>> SQL TRACE ENTRY
+    IFR_SQL_TRACE << endl << "::GET LAST SERIAL " << m_CursorName << endl;
+    //<<< SQL TRACE ENTRY
+    
+    IFR_Retcode rc        = IFR_OK;
+    IFR_Bool    memory_ok = true;
+    error().clear();
+
+    if (!m_Connection->isConnected()) {
+        error().setRuntimeError(IFR_ERR_SESSION_NOT_CONNECTED);
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+
+    if (size < 0){
+        error().setRuntimeError(IFR_ERR_NEGATIVE_BUFFERLEN_I, size);
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    
+    if(tag != SQLDBC_FIRST_INSERTED_SERIAL && 
+       tag != SQLDBC_LAST_INSERTED_SERIAL) {
+        error().setRuntimeError(IFR_ERR_INVALID_SERIAL_VALUE_TAG);
+        DBUG_RETURN(IFR_NOT_OK);
+    }
+    
+    if(m_lastserial == 0) {
+        IFR_SQL_TRACE << "*** NOT FOUND ***" << endl;
+        DBUG_RETURN(IFR_NO_DATA_FOUND);
+    }
+
+    IFR_Parameter parameter(hosttype,
+                            terminate,
+                            paramAddr,
+                            lengthIndicator,
+                            size);
+    
+    const char *tag_str;
+    switch(tag) {
+    case SQLDBC_FIRST_INSERTED_SERIAL:
+        tag_str = "FST";
+        break;
+    case SQLDBC_LAST_INSERTED_SERIAL:
+        tag_str = "LST";
+        break;
+    default:
+        tag_str = "???";
+    }
+    
+    //>>> SQL TRACE
+    if(IFR_SQL_TRACE_ENABLED) {
+        IFR_TRACE_STREAM << "PARAMETER" << endl
+                         << IFR_SQLTRACE_PARAMETER_HEADER << endl;
+        parameter.sqlTraceParameter(IFR_TRACE_STREAM, tag_str);
+        IFR_TRACE_STREAM << endl;
+    }
+    //<<< SQL TRACE
+    
+    IFRConversion_Converter * info = getConnection()->findSpecialParameter(tag);
+    if(info == 0) {
+        IFR_SQL_TRACE << "*** NOT FOUND ***" << endl;
+        DBUG_RETURN(IFR_NO_DATA_FOUND);
+    }
+    IFR_Length  dataoffset = 0;
+    IFR_Length  offset     = 0;
+    rc = info->translateOutput(*m_lastserial,
+                               parameter,
+                               *this,
+                               0,
+                               0);
+    //<<< SQL TRACE
+    if(IFR_SQL_TRACE_ENABLED) {
+        IFR_TRACE_STREAM << "DATA" << endl
+                         << IFR_SQLTRACE_DATA_HEADER << endl;
+        parameter.sqlTraceParameterData(IFR_TRACE_STREAM, 1, rc, 0, 0, 0, 0, true);
+        IFR_SQL_TRACE << endl;
+    }
+    //>>> SQL TRACE
+    
+    DBUG_RETURN(rc);
+}
+                                  
+
 //----------------------------------------------------------------------
 // Gives a crude, but fast enough upper bound on the string size
 // of a string in a specific encoding
@@ -1069,7 +1275,7 @@ IFR_Statement::executeBatch()
                     ++current;
                 }
             } while(!full && current<num_batchelements);
-            rc = m_Connection->sqlaexecute(requestpacket, replypacket, false, error());
+            rc = m_Connection->sqlaexecute(requestpacket, replypacket, IFR_Connection::AppendNotAllowed_C, error());
             if(rc != IFR_OK) {
                 DBUG_RETURN(rc);
             } else if(error()) {
@@ -1114,11 +1320,57 @@ void
 IFR_Statement::resetResults()
 {
     DBUG_METHOD_ENTER(IFR_Statement, resetResults);
-    IFRUtil_Delete(this->m_resultset, allocator);
-    this->m_resultset = 0;
-    this->m_RowsAffected = 0;
+    IFRUtil_Delete(m_resultset, allocator);
+    IFRUtil_Delete(m_lastserial, allocator);
+    m_lastserial = 0;
+    m_resultset = 0;
+    m_RowsAffected = 0;
 }
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFR_Statement::setSerialPart(IFRPacket_DataPart& serialData)
+{
+    DBUG_METHOD_ENTER(IFR_Statement, setSerialPart);
+    if(serialData.isValid ()) {
+        IFR_size_t copysize=sizeof(tsp1_part_header) + serialData.Length();
+        IFR_Byte *bsource=(IFR_Byte*) serialData.GetRawPart();
+        IFR_Byte *bdest = (IFR_Byte *) IFR_ALLOCATOR(allocator).Allocate(copysize);
+        if(bdest == 0) {
+            error().setMemoryAllocationFailed();
+            DBUG_RETURN(IFR_NOT_OK);
+        }
+        memcpy(bdest, bsource, copysize);
+        if(m_lastserial) {
+            IFRUtil_Delete(m_lastserial, allocator);
+        }
+        m_lastserial=new IFR_ALLOCATOR(allocator) CopiedDataPart(bdest,
+                                                                 serialData.getEncoding(),
+                                                                 allocator);
+        if(m_lastserial == 0) {
+            allocator.Deallocate(bdest);
+            error().setMemoryAllocationFailed();
+            DBUG_RETURN(IFR_NOT_OK);
+        }
+    }
+    DBUG_RETURN(IFR_OK);
+}
+
+//----------------------------------------------------------------------
+IFR_Retcode
+IFR_Statement::updateSerialPart(IFRPacket_DataPart& serialData)
+{
+    DBUG_METHOD_ENTER(IFR_Statement, setSerialPart);
+    if(serialData.isValid () && m_lastserial) {
+        // Copy the 2nd number on the 1st for batch
+        char *src = serialData.getOutputData(21);
+        char *dest = m_lastserial->getOutputData(21);
+        memcpy(dest, src, 21);
+    }
+    DBUG_RETURN(IFR_OK);
+}
+
+
 //----------------------------------------------------------------------
 IFR_TraceStream&
 operator << (IFR_TraceStream& s, const IFR_Statement& statement)
