@@ -220,6 +220,16 @@ public:
      */
     virtual IFR_Retcode clearBatch();
 
+    /**
+     * Gets the last inserted key.
+     */
+    IFR_Retcode getLastInsertedKey(IFR_Int4     tag,
+                                   IFR_HostType type,
+                                   void            *paramAddr,
+                                   IFR_Length   *lengthIndicator,
+                                   IFR_Length    size,
+                                   IFR_Bool      terminate);
+    
     /**
      * Executes the batch commands. 
      * @return #IFR_OK on success, #IFR_NOT_OK on error, in which case the row status array 
@@ -305,6 +315,23 @@ public:
     IFR_Retcode setCursorName(const char *name, 
                        IFR_Length length, 
                        IFR_StringEncoding encoding);
+    
+    /**
+     * Sets the command info (for example name of programm) and a line number. 
+     * This will be send to the database kernel as part of a parse order. 
+     * If the kernel runs in a special diagnose mode these information will be 
+     * stored in a system table and can be used to profile your application.
+     *
+     * @param commandInfo the command info.
+     * @param length The length of commandInfo, or #IFR_NTS to signal a null-terminated string,
+     * or SQLDBC_NULL_DATA to unset the command info.
+     * @param lineNumber line number.
+     * @return @c IFR_OK on success, @c IFR_NOT_OK on failure, in which
+     *         case the error is set.
+     */
+    IFR_Retcode setCommandInfo(const char *commandInfo, 
+                               SQLDBC_Length bufferLength, 
+                               SQLDBC_Int4 lineNumber);                   
   
     /**
      * Checks if the statement is a query. A statement is a query only
@@ -404,7 +431,12 @@ public:
      * @return A pointer to the connection object.
      */  
     virtual IFR_Connection *getConnection() const;
-      
+
+    inline void setCursorClosed()
+    {
+        m_cursorstate = CursorStateUsedClosed_C;
+    }
+    
 protected:
     /**
      * @brief Sets the size of the row array expected at a batch execute.
@@ -455,7 +487,7 @@ protected:
                        IFR_Bool masscommand,
                        IFR_Bool parseonly,
                        IFR_Bool parseAgain,
-                       IFR_Bool allowAppended);
+                       IFR_Connection::AppendMode appendFlag);
   
   /**
      @brief Sends the given SQL command to the database.
@@ -468,7 +500,7 @@ protected:
      @param parseonly [in] Whether this command should be parsed only (<code>true</code>) or executed (<code>false</code>).
      @param parseAgain [in] Whether this command should be parsed with a 
      'parse again' mark set.
-     @param allowAppended [in] Whether appended housekeeping commands are allowed.
+     @param appendFlag [in] Whether appended housekeeping commands are allowed.
      @param replyPacket [out] The reply packet, to which the reply is assigned.
      @return <code>IFR_OK</code> if the request was ok, <code>IFR_NOT_OK</code>
      *       otherwise (in which case the error in this statement is set).
@@ -478,7 +510,7 @@ protected:
                            IFR_Bool masscommand,
                            IFR_Bool parseonly,
                            IFR_Bool parseAgain,
-                           IFR_Bool allowAppended,
+                           IFR_Connection::AppendMode appendFlag,
                            IFRPacket_ReplyPacket& replyPacket);
     
   /**
@@ -496,10 +528,12 @@ protected:
      @todo Implement UPDATE tableName WHERE CURRENT OF cursorName
      */
   virtual IFR_Retcode createResultSet (IFR_String& tableName,
-			       IFR_String& cursorName,
-			       IFR_Bool rowNotFound,
-                   IFRConversion_Converter **info=0,
-			       IFRUtil_Vector<IFR_String>* colName=0);
+                                       IFR_String& cursorName,
+                                       IFR_Bool rowNotFound,
+                                       IFRConversion_ConverterList& info,
+                                       IFRUtil_Vector<IFR_String>* colName,
+                                       IFR_Bool closeOnLast,
+                                       IFR_FetchChunk *firstchunk);
   
     
   /*!
@@ -519,6 +553,14 @@ protected:
     */  
     IFR_Bool setCommandEncoding(IFR_StringEncoding encodingOfCmd, IFR_Bool swapped);
     
+    /**
+     * Forcibly sets the command encoding. No checks are done. This is used in
+     * case the encoding is stored e.g. in parseinfo cache.
+     */
+    inline void unsafeSetCommandEncoding(IFR_StringEncoding encoding)
+    {
+        m_CommandEncoding = encoding;
+    }
     
     /**
      * @brief Checks whether the statement's first command is a SELECT.
@@ -531,7 +573,8 @@ protected:
      *       <code>false</code> otherwise.
      */
     // static IFR_Bool isQuery(const char *sql, IFR_StringEncoding encoding);
-  
+    
+
 protected:
     /**
      * Closes the current result set and resets the row count.
@@ -576,10 +619,48 @@ protected:
      */
     static IFR_Bool isQuery(const IFR_String& sql);
     
+    /**
+     * Sets a new serial part.
+     * @param part The serial part.
+     * @return @c IFR_OK on success, @c IFR_NOT_OK if a 
+     * memory allocation error occurs.
+     */
+    IFR_Retcode setSerialPart(IFRPacket_DataPart& part);
+
+    IFR_Retcode updateSerialPart(IFRPacket_DataPart& part);
 
     friend class IFR_FetchInfo; //!< The fetch info needs access to cursor name etc. directly.
     friend class IFR_Connection; //!< For calling the private constructor.    
 
+    class CopiedDataPart
+        : public IFRPacket_DataPart
+    {
+    public:
+        inline CopiedDataPart(IFR_Byte *copiedbuffer,
+                              IFR_StringEncoding encoding,
+                              SAPDBMem_IRawAllocator& _allocator)
+        :IFRPacket_DataPart((tsp1_part*)copiedbuffer, encoding),
+         allocator(_allocator)
+        {
+        }
+        
+        inline ~CopiedDataPart()
+        {
+            IFR_Byte *to_del=(IFR_Byte*)GetRawPart();
+            IFR_ALLOCATOR(allocator).Deallocate(to_del);
+        }
+        
+    private:
+        SAPDBMem_IRawAllocator& allocator;
+    };
+    
+    enum CursorState {
+        CursorStateNotUsed_C     = 0, //!< 
+        CursorStateUsed_C        = 1, //!< Used, must be closed.
+        CursorStateUsedClosed_C  = 2  //!< Used and implicitely closed (Oracle, SELECT FETCH Optimization)
+    };
+
+
     IFR_Int4 m_RowsAffected;                  //!< Number of rows affected by last command (-1 for unknown).
     ResultSetType m_ResultSetType;        //!< Sensitivity of the result set (changes are visible or not, use 
                                               //!< of <code>FOR REUSE</code>)
@@ -589,17 +670,21 @@ protected:
     IFR_Int4 m_FetchSize;                     //!< Number of rows to fetch at once (performance hint).
     IFR_Int4 m_MaxRows;                       //!< Maximum number of rows in the result set (fetch will be limited). 
     IFR_Int4 m_QueryTimeout;   //!< Execution time out for sql command    
+    IFR_Int4         m_commandLineNumber; //!< line number used in command info.    
+    IFR_String       m_commandInfo;//!< command info of the statement.
     IFR_String       m_CursorName; //!< Cursor name of the statement.
     IFR_String       m_TableName;  //!< Table name of the statement.
     IFR_ResultSet   *m_resultset;  //!< Result set of this statement.
     IFR_Connection  *m_Connection; //!< Connection of this statement.    
     IFR_StringEncoding m_CommandEncoding; //!< Encoding of the executed command.
-    IFR_Bool         m_cursorreallyused; //!< Marks if a result set was created and used by a cursor.
+    IFR_Int4           m_cursorstate;     //!< Status of cursor.
 
     IFR_Length                    m_rowarraysize;   //!< number of rows that are subject to the statement
     IFR_Int4                     *m_rowstatusarray; //!< status array for batch execute
     IFR_Length                    m_rowstatussize;  //!< physical size of row status array.
     IFRUtil_Vector<IFR_String>   *m_batchelements;  //!< Elements of batch execution, if any.
+    
+    CopiedDataPart               *m_lastserial;     //!< Last inserted serials, if any.
 };
 
 /**
