@@ -61,7 +61,7 @@ IFR_String::IFR_String(IFR_StringEncoding encoding, SAPDBMem_IRawAllocator& allo
 IFR_String::~IFR_String()
 {
     if(m_buffer) {
-        m_allocator.Deallocate(m_buffer);
+        IFR_ALLOCATOR(m_allocator).Deallocate(m_buffer);
     }
 }
 
@@ -83,7 +83,7 @@ IFR_String::IFR_String(const IFR_String& src, IFR_Bool& memory_ok)
     if(m_bufsize==0) {
         m_buffer=0;
     } else {
-        m_buffer=(char*) m_allocator.Allocate(m_bufsize);
+        m_buffer=(char*) IFR_ALLOCATOR(m_allocator).Allocate(m_bufsize);
         //<<< MEMCHECK
         if(m_buffer == 0) {
             memory_ok = false;
@@ -123,7 +123,7 @@ IFR_String::IFR_String(SAPDBMem_IRawAllocator& allocator,
     if(m_bufsize==0) {
         m_buffer=0;
     } else {
-        m_buffer=(char*) m_allocator.Allocate(m_bufsize);
+        m_buffer=(char*) IFR_ALLOCATOR(m_allocator).Allocate(m_bufsize);
         //<<< MEMCHECK
         if(m_buffer == 0) {
             memory_ok = false;
@@ -204,7 +204,7 @@ IFR_String::IFR_String(const char *buf,
         m_strlen = tmp_strlen;
         m_bufsize=m_length_in_bytes=bytelength;
         m_bufsize+=IFR_ENCODING(m_encoding_type)->terminatorSize;
-        m_buffer=(char*)m_allocator.Allocate(m_bufsize);
+        m_buffer=(char*)IFR_ALLOCATOR(m_allocator).Allocate(m_bufsize);
         //<<< MEMCHECK
         if(m_buffer == 0) {
             memory_ok = false;
@@ -259,7 +259,7 @@ IFR_String::IFR_String(const char *buf,
         m_strlen = tmp_strlen;
         m_bufsize=m_length_in_bytes=bytelength;
         m_bufsize+=IFR_ENCODING(m_encoding_type)->terminatorSize;
-        m_buffer=(char*)m_allocator.Allocate(m_bufsize);
+        m_buffer=(char*)IFR_ALLOCATOR(m_allocator).Allocate(m_bufsize);
         //<<< MEMCHECK
         if(m_buffer == 0) {
             memory_ok = false;
@@ -314,9 +314,9 @@ IFR_String::setBuffer(const char *buf,
 
     if(bufferlength > m_bufsize) {
         if(m_buffer) {
-            m_allocator.Deallocate(m_buffer);
+            IFR_ALLOCATOR(m_allocator).Deallocate(m_buffer);
         }
-        m_buffer = (char*) m_allocator.Allocate(bufferlength);
+        m_buffer = (char*) IFR_ALLOCATOR(m_allocator).Allocate(bufferlength);
         //<<< MEMCHECK
         if(!m_buffer) {
             memory_ok         = false;
@@ -373,7 +373,7 @@ IFR_String::assign(const IFR_String& src, IFR_Bool& memory_ok)
     //<<< MEMCHECK
     if(!memory_ok) {
         if(m_buffer) {
-            m_allocator.Deallocate(m_buffer);
+            IFR_ALLOCATOR(m_allocator).Deallocate(m_buffer);
             m_buffer = 0;
         }
         m_bufsize         = 0;
@@ -383,8 +383,8 @@ IFR_String::assign(const IFR_String& src, IFR_Bool& memory_ok)
     //>>> MEMCHECK
     
     if(src.m_bufsize > m_bufsize) {
-        if(m_buffer) m_allocator.Deallocate(m_buffer);
-        m_buffer = (char *) m_allocator.Allocate(src.m_bufsize);
+        if(m_buffer) IFR_ALLOCATOR(m_allocator).Deallocate(m_buffer);
+        m_buffer = (char *) IFR_ALLOCATOR(m_allocator).Allocate(src.m_bufsize);
         //<<< MEMCHECK
         if(!m_buffer) {
             memory_ok         = false;
@@ -530,15 +530,15 @@ IFR_String::expand(IFR_Length newbufsize, IFR_Bool& memory_ok)
     if(usefornewbuf < newbufsize) {
         usefornewbuf=newbufsize;
     }
-    char *tmp_buffer=(char*)m_allocator.Allocate(usefornewbuf);
+    char *tmp_buffer=(char*)IFR_ALLOCATOR(m_allocator).Allocate(usefornewbuf);
     // give it another try 
     if(tmp_buffer == 0) {
         usefornewbuf = newbufsize;
-        tmp_buffer=(char*)m_allocator.Allocate(usefornewbuf);
+        tmp_buffer=(char*)IFR_ALLOCATOR(m_allocator).Allocate(usefornewbuf);
         //<<< MEMCHECK
         if(!tmp_buffer) {
             if(m_buffer) {
-                m_allocator.Deallocate(m_buffer);
+                IFR_ALLOCATOR(m_allocator).Deallocate(m_buffer);
                 m_buffer = 0;
             }
             m_bufsize         = 0;
@@ -550,7 +550,7 @@ IFR_String::expand(IFR_Length newbufsize, IFR_Bool& memory_ok)
     }
     if(m_buffer) {
         memcpy(tmp_buffer, m_buffer, m_bufsize);
-        m_allocator.Deallocate(m_buffer);
+        IFR_ALLOCATOR(m_allocator).Deallocate(m_buffer);
     }
     m_buffer=tmp_buffer;
     m_bufsize=usefornewbuf;
@@ -568,8 +568,14 @@ IFR_String::compare(const IFR_String& a, const IFR_String& b, IFR_Bool& memory_o
     //>>> MEMCHECK
     
     if(a.getEncoding() == b.getEncoding()) {
-        // Due to 0-termination, this will stop at the zero byte.
-        return memcmp(a.getBuffer(), b.getBuffer(), b.getLength());
+        IFR_size_t comparelength = a.getLength();
+        if(b.getLength() < comparelength) {
+            comparelength = b.getLength();
+        }
+        // Due to 0-termination, this will stop at the zero byte. 
+        // However, some memory checks complain about this if the 
+        // length is larger than one buffer.
+        return memcmp(a.getBuffer(), b.getBuffer(), comparelength);
     } else {
         unsigned int a_charlength;
         unsigned int a_bytelength;
@@ -788,22 +794,4 @@ IFR_String::convertInto(char               *buffer,
     }
 }
 
-#ifndef SVR4
-//----------------------------------------------------------------------
-IFR_Bool
-operator ==(IFR_StringEncodingType::Encoding encoding1,
-            IFR_StringEncodingType::Encoding encoding2)
-{
-    return IFR_ENCODING(encoding1) == IFR_ENCODING(encoding2);
-}
-
-//----------------------------------------------------------------------
-IFR_Bool
-operator !=(IFR_StringEncodingType::Encoding encoding1,
-            IFR_StringEncodingType::Encoding encoding2)
-{
-    return IFR_ENCODING(encoding1) != IFR_ENCODING(encoding2);
-}
-
-#endif
 
