@@ -65,13 +65,6 @@ public:
 SAPDBMEM_STATIC_RAW_ALLOCATE(ifrtrace_dummystream_memory, 
                              sizeof(IFR_Trace_DummyOutputStream));
 
-//----------------------------------------------------------------------
-static IFR_Trace_DummyOutputStream *
-IFR_Trace_GetDummyStream(void)
-{
-    void *p = (void *) & ifrtrace_dummystream_memory;
-    return new (p) IFR_Trace_DummyOutputStream();
-}
 
 //----------------------------------------------------------------------
 static const char hexchar[] ={ '0', '1', '2', '3', '4', '5', '6', '7', 
@@ -97,7 +90,7 @@ static inline void hexifystr(const char *dest,
 
 //----------------------------------------------------------------------
 IFR_TraceStream::IFR_TraceStream(IFR_ITraceController* controller)
-:PIn_TraceStream(IFR_Trace_GetDummyStream()),
+:PIn_TraceStream(0),
  m_controller(controller)
 {}
 
@@ -132,7 +125,14 @@ IFR_TraceStream::operator << (const char *sb)
         inputlengthc = controller_traceflags->inputlength;
         hexc         = controller_traceflags->hex; 
     }
-
+    
+    if(inputlengthc == 0) {
+        if(m_controller) {
+            m_controller->clearFlags();
+        }
+        return *this;
+    }
+    
     // sanity check: If we have an invalid encoding, we fall back to ASCII.
     if( ((int)inputenc) < 1 ||
         ((int)inputenc) > 4) {
