@@ -46,12 +46,26 @@ struct lpad;
 struct inputencoding;
 extern SAPDB_Bool ifr_dbug_trace;
 
+#if defined(BIT64)
+#  define IFR_SQLTRACE_PARAMETER_HEADER "I   T          AT L          I                  D                  P"
+#  define IFR_SQLTRACE_DATA_HEADER      "I   T          AT L          I           DATA"
+#else
+#  define IFR_SQLTRACE_PARAMETER_HEADER "I   T          AT L          I          D          P"
+                                                                              
+#  define IFR_SQLTRACE_DATA_HEADER      "I   T          AT L          I           DATA"
+#endif
+
 #if defined(IFR_NO_TRACE)
 #  define IFR_DBUG_TRACE (false)
 #else
 #  define IFR_DBUG_TRACE (ifr_dbug_trace)
 #endif
 
+/**
+ * Number of bytes to be traced of data buffers.
+ */
+#define IFR_MAX_TRACELENGTH 100
+
 /**
  * @ingroup IFR_Trace
  * Structure to combine some constants.
@@ -108,6 +122,12 @@ public:
      */
     virtual SAPDBMem_IRawAllocator& getAllocator() = 0;
     
+
+    /**
+     * Called when an error had happened.
+     * This is used for the "STOP ON ERROR" feature.
+     */
+    virtual void traceError(IFR_Int4 errorcode) = 0;
     
     /**
      * Set the output format to decimal.
@@ -498,6 +518,13 @@ public:
      */
     void println(const char *s, IFR_Int4 n);   
 
+    /**
+     * Called when an error is traced.
+     */
+    inline void traceError(IFR_Int4 errorcode)
+    {
+        if(m_controller) m_controller->traceError(errorcode);
+    }
 private:
     IFR_ITraceController *m_controller;  //!< The trace controller used.
 
