@@ -3,7 +3,7 @@
   @author         DL SAP DB INTERFACES
   @ingroup        IFR_Common
   @brief          Basic types and macros
-  @see            
+  @see
 
 \if EMIT_LICENCE
 
@@ -35,6 +35,7 @@
 #include <math.h>
 #include "SAPDB/Interfaces/SQLDBC/SQLDBC_Types.h"
 
+
 /** @ingroup IFR_Common
  * @def IFR_NEED_VIRTUAL_DELEGATION
  * This define is set if the the compiler requires that if more than
@@ -42,7 +43,7 @@
  * a different signature, either none or all functions need to be
  * redefined in a subclass. This is true for Sun and HP Unix.
  */
-#if defined(SUN) || defined(HPUX) 
+#if defined(SUN) || defined(HPUX)
 #  define IFR_NEED_VIRTUAL_DELEGATION  1
 #else
 #  define  IFR_NEED_VIRTUAL_DELEGATION 0
@@ -54,10 +55,10 @@
  * depending whether the compiler requires and understands the
  * <code>typename</code> keyword, or forbids it (IA 64).
  */
-#if defined(_WIN64) 
-/* IA64 does not like 'typename'. */
-#  define IFR_TYPENAME 
+#if defined(_M_IA64)
+#  define IFR_TYPENAME
 #else
+/* IA64 does not like 'typename'. */
 #  define IFR_TYPENAME typename
 #endif
 
@@ -66,12 +67,12 @@
 #ifndef HPUX
 #ifndef MAX
 /** @ingroup IFR_Common
- * Definition of maximum computation. 
+ * Definition of maximum computation.
  */
 #define MAX(a, b) (((a) > (b)) ? (a) : (b))
 #endif
 #ifndef MIN
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Definition of minimum computation. */
 #define MIN(a, b) (((a) < (b)) ? (a) : (b))
 #endif
@@ -87,7 +88,7 @@ typedef unsigned char       IFR_Bool1;        /* byte (1 byte, unsigned) */
 /** @ingroup IFR_Common
  * The <code>true</code> value for the <code>IFR_Bool1</code> type. */
 #define IFR_TRUE          SQLDBC_TRUE
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  *The <code>false</code> value for the <code>IFR_Bool1</code> type. */
 #define IFR_FALSE         SQLDBC_FALSE
 
@@ -107,12 +108,12 @@ typedef unsigned char       IFR_Bool1;        /* byte (1 byte, unsigned) */
  * Definition of the boolean type, this is either the C++ <code>bool</code>
  * type, or if missing a <tt>typedef</tt> to an <tt>unsigned char</tt>.
  */
-typedef SQLDBC_Bool IFR_Bool; 
+typedef SQLDBC_Bool IFR_Bool;
 /** @ingroup IFR_Common
  * @deprecated
  * Type definition for the 'bool' type as a helper for old sources
  */
-typedef IFR_Bool  bool; 
+typedef IFR_Bool  bool;
 #endif
 #else
 typedef bool IFR_Bool;
@@ -131,12 +132,12 @@ typedef bool IFR_Bool;
 typedef SQLDBC_ULong    IFR_ULong;      /* long (fast, unsigned) */
 typedef SQLDBC_Long     IFR_Long;       /* long (fast, signed)   */
 #else
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * An unsigned long type, which has 64 bit length on 64 bit
  * platforms, and 32 bit length on 32 bit platforms.
  */
 typedef SQLDBC_ULong     IFR_ULong;      /* long (fast, unsigned) */
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * A signed long type, which has 64 bit length on 64 bit
  * platforms, and 32 bit length on 32 bit platforms.
  */
@@ -174,37 +175,37 @@ typedef SQLDBC_size_t       IFR_size_t;
 typedef IFR_Long            IFR_ptrdiff_t;
 
 
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Unsigned 1 byte integer. */
 typedef SQLDBC_UInt1       IFR_UInt1;
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Signed 1 byte integer. */
 typedef SQLDBC_Int1        IFR_Int1;
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Unsigned 2 byte integer. */
 typedef SQLDBC_UInt2       IFR_UInt2;
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Signed 2 byte integer. */
 typedef SQLDBC_Int2        IFR_Int2;
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Unsigned 4 byte integer. */
 typedef SQLDBC_UInt4       IFR_UInt4;
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Signed 4 byte integer. */
 typedef SQLDBC_Int4        IFR_Int4;
 
 #if defined(WIN32) || defined(_WIN64)
-/** @ingroup IFR_Common 
- * Unsigned 8 byte integer. */
-typedef SQLDBC_UInt8       IFR_UInt8;      
-/** @ingroup IFR_Common 
- * Signed 8 byte integer. */
-typedef SQLDBC_Int8        IFR_Int8;       
-#else
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Unsigned 8 byte integer. */
 typedef SQLDBC_UInt8       IFR_UInt8;
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
+ * Signed 8 byte integer. */
+typedef SQLDBC_Int8        IFR_Int8;
+#else
+/** @ingroup IFR_Common
+ * Unsigned 8 byte integer. */
+typedef SQLDBC_UInt8       IFR_UInt8;
+/** @ingroup IFR_Common
  * Signed 8 byte integer. */
 typedef SQLDBC_Int8        IFR_Int8;
 #endif
@@ -218,53 +219,53 @@ typedef SQLDBC_Length      IFR_Length;
 
 
 
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  *  Standard unsigned integer type of the platform. */
 typedef SQLDBC_UInt        IFR_UInt;
 
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Standard signed integer type of the platform. */
 typedef SQLDBC_Int         IFR_Int;
 
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Maximum value for signed 1 byte integer. */
 #define MAX_IFR_INT1  ((IFR_Int1)  127)
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Minimum value for signed 1 byte integer. */
 #define MIN_IFR_INT1  ((IFR_Int1) -128)
 
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Maximum value for unsigned 1 byte integer. */
 #define MAX_IFR_UINT1  ((IFR_UInt1) 255)
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Minimum value for unsigned 1 byte integer. */
-#define MIN_IFR_UINT1  ((IFR_UInt1) 0) 
+#define MIN_IFR_UINT1  ((IFR_UInt1) 0)
 
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Maximum value for signed 2 byte integer. */
 #define MAX_IFR_INT2   ((IFR_Int2)32767)
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Minimum value for signed 2 byte integer. */
 #define MIN_IFR_INT2   ((IFR_Int2)(-MAX_IFR_INT2 - 1))
 
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Maximum value for unsigned 2 byte integer. */
 #define MAX_IFR_UINT2  ((IFR_UInt2)65535)
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Minimum value for unsigned 2 byte integer. */
 #define MIN_IFR_UINT2  ((IFR_UInt2)0)
 
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Maximum value for signed 4 byte integer. */
 #define MAX_IFR_INT4   ((IFR_Int4)2147483647)
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Minimum value for signed 4 byte integer. */
-#define MIN_IFR_INT4   ((IFR_Int4)(-MAX_IFR_INT4 - 1)) 
+#define MIN_IFR_INT4   ((IFR_Int4)(-MAX_IFR_INT4 - 1))
 
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Maximum value for unsigned 4 byte integer. */
 #define MAX_IFR_UINT4   ((IFR_UInt4)4294967295U)
-/** @ingroup IFR_Common 
+/** @ingroup IFR_Common
  * Minimum value for unsigned 4 byte integer. */
 #define MIN_IFR_UINT4   ((IFR_UInt4)0)
 
@@ -278,7 +279,7 @@ typedef SQLDBC_Int         IFR_Int;
 /** @ingroup IFR_Common
  * Macro for making a signed 64-bit constant. As this is
  * marked different for different systems, there is a helper
- * macro defined. It appends 'I64', 'L', or 'LL' to the 
+ * macro defined. It appends 'I64', 'L', or 'LL' to the
  * argument, depending on the platform.
  * @param x The constant number.
  */
@@ -286,7 +287,7 @@ typedef SQLDBC_Int         IFR_Int;
 /** @ingroup IFR_Common
  * Macro for making an unsigned 64-bit constant. As this is
  * marked different for different systems, there is a helper
- * macro defined. It appends 'I64', 'L', or 'LL' to the 
+ * macro defined. It appends 'I64', 'L', or 'LL' to the
  * argument, depending on the platform.
  * @param x The constant number.
  */
@@ -298,7 +299,7 @@ typedef SQLDBC_Int         IFR_Int;
 #define MAX_IFR_INT8   ((IFR_Int8)IFR_INT64_CONSTANT(9223372036854775807))
 
 
-#if defined(HPUX) && defined(BIT64) 
+#if defined(HPUX) && defined(BIT64)
 #  define MIN_IFR_INT8  ((IFR_Int8)-9223372036854775808L)
 #else
 /** @ingroup IFR_Common
@@ -358,7 +359,7 @@ const int IFR_ParseID_Size = 12;
 /** @ingroup IFR_Packet
  * A parse id is returned by the database server after preparing
  * or parsing a command, and refers to this command.
- * 
+ *
  */
 class IFR_ParseID
 {
@@ -366,29 +367,29 @@ public:
     /**
      * Default constructor. Creates an invalid parse id.
      */
-    inline IFR_ParseID() 
-    { 
+    inline IFR_ParseID()
+    {
         memset(data, 0, IFR_ParseID_Size);
         connectCount = -1;
     }
-    
+
     /**
      * Copy constructor.
      * @param parseid The parse id to copy.
      */
-    inline IFR_ParseID(const IFR_ParseID& parseid) 
+    inline IFR_ParseID(const IFR_ParseID& parseid)
     :connectCount(parseid.connectCount)
-    { 
+    {
         memcpy(data, parseid.data, IFR_ParseID_Size);
     }
-    
+
     /**
-     * Assignment operator. Copies the data from a 
+     * Assignment operator. Copies the data from a
      * memory area.
      * @param d Pointer to the data of the parse id.
      * @return <code>*this</code>.
      */
-    inline IFR_ParseID &operator=(const unsigned char *d) 
+    inline IFR_ParseID &operator=(const unsigned char *d)
     {
         memcpy(data, d, IFR_ParseID_Size);
         return *this;
@@ -411,7 +412,7 @@ public:
     {
         return data;
     }
-    
+
     /**
      * Gets the connection id that is stored in the parse id,
      * that is, the 32 bit integer stored in the first 4 bytes.
@@ -426,7 +427,7 @@ public:
         return tmp;
 #endif
     }
-            
+
     /**
      * Assignment operator.
      * @param parseid The other parse id.
@@ -440,7 +441,7 @@ public:
         }
         return *this;
     }
-    
+
     /**
      * Returns whether this parse id is a mass command.
      * @return <code>true</code> is the next-to-last byte has
@@ -452,7 +453,17 @@ public:
     }
     
     /**
-     * Modifies the 'next-to-last' byte of the parse 
+     * Returns whether the command is already executed.
+     * @return @c true if the command was already executed
+     *   during parsing.
+     */
+    inline IFR_Bool isExecuted() const
+    {
+        return data[10] == 1;
+    }
+
+    /**
+     * Modifies the 'next-to-last' byte of the parse
      * id so that a parse id of a mass command will
      * again represent the single command.
      */
@@ -463,7 +474,7 @@ public:
 
         }
     }
-    
+
     /**
      * Invalidates the parse id by deleting the connection id.
      */
@@ -474,7 +485,7 @@ public:
 
     /**
      * Checks that the parse id is valid, i.e. the connection id is not 0.
-     * It will also check whether a connection is the same as the one that was 
+     * It will also check whether a connection is the same as the one that was
      * used when parsing the command, if a @c connectcount is provided.
      * @param connectcount The connect count of the current connection.
      * @return <code>true</code> if the parse id is valid, <code>false</code>
@@ -484,7 +495,7 @@ public:
     {
         return (getConnectionID() != 0) && (connectcount==-1 || connectcount == this->connectCount);
     }
-    
+
     /**
      * Sets the connect count.
      * @param count The connect count value.
@@ -493,16 +504,17 @@ public:
     {
         connectCount = count;
     }
-    
+
     /**
      * Retrieve the connect count.
-     * @return The connect count, or 
+     * @return The connect count, or
      *         -1 if the connect count is not set.
      */
     inline IFR_Int4 getConnectCount() const
     {
         return connectCount;
     }
+
     
 private:
     unsigned char data[IFR_ParseID_Size]; //!< Parse ID data.
@@ -567,7 +579,7 @@ IFR_TraceStream& operator << (IFR_TraceStream& s, const IFR_ParseID& p);
 /** @ingroup IFR_Common
  * Swap kind (or byte order).
  */
-enum IFR_ClientSwap 
+enum IFR_ClientSwap
 {
     SwapUnknown,   //!< Unknown byte order.
     SwapNormal,    //!< Big endian.
@@ -577,9 +589,9 @@ enum IFR_ClientSwap
 
 /**
  * @ingroup IFR_Common
- * possible SQL modes. 
+ * possible SQL modes.
  */
-enum IFR_SQLMode { 
+enum IFR_SQLMode {
      IFR_Nil            = 0,                  //!< Unknown SQL Mode.
      IFR_SessionSqlmode = 1,                  //!< SQL Mode of current session.
      IFR_INTERNAL       = SQLDBC_INTERNAL,    //!< SQL mode internal.
@@ -592,21 +604,21 @@ enum IFR_SQLMode {
 /**
  * @ingroup IFR_Common
  * Return code of functions. This is not an error code,
- * it only indicates the status of the function call. 
+ * it only indicates the status of the function call.
  */
-enum IFR_Retcode { 
+enum IFR_Retcode {
      IFR_OK                = SQLDBC_OK,     //!< Function call went ok.
-     IFR_NOT_OK            = SQLDBC_NOT_OK, //!< Function call went not ok. Further information 
-                                            //!< is in the corresponding error object. 
-     IFR_DATA_TRUNC        = SQLDBC_DATA_TRUNC, //!< Data was truncated during the call. 
-     IFR_OVERFLOW          = SQLDBC_OVERFLOW,   //!< Signal of a numeric overflow.  
-     IFR_SUCCESS_WITH_INFO = SQLDBC_SUCCESS_WITH_INFO, //!< The method succeeded, but there were warnings. 
+     IFR_NOT_OK            = SQLDBC_NOT_OK, //!< Function call went not ok. Further information
+                                            //!< is in the corresponding error object.
+     IFR_DATA_TRUNC        = SQLDBC_DATA_TRUNC, //!< Data was truncated during the call.
+     IFR_OVERFLOW          = SQLDBC_OVERFLOW,   //!< Signal of a numeric overflow.
+     IFR_SUCCESS_WITH_INFO = SQLDBC_SUCCESS_WITH_INFO, //!< The method succeeded, but there were warnings.
      IFR_NO_DATA_FOUND     = SQLDBC_NO_DATA_FOUND, //!< Expected data was not found.
      IFR_NEED_DATA         = SQLDBC_NEED_DATA      //!< Late binding, data is needed for execution.
 };
 
 /**
- * @ingroup IFR_Common 
+ * @ingroup IFR_Common
  * Row status for mass operations. The status for
  * an inserted row is either one of these constants, or the row count
  * of the individual column.
@@ -614,16 +626,30 @@ enum IFR_Retcode {
 enum IFR_Rowstatus
 {
     IFR_EXECUTE_FAILED  = SQLDBC_EXECUTE_FAILED,  //!< Execute of this row failed.
-    IFR_SUCCESS_NO_INFO = SQLDBC_SUCCESS_NO_INFO  //!< Execute succeeded, no information about 
+    IFR_SUCCESS_NO_INFO = SQLDBC_SUCCESS_NO_INFO  //!< Execute succeeded, no information about
                                                   //!< affected rows available.
 };
 
 /**
- * Sets the memory failure. 
+ * Sets the memory failure.
  * @param var The 'out of memory' indicator.
  */
 #define IFR_MFAIL(var) var=false
 
 #define IFR_MFAIL_DBUG_RETURN(var, retval) if(!var) DBUG_RETURN(retval)
 
+#if (<MAJOR_VERSION> == 7) && (<MINOR_VERSION> >=6)
+#  define IFR_HAS_VARDATA 1
+#else
+#  define IFR_HAS_VARDATA 0
+#endif
+
+
+#if (<MAJOR_VERSION> == 7) && (<MINOR_VERSION> >=6)
+#  define IFR_HAS_FEATUREPART 1
+#else
+#  define IFR_HAS_FEATUREPART 0
+#endif
+
+
 #endif
