@@ -44,22 +44,6 @@
 
 #define DBUG_NULL_METHOD_ENTER(x,y) DBUG_CONTEXT_METHOD_ENTER(x, y, (IFR_TraceStream *)0)
 
-//----------------------------------------------------------------------
-IFR_Int2
-IFRPacket_Part::partArguments() const
-{
-  DBUG_NULL_METHOD_ENTER(IFRPacket_Part,partArguments);
-  DBUG_RETURN((isValid() ? getInt2(IFRPacket_Part_ArgCount_O) : 0));
-}
-
-//----------------------------------------------------------------------
-IFR_Int4
-IFRPacket_Part::bufferLength() const
-{
-  DBUG_NULL_METHOD_ENTER(IFRPacket_Part,bufferLength);
-  DBUG_RETURN((isValid() ? getInt4(IFRPacket_Part_BufLen_O) : 0));
-}
-
 //----------------------------------------------------------------------
 IFR_Int2
 IFRPacket_Part::getInt2(IFR_Int4 position) const
@@ -102,7 +86,7 @@ IFRPacket_Part::getInt4(IFR_Int4 position) const
     
 //----------------------------------------------------------------------
 IFR_Retcode
-IFRPacket_Part::getText(IFR_String& text, IFR_Bool& memory_ok)
+IFRPacket_Part::getText(IFR_String& text, IFR_Bool& memory_ok, IFR_Bool append)
 {
     DBUG_NULL_METHOD_ENTER(IFRPacket_Part, getText);
     if(!memory_ok || !isValid()) {
@@ -110,10 +94,14 @@ IFRPacket_Part::getText(IFR_String& text, IFR_Bool& memory_ok)
     }
 
     const teo00_Byte *data=GetReadData();
-    IFR_Int4 len=bufferLength();
+    IFR_Int4 len=getBufferLength();
     DBUG_PRINT(len);
-    if(partArguments()>0 && len>0 && data) {
-        text.setBuffer((char*)data, len, m_encoding, memory_ok);
+    if(getPartArguments()>0 && len>0 && data) {
+        if(append) {
+            text.append((char*)data, m_encoding, len, memory_ok);
+        } else {
+            text.setBuffer((char*)data, len, m_encoding, memory_ok);
+        }
         if(!memory_ok) {
             DBUG_RETURN(IFR_NO_DATA_FOUND);
         }  
@@ -123,6 +111,27 @@ IFRPacket_Part::getText(IFR_String& text, IFR_Bool& memory_ok)
     }
 }   
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRPacket_ErrorTextPart::setErrorText(IFR_ErrorHndl& error)
+{
+    if(!isValid()) {
+        return IFR_NOT_OK;
+    }
+    
+    const char *errortext = error.getErrorText();
+    IFR_size_t  errortext_length = strlen(errortext);
+    
+    if(getRemainingBytes() >= errortext_length) {
+        AddData(errortext, (IFR_Int4)errortext_length);    
+        GetRawPart()->sp1p_part_header().sp1p_arg_count = 1;
+        return IFR_OK;
+    } else {
+        return IFR_NOT_OK;
+    }
+}
+
+
 //----------------------------------------------------------------------
 IFR_Retcode
 IFRPacket_ParseIDPart::getParseID(IFR_ParseID& parseid) const
@@ -133,7 +142,7 @@ IFRPacket_ParseIDPart::getParseID(IFR_ParseID& parseid) const
     }
 
     const teo00_Byte *data=GetReadData();
-    if(partArguments()>0 && bufferLength()==IFR_ParseID_Size && data) {
+    if(getPartArguments()>0 && getBufferLength()==IFR_ParseID_Size && data) {
         parseid.setParseID(data);
         return IFR_OK;
     } else {
@@ -145,7 +154,7 @@ IFRPacket_ParseIDPart::getParseID(IFR_ParseID& parseid) const
 IFR_Retcode IFRPacket_ShortInfoPart::getColCount (IFR_Int2& ColCount) const
 {
   if(isValid()) {
-    ColCount = partArguments();
+    ColCount = getPartArguments();
     return IFR_OK;
   }
   return IFR_NO_DATA_FOUND;
@@ -156,30 +165,14 @@ IFRPacket_ShortInfoPart::getShortInfos(IFR_ShortInfo* shortinfo)
 {
   DBUG_NULL_METHOD_ENTER(IFRPacket_ShortInfoPart,getShortInfos);
     IFR_Int4 args;
-    if(!isValid() || (args = partArguments())==0) {
+    if(!isValid() || (args = getPartArguments())==0) {
         return IFR_NO_DATA_FOUND;
-    }    
-    if(sizeof(IFR_ShortInfo)==12) {
-        memcpy(shortinfo, GetReadData(), args * 12);
-    } else {
-        char *p=(char*) GetReadData();
-        for(IFR_Int4 i=0; i<args; ++i) {
-            memcpy(&shortinfo[i].mode, p, 1);
-            ++p;
-            memcpy(&shortinfo[i].iotype, p, 1);
-            ++p;
-            memcpy(&shortinfo[i].datatype, p, 1);
-            ++p;
-            memcpy(&shortinfo[i].frac, p, 1);
-            ++p;
-            memcpy(&shortinfo[i].length, p, 2);
-            p+=2;
-            memcpy(&shortinfo[i].iolength, p, 2);
-            p+=2;
-            memcpy(&shortinfo[i].bufpos, p, 4);
-            p+=4;
-        }
     }
+    char *p=(char*) GetReadData();
+    for(IFR_Int4 i=0; i<args; ++i,p+=12) {
+        memcpy(&shortinfo[i], p, 12);
+    }
+#if 0
     // detect unicode columns and update the encoding for this
     IFR_StringEncoding encoding = this->getEncoding();
     if (encoding == IFR_StringEncodingAscii) {
@@ -196,33 +189,50 @@ IFRPacket_ShortInfoPart::getShortInfos(IFR_ShortInfo* shortinfo)
         shortinfo[i].encoding = encoding;
       }
     }
+#endif
     return IFR_OK;
 }
 
 //----------------------------------------------------------------------
 IFR_Retcode
-IFRPacket_ShortInfoPart::parseShortFields(IFRConversion_Converter**& CnvCon,
+IFRPacket_ShortInfoPart::parseShortFields(IFRConversion_ConverterList& convList,
                                           IFR_ConnectionItem& clink)
 {
   DBUG_CONTEXT_METHOD_ENTER(IFRPacket_ShortInfoPart, parseShortFields, &clink);
-  IFRConversion_Converter **result;
-  IFR_Int4 colcnt;
 
-  if(!isValid() || (colcnt = partArguments())==0) {
+  
+
+  IFR_Int4 colcnt;
+  if(!isValid() || (colcnt = getPartArguments())==0) {
       DBUG_RETURN(IFR_NO_DATA_FOUND);
   }
-    
   SAPDBMem_IRawAllocator& allocator=clink.getConnection()->allocator;
-
-  IFR_ShortInfo *si=new (allocator) IFR_ShortInfo[colcnt];
+  IFR_ShortInfo *si=new IFR_ALLOCATOR(allocator) IFR_ShortInfo[colcnt];
+  if(si == 0) {
+      clink.error().setMemoryAllocationFailed();
+      DBUG_RETURN(IFR_NOT_OK);
+  }
   getShortInfos(si);
-  result = new (allocator) IFRConversion_Converter*[colcnt];
+  IFR_Bool memory_ok = true;
   for(IFR_Int4 i=0; i<colcnt; ++i) {
-      result[i]=IFRConversion_Factory::createInstance(si[i], 
-        *(clink.getConnection()));
+      IFRConversion_Converter *c = 
+          IFRConversion_Factory::createInstance(si[i], 
+                                                *(clink.getConnection()),
+                                                m_encoding != IFR_StringEncodingAscii);
+      if(c != 0) {
+          convList.add(c, memory_ok);          
+      }
+      if(c == 0 || !memory_ok) {
+          if(c != 0) {
+              IFRUtil_Delete(c, allocator);
+          }
+          convList.clear();
+          IFRUtil_DeleteArray(si, colcnt, IFR_ALLOCATOR(allocator));
+          clink.error().setMemoryAllocationFailed();
+          DBUG_RETURN(IFR_NOT_OK);
+      } 
   } 
-  IFRUtil_DeleteArray(si, colcnt, allocator);
-  CnvCon = result;
+  IFRUtil_DeleteArray(si, colcnt, IFR_ALLOCATOR(allocator));
   DBUG_RETURN(IFR_OK);  
 }
 
@@ -255,7 +265,7 @@ IFRPacket_CommandPart::addText(const IFR_String& text, IFR_ErrorHndl &error)
     IFR_size_t text_length=text.getLength();
     
     if(encoding == text_encoding) {
-        if(remainingBytes() < text_length) {
+        if(getRemainingBytes() < text_length) {
           error.setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
           rc = IFR_DATA_TRUNC;
         } else {
@@ -266,7 +276,7 @@ IFRPacket_CommandPart::addText(const IFR_String& text, IFR_ErrorHndl &error)
 
     // command encoding is ascii, source encoding is something other.
     if(encoding == IFR_StringEncodingAscii) {
-        if(remainingBytes() < text_length) {
+        if(getRemainingBytes() < text_length) {
           error.setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
           rc = IFR_DATA_TRUNC;
         } else {
@@ -285,7 +295,7 @@ IFRPacket_CommandPart::addText(const IFR_String& text, IFR_ErrorHndl &error)
                     }
                 }
                 if(onlyAscii) {
-                    if(remainingBytes() < text_length) {
+                    if(getRemainingBytes() < text_length) {
                       error.setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
                       rc = IFR_DATA_TRUNC;
                     } else {
@@ -301,7 +311,7 @@ IFRPacket_CommandPart::addText(const IFR_String& text, IFR_ErrorHndl &error)
         return rc;
     } else  if(encoding == IFR_StringEncodingUCS2 || encoding == IFR_StringEncodingUCS2Swapped) {
         if(text.getEncoding() == IFR_StringEncodingAscii) {
-            if(remainingBytes() < text.getLength() * 2) {
+            if(getRemainingBytes() < text.getLength() * 2) {
                 error.setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
                 return IFR_DATA_TRUNC;
             } else {
@@ -309,7 +319,7 @@ IFRPacket_CommandPart::addText(const IFR_String& text, IFR_ErrorHndl &error)
                                    encoding == IFR_StringEncodingUCS2 ? 0 : 1);
             }
         } else if (text_encoding == IFR_StringEncodingUCS2 || text_encoding == IFR_StringEncodingUCS2Swapped) {
-            if(remainingBytes() < text.getLength()) {
+            if(getRemainingBytes() < text.getLength()) {
                 error.setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
                 return IFR_DATA_TRUNC;
             }
@@ -326,7 +336,7 @@ IFRPacket_CommandPart::addText(const IFR_String& text, IFR_ErrorHndl &error)
             part->sp1p_buf_len() += text_length;
         } else { // UTF 8 not yet there
 
-            if((IFR_Length)remainingBytes() < text.getStrLen() * 2) {
+            if((IFR_Length)getRemainingBytes() < text.getStrLen() * 2) {
                 error.setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
                 return IFR_DATA_TRUNC;
             } else {
@@ -340,14 +350,41 @@ IFRPacket_CommandPart::addText(const IFR_String& text, IFR_ErrorHndl &error)
       error.setRuntimeError(IFR_ERR_NOT_IMPLEMENTED("command encoding UTF8"));
       return IFR_NOT_OK;
     }
-    return rc;
 }
 
 //----------------------------------------------------------------------
 IFR_Retcode
-IFRPacket_ParseIDPart::setParseID(IFR_ParseID& parseid)
+IFRPacket_CommandInfoPart::addCommandInfo(const IFR_String& text, IFR_Int4 _lineNumber, IFR_ErrorHndl &error)
 {
-  DBUG_NULL_METHOD_ENTER(IFRPacket_ParseIDPart, setParseID);
+  DBUG_NULL_METHOD_ENTER(IFRPacket_CommandInfoPart, addCommandInfo);
+  DBUG_PRINT(text);
+  DBUG_PRINT(_lineNumber);
+  if(!isValid()) {
+    return IFR_NOT_OK;
+  }
+  char lineNum[12];
+  unsigned char text_length = (unsigned char) text.getLength();
+  unsigned char lineNumLen  = (unsigned char) sp77sprintf (lineNum, sizeof(lineNum),"%d", _lineNumber);
+
+  IFR_Retcode rc=IFR_OK;
+    
+  if(getRemainingBytes() < (IFR_UInt4)(text_length+lineNumLen+2)) {
+     error.setRuntimeError(IFR_ERR_PACKET_EXHAUSTED);
+     rc = IFR_DATA_TRUNC;
+  } else {
+     AddData(&text_length, 1);
+     AddData(text.getBuffer(), (IFR_Int4)text_length);
+     AddData(&lineNumLen, 1);
+     AddData(lineNum, (IFR_Int4)lineNumLen);
+     this->setArgCount(2);    
+  } 
+  return rc;
+}
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRPacket_ParseIDPart::addParseID(IFR_ParseID& parseid)
+{
+  DBUG_NULL_METHOD_ENTER(IFRPacket_ParseIDPart, addParseID);
   DBUG_PRINT(parseid);
     if(!isValid()) {
         return IFR_NOT_OK;
@@ -357,115 +394,241 @@ IFRPacket_ParseIDPart::setParseID(IFR_ParseID& parseid)
     }
 }
 
-//----------------------------------------------------------------------
-// stolen from vin201.cpp
-// does the same, except that the part argument count is not
-// modified.
-void
-IFRPacket_DataPart::addParameter(const void *buffer,
-                                 IFR_Int4 position,
-                                 IFR_Int4 length,
-                                 IFR_Int4 iolength,
-                                 char definedbytepadding)
-{
-  DBUG_NULL_METHOD_ENTER(IFRPacket_DataPart, addParameter);
-  DBUG_PRINT(position);
-  DBUG_PRINT(length);
-  DBUG_PRINT(iolength);
-  DBUG_PRINT(definedbytepadding);
-    IFR_Int4 len;
-    IFR_Int4 offset=m_massextent;
 
-    
-    char *data = GetRawPart()->sp1p_buf().asCharp();
-    char *p    = data + offset + position - 1;
-    
-    if(0 == buffer) {
-        *p = (char) csp_undef_byte;
-    } else {
-        *p = definedbytepadding;
-        ++p;
-        --iolength;
-        IFR_Int4 cplen;
-        IFR_Int4 padlen;
-        if(length < iolength) {
-            cplen  = length;
-            padlen = iolength - length;
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRPacket_DataPart::addBinaryParameter(const void *buffer,
+                                       IFR_Int4 length,
+                                       IFR_ShortInfo &shortinfo)
+{
+    DBUG_NULL_METHOD_ENTER(IFRPacket_DataPart, addBinaryParameter);
+    DBUG_PRINT(length);
+    if(m_variableinput) {
+        char *elementstart = GetRawPart()->sp1p_buf().asCharp() + GetRawPart()->sp1p_buf_len();
+        char *datastart;
+        m_extent = GetRawPart()->sp1p_buf_len();
+        IFR_Int4 maxlen = shortinfo.iolength - 1;
+        IFR_Int4 usedlength = maxlen > length ? length : maxlen;
+        if(maxlen > 250) {
+            elementstart[0] = (char) 0xFF;
+            elementstart[1] = (usedlength >> 8);
+            elementstart[2] = (usedlength & 0xFF);
+            datastart = elementstart + 3;
         } else {
-            cplen  = iolength;
-            padlen = 0;
+            elementstart[0] = usedlength;
+            datastart = elementstart + 1;
         }
-        memcpy(p, buffer, cplen);
-        if(padlen) {
-            memset(p + cplen, definedbytepadding, padlen);
+        memcpy(datastart, buffer, usedlength);
+        if(maxlen > 250) {
+            GetRawPart()->sp1p_buf_len () += usedlength + 3;
+        } else {
+            GetRawPart()->sp1p_buf_len () += usedlength + 1;
         }
+        if(usedlength < length) {
+            DBUG_RETURN(IFR_DATA_TRUNC);
+        } else {
+            DBUG_RETURN(IFR_OK);
+        }
+    } else {
+        IFR_Int4 len;
+        IFR_Int4 offset=m_massextent;
+        char *data = GetRawPart()->sp1p_buf().asCharp();
+        char *p    = data + offset + shortinfo.pos.bufpos - 1;
+        IFR_UInt2 iolength = shortinfo.iolength;
+        IFR_Retcode rc = IFR_OK;
+        if(0 == buffer) {
+            *p = (char) csp_undef_byte;
+            // memset(p+1, 0, iolength -1);
+        } else {
+            *p = shortinfo.getDefinedByte(this->getEncoding()!=IFR_StringEncodingAscii);
+            ++p;
+            --iolength;
+            if(length > iolength) {
+                memcpy(p, buffer, iolength);
+                rc = IFR_DATA_TRUNC;
+            } else {
+                memcpy(p, buffer, length);
+                memset(p + length, 0, iolength - length);
+            }
+        }
+        len = (IFR_Int4)(shortinfo.pos.bufpos + iolength);
+        GetRawPart()->sp1p_buf_len() =  MAX (this->GetRawPart()->sp1p_buf_len (), offset+len);
+        // no argument count modification
+        return rc;
     }
-    len = position + iolength;
-    GetRawPart()->sp1p_buf_len() =  MAX (this->GetRawPart()->sp1p_buf_len (), offset+len);
-    m_extent = MAX (m_extent, len);
-    // no argument count modification
-    return;
 }
 
+
 //----------------------------------------------------------------------
 IFR_Retcode
 IFRPacket_DataPart::addParameter(const void *buffer,
                                  IFR_Int4 length,
                                  IFR_StringEncoding srcEncoding,
+                                 IFR_StringEncoding targetEncoding,
                                  IFR_ShortInfo &shortinfo)
 {
     DBUG_NULL_METHOD_ENTER(IFRPacket_DataPart, addParameter);
     DBUG_PRINT(length);
     DBUG_PRINT(srcEncoding);
     
-    IFR_Int4 len;
-    IFR_Int4 offset=m_massextent;
-    char *data = GetRawPart()->sp1p_buf().asCharp();
-    char *p    = data + offset + shortinfo.bufpos - 1;
-    IFR_UInt2 iolength = shortinfo.iolength;
-    IFR_Retcode rc = IFR_OK;
-    if(0 == buffer) {
-        *p = (char) csp_undef_byte;
-    } else {
-        *p = shortinfo.getDefinedByte(false);
-        ++p;
-        --iolength;
+    if(m_variableinput) {
+        IFR_Retcode rc = IFR_OK;
+        char *elementstart = GetRawPart()->sp1p_buf().asCharp() + GetRawPart()->sp1p_buf_len();
+        m_extent = GetRawPart()->sp1p_buf_len();
+        char *datastart;
+        IFR_UInt4 maxlen = shortinfo.iolength - 1;
+        if(maxlen > 250) {
+            datastart = elementstart + 3;
+            elementstart[0] = (char) 255;
+        } else {
+            datastart = elementstart + 1;
+        }
+        const tsp77encoding* target_encoding_sp77 = IFR_ENCODING(targetEncoding);
         IFR_UInt4 destbyteswritten;
         IFR_UInt4 srcbytesparsed;
         tsp78ConversionResult convres =
-          sp78convertBuffer(IFR_ENCODING(shortinfo.encoding), p, iolength,
-                            &destbyteswritten,
-                            IFR_ENCODING(srcEncoding), buffer, length, 
-                            &srcbytesparsed);
+            sp78convertBuffer(target_encoding_sp77, datastart, maxlen,
+                              &destbyteswritten,
+                              IFR_ENCODING(srcEncoding), buffer, length, 
+                              &srcbytesparsed);
         if (convres != sp78_Ok && convres != sp78_TargetExhausted) {
-          return IFR_NOT_OK;
+            return IFR_NOT_OK;
         }
         if (convres == sp78_TargetExhausted) {
-          rc = IFR_DATA_TRUNC;
+            rc = IFR_DATA_TRUNC;
         }
-        else {
-          void *f = p+destbyteswritten;
-          unsigned int padlen = iolength-destbyteswritten;
-          int bufferSwapped = (shortinfo.encoding == IFR_StringEncodingUCS2Swapped);
-          IFR_ENCODING(shortinfo.encoding)->fillString(&f, 
-                                                       &padlen,
-                                                       padlen/IFR_ENCODING(shortinfo.encoding)->terminatorSize, 
-                                                       ' ');          
+        if(maxlen > 250) {
+            elementstart[1] = (destbyteswritten >> 8) & 0xFF;
+            elementstart[2] = (destbyteswritten & 0xFF);
+            GetRawPart()->sp1p_buf_len() += destbyteswritten + 3;
+        } else {
+            elementstart[0] = (destbyteswritten & 0xFF);
+            GetRawPart()->sp1p_buf_len() += destbyteswritten + 1;
         }
+        DBUG_RETURN(rc);
+    } else {
+        IFR_Int4 len;
+        IFR_Int4 offset=m_massextent;
+        char *data = GetRawPart()->sp1p_buf().asCharp();
+        char *p    = data + offset + shortinfo.pos.bufpos - 1;
+        IFR_UInt2 iolength = shortinfo.iolength;
+        IFR_Retcode rc = IFR_OK;
+        if(0 == buffer) {
+            *p = (char) csp_undef_byte;
+        } else {
+            *p = shortinfo.getDefinedByte(this->getEncoding()!=IFR_StringEncodingAscii);
+            ++p;
+            --iolength;
+            
+            const tsp77encoding* target_encoding_sp77 = IFR_ENCODING(targetEncoding);
+            
+            IFR_UInt4 destbyteswritten;
+            IFR_UInt4 srcbytesparsed;
+            tsp78ConversionResult convres =
+                sp78convertBuffer(target_encoding_sp77, p, iolength,
+                                  &destbyteswritten,
+                                  IFR_ENCODING(srcEncoding), buffer, length, 
+                                  &srcbytesparsed);
+            if (convres != sp78_Ok && convres != sp78_TargetExhausted) {
+                return IFR_NOT_OK;
+            }
+            if (convres == sp78_TargetExhausted) {
+                rc = IFR_DATA_TRUNC;
+            } else {
+                void *f = p+destbyteswritten;
+                unsigned int padlen = iolength-destbyteswritten;
+                target_encoding_sp77->fillString(&f, 
+                                                 &padlen,
+                                                 padlen / target_encoding_sp77->terminatorSize, 
+                                                 shortinfo.getPaddingCharacter());          
+            }
+        }
+        len = (IFR_Int4)(shortinfo.pos.bufpos + iolength);
+        
+        GetRawPart()->sp1p_buf_len() =  MAX (this->GetRawPart()->sp1p_buf_len (), offset+len);
+        // no argument count modification
+        return rc;
     }
-    len = (IFR_Int4)(shortinfo.bufpos + iolength);
-    
-    GetRawPart()->sp1p_buf_len() =  MAX (this->GetRawPart()->sp1p_buf_len (), offset+len);
-    m_extent = MAX (m_extent, len);
-    // no argument count modification
-    return rc;
 }
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRPacket_DataPart::appendBinaryToParameter(const void *buffer,
+                                            IFR_Int4 length,
+                                            IFR_ShortInfo &shortinfo,
+                                            IFR_Length& offset)
+{
+    DBUG_NULL_METHOD_ENTER(IFRPacket_DataPart, appendBinaryToParameter);
+    DBUG_PRINT(length);
+    DBUG_PRINT(offset);
+    if(m_variableinput) {
+        char *datastart = GetRawPart()->sp1p_buf().asCharp() + m_extent;
+        IFR_Int4 maxlength = shortinfo.iolength - 1;
+        if(offset >= maxlength) {
+            DBUG_RETURN(IFR_DATA_TRUNC);
+        }
+        char *datapos = datastart + ((maxlength) > 250 ? 3 : 1) + offset;
+        if(maxlength - offset < length) {
+            memcpy(datapos, buffer, maxlength - offset);
+            GetRawPart()->sp1p_buf_len() 
+                = MAX( GetRawPart()->sp1p_buf_len(),
+                       m_extent + ((maxlength) > 250 ? 3 : 1) + maxlength );
+            offset = maxlength;
+            if(maxlength > 250) {
+                datastart[0] = (char) 255;
+                datastart[1] = (char) (maxlength >> 8);
+                datastart[2] = (char) (maxlength & 0xFF);
+            } else {
+                datastart[0] = maxlength;
+            }
+            DBUG_RETURN(IFR_DATA_TRUNC);
+        } else {
+            memcpy(datapos, buffer, length);
+            offset += length;
+            GetRawPart()->sp1p_buf_len() 
+                = MAX( GetRawPart()->sp1p_buf_len(),
+                       m_extent + ((maxlength) > 250 ? 3 : 1) + offset);
+            if(maxlength > 250) {
+                datastart[0] = (char) 255;
+                datastart[1] = (char) (offset >> 8);
+                datastart[2] = (char) (offset & 0xFF);
+            } else {
+                datastart[0] = offset;
+            }
+            DBUG_RETURN(IFR_OK);
+        }
+    } else {
+        IFR_Int4 dataoffset=m_massextent;
+        char *data = GetRawPart()->sp1p_buf().asCharp();
+        char *p    = data + dataoffset + shortinfo.pos.bufpos + offset;
+        IFR_UInt2 iolength = shortinfo.iolength-1; // iolength has the defined byte
+        IFR_Retcode rc = IFR_OK;
+        
+        if(offset >= iolength) {
+            DBUG_RETURN(IFR_DATA_TRUNC);
+        }
+
+        if(iolength-offset < length) {
+            memcpy(p, buffer, iolength - offset);
+            offset += (iolength - offset);
+            rc = IFR_DATA_TRUNC;
+        } else {
+            memcpy(p, buffer, length);
+        offset += length;
+        }
+        // It is not extended, as an add operation was done previously.
+        return rc;
+    }
+}
+
+
+
 //----------------------------------------------------------------------
 IFR_Retcode
 IFRPacket_DataPart::appendToParameter(const void *buffer,
                                       IFR_Int4 length,
                                       IFR_StringEncoding srcEncoding,
+                                      IFR_StringEncoding targetEncoding,
                                       IFR_ShortInfo &shortinfo,
                                       IFR_Length& offset)
 {
@@ -474,39 +637,131 @@ IFRPacket_DataPart::appendToParameter(const void *buffer,
     DBUG_PRINT(srcEncoding);
     DBUG_PRINT(offset);
     
-    IFR_Int4 dataoffset=m_massextent;
-    char *data = GetRawPart()->sp1p_buf().asCharp();
-    char *p    = data + dataoffset + shortinfo.bufpos + offset;
-    IFR_UInt2 iolength = shortinfo.iolength-1; // iolength has the defined byte
-    
-    if(offset >= iolength) {
-        DBUG_RETURN(IFR_DATA_TRUNC);
-    }
-    
-    IFR_Retcode rc = IFR_OK;
-    IFR_UInt4 destbyteswritten;
-    IFR_UInt4 srcbytesparsed;
-    tsp78ConversionResult convres =
-        sp78convertBuffer(IFR_ENCODING(shortinfo.encoding), p, iolength - offset,
-                          &destbyteswritten,
-                          IFR_ENCODING(srcEncoding), buffer, length, 
-                          &srcbytesparsed);
-    if (convres != sp78_Ok && convres != sp78_TargetExhausted) {
-        return IFR_NOT_OK;
-    }
-    if (convres == sp78_TargetExhausted) {
+    if(m_variableinput) {
+            char *datastart = GetRawPart()->sp1p_buf().asCharp() + m_extent;
+        IFR_Int4 maxlength = shortinfo.iolength - 1;
+        if(offset >= maxlength) {
+            DBUG_RETURN(IFR_DATA_TRUNC);
+        }
+        char *datapos = datastart + ((maxlength) > 250 ? 3 : 1) + offset;
+        IFR_UInt4 destbyteswritten;
+        IFR_UInt4 srcbytesparsed;
+        tsp78ConversionResult convres =
+            sp78convertBuffer(IFR_ENCODING(targetEncoding), datapos, maxlength - offset,
+                              &destbyteswritten,
+                              IFR_ENCODING(srcEncoding), buffer, length, 
+                              &srcbytesparsed);
+        if(convres != sp78_Ok && convres != sp78_TargetExhausted) {
+            DBUG_RETURN(IFR_NOT_OK);
+        } 
         offset += destbyteswritten;
-        rc = IFR_DATA_TRUNC;
+        // extend the buffer
+        GetRawPart()->sp1p_buf_len() 
+            = MAX( GetRawPart()->sp1p_buf_len(),
+                   m_extent + ((maxlength) > 250 ? 3 : 1) + offset );
+        if(maxlength > 250) {
+            datastart[0] = (char) 255;
+            datastart[1] = (char) (offset >> 8);
+            datastart[2] = (char) (offset & 0xFF);
+        } else {
+            datastart[0] = offset;
+        }
+        if(convres == sp78_TargetExhausted) {
+            DBUG_RETURN(IFR_DATA_TRUNC);
+        } else {
+            DBUG_RETURN(IFR_OK);
+        }
     } else {
-        // No padding, as the first add has padded.
-        offset += destbyteswritten;
+        IFR_Int4 dataoffset=m_massextent;
+        char *data = GetRawPart()->sp1p_buf().asCharp();
+        char *p    = data + dataoffset + shortinfo.pos.bufpos + offset;
+        IFR_UInt2 iolength = shortinfo.iolength-1; // iolength has the defined byte
+        
+        if(offset >= iolength) {
+            DBUG_RETURN(IFR_DATA_TRUNC);
+        }
+        
+        IFR_Retcode rc = IFR_OK;
+        IFR_UInt4 destbyteswritten;
+        IFR_UInt4 srcbytesparsed;
+        tsp78ConversionResult convres =
+            sp78convertBuffer(IFR_ENCODING(targetEncoding), p, iolength - offset,
+                              &destbyteswritten,
+                              IFR_ENCODING(srcEncoding), buffer, length, 
+                              &srcbytesparsed);
+        if (convres != sp78_Ok && convres != sp78_TargetExhausted) {
+            return IFR_NOT_OK;
+        }
+        if (convres == sp78_TargetExhausted) {
+            offset += destbyteswritten;
+            rc = IFR_DATA_TRUNC;
+        } else {
+            // No padding, as the first add has padded.
+            offset += destbyteswritten;
+        }
+        
+        // It is not extended, as an add operation was done previously.
+        return rc;
     }
-  
-    // It is not extended, as an add operation was done previously.
-    return rc;
 }
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRPacket_DataPart::addNullInput(IFR_ShortInfo& shortinfo)
+{
+    if(m_variableinput) {
+        char *data = GetRawPart()->sp1p_buf().asCharp() + GetRawPart()->sp1p_buf_len();
+        *data = (char) 0xFE;
+        GetRawPart()->sp1p_buf_len() += 1;
+        return IFR_OK;
+    } else {
+        addData((IFR_Int1)csp_undef_byte, shortinfo.pos.bufpos, shortinfo.iolength);
+        return IFR_OK;
+    }
+}
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRPacket_DataPart::addDefaultInput(IFR_ShortInfo& shortinfo)
+{
+    if(m_variableinput) {
+        char *data = GetRawPart()->sp1p_buf().asCharp() + GetRawPart()->sp1p_buf_len();
+        *data = (char) 0xFE;
+        GetRawPart()->sp1p_buf_len() += 1;
+        return IFR_OK;
+    } else {
+        addData((IFR_Int1)csp_default_byte, shortinfo.pos.bufpos, shortinfo.iolength);
+        return IFR_OK;
+    }
+}
+
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRPacket_DataPart::finishData(IFR_Length length, IFR_ShortInfo& shortinfo)
+{
+    if(m_variableinput) {
+        IFR_UInt4 maxlength = shortinfo.iolength - 1;
+        char *data = GetRawPart()->sp1p_buf().asCharp() + GetRawPart()->sp1p_buf_len();
+        if(maxlength > 250) {
+            data[0] = (char) 255;
+            data[1] = (char) (maxlength >> 8);
+            data[2] = (char) (maxlength & 0xFF);
+            GetRawPart()->sp1p_buf_len() += (3 + maxlength);
+        } else {
+            data[0] = maxlength;
+            GetRawPart()->sp1p_buf_len() += (1 + maxlength);
+        }
+        return IFR_OK;
+    } else {
+        IFR_Int4 offset = m_massextent;
+        char *data = GetRawPart()->sp1p_buf().asCharp();
+        *(data + offset + shortinfo.pos.bufpos -1) = shortinfo.getDefinedByte(this->getEncoding()!=IFR_StringEncodingAscii);
+        IFR_Int4 len = shortinfo.pos.bufpos + shortinfo.iolength - 1;
+        GetRawPart()->sp1p_buf_len() =  
+            MAX (this->GetRawPart()->sp1p_buf_len (), offset+len);
+        return IFR_OK;
+    }
+}
 
 //----------------------------------------------------------------------
 void
@@ -527,7 +782,6 @@ IFRPacket_DataPart::addData(IFR_Int1 definedbyte,
 
     GetRawPart()->sp1p_buf_len() =  
         MAX (this->GetRawPart()->sp1p_buf_len (), offset+len);
-    m_extent = MAX (m_extent, len);
 }
 
 
@@ -550,7 +804,6 @@ IFRPacket_DataPart::addData(void *buffer,
     len = position + iolength - 1;
 
     GetRawPart()->sp1p_buf_len() =  MAX (this->GetRawPart()->sp1p_buf_len (), offset+len);
-    m_extent = MAX (m_extent, len);
     return;
 }
 
@@ -561,8 +814,8 @@ IFRPacket_ResultCountPart::getResultCount(IFR_Int4& ResCount) const
   ResCount = 0;
   if(isValid()) {
     IFR_Byte *data = const_cast<IFR_Byte *>(GetReadData());
-    if(partArguments() && data) {
-      IFR_Int4 len=bufferLength();
+    if(getPartArguments() && data) {
+      IFR_Int4 len=getBufferLength();
       if (len > 0 && *data != csp_undef_byte) {
         data++; // strip defined byte
         rc = IFRUtil_VDNNumber::numberToInt4(data, ResCount, len);
@@ -581,8 +834,8 @@ IFRPacket_TableNamePart::getResultName(IFR_String& ResName, IFR_Bool& memory_ok)
   ResName.setBuffer("", 0, IFR_StringEncodingAscii, memory_ok);
   if(isValid()) {
     const char *data = REINTERPRET_CAST(const char *,GetReadData());
-    if(partArguments() && data) {
-      IFR_Int4 len=bufferLength();
+    if(getPartArguments() && data) {
+      IFR_Int4 len=getBufferLength();
       if (len > 0) {
         ResName.setBuffer(data, len, this->getEncoding(), memory_ok);
         rc = IFR_OK;
@@ -634,7 +887,7 @@ IFRPacket_ResultCountPart::setUndefResultCount()
     tsp1_part *rawpart=GetRawPart();
     
     // check whether there are enough bytes 
-    if(remainingBytes() < sizeof(vdn) - rawpart->sp1p_buf_len()) {
+    if(getRemainingBytes() < sizeof(vdn) - rawpart->sp1p_buf_len()) {
         return IFR_NOT_OK;
     }
     rawpart->sp1p_part_header().sp1p_arg_count = 1;    
@@ -660,7 +913,7 @@ IFRPacket_DataPart::hasRoomFor(IFR_UInt4 bytes)
     // - minimum size on 3rd and up s again like 1st
     // 
     // 
-    return remainingBytes() >= bytes + 4096;
+    return getRemainingBytes() >= bytes + 4096;
 }
 
 
@@ -695,7 +948,7 @@ IFRPacket_DataPart::addStreamData(char *&datastart,
     DBUG_NULL_METHOD_ENTER(IFRPacket_DataPart, addStreamData);
     IFR_Retcode rc=IFR_OK;
 
-    IFR_Int4 datasize = remainingBytes();
+    IFR_Int4 datasize = getRemainingBytes();
 
     datasize /=  8;
     datasize *=  8;   
@@ -708,7 +961,7 @@ IFRPacket_DataPart::addStreamData(char *&datastart,
     // we initialise the VALPOS if it is not, and we will initialise
     // in that case the data part to something.
     if(longdesc_valpos == 0) {
-        longdesc_valpos = m_massextent + m_extent + 1;
+        longdesc_valpos = GetRawPart()->sp1p_buf_len() + 1;
         memcpy(longdescriptor + IFRPacket_LongDescriptor::ValPos_O, &longdesc_valpos, sizeof(IFR_Int4));
         setStreamValMode(longdescriptor, IFRPacket_LongDescriptor::DataPart_C);
     }
@@ -716,7 +969,7 @@ IFRPacket_DataPart::addStreamData(char *&datastart,
     tsp00_Uint4 bytes_written = 0;
     tsp00_Uint4 srcbytes_parsed = 0;
 
-    char *dest = GetRawPart()->sp1p_buf().asCharp() + m_massextent + m_extent;
+    char *dest = GetRawPart()->sp1p_buf().asCharp() + GetRawPart()->sp1p_buf_len();
     IFR_Int4 source_size = dataend - datastart;
     tsp78ConversionResult conv_res = sp78convertString(IFR_ENCODING(targetencoding),
                                                        dest,
@@ -736,12 +989,7 @@ IFRPacket_DataPart::addStreamData(char *&datastart,
         clink.error().setRuntimeError(IFR_ERR_STRING_CONVERSION);
         DBUG_RETURN(IFR_NOT_OK);
     }
-    IFR_Int4 newbuflen = (IFR_Int4) GetRawPart()->sp1p_buf_len();
-    if(newbuflen < (IFR_Int4) (m_massextent + m_extent + bytes_written)) {
-        newbuflen = m_massextent + m_extent + bytes_written;
-    }
-    GetRawPart()->sp1p_buf_len() = newbuflen;
-    m_extent += bytes_written; 
+    GetRawPart()->sp1p_buf_len() += bytes_written;
     longdesc_vallen += bytes_written;
     memcpy(longdescriptor + IFRPacket_LongDescriptor::ValLen_O, &longdesc_vallen, sizeof(IFR_Int4));
     datastart += srcbytes_parsed;
@@ -758,38 +1006,25 @@ IFRPacket_DataPart::setStreamValMode(char *longdescriptor,
     return;
 }
 
-//----------------------------------------------------------------------
-void
-IFRPacket_DataPart::moveRecordBase ()
-{
-  DBUG_NULL_METHOD_ENTER(IFRPacket_DataPart, moveRecordBase);
-
-  m_massextent += m_extent;
-  m_extent = 0;
-}
-
-
 //----------------------------------------------------------------------
 IFR_Retcode
-IFRPacket_LongDataPart::addDescriptor(IFRPacket_LongDescriptor& longdesc)
+IFRPacket_DataPart::addDescriptor(IFRPacket_LongDescriptor& longdesc)
 {
     // We must ensure that the descriptor is inserted *behind* the current data.
     // Otherwise it will be inserted at the beginning of the part.
     moveRecordBase();    
     
-    IFR_Int4 datasize = remainingBytes() & ~7;
+    IFR_Int4 datasize = getRemainingBytes() & ~7;
     
-    if(datasize < IFRPacket_LongDescriptor::Size + 1) {
+    if(datasize < IFRPacket_LongDescriptor::IOLength) {
         return IFR_NOT_OK;
     }
     
-    addParameter(&longdesc,
-                 1,
-                 IFRPacket_LongDescriptor::Size,
-                 IFRPacket_LongDescriptor::Size + 1, // io length is plus the defined byte
-                 0);
-
-    GetRawPart()->sp1p_part_header().sp1p_arg_count++;
+    char      *p = GetRawPart()->sp1p_buf().asCharp() + m_massextent;
+    *p = 0;
+    memcpy(p+1, &longdesc, IFRPacket_LongDescriptor::Size);
+    GetRawPart()->sp1p_buf_len() = MAX(GetRawPart()->sp1p_buf_len(), m_massextent + IFRPacket_LongDescriptor::IOLength);
+    GetRawPart()->sp1p_part_header().sp1p_arg_count += 1;
     return IFR_OK;
 }
 
@@ -807,7 +1042,7 @@ IFRPacket_ApplParamPart::addArgument(IFR_SQLType datatype,
                                      IFR_Int1    frac,
                                      IFR_size_t  length)
 {
-    IFR_size_t currentArguments=(IFR_size_t)partArguments();
+    IFR_size_t currentArguments=(IFR_size_t)getPartArguments();
     char *data = GetRawPart()->sp1p_buf().asCharp() + currentArguments * 4;
     *data=(IFR_Int1)datatype;
     *(data+1)=frac;
@@ -815,10 +1050,36 @@ IFRPacket_ApplParamPart::addArgument(IFR_SQLType datatype,
     memcpy(data+2, &used_length, sizeof(IFR_Int2));
     GetRawPart()->sp1p_buf_len() =  
         MAX (this->GetRawPart()->sp1p_buf_len (), (IFR_Int4) (currentArguments +1) * 4);
-    GetRawPart()->sp1p_part_header().sp1p_arg_count++;
+    ++(GetRawPart()->sp1p_part_header().sp1p_arg_count);
     return;
 }
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRPacket_FeaturePart::requestFeature(IFRPacket_Feature::Feature feature)
+{
+    IFR_Int2 fcount = getPartArguments();
+    char *data = GetRawPart()->sp1p_buf().asCharp();
+    for(IFR_Int2 i=0; i<fcount; ++i) {
+        if(data[i*2] == (char) feature) {
+            return IFR_OK;
+        }
+    }
+    
+    if(BytesRemaining() < 2) {
+        return IFR_NOT_OK;
+    }
+
+    data += (fcount * 2);
+    data[0] = (char) feature;
+    data[1] = 1;
+    GetRawPart()->sp1p_buf_len() =  
+        MAX (this->GetRawPart()->sp1p_buf_len (), fcount * 2 + 2);
+    ++(GetRawPart()->sp1p_part_header().sp1p_arg_count);
+    return IFR_OK;
+}
+
+
 //----------------------------------------------------------------------
 IFR_Retcode
 IFRPacket_ColumnNamesPart::getColumnNames(IFRUtil_Vector<IFR_String>& columnnames, 
@@ -832,7 +1093,7 @@ IFRPacket_ColumnNamesPart::getColumnNames(IFRUtil_Vector<IFR_String>& columnname
     if(!isValid()) {
         return IFR_NO_DATA_FOUND;
     }
-    IFR_Int2 pa=partArguments();
+    IFR_Int2 pa=getPartArguments();
     columnnames.Clear();
     const teo00_Byte *data=GetReadData();    
     for(IFR_Int2 i=0; i<pa; ++i) {
@@ -903,6 +1164,25 @@ IFRPacket_LongDemandPart::addPosParams(const IFR_Bool immediately,
   return rc;
 }
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFRPacket_VarDataShortInfoPart::parseShortFields(IFRConversion_ConverterList& convList,
+                                                 IFR_ConnectionItem& clink)
+{
+    DBUG_CONTEXT_METHOD_ENTER(IFRPacket_VarDataShortInfoPart, parseShortFields, &clink);
+    IFR_Retcode rc = IFRPacket_ShortInfoPart::parseShortFields(convList, clink);
+    if(rc == IFR_OK) {
+        IFR_Bool memory_ok = true;
+        convList.buildIndex(memory_ok);
+        if(!memory_ok) {
+            clink.error().setMemoryAllocationFailed();
+            DBUG_RETURN(IFR_NOT_OK);
+        }
+    }
+    DBUG_RETURN(rc);
+}
+
+//----------------------------------------------------------------------
 class IFR_TraceStream;
 IFR_TraceStream& operator << (IFR_TraceStream& s, const enum IFRPacket_PartKind::PartKind p )
 {
@@ -936,6 +1216,14 @@ IFR_TraceStream& operator << (IFR_TraceStream& s, const enum IFRPacket_PartKind:
     IFRCASE(AbapOStream_C);
     IFRCASE(AbapInfo_C);
     IFRCASE(LongDemand_C);
+    IFRCASE(MessageList_C);
+#if IFR_HAS_VARDATA
+    IFRCASE(VarDataShortInfo_C);
+    IFRCASE(VarData_C);
+#endif
+#if IFR_HAS_FEATUREPART
+    IFRCASE(Feature_C);
+#endif
 #undef IFRCASE
   default:{s << "(unknown " << (int)p << ")";break;}
   }
