@@ -37,51 +37,16 @@
 #include "Interfaces/Runtime/IFR_Datatypes.h"
 #include "Interfaces/Runtime/IFR_ErrorHndl.h"
 #include "Interfaces/Runtime/Util/IFRUtil_Vector.h"
-#include "Interfaces/Runtime/Util/IFRUtil_Stream.h"
+#include "Interfaces/Runtime/IFR_ShortInfo.h"
 
 class IFRConversion_Converter;
+class IFRConversion_ConverterList;
 class IFR_ShortInfo;
 class IFR_ConnectionItem;
 struct IFRPacket_LongDescriptor;
 
-/**
- * Offset of the part kind from the part header.
- */
-const int IFRPacket_Part_PartKind_O   = 0;
-
-/**
- * Offset of the part attributes from the part header.
- */
-const int IFRPacket_Part_Attributes_O = 1;
-
-/**
- * Offset of the part argument count from the part header.
- */
-const int IFRPacket_Part_ArgCount_O   = 2;
-
-/**
- * Offset of the part segment offset from the part header.
- */
-const int IFRPacket_Part_SegmOffs_O   = 4;
-
-/**
- * Offset of the part buffer length from the part header.
- */
-const int IFRPacket_Part_BufLen_O     = 8;
-
-/**
- * Offset of the part buffer size from the part header.
- */
-const int IFRPacket_Part_BufSize_O    = 12;
-
-/**
- * Offset of the part data from the part header.
- */
-const int IFRPacket_Part_Data_O       = 16;
-
 /**
  * Class to encapsulate part attributes.
- * @todo Refer to the respective <code>gsp/vsp</code> constants.
  */
 class IFRPacket_PartAttributes
 {
@@ -130,7 +95,21 @@ public:
         AbapIStream_C               =sp1pk_abap_istream,            
         AbapOStream_C               =sp1pk_abap_ostream,            
         AbapInfo_C                  =sp1pk_abap_info,
-        LongDemand_C                =sp1pk_long_demand
+        LongDemand_C                =sp1pk_long_demand,
+        MessageList_C               =sp1pk_message_list,
+#if IFR_HAS_VARDATA
+        VarDataShortInfo_C          =sp1pk_vardata_shortinfo,
+        VarData_C                   =sp1pk_vardata,
+#else
+        VarDataShortInfo_C          =sp1pk_nil,
+        VarData_C                   =sp1pk_nil,
+#endif
+
+#if IFR_HAS_FEATUREPART
+        Feature_C                   =sp1pk_feature
+#else
+        Feature_C                   =sp1pk_nil
+#endif
     };
 };
 
@@ -156,7 +135,7 @@ public:
      * @param part The <code>PIn_Part</code> where this part should be the overlay.
      * @param encoding the encoding to use for the part, default is raw ascii.
      */ 
-    inline IFRPacket_Part (const PIn_Part &part, IFR_StringEncoding encoding=IFR_StringEncodingAscii)
+    inline IFRPacket_Part (const PIn_Part &part, IFR_StringEncoding encoding)
     :PIn_Part (const_cast<tsp1_part*>(part.GetRawPart())),
      m_encoding(encoding)
     {
@@ -168,7 +147,7 @@ public:
      * @param rawpart The raw part.
      * @param encoding the encoding to use for the part, default is raw ascii.
      */
-    inline IFRPacket_Part(tsp1_part* rawpart, IFR_StringEncoding encoding=IFR_StringEncodingAscii)
+    inline IFRPacket_Part(tsp1_part* rawpart, IFR_StringEncoding encoding)
     :PIn_Part(rawpart),
      m_encoding(encoding)
     {
@@ -178,14 +157,42 @@ public:
      * Get the number of arguments this part has.
      * @return the number of part arguments, 0 if the part is not valid.
      */
-    IFR_Int2 partArguments() const;
+    inline IFR_Int2 getPartArguments() const
+    {
+        const tsp1_part * part=GetRawPart();
+        if(part) {
+            return part->sp1p_part_header().sp1p_arg_count;
+        } else {
+            return 0;
+        }
+    }
     
     /**
      * Returns the buffer length of this part.
      * @return the buffer length, 0 if the part is not valid.
      */
-    IFR_Int4 bufferLength() const;
+    inline IFR_Int4 getBufferLength() const
+    {
+        const tsp1_part * part=GetRawPart();
+        if(part) {
+            return part->sp1p_part_header().sp1p_buf_len;
+        } else {
+            return 0;
+        }
+    }
 
+    /**
+     * Sets the part's buffer length.
+     * @param newLength The new buffer length.
+     */
+    inline void setBufferLength(IFR_Int4 newLength)
+    {
+        tsp1_part * part=GetRawPart();
+        if(part) {
+            part->sp1p_part_header().sp1p_buf_len = newLength;
+        }
+    }
+    
     /**
      * Get the character encoding of the part.
      */
@@ -212,18 +219,19 @@ public:
      * Gets the part's contents as text. 
      * @param text the will be returned in this string.
      * @param memory_ok Flag to indicate out of memory condition.
+     * @param append Whether to append or assign the string, default is assign.
      * @return <code>IFR_OK</code> on success, 
      * <code>IFR_NO_DATA_FOUND</code> if the part is 
      * invalid or an out of memory error occurs.
      */
-    IFR_Retcode getText(IFR_String& text,IFR_Bool& memory_ok);
+    IFR_Retcode getText(IFR_String& text,IFR_Bool& memory_ok, IFR_Bool append=false);
     
     /**
      * Sets the 'first packet' part attribute.
      */
     inline void setFirstPacket()
     {
-        setPartAttribute(IFRPacket_PartAttributes::FirstPacket_C);
+        setPartAttribute(IFRPacket_PartAttributes::FirstPacket_C);    
     }
 
     /**
@@ -231,7 +239,7 @@ public:
      */
     inline void setLastPacket()
     {
-        setPartAttribute(IFRPacket_PartAttributes::LastPacket_C);
+        setPartAttribute(IFRPacket_PartAttributes::LastPacket_C);    
     }
 
     /**
@@ -239,14 +247,13 @@ public:
      */
     inline void setNextPacket()
     {
-        setPartAttribute(IFRPacket_PartAttributes::NextPacket_C);
-    }
+        setPartAttribute(IFRPacket_PartAttributes::NextPacket_C);    }
 
     /**
      * Gets the remaining bytes.
      * @return The number of bytes that remain in the part.
      */
-    inline IFR_size_t remainingBytes() const
+    inline IFR_size_t getRemainingBytes() const
     {
         return (IFR_size_t)BytesRemaining();
     }
@@ -254,6 +261,8 @@ public:
 
     /**
      * Gets the part kind.
+     * @return The part kind of this part, or @c IFRPacket_PartKind::Nil_C if the part is not
+     *         valid.
      */
     inline IFRPacket_PartKind::PartKind getPartKind() 
     {
@@ -264,6 +273,57 @@ public:
         }
     }
 
+    /**
+     * Determines if last part was set in part attributes
+     * @return <code>true</code> if last part attribute has been set for this part
+     *         <code>false</code> otherwise
+     */
+    inline IFR_Bool isLastPacket() const
+    {
+        const tsp1_part * part = GetRawPart();
+        if(part == 0) {
+            return false;
+        } else {
+            char *p =(char *) ((const char *)&(part->sp1p_part_header()));
+            ++p;
+            return (*p & (char)IFRPacket_PartAttributes::LastPacket_C) != 0;
+        }
+    }
+
+    /**
+     * Determines if first part was set in part attributes
+     * @return <code>true</code> if first part attribute has been set for this part
+     *         <code>false</code> otherwise
+     */
+    inline IFR_Bool isFirstPacket() const
+    {
+        const tsp1_part * part = GetRawPart();
+        if(part == 0) {
+            return false;
+        } else {
+            char *p =(char *) ((const char *)&(part->sp1p_part_header()));
+            ++p;
+            return (*p & (char)IFRPacket_PartAttributes::FirstPacket_C) != 0;
+        }
+    }
+
+    /**
+     * Sets the number of arguments in the data part. This is
+     * the number of rows in a mass command, or the number of
+     * LONG descriptors in a LONG data part.
+     * @param argcount The number of arguments.
+     * @return <code>IFR_OK</code> if correct, <code>IFR_NOT_OK</code> if the argument was 
+     * less or equal 0.
+     */
+    inline IFR_Retcode setArgCount(IFR_Int2 argcount)
+    {
+        if(argcount <= 0) {
+            return IFR_NOT_OK;
+        }
+        GetRawPart()->sp1p_part_header().sp1p_arg_count = argcount;
+        return IFR_OK;
+    }
+    
 protected:
     /**
      * Gets an 8 bit integer from the specified offset.
@@ -274,23 +334,21 @@ protected:
     /**
      * Gets a 16 bit integer from the specified offset.
      * @param position position within the part.
-     * @todo take honor of swap
      */
     IFR_Int2 getInt2(IFR_Int4 position) const;
 
     /**
      * Gets a 32 bit integer from the specified offset.
      * @param position position within the part.
-     * @todo take honor of swap
      */
     IFR_Int4 getInt4(IFR_Int4 position) const;
-
-
-
+    
+   
+    IFR_StringEncoding m_encoding; //!< encoding for text retrieved/set into this part
 private:
     void setPartAttribute(int attribute);
     
-    IFR_StringEncoding m_encoding; //!< encoding for text retrieved/set into this part
+    
 };
 
 
@@ -298,10 +356,6 @@ private:
 // various classes for different kinds of parts
 //----------------------------------------------------------------------
 
-/**
- * An error text part, which has no special properties. 
- */
-typedef IFRPacket_Part IFRPacket_ErrorTextPart; 
 
 /**
  * A result table name part, which has no special properties.
@@ -309,6 +363,34 @@ typedef IFRPacket_Part IFRPacket_ErrorTextPart;
 typedef IFRPacket_Part IFRPacket_ResultTableNamePart; 
 
 
+/**
+ * An error text part. You can add an error text.
+ */
+class IFRPacket_ErrorTextPart
+    : public IFRPacket_Part
+{
+public:
+    /**
+     * Constructor.
+     */
+    inline IFRPacket_ErrorTextPart() 
+    {}
+
+    /**
+     * Constructor. Overlay on <code>PIn_Part</code>
+     */
+    inline IFRPacket_ErrorTextPart(const PIn_Part& cpart, IFR_StringEncoding encoding)
+    :IFRPacket_Part(cpart, encoding)
+    {}
+    
+    /**
+     * Set the error text.
+     * @param error The error that contains the text.
+     */
+    IFR_Retcode setErrorText(IFR_ErrorHndl &error);
+    
+};
+
 
 /**
  * A parse id part. Contains a parse id.
@@ -326,8 +408,8 @@ public:
     /**
      * Constructor. Overlay on <code>PIn_Part</code>
      */
-    inline IFRPacket_ParseIDPart(const PIn_Part& cpart)
-    :IFRPacket_Part(cpart)
+    inline IFRPacket_ParseIDPart(const PIn_Part& cpart, IFR_StringEncoding encoding)
+    :IFRPacket_Part(cpart, encoding)
     {}
     
     /**
@@ -339,11 +421,11 @@ public:
     IFR_Retcode getParseID(IFR_ParseID& parseid) const;
     
     /**
-     * Sets the parse id.
+     * Adds a parse id.
      * @param parseid the parse id to be set.
      * @return <code>IFR_OK</code> on success.
      */
-    IFR_Retcode setParseID(IFR_ParseID& parseid);
+    IFR_Retcode addParseID(IFR_ParseID& parseid);
 };
 
 /**
@@ -367,7 +449,13 @@ public:
      */
     IFR_Retcode getShortInfos(IFR_ShortInfo* shortinfo);
 
-    IFR_Retcode parseShortFields(IFRConversion_Converter**& CnvCon, IFR_ConnectionItem& clink);
+    /**
+     * Parses the short fields, hereby creating converter instances for
+     * all parameter/columns.
+     * @param convList Vector of converter instances.
+     * @param clink Link to connection.
+     */
+    IFR_Retcode parseShortFields(IFRConversion_ConverterList& convList,  IFR_ConnectionItem& clink);
 
     /**
      * Gets the number of short info fields.
@@ -379,6 +467,24 @@ public:
 
 };
 
+/**
+ * Part for variable input short infos.
+ */
+class IFRPacket_VarDataShortInfoPart 
+    : public IFRPacket_ShortInfoPart
+{
+public:
+    inline IFRPacket_VarDataShortInfoPart() 
+    {}
+
+    inline IFRPacket_VarDataShortInfoPart(const IFRPacket_Part& part)
+    :IFRPacket_ShortInfoPart(part)
+    {}
+
+    IFR_Retcode parseShortFields(IFRConversion_ConverterList& convList,  IFR_ConnectionItem& clink);
+};
+
+
 
 /**
  * A command part makes it possible to set and add text (the command).
@@ -402,9 +508,6 @@ public:
      */
     IFR_Retcode setText(const IFR_String& text, IFR_ErrorHndl &error);
     IFR_Retcode addText(const IFR_String& text, IFR_ErrorHndl &error);
-    // IFR_Retcode addText(const char *buffer, const IFR_UInt4 length=(IFR_UInt4) IFR_NTS, 
-    //  const IFR_StringEncoding encoding=IFR_StringEncodingAscii);
-    
 };
 
 /**
@@ -416,19 +519,22 @@ class IFRPacket_DataPart
 {
 public:
     inline IFRPacket_DataPart() 
-      :m_currentrecord(0),
-       m_recordsize(0),
-       m_extent(0),
-       m_massextent(0)
+    :m_currentrecord(0),
+     m_recordsize(0),
+     m_extent(0),
+     m_massextent(0),
+     m_variableinput(false)
     {}
 
-    inline IFRPacket_DataPart(const PIn_Part& cpart)
-    :IFRPacket_Part(cpart),
+    inline IFRPacket_DataPart(const PIn_Part& cpart, IFR_StringEncoding encoding)
+    :IFRPacket_Part(cpart, encoding),
      m_currentrecord(0),
      m_recordsize(0),
      m_extent(0),
-     m_massextent(0)
-    {}
+     m_massextent(0),
+     m_variableinput(false)
+    {
+    }
 
     /**
      * Creates a data part on a <code>tsp1_part</code> structure that is
@@ -437,36 +543,38 @@ public:
      * @param currentrecord The current record in the part (start with 0).
      * @param recordsize The record size of one row.
      */
-    inline IFRPacket_DataPart(tsp1_part *rawpart, IFR_Int2 currentrecord, IFR_Int2 recordsize)
-    :IFRPacket_Part(rawpart),
+    inline IFRPacket_DataPart(tsp1_part *rawpart, IFR_StringEncoding encoding, IFR_Int2 currentrecord, IFR_Int2 recordsize)
+    :IFRPacket_Part(rawpart, encoding),
      m_currentrecord(currentrecord),
      m_recordsize(recordsize),
      m_extent(0),
-     m_massextent(0)
+     m_massextent(0),
+     m_variableinput(false)
     {}
     
-    /**
-     * Adds a parameter with padding and defined byte. Also, NULL values are honored. The
-     * parameter is added at the current record position.
-     * @param buffer the data
-     * @param position the position of the data (defined byte will be written at <code>position -1</code>.
-     * @param length the length of the data
-     * @param iolength the length of the data as specified in the parameter info
-     * @param definedbytepadding the char used for defined byte and for padding
-     */
-    void addParameter(const void *buffer,
-                      IFR_Int4 position,
-                      IFR_Int4 length,
-                      IFR_Int4 iolength,
-                      char definedbytepadding);
 
+    /**
+     * Add binary data 
+     * @param buffer the data
+     * @param length the length of the data
+     * @param shortinfo the shorfield info of the parameter
+     * @return <code>IFR_OK</code> if the parameter is added, 
+     * <code>IFR_DATA_TRUNC</code> if the argument was added but truncated
+     * or <code>IFR_NOT_OK</code> if the given buffer argument can't convert
+     * to the datapart.
+     */
+    IFR_Retcode addBinaryParameter(const void *buffer,
+                                   IFR_Int4 length,
+                                   IFR_ShortInfo &shortinfo);
+    
     /**
      * Adds a parameter with the encoding of the shortfield info and the
      * defined byte. The argument is padded if neccessary. Also, NULL values 
      * are honored. The parameter is added at the current record position.
      * @param buffer the data
      * @param length the length of the data
-     * @param srcEncoding the encoding of the data
+     * @param srcEncoding The encoding of the data
+     * @param targetEncoding The target encoding.
      * @param shortinfo the shorfield info of the parameter
      * @return <code>IFR_OK</code> if the parameter is added, 
      * <code>IFR_DATA_TRUNC</code> if the argument was added but truncated
@@ -476,6 +584,7 @@ public:
     IFR_Retcode addParameter(const void *buffer,
                              IFR_Int4 length,
                              IFR_StringEncoding srcEncoding,
+                             IFR_StringEncoding targetEncoding,
                              IFR_ShortInfo &shortinfo);
 
     /**
@@ -494,9 +603,30 @@ public:
     IFR_Retcode appendToParameter(const void *buffer,
                                   IFR_Int4 length,
                                   IFR_StringEncoding srcEncoding,
+                                  IFR_StringEncoding targetEncoding,
                                   IFR_ShortInfo &shortinfo,
                                   IFR_Length&    offset);
+
+    IFR_Retcode appendBinaryToParameter(const void *buffer,
+                                        IFR_Int4 length,
+                                        IFR_ShortInfo &shortinfo,
+                                        IFR_Length&    offset);
     
+    
+    /**
+     * Adds a @c NULL value.
+     * @param shortinfo Parameter info.
+     */
+    IFR_Retcode addNullInput(IFR_ShortInfo& shortinfo);
+    
+    /**
+     * Adds a @c DEFAULT value.
+     * @param shortinfo Parameter info.
+     */
+    IFR_Retcode addDefaultInput(IFR_ShortInfo& shortinfo);
+
+    IFR_Retcode finishData(IFR_Length length, IFR_ShortInfo& shortinfo);
+private:
     /**
      * Adds plain data at the specified position. The data must
      * include the defined byte.
@@ -520,24 +650,8 @@ public:
     void addData(IFR_Int1 definedbyte,
                  IFR_Int4 position,
                  IFR_Int4 iolength);
-    
-    /**
-     * Sets the number of arguments in the data part. This is
-     * the number of rows in a mass command, or the number of
-     * LONG descriptors in a LONG data part.
-     * @param argcount The number of arguments.
-     * @return <code>IFR_OK</code> if correct, <code>IFR_NOT_OK</code> if the argument was 
-     * less or equal 0.
-     */
-    inline IFR_Retcode setArgCount(IFR_Int2 argcount)
-    {
-        if(argcount <= 0) {
-            return IFR_NOT_OK;
-        }
-        GetRawPart()->sp1p_part_header().sp1p_arg_count = argcount;
-        return IFR_OK;
-    }
 
+public:
     /**
      * Gets the data in this part at this position in the current
      * row. The absolute position looked up is <code>currentrecord *
@@ -546,19 +660,28 @@ public:
      * @return A pointer to the data, positioned at the <i>defined byte</i>
      *         of the data entry.
      */
-    inline char* getParameterData(IFR_Int4 position)
+    inline char* getOutputData(IFR_Int4 position)
     {
         return (char*)GetReadData(0) + m_massextent + position;
     }
 
-
-    /**
-     * @brief Sets the mass extent
-     * @param val The value to be set
-     */
-    inline void setMassExtent (IFR_Int4 val)
+    
+    inline char* getInputData(IFR_ShortInfo& shortinfo) 
     {
-      m_massextent = val;
+        if(m_variableinput) {
+            return (char * )GetReadData(0) + GetRawPart()->sp1p_buf_len() + (shortinfo.iolength > 251 ? 3 : 1);
+        }  else {
+            return (char *)GetReadData(0) + m_massextent + shortinfo.pos.bufpos;
+        }
+    }
+    
+    /**
+     * Set the start position of the current row (0-based).
+     * @param val The start position.
+     */
+    inline void setCurrentRowStart(IFR_Int4 val)
+    {
+        m_massextent = val;
     }
 
     
@@ -610,7 +733,10 @@ public:
     /**
      * @brief Moves to the next record in data part (for mass cmds)
      */
-    void moveRecordBase ();
+    inline void moveRecordBase()
+    {
+        m_massextent = GetRawPart()->sp1p_buf_len();
+    }
 
 
     /**
@@ -653,11 +779,12 @@ public:
     void setStreamValMode(char *longdescriptor, IFR_Int1 valmode);
     
     /**
-     * Clears the offsets. Both the extent and the mass command extent are set to 0.
+     * Clears the offsets. The start of the current row is set to zero.
      */
     inline void clearOffsets()
     {
-        m_extent=m_massextent=0;
+        m_currentrecord = 0;
+        m_massextent    = 0;
     }
 
     /**
@@ -667,30 +794,53 @@ public:
      * that means the extent passed to this method is allocated in the datapart.
      * @param extent The new extent.
      */
-    inline void setExtent(IFR_Int4 extent)
+    inline void extendRow(IFR_Int4 extent)
     {
-        m_extent = m_extent > extent ? m_extent : extent;
-        GetRawPart()->sp1p_buf_len() = MAX(GetRawPart()->sp1p_buf_len(), m_massextent + m_extent);
+        if(!m_variableinput) {
+            GetRawPart()->sp1p_buf_len() = MAX(GetRawPart()->sp1p_buf_len(), m_massextent + extent);
+        }
     }
 
-
     /**
-     * Determines if last part was set in part attributes
-     * @return <code>true</code> if last part attribute has been set for this datapart
-     *         <code>false</code> otherwise
+     * Sets the input parameter count, necessary for each row on variable
+     * input.
+     * @param inputparametercount The number of input parameters.
      */
-    inline bool wasLastPart ()
+    inline void setInputParameterCount(IFR_Int2 inputparametercount)
     {
-        return (*((IFR_Byte *) GetRawHeader () + IFRPacket_Part_Attributes_O) 
-                & IFRPacket_PartAttributes::LastPacket_C) != 0;        
+        if(m_variableinput) {
+            char *p = GetRawPart()->sp1p_buf().asCharp() + m_massextent;
+            *p     = (char) (inputparametercount >> 8);
+            *(p+1) = (char) (inputparametercount & 0xFF);
+            if((m_massextent + 2) > GetRawPart()->sp1p_buf_len()) {
+                GetRawPart()->sp1p_buf_len() = m_massextent + 2;
+            }
+        }
     }
 
+    inline IFR_Bool hasVariableInput()
+    {
+        return m_variableinput;
+    }
+
+    inline void setVariableInput(IFR_Bool variableinput)
+    {
+        m_variableinput = variableinput;
+    }
+
+    /**
+     * Adds a LONG descriptor. The long descriptor is added, the argument count 
+     * is incremented by one.
+     */
+    IFR_Retcode addDescriptor(IFRPacket_LongDescriptor& longdesc);
+    
 private:
     IFR_Int2 m_currentrecord;     //!< The index of the current record within the data part.
     IFR_Int2 m_recordsize;        //!< The size of one row.
 protected:
-    IFR_Int4 m_extent;            //!< The actual position in the current record.
-    IFR_Int4 m_massextent;        //!< The actual postion in the data part
+    IFR_Int4 m_extent;            //!< End of current input parameter that is filled in.
+    IFR_Int4 m_massextent;        //!< The current row start in the data part. 
+    IFR_Bool m_variableinput;     //!< Whether this is a vardata part.
 };
 
 /**
@@ -721,10 +871,6 @@ public:
      */
     IFR_Retcode closePutval();
 
-    /**
-     * Adds a LONG descriptor.
-     */
-    IFR_Retcode addDescriptor(IFRPacket_LongDescriptor& longdesc);
 };
 
 /**
@@ -736,15 +882,18 @@ class IFRPacket_LongDemandPart
 {
 public:
     inline IFRPacket_LongDemandPart() 
-    :IFRPacket_DataPart()
+    :IFRPacket_DataPart(),
+     m_extent(0)
     {}
 
     inline IFRPacket_LongDemandPart(const IFRPacket_DataPart& datapart)
-    :IFRPacket_DataPart(datapart)
+    :IFRPacket_DataPart(datapart),
+     m_extent(0)
     {}
 
     inline IFRPacket_LongDemandPart(const IFRPacket_LongDemandPart& datapart)
-    :IFRPacket_DataPart(datapart)
+    :IFRPacket_DataPart(datapart),
+     m_extent(datapart.m_extent)
     {}
 
     /**
@@ -760,6 +909,8 @@ public:
     IFR_Retcode addPosParams (const IFR_Bool immediately, 
                               const IFR_Int4 position, 
                               const IFR_UInt4 length);
+private:
+    IFR_Int4 m_extent; //!< to be deleted.
 };
 
 /**
@@ -777,8 +928,8 @@ public:
      * Constructor. Overlay on <code>PIn_Part</code>
      * @param cpart the part structure this part will reuse.
      */
-    inline IFRPacket_SessionInfoPart(const PIn_Part& cpart)
-    :IFRPacket_Part(cpart)
+    inline IFRPacket_SessionInfoPart(const PIn_Part& cpart, IFR_StringEncoding encoding)
+    :IFRPacket_Part(cpart, encoding)
     {}
     
     /**
@@ -808,8 +959,8 @@ public:
     /**
      * Constructor. Overlay on <code>PIn_Part</code>
      */
-    inline IFRPacket_ResultCountPart(const PIn_Part& cpart)
-    :IFRPacket_Part(cpart)
+    inline IFRPacket_ResultCountPart(const PIn_Part& cpart, IFR_StringEncoding encoding)
+    :IFRPacket_Part(cpart, encoding)
     {}
     
     /**
@@ -893,8 +1044,8 @@ public:
     /**
      * Constructor. Overlay on <code>PIn_Part</code>
      */
-    inline IFRPacket_ColumnNamesPart(const PIn_Part& cpart)
-    :IFRPacket_Part(cpart)
+    inline IFRPacket_ColumnNamesPart(const PIn_Part& cpart, IFR_StringEncoding encoding)
+    :IFRPacket_Part(cpart, encoding)
     {}
 
     /**
@@ -928,8 +1079,8 @@ public:
     /**
      * Constructor. Overlay on <code>PIn_Part</code>
      */
-    inline IFRPacket_ApplParamPart(const PIn_Part& cpart)
-    :IFRPacket_Part(cpart)
+    inline IFRPacket_ApplParamPart(const PIn_Part& cpart, IFR_StringEncoding encoding)
+    :IFRPacket_Part(cpart, encoding)
     {}
     
     /**
@@ -943,7 +1094,157 @@ public:
                      IFR_Int1    frac,
                      IFR_size_t  length);
     
-friend IFR_TraceStream& operator << (IFR_TraceStream& s, const enum IFRPacket_PartKind::PartKind p );
+    friend IFR_TraceStream& operator << (IFR_TraceStream& s, const enum IFRPacket_PartKind::PartKind p );
 };
 
+/**
+ * An 'ABAP Input Stream' part.
+ */
+class IFRPacket_AbapIStreamPart 
+    : public IFRPacket_Part
+{
+public:
+    inline IFRPacket_AbapIStreamPart()
+    {}
+
+    inline IFRPacket_AbapIStreamPart(const PIn_Part& cpart, IFR_StringEncoding encoding)
+    :IFRPacket_Part(cpart, encoding)
+    {}
+
+    inline IFR_Retcode getABAPTabId(IFR_Int4& tabid)
+    {
+        if(isValid()) {
+            memcpy(&tabid, GetReadData(), sizeof(IFR_Int4));
+            return IFR_OK;
+        } else {
+            return IFR_NO_DATA_FOUND;
+        }
+    }
+    
+    inline void * getMask()
+    {
+        if(Length() <= sizeof(IFR_Int4)) {
+            return 0;
+        } else {
+            return (void *) (GetReadData() + sizeof(IFR_Int4)); 
+        }
+    }
+    
+    inline IFR_Length getMaskLength()
+    {
+        return Length() <= sizeof(IFR_Int4) ? 0 : Length() - sizeof(IFR_Int4);
+    }
+    
+};
+
+/**
+ * An 'ABAP Output Stream' part.
+ */
+class IFRPacket_AbapOStreamPart 
+    : public IFRPacket_Part
+{
+public:
+    inline IFRPacket_AbapOStreamPart()
+    {}
+
+    inline IFRPacket_AbapOStreamPart(const PIn_Part& cpart, IFR_StringEncoding encoding)
+    :IFRPacket_Part(cpart, encoding)
+    {}
+
+    inline IFR_Retcode getABAPTabId(IFR_Int4& tabid)
+    {
+        if(isValid()) {
+            memcpy(&tabid, GetReadData(), sizeof(IFR_Int4));
+            return IFR_OK;
+        } else {
+            return IFR_NO_DATA_FOUND;
+        }
+    }
+
+    inline void * getMask()
+    {
+        if(Length() <= sizeof(IFR_Int4)) {
+            return 0;
+        } else {
+            return (void *) (GetReadData() + sizeof(IFR_Int4)); 
+        }
+    }
+
+    inline IFR_Length getMaskLength()
+    {
+        return Length() <= sizeof(IFR_Int4) ? 0 : Length() - sizeof(IFR_Int4);
+    }
+};
+
+/**
+ * A command info part makes it possible to set and add a command info.
+ */
+class IFRPacket_CommandInfoPart
+    : public IFRPacket_Part
+{
+public:
+    inline IFRPacket_CommandInfoPart() 
+    {}
+
+    inline IFRPacket_CommandInfoPart(const PIn_Part& part, IFR_StringEncoding encoding)
+    :IFRPacket_Part(part, encoding)
+    {}
+    
+    /**
+     * Set the command info text.
+     * @param text the command info text.
+     * @param lineNumber the command info line number text.
+     * @param clink Connection link for error reporting.
+     * @return <code>IFR_OK</code> on success.
+     */
+    IFR_Retcode addCommandInfo(const IFR_String& text, IFR_Int4 lineNumber, IFR_ErrorHndl &error);
+};
+
+
+struct IFRPacket_Feature
+{
+    enum Feature {
+#if IFR_HAS_FEATUREPART
+        MultipleDropParseID_C   = sp1f_multiple_drop_parseid,
+        SpaceOption_C           = sp1f_space_option,
+        VariableInput_C         = sp1f_variable_input,
+        OptimizedStreams_C      = sp1f_optimized_streams,
+        CheckScrollableOption_C = sp1f_check_scrollableoption
+#else
+        NoFeature_C
+#endif
+    };
+};
+
+
+/**
+ * A feature request/reply part.
+ */
+class IFRPacket_FeaturePart
+    : public IFRPacket_Part
+{
+public:
+    /**
+     * Default constructor.
+     */
+    inline IFRPacket_FeaturePart()
+    {}
+    
+    /**
+     * Requests a feature. If the feature is already reqeuested, nothing
+     * happens, else the part is extended so that the feature is requested.
+     */
+    IFR_Retcode requestFeature(IFRPacket_Feature::Feature feature);
+
+    /**
+     * Access the data for reading.
+     * @return Pointer to data buffer.
+     */
+    inline char *getData()
+    {
+        return GetRawPart()->sp1p_buf().asCharp();
+    }
+};
+
+
 #endif // IFRPACKET_PART_H
