@@ -46,7 +46,7 @@ IFRPacket_RequestPacket::IFRPacket_RequestPacket (tsp1_packet *packetptr,
                                                   const char *client_version,
                                                   IFRUtil_RuntimeItem& runtimeitem,
                                                   IFR_Bool& memory_ok)
-:PIn_RequestPacket(packetptr, size, unicode),
+:PIn_RequestPacket(packetptr, size, unicode, client_application, client_version),
  IFRUtil_RuntimeItem(runtimeitem),
  m_CurrentSqlMode (sqlmode),
  m_hasexclusivelock(false),
@@ -59,12 +59,9 @@ IFRPacket_RequestPacket::IFRPacket_RequestPacket (tsp1_packet *packetptr,
     DBUG_PRINT(sqlmode);
     DBUG_PRINT(client_application);
     DBUG_PRINT(client_version); 
-    if(packetptr && client_application) {
-        initVersion(client_application, client_version);
-    }
-    m_lock=new (allocator) IFRPacket_RootLock(runtime, allocator, memory_ok);
+    m_lock=new IFR_ALLOCATOR(allocator) IFRPacket_RootLock(runtime, allocator, memory_ok);
     if(IFR_MEM_NOT_OK) {
-        delete m_lock;
+        IFRUtil_Delete(m_lock, allocator);
         m_lock = 0;
     } else if(m_lock==0) {
         IFR_MEM_FAIL;
@@ -81,7 +78,7 @@ IFRPacket_RequestPacket::IFRPacket_RequestPacket(tsp1_packet *packetptr,
                                                  IFRPacket_RequestPacket::PacketType packettype,
                                                  IFRPacket_Lock *lockinstance,
                                                  IFRUtil_RuntimeItem& runtimeitem)
-:PIn_RequestPacket(packetptr, size, unicode),
+:PIn_RequestPacket(packetptr, size, unicode, client_application, client_version),
  IFRUtil_RuntimeItem(runtimeitem),
  m_CurrentSqlMode (sqlmode),
  m_hasexclusivelock(false),
@@ -94,11 +91,6 @@ IFRPacket_RequestPacket::IFRPacket_RequestPacket(tsp1_packet *packetptr,
     DBUG_PRINT(client_application);
     DBUG_PRINT(client_version);
     DBUG_PRINT(packettype);
-
-  
-    if(packetptr && client_application) {
-        initVersion(client_application, client_version);
-    }
     m_lock=lockinstance;
     if(packettype==Dynamic_C) {
         m_lock->acquireExclusiveLock();
@@ -130,10 +122,6 @@ IFRPacket_RequestPacket(const IFRPacket_RequestPacket& requestpacket)
     case (Root_Ref_C):
         m_packettype=Root_Ref_C;
         break;
-    case (Synthetic_C):
-    case (Synthetic_Ref_C):
-        m_packettype=Synthetic_Ref_C;
-        break;
     case (Dynamic_C):
         m_packettype=Dynamic_C;
     };
@@ -175,27 +163,20 @@ void
 IFRPacket_RequestPacket::destroyPacketData()
 {
     DBUG_METHOD_ENTER(IFRPacket_RequestPacket, destroyPacketData);
-
-    if(m_packettype==Root_C || m_packettype==Synthetic_C) {
+    if(m_packettype==Root_C) {
         IFRUtil_Delete(m_lock, allocator);
     } else {
         if(m_hasexclusivelock) {
             m_lock->releaseExclusiveLock();
         }
     }
-    if(m_packettype==Synthetic_C) {
-        char * to_delete = (char*)(this->rawPacket);
-        allocator.Deallocate(to_delete);
-    }
     if(m_packettype==Dynamic_C) {
         IFRPacket_DynamicLock *dlock=(IFRPacket_DynamicLock*)m_lock;
         dlock->lock();
         if(m_lock->isLocked()) {
             dlock->unlock();
         } else {
-            char * to_delete = (char*)(this->rawPacket);
-            
-            allocator.Deallocate(to_delete);
+            dlock->freePacket(this->rawPacket);
             IFRUtil_Delete(m_lock, allocator);
         }
     }
@@ -214,7 +195,7 @@ IFRPacket_RequestPacket::operator =(const IFRPacket_RequestPacket& rhs)
 
 //----------------------------------------------------------------------
 IFR_Retcode
-IFRPacket_RequestPacket::assign(const IFRPacket_RequestPacket& rhs)
+IFRPacket_RequestPacket::assign(const IFRPacket_RequestPacket& rhs, IFR_Int4* in_use)
 {
     if(this == &rhs) {
         // noop for self-assignment, this may actually happen
@@ -233,16 +214,23 @@ IFRPacket_RequestPacket::assign(const IFRPacket_RequestPacket& rhs)
     case (Root_Ref_C):
         m_packettype=Root_Ref_C;
         break;
-    case (Synthetic_C):
-    case (Synthetic_Ref_C):
-        m_packettype=Synthetic_Ref_C;
-        break;
     case (Dynamic_C):
         m_packettype=Dynamic_C;
     };
 
     if(m_lock) {
         m_lock->acquireExclusiveLock();
+
+        // in an mt scenarion we can only have aquired the
+        // exclusive lock when 'in_use' is not longer 'in_use'.
+        if(rhs.m_packettype==Root_C || rhs.m_packettype==Root_Ref_C) {
+            if(in_use && *in_use==1) {
+                m_lock->releaseExclusiveLock();
+                m_hasexclusivelock = false;
+                this->rawPacket = 0;
+                return IFR_NO_DATA_FOUND;
+            }
+        }
         m_hasexclusivelock=true;
     } else {
         m_hasexclusivelock=false;
@@ -252,16 +240,6 @@ IFRPacket_RequestPacket::assign(const IFRPacket_RequestPacket& rhs)
 }
 
 
-//----------------------------------------------------------------------
-void
-IFRPacket_RequestPacket::initVersion(const char *application,
-                                     const char *appversion)
-{
-    DBUG_METHOD_ENTER(IFRPacket_RequestPacket, initVersion);
-    PIn_RequestPacket::InitVersion(application,
-                                      appversion==0?"70404":appversion);
-}
-
 //----------------------------------------------------------------------
 const IFR_StringEncoding
 IFRPacket_RequestPacket::getEncoding() const
@@ -296,16 +274,22 @@ IFRPacket_RequestPacket::setEncoding(const IFR_StringEncoding encoding) const
       return old_encoding;
     tsp1_packet_header *header = (tsp1_packet_header *) GetRawHeader();
     if (header) {
-      if (encoding == IFR_StringEncodingAscii) 
-        header->sp1h_mess_code = csp_ascii;
-      else if (encoding == IFR_StringEncodingUCS2)
-        header->sp1h_mess_code = csp_unicode;
-      else if (encoding == IFR_StringEncodingUCS2Swapped)
-        header->sp1h_mess_code = csp_unicode_swap;
-      else if (encoding == IFR_StringEncodingUTF8)
-        header->sp1h_mess_code = csp_utf8;
-      else 
-        DBUG_RETURN((IFR_StringEncoding)0);
+        switch(encoding) {
+        case IFR_StringEncodingAscii:
+            header->sp1h_mess_code = csp_ascii;  
+            break;
+        case IFR_StringEncodingUCS2:
+            header->sp1h_mess_code = csp_unicode;
+            break;
+        case IFR_StringEncodingUCS2Swapped:
+            header->sp1h_mess_code = csp_unicode_swap;
+            break;
+        case IFR_StringEncodingUTF8:
+            header->sp1h_mess_code = csp_utf8;
+            break;
+        default:
+            DBUG_RETURN(IFR_StringEncodingUnknown);
+        }
     }
     DBUG_RETURN(old_encoding);
 }
@@ -317,15 +301,6 @@ IFRPacket_RequestPacket::isSwapped() const
     return header->sp1h_mess_swap == sw_full_swapped;
 }
 
-
-int IFRPacket_RequestPacket::switchSqlMode (int newMode)
-{
-  int result = this->m_CurrentSqlMode;
-
-  this->m_CurrentSqlMode = newMode;
-  return result;
-}
-
 IFR_Byte *IFRPacket_RequestPacket::getFirstSegmentPtr ()
 {
   return ((IFR_Byte *) &rawPacket->sp1_segm ());
@@ -370,13 +345,6 @@ void IFRPacket_RequestPacket::setParseAgain ()
   memset(hdr, 1, 1);
 }
 
-//----------------------------------------------------------------------
-PIn_RequestSegment
-IFRPacket_RequestPacket::addSegment(int messagetype, IFR_Bool parseagain)
-{
-    return addSegment(messagetype, parseagain, m_CurrentSqlMode);
-}
-
 //----------------------------------------------------------------------
 PIn_RequestSegment
 IFRPacket_RequestPacket::addSegment(int messagetype, IFR_Bool parseagain, int sqlmode)
@@ -438,7 +406,7 @@ IFR_Retcode
 IFRPacket_RequestPacket::replaceParseIDAdjustData(IFR_ParseID* parseid,
                                                   IFR_size_t   length,
                                                   IFRUtil_Vector<IFR_Int4>& old_bufpos,
-                                                  IFRConversion_Converter **shortinfo,
+                                                  IFRConversion_ConverterList& shortinfo,
                                                   IFR_ConnectionItem& clink)
 {
     DBUG_METHOD_ENTER(IFRPacket_RequestPacket, replaceParseIDAdjustData);
@@ -507,7 +475,8 @@ IFRPacket_RequestPacket::replaceParseIDAdjustData(IFR_ParseID* parseid,
     // now we have the tsp1_part, fine ...
     char     * data     = part->sp1p_buf().asCharp();
     IFR_Int2   argCount = part->sp1p_part_header().sp1p_arg_count;
-    char     *tmp_buffer=new (allocator) char[rowsize];
+    char     *tmp_buffer= (char *) alloca(rowsize);
+
    
     for(IFR_Int2 i=0; i<argCount; ++i) {
         // evacuate the data
@@ -530,8 +499,6 @@ IFRPacket_RequestPacket::replaceParseIDAdjustData(IFR_ParseID* parseid,
         }
         data += offset; 
     }
-
-    IFRUtil_DeleteArray(tmp_buffer, rowsize, allocator);
     DBUG_RETURN(IFR_OK);
 }
 
@@ -548,12 +515,6 @@ operator << (IFR_TraceStream& s,
     case IFRPacket_RequestPacket::Root_Ref_C:
         s << "ROOT REF";
         break;
-    case IFRPacket_RequestPacket::Synthetic_C:
-        s << "SYNTHETIC";
-        break;
-    case IFRPacket_RequestPacket::Synthetic_Ref_C:
-        s << "SYNTHETIC REF";
-        break;
     case IFRPacket_RequestPacket::Dynamic_C:
         s << "DYNAMIC";
         break;
