@@ -255,7 +255,7 @@ private:
      
     Node* m_get_node() 
     { 
-        Node *np=(Node*)m_node_allocator.Allocate(sizeof(Node));
+        Node *np=(Node*)IFR_ALLOCATOR(m_node_allocator).Allocate(sizeof(Node));
         new (np) Node();
         return np;
     }
@@ -263,7 +263,7 @@ private:
     void m_put_node(Node* p) 
     { 
         IFRUtil_Destroy(p);
-        m_node_allocator.Deallocate(p);
+        IFR_ALLOCATOR(m_node_allocator).Deallocate(p);
     }
 
 private:
