@@ -38,6 +38,7 @@
 #include "Interfaces/Runtime/Util/IFRUtil_SQLNumeric.h"
 
 #include <stdlib.h>
+#include <ctype.h>
 
 //----------------------------------------------------------------------
 // Converts the SQL Numeric integer number into a string 
@@ -81,8 +82,8 @@ static void integer2string(const unsigned char *val,
         ;
     
     if(maxUsedByte==0 && val[0]==0) {
-        **str='0';
-        --(*str);
+        *(--(*str)) = '0';
+        return;
     }
 
 
@@ -144,8 +145,9 @@ IFRUtil_SQLNumeric::numericToNumber(const SQL_NUMERIC_STRUCT &number,
                                     IFR_Int4 digits)
 {
     // DBUG_METHOD_ENTER(IFRUtil_SQLNumeric, numericToNumber);
-    
     char tmpNum[SQL_MAX_NUMERIC_DIGITS];
+    // memset(tmpNum, 0, SQL_MAX_NUMERIC_DIGITS);
+    
     char *p=tmpNum + SQL_MAX_NUMERIC_DIGITS;
     integer2string(number.val, &p);
     
@@ -352,7 +354,116 @@ static void string2integer(unsigned char *string,
     }
 }
 
-
+//----------------------------------------------------------------------
+// XXX -- add means to supply scale and precision
+IFR_Retcode
+IFRUtil_SQLNumeric::asciiStringToNumeric(const char *buffer,
+                                         SQL_NUMERIC_STRUCT& number)
+{
+    memset(&number, 0, sizeof(SQL_NUMERIC_STRUCT));
+    number.sign = 1; // assume number > 0
+    number.precision = 38;
+    
+    char *p = (char *)buffer;
+    while(isspace(*p) && *p) {
+        ++p;
+    }
+    if(*p == '+') {
+        ++p;
+    } else if(*p == '-') {
+        number.sign = 0;
+        ++p;
+    }
+    if(!(*p)) {
+        return IFR_NOT_OK;
+    }
+    // skip leading zeros
+    while((*p) == '0') {
+        ++p;
+    }
+    if(!(*p)) {
+        return IFR_NOT_OK;
+    }
+    char digitbuffer[39];
+    int  digits_in_buffer = 0;
+    char *dotpos = strchr(p, '.');
+    if(dotpos != 0 && (dotpos - p) > 38) {
+        return IFR_OVERFLOW;
+    }
+    if(dotpos) {
+        memcpy(digitbuffer, p, dotpos - p);
+        digits_in_buffer = dotpos - p;
+        p = dotpos + 1;
+        int totaldigits = 0;
+        int significantdigits = 0;
+        while(*p) {
+            if(!isdigit(*p)) {
+                break;
+            }
+            if(*p == '0') {
+                ++totaldigits;
+            } else {
+                ++totaldigits;
+                significantdigits = totaldigits;
+            }
+            ++p;
+        }
+        
+        if(*p) {
+            while((*p) && isspace(*p)) {
+                ++p;
+            }
+            if(*p) {
+                return IFR_NOT_OK;
+            }
+        }
+        // truncate 
+        if(significantdigits > (38 - digits_in_buffer)) {
+            significantdigits = 38 - digits_in_buffer;
+        }
+        
+        number.scale = significantdigits;
+        memcpy(digitbuffer + digits_in_buffer, dotpos + 1, significantdigits);
+        digits_in_buffer += significantdigits;
+        
+    } else {
+        char *digitstart = p;
+        while(*p && isdigit(*p)) {
+            ++p;
+        }
+        if(p - digitstart > 38) {
+            return IFR_OVERFLOW;
+        }
+        memcpy(digitbuffer, digitstart, p-digitstart);
+        digits_in_buffer = p - digitstart;
+        while(*p) {
+            if(!isspace(*p)) {
+                return IFR_NOT_OK;
+            }
+            ++p;
+        }
+        number.scale=0;
+    }
+    IFR_Int4 startdigit = 0;
+    IFR_UInt4 tmpres = 0;
+    for(IFR_Int4 nbyte =0; nbyte < SQL_MAX_NUMERIC_LENGTH && (startdigit < digits_in_buffer || tmpres!=0); ++nbyte) {
+        IFR_Int4 pos = digits_in_buffer - 1 - startdigit;
+        for(IFR_Int4 ndigit = startdigit; ndigit < digits_in_buffer; ndigit ++) {
+            IFR_Int4 currentdigit;
+            if(!isdigit(digitbuffer[pos])) {
+                return IFR_NOT_OK;
+            }
+            currentdigit = digitbuffer[pos] - '0';
+            tmpres += currentdigit * string2integer_decfactors[ndigit][nbyte];
+            --pos;
+        }
+        number.val[nbyte] = tmpres & 0xFF;
+        tmpres >>= 8;
+        startdigit = string2integer_startdigits[nbyte + 1];
+    }
+    
+    return IFR_OK; 
+}
 
 
 //----------------------------------------------------------------------
