@@ -91,6 +91,37 @@ static unsigned char max_uint1_vdnnumber[20] = { 0xc3, 0x25, 0x50, 0x00, 0x00, 0
 
 #define ZERO_EXPONENT 0x80
 
+static inline void round_digits(IFR_Int4 *digits, 
+                                IFR_Int4  round_length,
+                                IFR_Int4  digit_length,
+                                IFR_Int4& exponent)
+{
+    if(digits[round_length] >= 5) {
+        for(IFR_Int4 i=round_length; i<digit_length; ++i) {
+            digits[i] = 0;
+        }
+        IFR_Int4 k = round_length - 1;
+        while(k >= 0) {
+            digits[k]++;
+            if(digits[k] == 10) {
+                digits[k] = 0;
+                --k;
+                continue;
+            }
+            break;
+        }
+        if(k == -1) {
+            digits[0]=1;
+            ++exponent;
+        }
+    } else {
+        for(IFR_Int4 i=round_length; i<digit_length; ++i) {
+            digits[i] = 0;
+        }
+    }
+}
+
+
 //----------------------------------------------------------------------
 // The individual conversions only differ in range checks,
 // and return types. Thus there is a template for them.
@@ -655,6 +686,16 @@ IFRUtil_VDNNumber::numberToStringAscii(unsigned char *number,
     IFR_Int4 exponent;
     if(number[0]==ZERO_EXPONENT) {
         *buffer='0';
+        if(fixed && precision > 0) {
+            IFR_Int4 filllength = precision;
+            if(filllength > IFR_Int4(bufferlength - 3)) {
+                filllength = (IFR_Int4)(bufferlength - 3);
+            }
+            *(buffer + 1) = '.';
+            memset(buffer + 2, '0', filllength);
+            *(buffer + 2 + filllength) = '\0';
+            return (IFR_size_t)precision > bufferlength - 3 ? IFR_DATA_TRUNC : IFR_OK;
+        }
         *(buffer+1)='\0';
         return IFR_OK;
     }
@@ -856,7 +897,7 @@ IFRUtil_VDNNumber::numberToStringUCS2(unsigned char *number,
     IFR_Int4 numberlength;
     IFR_Int4 exponent;
     if(number[0]==ZERO_EXPONENT) {
-        memcpy(buffer, "\00\0\0", 4);
+        memcpy(buffer, "\0000\0\0", 4);
         return IFR_OK;
     }
 
@@ -1252,7 +1293,7 @@ IFRUtil_VDNNumber::stringToNumber(char              *buffer,
 {
     if(encoding == IFR_StringEncodingAscii || encoding == IFR_StringEncodingUTF8) {
         return (stringToNumberAscii(buffer, bufferlength, nan, number, fixed, length, precision));
-    } else if(encoding == IFR_StringEncodingUCS2Swapped) {
+    } else if(encoding == IFR_StringEncodingUCS2) {
         if(bufferlength % 2) {
             return IFR_NOT_OK;
         }
@@ -1265,7 +1306,7 @@ IFRUtil_VDNNumber::stringToNumber(char              *buffer,
             }
         }
         return stringToNumberAscii(tmpbuffer, tmplen, nan, number, fixed, length, precision);
-    } else if(encoding == IFR_StringEncodingUCS2) {
+    } else if(encoding == IFR_StringEncodingUCS2Swapped) {
         if(bufferlength % 2) {
             return IFR_NOT_OK;
         }
@@ -1357,29 +1398,51 @@ IFRUtil_VDNNumber::stringToNumberAscii(char              *buffer,
     IFR_Int4  frac_digits=0;
     IFR_Int4  exponent=0;
     IFR_Bool  frac_part=false;
-
-    
+    IFR_Bool  frac_significant = false;
+    IFR_Int4  frac_zeros=0;
 
     do {
         if(*buffer == '.') {
             frac_part = true;
         } else  if((*buffer)>='0' && (*buffer)<='9') {
-            if(num_digits >= 39) {
-                if(frac_part) {
-                    rc = IFR_DATA_TRUNC;
+            if(!frac_part) {
+                frac_significant = true; // after having a number before the '.', the
+                // fractional part is always significant.
+                if(num_digits >= 39) {
+                    if(frac_part) {
+                        rc = IFR_DATA_TRUNC;
+                    } else {
+                        return IFR_OVERFLOW;
+                    }
                 } else {
-                    return IFR_OVERFLOW;
-                }
-            } else {
-                digits[num_digits] = *buffer - '0';
-                ++num_digits;
-                if(!frac_part) {
+                    digits[num_digits] = *buffer - '0';
+                    ++num_digits;
                     if(num_digits == 39) { 
                         return IFR_OVERFLOW; 
                     }
                     ++exponent;
+                }
+            } else {
+                if(*buffer != '0' || frac_significant) {
+                    frac_significant = true;
+                    if(num_digits >= 39) {
+                        if(frac_part) {
+                            rc = IFR_DATA_TRUNC;
+                        } else {
+                            return IFR_OVERFLOW;
+                        }
+                    } else {
+                        digits[num_digits] = *buffer - '0';
+                        ++num_digits;
+                        ++frac_digits;
+                    }
                 } else {
-                    ++frac_digits;
+                    --exponent;
+                    ++frac_zeros;
+                    if(remaining==1) {
+                        memcpy(number, zero_vdnnumber, digits_bytelength + 1);
+                        return IFR_OK;
+                    }
                 }
             }
         } else { 
@@ -1439,27 +1502,25 @@ IFRUtil_VDNNumber::stringToNumberAscii(char              *buffer,
         if( (num_digits - frac_digits) > (length - precision) ) {
             return IFR_OVERFLOW;
         }
-        if(frac_digits > precision) {
+        if((frac_digits + frac_zeros) > precision) {
             rc = IFR_DATA_TRUNC;
         }
     }
     
     if(num_digits > length) {
-        if(digits[length] >= 5) {
-            digits[length-1]++;
-        }
-        for(IFR_Int4 i=length; i<num_digits; ++i) {
-            digits[i]=0;
-        }
+        round_digits(digits, length, num_digits, exponent);
     }
-
+    
     IFR_Int4 lastnonzero=0;
     for(IFR_Int4 k=0; k<num_digits; ++k) {
         if(digits[k]) {
             lastnonzero = k;
         }
     }
-
+    
+    if(abs(exponent) > 64) {
+        return  IFR_OVERFLOW;
+    }
     // and now insert it !
 
     if(lastnonzero == 0 && digits[0]==0) {
