@@ -82,8 +82,14 @@ private:
     SizeType m_Size;                        //!< The number of vector elements
     SizeType m_Capacity;                    //!< The number of vector elements that can be stored without resizing the vector.
     ValueType m_InitVal;                    //!< The initialisation value. 
+
+protected:
+    inline void setSize(SizeType size) 
+    {
+        m_Size = size;
+    }
+
 public:
-    
     /**
      * Constructs a vector that will use the provided allocator.
      * @param RawAlloc The raw allocator to be used for this vector.
@@ -113,7 +119,7 @@ public:
      m_InitVal()
     {
         if (Size > 0) {
-            IFRUtil_NewArray(m_ArrayPtr, Size, (*m_rawallocator));
+            IFRUtil_NewArray(m_ArrayPtr, Size, IFR_ALLOCATOR(*m_rawallocator));
         } else {
             m_ArrayPtr=0;
         }
@@ -145,7 +151,7 @@ public:
         }
         if (Size > 0) {
             IFRUtil_NewArray(m_ArrayPtr, Size, Init, 
-                             (*m_rawallocator), 
+                             IFR_ALLOCATOR(*m_rawallocator), 
                              memory_ok);
             if(!memory_ok) {
                 m_Size = m_Capacity = 0;
@@ -179,7 +185,7 @@ public:
             IFRUtil_NewArray(m_ArrayPtr, 
                              InitVector.m_ArrayPtr, 
                              InitVector.m_ArrayPtr + m_Size, 
-                             (*m_rawallocator),
+                             IFR_ALLOCATOR(*m_rawallocator),
                              memory_ok);
         } else {
             m_ArrayPtr = 0;
@@ -211,7 +217,7 @@ public:
             IFRUtil_NewArray(m_ArrayPtr, 
                              InitVector.m_ArrayPtr, 
                              InitVector.m_ArrayPtr + m_Size, 
-                             (*m_rawallocator),
+                             IFR_ALLOCATOR(*m_rawallocator),
                              memory_ok);
         } else {
             m_ArrayPtr = 0;
@@ -223,7 +229,12 @@ public:
      */
     inline ~IFRUtil_Vector()
     {
-        IFRUtil_DeleteArray(m_ArrayPtr, m_Size, (*m_rawallocator));
+        IFRUtil_DeleteArray(m_ArrayPtr, m_Size, IFR_ALLOCATOR(*m_rawallocator));
+    }
+
+    inline SAPDBMem_IRawAllocator& allocator()
+    {
+        return *m_rawallocator;
     }
 
     /**
@@ -235,6 +246,15 @@ public:
         return (ConstPointer) m_ArrayPtr;
     } 
 
+    /**
+     * Get the data.
+     * @return The pointer to the array data.
+     */
+    inline T* Data()
+    {
+        return (Pointer) m_ArrayPtr;
+    } 
+
     /**
      * Returns the number of vector elements within the vector
      * @return The number of elements in the vector.
@@ -301,7 +321,7 @@ public:
      * @param Iter The iterator.
      * @return The index of the value identified by the iterator.
      */
-    SizeType GetIndex(const Iterator& Iter) const
+    inline SizeType GetIndex(const Iterator& Iter) const
     {
         return (Iter - m_ArrayPtr);
     }
@@ -385,13 +405,13 @@ public:
         while(sz < NewCapacity) 
             sz = sz << 1;
         // Currently, we use simple allocation without hint.
-        Pointer new_ArrayPtr = (Pointer) (*m_rawallocator).Allocate(sz * sizeof(ValueType));
+        Pointer new_ArrayPtr = (Pointer) IFR_ALLOCATOR(*m_rawallocator).Allocate(sz * sizeof(ValueType));
         if(new_ArrayPtr == 0) {
             memory_ok=false;
         } else {
             memcpy(new_ArrayPtr, m_ArrayPtr, sizeof(ValueType) * m_Size);
             if(m_ArrayPtr) {
-                (*m_rawallocator).Deallocate(m_ArrayPtr);
+                IFR_ALLOCATOR(*m_rawallocator).Deallocate(m_ArrayPtr);
             }
             m_ArrayPtr=new_ArrayPtr;
             m_Capacity=sz;
@@ -405,7 +425,7 @@ public:
      * @param InitVector The vector to assign to this vector.
      * @return <code>*this</code>.
      */
-    void assign(const IFRUtil_Vector<T>& InitVector, IFR_Bool& memory_ok)
+    inline void assign(const IFRUtil_Vector<T>& InitVector, IFR_Bool& memory_ok)
     {
         
         if(&InitVector == this) {
@@ -635,8 +655,6 @@ public:
      */
     inline void Delete(const Iterator& From, const Iterator& To)
     {
-      //        IFR_ASSERT(!IsEmpty());
-      //        IFR_ASSERT( (To-From) >= 0 );
         Iterator dest = From;
         Iterator src = To;
         while((src!=End())) {
@@ -695,7 +713,7 @@ public:
     
 private:
     /** Dummy. */
-    IFRUtil_Vector(const IFRUtil_Vector<T>& c)
+    inline IFRUtil_Vector(const IFRUtil_Vector<T>& c)
     :m_rawallocator(c.m_rawallocator), 
      m_Size(0), 
      m_Capacity(0), 
@@ -705,7 +723,7 @@ private:
     }
     
     /** Dummy. */
-    IFRUtil_Vector<T>& operator=(const IFRUtil_Vector<T>&)
+    inline IFRUtil_Vector<T>& operator=(const IFRUtil_Vector<T>&)
     {
         return *this;
     }
