@@ -147,7 +147,7 @@ SQLDBC_ConnectionItem::SQLDBC_ConnectionItem(IFR_ConnectionItem *item)
           // subsequent calls will return SQLDBC_INVALID_OBJECT 
           m_citem = 0;
         }
-        m_citem = new (*(item->getAllocator())) SQLDBC_ConnectionItemStorage(item);
+        m_citem = new IFR_ALLOCATOR(*(item->getAllocator())) SQLDBC_ConnectionItemStorage(item);
     }
 }
 
@@ -213,7 +213,7 @@ void SQLDBC_ConnectionItem::clearError()
 
 SQLDBC_Environment::SQLDBC_Environment(SQLDBC_IRuntime* runtime)
 {
-    m_env = new (runtime->getGlobalAllocator()) IFR_Environment(runtime);
+    m_env = new IFR_ALLOCATOR(runtime->getGlobalAllocator()) IFR_Environment(runtime);
 }
 
 SQLDBC_Environment::~SQLDBC_Environment()
@@ -238,7 +238,7 @@ SQLDBC_Connection *SQLDBC_Environment::createConnection() const
         if(contained_connection == 0) {
             return 0;
         }
-        SQLDBC_Connection *result = new (m_env->allocator) SQLDBC_Connection(contained_connection);
+        SQLDBC_Connection *result = new IFR_ALLOCATOR(m_env->allocator) SQLDBC_Connection(contained_connection);
         if(result == 0) {
             m_env->releaseConnection(contained_connection);
         }
@@ -251,7 +251,7 @@ SQLDBC_Connection *SQLDBC_Environment::createConnection() const
 SQLDBC_Connection *SQLDBC_Environment::createConnection(SAPDBMem_IRawAllocator& allocator) const
 {
     if(m_env) {
-        return new (allocator) SQLDBC_Connection(m_env->getConnection(allocator));
+        return new IFR_ALLOCATOR(allocator) SQLDBC_Connection(m_env->getConnection(allocator));
     } else {
         return 0;
     }
@@ -271,15 +271,35 @@ SAPDBMem_IRawAllocator& SQLDBC_Environment::getAllocator()
     return m_env->allocator;
 }
 
+
+void SQLDBC_Environment::setTraceOptions(const SQLDBC_ConnectProperties& traceoptions)
+{
+    m_env->setTraceOptions(*(traceoptions.m_prop));
+}
+
+
+void SQLDBC_Environment::getTraceOptions(SQLDBC_ConnectProperties& traceoptions)
+{
+    m_env->setTraceOptions(*(traceoptions.m_prop));
+}
+
+
 /********************************/
 
 SQLDBC_ConnectProperties::SQLDBC_ConnectProperties()
 {
     IFR_Bool memory_ok_ignored=true;
-    m_prop = new (RTEMem_RteAllocator::Instance()) 
+    m_prop = new IFR_ALLOCATOR(RTEMem_RteAllocator::Instance()) 
         IFR_ConnectProperties(RTEMem_RteAllocator::Instance(), memory_ok_ignored);
 }
 
+SQLDBC_ConnectProperties::SQLDBC_ConnectProperties(SAPDBMem_IRawAllocator& allocator)
+{
+    IFR_Bool memory_ok_ignored=true;
+    m_prop = new IFR_ALLOCATOR(allocator) IFR_ConnectProperties(allocator, memory_ok_ignored);
+}
+
+
 SQLDBC_ConnectProperties::SQLDBC_ConnectProperties(const SQLDBC_ConnectProperties& copy)
 :m_prop(copy.m_prop)
 {}
@@ -359,7 +379,7 @@ SQLDBC_Retcode SQLDBC_Connection::disconnect()
   SQLDBC_CHECK_CITEM_ARG(SQLDBC_NOT_OK);
   IFR_Connection *conn = (IFR_Connection *)m_citem->m_item;
   conn->clearError();
-  return (SQLDBC_Retcode)conn->close(true, true);
+  return (SQLDBC_Retcode)conn->close(false, true);
 }
 
 SQLDBC_Retcode SQLDBC_Connection::commit()
@@ -377,7 +397,7 @@ SQLDBC_Statement *SQLDBC_Connection::createStatement()
     SQLDBC_CHECK_CITEM_ARG(0);
     IFR_Connection *conn = (IFR_Connection *)m_citem->m_item;
     conn->clearError();
-    return new (conn->allocator) SQLDBC_Statement(conn->createStatement());
+    return new IFR_ALLOCATOR(conn->allocator) SQLDBC_Statement(conn->createStatement());
 }
 
 SQLDBC_PreparedStatement *SQLDBC_Connection::createPreparedStatement()
@@ -386,7 +406,7 @@ SQLDBC_PreparedStatement *SQLDBC_Connection::createPreparedStatement()
     SQLDBC_CHECK_CITEM_ARG(0);
     IFR_Connection *conn = (IFR_Connection *)m_citem->m_item;
     conn->clearError();
-    return new (conn->allocator) SQLDBC_PreparedStatement(conn->createPreparedStatement());
+    return new IFR_ALLOCATOR(conn->allocator) SQLDBC_PreparedStatement(conn->createPreparedStatement());
 }
 
 
@@ -701,12 +721,23 @@ SQLDBC_RowSet::getObject(const SQLDBC_Int4     columnIndex,
 {
   SQLDBC_CHECK(this, SQLDBC_INVALID_OBJECT);
   IFR_ResultSet *resultset = (IFR_ResultSet *)m_citem->m_item;
-  return (SQLDBC_Retcode)resultset->getRowSet ()->getObject(columnIndex, 
-							    columnAddr, 
-							    columnSize, 
-							    (IFR_HostType)columnType,
-							    columnLengthIndicator,
-							    terminate);  
+  IFR_RowSet *rowset = resultset->getRowSet();
+  if(rowset) {
+    SQLDBC_Retcode rc = (SQLDBC_Retcode) rowset->getObject(columnIndex, 
+							   columnAddr, 
+							   columnSize, 
+							   (IFR_HostType)columnType,
+							   columnLengthIndicator,
+							   terminate);  
+    if(rc != SQLDBC_OK) {
+      if(rowset->error() && !resultset->error()) {
+	resultset->error() = rowset->error();
+      }
+    }
+    return rc;
+  } else {
+    return SQLDBC_INVALID_OBJECT;
+  }
 }
 
 SQLDBC_Retcode
@@ -720,13 +751,24 @@ SQLDBC_RowSet::getObject(const SQLDBC_Int4     columnIndex,
 {
   SQLDBC_CHECK(this, SQLDBC_INVALID_OBJECT);
   IFR_ResultSet *resultset = (IFR_ResultSet *)m_citem->m_item;
-  return (SQLDBC_Retcode)resultset->getRowSet ()->getObject(columnIndex, 
-							    columnAddr, 
-							    columnSize,
-							    (IFR_HostType)columnType,
-							    columnLengthIndicator,
-							    terminate,
-							    &columnStartPos);  
+  IFR_RowSet *rowset = resultset->getRowSet();
+  if(rowset) {
+    SQLDBC_Retcode rc = (SQLDBC_Retcode) rowset->getObject(columnIndex, 
+							   columnAddr, 
+							   columnSize, 
+							   (IFR_HostType)columnType,
+							   columnLengthIndicator,
+							   terminate,
+							   &columnStartPos);  
+    if(rc != SQLDBC_OK) {
+      if(rowset->error() && !resultset->error()) {
+	resultset->error() = rowset->error();
+      }
+    }
+    return rc;
+  } else {
+    return SQLDBC_INVALID_OBJECT;
+  }
 }
 
 SQLDBC_Retcode SQLDBC_RowSet::fetch()
@@ -812,7 +854,7 @@ SQLDBC_ResultSet::SQLDBC_ResultSet(SQLDBC_Statement *statement,
                                    IFR_ResultSet *resultset)
   : SQLDBC_ConnectionItem(resultset)
 {
-    m_cresult = new (resultset->allocator) SQLDBC_ResultSetStorage(statement, resultset);
+    m_cresult = new IFR_ALLOCATOR(resultset->allocator) SQLDBC_ResultSetStorage(statement, resultset);
 }
 
 SQLDBC_ResultSet::~SQLDBC_ResultSet()
@@ -1060,13 +1102,13 @@ SQLDBC_ResultSet::getObject(const SQLDBC_Int4     columnIndex,
 SQLDBC_Statement::SQLDBC_Statement(IFR_Statement *stmt)
   : SQLDBC_ConnectionItem(stmt)
 {
-    m_cstmt = new (stmt->allocator) SQLDBC_StatementStorage(stmt, (void *)this);
+    m_cstmt = new IFR_ALLOCATOR(stmt->allocator) SQLDBC_StatementStorage(stmt, (void *)this);
 }
 
 SQLDBC_Statement::SQLDBC_Statement(IFR_PreparedStmt* stmt)
 : SQLDBC_ConnectionItem(stmt)
 {
-    m_cstmt = new (stmt->allocator) SQLDBC_StatementStorage(stmt, 0);
+    m_cstmt = new IFR_ALLOCATOR(stmt->allocator) SQLDBC_StatementStorage(stmt, 0);
 }
 
 SQLDBC_Statement::~SQLDBC_Statement()
@@ -1135,7 +1177,7 @@ SQLDBC_ResultSet *SQLDBC_Statement::getResultSet()
       IFR_Statement *stmt = (IFR_Statement *)m_citem->m_item;
       IFR_ResultSet *resultset = stmt->getResultSet();
       if(resultset) {
-          m_cstmt->m_resultset = new (m_cstmt->m_allocator)
+          m_cstmt->m_resultset = new IFR_ALLOCATOR(m_cstmt->m_allocator)
               SQLDBC_ResultSet(this, resultset);
       }
   }
@@ -1181,7 +1223,15 @@ void SQLDBC_Statement::setResultSetConcurrencyType (ConcurrencyType conType)
   IFR_Statement *stmt = (IFR_Statement *)m_citem->m_item;
   return (SQLDBC_Statement::ConcurrencyType) stmt->getResultSetConcurrencyType();
 }
-
+//----------------------------------------------------------------------
+SQLDBC_Retcode 
+SQLDBC_Statement::setCommandInfo(const char *buffer, 
+                               SQLDBC_Length bufferLength, 
+                               SQLDBC_Int4 lineNumber)
+{
+    IFR_Statement *stmt = (IFR_Statement *)m_citem->m_item;
+    return (SQLDBC_Retcode) stmt->setCommandInfo(buffer, bufferLength, lineNumber);
+}                                                  
 //----------------------------------------------------------------------
 void
 SQLDBC_Statement::setCursorName(const char *buffer,
@@ -1292,6 +1342,26 @@ SQLDBC_Statement::clearBatch()
   return;
 }
 
+SQLDBC_Retcode
+SQLDBC_Statement::getLastInsertedKey(SQLDBC_Int4     tag,
+                                     SQLDBC_HostType type,
+                                     void            *paramAddr,
+                                     SQLDBC_Length   *lengthIndicator,
+                                     SQLDBC_Length    size,
+                                     SQLDBC_Bool      terminate)
+{
+  if (!this) {
+      return SQLDBC_INVALID_OBJECT;
+  }
+  IFR_Statement *stmt = (IFR_Statement *)m_citem->m_item;
+  return (SQLDBC_Retcode) stmt->getLastInsertedKey(tag, 
+                                                   (IFR_HostType)type, 
+                                                   paramAddr, 
+                                                   lengthIndicator, 
+                                                   size, 
+                                                   terminate);
+}
+
   
 /********************************/
 struct SQLDBC_PreparedStatementStorage 
@@ -1311,7 +1381,7 @@ SQLDBC_PreparedStatement::SQLDBC_PreparedStatement(IFR_PreparedStmt *stmt)
 :SQLDBC_Statement(stmt)
 {
     m_cstmt->m_self = this;
-    m_pcstmt = new (stmt->allocator) SQLDBC_PreparedStatementStorage(stmt);
+    m_pcstmt = new IFR_ALLOCATOR(stmt->allocator) SQLDBC_PreparedStatementStorage(stmt);
 }
 
 SQLDBC_PreparedStatement::~SQLDBC_PreparedStatement()
@@ -1330,6 +1400,14 @@ SQLDBC_PreparedStatement::setBatchSize(SQLDBC_UInt4 rowarraysize)
   return (SQLDBC_Retcode)stmt->setRowArraySize(rowarraysize);
 }
 
+SQLDBC_UInt4
+SQLDBC_PreparedStatement::getPreferredBatchSize()
+{
+    SQLDBC_CHECK(this, SQLDBC_INVALID_OBJECT);
+    IFR_PreparedStmt *stmt = (IFR_PreparedStmt *)m_citem->m_item;
+    return stmt->getPreferredRowArraySize();
+}
+
 
 SQLDBC_Retcode
 SQLDBC_PreparedStatement::clearParameters()
