@@ -35,9 +35,13 @@
 #include "Interfaces/SQLDBC/SQLDBC_Types.h"
 #include "SAPDBCommon/SAPDB_Types.h"
 #include "SAPDBCommon/SAPDB_ReuseLib.h"
+#include "SAPDB/SAPDBCommon/ErrorsAndMessages/SAPDBErr_MessageList.hpp"
+
+#ifndef Msg_List
+#  define Msg_List SAPDBErr_MessageList
+#endif
 
 class SAPDBMem_IRawAllocator;
-class SAPDBErr_MessageList;
 
 #ifdef _WIN32 
 #define SQLDBC_DLLEXPORT __declspec(dllexport)
@@ -177,7 +181,7 @@ public:
                                       char *username,
                                       SAPDB_Int4& usernamelength,
                                       SQLDBC_StringEncoding& usernameEncoding,
-                                      SAPDBErr_MessageList& errorMessages)
+                                      Msg_List& errorMessages)
     {
         return false;
     }
@@ -245,7 +249,7 @@ public:
                                   SAPDB_Int4& packetListSize,
                                   void **packetList,  
                                   void **sessionInfoReply,
-                                  SAPDBErr_MessageList& errorMessages,
+                                  Msg_List& errorMessages,
                                   SAPDBMem_IRawAllocator* allocator = 0) = 0;
 
     /**
@@ -257,7 +261,7 @@ public:
      *         if it fails. In this case the message list is filled.
      */
     virtual SAPDB_Bool releaseSession(SAPDB_Int8 sessionID,
-                                      SAPDBErr_MessageList& errorMessages) = 0;
+                                      Msg_List& errorMessages) = 0;
     
     /**
      * Performs a request to the database kernel. 
@@ -272,7 +276,7 @@ public:
     virtual SAPDB_Bool request(SAPDB_Int8  sessionID,
                                void       *requestData,
                                SAPDB_UInt4 requestDataLength,
-                               SAPDBErr_MessageList& errorMessages) = 0; 
+                               Msg_List& errorMessages) = 0; 
 
     /**
      * Retrieves data from the database kernel. 
@@ -286,7 +290,7 @@ public:
     virtual SAPDB_Bool receive(SAPDB_Int8 sessionID,
                                void **replyData,
                                SAPDB_Int4& replyDataLength,
-                               SAPDBErr_MessageList& errorMessages) = 0;
+                               Msg_List& errorMessages) = 0;
 
     /**
      * Cancels the current SQL statment.
@@ -296,7 +300,7 @@ public:
      *         if it fails. In this case the message list is filled.
      */
     virtual SAPDB_Bool cancelCurrentCommand(SAPDB_Int8 sessionID,
-                                            SAPDBErr_MessageList& errorMessages) = 0;
+                                            Msg_List& errorMessages) = 0;
 
     // -- synchronization primitives
     /**
@@ -310,7 +314,7 @@ public:
      */    
     virtual SAPDB_Bool createMutex(MutexHandle& mutexHandle, 
                                    SAPDBMem_IRawAllocator& allocator,
-                                   SAPDBErr_MessageList& errorMessages)  = 0;
+                                   Msg_List& errorMessages)  = 0;
 
     /**
      * Locks a mutex.
@@ -338,7 +342,7 @@ public:
      */    
     virtual SAPDB_Bool destroyMutex(MutexHandle& mutexHandle, 
                                     SAPDBMem_IRawAllocator& allocator,
-                                    SAPDBErr_MessageList& errorMessages) = 0;
+                                    Msg_List& errorMessages) = 0;
     
     /**
      * Creates an atomic counter.
@@ -350,7 +354,7 @@ public:
      */    
     virtual SAPDB_Bool  createCounter(CounterHandle& counterHandle,
                                       SAPDBMem_IRawAllocator& allocator,
-                                      SAPDBErr_MessageList& errorMessages)  = 0;
+                                      Msg_List& errorMessages)  = 0;
 
     
     /** 
@@ -371,7 +375,7 @@ public:
      */    
     virtual SAPDB_Bool  destroyCounter(CounterHandle counterHandle,
                                       SAPDBMem_IRawAllocator& allocator,
-                                      SAPDBErr_MessageList& errorMessages)  = 0;
+                                      Msg_List& errorMessages)  = 0;
            
     /**
      * Returns the ID of the current task. If it is single threaded, the 
@@ -393,7 +397,7 @@ public:
     virtual SAPDB_Bool createSemaphore(SemaphoreHandle& semaphoreHandle,
                                        SAPDB_Int4 initialValue,
                                        SAPDBMem_IRawAllocator& allocator,
-                                       SAPDBErr_MessageList errorMessages) = 0;
+                                       Msg_List& errorMessages) = 0;
 
     /**
      * Waits for the semaphore, decreases its value by one and
@@ -423,7 +427,7 @@ public:
      */    
     virtual SAPDB_Bool destroySemaphore(SemaphoreHandle& semaphoreHandle,
                                         SAPDBMem_IRawAllocator& allocator,
-                                        SAPDBErr_MessageList& errorMessages)  = 0;
+                                        Msg_List& errorMessages)  = 0;
     
     // -- Tracing 
     
@@ -456,12 +460,36 @@ public:
      * Updates the internal trace flags, so that changes in trace settings are
      * detected. This method is called by the interface runtime from time to 
      * check whether updates are available. If the update fails, there shall
-     * be no error reported, but of cu
+     * be no error reported.
      * @param traceEnabled Flag, which is is set to  @c true if any trace is
      *     enabled, and set to @c false if there is no trace that can be 
      *     enabled.
      */ 
     virtual void updateTraceFlags(SAPDB_Bool& traceEnabled) { traceEnabled=true; return; }
+
+    /**
+     * Called when an error is traced. The runtime may count
+     * or disable tracing as result of this method call.
+     * @param errorcode The error code that has happened.
+     * @return @c true if some trace was switched off.
+     */
+    virtual SAPDB_Bool traceError(SAPDB_Int4 errorcode) { return false; }
+
+    //-- internally for client runtime used, no use in Kernel/OMS/...
+    
+    virtual void setTraceOptions(const char *optionstring) 
+    { 
+        return; 
+    }
+    
+    virtual SAPDB_Int4 getTraceOptions(char *options, SAPDB_Int4 optionslength) 
+    { 
+        if(options && optionslength>0) {
+            *options = '\0';
+        }
+        return 0; 
+    }
+
 };
 
 #endif
