@@ -33,6 +33,12 @@
 #include "Interfaces/Runtime/Packet/IFRPacket_ReplySegment.h"
 #include "Interfaces/Runtime/Packet/IFRPacket_Part.h"
 
+#include "SAPDB/SAPDBCommon/ErrorsAndMessages/SAPDBErr_MessageList.hpp"
+
+#ifndef Msg_List
+#  define Msg_List SAPDBErr_MessageList
+#endif
+
 #ifdef WIN32
 #  define STRICMP _stricmp
 #else
@@ -55,10 +61,10 @@ typedef void* (*SQLONCE_InitFuncType)(void*);
 #define MYSQL_HASHED_PASSWORD_LENGTH  9
 
 #define OOM_ERROR(messagelist)                                          \
-SAPDBErr_MessageList __tmp__messagelist("SQLDBC",                       \
+Msg_List __tmp__messagelist("SQLDBC",                       \
                                          __FILE__,                      \
                                          __LINE__,                      \
-                                         SAPDBErr_MessageList::Error,   \
+                                         Msg_List::Error,   \
                                          -10760,                        \
                                          "Memory allocation failed.",   \
                                          0);                            \
@@ -72,7 +78,7 @@ SQLDBC_ProxyRuntime::SQLDBC_ProxyRuntime()
     for(SQLDBC_size_t i=0; i< sizeof(pendingSessions)/sizeof(SessionInfo); ++i) {
         pendingSessions[i].sessionid = -1;
     }
-    SAPDBErr_MessageList ignored;
+    Msg_List ignored;
     createMutex(pendingSessionsMutex, getGlobalAllocator(), ignored);
 }
 
@@ -104,7 +110,7 @@ SQLDBC_ProxyRuntime::checkKeyOption(const char *keyoption,
                                     char *username,
                                     SAPDB_Int4& usernamelength,
                                     SQLDBC_StringEncoding& usernameEncoding,
-                                    SAPDBErr_MessageList& errorMessages)
+                                    Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::checkKeyOption(keyoption, 
                                                 servernode,
@@ -139,7 +145,7 @@ SQLDBC_ProxyRuntime::getSession(const char *connectUrl,
                                 SAPDB_Int4& packetListSize,
                                 void **packetList,  
                                 void **sessionInfoReply,
-                                SAPDBErr_MessageList& errorMessages,
+                                Msg_List& errorMessages,
                                 SAPDBMem_IRawAllocator* allocator)
 {
     // Use global allocator if no allocator passed.
@@ -159,10 +165,10 @@ SQLDBC_ProxyRuntime::getSession(const char *connectUrl,
     
     const char *sessionid = properties.getProperty("SESSIONID");
     if(sessionid == 0) {
-        SAPDBErr_MessageList tmpMessageList("SQLDBC",
+        Msg_List tmpMessageList("SQLDBC",
                                             __FILE__,
                                             __LINE__,
-                                            SAPDBErr_MessageList::Error,
+                                            Msg_List::Error,
                                             0,
                                             "No session id found.",
                                             0);
@@ -174,10 +180,10 @@ SQLDBC_ProxyRuntime::getSession(const char *connectUrl,
     if(*errp && errno!=ERANGE) {
         char errmesg[80];
         sp77sprintf(errmesg, 80, "Invalid session number %s specified.", sessionid);
-        SAPDBErr_MessageList tmpMessageList("SQLDBC",
+        Msg_List tmpMessageList("SQLDBC",
                                             __FILE__,
                                             __LINE__,
-                                            SAPDBErr_MessageList::Error,
+                                            Msg_List::Error,
                                             0,
                                             errmesg,
                                             0);
@@ -188,10 +194,10 @@ SQLDBC_ProxyRuntime::getSession(const char *connectUrl,
     if(!session) {
         char errmesg[80];
         sp77sprintf(errmesg, 80, "Invalid session number %s specified.", sessionid);
-        SAPDBErr_MessageList tmpMessageList("SQLDBC",
+        Msg_List tmpMessageList("SQLDBC",
                                             __FILE__,
                                             __LINE__,
-                                            SAPDBErr_MessageList::Error,
+                                            Msg_List::Error,
                                             0,
                                             errmesg,
                                             0);
@@ -208,10 +214,10 @@ SQLDBC_ProxyRuntime::getSession(const char *connectUrl,
     if(!packetList[0]) {
         char errmesg[80];
         sp77sprintf(errmesg, 80, "No request packet available.");
-        SAPDBErr_MessageList tmpMessageList("SQLDBC",
+        Msg_List tmpMessageList("SQLDBC",
                                             __FILE__,
                                             __LINE__,
-                                            SAPDBErr_MessageList::Error,
+                                            Msg_List::Error,
                                             0,
                                             errmesg,
                                             0);
@@ -230,10 +236,10 @@ SQLDBC_ProxyRuntime::getSession(const char *connectUrl,
     // no applencrypt, the password that is provided is already what we need ...
     // password is 
     if(passwordLength != MYSQL_HASHED_PASSWORD_LENGTH) {
-        SAPDBErr_MessageList tmpMessageList("SQLDBC",
+        Msg_List tmpMessageList("SQLDBC",
                                             __FILE__,
                                             __LINE__,
-                                            SAPDBErr_MessageList::Error,
+                                            Msg_List::Error,
                                             0,
                                             "Invalid length of password hash.",
                                             0);
@@ -274,14 +280,14 @@ SQLDBC_ProxyRuntime::getSession(const char *connectUrl,
 
     // -- create the packet
     IFRPacket_RequestPacket *requestpacket = 
-        new (*allocator) IFRPacket_RequestPacket((tsp1_packet*)packetList[0],
-                                                packetSize,
-                                                client_unicode,
-                                                client_sqlmode,
-                                                client_application,
-                                                client_version,
-                                                 runtimeitem,
-                                                 memory_ok);
+        new IFR_ALLOCATOR(*allocator) IFRPacket_RequestPacket((tsp1_packet*)packetList[0],
+                                                              packetSize,
+                                                              client_unicode,
+                                                              client_sqlmode,
+                                                              client_application,
+                                                              client_version,
+                                                              runtimeitem,
+                                                              memory_ok);
     if(IFR_MEM_NOT_OK || requestpacket==0) {
         OOM_ERROR(errorMessages);
         return false;
@@ -297,10 +303,10 @@ SQLDBC_ProxyRuntime::getSession(const char *connectUrl,
     }
     IFR_ErrorHndl tmperror(*allocator);
     if (IFR_OK != connect_cmdpart.setText(connectCommandString, tmperror)) {
-      SAPDBErr_MessageList tmpMessageList("SQLDBC",
+      Msg_List tmpMessageList("SQLDBC",
                                           __FILE__,
                                           __LINE__,
-                                          SAPDBErr_MessageList::Error,
+                                          Msg_List::Error,
                                           tmperror.getErrorCode(),
                                           tmperror.getErrorText(),
                                           0);
@@ -356,7 +362,7 @@ SQLDBC_ProxyRuntime::getSession(const char *connectUrl,
 //----------------------------------------------------------------------
 SAPDB_Bool 
 SQLDBC_ProxyRuntime::releaseSession(SAPDB_Int8 sessionID,
-                                    SAPDBErr_MessageList& errorMessages)
+                                    Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::releaseSession(sessionID, errorMessages);
 }
@@ -366,7 +372,7 @@ SAPDB_Bool
 SQLDBC_ProxyRuntime::request(SAPDB_Int8  sessionID,
                              void       *requestData,
                              SAPDB_UInt4 requestDataLength,
-                             SAPDBErr_MessageList& errorMessages)
+                             Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::request(sessionID, 
                                          requestData,
@@ -379,7 +385,7 @@ SAPDB_Bool
 SQLDBC_ProxyRuntime::receive(SAPDB_Int8 sessionID,
                              void **replyData,
                              SAPDB_Int4& replyDataLength,
-                             SAPDBErr_MessageList& errorMessages)
+                             Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::receive(sessionID,
                                          replyData,
@@ -389,7 +395,7 @@ SQLDBC_ProxyRuntime::receive(SAPDB_Int8 sessionID,
 //----------------------------------------------------------------------
 SAPDB_Bool 
 SQLDBC_ProxyRuntime::cancelCurrentCommand(SAPDB_Int8 sessionID,
-                                          SAPDBErr_MessageList& errorMessages)
+                                          Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::cancelCurrentCommand(sessionID, errorMessages);
 }
@@ -398,7 +404,7 @@ SQLDBC_ProxyRuntime::cancelCurrentCommand(SAPDB_Int8 sessionID,
 SAPDB_Bool 
 SQLDBC_ProxyRuntime::createMutex(MutexHandle& mutexHandle, 
                                  SAPDBMem_IRawAllocator& allocator,
-                                 SAPDBErr_MessageList& errorMessages)
+                                 Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::createMutex(mutexHandle, allocator, errorMessages);
 }
@@ -421,7 +427,7 @@ SQLDBC_ProxyRuntime::releaseMutex(MutexHandle mutexHandle)
 SAPDB_Bool 
 SQLDBC_ProxyRuntime::destroyMutex(MutexHandle& mutexHandle, 
                                   SAPDBMem_IRawAllocator& allocator,
-                                  SAPDBErr_MessageList& errorMessages)
+                                  Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::destroyMutex(mutexHandle,
                                               allocator,
@@ -432,7 +438,7 @@ SQLDBC_ProxyRuntime::destroyMutex(MutexHandle& mutexHandle,
 SAPDB_Bool  
 SQLDBC_ProxyRuntime::createCounter(CounterHandle& counterHandle,
                                    SAPDBMem_IRawAllocator& allocator,
-                                   SAPDBErr_MessageList& errorMessages)
+                                   Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::createCounter(counterHandle, allocator, errorMessages);
 }
@@ -448,7 +454,7 @@ SQLDBC_ProxyRuntime::nextCounter(CounterHandle counterHandle)
 SAPDB_Bool  
 SQLDBC_ProxyRuntime::destroyCounter(CounterHandle counterHandle,
                                     SAPDBMem_IRawAllocator& allocator,
-                                    SAPDBErr_MessageList& errorMessages)
+                                    Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::destroyCounter(counterHandle, 
                                                 allocator,
@@ -466,7 +472,7 @@ SAPDB_Bool
 SQLDBC_ProxyRuntime::createSemaphore(SemaphoreHandle& semaphoreHandle,
                                      SAPDB_Int4 initialValue,
                                      SAPDBMem_IRawAllocator& allocator,
-                                     SAPDBErr_MessageList errorMessages)
+                                     Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::createSemaphore(semaphoreHandle,
                                                  initialValue,
@@ -492,7 +498,7 @@ SQLDBC_ProxyRuntime::signalSemaphore(SemaphoreHandle semaphoreHandle)
 SAPDB_Bool 
 SQLDBC_ProxyRuntime::destroySemaphore(SemaphoreHandle& semaphoreHandle,
                                       SAPDBMem_IRawAllocator& allocator,
-                                      SAPDBErr_MessageList& errorMessages)
+                                      Msg_List& errorMessages)
 {
     return SQLDBC_ClientRuntime::destroySemaphore(semaphoreHandle, 
                                                   allocator, 
@@ -551,7 +557,7 @@ SQLDBC_ProxyRuntime::getChallenge(const char    *servernode,
                                   char          *errortext,
                                   SQLDBC_Length  errortextLength)
 {
-    SAPDBErr_MessageList messages;
+    Msg_List messages;
     char urlbuffer[256];
     if(!servernode || *servernode=='\0') {
         servernode="localhost";
@@ -581,7 +587,7 @@ SQLDBC_ProxyRuntime::getChallenge(const char    *connectURL,
                                   SAPDB_Int8&    sessionid,
                                   char          *challenge,
                                   SQLDBC_Length& challengeLength,
-                                  SAPDBErr_MessageList& errorMessages,
+                                  Msg_List& errorMessages,
                                   SAPDBMem_IRawAllocator* allocator)
 {
     sessionid = -1;
@@ -646,10 +652,10 @@ SQLDBC_ProxyRuntime::getChallenge(const char    *connectURL,
                           IFR_StringEncodingAscii, 
                           *allocator,
                           memory_ok);
-        SAPDBErr_MessageList tmpMessageList("SQLDBC",
+        Msg_List tmpMessageList("SQLDBC",
                                             __FILE__,
                                             __LINE__,
-                                            SAPDBErr_MessageList::Error,
+                                            Msg_List::Error,
                                             0,
                                             errStr.getBuffer(),
                                             0);
@@ -663,14 +669,14 @@ SQLDBC_ProxyRuntime::getChallenge(const char    *connectURL,
 
     // -- create the packet
     IFRPacket_RequestPacket *requestpacket = 
-        new (*allocator) IFRPacket_RequestPacket((tsp1_packet*)session->packetlist[0],
-                                                 session->packetsize,
-                                                 0,
-                                                 IFR_INTERNAL,
-                                                 "ODB",
-                                                 "70404",
-                                                 runtimeitem,
-                                                 memory_ok);
+        new IFR_ALLOCATOR(*allocator) IFRPacket_RequestPacket((tsp1_packet*)session->packetlist[0],
+                                                              session->packetsize,
+                                                              0,
+                                                              IFR_INTERNAL,
+                                                              "ODB",
+                                                              "70404",
+                                                              runtimeitem,
+                                                              memory_ok);
     if(requestpacket == 0 || IFR_MEM_NOT_OK) {
         IFRUtil_Delete(requestpacket, *allocator);
         OOM_ERROR(errorMessages);
@@ -705,10 +711,10 @@ SQLDBC_ProxyRuntime::getChallenge(const char    *connectURL,
     IFRPacket_DataPart datapart;
     IFR_Retcode part_rc=firstsegment.getPart(datapart);
     if(part_rc == IFR_NO_DATA_FOUND) {
-        SAPDBErr_MessageList tmpMessageList("SQLDBC",
+        Msg_List tmpMessageList("SQLDBC",
                                             __FILE__,
                                             __LINE__,
-                                            SAPDBErr_MessageList::Error,
+                                            Msg_List::Error,
                                             0,
                                             "No challenge part found in database reply.",
                                             0);
@@ -718,16 +724,16 @@ SQLDBC_ProxyRuntime::getChallenge(const char    *connectURL,
         releaseSessionInfo(session);
         return false;
     }
-    if(challengeLength < datapart.bufferLength()) {
+    if(challengeLength < datapart.getBufferLength()) {
         char text[80];
         sp77sprintf(text, 
                     80, 
                     "Length of buffer for challenge too small, need (%d) bytes.", 
-                    datapart.bufferLength());
-        SAPDBErr_MessageList tmpMessageList("SQLDBC",
+                    datapart.getBufferLength());
+        Msg_List tmpMessageList("SQLDBC",
                                             __FILE__,
                                             __LINE__,
-                                            SAPDBErr_MessageList::Error,
+                                            Msg_List::Error,
                                             0,
                                             text,
                                             0);
@@ -738,7 +744,7 @@ SQLDBC_ProxyRuntime::getChallenge(const char    *connectURL,
         return false;
     }
     sessionid = session->sessionid;
-    challengeLength = datapart.bufferLength();
+    challengeLength = datapart.getBufferLength();
     memcpy(challenge, datapart.GetReadData(0), challengeLength);
     replypacket.setPacket(0,0);
     IFRUtil_Delete(requestpacket, *allocator);
