@@ -54,13 +54,16 @@ struct TraceFlags
      m_sql(false),
      m_packetsize(1000),
      m_filesize(-1),
-     m_timestamp(false)
+     m_timestamp(false),
+     m_stop_on_error(false),
+     m_errorcode(0),
+     m_errorcount(0)
     {
         char *p=(char *) alloca(strlen(flags));
         strcpy(p, flags);
         while(p && *p) {
             char *cpos=strchr(p, ':');
-            if(cpos != 0) 
+            if(cpos != 0)
                 *cpos=0;
             switch(*p) {
             case 'c':
@@ -87,7 +90,7 @@ struct TraceFlags
                 ++p;
                 if(*p) {
                     m_filesize=atoi(p);
-                    if(m_filesize!=-1 && 
+                    if(m_filesize!=-1 &&
                        m_filesize < 8192) {
                         m_filesize=8192;
                     }
@@ -96,6 +99,27 @@ struct TraceFlags
             case 'T':
                 m_timestamp = true;
                 break;
+            case 'e': {
+                ++p;
+                char *slpos =strchr(p, '/');
+                if(slpos == 0) {
+                    m_errorcode=atoi(p);
+                    if(m_errorcode != 0) {
+                        m_stop_on_error = true;
+                    }
+                } else {
+                    *slpos = '\0';
+                    m_errorcode = atoi(p);
+                    if(m_errorcode != 0) {
+                        m_stop_on_error = true;
+                        m_errorcount = atoi(slpos + 1);
+                        if(m_errorcount < 0) {
+                            m_errorcount = 0;
+                        }
+                    }
+                }
+                break;
+            }
             }
             if(cpos != 0) {
                 p=cpos + 1;
@@ -104,7 +128,7 @@ struct TraceFlags
             }
         }
     }
-    
+
     void toString(char *str)
     {
         *str='\0';
@@ -140,9 +164,24 @@ struct TraceFlags
             sp77sprintf(buffer, 128, optionset ? ":s%d" : "s%d", m_filesize);
             strcat(str, buffer);
         }
+
+        if(m_stop_on_error) {
+            if(m_errorcount == 0) {
+                char buffer[100];
+                sp77sprintf(buffer, 100, optionset?":e%d":"e%d", m_errorcode);
+                strcat(str, buffer);
+            } else {
+                char buffer[100];
+                sp77sprintf(buffer, 100, optionset?":e%d/%d":"e%d/%d",
+                            m_errorcode,
+                            m_errorcount);
+                strcat(str, buffer);
+            }
+        }
+
         return;
     }
-    
+
     IFR_Bool m_call;
     IFR_Bool m_debug;
     IFR_Bool m_profile;
@@ -151,6 +190,9 @@ struct TraceFlags
     IFR_Int4 m_packetsize;
     IFR_Int4 m_filesize;
     IFR_Bool m_timestamp;
+    IFR_Bool m_stop_on_error;
+    IFR_Int4 m_errorcode;
+    IFR_Int4 m_errorcount;
 };
 
 #define VERB_STREAM if(!m_verbose) {} else stdoutstream()
@@ -247,6 +289,11 @@ IFTools_Output::getAllocator()
     return RTEMem_RteAllocator::Instance();
 }
 
+//----------------------------------------------------------------------
+void
+IFTools_Output::traceError(IFR_Int4 errorcode)
+{}
+
 
 //----------------------------------------------------------------------
 IFTools_Console::IFTools_Console()
@@ -383,11 +430,13 @@ IFTools_Console::parseArguments(int argc, char **argv)
                 m_command = C_CONFIG_TRACE_SIZE;
             } else if(IFTools_Console_matchCommand(argv[pos], "FILENAME",1)) {
                 m_command = C_CONFIG_TRACE_FILENAME;
+            } else if(IFTools_Console_matchCommand(argv[pos], "STOP", 2)) {
+                m_command = C_CONFIG_TRACE_STOP_ON_ERROR;
             } else {
-                SYNTAX_ERROR("one of SHORT,LONG,PROFILE,PACKET,TIMESTAMP,SIZE,FILENAME,SQL");
+                SYNTAX_ERROR("one of SHORT,LONG,PROFILE,PACKET,TIMESTAMP,SIZE,FILENAME,SQL,STOP");
             }
             if(m_command == C_CONFIG_TRACE_SHORT
-               || m_command == C_CONFIG_TRACE_LONG 
+               || m_command == C_CONFIG_TRACE_LONG
                || m_command == C_CONFIG_TRACE_PROFILE
                || m_command == C_CONFIG_TRACE_TIMESTAMP
                || m_command == C_CONFIG_TRACE_SQL) {
@@ -403,7 +452,7 @@ IFTools_Console::parseArguments(int argc, char **argv)
             } else if (m_command == C_CONFIG_TRACE_PACKET) {
                 ++pos;
                 CHECK_END("one of ON, OFF, <size>");
-                
+
                 if(IFTools_Console_matchCommand(argv[pos], "ON", 2)) {
                     m_on   = true;
                     m_size = 1000;
@@ -420,24 +469,67 @@ IFTools_Console::parseArguments(int argc, char **argv)
             } else if (m_command == C_CONFIG_TRACE_SIZE) {
                 ++pos;
                 CHECK_END("one of UNLIMITED, <size>");
-                
+
                 if(IFTools_Console_matchCommand(argv[pos], "UNLIMITED", 1)) {
                     m_size = -1;
                 } else {
                     m_size = atoi(argv[pos]);
                     if(m_size < 8192) {
-                        stderrstream() << "Error: Syntax error, found " 
-                                       << argv[pos] 
-                                       << ", expected UNLIMITED or a number > 8192." 
+                        stderrstream() << "Error: Syntax error, found "
+                                       << argv[pos]
+                                       << ", expected UNLIMITED or a number > 8192."
                                        << endl;
                         return IFR_NOT_OK;
                     }
                 }
-                
+
             } else if (m_command == C_CONFIG_TRACE_FILENAME) {
                 ++pos;
                 CHECK_END("<filename>");
                 m_filename=argv[pos];
+            } else if (m_command = C_CONFIG_TRACE_STOP_ON_ERROR) {
+                ++pos;
+                CHECK_END("ON");
+                if(!IFTools_Console_matchCommand(argv[pos],"ON", 1)) {
+                    SYNTAX_ERROR("ON");
+                }
+                ++pos;
+                CHECK_END("ERROR");
+                if(!IFTools_Console_matchCommand(argv[pos],"ERROR", 1)) {
+                    SYNTAX_ERROR("ON");
+                }
+                ++pos;
+                CHECK_END("<errorcode> or OFF");
+
+                if(IFTools_Console_matchCommand(argv[pos], "OFF", 1)) {
+                    m_stop_on_error = false;
+                } else {
+                    m_stop_on_error = true;
+                    m_errorcode = atoi(argv[pos]);
+                    if(m_errorcode == 0) {
+                        stderrstream() << "Error: Syntax error, found "
+                                       << argv[pos]
+                                       << ", expected an error code."
+                                       << endl;
+                        return IFR_NOT_OK;
+                    }
+                    ++pos;
+                    if(pos != argc) {
+                        if(!IFTools_Console_matchCommand(argv[pos],"COUNT",1)) {
+                            SYNTAX_ERROR("COUNT");
+                        }
+                        ++pos;
+                        CHECK_END("<errorcount>");
+                        m_errorcount = atoi(argv[pos]);
+                        if(m_errorcount <= 0) {
+                            stderrstream() << "Error: Syntax error, found "
+                                           << argv[pos]
+                                           << ", expected a positive number."
+                                       << endl;
+                            return IFR_NOT_OK;
+                        }
+                    }
+                }
             }
         } else {
             SYNTAX_ERROR("TRACE");
@@ -463,12 +555,14 @@ IFTools_Console::parseArguments(int argc, char **argv)
         } else if(IFTools_Console_matchCommand(argv[pos], "OFF", 1)) {
             m_command = C_TRACE_OFF;
             return IFR_OK;
+        } else if(IFTools_Console_matchCommand(argv[pos],"STOP", 2)) {
+            m_command = C_TRACE_STOP_ON_ERROR;
         } else {
-            SYNTAX_ERROR("one of SHORT,LONG,PROFILE,PACKET,TIMESTAMP,SIZE,REFRESH,OFF,SQL");
+            SYNTAX_ERROR("one of SHORT,LONG,PROFILE,PACKET,TIMESTAMP,SIZE,REFRESH,OFF,SQL,STOP");
         }
 
         if(m_command == C_TRACE_SHORT
-           || m_command == C_TRACE_LONG 
+           || m_command == C_TRACE_LONG
            || m_command == C_TRACE_PROFILE
            || m_command == C_TRACE_TIMESTAMP
            || m_command == C_TRACE_SQL) {
@@ -484,7 +578,7 @@ IFTools_Console::parseArguments(int argc, char **argv)
         } else if (m_command == C_TRACE_PACKET) {
             ++pos;
             CHECK_END("one of ON, OFF, <size>");
-            
+
             if(IFTools_Console_matchCommand(argv[pos], "ON", 2)) {
                 m_on   = true;
                 m_size = 1000;
@@ -501,21 +595,64 @@ IFTools_Console::parseArguments(int argc, char **argv)
         } else if (m_command == C_TRACE_SIZE) {
             ++pos;
             CHECK_END("one of UNLIMITED, <size>");
-            
+
             if(IFTools_Console_matchCommand(argv[pos], "UNLIMITED", 1)) {
                 m_size = -1;
             } else {
                 m_size = atoi(argv[pos]);
                 if(m_size < 8192) {
-                    stderrstream() << "Error: Syntax error, found " 
-                                   << argv[pos] 
-                                   << ", expected UNLIMITED or a number > 8192." 
+                    stderrstream() << "Error: Syntax error, found "
+                                   << argv[pos]
+                                   << ", expected UNLIMITED or a number > 8192."
                                    << endl;
                     return IFR_NOT_OK;
                 }
             }
-            
-        } 
+
+        } else if(m_command == C_TRACE_STOP_ON_ERROR) {
+            ++pos;
+            CHECK_END("ON");
+            if(!IFTools_Console_matchCommand(argv[pos],"ON", 1)) {
+                SYNTAX_ERROR("ON");
+                }
+            ++pos;
+            CHECK_END("ERROR");
+            if(!IFTools_Console_matchCommand(argv[pos],"ERROR", 1)) {
+                SYNTAX_ERROR("ON");
+            }
+            ++pos;
+            CHECK_END("<errorcode> or OFF");
+
+            if(IFTools_Console_matchCommand(argv[pos], "OFF", 1)) {
+                m_stop_on_error = false;
+            } else {
+                m_stop_on_error = true;
+                m_errorcode = atoi(argv[pos]);
+                if(m_errorcode == 0) {
+                    stderrstream() << "Error: Syntax error, found "
+                                   << argv[pos]
+                                   << ", expected an error code."
+                                   << endl;
+                    return IFR_NOT_OK;
+                }
+                ++pos;
+                if(pos != argc) {
+                    if(!IFTools_Console_matchCommand(argv[pos],"COUNT",1)) {
+                        SYNTAX_ERROR("COUNT");
+                    }
+                    ++pos;
+                    CHECK_END("<errorcount>");
+                    m_errorcount = atoi(argv[pos]);
+                    if(m_errorcount <= 0) {
+                        stderrstream() << "Error: Syntax error, found "
+                                       << argv[pos]
+                                       << ", expected a positive number."
+                                       << endl;
+                        return IFR_NOT_OK;
+                    }
+                }
+            }
+        }
     } else if(IFTools_Console_matchCommand(argv[pos], "SHOW", 1)) {
         ++pos;
         CHECK_END("one of CONFIG,TRACESETTINGS,ALL");
@@ -570,6 +707,9 @@ IFTools_Console::usage(IFR_TraceStream &usedstream)
         << "  [CONFIG] TRACE SIZE <size>"                                        << endl
         << "    Limits the size of the trace file to <size> bytes, at least"     << endl
         << "    8192 bytes are required."                                        << endl
+        << "  [CONFIG] TRACE STOP ON ERROR <error>|OFF [COUNT <number>]"         << endl
+        << "    Stops tracing after the error <error> has happened <number>"     << endl
+        << "    times (default is 1), or switches the trace stop feature off."   << endl
         << "  CONFIG TRACE FILENAME <filename>"                                  << endl
         << "    Sets the name of the trace file name. A '%p' in the name is"     << endl
         << "    replaced by the process id."                                     << endl
@@ -626,7 +766,14 @@ IFTools_Console::showConfig()
             if(flags.m_filesize == -1) {
                 stdoutstream() << "    Trace file size    : not limited" << endl;
             } else {
-                stdoutstream() << "    Trace file size    : limited to " << flags.m_filesize << " bytes"<< endl;            
+                stdoutstream() << "    Trace file size    : limited to " << flags.m_filesize << " bytes"<< endl;
+            }
+            if(flags.m_stop_on_error) {
+                IFR_Int4 errorcount = flags.m_errorcount>0?flags.m_errorcount:1;
+                stdoutstream() << "    Stop on error      : " << flags.m_errorcode << " after " <<
+                    errorcount << ((errorcount>1)?" occurrences":" occurence") << endl;
+            } else {
+                stdoutstream() << "    Stop on error      : disabled." << endl;
             }
         }
     } else {
@@ -651,7 +798,7 @@ IFTools_Console::showConfig()
 
     if(RTE_GetUserSpecificConfigPath(fileLocation, false, errtext)) {
         stdoutstream() << "  Files will be created in path "
-                       << fileLocation << "," 
+                       << fileLocation << ","
                        << " unless specified as absolute paths."
                        << endl;
     } else {
@@ -671,12 +818,12 @@ IFTools_Console::showTracesettings()
 
     sharedMemory.attach(sharedMemoryError);
     if(sharedMemoryError) {
-        stderrstream() << "Error opening shared memory: " 
-                       << sharedMemoryError.getErrorText() 
+        stderrstream() << "Error opening shared memory: "
+                       << sharedMemoryError.getErrorText()
                        << ", aborting." << endl;
         return IFR_NOT_OK;
     }
-    IFR_Int4 partcount = (sharedMemory.getHeader()->totalSize - 
+    IFR_Int4 partcount = (sharedMemory.getHeader()->totalSize -
                           SHAREDMEM_HEADER_SIZE)/sizeof(IFRUtil_TraceSharedMemory::Part);
     if(sharedMemory.getHeader()->totalSize == 0) {
         partcount=0;
@@ -696,10 +843,10 @@ IFTools_Console::showTracesettings()
         stdoutstream() <<   "  Process  Update count  Flags" << endl;
         stdoutstream() <<   "  ----------------------------------------------------------------------" << endl;
     }
-    
+
     for(IFR_Int4 i=0; i<partcount; ++i) {
         char buffer[1024];
-        sprintf(buffer, "    %5.d         %5.d  %s", 
+        sprintf(buffer, "    %5.d         %5.d  %s",
                 parts[i].processID,
                 parts[i].readCount,
                 parts[i].flags);
@@ -709,7 +856,7 @@ IFTools_Console::showTracesettings()
 }
 
 //----------------------------------------------------------------------
-IFR_Retcode 
+IFR_Retcode
 IFTools_Console::configTraceShort()
 {
     char buffer[1024];
@@ -717,17 +864,17 @@ IFTools_Console::configTraceShort()
     if(rc != IFR_OK) {
         return IFR_NOT_OK;
     }
-    
+
     TraceFlags flags(buffer);
     flags.m_call = m_on;
     flags.toString(buffer);
-    
+
     rc=IFRUtil_Configuration::setTraceFlags(buffer);
     return rc;
 }
 
 //----------------------------------------------------------------------
-IFR_Retcode 
+IFR_Retcode
 IFTools_Console::configTraceLong()
 {
     char buffer[1024];
@@ -735,17 +882,17 @@ IFTools_Console::configTraceLong()
     if(rc != IFR_OK) {
         return IFR_NOT_OK;
     }
-    
+
     TraceFlags flags(buffer);
     flags.m_debug = m_on;
     flags.toString(buffer);
-    
+
     rc=IFRUtil_Configuration::setTraceFlags(buffer);
     return rc;
 }
 
 //----------------------------------------------------------------------
-IFR_Retcode 
+IFR_Retcode
 IFTools_Console::configTraceTimestamp()
 {
     char buffer[1024];
@@ -753,17 +900,17 @@ IFTools_Console::configTraceTimestamp()
     if(rc != IFR_OK) {
         return IFR_NOT_OK;
     }
-    
+
     TraceFlags flags(buffer);
     flags.m_timestamp = m_on;
     flags.toString(buffer);
-    
+
     rc=IFRUtil_Configuration::setTraceFlags(buffer);
     return rc;
 }
 
 //----------------------------------------------------------------------
-IFR_Retcode 
+IFR_Retcode
 IFTools_Console::configTraceProfile()
 {
     char buffer[1024];
@@ -771,11 +918,11 @@ IFTools_Console::configTraceProfile()
     if(rc != IFR_OK) {
         return IFR_NOT_OK;
     }
-    
+
     TraceFlags flags(buffer);
     flags.m_profile = m_on;
     flags.toString(buffer);
-    
+
     rc=IFRUtil_Configuration::setTraceFlags(buffer);
     return rc;
 }
@@ -789,17 +936,17 @@ IFTools_Console::configTraceSQL()
     if(rc != IFR_OK) {
         return IFR_NOT_OK;
     }
-    
+
     TraceFlags flags(buffer);
     flags.m_sql = m_on;
     flags.toString(buffer);
-    
+
     rc=IFRUtil_Configuration::setTraceFlags(buffer);
     return rc;
 }
 
 //----------------------------------------------------------------------
-IFR_Retcode 
+IFR_Retcode
 IFTools_Console::configTracePacket()
 {
     char buffer[1024];
@@ -807,7 +954,7 @@ IFTools_Console::configTracePacket()
     if(rc != IFR_OK) {
         return IFR_NOT_OK;
     }
-    
+
     TraceFlags flags(buffer);
     if(m_on) {
         flags.m_packet     = true;
@@ -817,13 +964,13 @@ IFTools_Console::configTracePacket()
         flags.m_packetsize = m_size;
     }
     flags.toString(buffer);
-    
+
     rc=IFRUtil_Configuration::setTraceFlags(buffer);
     return rc;
 }
 
 //----------------------------------------------------------------------
-IFR_Retcode 
+IFR_Retcode
 IFTools_Console::configTraceSize()
 {
     char buffer[1024];
@@ -831,11 +978,11 @@ IFTools_Console::configTraceSize()
     if(rc != IFR_OK) {
         return IFR_NOT_OK;
     }
-    
+
     TraceFlags flags(buffer);
     flags.m_filesize = m_size;
     flags.toString(buffer);
-    
+
     rc=IFRUtil_Configuration::setTraceFlags(buffer);
     return rc;
 }
@@ -847,6 +994,25 @@ IFTools_Console::configTraceFilename()
     return IFRUtil_Configuration::setTraceFileName(m_filename);
 }
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFTools_Console::configTraceStopOnError()
+{
+    char buffer[1024];
+    IFR_Retcode rc=IFRUtil_Configuration::getTraceFlags(buffer, 1024);
+    if(rc != IFR_OK) {
+        return IFR_NOT_OK;
+    }
+    TraceFlags flags(buffer);
+    flags.m_stop_on_error = m_stop_on_error;
+    flags.m_errorcode     = m_errorcode;
+    flags.m_errorcount    = m_errorcount;
+    flags.toString(buffer);
+    rc=IFRUtil_Configuration::setTraceFlags(buffer);
+    return rc;
+}
+
+
 //----------------------------------------------------------------------
 IFR_Retcode
 IFTools_Console::traceRefresh()
@@ -856,8 +1022,8 @@ IFTools_Console::traceRefresh()
         IFRUtil_TraceSharedMemory sharedMemory;
         sharedMemory.attach(sharedMemoryError);
         if(sharedMemoryError) {
-            stderrstream() << "Error opening shared memory: " 
-                           << sharedMemoryError.getErrorText() 
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
                            << ", aborting." << endl;
             return IFR_NOT_OK;
         }
@@ -868,8 +1034,8 @@ IFTools_Console::traceRefresh()
         IFRUtil_TraceSharedMemory sharedMemory;
         sharedMemory.attach(sharedMemoryError);
         if(sharedMemoryError) {
-            stderrstream() << "Error opening shared memory: " 
-                           << sharedMemoryError.getErrorText() 
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
                            << ", aborting." << endl;
             return IFR_NOT_OK;
         }
@@ -878,6 +1044,56 @@ IFTools_Console::traceRefresh()
     }
 }
 
+//----------------------------------------------------------------------
+IFR_Retcode
+IFTools_Console::traceStopOnError()
+{
+
+    if(m_pidpresent) {
+        IFR_ErrorHndl sharedMemoryError(RTEMem_RteAllocator::Instance());
+        IFRUtil_TraceSharedMemory sharedMemory;
+        sharedMemory.attach(sharedMemoryError);
+        if(sharedMemoryError) {
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
+                           << ", aborting." << endl;
+            return IFR_NOT_OK;
+        }
+
+        IFRUtil_TraceSharedMemory::Part *p     = sharedMemory.getPart(m_pid);
+        char buffer[256];
+
+        if(p == 0) {
+            p = sharedMemory.getPart(m_pid, true);
+            if(IFRUtil_Configuration::getTraceFlags(buffer, 256) != IFR_OK) {
+                stderrstream() << "Error getting configuration default." << endl;
+                return IFR_NOT_OK;
+            }
+        } else {
+            memcpy(buffer, p->flags, 256);
+        }
+
+        TraceFlags tf(buffer);
+
+        tf.m_stop_on_error = m_stop_on_error;
+        tf.m_errorcode = m_errorcode;
+        tf.m_errorcount = m_errorcount;
+
+        tf.toString(buffer);
+        if(!sharedMemory.setFlags(m_pid, buffer)) {
+            stderrstream() << "Error setting trace flags." << endl;
+            return IFR_NOT_OK;
+        }
+        return IFR_OK;
+    } else {
+        IFR_Retcode rc=configTraceStopOnError();
+        if(rc == IFR_OK) {
+            rc = traceRefresh();
+        }
+        return rc;
+    }
+}
+
 
 //----------------------------------------------------------------------
 IFR_Retcode
@@ -888,15 +1104,15 @@ IFTools_Console::traceShort()
         IFRUtil_TraceSharedMemory sharedMemory;
         sharedMemory.attach(sharedMemoryError);
         if(sharedMemoryError) {
-            stderrstream() << "Error opening shared memory: " 
-                           << sharedMemoryError.getErrorText() 
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
                            << ", aborting." << endl;
             return IFR_NOT_OK;
         }
-        
+
         IFRUtil_TraceSharedMemory::Part *p     = sharedMemory.getPart(m_pid);
         char buffer[256];
-        
+
         if(p == 0) {
             p = sharedMemory.getPart(m_pid, true);
             if(IFRUtil_Configuration::getTraceFlags(buffer, 256) != IFR_OK) {
@@ -906,7 +1122,7 @@ IFTools_Console::traceShort()
         } else {
             memcpy(buffer, p->flags, 256);
         }
-        
+
         TraceFlags tf(buffer);
 
         // traceShort
@@ -927,7 +1143,7 @@ IFTools_Console::traceShort()
         }
         return rc;
     }
-    
+
 }
 
 //----------------------------------------------------------------------
@@ -939,15 +1155,15 @@ IFTools_Console::traceLong()
         IFRUtil_TraceSharedMemory sharedMemory;
         sharedMemory.attach(sharedMemoryError);
         if(sharedMemoryError) {
-            stderrstream() << "Error opening shared memory: " 
-                           << sharedMemoryError.getErrorText() 
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
                            << ", aborting." << endl;
             return IFR_NOT_OK;
         }
-        
+
         IFRUtil_TraceSharedMemory::Part *p     = sharedMemory.getPart(m_pid);
         char buffer[256];
-        
+
         if(p == 0) {
             p = sharedMemory.getPart(m_pid, true);
             if(IFRUtil_Configuration::getTraceFlags(buffer, 256) != IFR_OK) {
@@ -957,7 +1173,7 @@ IFTools_Console::traceLong()
         } else {
             memcpy(buffer, p->flags, 256);
         }
-        
+
         TraceFlags tf(buffer);
 
         // traceLong
@@ -972,7 +1188,7 @@ IFTools_Console::traceLong()
         }
         return IFR_OK;
     } else {
-        IFR_Retcode rc=configTraceShort();
+        IFR_Retcode rc=configTraceLong();
         if(rc == IFR_OK) {
             rc = traceRefresh();
         }
@@ -990,15 +1206,15 @@ IFTools_Console::traceProfile()
         IFRUtil_TraceSharedMemory sharedMemory;
         sharedMemory.attach(sharedMemoryError);
         if(sharedMemoryError) {
-            stderrstream() << "Error opening shared memory: " 
-                           << sharedMemoryError.getErrorText() 
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
                            << ", aborting." << endl;
             return IFR_NOT_OK;
         }
-        
+
         IFRUtil_TraceSharedMemory::Part *p     = sharedMemory.getPart(m_pid);
         char buffer[256];
-        
+
         if(p == 0) {
             p = sharedMemory.getPart(m_pid, true);
             if(IFRUtil_Configuration::getTraceFlags(buffer, 256) != IFR_OK) {
@@ -1008,7 +1224,7 @@ IFTools_Console::traceProfile()
         } else {
             memcpy(buffer, p->flags, 256);
         }
-        
+
         TraceFlags tf(buffer);
 
 
@@ -1022,7 +1238,7 @@ IFTools_Console::traceProfile()
         }
         return IFR_OK;
     } else {
-        IFR_Retcode rc=configTraceShort();
+        IFR_Retcode rc=configTraceProfile();
         if(rc == IFR_OK) {
             rc = traceRefresh();
         }
@@ -1039,15 +1255,15 @@ IFTools_Console::traceSQL()
         IFRUtil_TraceSharedMemory sharedMemory;
         sharedMemory.attach(sharedMemoryError);
         if(sharedMemoryError) {
-            stderrstream() << "Error opening shared memory: " 
-                           << sharedMemoryError.getErrorText() 
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
                            << ", aborting." << endl;
             return IFR_NOT_OK;
         }
-        
+
         IFRUtil_TraceSharedMemory::Part *p     = sharedMemory.getPart(m_pid);
         char buffer[256];
-        
+
         if(p == 0) {
             p = sharedMemory.getPart(m_pid, true);
             if(IFRUtil_Configuration::getTraceFlags(buffer, 256) != IFR_OK) {
@@ -1057,7 +1273,7 @@ IFTools_Console::traceSQL()
         } else {
             memcpy(buffer, p->flags, 256);
         }
-        
+
         TraceFlags tf(buffer);
 
 
@@ -1088,15 +1304,15 @@ IFTools_Console::traceTimestamp()
         IFRUtil_TraceSharedMemory sharedMemory;
         sharedMemory.attach(sharedMemoryError);
         if(sharedMemoryError) {
-            stderrstream() << "Error opening shared memory: " 
-                           << sharedMemoryError.getErrorText() 
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
                            << ", aborting." << endl;
             return IFR_NOT_OK;
         }
-        
+
         IFRUtil_TraceSharedMemory::Part *p     = sharedMemory.getPart(m_pid);
         char buffer[256];
-        
+
         if(p == 0) {
             p = sharedMemory.getPart(m_pid, true);
             if(IFRUtil_Configuration::getTraceFlags(buffer, 256) != IFR_OK) {
@@ -1106,7 +1322,7 @@ IFTools_Console::traceTimestamp()
         } else {
             memcpy(buffer, p->flags, 256);
         }
-        
+
         TraceFlags tf(buffer);
 
         // traceShort
@@ -1121,7 +1337,7 @@ IFTools_Console::traceTimestamp()
         }
         return IFR_OK;
     } else {
-        IFR_Retcode rc=configTraceShort();
+        IFR_Retcode rc=configTraceTimestamp();
         if(rc == IFR_OK) {
             rc = traceRefresh();
         }
@@ -1139,15 +1355,15 @@ IFTools_Console::tracePacket()
         IFRUtil_TraceSharedMemory sharedMemory;
         sharedMemory.attach(sharedMemoryError);
         if(sharedMemoryError) {
-            stderrstream() << "Error opening shared memory: " 
-                           << sharedMemoryError.getErrorText() 
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
                            << ", aborting." << endl;
             return IFR_NOT_OK;
         }
-        
+
         IFRUtil_TraceSharedMemory::Part *p     = sharedMemory.getPart(m_pid);
         char buffer[256];
-        
+
         if(p == 0) {
             p = sharedMemory.getPart(m_pid, true);
             if(IFRUtil_Configuration::getTraceFlags(buffer, 256) != IFR_OK) {
@@ -1157,7 +1373,7 @@ IFTools_Console::tracePacket()
         } else {
             memcpy(buffer, p->flags, 256);
         }
-        
+
         TraceFlags tf(buffer);
 
         if(m_on) {
@@ -1177,7 +1393,7 @@ IFTools_Console::tracePacket()
         }
         return IFR_OK;
     } else {
-        IFR_Retcode rc=configTraceShort();
+        IFR_Retcode rc=configTracePacket();
         if(rc == IFR_OK) {
             rc = traceRefresh();
         }
@@ -1195,15 +1411,15 @@ IFTools_Console::traceSize()
         IFRUtil_TraceSharedMemory sharedMemory;
         sharedMemory.attach(sharedMemoryError);
         if(sharedMemoryError) {
-            stderrstream() << "Error opening shared memory: " 
-                           << sharedMemoryError.getErrorText() 
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
                            << ", aborting." << endl;
             return IFR_NOT_OK;
         }
-        
+
         IFRUtil_TraceSharedMemory::Part *p     = sharedMemory.getPart(m_pid);
         char buffer[256];
-        
+
         if(p == 0) {
             p = sharedMemory.getPart(m_pid, true);
             if(IFRUtil_Configuration::getTraceFlags(buffer, 256) != IFR_OK) {
@@ -1213,7 +1429,7 @@ IFTools_Console::traceSize()
         } else {
             memcpy(buffer, p->flags, 256);
         }
-        
+
         TraceFlags tf(buffer);
 
         tf.m_filesize = m_size;
@@ -1226,7 +1442,7 @@ IFTools_Console::traceSize()
         }
         return IFR_OK;
     } else {
-        IFR_Retcode rc=configTraceShort();
+        IFR_Retcode rc=configTraceSize();
         if(rc == IFR_OK) {
             rc = traceRefresh();
         }
@@ -1244,16 +1460,16 @@ IFTools_Console::traceOff()
         IFRUtil_TraceSharedMemory sharedMemory;
         sharedMemory.attach(sharedMemoryError);
         if(sharedMemoryError) {
-            stderrstream() << "Error opening shared memory: " 
-                           << sharedMemoryError.getErrorText() 
+            stderrstream() << "Error opening shared memory: "
+                           << sharedMemoryError.getErrorText()
                            << ", aborting." << endl;
             return IFR_NOT_OK;
         }
-        
+
         IFRUtil_TraceSharedMemory::Part *p     = sharedMemory.getPart(m_pid);
         if(p == 0) {
             p = sharedMemory.getPart(m_pid, true);
-        } 
+        }
         if(!sharedMemory.setFlags(m_pid, (char*)"")) {
             stderrstream() << "Error setting trace flags." << endl;
             return IFR_NOT_OK;
@@ -1310,6 +1526,8 @@ IFTools_Console::run()
         return configTraceFilename();
     case C_CONFIG_TRACE_SQL:
         return configTraceSQL();
+    case C_CONFIG_TRACE_STOP_ON_ERROR:
+        return configTraceStopOnError();
     case C_TRACE_REFRESH:
         return traceRefresh();
     case C_TRACE_SHORT:
@@ -1326,6 +1544,8 @@ IFTools_Console::run()
         return tracePacket();
     case C_TRACE_SQL:
         return traceSQL();
+    case C_TRACE_STOP_ON_ERROR:
+        return traceStopOnError();
     case C_TRACE_OFF:
         return traceOff();
     }
