@@ -88,9 +88,11 @@ IFTools_sqlcliInput::cmdKeywordTable IFTools_sqlcliInput::mycmdKeywordTab [ ]= {
   { cmdtype_connect,    {{"\\C"         ,0},{"",0},{"",0}} },
   { cmdtype_connect,    {{"\\CONNECT"   ,0},{"",0},{"",0}} },
   { cmdtype_showColumns,{{"\\DC"        ,0},{"",0},{"",0}} },
+  { cmdtype_showIndices,{{"\\DE"        ,0},{"",0},{"",0}} },
   { cmdtype_disconnect, {{"\\DI"        ,0},{"",0},{"",0}} },
   { cmdtype_disconnect, {{"\\DISCONNECT",0},{"",0},{"",0}} },
   { cmdtype_showProcs,  {{"\\DP"        ,0},{"",0},{"",0}} },
+  { cmdtype_showSchemas,{{"\\DS"        ,0},{"",0},{"",0}} },
   { cmdtype_showTables, {{"\\DT"        ,0},{"",0},{"",0}} },
   { cmdtype_showUsers,  {{"\\DU"        ,0},{"",0},{"",0}} },
   { cmdtype_showViews,  {{"\\DV"        ,0},{"",0},{"",0}} },
@@ -107,7 +109,9 @@ IFTools_sqlcliInput::cmdKeywordTable IFTools_sqlcliInput::mycmdKeywordTab [ ]= {
   { cmdtype_outfile,    {{"\\O"         ,0},{"",0},{"",0}} },
   { cmdtype_outfile,    {{"\\OUTPUT"    ,0},{"",0},{"",0}} },
   { cmdtype_print,      {{"\\P"         ,0},{"",0},{"",0}} },
+  { cmdtype_usePreparedStmt, {{"\\PREPARED"  ,0},{"",0},{"",0}} },
   { cmdtype_print,      {{"\\PRINT"     ,0},{"",0},{"",0}} },
+  { cmdtype_usePreparedStmt, {{"\\PS"        ,0},{"",0},{"",0}} },
   { cmdtype_exit,       {{"\\Q"         ,0},{"",0},{"",0}} },
   { cmdtype_exit,       {{"\\QUIT"      ,0},{"",0},{"",0}} },
   { cmdtype_reset,      {{"\\R"         ,0},{"",0},{"",0}} },
@@ -128,6 +132,7 @@ const char * IFTools_sqlcliInput::getshortcutHelp(){
     "\n"
     " \\a[utocommit]  [ON|OFF] switch autocommit mode on or off\n"
     " \\m[ode]        [INTERNAL|ORACLE|ANSI|DB2] change SQL mode\n"
+    " \\ps            [ON|OFF] toggle the usage of prepared statements on or off\n"
     " \\o[utput]      <filename> send all query results to file\n"
     " \\i[nput]       <filename> read input from the file <filename>\n"
     "\n"
@@ -142,7 +147,9 @@ const char * IFTools_sqlcliInput::getshortcutHelp(){
     " \\!<command>    execute <command> in shell\n"
     "\n"
     " \\dc [PATTERN]  list columns\n"
+    " \\de [PATTERN]  list indices\n"
     " \\dp [PATTERN]  list procedures\n"
+    " \\ds [NAME]     list schemas\n"
     " \\dt [PATTERN]  list tables\n"
     " \\du [NAME]     list users\n"
     " \\dv [PATTERN]  list views\n"
@@ -212,7 +219,7 @@ public:
       }
       
       int printPrompt(){
-        if (m_sqlcliProp->isInteractive())
+        if (m_sqlcliProp->isReadingFromStdin())
           if (m_multiLinePrompt)
             return fprintf (stdout,"> ");
           else{
@@ -284,7 +291,7 @@ public:
           m_alreadyExecuted= SQLDBC_TRUE;
           IFTools_sqlcliInput::printSplashScreen(this->m_sqlcliProp->getoutfile());
           if (   !m_sqlcliProp->isConnected() 
-              && m_sqlcliProp->isInteractive()
+              && m_sqlcliProp->isReadingFromStdin()
               && !strlen(m_sqlcliProp->getpassword())
               && strlen(m_sqlcliProp->getusername())
               && strlen(m_sqlcliProp->getdbname())){
@@ -484,9 +491,12 @@ IFTools_sqlcliInput* IFTools_sqlcliInput::getNewInputHandler(
         this->m_sqlcliProp->getifrprops().setProperty("SQLMODE", IFTools_sqlcliProperties::trimString(&pIdentifier[currPos]));
         break;
       } 
+      case(IFTools_sqlcliInput::cmdtype_showUsers):
+      case(IFTools_sqlcliInput::cmdtype_showSchemas):
       case(IFTools_sqlcliInput::cmdtype_showColumns):
       case(IFTools_sqlcliInput::cmdtype_showProcs):
       case(IFTools_sqlcliInput::cmdtype_showTables):
+      case(IFTools_sqlcliInput::cmdtype_showIndices):
       case(IFTools_sqlcliInput::cmdtype_showViews): {
         currPos+=symbolLen+1;
         symbolLen = this->nextSymbol(&pIdentifier[currPos], &currPos);
@@ -578,6 +588,17 @@ IFTools_sqlcliInput* IFTools_sqlcliInput::getNewInputHandler(
         }
         break;
       } 
+     case(IFTools_sqlcliInput::cmdtype_usePreparedStmt):{
+        currPos+=symbolLen+1;
+        symbolLen = this->nextSymbol(&pIdentifier[currPos], &currPos);
+        if (! symbolLen){
+			m_sqlcliProp->setUsePreparedStmt();
+        } else {
+          sqlcliInput_autocommittype actyp = (IFTools_sqlcliInput::sqlcliInput_autocommittype)this->analyzeKeyword(&pIdentifier[currPos],symbolLen, autocommitKeywordTab, AUTOCOMMITKEYWORDTABLESIZE);        
+          m_sqlcliProp->setUsePreparedStmt((actyp == IFTools_sqlcliInput::autocommit_on)?SQLDBC_TRUE:SQLDBC_FALSE);
+        }
+        break;
+      } 
       case(IFTools_sqlcliInput::cmdtype_status):{
         currPos+=symbolLen+1;
         symbolLen = this->nextSymbol(&pIdentifier[currPos], &currPos);
