@@ -50,15 +50,14 @@ public:
 
       ~IFTools_sqlcliOutputDefPrinter   (){}
 
-      virtual void printColName(IFTools_sqlcliOutput *out){
+      virtual SQLDBC_Bool printColName(IFTools_sqlcliOutput *out){
         if (m_colLen){
-          out->printResult("%-*.*s",m_colLen,(m_colLen>17)?17:m_colLen,
-            m_colname);
+          return out->printResult("%-*.*s",m_colLen,(m_colLen>17)?17:m_colLen, m_colname);
         }
+        return SQLDBC_OK;
       }	
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
-        out->printResult("%-*.*s", m_colLen, (bufLen > m_colLen)?m_colLen:bufLen, buffer);        
-        return SQLDBC_TRUE;
+        return out->printResult("%-*.*s", m_colLen, (bufLen > m_colLen)?m_colLen:bufLen, buffer);        
       }
 }; 
 //----------------------------------------------------------------------------------
@@ -74,8 +73,7 @@ public:
       ~IFTools_sqlcliOutputFixPointPrinter   (){}
 
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
-        out->printResult("%*.*s", m_colLen, (bufLen > m_colLen)?m_colLen:bufLen, buffer);        
-        return SQLDBC_TRUE;
+        return out->printResult("%*.*s", m_colLen, (bufLen > m_colLen)?m_colLen:bufLen, buffer);        
       }
 
 private:
@@ -97,8 +95,7 @@ public:
       ~IFTools_sqlcliOutputFloatPrinter   (){}
 
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
-        out->printResult("%*.*s", m_colLen, (bufLen > m_colLen)?m_colLen:bufLen, buffer);        
-        return SQLDBC_TRUE;
+        return out->printResult("%*.*s", m_colLen, (bufLen > m_colLen)?m_colLen:bufLen, buffer);        
       }
 
 };
@@ -114,7 +111,7 @@ public:
 
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
         SQLDBC_Int4  maxOutLen = ((m_colLen-2) < bufLen*2)?(m_colLen/2-1):bufLen; 
-        out->printResult("0x");
+        if (out->printResult("0x")== SQLDBC_FALSE) return SQLDBC_FALSE;
         return printBinaryData(out, buffer, maxOutLen);
       }
 
@@ -123,7 +120,7 @@ protected:
         unsigned char *ptr;
         SQLDBC_Int4 i;
         for (ptr=(unsigned char *)buffer, i=0; i<bufLen; i++,ptr++)
-          out->printResult("%02X",*ptr);
+          if (out->printResult("%02X",*ptr)==SQLDBC_FALSE) return SQLDBC_FALSE;
         return SQLDBC_TRUE;
       };
 };
@@ -137,15 +134,14 @@ public:
       {}
       ~IFTools_sqlcliOutputUnicodePrinter   (){}
       
-      virtual void printColName(IFTools_sqlcliOutput *out){
+      virtual SQLDBC_Bool printColName(IFTools_sqlcliOutput *out){
         if (m_colLen){
-          out->printResult("%-*.*s",m_colLen,(m_colLen>17)?17:m_colLen,
-            m_colname);
+          return out->printResult("%-*.*s",m_colLen,(m_colLen>17)?17:m_colLen, m_colname);
         }
+        return SQLDBC_TRUE;
       }	
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
-        out->printResult("%-*.*s", m_colLen, (bufLen > m_colLen)?m_colLen:bufLen, buffer);        
-        return SQLDBC_TRUE;
+        return out->printResult("%-*.*s", m_colLen, (bufLen > m_colLen)?m_colLen:bufLen, buffer);        
       }
 
 };
@@ -161,8 +157,7 @@ public:
 
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
         SQLDBC_Int1* erg = (SQLDBC_Int1*)buffer;
-        out->printResult("% 5s", (*erg)?"TRUE":"FALSE");
-        return SQLDBC_TRUE;
+        return out->printResult("% 5s", (*erg)?"TRUE":"FALSE");
       };
 };
 
@@ -207,7 +202,7 @@ public:
       ~IFTools_sqlcliOutputWholeLongBinaryPrinter   (){}
 
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
-        out->printResult("0x");
+        if (out->printResult("0x")==SQLDBC_FALSE) return SQLDBC_FALSE;
         return printBinaryData(out, buffer, bufLen);
       }       
       
@@ -224,8 +219,7 @@ public:
       ~IFTools_sqlcliOutputWholeLongAsciiPrinter   (){}
 
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
-        out->printResult("%.*s", bufLen, buffer);
-        return SQLDBC_TRUE;
+        return out->printResult("%.*s", bufLen, buffer);
       }       
 
       virtual SQLDBC_Bool printNextChunk(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
@@ -241,8 +235,7 @@ public:
       ~IFTools_sqlcliOutputWholeLongUniPrinter   (){}
       
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
-        out->printResult("%.*s", bufLen, buffer);
-        return SQLDBC_TRUE;
+        return out->printResult("%.*s", bufLen, buffer);
       }       
 
       virtual SQLDBC_Bool printNextChunk(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
@@ -258,54 +251,56 @@ public:
 
       ~IFTools_sqlcliOutputUnAlignedDefPrinter   (){}
 
-      virtual void printColName(IFTools_sqlcliOutput *out){
+      virtual SQLDBC_Bool printColName(IFTools_sqlcliOutput *out){
         if (m_colLen){
-          out->printResult("%s", m_colname);
+          return out->printResult("%s", m_colname);
         }
+        return SQLDBC_FALSE;
       }	
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
         SQLDBC_Length i, curLen=0, curPos=0;
-        out->printResult("\"");
+        if (out->printResult("\"")==SQLDBC_FALSE) return SQLDBC_FALSE;
         for (i=0; i< bufLen; i++ ){
           switch(buffer[i]) {
             case('\"'):{
-              this->printRest(out, buffer, &curPos, &curLen);
-              out->printResult("\\\""), curPos+=1;
+              if (this->printRest(out, buffer, &curPos, &curLen)==SQLDBC_FALSE) return SQLDBC_FALSE;
+              if (out->printResult("\\\"")==SQLDBC_FALSE) return SQLDBC_FALSE; 
+              curPos+=1;
               break;
             }
             case('\n'):{
-              this->printRest(out, buffer, &curPos, &curLen);
-              out->printResult("\\n"), curPos+=1;
+              if (this->printRest(out, buffer, &curPos, &curLen)==SQLDBC_FALSE) return SQLDBC_FALSE;
+              if (out->printResult("\\n")==SQLDBC_FALSE) return SQLDBC_FALSE; curPos+=1;
               break;
             }
             case('\b'):{
-              this->printRest(out, buffer, &curPos, &curLen);
-              out->printResult("\\b"), curPos+=1;
+              if (this->printRest(out, buffer, &curPos, &curLen)==SQLDBC_FALSE) return SQLDBC_FALSE;
+              if (out->printResult("\\b")==SQLDBC_FALSE) return SQLDBC_FALSE; curPos+=1;
               break;
             }
             case('\r'):{
-              this->printRest(out, buffer, &curPos, &curLen);
-              out->printResult("\\r"), curPos+=1;
+              if (this->printRest(out, buffer, &curPos, &curLen)==SQLDBC_FALSE) return SQLDBC_FALSE;
+              if (out->printResult("\\r")==SQLDBC_FALSE) return SQLDBC_FALSE; curPos+=1;
               break;
             }
             case('\a'):{
-              this->printRest(out, buffer, &curPos, &curLen);
-              out->printResult("\\a"), curPos+=1;
+              if (this->printRest(out, buffer, &curPos, &curLen)==SQLDBC_FALSE) return SQLDBC_FALSE;
+              if (out->printResult("\\a")==SQLDBC_FALSE) return SQLDBC_FALSE; curPos+=1;
               break;
             }
             case('\f'):{
-              this->printRest(out, buffer, &curPos, &curLen);
-              out->printResult("\\f"), curPos+=1;
+              if (this->printRest(out, buffer, &curPos, &curLen)==SQLDBC_FALSE) return SQLDBC_FALSE;
+              if (out->printResult("\\f")==SQLDBC_FALSE) return SQLDBC_FALSE; curPos+=1;
               break;
             }
             case('\t'):{
-              this->printRest(out, buffer, &curPos, &curLen);
-              out->printResult("\\t"), curPos+=2;
+              if (this->printRest(out, buffer, &curPos, &curLen)==SQLDBC_FALSE) return SQLDBC_FALSE;
+              if (out->printResult("\\t")==SQLDBC_FALSE) return SQLDBC_FALSE; curPos+=2;
               break;
             }
             case('\\'):{
-              this->printRest(out, buffer, &curPos, &curLen);
-              out->printResult("\\\\"), curPos+=1;
+              if (this->printRest(out, buffer, &curPos, &curLen)==SQLDBC_FALSE) return SQLDBC_FALSE;
+              if (out->printResult("\\\\")==SQLDBC_FALSE) return SQLDBC_FALSE; curPos+=1;
               break;
             }
             default:{
@@ -314,17 +309,18 @@ public:
             }
           }
         }        
-        this->printRest(out, buffer, &curPos, &curLen);
-        out->printResult("\"");
+        if (this->printRest(out, buffer, &curPos, &curLen)==SQLDBC_FALSE) return SQLDBC_FALSE;
+        if (out->printResult("\"")==SQLDBC_FALSE) return SQLDBC_FALSE;
         return SQLDBC_TRUE;
       }
    private:
-     void printRest(IFTools_sqlcliOutput *out, char* buf, SQLDBC_Length* pos, SQLDBC_Length* len ){
+     SQLDBC_Bool printRest(IFTools_sqlcliOutput *out, char* buf, SQLDBC_Length* pos, SQLDBC_Length* len ){
        if (*len) {
-        out->printResult("%.*s", *len, &buf[(*pos)]);
+        if (out->printResult("%.*s", *len, &buf[(*pos)])==SQLDBC_FALSE) return SQLDBC_FALSE;
         *pos += *len;
         *len=0;
-       }  
+       }
+       return SQLDBC_TRUE;  
      }    
 }; 
 //----------------------------------------------------------------------------------
@@ -337,12 +333,11 @@ public:
 
       ~IFTools_sqlcliOutputUnalignedFixPointPrinter   (){}
 
-      virtual void printColName(IFTools_sqlcliOutput *out){
-        out->printResult("%s", m_colname);
+      virtual SQLDBC_Bool printColName(IFTools_sqlcliOutput *out){
+        return out->printResult("%s", m_colname);
       }	
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
-        out->printResult("%s",buffer);        
-        return SQLDBC_TRUE;
+        return out->printResult("%s",buffer);        
       }
 };
 
@@ -355,12 +350,11 @@ public:
       {}
       ~IFTools_sqlcliOutputUnalignedFloatPrinter   (){}
 
-      virtual void printColName(IFTools_sqlcliOutput *out){
-        out->printResult("%s", m_colname);
+      virtual SQLDBC_Bool printColName(IFTools_sqlcliOutput *out){
+        return out->printResult("%s", m_colname);
       }	
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
-        out->printResult("%s", buffer);        
-        return SQLDBC_TRUE;
+        return out->printResult("%s", buffer);        
       }
 
 };
@@ -376,9 +370,9 @@ public:
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
         unsigned char *ptr;
         SQLDBC_Int4 i, maxOutLen = ((m_colLen-2) < bufLen*2)?(m_colLen/2-1):bufLen; 
-        out->printResult("0x");
+        if (out->printResult("0x")==SQLDBC_FALSE) return SQLDBC_FALSE;
         for (ptr=(unsigned char *)buffer, i=0; i<maxOutLen; i++,ptr++)
-          out->printResult("%02X",*ptr);
+          if (out->printResult("%02X",*ptr)==SQLDBC_FALSE) return SQLDBC_FALSE;
         return SQLDBC_TRUE;
       };
 };
@@ -394,8 +388,7 @@ public:
 
       virtual SQLDBC_Bool printValue(IFTools_sqlcliOutput *out, char* buffer, SQLDBC_Int4 bufLen){
         SQLDBC_Int1* erg = (SQLDBC_Int1*)buffer;
-        out->printResult("%s", (*erg)?"TRUE":"FALSE");
-        return SQLDBC_TRUE;
+        return out->printResult("%s", (*erg)?"TRUE":"FALSE");
       };
 };
 
@@ -479,6 +472,7 @@ void IFTools_sqlcliOutput::printError(SQLDBC_ErrorHndl error)
   else
     fprintf (m_sqlcliEnv->geterrfile(),"* %d: %s\n", error.getErrorCode(),
     error.getErrorText());
+    this->m_sqlcliEnv->forceProgramExit();
 }
 
 //----------------------------------------------------------------------
@@ -489,6 +483,7 @@ void IFTools_sqlcliOutput::printError(const char *fmt, ...)
   va_start(ap, fmt);
   vfprintf(m_sqlcliEnv->geterrfile(), fmt, ap);
   va_end(ap); 
+  this->m_sqlcliEnv->forceProgramExit();
 }
 
 //----------------------------------------------------------------------
@@ -511,11 +506,20 @@ void IFTools_sqlcliOutput::printComment(const char *fmt, ...){
 }
 
 //----------------------------------------------------------------------
-void IFTools_sqlcliOutput::printResult(const char *fmt, ...){
+SQLDBC_Bool IFTools_sqlcliOutput::printResult(const char *fmt, ...){
   va_list ap;
   va_start(ap, fmt);
+  
+  errno=0;
+  
   vfprintf(m_sqlcliEnv->getoutfile(), fmt, ap);
-  va_end(ap); 
+  
+  if(errno){
+    return SQLDBC_FALSE;
+  }
+  
+  va_end(ap);
+  return SQLDBC_TRUE; 
 }
 
 //----------------------------------------------------------------------
@@ -665,44 +669,206 @@ SQLDBC_Bool IFTools_sqlcliOutput::registerResultSet(SQLDBC_ResultSetMetaData* rs
   return SQLDBC_TRUE;
 }        
 
-void IFTools_sqlcliOutput::printColumnNames(){
-  if (this->m_sqlcliEnv->getnoHeader()) return;
+//----------------------------------------------------------------------
+SQLDBC_Bool IFTools_sqlcliOutput::registerOutputValues(SQLDBC_ParameterMetaData* rsmd){
+
+  SQLDBC_Int4 length,frac,maxColLength=0; 
+  SQLDBC_Length bufferlength;
+  SQLDBC_Bool aligned = m_sqlcliEnv->getalignedOutput();
+  m_printerLen = rsmd->getParameterCount();
+  m_printer = new IFTools_sqlcliOutputPrinter* [m_printerLen];
+
+  for(SQLDBC_Int2 i = 1; i <= m_printerLen; i++)
+  {
+	if (rsmd->getParameterMode(i) == SQLDBC_ParameterMetaData::parameterModeIn) {
+	  continue;
+	}
+    char *colName = new char[m_sqlcliEnv->maxKnlIdentifierUTF8];
+
+    if (//rsmd->getColumnName (i, colName, SQLDBC_StringEncodingUTF8, m_sqlcliEnv->maxKnlIdentifierUTF8)
+	  rsmd->getParameterName(i, colName,
+      SQLDBC_StringEncodingUTF8,
+      m_sqlcliEnv->maxKnlIdentifierUTF8,
+      &bufferlength)
+
+      ) {
+        this->printError (ERR_GET_COLUMNNAME, i);
+        this->deletePrinter(SQLDBC_TRUE);                  
+        return SQLDBC_FALSE;                  
+      }
+      frac    = rsmd->getScale(i); 
+      length  = strlen (colName); 
+	  if (rsmd->getParameterLength(i)>length)length = rsmd->getParameterLength(i);
+
+	  switch(rsmd->getParameterType (i))
+      {
+      case SQLDBC_SQLTYPE_FIXED         :
+      case SQLDBC_SQLTYPE_NUMBER        :
+      case SQLDBC_SQLTYPE_SMALLINT      :
+      case SQLDBC_SQLTYPE_INTEGER       : {
+        if (aligned)
+          m_printer[i-1] = new   IFTools_sqlcliOutputFixPointPrinter(colName, length, frac);
+        else
+          m_printer[i-1] = new   IFTools_sqlcliOutputUnalignedFixPointPrinter(colName, length, frac);
+        break;
+                                          }
+
+      case SQLDBC_SQLTYPE_FLOAT         :
+      case SQLDBC_SQLTYPE_VFLOAT        :{
+        m_printer[i-1] = (aligned)
+        ?
+        new   IFTools_sqlcliOutputFloatPrinter(colName, length, frac)
+        :
+        new   IFTools_sqlcliOutputUnalignedFloatPrinter(colName, length, frac); 
+        break;                         
+                                         }
+      case SQLDBC_SQLTYPE_CHB           :
+      case SQLDBC_SQLTYPE_ROWID         :
+      case SQLDBC_SQLTYPE_VARCHARB      :{
+        if (aligned)
+          m_printer[i-1] = new   IFTools_sqlcliOutputBinaryPrinter(colName, length, frac);
+        else
+          m_printer[i-1] = new   IFTools_sqlcliOutputUnalignedBinaryPrinter(colName, length, frac);
+        break;                         
+                                         }
+      case SQLDBC_SQLTYPE_CHA           :               
+      case SQLDBC_SQLTYPE_CHE           :
+      case SQLDBC_SQLTYPE_VARCHARA      :
+      case SQLDBC_SQLTYPE_VARCHARE      :
+
+      case SQLDBC_SQLTYPE_DATE          :
+      case SQLDBC_SQLTYPE_TIME          :
+      case SQLDBC_SQLTYPE_TIMESTAMP     :{
+        if (aligned)
+          m_printer[i-1] = new   IFTools_sqlcliOutputDefPrinter(colName, length, frac, SQLDBC_HOSTTYPE_ASCII);
+        else
+          m_printer[i-1] = new   IFTools_sqlcliOutputUnAlignedDefPrinter(colName, length, frac, SQLDBC_HOSTTYPE_ASCII);
+        break;                         
+                                         }
+      case SQLDBC_SQLTYPE_UNICODE       :
+      case SQLDBC_SQLTYPE_VARCHARUNI    :{
+        if (aligned) 
+          m_printer[i-1] = new   IFTools_sqlcliOutputUnicodePrinter(colName,length,frac);
+        else
+          m_printer[i-1] = new   IFTools_sqlcliOutputUnalignedUniPrinter(colName,length,frac);
+        break;                         
+                                         }
+      case SQLDBC_SQLTYPE_BOOLEAN       :{
+        if (aligned)
+          m_printer[i-1] = new   IFTools_sqlcliOutputBooleanPrinter(colName, length, frac);
+        else
+          m_printer[i-1] = new   IFTools_sqlcliOutputUnalignedBooleanPrinter(colName, length, frac);
+        break;                         
+                                         }
+      case SQLDBC_SQLTYPE_ABAPTABHANDLE :{
+        break;
+                                         }
+
+      case SQLDBC_SQLTYPE_STRB          :
+      case SQLDBC_SQLTYPE_LONGB         :{
+        length = m_sqlcliEnv->getmaxLongLenth();   
+        if (length == IFTOOLS_PROP_WHOLE_LONG){
+          length = IFTOOLS_OUT_LONG_BUF;
+          m_printer[i-1] = new   IFTools_sqlcliOutputWholeLongBinaryPrinter(colName, frac);
+        } else if (aligned)
+          m_printer[i-1] = new   IFTools_sqlcliOutputLongBinaryPrinter(colName, length, frac);
+        else
+          m_printer[i-1] = new   IFTools_sqlcliOutputUnalignedLongBinaryPrinter(colName, length, frac);
+        break;                         
+                                         }
+
+      case SQLDBC_SQLTYPE_STRA          :
+      case SQLDBC_SQLTYPE_STRE          :
+      case SQLDBC_SQLTYPE_LONGA         :
+      case SQLDBC_SQLTYPE_LONGE         :{
+        length = m_sqlcliEnv->getmaxLongLenth();   
+        if (length == IFTOOLS_PROP_WHOLE_LONG){
+          length = IFTOOLS_OUT_LONG_BUF;
+          m_printer[i-1] = new   IFTools_sqlcliOutputWholeLongAsciiPrinter(colName, frac);
+        } else if (aligned)
+          m_printer[i-1] = new   IFTools_sqlcliOutputLongAsciiPrinter(colName, length, frac);
+        else  
+          m_printer[i-1] = new   IFTools_sqlcliOutputUnalignedLongAsciiPrinter(colName, length, frac);
+        break;                         
+                                         }
+      case SQLDBC_SQLTYPE_STRUNI        :
+      case SQLDBC_SQLTYPE_LONGUNI       :{
+        length = m_sqlcliEnv->getmaxLongLenth();   
+        if (length == IFTOOLS_PROP_WHOLE_LONG){
+          length = IFTOOLS_OUT_LONG_BUF;
+          m_printer[i-1] = new   IFTools_sqlcliOutputWholeLongUniPrinter(colName, frac);
+        } else if (aligned)
+          m_printer[i-1] = new   IFTools_sqlcliOutputLongUniPrinter(colName, length, frac);
+        else
+          m_printer[i-1] = new   IFTools_sqlcliOutputUnalignedLongUniPrinter(colName, length, frac);
+        break;                         
+                                         }  
+      default: {// unknown type - should not occur 
+		  this->printError (ERR_UNKNOWN_COLTYPE, rsmd->getParameterType (i), i);
+        this->deletePrinter(SQLDBC_TRUE);                  
+        return SQLDBC_FALSE;                  
+               }
+      } /*end switch*/ 
+      if (maxColLength<length) maxColLength = length;
+  } /*end for*/
+  if ((++maxColLength) > m_bufLen){
+    delete m_buffer;        
+    m_buffer = new char[maxColLength];
+    m_bufLen = maxColLength; 
+  }
+  return SQLDBC_TRUE;
+}
+//----------------------------------------------------------------------
+SQLDBC_Bool IFTools_sqlcliOutput::printColumnNames(){
+  if (this->m_sqlcliEnv->getnoHeader()) return SQLDBC_TRUE;
   SQLDBC_Int4 length, rowlength=0;
-  this->printResult("%s",this->m_sqlcliEnv->getrowprefix()); 
+  if (this->printResult("%s",this->m_sqlcliEnv->getrowprefix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
   for(SQLDBC_Int2 i = 0; i < m_printerLen; i++)
   {
     if (0 != (length = m_printer[i]->getColLength()))
     {
-      this->printResult("%s",this->m_sqlcliEnv->getcolumnprefix()); 
-      this->printResult("%-*.*s",length,(length>18)?18:length, m_printer[i]->getColName());
-      this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix()); 
+      if (this->printResult("%s",this->m_sqlcliEnv->getcolumnprefix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+      if (this->printResult("%-*.*s",length,(length>18)?18:length, m_printer[i]->getColName())==SQLDBC_FALSE) return SQLDBC_FALSE;
+      if (this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
     }
   }
-  this->printResult("%s",this->m_sqlcliEnv->getrowsuffix()); 
+  if (this->printResult("%s",this->m_sqlcliEnv->getrowsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
 
   if (this->m_sqlcliEnv->getalignedOutput()){
-    this->printResult("%s",this->m_sqlcliEnv->getrowprefix()); 
+    if (this->printResult("%s",this->m_sqlcliEnv->getrowprefix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
     for(SQLDBC_Int2 m = 0; m < m_printerLen; m++)
     {
-      this->printResult("%s",this->m_sqlcliEnv->getcolumnprefix()); 
+      if (this->printResult("%s",this->m_sqlcliEnv->getcolumnprefix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
       for (SQLDBC_Int2 l=0;l<m_printer[m]->getColLength();l++) this->printResult("-"); /* underline */
-      this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix()); 
+      if (this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
     }
-    this->printResult("%s",this->m_sqlcliEnv->getrowsuffix()); 
+    if (this->printResult("%s",this->m_sqlcliEnv->getrowsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
   }
-}
-
-SQLDBC_Bool IFTools_sqlcliOutput::printRow(SQLDBC_ResultSet *rs){
-  this->printResult("%s",this->m_sqlcliEnv->getrowprefix()); 
-  for(SQLDBC_Int2 i = 0; i < m_printerLen; i++){
-    this->printResult("%s",this->m_sqlcliEnv->getcolumnprefix()); 
-    if( !  this->printRecord(rs, i+1, m_printer[i])) return SQLDBC_FALSE;     
-    this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix()); 
-  }
-  this->printResult("%s",this->m_sqlcliEnv->getrowsuffix()); 
   return SQLDBC_TRUE;
 }
-
+//----------------------------------------------------------------------
+SQLDBC_Bool IFTools_sqlcliOutput::printOutputValues(SQLDBC_PreparedStatement *rs){
+  if (this->printResult("%s",this->m_sqlcliEnv->getrowprefix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+  for(SQLDBC_Int2 i = 0; i < m_printerLen; i++){
+    if (this->printResult("%s",this->m_sqlcliEnv->getcolumnprefix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+    if( !  this->printOutputValue(rs, i+1, m_printer[i])) return SQLDBC_FALSE;     
+    if (this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE;
+  }
+  if (this->printResult("%s",this->m_sqlcliEnv->getrowsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+  return SQLDBC_TRUE;
+}
+//----------------------------------------------------------------------
+SQLDBC_Bool IFTools_sqlcliOutput::printRow(SQLDBC_ResultSet *rs){
+  if (this->printResult("%s",this->m_sqlcliEnv->getrowprefix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+  for(SQLDBC_Int2 i = 0; i < m_printerLen; i++){
+    if (this->printResult("%s",this->m_sqlcliEnv->getcolumnprefix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+    if( !  this->printRecord(rs, i+1, m_printer[i])) return SQLDBC_FALSE;     
+    if (this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE;
+  }
+  if (this->printResult("%s",this->m_sqlcliEnv->getrowsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+  return SQLDBC_TRUE;
+}
+//----------------------------------------------------------------------
 SQLDBC_Bool  IFTools_sqlcliOutput::printRecord(SQLDBC_ResultSet * rs, SQLDBC_Int4 colIndex, IFTools_sqlcliOutputPrinter *outp){
   SQLDBC_Length indicator=0;
   SQLDBC_Retcode erg = rs->getRowSet()->getObject(colIndex, outp->gethostType(), m_buffer,  &indicator, m_bufLen);
@@ -720,8 +886,8 @@ SQLDBC_Bool  IFTools_sqlcliOutput::printRecord(SQLDBC_ResultSet * rs, SQLDBC_Int
         indicator=0;
         erg = rs->getRowSet()->getObject(colIndex, outp->gethostType(), m_buffer,  &indicator, m_bufLen);  
         if (erg != SQLDBC_OK && erg != SQLDBC_DATA_TRUNC && erg != SQLDBC_NO_DATA_FOUND){
-          this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix()); 
-          this->printResult("%s",this->m_sqlcliEnv->getrowsuffix()); 
+          if (this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+          if (this->printResult("%s",this->m_sqlcliEnv->getrowsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
           this->printError (rs->error());
           return SQLDBC_FALSE;                  
         } else {
@@ -736,8 +902,8 @@ SQLDBC_Bool  IFTools_sqlcliOutput::printRecord(SQLDBC_ResultSet * rs, SQLDBC_Int
       return SQLDBC_TRUE;                 
     }  
   } else if (erg != SQLDBC_OK){
-    this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix()); 
-    this->printResult("%s",this->m_sqlcliEnv->getrowsuffix()); 
+    if (this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+    if (this->printResult("%s",this->m_sqlcliEnv->getrowsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
     this->printError (rs->error());
     return SQLDBC_FALSE;                  
   } else { 
@@ -749,3 +915,51 @@ SQLDBC_Bool  IFTools_sqlcliOutput::printRecord(SQLDBC_ResultSet * rs, SQLDBC_Int
     return SQLDBC_TRUE;
   }
 }
+
+//----------------------------------------------------------------------
+SQLDBC_Bool  IFTools_sqlcliOutput::printOutputValue(SQLDBC_PreparedStatement * rs, SQLDBC_Int4 colIndex, IFTools_sqlcliOutputPrinter *outp){
+  SQLDBC_Length indicator=0;
+  SQLDBC_Retcode erg = rs->getObject(colIndex, outp->gethostType(), m_buffer,  &indicator, m_bufLen);
+
+  //     SQLDBC_Retcode getObject(SQLDBC_Int4 columnIndex, 
+  //                            SQLDBC_HostType columnType, 
+  //                            void* columnAddr, 
+  //                            SQLDBC_Length * columnLengthIndicator,
+  //                            const SQLDBC_Length columnSize);
+
+  if (erg == SQLDBC_DATA_TRUNC){
+    outp->printValue(this, m_buffer, indicator);
+    if (outp->streamAllData()){
+      while(erg == SQLDBC_DATA_TRUNC){
+        indicator=0;
+        erg = rs->getObject(colIndex, outp->gethostType(), m_buffer,  &indicator, m_bufLen);  
+        if (erg != SQLDBC_OK && erg != SQLDBC_DATA_TRUNC && erg != SQLDBC_NO_DATA_FOUND){
+          if (this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+          if (this->printResult("%s",this->m_sqlcliEnv->getrowsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+          this->printError (rs->error());
+          return SQLDBC_FALSE;                  
+        } else {
+          outp->printValue(this, m_buffer, indicator);
+        }
+      }
+      return SQLDBC_TRUE; 
+    } else {
+      //this->printResult("%s",this->m_sqlcliEnv->getrecordsep()); /* linebreak */
+      //this->printError (ERR_INTERNAL_TRUNC, colIndex,outp->getColLength(),m_bufLen);
+      //return SQLDBC_FALSE;                    
+      return SQLDBC_TRUE;                 
+    }  
+  } else if (erg != SQLDBC_OK){
+    if (this->printResult("%s",this->m_sqlcliEnv->getcolumnsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+    if (this->printResult("%s",this->m_sqlcliEnv->getrowsuffix())==SQLDBC_FALSE) return SQLDBC_FALSE; 
+    this->printError (rs->error());
+    return SQLDBC_FALSE;                  
+  } else { 
+    if(indicator == SQLDBC_NULL_DATA){ 
+      this->printNullValue(outp->getColLength());     
+      return SQLDBC_TRUE;
+    }
+    outp->printValue(this, m_buffer, indicator);
+    return SQLDBC_TRUE;
+  }
+}
\ No newline at end of file
