@@ -49,7 +49,8 @@
 */
 #define DEFAULT_PAGER "more"
 
-SQLDBC_Environment* IFTools_sqlcliProperties::m_environment=new SQLDBC_Environment(SQLDBC_ClientRuntimeInit());
+SQLDBC_Environment* IFTools_sqlcliProperties::m_environment = 0;
+//new SQLDBC_Environment(SQLDBC_ClientRuntimeInit());
 
 
 
@@ -62,6 +63,7 @@ const teo81_EvalProgArgs::teo81_OptionDesc IFTools_sqlcliProperties::Options[] =
     { option_cmdseparator    , _T('c'), _T("cmdseparator"),    true,  true, _T("\"<separator>\""),                                 _T("Use separator to separate comamnds in batch files.") },
     { option_dbname          , _T('d'), _T("dbname") ,         true,  true, _T("\"<database name>\""),                             _T("Specifies the name of the database to connect to.") },
     { option_dateformat      , _T('D'), _T("dateformat") ,     true,  true, _T("\"[INTERNAL|ISO|USA|EUR|JIS]\""),                  _T("Use this date and/or time format instead of the one specified in the database set command. Possible options are INTERNAL, ISO, USA, EUR or JIS") },
+    { option_stopOnError      , _T('E'), _T("stoponerror") ,   true,  true, _T("exit code"),                  _T("exit sqlcli returning <exit code> in case of an error") },
     { option_writeCmdProt    , _T('f'), _T("cmdtrace") ,       false,  true, _T("\"<protocol name>\""),                            _T("Printout the SQL commands") },
     { option_separator       , _T('F'), _T("fieldseparator") , true,  true, _T("\"<separator>\""),                                 _T("Use separator as the field separator.") },
     { option_nullval         , _T('g'), _T("null") ,           true,  true, _T("\"<null string>\""),                               _T("The argument is a string that should be printed whenever a field is null. The default is '?'.") },
@@ -74,7 +76,7 @@ const teo81_EvalProgArgs::teo81_OptionDesc IFTools_sqlcliProperties::Options[] =
     { option_rowprefix       , _T('p'), _T("rowprefix")       ,true,  true, _T(""),                                                _T("") },
     { option_rowsuffix       , _T('P'), _T("rowsuffix")       ,true,  true, _T(""),                                                _T("") },
     { option_oneColperLine   , _T('Q'), _T("onecolumnperline"),false, true, _T("\"Show each column on a separate line.\""),        _T("Show each column on a separate line") },
-//    { option_rowseparator    , _T('R'), _T("rowseparator"),    true,  true, _T("\"<separator>\""),                                 _T("Use separator as the row separator.") },
+    { option_useDBSStmt      , _T('r'), _T("noPreparedStatement"),false,  true, _T(""),                                            _T("do not use prepared statements") },
     { option_sqlmode         , _T('S'), _T("sqlmode") ,        true,  true, _T("\"[INTERNAL|ORACLE|ANSI|DB2]\""),                  _T("The SQL Mode. One of \"INTERNAL\",\"ORACLE\",\"ANSI\", or \"DB2\". ") },
     { option_trace           , _T('T'), _T("trace")  ,         true,  true, _T("\"<tracefile name>\""),                            _T("Write a trace file.") },
     { option_userpasswd      , _T('u'), _T("user")   ,         true,  true, _T("\"<user>,<password>\""),                           _T("Username and password to be connected to the database.") },
@@ -113,6 +115,7 @@ void IFTools_sqlcliProperties::eo80_vHelp ( const TCHAR*         OptionArgument
         "                          \"INTERNAL\",\"ORACLE\",\"ANSI\", or \"DB2\""));
 	puts(("  -y                      sets session timeout in seconds (0 means no timeout)"));
 	puts(("  -z                      switches autocommit off"));
+	puts(("  -r                      suppress usage of prepared statements"));
 	
 	puts(("\nInput and output options:"));
 	puts(("  -i <file_name>  use file <file_name> to input queries (default: stdin)"));
@@ -137,14 +140,15 @@ void IFTools_sqlcliProperties::eo80_vHelp ( const TCHAR*         OptionArgument
 	puts(("General options:"));
 	puts(("  -h              show help, then exit"));
 	puts(("  -v              printout version information"));
+	puts(("  -E <exit code>  exit sqlcli returning <exit code> in case of an error"));
 	puts(("  -t              printout debug information"));
-	puts(("  -T              write a database interface trace"));
+	puts(("  -T <filename>   write a database interface trace (SQL trace)"));
 	
 	puts((
 		   "\nFor further information, type \"\\?\" (for internal commands) from \n"
 		   "within SQLCLI or consult the SQLCLI documentation.\n"));
 
-   exit(0);
+   this->exitProgram();
 }
 
 static char Utfmap[] ={  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x5F, 0x5F, 0x20, 0x20, 0x20, 0x20, 0x5F, 0x5F, 0x20,
@@ -238,6 +242,7 @@ IFTools_sqlcliProperties::IFTools_sqlcliProperties ( int argc, TCHAR** argv )
         m_cmdtrace  (SQLDBC_FALSE),          
         m_autocommit(SQLDBC_TRUE),     
         m_noHeader  (SQLDBC_FALSE),       
+		m_usePreparedStmt  (SQLDBC_TRUE),       
         m_batch     (SQLDBC_FALSE),          
         m_rollbackOnError (SQLDBC_FALSE),
         m_noComment(SQLDBC_FALSE),
@@ -250,6 +255,9 @@ IFTools_sqlcliProperties::IFTools_sqlcliProperties ( int argc, TCHAR** argv )
         m_connectState(SQLDBC_FALSE),
         m_pagerOutput(SQLDBC_FALSE),
         m_versionInfo(SQLDBC_FALSE),
+		m_exitCode(0),
+		m_ExitForced(SQLDBC_FALSE),
+        m_StopOnError(SQLDBC_FALSE),
         teo81_EvalProgArgs( argc, argv, 
                            (teo81_EvalProgArgs::teo81_OptionDesc*)&Options, 
                             NUM_OF_OPTIONS_EO80(Options),
@@ -276,11 +284,12 @@ IFTools_sqlcliProperties::IFTools_sqlcliProperties ( int argc, TCHAR** argv )
  setdefaulteditor ("vi"); 
 #endif
  m_ifrprops.setProperty("SQLMODE", "INTERNAL");
- m_ifrprops.setProperty("APPLICATION ", "ODB");
+ m_ifrprops.setProperty("STATEMENTCACHESIZE", "0");
+ m_ifrprops.setProperty("SELECTFETCHOPTIMIZE", "1");
 
  if (m_environment == 0) 
      m_environment=new SQLDBC_Environment(SQLDBC_ClientRuntimeInit());
-
+ 
 //  m_properties.setProperty  ("COMPNAME",  "CLI"  ); 
   if(this->eo80_ParseArguments() == SQLDBC_OK) {}
 };
@@ -309,9 +318,13 @@ bool IFTools_sqlcliProperties::eo80_vNextValidOption ( const int            Opti
       break;
     }  
     case option_trace           : {
-      this->m_ifrprops.setProperty("TRACEFILE", "sql.pct");
-//      DBUG_SETTRACEFLAGS("osqlcli.prt:d:c");
-      break;
+        m_environment->getTraceOptions(m_traceprops);
+        m_traceprops.setProperty("SQL", "TRUE");
+        if(OptionArgument && strlen(OptionArgument)) {
+            m_traceprops.setProperty("FILENAME", OptionArgument);
+        }
+        m_environment->setTraceOptions(m_traceprops);
+        break;
     }  
     case option_separator       : {
       setfieldseparator(OptionArgument, strlen(OptionArgument));
@@ -332,6 +345,10 @@ bool IFTools_sqlcliProperties::eo80_vNextValidOption ( const int            Opti
     case option_noHeader        : {
       this->m_noHeader = true;
       break;
+    }  
+	case option_useDBSStmt      : {
+		this->m_usePreparedStmt = SQLDBC_FALSE;
+      break;
     }  
     case option_batch           : { 
       setinfile(OptionArgument);
@@ -342,6 +359,11 @@ bool IFTools_sqlcliProperties::eo80_vNextValidOption ( const int            Opti
       setoutfile(OptionArgument);
       break;
     }
+    case option_stopOnError     : {
+      setStopOnError(SQLDBC_TRUE);
+      setExitCode(atoi(OptionArgument));
+      break;
+    }
     case option_oneColperLine   : {
       setcolumnprefix  ("",0);
       setrowprefix     ("",0);
@@ -483,11 +505,11 @@ void IFTools_sqlcliProperties::dumpClass(){
     fprintf(m_errfile,"CMDTrace  : \"%s\"\n",  (m_cmdtrace  ==true)?"TRUE":"FALSE");
     fprintf(m_errfile,"autocommit: \"%s\"\n",  (m_autocommit==true)?"TRUE":"FALSE");
     fprintf(m_errfile,"noHeader  : \"%s\"\n",  (m_noHeader  ==true)?"TRUE":"FALSE");
+	fprintf(m_errfile,"useDBS    : \"%s\"\n",  (m_usePreparedStmt ==true)?"TRUE":"FALSE");
     fprintf(m_errfile,"batch     : \"%s\"\n",  (m_batch     ==true)?"TRUE":"FALSE");
 
     fprintf(m_errfile,"rollbOnErr: \"%s\"\n",  (m_rollbackOnError==true)?"TRUE":"FALSE");
     fprintf(m_errfile,"SQL mode  : \"%s\"\n",  m_ifrprops.getProperty("SQLMODE"));
-    fprintf(m_errfile,"trace     : \"%s\"\n",  m_ifrprops.getProperty("TRACEFILE"));
 /*    fprintf(m_errfile,"Screensize: \"%d\"\n",  getConsoleScreenSize()); */
     fprintf(m_errfile,"Command   : \"%s\"\n", (m_command)?m_command:"NULL");
 }
@@ -768,7 +790,10 @@ SQLDBC_Bool IFTools_sqlcliProperties::setfieldseparator   (const char * name, SQ
   }
   return SQLDBC_TRUE;   
 }
-
+//--------------------------------------------------------------------------
+void IFTools_sqlcliProperties::exitProgram(){
+	exit(this->m_exitCode);
+}
 //--------------------------------------------------------------------------
 /*
 #SQLDBC_Int4 IFTools_sqlcliProperties::getConsoleScreenSize(){
