@@ -1,25 +1,58 @@
+/*
+
+
+    ========== licence begin  GPL
+    Copyright (c) 2003-2004 SAP AG
+
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
+    as published by the Free Software Foundation; either version 2
+    of the License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    ========== licence end
+
+
+*/
+
 #include "heo00x.h" // suspend reasons
 #include "heo56.h"  // vsuspend, vresume
+#include "heo670.h" // eo670_CTraceStack      
 #include "heo922.h" // vGetRteInfoTaskDetails
 #include "hkb53.h"
-#include "gkb07.h"
+#include "ggg07.h"
 #include "hkb90.h"
+#include "hbd01.h"
 #include "hbd02.h"
 #include "hbd03.h"
+#include "hbd21.h"
 #include "hgg01.h"
 #include "hgg01_3.h"
 #include "hgg04.h"
 #include "hkb742.h"
 
+#include "hta01.h"
+#include "hta01_3.h"
+
+
 #include "RunTime/Synchronisation/RTESync_Spinlock.hpp"
+#include "RunTime/System/RTESys_MemoryBarrier.h"
 #include "Container/Container_Vector.hpp"
 #include "SQLManager/SQLMan_Context.hpp"
 #include "SQLManager/SQLMan_MessBlock.hpp"
-#include "SAPDB/SAPDBCommon/ErrorsAndMessages/SAPDBErr_Assertions.hpp"
-#include "SAPDB/Join/Join_InvSelectIterator.hpp"
+#include "SAPDBCommon/ErrorsAndMessages/SAPDBErr_Assertions.hpp"
+#include "Join/Join_InvSelectIterator.hpp"
 #include "SAPDBCommon/Tracing/SAPDBTrace_Streaming.hpp"
 #include "IOManager/IOMan_IDataInfo.hpp"
 #include "KernelCommon/Kernel_VTrace.hpp"
+#include "KernelCommon/Kernel_OpMsg.hpp"
 
 extern SAPDBTrace_Topic Join_Trace;   //!< join tracing
 
@@ -172,6 +205,8 @@ void Join_InvSelectServer::EnterRequest(SAPDB_Byte* pBuffer, int keyCount, bool
     m_pKeyBuffer    = pBuffer;
     m_keyCount      = keyCount;
     m_storeLastKey  = storeLastKey;
+    m_read              = 0;
+    m_written           = 0;
     ++m_statistics->m_requests;
     m_statistics->m_keyCount += keyCount;
     this->Resume(true);
@@ -188,6 +223,7 @@ tgg00_BasisError Join_InvSelectServer::GetNextRow (void* buf, int bufSize)
     }
     if (m_written > m_read)
     {
+        RTESys_ReadMemoryBarrier();
         memcpy (buf, m_pRowCoordinator, m_rowSize);
         ++m_read;
         m_pRowCoordinator = IncRowBufferPointer(m_pRowCoordinator);
@@ -242,6 +278,7 @@ inline void Join_InvSelectServer::IncWritten()
 {
     SAPDBTRACE_METHOD_DEBUG( "Join_InvSelectServer::IncWritten", Join_Trace, 1 );
     m_pRowServer = this->IncRowBufferPointer(m_pRowServer);
+    RTESys_WriteMemoryBarrier();
     ++m_written;
     this->NotifyIterator();
 }
@@ -291,10 +328,10 @@ void Join_InvSelectServer::RegisterRowBuffer (SAPDB_Byte* pBuffer, int bufferSiz
     m_bufferSize      = bufferSize;
     m_rowSize         = rowSize;
     m_maxRowCount     = bufferSize / rowSize;
-    m_written         = 1;
-    m_read            = 0;
     m_pRowServer      = this->IncRowBufferPointer(pBuffer);
     m_pRowCoordinator = pBuffer;
+    m_written         = 1;
+    m_read            = 0;
     this->NotifyIterator();
 }
 
@@ -303,28 +340,25 @@ void Join_InvSelectServer::RegisterRowBuffer (SAPDB_Byte* pBuffer, int bufferSiz
 tsp00_MoveObjPtr Join_InvSelectServer::ReserveRowBuffer (SAPDB_Int4& bufferSize)
 {
     SAPDBTRACE_METHOD_DEBUG( "Join_InvSelectServer::ReserveRowBuffer", Join_Trace, 1 );
-    if (m_written - m_read == m_maxRowCount)
+    bool doSuspend;
     {
-        bool doSuspend;
+        RTESync_LockedScope lockedScope(m_lock);
+        if (m_stopServer) return 0;
+        doSuspend         = (m_written - m_read == m_maxRowCount);
+        m_serverSuspended = doSuspend;
+    }
+    if (doSuspend)
+    {
+        SAPDBTRACE_WRITELN( Join_Trace, 1, "Suspend Server(" << m_serverTaskId << "), buffer full");
+        if (m_unRecognizedRows > 0)
         {
-            RTESync_LockedScope lockedScope(m_lock);
-            if (m_stopServer) return 0;
-            doSuspend         = (m_written - m_read == m_maxRowCount);
-            m_serverSuspended = doSuspend;
-        }
-        if (doSuspend)
-        {
-            SAPDBTRACE_WRITELN( Join_Trace, 1, "Suspend Server(" << m_serverTaskId << "), buffer full");
-            if (m_unRecognizedRows > 0)
-            {
-                // there are rows that have not been noted by the iterator
-                // do it now before suspending
-                m_iterator.IncRowCount(m_unRecognizedRows);
-                m_unRecognizedRows = 0;
-            }
-            ++m_statistics->m_bufferOverflows;
-            vsuspend (m_serverTaskId, SUSPEND_REASON_INV_SEL_SERVER_RESERVE_ROW_BUFFER);  
+            // there are rows that have not been noted by the iterator
+            // do it now before suspending
+            m_iterator.IncRowCount(m_unRecognizedRows);
+            m_unRecognizedRows = 0;
         }
+        ++m_statistics->m_bufferOverflows;
+        vsuspend (m_serverTaskId, SUSPEND_REASON_INV_SEL_SERVER_RESERVE_ROW_BUFFER);  
     }
     bufferSize = m_rowSize;
     return reinterpret_cast<tsp00_MoveObjPtr>(m_pRowServer);  
@@ -399,7 +433,6 @@ void Join_InvSelectServer::StoreStatistics (SAPDB_UInt8 ioCount)
 //-------------------------------------------------------------------------------
 
 bool Join_InvSelectIterator::m_serversInUse   = false;
-int  Join_InvSelectIterator::m_maxServerTasks = 0;
 
 //-------------------------------------------------------------------------------
 
@@ -409,7 +442,8 @@ Join_InvSelectIterator::Join_InvSelectIterator
  SQLMan_MessBlock        &messBlock,
  tgg00_FileId            &InvTree,
  tgg00_SelectFieldsParam &SelectParam,
- tgg00_StackDesc         &StackDesc)
+ tgg00_StackDesc         &StackDesc,
+ const SAPDB_Int4        &maxServerTasks)
  : m_lock()
  , m_messBlock(messBlock)
  , m_selFieldsParam(SelectParam)
@@ -420,9 +454,10 @@ Join_InvSelectIterator::Join_InvSelectIterator
  , m_allocator(SQLMan_Context::GetContext()->GetAllocator())
  , m_stop(false)
  , m_suspended(false)
+ , m_isOpen(false)
  , m_error(e_ok)
  , m_bdInvSet()
- , m_maxServers(m_maxServerTasks)
+ , m_maxServers(maxServerTasks)
  , m_servers(m_allocator)
  , m_coordinatorTrans(Trans)
  , m_countRowsProvided(0)
@@ -441,10 +476,6 @@ Join_InvSelectIterator::~Join_InvSelectIterator()
 {
     SAPDBTRACE_METHOD_DEBUG( "Join_InvSelectIterator::~Join_InvSelectIterator", Join_Trace, 1 );
     this->StopServers();
-    for (int ix = 0; ix < m_startedServers; ++ix)
-    {
-        destroy (m_servers[ix], m_allocator);
-    }
     if ( Join_Trace.TracesLevel( 1 ) )
     {
         Kernel_VTrace() << "parallel join coordinator";
@@ -465,10 +496,7 @@ void Join_InvSelectIterator::Close()
 {
     SAPDBTRACE_METHOD_DEBUG( "Join_InvSelectIterator::Close", Join_Trace, 1 );
     this->StopServers();
-    for (int ix = 0; ix < m_startedServers; ++ix)
-    {
-        destroy (m_servers[ix], m_allocator);
-    }
+    m_isOpen = false;
 }
 
 //-------------------------------------------------------------------------------
@@ -536,7 +564,14 @@ bool Join_InvSelectIterator::CreatedParallelServers()
         // are servers currently in use ?
         if (!m_serversInUse)
         {
-            m_serversInUse = true;
+            const bool c_inRegion = true; 
+            int availableServers = bd01UsableServerForParallelExecution (m_coordinatorTrans.trTaskId_gg00,
+                m_select_row, !c_inRegion);
+            m_serversInUse = availableServers >= m_maxServers;
+            if (!m_serversInUse)
+            {
+                return false;
+            }
         }
         else
         {
@@ -581,6 +616,7 @@ bool Join_InvSelectIterator::CreatedParallelServers()
         this->StopServers();
         return false;
     }
+    m_coordinatorTrans.trSyncParallelServers_gg00 = true;
     return true;
 }
 
@@ -691,9 +727,21 @@ inline bool Join_InvSelectIterator::IsSuspended() const
          m_selFieldsParam.sfp_m_result_cnt()  = 1; // at most one result
          SAPDBTRACE_WRITELN( Join_Trace, 5, "result_addr : " << (char*) ToStr((void*) m_selFieldsParam.sfp_m_result_addr()));
          m_tree.fileBdUse_gg00().clear();
+         
+         m_selFieldsParam.sfp_rows_read() = 0;
+         m_selFieldsParam.sfp_rows_qual() = 0;
+            
          b02kb_select_rec (m_coordinatorTrans, m_tree, m_pKeyBuffer, keyLength,
              m_pKeyBuffer, keyLength, 0, 0, false, m_selFieldsParam, m_messBlock.mb_qual()->mstack_desc(),
              dummyBool, dummyGranted);
+
+        if ( m_messBlock.mb_trns()->trBdTcachePtr_gg00 != NULL )
+        {
+            b21mp_rows_read_put (m_messBlock.mb_trns()->trBdTcachePtr_gg00, m_selFieldsParam.sfp_rows_read());
+            b21mp_rows_qual_put (m_messBlock.mb_trns()->trBdTcachePtr_gg00, m_selFieldsParam.sfp_rows_qual());
+        }
+        
+
          e =  m_coordinatorTrans.trError_gg00;
          bool found;
          if (e_ok == e)
@@ -791,7 +839,6 @@ inline bool Join_InvSelectIterator::IsSuspended() const
                  return this->NextRowResult (e_no_next_record);
              }
              // no row available, wait ...
-             m_lock.Unlock();
              this->Suspend();
              // note : coordinator is waked up if m_runningServer is 0 or at least one row is available
              if ((0 == m_runningServers) && (m_countRowsHandled == m_countRowsProvided))
@@ -871,13 +918,20 @@ void Join_InvSelectIterator::NotifyRequestFinished()
 
  //-------------------------------------------------------------------------------
 
- void Join_InvSelectIterator::Open (
+tgg00_BasisError Join_InvSelectIterator::Open (
      tgg00_Lkey          &invKey,
      tgg00_Lkey          &startKey,
      tgg00_Lkey          &stopKey,
      tgg00_BdInvSet      &InvRangeSet)
 {
      SAPDBTRACE_METHOD_DEBUG( "Join_InvSelectIterator::Open", Join_Trace, 1 );
+     if (m_isOpen)
+     {
+         Kernel_OpError (csp3_msg, csp3_n_join) << "Join_InvSelectIterator::Open : already open ";
+         eo670_CTraceStack();   
+         m_error = e_invalid;
+         return m_error;
+     }
      m_pInvKey  = &invKey;
      m_pStopKey = &stopKey;
      m_bdInvSet = InvRangeSet;
@@ -892,13 +946,8 @@ void Join_InvSelectIterator::NotifyRequestFinished()
      m_keyCount          = 0;
      m_getInvError       = e_buffer_limit;
      m_currServer        = 0;
-}
-
-//-------------------------------------------------------------------------------
-
-bool Join_InvSelectIterator::ParallelEnabled()
-{
-    return (m_maxServerTasks > 0);
+     m_isOpen            = true;
+     return e_ok;
 }
 
 //-------------------------------------------------------------------------------
@@ -917,23 +966,25 @@ inline void Join_InvSelectIterator::Resume()
 
 //-------------------------------------------------------------------------------
 
-void Join_InvSelectIterator::SetMaxServerTasks(int max)
+void Join_InvSelectIterator::SetMaxServerTasks(SAPDB_Int4 max)
 {
     if (max > IOMan_IDataInfo::GetInstance().ConfiguredDataVolumes() - 1)
     {
-        m_maxServerTasks = IOMan_IDataInfo::GetInstance().ConfiguredDataVolumes() - 1;
+        m_maxServers = IOMan_IDataInfo::GetInstance().ConfiguredDataVolumes() - 1;
+        SAPDBTRACE_WRITELN( Join_Trace, 1, "max Servers adjusted" );
     }
     else
     {
-        m_maxServerTasks = max;
+        m_maxServers = max;
     }
+    SAPDBTRACE_WRITELN( Join_Trace, 1, "max Servers " << m_maxServers );
 }
 
 //-------------------------------------------------------------------------------
 
 SAPDB_Int Join_InvSelectIterator::GetMaxServerTasks()
 {
-    return m_maxServerTasks;
+    return m_maxServers;
 }
 
 //-------------------------------------------------------------------------------
@@ -960,31 +1011,45 @@ void Join_InvSelectIterator::StopServers()
 {
     SAPDBTRACE_METHOD_DEBUG( "Join_InvSelectIterator::StopServers", Join_Trace, 1 );
     m_stop = true;
-    for (int serverId = 0; serverId < m_startedServers; ++serverId)
+    if (m_startedServers > 0)
     {
-        m_servers[serverId]->StopServer();
-    }
-    tgg00_MessBlock     ReceiveMBlock;
-    tgg00_TransChild    ReceiveTrans;
-    tsp00_Buf           ReceiveDataPart;
-    tgg00_BasisError    e = e_ok;
-    while (0 < m_startedServers)
-    {
-        m_coordinatorTrans.trError_gg00 = e_ok;
-        g01mblock_init (m_coordinatorTrans, m_nil, mm_nil, ReceiveMBlock);
-        g01datapart_init (ReceiveMBlock, 
-            REINTERPRET_CAST (tgg00_DataPartPtr, &ReceiveDataPart), 
-            sizeof (ReceiveDataPart));
-        k90rcv_child (ReceiveMBlock, ReceiveTrans);
-        if (e_ok != m_coordinatorTrans.trError_gg00) 
+        for (int serverId = 0; serverId < m_startedServers; ++serverId)
         {
-            e = m_coordinatorTrans.trError_gg00;
+            m_servers[serverId]->StopServer();
         }
-        --m_startedServers;
-        SAPDBTRACE_WRITELN( Join_Trace, 1, "still running servers : " << m_startedServers);
+        tgg00_MessBlock     ReceiveMBlock;
+        tgg00_TransChild    ReceiveTrans;
+        tsp00_Buf           ReceiveDataPart;
+        tgg00_BasisError    e = e_ok;
+        int startedServers = m_startedServers;
+        while (0 < m_startedServers)
+        {
+            m_coordinatorTrans.trError_gg00 = e_ok;
+            g01mblock_init (m_coordinatorTrans, m_nil, mm_nil, ReceiveMBlock);
+            g01datapart_init (ReceiveMBlock, 
+                REINTERPRET_CAST (tgg00_DataPartPtr, &ReceiveDataPart), 
+                sizeof (ReceiveDataPart));
+            k90rcv_child (ReceiveMBlock, ReceiveTrans);
+            if (e_ok != m_coordinatorTrans.trError_gg00) 
+            {
+                e = m_coordinatorTrans.trError_gg00;
+                Kernel_OpError (csp3_msg, csp3_n_join) << "join server returned error " << e;
+            }
+            --m_startedServers;
+            SAPDBTRACE_WRITELN( Join_Trace, 1, "still running servers : " << m_startedServers);
+        }
+        for (int ix = 0; ix < startedServers; ++ix)
+        {
+            destroy (m_servers[ix], m_allocator);
+        }
+        // TODO message list
+        m_serversInUse = (e != e_ok); // don't use servers any more, if rcv_child returned an error
+        if (m_serversInUse)
+        {
+            Kernel_OpError (csp3_msg, csp3_n_join) << "parallel join disabled";
+        }
+        m_coordinatorTrans.trSyncParallelServers_gg00 = false;
     }
-    // TODO message list
-    m_serversInUse = (e != e_ok); // don't use servers any more, if rcv_child returned an error
 }
 
 //-------------------------------------------------------------------------------
@@ -1002,6 +1067,7 @@ void Join_InvSelectIterator::Suspend()
 {
     SAPDBTRACE_METHOD_DEBUG( "Join_InvSelectIterator::Suspend", Join_Trace, 1 );
     m_suspended = true;
+    m_lock.Unlock(); // PTS 1131505
     SAPDBTRACE_WRITELN( Join_Trace, 1, "Suspend Coordinator");
     vsuspend ( m_coordinatorTrans.trTaskId_gg00, SUSPEND_REASON_INV_SEL_ITER_SUSPEND ); 
 }
@@ -1011,14 +1077,15 @@ void Join_InvSelectIterator::Suspend()
 tgg00_VoidPtr k742CreateInvIterator (
     tgg00_MessBlock          VAR_VALUE_REF  m,
     tgg00_SelectFieldsParam  VAR_VALUE_REF  sel,
-    tkb07_select_param       VAR_VALUE_REF  selrec)
+    tgg07_select_param       VAR_VALUE_REF  selrec)
 {
     SAPDBTRACE_ROUTINE_DEBUG ("k742CreateInvIterator", Join_Trace, 1 );
-    if (Join_InvSelectIterator::ParallelEnabled())
+    if ( g01optimize_parallel_server() > 0 )
     {
         SQLMan_MessBlock* pMessBlock = reinterpret_cast<SQLMan_MessBlock*>(&m);
         Join_InvSelectIterator* pIterator = new(SQLMan_Context::GetContext()->GetAllocator()) 
-            Join_InvSelectIterator(*m.mb_trns(), *pMessBlock, selrec.selr_inv_id(), sel, m.mb_qual()->mstack_desc());
+            Join_InvSelectIterator(*m.mb_trns(), *pMessBlock, selrec.selr().selr_inv_id, sel, 
+                    m.mb_qual()->mstack_desc(), g01optimize_parallel_server());
         return pIterator;
     }
     return 0;
@@ -1056,14 +1123,14 @@ tgg00_BasisError k742Next (tgg00_VoidPtr pIter)
 
 void k742OpenIterator(tgg00_VoidPtr                           pIter,
                       tgg00_SelectFieldsParam  VAR_VALUE_REF  sel,
-                      tkb07_select_param       VAR_VALUE_REF  selrec,
+                      tgg07_select_param       VAR_VALUE_REF  selrec,
                       tgg00_BdInvSet           VAR_ARRAY_REF  bdInvSet)
 {
     SAPDBTRACE_ROUTINE_DEBUG ("k742OpenIterator", Join_Trace, 1 );
     Join_InvSelectIterator* pIterator = reinterpret_cast<Join_InvSelectIterator*>(pIter);
     sel.sfp_m_result_addr() = reinterpret_cast<tsp00_MoveObjPtr>(&selrec.selr_selectbuffer().rbuf);
     sel.sfp_m_result_size() = sizeof (selrec.selr_selectbuffer().rbuf);
-    pIterator->Open (selrec.selr_invkey(), selrec.selr_startkey(), selrec.selr_stopkey(), bdInvSet);
+    (void) pIterator->Open (selrec.selr().selr_invkey, selrec.selr().selr_startkey, selrec.selr().selr_stopkey, bdInvSet);
 }
 
 //-------------------------------------------------------------------------------
@@ -1076,10 +1143,12 @@ externPascal void k742Select (
     Join_InvSelectServer*   _pInvSelectServer;
     tgg00_TransContext*     _pCoordinatorTrans;  
     Join_InvSelectIterator::GetServerContext(m, _pInvSelectIterator, _pInvSelectServer, _pCoordinatorTrans); 
+    m.mb_trns()->trSessionId_gg00           = _pCoordinatorTrans->trSessionId_gg00;
     m.mb_trns()->trIndex_gg00               = _pCoordinatorTrans->trIndex_gg00;
     m.mb_trns()->trTransId_gg00             = _pCoordinatorTrans->trTransId_gg00;
     m.mb_trns()->trWriteTransId_gg00        = _pCoordinatorTrans->trWriteTransId_gg00;
     m.mb_trns()->trSubtransId_gg00          = _pCoordinatorTrans->trSubtransId_gg00;
+    m.mb_trns()->trConsistView_gg00         = _pCoordinatorTrans->trConsistView_gg00;
     m.mb_trns()->trState_gg00               = _pCoordinatorTrans->trState_gg00;
     m.mb_trns()->trAcvPtr_gg00              = _pCoordinatorTrans->trAcvPtr_gg00;
     m.mb_trns()->trIoCount_gg00             = 0; 
