@@ -99,6 +99,8 @@ private: // interfaces
         SAPDB_Int4          &badIndexCount,
         SAPDB_Int4          &totalBadCount );
     
+    virtual void Diagnose( tgg00_MessBlock  &messBlock );
+
     virtual void ResumeLogWriter (tsp00_TaskId taskid);
     virtual void CreateSnapShot (tgg00_TransContext &trans);
     virtual void ActivateSnapShot (tgg00_TransContext &trans);
@@ -161,17 +163,24 @@ private: // interfaces
     virtual void SetNewDBIdentifierIfHistoryLost (tsp00_TaskId taskid);
     virtual bool DenyBackupAndRestore();
     virtual void PermitBackupAndRestore();
-    virtual void GetBlockAddress ( tsp00_TaskId               taskid,
-                                   const Data_PageNo         &pageno,
-                                   const Data_AddressingMode &addressmode,
-                                   IOMan_BlockAddress        &blockaddress );
-    virtual bool SetNewBlockAddress ( tsp00_TaskId               taskid,
-                                      const Data_PageNo         &pageno,
-                                      const Data_AddressingMode &addressmode,
-                                      const IOMan_BlockAddress  &blockaddress );
-    virtual bool DeleteBlockAddress( tsp00_TaskId               taskid,
-                                     const Data_PageNo         &pageno,
-                                     const Data_AddressingMode &addressmode );
+
+    virtual void GetBlockAddress ( 
+        const tsp00_TaskId           taskid,
+        const Data_PageNo            &pageno,
+        const Data_PageRecoveryMode  &recMode,
+        IOMan_BlockAddress           &block );
+    
+    virtual bool SetNewBlockAddress ( 
+        const tsp00_TaskId          taskid,
+        const Data_PageNo           &pageno,
+        const Data_PageRecoveryMode &recMode,
+        const IOMan_BlockAddress    &block );
+    
+    virtual bool DeleteBlockAddress( 
+        const tsp00_TaskId             taskid,
+        const Data_PageNo              &pageno,
+        const Data_PageRecoveryMode    &recMode );
+    
     virtual bool ReadRestartRecord ( tsp00_TaskId taskid);
     virtual bool FlushRestartRecord ( tsp00_TaskId taskid);
     virtual void DetermineHotStandbyRole();
@@ -210,7 +219,17 @@ private: // interfaces
     virtual TriggerSynchronizeResult TriggerSynchronize (tgg00_TransContext &serverTaskStartContext);
     virtual void SynchronizeServer ( tsp00_TaskId            taskid,
                                      SAPDBMem_IRawAllocator &allocator);
-	virtual void RestartStandbyServer (tgg00_TransContext &restartContext);
+    virtual void BroadcastBackupHistoryEntry (
+                            tsp00_TaskId            taskid,
+                            const char * line,
+                            const SAPDB_UInt lineLength);
+    virtual void BroadcastBackupMediumDefEntry (
+                            tsp00_TaskId            taskid,
+                            const char * line,
+                            const SAPDB_UInt lineLength);
+                                     
+    virtual void RestartStandbyServer (tgg00_TransContext &restartContext);
+
     virtual void GetDelayTimeStamp (Kernel_Date &date,
                                     Kernel_Time &time);
 private:
@@ -271,6 +290,29 @@ private:
     void ClearLogArea( tsp00_TaskId taskid );
     /// returns true if the master database can be accessed
     bool IsMasterAlive(SAPDBErr_MessageList &errlist); // PTS 1127374 UH 2004-01-23 new
+    
+    /// defines where the call to BroadcastLineToStandbys shall append the given line
+    enum BroadcastDestination {
+        BackupHistoryFile,			                    ///< destination is dbm.knl
+        BackupMediumDefinitionFile                      ///< destination is dbm.mdf
+    } ;
+    /// sends the given Line to all standbys. Depending on dest, the line will be
+    /// appended to the backup history file or the backup medium definietion file
+    void BroadcastLineToStandbys (
+                            const BroadcastDestination    dest,
+                                  tsp00_TaskId            taskid,
+                            const char *                  line,
+                            const SAPDB_UInt              lineLength);
+
+    /// in and outcopy of pages in case of diagnose repair
+    bool DiagnoseHandling(
+        tgg00_TransContext     &trans, 
+        tgg00_MessBlock        &messBlock,
+        tsp00_PageAddr         pPage,
+        const tgg00_DiagType   diagType,
+        const bool             bIsRestore,
+        bool                   &bPageWasExtracted,
+        bool                   &bCloseVolumes );
 
 private:
     /// pointer to singleton instance
