@@ -321,6 +321,8 @@
 #define KERNELPARAM_VALLOC_MEM_POOL (SAPDB_UTF8 *)"VALLOC_MEM_POOL"
 /*! cserv name of the configuration parameter __PARAM_CHANGED___ */
 #define KERNELPARAM___PARAM_CHANGED___ (SAPDB_UTF8 *)"__PARAM_CHANGED___"
+/*! cserv name of the configuration parameter USE_UCONTEXT */
+#define KERNELPARAM_FORBID_LOAD_BALANCING   (SAPDB_UTF8 *)"FORBID_LOAD_BALANCING"
 /*++++++++++++++++*OS DEPENDEND PARAMETERS *+++++++++++++++++++*/
 /***************************************************************/
 #if defined(_WIN32)
@@ -359,6 +361,8 @@
 #define KERNELPARAM_USE_STACK_ON_STACK (SAPDB_UTF8 *)"USE_STACK_ON_STACK"
 /*! cserv name of the configuration parameter USE_UCONTEXT */
 #define KERNELPARAM_USE_UCONTEXT   (SAPDB_UTF8 *)"USE_UCONTEXT"
+/*! cserv name of the configuration parameter SIMULATE_VECTORIO */
+#define KERNELPARAM_SIMULATE_VECTORIO   (SAPDB_UTF8 *)"SIMULATE_VECTORIO"
 
 /***************************************************************/
 #endif
