@@ -202,8 +202,6 @@ public:
     static void AttachNewSink(OmsHandle &obj);
 
 protected:
-    friend class LVCSim_ProcServerInterface;
-
     /// Detach the handle from the sink in the simulator (internal).
     static void Detach(OmsHandle &obj);
 
@@ -445,15 +443,6 @@ public:
     }
     //@}
 
-    /// Start COM routine in simulated ProcServer (in new pseudo-session).
-    static void StartInProcServer(OmsHandle &h, const char *libname, const char *methodname, ...);
-
-    /// Commit in simulated ProcServer (in new pseudo-session).
-    static void CommitInProcServer(OmsHandle &h, const char *libname);
-
-    /// Rollback in simulated ProcServer (in new pseudo-session).
-    static void RollbackInProcServer(OmsHandle &h, const char *libname);
-
 protected:
     // definition of extensions for liveCache simulator in Oms
     friend class LVCSim_liveCacheSink;
@@ -556,8 +545,16 @@ private:
 #ifndef	LVCSIM_ONLY_GLOBALS
 
 // Compatibility classes for old simulator
-
-#include <iostream.h>
+#if defined _M_AMD64
+// on newer platfroms only standard c++ libraries are available
+  #include <iostream>
+  using namespace std;
+#elif _MSC_VER == 1310
+  #include <iostream>   //   PTS 1129537
+  using namespace std;  //   PTS 1129537
+#else
+  #include <iostream.h>
+#endif
 
 LVCSIM_API inline ostream& operator<<(ostream &os, const OmsObjectId &r)
 {
@@ -774,6 +771,7 @@ protected:
     colDesc[0].length = size;
     colDesc[0].memberType = 0;
     colDesc[0].offset = 0;
+    colDesc[0].inout = 0;
     colDesc[0].dec = 0;
   }
 
@@ -876,6 +874,7 @@ protected:
     colDesc[0].length = rowSize;
     colDesc[0].abap_type = 0;
     colDesc[0].offset = 0;
+    colDesc[0].inout = 0;
     colDesc[0].dec = 0;
   }
 
