@@ -29,6 +29,7 @@
 #include "LVCSimulator/LVCSim_Internal.hpp"
 #include "LVCSimulator/LVCSim_DebugAllocator.hpp"
 #include "RunTime/Synchronisation/RTESync_Spinlock.hpp"
+#include "SAPDBCommon/SAPDB_MemCopyMove.hpp"
 #include <stdlib.h>
 #include <string.h>
 #include "hsp77.h"
@@ -64,6 +65,15 @@ bool                    __debug_lvcsim_throw_badalloc = false;
 #define	MAX_FL		32768	// maximum alloc for "small" block - in freelists
 #define	FL_COUNT	12		// freelist count
 
+#if defined(_WIN64)
+// the GetStackTrace() method is extremly expensive on WindowsIA64, 
+// so we deactivate StackBackTrace for allocation and deallocation on this platform
+static bool allocTraceEnabled = false;   
+#else
+// activate StackBackTrace for allocation and deallocation of memory
+static bool allocTraceEnabled = true;
+#endif
+
 #if defined( SUN )
 #define TRACE_SKIP  2       // skip levels
 #elif !defined( WIN32 )
@@ -142,7 +152,7 @@ public:
     next = n;
     key = k;
     reported_count = 0;
-    memcpy(&trace[0], t, sizeof(trace));
+    SAPDB_MemCopyNoCheck(&trace[0], t, sizeof(trace));
   }
 
   bool traceEqual(voidPtr *t)
@@ -528,9 +538,13 @@ bool LVCSim_AllocFreelist::checkFreeBlock(LVCSim_AllocHeader *blk)
     }
     ++p;
   }
+  
+  if(allocTraceEnabled)
+    blk->deallocTrKey = getTrace(TRACE_SKIP);
+  else
+    blk->deallocTrKey = 0;
 
   // unlink it from allocated list
-  blk->deallocTrKey = getTrace(TRACE_SKIP);
   if (blk->prev) {
     checkRead(blk->prev, blk);
     if (blk->prev->next != blk) {
@@ -705,7 +719,12 @@ void LVCSim_AllocFreelist::registerAlloc(LVCSim_AllocHeader *newblk)
   memset(reinterpret_cast<char*>(newblk + 1) + newblk->blksize, FENCECHAR, cnt);
   newblk->prev = NULL;
   newblk->next = first;
-  newblk->allocTrKey = getTrace(TRACE_SKIP);
+
+  if(allocTraceEnabled)
+    newblk->allocTrKey = getTrace(TRACE_SKIP);
+  else
+    newblk->allocTrKey = 0;
+
   if (first) {
     checkRead(first, NULL);
     if (first->prev != NULL) {
