@@ -678,7 +678,7 @@ public:
 
 static int tls_stack_key = 0;
 
-#if (defined(WIN32) && !defined(_WIN64)) || (defined(LINUX) && !(defined(IA64)||defined(S390X)))
+#if (defined(WIN32) && !defined(_WIN64)) || (defined(LINUX) && defined(I386))
 void LVCSim::SetStackBase(void *ptr)
 {
     int err;
@@ -706,7 +706,7 @@ void LVCSim::SetStackBase(void *ptr)
 extern "C" void *LVCSim_ThreadMainFunction(void *arg)
 {
     LVCSim_ThreadParams *p = (LVCSim_ThreadParams*) arg;
-#if (defined(WIN32) && !defined(_WIN64)) || (defined(LINUX) && !(defined(IA64)||defined(S390X)))
+#if (defined(WIN32) && !defined(_WIN64)) || (defined(LINUX) && defined(I386))
     p->top = (void*) ((((int) &p) + 4095) & ~4095);
     p->bottom = ((char*) p->top) - p->stacksize;
 #endif
@@ -1102,7 +1102,7 @@ int LVCSim::GetStackTrace(void** &trace, size_t traceSize, size_t traceSkip)
           *(trace++) = 0;
         return 0;
     }
-#if (defined(WIN32) && !defined(_WIN64)) || (defined(LINUX) && !(defined(IA64)||defined(S390X)))
+#if (defined(WIN32) && !defined(_WIN64)) || (defined(LINUX) && defined(I386))
     LVCSim_ThreadParams *sinfo = (LVCSim_ThreadParams*) TLSGet(tls_stack_key);
     if (!sinfo) {
         // cannot create stack trace - no stack info
@@ -1137,26 +1137,6 @@ int LVCSim::GetStackTrace(void** &trace, size_t traceSize, size_t traceSkip)
     return level;
 }
 
-void LVCSim::StartInProcServer(OmsHandle &h, const char *libname, const char *methodname, ...)
-{
-  va_list vargs;
-  va_start(vargs, methodname);
-  (*GET_SINK(h))->StartInProcServer(libname, methodname, vargs);
-  va_end(vargs);
-}
-
-void LVCSim::CommitInProcServer(OmsHandle &h, const char *libname)
-{
-  (*GET_SINK(h))->CommitInProcServer(libname);
-  h.omsCommit();
-}
-
-void LVCSim::RollbackInProcServer(OmsHandle &h, const char *libname)
-{
-  (*GET_SINK(h))->RollbackInProcServer(libname);
-  h.omsRollback();
-}
-
 
 // Initialization
 
@@ -1173,7 +1153,7 @@ class LVCSim_Initer {
 public:
   LVCSim_Initer()
   {
-#if (defined(WIN32) && !defined(_WIN64)) || (defined(LINUX) && !(defined(IA64)||defined(S390X)))
+#if (defined(WIN32) && !defined(_WIN64)) || (defined(LINUX) && defined(I386))
     int variableonstack;
     LVCSim::SetStackBase(&variableonstack);
 #endif
@@ -1229,4 +1209,4 @@ private:
   }
 };
 
-static LVCSim_Initer	initer_instance;
\ No newline at end of file
+static LVCSim_Initer	initer_instance;
