@@ -247,7 +247,7 @@ bool LVCSim_ObjectHandle::isContObj(LVCSim_SID sid)
 	while (v) {
 		if (v->m_tid_min <= tid && sid->sees(v->m_tid_min)) {
 			// ok, we see this object version
-      return v->contObj;
+      return v->m_contObj;
 		}
 		v = v->m_next;
 	}
@@ -285,7 +285,7 @@ short LVCSim_ObjectHandle::isVisibleForChangeNoLock(LVCSim_SID sid) const
     	// TODO: now wait until the m_lock is possible, then retest
     	return e_request_timeout;
     }
-    if (m_tid_max != LVCSim_TID_MAX || m_versions->m_tid_min > sid->getCommitTime()) {
+    if (m_tid_max != LVCSim_TID_MAX || (m_versions->m_tid_min > sid->m_readView && sid->m_readView != -1)) { //(m_versions->m_tid_min > sid->getCommitTime())) {
         return e_object_dirty;
     }
     if (sid->hasReadView()) {
@@ -406,7 +406,7 @@ short LVCSim_ObjectHandle::lockObject(LVCSim_SID sid, size_t updateLen, bool ign
         // simulate update on the object by creating a new version
 		m_versions = new(updateLen) LVCSim_ObjectVerHandle(
             sid->getCommitTime(),
-			m_versions->m_data.fix, updateLen, false, false, m_versions);
+			m_versions->m_data.fix, updateLen, false, false, m_versions );
 	}
 
 	return e_ok;	// locked
