@@ -51,7 +51,6 @@
 #include "LVCSimulator/LVCSim_CheckpointIO.hpp"
 
 #include "liveCache/LVC_IliveCacheSink.hpp"
-#include "LVCSimulator/LVCSim_ProcServerInterface.hpp"
 #include "geo07.h"
 
 
@@ -71,8 +70,6 @@ class LVCSim_GlobalAllocator;
 class LVCSim_ContainerID;
 
 class LVC_Dispatcher;
-class LVC_ProcServerExecutor;
-class LVC_ProcServerInterface;
 
 /*!
  * \brief LiveCache session-specific kernel sink.
@@ -208,18 +205,6 @@ protected:
   /// Set dump count.
   static void SetDumpCount(int param);
 
-  /// Start a routine in simulated ProcServer.
-  void StartInProcServer(const char *libname, const char *methodname, va_list &args);
-
-  /// Start a routine in simulated ProcServer.
-  void StartInProcServer(const char *libname, const char *methodname, int dummy, ...);
-
-  /// COMMIT in simulated ProcServer.
-  void CommitInProcServer(const char *libname);
-
-  /// ROLLBACK in simulated ProcServer.
-  void RollbackInProcServer(const char *libname);
-
 	/// Create new sink.
 	LVCSim_liveCacheSink(bool appendToList = true);
 	/// Destroy the sink.
@@ -248,16 +233,6 @@ protected:
 	/// Currently active persistent version or NULL.
     ActiveVersion           *m_activeVersion;   
 
-  /// Communication buffer for simulated ProcServer.
-  LVCSim_ProcServerBuffer   *m_procServerBuffer;
-  /// Local end for ProcServer communication interface for simulated ProcServer.
-  LVC_ProcServerInterface   *m_procServerLocal;
-  /// Local end for ProcServer communication interface for simulated ProcServer.
-  LVC_ProcServerInterface   *m_procServerRemote;
-  /// Thread of the other end in ProcServer simulator.
-  LVCSim_Thread             *m_procServerThread;
-
-
 public:
 
 	STDMETHODIMP QueryInterface(const IID& iid, LPVOID* ppv) 
@@ -337,6 +312,7 @@ public:
     tgg00_FileId             &objFile,
     tgg01_OmsVersionContext  *lpVersionContext,
     void                    **pKeyRangeIterator,
+    tsp00_Int4                bodySize,
     tsp00_Int4                keyLen,
     const void               *pStartKey,              
     const void               *pLowerKey,             
@@ -344,6 +320,7 @@ public:
     tsp00_Int4               &noOfOid,
     OmsTypeOid               *pResultOId,  
     tgg91_PageRef            *pObjSeq,
+    unsigned char            *pBody,
     tsp00_Int4               &LogReadAccesses,
     tgg00_BasisError         &e,
     OmsIterDirection          direction = OMS_ASCENDING);
@@ -541,11 +518,13 @@ public:
     tgg01_ContainerId       &ObjFile,
     tgg01_OmsVersionContext *VersionContext,
     void                    *pKeyIterator,
+    tsp00_Int4               bodySize,
     tsp00_Int4               KeyLen,
     void                    *pRestartKey, 
     tsp00_Int4              &NoOfOId,
     OmsTypeOid              *pResultOId, 
     tgg91_PageRef           *pObjVers,
+    unsigned char           *pBody,
     tsp00_Int4              &objHistReadCount,
     tgg00_BasisError        &e);
 
@@ -564,11 +543,13 @@ public:
     tgg01_ContainerId       &ObjFile,
     tgg01_OmsVersionContext *VersionContext,
     void                    *pKeyIterator,
+    tsp00_Int4               bodySize,
     tsp00_Int4               KeyLen,
     void                    *pRestartKey, 
     tsp00_Int4              &NoOfOId,
     OmsTypeOid              *pResultOId, 
     tgg91_PageRef           *pObjVers,
+    unsigned char           *pBody,
     tsp00_Int4              &objHistReadCount,
     tgg00_BasisError        &e);
 
@@ -892,44 +873,6 @@ public:
       bool  &isUnicodeInstance,
       short &sqlError);
 
-    virtual HRESULT STDMETHODCALLTYPE VersionDictInsert(
-      const OmsVersionId  &versionId,
-      const OMS_Context   *pVersionContext,
-      tgg00_BasisError    &error);
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictFind(
-      const OmsVersionId   &versionId,
-      OMS_Context         **ppVersionContext);
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictDrop(
-      const OmsVersionId  &versionId,
-      tgg00_BasisError    &error);
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictShutdown(); 
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictUnloadOldestVersion(
-      bool &unloaded); 
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictMarkNotUnloadable(
-      OMS_Context *pVersionContext);
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictMarkUnloadable(
-      OMS_Context *pVersionContext,
-      bool         callFromDestructor);
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictCreateIter(
-      OMS_LockMode   lockMode, 
-      void        **ppIter, 
-      OMS_Context **ppContext);
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictGetNext(
-      void         *pIter, 
-      bool          dropCurr,
-      OMS_Context **ppContext);
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictDestructIter(
-      void        *pIter);
-
   virtual int STDMETHODCALLTYPE GetSinkType();
 
 private:
