@@ -899,8 +899,12 @@ inline void Log_InfoPage::WriteSavepointParameters (const SAPDBFields_Field& rrp
     SAPDBERR_ASSERT_ARGUMENT(rrpart.GetLength() == sizeof(param));
     memcpy(&param, rrpart.GetDataPtr(), rrpart.GetLength());
 
-    WriteLastKnownIOSequenceNo(ReadSavepointIOSequenceNo());
-    WriteLastKnownOffset(ReadSavepointEntryOffset());
+    if (ReadSavepointEntryOffset().IsValid()) // PTS 1133214 mb 2004-01-04
+    // For Redosavepoints without offset (e.g. from tape) do not set the lastKnownOffset to invalid
+    {
+        WriteLastKnownIOSequenceNo(ReadSavepointIOSequenceNo());
+        WriteLastKnownOffset(ReadSavepointEntryOffset());
+    }
     // PTS 1128080 mb 2004-02-26 change OldestKnown not for restart-savepoints
     if (!Kernel_IAdminInfo::Instance().KernelStateIsRestart())
     {
