@@ -74,7 +74,7 @@ void Log_Queue::FindStartPointToReserve (tsp00_TaskId                      usert
 
     bool     wasInterupted;
     SizeType available;
-        
+
     do
     {
         wasInterupted = false;
@@ -94,6 +94,7 @@ void Log_Queue::FindStartPointToReserve (tsp00_TaskId                      usert
         
         SAPDBERR_ASSERT_ARGUMENT( ! m_CurrentAppend->IsMarkedForIO() );
         
+        
         // ----------------------------------------------------------
         // check if the the unsplit part fits
         // ----------------------------------------------------------
@@ -146,6 +147,8 @@ void Log_Queue::FindSpaceToReserve (tsp00_TaskId                      usertask,
 
     do
     {    
+        wanted = size; // PTS 1129382 mb 2004-04-28
+
         wasInterupted = false;
         
         FindStartPointToReserve (usertask, waitcontext, size, unsplitsize);
@@ -398,11 +401,14 @@ void Log_Queue::UserTaskEOTReady
     vsuspend(usertaskid, LOG_QUEUE_IO_WAIT_SUSPEND);
 }
 // -----------------------------------------------------------------------------
-void Log_Queue::WaitForFlushLastPage (TaskID          backupCoordinator)
+void Log_Queue::ForceNewPage (TaskID               backupCoordinator,
+                              Log_RawDeviceOffset &offsetOfLastWrittenPage)
 {
-    SAPDBTRACE_METHOD_DEBUG ("Log_Queue::WaitForFlushLastPage", LogVolume_Trace, 5);
+    SAPDBTRACE_METHOD_DEBUG ("Log_Queue::ForceNewPage", LogVolume_Trace, 5);
     
     bool doSuspend = false;
+    Log_IOSequenceNo                   dummyLogIOSequence;
+    Kernel_TaskWaitList::WaitContext   waitContext;
 
     EnterRegion(backupCoordinator); 
     
@@ -410,25 +416,15 @@ void Log_Queue::WaitForFlushLastPage (TaskID          backupCoordinator)
     // if the currentPage is empty, then it need not be flushed 
     {
         m_ForceNewPage = true;
-
-        Log_IOSequenceNo                   dummyLogIOSequence;
-        Log_RawDeviceOffset                dummyLogOffset;
-        Kernel_TaskWaitList::WaitContext   waitContext;
-        if (!m_CurrentAppend->RegisterForIOWait(backupCoordinator, waitContext, dummyLogIOSequence, dummyLogOffset))
+        if (!m_CurrentAppend->RegisterForIOWait(backupCoordinator, waitContext, dummyLogIOSequence, offsetOfLastWrittenPage))
         {
             WriteToTrace();
             RTE_Crash( SAPDBErr_Exception(__FILE__, __LINE__,
                                           SAPDBERR_ASSERT_STATE_FAILED,
-                                          "Log_Queue::WaitForFlushLastPage: RegisterForIOWait() failed") );
+                                          "Log_Queue::ForceNewPage: RegisterForIOWait() failed") );
         }
     
-        // insert task in waitqueue of the last page
-        if (OutputAvailableAt(backupCoordinator, m_CurrentOutput))
-        {
-            // There are pages available for output.
-            // Resume writer task if not already running.
-            m_WriterTaskSync.Resume();
-        }
+        m_WriterTaskSync.Resume();
         doSuspend = true;
     }
 
