@@ -242,10 +242,9 @@ public:
     /*!
         @brief          When saving Log online, the the last page must be flushed
                         in order to save it. (PTS 1123761 mb 2003-09-04)
-        @param          backupCoordinator   Taskid des backupCoordinators, der suspended wird, bis die letzte
-                                            Seite geschrieben wurde.
     */
-    void WaitForFlushLastPage (TaskID          backupCoordinator);
+    void ForceNewPage (TaskID               backupCoordinator,
+                       Log_RawDeviceOffset &offsetOfLastWrittenPage); // PTS  1129651 mb 2004-05-13
 
     /*!
         @brief          Puts together filled log pages for write i/o
