@@ -160,10 +160,8 @@ void Log_Savepoint::StartCoordinator (tgg00_TransContext &Trans)
     SendMessBlock.mb_reply() = false;
     k90send (SendMessBlock, ChildTrans);
     if ( Trans.trError_gg00 != e_ok )
-    {
-        RTE_Crash( SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
-                                      "Log_Savepoint::Trigger: k90send failed") );
-    }
+        RTE_Crash( Log_Exception(__CONTEXT__, LOG_START_SAVEPOINT_FAILED, // PTS 1129781 UH 2004-05-24
+                                 SAPDB_ToString(Trans.trError_gg00)) );
     SAPDBTRACE_WRITELN (LogTrans_Trace, 6, "end: triggered");
 }
 
@@ -213,18 +211,37 @@ bool Log_Savepoint::IsSavepointAllowed() const
     return false;
 }
 /*---------------------------------------------------------------------------*/
-void Log_Savepoint::StartSavepointAndWait (tgg00_TransContext  &Trans,
+void Log_Savepoint::StartSavepointAndWait (tgg00_TransContext  &trans,
                                            Log_SavepointReason  Reason)
 {
     SAPDBTRACE_METHOD_DEBUG ("Log_Savepoint::StartSavepointAndWait", LogTrans_Trace, 5);
 
     if ( ! SystemIsReadyForSavepoint() )
+    {
+        // PTS 1129993 UH 2004-06-16
+        RTE_Message( Log_Exception(__CONTEXT__, LOG_SAVEPOINT_WANTED_BUT_DENIED,
+                     Log_SavepointReasonStrings[Reason],
+                     "Not SystemIsReadyForSavepoint") );
         return;
-
-    tsp00_TaskId        &taskid = Trans.trTaskId_gg00;
+    }
+    
+    tsp00_TaskId        &taskid = trans.trTaskId_gg00;
     Log_SavepointReason  reason = Reason;
     
-    SAPDBTRACE_WRITELN (LogTrans_Trace, 6, "reason: " << Log_SavepointReasonStrings[reason]);
+    if ( ! Log_Volume::Instance().ReserveForEotWillNotBlock() // PTS 1130928 mb 2004-08-10
+         &&
+         (Log_SVPReasonSaveData  == reason ||
+          Log_SVPReasonSavePages == reason) )
+    {
+        // PTS 1129993 UH 2004-06-16 added if-clause
+        RTE_Message( Log_Exception(__CONTEXT__, LOG_SAVEPOINT_WANTED_BUT_DENIED,
+                     Log_SavepointReasonStrings[reason],
+                     "LOG FULL") );
+        trans.trError_gg00 = e_log_full;
+        if ( Log_Volume::Instance().GetDeviceState() == Log_DeviceStateHistoryLost )
+            RTE_Message( Log_Exception(__CONTEXT__, LOG_BACKUP_AFTER_DATA_BACKUP_NEEDED_IN_ADMIN_MODE) );
+        return;
+    }
 
     bool startCoordinator = false;
     
@@ -241,7 +258,7 @@ void Log_Savepoint::StartSavepointAndWait (tgg00_TransContext  &Trans,
     }
     
     if ( ! IsDBManagerReason(Reason) )
-        CurrentWaitList().InsertAsFirst(taskid, Trans.trWaitContext_gg00);
+        CurrentWaitList().InsertAsFirst(taskid, trans.trWaitContext_gg00);
     else
     {
         if ( m_WaitingDBmanager == cgg_nil_pid )
@@ -267,13 +284,13 @@ void Log_Savepoint::StartSavepointAndWait (tgg00_TransContext  &Trans,
     }
     EndRegion (taskid);
 
-    PrintReason (Trans.trTaskId_gg00, Reason, startCoordinator);
+    PrintReason (taskid, Reason, startCoordinator);
 
     if ( startCoordinator )
     {
-        StartCoordinator (Trans);
-        if ( Trans.trError_gg00 != e_ok )
-            RTE_Crash( SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
+        StartCoordinator (trans);
+        if ( trans.trError_gg00 != e_ok )
+            RTE_Crash( Log_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                        "Log_Savepoint::StartSavepointAndWait: StartCoordinator() failed") );
         if ( LogTrans_Trace.TracesLevel(6) )                                          
             Kernel_VTrace() << "*** SAVEPOINT STARTED AND WAIT T" << taskid;
@@ -290,10 +307,31 @@ void Log_Savepoint::StartSavepoint (tgg00_TransContext  &trans,
     SAPDBTRACE_METHOD_DEBUG ("Log_Savepoint::StartSavepoint", LogTrans_Trace, 5);
 
     if ( ! SystemIsReadyForSavepoint() )
+    {
+        // PTS 1129993 UH 2004-06-16
+        RTE_Message( Log_Exception(__CONTEXT__, LOG_SAVEPOINT_WANTED_BUT_DENIED,
+                     Log_SavepointReasonStrings[reason],
+                     "Not SystemIsReadyForSavepoint") );
         return;
+    }
 
     tsp00_TaskId &taskid = trans.trTaskId_gg00;
 
+    if ( ! Log_Volume::Instance().ReserveForEotWillNotBlock() // PTS 1130928 mb 2004-08-10
+         &&
+         (Log_SVPReasonSaveData  == reason ||
+          Log_SVPReasonSavePages == reason) )
+    {
+        // PTS 1129993 UH 2004-06-16 added if-clause
+        RTE_Message( Log_Exception(__CONTEXT__, LOG_SAVEPOINT_WANTED_BUT_DENIED,
+                     Log_SavepointReasonStrings[reason],
+                     "LOG FULL") );
+        trans.trError_gg00 = e_log_full;
+        if ( Log_Volume::Instance().GetDeviceState() == Log_DeviceStateHistoryLost )
+            RTE_Message( Log_Exception(__CONTEXT__, LOG_BACKUP_AFTER_DATA_BACKUP_NEEDED_IN_ADMIN_MODE) );
+        return;
+    }
+
     bool startCoordinator = false;
     
     BeginRegion (taskid);
@@ -378,11 +416,20 @@ void Log_Savepoint::Coordinator (tgg00_TransContext &Trans)
                                                                        : Log_SVPReasonUnknown;
         m_DBmanagerReason  = Log_SVPReasonUnknown;
         m_WaitingDBmanager = cgg_nil_pid;
-        
         saveptParam        = k57restartrec->rstLastSavept_kb00();
         
         EndRegion (Trans.trTaskId_gg00);
 
+		if ( ! Log_Volume::Instance().ReserveForEotWillNotBlock() // PTS 1130928 mb 2004-08-10
+			 &&
+			 Log_SVPReasonShutdown == Reason )
+		{
+	        // PTS 1129993 UH 2004-06-16 added if-clause
+			Trans.trError_gg00 = e_log_full;
+            ShutdownFilesystem (Trans);
+			// never come to this line
+		}
+
         if ( LogTrans_Trace.TracesLevel(6) )                                          
             Kernel_VTrace() << "*** SAVEPOINT START T" << Trans.trTaskId_gg00
                             << " (" << Log_SavepointReasonStrings[Reason] << ")" << NewLine;
@@ -499,6 +546,12 @@ void Log_Savepoint::Coordinator (tgg00_TransContext &Trans)
         k51leave_all_trans_regions (Trans.trTaskId_gg00);
         SAPDBTRACE_WRITELN (LogTrans_Trace, 6, "SVP: new write transactions can be started or removed again");
 
+        if(( Log_SVPReasonShutdown != Reason ) && ( TRUE == g01is_livecache())) // PTS 1129813 TS 2004-05-25
+        {
+            bd91StartOMSGarbageCollectionAfterSpecialOp( Trans.trTaskId_gg00 );
+            SAPDBTRACE_WRITELN (LogTrans_Trace, 6, "SVP: all waiting tasks were resumed");
+        }
+
         Pager_ISavepoint::GetInstance().FlushDataCacheAfterPrepare( Trans.trTaskId_gg00, numPages, numIO );
         numPagesTotal += numPages;
         numIOTotal += numIO;
@@ -539,10 +592,6 @@ void Log_Savepoint::Coordinator (tgg00_TransContext &Trans)
         else
         {
             Pager_ISavepoint::GetInstance().EndSavepoint( Trans.trTaskId_gg00, ! DISABLE_DATAWRITER );
-            if( TRUE == g01is_livecache() )
-                bd91StartOMSGarbageCollectionAfterSpecialOp( Trans.trTaskId_gg00 );
-
-            SAPDBTRACE_WRITELN (LogTrans_Trace, 6, "SVP: all waiting tasks were resumed");
         }
 
         if ( WaitingDBmanager != cgg_nil_pid )
@@ -735,12 +784,20 @@ void Log_Savepoint::FlushCompleted (tsp00_TaskId            TaskId,
     Converter_Version newConverterVersion;
     Converter_ISavePoint::Instance().EndSavepoint( TaskId, newConverterVersion );
 
+    // Set all blocks from state free after savepoint to free. Note that this
+    // has to take place before the data cache is informed about the savepoint end
+    // situation, because this guarantees that blocks which are written after the 
+    // savepoint won't be set to free, because of the following call. 
+    // PTS 1132143 TS 2004-10-26
+
+    FBM_IManager::Instance().SetAllBlocksMarkedAsFreeAfterSVPToFree( TaskId );
+
     // Reset savepoint state and enable data cache to write non 
     // savepoint relevant data pages to the data area
     bd20SavepointCompleted( TaskId, converterVersion );
 
-    // Set all blocks from state free after savepoint to free
-    FBM_IManager::Instance().SetAllBlocksMarkedAsFreeAfterSVPToFree( TaskId );
+    // Set all blocks from state free after savepoint to free  
+    //FBM_IManager::Instance().SetAllBlocksMarkedAsFreeAfterSVPToFree( TaskId ); PTS 1132143 TS 2004-10-26
     
     // Free pnos removed during prepare phase 
     Converter_ISavePoint::Instance().FreePageNosAfterSavepoint( TaskId );
