@@ -69,6 +69,7 @@
 #include "Logging/Log_ActionUnloadFile.hpp"
 #include "Logging/Log_ActionUpdateRec.hpp"
 #include "Logging/Log_ActionPartialRollback.hpp"
+#include "Logging/Log_ActionSavepoint.hpp"
 #include "Restart/Rst_IRedoTrafficControl.hpp"
 #include "Logging/Log_Exceptions.hpp"
 #include "Logging/Log_History.hpp"
@@ -78,6 +79,7 @@
 #include "hkb54.h"  // k54rollback_temp
 #include "hkb57.h"  // k57surrogates_to_log
 #include "hgg01.h"  // g01is_livecache()
+#include "RunTime/Configuration/RTEConf_ParameterAccessKernelInterface.hpp"
 
 /*===========================================================================*
  *  DEFINES                                                                  *
@@ -89,6 +91,8 @@
  *  LOCAL *
  *===========================================================================*/
 
+/// 0 means is not set
+static SAPDB_UInt LocalRedoBufferSize = 0; // PTS 1131107 UH 2004-08-24 added
 
 /* -------------------------------------------------------------------------------- */
 static void TraceEntry ( const char*          title,
@@ -225,6 +229,15 @@ bool Log_Transaction::Initialize (tgg00_TransContext &Trans)
         Delete();
         return false;
     }
+
+	// PTS 1131107 UH 2004-08-24
+    if ( LocalRedoBufferSize != 0
+         &&
+         ! m_LocalRedoEntries.Initialize(Allocator(),LocalRedoBufferSize) )
+    {
+        Delete();
+        return false;
+    }
     
     return true;
 }
@@ -247,6 +260,9 @@ void Log_Transaction::Delete()
     {
         destroy (m_pSubtransStack, Allocator());
     }
+    
+    m_LocalRedoEntries.Delete();
+    
     m_pTrans          = NULL;
     m_pAllocator      = NULL;
 }
@@ -285,6 +301,15 @@ bool Log_Transaction::InitializeForRedoCopy (tgg00_TransContext     &redoReaderT
                    SAPDBERR_ASSERT_STATE_FAILED,
                    "Log_Transaction::InitializeForRedo() allocation failed") );
     
+	// PTS 1131107 UH 2004-08-24
+    if ( LocalRedoBufferSize != 0
+         &&
+         ! m_LocalRedoEntries.Initialize(Allocator(),LocalRedoBufferSize) )
+    {
+        Delete();
+        return false;
+    }
+
     return true;
 }
 
@@ -297,11 +322,14 @@ void Log_Transaction::DeleteAfterRedoCopy()
                    "Log_Transaction::DeleteAfterRedoCopy() allocator not assigned") );
     if ( m_pRedoFile != 0 )
     {
-        Allocator().Deallocate(m_pRedoFile);
+        m_pRedoFile->Delete();                // PTS 1131935 mb 2004-10-14
+        Allocator().Deallocate(m_pRedoFile); // PTS 1131935 mb 2004-10-14
         m_pRedoFile = 0;
     }
     m_pAllocator  = 0;
     m_pTrans      = 0;
+    
+    m_LocalRedoEntries.Delete();
 }
 
 /* -------------------------------------------------------------------------------- */
@@ -582,28 +610,6 @@ void Log_Transaction::EndOfTransHandling (TransState State)
     SetTransState (State);
     kb50LeaveTransRegion (m_pTrans->trTaskId_gg00, m_pTrans->trIndex_gg00);
 
-	#ifdef CHECK_COMMITTED_NEWS
-    if ( LogTrans_Check.ChecksLevel(6) )
-    {
-        if ( online == m_TransType
-             &&
-             m_ReservedObjects.Check() )
-        {
-            ReservedObjects::Iterator  iter = m_ReservedObjects.Check();
-            while ( iter )
-            {
-                Kernel_VTrace() << "reserved object: "
-                                << iter()->gg92GetPno() << "." << iter()->gg92GetPos();
-                ++iter;
-            }
-            // no reserved objects may exist in the list
-            RTE_Crash( SAPDBErr_Exception(__FILE__, __LINE__,
-                       SAPDBERR_ASSERT_STATE_FAILED,
-                       "reserved objects become committed (look into vtrace)") );
-        }
-	}
-	#endif
-
     #ifdef SAPDB_QUICK
     if ( online == m_TransType
          &&
@@ -1098,34 +1104,6 @@ void Log_Transaction::ExecuteUndoAction
                 return;
             }
             
-            #ifdef CHECK_COMMITTED_NEWS
-	        if ( LogTrans_Check.ChecksLevel(6) )
-            {
-                switch ( BeforeImage.GetActionType() )
-                {
-                    case Log_NewObject:
-                    {
-                        RemoveReservedObject (ActionObject.GetOid());
-                        break;
-                    }
-                    case Log_InsertObject:
-                    {
-                        AddReservedObject (ActionObject.GetOid());
-                        break;
-                    }
-                    case Log_DeleteObject:
-                    {
-                        if ( BeforeImage.GetPreviousState() == obsReserved_egg00
-                             ||
-                             BeforeImage.GetPreviousState() == obsKeyReserved_egg00 )
-                            AddReservedObject (ActionObject.GetOid());
-                        break;
-                    }
-                    default:
-                        break; // nothing
-                }
-            }
-            #endif
             // the undo file resources must be released to avoid
             // a deadlock
             
@@ -1328,7 +1306,6 @@ void Log_Transaction::ExecuteRedoAction
     
     if ( redocontrol.CheckIoSeq ( m_pTrans->trTaskId_gg00,
                                   m_pTrans->trWaitContext_gg00,
-                                  m_pTrans->trIndex_gg00,
                                   IOSequence ) )
     {
         iter.Break();
@@ -1619,6 +1596,10 @@ Data_SplitSpaceWriter Log_Transaction::ReserveSpaceForRedoEntryCopy
         m_OldestKnownIOSequence = IOSequence; // PTS 1127704 mb 2004-02-11
     }
     SAPDBERR_ASSERT_STATE ( m_pRedoFile->IsCreated() );
+    #ifdef SAPDB_SLOW
+    Kernel_VTrace() << "Length:        " << Length        << ", MinimalLength: " << MinimalLength;
+    Kernel_VTrace() << "EntrySequence: " << EntrySequence << ", IOSequence: " << IOSequence;
+    #endif
     Data_SplitSpaceWriter Writer = m_pRedoFile->ReserveSpace ( Length,
                                                                MinimalLength,
                                                                EntrySequence,
@@ -1649,6 +1630,12 @@ Log_Transaction::WriteRedoEntry (Log_AfterImage    &AfterImage,
     SAPDBTRACE_METHOD_DEBUG ("Log_Transaction::WriteRedoEntry", LogTrans_Trace, 5);
     
     SAPDBERR_ASSERT_STATE ( IsInitialized() );
+    #ifdef SAPDB_QUICK
+    if ( (void*)k50GetLogTransaction (m_pTrans->trIndex_gg00) != (void*)this )
+        RTE_Crash( SAPDBErr_Exception(__FILE__, __LINE__,SAPDBERR_ASSERT_STATE_FAILED,
+                   "Log_Transaction::WriteRedoEntry() logtransptr lost") );
+    #endif
+    
 
     Log_EntrySequence savedLastRedoEntrySequence = m_Context.LastRedoEntrySequence;
     if ( m_Context.LastRedoEntrySequence.IsValid() )
@@ -1671,17 +1658,63 @@ Log_Transaction::WriteRedoEntry (Log_AfterImage    &AfterImage,
             m_Context.TransNo = m_pTrans->trWriteTransId_gg00;
     }
 
-    #ifdef SAPDB_QUICK
-    if ( (void*)k50GetLogTransaction (m_pTrans->trIndex_gg00) != (void*)this )
-        RTE_Crash( SAPDBErr_Exception(__FILE__, __LINE__,SAPDBERR_ASSERT_STATE_FAILED,
-                   "Log_Transaction::WriteRedoEntry() logtransptr lost") );
-    #endif
-    
     AfterImage.SetTransNo          (m_Context.TransNo);
     AfterImage.SetSequence         (m_Context.LastRedoEntrySequence);
     AfterImage.SetLastUndoSequence (m_Context.LastUndoEntrySequence);
     
-	if ( LogTrans_Trace.TracesLevel(6) )
+
+    if ( LocalRedoBufferSize != 0 // PTS 1131107 UH 2004-08-24
+         &&
+         AfterImage.GetActionType() != Log_MultipleActions )
+    {
+        if ( Action.LocalRedoEntryCashingPossible() )
+        {
+            if ( m_LocalRedoEntries.Append(AfterImage, Action) )
+            {
+                if ( LogTrans_Check.ChecksLevel(5) )
+                    TraceEntry ( "WriteRedoEntry-Append",
+                                 m_Context.TransNo,
+                                 m_pTrans->trSubtransId_gg00,
+                                 m_Context.LastRedoEntrySequence,
+                                    AfterImage.GetActionType() );
+                return noError;
+            }
+            if ( ! m_LocalRedoEntries.IsEmpty() )
+            {
+                // local buffer is full so flush it and then append current entry
+                Log_AfterImage image (Log_MultipleActions);
+                const LogState result = WriteRedoEntry (image,m_LocalRedoEntries);
+                if ( result != noError )
+                    return result;
+                m_LocalRedoEntries.Reset();
+                if ( m_LocalRedoEntries.Append(AfterImage, Action) )
+                {
+                    if ( LogTrans_Check.ChecksLevel(5) )
+                        TraceEntry ( "WriteRedoEntry-Append",
+                                     m_Context.TransNo,
+                                     m_pTrans->trSubtransId_gg00,
+                                     m_Context.LastRedoEntrySequence,
+                                        AfterImage.GetActionType() );
+                    return noError;
+                }
+            }
+            // here: appending is not possible write the entry to the log as usual
+        }
+        else
+        {
+            // cached redo entries must be flushed before end of trans log entry
+            if ( ! m_LocalRedoEntries.IsEmpty() )
+            {
+                Log_AfterImage image (Log_MultipleActions);
+                const LogState result = WriteRedoEntry (image,m_LocalRedoEntries);
+                if ( result != noError )
+                    return result;
+                m_LocalRedoEntries.Reset();
+            }
+        }
+    }
+
+    if ( LogTrans_Trace.TracesLevel(6) )
     	TraceEntry ( "WriteRedoEntry",
         	         m_Context.TransNo,
             	     m_pTrans->trSubtransId_gg00,
@@ -1708,31 +1741,6 @@ Log_Transaction::WriteRedoEntry (Log_AfterImage    &AfterImage,
                    "Log_Transaction::WriteRedoEntry: space init failed") );
     }
 
-    #ifdef CHECK_COMMITTED_NEWS
-    if ( LogTrans_Check.ChecksLevel(6) )
-    {
-        switch ( AfterImage.GetActionType() )
-        {
-            case Log_NewObject:
-            {
-                const Log_ActionObject &objAction = *(reinterpret_cast<const Log_ActionObject*>(&Action));
-                AddReservedObject (objAction.GetOid());
-                break;
-            }
-            case Log_InsertObject:
-            case Log_UpdateObject:
-            case Log_DeleteObject:
-            {
-                const Log_ActionObject &objAction = *(reinterpret_cast<const Log_ActionObject*>(&Action));
-                RemoveReservedObject (objAction.GetOid());
-                break;
-            }
-            default:
-                break; // nothing
-        }
-    }
-    #endif
-
     if ( m_pQueue != NULL )
     {        
         SAPDB_UInt2 offsetInPage;
@@ -2054,7 +2062,7 @@ void  Log_Transaction::RedoPartialRollback (Log_EntrySequence UndoStopSequence,
     }
     // PTS 1113230 UH 2001-12-20 end
         
-    if ( ! m_pRedoFile->IsCreated() )
+    if ( GetTransState() != opened || ! m_pRedoFile->IsCreated() )
         // The partial rollback entry may be the first entry read after the savepoint
         // for a transaction which was open during savepoint.
         // So there is no redofile which can be truncated.
@@ -2286,6 +2294,153 @@ const char* Log_RollbackResultStrings [] =
     "undoLogFull"
 };
 
+/* -------------------------------------------------------------------------------- */
+inline SAPDB_UInt GetMinimalLengthOfActionToCopy (Log_ActionType type)
+{
+	// PTS 1131107 UH 2004-08-24 added
+    switch (type)
+    {
+        case Log_NewObject:
+        case Log_InsertObject:
+        case Log_UpdateObject:
+        case Log_DeleteObject:
+            return Log_ActionObject::GetMinimalLengthToCopyEntry();
+            break;
+        case Log_PartialRollback:
+            return Log_ActionPartialRollback::GetMinimalLengthToCopyEntry();
+            break;
+        case Log_SavepointEntry:
+        case Log_FinishOpenTrans:
+            return Log_ActionSavepoint::GetMinimalLengthToCopyEntry();
+            break;
+        case Log_MultipleActions:
+            return Log_ActionMultiple::GetMinimalLengthToCopyEntry();
+            break;
+        default:
+            return 0;
+    }
+    return 0;
+}
+/* -------------------------------------------------------------------------------- */
+Data_SplitSpaceWriter Log_Transaction::GetWriterForLocalRedoEntries(SAPDB_Byte *pActionHead,
+                                                                    SAPDB_UInt sizeToReserve)
+{
+	// PTS 1131107 UH 2004-08-24 new
+    SAPDBTRACE_METHOD_DEBUG ("Log_Transaction::GetWriterForLocalRedoEntries", LogTrans_Trace, 5);
+    if (! m_LocalRedoEntries.Reserve(pActionHead, sizeToReserve) )
+        RTE_Crash( Log_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
+                   "m_LocalRedoEntries.Reserve()") );
+    return Data_SplitSpaceWriter(m_LocalRedoEntries.GetBuffer());
+}
+
+/* -------------------------------------------------------------------------------- */
+bool Log_Transaction::RestoreLocalRedoEntries (Log_IOSequenceNo iosequence)
+{
+	// PTS 1131107 UH 2004-08-24 new
+    SAPDBTRACE_METHOD_DEBUG ("Log_Transaction::RestoreLocalRedoEntries", LogTrans_Trace, 5);
+
+    Log_AfterImage                image;
+    SAPDB_UInt                    actionLength;
+    const SAPDB_Byte             *pActionSpace;
+    Log_ActionMultiple::Iterator  actioniter  = m_LocalRedoEntries.GetFirstAction();
+    Data_SplitSpaceWriter::Result writeResult = Data_SplitSpaceWriter::ok;
+    bool                          isOK        = true;
+
+    #ifdef SAPDB_SLOW
+    m_LocalRedoEntries.WriteToTrace("Log_Transaction::RestoreLocalRedoEntries()");
+    SAPDB_UInt actionno = 1;
+    #endif
+    
+    while ( actioniter.GetAction(image, actionLength, pActionSpace) )
+    {
+        #ifdef SAPDB_SLOW
+        Kernel_VTrace() << "MultipleActions: internal #" << actionno++;
+        image.WriteToTrace("IMAGE");
+        Kernel_VTrace() << "GetLastCopiedRedoEntrySequence() = " << GetLastCopiedRedoEntrySequence();
+        #endif
+        if ( actionLength > LocalRedoBufferSize )
+        {
+            Kernel_VTrace() << "actionLength is invalid" << actionLength;
+            return false;
+        }
+        
+        if ( (m_Context.LastRedoEntrySequence.IsInvalid()
+              ||
+              image.GetSequence() >= m_Context.LastRedoEntrySequence)
+             &&
+             (GetLastCopiedRedoEntrySequence().IsInvalid()
+              ||
+              image.GetSequence() > GetLastCopiedRedoEntrySequence()) )
+        {
+            #ifdef SAPDB_SLOW
+            Kernel_VTrace() << "IS COPIED";
+            #endif
+            Data_SplitSpaceWriter writer = ReserveSpaceForRedoEntryCopy
+                                ( actionLength + image.GetPersistentLength(),
+                                  GetMinimalLengthOfActionToCopy(image.GetActionType())
+                                  + image.GetPersistentLength(),
+                                  image.GetSequence(),
+                                  iosequence );
+            image.SetTransNo(GetRedoTransNo());
+            image.WritePersistentFormat(writer, isOK);
+            if ( ! isOK )
+                return false;
+                
+            writer.Write (pActionSpace, actionLength, writeResult);
+            
+            if ( writeResult != Data_SplitSpaceWriter::ok )
+            {
+                ReleaseSpaceAfterRedoEntryCopy(false);
+                return false;
+            }
+            ReleaseSpaceAfterRedoEntryCopy(true);
+        }
+        else
+        {
+            #ifdef SAPDB_SLOW
+            Kernel_VTrace() << "IS NOT COPIED";
+            #endif
+        }
+        actioniter.Next();
+    }
+    m_LocalRedoEntries.Reset();
+    return true;
+}
+
+// -----------------------------------------------------------------------------
+void Log_Transaction::DetermineLocalRedoBufferSize()
+{
+	// PTS 1131107 UH 2004-08-24
+    const SAPDB_UTF8 *         paramName    = UTF8("LOCAL_REDO_LOG_BUFFER_SIZE");
+    RTEConf_Parameter::Integer size = 0;
+    SAPDBErr_MessageList       error;
+
+    if ( RTEConf_ParameterAccess::Instance() == 0 )
+    {
+        RTE_Message (Log_Exception(__CONTEXT__,LOG_LOCAL_REDO_BUFFER_SIZE_SET, "0"));
+        return;
+    }
+
+    if ( ! RTEConf_ParameterAccess::Instance()->GetInteger(paramName, size, error) )
+    {
+        RTE_Message (error);
+        RTE_Message (Log_Exception(__CONTEXT__,LOG_LOCAL_REDO_BUFFER_SIZE_SET, "0"));
+        return;
+    }
+    
+    if ( size > SAPDB_MAX_UINT4 )
+    {
+        RTE_Message(Log_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"size <= SAPDB_MAX_UINT4"));
+        RTE_Message(Log_Exception(__CONTEXT__,LOG_LOCAL_REDO_BUFFER_SIZE_SET, "0"));
+        return;
+    }
+
+    LocalRedoBufferSize = static_cast<SAPDB_UInt>(size);
+    
+	RTE_Message(Log_Exception(__CONTEXT__,LOG_LOCAL_REDO_BUFFER_SIZE_SET,
+                              SAPDB_ToString(LocalRedoBufferSize)));
+}
+
 /*===========================================================================*
  *  END OF CODE                                                              *
  *===========================================================================*/
