@@ -35,11 +35,6 @@
 #ifndef LOG_TRANSACTION_HPP
 #define LOG_TRANSACTION_HPP
 
-#ifndef SUN
-/// debug-flag for checking reserved objects
-#define CHECK_COMMITTED_NEWS 1
-#endif
-
 #include "ggg00.h"
 #include "ggg92.h"
 
@@ -49,8 +44,8 @@
 #include "Container/Container_Stack.hpp"
 #include "Logging/Log_Types.hpp"
 #include "Logging/Log_UndoFile.hpp"
+#include "Logging/Log_ActionMultiple.hpp"
 #include "Restart/Rst_RedoFile.hpp"
-#include "liveCache/LVC_NewObjectChecker.hpp"
 
 class SAPDBMem_IRawAllocator;
 class Log_Queue;
@@ -174,7 +169,7 @@ public:
     }
     
     /// used by redo to get the open trans info.
-    tgg91_TransNo GetRedoTransNo() const
+    const tgg91_TransNo& GetRedoTransNo() const
     {
         return m_Context.TransNo;
     }
@@ -186,7 +181,7 @@ public:
     }
     
     /// used by redo reader to not copy entries, already copied
-    Log_EntrySequence GetLastRedoEntrySequence() const
+    Log_EntrySequence GetLastCopiedRedoEntrySequence() const
     {
         return m_Context.LastCopiedRedoEntrySequence;
     }
@@ -401,20 +396,18 @@ public:
         return *m_pTrans;
     }
 
-    #ifdef CHECK_COMMITTED_NEWS
-
-    /// adds an object into the list of reserved objects
-    void AddReservedObject (const tgg92_KernelOid& oid)
+    /// During redo copy the local entries are restored and distributed to the redo file
+    bool RestoreLocalRedoEntries(Log_IOSequenceNo iosequence);
+	/// returns a split space writer which works on the local redo entry buffer
+    Data_SplitSpaceWriter GetWriterForLocalRedoEntries(SAPDB_Byte *pActionHead,
+                                                       SAPDB_UInt  sizeToReserve);
+	/// This resets the internal redo entries and empties the buffer
+    void ResetLocalRedoEntries()
     {
-        m_ReservedObjects.Insert(oid);
+        m_LocalRedoEntries.Reset();
     }
-
-    /// removes an object from the list of reserved objects
-    void RemoveReservedObject (const tgg92_KernelOid& oid)
-    {
-        m_ReservedObjects.Delete(oid);
-    }
-    #endif
+    /// This reads the paramter LOCAL_REDO_LOG_BUFFER_SIZE and sets local variable
+    static void DetermineLocalRedoBufferSize();
 
 private: 
 
@@ -571,15 +564,8 @@ private: // member
     /// @attention this MUST not copied in the copy-operator, because the RTE must allocate
     /// a new spinlock for undo/redo
     mutable RTESync_Spinlock m_PersistentAccess;
-    
-    #ifdef CHECK_COMMITTED_NEWS
-    
-    /// A list which holds all objects which are reserved, but not filled.
-    typedef LVC_NewObjectChecker<tgg92_KernelOid> ReservedObjects;
-    
-    /// A list which holds all objects which are reserved, but not filled.
-    ReservedObjects m_ReservedObjects;
-    #endif
+	/// redo entries which are written as a block to the redo log
+    Log_ActionMultiple m_LocalRedoEntries; // PTS 1131107 UH 2004-08-24 added
 };
 
 /// for converting enum type to string
@@ -658,6 +644,12 @@ public:
         }
     }
 
+    /// if this returns false a local redo entry cash must be flushed
+    virtual bool LocalRedoEntryCashingPossible() const
+    {
+        return false;
+    }
+
     const Kernel_Date GetCommitDate() const
     {
         return m_commitTimestamp.CommitDate;
