@@ -101,7 +101,7 @@ public:
     bool IsInRange( const Log_IOSequenceNo leftBorder,
                     const Log_IOSequenceNo rightBorder) const
     {
-        if (leftBorder < rightBorder)
+        if (leftBorder <= rightBorder) // PTS 1131546 mb 2004-09-27
         // no IOSequence overflow within logBackup
         {
             if ((leftBorder <= *this) && (*this <= rightBorder))
@@ -415,7 +415,9 @@ enum Log_ActionType
     Log_UnloadFile     = 34,
     Log_SetFileState   = 35,
     //
-    Log_ActionType_DO_NOT_USE = 36
+    Log_MultipleActions = 36,
+    //
+    Log_ActionType_DO_NOT_USE = 37
 };
 /// for tracing the action types
 extern const char* Log_ActionTypeStrings[Log_ActionType_DO_NOT_USE];
