@@ -105,7 +105,6 @@ public:
     }
     /*!
        @brief          Constructor
-       @param          taskid [in] id of the calling task
        @param          writertaskid [in] id of the associated writer task
        @param          alloc [in] raw allocator to be used
        @param          framealloc [in] raw allocator for allocating pageframes
@@ -113,8 +112,7 @@ public:
        @param          flushmode [in] log queue flush mode
      */
     Log_Volume 
-        (TaskID          taskid,
-         TaskID          writertaskid,
+        (TaskID          writertaskid,
          Allocator&      alloc,
          FrameAllocator &framealloc,
          QueueSyncMode   syncmode,
@@ -129,7 +127,8 @@ public:
          m_LogWriter      (m_Allocator, m_LogQueues, IOMan_ILogIO::GetInstance(), m_WriterTaskSync),
          m_IOPages        (m_Allocator, UTF8( "Log_Volume::WriteLogPages" )),
          m_QueueSyncMode  (syncmode),
-         m_FlushMode      (flushmode)
+         m_FlushMode      (flushmode),
+         m_SaveIterator   (alloc, framealloc, m_PageReader)
     {}
     /*!
        @brief   Creates archive log for new database
@@ -227,23 +226,33 @@ public:
     }
 
     /// describes the state of a save-iterator
-    enum SaveInfo
+    enum InitLogSaveResult
     {
-        okay,
-        noLogToSave,
-        historyIsLost,
-        incompleteSegment
+        Okay,
+        NoLogToSave,
+        HistoryIsLost,
+        IncompleteSegment,
+        InitializeFailed
+    };
+
+    // defines how much of the unsaved log-pages will be denoted by the 
+    // newly constructed Log_SaveIterator (PTS 1128703 mb 2004-04-23)
+    enum SaveRange
+    {
+        AllAtOnce,                     ///< saves all Log_Pages with one Log-SaveIterator
+        AllIncompleteSegmentsAllowed,  ///< saves in Segments, the last may be incomplete
+        OnlyCompleteLogSegment         ///< saves only complete segments
     };
     
     /// allocates an iterator for saving the log
-    void GetSaveLogIterator( tsp00_TaskId            TaskId,
-                             SAPDBMem_IRawAllocator &Allocator,
-                             bool                    IsCold,
-                             bool                    ForAutosave,
-                             bool                    completeLog,
-                             Log_DeviceBlockCount    blockSize,
-                             Log_SaveIterator*      &iterator,
-                             SaveInfo               &saveInfo);
+    const InitLogSaveResult InitLogSaveIterator( const tsp00_TaskId            taskId,
+                                                       SAPDBMem_IRawAllocator &allocator,
+                                                 const SaveRange               rangeToBeSaved, // PTS 1128703  
+                                                 const SAPDB_Bool              repeatable,                           
+                                                 const Log_DeviceBlockCount    blockSize);
+
+    // returns the current iterator for saving log
+    Log_SaveIterator & GetLogSaveIter() { return m_SaveIterator; };
 
     /*!
        @brief          returns a new backup-count for numbering the log_save-files
@@ -267,12 +276,38 @@ public:
     /// returns the flag whether the log may overwritten without save_log
     bool IsAutomaticOverwrite (tsp00_TaskId taskid);
 
+    /// result set for FreeLog-calls
+    enum FreeLogResult
+    {
+        /// the operation succeded
+        FreeLogOk,
+
+        /// the IOSequence of the free command does not match to the last saved segment 
+        IOSequenceDoesNotMatch,     
+
+        /// open of log volume failed
+        LogVolumeNotAccessible,
+        
+        /// no save log done before
+        NoSaveLogExecutedBefore,
+        
+        /// no log freed due to repeatable-flage
+        RepeatedLogSaveNeeded
+    };
+    
+    /*!
+    @brief frees the logPages after a save into a pipe
+
+    frees the log which had been saved by the previous "save log into pipe".
+    It is checked, that the given IOSequences matches the saved values from
+    the previous save
+    */
+    FreeLogResult FreeLogForPipe(const tsp00_TaskId         taskid,
+                                 const Log_IOSequenceNo     startIOSeq,
+                                 const Log_IOSequenceNo     endIOSeq);
+
     /// This releases log pages, which are already backuped.
-    bool FreeLog (tsp00_TaskId        taskid, 
-                  Log_IOSequenceNo    oldLastSavedIOsequence,
-                  Log_RawDeviceOffset oldLastSavedOffset,
-                  Log_IOSequenceNo    newLastSavedIOsequence,
-                  Log_RawDeviceOffset newLastSavedOffset);
+    FreeLogResult FreeLog (const tsp00_TaskId        taskid);
 
     /// This releases entries in a logpage up to a specified IOSequence 
     /// @attention This call is not synchronized with other accesses
@@ -289,77 +324,6 @@ public:
                           const Log_RawDeviceOffset  pageOffset,
                                 bool                &isEmpty);
 
-    /// result set for operations which has to be performed
-    /// additionally for saving into a pipe:
-    enum PipeHandlingResult
-    {
-        /// the operation succeded
-        pipeOk,
-
-        /// the IOSequence of the free command does not match to the last
-        /// saved segment 
-        IOSequenceDoesNotMatch,     
-
-        /// the last saved segment was no pipe
-        lastSaveWasNot2Pipe,
-
-        /// after the last save into a pipe, no freeLog had
-        /// been called
-        noFreeLogCalledInbetween,
-        
-        /// open on log volumes failed
-        logVolumeNotAccessible
-    };
-    
-    /// saves information about the saved log_segment for a subsequent call
-    /// to ExecuteFreeLogForPipe
-    PipeHandlingResult PrepareFreeLogForPipe(
-                        const Log_IOSequenceNo     firstIOSeqSaved,
-                        const Log_IOSequenceNo     lastIOSeqSaved,
-                        const Log_RawDeviceOffset  firstPageSaved,
-                        const Log_RawDeviceOffset  lastPageSaved    );
-
-    /*!
-    @brief frees the logPages after a save into a pipe
-
-    frees the log which had been saved by the previous "save log into pipe".
-    It is checked, that the given IOSequences matches the saved values from
-    the previous PrepareFreeLogForPipe
-    */
-    PipeHandlingResult ExecuteFreeLogForPipe(
-                        const tsp00_TaskId         taskid,
-                        const Log_IOSequenceNo     startIOSeq,
-                        const Log_IOSequenceNo     endIOSeq);
-private:
-
-    /// summarizes all information for the handling of saving into pipes
-    struct  PipeSaveState
-    {
-        /// true, if no FreeLog had been executed yet after a save into a pipe
-        bool                 waitingForExecuteFreeLogForPipe;
-
-        /// first iosequence of the first LogPage saved
-        Log_IOSequenceNo     firstIOSeqSaved;
-        
-        /// last iosequence of the last LogPage saved
-        Log_IOSequenceNo     lastIOSeqSaved;
-
-        /// offset of the first pages saved
-        Log_RawDeviceOffset  firstPageSaved;
-
-        /// offset of the last page saved
-        Log_RawDeviceOffset  lastPageSaved; 
-        
-        /// default init: set waitingForExecuteFreeLogForPipe to false
-        PipeSaveState()
-        {
-            waitingForExecuteFreeLogForPipe = false;
-        }
-    };
-    
-    ///  state of the last save of the log (if it was a pipe)
-    PipeSaveState m_pipeSaveState;
-        
 public:
     /// Returns lowest offset on raw log device to be used for cyclical log write i/o
     /// Offset is located behind the leading log info page block(s) on the raw device.
@@ -394,9 +358,14 @@ public:
         return  m_LogInfoPage.ReadOldestKnownIOSequenceNo();
     }
     
+    /// returns true, if a subsequent ResoreveForEot will not block because of LogFull
+    bool ReserveForEotWillNotBlock() const
+    {
+        return m_LogWriter.PagesAreFreeForWrite();
+    }
+    
     /// Checks whether no more log can be written (Not valid for utility)
-    SAPDB_Bool LogIsFull 
-        (TaskID                         taskid)
+    SAPDB_Bool LogIsFull ()
     {
         return m_LogWriter.LogIsFull();
     }
@@ -562,7 +531,7 @@ public:
     const;
 
     /// The iterator is set to the last savepoint entry of the restart record.
-    void SetToLastSavepointEntry (Log_VolumeIterator &iter);
+    void SetToLastSavepointEntry (tsp00_TaskId  taskId,Log_VolumeIterator &iter);
 
     /// reads the iosequence of the last savepoint from the restartrecord
     Log_IOSequenceNo  GetSavepointIOSeq();
@@ -599,6 +568,11 @@ public:
     /// @attention  Must not be called after Restart
     void ScanLogForFirstKnown (TaskID           taskid);
 
+    /// searches the log_volume for the given IOSequence and returns the offset,
+    /// on which this Log_IOSequenceNo resides (PTS 1129184)
+    Log_RawDeviceOffset SearchSavepointOffset(      TaskID              taskid,
+                                              const Log_IOSequenceNo    savepointIOSequence);
+
     /// This method is returns the values, which had been set by ScanLogForFirstKnown
     void GetFirstKnownBeforeRestoreLog (RawDeviceOffset     &firstKnownOffset,
                                         IOSequenceNo        &firstKnownIOSeq) 
@@ -887,7 +861,10 @@ private:
     IOSequenceNo                                        m_FirstKnownIOSeq;
 	
     /// holds the size of a saved LogSegment in pages (PTS 1113361 mb 2002-01-11)
-	DeviceBlockCount               						m_autosaveDistance;
+    DeviceBlockCount                                    m_autosaveDistance;
+    
+    /// the iterator, which is used to read the logpages to be saved
+    Log_SaveIterator                                    m_SaveIterator;
 };
 
 
