@@ -84,7 +84,8 @@ Log_Writer::Log_Writer (SAPDBMem_IRawAllocator     &alloc,
   m_CheckOffsetCount  (0),
   m_SuspendedByUser   (false),          // PTS 1114053 UH 2002-10-22 new
   m_UserWaitingForSuspend (cgg_nil_pid),// PTS 1114053 UH 2002-10-22 new
-  m_LogUsageEvents    (sp31ei_log_above_limit, sp31ei_nil) // PTS 1124743 UH 2003-10-21
+  m_LogUsageEvents    (sp31ei_log_above_limit, sp31ei_nil), // PTS 1124743 UH 2003-10-21
+  m_PagesFreeOnDevice (0) // PTS 1129443 mb 2004-05-03
 {
     SAPDBTRACE_METHOD_DEBUG ("Log_Writer::Log_Writer", LogVolume_Trace, 5);
     // PTS 1124743 UH 2003-10-21 added events
@@ -132,9 +133,9 @@ void Log_Writer::Run()
     // set start position for queue loop:
     // ----------------------------------------------------------
     QueueIterator        queueiter         = m_LogQueues.Anchor();
-    Log_DeviceBlockCount pagesFreeOnDevice = FreePagesForLogWriter();
+    m_PagesFreeOnDevice = FreePagesForLogWriter(); // PTS 1130059 mb 2004-06-09
     
-    if ( EnoughPagesFreeForUserTaskWrite(pagesFreeOnDevice) )                   // PTS 1115488 mb 2002-05-03
+    if ( EnoughPagesFreeForUserTaskWrite(m_PagesFreeOnDevice) )                   // PTS 1115488 mb 2002-05-03
     {
         // enable write for everybody into the Log_Queue
         SetLogFullQueueState(m_WriterTaskSync.GetTaskID(), false);
@@ -145,7 +146,7 @@ void Log_Writer::Run()
         // ----------------------------------------------------------
         // check for log full and suspend if necessary
         // ----------------------------------------------------------
-        while ( pagesFreeOnDevice == 0 )
+        while ( m_PagesFreeOnDevice == 0 )
         {
             RTE_Message(Log_Exception(__CONTEXT__,LOG_IS_FULL));
             
@@ -154,9 +155,9 @@ void Log_Writer::Run()
             if (m_Active == false)
                 return;
             
-            pagesFreeOnDevice = FreePagesForLogWriter();
+            m_PagesFreeOnDevice = FreePagesForLogWriter();
             
-            if ( ! EnoughPagesFreeForUserTaskWrite(pagesFreeOnDevice) )                   // PTS 1115488 mb 2002-05-03
+            if ( ! EnoughPagesFreeForUserTaskWrite(m_PagesFreeOnDevice) )                   // PTS 1115488 mb 2002-05-03
             {
                 // only commit,rollback and savepoint are allowed due to log full
                 SetLogFullQueueState(m_WriterTaskSync.GetTaskID(), true);
@@ -188,7 +189,7 @@ void Log_Writer::Run()
             SAPDB_UInt pagecount = 0;
             
             queue.GetOutput (m_WriterTaskSync.GetTaskID(), 
-                             pagesFreeOnDevice - m_RequestPages.GetPageCount(),
+                             m_PagesFreeOnDevice - m_RequestPages.GetPageCount(),
                              m_RequestPages,
                              pagecount,
                              firstflushedagain);
@@ -202,7 +203,7 @@ void Log_Writer::Run()
                 &&
                 (queueiter != fullloop)
                 &&
-                (pagesFreeOnDevice - m_RequestPages.GetPageCount() > 0) );
+                (m_PagesFreeOnDevice - m_RequestPages.GetPageCount() > 0) );
         
         // ----------------------------------------------------------
         // check whether there is output available:
@@ -234,7 +235,7 @@ void Log_Writer::Run()
 
                 ioseqnoBeforeFlush   += m_QueueRequests[i].pageCount;
                 firstWrittenPosition += m_QueueRequests[i].pageCount;
-                pagesFreeOnDevice    -= m_QueueRequests[i].pageCount;
+                m_PagesFreeOnDevice    -= m_QueueRequests[i].pageCount;
             }
 
             // ----------------------------------------------------------
@@ -244,13 +245,13 @@ void Log_Writer::Run()
             if ( EnableAddedLogDeviceIsPossible() )
             {
                 EnableAddedLogDevice();
-                pagesFreeOnDevice = FreePagesForLogWriter();
+                m_PagesFreeOnDevice = FreePagesForLogWriter();
             }
 
-            if ( ! EnoughPagesFreeForUserTaskWrite(pagesFreeOnDevice) )                   // PTS 1115488 mb 2002-05-03
+            if ( ! EnoughPagesFreeForUserTaskWrite(m_PagesFreeOnDevice) )                   // PTS 1115488 mb 2002-05-03
             {
-                pagesFreeOnDevice = FreePagesForLogWriter();
-                if ( ! EnoughPagesFreeForUserTaskWrite(pagesFreeOnDevice) )                   // PTS 1115488 mb 2002-05-03
+                m_PagesFreeOnDevice = FreePagesForLogWriter();
+                if ( ! EnoughPagesFreeForUserTaskWrite(m_PagesFreeOnDevice) )                   // PTS 1115488 mb 2002-05-03
                 {
                     // only commit,rollback and savepoint are allowed due to log full
                     SetLogFullQueueState(m_WriterTaskSync.GetTaskID(), true);
@@ -649,7 +650,10 @@ Log_DeviceBlockCount Log_Writer::UsedPages() const
 Log_DeviceBlockCount Log_Writer::UserLogSize() const
 {
     // PTS 1124876 UH 2003-10-23 new
-    return m_WriteIterator.GetSize() - m_PagesNotAvailableForUsers - LOG_RESTART_RESERVE_SPACE;
+    if ( m_WriteIterator.GetSize() < m_PagesNotAvailableForUsers + LOG_RESTART_RESERVE_SPACE )
+        return 0;
+	else
+        return m_WriteIterator.GetSize() - m_PagesNotAvailableForUsers - LOG_RESTART_RESERVE_SPACE;
 }
 
 // -----------------------------------------------------------------------------
