@@ -222,6 +222,7 @@ public:
         m_IOSequenceNo = newIOSequence.IsValid() ? newIOSequence
                                                  : LOG_INITIAL_IOSEQ;
         m_WriteIterator = newWritePosition;
+        m_PagesFreeOnDevice = FreePagesForLogWriter(); // PTS 1129443 mb 2004-05-03
     }
     
     /*!
@@ -270,11 +271,19 @@ public:
     Log_DeviceBlockCount UsedPages() const;
 
     /// returns true if no user can write more redo log
-	bool LogIsFull()  
+	bool LogIsFull() const
 	{
 		SAPDBTRACE_METHOD_DEBUG ("Log_Writer::LogIsFull", LogVolume_Trace, 5);
 		return UsedPages() >= UserLogSize();
 	}
+    
+    /// returns true, if commit/savepoints are possible. I.E. more than the restart-reserve-pages
+    /// are free on the LogVolume, so that the LogWirter would not be suspended
+    /// PTS 1130928 mb 2004-08-10
+    bool PagesAreFreeForWrite() const
+    {
+        return FreePagesForLogWriter() > 0;
+    }
 
     /*!
        @brief          returns the number of pages reserved for special purposes
@@ -475,5 +484,10 @@ private:
     SAPDB_Int                           m_LastSavepointTriggerTime;
     ///  event handler for log volume filling
     Kernel_EventSet                     m_LogUsageEvents; // PTS 1124876 UH 2003-10-23 added
+    
+    /// holds the number of currently free pages. If It's zero, then it's calculated
+    /// again, because it does not hold an exact value.
+    Log_DeviceBlockCount                m_PagesFreeOnDevice;  // PTS 1129443 mb 2004-05-03
+
 };
 #endif // LOG_WRITER_HPP
