@@ -60,13 +60,13 @@ tgg00_BasisError OMS_ContainerDictionary::DropContainer (IliveCacheSink* lcSink,
   cs.Enter();
   GUID_TEMP(lguid, rguid);
   OMS_HResult hr = lcSink->DropContainer (schema, lguid, cno, DBError);
-  if (e_ok == DBError) {
+  //if (e_ok == DBError) {
     OMS_ContainerInfo* p = this->FindGuid(guid, schema, cno);
     if (NULL != p) {
       p->MarkDropped();
       ++m_dropId;   // PTS 1120873
     }
-  }
+  //}
   return DBError;
 }
 
@@ -395,8 +395,7 @@ void OMS_ContainerDictionary::Insert (OMS_ContainerInfo* pContainerInfo)
 void OMS_ContainerDictionary::ReleaseContainerInfo(IliveCacheSink* lcSink, OMS_ContainerInfoPtr p) 
 {
   OMS_SinkCriticalSection cs(lcSink, RGN_CLASS_DIR);
-  if (!OMS_Globals::m_globalsInstance->InSimulator() && 
-    !OMS_Globals::m_globalsInstance->InProcServer()) {
+  if (!OMS_Globals::m_globalsInstance->InSimulator()) {
     cs.Enter();
   }
   OMS_TRACE(omsTrContainerDir, lcSink, "ReleaseContainerInfo, RefCnt : " << p->m_refCnt);
