@@ -35,6 +35,7 @@
 #include "Oms/OMS_ObjectContainer.hpp"
 #include "hsp77.h"
 
+#define REGISTER_ALLOCATOR true
 
 //externCpp
 static 
@@ -67,7 +68,7 @@ OMS_Context::OMS_Context
  , m_isVersion(false)
 #ifndef USE_SYSTEM_ALLOC_CO13
  , m_heap((const SAPDB_UTF8*) "", *OMS_Globals::GetKernelInterface()->GetOmsAllocator(), 32 * 1024, 32 * 1024, 
- SAPDBMem_RawAllocator::FREE_RAW_EXTENDS) 
+ SAPDBMem_RawAllocator::FREE_RAW_EXTENDS, SAPDBMEM_ALLOCATOR_UNLIMITED, !REGISTER_ALLOCATOR) 
  , m_newObjCache(m_heap)
 #endif
  , m_next(0)
@@ -91,7 +92,7 @@ OMS_Context::OMS_Context
   if (NULL != vid) {
     m_isVersion      = true;
     m_versionContext = *vctxt;
-    memcpy (m_version, vid, sizeof(*vid));
+    SAPDB_MemCopyNoCheck (m_version, vid, sizeof(*vid));
     m_isOpen         = true;
     sp77sprintf (REINTERPRET_CAST(char*, &heapName[0]), sizeof(heapName), "OMS Version %s", &vid[0]);
   }
@@ -178,7 +179,7 @@ void OMS_Context::Dump(OMS_DumpInterface& dumpObj) const
   } contextDumpInfo;
 
   contextDumpInfo.m_this            = CONST_CAST(OMS_Context*, this);
-  memcpy(&contextDumpInfo.m_version[0], &m_version[0], sizeof(m_version));
+  SAPDB_MemCopyNoCheck(&contextDumpInfo.m_version[0], &m_version[0], sizeof(m_version));
   contextDumpInfo.m_filler1         = 0;
   contextDumpInfo.m_session         = m_session;
   contextDumpInfo.m_currLcSink      = m_currLcSink;
@@ -383,14 +384,14 @@ void OMS_Context::FlushVarObject (OmsObjectContainerPtr pObj) {
     }
     else {
       if (pInfoNode->m_vobjNext) {
-        memcpy (&varObjContainer.m_vobjNext, &pInfoNode->m_vobjNext->m_vobjOid, 
+        SAPDB_MemCopyNoCheck (&varObjContainer.m_vobjNext, &pInfoNode->m_vobjNext->m_vobjOid, 
           sizeof (pInfoNode->m_vobjNext->m_vobjOid));
       }
       else {
         if (IsVersion()) /* PTS 1110149 */
         {
           OmsObjectId newOid = VersionNewOid();
-          memcpy(&varObjContainer.m_vobjNext, &newOid, sizeof(varObjContainer.m_vobjNext));
+          SAPDB_MemCopyNoCheck(&varObjContainer.m_vobjNext, &newOid, sizeof(varObjContainer.m_vobjNext));
           newObjSeq.gg91SetNilRef();
         }
         else
@@ -411,7 +412,7 @@ void OMS_Context::FlushVarObject (OmsObjectContainerPtr pObj) {
         pInfoNode->m_vobjNext = new(*this) OMS_VarObjInfoNode (varObjContainer.m_vobjNext, newObjSeq);
       }
     }
-    memcpy (&varObjContainer.vobjInfo[0], p, moveLen);
+    SAPDB_MemCopyNoCheck (&varObjContainer.vobjInfo[0], p, moveLen);
     if ((!isLocked) && (!IsVersion())) {
       hr = m_currLcSink->LockObj (
         (unsigned char*) &m_consistentView,
@@ -553,7 +554,7 @@ void OMS_Context::FlushObjCache(bool emptyObjCache,
             appendToFreeList = false;
 
             // Collect all variable sized objects in a list for later processing
-            curr->m_hashnext = pDelVarHead;
+            curr->SetNext(pDelVarHead);
             pDelVarHead = curr;
             OMS_DETAIL_TRACE(omsTrFlushCache, m_session->m_lcSink, " Object " << curr->m_oid << " append to var delete list");
           }
@@ -582,14 +583,14 @@ void OMS_Context::FlushObjCache(bool emptyObjCache,
             if (curr->GetContainerInfoNoCheck(this) == pClsInfoVarObj){
               // Object belongs to the 'container' which is currently collected in the 
               // the list pFlushVarHead.
-              curr->m_hashnext = pFlushVarHead;
+              curr->SetNext(pFlushVarHead);
               pFlushVarHead = curr;
               OMS_DETAIL_TRACE(omsTrFlushCache, m_session->m_lcSink, " Object " << curr->m_oid << " append to var update list1");
             }
             else {
               // Object belongs to another container and therefore this object is inserted
               // into the 'mixed' list 
-              curr->m_hashnext = pFlushVarHeadOtherClasses;
+              curr->SetNext(pFlushVarHeadOtherClasses);
               pFlushVarHeadOtherClasses = curr;
               OMS_DETAIL_TRACE(omsTrFlushCache, m_session->m_lcSink, " Object " << curr->m_oid << " append to var update list2");
             }
@@ -625,7 +626,7 @@ void OMS_Context::FlushObjCache(bool emptyObjCache,
         // DeleteVarObjMass. Those objects will be added in the corresponding 
         // method to the list.
         if ((emptyObjCache || curr->VarObjFlag()) && appendToFreeList) {
-          curr->m_hashnext = pFreeHead;
+          curr->SetNext(pFreeHead);
           pFreeHead = curr;
           OMS_DETAIL_TRACE(omsTrFlushCache, m_session->m_lcSink, " Object " << curr->m_oid << " appended to free list");
         }
@@ -701,7 +702,7 @@ void OMS_Context::FlushObjCache(bool emptyObjCache,
       while (iter){
         OmsObjectContainer *curr = iter();
         ++iter;
-        curr->m_hashnext = pFreeHead;
+        curr->SetNext(pFreeHead);
         pFreeHead = curr;
         OMS_DETAIL_TRACE(omsTrFlushCache, m_session->m_lcSink, " Object " << curr->m_oid << " appended to free list");
       }
@@ -715,9 +716,9 @@ void OMS_Context::FlushObjCache(bool emptyObjCache,
     
     // Release all memory of objects  
     while (pFreeHead){
-      OmsObjectContainer *pNext = pFreeHead->m_hashnext;
+      OmsObjectContainer *pNext = pFreeHead->GetNext();
       OMS_ClassIdEntry *pClsInfo = pFreeHead->GetContainerInfoNoCheck(this);
-      pClsInfo->chainFree(*this, pFreeHead);
+      pClsInfo->chainFree(*this, pFreeHead, 4);
       pFreeHead = pNext;
     }
   }
@@ -729,7 +730,7 @@ void OMS_Context::FlushObjCache(bool emptyObjCache,
     // Free only the in-memory representation of the var-objects.
     // All other memory is freed when the corresponding context is dropped.
     while (pFreeHead){
-      OmsObjectContainer *pNext = pFreeHead->m_hashnext;
+      OmsObjectContainer *pNext = pFreeHead->GetNext();
       reinterpret_cast<OMS_VarObjInfo*>(&pFreeHead->m_pobj)->freeVarObj(this);
       pFreeHead = pNext;
     }
@@ -827,11 +828,11 @@ void OMS_Context::DeleteVarObjMass(OmsObjectContainer  *pHead,
 
       if (pInfoNode == NULL){
         // Previous object is completely flushed, therefore consider next object
-        OmsObjectContainer *pNext = pHead->m_hashnext;
+        OmsObjectContainer *pNext = pHead->GetNext();
         
         // Append object to list for later freeing of memory
-        pHead->m_hashnext     = *ppFreeList;
-        *ppFreeList           = pHead;
+        pHead->SetNext(*ppFreeList);
+        *ppFreeList = pHead;
 
         pHead = pNext;
       }
@@ -965,7 +966,7 @@ void OMS_Context::FlushVarObjMass (OmsObjectContainer  *pHeadOneClass,
 
     pObjContainer = pHeadOneClass;
     while (pObjContainer){
-      pNext = pObjContainer->m_hashnext;
+      pNext = pObjContainer->GetNext();
 
       pObjInfo       = REINTERPRET_CAST (OMS_VarObjInfo*, &pObjContainer->m_pobj);
       pFirstInfoNode = &pObjInfo->m_vobjInfo;
@@ -1029,8 +1030,8 @@ void OMS_Context::FlushVarObjMass (OmsObjectContainer  *pHeadOneClass,
           if (pPrev == NULL)
             pHeadOneClass             = pNext;
           else
-            pPrev->m_hashnext         = pNext;
-          pObjContainer->m_hashnext   = pUnfinishedObjs;
+            pPrev->SetNext(pNext);
+          pObjContainer->SetNext(pUnfinishedObjs);
           pUnfinishedObjs             = pObjContainer;
           neededInfoNodeCnt          += (pagesNeeded-pagesCurr);
 
@@ -1129,12 +1130,12 @@ void OMS_Context::FlushVarObjMass (OmsObjectContainer  *pHeadOneClass,
         // with these objects. So these objects are considered twice.
         if (pObjContainer == pUnfinishedObjs){
           if (pPrev != NULL)
-            pPrev->m_hashnext = pUnfinishedObjs;
+            pPrev->SetNext(pUnfinishedObjs);
           else
             pHeadOneClass     = pUnfinishedObjs;
         }
         else {
-          pObjContainer->m_hashnext = pUnfinishedObjs;
+          pObjContainer->SetNext(pUnfinishedObjs);
           pPrev                     = pObjContainer;
         }
         pObjContainer   = pUnfinishedObjs;
@@ -1253,10 +1254,10 @@ void OMS_Context::FlushVarObjMass (OmsObjectContainer  *pHeadOneClass,
 
         if (pInfoNode == NULL){
           // Previous object is completely flushed, therefore consider next object
-          pNext = pObjContainer->m_hashnext;
+          pNext = pObjContainer->GetNext();
 
           // Append object to list for later freeing of memory
-          pObjContainer->m_hashnext = *ppFreeList;
+          pObjContainer->SetNext(*ppFreeList);
           *ppFreeList               = pObjContainer;
 
           pObjContainer = pNext;  
@@ -1305,13 +1306,13 @@ void OMS_Context::FlushVarObjMass (OmsObjectContainer  *pHeadOneClass,
       OmsObjectContainer *prev = NULL;
       OmsObjectContainer *curr = pHeadOtherClasses;
       while (curr){
-        next = curr->m_hashnext;
+        next = curr->GetNext();
         if (curr->GetContainerInfoNoCheck(this) == pClsInfo){
           if (prev)
-            prev->m_hashnext = next;
+            prev->SetNext(next);
           else 
-            pHeadOtherClasses = curr->m_hashnext;
-          curr->m_hashnext = pHeadOneClass; 
+            pHeadOtherClasses = curr->GetNext();
+          curr->SetNext(pHeadOneClass); 
           pHeadOneClass = curr;
         }
         else {
@@ -1383,7 +1384,7 @@ void OMS_Context::FlushVersionObjCache()
         }
 
         // Append object frame to free-list
-        pInfo->chainFree(*this, curr);
+        pInfo->chainFree(*this, curr, 5);
       }
     }
   }
@@ -1450,20 +1451,30 @@ void  OMS_Context::ResetVersion(OMS_Session& session)
 
 /*----------------------------------------------------------------------*/
 
-void OMS_Context::UnLoad() {
+void OMS_Context::UnLoad() 
+{
   const bool emptyObjCache = true;
   pasbool*     pToCancel;
   tsp00_TaskId taskId;
-  // reassign pointer to session, may not be valid, if called from
-  // bad alloc handling
+  OMS_Session *oldSession = m_session;
+
+  // reassign pointer to session, may not be valid, if called from bad alloc handling
   OMS_Globals::GetCurrentLcSink()->GetDefaultContext(REINTERPRET_CAST(void**, &this->m_session), &pToCancel, taskId);
+  AssignLcSink(m_session->m_lcSink);  // Reassign sink pointer, too  PTS 1133345
+  
   m_pVersionContext = &m_versionContext;
   FlushObjCache(emptyObjCache, false);  // No hash-adaptation PTS 1118855 
-  m_containerDir.ClearFreeLists();
+  m_containerDir.ClearFreeLists(1);
   for (OMS_ClassIdHash::Iter iter = m_containerDir.First(); iter; ++iter) {
     iter()->VersionDelIndex(false, this);
   }
   OMS_Globals::GetKernelInterface()->IncOmsVersionUnloadCounter();
+
+  // Reset local pointers if they have been set by this routine  PTS 1133345
+  if (oldSession != m_session){
+    m_session = NULL;
+    AssignLcSink(NULL);
+  }
 }
 
 /*----------------------------------------------------------------------*/
@@ -1590,8 +1601,8 @@ void OMS_Context::ReleaseAllUnchanged()   // PTS 1128262
   while (iter) {
     OmsObjectContainer *pObj = iter();
 
-    m_session->ReleaseObj(pObj);
     ++iter;
+    m_session->ReleaseObj(pObj);
   }
 }
 
