@@ -40,7 +40,7 @@
 #include "Oms/OMS_ClassIdEntry.hpp"
 #include "Oms/OMS_CacheMissEntry.hpp"
 #include "Oms/OMS_VarOid.hpp"
-
+#include "SAPDBCommon/SAPDB_MemCopyMove.hpp"
 
 
 inline void OMS_Context::AssignLcSink(IliveCacheSink* pSink) 
@@ -186,7 +186,7 @@ inline void  OMS_Context::ClearObjCache()
 
 inline void OMS_Context::CloseVersion()
 {
-  m_containerDir.ClearFreeLists();
+  m_containerDir.ClearFreeLists(2);
   m_isOpen = false;
 }
 
@@ -211,12 +211,15 @@ inline void OMS_Context::DeleteObjInContext (OmsObjectContainerPtr p, OMS_ClassI
     OMS_DETAIL_TRACE(p->IsNewObject() ? (omsTrContext | omsTrNewObj) : omsTrContext, 
         m_session->m_lcSink, "OMS_Context::DeleteObjInContext : " << p->m_oid
         << ", class: " << clsinfo->GetClassInfoPtr()->GetClassName());
+    
     if (p->IsVerNewRegistered())
         m_newObjCache.removeObject(p, this);
+
     const bool c_updateKeyCache = true;
-    m_oidDir.HashDelete(p->m_oid, c_updateKeyCache);
-    if (NULL != clsinfo) {
-        clsinfo->chainFree(*this, p);
+    bool found = m_oidDir.HashDelete(p->m_oid, c_updateKeyCache);
+    
+    if (NULL != clsinfo && found) {
+        clsinfo->chainFree(*this, p, 6);
     }
 }
 
@@ -251,7 +254,7 @@ inline OmsObjectContainerPtr OMS_Context::FindVarObjInContext (const OmsObjectId
       m_session->ThrowDBError (e_too_small_packet_size, msg, oid, __MY_FILE__, __LINE__);
     }
     if (NULL != buf) {
-      memcpy (buf, pObjInfo->m_pvobj, pObjInfo->m_vobjSize);
+      SAPDB_MemCopyNoCheck (buf, pObjInfo->m_pvobj, pObjInfo->m_vobjSize);
     }
     return po;
   }
@@ -423,7 +426,7 @@ inline OmsObjectContainerPtr OMS_Context::GetObjFromLiveCacheBase(const OmsObjec
     else {
       OMS_ClassIdEntry* containerInfo = this->GetContainerInfo(FileId.GetContainerHandle());
       pObj = m_session->GetMemory (*containerInfo);
-      memcpy (REINTERPRET_CAST(unsigned char*, &pObj->m_pobj) + sizeof(void*), &m_session->m_currVarObjChunk.m_container,
+      SAPDB_MemCopyNoCheck (REINTERPRET_CAST(unsigned char*, &pObj->m_pobj) + sizeof(void*), &m_session->m_currVarObjChunk.m_container,
         containerInfo->GetPersistentSize());
       pObj->m_oid    = oid;
       pObj->m_objseq = m_session->m_currVarObjChunk.m_seq;
@@ -495,7 +498,7 @@ inline OmsObjectContainerPtr OMS_Context::GetObjFromLiveCacheBase(const ClassIDR
       objHistReadCount,
       DBError);
     if (0 != DBError) {
-      pClassInfo->ChainFree(*this, pObj); 
+      pClassInfo->ChainFree(*this, pObj, 50); 
       if (e_object_not_found != DBError) {
         switch (doLockObj) {
         case OMS_Context::Lock:
@@ -631,7 +634,7 @@ inline int OMS_Context::LoadObjsFromLiveCacheBase(int                  noOfOid,
     for (i=0; i<noOfOid; ++i){
       if (pDBError[i] != 0){
         // Release allocated memory
-        ppClassInfo[i]->ChainFree(*this, ppObjContainer[i]); 
+        ppClassInfo[i]->ChainFree(*this, ppObjContainer[i], 51); 
         ppObjContainer[i] = NULL;
 
         if (pDBError[i] == e_object_not_found){
@@ -642,7 +645,7 @@ inline int OMS_Context::LoadObjsFromLiveCacheBase(int                  noOfOid,
         else {
           // Release allocated memory for all remaining objects
           for(int j=i ; j<noOfOid; ++j){
-            ppClassInfo[j]->ChainFree(*this, ppObjContainer[j]); 
+            ppClassInfo[j]->ChainFree(*this, ppObjContainer[j], 52); 
             ppObjContainer[j] = NULL;
             pDBError[j]       = -1;
           }
@@ -724,7 +727,7 @@ inline OmsObjectContainerPtr OMS_Context::GetObjViaKey(OMS_ClassIdEntry &Contain
     OmsObjectContainer* found = ContainerInfo.VersionFindKey(pBinaryKey);
     if (found) {
       // Release memory
-      ContainerInfo.chainFree(*this, obj);
+      ContainerInfo.chainFree(*this, obj, 7);
 
       m_session->IncCacheHit(found->m_oid);
 
@@ -754,7 +757,7 @@ inline OmsObjectContainerPtr OMS_Context::GetObjViaKey(OMS_ClassIdEntry &Contain
 
   if (DBError) {
     // Release memory
-    ContainerInfo.chainFree(*this, obj);
+    ContainerInfo.chainFree(*this, obj, 8);
 
     if (e_hash_key_not_found != DBError) {
       // Error handling
@@ -788,7 +791,7 @@ inline OmsObjectContainerPtr OMS_Context::GetObjViaKey(OMS_ClassIdEntry &Contain
       }
 
       // Release memory of object read from the kernel
-      ContainerInfo.chainFree(*this, obj);
+      ContainerInfo.chainFree(*this, obj, 9);
       
       return found->DeletedFlag()
              ? NULL
@@ -849,7 +852,7 @@ OMS_Context::GetObjViaCachedKey(OMS_ClassIdEntry& ContainerInfo,
         m_session->IncCacheHit(found->m_oid);
         
         // Release memory
-        ContainerInfo.chainFree(*this, obj);
+        ContainerInfo.chainFree(*this, obj, 10);
         
         if (doLock && !found->LockedFlag() && !IsVersionOid(found->m_oid)) {
             // Get lock if requested. As the object has been read from the kernel
@@ -867,7 +870,7 @@ OMS_Context::GetObjViaCachedKey(OMS_ClassIdEntry& ContainerInfo,
         m_session->IncCacheMiss();		
         
         // Release memory
-        ContainerInfo.chainFree(*this, obj);
+        ContainerInfo.chainFree(*this, obj, 11);
         
         return NULL;
     }
@@ -894,7 +897,7 @@ OMS_Context::GetObjViaCachedKey(OMS_ClassIdEntry& ContainerInfo,
             // Read from kernel was not successful 
             
             // Release Memory
-            ContainerInfo.chainFree(*this, obj);
+            ContainerInfo.chainFree(*this, obj, 12);
             
             
             if (DBError == e_hash_key_not_found) {
@@ -933,7 +936,7 @@ OMS_Context::GetObjViaCachedKey(OMS_ClassIdEntry& ContainerInfo,
                 // already been loaded into the context
                 
                 found->m_objseq = obj->m_objseq;
-                ContainerInfo.chainFree(*this, obj);
+                ContainerInfo.chainFree(*this, obj, 13);
                 
                 if (doLock && !found->LockedFlag()) {
                     found->MarkLocked();
@@ -991,7 +994,7 @@ inline int OMS_Context::LoadObjsViaKeyFromLCBase(int                   noOfObj,
     for (int i=0; i<noOfObj; ++i){
       if (pDBError[i] != 0){
         // Release allocated memory
-        ppClsInfo[i]->chainFree(*this, ppObjContainer[i]); 
+        ppClsInfo[i]->chainFree(*this, ppObjContainer[i], 14); 
         ppObjContainer[i] = NULL;
 
         if (pDBError[i] == e_hash_key_not_found){
@@ -1002,7 +1005,7 @@ inline int OMS_Context::LoadObjsViaKeyFromLCBase(int                   noOfObj,
         else {
           //// Release allocated memory for all remaining objects
           //for(int j=i ; j<noOfObj; ++j){
-          //  ppClsInfo[j]->chainFree(*this, ppObjContainer[j]); 
+          //  ppClsInfo[j]->chainFree(*this, ppObjContainer[j], 15); 
           //  ppObjContainer[j] = NULL;
           //  pDBError[j]       = -1;
           //}
@@ -1019,7 +1022,7 @@ inline int OMS_Context::LoadObjsViaKeyFromLCBase(int                   noOfObj,
       OmsObjectContainer* found = FindObjInContext(&pOid[i], true /*ignoreGeneration*/);
       if (found){
         // Object is already stored in the oms-cache
-        ppClsInfo[i]->chainFree(*this, ppObjContainer[i]);
+        ppClsInfo[i]->chainFree(*this, ppObjContainer[i], 16);
         ppObjContainer[i] = found;
 
         ppObjContainer[i]->m_objseq = pObjVers[i];
@@ -1203,7 +1206,7 @@ inline OmsObjectContainerPtr OMS_Context::LoadVarObject (const OmsVarOid& oid, O
       m_session->ThrowDBError(DBError, msg, __MY_FILE__, __LINE__);
     }
   }
-  memcpy (pObjInfo->m_pvobj, &chunk.m_container, moveLen);
+  SAPDB_MemCopyNoCheck (pObjInfo->m_pvobj, &chunk.m_container, moveLen);
   p  = pObjInfo->m_pvobj + moveLen;
   try {
     bool overflow = false;                            // PTS 1121449
@@ -1225,7 +1228,7 @@ inline OmsObjectContainerPtr OMS_Context::LoadVarObject (const OmsVarOid& oid, O
         ObjHistReadCount,
         DBError);
       this->CheckDBError(DBError, msg, oid);
-      memcpy (p, &chunk.m_container, moveLen);
+      SAPDB_MemCopyNoCheck (p, &chunk.m_container, moveLen);
       p       += moveLen;
       objSize -= moveLen;
       pInfoNode->m_vobjNext = new(*this) OMS_VarObjInfoNode (nextOid, chunk.m_seq);
@@ -1243,7 +1246,7 @@ inline OmsObjectContainerPtr OMS_Context::LoadVarObject (const OmsVarOid& oid, O
       pInfoNode = pInfoNode->m_vobjNext;
     }
     if (buf != NULL) {
-      memcpy (buf, pObjInfo->m_pvobj, pObjInfo->m_vobjSize);
+      SAPDB_MemCopyNoCheck (buf, pObjInfo->m_pvobj, pObjInfo->m_vobjSize);
     }
   } 
   catch (DbpError &e) {
@@ -1493,7 +1496,7 @@ inline bool OMS_Context::PutObjectIntoContextReg (OmsObjectContainerPtr p, tsp00
 
 inline void OMS_Context::RenameVersion(const OmsVersionId& OldId, const OmsVersionId& NewId) 
 {
-  memcpy (m_version, NewId, sizeof(m_version));
+  SAPDB_MemCopyNoCheck (m_version, NewId, sizeof(m_version));
 }
 
 /*----------------------------------------------------------------------*/
@@ -1553,7 +1556,7 @@ inline void OMS_Context::SetVersionDesc(const OmsTypeWyde* desc)
   // Every desciption longer than MAX_... will be truncated.
   // Memory will be freed in destructor of omsContext_co12.
   m_versionDesc = reinterpret_cast<OmsTypeWyde*>(allocate((len + 1) * sizeof(OmsTypeWyde)));
-  memcpy(m_versionDesc, desc, len * sizeof(OmsTypeWyde));
+  SAPDB_MemCopyNoCheck(m_versionDesc, desc, len * sizeof(OmsTypeWyde));
   m_versionDesc[len] = 0;  // ensure that string in null-terminated
 }
 
