@@ -35,6 +35,7 @@
 #include "Oms/OMS_DbpBase.hpp"
 #include "Oms/OMS_Globals.hpp"
 #include "liveCache/LVC_IliveCacheSink.hpp"
+#include "SAPDBCommon/SAPDB_MemCopyMove.hpp"
 #include "heo02x.h"
 #include "hsp77.h"
 
@@ -147,7 +148,7 @@ void DbpBase::dbpGetDbrootSapPath(char* path, int bufSize)
   }
   int len = (int)strlen(pathBuf);
   if (len < bufSize) {
-    memcpy (path, &pathBuf[0], len + 1);
+    SAPDB_MemCopyNoCheck (path, &pathBuf[0], len + 1);
   }
   else {
     throw DbpError(DbpError::DB_ERROR, e_directory_not_found, __MY_FILE__, __LINE__);
@@ -196,7 +197,7 @@ void DbpBase::dbpGetWorkingDirectory(char* path, int bufSize)
   }
   int len = (int)strlen(pathBuf);
   if (len < bufSize) {
-    memcpy (path, &pathBuf[0], len + 1);
+    SAPDB_MemCopyNoCheck (path, &pathBuf[0], len + 1);
   }
   else {
     throw DbpError(DbpError::DB_ERROR, e_directory_not_found, __MY_FILE__, __LINE__);
