@@ -85,6 +85,7 @@ class OMS_DummyKernelCallback : public IliveCacheSink
     tgg00_FileId             &objFile,
     tgg01_OmsVersionContext  *lpVersionContext,
     void                    **pKeyRangeIterator,
+    tsp00_Int4                bodySize,
     tsp00_Int4                keyLen,
     const void               *pStartKey,              
     const void               *pLowerKey,             
@@ -92,6 +93,7 @@ class OMS_DummyKernelCallback : public IliveCacheSink
     tsp00_Int4               &noOfOid,
     OmsTypeOid               *pResultOId,  
     tgg91_PageRef            *pObjSeq,
+    unsigned char            *pBody,
     tsp00_Int4               &LogReadAccesses,
     tgg00_BasisError         &e,
     OmsIterDirection          direction = OMS_ASCENDING){ return S_OK; }
@@ -289,11 +291,13 @@ class OMS_DummyKernelCallback : public IliveCacheSink
     tgg01_ContainerId       &ObjFile,
     tgg01_OmsVersionContext *VersionContext,
     void                    *pKeyIterator,
+    tsp00_Int4               bodySize,
     tsp00_Int4               KeyLen,
     void                    *pRestartKey, 
     tsp00_Int4              &NoOfOId,
     OmsTypeOid              *pResultOId, 
     tgg91_PageRef           *pObjVers,
+    unsigned char           *pBody,
     tsp00_Int4              &objHistReadCount,
     tgg00_BasisError        &e){ return S_OK; }
 
@@ -312,11 +316,13 @@ class OMS_DummyKernelCallback : public IliveCacheSink
     tgg01_ContainerId       &ObjFile,
     tgg01_OmsVersionContext *VersionContext,
     void                    *pKeyIterator,
+    tsp00_Int4               bodySize,
     tsp00_Int4               KeyLen,
     void                    *pRestartKey, 
     tsp00_Int4              &NoOfOId,
     OmsTypeOid              *pResultOId, 
     tgg91_PageRef           *pObjVers,
+    unsigned char           *pBody,
     tsp00_Int4              &objHistReadCount,
     tgg00_BasisError        &e){ return S_OK; }
 
@@ -685,54 +691,6 @@ class OMS_DummyKernelCallback : public IliveCacheSink
         return S_OK; 
     }
 
-    virtual HRESULT STDMETHODCALLTYPE VersionDictInsert(
-      const OmsVersionId  &versionId,
-      const OMS_Context   *pVersionContext,
-      tgg00_BasisError    &error)
-    { return S_OK; }
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictFind(
-      const OmsVersionId   &versionId,
-      OMS_Context         **ppVersionContext)
-    { return S_OK; }
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictDrop(
-      const OmsVersionId  &versionId,
-      tgg00_BasisError    &error)
-    { return S_OK; }
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictShutdown() 
-    { return S_OK; }
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictUnloadOldestVersion(
-      bool &unloaded) 
-    { return S_OK; }
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictMarkNotUnloadable(
-      OMS_Context *pVersionContext)
-    { return S_OK; }
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictMarkUnloadable(
-      OMS_Context *pVersionContext,
-      bool         callFromDestructor)
-    { return S_OK; }
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictCreateIter(
-      OMS_LockMode   lockMode, 
-      void         **ppIter, 
-      OMS_Context  **ppContext)
-    { return S_OK; }
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictGetNext(
-      void         *pIter, 
-      bool          dropCurr, 
-      OMS_Context **pContext)
-    { return S_OK; }
-
-    virtual HRESULT STDMETHODCALLTYPE VersionDictDestructIter(
-      void* pIter)
-    { return S_OK; }
-
     virtual int STDMETHODCALLTYPE GetSinkType()
     {
       return 0;
