@@ -943,21 +943,28 @@ public:
   **        buffer for the name must be provided by the invoker.
   ** \param classNameBufSize Size of the buffer for the class name in bytes.
   ** \param cno Class-specific number of the container in which the instance is stored.
+  ** \param excSafe if this parameter equals true, then no exception is thrown, if no
+  **        object is found for the given oid.
   ** 
+  ** \return 0 if the oid corresponds to an existing object,
+  **        -1 if no object for the given oid was found (only if excSafe==true).
+  **
   ** \attention The determination of the schemaName is quite expensive. Therefore if
   ** this information is not needed, the parameter schemaNameBufSize should be set equal
   ** zero. In this case the schema name is not returned; therefore the pointer to the buffer
   ** schemaName can be equal NULL.
   */
 #if defined(OMS_IMPLEMENTATION) || !defined(SAPDB_UNICODE)
-  void        omsOidInfo (const OmsObjectId& oid, ClassID& guid, 
+  int         omsOidInfo (const OmsObjectId& oid, ClassID& guid, 
                           OmsSchemaHandle& sh, char* schemaName, int schemaNameBufsize, 
-                          char* className, int classNameBufSize, OmsContainerNo& cno); 
+                          char* className, int classNameBufSize, OmsContainerNo& cno,
+                          bool excSafe=false); 
 #endif
 #if defined(OMS_IMPLEMENTATION) || defined(SAPDB_UNICODE)
-  void        omsOidInfo (const OmsObjectId& oid, ClassID& guid, 
+  int         omsOidInfo (const OmsObjectId& oid, ClassID& guid, 
                           OmsSchemaHandle& sh, OmsTypeWyde* schemaName, int schemaNameBufSize, 
-                          char* className, int classNameBufSize, OmsContainerNo& cno); 
+                          char* className, int classNameBufSize, OmsContainerNo& cno,
+                          bool excSafe=false); 
 #endif
 
   /*--------------------------------------------------------------------------*/
