@@ -1,6 +1,6 @@
 /*!
  * \file    OMS_KernelKeyIter.hpp
- * \author  MarkusS
+ * \author  MarkusSi
  * \brief   iterator which reads objects from the kernel
  */
 
@@ -43,6 +43,11 @@
 class OMS_KernelKeyIter : public OMS_BasisKeyIterBase 
 {
 private :
+  /*! As UDE-Server is not active in this release, set buffer size as small as
+  **  possible 
+  */
+  enum {MAX_BUFFER_SIZE=1};
+
   /*! When loading object oid's from the kernel this member is set to false. 
   **  Then it is checked whether the objects, corresponding to these oid's   
   **  are already stored in the local oms cache and all objects which are    
@@ -56,6 +61,15 @@ private :
   **  might some oids left in the local buffer.                              
   */
   bool                 m_allObjReadFromKernel;
+
+  /*! If this variable is set to true, then in every step of the iteration only
+  **  a single object is requested from the kernel, and not a whole array. But
+  **  in contrast to the mass-operation not only the oid is returned but the 
+  **  object itself is directly delivered. This is advantagous if the communication
+  **  cost between the liboms and the kernel is low.
+  */
+  bool                 m_noMassOperation;
+
   /*! This member contains the index of the current object in the array      
   **  OMS_KernelKeyIter::m_pOid.                                             
   */ 
@@ -95,7 +109,7 @@ private :
   **  the memory is allocated using a character type and later on the correctly
   **  typed pointer is assigned to this array.
   */
-  char m_pOid_u[OMS_MASS_OPERATION_CNT * sizeof(OmsObjectId)];
+  char m_pOid_u[MAX_BUFFER_SIZE * sizeof(OmsObjectId)];
   OmsObjectId        *m_pOid;
 
   /*! In this array the Log-Oid of the oids are stored. These oids can be used
@@ -104,22 +118,22 @@ private :
   **  and then searching the history chain for the version corresponding to the
   **  current consistent view.
   */
-  tgg91_PageRef        m_pObjVers[OMS_MASS_OPERATION_CNT];
+  tgg91_PageRef        m_pObjVers[MAX_BUFFER_SIZE];
 
   /*! 
   ** \name Arrays which are needed for the mass deref 
   */
   //@{
   // Prevent running the constructor for every entry of the array
-  //OmsObjectId          m_pLoadOid      [OMS_MASS_OPERATION_CNT];
-  char                   m_pLoadOid_u    [OMS_MASS_OPERATION_CNT * sizeof(OmsObjectId)];
+  //OmsObjectId          m_pLoadOid      [MAX_BUFFER_SIZE];
+  char                   m_pLoadOid_u    [MAX_BUFFER_SIZE * sizeof(OmsObjectId)];
   OmsObjectId           *m_pLoadOid;
-  tgg00_BasisError       m_pDBError      [OMS_MASS_OPERATION_CNT];
-  OmsObjectContainer    *m_ppObjContainer[OMS_MASS_OPERATION_CNT];
-  OmsAbstractObject     *m_ppObj         [OMS_MASS_OPERATION_CNT];
-  OMS_GuidEntry         *m_ppClassInfo   [OMS_MASS_OPERATION_CNT];
-  tgg01_ContainerId      m_pContainerId  [OMS_MASS_OPERATION_CNT];
-  int                    m_pObjSize      [OMS_MASS_OPERATION_CNT];
+  tgg00_BasisError       m_pDBError      [MAX_BUFFER_SIZE];
+  OmsObjectContainer    *m_ppObjContainer[MAX_BUFFER_SIZE];
+  OmsAbstractObject     *m_ppObj         [MAX_BUFFER_SIZE];
+  OMS_GuidEntry         *m_ppClassInfo   [MAX_BUFFER_SIZE];
+  tgg01_ContainerId      m_pContainerId  [MAX_BUFFER_SIZE];
+  int                    m_pObjSize      [MAX_BUFFER_SIZE];
   //@}
 
 public :
@@ -148,7 +162,7 @@ private:
   /// delivers the OID of the object the iterator is currently pointing to
   OmsObjectId&        GetCurrOid() { return m_pOid[m_currIdx]; }
   /// loads the object-container from the kernel into the oms-cache
-  inline OmsObjectContainer* LoadObj();
+  inline OmsObjectContainer* LoadObj(OmsObjectContainer* pObj);
 };
 
 
@@ -203,6 +217,8 @@ inline void OMS_KernelKeyIter::operator++()
   const char* msg = "OMS_KernelKeyIter::++ ";
 
   do {
+    OmsObjectContainer* pObj=NULL;
+
     if (m_currIdx < m_maxIdx && m_maxIdx != -1){
       // next object can be read from local buffer
       ++m_currIdx;
@@ -212,6 +228,16 @@ inline void OMS_KernelKeyIter::operator++()
       int objHistReadCount; 
 
       if (!m_allObjReadFromKernel){
+        if (m_noMassOperation){
+          // Get memory to store the object. 
+          if (m_pSession->InVersion()) {
+            pObj = m_pSession->GetMemoryInVersion(*m_pContainerInfo);
+          } 
+          else {
+            pObj = m_pSession->GetMemory(*m_pContainerInfo);
+          }
+        }
+
         // Get next objects from livecache 
         tgg00_BasisError DBError;
         m_pSession->m_lcSink->NextObjFromKeyIterator (
@@ -219,11 +245,13 @@ inline void OMS_KernelKeyIter::operator++()
             m_pContainerInfo->GetFileId(),
             REINTERPRET_CAST(tgg01_OmsVersionContext*, m_pSession->m_context->VersionContext()),
             m_pKBIterator,
+            m_pContainerInfo->GetPersistentSize(),
             m_keyLen,
             m_pRestartKey != NULL ? m_pRestartKey : GetCurrKey(),       
             noOfOid,            // changed after call
             REINTERPRET_CAST(OmsTypeOid*, m_pOid),
             m_pObjVers,
+            m_noMassOperation ? ((unsigned char*) &pObj->m_pobj) + sizeof(void*) : NULL,
             objHistReadCount,
             DBError); 
         m_pRestartKey = NULL;
@@ -259,7 +287,7 @@ inline void OMS_KernelKeyIter::operator++()
     }
 
     if (!m_end) { 
-      m_pCurrObj = LoadObj();
+      m_pCurrObj = LoadObj(pObj);
       if (m_pCurrObj != NULL){
         // Object is not marked as deleted
         OMS_TRACE(omsTrKeyRange, m_pSession->m_lcSink, msg 
@@ -283,6 +311,8 @@ inline void OMS_KernelKeyIter::operator--()
   const char* msg = "OMS_KernelKeyIter::-- ";
  
   do {
+    OmsObjectContainer* pObj=NULL;
+
     if (m_currIdx > 0 && m_maxIdx != -1){
       // Next bject can be read from local buffer
       --m_currIdx;
@@ -292,6 +322,16 @@ inline void OMS_KernelKeyIter::operator--()
       int objHistReadCount;
 
       if (!m_allObjReadFromKernel){
+        if (m_noMassOperation){
+          // Get memory to store the object. 
+          if (m_pSession->InVersion()) {
+            pObj = m_pSession->GetMemoryInVersion(*m_pContainerInfo);
+          } 
+          else {
+            pObj = m_pSession->GetMemory(*m_pContainerInfo);
+          }
+        }
+
         // Get next objects from livecache 
         tgg00_BasisError DBError;
         m_pSession->m_lcSink->PrevObjFromKeyIterator (
@@ -299,11 +339,13 @@ inline void OMS_KernelKeyIter::operator--()
             m_pContainerInfo->GetFileId(),
             REINTERPRET_CAST(tgg01_OmsVersionContext*, m_pSession->m_context->VersionContext()),
             m_pKBIterator,
+            m_pContainerInfo->GetPersistentSize(),
             m_keyLen,
             m_pRestartKey != NULL ? m_pRestartKey : GetCurrKey(),       
             noOfOid,    // changed after call
             REINTERPRET_CAST(OmsTypeOid*, m_pOid),
             m_pObjVers,
+            m_noMassOperation ? ((unsigned char*) &pObj->m_pobj) + sizeof(void*) : NULL,
             objHistReadCount,
             DBError); 
         m_pRestartKey = NULL;
@@ -339,7 +381,7 @@ inline void OMS_KernelKeyIter::operator--()
     }
 
     if (!m_end) {
-      m_pCurrObj = LoadObj();
+      m_pCurrObj = LoadObj(pObj);
       if (m_pCurrObj != NULL){
         // Object is not marked as deleted
         OMS_TRACE(omsTrKeyRange, m_pSession->m_lcSink, msg 
@@ -359,32 +401,55 @@ inline void OMS_KernelKeyIter::operator--()
 **  \return pointer to the object frame in the oms cache of the current object.      
 */
 /*===================================================================================*/
-inline OmsObjectContainer* OMS_KernelKeyIter::LoadObj() 
+inline OmsObjectContainer* OMS_KernelKeyIter::LoadObj(OmsObjectContainer* pObj) 
 { 
   const char* msg = "OMS_KernelKeyIter::LoadObj ";
   OMS_TRACE(omsTrKeyRange, m_pSession->m_lcSink, msg);
 
   OmsObjectContainerPtr found = NULL;
 
-  if (!OMS_Globals::m_globalsInstance->InProcServer()){
-    // InProc case: Single derefs from the kernel
-    m_pSession->IncDeref();
-    found = m_pSession->m_context->FindObjInContext(&(GetCurrOid()), true /*ignoreGeneration*/);
-    if (found == NULL) {
-      found = m_pSession->m_context->GetObjFromLiveCacheBase(m_pContainerInfo->GetGuid(), GetCurrOid(), OMS_Context::NoLock, &m_pObjVers[m_currIdx]);
+  if (m_noMassOperation){
+    // Object body was already delivered by the create or the ++,-- operator.
+    // Check if object does already exist in local cache.
+    OmsObjectContainerPtr found = m_pSession->m_context->FindObjInContext(&m_pOid[m_currIdx], true /*ignoreGeneration*/); 
+    if (found != NULL){
+      // Object does already exist, therefore free memory
+      m_pContainerInfo->chainFree(*m_pSession->m_context, pObj, 17);
+      m_pCurrObj = NULL;
+
+      // If object is not marked as deleted, then let m_pCurrObj point to this object
+      // in the oms-cache
+      if (found->DeletedFlag()){
+        m_pCurrObjDel = found;
+        return NULL;
+      }
+      else {
+        m_pCurrObjDel = NULL;
+        return found;
+      }
+    }
+    else {
+      // Object does not exist locally. 
+      // Fill header of object container
+      pObj->m_oid    = m_pOid[0];
+      pObj->m_objseq = m_pObjVers[0];
+
+      // Insert object into oms-cache
+      m_pSession->m_context->PutObjectIntoContext (pObj, m_pContainerInfo->GetContainerHandle());
+
+      return pObj;
     }
-    m_objectsAreLoaded = false;
   }
   else {
-    // ProcServer case: Mass derefs from the kernel
+    // Mass derefs from the kernel
     if (!m_objectsAreLoaded){
       OMS_GuidEntry *pClassInfo = m_pSession->GetClassInfo(m_pContainerInfo->GetGuid());
 
       int i = 0;
       while (i<=m_maxIdx){
         int noOfLoadObj = 0;
-        // Deref oids in portion of size OMS_MASS_OPERATION_CNT from the kernel
-        while (noOfLoadObj < OMS_MASS_OPERATION_CNT && i<=m_maxIdx){
+        // Deref oids in portion of size MAX_BUFFER_SIZE from the kernel
+        while (noOfLoadObj < MAX_BUFFER_SIZE && i<=m_maxIdx){
           // Check whether object are already stored in local oms-cache; 
           // if not, remember them in an array
           m_pSession->IncDeref();
