@@ -74,6 +74,9 @@ void OMS_LibOmsInterfaceInstance::AdviseKernelInterface(LVC_KernelInterface& knl
 
     // PTS 1124533
     OMS_Globals::m_globalsInstance->m_versionDictionary.m_useRWLocks = (knlInterface.UseReaderWriterLocks() ? true : false);
+
+    // PTS 1129082
+    OMS_Globals::m_globalsInstance->m_versionBuffer.Advise(&OMS_Globals::m_sharedMemAllocatorInstance, 10);   // PTS 1129082
 }
 
 /*----------------------------------------------------------------------*/
@@ -133,15 +136,17 @@ tsp00_Bool OMS_LibOmsInterfaceInstance::GetFirstVersion(IliveCacheSink*     lcSi
     // structure. It is NOT used to synchronize access to the version dictionary.
     lcSink->EnterCriticalSection(RGN_VERSION_DIR);
 
-    // Reset iterator to the first element
-    OMS_Globals::m_globalsInstance->m_versionIter.first(OMS_LOCK_EXCLUSIVE);
-  } catch (DbpError &e){  // PTS 1126697
+    // Reset the global iterator
+    OMS_Globals::m_globalsInstance->m_versionIter.Reset(OMS_LOCK_EXCLUSIVE);
+  } 
+  catch (DbpError &e){  // PTS 1126700
+
     DbpErrorCallbackInterface *pCBInterface = DbpError::dbpGetCallback();  // PTS 1127871
     if (pCBInterface){
       pCBInterface->dbpCaughtError(e);
     }
 
-    OMS_Globals::m_globalsInstance->m_versionIter.stop();
+    OMS_Globals::m_globalsInstance->m_versionIter.Stop();
     lcSink->LeaveCriticalSection(RGN_VERSION_DIR);
     error = e.m_errorNo;
     return false;
@@ -198,40 +203,115 @@ tsp00_Bool OMS_LibOmsInterfaceInstance::GetNextVersion(IliveCacheSink*     lcSin
                                                  short&              error)  // PTS 1126697 
 {
   try {
-  error = e_ok;
-  while (true) {
-    if (OMS_Globals::m_globalsInstance->m_versionIter) {
-      OMS_Context* pContext = OMS_Globals::m_globalsInstance->m_versionIter();
-      if (pContext->IsDropped()) {
-        // version is marked as dropped, try to drop it now
-        //++OMS_Globals::m_globalsInstance->m_versionIter;   // PTS 1126697
-        if (!pContext->IsBoundToTrans()) {
-          // version no longer bound to transaction, i.e. can be dropped now
-          pContext->AssignLcSink(lcSink);  /* PTS 1107209 */
-          OMS_Globals::m_globalsInstance->m_versionDictionary.MarkNotUnloadable(lcSink, pContext);
-          OMS_Globals::m_globalsInstance->m_versionIter.getNext(true /*dropCurr*/);
-          pContext->DeleteSelf();
-        }
-        else {                                                // PTS 1126697
-          ++OMS_Globals::m_globalsInstance->m_versionIter;
+    const unsigned int MAX_BUFFER_SIZE = 100;
+    int                cnt = 0;
+    OMS_Context       *ppContext[MAX_BUFFER_SIZE];
+    OMS_Context       *pContext = NULL;
+    OMS_Session       *pSession = NULL;
+    bool               checkForSession = true;
+
+    error = e_ok;
+  
+    while (true){
+      if (OMS_Globals::m_globalsInstance->m_versionBuffer.IsEmpty()){
+        // Position the iterator on the next slot in the hash of the version directory.
+        pContext = OMS_Globals::m_globalsInstance->m_versionIter.GetFirstInSlot();
+
+        // Read all versions which belong to the next slot in the hash of the version 
+        // directory and insert them into the buffer. After all versions have been read 
+        // the corresponding lock is freed again.
+        while (pContext){
+          OMS_Context *pNext = OMS_Globals::m_globalsInstance->m_versionIter.GetNextInSlot();
+          if (!pContext->IsDropped()){
+            VersionEntry   versionEntry;
+            memcpy(&versionEntry.versionId[0], &pContext->GetVersionId()[0], sizeof(OmsVersionId)); 
+            versionEntry.createDate     = pContext->GetCreateDate();
+            versionEntry.createTime     = pContext->GetCreateTime();
+            versionEntry.openDate       = pContext->GetOpenDate();
+            versionEntry.openTime       = pContext->GetOpenTime();
+            versionEntry.consistentView = pContext->m_consistentView;
+            versionEntry.isMarked       = pContext->IsMarked();
+            versionEntry.isOpen         = pContext->IsVersionOpen();
+            versionEntry.isUnloaded     = pContext->IsUnloaded();
+            versionEntry.heapUsage      = pContext->HeapUsed();
+            versionEntry.hashSize       = pContext->GetHashSize();
+            versionEntry.versionTree    = pContext->VersionFile();
+            versionEntry.versionInvTree = pContext->VersionInvFile();
+            if (pContext->GetVersionDesc() != NULL){
+              versionEntry.hasVersionDesc = true;
+              memcpy(&versionEntry.versionDesc[0], pContext->GetVersionDesc(), sizeof(versionEntry.versionDesc));
+            }
+            else {
+              versionEntry.hasVersionDesc = false;
+            }
+
+            OMS_Globals::m_globalsInstance->m_versionBuffer.Push(versionEntry);
+          }
+          else {
+            if (checkForSession && pSession == NULL){
+              // Get session if not already determined If no com-routine has been executed
+              // before in this connection, then no session exists. In this case, the versions,
+              // which are marked as deleted, will not be removed. 
+              pasbool         *pToCancel;
+              tsp00_TaskId     taskId;
+              lcSink->GetDefaultContext(reinterpret_cast<void**>(&pSession), &pToCancel, taskId);
+              checkForSession = false;
+            }
+ 
+            if (pSession != NULL){
+              if (!pContext->IsBoundToTrans()) {
+                pContext->AssignLcSink(lcSink);  
+                OMS_Globals::m_globalsInstance->m_versionDictionary.MarkNotUnloadable(lcSink, pContext);
+                OMS_Globals::m_globalsInstance->m_versionDictionary.DropVersion(pContext->GetVersionId()); 
+                if (cnt < MAX_BUFFER_SIZE){
+                  // Remember a pointer to the context for later destruction. As the version is
+                  // not registered any more in the version directory, nobody else can change
+                  // the pointer.
+                  ppContext[cnt++] = pContext;
+                }
+                else {
+                  // To prevent lock contention problems this should never occur. Normally the
+                  // destruction of the version should be done outside the exclusive region.
+                  // But if it occurs inside, then 'only' performance will degrade.
+                  pSession->DropVersionEpilog(pContext);
+                }
+              }
+            }
+          }
+          pContext = pNext;
         }
       }
+ 
+      if (pSession != NULL){
+        // Destruct the versions, which are marked as deleted, outside the exclusive region
+        for (int i=0; i<cnt; ++i){
+          pSession->DropVersionEpilog(ppContext[i]);
+        }
+        cnt = 0;
+      }
+    
+      if (OMS_Globals::m_globalsInstance->m_versionBuffer.IsEmpty()){
+        // All versions have been considered
+        lcSink->LeaveCriticalSection(RGN_VERSION_DIR);
+        return false;
+      }
       else {
-        memset (&versionId[0], ' ', sizeof(versionId));
-        memcpy(&versionId[0], &pContext->GetVersionId()[0], sizeof(OmsVersionId)); 
-        createDate     = pContext->GetCreateDate();
-        createTime     = pContext->GetCreateTime();
-        openDate       = pContext->GetOpenDate();
-        openTime       = pContext->GetOpenTime();
-        consistentView = pContext->m_consistentView;
-        isMarked       = pContext->IsMarked();
-        isOpen         = pContext->IsVersionOpen();
-        isUnloaded     = pContext->IsUnloaded();
-        heapUsage      = pContext->HeapUsed();
-        hashSize       = pContext->GetHashSize();
-        versionTree    = pContext->VersionFile();
-        versionInvTree = pContext->VersionInvFile();
-        
+        // Return last entry from the buffer
+        const VersionEntry *pVersionEntry = OMS_Globals::m_globalsInstance->m_versionBuffer.Pop();
+        memcpy(&versionId[0], &pVersionEntry->versionId[0], sizeof(OmsVersionId)); 
+        createDate     = pVersionEntry->createDate;
+        createTime     = pVersionEntry->createTime;
+        openDate       = pVersionEntry->openDate;
+        openTime       = pVersionEntry->openTime;
+        consistentView = pVersionEntry->consistentView;
+        isMarked       = pVersionEntry->isMarked;
+        isOpen         = pVersionEntry->isOpen;
+        isUnloaded     = pVersionEntry->isUnloaded;
+        heapUsage      = pVersionEntry->heapUsage;
+        hashSize       = pVersionEntry->hashSize;
+        versionTree    = pVersionEntry->versionTree;
+        versionInvTree = pVersionEntry->versionInvTree;
+
         // PTS 1117690
         // Initialize version-desciption
         if (omsInUnicodeMode){
@@ -243,11 +323,11 @@ tsp00_Bool OMS_LibOmsInterfaceInstance::GetNextVersion(IliveCacheSink*     lcSin
           }
         }
         else {
-          memset(versionDesc, ' ', sizeof(versionDesc));
+          memset(&versionDesc[0], ' ', sizeof(versionDesc));
         }
-        
-        if (pContext->GetVersionDesc() != NULL){
-          int len = OMS_Globals::WideStrlen(pContext->GetVersionDesc());
+
+        if (pVersionEntry->hasVersionDesc){
+          int len = OMS_Globals::WideStrlen(pVersionEntry->versionDesc);
           
           if (omsInUnicodeMode){
             // Source and Target in UCS2 => No Convertion needed but output string could be
@@ -256,7 +336,7 @@ tsp00_Bool OMS_LibOmsInterfaceInstance::GetNextVersion(IliveCacheSink*     lcSin
                          ? sizeof(versionDesc)
                          : len * sizeof(OmsTypeWyde));
 
-            memcpy(versionDesc, pContext->GetVersionDesc(), len_byte); 
+            memcpy(versionDesc, pVersionEntry->versionDesc, len_byte); 
             
             // check if endianness of maschine is the same as the endianness of the kernel
             if (sp77encodingUCS2Native != sp77encodingUCS2){
@@ -281,28 +361,18 @@ tsp00_Bool OMS_LibOmsInterfaceInstance::GetNextVersion(IliveCacheSink*     lcSin
                                       &destBytesWritten,              // destBytesWritten
                                       false,                          // addZeroTerminator
                                       sp77encodingUCS2Native,         // srcEncoding
-                                      pContext->GetVersionDesc(),     // srcBuffer
+                                      pVersionEntry->versionDesc,     // srcBuffer
                                       len * sizeof(OmsTypeWyde),      // srcBufferLengthInBytes
                                       &srcBytesParsed);               // srcBytesParsed
           }
         }
-        
-        ++OMS_Globals::m_globalsInstance->m_versionIter;
+
         return true;
       }
     }
-    else {
-      lcSink->LeaveCriticalSection(RGN_VERSION_DIR);
-      return false;
-    }
-  }
-  } catch (DbpError &e){  // PTS 1126697
-    DbpErrorCallbackInterface *pCBInterface = DbpError::dbpGetCallback();  // PTS 1127871
-    if (pCBInterface){
-      pCBInterface->dbpCaughtError(e);
-    }
-
-    OMS_Globals::m_globalsInstance->m_versionIter.stop();
+  } 
+  catch (DbpError &e){  // PTS 1126700
+    OMS_Globals::m_globalsInstance->m_versionIter.Stop();
     lcSink->LeaveCriticalSection(RGN_VERSION_DIR);
     error = e.m_errorNo;
     return false;
@@ -409,126 +479,59 @@ bool OMS_LibOmsInterfaceInstance::VersionUnloaded()
 
 /*-------------------------------------------------------------------------*/
 
-void OMS_LibOmsInterfaceInstance::VersionDictInsert(
-  const OmsVersionId &versionId, 
-  const OMS_Context  *pVersionContext,
-  tgg00_BasisError   &error)
+void OMS_LibOmsInterfaceInstance::CancelVersion(
+  tgg91_TransNo &ConsistViewNo) 
 {
-  error = OMS_Globals::m_globalsInstance->m_versionDictionary.InsertVersion(versionId, 
-    const_cast<OMS_Context*>(pVersionContext));
-}
+  OMS_VersionDictionary::Iter VersionIter(OMS_LOCK_EXCLUSIVE); 
+  while (true)
+  {
+    OMS_Context* pContext = VersionIter.GetFirstInSlot();
+    if (NULL == pContext)
+      break;
 
-/*-------------------------------------------------------------------------*/
+    while (pContext)
+    {
+      if (pContext->m_consistentView == ConsistViewNo)
+      {
+        IliveCacheSink* pSink = OMS_Globals::m_globalsInstance->GetCurrentLcSink();
+        pContext->AssignLcSink(pSink);
+        DbpBase b(pSink);
+        char versionId[sizeof(OmsVersionId)+1];
+        SAPDB_MemCopyNoCheck(&versionId[0], &pContext->GetVersionId()[0], sizeof(OmsVersionId));
+        versionId[sizeof(OmsVersionId)] = 0;
+        b.dbpOpMsg("Cancel OMS version: %s", &versionId[0]);
+        OMS_Globals::m_globalsInstance->m_versionDictionary.MarkNotUnloadable(pSink, pContext);
+        if (!pContext->IsBoundToTrans())
+        {
+          OMS_Globals::m_globalsInstance->m_versionDictionary.DropVersion(pContext->GetVersionId());
 
-void OMS_LibOmsInterfaceInstance::VersionDictFind(
-  const OmsVersionId  &versionId,
-  OMS_Context        **ppVersionContext)
-{
- *ppVersionContext = OMS_Globals::m_globalsInstance->m_versionDictionary.FindVersion(versionId);
-}
+          //pContext->DeleteSelf();
 
-/*-------------------------------------------------------------------------*/
+          tgg01_OmsVersionContext  m_versionContext;
+          m_versionContext.ovc_tree          = pContext->VersionFile();
+          m_versionContext.ovc_index_tree    = pContext->VersionInvFile();
+          m_versionContext.ovc_key           = pContext->CurrentVersionKey();
+          m_versionContext.ovc_trans_version = ConsistViewNo;
+          tgg00_BasisError DBError;
+          OMS_HResult hr = pSink->DropVersion((unsigned char*) &m_versionContext, &DBError);
+          if (0 != DBError)
+          {
+            // just write a message into knldiag and ignore error
+            DbpBase opMsg(pSink);
+            opMsg.dbpOpError("cancel version, error %d ignored", DBError);
+          }
+          pContext->OMS_Context::~OMS_Context();
+          OMS_Globals::m_sharedMemAllocatorInstance.deallocate(pContext);
+        }
+        else
+        {
+          pContext->MarkDropped();
+        }
+      }
 
-void OMS_LibOmsInterfaceInstance::VersionDictDrop(
-  const OmsVersionId &versionId, 
-  tgg00_BasisError   &error)
-{
-  error = OMS_Globals::m_globalsInstance->m_versionDictionary.DropVersion(versionId);
-}
-
-/*-------------------------------------------------------------------------*/
-
-void OMS_LibOmsInterfaceInstance::VersionDictShutdown()
-{
-  OMS_Globals::m_globalsInstance->m_versionDictionary.Shutdown();
-}
-
-/*-------------------------------------------------------------------------*/
-
-bool OMS_LibOmsInterfaceInstance::VersionDictUnloadOldestVersion()
-{
-  IliveCacheSink *pSink = OMS_Globals::m_globalsInstance->GetCurrentLcSink();
-  return OMS_Globals::m_globalsInstance->m_versionDictionary.UnloadOldestVersion(pSink);
-}
-
-
-/*-------------------------------------------------------------------------*/
-
-void OMS_LibOmsInterfaceInstance::VersionDictMarkNotUnloadable(
-  OMS_Context    *pVersionContext)
-{
-  IliveCacheSink *pSink = OMS_Globals::m_globalsInstance->GetCurrentLcSink();
-  OMS_Globals::m_globalsInstance->m_versionDictionary.MarkNotUnloadable(pSink, pVersionContext);
-}
-
-/*-------------------------------------------------------------------------*/
-
-void OMS_LibOmsInterfaceInstance::VersionDictMarkNotUnloadable(
-  OMS_Context  *pVersionContext,
-  bool          callFromDestructor)
-{
-  OMS_Globals::m_globalsInstance->m_versionDictionary.MarkUnloadable(pVersionContext, callFromDestructor);
-}
-
-/*-------------------------------------------------------------------------*/
-
-void OMS_LibOmsInterfaceInstance::VersionDictCreateIter(
-  OMS_LockMode   lockMode, 
-  void         **ppIter, 
-  OMS_Context  **ppContext)
-{
-  *ppIter = OMS_Globals::m_sharedMemAllocatorInstance.allocate(sizeof(OMS_VersionDictionary::Iter));
-  OMS_VersionDictionary::Iter* pIter = reinterpret_cast<OMS_VersionDictionary::Iter*>(*ppIter);
-  
-  pIter->first(lockMode);
-
-  if (*pIter){
-    *ppContext = (*pIter)();
+      pContext = VersionIter.GetNextInSlot();
+    }
   }
-  else {
-    // Iterator is empty
-    OMS_Globals::m_sharedMemAllocatorInstance.deallocate(*ppIter);
-    *ppIter    = NULL;
-    *ppContext = NULL;
-  }
-}
-
-/*-------------------------------------------------------------------------*/
-
-void OMS_LibOmsInterfaceInstance::VersionDictGetNext(
-  void         *p, 
-  bool          dropCurr,
-  OMS_Context **ppContext)
-{
-  OMS_VersionDictionary::Iter* pIter = reinterpret_cast<OMS_VersionDictionary::Iter*>(p);
-
-  pIter->getNext(dropCurr);
-  if (*pIter){
-    *ppContext = (*pIter)();
-  }
-  else {
-    // Iterator is empty
-    OMS_Globals::m_sharedMemAllocatorInstance.deallocate(p);
-    *ppContext = NULL;
-  }
-}
-
-/*-------------------------------------------------------------------------*/
-
-void OMS_LibOmsInterfaceInstance::VersionDictDestructIter(
-  void  *p) 
-{
-  if (p == NULL)
-    return;
-
-  OMS_VersionDictionary::Iter* pIter = reinterpret_cast<OMS_VersionDictionary::Iter*>(p);
-  if (*pIter){
-    // Ensure that all locks are freed
-    pIter->stop();
-  }
-    
-  // Iterator is empty
-  OMS_Globals::m_sharedMemAllocatorInstance.deallocate(pIter);
-}
+};
 
 /*-------------------------------------------------------------------------*/
