@@ -35,6 +35,7 @@
 #include "Oms/OMS_ISessionLockObjects.hpp"
 #include "Oms/OMS_Session.hpp"
 #include "Oms/OMS_Handle.hpp"
+#include "SAPDBCommon/SAPDB_MemCopyMove.hpp"
 #include "gsp00.h"
 #include "ggg251.h"
 
@@ -983,23 +984,23 @@ bool OMS_LockEntryHash::NextLockInfo(void** pVoidLockInfo, tgg01_OmsLockInfo& lo
   if (NULL != pLockInfo)
   {
     *pVoidLockInfo = pLockInfo->m_next;
-    memcpy(&lockInfo.oli_handle, &pLockInfo->m_lockHandle, sizeof(lockInfo.oli_handle));
+    SAPDB_MemCopyNoCheck(&lockInfo.oli_handle, &pLockInfo->m_lockHandle, sizeof(lockInfo.oli_handle));
     switch (pLockInfo->m_lockKind)
     {
     case LockInfoRow::LOCK_SHARE : 
-      memcpy(&lockInfo.oli_lockmode, "lock_share  ", sizeof(lockInfo.oli_lockmode));
+      SAPDB_MemCopyNoCheck(&lockInfo.oli_lockmode, "lock_share  ", sizeof(lockInfo.oli_lockmode));
       memset(&lockInfo.oli_requestmode, ' ', sizeof(lockInfo.oli_requestmode));
       break;
     case LockInfoRow::LOCK_EXCLUSIVE : 
-      memcpy(&lockInfo.oli_lockmode, "lock_excl   ", sizeof(lockInfo.oli_lockmode));
+      SAPDB_MemCopyNoCheck(&lockInfo.oli_lockmode, "lock_excl   ", sizeof(lockInfo.oli_lockmode));
       memset(&lockInfo.oli_requestmode, ' ', sizeof(lockInfo.oli_requestmode));
       break;
     case LockInfoRow::REQ_EXCLUSIVE :
-      memcpy(&lockInfo.oli_requestmode, "req_excl    ", sizeof(lockInfo.oli_requestmode));
+      SAPDB_MemCopyNoCheck(&lockInfo.oli_requestmode, "req_excl    ", sizeof(lockInfo.oli_requestmode));
       memset(&lockInfo.oli_lockmode, ' ', sizeof(lockInfo.oli_lockmode));
       break;
     case LockInfoRow::REQ_SHARE :
-      memcpy(&lockInfo.oli_requestmode, "req_share   ", sizeof(lockInfo.oli_requestmode));
+      SAPDB_MemCopyNoCheck(&lockInfo.oli_requestmode, "req_share   ", sizeof(lockInfo.oli_requestmode));
       memset(&lockInfo.oli_lockmode, ' ', sizeof(lockInfo.oli_lockmode));
       break;
     }
