@@ -151,6 +151,11 @@ const OmsObjectId& OMS_Session::CastOid(const ClassIDRef castToGuid, const OmsOb
     return oid;
   }
   OmsObjectContainerPtr p = m_context->GetObj(oid, false);
+  // PTS 1132363
+  if (p == NULL){
+    ThrowDBError (e_object_not_found, "OMS_Session::CastOid", oid, __MY_FILE__, __LINE__);
+  }
+
   OMS_ClassIdEntry*     pFrom = p->GetContainerInfo(m_context);
   if (pFrom->GetGuid() == castToGuid) {
     return oid;
@@ -197,7 +202,7 @@ void OMS_Session::CleanAfterDropSchema()
 void OMS_Session::ClearDefaultContext()
 {
   m_defaultContext->ClearObjCache();
-  this->ClearFreeLists();
+  this->ClearFreeLists(3);
   this->FreeStackHeap();
 }
 
@@ -253,7 +258,11 @@ void OMS_Session::CreateVersion(const OmsVersionId& versionId, const OmsTypeWyde
     context = new OMS_Context(this, &versionId, &versionContext);
     context->NewConsistentView();
     context->ResetLcSink();
+
+    OpenVersionProlog(versionId, true);  // PTS 1129082
     this->OpenVersion(context, true);
+    OpenVersionEpilog();  // PTS 1129082 
+
     context->SetVersionDesc(desc);     // PTS 1117690
   }
   catch (DbpError& e) {
@@ -315,21 +324,23 @@ void OMS_Session::CurrentMethodCallEpilog(tsp00_Int4 runTime)
 		{
       m_monitor_curr->cmiCounters_gg00[cmiCacheSize] = heapUsed;
 		}
-		tsp00_8ByteCounter heapDelta = heapUsed - m_heapAtMethodBegin;  
-    if (heapDelta < m_monitor_curr->cmiCounters_gg00[cmiDeltaMin])
-    {
-      m_monitor_curr->cmiCounters_gg00[cmiDeltaMin] = heapDelta; 
-    }
-    if (heapDelta > m_monitor_curr->cmiCounters_gg00[cmiDeltaMax])
-    {
-      m_monitor_curr->cmiCounters_gg00[cmiDeltaMax] = heapDelta;
-    }
-    tsp00_8ByteCounter callCnt = m_monitor_curr->cmiCallCnt_gg00;
+    tsp00_8ByteCounter heapDelta = heapUsed - m_heapAtMethodBegin;  
+    tsp00_8ByteCounter callCnt = ++m_monitor_curr->cmiCallCnt_gg00; // PTS 1133312
     if (callCnt > 0)
     {
-      m_monitor_curr->cmiCounters_gg00[cmiDeltaAvg] =
-        (m_monitor_curr->cmiCounters_gg00[cmiDeltaAvg] * (callCnt - 1) + heapDelta) / callCnt;
+        if (heapDelta < m_monitor_curr->cmiCounters_gg00[cmiDeltaMin])
+        {
+            m_monitor_curr->cmiCounters_gg00[cmiDeltaMin] = heapDelta; 
+        }
+        if (heapDelta > m_monitor_curr->cmiCounters_gg00[cmiDeltaMax])
+        {
+            m_monitor_curr->cmiCounters_gg00[cmiDeltaMax] = heapDelta;
+        }
+        tsp00_8ByteCounter cnt = m_monitor_curr->cmiCounters_gg00[cmiDeltaAvg];
+        m_monitor_curr->cmiCounters_gg00[cmiDeltaAvg] = 
+            (m_monitor_curr->cmiCounters_gg00[cmiDeltaAvg] * (callCnt - 1) + heapDelta) / callCnt;
     }
+    // End PTS 1133312
   }
 }
 
@@ -414,7 +425,9 @@ void OMS_Session::DropVersionProlog(OMS_Context* context)
     if ((context != CurrentContext()) && (!this->VersionBoundByMe(context))) {
       this->ThrowDBError (e_missing_privilege, msg, context->GetVersionId(), __MY_FILE__, __LINE__);
     }
-    context->CloseVersion();
+
+    //context->CloseVersion();   // moved to Epilog  PTS 1129082
+
     this->RemoveFromTransVersion(context);
   }
   else {
@@ -427,6 +440,11 @@ void OMS_Session::DropVersionProlog(OMS_Context* context)
 void OMS_Session::DropVersionEpilog(OMS_Context* context)
 {
   m_beforeImages.removeContext(context);
+
+  if (context->IsBoundToTrans()) {    // PTS 1129082
+    context->CloseVersion();
+  }
+
   /* free context */
   context->DeleteSelf();
   /* return to default context */
@@ -501,7 +519,7 @@ bool OMS_Session::GetMethodCallMonitorInfo (OmsMonitorData& info) const
     m_monitor.GetCallStatistics(info);
     const bool AllocatorOnly = true;
     OMS_HResult hr = m_lcSink->EvalCallStatistics(!AllocatorOnly, info);
-    memcpy(&info.m_methodName[0], &m_monitor_curr->cmiMethodName_gg00[0], sizeof(info.m_methodName));
+    SAPDB_MemCopyNoCheck(&info.m_methodName[0], &m_monitor_curr->cmiMethodName_gg00[0], sizeof(info.m_methodName));
     return true;
   }
   return false;
@@ -1106,7 +1124,7 @@ int OMS_Session::MassDerefViaKey(OmsIDerefKeyIter &derefIter)
           IncCacheHit(found->m_oid);
 
           // release memory
-          pClsInfo->chainFree(*m_context, pObj);
+          pClsInfo->chainFree(*m_context, pObj, 22);
 
           if (found->DeletedFlag()) 
             derefIter.omsSetResult(ppAddr, pKey, pClsInfo->GetKeyDesc().GetLen(),
@@ -1120,7 +1138,7 @@ int OMS_Session::MassDerefViaKey(OmsIDerefKeyIter &derefIter)
           IncCacheMiss();
 
           // release memory
-          pClsInfo->chainFree(*m_context, pObj);
+          pClsInfo->chainFree(*m_context, pObj, 23);
 
           derefIter.omsSetResult(ppAddr, pKey, pClsInfo->GetKeyDesc().GetLen(), 
                                  NULL, pErrAddr, e_object_not_found);
@@ -1253,7 +1271,7 @@ int OMS_Session::MassDerefViaKeyForUpd(OmsIDerefKeyIterForUpd &derefIter, bool d
           IncCacheHit(found->m_oid);
 
           // release memory
-          pClsInfo->chainFree(*m_context, pObj);
+          pClsInfo->chainFree(*m_context, pObj, 24);
 
           if (found->DeletedFlag()) 
             // Object is stored in local cache, but marked as deleted
@@ -1279,7 +1297,7 @@ int OMS_Session::MassDerefViaKeyForUpd(OmsIDerefKeyIterForUpd &derefIter, bool d
           IncCacheMiss();
 
           // release memory
-          pClsInfo->chainFree(*m_context, pObj);
+          pClsInfo->chainFree(*m_context, pObj, 25);
 
           derefIter.omsSetResult(ppAddr, pKey, pClsInfo->GetKeyDesc().GetLen(), 
                                  NULL, pErrAddr, e_object_not_found);
@@ -1446,7 +1464,7 @@ OmsAbstractObject* OMS_Session::NewKeyedObject(const ClassIDRef     guid,
     bool keyInCache = (NULL != found);
     if (NULL != found && m_context->IsVersionOid(found->m_oid)) { // PTS 1117571
       if (!found->DeletedFlag()) {
-        pContainerInfo->chainFree(*m_context, po);
+        pContainerInfo->chainFree(*m_context, po, 26);
         throw OmsDuplicateKey (e_duplicate_hash_key, found->m_oid, __MY_FILE__, __LINE__);
       }
     }
@@ -1462,11 +1480,11 @@ OmsAbstractObject* OMS_Session::NewKeyedObject(const ClassIDRef     guid,
         false, pContainerInfo->GetPersistentSize(), ((unsigned char*) &po->m_pobj) + sizeof(void*),
         REINTERPRET_CAST(OmsTypeOid*, &po->m_oid), (unsigned char*) &po->m_objseq, &ObjHistReadCount, &DBError);
       if FAILED (hr) {
-        pContainerInfo->chainFree(*m_context, po);
+        pContainerInfo->chainFree(*m_context, po, 27);
       }
       if ( DBError != 0 ) {
         if ( e_hash_key_not_found != DBError ) {
-          pContainerInfo->chainFree(*m_context, po);
+          pContainerInfo->chainFree(*m_context, po, 28);
           this->ThrowDBError (DBError, msg, __MY_FILE__, __LINE__);
         }
       }
@@ -1475,13 +1493,13 @@ OmsAbstractObject* OMS_Session::NewKeyedObject(const ClassIDRef     guid,
         OmsObjectContainerPtr found = m_context->FindObjInContext(&po->m_oid, true /*ignoreGeneration*/);  // PTS 1125361
         if (found) {
           if (!found->DeletedFlag()) {
-            pContainerInfo->chainFree(*m_context, po);
+            pContainerInfo->chainFree(*m_context, po, 29);
             throw OmsDuplicateKey (e_duplicate_hash_key, found->m_oid, __MY_FILE__, __LINE__);
           }
         }
         else {
             OmsObjectId oid = po->m_oid;
-            pContainerInfo->chainFree(*m_context, po);
+            pContainerInfo->chainFree(*m_context, po, 30);
             throw OmsDuplicateKey (e_duplicate_hash_key, oid, __MY_FILE__, __LINE__);
         }
       }
@@ -1575,7 +1593,7 @@ OmsAbstractObject* OMS_Session::NewKeyedObject(const ClassIDRef     guid,
              if (REUSE_FRAME) {
                // Reuse the same frame for the new object. This means, that the oid
                // of the new object is the same as the oid of the deleted object
-               pContainerInfo->chainFree(*m_context, po);
+               pContainerInfo->chainFree(*m_context, po, 31);
 
                if (found->IsNewObject()) {
                  // oid has already been counted as new
@@ -1623,7 +1641,7 @@ OmsAbstractObject* OMS_Session::NewKeyedObject(const ClassIDRef     guid,
            }
            else {
              // Object does already exist, is stored in the cache and is not marked as deleted
-             pContainerInfo->chainFree(*m_context, po);
+             pContainerInfo->chainFree(*m_context, po, 32);
              throw OmsDuplicateKey(e_duplicate_hash_key, found->m_oid, __MY_FILE__, __LINE__);
              return NULL;
            }
@@ -1631,7 +1649,7 @@ OmsAbstractObject* OMS_Session::NewKeyedObject(const ClassIDRef     guid,
         else {
           // Object does already exist in the kernel
           OmsObjectId oid = po->m_oid;
-          pContainerInfo->chainFree(*m_context, po);
+          pContainerInfo->chainFree(*m_context, po, 33);
           throw OmsDuplicateKey(e_duplicate_hash_key, oid, __MY_FILE__, __LINE__);
           return NULL;
         }
@@ -1649,14 +1667,14 @@ OmsAbstractObject* OMS_Session::NewKeyedObject(const ClassIDRef     guid,
         //                                  NewObj(b,k)              GetObjWithKey(k)
         //                                  dup_key                  not_found
         OmsObjectId oid = po->m_oid;
-        pContainerInfo->chainFree(*m_context, po);
+        pContainerInfo->chainFree(*m_context, po, 34);
         throw OmsDuplicateKey(e_duplicate_hash_key, oid, __MY_FILE__, __LINE__);
         return NULL;
       }
       else {
         // Unhandled error occured while executing GetObjWithKey
         OmsObjectId oid = po->m_oid;
-        pContainerInfo->chainFree(*m_context, po);
+        pContainerInfo->chainFree(*m_context, po, 35);
         char s[100]; 
         sp77sprintf(s, sizeof(s), "%s:%d", __MY_FILE__, __LINE__);
         ThrowDBError(DBError2, s, oid, __MY_FILE__, __LINE__);
@@ -1664,20 +1682,32 @@ OmsAbstractObject* OMS_Session::NewKeyedObject(const ClassIDRef     guid,
       }
     } 
 
-    else if (  /*e_OMS_request_timeout*/ 500 == DBError1    // PTS 1124935
-            || /*e_OMS_lock_collision*/  400 == DBError1){  // PTS 1124935
+    else if (e_key_still_visible == DBError1){
+      // The object is existing in the current consistent view, but a concurrent running
+      // transaction has deleted the object and this transaction is committed.  
+      //  Trans 1: read(a)                 new(a)
+      //  Trans 2:          del(a) commit
+      OmsObjectId oid = po->m_oid;
+      pContainerInfo->chainFree(*m_context, po, 36);
+      throw OmsDuplicateKey(DBError1, oid, __MY_FILE__, __LINE__);
+      return NULL;
+    }
+
+    else if (  /*e_request_timeout*/ 2600 == DBError1    // PTS 1124935
+            || /*e_lock_collision*/  5000 == DBError1){  // PTS 1124935
       // The keyed object can not be created because another session has 
       // already created an object with the same key, but without having 
       // committed the creation.
       OmsObjectId oid = po->m_oid;
-      pContainerInfo->chainFree(*m_context, po);
+      pContainerInfo->chainFree(*m_context, po, 37);
+        
       throw OmsKeyInUse(e_hash_key_in_use, oid, __MY_FILE__, __LINE__);
       return NULL;
     }
 
     else {
       // Unhandled error occured while executing NewObj
-      pContainerInfo->chainFree(*m_context, po);
+      pContainerInfo->chainFree(*m_context, po, 38);
       ThrowDBError(DBError1, msg, __MY_FILE__, __LINE__);
       return NULL;
     }
@@ -1741,7 +1771,7 @@ OmsAbstractObject* OMS_Session::NewObject(const ClassIDRef guid, OmsSchemaHandle
       (unsigned char*) &po->m_objseq,
       &DBError);
     if ( DBError != 0 ) {
-      pContainerInfo->chainFree(*this->CurrentContext(), po);
+      pContainerInfo->chainFree(*this->CurrentContext(), po, 39);
       this->ThrowDBError (DBError, msg, __MY_FILE__, __LINE__);
     }
   }
@@ -1787,7 +1817,7 @@ const OmsVarOid& OMS_Session::NewVarObject(OmsSchemaHandle Schema, OmsContainerN
       (unsigned char*) &po->m_objseq,
       &DBError);
     if ( DBError != 0 ) {
-      pContainerInfo->chainFree(*CurrentContext(), po);
+      pContainerInfo->chainFree(*CurrentContext(), po, 40);
       this->ThrowDBError (DBError, msg, __MY_FILE__, __LINE__);
     }
   }
@@ -1846,7 +1876,7 @@ void OMS_Session::RegContainer (OmsSchemaHandle Schema,
     this->AddContainerInfo (pContainerInfo);
   }
   if (!isVarObject) {
-    pContainerInfo->chainFree(*this->CurrentContext(), Obj);
+    pContainerInfo->chainFree(*this->CurrentContext(), Obj, 41);
   }
 }
 
@@ -1977,15 +2007,13 @@ void OMS_Session::ThrowDBError(tsp00_Int2 e, const char* msg, const OmsVersionId
     if (msgLen < sizeof(resultMsg) - sizeof(v) - 2)
     {
         char* pChar = &resultMsg[0];
-        memcpy(pChar, &msg[0], msgLen);
+        SAPDB_MemCopyNoCheck(pChar, &msg[0], msgLen);
         pChar += msgLen;
         *pChar = ' ';
         ++pChar;
-        memcpy (pChar, &v, sizeof(v));
+        SAPDB_MemCopyNoCheck (pChar, &v, sizeof(v));
         pChar += sizeof(v);
         *pChar = 0;
-        DbpBase b(m_lcSink);
-        b.dbpOpError ("%d, %s", e, &resultMsg[0]);
         this->ThrowDBError(e, &resultMsg[0], pFileName, line);
     }
     else
@@ -2067,7 +2095,7 @@ void OMS_Session::TransEnd()
       (unsigned char*) &m_defaultContext->m_consistentView, &DBError);
   }
   m_currVarObjChunk.Clear();
-  this->ClearFreeLists();
+  this->ClearFreeLists(4);
   this->ClearTransVersions();
   m_defaultContext->CleanContainerDir();
 
