@@ -34,6 +34,7 @@
 #include "Oms/OMS_ISessionLockObjects.hpp"
 #include "Oms/OMS_ObjectContainer.hpp"
 #include "Oms/OMS_VersionDirRegions.hpp"
+#include "SAPDBCommon/SAPDB_MemCopyMove.hpp"
 
 class OMS_SessionCriticalSection : public OMS_SinkCriticalSection {
 public:
@@ -93,9 +94,9 @@ inline void OMS_Session::CloseVersion()
 
 /*----------------------------------------------------------------------*/
 
-inline void OMS_Session::ClearFreeLists()
+inline void OMS_Session::ClearFreeLists(int caller)
 {
-  m_defaultContext->m_containerDir.ClearFreeLists();
+  m_defaultContext->m_containerDir.ClearFreeLists(caller);
 }
 
 /*----------------------------------------------------------------------*/
@@ -384,8 +385,9 @@ inline OmsObjectContainerPtr OMS_Session::GetMemory (OMS_ClassIdEntry& clsid)
   OmsObjectContainerPtr p;
 
   p = clsid.m_freeHead->free;
-  if (NULL != p) { 
-    clsid.m_freeHead->free = p->m_hashnext;
+  if (NULL != p) {
+    p->CheckFreeListState();
+    clsid.m_freeHead->free = p->GetNextFreeList();
   }
   else {
     if (this->InVersion()) {
@@ -414,7 +416,8 @@ inline OmsObjectContainerPtr OMS_Session::GetMemoryInVersion (OMS_ClassIdEntry&
   tsp00_Int4 size = (clsid.GetObjectSize() + sizeof(void *) - 1) & ~(sizeof(void *) - 1);
   p = clsid.m_freeHeadInVersion->free;
   if (NULL != p) { 
-    clsid.m_freeHeadInVersion->free = p->m_hashnext;
+    p->CheckFreeListState();
+    clsid.m_freeHeadInVersion->free = p->GetNextFreeList();
   }
   else {
     p = (OmsObjectContainerPtr) m_context->allocate(size + 2 * sizeof(void*));
@@ -435,7 +438,8 @@ inline OmsObjectContainerPtr OMS_Session::GetMemory (OMS_GuidEntry& classInfo)
 
   p = classInfo.m_freeHead->free;
   if (NULL != p) { 
-    classInfo.m_freeHead->free = p->m_hashnext;
+    p->CheckFreeListState();
+    classInfo.m_freeHead->free = p->GetNextFreeList();
   }
   else {
     if (this->InVersion()) {
@@ -985,10 +989,8 @@ inline void OMS_Session::MonitorWaitNewConsistentView(tsp00_Int4 runTime)
 
 /*----------------------------------------------------------------------*/
 
-inline void OMS_Session::OpenVersion(OMS_Context* context, bool create)
+inline void OMS_Session::OpenVersionProlog(const OmsVersionId &versionId, bool create)   // PTS 1129082
 {
-  short   DBError;
-  HRESULT hr;
   if (create) {
     // no modifications allowed before create
     m_defaultContext->VersionClearObjCache();
@@ -998,7 +1000,7 @@ inline void OMS_Session::OpenVersion(OMS_Context* context, bool create)
 
     while (iter) {
       OMS_Context* pContext = *iter();
-      if (context == pContext) {
+      if (!memcmp(pContext->GetVersionId(), versionId, sizeof(OmsVersionId))) {
         // created in this trans
         m_defaultContext->VersionClearObjCache();
         break;
@@ -1006,19 +1008,31 @@ inline void OMS_Session::OpenVersion(OMS_Context* context, bool create)
       ++iter;
     }
   }
-  if (IsDefaultContext(m_context)) {
-    /* current context is default context, release current consistent view */
-    if (m_context->ExistsConsistentView()) {
-      hr = m_lcSink->EndConsistentView(
-        (unsigned char*) &m_context->m_consistentView, &DBError);
-      if FAILED (hr) {
-         throw DbpError (DbpError::HRESULT_ERROR, hr, __MY_FILE__, __LINE__);
-      }
-      if ( 0 != DBError ) { 
-        this->ThrowDBError(DBError, "OmsSession::OpenVersion", context->GetVersionId(), __MY_FILE__, __LINE__);
-      }
+}
+
+/*----------------------------------------------------------------------*/
+ 
+inline void OMS_Session::OpenVersionEpilog()  // PTS 1129082
+{
+  short   DBError;
+  HRESULT hr;
+
+  /* current context is default context, release current consistent view */
+  if (m_defaultContext->ExistsConsistentView()) {
+    hr = m_lcSink->EndConsistentView((unsigned char*) &m_defaultContext->m_consistentView, &DBError);
+    if FAILED (hr) {
+      throw DbpError (DbpError::HRESULT_ERROR, hr, __MY_FILE__, __LINE__);
+    }
+    if ( 0 != DBError ) { 
+      throw DbpError (DbpError::DB_ERROR, DBError, "OmsSession::OpenVersionEpilog", __MY_FILE__, __LINE__);
     }
   }
+}
+ 
+/*----------------------------------------------------------------------*/
+
+inline void OMS_Session::OpenVersion(OMS_Context* context, bool create) 
+{
   m_context = context;
   m_context->OpenVersion(this);
   this->ChangedConsistentView();
@@ -1053,6 +1067,8 @@ inline void OMS_Session::ReleaseObj(OmsObjectContainerPtr p)
 
   const bool releaseUpdatedObj = ((!m_context->IsVersion()) && (!this->IsSubtransOpen()));
 
+  m_monitor.OmsIncOmsRelease(); // PTS 1133312
+
   if (p->DeletedFlag()) 
   {
     if (releaseUpdatedObj) 
@@ -1066,6 +1082,7 @@ inline void OMS_Session::ReleaseObj(OmsObjectContainerPtr p)
           << ", class: " << p->GetContainerInfo(m_context)->GetClassInfoPtr()->GetClassName());
         m_context->DecNewObjectsToFlush();
       }
+      m_monitor.OmsIncOmsReleaseExecuted(); // PTS 1133314
       m_context->FlushDelete (p, true);
     }
     return;
@@ -1100,7 +1117,7 @@ inline void OMS_Session::ReleaseObj(OmsObjectContainerPtr p)
       return;
     }
   }
-
+  m_monitor.OmsIncOmsReleaseExecuted(); // PTS 1133312
   m_context->DeleteObjInContext(p, p->GetContainerInfo (m_context));
 }
 
@@ -1281,7 +1298,7 @@ inline void OMS_Session:: StoreVarObject (const OmsVarOid& oid, const void* pVar
     objInfo->m_pvobj = this->GetVarObjMemory(objLength);
   }
   objInfo->m_vobjSize = objLength;
-  memcpy (objInfo->m_pvobj, pVarObj, objLength);
+  SAPDB_MemCopyNoCheck (objInfo->m_pvobj, pVarObj, objLength);
   found->MarkStored();
 }
 
