@@ -85,7 +85,7 @@ public:
 #endif
   void                  CleanAfterDropSchema();
   void                  ClearDefaultContext();
-  inline void                  ClearFreeLists();
+  inline void                  ClearFreeLists(int caller);
   inline void                  CloseVersion();
   inline void                  CommitSubtrans(int requiredLevel);
   inline void                  ClearTransVersions();
@@ -195,6 +195,8 @@ public:
   OmsAbstractObject*    NewObject(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo);
   const OmsVarOid&      NewVarObject(OmsSchemaHandle Schema, OmsContainerNo ContainerNo);
   inline void                  OpenVersion(OMS_Context* context, bool create);
+  inline void                  OpenVersionProlog(const OmsVersionId &versionId, bool create);  // PTS 1129082
+  inline void                  OpenVersionEpilog();  // PTS 1129082
   void                  RegContainer(OmsSchemaHandle Schema,
     const ClassIDRef   guid,
     const char*        ClassName, 
