@@ -48,6 +48,7 @@
 #include "Oms/OMS_Handle.hpp"
 #include "Oms/OMS_Session.hpp"
 #include "Oms/OMS_ObjectContainer.hpp"
+#include "SAPDBCommon/SAPDB_MemCopyMove.hpp"
 #include "hsp20.h"
 #include <new.h>
 
@@ -346,7 +347,17 @@ void OMS_StreamBody::IncopyStream(const SAPDB_Byte* buffer)
                 case OMS_SwapAndModifyInfo::swap_2 :
                     {
                         // swap 2 bytes integers
-                        for (int l = 0; l < abapDesc.length; ++l)
+                        // PTS 1132754, PG
+                        int intCnt = 0;
+                        if (abapDesc.abap_type == ABTYPINT2)
+                        {
+                            intCnt = 1; // == abapDesc.length/2
+                        }
+                        else // ABTYPWYDE
+                        {
+                            intCnt = abapDesc.length;
+                        }
+                        for (int l = 0; l < intCnt; ++l)
                         {
                             CopyAndSwapI2(pSource, pDest);
                             pSource += 2;
@@ -408,7 +419,7 @@ void OMS_StreamBody::IncopyStream(const SAPDB_Byte* buffer)
                 default :
                     {
                         // just copy without any translation 
-                        memcpy(pDest, pSource, abapDesc.length);
+                        SAPDB_MemCopyNoCheck(pDest, pSource, abapDesc.length);
                     }
                 } // switch
             } // for every member
@@ -466,7 +477,17 @@ void OMS_StreamBody::Write(SAPDB_Byte* pObj)
         case OMS_SwapAndModifyInfo::swap_2 :
             {
                 // swap 2 bytes integers
-                for (int l = 0; l < abapDesc.length; ++l)
+                // PTS 1132754, PG
+                int intCnt = 0;
+                if (abapDesc.abap_type == ABTYPINT2) 
+                {
+                    intCnt = 1; // == abapDesc.length/2
+                }
+                else // ABTYPWYDE
+                {
+                    intCnt = abapDesc.length;
+                }
+                for (int l = 0; l < intCnt; ++l)
                 {
                     CopyAndSwapI2(pSource, pDest);
                     pSource += 2;
@@ -529,11 +550,12 @@ void OMS_StreamBody::Write(SAPDB_Byte* pObj)
         default :
             {
                 // just copy without any translation 
-                memcpy(pDest, pSource, abapDesc.length);
+                SAPDB_MemCopyNoCheck(pDest, pSource, abapDesc.length);
             }
         } // case
     }
     ++m_rowCount;
+    m_handle.m_pSession->IncWriteStreamRow();
     m_curr += m_clientRowSize;
 }
 
