@@ -178,6 +178,8 @@ void Rst_RedoReadTask::InternReadLogAndCreateRedoFile(tgg00_TransContext      &r
                                     Rst_LogEntryIterator    &iter,
                                     SAPDBMem_IRawAllocator  &allocator)
 {
+    SAPDBTRACE_METHOD_DEBUG ("Rst_RedoReadTask::InternReadLogAndCreateRedoFile", LogTrans_Trace, 5);
+
     Log_AfterImageHeadSpace  entryHeadSpace;
     Log_AfterImage           afterImage;
     bool                     isOK            = true;
@@ -185,7 +187,6 @@ void Rst_RedoReadTask::InternReadLogAndCreateRedoFile(tgg00_TransContext      &r
                              = Log_Savepoint::GetLastSavepointEntryType() != Log_NoOp; // for migration
     bool                     nextEntryMustBeFinishOpenTrans = false; // PTS 1115065 UH 2002-04-10
     bool                     endOfTransAlreadyProcessed = false; // PTS 1123461 UH 2003-08-28
-    bool                     nextReadTransNoMustBeGreater = false; // PTS 1127838 UH 2004-02-20
     tgg00_TransIndex         currTransIndex;
     Log_Transaction         *pTransaction    = NULL;
     SAPDB_UInt               readEntryCount  = 0;
@@ -260,30 +261,6 @@ void Rst_RedoReadTask::InternReadLogAndCreateRedoFile(tgg00_TransContext      &r
 
         if ( ! afterImage.GetTransNo().gg90IsNil() )
         {
-            // PTS 1127838 UH 2004-02-20 added check: nextReadTransNoMustBeGreater
-            if ( nextReadTransNoMustBeGreater )
-            {
-                if ( k57restartrec->rstLastSurrogate_kb00().srgWriteTrans_kb00
-                     >=
-                     afterImage.GetTransNo() )
-                {
-                    Kernel_VTrace() << "Transno: T" << k57restartrec->rstLastSurrogate_kb00().srgWriteTrans_kb00.gg90GetInt4()
-                                    << " from restartrecord should be smaller than read transno.";
-                    Kernel_VTrace() << "current afterimage: T" << afterImage.GetTransNo().gg90GetInt4()
-                                    <<" (" << afterImage.GetSequence() << ") '"
-                                    << Log_ActionTypeStrings[afterImage.GetActionType()] << "' ["
-                                    << iter.IOSeqNo() << "]\n";
-                    iter.WriteToTrace ("LogEntryIterator");
-                    RTE_Message( Restart_Exception(__CONTEXT__, RST_TRANSNO_USED_AGAIN,
-                                                   SAPDB_ToString(k57restartrec->rstLastSurrogate_kb00().srgWriteTrans_kb00.gg90GetInt4()),
-                                                   SAPDB_ToString(afterImage.GetTransNo().gg90GetInt4()) ) );
-                    iter.Message ("", 0, afterImage.GetTransNo().gg90GetInt4(), afterImage.GetSequence());
-                    RTE_Message( SAPDBErr_Exception(__CONTEXT__, SAPDBERR_ASSERT_STATE_FAILED,
-                                 "Log was written by a different databas instance => Restart is impossible !") );
-                    Kernel_IAdminRestartShutdown::Instance().Offline(e_incompatible_log);
-                }
-                nextReadTransNoMustBeGreater = false;
-            }
             pTransaction = GetOrCreateRedoTransaction ( readerTrans,
                                                         allocator,
                                                         afterImage.GetTransNo(),
@@ -329,10 +306,14 @@ void Rst_RedoReadTask::InternReadLogAndCreateRedoFile(tgg00_TransContext      &r
             if ( (pTransaction->GetTransState() == Log_Transaction::committed
                   &&
                   afterImage.GetActionType() != Log_Commit)
+                  /* PTS 1132673 UH 2004-11-25
+                     if trans is marked as rollbacked the next entry may be of any kind
+                     but it must be the last entry in the log !!!
                  ||
                  (pTransaction->GetTransState() == Log_Transaction::rollbacked
                   &&
                   afterImage.GetActionType() != Log_Rollback)
+                 */
                  ||
                  (pTransaction->GetTransState() == Log_Transaction::opened) )
             {
@@ -415,7 +396,6 @@ void Rst_RedoReadTask::InternReadLogAndCreateRedoFile(tgg00_TransContext      &r
                     CRASH("restartsavepoint with too new Softwareversion");
                 }
             }
-            nextReadTransNoMustBeGreater = true; // PTS 1127838 UH 2004-02-20 added
             CloseOpenTransactions (readerTrans, lastIOSequence);
             break;
         }
@@ -457,11 +437,12 @@ void Rst_RedoReadTask::InternReadLogAndCreateRedoFile(tgg00_TransContext      &r
                 if ( LogTrans_Trace.TracesLevel(6) )
                 {
                     Kernel_VTrace() << "nextEntryMustBeFinishOpenTrans:\n";
-                    Kernel_VTrace() << "readlog: T" << afterImage.GetTransNo().gg90GetInt4()
-                                    <<" (" << afterImage.GetSequence() << ") '"
-                                    << Log_ActionTypeStrings[afterImage.GetActionType()] << "' ["
-                                    << iter.IOSeqNo() << "]\n";
-                    iter.WriteToTrace ("logEntryIter");
+                    iter.WriteToTrace ("read iterator");
+                    if ( iter.GetState() == Log_VolumeIterator::Iter_ok )
+                        Kernel_VTrace() << "readlog: T" << afterImage.GetTransNo().gg90GetInt4()
+                                        <<" (" << afterImage.GetSequence() << ") '"
+                                        << Log_ActionTypeStrings[afterImage.GetActionType()] << "' ["
+                                        << iter.IOSeqNo() << "]\n";
                 }
             }
             break;
@@ -529,6 +510,8 @@ void Rst_RedoReadTask::CloseOpenTransactions (tgg00_TransContext &redoReadTrans,
         openTransIndex = cgg_nil_transindex;
         kb51GetFirstOpenRedoTrans (redoReadTrans.trTaskId_gg00, openTransIndex);
     }//while
+    // PTS 1129746 UH 2004-06-18 added wait for all transactions which end at this time
+    Rst_IRedoTrafficControl::Instance().WaitForAllRedoTasks(redoReadTrans.trTaskId_gg00);
 }
 /* --------------------------------------------------------------------------- */
 void Rst_RedoReadTask::ReCreateOpenTransactions (tgg00_TransContext     &redoReaderTrans,
@@ -689,25 +672,32 @@ bool Rst_RedoReadTask::CopyEntry (Log_Transaction        &transaction, // PTS 11
                                   tgg00_TransContext     &readerTrans)
 {
     SAPDBTRACE_METHOD_DEBUG ("Rst_RedoReadTask::CopyEntry", LogTrans_Trace, 5);
-
-    // PTS 1127732 UH 2004-02-11 added SkipCurrentEntry 
-    const bool SkipCurrentEntry = transaction.GetLastRedoEntrySequence().IsValid()
-                                  &&
-                                  afterimage.GetSequence() <= transaction.GetLastRedoEntrySequence();
-    // If SkipCurrentEntry is true then nothing is to do - this is a redo of a redo.
-    // This entry was already copied into redo file.
-    // PTS 1115065 UH 2002-04-10
-
     SAPDBTRACE_WRITELN (LogTrans_Trace, 7, "CopyEntry: length:" << iter.RestLength() <<
                                            ", minimal: " << GetMinimalLengthToCopyEntry (afterimage.GetActionType()));
 
+    // PTS 1127732 UH 2004-02-11 added SkipCurrentEntry 
+    // If SkipCurrentEntry is true then nothing is to do - this is a redo of a redo.
+    // This entry was already copied into redo file.
+    // PTS 1115065 UH 2002-04-10
+    const bool SkipCurrentEntry = transaction.GetLastCopiedRedoEntrySequence().IsValid()
+                                  &&
+                                  afterimage.GetSequence() <= transaction.GetLastCopiedRedoEntrySequence();
+    // PTS 1131107 UH 2004-08-24 added handling for Log_MultipleActions
+    const bool isMultipleAction = Log_MultipleActions == afterimage.GetActionType();
     Log_AfterImageHeadSpace dummySplitSpace;
-    Data_SplitSpaceWriter   Writer = SkipCurrentEntry          // PTS 1127732 UH 2004-02-11
-        ? Data_SplitSpaceWriter(dummySplitSpace)   // PTS 1127732 UH 2004-02-11 use dummy writer
-        : transaction.ReserveSpaceForRedoEntryCopy ( iter.RestLength(),
-                                                     GetMinimalLengthToCopyEntry (afterimage.GetActionType()),
-                                                     afterimage.GetSequence(),
-                                                     iter.IOSeqNo() );
+    Data_SplitSpaceWriter   Writer =
+                                SkipCurrentEntry          // PTS 1127732 UH 2004-02-11
+                                ? Data_SplitSpaceWriter(dummySplitSpace)   // PTS 1127732 UH 2004-02-11 use dummy writer
+                                : isMultipleAction
+                                  ? transaction.GetWriterForLocalRedoEntries
+                                          (iter.GetValuePtr() + Log_AfterImage::GetHeaderSize(),
+                                           iter.RestLength() - Log_AfterImage::GetHeaderSize())
+                                  : transaction.ReserveSpaceForRedoEntryCopy
+                                            ( iter.RestLength(),
+                                              GetMinimalLengthToCopyEntry (afterimage.GetActionType()),
+                                              afterimage.GetSequence(),
+                                              iter.IOSeqNo() );
+
 
     Data_SplitSpaceWriter::Result writeResult      = Data_SplitSpaceWriter::ok;
     bool                          loopagain        = false;
@@ -742,7 +732,9 @@ bool Rst_RedoReadTask::CopyEntry (Log_Transaction        &transaction, // PTS 11
 				Writer.WriteToTrace("writer");
 			}
             writeResult = Data_SplitSpaceWriter::ok;
-            if ( ! SkipCurrentEntry ) // PTS 1127732 UH 2004-02-11
+            if ( ! SkipCurrentEntry // PTS 1127732 UH 2004-02-11
+                 &&
+                 ! isMultipleAction )
                 transaction.ReleaseSpaceAfterRedoEntryCopy(false); // release resources unchanged
             break;
         }
@@ -751,10 +743,31 @@ bool Rst_RedoReadTask::CopyEntry (Log_Transaction        &transaction, // PTS 11
             loopagain = iter.RestLength() > iter.Length();
         else
         {
-            SAPDBTRACE_WRITELN (LogTrans_Trace, 7, "CopyEntry: write:" << iter.Length());
-    
-            Writer.Write (iter.GetValuePtr(), iter.Length(), writeResult);
-            
+            if ( ! loopagain && isMultipleAction )
+            {
+                const SAPDB_UInt firstTimeSkipLength = Log_AfterImage::GetHeaderSize()
+                                                       +
+                                                       Log_ActionMultiple::GetHeaderSize();
+                if ( firstTimeSkipLength < iter.Length() )
+                {
+                    SAPDBTRACE_WRITELN (LogTrans_Trace, 7, "CopyEntry: write: length = " << iter.Length() << " - " << firstTimeSkipLength);
+                    Writer.Write ( iter.GetValuePtr() + firstTimeSkipLength,
+                                   iter.Length()      - firstTimeSkipLength, writeResult );
+                }
+                else
+                {
+                    SAPDBTRACE_WRITELN (LogTrans_Trace, 7, "CopyEntry: write: NO WRITE firstTimeSkipLength:" \
+                                                            << firstTimeSkipLength << " < iter.Length():" \
+                                                            << iter.Length());
+                    writeResult = Data_SplitSpaceWriter::moreSpaceAvailable;
+                }
+            }
+            else   
+            {
+                SAPDBTRACE_WRITELN (LogTrans_Trace, 7, "CopyEntry: write: length = " << iter.Length());
+                Writer.Write (iter.GetValuePtr(), iter.Length(), writeResult);
+            }
+
             if ( writeResult != Data_SplitSpaceWriter::moreSpaceAvailable
                  &&
                  writeResult != Data_SplitSpaceWriter::ok )
@@ -777,7 +790,24 @@ bool Rst_RedoReadTask::CopyEntry (Log_Transaction        &transaction, // PTS 11
          ! entryInterrupted )
     {
         if ( ! SkipCurrentEntry ) // PTS 1127732 UH 2004-02-11
-            transaction.ReleaseSpaceAfterRedoEntryCopy(true);
+        {
+            if ( ! isMultipleAction )
+                transaction.ReleaseSpaceAfterRedoEntryCopy(true);
+            else
+            {
+                if ( iter.GetState() != Log_VolumeIterator::Iter_ok )
+                {
+                    transaction.ResetLocalRedoEntries();
+                    return false;
+                }
+                if (! transaction.RestoreLocalRedoEntries (iter.IOSeqNo()))
+                {
+                    Writer.WriteToTrace("writer");
+                    iter.WriteToTrace ("read iterator");
+                    RTE_Crash(Log_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,"transaction.RestoreLocalRedoEntries"));
+                }
+            }
+        }
         return true;
     }
     // PTS 1115065 UH 2002-04-10 no abort and no change
@@ -788,7 +818,9 @@ bool Rst_RedoReadTask::CopyEntry (Log_Transaction        &transaction, // PTS 11
     }
     if ( ! entryInterrupted
          &&
-         ! SkipCurrentEntry ) // PTS 1127732 UH 2004-02-11
+         ! SkipCurrentEntry // PTS 1127732 UH 2004-02-11
+         &&
+         ! isMultipleAction ) 
         transaction.ReleaseSpaceAfterRedoEntryCopy(false);
     return false;
 }
@@ -902,6 +934,9 @@ SAPDB_UInt Rst_RedoReadTask::GetMinimalLengthToCopyEntry (Log_ActionType type)
         case Log_FinishOpenTrans:
             length += Log_ActionSavepoint::GetMinimalLengthToCopyEntry();
             break;
+        case Log_MultipleActions:
+            length += Log_ActionMultiple::GetMinimalLengthToCopyEntry();
+            break;
         default:
             break;
     }
