@@ -53,6 +53,8 @@
 #define LOG_WAIT_FOR_VACANT_ACTION   231
 /// suspend reason used waiting for another trans, which has to be commited before
 #define LOG_WAIT_FOR_PRECEDING_TRANS 232
+/// WAIT_REASON_EMPTY_LIST
+#define WAIT_REASON_EMPTY_LIST       228
 /// global instance access (singleton)
 Rst_IRedoTrafficControl* Rst_IRedoTrafficControl::m_pInstance = NULL;
 /// forward declaration
@@ -386,7 +388,6 @@ public:
      */
     void WaitForIoSeq (tsp00_TaskId       RedoTaskId,
                        tgg00_WaitContext &WaitContext,
-                       Rst_IoSeqEntry    *pOwnEotSeqEntry,
                        Log_IOSequenceNo   IoSeq,
                        Log_IOSequenceNo   FirstIoSeq,
                        bool              &WaitInserted)
@@ -395,10 +396,13 @@ public:
         
         WaitInserted = false;
         
+        if ( GetFirst() == 0 )
+            return; // list is empty
+            
         if ( LogTrans_Trace.TracesLevel(6) )
             Kernel_VTrace() << "REDOTRAFFIC: check ioseq: " << IoSeq
                             << ", first eotseq: " << GetFirst()->GetIoSeq() << NewLine;
-    
+
         if ( IoSeq == GetFirst()->GetIoSeq()
              ||
              // PTS 1120201 UH 2003-02-17
@@ -479,6 +483,8 @@ private:
     bool                    m_EndOfLog;
     /// List of idle ReDo-Tasks
     Kernel_TaskWaitList     m_VacantJobWait;
+    /// redo reader task waiting in CloseOpenTrans
+    tsp00_TaskId            m_TaskWaitingForEmptyList;
     /// list of RedoTransactions sorted by the EndOfTrans-IOSequence
     Rst_IoSeqList           m_IoSeqList;
     /// list of free RedoTransactions
@@ -508,6 +514,7 @@ private:
     : m_NumberOfTransEntries    (0),
       m_EndOfLog                (false),
       m_pFirstVacancy           (NULL),
+      m_TaskWaitingForEmptyList (cgg_nil_pid),
       m_pIoSeqContainer         (NULL),
       m_pJobContainer           (NULL),
       m_RawAllocator            (RawAllocator),
@@ -709,28 +716,23 @@ public:
         @brief  a redo task checks if preceding transactions are closed
         @param  RedoTaskId [in] 
         @param  WaitContext [in/out] 
-        @param  TransIndex [in] reference to the transaction handled by the redo task
         @param  IoSeq [in] sequence of the log entry handled by the redo task
+        @return true if caller has to wait
      */
     virtual bool CheckIoSeq (tsp00_TaskId       RedoTaskId,
                              tgg00_WaitContext &WaitContext,
-                             tgg00_TransIndex   TransIndex,
                              Log_IOSequenceNo   IoSeq)
     {
         SAPDBTRACE_METHOD_DEBUG ("Rst_RedoTraffic::CheckIoSeq", LogTrans_Trace, 5);
         
-        bool            WaitInserted;
-        Rst_IoSeqEntry *pOwnEotSeqEntry;
-        
-        WaitInserted = false;
+        bool WaitInserted = false;
         {
             Rst_RedoSync RedoSync (RedoTaskId);
-    
-            pOwnEotSeqEntry = (m_pJobContainer+TransIndex-1)->GetIoSeqEntry();
-    
-            m_IoSeqList.WaitForIoSeq (RedoTaskId, WaitContext, pOwnEotSeqEntry, IoSeq, m_OldestKnownEOTSequence, WaitInserted);
+            if ( LogTrans_Trace.TracesLevel(6) )
+                WriteToTrace ("REDOTRAFFIC: WaitForIoSeq");
+            m_IoSeqList.WaitForIoSeq (RedoTaskId, WaitContext, IoSeq,
+                                      m_OldestKnownEOTSequence, WaitInserted);
         }
-    
         return WaitInserted;
     }
     /// a redo task waits until preceding transactions are closed
@@ -739,6 +741,22 @@ public:
         SAPDBTRACE_METHOD_DEBUG ("Rst_RedoTraffic::WaitForIoSeq", LogTrans_Trace, 5);
         vsuspend (RedoTaskId, LOG_WAIT_FOR_PRECEDING_TRANS);
     }
+    /// This is used to wait until the whole list is empty
+    virtual void WaitForAllRedoTasks ( tsp00_TaskId RedoTaskId )
+	{
+        {
+            Rst_RedoSync RedoSync (RedoTaskId);
+            if ( m_TaskWaitingForEmptyList != cgg_nil_pid )
+                RTE_Crash( SAPDBErr_Exception(__CONTEXT__, SAPDBERR_ASSERT_STATE_FAILED,
+                           "Rst_RedoTrafficControl::WaitForAllRedoTasks: m_TaskWaitingForEmptyList != nil_pid") );
+            if ( m_IoSeqList.IsEmpty() )
+                return;
+            m_TaskWaitingForEmptyList = RedoTaskId;
+            m_VacantJobWait.ResumeAll();
+        }
+        vsuspend (RedoTaskId, WAIT_REASON_EMPTY_LIST);
+        m_TaskWaitingForEmptyList = cgg_nil_pid;
+	}
     /// The log reader calls this after recreation of all open transactions from last savepoint
     virtual void EnableRedoProcessing (tsp00_TaskId LogReaderTaskId)
     // PTS 1124684 UH 2003-10-16 new
@@ -862,7 +880,6 @@ void Rst_RedoTrafficControl::ExecuteJobs (tgg00_TransContext &TransContext)
                         // synchronize with other commits
                         if ( CheckIoSeq (TransContext.trTaskId_gg00,
                                          TransContext.trWaitContext_gg00,
-                                         pJob->GetTransIndex(),
                                          pJob->GetIoSeqEntry()->GetIoSeq()) )
                             WaitForIoSeq (TransContext.trTaskId_gg00);
                     }
@@ -891,7 +908,6 @@ void Rst_RedoTrafficControl::ExecuteJobs (tgg00_TransContext &TransContext)
 
                     if ( CheckIoSeq (TransContext.trTaskId_gg00,
                                      TransContext.trWaitContext_gg00,
-                                     pJob->GetTransIndex(),
                                      pJob->GetIoSeqEntry()->GetIoSeq()) )
                         WaitForIoSeq (TransContext.trTaskId_gg00);
                     
@@ -928,34 +944,32 @@ void Rst_RedoTrafficControl::ExecuteJobs (tgg00_TransContext &TransContext)
                 RTE_Crash( SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
                            "Rst_RedoTrafficControl::ExecuteJobs() transindex mismatch") );
 
-            tgg00_BasisError auxerror = TransContext.trError_gg00;
-            TransContext.trError_gg00 = e_ok;
 
-            // The following sequence of CleanUp is important  !!!
-            
-            RemoveJob (TransContext.trTaskId_gg00, pJob);
-            
-            pJob->GetTransaction().Delete();
-
-            if (e_ok != TransContext.trError_gg00)
-                RTE_Crash( SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
-                           "Rst_RedoTrafficControl::ExecuteJobs() LogTrans.Delete()") );
-
-            k51remove_locks                 ( TransContext.trTaskId_gg00,
-                                              TransContext.trIndex_gg00,
-                                              TransContext.trState_gg00 );
-            k51del_transentry_from_locklist ( TransContext.trTaskId_gg00,
-                                              TransContext.trIndex_gg00 );
-
-            // really drop all dropped files
-            k53server_drop_aux_files (TransContext);
-
-            TransContext.trWriteTransId_gg00.gg90SetNil();
-            TransContext.trLogContext_gg00 = NULL;
-            TransContext.trIndex_gg00      = cgg_nil_transindex;
-
-            if( auxerror != e_ok )
-                TransContext.trError_gg00 = auxerror;
+            if ( e_ok == TransContext.trError_gg00 ) // PTS 1132836 UH 2004-12-03 do not remove trans on error
+            {
+                // The following sequence of CleanUp is important  !!!
+                
+                RemoveJob (TransContext.trTaskId_gg00, pJob);
+                
+                pJob->GetTransaction().Delete();
+    
+                if (e_ok != TransContext.trError_gg00)
+                    RTE_Crash( SAPDBErr_Exception(__CONTEXT__,SAPDBERR_ASSERT_STATE_FAILED,
+                               "Rst_RedoTrafficControl::ExecuteJobs() LogTrans.Delete()") );
+    
+                k51remove_locks                 ( TransContext.trTaskId_gg00,
+                                                  TransContext.trIndex_gg00,
+                                                  TransContext.trState_gg00 );
+                k51del_transentry_from_locklist ( TransContext.trTaskId_gg00,
+                                                  TransContext.trIndex_gg00 );
+    
+                // really drop all dropped files
+                k53server_drop_aux_files (TransContext);
+    
+                TransContext.trWriteTransId_gg00.gg90SetNil();
+                TransContext.trLogContext_gg00 = NULL;
+                TransContext.trIndex_gg00      = cgg_nil_transindex;
+            }
         }
     }
 
@@ -1119,7 +1133,7 @@ void Rst_RedoTrafficControl::WriteToTrace (const char *title) const
     pEotIoSeqEntry = m_IoSeqList.GetFirst();
 
 
-	Kernel_VTrace() << "REDOLIST:";
+	Kernel_VTrace() << "REDOLIST: (oldestknown=" << m_OldestKnownEOTSequence << ")";
 
 	while ( pEotIoSeqEntry != NULL )
 	{
@@ -1165,6 +1179,12 @@ void Rst_RedoTrafficControl::RemoveJob (tsp00_TaskId  RedoTaskId,
         m_IoSeqList.Remove (pEotSeqEntry);
         PutIntoFreeIoSeqList (pEotSeqEntry); // added by UH 2001-07-03
     }
+    if ( m_IoSeqList.IsEmpty()
+         &&
+         m_TaskWaitingForEmptyList != cgg_nil_pid )
+    {
+        vresume(m_TaskWaitingForEmptyList);
+    }
 
     ++m_removed;
     // WriteToTrace("REDOTRAFFIC: after remove");
