@@ -50,6 +50,7 @@ extern SAPDBTrace_Topic Runtime_Trace;
 
 #include "geo50_0.h"    // KGS
 #include "SAPDBCommon/SAPDB_sprintf.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 #ifdef _WIN32           /* & if $OSSPEC = WIN32*/
 #include "gos00k.h"     /* nocheck */
@@ -359,7 +360,6 @@ static struct xparam_strcmp_table RteXparamTable2[] =
  { KERNELPARAM_GATEWAYTYPE,         XP_V_IGNORED,    0, 0 },
  { KERNELPARAM_OPMSG1,              XP_V_STRING,    XP_SZ_OFF(szOpMsg1)              },
  { KERNELPARAM_OPMSG2,              XP_V_STRING,    XP_SZ_OFF(szOpMsg2)              },
- { KERNELPARAM__SHMCHUNK,           XP_V_LONG,      XP_SZ_OFF(lShmChunk)             },
  { KERNELPARAM__SHMKERNEL,          XP_V_LONG,      XP_SZ_OFF(lShmKernel)            },
  { KERNELPARAM__TASK_STACK_AREA,    XP_V_IGNORED,   0, 0                             },
  { KERNELPARAM__USE_PROC_SCOPE,     XP_V_BOOL_Y,    XP_SZ_OFF(fUseProcessScope)      },  /* PTS 1109046 */
@@ -367,7 +367,9 @@ static struct xparam_strcmp_table RteXparamTable2[] =
  { KERNELPARAM_USE_COROUTINES,      XP_V_BOOL_Y,    XP_SZ_OFF(fUseCoroutines)        },
  { KERNELPARAM_USE_STACK_ON_STACK,  XP_V_BOOL_Y,    XP_SZ_OFF(fUseStackOnStack)      },
  { KERNELPARAM_USE_UCONTEXT,        XP_V_BOOL_Y,    XP_SZ_OFF(fUseUcontext)          },
+ { KERNELPARAM_SIMULATE_VECTORIO,   XP_V_STRING,    XP_SZ_OFF(szVectorIoMode)        },
 #endif
+ { KERNELPARAM_FORBID_LOAD_BALANCING, XP_V_BOOL_Y,  XP_SZ_OFF(fForbidLoadBalancing)  },
 /***************************************************************/
 /* End of OS Depending Parameter                               */
 /***************************************************************/
@@ -747,6 +749,45 @@ static void RTEConf_ModifyParameters(RTE_XPARAM_REC *XParam)
         XParam->DevspDescr[RTE_DevSpaceMirrorData].Devspaces = 0;
         XParam->DevspDescr[RTE_DevSpaceMirrorData].ulNumDevs = 0 ;
     }
+
+    if ( XParam->fForbidLoadBalancing )
+    {
+        XParam->uLoadBalancingCheck = 0;
+    }
+
+#ifndef WIN32
+    /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1132876 */
+         if ( 0 == SAPDB_strcasecmp(XParam->szVectorIoMode.asCharp(), "NEVER") )
+    {
+        XParam->simulateVectorIo = RTE_SimVioNever;
+    }
+    else if ( 0 == SAPDB_strcasecmp(XParam->szVectorIoMode.asCharp(), "IF_OPEN_DIRECT") )
+    {
+        XParam->simulateVectorIo = RTE_SimVioIfOpenDirect;
+    }
+    else if ( 0 == SAPDB_strcasecmp(XParam->szVectorIoMode.asCharp(), "IF_RAW_DEVICE") )
+    {
+        XParam->simulateVectorIo = RTE_SimVioIfRawDevice;
+    }
+    else if ( 0 == SAPDB_strcasecmp(XParam->szVectorIoMode.asCharp(), "IF_OPEN_DIRECT_OR_RAW_DEVICE") )
+    {
+        XParam->simulateVectorIo = RTE_SimVioIfOpenDirectOrRawDevice;
+    }
+    else if ( 0 == SAPDB_strcasecmp(XParam->szVectorIoMode.asCharp(), "ALWAYS") )
+    {
+        XParam->simulateVectorIo = RTE_SimVioAlways;
+    }
+    else
+    {
+#if defined(LINUX)
+        SAPDB_strcpy(XParam->szVectorIoMode.asCharp(), "IF_OPEN_DIRECT_OR_RAW_DEVICE");
+        XParam->simulateVectorIo = RTE_SimVioIfOpenDirectOrRawDevice;
+#else
+        SAPDB_strcpy(XParam->szVectorIoMode.asCharp(), "NEVER");
+        XParam->simulateVectorIo = RTE_SimVioNever;
+#endif
+    }
+#endif
 }
 
 
@@ -760,6 +801,8 @@ externC SAPDB_Bool RTEConf_ReadAndAnalyzeParam ( tsp00_DbNamec VAR_ARRAY_REF
     /* --- Set XPARAM default values */
     RTEConf_InitParam ( XParam );
 
+    RTE_Message( SAPDBErr_MessageList(RTE_CONTEXT, RTEINFO_BEGIN_OF_STARTUP_ARGUMENTS) );
+
     /* read the special parameters other parameters depend on (MAXARCHIVELOGS etc.) */
     RTEConf_ReadSpecialParameters(XParam);
 
@@ -777,6 +820,7 @@ externC SAPDB_Bool RTEConf_ReadAndAnalyzeParam ( tsp00_DbNamec VAR_ARRAY_REF
         {
             RTEConf_Parameter::Integer value;
             RTEConf_ParameterAccess::Instance()->GetInteger(name,value,err);
+            RTE_Message( SAPDBErr_MessageList(RTE_CONTEXT, RTEINFO_STARTUP_ARGUMENT, (SAPDB_Char const *)name, SAPDB_ToString( (SAPDB_Int4)value )));
             valueRec.xp_value_t = NXP_INT_TYPE;
             valueRec.xp_type.xp_integer = (SAPDB_Int4)value;
             break;
@@ -787,6 +831,7 @@ externC SAPDB_Bool RTEConf_ReadAndAnalyzeParam ( tsp00_DbNamec VAR_ARRAY_REF
         {
             RTEConf_Parameter::String value;
             RTEConf_ParameterAccess::Instance()->GetString(name,value,err);
+            RTE_Message( SAPDBErr_MessageList(RTE_CONTEXT, RTEINFO_STARTUP_ARGUMENT, (SAPDB_Char const *)name, SAPDB_ToString( (SAPDB_Char const *)value )));
             valueRec.xp_value_t = NXP_STRING_TYPE;
             valueRec.xp_type.xp_string = value;
             break;
@@ -795,6 +840,7 @@ externC SAPDB_Bool RTEConf_ReadAndAnalyzeParam ( tsp00_DbNamec VAR_ARRAY_REF
         {
             RTEConf_Parameter::CryptInfo value;
             RTEConf_ParameterAccess::Instance()->GetCryptInfo(name,value,err);
+            RTE_Message( SAPDBErr_MessageList(RTE_CONTEXT, RTEINFO_STARTUP_ARGUMENT, (SAPDB_Char const *)name, "***") );
             valueRec.xp_value_t = NXP_CRYPT_TYPE;
             memcpy(valueRec.xp_type.xp_crypt,(SAPDB_Byte *)value,sizeof(RTEConf_Parameter::CryptInfo));
             break;
@@ -810,6 +856,8 @@ externC SAPDB_Bool RTEConf_ReadAndAnalyzeParam ( tsp00_DbNamec VAR_ARRAY_REF
         }
     }
 
+    RTE_Message( SAPDBErr_MessageList(RTE_CONTEXT, RTEINFO_END_OF_STARTUP_ARGUMENTS) );
+
     RTEConf_ModifyParameters(XParam);
 
     if(!(err.IsEmpty() || SAPDBErr_MessageList::Info == err.Type()) || !(paramReader.EndIteration(err)))
