@@ -26,9 +26,6 @@
 \endif
 */
 
-
-
-
 /*===========================================================================*
  *  INCLUDES                                                                 *
  *===========================================================================*/
@@ -48,7 +45,7 @@
 #include    "SAPDBCommon/Tracing/SAPDBTrace_Usage.hpp"
 #include    "RunTime/RTE_ConsoleTraceTopic.hpp"
 
-#if defined (LINUX)                                         /*&if $OSSPEC = LINUX*/
+#if defined (LINUX)            /*&if $OSSPEC = LINUX */
 #include    "RunTime/RTE_ConsoleSemaphoreTimeoutList.hpp"   /* nocheck */
 #endif                                                      /*&endif*/        
 
@@ -485,6 +482,21 @@ static  SAPDB_Bool   GetRequestFromParameter
                                         ArgN++;
                                     }
                                 }
+                                else if (( lParam == PARA_RWREGIONS ) && (ARGS(argc, ArgN)))
+                                {
+                                    if (0 != strcmp("all",argv[ArgN]))
+                                    { 
+                                        if (e92_get_tt_to_show (argv[ArgN], (int *)&xcr.lExtParam))
+                                        {
+                                            ArgN++;
+                                        }
+                                    }
+                                    else
+                                    {
+                                        xcr.lExtParam = 27 ;
+                                        ArgN++;
+                                    }
+                                }
                                 else if (lParam == PARA_TASKS || lParam == PARA_ACTIVE || lParam == PARA_RUNNABLE  || lParam == PARA_TASK_MOVE)
                                 {
                                     xcr.lExtParam = SHOW_TT_ALL_EO92;
@@ -860,6 +872,11 @@ static SAPDB_UInt4  GetShowResParam
         return PARA_AWE;
     }
 
+    if (e92_strcont ((char *)XCONS_CMD_RWREGIONS, pszParam, XCONS_MIN_CMD_LEN_RWREGIONS))
+    {
+        return PARA_RWREGIONS;
+    }
+
     if (e92_strcont ((char *)XCONS_CMD_REGIONS, pszParam, XCONS_MIN_CMD_LEN_REGIONS))
     {
         return PARA_REGIONS;
@@ -1204,4 +1221,4 @@ externC SAPDB_Bool  RTECons_ProcessStandardRequest
     //could be enqueued in the timeout list of the timeout thread. But at the moment there is no need for it.
 
     return true;
-}
\ No newline at end of file
+}
