@@ -209,6 +209,7 @@
 #define XCONS_CMD_CONNECTIONS              "CONNECTIONS"
 #define XCONS_CMD_MOVE_INFO                "MOVEINFO  "
 #define XCONS_CMD_TASK_MOVE                "T_MOVE    "
+#define XCONS_CMD_RWREGIONS                "RWREGIONS "
 
 #define XCONS_CMD_TRACE_XCONS              "XCONS     "
 #define XCONS_CMD_TRACE_CONSOLE            "CONSOLE   "
@@ -241,6 +242,7 @@
 #define XCONS_MIN_CMD_LEN_MOVE_INFO        4
 #define XCONS_MIN_CMD_LEN_TRACE_XCONS      2
 #define XCONS_MIN_CMD_LEN_TRACE_CONSOLE    4
+#define XCONS_MIN_CMD_LEN_RWREGIONS        2
 
 
 #define COM_UNDEF           0
@@ -296,6 +298,7 @@
 #define PARA_MOVE_INFO      0x400000
 #define PARA_TRACE_XCONS    0x1000000
 #define PARA_TRACE_CONSOLE  0x2000000
+#define PARA_RWREGIONS      0x4000000
 
 #define PARA_SET          0x01
 #define PARA_CLEAR        0x02
