@@ -1775,11 +1775,17 @@ ValidateOrCreateShare(SAPDB_Char const * volumePath)
         strcat(shareName, ApiCopy.id);
     }
 
+    memset(sharePath, 0, sizeof(RTE_Path));
     if ( readlink(shareName, sharePath, sizeof(RTE_Path)-1) < 0 )
     {
         if ( errno == ENOENT )
         {
-            symlink(ApiCopy.workingDirectory, shareName);
+            if ( 0 == symlink(ApiCopy.workingDirectory, shareName) )
+            {
+                return true;
+            }
+            ReportError("ValidateOrCreateShare cannot create read volume directory symlink %s:%d",
+                        shareName, errno);
             return true;
         }
 
