@@ -546,6 +546,34 @@ public:
                                 LogPosition          & firstStillNeededOffset,
                                 SAPDBErr_MessageList & errList );
 
+    /*!
+       @brief    Send backup history info line
+
+       Send from master instance to standby instance.
+       @param          standbyNodeName [in] node to send command to
+       @param          backupHistoryInfo [in] the info line (if needed quoted externally, not in this routine!)
+       @param          errList [out] set to error if call failed
+       @return   true if call suceeded, false if call failed
+
+     */
+     SAPDB_Bool SendBackupHistoryInfo( RTE_Nodename const  & standbyNodeName,
+                                       SAPDB_Char const    * backupHistoryInfo,
+                                       SAPDBErr_MessageList & errList );
+
+    /*!
+       @brief    Send backup medium information line
+
+       Send from master instance to standby instance.
+       @param          standbyNodeName [in] node to send command to
+       @param          backupMediumInfo [in] the info line (if needed quoted externally, not in this routine!)
+       @param          errList [out] set to error if call failed
+       @return   true if call suceeded, false if call failed
+
+     */
+     SAPDB_Bool SendBackupMediumInfo( RTE_Nodename const  & standbyNodeName,
+                                      SAPDB_Char const    * backupMediumInfo,
+                                      SAPDBErr_MessageList & errList );
+
 private:
     /*!
        @brief    The constructor
