@@ -35,7 +35,7 @@
 
 #include "gsp00.h"
 #if defined (_WIN32) /*&if $OSSPEC == WIN32 */
-#include "gos003.h"
+#include "gos003.h"  /* nocheck */
 #endif               /*&endif               */
 
 void
