@@ -107,11 +107,11 @@ RTEPPars_SAPDBProt::RTEPPars_SAPDBProt ()
                      m_PropertyTable( NULL ),
                      m_NumProperties( 0 )
 {
-  m_EntryPoints.AttachProperties = _AttachProperties;
+  m_EntryPoints.AttachProperties = reinterpret_cast <ATTACHPROPERTIES> (_AttachProperties);
   m_EntryPoints.Register         = _Register;
   m_EntryPoints.Deregister       = _Deregister;
-  m_EntryPoints.FormatProperties = _FormatProperties;
-  m_EntryPoints.RecognizeFrame   = _RecognizeFrame;
+  m_EntryPoints.FormatProperties = reinterpret_cast <FORMATPROPERTIES> (_FormatProperties);
+  m_EntryPoints.RecognizeFrame   = reinterpret_cast <RECOGNIZEFRAME> (_RecognizeFrame);
 }
 
 /*---------------------------------------------------------------------------*/
