@@ -70,6 +70,13 @@ extern "C"  // this is just for the sqlx*() wrappers
 #include "heo05.h"
 }
 
+// this for writing the state of the kernel process to the dbm protocol if the kernel process cannot be stopped
+// happens on HPUX only and is very platform specific
+#ifdef HPUX
+#include <sys/pstat.h>  /* nocheck */
+#include "hin100.h"     /* nocheck */
+#include "SAPDBCommon/ErrorsAndMessages/SAPDBErr_MessageEventConverter.hpp"     /* nocheck */
+#endif
 /*===========================================================================*
  *  DEFINES                                                                  *
  *===========================================================================*/
@@ -509,7 +516,6 @@ SAPDB_Bool RTE_ActiveDatabase::OpenKernel
             SAPDB_Bool firstCall = true;
             do
             {
-                SLEEP ( 1000 );
                 // Check the state of the service. To minimize the bad busy-waiting-effects,
                 // wait a second between the calls to the Windows service manager - but NOT
                 // before the first call!
@@ -760,6 +766,102 @@ void RTE_ActiveDatabase::saveErrorMessageToFile(SAPDB_Char *errorMessage)
     close(fd);
 }
 
+// this for writing the state of the kernel process to the dbm protocol if the kernel process cannot be stopped
+// happens on HPUX only and is very platform specific
+#ifdef HPUX
+static void outputProcessState(pid_t kernelPid,SAPDBErr_MessageList &err)
+{
+    struct pst_status psa;
+    if ( pstat_getproc(&psa, sizeof(struct pst_status), 0, kernelPid) == -1 )
+    {
+        return; //  error in pstat_getproc();
+    }
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_idx", SAPDB_ToString(psa.pst_idx));        /* Index for further pstat() requests int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_uid", SAPDB_ToString(psa.pst_uid));        /* Real UID         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_pid", SAPDB_ToString(psa.pst_pid));        /* Process ID         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_ppid", SAPDB_ToString(psa.pst_ppid));       /* Parent process ID         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_dsize", SAPDB_ToString(psa.pst_dsize));      /* # real pages used for data         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_tsize", SAPDB_ToString(psa.pst_tsize));      /* # real pages used for text         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_ssize", SAPDB_ToString(psa.pst_ssize));      /* # real pages used for stack         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_nice", SAPDB_ToString(psa.pst_nice));       /* Nice value         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_term.psd_minor", SAPDB_ToString(psa.pst_term.psd_minor));       /* TTY of this process; -1/-1 if there isn't one         struct psdev    */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_term.psd_major", SAPDB_ToString(psa.pst_term.psd_major));       /* TTY of this process; -1/-1 if there isn't one         struct psdev    */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_pgrp", SAPDB_ToString(psa.pst_pgrp));       /* Process group of this process         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_pri", SAPDB_ToString(psa.pst_pri));        /* priority of process         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_addr", SAPDB_ToString(psa.pst_addr));       /* address of process (in memory)         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_cpu", SAPDB_ToString(psa.pst_cpu));        /* processor utilization for scheduling         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_utime", SAPDB_ToString(psa.pst_utime));      /* user time spent executing (in seconds)         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_stime", SAPDB_ToString(psa.pst_stime));      /* system time spent executing (in seconds)         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_start", SAPDB_ToString(psa.pst_start));      /* time process started (seconds since epoch)         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_flag", SAPDB_ToString(psa.pst_flag));       /* flags associated with process         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_stat", SAPDB_ToString(psa.pst_stat));       /* Current status         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_wchan", SAPDB_ToString(psa.pst_wchan));      /* If state PS_SLEEP, value sleeping on         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_procnum", SAPDB_ToString(psa.pst_procnum));    /* processor this proc last run on         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_cmd", SAPDB_ToString(psa.pst_cmd));        /* Command line for the process, if available         char[PST_CLEN]            */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_time", SAPDB_ToString(psa.pst_time));       /* resident time for scheduling         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_cpticks", SAPDB_ToString(psa.pst_cpticks));    /* ticks of cpu time */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_cptickstotal", SAPDB_ToString(psa.pst_cptickstotal)); /* total ticks for life of process         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_fss", SAPDB_ToString(psa.pst_fss));        /* fair share scheduler group id         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_pctcpu", SAPDB_ToString(psa.pst_pctcpu));     /* %cpu for this process during p_time         float */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_rssize", SAPDB_ToString(psa.pst_rssize));     /* resident set size for process (private pages)         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_suid", SAPDB_ToString(psa.pst_suid));       /* saved UID         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_ucomm", SAPDB_ToString(psa.pst_ucomm));      /* executable basename the process is running         char[PST_UCOMMLEN]                 */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_shmsize", SAPDB_ToString(psa.pst_shmsize));    /* # real pages used for shared memory         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_mmsize", SAPDB_ToString(psa.pst_mmsize));     /* # real pages used for memory mapped files         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_usize", SAPDB_ToString(psa.pst_usize));      /* # real pages used for U-Area & K-Stack         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_iosize", SAPDB_ToString(psa.pst_iosize));     /* # real pages used for I/O device mapping         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_vtsize", SAPDB_ToString(psa.pst_vtsize));     /* # virtual pages used for text         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_vdsize", SAPDB_ToString(psa.pst_vdsize));     /* # virtual pages used for data         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_vssize", SAPDB_ToString(psa.pst_vssize));     /* # virtual pages used for stack         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_vshmsize", SAPDB_ToString(psa.pst_vshmsize));   /* # virtual pages used for shared memory         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_vmmsize", SAPDB_ToString(psa.pst_vmmsize));    /* # virtual pages used for mem-mapped files         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_vusize", SAPDB_ToString(psa.pst_vusize));     /* # virtual pages used for U-Area & K-Stack         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_viosize", SAPDB_ToString(psa.pst_viosize));    /* # virtual pages used for I/O dev mapping         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_minorfaults", SAPDB_ToString(psa.pst_minorfaults));/* # page reclaims for the process         uint64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_majorfaults", SAPDB_ToString(psa.pst_majorfaults));/* # page faults needing disk access         uint64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_nswap", SAPDB_ToString(psa.pst_nswap));      /* # of swaps for the process         uint64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_nsignals", SAPDB_ToString(psa.pst_nsignals));   /* # signals received by the process         uint64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_msgrcv", SAPDB_ToString(psa.pst_msgrcv));     /* # socket msgs received by the proc        uint64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_msgsnd", SAPDB_ToString(psa.pst_msgsnd));     /* # of socket msgs sent by the proc         uint64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_maxrss", SAPDB_ToString(psa.pst_maxrss));     /* highwater mark for proc resident set size         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_sid", SAPDB_ToString(psa.pst_sid));        /* session ID         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_schedpolicy", SAPDB_ToString(psa.pst_schedpolicy));/* scheduling policy for the process         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_ticksleft", SAPDB_ToString(psa.pst_ticksleft));  /* clock ticks left in process' RR timeslice         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_rdir.psf_fsid.psfs_id", SAPDB_ToString(psa.pst_rdir.psf_fsid.psfs_id));       /* File ID of the process' root directory         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_rdir.psf_fsid.psfs_type", SAPDB_ToString(psa.pst_rdir.psf_fsid.psfs_type));       /* File ID of the process' root directory         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_rdir.psf_fileid", SAPDB_ToString(psa.pst_rdir.psf_fileid));       /* File ID of the process' root directory         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_rdir.psf_spare", SAPDB_ToString(psa.pst_rdir.psf_spare));       /* File ID of the process' root directory         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_cdir.psf_fsid.psfs_id", SAPDB_ToString(psa.pst_cdir.psf_fsid.psfs_id));       /* File ID of the process' current directory         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_cdir.psf_fsid.psfs_type", SAPDB_ToString(psa.pst_cdir.psf_fsid.psfs_type));       /* File ID of the process' current directory         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_cdir.psf_fileid", SAPDB_ToString(psa.pst_cdir.psf_fileid));       /* File ID of the process' current directory         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_cdir.psf_spare", SAPDB_ToString(psa.pst_cdir.psf_spare));       /* File ID of the process' current directory         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_text.psf_fsid.psfs_id", SAPDB_ToString(psa.pst_text.psf_fsid.psfs_id));       /* File ID of the process' executable         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_text.psf_fsid.psfs_type", SAPDB_ToString(psa.pst_text.psf_fsid.psfs_type));       /* File ID of the process' executable         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_text.psf_fileid", SAPDB_ToString(psa.pst_text.psf_fileid));       /* File ID of the process' executable         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_text.psf_spare", SAPDB_ToString(psa.pst_text.psf_spare));       /* File ID of the process' executable         struct psfileid*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_highestfd", SAPDB_ToString(psa.pst_highestfd));  /* highest file descriptor currently opened         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_euid", SAPDB_ToString(psa.pst_euid));       /* Effective UID         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_egid", SAPDB_ToString(psa.pst_egid));       /* Effective GID         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_ioch", SAPDB_ToString(psa.pst_ioch));       /* # of characters read/written         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_usercycles.psc_hi", SAPDB_ToString(psa.pst_usercycles.psc_hi)); /* 64-bit user mode execution cycle count         struct pstcycles*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_usercycles.psc_lo", SAPDB_ToString(psa.pst_usercycles.psc_lo)); /* 64-bit user mode execution cycle count         struct pstcycles*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_systemcycles.psc_hi", SAPDB_ToString(psa.pst_systemcycles.psc_hi));/* 64-bit system mode execution cycle count         struct pstcycles*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_systemcycles.psc_lo", SAPDB_ToString(psa.pst_systemcycles.psc_lo));/* 64-bit system mode execution cycle count         struct pstcycles*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_interruptcycles.psc_hi", SAPDB_ToString(psa.pst_interruptcycles.psc_hi));/* 64-bit interrupt for proc cycle count         struct pstcycles*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_interruptcycles.psc_lo", SAPDB_ToString(psa.pst_interruptcycles.psc_lo));/* 64-bit interrupt for proc cycle count         struct pstcycles*/
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_gid", SAPDB_ToString(psa.pst_gid));        /* Real GID         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_sgid", SAPDB_ToString(psa.pst_sgid));       /* saved effective gid         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_nlwps", SAPDB_ToString(psa.pst_nlwps));      /* # lwps within this process         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_lwpid", SAPDB_ToString(psa.pst_lwpid));      /* LWP identifier. NOTE: If this process is multi-threaded, this is an lwpid of one of the LWPs in the process at this exact moment, which LWP is undefined (random) int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_valid", SAPDB_ToString(psa.pst_valid));      /* This is a vector that will indicate if a certain field in the structure is valid or not. int32_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_text_size", SAPDB_ToString(psa.pst_text_size)); /* Page size used for text objects.         int64_t */
+    err = err + SAPDBErr_MessageList (RTE_CONTEXT, RTEINFO_PROCESSSTATE, "psa.pst_data_size", SAPDB_ToString(psa.pst_data_size)); /* Page size used for data objects.         int64_t */
+
+    teo200_EventList eventList;
+    SAPDBErr_ConvertMessageToEventList(err,eventList); 
+    tin100_GlobalLog::writeEntry (eventList);
+}
+#endif
 /*-----------------------------------------------------------*/
 
 
@@ -929,6 +1031,17 @@ SAPDB_Bool RTE_ActiveDatabase::updateEnvironment(
         return false;
     }
 #  endif
+#endif
+
+    /* enforce linuxthread compatibility mode for non NPTL supporting database kernel */
+#if 0 /* This selects the wrong pthread library... maybe '2.4.1' is correct ... to be tested... */
+#if defined(LINUX) && !defined(HAS_NPTL) && defined(I386)
+    if ( putenv("LD_ASSUME_KERNEL=2.2.5") != 0 )
+    {
+        SAPDB_sprintf(errorText,maxErrorTextLengthInBytes,"start:   Cannot putenv LD_ASSUME_KERNEL=2.2.5" );
+        return false;
+    }
+#endif
 #endif
 
     return true;
@@ -1002,6 +1115,11 @@ SAPDB_Bool RTE_ActiveDatabase::doWaitForPid(
                 if( 0 == offlineTimeOut )
                 {
                     errList = SAPDBErr_MessageList (RTE_CONTEXT, RTEERR_DBREG_COULD_NOT_KILL_KERNEL,SAPDB_ToString(offlineTimeOut*2));
+// this for writing the state of the kernel process to the dbm protocol if the kernel process cannot be stopped
+// happens on HPUX only and is very platform specific
+#ifdef HPUX
+                    outputProcessState(kernelPid,errList);
+#endif
                     ret = false;
                 }
 
@@ -1627,13 +1745,13 @@ RTE_ActiveDatabase::Start( RTE_SpeedInfo &speed
     while(returnedPid != serverpid);
 
     /*
-        Now check if the kernel state is really admin -
+        Now check if the kernel state is really admin or reached already online state -
         To handle the case that the kernel crashed after closing the diag pipe correctly 
         http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1122382    
     */  
     SAPDB_UInt4 dbState;
     getDbPidStateFromShm(NULL,NULL,&dbState,0,0,errList);
-    if(SERVER_COLD != dbState)
+    if(SERVER_COLD != dbState && SERVER_WARM != dbState )
     {
         errList = SAPDBErr_MessageList (RTE_CONTEXT, RTEERR_DBREG_KERNEL_DIED_BEFORE_ADMIN);
         return false;
