@@ -46,6 +46,7 @@
 #include "geo002.h"
 
 
+
 /*===========================================================================*
  *  DEFINES                                                                  *
  *===========================================================================*/
@@ -106,6 +107,7 @@
 #define CON_REQ_PAGECACHE_RELEASE      51
 #define CON_REQ_DIAGNOSE_PAGECACHE     52
 
+#define CON_REQ_RWREGIONS              53
 
 #define CON_REQUEST(_request)              ((_request) & ~SECURITY_CHECK_ACCESS_RIGHTS)  
 #define CON_REQUEST_SECURITY(_request)     ((_request) != CON_REQUEST (_request))
@@ -117,6 +119,7 @@
 #define CON_REQ_CLOSE           0xFF010000
 #define CON_REQ_UNDEFINED       0xFFFF0000
 
+
 /* Tests */
 #define CON_REQ_TEST01          0xFFFF0001
 #define CON_REQ_TEST02          0xFFFF0002
@@ -306,6 +309,7 @@ typedef struct cons_ukt_info_record
     TID                             Tid;
     INT4                            ThrdState;
     double                          dDispatcherCalls;
+    double                          dTaskSwitchCalls;
     SAPDB_ULong                     ulActiveTasks;
     SAPDB_ULong                     ulTotalTasks;
     CHAR                            szCluster[80];
@@ -683,6 +687,8 @@ typedef struct cons_single_task_info_record
     SAPDB_ULong                     UktSleepCount;      /* CR 1115172 */
     SAPDB_Long                      WaitTime;           /* CR 1115172 */
     SAPDB_ULong                     ulDynPrio;          /* CR 1115172 */
+    void                           *RWRegion;
+    UINT1                           WaitingForExclusiveAccess;
 } CONS_SINGLE_TASK_INFO_REC;
 
 typedef CONS_SINGLE_TASK_INFO_REC *PCONS_SINGLE_TASK_INFO_REC;
@@ -780,6 +786,43 @@ typedef struct cons_check_interval_record
     SAPDB_Int4                      CheckIntervalTime;
 } CONS_CHECK_INTERVAL_RECORD;
 
+//
+//  RWRegions
+//
+
+typedef struct cons_rwregion_info_record
+{
+    union
+    {
+        SAPDB_ULong ulInfoNum;
+        SAPDB_Bool  showAll;
+    };
+} CONS_RWREGION_INFO_REC;
+
+typedef CONS_RWREGION_INFO_REC        *PCONS_RWREGION_INFO_REC;
+
+typedef struct cons_single_rwregion_info_record
+{
+    void        *pAddress;
+    enum RWRegionAccess
+    {
+        none=0,
+        holdingExclusive=1,
+        holdingNonExclusive=2,
+        waitingForExclusive=3,
+        waitingForNonExclusive=4
+    }
+    access;
+    RTE_TaskId  taskID;
+    SAPDB_UInt4 collisionCounter;
+    SAPDB_UInt4 exclusiveAccessCounter;
+    SAPDB_UInt4 nonExclusiveAccessCounter;
+    SAPDB_UInt4 waitCounter;
+    SAPDB_UInt4 tasCounter;
+} CONS_SINGLE_RWREGION_INFO_REC;
+
+typedef CONS_SINGLE_RWREGION_INFO_REC        *PCONS_SINGLE_RWREGION_INFO_REC;
+
 //
 // --- Setting trace level for console thread
 //
@@ -811,6 +854,7 @@ typedef union console_data_record
     CONS_TASK_MOVE_INFO_REC         TaskMoveInfo;
     CONS_CHECK_INTERVAL_RECORD      SetCheckInterval;
     CONS_SET_TRACELEVEL             SetTraceLevel;
+    CONS_RWREGION_INFO_REC          RWRegionInfo;
     //only for TEST!!!!!
     SAPDB_Byte                      test01String[5];
 } CONSOLE_DATA_REC;
