@@ -139,11 +139,11 @@
 #define XCONS_SHOW_RTE_CONCURR_DEV_WORKER  "\n %d IO Worker threads executing concurrently\n"
 
 #define XCONS_SHOW_RTE_UKT_HEADER          "\nUser Kernel Threads:\n"                                             \
-                                           " %-6.6s  %10.10s  %-11.11s  %13.13s  %6.6s  %5.5s   %s\n"             \
-                                           " %-6.6s  %10.10s  %-11.11s  %13.13s  %6.6s  %5.5s   %s\n",            \
-                                           "Thread", XCONS_SYS, "State", "Dispatch", "Active" , "Total" , "Task", \
-                                           "Name",   "Tid",     "",      "Counter",  "Tasks"  , "Tasks" , "Cluster"
-#define XCONS_SHOW_RTE_UKT_INFO            " UKT%-3d  %10.10s  %-11.11s %14.0f  %6d  %5d   %s \n"
+                                           " %-6.6s  %10.10s  %-11.11s  %13.13s  %13.13s  %6.6s  %5.5s   %s\n"             \
+                                           " %-6.6s  %10.10s  %-11.11s  %13.13s  %13.13s  %6.6s  %5.5s   %s\n",            \
+                                           "Thread", XCONS_SYS, "State", "Dispatch", "TaskSwitch", "Active" , "Total" , "Task", \
+                                           "Name",   "Tid",     "",      "Counter",  "Counter",    "Tasks"  , "Tasks" , "Cluster"
+#define XCONS_SHOW_RTE_UKT_INFO            " UKT%-3d  %10.10s  %-11.11s %14.0f %14.0f  %6d  %5d   %s \n"
 #define XCONS_SHOW_RTE_UKT_AFFIN_HEADER    "\nUser Kernel Threads affinity:\n"
 #define XCONS_SHOW_RTE_UKT_AFFIN_INFO      " UKT%-3d <-> CPU:%3d\n"
 
@@ -487,6 +487,8 @@
 #define XCONS_TSK_STATE_VVECTORIO          "Vvectorio     "
 #define XCONS_TSK_STATE_VDUALVECTORIO      "Vdualvectorio "
 #define XCONS_TSK_STATE_VCOPYVOLUME        "Vcopyvolume   "
+#define XCONS_TSK_STATE_VENTERRWREGION     "VenterRWRegion"
+#define XCONS_TSK_STATE_VLEAVERWREGION     "VleaveRWRegion"
 #define XCONS_TSK_STATE_UNKNOWN            "UNKNOWN       "
 
 
