@@ -48,11 +48,11 @@ extern RTE_CoroutineContext *RTE_GetTaskContext(RTE_TaskId id);
 extern pthread_t RTE_GetTaskThread(RTE_TaskId id);
 
 extern "C" {
-#if defined (I386) || defined(IA64) || defined(S390X)
+#if (defined(LINUX)) && (defined (I386) || defined(IA64) || defined(S390X) || defined(X86_64) || defined(SDBonPPC64))
 #  define MAKECONTEXT_USES_STACK_BASE
 #  define MAKECONTEXT_ZERO_ARGUMENT_COUNT 1
 typedef void * stackPointerType;
-#endif /* LINUX && IA64 */
+#endif /* LINUX && (I386 || IA64 || S390X || X86_64 || SDBonPPC64) */
 
 #if defined(ALPHA)
 #  define MAKECONTEXT_USES_STACK_BASE
@@ -97,6 +97,10 @@ struct RTE_pthread_tls_key
 } RTE_TLSKeys[THREAD_KEYS_FIRST_LEVEL*THREAD_KEYS_SECOND_LEVEL] = { 0 };
 #endif
 
+
+
+extern "C" void en88memcpy(void *a,void *b,SAPDB_UInt s);
+
 /*!--------------------------------------------------------------------
    @brief Switch from one coroutine context to another
 
@@ -122,7 +126,24 @@ MSGD(( INFO_TEST_WILDCARD, eBuffer ));
 
     if ( XPARAM(fUseUcontext) )
     {
+#ifdef HP_IA64
+/* This is a workaround for broken swapcontect() on HPUX IA64. During the swapcontext() call the page-fault- */
+/* handling is deactivated, so that ucontext structures that are paged out cannot be accessed. In such cases */
+/* the function fails and errno is EFAULT. if this is detected, the ucontext structures are copied (just to  */
+/* make sure they are paged into main memory) and swapcontext() is called again. This is done in a loop      */
+/* because the ucontexts may be paged out again just before the swapcontext() call. */
+/* HP has announced a patch for this problem for January 2005, this workaround code may be eliminated then...*/
+        int retval;
+        ucontext_t dummyContext;
+        while( ((retval = swapcontext(&oldContext.context.uC, &newContext.context.uC)) != 0) && (errno == EFAULT) )
+        {
+            en88memcpy(&dummyContext,&oldContext.context.uC,sizeof(ucontext_t));
+            en88memcpy(&dummyContext,&newContext.context.uC,sizeof(ucontext_t));
+        }
+        return ( 0 == retval );
+#else
         return ( swapcontext(&oldContext.context.uC, &newContext.context.uC) == 0 );
+#endif
     }
     else
     {
@@ -244,6 +265,20 @@ SAPDB_Bool RTE_ContextPrepare( RTE_CoroutineContext &    GivenContextPointer
 
         makecontext ( pContext, u.initialContextCall, MAKECONTEXT_ZERO_ARGUMENT_COUNT);
 
+#if defined(LINUX) && defined(X86_64)
+		// fix machine context stack pointer
+		// caused by broken makecontext ()
+		// according the x86-64 abi (%rsp - 8) must be a multiple of 16
+#ifndef REG_RSP
+#define REG_RSP 15
+#endif
+		unsigned long sp = pContext->uc_mcontext.gregs[REG_RSP];
+		if ((sp & (unsigned long) 15) != (unsigned long) 8) {
+			sp &= ~(unsigned long) 15;
+			sp -= 8;
+	        pContext->uc_mcontext.gregs[REG_RSP] = sp;
+		}
+#endif
         return true;
     }
     else
