@@ -49,7 +49,7 @@
 #define KERNEL_LZU
 #include "geo50_0.h"
 
-#if  defined(IA64) || (defined(WIN32) && defined(BIT64)) || defined (HP_IA64)
+#if  defined(IA64) || (defined(WIN32) && defined(_IA64)) || defined (HP_IA64)
 #  include "RunTime/RTE_IA64GetBsp.h" /* nocheck */
 #endif
 
@@ -137,7 +137,7 @@
 /*ALL*/           SAPDB_UInt4 currentStackUsage;
 #if defined(_WIN32)
 /*WIN*/           currentStackUsage = (SAPDB_UInt4)(((SAPDB_Byte *)pTaskCtrl->Context.pStackLimit) - ((SAPDB_Byte *)&pTaskCtrl));
-#  ifdef BIT64
+#  if defined(_IA64)
 /*WIN IA64 */     /* On IA64, there is a special memory area for swapping out registers used for local variables */
 /*WIN IA64 */     /* called backing store. On Windows, this area starts at the top of the stack area and is as   */
 /*WIN IA64 */     /* big as the stack. So, a stack overflow occurs when either the stack pointer reaches the     */
@@ -294,6 +294,10 @@
 /*ALL*/               --heuristicCounter;
 /*ALL*/           }
 /*ALL*/
+/*ALL*/           if ( pHeuristicCheck <= pLastUsedOfStack )
+/*ALL*/           {
+/*ALL*/               heuristicCounter = 0;
+/*ALL*/           }
 /*ALL*/       } while ( pStackImage > pLastUsedOfStack && heuristicCounter != 0 );
 /*ALL*/
 /*ALL*/       maximumStackUsage = (SAPDB_UInt4)(((SAPDB_Byte *)pFirstUsedOfStack) - (SAPDB_Byte *)pStackImage);
@@ -322,6 +326,11 @@
 /*ALL*/               --heuristicCounter;
 /*ALL*/           }
 /*ALL*/
+/*ALL*/           if ( pHeuristicCheck >= pLastUsedOfStack )
+/*ALL*/           {
+/*ALL*/               heuristicCounter = 0;
+/*ALL*/           }
+/*ALL*/
 /*ALL*/       } while ( pStackImage < pLastUsedOfStack && heuristicCounter != 0 );
 /*ALL*/
 /*ALL*/       maximumStackUsage = (SAPDB_UInt4)(((SAPDB_Byte *)pStackImage) - (SAPDB_Byte *)pFirstUsedOfStack);
@@ -364,14 +373,44 @@
 
 /*ALL*/static TASK_CTRL GetTaskControl(RTE_TaskId taskId)
 /*ALL*/{
-/*ALL*/    if ( 0 == taskId )
-/*ALL*/    {
-/*ALL*/        return CURR_TASK;
-/*ALL*/    }
-/*ALL*/    else
-/*ALL*/    {
-/*ALL*/        return (KGS->pFirstTaskCtrl + taskId - 1);
-/*ALL*/    }
+/*ALL*/           TASK_CTRL pTaskCtrl;
+/*ALL*/           if ( 0 == taskId )
+/*ALL*/           {
+#ifdef WIN32
+/*WIN32*/             PUKT_CTRL_REC pUkt = THIS_UKT_CTRL;
+#ifdef SAPDB_SLOW
+/*WIN*/               if ( !pUkt 
+/*WIN*/                 || !KGS 
+/*WIN*/                 || !KGS->pFirstUKTCtrl
+/*WIN*/                 || pUkt < KGS->pFirstUKTCtrl
+/*WIN*/                 || !KGS->pLastUKTCtrl
+/*WIN*/                 || pUkt > KGS->pLastUKTCtrl )
+/*WIN*/               {
+/*WIN*/                   return 0;
+/*WIN*/               }
+#endif
+/*WIN*/               pTaskCtrl = pUkt->pCTask;
+#else
+/*UNIX*/              ten50_UKT_Control * pUkt = THIS_UKT_CTRL;
+#ifdef SAPDB_SLOW
+/*UNIX*/              if ( !pUkt 
+/*UNIX*/                 || !KGS 
+/*UNIX*/                 || !KGS->first_kp 
+/*UNIX*/                 || pUkt < KGS->first_kp 
+/*UNIX*/                 || !KGS->last_kp 
+/*UNIX*/                 || pUkt > KGS->last_kp )
+/*UNIX*/              {
+/*UNIX*/                  return 0;
+/*UNIX*/              }
+#endif
+/*UNIX*/              pTaskCtrl = pUkt->curr_task;
+#endif
+/*ALL*/           }
+/*ALL*/           else
+/*ALL*/           {
+/*ALL*/               pTaskCtrl = (KGS->pFirstTaskCtrl + taskId - 1);
+/*ALL*/           }
+/*ALL*/           return pTaskCtrl;
 /*ALL*/}
 
 /*------------------------------*/
