@@ -377,7 +377,7 @@ bool RTE_DBRegister::RegisterInstallation( const RTE_Path dbRoot,
     RegistryEntries[1].ulValueSize  = (ULONG)strlen(version.key)+1;
     RegistryEntries[1].ulValueType  = REG_SZ;
 
-    rc = sql50_reg_put_applic_values (NULL, HKEY_LOCAL_MACHINE, szKey,
+    rc = sql50_reg_put_applic_values (NULL, NULL, HKEY_LOCAL_MACHINE, szKey,
                                       NULL, 2, RegistryEntries);
     if(rc != NO_ERROR)
     {
