@@ -46,6 +46,12 @@
 #include "RunTime/RTE_KGS.hpp"
 #include "heo07.h"
 #include "heo01.h"
+#include "RunTime/RTE_DiagPipe.h" 
+#if defined(WIN32)  /*&if $OSSPEC = WIN32 */
+#include "gos00k.h" /* nocheck but_imkdep */
+#else               /*&else */
+#include "gen41.h"  /* nocheck but_imkdep */
+#endif              /*&endif */
 
 class RTE_ExternalCall : public RTE_IExternalCall
 {
@@ -293,6 +299,8 @@ SAPDB_Bool RTE_ExternalCall::ExecuteCommand( SAPDB_Char const *command,
                                              RTE_IExternalCallReplyContext *replyContext,
                                              SAPDBErr_MessageList &errList)
 {
+    SAPDB_Bool callResult;
+
     if ( replyHandler != RTE_ExternalCallIgnoreReply )
     {
         if ( !m_ReplyBuffer )
@@ -366,6 +374,8 @@ SAPDB_Bool RTE_ExternalCall::ExecuteCommand( SAPDB_Char const *command,
                                         SAPDB_ToString(command),
                                         SAPDB_ToString(rc)
                                        );
+        DisconnectNamedPipe(pipeHandle);
+        CloseHandle(pipeHandle);
         return false;        
     }
     
@@ -375,6 +385,9 @@ SAPDB_Bool RTE_ExternalCall::ExecuteCommand( SAPDB_Char const *command,
                                             pipeHandle,
                                             errList) )
         {
+	        CloseHandle(processInfo.hProcess);
+            DisconnectNamedPipe(pipeHandle);
+	        CloseHandle(pipeHandle);
             return false;        
         }
     }
@@ -392,6 +405,9 @@ SAPDB_Bool RTE_ExternalCall::ExecuteCommand( SAPDB_Char const *command,
                                             SAPDB_ToString(command),
                                             SAPDB_ToString(rc)
                                            );
+	        CloseHandle(processInfo.hProcess);
+            DisconnectNamedPipe(pipeHandle);
+    	    CloseHandle(pipeHandle);
             return false;
         }
 
@@ -401,8 +417,12 @@ SAPDB_Bool RTE_ExternalCall::ExecuteCommand( SAPDB_Char const *command,
         }
 
     } while ( STILL_ACTIVE == terminationStatus );
+
     CloseHandle(processInfo.hProcess);
+    DisconnectNamedPipe(pipeHandle);
     CloseHandle(pipeHandle);
+
+    callResult = ( 0 == terminationStatus );
 #else
     RTESys_CommandOutputPipeContext cmdContext;
     RTE_SystemRc rc;
@@ -434,11 +454,13 @@ SAPDB_Bool RTE_ExternalCall::ExecuteCommand( SAPDB_Char const *command,
                                        );
         return false;        
     }
+
+    callResult = ( 0 == exitStatus );
 #endif
 
     if ( replyHandler != RTE_ExternalCallIgnoreReply )
     {
-        SAPDB_Bool callResult = (*replyHandler)(command, m_ReplyBuffer, replyContext, errList);
+        callResult = (*replyHandler)(command, m_ReplyBuffer, replyContext, errList);
 
         // free memory if we got an oversized reply...
         if ( m_ReplyBufferLength > m_MaxReplySize )
@@ -446,11 +468,9 @@ SAPDB_Bool RTE_ExternalCall::ExecuteCommand( SAPDB_Char const *command,
             m_ReplyBufferLength = 0;
             Deallocate(m_ReplyBuffer);
         }
-
-        return callResult;
     }
 
-    return true;
+    return callResult;
 } // RTE_ExternalCall::ExecuteCommand
 
 //---------------------------------------------
@@ -611,8 +631,12 @@ SAPDB_Bool RTE_ExternalCall::FillReplyBufferFromNamedPipe( HANDLE processHandle,
                             break;
                         }
 
-                        if ( terminationStatus == 0 )
-                        if ( STILL_ACTIVE != terminationStatus )
+                        if ( 0 == terminationStatus )
+                        {
+                            readPipeDone = true;
+                            returnValue = vf_eof;
+                        } 
+                        else if ( STILL_ACTIVE != terminationStatus )
                         {
                             lastTry = true;
                             localErrList = SAPDBErr_MessageList( RTE_CONTEXT,
@@ -710,7 +734,7 @@ static SAPDB_Bool SpawnDbmstartOrStop(void *startOrStop)
 
 //-----------------------------------------------------------------
 
-void RTE_SpawnDbmstartOrStop(void *startOrStop)
+SAPDB_Bool RTE_SpawnDbmstartOrStop(void *startOrStop)
 {
     if ( SpawnDbmstartOrStop(startOrStop) )
     {
@@ -724,33 +748,42 @@ void RTE_SpawnDbmstartOrStop(void *startOrStop)
             RTE_Message( SAPDBErr_MessageList( RTE_CONTEXT
                        , RTEINFO_AUTOMATIC_SHUTDOWN_INITIATED ) );
         }
+        return true;
+    }
+
+    if ( startOrStop )
+    {
+        RTE_Message( SAPDBErr_MessageList( RTE_CONTEXT
+                                         , RTEERR_AUTOMATIC_RESTART_FAILED ) );
     }
     else
     {
-        if ( startOrStop )
-        {
-            RTE_Message( SAPDBErr_MessageList( RTE_CONTEXT
-                                             , RTEERR_AUTOMATIC_RESTART_FAILED ) );
-        }
-        else
-        {
-            RTE_Message( SAPDBErr_MessageList( RTE_CONTEXT
-                                             , RTEERR_AUTOMATIC_SHUTDOWN_FAILED ) );
-        }
+        RTE_Message( SAPDBErr_MessageList( RTE_CONTEXT
+                                         , RTEERR_AUTOMATIC_SHUTDOWN_FAILED ) );
     }
+
+    return false;
 }
 
 //-----------------------------------------------------------------
     
-extern "C" void RTE_SpawnRestartShutdown(void *startOrStop)
+extern "C" SAPDB_Bool RTE_SpawnRestartShutdown(void *startOrStop)
 {
-    RTE_SpawnDbmstartOrStop(startOrStop);
+    return RTE_SpawnDbmstartOrStop(startOrStop);
 }
 
 //-----------------------------------------------------------------
     
 extern "C" void *RTE_AutoRestartShutdownKernel(void *startOrStop)
 {
+#if defined (_WIN32)    
+    __try 
+    {
+#       if defined(_FASTCAP)
+         CAPNameThread ("RTE_AutoRestartShutdownKernel");
+#       endif
+#endif
+
     SAPDB_Bool doRestart = (startOrStop != 0);
     if ( AutoRestartShutdownKernel(doRestart) )
     {
@@ -771,13 +804,42 @@ extern "C" void *RTE_AutoRestartShutdownKernel(void *startOrStop)
         {
             RTE_Message( SAPDBErr_MessageList( RTE_CONTEXT
                                              , RTEERR_AUTOMATIC_RESTART_FAILED ) );
+            /* tidy up... */
+            RTE_CloseDiagPipe();
+
+#if defined(_WIN32)
+            sql61k_post_xstart_sem ( kgs.szServerDB, false );
+#else
+            (void)en41RemovePipeFDFile(&kgs->serverdb[0]);
+#endif
         }
         else
         {
-            RTE_Message( SAPDBErr_MessageList( RTE_CONTEXT
-                                             , RTEERR_AUTOMATIC_SHUTDOWN_FAILED ) );
+#if defined(_WIN32)
+            if ( kgs.fWinShutdownPending )
+            {
+                RTE_Message( SAPDBErr_MessageList( RTE_CONTEXT
+                                               , RTEERR_AUTOMATIC_SHUTDOWN_FAILED ) );
+            }
+            else
+            {
+                RTE_Crash( SAPDBErr_MessageList( RTE_CONTEXT
+                                               , RTEERR_AUTOMATIC_SHUTDOWN_FAILED ) );
+            }
+#else
+            RTE_Crash( SAPDBErr_MessageList( RTE_CONTEXT
+                                           , RTEERR_AUTOMATIC_SHUTDOWN_FAILED ) );
+#endif
         }
     }
+#if defined (_WIN32)
+    }
+    __except( sql50k_excepthandler(GetExceptionInformation()) ) 
+    {
+        ;
+    }
+#endif
+
     return (void *)0;
 }
 
@@ -835,7 +897,7 @@ static SAPDB_Bool AutoRestartShutdownKernel(SAPDB_Bool doRestart)
 
     SAPDBErr_MessageList errList;
     
-    RTE_IExternalCall *pCall = RTE_IExternalCall::CreateExternalCall(errList);
+    RTE_IExternalCall *pCall = RTE_IExternalCall::CreateExternalCall(errList, 1024);
     if ( !pCall )
     {
         RTE_Message(errList);
@@ -858,4 +920,4 @@ static SAPDB_Bool AutoRestartShutdownKernel(SAPDB_Bool doRestart)
 
 /*===========================================================================*
  *  END OF CODE                                                              *
- *===========================================================================*/
\ No newline at end of file
+ *===========================================================================*/
