@@ -56,7 +56,7 @@
 #include <signal.h>
 #endif
 
-#include <RunTime/RTE_FileUser.hpp>
+#include "RunTime/RTE_FileUser.hpp"
 
 /*===========================================================================*
  *  DEFINES                                                                  *
@@ -952,7 +952,9 @@ psinit (struct procinfo **ps, struct modinfo **mi) {
 
 	return 0;
 }
+#endif
 
+#ifdef UNIX
 static struct procinfo *
 who_uses_module (struct procinfo *ps, struct modinfo *mi, const char *modname) {
 	pid_t pid;
@@ -974,7 +976,9 @@ who_uses_module (struct procinfo *ps, struct modinfo *mi, const char *modname) {
 	} else if (pid == 0) {
 		/* do a fuser command, redirect STDOUT and STDERR to parent */
 
-		char fusercmd[] = "/usr/sbin/fuser";
+		char fuserbincmd[] = "/bin/fuser";
+		char fusersbincmd[] = "/sbin/fuser";
+		char fuserposixcmd[] = "/usr/sbin/fuser";
 		struct stat sb[1];
 
 		close (fdout[0]);
@@ -982,13 +986,17 @@ who_uses_module (struct procinfo *ps, struct modinfo *mi, const char *modname) {
 		dup2 (fdout[1], 2);
 
 #ifdef _AIX
-		if (stat (fusercmd, sb) == 0)
-			execl (fusercmd, "fuser", "-x", modname, 0);
+		if (stat (fuserposixcmd, sb) == 0)
+			execl (fuserposixcmd, "fuser", "-x", modname, 0);
 		else
 			execlp ("fuser", "fuser", "-x", modname, 0);
 #else
-		if (stat (fusercmd, sb) == 0)
-			execl (fusercmd, "fuser", modname, 0);
+		if (stat (fuserposixcmd, sb) == 0)
+			execl (fuserposixcmd, "fuser", modname, 0);
+		else if (stat (fuserbincmd, sb) == 0)
+			execl (fuserbincmd, "fuser", modname, 0);
+		else if (stat (fusersbincmd, sb) == 0)
+			execl (fusersbincmd, "fuser", modname, 0);
 		else
 			execlp ("fuser", "fuser", modname, 0);
 #endif
@@ -1053,6 +1061,9 @@ who_uses_module (struct procinfo *ps, struct modinfo *mi, const char *modname) {
 #ifdef _AIX
 		/* aix uses a 's' for 'shared lib' and 'e' for 'executable */
 		if (*ptr == 's' || *ptr == 'e') {
+#elif defined linux
+		/* linux uses a 't' for 'text' or a 'e' for 'executable' and a 'm' for 'maped' */
+		if (*ptr == 'm' || *ptr == 't' || *ptr == 'e') {
 #else
 		/* unix uses a 't' for 'text' and a 'm' for 'maped' */
 		if (*ptr == 'm' || *ptr == 't') {
@@ -1295,6 +1306,7 @@ psinit (struct procinfo **ps, struct modinfo **mi) {
 		strcat (pidstatfilename, "/");
 		strcat (pidstatfilename, statfilename);
 
+#if 0
 		pidmapfilename = (char *)
 		malloc (strlen (piddirname) + 1 + strlen (mapfilename) + 1);
 
@@ -1310,6 +1322,7 @@ psinit (struct procinfo **ps, struct modinfo **mi) {
 		strcpy (pidmapfilename, piddirname);
 		strcat (pidmapfilename, "/");
 		strcat (pidmapfilename, mapfilename);
+#endif
 
 		free (piddirname);
 
@@ -1323,8 +1336,10 @@ psinit (struct procinfo **ps, struct modinfo **mi) {
 		fpstat = fopen (pidstatfilename, "r");
 		free (pidstatfilename);
 
+#if 0
 		fpmap = fopen (pidmapfilename, "r");
 		free (pidmapfilename);
+#endif
 
 		if (fpstat != 0) {
 			char state;
@@ -1353,6 +1368,7 @@ psinit (struct procinfo **ps, struct modinfo **mi) {
 			fclose (fpstat);
 		}
 
+#if 0
 		if (fpmap != 0) {
 			pid_t pid;
 			pid = (pid_t) atol (de->d_name);
@@ -1384,9 +1400,10 @@ psinit (struct procinfo **ps, struct modinfo **mi) {
 					return -1;
 				}
 			}
-
 			fclose (fpmap);
 		}
+#endif
+
 	}
 
 	closedir (dp);
@@ -1394,7 +1411,7 @@ psinit (struct procinfo **ps, struct modinfo **mi) {
 }
 #endif
 
-#if defined linux || defined WIN32
+#ifdef WIN32
 static struct procinfo *
 who_uses_module (struct procinfo *ps, struct modinfo *mi, const char *modname) {
 	struct modinfo *currmod;
