@@ -49,6 +49,9 @@
 #include "RunTime/RTE_Database.hpp"
 #include "SAPDBCommon/SAPDB_sprintf.h"
 #include "SAPDBCommon/SAPDB_Names.h"
+#ifndef _WIN32
+#include "RunTime/MemoryManagement/RTEMem_RteAllocator.hpp" /* nocheck */
+#endif
 
 /*===========================================================================*
  *  DEFINES                                                                  *
@@ -156,7 +159,9 @@ Container_List<RTE_Database>* RTE_Installation::GetDatabases(SAPDBErr_MessageLis
                         delete currentDataBase;
                     }
                     currentDataBase = new RTE_Database(*this,pszDBRoot,szServerDB);
+                    currentDataBase->m_speedList.Delete();  // we don't like the pre-filled speed list here as it may contain speeds that are not registered for this special database...
                 }
+                currentDataBase->m_speedList.InsertFront(speedInfo);
                 if(active)
                 {
                     currentDataBase->SetActiveSpeed(speedInfo);
@@ -204,20 +209,235 @@ SAPDB_Bool RTE_Installation::dbHasSpecificProcess (
 }
 
 /*----------------------------------------*/
+//#include <limits.h>
+#define NUMBER_OF_ELEMENTS_IN_PIDLIST_ 1
 
-SAPDB_Bool RTE_Installation::dbHasSpeedFile (
-    RTE_DBName       &dbname,
-    const SAPDB_UTF8 *speedBuffer,
-    SAPDB_Int4 sizeOfSpeedBuffer)
+class PidArray
 {
+public:
+    PidArray()
+        : m_pNext(NULL)
+    {}
+    SAPDB_UInt  m_elements[NUMBER_OF_ELEMENTS_IN_PIDLIST_];
+    PidArray   *m_pNext;
+};
+
+class PidList
+{
+public:
+    PidList()
+        : m_nextElementToFill(0)
+        , m_currentPidArray(&m_firstPidArray)
+    {}
+
+    ~PidList()
+    {
+        PidArray *pNext = m_firstPidArray.m_pNext;
+        while(pNext)
+        {
+            PidArray *pNextNext = pNext->m_pNext;
+            (RTEMem_RteAllocator::Instance()).Deallocate(pNext);
+            pNext = pNextNext;
+        }
+    }
+    
+    void Insert(SAPDB_UInt newPid)
+    {
+        if( NUMBER_OF_ELEMENTS_IN_PIDLIST_ == m_nextElementToFill )
+        {
+            PidArray *pNewArray = new(RTEMem_RteAllocator::Instance())PidArray;
+            m_currentPidArray->m_pNext = pNewArray;
+            m_nextElementToFill = 0;
+            m_currentPidArray = pNewArray;
+        }
+        m_currentPidArray->m_elements[m_nextElementToFill] = newPid;
+        m_nextElementToFill++;
+    }
+
+    void Dump(void)
+    {
+        SAPDB_UInt i;
+        PidArray *pNext = m_firstPidArray.m_pNext;
+        PidArray *currentArray = &m_firstPidArray;
+        do
+        {
+            for(i = 0;i < (pNext ? NUMBER_OF_ELEMENTS_IN_PIDLIST_ : m_nextElementToFill);i++)
+            {
+                printf("%u\n",currentArray->m_elements[i]);
+            }
+            if(pNext)
+            {
+                currentArray = pNext;
+                pNext = pNext->m_pNext;
+            }
+            else
+            {
+                break;
+            }
+        }
+        while(1);
+    }
+
+    SAPDB_Bool Contains(SAPDB_UInt pid)
+    {
+        SAPDB_UInt i;
+        PidArray *pNext = m_firstPidArray.m_pNext;
+        PidArray *currentArray = &m_firstPidArray;
+        do
+        {
+            for(i = 0;i < (pNext ? NUMBER_OF_ELEMENTS_IN_PIDLIST_ : m_nextElementToFill);i++)
+            {
+                if( pid == currentArray->m_elements[i])
+                {
+                    return true;
+                }
+            }
+            if(pNext)
+            {
+                currentArray = pNext;
+                pNext = pNext->m_pNext;
+            }
+            else
+            {
+                break;
+            }
+        }
+        while(1);
+        return false;
+    }
+private:
+    PidArray    m_firstPidArray;
+    SAPDB_UInt  m_nextElementToFill;
+    PidArray  *m_currentPidArray;
+};
+
+void RTE_Installation::CheckFuserExistence(void)
+{
+    // check if a fuser executable is accessible. It may be in the $PATH, it may be accessible
+    // via /usr/sbin or not accessible at all. If it is accessible, calling it without arguments
+    // produces something like "Usage:....." on stderr.
+    m_fuserState = noAccess;
+    FILE *p;
+#define FUSER_BUFFERSIZE_   256
+    SAPDB_Char buf[FUSER_BUFFERSIZE_];
+    // first try 
+    p=popen("fuser 2>&1","r");  // try to call it in the $PATH . 2>&1 is used to redirect stderr to stdout, which can be accessed by reading from p
+    buf[0]=0;
+    fgets(buf,FUSER_BUFFERSIZE_,p);
+    if(strstr(buf,"sage"))  // to match "usage" or "Usage"...
+    {
+        m_fuserState = inPath;
+    }
+    pclose(p);
+    
+    if( m_fuserState != inPath )    // if it was not found in the $PATH, try to call it explicitly in /usr/sbin
+    {
+        p=popen("/usr/sbin/fuser 2>&1","r");
+        buf[0]=0;
+        fgets(buf,FUSER_BUFFERSIZE_,p);
+        if(strstr(buf,"sage"))
+        {
+            m_fuserState = inUsrSbin;
+        }
+        pclose(p);
+    }
+    if( m_fuserState == noAccess )    // if it was not found in the $PATH, try to call it explicitly in /usr/sbin
+    {
+        p=popen("/sbin/fuser 2>&1","r");
+        buf[0]=0;
+        fgets(buf,FUSER_BUFFERSIZE_,p);
+        if(strstr(buf,"sage"))
+        {
+            m_fuserState = inSbin;
+        }
+        pclose(p);
+    }
+}
+
+void RTE_Installation::FillPidList(const SAPDB_Char *pExecutableName,PidList &pidList)
+{
+    RTE_Path fuserCall;
+    FILE *p;
+#define FUSER_BUFFERSIZE_   256
+
+#ifdef _IBMR2
+#define OS_SPECIFIC_FUSER_OPTION_ "-c"
+#else
+#define OS_SPECIFIC_FUSER_OPTION_ ""
+#endif
+    SAPDB_Char buf[FUSER_BUFFERSIZE_];
+
+    const SAPDB_Char *pFuserExecutable = NULL;
+    if( inPath == m_fuserState )
+    {
+        pFuserExecutable = "fuser";
+    }
+    else if( inUsrSbin == m_fuserState )
+    {
+        pFuserExecutable = "/usr/sbin/fuser";
+    }
+    else if( inSbin == m_fuserState )
+    {
+        pFuserExecutable = "/sbin/fuser";
+    }
+    if( pFuserExecutable )
+    {
+        SAPDB_sprintf(fuserCall,sizeof(RTE_Path),"%s %s %s/pgm/%s 2>/dev/null"
+            ,pFuserExecutable
+            ,OS_SPECIFIC_FUSER_OPTION_
+            ,m_dbRoot
+            ,pExecutableName 
+            );
+        if((p = popen(fuserCall,"r")) != NULL)
+        {
+            SAPDB_UInt index;
+            SAPDB_UInt currentPid=0;
+            while(fgets(buf,FUSER_BUFFERSIZE_,p) != NULL)
+            {
+                index = 0;
+                while(index < FUSER_BUFFERSIZE_ && '\0' != buf[index])
+                {
+                    if(isspace(buf[index]))
+                    {
+                        if(0 != currentPid)
+                        {
+                            pidList.Insert(currentPid);
+                        }
+                        currentPid = 0;
+                    }
+                    else if(isdigit(buf[index])) // the pids in the fuser output are followed by character flags. Usually these flags go to stderr, but linux is not usual...
+                    {
+                        currentPid *= 10;
+                        currentPid += (buf[index]-'0');
+                    }
+                    index++;
+                }
+            }
+            if(0 != currentPid)
+            {
+                pidList.Insert(currentPid);
+            }
+            pclose(p);
+        }
+    }
+}
+
+SAPDB_Bool RTE_Installation::dbHasRunningProcess ( RTE_DBName &dbname,RTE_SpeedInfo speed,PidList &pidsRunningKernel)
+{
+    SAPDB_UTF8 speedFromFile[RTE_DBREGISTRATION_MAX_SPEED_NAME_LENGTH];
+
     tsp00_TaskId pid;
 
-    if ( 0 != en41GetPPIDFromFile(dbname, &pid)
-      || pid == 0
-      || 0 != en41GetPIDFromFile(dbname, &pid)
-      || pid == 0
-      || 0 != en41GetDbSpeedFromFile(dbname, (char *)speedBuffer, sizeOfSpeedBuffer) )
-    {
+    if ( 0 != en41GetPPIDFromFile(dbname, &pid) // does the ppid file exist?
+      || pid == 0                               // does the ppid file contain a pid?
+      || 0 != kill(pid,0)                       // is the process with that pid alive?
+      || 0 != en41GetPIDFromFile(dbname, &pid)  // does the pid file exist?
+      || pid == 0                               // does the pid file contain a pid?
+      || 0 != kill(pid,0)                       // is the process with that pid alive?
+      || !((noAccess == m_fuserState) || pidsRunningKernel.Contains(pid))
+      || 0 != en41GetDbSpeedFromFile(dbname, (char *)speedFromFile, RTE_DBREGISTRATION_MAX_SPEED_NAME_LENGTH)   // does the speed file exist?
+      || 0 != strcasecmp((const char *)speedFromFile, (const char *)speed.Name() ) )
+    {                                           // any of conditions above not true --> not running
         return false;
     }
     else
@@ -226,9 +446,6 @@ SAPDB_Bool RTE_Installation::dbHasSpeedFile (
     }
 }
 
-#include <limits.h>
-
-
 Container_List<RTE_Database>* RTE_Installation::GetDatabases(SAPDBErr_MessageList &errList)
 {
     Container_List<RTE_Database> *liste = new Container_List<RTE_Database>(RTEMem_Allocator::Instance());
@@ -266,6 +483,13 @@ Container_List<RTE_Database>* RTE_Installation::GetDatabases(SAPDBErr_MessageLis
     {
         RTE_Database  *currentDataBase = NULL;
 
+        PidList pidsRunningKernel;
+
+        FillPidList(speedFast.ExecutableName(),pidsRunningKernel);
+        FillPidList(speedQuick.ExecutableName(),pidsRunningKernel);
+        FillPidList(speedSlow.ExecutableName(),pidsRunningKernel);
+        FillPidList(speedTest.ExecutableName(),pidsRunningKernel);
+
         int ok;
         do
         {
@@ -291,18 +515,13 @@ Container_List<RTE_Database>* RTE_Installation::GetDatabases(SAPDBErr_MessageLis
                     while(speedIterator != Container_Node<RTE_SpeedInfo>::InvalidNode)
                     {
                         SAPDB_Bool running = false;                 
-                        SAPDB_UTF8 speedFromFile[RTE_DBREGISTRATION_MAX_SPEED_NAME_LENGTH];
-
                         if(buildProcessList)
                         {
                             running =  dbHasSpecificProcess (dbname, (*speedIterator).ExecutableName()) ;
                         }
                         else
                         {
-                            if( dbHasSpeedFile(dbname,&speedFromFile[0],RTE_DBREGISTRATION_MAX_SPEED_NAME_LENGTH))
-                            {
-                                running = ( 0 == strcasecmp((const char *)speedFromFile, (const char *)(*speedIterator).Name() ) );
-                            }
+                            running = dbHasRunningProcess (dbname,*speedIterator,pidsRunningKernel);
                         }
                         if(running)
                         {
