@@ -626,7 +626,8 @@ SAPDB_Bool RTE_NTService::UpdateServiceEntry (
             serviceType = SERVICE_NO_CHANGE;
             if((SAPDB_Char *)RTE_SERVICE_ENTRY_UNCHANGED == pszAccountName)
             {
-                pszAccountName = NULL;
+                pszAccountName  = NULL;
+                pszPassword     = NULL;
             }
             else
             {
@@ -646,6 +647,7 @@ SAPDB_Bool RTE_NTService::UpdateServiceEntry (
                     }
                 }
             }
+            SAPDB_Bool changeDBRoot = stricmp(m_dbRoot, Params.szDBRoot) != 0 ;
         
             // from now on: (serviceType == SERVICE_NO_CHANGE && pszAccountName == NULL) -> do not change account
             //              (serviceType != SERVICE_NO_CHANGE && pszAccountName == NULL) -> change account to local system
@@ -657,7 +659,7 @@ SAPDB_Bool RTE_NTService::UpdateServiceEntry (
             tsp100_VersionResult Result = sp100_CompareVersionIDs(&Params.RTEVersionID);
 
             if ( 
-                ( stricmp(m_dbRoot, Params.szDBRoot)    != 0 )              ||
+                  changeDBRoot                                              ||
                 ( pszParameter                          != NULL )           ||
                 ( Result                                != IsEqual_esp100 ) ||
                 ( Params.ValUseIndepPath                != UseIndepPath )   ||
@@ -695,7 +697,7 @@ SAPDB_Bool RTE_NTService::UpdateServiceEntry (
                                 RegistryEntries[0].ulValueType  = REG_SZ;
                                 rc = sql50_reg_put_service_values(m_NodeName,m_serviceName,1,RegistryEntries);
                             }
-                            if( (serviceType != SERVICE_NO_CHANGE) || (startType != SERVICE_NO_CHANGE) )
+                            if( (serviceType != SERVICE_NO_CHANGE) || (startType != SERVICE_NO_CHANGE) || changeDBRoot )
                             {
                                 if (ChangeServiceConfig ( hS,
                                     serviceType,
