@@ -39,6 +39,7 @@
 
 #include "RunTime/RTE_MessageList.hpp"
 #include "RunTime/RTE_Message.hpp"
+#include "geo00_2.h"
 #  if defined(WIN32)
 // gos00.h
    typedef DWORD PID;
@@ -140,9 +141,9 @@ SAPDB_Bool CallRightProgram( char const *programToCall,
 static void Usage(SAPDB_Bool beVerbose)
 {
 #ifdef ALLOW_OWN_EXECUTABLE
-    printf("Usage: xstart DBName [-<speed>] [-x <executable>] [additional arguments for executable]\n");
+    printf("Usage: x_start DBName [-<speed>] [-x <executable>] [additional arguments for executable]\n");
 #else
-    printf("Usage: xstart DBName [-<speed>] [additional arguments for executable]\n");
+    printf("Usage: x_start DBName [-<speed>] [additional arguments for executable]\n");
 #endif
     if ( beVerbose )
     {
@@ -157,11 +158,11 @@ static void Usage(SAPDB_Bool beVerbose)
         printf("  -i or -admin allows to suppress automatic restart into online mode\n");
         printf("  -l allows to start kernel in lower priority mode\n");
         printf("  -D, -v, -V or -verbose allows to get the startup output even if successfully started\n\n");
-        printf("Example: xstart SAPDB                            start with default speed (fast)\n");
-        printf("         xstart SAPDB -slow                      start with 'slow' speed\n");
-        printf("         xstart SAPDB -test -admin               start into 'admin' mode using 'test' speed\n");
+        printf("Example: x_start SAPDB                            start with default speed (fast)\n");
+        printf("         x_start SAPDB -slow                      start with 'slow' speed\n");
+        printf("         x_start SAPDB -test -admin               start into 'admin' mode using 'test' speed\n");
 #ifdef ALLOW_OWN_EXECUTABLE
-        printf("         xstart SAPDB -own -x kernel_try -admin  start using special executable for own speed\n");
+        printf("         x_start SAPDB -own -x kernel_try -admin  start using special executable for own speed\n");
 #endif
     }
 }
@@ -436,7 +437,7 @@ int main(int argc, char *argv[])
     }
 
 #ifndef WIN32
-    if ( RTE_SetSapdbOwnerAndGroupId(errList) )
+    if ( !RTE_SetSapdbOwnerAndGroupId(errList) )
     {
         RTE_Message(errList, MSG_DIAG_CONSOLE);
         return 1;
@@ -522,13 +523,33 @@ int main(int argc, char *argv[])
     }
     else
     {
-        printf("OK\n");
-
         if ( beVerbose )
         {
             RTE_Message(errList, MSG_DIAG_CONSOLE);
         }
 
+        const RTE_Database::DBStatusInfo* pStatus;
+        pStatus = dbInstance->GetDBStatus(errList);
+        if ( !pStatus )
+        {
+            RTE_Message(errList, MSG_DIAG_CONSOLE);
+            result = 1;
+        }
+        else
+        {
+            printf("OK\n");
+
+            if ( (adminMode 
+               && pStatus->statusId != SERVER_ADMIN_EO00)
+             ||  (!adminMode 
+               && pStatus->statusId != SERVER_ONLINE_EO00
+               && pStatus->statusId != SERVER_STANDBY_EO00) )
+            {
+                printf( "Warning: Expected database state was %s\n", adminMode ? "ADMIN" : "ONLINE");
+            }
+            printf("Database %s started to status %s\n", dbname, pStatus->statusName );
+        }
+
         result = 0;
     }
 
