@@ -44,13 +44,14 @@
 /*===========================================================================*
  *  INCLUDES                                                                 *
  *===========================================================================*/
-
 #include "RunTime/RTE_ISystem.hpp"
 #include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"
 #include "RunTime/System/RTESys_Spinlock.h"
 #include "RunTime/System/RTESys_SystemInfo.h"
 #include "SAPDBCommon/ErrorsAndMessages/SAPDBErr_Assertions.hpp"
 #include "RunTime/MemoryManagement/RTEMem_SystemPageCache.hpp"
+#include "RunTime/RTE_Message.hpp"
+#include "RunTime/RTE_Messages.hpp"
 #if defined(OSF1)
 #define _XOPEN_SOURCE_EXTENDED /* needed for usleep */
 #endif
@@ -132,6 +133,15 @@ private:
 
   RTE_SystemUNIX(SAPDB_ULong MemoryLimitInBytes);
 
+/*
+   Called in out of memory situation to show detailed memory usage information.
+   If memory statistics was already added with an identical number of successfull
+   allocation calls, false is returned 
+ */
+  void AddMemoryStatistic(SAPDBErr_MessageList &messageList);
+
+  void ShowAllocationFailedMessage(SAPDBErr_MessageList &messageList);
+
   SAPDB_ULong m_SystemPageSize;
 
   SAPDB_UInt4 m_NumberOfCPU;
@@ -370,75 +380,293 @@ void * RTE_SystemUNIX::AllocSystemPagesAtFixedAddress(void *FixedAddress, SAPDB_
 
 /*---------------------------------------------------------------------------*/
 
+void RTE_SystemUNIX::ShowAllocationFailedMessage(SAPDBErr_MessageList &messageList)
+{
+    static SAPDB_UInt8 lastSuccessfullAllocs = 0;
+
+    SAPDB_UInt8 successfullAllocs;
+
+    while ( RTE_ISystem::TestAndLock(m_MemoryUsageLock) )     
+    {
+        RTE_ISystem::GiveUpTimeSlice();
+    }
+
+    successfullAllocs = m_SuccessfullAllocCalls;
+
+    RTE_ISystem::Unlock(m_MemoryUsageLock);
+
+    if ( successfullAllocs > lastSuccessfullAllocs )
+    {
+        RTE_Message(messageList);
+    }
+}
+
+//-----
+
+void RTE_SystemUNIX::AddMemoryStatistic(SAPDBErr_MessageList &messageList)
+{
+    SAPDB_ULong MemoryLimitInBytes;
+    SAPDB_ULong MemoryCommitedInBytes;
+    SAPDB_ULong MemoryUsedInBytes;
+    SAPDB_ULong MemoryMaxUsedInBytes;
+    SAPDB_UInt8 MemorySuccessfullAllocs;
+    SAPDB_UInt8 MemoryFailedAllocs;
+    SAPDB_UInt8 MemoryCommits;
+    SAPDB_UInt8 MemoryFrees;
+
+    GetMemoryStatistics( MemoryLimitInBytes,
+                         MemoryCommitedInBytes,
+                         MemoryUsedInBytes,
+                         MemoryMaxUsedInBytes,
+                         MemorySuccessfullAllocs,
+                         MemoryFailedAllocs,
+                         MemoryCommits,
+                         MemoryFrees );
+
+    if ( MemoryLimitInBytes == RTE_ISystem::UnlimitedMemory )
+    {
+        SAPDBErr_MessageList info( RTE_CONTEXT,
+                                   RTEINFO_RTEMEM_SYSTEM_HEAP_MEMORY_STATISTICS,
+                                   SAPDB_ToString(MemoryUsedInBytes),
+                                   SAPDB_ToString("system imposed limitation") );
+        messageList.AppendNewMessage( info );
+
+        SAPDBErr_MessageList details(
+            RTE_CONTEXT,
+            RTEINFO_RTEMEM_SYSTEM_HEAP_MEMORY_DETAILS,
+            SAPDB_ToString(MemorySuccessfullAllocs),
+ 	        SAPDB_ToString(MemoryFrees),
+ 	        SAPDB_ToString(MemoryFailedAllocs),
+ 	        SAPDB_ToString(MemoryMaxUsedInBytes) );
+        messageList.AppendNewMessage( details );
+    }
+    else
+    {
+        SAPDBErr_MessageList info( RTE_CONTEXT,
+                                   RTEINFO_RTEMEM_SYSTEM_HEAP_MEMORY_STATISTICS,
+                                   SAPDB_ToString(MemoryUsedInBytes),
+                                   SAPDB_ToString(MemoryLimitInBytes) );
+        messageList.AppendNewMessage( info );
+
+        SAPDBErr_MessageList details(
+            RTE_CONTEXT,
+            RTEINFO_RTEMEM_SYSTEM_HEAP_MEMORY_DETAILS,
+            SAPDB_ToString(MemorySuccessfullAllocs),
+ 	        SAPDB_ToString(MemoryFrees),
+ 	        SAPDB_ToString(MemoryFailedAllocs),
+ 	        SAPDB_ToString(MemoryMaxUsedInBytes) );
+        messageList.AppendNewMessage( details );
+    }
+
+#ifdef USE_FIXED_MMAP_ADDRESS
+    void *mmapAddress;
+
+    while ( RTE_ISystem::TestAndLock(m_nextMmapAddressLock) )
+    {
+        RTE_ISystem::GiveUpTimeSlice();
+    }
+
+    mmapAddress = m_nextMmapAddress;
+
+    RTE_ISystem::Unlock(m_nextMmapAddressLock);
+
+    {
+        SAPDBErr_MessageList info( RTE_CONTEXT,
+                           RTEINFO_RTEMEM_NEXT_RESERVED_ADDRESS,
+                           SAPDB_ToString(mmapAddress) );
+        messageList.AppendNewMessage( info );
+    }
+#endif /* USE_FIXED_MMAP_ADDRESS */
+
+}
+
+/*---------------------------------------------------------------------------*/
+
 void * RTE_SystemUNIX::UncachedAllocSystemPagesAtFixedAddress(void *FixedAddress, SAPDB_ULong SizeInBytes, bool DoCommit)
 {
     // Round up to Systempage size
-  SAPDB_ULong SystemSize = (SizeInBytes + GetSystemPageSize() - 1) & (~(GetSystemPageSize()-1));
-  void *ChunkAddress = 0;
-  bool  DoAllocate = IncrementUsedMemory(SystemSize);
+    SAPDB_ULong SystemSize = (SizeInBytes + GetSystemPageSize() - 1) & (~(GetSystemPageSize()-1));
+    void *ChunkAddress = 0;
+    bool  DoAllocate = IncrementUsedMemory(SystemSize);
 
+    if ( DoAllocate )
+    {
 #ifdef USE_VMALLOC_NOT_MMAP
 
-  ChunkAddress = valloc(SystemSize);
-  if ( !ChunkAddress )
-  {
-      DecrementUsedMemory(SystemSize);
-  }
-
+#ifdef HAS_NPTL
+        if ( posix_memalign(&ChunkAddress, GetSystemPageSize(), SizeInBytes) )
+        {
+            ChunkAddress = 0;
+        }
 #else
+        ChunkAddress = valloc(SystemSize);
+#endif
+        if ( !ChunkAddress )
+        {
+            SAPDB_Char buffer[256];
+            SAPDB_Char const *errorText;
+            switch( errno )
+            {
+            case EINVAL:
+                errorText = "EINVAL ? bad alignment";
+                break;
+            case ENOMEM:
+                errorText = "ENOMEM ? out of memory";
+                break;
+            default:
+                sprintf(buffer, "Unexpected error code %d", errno);
+                errorText = buffer;
+                break;
+            }
+            DecrementUsedMemory(SystemSize);
 
-  SAPDB_Char *mmapAddress = 0; // let system choose
+            SAPDBErr_MessageList messageList( RTE_CONTEXT,
+                                RTEERR_RTEMEM_NO_MORE_SYSTEM_PAGES,
+	                            SAPDB_ToString(SizeInBytes),
+	                            SAPDB_ToString(GetSystemPageSize()),
+#ifdef HAS_NPTL
+	                            SAPDB_ToString("posix_memalign"),
+#else
+	                            SAPDB_ToString("valloc"),
+#endif
+                                SAPDB_ToString(errorText) );
 
-  if ( FixedAddress != 0 )
-  {
-      mmapAddress = (SAPDB_Char *)FixedAddress;
-  }
-  else
-  {
-      mmapAddress = (SAPDB_Char *)ReserveSystemPages(SizeInBytes);
-  }
+            
+            AddMemoryStatistic(messageList);
+
+            // only output if meanwhile an allocation succeeded...
+            ShowAllocationFailedMessage( messageList );
+        }
+
+#else /* not USE_VMALLOC_NOT_MMAP */
+
+        SAPDB_Char *mmapAddress = 0; // let system choose
+
+        if ( FixedAddress != 0 )
+        {
+            mmapAddress = (SAPDB_Char *)FixedAddress;
+        }
+        else
+        {
+            mmapAddress = (SAPDB_Char *)ReserveSystemPages(SizeInBytes);
+        }
 
-  if ( DoAllocate )
-  {
 #if defined(SUN)
-      ChunkAddress = (void *)mmap( (char *)mmapAddress, SystemSize, PROT_READ|PROT_WRITE,
+        ChunkAddress = (void *)mmap( (char *)mmapAddress, SystemSize, PROT_READ|PROT_WRITE,
           mmapAddress != 0 ? MAP_FIXED|MAP_PRIVATE|MAP_ANONYMOUS : MAP_PRIVATE|MAP_ANONYMOUS, -1, 0 );
 #elif !defined(AIX)
-      ChunkAddress = (void *)mmap( mmapAddress, SystemSize, PROT_READ|PROT_WRITE,
+        ChunkAddress = (void *)mmap( mmapAddress, SystemSize, PROT_READ|PROT_WRITE,
           mmapAddress != 0 ? MAP_FIXED|MAP_PRIVATE|MAP_ANONYMOUS : MAP_PRIVATE|MAP_ANONYMOUS, -1, 0 );
 #else
-      /* AIX is very special... Two physical pages are allocated if MAP_PRIVATE|MAP_ANONYMOUS is used... */
-      ChunkAddress = (void *)mmap( (void *)0, SystemSize, PROT_READ|PROT_WRITE, MAP_SHARED|MAP_ANONYMOUS, -1, 0 );
+        /* AIX is very special... Two physical pages are allocated if MAP_PRIVATE|MAP_ANONYMOUS is used... */
+        ChunkAddress = (void *)mmap( (void *)0, SystemSize, PROT_READ|PROT_WRITE, MAP_SHARED|MAP_ANONYMOUS, -1, 0 );
 #endif
 
-      if ( ChunkAddress == (void *) MAP_FAILED )
-      {
-          DecrementUsedMemory(SystemSize);
-          ChunkAddress = (void *)0;
-      }
-  }
+        if ( ChunkAddress == (void *) MAP_FAILED )
+        {
+              SAPDB_Char buffer[256];
+              SAPDB_Char const *errorText;
+              switch( errno )
+              {
+              case EINVAL:
+                errorText = "EINVAL ? value too large";
+                break;
+              case EAGAIN:
+                errorText = "EAGAIN ? too much memory locked";
+                break;
+              case ENOMEM:
+                errorText = "ENOMEM ? out of memory";
+                break;
+              default:
+                sprintf(buffer, "Unexpected error code %d", errno);
+                errorText = buffer;
+                break;
+              }
+              DecrementUsedMemory(SystemSize);
+
+              SAPDBErr_MessageList messageList( RTE_CONTEXT,
+                                    RTEERR_RTEMEM_NO_MORE_SYSTEM_PAGES,
+	                                SAPDB_ToString(SizeInBytes),
+	                                SAPDB_ToString(GetSystemPageSize()),
+	                                SAPDB_ToString("mmap"),
+	                                SAPDB_ToString(errorText) );
+              AddMemoryStatistic(messageList);
+
+              // only output if meanwhile an allocation succeeded...
+              ShowAllocationFailedMessage( messageList );
+
+              ChunkAddress = (void *)0;
+        }
+
+        if ( 0 != ChunkAddress 
+          && 0 != FixedAddress 
+          && FixedAddress != ChunkAddress )
+        {
+            SAPDBErr_MessageList messageList( RTE_CONTEXT,
+                                              RTEERR_RTEMEM_OUT_OF_RESERVABLE_SYSTEM_PAGES,
+                                              SAPDB_ToString(SizeInBytes),
+                                              SAPDB_ToString(FixedAddress) );
+
+#ifdef USE_FIXED_MMAP_ADDRESS
+            void *mmapAddress;
+
+            while ( RTE_ISystem::TestAndLock(m_nextMmapAddressLock) )
+            {
+                RTE_ISystem::GiveUpTimeSlice();
+            }
+
+            mmapAddress = m_nextMmapAddress;
+
+            RTE_ISystem::Unlock(m_nextMmapAddressLock);
+
+            SAPDBErr_MessageList info( RTE_CONTEXT,
+                               RTEINFO_RTEMEM_NEXT_RESERVED_ADDRESS,
+                               SAPDB_ToString(mmapAddress) );
+            messageList.AppendNewMessage( info );
+#endif
+            // only output if meanwhile an allocation succeeded...
+            ShowAllocationFailedMessage( messageList );
+
+            // Only wanted address is acceptable...
+            UncachedFreeSystemPages(ChunkAddress, SizeInBytes);
+
+            ChunkAddress = 0;
+        }
 
 #endif /* USE_VMALLOC_NOT_MMAP */
+    }
+    else
+    {
+        SAPDB_ULong MemoryLimitInBytes;
 
-  if ( 0 != ChunkAddress 
-    && 0 != FixedAddress 
-    && FixedAddress != ChunkAddress )
-  {
-      // Only wanted address is acceptable...
-    UncachedFreeSystemPages(ChunkAddress, SizeInBytes);
-    ChunkAddress = 0;
-  }
+        while ( RTE_ISystem::TestAndLock(m_MemoryUsageLock) )
+        {
+            RTE_ISystem::GiveUpTimeSlice();
+        }
 
-  if ( ChunkAddress == 0 )
-  {
-    IncrementCallCounter(m_FailedAllocCalls);
-  } 
-  else
-  {
-    IncrementCallCounter(m_SuccessfullAllocCalls);
-  }
+        MemoryLimitInBytes = m_MemoryLimitInBytes;
+ 
+        RTE_ISystem::Unlock(m_MemoryUsageLock);
 
-  return ( ChunkAddress );
+        SAPDBErr_MessageList messageList( RTE_CONTEXT,
+                            RTEERR_RTEMEM_MEMORY_LIMIT_REACHED,
+	                        SAPDB_ToString(MemoryLimitInBytes),
+	                        SAPDB_ToString("Systemheap"),
+	                        SAPDB_ToString(SizeInBytes) );
+        // only output if meanwhile an allocation succeeded...
+        ShowAllocationFailedMessage( messageList );
+    }
+
+    if ( ChunkAddress == 0 )
+    {
+        IncrementCallCounter(m_FailedAllocCalls);
+    } 
+    else
+    {
+        IncrementCallCounter(m_SuccessfullAllocCalls);
+    }
+
+    return ( ChunkAddress );
 }
 
 /*---------------------------------------------------------------------------*/
