@@ -15,6 +15,7 @@
   -------------------------------------------------------------------------
 
 
+  copyright:    Copyright (c) 2000-2004 SAP AG
 
 
     ========== licence begin  GPL
@@ -43,11 +44,13 @@
 #include "RunTime/MemoryManagement/RTEMem_RteAllocator.hpp"
 #include "RunTime/RTE_Crash.hpp"
 #include "RunTime/System/RTESys_AtomicOperation.hpp"
+#include "RunTime/System/RTESys_MemoryBarrier.h"
 #include "RunTime/System/RTESys_MemProtect.h"
 #include "geo007_1.h" /* MSGD content... */
 #include "geo50_0.h" /* kgs for XPARAM */
 #include "geo002.h" /* XPARAM(showMaxStackUse/fUseProcessScope) */
 #include "geo70k.h" /* NUM_GUARDPAGES_EO70 */
+#include "heo07.h"
 /*
    local structure describing task chunk parameters
  */
@@ -69,6 +72,11 @@ protected:
 public:
     static RTE_TaskStackControl &Instance();
 
+    /*!
+      @brief Once called function to initialize task stack control
+      @return specific type of task stack control created
+     */
+    virtual RTE_ITaskStackControl::TypeOfStackControl const GetTypeOfStackControl() const { return m_TypeOfStackControl; }
     /*!
       @brief Once called function to initialize task stack control
       @param numberOfTasks [in] number of tasks
@@ -181,6 +189,7 @@ MSGD(( ERR_TEST_WILDCARD, eBuffer ));
         TaskStackMapping *pLastFirstMapping = m_FirstMapping;
         do {
             newMapping.m_Next = pLastFirstMapping;
+    		RTESys_WriteMemoryBarrier();
         } while ( !RTESys_CompareAndExchange( m_FirstMapping, 
 #ifdef OSF1
                                               pLastFirstMapping,
@@ -231,7 +240,7 @@ protected:
     void TryToGuard(SAPDB_Byte *startAddress, SAPDB_Long guardSize);
 
 protected:
-    RTE_TaskStackControl();
+    RTE_TaskStackControl(RTE_ITaskStackControl::TypeOfStackControl const typeOfStackControl);
 
     void ReserveOrAllocateBigChunk(SAPDB_Bool tryReserve);
 
@@ -252,6 +261,7 @@ protected:
     SAPDB_Bool  m_useReservedAreaAndGaps;   /* Flag to show mprotect calls are not needed */
 private:
     TaskStackMapping            *m_FirstMapping; /* first mapping */
+    RTE_ITaskStackControl::TypeOfStackControl const m_TypeOfStackControl; /* type of derived class */
 public:
     static RTE_ITaskStackControl *m_Instance;
 }; //  class RTE_TaskStackControl
@@ -365,19 +375,19 @@ private:
 class RTE_StackAddressSpaceTaskStackControl: public RTE_TaskStackControl
 {
 public:
-    RTE_StackAddressSpaceTaskStackControl()
-    {}
+    RTE_StackAddressSpaceTaskStackControl();
 
     virtual void Initialize( SAPDB_Long,
                              struct TASK_TYPE *,
                              struct TASK_TYPE * )
     {}
-}; // class RTE_LimitedAddressSpaceTaskStackControl: public RTE_TaskStackControl
+}; // class RTE_StackAddressSpaceTaskStackControl: public RTE_TaskStackControl
 
+//------------------
 
 RTE_ITaskStackControl & RTE_ITaskStackControl::Instance()
 {
-    if ( XPARAM(fUseCoroutines) && XPARAM(fUseStackOnStack) )
+    if ( KGS && XPARAM(fUseCoroutines) && XPARAM(fUseStackOnStack) )
     {
         SAPDBMEM_STATIC_RAW_ALLOCATE(Space, sizeof(RTE_StackAddressSpaceTaskStackControl));
         if ( !RTE_TaskStackControl::m_Instance )
@@ -411,7 +421,11 @@ RTE_ITaskStackControl & RTE_ITaskStackControl::Instance()
     return *RTE_TaskStackControl::m_Instance;
 }
 
-RTE_TaskStackControl::RTE_TaskStackControl()
+//-----------------------------------------------------------------------------
+//------------------ Contructors for singleton --------------------------------
+//-----------------------------------------------------------------------------
+
+RTE_TaskStackControl::RTE_TaskStackControl(RTE_ITaskStackControl::TypeOfStackControl const typeOfStackControl)
 : m_StartAddress(0),
   m_StartOfFirstTaskStack(0),
   m_TotalSize(0),
@@ -419,21 +433,38 @@ RTE_TaskStackControl::RTE_TaskStackControl()
   m_SingleGuardSize(0),
   m_SingleProtectedStackSize(0),
   m_NumberOfTasks(0),
+  m_maxTaskStackSize(0),
   m_useReservedAreaAndGaps(false),
-  m_FirstMapping(0)
+  m_FirstMapping(0),
+  m_TypeOfStackControl(typeOfStackControl)
 {}
 
-RTE_LimitedMmapSectionsTaskStackControl::RTE_LimitedMmapSectionsTaskStackControl()
+//--
+
+RTE_StackAddressSpaceTaskStackControl::RTE_StackAddressSpaceTaskStackControl()
+: RTE_TaskStackControl( RTE_ITaskStackControl::StackAddressSpace )
 {}
 
+//--
+
 RTE_LimitedAddressSpaceTaskStackControl::RTE_LimitedAddressSpaceTaskStackControl()
 : m_StartOfFirstUserTaskStack(0),
   m_SingleAlignedNonUserStackSize(0),
   m_SingleProtectedNonUserStackSize(0),
-  m_NumberOfNonUserTasks(0)
+  m_NumberOfNonUserTasks(0),
+  RTE_TaskStackControl(RTE_ITaskStackControl::LimitedAddressSpace)
 {}
 
+//--
+
+RTE_LimitedMmapSectionsTaskStackControl::RTE_LimitedMmapSectionsTaskStackControl()
+: RTE_TaskStackControl(RTE_ITaskStackControl::LimitedMmapSections)
+{}
+
+//--
+
 RTE_BigAddressSpaceTaskStackControl::RTE_BigAddressSpaceTaskStackControl()
+: RTE_TaskStackControl(RTE_ITaskStackControl::BigAddressSpace)
 {}
 
 //-----------------------------------------------------------------------------
@@ -455,9 +486,12 @@ void RTE_TaskStackControl::CalculateMaxTaskStack(
         }
     }
 
-    if ( m_maxTaskStackSize < sqlget_thread_min_stack() )
+    if ( !XPARAM(fUseCoroutines) )
     {
-        m_maxTaskStackSize = sqlget_thread_min_stack();
+        if ( m_maxTaskStackSize < sqlget_thread_min_stack() )
+        {
+            m_maxTaskStackSize = sqlget_thread_min_stack();
+        }
     }
 
     MSGD (( INFO_MAX_TASK_STACK, m_maxTaskStackSize/1024 ));
@@ -470,10 +504,13 @@ void RTE_TaskStackControl::CalculateMaxTaskStack(
 void RTE_TaskStackControl::TryToGuard(SAPDB_Byte *startAddress, SAPDB_Long guardSize)
 {
 #if !defined(USE_VMALLOC_NOT_MMAP) && defined(USE_FIXED_MMAP_ADDRESS)
-    if ( !RTE_ISystem::Instance().UncachedAllocSystemPagesAtFixedAddress(startAddress, guardSize, true) )
+    if ( RTE_ITaskStackControl::StackAddressSpace != m_TypeOfStackControl )
     {
-        MSGD(( ERR_REALLOCATING_TASK_STACK, startAddress, startAddress + guardSize - 1 ));
-        RTE_Crash(SAPDBErr_MessageList());
+        if ( !RTE_ISystem::Instance().UncachedAllocSystemPagesAtFixedAddress(startAddress, guardSize, true) )
+        {
+            MSGD(( ERR_REALLOCATING_TASK_STACK, startAddress, startAddress + guardSize - 1 ));
+            RTE_Crash(SAPDBErr_MessageList());
+        }
     }
 #endif /* !USE_VMALLOC_NOT_MMAP && USE_FIXED_MMAP_ADDRESS */
 
@@ -550,7 +587,14 @@ void RTE_LimitedMmapSectionsTaskStackControl::UpdateTaskControlRecords(
     for ( struct TASK_TYPE * tcb = pFirst; tcb <= pLast; ++tcb )
     {
         SAPDB_Int4 alignedStackSize = RTE_ISystem::Instance().RoundUpToSystemPageSize(tcb->stack_size);
- 
+        if ( !XPARAM(fUseCoroutines) )
+        {
+            if ( alignedStackSize < sqlget_thread_min_stack() )
+            {
+                alignedStackSize = RTE_ISystem::Instance().RoundUpToSystemPageSize(sqlget_thread_min_stack());
+            }
+        }
+
         tcb->StackChunkStart = stackStartAddress;
         tcb->StackChunkEnd   = stackStartAddress + alignedStackSize;
         tcb->stack_size      = alignedStackSize;
@@ -597,7 +641,14 @@ void RTE_BigAddressSpaceTaskStackControl::AllocateAndProtectTaskStacks(
     for ( struct TASK_TYPE *tcb = pFirst; tcb <= pLast; ++tcb )
     {
         SAPDB_Int4 alignedStackSize = RTE_ISystem::Instance().RoundUpToSystemPageSize(tcb->stack_size);
- 
+        if ( !XPARAM(fUseCoroutines) )
+        {
+            if ( alignedStackSize < sqlget_thread_min_stack() )
+            {
+                alignedStackSize = RTE_ISystem::Instance().RoundUpToSystemPageSize(sqlget_thread_min_stack());
+            }
+        }
+
         tcb->StackChunkStart = stackStartAddress;
         tcb->StackChunkEnd   = stackStartAddress + alignedStackSize;
         tcb->stack_size      = alignedStackSize;
@@ -690,7 +741,14 @@ void RTE_LimitedAddressSpaceTaskStackControl::AllocateAndProtectTaskStacks(
     for ( struct TASK_TYPE *tcb = pFirst; tcb <= pLast; ++tcb )
     {
         SAPDB_Int4 alignedStackSize = RTE_ISystem::Instance().RoundUpToSystemPageSize(tcb->stack_size);
- 
+        if ( !XPARAM(fUseCoroutines) )
+        {
+            if ( alignedStackSize < sqlget_thread_min_stack() )
+            {
+                alignedStackSize = RTE_ISystem::Instance().RoundUpToSystemPageSize(sqlget_thread_min_stack());
+            }
+        }
+
         tcb->StackChunkStart = stackStartAddress;
         tcb->StackChunkEnd   = stackStartAddress + alignedStackSize;
         tcb->stack_size      = alignedStackSize;
@@ -802,7 +860,7 @@ SAPDB_Long RTE_BigAddressSpaceTaskStackControl::TryToGuardStartPages(
     }
 
     return m_SingleGuardSize * (1 + m_NumberOfTasks);
-} // RTE_TaskStackControl::TryToGuardStartPages
+} // RTE_BigAddressSpaceTaskStackControl::TryToGuardStartPages
 
 // -------------------------------------------------------------
 
@@ -833,7 +891,7 @@ SAPDB_Long RTE_LimitedAddressSpaceTaskStackControl::TryToGuardStartPages(
     }
 
     return m_SingleGuardSize * (1 + m_NumberOfTasks);
-} // RTE_TaskStackControl::TryToGuardStartPages
+} // RTE_LimitedAddressSpaceTaskStackControl::TryToGuardStartPages
 
 // -----------------------------------------------------
 
@@ -860,15 +918,18 @@ SAPDB_ULong RTE_LimitedAddressSpaceTaskStackControl::CalculateMaxNonUserTaskStac
         }
     }
 
-    if ( maxNonUserTaskStackSize < sqlget_thread_min_stack() )
+    if ( !XPARAM(fUseCoroutines) )
     {
-        maxNonUserTaskStackSize = sqlget_thread_min_stack();
+        if ( maxNonUserTaskStackSize < sqlget_thread_min_stack() )
+        {
+            maxNonUserTaskStackSize = sqlget_thread_min_stack();
+        }
     }
 
     MSGD (( INFO_MAX_NON_USER_TASK_STACK, maxNonUserTaskStackSize/1024 ));
 
     return maxNonUserTaskStackSize;
-} // RTE_LimitedMmapSectionsTaskStackControl::CalculateMaxNonUserTaskStack()
+} // RTE_LimitedAddressSpaceTaskStackControl::CalculateMaxNonUserTaskStack()
 
 // -----------------------------------------------------
 // RTE_TaskStackControl::Initialize implementations
