@@ -51,7 +51,19 @@
 #include "SAPDBCommon/SAPDB_Types.h"
 
 #if defined (_WIN32) && !defined (NO_WINDOWSH)
+
+#define NEEDED_WIN32_WINNT 0x0500
+
+#if !defined(_WIN32_WINNT)
+# define _WIN32_WINNT  NEEDED_WIN32_WINNT
+#else
+# if _WIN32_WINNT < NEEDED_WIN32_WINNT
+#  undef _WIN32_WINNT
+#  define _WIN32_WINNT  NEEDED_WIN32_WINNT
+# endif
+#endif
 #include <windows.h>
+#include <tchar.h>
 #else
 #include <unistd.h>
 #include <sys/types.h>
@@ -64,7 +76,7 @@
 # include  <sys/atomic_op.h>
 #endif /*AIX*/
 
-#if defined(S390X)
+#if defined(S390X) 
 # include <asm/spinlock.h>
 #endif /* S390X */
 
@@ -124,7 +136,7 @@
 # undef USE_VMALLOC_NOT_MMAP
 
 /*---------------------- vmalloc() using platforms ------------*/
-# if (defined(LINUX) && !defined(BIT64)) || defined(AIX)
+# if (defined(LINUX) && (defined(I386) || defined(X86_64) || defined(SDBonPPC64))) || defined(AIX)
 /*! For platforms that do not support mmap sufficiently */
 #  define USE_VMALLOC_NOT_MMAP
 
@@ -238,6 +250,38 @@ typedef SAPDB_UInt4  RTE_TLSKey;             /* Thread Local Storage Key
 #endif
 #endif
 
+/*!
+  Description: Command line argument string type
+  
+               Its the type of the command line argument string (argv) as passed to main().
+               On Windows plattforms we might using _tmain() to pass unicode character string
+               to the program.
+*/
+#if defined (_WIN32)
+  typedef _TCHAR      RTE_ArgChar;    // on Windows plattforms we might using _tmain()
+#else
+  typedef char        RTE_ArgChar;
+#endif
+
+
+/*!
+  Common Runtime Process ID
+*/
+#if defined(_WIN32)
+ typedef DWORD           RTE_OSPid;
+#else
+#ifdef LINUX
+ /* special case... pid_t must not be used in typedef */
+ typedef int             RTE_OSPid;
+#else
+ typedef pid_t           RTE_OSPid;
+#endif
+#endif
+
+/*! used to specify undefined process id */
+#define RTE_UNDEF_OSPID        ((RTE_OSPid)-1)
+
+
 /*!--------------------------------------------------------------------------------------------
   Declaration: RTE_Lock
   Description: Memory object needed as core of a SpinLock
@@ -279,6 +323,7 @@ typedef SAPDB_UInt4  RTE_TLSKey;             /* Thread Local Storage Key
   typedef spinlock_t RTE_Lock;
 #endif
 
+
 /*
   End of this ugly RTE_Lock...
  *-------------------------------------------------------------------------------------------*/
@@ -462,6 +507,4 @@ typedef enum
 RTE_StartOption;
 
 
-
-
 #endif  /* RTE_TYPES_H */
