@@ -37,6 +37,7 @@
 #include "heo01.h"
 #include "SAPDBCommon/SAPDB_sprintf.h"
 #include "geo28.h"
+#include "RunTime/Synchronisation/RTESync_Spinlock.hpp"
 
 #include <unistd.h>
 /* this .... is needed due to corrupted header file grp.h on tru64... */
@@ -64,6 +65,11 @@
 
 /*--------------------*/
 
+static RTESync_Spinlock lock;   // to get user/group id by name, the system calls getpwnam()
+                                // and getgrnam() are used here. They are not reentrant, so
+                                // to avoid race conditions, these routines are called in a
+                                // locked scope. This is the spinlock used for the locked scope.
+                                // PTS 1129679
 SAPDB_Bool
 RTE_GetUserIdFromUsername(SAPDB_Char const *Username,
                           SAPDB_Int4       &userId,
@@ -71,6 +77,7 @@ RTE_GetUserIdFromUsername(SAPDB_Char const *Username,
 {
     if ( Username )
     {
+        RTESync_LockedScope scope(lock);
         struct passwd *pwEntry = getpwnam (Username);
 
         if ( ! pwEntry )
@@ -103,6 +110,7 @@ RTE_GetUserAndPrimaryGroupIdFromUsername( SAPDB_Char const     *Username,
 {
     if ( Username )
     {
+        RTESync_LockedScope scope(lock);
         struct passwd *pwEntry = getpwnam (Username);
 
         if ( ! pwEntry )
@@ -169,6 +177,7 @@ RTE_GetGroupIdFromGroupname( SAPDB_Char const *Groupname,
 {
     if ( Groupname )
     {
+        RTESync_LockedScope scope(lock);
         struct group *grEntry = getgrnam (Groupname);
 
         if ( ! grEntry )
@@ -870,8 +879,10 @@ RTE_VerifyUserAsSapdbAdministrator(
     }
 
     struct group *grEntry;
-
-    grEntry = getgrnam(GroupnameBuffer);
+    {
+        RTESync_LockedScope scope(lock);
+        grEntry = getgrnam(GroupnameBuffer);
+    }
     if ( !grEntry )
     {
         errList = SAPDBErr_MessageList(RTE_CONTEXT,
@@ -947,8 +958,10 @@ RTE_VerifyUserIdGroupIdCombinationAsSapdbAdministrator(
     }
 
     struct group *grEntry;
-
-    grEntry = getgrnam(GroupnameBuffer);
+    {
+        RTESync_LockedScope scope(lock);
+        grEntry = getgrnam(GroupnameBuffer);
+    }
     if ( !grEntry )
     {
         errList = SAPDBErr_MessageList(RTE_CONTEXT,
