@@ -47,6 +47,7 @@
 #include "SAPDBCommon/Tracing/SAPDBTrace_Usage.hpp"
 #include "SAPDBCommon/Tracing/SAPDBTrace_Topic.hpp"
 #include "RunTime/MemoryManagement/RTEMem_RteAllocator.hpp"
+#include "RunTime/Synchronisation/RTESync_SpinlockPoolRegister.hpp"
 
 #define USE_KGS
 #include "geo50_0.h"    // for things like KGS, TASK_CTRL...
@@ -73,6 +74,125 @@ extern SAPDBTrace_Topic RWRegion_Trace;
  *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
  *===========================================================================*/
 
+class RTESync_RWRegion;
+
+class RTESync_RWRegionRegister : public RTE_ItemRegister<RTESync_RWRegion>
+{
+public:
+    struct StatisticInfo
+    {
+        SAPDB_UTF8  PoolName[40+1];
+        SAPDB_ULong Address;
+        SAPDB_UInt4 CollisionCount;
+        SAPDB_UInt4 WaitCount;
+        SAPDB_ULong TasCount;
+        SAPDB_UInt4 ExclusiveAccessCount;
+        SAPDB_UInt4 NonExclusiveAccessCount;
+        RTE_TaskId  TaskId;
+        SAPDB_Int4  CurrentAccesses;
+    };
+
+    /*!
+        Function: Instance
+
+        description: Return reference to single instance of RTESync_SpinlockRegister
+
+        The InitializeInstance function must have been called before.
+
+        Return value: Reference to singleton instance
+     */
+    static RTESync_RWRegionRegister & Instance( )
+    {
+        SAPDBMEM_STATIC_RAW_ALLOCATE(Space, sizeof(RTESync_RWRegionRegister));
+
+        if ( !m_Instance )
+        {
+#if defined (SAPDB_SLOW)
+            m_Instance = new ( Space ) RTESync_RWRegionRegister(true);
+#else
+            m_Instance = new ( Space ) RTESync_RWRegionRegister(false);
+#endif
+        }
+
+        return *m_Instance;
+    };
+
+    /*!
+       Function: GetStatisticInfo
+       Description: Get a snapshot of all statistic informations
+       Arguments: pInfoArray[out] Pointer to array of StatisticInfo
+
+       Return value: True if info array fill, false if not not enough memory
+     */
+    SAPDB_Bool GetStatisticInfo(StatisticInfo * &pInfoArray, SAPDB_Int4 &itemCount,RTESync_IRWRegionReader::Mode mode,RTESync_IRWRegionReader::Selection  selection)
+    {
+        SAPDB_Int4 currentCount;
+        StatisticInfo *pSnapshotInfoArray = 0;
+
+        while ( 0 == pSnapshotInfoArray )
+        {
+            currentCount = m_Count; // dirty read... this leads to the necessity of 'while()' loop. See below
+            if ( 0 == currentCount )
+            {
+                return false;
+            }
+
+            pSnapshotInfoArray = reinterpret_cast<StatisticInfo *>(RTEMem_RteAllocator::Instance().Allocate(currentCount * sizeof(StatisticInfo)));
+            if ( 0 == pSnapshotInfoArray )
+            {
+                return false;
+            }
+
+            memset( pSnapshotInfoArray, 0, currentCount * sizeof(StatisticInfo));
+
+            /* This is using a locked scope... */
+            if ( !FillStatisticInfo(pSnapshotInfoArray, currentCount, itemCount,mode,selection) )
+            {
+                /* FillInfoArray() only reason to fail, is that the most unlikely case happened and */
+                /* in the meantime more allocators were registered. This is handled with forcing a loop here... */
+                RTEMem_RteAllocator::Instance().Deallocate(pSnapshotInfoArray);
+                pSnapshotInfoArray = 0;
+            }
+        }
+
+        pInfoArray = pSnapshotInfoArray;
+
+        return true;
+    };
+
+    /*!
+       Function: FreeStatisticInfo
+       Description: Free the snapshot of all statistic informations collected by GetStatisticInfo
+
+       Arguments: pInfoArray[in] Pointer to array of StatisticInfo
+     */
+    void FreeStatisticInfo(StatisticInfo *pInfoArray)
+    {
+        RTEMem_RteAllocator::Instance().Deallocate(pInfoArray);
+    };
+
+    SAPDB_UInt4 GetCount(void)
+    {
+        return m_Count;
+    }
+private:
+    RTESync_RWRegionRegister(SAPDB_Bool makeBackupCopy):RTE_ItemRegister<RTESync_RWRegion>(makeBackupCopy) {}
+
+    /*!
+       Function: FillStatisticInfo
+       Description: Fill the statistic informations
+       
+       Called by GetStatisticInfo, to call at least memory allocation outside of locked registration code
+
+       Arguments: pInfoArray[in] Pointer to array of StatisticInfo
+     */
+    SAPDB_Bool FillStatisticInfo(StatisticInfo * &pInfoArray, SAPDB_Int4 maxCount, SAPDB_Int4 &itemCount,RTESync_IRWRegionReader::Mode mode,RTESync_IRWRegionReader::Selection  selection);
+
+    static RTESync_RWRegionRegister *m_Instance;
+};
+
+RTESync_RWRegionRegister *RTESync_RWRegionRegister::m_Instance=NULL;
+
 /*!
    @class RTESync_RWRegion
    @brief class containing methods common to all flavours of RWRegions
@@ -92,6 +212,7 @@ public:
      */
     RTESync_RWRegion(SAPDB_Int8 id,RTESync_BaseSpinlockPool &pool,SAPDB_Bool enterCollisionCounterInTcb=false);
 
+    ~RTESync_RWRegion();
     /*!
       @brief
 
@@ -149,17 +270,42 @@ public:
         return m_lastEntered;
     };
 
+    virtual SAPDB_UInt4 GetCollisionCounter(void)
+    {
+        return m_collisionCounter;
+    }
+
+    virtual SAPDB_UInt4 GetExclusiveAccessCounter(void)
+    {
+        return m_exclusiveAccessCounter;
+    }
+
+    virtual SAPDB_UInt4 GetNonExclusiveAccessCounter(void)
+    {
+        return m_nonExclusiveAccessCounter;
+    }
+
+    virtual SAPDB_UInt4 GetWaitCounter(void)
+    {
+        return m_waitCounter;
+    }
+
+    virtual SAPDB_UInt4 GetTasCounter(void)
+    {
+        return m_tasCounter;
+    }
+
 protected:
     SAPDB_Int8      m_id;
     RTE_TaskId      m_lastEntered;
     volatile int    m_nonExclusive;
     SAPDB_UInt      m_exclusiveRequests;
-
+    SAPDB_ULong     m_tasCounter;
+    SAPDB_UInt4     m_waitCounter;
+/*
     SAPDB_ULong     m_attemptsToGetLockOnEnter;
     SAPDB_ULong     m_attemptsToGetLockOnLeave;
-
-    SAPDB_Long      m_collisionCounter;
-
+*/
     void get_lock(RTE_TCB &tcb,RTE_UKT &ukt, SAPDB_Bool busyWaiting,SAPDB_ULong &totalTasCnt);
 
     void free_lock();
@@ -168,6 +314,7 @@ private:
     RTESync_BaseSpinlockPool  & m_pool;
     SAPDB_Int4                  m_poolLockIndex;
     SAPDB_Bool                  m_enterCollisionCounterInTcb;   // only for vbegexcl()/vendexcl() emulation
+    RTE_ItemRegister<RTESync_RWRegion>::Info    m_info;
 
     virtual void enterRegionExclusive(RTE_TCB tcb,RTE_UKT ukt,SAPDB_Long maxCollisions,SAPDB_Long &collisionCounter)=0;
 
@@ -184,6 +331,11 @@ private:
     SAPDB_Bool tryEnterRegionNonExclusiveNonBlocking(RTE_TCB tcb,RTE_UKT ukt);
 
     void check_indefinite_loop(SAPDB_ULong &tasCount,RTE_TCB tcb);
+
+    SAPDB_UInt4 m_exclusiveAccessCounter;
+    SAPDB_UInt4 m_nonExclusiveAccessCounter;
+    SAPDB_UInt4 m_collisionCounter;
+friend class RTESync_RWRegionRegister;
 };
 
 /*!
@@ -322,12 +474,33 @@ RTESync_RWRegion::RTESync_RWRegion(SAPDB_Int8 id,RTESync_BaseSpinlockPool &pool,
     , m_exclusiveRequests(0)
     , m_lastEntered(-1)
     , m_collisionCounter(0)
+    , m_waitCounter(0)
+    , m_tasCounter(0)
+    , m_exclusiveAccessCounter(0)
+    , m_nonExclusiveAccessCounter(0)
+#ifdef RESET_STATISTICS_SUPPORTED
+    , m_collisionCounterAtReset(0)
+    , m_waitCounterAtReset(0)
+    , m_tasCounterAtReset(0)
+    , m_exclusiveAccessCounterAtReset(0)
+    , m_nonExclusiveAccessCounterAtReset(0)
+#endif
     , m_enterCollisionCounterInTcb(enterCollisionCounterInTcb)
+    , m_info(NULL,NULL)
 {
     if(0 == id)
     {
         RTE_Crash(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_ID_ZERO,"","constructor"));
     }
+    new ( ((void *)&m_info) ) RTE_ItemRegister<RTESync_RWRegion>::Info ( (const SAPDB_UTF8 *)"", this );
+    RTESync_RWRegionRegister::Instance().Register(m_info);
+}
+
+/*---------------------------------------------------------------------------*/
+
+RTESync_RWRegion::~RTESync_RWRegion()
+{
+    RTESync_RWRegionRegister::Instance().Deregister(m_info);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -350,10 +523,11 @@ void RTESync_RWRegion::enter(SAPDB_Bool exclusive,RTE_TaskId pid)
     tcb.beginExclusiveCalls()++;
     SAPDB_Long maxCollisions = XPARAM(ulRegionCollisionLoop);
     SAPDB_Long collisionCounter = 0;
+    tcb.RWRegion() = this;
     tcb.inRegionID() = (RTE_RegionId)m_id;
     tcb.exclusiveNestingCount()++;
     tcb.beginExclusiveCount()++;
-    tcb.taskState() = TSK_VBEGEXCL;
+    tcb.taskState() = TSK_ENTERRWREGION_EO00;
     if(exclusive)
     {
         if(pid == m_lastEntered)
@@ -361,10 +535,12 @@ void RTESync_RWRegion::enter(SAPDB_Bool exclusive,RTE_TaskId pid)
             RTE_Crash(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_ALREADY_ENTERED,SAPDB_ToString(tcb.taskIndex()),SAPDB_ToString(m_id)));
         }
         enterRegionExclusive(tcb,ukt,maxCollisions,collisionCounter);
+        m_exclusiveAccessCounter++;
     }
     else
     {
         enterRegionNonExclusive(tcb,ukt,maxCollisions,collisionCounter);
+        m_nonExclusiveAccessCounter++;
     }
     m_collisionCounter += collisionCounter;
     if(m_enterCollisionCounterInTcb)
@@ -391,7 +567,7 @@ void RTESync_RWRegion::leave(SAPDB_Bool exclusive,RTE_TaskId pid)
 
     RTETask_getTCBandUKTCB( tcb, ukt, pid );
 
-    tcb.taskState() = TSK_VENDEXCL;
+    tcb.taskState() = TSK_LEAVERWREGION_EO00;
 
     if(exclusive)
     {
@@ -421,6 +597,7 @@ void RTESync_RWRegion::leave(SAPDB_Bool exclusive,RTE_TaskId pid)
     }
 
     tcb.inRegionID() = 0;
+    tcb.RWRegion() = NULL;
     tcb.taskState() = TSK_RUNNING;
 }
 
@@ -442,7 +619,7 @@ SAPDB_Bool RTESync_RWRegion::tryEnter(SAPDB_Bool exclusive,RTE_TaskId pid)
     RTETask_getTCBandUKTCB ( tcb, ukt, pid );
 
     tcb.inRegionID() = (RTE_RegionId)m_id;
-    tcb.taskState()   = TSK_VBEGEXCL;
+    tcb.taskState()   = TSK_ENTERRWREGION_EO00;
 
     SAPDB_Bool canEnter;
     if(exclusive)
@@ -482,8 +659,8 @@ void RTESync_RWRegion::SwitchToNonExclusiveAccess(RTE_TaskId pid)
 /*---------------------------------------------------------------------------*/
 SAPDB_Bool RTESync_RWRegion::tryEnterRegionExclusiveNonBlocking(RTE_TCB tcb,RTE_UKT ukt)
 {
-    get_lock(tcb,ukt,false,m_attemptsToGetLockOnEnter);
-    if(m_nonExclusive < 0)
+    get_lock(tcb,ukt,false,m_tasCounter);
+    if(m_nonExclusive != 0) // the region has been locked ("exclusive" or "non-exclusive") -> can't enter
     {
         free_lock();
         return false;
@@ -502,8 +679,8 @@ SAPDB_Bool RTESync_RWRegion::tryEnterRegionExclusiveNonBlocking(RTE_TCB tcb,RTE_
 
 SAPDB_Bool RTESync_RWRegion::tryEnterRegionNonExclusiveNonBlocking(RTE_TCB tcb,RTE_UKT ukt)
 {
-    get_lock(tcb,ukt,false,m_attemptsToGetLockOnEnter);
-    if(m_nonExclusive)
+    get_lock(tcb,ukt,false,m_tasCounter);
+    if(m_nonExclusive < 0)  // the region has been locked "exclusive" -> can't enter
     {
         free_lock();
         return false;
@@ -592,7 +769,7 @@ void RTESync_RWRegionNoSemalist::enterRegionExclusive(RTE_TCB tcb,RTE_UKT ukt,SA
     if(XPARAM(ulRegionCollisionLoop)  &&
        ( !tcb.prioFlag() || !XPARAM(fBegExclQueWhenPrio)))
     {
-        get_lock(tcb,ukt,false,m_attemptsToGetLockOnEnter);
+        get_lock(tcb,ukt,false,m_tasCounter);
     }
     else
     {
@@ -601,7 +778,7 @@ void RTESync_RWRegionNoSemalist::enterRegionExclusive(RTE_TCB tcb,RTE_UKT ukt,SA
             fRegionFree = (0==m_nonExclusive); /* dirty read */
             if ( fRegionFree )
             {
-                get_lock(tcb,ukt,false,m_attemptsToGetLockOnEnter);
+                get_lock(tcb,ukt,false,m_tasCounter);
                 fRegionFree = (0==m_nonExclusive); /* real read */
                 if ( !fRegionFree )
                     free_lock();
@@ -633,7 +810,7 @@ void RTESync_RWRegionNoSemalist::enterRegionNonExclusive(RTE_TCB tcb,RTE_UKT ukt
 {
     do
     {
-        get_lock(tcb,ukt,false,m_attemptsToGetLockOnEnter);
+        get_lock(tcb,ukt,false,m_tasCounter);
         if(m_nonExclusive < 0)
         {
             free_lock();
@@ -657,8 +834,8 @@ void RTESync_RWRegionNoSemalist::enterRegionNonExclusive(RTE_TCB tcb,RTE_UKT ukt
     if(0 == m_nonExclusive)
     {
         SAPDB_Char msg[256];
-        SAPDB_sprintf(msg,256," m_nonExclusive is 0 after enter (non-Exclusive)!!(pid is %d)\n",tcb.taskIndex());
-        RTE_Message(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
+        SAPDB_sprintf(msg,256," m_nonExclusive is 0 after enter (non-Exclusive)!!(pid is %d region id is %d)\n",tcb.taskIndex(),m_id);
+        RTE_Crash(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
     }
     free_lock();
 }
@@ -680,12 +857,12 @@ void RTESync_RWRegionNoSemalist::leaveRegionNonExclusive(RTE_TCB tcb,RTE_UKT ukt
 /*---------------------------------------------------------------------------*/
 void RTESync_RWRegionNoSemalist::SwitchNonExclusive(RTE_TCB tcb,RTE_UKT ukt)
 {
-    get_lock(tcb,ukt,false,m_attemptsToGetLockOnEnter);
+    get_lock(tcb,ukt,false,m_tasCounter);
     if(-1 != m_nonExclusive)
     {
         SAPDB_Char msg[256];
-        SAPDB_sprintf(msg,256,"SwitchToNonExclusiveAccess() called but m_nonExclusive is != -1 (%d)!!(pid is %d)\n",m_nonExclusive,tcb.taskIndex());
-        RTE_Message(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
+        SAPDB_sprintf(msg,256,"SwitchToNonExclusiveAccess() called but m_nonExclusive is != -1 (%d)!!(pid is %d region id is %d)\n",m_nonExclusive,tcb.taskIndex(),m_id);
+        RTE_Crash(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
     }
     m_nonExclusive = 1;
     free_lock();
@@ -753,7 +930,7 @@ void RTESync_RWRegionWithSemalist::enterRegionExclusive(RTE_TCB tcb,RTE_UKT ukt,
 {
     do
     {
-        get_lock(tcb,ukt,false,m_attemptsToGetLockOnEnter);
+        get_lock(tcb,ukt,false,m_tasCounter);
         if(m_nonExclusive != 0)   // not free?
         {
             collisionCounter++;
@@ -772,6 +949,7 @@ void RTESync_RWRegionWithSemalist::enterRegionExclusive(RTE_TCB tcb,RTE_UKT ukt,
     m_exclusiveRequests++;
     if(m_nonExclusive != 0)
     {
+        m_waitCounter++;
         m_waitQueue.insert(true,tcb);
         free_lock();
         (RTETask_IScheduling::Instance()).Reschedule(ukt);
@@ -790,7 +968,7 @@ void RTESync_RWRegionWithSemalist::enterRegionNonExclusive(RTE_TCB tcb,RTE_UKT u
 {
     do
     {
-        get_lock(tcb,ukt,false,m_attemptsToGetLockOnEnter);
+        get_lock(tcb,ukt,false,m_tasCounter);
         if(m_nonExclusive < 0)   // not free?
         {
             collisionCounter++;
@@ -808,6 +986,7 @@ void RTESync_RWRegionWithSemalist::enterRegionNonExclusive(RTE_TCB tcb,RTE_UKT u
     while(true);
     if(m_nonExclusive < 0 || m_exclusiveRequests > 0)
     {
+        m_waitCounter++;
         m_waitQueue.insert(false,tcb);
         free_lock();
         (RTETask_IScheduling::Instance()).Reschedule(ukt);
@@ -819,8 +998,8 @@ void RTESync_RWRegionWithSemalist::enterRegionNonExclusive(RTE_TCB tcb,RTE_UKT u
         if(0 == m_nonExclusive)
         {
             SAPDB_Char msg[256];
-            SAPDB_sprintf(msg,256," m_nonExclusive is 0 after enter (non-Exclusive)!!(pid is %d)\n",tcb.taskIndex());
-            RTE_Message(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
+            SAPDB_sprintf(msg,256," m_nonExclusive is 0 after enter (non-Exclusive)!!(pid is %d, region id is %d)\n",tcb.taskIndex(),m_id);
+            RTE_Crash(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
         }
         free_lock();
     }
@@ -831,7 +1010,7 @@ void RTESync_RWRegionWithSemalist::enterRegionNonExclusive(RTE_TCB tcb,RTE_UKT u
 void RTESync_RWRegionWithSemalist::leaveRegionExclusive(RTE_TCB tcb,RTE_UKT ukt)
 {
     SAPDB_Bool readerActivated = false;   // indicate that at least one reader has been activated. In this case, no writer must be activated
-    get_lock(tcb,ukt,false,m_attemptsToGetLockOnLeave);
+    get_lock(tcb,ukt,false,m_tasCounter);
     if(0 <= m_nonExclusive)
     {
         RTE_Crash(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_LEAVE_WITHOUT_ENTER,SAPDB_ToString(tcb.taskIndex()),SAPDB_ToString(m_id)));
@@ -858,6 +1037,7 @@ void RTESync_RWRegionWithSemalist::leaveRegionExclusive(RTE_TCB tcb,RTE_UKT ukt)
             else
             {
                 m_waitQueue.removeFirst();
+                m_waitCounter--;
                 if(entryTcb.taskIsWaitingForExclusiveAccess())
                 {
                     m_nonExclusive = -1;
@@ -868,8 +1048,8 @@ void RTESync_RWRegionWithSemalist::leaveRegionExclusive(RTE_TCB tcb,RTE_UKT ukt)
                     if(0 == m_nonExclusive)
                     {
                         SAPDB_Char msg[256];
-                        SAPDB_sprintf(msg,256," m_nonExclusive is 0 after enter-after-leave (Exclusive)!!(pid is %d)\n",tcb.taskIndex());
-                        RTE_Message(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
+                        SAPDB_sprintf(msg,256," m_nonExclusive is 0 after enter-after-leave (Exclusive)!!(pid is %d region id is %d)\n",tcb.taskIndex(),m_id);
+                        RTE_Crash(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
                     }
                 }
                 m_lastEntered = entryTcb.taskIndex();
@@ -881,8 +1061,9 @@ void RTESync_RWRegionWithSemalist::leaveRegionExclusive(RTE_TCB tcb,RTE_UKT ukt)
                 }
                 else
                 {
+                    SAPDB_Bool nextTaskIsWaitingForExclusiveAccess = entryTcb.taskIsWaitingForExclusiveAccess();
                     activateTaskInOtherUKT(tcb,ukt,entryTcb);
-                    if(entryTcb.taskIsWaitingForExclusiveAccess())
+                    if(nextTaskIsWaitingForExclusiveAccess)
                     {
                         free_lock();
                         break;
@@ -904,7 +1085,7 @@ void RTESync_RWRegionWithSemalist::leaveRegionExclusive(RTE_TCB tcb,RTE_UKT ukt)
 void RTESync_RWRegionWithSemalist::leaveRegionNonExclusive(RTE_TCB tcb,RTE_UKT ukt)
 {
     SAPDB_Bool readerActivated = false;   // indicate that at least one reader has been activated. In this case, no writer must be activated
-    get_lock(tcb,ukt,false,m_attemptsToGetLockOnLeave);
+    get_lock(tcb,ukt,false,m_tasCounter);
     if(0 >= m_nonExclusive)
     {
         RTE_Crash(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_LEAVE_WITHOUT_ENTER,SAPDB_ToString(tcb.taskIndex()),SAPDB_ToString(m_id)));
@@ -930,6 +1111,7 @@ void RTESync_RWRegionWithSemalist::leaveRegionNonExclusive(RTE_TCB tcb,RTE_UKT u
             else
             {
                 m_waitQueue.removeFirst();
+                m_waitCounter--;
                 if(entryTcb.taskIsWaitingForExclusiveAccess())
                     m_nonExclusive = -1;
                 else
@@ -938,8 +1120,8 @@ void RTESync_RWRegionWithSemalist::leaveRegionNonExclusive(RTE_TCB tcb,RTE_UKT u
                     if(0 == m_nonExclusive)
                     {
                         SAPDB_Char msg[256];
-                        SAPDB_sprintf(msg,256," m_nonExclusive is 0 after enter-after-leave (non-Exclusive)!!(pid is %d)\n",tcb.taskIndex());
-                        RTE_Message(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
+                        SAPDB_sprintf(msg,256," m_nonExclusive is 0 after enter-after-leave (non-Exclusive)!!(pid is %d region is is %d)\n",tcb.taskIndex(),m_id);
+                        RTE_Crash(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
                     }
                 }
                 m_lastEntered = entryTcb.taskIndex();
@@ -951,8 +1133,9 @@ void RTESync_RWRegionWithSemalist::leaveRegionNonExclusive(RTE_TCB tcb,RTE_UKT u
                 }
                 else
                 {
+                    SAPDB_Bool nextTaskIsWaitingForExclusiveAccess = entryTcb.taskIsWaitingForExclusiveAccess();
                     activateTaskInOtherUKT(tcb,ukt,entryTcb);
-                    if(entryTcb.taskIsWaitingForExclusiveAccess())
+                    if(nextTaskIsWaitingForExclusiveAccess)
                     {
                         free_lock();
                         break;
@@ -972,14 +1155,15 @@ void RTESync_RWRegionWithSemalist::leaveRegionNonExclusive(RTE_TCB tcb,RTE_UKT u
 /*---------------------------------------------------------------------------*/
 void RTESync_RWRegionWithSemalist::SwitchNonExclusive(RTE_TCB tcb,RTE_UKT ukt)
 {
-    get_lock(tcb,ukt,false,m_attemptsToGetLockOnEnter);
+    get_lock(tcb,ukt,false,m_tasCounter);
     if(-1 != m_nonExclusive)
     {
         SAPDB_Char msg[256];
-        SAPDB_sprintf(msg,256,"SwitchToNonExclusiveAccess() called but m_nonExclusive is != -1 (%d)!!(pid is %d)\n",m_nonExclusive,tcb.taskIndex());
-        RTE_Message(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
+        SAPDB_sprintf(msg,256,"SwitchToNonExclusiveAccess() called but m_nonExclusive is != -1 (%d)!!(pid is %d region id is %d)\n",m_nonExclusive,tcb.taskIndex(),m_id);
+        RTE_Crash(SAPDBErr_MessageList(RTE_CONTEXT,RTEERR_SYNC_RWREGION_INCONSISTENCY,msg));
     }
     m_nonExclusive = 1;
+    m_exclusiveRequests--;
     do
     {
         if(m_waitQueue.empty()) // no tasks that could be activated -> leave
@@ -1000,9 +1184,18 @@ void RTESync_RWRegionWithSemalist::SwitchNonExclusive(RTE_TCB tcb,RTE_UKT ukt)
                 if(entryTcb.ukt() != ukt)   // the waiting task is in another UKT -> activate it
                 {
                     m_waitQueue.removeFirst();
+                    m_waitCounter--;
                     m_nonExclusive++;
                     activateTaskInOtherUKT(tcb,ukt,entryTcb);
                 }
+                else
+                {
+                    free_lock();
+//http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1129806:
+// activate tasks in my own task scheduler, too
+                    activateTaskInMyUKT(tcb,ukt,entryTcb);
+                    break;
+                }
             }
         }
     }
@@ -1060,7 +1253,11 @@ RTESync_BaseSpinlockPool::RTESync_BaseSpinlockPool( SAPDB_UTF8 const *poolIdenti
                                                     SAPDBMem_IRawAllocator &allocator)
     : m_numberOfElements(numberOfElements)
     , m_allocator(allocator)
+    , m_info(0,0)   // must be done this way, because we do not know "this" at this point
 {
+    strncpy((char *)&m_Name[0], (const char *)poolIdentifier, sizeof(m_Name)-1);
+    m_Name[sizeof(m_Name)-1] = 0;
+
     m_pool = (RTESync_NamedSpinlock *)((SAPDB_Byte *)m_allocator.Allocate(sizeof(RTESync_NamedSpinlock) * m_numberOfElements + 2 * SAFETY_OFFSET_)+SAFETY_OFFSET_);
     if ( !m_pool )
     {
@@ -1073,6 +1270,8 @@ RTESync_BaseSpinlockPool::RTESync_BaseSpinlockPool( SAPDB_UTF8 const *poolIdenti
         SAPDB_sprintf((SAPDB_Char *)myName, sizeof(myName), "%s%d", poolIdentifier, i);
         new(&m_pool[i])RTESync_NamedSpinlock(myName);
     }
+    new ( ((void *)&m_info) ) RTE_ItemRegister<RTESync_BaseSpinlockPool>::Info ( (const SAPDB_UTF8 *)&m_Name[0], this );
+    RTESync_SpinlockPoolRegister::Instance().Register(m_info);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1134,6 +1333,283 @@ void RTESync_DestroyRWRegion(RTESync_IRWRegion *region,SAPDBMem_IRawAllocator &a
     allocator.Deallocate(region);
 }
 
+SAPDB_Bool RTESync_RWRegionRegister::FillStatisticInfo(StatisticInfo * &pInfoArray, SAPDB_Int4 maxCount, SAPDB_Int4 &itemCount,RTESync_IRWRegionReader::Mode mode,RTESync_IRWRegionReader::Selection  selection)
+{
+    RTESync_LockedScope lock(Spinlock());
+
+    if ( this->m_Count > maxCount )
+    {
+        return false;
+    }
+    itemCount = 0;
+    StatisticInfo *pFill = pInfoArray;
+
+    for ( RTE_ItemRegister<RTESync_RWRegion>::Info * pInfo = this->m_First; 0 != pInfo; pInfo = pInfo->GetSuccessor() )
+    {
+        RTESync_RWRegion *pCurrent = &(pInfo->GetItem());
+        if( (selection == RTESync_IRWRegionReader::all) || (pCurrent->m_nonExclusive != 0)/*(pCurrent->m_lastEntered != -1)*/ )
+        {
+            pFill->ExclusiveAccessCount     = pCurrent->m_exclusiveAccessCounter;
+            pFill->NonExclusiveAccessCount  = pCurrent->m_nonExclusiveAccessCounter;
+            pFill->CollisionCount           = pCurrent->m_collisionCounter;
+            pFill->WaitCount                = pCurrent->m_waitCounter;
+            pFill->TasCount                 = pCurrent->m_tasCounter;
+            pFill->CurrentAccesses          = pCurrent->m_nonExclusive;
+#ifdef RESET_STATISTICS_SUPPORTED
+            if(RTESync_IRWRegionReader::reset == mode)
+            {
+                pFill->ExclusiveAccessCount     -= pCurrent->m_exclusiveAccessCounterAtReset;
+                pFill->NonExclusiveAccessCount  -= pCurrent->m_nonExclusiveAccessCounterAtReset;
+                pFill->CollisionCount           -= pCurrent->m_collisionCounterAtReset;
+                pFill->WaitCount                -= pCurrent->m_waitCounterAtReset;
+                pFill->CollisionCount           -= pCurrent->m_tasCounterAtReset;
+            }
+#endif
+            strcpy((SAPDB_Char *)pFill->PoolName, (SAPDB_Char *)pCurrent->m_pool.Name());
+            pFill->TaskId = pCurrent->m_lastEntered;
+    #ifdef ADD_WAITERS_
+            pFill->Waiters[0]=0;
+            if(-1 != pFill->TaskId)
+            {
+                if(pCurrent->getWaitQueue())
+                {
+                    RTETask_Task *pCurrenWaitQueueEntry = pCurrent->getWaitQueue()->getFirst();
+                    if(pCurrenWaitQueueEntry)
+                    {
+                        SAPDB_sprintf(&pFill->Waiters[0],sizeof(pFill->Waiters),"%d",pCurrenWaitQueueEntry->ID());
+                        pCurrenWaitQueueEntry = pCurrenWaitQueueEntry->NextTaskInWaitQueue();
+                    }
+                    while(pCurrenWaitQueueEntry)
+                    {
+                        SAPDB_sprintf(&pFill->Waiters[0],sizeof(pFill->Waiters),"%s,%d",pFill->Waiters,pCurrenWaitQueueEntry->ID());
+                        pCurrenWaitQueueEntry = pCurrenWaitQueueEntry->NextTaskInWaitQueue();
+                    }
+                }
+            }
+    #endif
+            pFill->Address = (SAPDB_ULong)pCurrent;
+            pFill++;
+            itemCount++;
+        }
+    }
+    return true;
+        
+}
+
+
+class RTESync_RWRegionReader : public RTESync_IRWRegionReader
+{
+
+public:
+
+    /* -----------------------------------------------------------------------*/
+    /*! chapter: Constructors and initialization                              */
+    /* -----------------------------------------------------------------------*/
+
+    RTESync_RWRegionReader (void)
+    {
+    }
+    
+    ~RTESync_RWRegionReader (void)
+    {
+    }
+
+    /* -----------------------------------------------------------------------*/
+    /*! endchapter: Constructors and initialization                           */
+    /* -----------------------------------------------------------------------*/
+
+    /* -----------------------------------------------------------------------*/
+    /*! chapter: Iteration                                                    */
+    /* -----------------------------------------------------------------------*/
+
+    SAPDB_Bool First (
+        RTESync_IRWRegionReader::Mode       mode        = RTESync_IRWRegionReader::normal,
+        RTESync_IRWRegionReader::Selection  selection   = RTESync_IRWRegionReader::all
+    )
+       {
+        if ( RTESync_RWRegionRegister::Instance().GetStatisticInfo(m_pInfoArray, m_Elements,mode,selection) )
+        {
+            m_Current = -1;
+            return (true);
+        }
+        else
+        {
+            return (false);
+        }
+    }
+
+    SAPDB_Bool Next (void)
+    {
+        m_Current ++;
+
+        if ((!m_pInfoArray) || (m_Current >= m_Elements))
+        {
+            FreeArray();
+            return (false);
+        }
+        else
+        {
+            return (true);
+        }
+    }
+
+    /* -----------------------------------------------------------------------*/
+    /*! endchapter: Iteration                                                 */
+    /* -----------------------------------------------------------------------*/
+
+    /* -----------------------------------------------------------------------*/
+    /*! chapter: Data Access                                                  */
+    /* -----------------------------------------------------------------------*/
+
+    SAPDB_Bool SpinlockPoolName         (SAPDB_UTF8 name[])
+    {
+        if ((m_pInfoArray) && (m_Current < m_Elements))
+        {
+            strcpy((SAPDB_Char *)name, (SAPDB_Char *)m_pInfoArray[m_Current].PoolName);
+            return (true);
+        }
+        else
+        {
+            return (true);
+        }
+    }
+
+    SAPDB_Bool Address                  (SAPDB_UInt8 &address)
+    {
+        if ((m_pInfoArray) && (m_Current < m_Elements))
+        {
+            address = m_pInfoArray[m_Current].Address;
+            return (true);
+        }
+        else
+        {
+            return (true);
+        }
+    }
+
+    SAPDB_Bool CollisionCount           (SAPDB_UInt8 &collisioncount)
+    {
+        if ((m_pInfoArray) && (m_Current < m_Elements))
+        {
+            collisioncount = m_pInfoArray[m_Current].CollisionCount;
+            return (true);
+        }
+        else
+        {
+            return (true);
+        }
+    }
+
+    SAPDB_Bool WaitCount           (SAPDB_UInt8 &waitcount)
+    {
+        if ((m_pInfoArray) && (m_Current < m_Elements))
+        {
+            waitcount = m_pInfoArray[m_Current].WaitCount;
+            return (true);
+        }
+        else
+        {
+            return (true);
+        }
+    }
+
+    SAPDB_Bool SpinLockCollisionCount   (SAPDB_UInt8 &tascount)
+    {
+        if ((m_pInfoArray) && (m_Current < m_Elements))
+        {
+            tascount = m_pInfoArray[m_Current].TasCount;
+            return (true);
+        }
+        else
+        {
+            return (true);
+        }
+    }
+
+    SAPDB_Bool ExclusiveAccessCount     (SAPDB_UInt8 &exclusiveaccesscount)
+    {
+        if ((m_pInfoArray) && (m_Current < m_Elements))
+        {
+            exclusiveaccesscount = m_pInfoArray[m_Current].ExclusiveAccessCount;
+            return (true);
+        }
+        else
+        {
+            return (true);
+        }
+    }
+
+    SAPDB_Bool NonExclusiveAccessCount  (SAPDB_UInt8 &nonexclusiveaccesscount)
+    {
+        if ((m_pInfoArray) && (m_Current < m_Elements))
+        {
+            nonexclusiveaccesscount = m_pInfoArray[m_Current].NonExclusiveAccessCount;
+            return (true);
+        }
+        else
+        {
+            return (true);
+        }
+    }
+
+    SAPDB_Bool LastEnteredTask          (SAPDB_Int4 &taskId)
+    {
+        if ((m_pInfoArray) && (m_Current < m_Elements))
+        {
+            taskId = m_pInfoArray[m_Current].TaskId;
+            return (true);
+        }
+        else
+        {
+            return (true);
+        }
+    }
+
+    SAPDB_Bool CurrentAccesses          (SAPDB_Int4 &currentAccesses)
+    {
+        if ((m_pInfoArray) && (m_Current < m_Elements))
+        {
+            currentAccesses = m_pInfoArray[m_Current].CurrentAccesses;
+            return (true);
+        }
+        else
+        {
+            return (true);
+        }
+    }
+
+    SAPDB_UInt4 GetNumberOfItems(void)
+    {
+        return RTESync_RWRegionRegister::Instance().GetCount();
+    }
+    /* -----------------------------------------------------------------------*/
+    /*! endchapter: Data Access                                               */
+    /* -----------------------------------------------------------------------*/
+
+
+private:
+
+    void FreeArray (void)
+    {
+        if (m_pInfoArray)
+        {
+            RTESync_RWRegionRegister::Instance().FreeStatisticInfo(m_pInfoArray);
+            m_pInfoArray  = NULL;
+            m_Elements    = 0;
+            m_Current     = 0;
+        }
+    };
+
+    RTESync_RWRegionRegister::StatisticInfo *m_pInfoArray;
+    SAPDB_Int4                               m_Elements;
+    SAPDB_Int4                               m_Current;
+};
+
+RTESync_IRWRegionReader *RTESync_GetRWRegionReader(SAPDBMem_IRawAllocator &allocator)
+{
+    return new (allocator) RTESync_RWRegionReader();
+}
+
 /*---------------------------------------------------------------------------*/
 /*!
     The following code can be used to redirect calls to vbegexcl()/vendexcl()/visexcl() 
