@@ -107,6 +107,16 @@ static void AtomicLockPoolUnlock(void *memoryAddress, SAPDB_UInt4 lockIndex);
 /*! Atomic read/write of 64Bit values */
 /*=========================================================================================*/
 
+#if !defined( InterlockedCompareExchange ) && ( ( _MSC_VER >= 1310 ) || defined( __INTEL_COMPILER ) )
+#pragma intrinsic( _InterlockedCompareExchange )
+#define InterlockedCompareExchange _InterlockedCompareExchange
+#endif
+
+#if !defined( InterlockedCompareExchange ) && ( ( _MSC_VER >= 1310 ) || defined( __INTEL_COMPILER ) )
+#pragma intrinsic( _InterlockedCompareExchangePointer )
+#define InterlockedCompareExchangePointer _InterlockedCompareExchangePointer
+#endif
+
 #if !defined(BIT64)
 /*!
    @brief Atomic read operation on 8byte aligned signed value
@@ -146,7 +156,7 @@ SAPDB_UInt8 RTESys_AtomicReadUInt8( SAPDB_UInt8 * memoryPosition )
 void RTESys_AtomicWriteInt8( SAPDB_Int8 VAR_VALUE_REF memoryPosition, SAPDB_Int8 const newValue )
 {
     SAPDB_Int8 expectedValue = *memoryPosition; /* dirty read */
-    while ( RTESys_CmpxchgInt8(memoryPosition, expectedValue, newValue, &expectedValue) )
+    while ( !RTESys_CmpxchgInt8(memoryPosition, expectedValue, newValue, &expectedValue) )
         ; /* should not loop too often... */
 }
 
@@ -158,7 +168,7 @@ void RTESys_AtomicWriteInt8( SAPDB_Int8 VAR_VALUE_REF memoryPosition, SAPDB_Int8
 void RTESys_AtomicWriteUInt8( SAPDB_UInt8 VAR_VALUE_REF memoryPosition, SAPDB_UInt8 const newValue )
 {
     SAPDB_UInt8 expectedValue = *memoryPosition; /* dirty read */
-    while ( RTESys_CmpxchgUInt8(memoryPosition, expectedValue, newValue, &expectedValue) )
+    while ( !RTESys_CmpxchgUInt8(memoryPosition, expectedValue, newValue, &expectedValue) )
         ; /* should not loop too often... */
 }
 #endif
@@ -370,6 +380,7 @@ SAPDB_Bool RTESys_CmpxchgInt8( SAPDB_Int8 *memoryPosition,
     SAPDB_UInt4 resultLow = newValue >> 32;
     SAPDB_UInt4 resultHigh = expectedValue >> 32;
     __asm__ __volatile__ (
+		"push %%ebx\n\t"
 		"movl %3, %%eax\n\t"
 		"movl %4, %%edx\n\t"
 		"movl %1, %%ebx\n\t"
@@ -377,6 +388,7 @@ SAPDB_Bool RTESys_CmpxchgInt8( SAPDB_Int8 *memoryPosition,
 		"lock; cmpxchg8b %0\n\t"
 		"movl %%edx, %6\n\t"
 		"movl %%eax, %5\n\t"
+		"pop %%ebx\n\t"
 		: 
 		: "m"(*memoryPosition), 
 		  "m"(newValue),
@@ -420,6 +432,7 @@ SAPDB_Bool RTESys_CmpxchgUInt8( SAPDB_UInt8 *memoryPosition,
     SAPDB_UInt4 resultLow = newValue >> 32;
     SAPDB_UInt4 resultHigh = expectedValue >> 32;
     __asm__ __volatile__ (
+		"push %%ebx\n\t"
 		"movl %3, %%eax\n\t"
 		"movl %4, %%edx\n\t"
 		"movl %1, %%ebx\n\t"
@@ -427,6 +440,7 @@ SAPDB_Bool RTESys_CmpxchgUInt8( SAPDB_UInt8 *memoryPosition,
 		"lock; cmpxchg8b %0\n\t"
 		"movl %%edx, %6\n\t"
 		"movl %%eax, %5\n\t"
+		"pop %%ebx\n\t"
 		: 
 		: "m"(*memoryPosition), 
 		  "m"(newValue),
@@ -457,6 +471,83 @@ SAPDB_Bool RTESys_CmpxchgPointer( void * *memoryPosition,
     return ( expectedValue == (*pOldValue) );
 }
 
+#elif defined(LINUX) && defined(X86_64)
+
+SAPDB_Bool RTESys_CmpxchgInt4( SAPDB_Int4 *memoryPosition,
+                               SAPDB_Int4 const expectedValue,
+                               SAPDB_Int4 const newValue,
+                               SAPDB_Int4 *pOldValue)
+{
+	__asm__ __volatile__ (
+		"movl %3, %%eax\n\t"
+		"lock; cmpxchgl %2, %1\n\t"
+		"movl %%eax, %0\n\t"
+		: "=g"(*pOldValue): "m"(*memoryPosition), "r"(newValue), "g"(expectedValue) 
+		: "eax", "memory"
+	);
+    return ( expectedValue == (*pOldValue) );
+}
+
+SAPDB_Bool RTESys_CmpxchgUInt4( SAPDB_UInt4 *memoryPosition,
+                                SAPDB_UInt4 const expectedValue,
+                                SAPDB_UInt4 const newValue,
+                                SAPDB_UInt4 *pOldValue)
+{
+	__asm__ __volatile__ (
+		"movl %3, %%eax\n\t"
+		"lock; cmpxchgl %2, %1\n\t"
+		"movl %%eax, %0\n\t"
+		: "=g"(*pOldValue): "m"(*memoryPosition), "r"(newValue), "g"(expectedValue) 
+		: "eax", "memory"
+	);
+    return ( expectedValue == (*pOldValue) );
+}
+
+SAPDB_Bool RTESys_CmpxchgInt8( SAPDB_Int8 *memoryPosition,
+                               SAPDB_Int8 const expectedValue,
+                               SAPDB_Int8 const newValue,
+                               SAPDB_Int8 *pOldValue)
+{
+	__asm__ __volatile__ (
+		"movq %3, %%rax\n\t"
+		"lock; cmpxchgq %2, %1\n\t"
+		"movq %%rax, %0\n\t"
+		: "=g"(*pOldValue): "m"(*memoryPosition), "r"(newValue), "g"(expectedValue) 
+		: "rax", "memory"
+	);
+    return ( expectedValue == (*pOldValue) );
+}
+
+SAPDB_Bool RTESys_CmpxchgUInt8( SAPDB_UInt8 *memoryPosition,
+                                SAPDB_UInt8 const expectedValue,
+                                SAPDB_UInt8 const newValue,
+                                SAPDB_UInt8 *pOldValue)
+{
+	__asm__ __volatile__ (
+		"movq %3, %%rax\n\t"
+		"lock; cmpxchgq %2, %1\n\t"
+		"movq %%rax, %0\n\t"
+		: "=g"(*pOldValue): "m"(*memoryPosition), "r"(newValue), "g"(expectedValue) 
+		: "rax", "memory"
+	);
+    return ( expectedValue == (*pOldValue) );
+}
+
+SAPDB_Bool RTESys_CmpxchgPointer( void * *memoryPosition,
+                                  void * const expectedValue,
+                                  void * const newValue,
+                                  void * *pOldValue)
+{
+	__asm__ __volatile__ (
+		"movq %3, %%rax\n\t"
+		"lock; cmpxchgq %2, %1\n\t"
+		"movq %%rax, %0\n\t"
+		: "=g"(*pOldValue): "m"(*memoryPosition), "r"(newValue), "g"(expectedValue) 
+		: "rax", "memory"
+	);
+    return ( expectedValue == (*pOldValue) );
+}
+
 #elif defined(LINUX) && defined(IA64)
 
 SAPDB_Bool RTESys_CmpxchgInt4( SAPDB_Int4 *memoryPosition,
@@ -464,16 +555,15 @@ SAPDB_Bool RTESys_CmpxchgInt4( SAPDB_Int4 *memoryPosition,
                                SAPDB_Int4 const newValue,
                                SAPDB_Int4 *pOldValue)
 {
-#if defined __GNUC__
+#if defined __INTEL_COMPILER
+/*LINUX && IA64*/	*pOldValue = _InterlockedCompareExchange_rel (memoryPosition, newValue, expectedValue);
+#elif defined __GNUC__
 /*LINUX && IA64*/	SAPDB_Int8 expectedValueAsLong = expectedValue;
 /*LINUX && IA64*/   __asm__ __volatile__ ("mov ar.ccv = %0 ;;" :: "rO"(expectedValueAsLong) );
 /*LINUX && IA64*/	__asm__ __volatile__ ("cmpxchg4.rel %0=%2,%3,ar.ccv"
 /*LINUX && IA64*/			: "=r"(*pOldValue),"=m"(*memoryPosition)
 /*LINUX && IA64*/			: "m"(*memoryPosition), "r"(newValue));
 #endif
-#if defined __INTEL_COMPILER
-/*LINUX && IA64*/	*pOldValue = _InterlockedCompareExchange_rel (memoryPosition, newValue, expectedValue);
-#endif
 /*LINUX && IA64*/  return ( expectedValue == (*pOldValue) );
 }
 
@@ -482,16 +572,15 @@ SAPDB_Bool RTESys_CmpxchgUInt4( SAPDB_UInt4 *memoryPosition,
                                 SAPDB_UInt4 const newValue,
                                 SAPDB_UInt4 *pOldValue)
 {
-#if defined __GNUC__
+#if defined __INTEL_COMPILER
+/*LINUX && IA64*/	*pOldValue = _InterlockedCompareExchange_rel (memoryPosition, newValue, expectedValue);
+#elif defined __GNUC__
 /*LINUX && IA64*/	SAPDB_UInt8 expectedValueAsLong = expectedValue;
 /*LINUX && IA64*/   __asm__ __volatile__ ("mov ar.ccv = %0 ;;" :: "rO"(expectedValueAsLong) );
 /*LINUX && IA64*/	__asm__ __volatile__ ("cmpxchg4.rel %0=%2,%3,ar.ccv"
 /*LINUX && IA64*/			: "=r"(*pOldValue),"=m"(*memoryPosition)
 /*LINUX && IA64*/			: "m"(*memoryPosition), "r"(newValue));
 #endif
-#if defined __INTEL_COMPILER
-/*LINUX && IA64*/	*pOldValue = _InterlockedCompareExchange_rel (memoryPosition, newValue, expectedValue);
-#endif
 /*LINUX && IA64*/  return ( expectedValue == (*pOldValue) );
 }
 
@@ -500,15 +589,14 @@ SAPDB_Bool RTESys_CmpxchgInt8( SAPDB_Int8 *memoryPosition,
                                SAPDB_Int8 const newValue,
                                SAPDB_Int8 *pOldValue)
 {
-#if defined __GNUC__
+#if defined __INTEL_COMPILER
+/*LINUX && IA64*/	*pOldValue = _InterlockedCompareExchange64_rel (memoryPosition, newValue, expectedValue);
+#elif defined __GNUC__
 /*LINUX && IA64*/	__asm__ __volatile__ ("mov ar.ccv = %0 ;;" :: "rO"(expectedValue) );
 /*LINUX && IA64*/	__asm__ __volatile__ ("cmpxchg8.rel %0=%2,%3,ar.ccv"
 /*LINUX && IA64*/			: "=r"(*pOldValue),"=m"(*memoryPosition)
 /*LINUX && IA64*/			: "m"(*memoryPosition), "r"(newValue));
 #endif
-#if defined __INTEL_COMPILER
-/*LINUX && IA64*/	*pOldValue = _InterlockedCompareExchange_rel (memoryPosition, newValue, expectedValue);
-#endif
 /*LINUX && IA64*/  return ( expectedValue == (*pOldValue) );
 }
 
@@ -517,15 +605,14 @@ SAPDB_Bool RTESys_CmpxchgUInt8( SAPDB_UInt8 *memoryPosition,
                                 SAPDB_UInt8 const newValue,
                                 SAPDB_UInt8 *pOldValue)
 {
-#if defined __GNUC__
+#if defined __INTEL_COMPILER
+/*LINUX && IA64*/	*pOldValue = _InterlockedCompareExchange64_rel (memoryPosition, newValue, expectedValue);
+#elif defined __GNUC__
 /*LINUX && IA64*/	__asm__ __volatile__ ("mov ar.ccv = %0 ;;" :: "rO"(expectedValue) );
 /*LINUX && IA64*/	__asm__ __volatile__ ("cmpxchg8.rel %0=%2,%3,ar.ccv"
 /*LINUX && IA64*/			: "=r"(*pOldValue),"=m"(*memoryPosition)
 /*LINUX && IA64*/			: "m"(*memoryPosition), "r"(newValue));
 #endif
-#if defined __INTEL_COMPILER
-/*LINUX && IA64*/	*pOldValue = _InterlockedCompareExchange_rel (memoryPosition, newValue, expectedValue);
-#endif
 /*LINUX && IA64*/  return ( expectedValue == (*pOldValue) );
 }
 
@@ -534,15 +621,14 @@ SAPDB_Bool RTESys_CmpxchgPointer( void * *memoryPosition,
                                   void * const newValue,
                                   void * *pOldValue)
 {
-#if defined __GNUC__
+#if defined __INTEL_COMPILER
+/*LINUX && IA64*/	*pOldValue = _InterlockedCompareExchangePointer (memoryPosition, newValue, expectedValue);
+#elif defined __GNUC__
 /*LINUX && IA64*/	__asm__ __volatile__ ("mov ar.ccv = %0 ;;" :: "rO"(expectedValue) );
 /*LINUX && IA64*/	__asm__ __volatile__ ("cmpxchg8.rel %0=%2,%3,ar.ccv"
 /*LINUX && IA64*/			: "=r"(*pOldValue),"=m"(*memoryPosition)
 /*LINUX && IA64*/			: "m"(*memoryPosition), "r"(newValue));
 #endif
-#if defined __INTEL_COMPILER
-/*LINUX && IA64*/	*pOldValue = _InterlockedCompareExchange_rel (memoryPosition, newValue, expectedValue);
-#endif
 /*LINUX && IA64*/  return ( expectedValue == (*pOldValue) );
 }
 
@@ -657,7 +743,7 @@ SAPDB_Bool RTESys_CmpxchgInt4( SAPDB_Int4 *memoryPosition,
     }
     else
     {
-        *pOldValue = (SAPDB_Int4)oldValue;
+        *pOldValue = *memoryPosition;
         return false;
     }
 }
@@ -678,7 +764,7 @@ SAPDB_Bool RTESys_CmpxchgUInt4( SAPDB_UInt4 *memoryPosition,
     }
     else
     {
-        *pOldValue = (SAPDB_UInt4)oldValue;
+        *pOldValue = *memoryPosition;
         return false;
     }
 }
@@ -699,7 +785,7 @@ SAPDB_Bool RTESys_CmpxchgInt8( SAPDB_Int8 *memoryPosition,
     }
     else
     {
-        *pOldValue = (SAPDB_Int8)oldValue;
+        *pOldValue = *memoryPosition;
         return false;
     }
 }
@@ -720,7 +806,7 @@ SAPDB_Bool RTESys_CmpxchgUInt8( SAPDB_UInt8 *memoryPosition,
     }
     else
     {
-        *pOldValue = (SAPDB_UInt8)oldValue;
+        *pOldValue = *memoryPosition;
         return false;
     }
 }
@@ -1019,7 +1105,14 @@ static void AtomicLockPoolInitialize()
                               PAGE_READWRITE );
     }
 #else
-    pLock = valloc( RTESYS_ATOMIC_OPERATION_POOL_SIZE * lockSize );
+    {
+        SAPDB_Int4 systemPageMask = (RTESys_SystemPageSize() - 1);
+        SAPDB_Byte *lockPoolPtr;
+        lockPoolPtr = (SAPDB_Byte *)malloc( systemPageMask + (RTESYS_ATOMIC_OPERATION_POOL_SIZE * lockSize) );
+        pLock = (RTE_Lock *)(
+                (SAPDB_Byte *)0 + ( ((lockPoolPtr - (SAPDB_Byte *)0) + systemPageMask)&(~systemPageMask) ) 
+                            );
+    }
 #endif
     for ( iLock = 0; iLock < RTESYS_ATOMIC_OPERATION_POOL_SIZE; iLock++ )
     {
@@ -1046,7 +1139,7 @@ static SAPDB_UInt4 AtomicLockPoolLock(void *memoryAddress)
 
     poolIndex = (SAPDB_UInt4)(byteOffset % RTESYS_ATOMIC_OPERATION_POOL_SIZE);
     
-    while ( !RTESys_TestAndLock(poolLockArray[poolIndex]) )
+    while ( RTESys_TestAndLock(poolLockArray[poolIndex]) )
         ;
 
     return poolIndex;
