@@ -272,12 +272,9 @@ SAPDB_Bool RTESys_CompareAndExchange( PointerType * &     pointerPosition,
 {
     void **voidPosition = (void **)&pointerPosition;
     void *oldValueFound;
-    if ( RTESys_CmpxchgPointer(*voidPosition, expectedValue, newValue, oldValueFound) )
-    {
-        oldValue = (PointerType *)oldValueFound; 
-        return true;
-    }
-    return false;
+    SAPDB_Bool result = RTESys_CmpxchgPointer(*voidPosition, expectedValue, newValue, oldValueFound);
+    oldValue = (PointerType *)oldValueFound; 
+    return result;
 }
 
 //--------------------------------------------------------------------------------
