@@ -96,10 +96,13 @@ externC SAPDB_Char *RTESys_StrError(RTE_SystemRc errCode, RTESys_ErrorMessage sp
       NULL, errCode, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf, 0, NULL );
     strncpy(&spaceToFill[0], (char *)lpMsgBuf, sizeof(RTESys_ErrorMessage)-1);
     LocalFree(lpMsgBuf);
+#elif defined(LINUX) || defined(FREEBSD) || defined(AIX)
+    if ( strerror_r(errCode, &spaceToFill[0], sizeof(RTESys_ErrorMessage)-1) )
+    {
+        sp77sprintf(&spaceToFill[0], sizeof(RTESys_ErrorMessage), "unknown error %d", (SAPDB_Int)errCode);
+    }
 #else
-    /* all platform seem to have it now..., */
-    /* Otherwise this should return sys_errlist[errCode] if errCode is in range [0...sys_nerrlist] */
-    strncpy(&spaceToFill[0], strerror(errCode), sizeof(RTESys_ErrorMessage)-1);
+    strncpy(&spaceToFill[0], RTE_save_strerror(errCode), sizeof(RTESys_ErrorMessage)-1);
 #endif
     spaceToFill[sizeof(RTESys_ErrorMessage)-1] = 0;
     return &spaceToFill[0];
@@ -609,8 +612,18 @@ externC void RTESys_IOSeek ( const tRTESys_IOHandle      FileHandle,
 }
 /*---------------------------------------------------------------------------*/
 externC void RTESys_IOCopyFile ( const tsp00_Char     *Original,
-                            const tsp00_Char     *Destination,
-                            tsp00_VfReturn_Param *ReturnStatus )
+                                 const tsp00_Char     *Destination,
+                                 tsp00_VfReturn_Param *ReturnStatus )
+{
+    RTESys_IOCopyFileWithDestinationAtribute ( Original,Destination,DEFAULT_ATTRIBUTE_RTESYS_IO,ReturnStatus );
+}
+
+/*---------------------------------------------------------------------------*/
+externC void RTESys_IOCopyFileWithDestinationAtribute
+                              ( const tsp00_Char                   *Original,
+                                const tsp00_Char                   *Destination,
+                                SAPDB_Int                           DestinationAttribute,
+                                tsp00_VfReturn_Param               *ReturnStatus )
 {
 #if !defined(_WIN32)
   tRTESys_IOPosition BytesWritten;
@@ -621,16 +634,41 @@ externC void RTESys_IOCopyFile ( const tsp00_Char     *Original,
   ROUTINE_DBG_MEO60 (_T("RTESys_IOCopyFile"));
 
 #if defined(_WIN32)
+  SetFileAttributes( Destination,GetFileAttributes(Destination) & ~FILE_ATTRIBUTE_READONLY);
+ 
   *ReturnStatus = ( CopyFile( Original, Destination, false ) ? vf_ok : vf_notok );
+ 
+  if(DEFAULT_ATTRIBUTE_RTESYS_IO != DestinationAttribute)
+  {
+    SetFileAttributes( Destination,DestinationAttribute );
+  }
 #else
   tRTESys_IOHandle OriginalFd;
   tRTESys_IOHandle DestinationFd;
   tsp00_VfReturn_Param IgnoredError; /* close errors of temporary files are ignored here */
-  
+  SAPDB_Bool DestinationAttributeSetOnOpen = false;  
+
   RTESys_IOOpen( &OriginalFd, Original, RTESys_IOReadOnly, false, DEFAULT_SECURITY_RTESYS_IO, ReturnStatus );
   if ( vf_ok == *ReturnStatus )
   {
-    RTESys_IOOpen( &DestinationFd, Destination, RTESys_IOWriteOnly, true, DEFAULT_SECURITY_RTESYS_IO, ReturnStatus );
+    if( DEFAULT_ATTRIBUTE_RTESYS_IO == DestinationAttribute )
+    {
+      struct stat orgBuf;
+      if ( 0 == RTE_save_stat(Original, &orgBuf) )
+      {
+        DestinationAttribute = orgBuf.st_mode;
+      }
+      else
+      {
+        DestinationAttribute = DEFAULT_SECURITY_RTESYS_IO;
+      }
+    }
+    else
+    {
+      DestinationAttributeSetOnOpen = true;
+    } 
+       
+    RTESys_IOOpen( &DestinationFd, Destination, RTESys_IOWriteOnly, true, DestinationAttribute, ReturnStatus );
     if ( vf_ok != *ReturnStatus )
     {
       RTESys_IOOpen( &DestinationFd, Destination, RTESys_IOWriteOnly, false, DEFAULT_SECURITY_RTESYS_IO, ReturnStatus );
@@ -666,21 +704,24 @@ externC void RTESys_IOCopyFile ( const tsp00_Char     *Original,
       *ReturnStatus = vf_ok;
     }
 
-    if ( vf_ok == *ReturnStatus )
-    {
-        /* Ignore any errors if mode setting fails... */
-        struct stat orgBuf;
-        if ( 0 == RTE_save_stat(Original, &orgBuf) )
-        {
-            if ( 0 != RTE_save_chmod(Destination, orgBuf.st_mode) )
-            {
-                RTESys_IOShowLastOsError("Ignoring bad chmod(Destination)", Destination );
-            }
-        }
-        else
-        {
-            RTESys_IOShowLastOsError("Ignoring bad stat(Original)", Original );
-        }
+    if(!DestinationAttributeSetOnOpen) 
+    {   
+      if ( vf_ok == *ReturnStatus )
+      {
+          /* Ignore any errors if mode setting fails... */
+          struct stat orgBuf;
+          if ( 0 == RTE_save_stat(Original, &orgBuf) )
+          {
+              if ( 0 != RTE_save_chmod(Destination, orgBuf.st_mode) )
+              {
+                  RTESys_IOShowLastOsError("Ignoring bad chmod(Destination)", Destination );
+              }
+          }
+          else
+          {
+              RTESys_IOShowLastOsError("Ignoring bad stat(Original)", Original );
+          }
+      }
     }
   }
   else
