@@ -1235,7 +1235,7 @@ inline SAPDB_UInt4  RTESys_NTAWEPhysMem::ReservePhysPage ( SAPDB_UInt4
   // Locked scope
   { RTESync_LockedScope LockedScope( m_Lock );
 
-    if ( m_FreeAWEPhysPages )
+    if (( m_FreeAWEPhysPages ) && ( NO_MORE_ENTRIES != m_FirstFreePPInfo ))
     {
       SAPDBERR_ASSERT_STATE(m_pPPInfos[m_FirstFreePPInfo].Reserved == 0);
 
