@@ -54,7 +54,9 @@
  *  INCLUDES                                                                 *
  *===========================================================================*/
 
+#ifndef _WIN32_WINNT
 #define _WIN32_WINNT  0x0500
+#endif
 #include <windows.h>
 #include "SAPDBCommon/SAPDB_Types.h"
 #include "SAPDBCommon/ErrorsAndMessages/SAPDBErr_Assertions.hpp"
