@@ -46,13 +46,14 @@
 #include "RunTime/RTE_MessageList.hpp"
 #include "RunTime/RTE_Messages.hpp"
 #include "RunTime/RTE_Message.hpp"
+#include "RunTime/System/RTESys_MemoryBarrier.h"
 
 
 /*===========================================================================*
  *  DEFINES                                                                  *
  *===========================================================================*/
 
-//#define HIGHRES_SERIALIZE
+
 
 /*===========================================================================*
  *  MACROS                                                                   *
@@ -76,6 +77,10 @@ extern "C" {
 # define _IA64_REG_AR_ITC 3116
 #endif
 
+#if defined( _M_AMD64 )
+#define CPUID
+#endif
+
 /*===========================================================================*
  *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
  *===========================================================================*/
@@ -204,17 +209,14 @@ SAPDB_UInt8 NTHighResTimer_ASM::Counter()
   { // Locked scope
     RTESync_LockedScope LockedScope( m_Lock ); 
 
+    RTESys_MemoryBarrier ();
+
 #   if defined( _M_IX86 )
     {
      LARGE_INTEGER Val;
+
      __asm
      {
-#      ifdef HIGHRES_SERIALIZE
-        XOR   EAX, EAX           /* to have a defined input for CPUID    */
-        CPUID                    /* serializing instruction              */
-                                 /* CPUID CHANGES EBX: SHOW IT TO CC     */
-#      endif
-
        _emit 0x0F                /* rdtsc - result in EDX:EAX            */
        _emit 0x31
 
@@ -224,9 +226,6 @@ SAPDB_UInt8 NTHighResTimer_ASM::Counter()
      NewVal = Val.QuadPart;
     }
 #   elif defined( _M_IA64 )
-#    ifdef HIGHRES_SERIALIZE
-      __memory_barrier();
-#    endif
      NewVal = __getReg(_IA64_REG_AR_ITC);
 #   elif defined( _M_AMD64 )
      NewVal = ReadTimeStampCounter();
@@ -248,8 +247,8 @@ SAPDB_UInt8 NTHighResTimer_ASM::Counter()
 
     m_LastCounterVal = NewVal; // set last counter value;
     CounterOffset    = m_CounterOffset;
+    RTESys_MemoryBarrier ();
   }
-
 #if 0
   if ( LeapWarning )
   { // The leap message will be  written here, because we schould not 
@@ -500,6 +499,12 @@ bool RTESys_NTHighResTimer::LeapCheck()
 
 RTESys_NTHighResTimer & RTESys_NTHighResTimer::Instance()
 {
+  if ( m_Instance)
+     return *m_Instance;
+
+  TimerType type = CPUTimer;
+  Initialize ( type );
+
   SAPDBERR_ASSERT_STATE(m_Instance != NULL);
 
   return *m_Instance;
