@@ -1272,6 +1272,7 @@ externC SAPDB_Real8 RTESys_GetCPULoad(void)
         fd = open("/proc/loadavg",0);
         if(fd < 0)
             return 0;
+        initialized = true;
     }
     else
     {
@@ -1346,15 +1347,15 @@ void RTESys_GetMachineTimes(SAPDB_UInt8 *secondsUser,SAPDB_UInt8 *secondsSystem,
         {            
             time64.LowPart  = idleTime.dwLowDateTime;
             time64.HighPart = idleTime.dwHighDateTime;
-            *secondsIdle    = (time64.QuadPart / 10000000)/(getSystemInfo()->dwNumberOfProcessors);
+            *secondsIdle    = (time64.QuadPart / 10000000);
 
             time64.LowPart  = kernelTime.dwLowDateTime;
             time64.HighPart = kernelTime.dwHighDateTime;
-            *secondsSystem  = (time64.QuadPart / 10000000)/(getSystemInfo()->dwNumberOfProcessors) - *secondsIdle;
+            *secondsSystem  = (time64.QuadPart / 10000000) - *secondsIdle;
 
             time64.LowPart  = userTime.dwLowDateTime;
             time64.HighPart = userTime.dwHighDateTime;
-            *secondsUser    = (time64.QuadPart / 10000000)/(getSystemInfo()->dwNumberOfProcessors);
+            *secondsUser    = (time64.QuadPart / 10000000);
         }
     }
 #elif defined (AIX)
@@ -1362,18 +1363,19 @@ void RTESys_GetMachineTimes(SAPDB_UInt8 *secondsUser,SAPDB_UInt8 *secondsSystem,
         SAPDB_UInt nHz   = (SAPDB_UInt)sysconf(_SC_CLK_TCK);
         perfstat_cpu_total_t cpuPerfStat;
         perfstat_cpu_total( NULL,&cpuPerfStat,sizeof(cpuPerfStat),1);
-        *secondsIdle    = (cpuPerfStat.idle  / cpuPerfStat.ncpus)/nHz;
-        *secondsSystem  = (cpuPerfStat.sys   / cpuPerfStat.ncpus)/nHz;
-        *secondsUser    = (cpuPerfStat.user  / cpuPerfStat.ncpus)/nHz;
+        *secondsIdle    = (cpuPerfStat.idle)/nHz;
+        *secondsSystem  = (cpuPerfStat.sys)/nHz;
+        *secondsUser    = (cpuPerfStat.user)/nHz;
     }
 #elif defined (HPUX)
     {
         struct pst_dynamic pstd;
         SAPDB_UInt nHz   = (SAPDB_UInt)sysconf(_SC_CLK_TCK);
+        SAPDB_UInt4 nProc = RTESys_NumberOfCPU();
         pstat_getdynamic(&pstd,sizeof(pstd),0,0);
-        *secondsIdle    = (pstd.psd_cpu_time[CP_IDLE])/nHz;
-        *secondsSystem  = (pstd.psd_cpu_time[CP_SYS] + pstd.psd_cpu_time[CP_WAIT])/nHz;
-        *secondsUser    = (pstd.psd_cpu_time[CP_USER] + pstd.psd_cpu_time[CP_NICE])/nHz;
+        *secondsIdle    = ((pstd.psd_cpu_time[CP_IDLE])/nHz)*nProc;
+        *secondsSystem  = ((pstd.psd_cpu_time[CP_SYS] + pstd.psd_cpu_time[CP_WAIT])/nHz)*nProc;
+        *secondsUser    = ((pstd.psd_cpu_time[CP_USER] + pstd.psd_cpu_time[CP_NICE])/nHz)*nProc;
     }
 #elif defined (LINUX)
     {
@@ -1399,9 +1401,9 @@ void RTESys_GetMachineTimes(SAPDB_UInt8 *secondsUser,SAPDB_UInt8 *secondsSystem,
         if(read(fd,buf,sizeof(buf)-1) < 0)
             return;
         sscanf(buf,"%*s%ld%ld%ld%ld",&user,&nice,&system,&idle);
-        *secondsIdle    = (idle/nProc)/nHz;
-        *secondsSystem  = (system/nProc)/nHz;
-        *secondsUser    = ((user+nice)/nProc)/nHz;
+        *secondsIdle    = (idle)/nHz;
+        *secondsSystem  = (system)/nHz;
+        *secondsUser    = ((user+nice))/nHz;
     }
 #elif defined (OSF1)
     {
@@ -1409,9 +1411,9 @@ void RTESys_GetMachineTimes(SAPDB_UInt8 *secondsUser,SAPDB_UInt8 *secondsSystem,
         SAPDB_UInt nProc = (SAPDB_UInt)sysconf(_SC_NPROCESSORS_ONLN);
         table ( TBL_SYSINFO , 0 , (char *)&sysinfo,1,sizeof(sysinfo));
 
-        *secondsIdle    = (sysinfo.si_idle/sysinfo.si_hz)/nProc;
-        *secondsSystem  = (sysinfo.si_sys/sysinfo.si_hz)/nProc;
-        *secondsUser    = ((sysinfo.si_user+sysinfo.si_nice)/sysinfo.si_hz)/nProc;
+        *secondsIdle    = (sysinfo.si_idle/sysinfo.si_hz);
+        *secondsSystem  = (sysinfo.si_sys/sysinfo.si_hz);
+        *secondsUser    = ((sysinfo.si_user+sysinfo.si_nice)/sysinfo.si_hz);
     }
 #elif defined (SUN)
     {
@@ -1426,11 +1428,12 @@ void RTESys_GetMachineTimes(SAPDB_UInt8 *secondsUser,SAPDB_UInt8 *secondsSystem,
         }
         if(kc && ksp)
         {
+            SAPDB_UInt4 nProc = (SAPDB_UInt4)sysconf(_SC_NPROCESSORS_ONLN);
             cpu_stat_t     statlast, cpu_stat;
             kstat_read(kc, ksp, &cpu_stat);
-            *secondsIdle    = (cpu_stat.cpu_sysinfo.cpu[CPU_IDLE])/nHz;
-            *secondsUser    = (cpu_stat.cpu_sysinfo.cpu[CPU_USER])/nHz;
-            *secondsSystem  = (cpu_stat.cpu_sysinfo.cpu[CPU_KERNEL]+cpu_stat.cpu_sysinfo.cpu[CPU_WAIT])/nHz;
+            *secondsIdle    = ((cpu_stat.cpu_sysinfo.cpu[CPU_IDLE])/nHz)*nProc;
+            *secondsUser    = ((cpu_stat.cpu_sysinfo.cpu[CPU_USER])/nHz)*nProc;
+            *secondsSystem  = ((cpu_stat.cpu_sysinfo.cpu[CPU_KERNEL]+cpu_stat.cpu_sysinfo.cpu[CPU_WAIT])/nHz)*nProc;
         }
     }
 #endif
