@@ -135,15 +135,10 @@ static  SAPDB_UInt4 ConsoleOpenConnection (RTEThread_ConsoleRequest const    &
                                            SAPDB_UInt4 const                     threadIndex,
                                            SAPDBErr_MessageList              &   messageList)
 {
-/*===========================================================================*
- *  Locals                                                                   *
- *===========================================================================*/
     RTEThread_ConsoleWorkerBase            *pWorker;
     SAPDB_UInt4                             rc = RTE_CONS_NO_ERROR;
     RTEThread_ConsoleConnectionListItem    *pConnectionItem;
-/*===========================================================================*
- *  Instructions                                                             *
- *===========================================================================*/
+
     SAPDBTRACE_ROUTINE_DEBUG("ConsoleOpenConnection", Console_Trace, 1);
 
     /*--- Searching free connection slot ------------------------------------*/
@@ -246,16 +241,10 @@ static  SAPDB_UInt4  ConsoleReleaseConnection (RTEThread_ConsoleRequest const
                                                SAPDB_UInt4 const                     threadIndex,
                                                SAPDBErr_MessageList              &   messageList)
 {
-   /*===========================================================================*
-    *  Locals                                                                   *
-    *===========================================================================*/
     RTEThread_ConsoleWorkerBase            *pWorker;
     RTEThread_ConsoleConnectionListItem    *pConnectionItem;
     SAPDB_UInt4                             rc = RTE_CONS_NO_ERROR;
 
-   /*===========================================================================*
-    *  Instructions                                                             *
-    *===========================================================================*/
     SAPDBTRACE_ROUTINE_DEBUG("ConsoleReleaseConnection", Console_Trace, 1);
 
     if (RTE_CONS_NO_ERROR 
@@ -298,17 +287,11 @@ static  void    ExecuteConsoleCommand
     SAPDB_UInt4 const                         threadIndex
 )
 {
-/*===========================================================================*
- *  Locals                                                                   *
- *===========================================================================*/
     RTEThread_ConsoleWorkerBase            *pWorker;
     SAPDBErr_MessageList                    messageList;
     RTEThread_ConsoleConnectionListItem    *pConnectionItem;
     SAPDB_UInt4                             rc = RTE_CONS_NO_ERROR;
     
-/*===========================================================================*
- *  Instructions                                                             *
- *===========================================================================*/
     SAPDBTRACE_ROUTINE_DEBUG("ExecuteConsoleCommand", Console_Trace, 1);
 
     if (OPEN_REQ == request.reqType)
@@ -429,17 +412,11 @@ static  void    ExecuteConsoleCommand
 
 externC void *ConsoleWorker ( void *arg )
 {
-/*===========================================================================*
- *  Locals                                                                   *
- *===========================================================================*/
     RTEThread_ConsoleRequest            request;
     teo07_ThreadErr                     err = THR_OK_EO07;
 	tsp00_ErrTextc                      errtext;
     ConsoleWorkerInfo               &   threadInfo = *(ConsoleWorkerInfo *)arg;
     
-/*===========================================================================*
- *  Instructions                                                             *
- *===========================================================================*/
     SAPDBTRACE_ROUTINE_DEBUG("ConsoleWorker", Console_Trace, 1);
 
 #   if defined(_FASTCAP)
@@ -498,25 +475,44 @@ externC void *ConsoleWorker ( void *arg )
 //                                      SAPDB_ToString (threadInfo.index)));
 
     sqlendthread(0);
+
     return NULL;
 }
 
 /*---------------------------------------------------------------------------*/
 
+externC void *ConsoleWorkerThread ( void *arg )
+{
+    void * ret = 0;
+
+#if defined (_WIN32)
+    __try
+    {
+#endif
+        ret = ConsoleWorker(arg);
+
+#if defined (_WIN32)
+    }
+    __except( sql50k_excepthandler(GetExceptionInformation()) )
+    {
+        ;
+    }
+#endif
+
+    return ret;
+}
+
+/*---------------------------------------------------------------------------*/
+
 SAPDB_Bool StartConsoleWorkerThread (ConsoleWorkerInfo &threadInfo, SAPDB_UInt4  threadIndex)
 {
-/*===========================================================================*
- *  Locals                                                                   *
- *===========================================================================*/
     tsp00_ErrTextc threadErrtext;
     teo07_ThreadErr threadOk;
-/*===========================================================================*
- *  Instructions                                                             *
- *===========================================================================*/
+
     SAPDBTRACE_ROUTINE_DEBUG("StartConsoleWorkerThread", Console_Trace, 1);
 
     sqlbeginthread(CONSOLE_WORKER_STACK_SIZE,
-                   ConsoleWorker,
+                   ConsoleWorkerThread,
                    &threadInfo,
                    THR_CREATE_SUSPENDED_EO07,
                    &threadInfo.thread,
@@ -538,4 +534,4 @@ SAPDB_Bool StartConsoleWorkerThread (ConsoleWorkerInfo &threadInfo, SAPDB_UInt4
 
 /*===========================================================================*
  *  END OF CODE                                                              *
- *===========================================================================*/
\ No newline at end of file
+ *===========================================================================*/
