@@ -43,6 +43,7 @@
 #include    "RunTime/RTE_MessageList.hpp"
 #include    "RunTime/RTE_Message.hpp"
 #include    "RunTime/RTE_Console_Thread_Messages.hpp"
+#include    "RunTime/System/RTESys_ProcessState.hpp"
 
 
 
@@ -567,40 +568,64 @@ void   RTEThread_ConsoleConnectionList::ScanConnectionsForTimeout
         lastAccessTime = pWorker->GetConsoleLastAccessTime ();
         currentTime = RTESys_Time ();
 
-        if (!pWorker->IsBusy () && currentTime - lastAccessTime > RTE_MAX_CONSOLE_IDLE_TIME)
+        if (!pWorker->IsBusy ())
         {
-            //Could be optimized by copying the connection data temporarily. Then
-            //the deletion of system resources and the message to the application
-            //could be done after UnLock. Here we would only have to delete the 
-            //connection data in our list. But this seems not very meaningful at 
-            //the moment
-
-            //Send timeout to XCons
-
-            messageList = SAPDBErr_MessageList(RTE_CONTEXT, RTEINFO_THREAD_CONS_TIMEOUT, 
-                                               SAPDB_ToString (pConnectionItem->GetHandle ()),
-                                               SAPDB_ToString (pConnectionItem->GetRemotePID ()),
-                                               SAPDB_ToString (pWorker->GetClientReference ()),
-                                               SAPDB_ToString (idx));
-           
-            //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            //The following send may not be blocking! Since XCons is not awaiting an answer
-            //at the moment we have to be sure that all error data fit in one packet.
-            //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-
-            if (!pWorker->SendError (TIMEOUT, RTE_CONS_REMOTE_TIMEOUT, messageList))
+            if (currentTime - lastAccessTime > RTE_MAX_CONSOLE_IDLE_TIME)
             {
-                messageList = messageList + SAPDBErr_MessageList (RTE_CONTEXT, RTEERR_THREAD_CONS_SEND_ERR);
-            }
-     
-            if (!ReleaseConnection (pConnectionItem, pWorker->GetClientReference (), messageList))
-            {
-                RTE_Crash (messageList);
-            }
+                //Could be optimized by copying the connection data temporarily. Then
+                //the deletion of system resources and the message to the application
+                //could be done after UnLock. Here we would only have to delete the 
+                //connection data in our list. But this seems not very meaningful at 
+                //the moment
 
-            RTE_Message (messageList);
-            SAPDBTRACE_WRITELN(Console_Trace, 5, "Slot " << idx << ": Last access time"
-                                                 << lastAccessTime <<" -> Connection timeout");
+                //Send timeout to XCons
+
+                RTE_Message (SAPDBErr_MessageList(RTE_CONTEXT, RTEINFO_THREAD_CONS_TIMEOUT, 
+                    SAPDB_ToString (pConnectionItem->GetHandle ()),
+                    SAPDB_ToString (pConnectionItem->GetRemotePID ()),
+                    SAPDB_ToString (pWorker->GetClientReference ()),
+                    SAPDB_ToString (idx)));
+
+                //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+                //The following send may not be blocking! Since XCons is not awaiting an answer
+                //at the moment we have to be sure that all error data fit in one packet.
+                //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+
+                if (!pWorker->SendError (TIMEOUT, RTE_CONS_REMOTE_TIMEOUT, messageList))
+                {
+                    RTE_Message (SAPDBErr_MessageList (RTE_CONTEXT, RTEERR_THREAD_CONS_SEND_ERR));
+                }
+
+                if (!ReleaseConnection (pConnectionItem, pWorker->GetClientReference (), messageList))
+                {
+                    RTE_Crash (messageList);
+                }
+
+                SAPDBTRACE_WRITELN(Console_Trace, 5, "Slot " << idx << ": Last access time"
+                    << lastAccessTime <<" -> Connection timeout");
+            }
+            else
+            {
+                //Check if the client is still alive
+                RTESys_ProcessState     clientProcess (pConnectionItem->GetRemotePID ());
+
+                if (RTESys_ProcessState::Alive != clientProcess.State ())
+                {
+                    RTE_Message (SAPDBErr_MessageList(RTE_CONTEXT, RTEINFO_THREAD_CONS_CLIENT_DIED, 
+                        SAPDB_ToString (pConnectionItem->GetHandle ()),
+                        SAPDB_ToString (pConnectionItem->GetRemotePID ()),
+                        SAPDB_ToString (pWorker->GetClientReference ()),
+                        SAPDB_ToString (idx)));
+
+                    if (!ReleaseConnection (pConnectionItem, pWorker->GetClientReference (), messageList))
+                    {
+                        RTE_Crash (messageList);
+                    }
+
+                    SAPDBTRACE_WRITELN(Console_Trace, 5, "Client connection " << (SAPDB_UInt8)pConnectionItem->GetRemotePID () 
+                        << " has died");
+                }
+            }
         }
         else
         {
