@@ -669,17 +669,13 @@ private:
 	            /* first child */
 	            unsigned int j = 1 + 2*k;
             	   
-                TData* Dj = &mSource.GetListElement(mQueue[j]);
-
 	            if ( j < (mSize - 1) )      // 'usedSize' may not be 0 here. 
-                {
-                    if ( mComparator.IsPredecessor(mSource.GetListElement(mQueue[1+j]), *Dj) ) 
+                    if ( mComparator.IsPredecessor(mSource.GetListElement(mQueue[1+j]), mSource.GetListElement(mQueue[j])) ) 
 	                { /* there is a child '1+j' and it is smaller than child 'j' */
 		                ++j;
 	                }
-                }
-            	   
-                if ( !mComparator.IsPredecessor(*Dj, mSource.GetListElement(SourceIdx)) )
+
+				if ( !mComparator.IsPredecessor(mSource.GetListElement(mQueue[j]), mSource.GetListElement(SourceIdx)) )
 	            { /* heap condition fulfilled */
 		            break;
 	            }
