@@ -15,7 +15,7 @@
 
 
 
-    ========== licence begin  GPL
+    ========== licence begin  GPL 
     Copyright (c) 1999-2004 SAP AG
 
     This program is free software; you can redistribute it and/or
@@ -1171,7 +1171,8 @@ SAPDBMem_RawAllocator::SAPDBMem_RawAllocator
     SAPDB_ULong                FirstBlockSize,
     SAPDB_ULong                SupplementBlockSize,
     FreeRawExtendsEnum         FreeRawExtends,
-    SAPDB_ULong                MaxSize
+    SAPDB_ULong                MaxSize,
+    bool                       DoRegister
  ) 
  : m_BlockAllocator(NULL)
  , m_RawAllocator(&RawAllocator)
@@ -1187,6 +1188,7 @@ SAPDBMem_RawAllocator::SAPDBMem_RawAllocator
  , m_freeRawExtends(FreeRawExtends)
  , m_opMessages(true)
  , m_writeProtected(false)
+ , m_doUnregister(DoRegister)
  , m_monitorCallStackLevel(0)
  , m_errorCount(0)
  , m_badAllocFunc(SAPDBMem_RawAllocator::InitBadAllocHandler())
@@ -1213,7 +1215,10 @@ SAPDBMem_RawAllocator::SAPDBMem_RawAllocator
     m_AllocatorInfo = RTEMem_AllocatorInfo(&m_Name[0], this, RawAllocator.GetIdentifier());
 
 #if defined(KERNEL)
-    RTEMem_AllocatorRegister::Instance().Register(m_AllocatorInfo);
+    if (DoRegister)
+    {
+        RTEMem_AllocatorRegister::Instance().Register(m_AllocatorInfo);
+    }
 #endif    
 }
 
@@ -1243,6 +1248,7 @@ SAPDBMem_RawAllocator::SAPDBMem_RawAllocator
  , m_freeRawExtends(FreeRawExtends)
  , m_opMessages(true)
  , m_writeProtected(false)
+ , m_doUnregister(true)
  , m_monitorCallStackLevel(0)
  , m_errorCount(0)
  , m_badAllocFunc(SAPDBMem_RawAllocator::InitBadAllocHandler())
@@ -1297,6 +1303,7 @@ SAPDBMem_RawAllocator::SAPDBMem_RawAllocator
  , m_freeRawExtends(FreeRawExtends)
  , m_opMessages(true)
  , m_writeProtected(false)
+ , m_doUnregister(true)
  , m_monitorCallStackLevel(0)
  , m_errorCount(0)
  , m_badAllocFunc(SAPDBMem_RawAllocator::InitBadAllocHandler())
@@ -1314,6 +1321,7 @@ SAPDBMem_RawAllocator::SAPDBMem_RawAllocator
 #if defined(KERNEL)
   SetIdentifier(Identifier);
   m_AllocatorInfo = RTEMem_AllocatorInfo( &m_Name[0], this, BlockAllocator.GetIdentifier());
+
   RTEMem_AllocatorRegister::Instance().Register(m_AllocatorInfo);
 #endif
   if (m_checkFlags & FL_DOUBLE_BOOKKEEPING)
@@ -1334,13 +1342,6 @@ INLINE SAPDBMem_RawAllocator::ChunkSize SAPDBMem_RawAllocator::AlignedSize(SAPDB
   (((req) + (sizeof(SAPDBMem_RawAllocator::ChunkSize) + MALLOC_ALIGN_MASK)) & ~(MALLOC_ALIGN_MASK)));
 }
 
-/*-------------------------------------------------------------------*/
- 
-INLINE void SAPDBMem_RawAllocator::DecrementUsed(SAPDBMem_RawAllocator::ChunkSize decrement)
-{
-    m_heapInfo.used -= decrement;
-}
-
 /*-------------------------------------------------------------------*/
 
 INLINE void SAPDBMem_RawAllocator::FillFreeChunk(void* p, int offset, int size, tsp00_Uint4 pattern)
@@ -1457,15 +1458,6 @@ bool SAPDBMem_RawAllocator::IsWriteProtected() const
 
 /*-------------------------------------------------------------------*/
 
-INLINE void SAPDBMem_RawAllocator::IncrementUsed(SAPDBMem_RawAllocator::ChunkSize increment)
-{
-  m_heapInfo.used += increment;
-  if (m_heapInfo.used > m_heapInfo.maxused)
-      m_heapInfo.maxused = m_heapInfo.used;
-}
-
-/*-------------------------------------------------------------------*/
-
 SAPDB_UInt4 SAPDBMem_RawAllocator::GetCheckLevel()
 {
 #if defined(KERNEL)
@@ -1677,7 +1669,10 @@ bool SAPDBMem_RawAllocator::CheckPointer(void* p, bool abort)
 SAPDBMem_RawAllocator::~SAPDBMem_RawAllocator() 
 {
 #if defined(KERNEL)
-  RTEMem_AllocatorRegister::Instance().Deregister(m_AllocatorInfo);
+  if (m_doUnregister)
+  {
+      RTEMem_AllocatorRegister::Instance().Deregister(m_AllocatorInfo);
+  }
 #endif
   this->Destructor();
 }
