@@ -128,8 +128,11 @@ private :
  */
 
 
-/*! define 4 gigabyte */
-#define GB4 0xFFFFFFFF
+#if defined(BIT64)
+#define SAPDBMEM_ALLOCATOR_UNLIMITED 0xFFFFFFFFFFFFFFFFUL
+#else
+#define SAPDBMEM_ALLOCATOR_UNLIMITED 0xFFFFFFFF
+#endif
 
 #if ASSERT_STATE_OPTION == SAPDBERR_OFF
 #undef DEBUG_BABDL
@@ -234,7 +237,8 @@ public:
     SAPDB_ULong                FirstBlockSize,
     SAPDB_ULong                SupplementBlockSize,
     FreeRawExtendsEnum         FreeRawExtends,
-    SAPDB_ULong                MaxSize = GB4
+    SAPDB_ULong                MaxSize = SAPDBMEM_ALLOCATOR_UNLIMITED,
+    bool                       DoRegister = true
     ); 
 
 
@@ -255,7 +259,7 @@ public:
     SAPDB_ULong                FirstBlockSize,
     SAPDB_ULong                SupplementBlockSize,
     FreeRawExtendsEnum         FreeRawExtends,
-    SAPDB_ULong                MaxSize = GB4
+    SAPDB_ULong                MaxSize = SAPDBMEM_ALLOCATOR_UNLIMITED
     ); 
   
   /*!
@@ -277,7 +281,7 @@ public:
     SAPDB_ULong                FirstBlockSize,
     SAPDB_ULong                SupplementBlockSize,
     FreeRawExtendsEnum         FreeRawExtends,
-    SAPDB_ULong                MaxSize = GB4
+    SAPDB_ULong                MaxSize = SAPDBMEM_ALLOCATOR_UNLIMITED
     ); 
 
    /*!
@@ -563,6 +567,51 @@ public:
   /* aix compiler                                                    */
   class   CChunk;
   typedef CChunk* CChunkPtr; //!< a pointer to a chunk
+
+protected:
+
+/*! define inline */
+
+#define INLINE inline 
+  
+   // PTS 1129440
+   /*! protected methods to set the heapInfo statistics */
+
+   INLINE void IncrementUsed(ChunkSize increment)         //!< increments the used memory counter
+   {
+     m_heapInfo.used += increment;
+     if (m_heapInfo.used > m_heapInfo.maxused)
+       m_heapInfo.maxused = m_heapInfo.used;
+   }
+   INLINE void DecrementUsed(ChunkSize decrement)         //!< decrements the used memory counter
+   {
+     m_heapInfo.used -= decrement;
+   }
+   INLINE void IncrementAllocated(ChunkSize increment)    //!< increments the allocated memory counter
+   {
+      m_heapInfo.allocated += increment;
+   }   
+   INLINE void DecrementAllocated(ChunkSize decrement)    //!< decrements the allocated memory counter
+   {
+     m_heapInfo.allocated -= decrement;
+   }
+   INLINE void IncrementAllocCnt()    //!< increments the allocation counter
+   {
+     ++m_heapInfo.cntAlloc;
+   }
+   INLINE void IncrementDeallocCnt()  //!< increments the deallocation counter
+   {
+     ++m_heapInfo.cntDealloc;
+   }
+   INLINE void IncrementRawAllocCnt()    //!< increments allocate calls to raw allocator counter
+   {
+     ++m_heapInfo.cntRawAlloc;
+   }
+   INLINE void IncrementRawDeallocCnt()  //!< increments deallocate calls to raw allocator counter
+   {
+     ++m_heapInfo.cntRawDealloc;
+   }
+
 private:
     /*! defines allocator check levels */ 
     enum checkFlags { 
@@ -584,10 +633,6 @@ private:
 
   struct  TNodeBuffer;
 
-/*! define inline */
-
-#define INLINE inline 
-
   /*!
   @brief manages free chunks
   */
@@ -639,7 +684,6 @@ private:
   INLINE void             CheckNoMansLand(CChunkPtr pChunk);                         //!< checks the no mans land pattern  
          bool             CheckPointer(void* p, bool abort = true);                  //!< verifies a pointer, must lie in any raw chunk 
          void             DeallocateRaw(void* p, SAPDB_ULong BlockCount);            //!< returns memory to the base allocator
-  INLINE void             DecrementUsed(ChunkSize decrement);                        //!< decrements the used memory counter
          void             Destructor();                                              //!< destructs the allocator
          void             Dump();                                                    //!< dumps the allocator using m_tracer
          void             DumpChunk(void* p, int length);                            //!< dumps a chunk using m_tracer
@@ -653,7 +697,6 @@ private:
   static SAPDB_UInt4      GetCheckLevel();                           //!< return the current check level
   INLINE IteratorState&   GetIteratorState();                        //!< returns the current iterator state
   INLINE SAPDB_UInt4      Idx2BinBlock(int);                         //!< index
-  INLINE void             IncrementUsed(ChunkSize increment);        //!< increments the used byte counter
   static BadAllocFunc*    InitBadAllocHandler();                     //!< returns the initial bad alloc handler
          bool             MallocExtend(ChunkSize required);          //!< required memory from the base allocator
          void             NewNodeBuffer();                           //!< requests a new buffer for nodes of tree managing big chunks
@@ -697,6 +740,7 @@ private:
   FreeRawExtendsEnum          m_freeRawExtends;          //!< return free base chunks to base allocator ?
   bool                        m_opMessages;              //!< if true, op messages are written to the knldiag in error conditions
   bool                        m_writeProtected;          //!< true if allocator is write protected
+  bool                        m_doUnregister;            //!< true, if allocator has to be unregistered at destruction
   int                         m_monitorCallStackLevel;   //!< depth of back trace
   int                         m_errorCount;              //!< numbers of error detected
   CFreeChunkLists             m_freeLists;               //!< free lists of small chunks
