@@ -178,13 +178,13 @@ public:
                     by the same allocator 'm_Allocator'.
                   - Nothing happens with the initially hosted object.
     arguments:    pmem [in] pointer to memory block
-    return:       none
+    return:       assigned pointer
     --------------------------------------------------------------------------*/
-    void SetPtr
+    Pointer SetPtr
         (Pointer                        pmem)
     {
         SAPDBERR_ASSERT_STATE(m_Allocator != 0);
-        m_DataPtr = pmem;
+        return m_DataPtr = pmem;
     }
 
     /*!-------------------------------------------------------------------------
@@ -517,13 +517,13 @@ public:
                     by the same allocator 'm_Allocator'.
                   - Nothing happens with the initially hosted object.
     arguments:    ptr [in] pointer to object
-    return:       none
+    return:       assigned pointer
     --------------------------------------------------------------------------*/
-    void SetPtr
+    Pointer SetPtr
         (Pointer                        pobj)
     {
         SAPDBERR_ASSERT_STATE(m_Allocator != 0);
-        m_ObjectPtr = pobj;
+        return m_ObjectPtr = pobj;
     }
 
     /*!-------------------------------------------------------------------------
