@@ -1546,34 +1546,31 @@ SAPDB_OStream & operator<<( SAPDB_OStream & os, const tgg07_StratEnum &value )
         case strat_key_equal:
             os << "strat_key_equal";
             break;
-        case strat_key_equal_sub:
-            os << "strat_key_equal_sub";
+        case strat_key_subq:
+            os << "strat_key_subq";
             break;
         case strat_key_in:
             os << "strat_key_in";
             break;
-             /***/
+             /* */
         case  strat_key_range:
             os << "strat_key_range";
             break;
-        case  strat_key_range_sub:
-            os << "strat_key_range_sub";
-            break;
-             /***/
+             /* */
         case strat_inv_range:
             os << "strat_inv_range";
             break;
-        case strat_inv_range_sub:
-            os << "strat_inv_range_sub";
+        case strat_inv_subq:
+            os << "strat_inv_subq";
             break;
         case strat_inv_in:
             os << "strat_inv_in";
             break;
-             /***/
+             /* */
         case strat_undecided:
             os << "strat_undecided";
             break;
-             /***/
+             /* */
         case strat_key_range_fetch:
             os << "strat_key_range_fetch";
             break;
@@ -1583,7 +1580,7 @@ SAPDB_OStream & operator<<( SAPDB_OStream & os, const tgg07_StratEnum &value )
         case strat_inv_range_merge_fetch:
             os << "strat_inv_range_merge_fetch";
             break;
-             /***/
+             /* */
         case strat_join_key_equal:
             os << "strat_join_key_equal";
             break;
@@ -1608,15 +1605,15 @@ SAPDB_OStream & operator<<( SAPDB_OStream & os, const tgg07_StratEnum &value )
         case strat_join_inv_range:
             os << "strat_join_inv_range";
             break;
-             /***/
+             /* */
         case strat_more_than_one:
             os << "strat_more_than_one";
             break;
-             /***/
+             /* */
         case strat_no_result:
             os << "strat_no_result";
             break;
-             /***/
+             /* */
         case strat_catalog:
             os << "strat_catalog";
             break;
@@ -2112,14 +2109,14 @@ SAPDB_OStream & operator<< (SAPDB_OStream &os, const tgg00_MessType2 &value)
         case mm_last :
             os << "mm_last";
             break;
-        case mm_free1 :
-            os << "mm_free1";
+        case mm_operator_join :
+            os << "mm_operator_join";
             break;
         case mm_lock :
             os << "mm_lock";
             break;
-        case mm_free6 :
-            os << "mm_free6";
+        case mm_first_operator_join :
+            os << "mm_first_operator_join";
             break;
         case mm_lock_timeout :
             os << "mm_lock_timeout";
@@ -2145,8 +2142,8 @@ SAPDB_OStream & operator<< (SAPDB_OStream &os, const tgg00_MessType2 &value)
         case mm_ok_not_allowed :
             os << "mm_ok_not_allowed";
             break;
-        case mm_free2 :
-            os << "mm_free2";
+        case mm_last_operator_join :
+            os << "mm_last_operator_join";
             break;
         case mm_optimize :
             os << "mm_optimize";
