@@ -52,12 +52,14 @@ SQLManErr_Instance::SQLManErrorObject SQLManErr_Instance::errors [] =
     {e_requested_dbperror,false,sqlm_internal,-28999,"required DBPError","requested_dbperror"},
     {e_invalid_deallocate,false,sqlm_internal,-28951,"System error : invalid deallocation","invalid_deallocate"}, 
     {e_missing_object_flush,false,sqlm_internal,-28950,"System error : missing object flush during commit","missing_object_flush"},
+    {e_stop_0,false,sqlm_internal,-28910,"STOP(0) not allowed","stop_0_not_allowed"},
     {e_dbfunc_code_not_found,false,sqlm_internal,-28906,"DBFunction code not found","dbfunc_code_not_found"},
     {e_dbfunc_index_error,false,sqlm_internal,-28905,"DBFunction called for indexing failed","dbfunc_index_error"},
     {e_dbfunc_with_sql_not_allowed,false,sqlm_internal,-28904,"DBFunction containing sql statements not allowed","dbfunc_with_sql_not_allowed"},
     {e_missing_return,false,sqlm_internal,-28903,"DBFunction $1 must return a value","missing_return"},
     {e_too_many_catch_levels,false,sqlm_internal,-28902,"Too many nested try/catch statements","too_many_catch_levels"},
     {e_case_not_found,false,sqlm_internal,-28901,"No case match found","case_not_found"},
+    {e_key_still_visible,false,sqlm_internal,-28835,"Key of new object is still visible within consistent version","key_still_visible"},
     {e_container_dropped,false,sqlm_internal,-28832,"Tried to access already dropped container","container_dropped"},
     {e_wrong_obj_count,false,sqlm_internal,-28831,"Wrong page count (internal error)","wrong_obj_count"},
     {e_inconsistent_drop_info,false,sqlm_internal,-28828,"Inconsistent information concerning 'is container dropped'","inconsistent_drop_info"},
@@ -152,9 +154,10 @@ SQLManErr_Instance::SQLManErrorObject SQLManErr_Instance::errors [] =
     {e_too_many_intern_parsinfos,false,sqlm_internal,-9411,"System error: AK Too many intern parsinfos","too_many_intern_parsinfos"},
     {e_invalid_segment_construction,false,sqlm_internal,-9409,"System error: AK Invalid segment construction","invalid_segment_construction"},
     {e_unitrans_src_too_short,false,sqlm_internal,-9408,"System error: AK Unitrans SRC too short","unitrans_src_too_short"},
+    {e_system_error,false,sqlm_internal,-9407,"System error: unexpected error","internal_error"},
     {-9406,false,sqlm_internal,-9406,"System error: DDL trigger failed"},
     {e_too_short_datapart,false,sqlm_internal,-9405,"System error: AK Datapart too short","too_short_datapart"},
-    {e_system_error,false,sqlm_internal,-9404,"System error: AK System error","system_error"},
+    {e_ak_system_error,false,sqlm_internal,-9404,"System error: AK System error","system_error"},
     {e_unknown_strategy,false,sqlm_internal,-9402,"System error: AK Unexpected strategy information","unknown_strategy"},
     {e_cachedirectory_full,false,sqlm_internal,-9400,"System error: AK Cachedirectory full","cachedirectory_full"},
     {e_nil_transindex,false,sqlm_internal,-9232,"System error: KB Nil transindex","nil_transindex"},
@@ -220,6 +223,12 @@ SQLManErr_Instance::SQLManErrorObject SQLManErr_Instance::errors [] =
     {e_illegal_branchlength,false,sqlm_internal,-9002,"System error: BD Illegal branchlength","illegal_branchlength"},
     {e_invalid_root,false,sqlm_internal,-9001,"System error: BD Invalid root","invalid_root"},
     {e_not_implemented,false,sqlm_internal,-9000,"System error: Not yet implemented","not_implemented"},
+    /* 
+     * -8052 used for error message returned by substitute triggers
+     * that were created for broken triggers during the unicode
+     * migration; message is 'trigger dropped during migration; check
+     * view migrationerrorprotocol'
+    */
     {e_no_snapshot_available,false,sqlm_internal,-8051,"No snapshot available","no_snapshot_available"},
     {e_migration_error,false,sqlm_internal,-8050,"Migration error","migration_error"},
     {e_not_prepared_for_migration,false,sqlm_internal,-8049,"Prepare migration required","not_prepared_for_migration"},
@@ -270,7 +279,7 @@ SQLManErr_Instance::SQLManErrorObject SQLManErr_Instance::errors [] =
     {e_backup_not_running,false,sqlm_internal,-7078,"No backup is running","backup_not_running"},
     {e_synonym_not_allowed,false,sqlm_internal,-7077,"Synonym not allowed","synonym_not_allowed"},
     {e_no_data_on_tape,false,sqlm_internal,-7076,"No more data to read from tape","no_data_on_tape"},
-    {e_save_skipped,false,sqlm_internal,-7075,"Current SAVE SKIPPED, next is ready to take on this tape","save_skipped"},
+    {e_save_skipped,false,sqlm_internal,-7075,"Current save skipped, next is ready to take on this tape","save_skipped"},
     {e_role_cycle,false,sqlm_internal,-7074,"Cycle in granted roles not allowed","role_cycle"},
     {e_too_many_serials,false,sqlm_internal,-7072,"More than one column with default SERIAL not allowed","too_many_serials"},
     {e_autosave_running,false,sqlm_internal,-7071,"AUTOSAVE LOG must be stopped","autosave_running"},
