@@ -53,7 +53,7 @@
 #include "SAPDBCommon/ErrorsAndMessages/SAPDBErr_MessageEventConverter.hpp"
 
 #include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"
-//#include "SAPDBCommon/Tracing/SAPDBTrace_Hex.hpp"
+#include "SAPDBCommon/Tracing/SAPDBTrace_Hex.hpp"
 #include "SQLManager/SQLMan_Context.hpp"
 #include "SQLManager/SQLMan_Tracing.hpp"
 #include "SQLManager/SQLMan_Messages.hpp"
@@ -69,8 +69,8 @@
 #include "RunTime/RTE_Message.hpp"
 #include "KernelCommon/Kernel_OpMsg.hpp"
 #include "RunTime/RTE_MySQLProxy.h"
-#include "Join/Join_InvSelectIterator.hpp"
 #include "Join/Join_HashAccessAllocator.hpp"
+#include "IOManager/IOMan_IDataInfo.hpp"
 
 // for test purposes the shared sql allocator can be write protected
 // except in the shared sql interface routines. This allows to find
@@ -81,17 +81,17 @@
 // DirkT: To switch on SharedSQL Protection use praeprozesor define in SharedSQL_Types.hpp instead of the following !!!
 //#define SHAREDSQL_PROTECT 1  
 
-//#define SQLMAN_SHARED_SQL_TRACE(X) \
-//if (SharedSQL_Trace.TracesLevel (3)) {\
-//    SAPDBTrace_Stream stream (&SharedSQL_Trace.GetTracePrinter ());\
-//    stream << X << NewLine;\
-//}
-//
-//inline SAPDB_OStream & operator<< (SAPDB_OStream & ostream, const SharedSQL_ParseID & parseId)
-//{
-//    SAPDBTrace_Hex hex(&parseId, sizeof(parseId));
-//    return hex.Print(ostream);
-//}
+#define SQLMAN_SHARED_SQL_TRACE(X) \
+if (SharedSQL_Trace.TracesLevel (3)) {\
+    SAPDBTrace_Stream stream (&SharedSQL_Trace.GetTracePrinter ());\
+    stream << X << NewLine;\
+}
+
+inline SAPDB_OStream & operator<< (SAPDB_OStream & ostream, const SharedSQL_ParseID & parseId)
+{
+    SAPDBTrace_Hex hex(&parseId, sizeof(parseId));
+    return hex.Print(ostream);
+}
 
 #if defined (SHAREDSQL_PROTECT)
 class SQLMan_AllocatorWriteUnProtector
@@ -442,7 +442,7 @@ void* a101_GetExecuteHandle (
     
     SharedSQL_ParseID& ParseId      = *REINTERPRET_CAST(SharedSQL_ParseID*, &parseId);
 
-    //SQLMAN_SHARED_SQL_TRACE ("GetExecuteHandle " << ParseId);
+    SQLMAN_SHARED_SQL_TRACE ("GetExecuteHandle " << ParseId);
 
     SAPDB_Bool Valid, minus9;
     SharedSQL_IExecuteHandle* pExecuteHandle = sharedSQLCache.NewExecuteHandle(context.GetAllocator(), ParseId, (CountExecuting?true:false), Valid, minus9);
@@ -450,16 +450,16 @@ void* a101_GetExecuteHandle (
 	{
         if (!Valid)
         {
-            //SQLMAN_SHARED_SQL_TRACE ("Plan invalid , -9 = " << ToStr(minus9));
+            SQLMAN_SHARED_SQL_TRACE ("Plan invalid , -9 = " << ToStr(minus9));
             context.ThrowError (e_old_fileversion);
             return pExecuteHandle;
         }
 		ParseId = *(pExecuteHandle->GetMasterParseID());
-        //SQLMAN_SHARED_SQL_TRACE ("Plan valid, master parseid = " << ParseId << " handle " << ToStr(pExecuteHandle));
+        SQLMAN_SHARED_SQL_TRACE ("Plan valid, master parseid = " << ParseId << " handle " << ToStr(pExecuteHandle));
 	}
     else
     {
-        //SQLMAN_SHARED_SQL_TRACE ("no execute handle found");
+        SQLMAN_SHARED_SQL_TRACE ("no execute handle found");
     }
     return pExecuteHandle; 
 }
@@ -645,8 +645,11 @@ pasbool a101_PutParseId (
 
     SharedSQL_ParseID& ParseId = *REINTERPRET_CAST(SharedSQL_ParseID*, &parseId);
     SharedSQL_ModuleName Module;
-    Module.mName = acv.a_modul_name;  
-    Module.mSize = sizeof (acv.a_modul_name);
+    if (acv.a_modul_name != a01_il_b_identifier)
+    {
+        Module.mName = acv.a_modul_name;  
+        Module.mSize = sizeof (acv.a_modul_name);
+    }
     //SQLMAN_SHARED_SQL_TRACE ("PutParseId " << ParseId);
     return sharedSQLCache.PutParseID(cmdHandle, ParseId, Module, (0 != acv.a_initial_segment_header.sp1c_prepare()));
 	//if ( cmdHandle )
@@ -690,15 +693,16 @@ void a101_GetModuleName (
 pasbool a101_SetExecuteStatisticData (
     tak_all_command_glob  VAR_VALUE_REF  acv, 
     tak_parsid            VAR_VALUE_REF  parseId,
-    tsp00_Uint8                          ExecuteTimeSec,
-    tsp00_Uint8                          ExecuteTimeMicroSec,
-    tsp00_Int8                           ReadRowCount,
-    tsp00_Int8                           QualifiedRowCount,
-    tsp00_Int8                           VirtualReadCount,
-    tsp00_Int8                           PhysicalReadCount,
-    tsp00_Int8                           FetchRowCount,
-    tsp00_Int8                           SuspendCount,
-    tsp00_Int8                           WaitCount)
+    tgg00_Surrogate       VAR_VALUE_REF  CmdID,
+    tsp00_Int4                           ExecuteTimeSec,
+    tsp00_Int4                           ExecuteTimeMicroSec,
+    tsp00_Int4                           ReadRowCount,
+    tsp00_Int4                           QualifiedRowCount,
+    tsp00_Int4                           VirtualReadCount,
+    tsp00_Int4                           PhysicalReadCount,
+    tsp00_Int4                           FetchRowCount,
+    tsp00_Int4                           SuspendCount,
+    tsp00_Int4                           WaitCount)
 {
     SAPDBTRACE_ROUTINE_DEBUG ("a101_SetExecuteStatisticData", SQLMan_Trace, 1);
     //SAPDBERR_ASSERT_STATE( a01SharedSqlEnabled );
@@ -706,8 +710,10 @@ pasbool a101_SetExecuteStatisticData (
     SQLMan_Context& Context = SQLMan_Context::AcvToContext(acv);
     SharedSQL_ISQLCache& SharedSQLCache = Context.GetSharedSQL_SQLCache();
     SharedSQL_ParseID& ParseId = *REINTERPRET_CAST(SharedSQL_ParseID*, &parseId);
+
     return SharedSQLCache.SetExecuteStatisticData(
             ParseId,  
+            (SharedSQL_CommandID&) CmdID, // output parameter
             ExecuteTimeSec, ExecuteTimeMicroSec,
             ReadRowCount, QualifiedRowCount,
             VirtualReadCount, PhysicalReadCount, 
@@ -753,18 +759,18 @@ void a101_ReleaseExecuteHandle (
 
 /*----------------------------------------------------------------*/
 
-void a101_SetMaxParallelServers (
-    integer max)
+tsp00_Int4 a101_SetMaxParallelServers (
+    tsp00_Int4  max)
 {
     SAPDBTRACE_ROUTINE_DEBUG ("a101_SetMaxParallelServers", SQLMan_Trace, 1);
-    Join_InvSelectIterator::SetMaxServerTasks(max);
-}
-
-/*----------------------------------------------------------------*/
-
-externPascal pasbool a101_JoinParallelIndexEnabled()
-{
-    return Join_InvSelectIterator::ParallelEnabled();
+    if (max > IOMan_IDataInfo::GetInstance().ConfiguredDataVolumes() - 1)
+    {
+        return IOMan_IDataInfo::GetInstance().ConfiguredDataVolumes() - 1;
+    }
+    else
+    {
+        return max > 0 ? max : 0;
+    }
 }
 
 /*----------------------------------------------------------------*/
@@ -888,7 +894,7 @@ void a101_InvalidateAll (tak_all_command_glob  VAR_VALUE_REF  acv)
 
 void a101_ResetSharedSQLStatistics (tak_all_command_glob  VAR_VALUE_REF  acv)
 {
-    SAPDBTRACE_ROUTINE_DEBUG ("a101_InvalidateAll", SQLMan_Trace, 1);
+    SAPDBTRACE_ROUTINE_DEBUG ("a101_ResetSharedSQLStatistics", SQLMan_Trace, 1);
 
     SharedSQL_ISQLCache& sharedSQLCache = SQLMan_Context::AcvToContext(acv).GetSharedSQL_SQLCache();
     SQLMAN_UNPROTECT_ALLOCATOR (sharedSQLCache.GetCommandCacheBaseRawAllocator());    
