@@ -179,6 +179,8 @@ SharedSQL_CommandCache::SharedSQL_CommandCache( SAPDB_ULong KBSize, SharedSQL_Ha
     mNumberOfSlots(Slots),
     mFirstLRUCommand(0),
     mLastLRUCommand(0),
+    mMiddleLRUCommand(0),
+    mLRUCounter(0),
     mCacheInfo(),
     mCacheInfoReset()
 {
@@ -317,11 +319,11 @@ SharedSQL_IPrepareHandle* SharedSQL_CommandCache::NewPrepareHandle( SAPDBMem_IRa
                             ActCommand->Plan().Drop(true);
     	                    ActCommand->Statistics().IncTotalPrepareCount();
                             ActCommand->SetStatus(Preparing);
-	                        H->SetMustPrepare(SAPDB_TRUE); 
+                            H->SetMustPrepare(SAPDB_TRUE); 
                         }
                         else
                         {
-            				++mCacheInfoReset.SuccessfulAccessCountPrepare;
+            		    ++mCacheInfoReset.SuccessfulAccessCountPrepare;
                             ActCommand->Plan().SwitchLockToNonExclusive();
     	                    H->SetMustPrepare(SAPDB_FALSE); 
                         }
@@ -447,10 +449,12 @@ inline SharedSQL_CommandID SharedSQL_CommandCache::NewCommandID( void )
 inline SharedSQL_HashValue	SharedSQL_CommandCache::CalcHashValue( const SharedSQL_SQLContext& Context, const SharedSQL_SQLStmt& Stmt )
 {
     SAPDBTRACE_METHOD_DEBUG ("SharedSQL_CommandCache::CalcHashValue", SharedSQL_Trace, 8);
-    SAPDBERR_ASSERT_ARGUMENT( Stmt.mStmt != 0 );
+    SAPDBERR_ASSERT_ARGUMENT( Stmt.mStmt != 0 && Stmt.mStmtSize > 0 );
 
     SharedSQL_HashValue Hash = FNV_32_INIT;
-    FNV32Hash((void*)Stmt.mStmt, Stmt.mStmtSize+Stmt.mDescriptionSize, Hash); // mStmt points to memory allocated for mStmt+mDescription !!!
+    FNV32Hash((void*)Stmt.mStmt, Stmt.mStmtSize, Hash); 
+	if ( Stmt.mDescriptionSize > 0 )
+		FNV32Hash((void*)Stmt.mDescription, Stmt.mDescriptionSize, Hash); 
     FNV32Hash((void*)&Context.UserID, sizeof(Context.UserID), Hash);
     return Hash;
 }
@@ -589,11 +593,6 @@ inline SAPDB_Bool SharedSQL_CommandCache::CleanUpCommand( tgg00_TransContext& Tr
                     Deallocate(Command->SQLStmt().mStmt);
                     Command->SQLStmt().mStmt = 0;
                 }
-                //if (Command->SQLStmt().mDescription) // mStmt points to memory allocated for mStmt+mDescription !!!
-                //{
-                //    Deallocate(Command->SQLStmt().mDescription);
-                //    Command->SQLStmt().mDescription = 0;
-                //}
                 if ( Command->GetFlgExtern() )
                 {
                     DeleteUnloadedSQLStmt(TransContext, Command);
@@ -882,7 +881,10 @@ inline SAPDB_Bool SharedSQL_CommandCache::ReadSQLStmtFromTempFile( tgg00_TransCo
             return SAPDB_FALSE;
         }
     }
-    Stmt->mDescription = ((char*)Stmt->mStmt) + Stmt->mDescriptionSize; // mStmt point to the memory allocated for mStmt + mDescription !!!
+    if ( Stmt->mDescriptionSize )
+        Stmt->mDescription = ((char*)Stmt->mStmt) + Stmt->mStmtSize; // mStmt point to the memory allocated for mStmt + mDescription !!!
+    else
+        Stmt->mDescription = 0;
     return SAPDB_TRUE;
 }
 
@@ -1008,34 +1010,45 @@ inline void SharedSQL_CommandCache::LRUListMoveCommandToTop( SharedSQL_ICachedCo
     SAPDBTRACE_METHOD_DEBUG ("SharedSQL_CommandCache::LRUListMoveCommandToTop", SharedSQL_Trace, 5);
     SAPDBERR_ASSERT_ARGUMENT( Cmd != 0 );
 
-    SharedSQL_LockedScope Lock(mpLRULock, SharedSQL_LockedScope::Exclusive);
-
-    SharedSQL_CachedCommand* Command = REINTERPRET_CAST(SharedSQL_CachedCommand*, Cmd);
-
-    if ( mFirstLRUCommand != Command )
+    if ( Cmd->GetFlgLRUListOld() )  // dirty read
     {
-        // Detach
-        if ( Command->PrevLRUCommand() )
-		    Command->PrevLRUCommand()->NextLRUCommand() = Command->NextLRUCommand();
-	    if (Command->NextLRUCommand())
-		    Command->NextLRUCommand()->PrevLRUCommand() = Command->PrevLRUCommand();
-	    else 
-	    {
-		    if ( mLastLRUCommand == Command ) 
-			    mLastLRUCommand = Command->PrevLRUCommand();
-            else
-                RTE_Message(SAPDBErr_MessageList("SharedSQL", __CONTEXT__, SHAREDSQL_ERR_LRULIST));
-	    }
-        // Attach at first position
-	    Command->PrevLRUCommand() = 0;
-	    if ( mFirstLRUCommand )
+        SharedSQL_LockedScope Lock(mpLRULock, SharedSQL_LockedScope::Exclusive);
+        if ( Cmd->GetFlgLRUListOld() )  // double check
         {
-            mFirstLRUCommand->PrevLRUCommand() = Command;
-    	    Command->NextLRUCommand() = mFirstLRUCommand;
+
+            SharedSQL_CachedCommand* Command = REINTERPRET_CAST(SharedSQL_CachedCommand*, Cmd);
+
+            if ( mFirstLRUCommand != Command )
+            {
+                // Detach
+                if ( Command->PrevLRUCommand() )
+		            Command->PrevLRUCommand()->NextLRUCommand() = Command->NextLRUCommand();
+	            if ( Command->NextLRUCommand() )
+		            Command->NextLRUCommand()->PrevLRUCommand() = Command->PrevLRUCommand();
+	            else 
+	            {
+		            if ( mLastLRUCommand == Command ) 
+			            mLastLRUCommand = Command->PrevLRUCommand();
+                    else
+                        RTE_Message(SAPDBErr_MessageList("SharedSQL", __CONTEXT__, SHAREDSQL_ERR_LRULIST));
+	            }
+                // Attach at first position
+	            Command->PrevLRUCommand() = 0;
+	            if ( mFirstLRUCommand )
+                {
+                    mFirstLRUCommand->PrevLRUCommand() = Command;
+    	            Command->NextLRUCommand() = mFirstLRUCommand;
+                }
+                mFirstLRUCommand = Command;
+                if ( !mLastLRUCommand )
+                    mLastLRUCommand = Command;
+
+                Command->SetFlgLRUListOld(false);
+                SAPDBERR_ASSERT_STATE( mMiddleLRUCommand != 0 );
+                mMiddleLRUCommand->SetFlgLRUListOld(true);
+                mMiddleLRUCommand = mMiddleLRUCommand->PrevLRUCommand();
+            }
         }
-        mFirstLRUCommand = Command;
-        if ( !mLastLRUCommand )
-            mLastLRUCommand = Command;
     }
 }
 
@@ -1055,6 +1068,18 @@ inline void SharedSQL_CommandCache::LRUListAttachCommand( SharedSQL_CachedComman
     mFirstLRUCommand = Command;
     if ( !mLastLRUCommand )
         mLastLRUCommand = Command;
+
+    //
+    if ( !mMiddleLRUCommand )
+        mMiddleLRUCommand = Command;
+    ++mLRUCounter;
+    if ( mLRUCounter % SharedSQL_OldLRUPart == 0 )
+    {
+        SAPDBERR_ASSERT_STATE( mMiddleLRUCommand != 0 );
+        mMiddleLRUCommand->SetFlgLRUListOld(true);
+        mMiddleLRUCommand = mMiddleLRUCommand->PrevLRUCommand();
+        SAPDBERR_ASSERT_STATE( mMiddleLRUCommand != 0 );
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -1064,6 +1089,32 @@ inline void SharedSQL_CommandCache::LRUListDetachCommand( SharedSQL_CachedComman
     SAPDBTRACE_METHOD_DEBUG ("SharedSQL_CommandCache::LRUListDetachCommand", SharedSQL_Trace, 8);
     SAPDBERR_ASSERT_ARGUMENT( Command != 0 );
 
+    if ( Command->GetFlgLRUListOld() )
+    {
+        if ( mLRUCounter % SharedSQL_OldLRUPart != 0 )
+        {
+            SAPDBERR_ASSERT_STATE( mMiddleLRUCommand != 0 );
+            mMiddleLRUCommand->SetFlgLRUListOld(true);
+            mMiddleLRUCommand = mMiddleLRUCommand->PrevLRUCommand();
+            SAPDBERR_ASSERT_STATE( mMiddleLRUCommand != 0 );
+        }
+    }
+    else
+    {
+        if ( mLRUCounter % SharedSQL_OldLRUPart == 0 )
+        {
+            SAPDBERR_ASSERT_STATE( mMiddleLRUCommand != 0 );
+            if ( mMiddleLRUCommand = mMiddleLRUCommand->NextLRUCommand() )
+                mMiddleLRUCommand->SetFlgLRUListOld(false);
+        }
+        if ( Command == (SharedSQL_CachedCommand*)mMiddleLRUCommand )
+        {
+            SAPDBERR_ASSERT_STATE( mMiddleLRUCommand != 0 );
+            mMiddleLRUCommand = mMiddleLRUCommand->PrevLRUCommand();
+        }
+    }
+	--mLRUCounter;
+
     if ( Command->PrevLRUCommand() )
 		Command->PrevLRUCommand()->NextLRUCommand() = Command->NextLRUCommand();
 	else 
