@@ -1059,6 +1059,200 @@ SAPDB_Bool StudioOAL_WResult::openResult(const StudioOAL_String &sCmdStr)
     }    
     return SAPDB_TRUE;
 
+}
+/*===========================================================================*
+ *  StudioOAL_WResult::openResultPrepare
+ *===========================================================================*/
+SAPDB_Bool StudioOAL_WResult::openResultPrepare(const StudioOAL_String &sCmdStr)
+{ 
+ 
+    //check it
+	if( m_pError == NULL) return SAPDB_FALSE;
+    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV)  return SAPDB_FALSE;
+	
+    //init	variables
+	SQLRETURN  nRc = SQL_SUCCESS;    
+    m_sStatement   = sCmdStr;
+    m_lFetchedRows = 0;
+    m_bNoRows      = SAPDB_TRUE;	
+
+    //if there is a open result the close it
+    if(m_hStmt != SQL_NULL_HSTMT) 
+	   closeResult();
+
+    //allocate handle for statement    
+	nRc = SQLAllocHandle(SQL_HANDLE_STMT,m_hDbc,&m_hStmt);
+	if( nRc != SQL_SUCCESS)
+	{
+		if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+
+			SQLFreeStmt(m_hStmt, SQL_DROP);
+			m_hStmt = SQL_NULL_HSTMT;
+			return SAPDB_FALSE;
+		}
+	}
+	
+	nRc = SQLSetStmtOption(m_hStmt,SQL_SQLMODE,getSqlMode()); 
+    if( nRc != SQL_SUCCESS ) {
+    
+		if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+    
+            SQLFreeStmt(m_hStmt, SQL_DROP);
+			m_hStmt = SQL_NULL_HSTMT;
+            return SAPDB_FALSE;
+        }
+    }
+
+    nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_ROW_BIND_TYPE, SQL_BIND_BY_COLUMN);
+    if( nRc != SQL_SUCCESS ) {
+    
+		if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+    
+            SQLFreeStmt(m_hStmt, SQL_DROP);
+			m_hStmt = SQL_NULL_HSTMT;
+            return SAPDB_FALSE;
+        }
+    }
+    
+    switch (m_enumResultCursorType)
+    {
+        case stoalCursorForwardOnly:
+    
+            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CONCURRENCY, SQL_CONCUR_READ_ONLY);
+            if( nRc != SQL_SUCCESS ) {
+                
+                if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+        
+                    SQLFreeStmt(m_hStmt, SQL_DROP);
+					m_hStmt = SQL_NULL_HSTMT;
+                    return SAPDB_FALSE;
+                }
+            }
+            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CURSOR_TYPE, SQL_CURSOR_FORWARD_ONLY);
+            if( nRc != SQL_SUCCESS ) {
+              if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+    
+                    SQLFreeStmt(m_hStmt, SQL_DROP);
+					m_hStmt = SQL_NULL_HSTMT;
+                    return SAPDB_FALSE;
+                }
+            }
+
+			break;
+        
+        case stoalCursorStatic:
+
+            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CONCURRENCY, SQL_CONCUR_ROWVER);
+            if( nRc != SQL_SUCCESS ) {
+                if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+        
+                    SQLFreeStmt(m_hStmt, SQL_DROP);
+					m_hStmt = SQL_NULL_HSTMT;
+                    return SAPDB_FALSE;
+                }
+            }
+            
+            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CURSOR_TYPE, SQL_CURSOR_STATIC);
+            if( nRc != SQL_SUCCESS ) {
+                
+                if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+                
+                    SQLFreeStmt(m_hStmt, SQL_DROP);
+					m_hStmt = SQL_NULL_HSTMT;
+                    return SAPDB_FALSE;
+                }
+            }
+			break;
+            
+		case stoalOptimizeCursor:
+    
+			nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CONCURRENCY, SQL_CONCUR_READ_ONLY);
+			if( nRc != SQL_SUCCESS ) {
+            
+				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+    
+					SQLFreeStmt(m_hStmt, SQL_DROP);
+					m_hStmt = SQL_NULL_HSTMT;
+					return SAPDB_FALSE;
+				}
+			}
+        
+			nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_OPTIMIZE_CURSOR, 1);
+			if( nRc != SQL_SUCCESS ) {
+        
+				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+
+					//Die Option ExtendedFetch ohne for reuse gibts erst ab 6.2.10
+					if(m_pError->getSQLStateErrorNumber() == "S1092") {  // Out of Option range
+							
+                
+						SQLFreeStmt(m_hStmt, SQL_DROP);
+						m_hStmt = SQL_NULL_HSTMT;
+						return SAPDB_FALSE;
+					}
+				}
+			}
+			nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CURSOR_TYPE, SQL_CURSOR_KEYSET_DRIVEN);
+			if( nRc != SQL_SUCCESS ) {
+        
+				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+
+					SQLFreeStmt(m_hStmt, SQL_DROP);
+					m_hStmt = SQL_NULL_HSTMT;
+					return SAPDB_FALSE;
+				}
+			}
+			break;
+         
+		default:
+			return SAPDB_FALSE;
+			break;   
+   }
+
+   nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_USE_BOOKMARKS, SQL_UB_ON);
+   if( nRc != SQL_SUCCESS ) {
+
+        if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+    
+            SQLFreeStmt(m_hStmt, SQL_DROP);
+			m_hStmt = SQL_NULL_HSTMT;
+            return SAPDB_FALSE;
+        }
+    }
+    
+    nRc = SQLSetStmtOption(m_hStmt, SQL_ROWSET_SIZE, 1);
+    if( nRc != SQL_SUCCESS ) {
+
+        if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
+    
+            SQLFreeStmt(m_hStmt, SQL_DROP);
+			m_hStmt = SQL_NULL_HSTMT;
+            return SAPDB_FALSE;
+        }
+    }
+    
+    //prepare the result set
+    if(!prepareStmt()){
+
+        //free statement
+        SQLFreeStmt(m_hStmt, SQL_DROP);
+		m_hStmt = SQL_NULL_HSTMT;
+        return SAPDB_FALSE;
+    }
+    
+    if(!getColDescriptions()) 	return SAPDB_FALSE;
+	
+    //is there a result
+	m_lRowCount = rowCount(&m_hStmt);
+    if( m_lRowCount == 0 ) {
+        
+        return SAPDB_TRUE;
+	}
+	else{
+        m_bNoRows = SAPDB_FALSE;
+    }    
+    return SAPDB_TRUE;
+
 }
 /*===========================================================================*
  *  StudioOAL_WResult::rowCount()
@@ -1254,6 +1448,92 @@ SAPDB_Bool StudioOAL_WResult::execStmt()
 
 	return SAPDB_TRUE;	
 }
+/*===========================================================================*
+ *  StudioOAL_WResult::prepareStmt()
+ *===========================================================================*/
+SAPDB_Bool StudioOAL_WResult::prepareStmt() 
+{    
+	/*-------------------------------------------------------------------------*
+     * check if pointer to error class is not null 						       *
+     *-------------------------------------------------------------------------*/
+	if (m_pError == NULL)	return SAPDB_FALSE;	
+
+	/*-------------------------------------------------------------------------*
+     * check if dbc handle and enviroment handle are not null 			       *
+     *-------------------------------------------------------------------------*/
+    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV) return SAPDB_FALSE;	
+
+	/*-------------------------------------------------------------------------*
+     * local variables										 			       *
+     *-------------------------------------------------------------------------*/
+	SQLRETURN	nRc = 0;	
+	tsp81_UCS2Char * pConvertOut = NULL;
+	
+   /*-------------------------------------------------------------------------*
+    * use a copy for conversion	to ucs2pointer								  *
+    *-------------------------------------------------------------------------*/
+	StudioOAL_String strTemp = m_sStatement;
+
+	/*-------------------------------------------------------------------------*
+     * init conversion buffer								 			       *
+     *-------------------------------------------------------------------------*/
+	memset(m_strCmdBuffer,0,sizeof(m_strCmdBuffer));
+
+	/*-------------------------------------------------------------------------*
+     * convert utf8 to ucs2									 			       *
+     *-------------------------------------------------------------------------*/
+	if (strTemp.ConvertToUCS2_Unaligned_Swap(m_strCmdBuffer,
+                                                 (m_strCmdBuffer+(sizeof(m_strCmdBuffer)/sizeof(SAPDB_UCS2))),
+                                                 pConvertOut) != Tools_UTF8Basis::Success )
+	{
+		return SAPDB_FALSE;		
+	}
+	
+	/*-------------------------------------------------------------------------*
+     * check if something was converted						 			       *
+     *-------------------------------------------------------------------------*/
+	if(pConvertOut<=m_strCmdBuffer) return SAPDB_FALSE;
+
+	/*-------------------------------------------------------------------------*
+         * prepare statement									 	       *
+         *-------------------------------------------------------------------------*/
+	nRc = SQLPrepareW(m_hStmt,(SQLWCHAR*)m_strCmdBuffer, ((SQLINTEGER)(pConvertOut-m_strCmdBuffer)));
+	if (nRc != SQL_SUCCESS) {
+    
+        if (!(m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    
+
+			return SAPDB_FALSE;
+		}
+	}    
+
+	return SAPDB_TRUE;	
+}
+/*===========================================================================*
+ *  StudioOAL_WResult::executePreparedStmt()
+ *===========================================================================*/
+SAPDB_Bool StudioOAL_WResult::executePreparedStmt() 
+{
+  if (m_hStmt == SQL_NULL_HSTMT)
+    return SAPDB_FALSE;
+  else {
+    SQLRETURN nRc = SQLExecute (m_hStmt);
+    if (nRc != SQL_SUCCESS) {
+      if (!(m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    
+        return SAPDB_FALSE;
+      }
+    } else {
+      //is there a result
+      m_lRowCount = rowCount(&m_hStmt);
+      if( m_lRowCount == 0 ) {
+        return SAPDB_TRUE;
+      }
+      else{
+        m_bNoRows = SAPDB_FALSE;
+      }    
+    }
+  }
+  return SAPDB_TRUE;	
+}
 /*===========================================================================*
  *  StudioOAL_WResult::fetch() 
  *===========================================================================*/
