@@ -103,6 +103,7 @@ StudioTD_String
 StudioTD_ColumnDef::genCreateStatement (StudioTD_Mode mode, SAPDB_Bool casesensitive)
 {
     StudioTD_String  sql;
+	StudioTD_String  escapedName;
     StudioTD_String  attributes;
     StudioTD_String  delimiter;
 
@@ -122,7 +123,9 @@ StudioTD_ColumnDef::genCreateStatement (StudioTD_Mode mode, SAPDB_Bool casesensi
      * Insert a TAB and the name of the column.                                *
      *-------------------------------------------------------------------------*/
     sql += "\t\"";
-    sql += CASE(mName);
+	escapedName = CASE(mName);
+	escapeQuoteChar(escapedName,"\"");
+    sql += escapedName;
     sql += "\"";
 
     /*-------------------------------------------------------------------------*
@@ -297,19 +300,35 @@ StudioTD_String StudioTD_ColumnDef::genCreateXML( SAPDB_Bool casesensitive )
    deployedcolumn += "";
    deployedcolumn += "\">\n";
 
-   // sapdbm:<data_typ>
+   // sdbm:<data_typ>
 
   /*-------------------------------------------------------------------------*
    * Insert the datatype fo the column.                                      *
    *-------------------------------------------------------------------------*/
 
-   deployedcolumn += "<sapdbm:";                         // SQLColAttribute 
-   deployedcolumn += mDataType.ToUpper();                // SQL_DESC_TYPE(ODBC 3.0), SQL_DESC_TYPE_NAME(ODBC 1.0)
+   deployedcolumn += "<sdbm:";                         // SQLColAttribute 
+   if (( mDataType == "Char" || mDataType == "Varchar") && 
+	     mCodeType == "BYTE" ) 
+         deployedcolumn += "BINARY";
+   else
+         deployedcolumn += mDataType.ToUpper();           // SQL_DESC_TYPE(ODBC 3.0), SQL_DESC_TYPE_NAME(ODBC 1.0)
    deployedcolumn += " oim:href=\"#_";
-   deployedcolumn += "sapdbm:";           
-   deployedcolumn += mDataType.ToUpper(); 
+   deployedcolumn += "sdbm:";   
+   if ((mDataType == "CHAR" || mDataType == "VARCHAR") ) 
+   {
+        if (mCodeType == "BYTE" )
+		{
+            deployedcolumn += "BINARY";
+		}
+        else 
+		if (mCodeType == "ASCII" || mCodeType == "UNICODE" )  
+        {
+            deployedcolumn += mDataType.ToUpper(); 
+	        deployedcolumn += "\" sdbm:Encoding=\"";         
+	        deployedcolumn += mCodeType;
+		}
+   }
    deployedcolumn += "\"/>\n";
-
    deployedcolumn += "</dbm:DeployedColumn>\n";
   
    return deployedcolumn;
@@ -324,6 +343,8 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
                                                        SAPDB_Bool casesensitive)
 {
     StudioTD_String  sql;
+	StudioTD_String  escapedOwner,escapedTable;
+	StudioTD_String  escapedName,escapedOldName;
     StudioTD_String  sql_default_attr;    // we cannot both add a new col and set its default, null, or
                                           // key attributes.
     StudioTD_String  sql_null_attr;
@@ -335,22 +356,31 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
     StudioTD_String  escaped_Default = mDefault;
     StudioTD_String  escaped_Comment = mComment;
 
-
     escapeQuoteChar(escaped_Default, "'");
     escapeQuoteChar(escaped_Comment, "'");
 
-
     intro  = "ALTER TABLE \"";
 
     if(owner != "")
     {
-        intro += owner;
+        escapedOwner = owner;
+		escapeQuoteChar(escapedOwner,"\"");
+		intro += escapedOwner;
         intro += "\".\"";
     }
 
-    intro += table;
+	escapedTable = CASE(table);
+	escapeQuoteChar(escapedTable,"\"");
+    intro += escapedTable;
     intro += "\" ";
 
+	escapedName = CASE(mName);
+	escapeQuoteChar(escapedName,"\"");
+
+	if(oldColumn != NULL){
+		escapedOldName = CASE(oldColumn->name());
+		escapeQuoteChar(escapedOldName,"\"");
+	}
     if (mAlterType == AlterChange)
     {
         /*-----------------------------------------------------------------------*
@@ -362,7 +392,7 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
         {
             sql += intro;
             sql += "MODIFY (\"";
-            sql += CASE(oldColumn->name());
+            sql += escapedOldName;
             sql += "\" ";
             sql += mDataType;
 
@@ -403,7 +433,7 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
         {
             sql += intro;
             sql += "COLUMN \"";
-            sql += CASE(oldColumn->name());
+            sql += escapedOldName;
             sql += "\" ";
             sql += (mNotNull ? "NOT" : "DEFAULT");
             sql += " NULL";
@@ -423,7 +453,7 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
 
             sql += intro;
             sql += "COLUMN \"";
-            sql += CASE(oldColumn->name());
+            sql += escapedOldName;
             sql += "\"";
 
             if (mDefault == "")
@@ -454,9 +484,10 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
         if (mComment != oldColumn->comment())
         {
             sql += "COMMENT ON COLUMN \"";
-            sql += CASE(table);
+            sql += escapedTable;
+
             sql += "\".\"";
-            sql += CASE(oldColumn->name());
+            sql += escapedOldName;
             sql += "\" IS '";
             sql += escaped_Comment;  
             sql += "'";
@@ -469,12 +500,12 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
         if (mName != oldColumn->name())
         {
             sql += "RENAME COLUMN \"";
-            sql += CASE(table);
+            sql += escapedTable;
             sql += "\".\"";
-            sql += CASE(oldColumn->name());
+            sql += escapedOldName;
             sql += "\" ";
             sql += "TO \"";
-            sql += CASE(mName);
+            sql += escapedName;
             sql += "\"";
             sql += SEP;
         }
@@ -492,7 +523,7 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
          *-----------------------------------------------------------------------*/
         sql += intro;
         sql += "ADD (\"";
-        sql += CASE(mName);
+        sql += escapedName;
         sql += "\" ";
         sql += mDataType;
 
@@ -533,7 +564,7 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
         {
             sql_null_attr += intro;
             sql_null_attr += "COLUMN \"";
-            sql_null_attr += CASE(mName);
+            sql_null_attr += escapedName;
             sql_null_attr += "\" ";
             sql_null_attr += "NOT NULL";
             sql_null_attr += SEP;
@@ -555,7 +586,7 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
 
             sql_default_attr += intro;
             sql_default_attr += "COLUMN \"";
-            sql_default_attr += CASE(mName);
+            sql_default_attr += escapedName;
             sql_default_attr += "\"";
             sql_default_attr += " ADD DEFAULT ";
             sql_default_attr += ( ( (mCodeType == "BYTE") && (SAPDB_TRUE == mbUserDefinedDefault) ) ? "x" : "" );
@@ -586,9 +617,10 @@ StudioTD_String StudioTD_ColumnDef::genAlterStatement (StudioTD_ColumnDef *oldCo
         if (!mComment.Empty())
         {
             sql += "COMMENT ON COLUMN \"";
-            sql += CASE(table);
+
+			sql += escapedTable;
             sql += "\".\"";
-            sql += CASE(mName);
+			sql += escapedName;
             sql += "\" IS '";
             sql += escaped_Comment;  
             sql += "'";
