@@ -230,8 +230,9 @@ StudioTD_String StudioTD_ConstraintDef::findIncomplete ()
 StudioTD_String StudioTD_ConstraintDef::genCreateStatement (StudioTD_Mode mode, SAPDB_Bool casesensitive)
 {
   StudioTD_String  sql;
+  StudioTD_String  escapedColumnName;
   StudioTD_String  conditions;
-  Studio_ListData   cond;
+  Studio_ListData  cond;
 
   /*-------------------------------------------------------------------------*
    * Generate the intro of the CONSTRAINT clause.                            *
@@ -244,7 +245,9 @@ StudioTD_String StudioTD_ConstraintDef::genCreateStatement (StudioTD_Mode mode,
   if (!mName.Empty())
   {
     sql += "\"";
-    sql += CASE(mName);
+	escapedColumnName = CASE(mName);
+    escapeQuoteChar(escapedColumnName,"\"");
+    sql += escapedColumnName;
     sql += "\" CHECK\t";
   }
   else
@@ -292,6 +295,7 @@ StudioTD_String StudioTD_ConstraintDef::genCreateXML (StudioTD_Mode mode, SAPDB_
 StudioTD_String StudioTD_ConstraintDef::genAlterStatement (SAPDB_Bool casesensitive)
 {
   StudioTD_String  sql;
+  StudioTD_String  escapedColumnName;
   SAPDB_Bool       bChanged = false;
   SAPDB_UInt       pos;
 
@@ -324,7 +328,9 @@ StudioTD_String StudioTD_ConstraintDef::genAlterStatement (SAPDB_Bool casesensit
     else
     {
       sql += "\"";
-      sql += CASE(cond->column());
+	  escapedColumnName = cond->column();
+	  escapeQuoteChar(escapedColumnName,"\"");
+      sql += CASE(escapedColumnName);
       sql += "\"";
     }
     
@@ -500,8 +506,8 @@ SAPDB_Bool StudioTD_ConstraintDef::extractPredicate(const StudioTD_String& token
  *===========================================================================*/
 void StudioTD_ConstraintDef::parseConditions (StudioTD_String conditions)
 {
-  Studio_List             tokens;
-  Studio_ListData         t;
+  Studio_List            tokens;
+  Studio_ListData        t;
   StudioTD_String        token;
   StudioTD_String        sep;
   StudioTD_String        s;
@@ -536,9 +542,10 @@ void StudioTD_ConstraintDef::parseConditions (StudioTD_String conditions)
      *-----------------------------------------------------------------------*/
     if (token[0] == (SAPDB_UTF8)'"')
     {
-      end = token.Find  ("\"", 1);
+	  end = token.Find ("\"", 1);
       cond->setColumn (token.SubStrBasis (1, end));
-      start = end+1;
+
+  	  start = end+1;
 	  if(extractPredicate(token,start,s,end) == SAPDB_FALSE){
 			return;
 	  }	
