@@ -102,7 +102,8 @@ StudioTD_String StudioTD_ForeignKeyDef::findIncomplete ()
 StudioTD_String StudioTD_ForeignKeyDef::genCreateStatement (StudioTD_Mode mode, SAPDB_Bool casesensitive)
 {
   StudioTD_String  sql;
-  SAPDB_UInt       start, end;
+  StudioTD_String  escapedName,escapedColumnName,escapedReference;
+  SAPDB_Int        start, end;
   
   /*-------------------------------------------------------------------------*
    * If referencing columns or reference is empty the foreign key is not     *
@@ -122,7 +123,9 @@ StudioTD_String StudioTD_ForeignKeyDef::genCreateStatement (StudioTD_Mode mode,
   if (!mName.Empty())
   {
     sql += "\"";
-    sql += CASE(mName);
+	escapedName = CASE(mName);
+	escapeQuoteChar(escapedName,"\"");
+    sql += escapedName;
     sql += "\"\t";
   }
   else
@@ -139,14 +142,18 @@ StudioTD_String StudioTD_ForeignKeyDef::genCreateStatement (StudioTD_Mode mode,
   while (end != StudioTD_String::NPos)
   {
     sql  += "\"";
-    sql  += CASE(mRefColumns.SubStrBasis (start, end));
+	escapedColumnName = CASE(mRefColumns.SubStrBasis (start, end));
+	escapeQuoteChar(escapedColumnName,"\"");
+    sql  += escapedColumnName;
     sql  += "\",";
     start = end + 1;
     end   = mRefColumns.Find (",", start);
   }
   
   sql += "\"";
-  sql += CASE(mRefColumns.SubStr (start));
+  escapedColumnName = CASE(mRefColumns.SubStr (start));
+  escapeQuoteChar(escapedColumnName,"\"");
+  sql  += escapedColumnName;
   sql += "\")";
 
   /*-------------------------------------------------------------------------*
@@ -160,15 +167,19 @@ StudioTD_String StudioTD_ForeignKeyDef::genCreateStatement (StudioTD_Mode mode,
   if (StudioTD_String::NPos != start)       // no owner name in reference
   {
     sql += "\"";
-    sql += mReference.SubStrBasis (0, start);
-    sql += "\".";
+	escapedReference = mReference.SubStrBasis (0, start);
+    escapeQuoteChar(escapedReference,"\"");
+    sql  += escapedReference;
+	sql += "\".";
   }
   else
   {
     start = -1;
   }
   sql += "\"";
-  sql += mReference.SubStrBasis (start + 1, end - 1);
+  escapedReference = mReference.SubStrBasis (start + 1, end - 1);
+  escapeQuoteChar(escapedReference,"\"");
+  sql  += escapedReference;
   sql += "\" (";
   
   start = end + 1;
@@ -177,14 +188,18 @@ StudioTD_String StudioTD_ForeignKeyDef::genCreateStatement (StudioTD_Mode mode,
   while (end != StudioTD_String::NPos)
   {
     sql  += "\"";
-    sql  += mReference.SubStrBasis (start, end);
+	escapedColumnName = mReference.SubStrBasis (start, end);
+	escapeQuoteChar(escapedColumnName,"\"");
+    sql  += escapedColumnName;
     sql  += "\",";
     start = end + 1;
     end   = mReference.Find (",", start);
   }
 
   sql += "\"";
-  sql += mReference.SubStrBasis (start, mReference.Length() - 1);
+  escapedColumnName = mReference.SubStrBasis (start, mReference.Length() - 1);
+  escapeQuoteChar(escapedColumnName,"\"");
+  sql  += escapedColumnName;
   sql += "\")";
 
   /*-------------------------------------------------------------------------*
@@ -209,8 +224,8 @@ StudioTD_String StudioTD_ForeignKeyDef::genCreateXML( StudioTD_String owner,Stud
    StudioTD_String   owner_referenced , tablename_referenced , id_referenced;
    StudioTD_String   owner_referencing, tablename_referencing, id_referencing;
 
-   SAPDB_UInt        start_referencing, end_referencing;
-   SAPDB_UInt        start_referenced , end_referenced;
+   SAPDB_Int         start_referencing, end_referencing;
+   SAPDB_Int         start_referenced , end_referenced;
 
    // comment();
 
@@ -349,23 +364,29 @@ StudioTD_String StudioTD_ForeignKeyDef::genCreateXML( StudioTD_String owner,Stud
 StudioTD_String StudioTD_ForeignKeyDef::genAlterStatement (StudioTD_String owner,StudioTD_String table, SAPDB_Bool casesensitive)
 {
   StudioTD_String   sql;
+  StudioTD_String   escapedTable,escapedOwner,escapedName,escapedColumnName,escapeReference;
   SAPDB_UInt        start, end;
   StudioTD_String   intro;
 
 
   StudioTD_String  escaped_Comment = mComment;
 
-  escapeQuoteChar(escaped_Comment,
-                  "'");
+  escapeQuoteChar(escaped_Comment,"'");
 
-  intro  = "ALTER TABLE \"";
-
-  if(owner != ""){
-      intro += owner;
+  intro  = "ALTER TABLE \""; 
+  if(owner != "")
+  {
+      escapedOwner = owner;
+	  escapeQuoteChar(escapedOwner,"\"");
+	  intro += escapedOwner;
       intro += "\".\"";
   }
-  intro += table;
+
+  escapedTable = CASE(table);
+  escapeQuoteChar(escapedTable,"\"");
+  intro += escapedTable;
   intro += "\" ";
+
   
   /*-------------------------------------------------------------------------*
    * If referencing columns or reference is empty the foreign key is not     *
@@ -386,7 +407,9 @@ StudioTD_String StudioTD_ForeignKeyDef::genAlterStatement (StudioTD_String owner
   if (!mName.Empty())
   {
     sql += "\"";
-    sql += CASE(mName);
+	escapedName = CASE(mName);
+	escapeQuoteChar(escapedName,"\"");
+    sql += escapedName;
     sql += "\" ";
   }
 
@@ -400,15 +423,19 @@ StudioTD_String StudioTD_ForeignKeyDef::genAlterStatement (StudioTD_String owner
   
   while (end != StudioTD_String::NPos)
   {
-    sql  += "\"";
-    sql  += CASE(mRefColumns.SubStrBasis (start, end));
+    sql  += "\"";    
+	escapedColumnName = CASE(mRefColumns.SubStrBasis (start, end));
+	escapeQuoteChar(escapedColumnName,"\"");
+	sql  += escapedColumnName;
     sql  += "\",";
     start = end + 1;
     end   = mRefColumns.Find (",", start);
   }
   
   sql += "\"";
-  sql += CASE(mRefColumns.SubStr (start));
+  escapedColumnName = CASE(mRefColumns.SubStr (start));
+  escapeQuoteChar(escapedColumnName,"\"");
+  sql += escapedColumnName;
   sql += "\")";
 
   /*-------------------------------------------------------------------------*
@@ -419,11 +446,26 @@ StudioTD_String StudioTD_ForeignKeyDef::genAlterStatement (StudioTD_String owner
   start = mReference.Find (".");
   end   = mReference.Find ("(");
   
-  sql += "\"";
-  sql += mReference.SubStrBasis (0, start);
-  sql += "\".\"";
-  sql += mReference.SubStrBasis (start + 1, end - 1);
-  sql += "\" (";
+  if ( start != StudioTD_String::NPos)
+  {
+	   sql += "\"";
+	   escapeReference = mReference.SubStrBasis (0, start);
+	   escapeQuoteChar(escapeReference,"\"");
+	   sql += escapeReference;
+	   sql += "\".\"";
+	   escapeReference = mReference.SubStrBasis (start + 1, end - 1);
+	   escapeQuoteChar(escapeReference,"\"");
+	   sql += escapeReference;
+	   sql += "\" (";
+  }
+  else 
+  {
+	   sql += "\"";
+	   escapeReference = mReference.SubStrBasis (0, end - 1);
+	   escapeQuoteChar(escapeReference,"\"");
+	   sql += escapeReference;
+	   sql  += "\" (";
+  }
   
   start = end + 1;
   end   = mReference.Find (",", start);
@@ -431,14 +473,18 @@ StudioTD_String StudioTD_ForeignKeyDef::genAlterStatement (StudioTD_String owner
   while (end != StudioTD_String::NPos)
   {
     sql  += "\"";
-    sql  += mReference.SubStrBasis (start, end);
+    escapedColumnName = mReference.SubStrBasis (start, end);
+	escapeQuoteChar(escapedColumnName,"\"");
+	sql  += escapedColumnName;
     sql  += "\",";
     start = end + 1;
     end   = mReference.Find (",", start);
   }
 
   sql += "\"";
-  sql += mReference.SubStrBasis (start, mReference.Length() - 1);
+  escapedColumnName = mReference.SubStrBasis (start, mReference.Length() - 1);
+  escapeQuoteChar(escapedColumnName,"\"");
+  sql += escapedColumnName;
   sql += "\")";
 
   /*-------------------------------------------------------------------------*
@@ -458,9 +504,9 @@ StudioTD_String StudioTD_ForeignKeyDef::genAlterStatement (StudioTD_String owner
   if ((!mName.Empty()) && (!mComment.Empty()))
   {
     sql += "COMMENT ON FOREIGN KEY \"";
-    sql += CASE(table);
+    sql += escapedTable;
     sql += "\".\"";
-    sql += CASE(mName);
+    sql += CASE(escapedName);
     sql += "\" IS '";
     sql += escaped_Comment; 
     sql += "'";
@@ -471,7 +517,6 @@ StudioTD_String StudioTD_ForeignKeyDef::genAlterStatement (StudioTD_String owner
   
 }
 
-
 /*===========================================================================*
  *     StudioTD_ForeignKeyDef::genDescription                                    *
  *===========================================================================*/
