@@ -240,10 +240,10 @@ StudioTD_ColumnDef * StudioTD_TableDef::column (SAPDB_UInt index)
 /*===========================================================================*
  *     StudioTD_TableDef::findColumn                                         *
  *===========================================================================*/
-SAPDB_UInt StudioTD_TableDef::findColumn(SAPDB_UInt2 pos)
+SAPDB_Int StudioTD_TableDef::findColumn(SAPDB_UInt2 pos)
 {
     StudioTD_ColumnDef *pCol = NULL;
-    SAPDB_UInt          i    = 0;
+    SAPDB_Int           i    = 0;
 
     /*-------------------------------------------------------------------------*
      * Search the list for an item with the given position.                    *
@@ -554,7 +554,8 @@ void StudioTD_TableDef::reset ()
  *     StudioTD_TableDef::readTableDefinition                                *
  *===========================================================================*/
 SAPDB_UInt
-StudioTD_TableDef::readTableDefinition (StudioTD_String tablename, StudioTD_String owner)
+StudioTD_TableDef::readTableDefinition (StudioTD_String tablename, StudioTD_String owner,SAPDB_Bool ownerIsSchema)
+
 {
     StudioTD_String         sql;
     StudioTD_String         name;
@@ -565,7 +566,7 @@ StudioTD_TableDef::readTableDefinition (StudioTD_String tablename, StudioTD_Stri
     StudioTD_ColumnDef     *col;
     StudioTD_ConstraintDef *constr;
     StudioTD_ForeignKeyDef *forkey;
-    SAPDB_Bool            bColIsNull[11];
+    SAPDB_Bool              bColIsNull[11];
 
     StudioTD_String         escaped_tablename = tablename;
     StudioTD_String         escaped_owner = owner;
@@ -584,56 +585,16 @@ StudioTD_TableDef::readTableDefinition (StudioTD_String tablename, StudioTD_Stri
     mOwner = owner;
     mName = tablename;
 
-
-  /*SELECT
-       A.columnname,
-       A.datatype,
-       A.len,
-       A.dec,
-       A.codetype,
-       A.nullable,
-       A.mode,
-       A.keypos,
-       A.default,
-       A.comment,
-       B.type
-    FROM
-       domain.columns A,
-      (select
-          owner,
-          tablename,
-          columnname,
-          type,
-          indexname
-       from
-          domain.indexcolumns outertab
-       where
-          outertab.tablename = 'Test' AND
-          outertab.owner = 'DEMO' AND
-          type = 'UNIQUE' and indexname <> 'SYSPRIMARYKEYINDEX' and
-         1 = ( select
-                     count(indexname)
-                  from
-                     domain.indexcolumns innertab
-                  where
-                     innertab.tablename = 'Test' AND
-                     innertab.owner = 'DEMO' AND
-                     innertab.indexname = outertab.indexname)
-      ) B
-    WHERE
-       A.tablename = 'Test' AND
-       A.owner = 'DEMO' AND
-       A.owner = B.owner (+) AND
-       A.tablename = B.tablename (+) AND
-       A.columnname = B.columnname (+)
-    ORDER BY A.pos*/
-
     /*-------------------------------------------------------------------------*
      * Prepare the WHERE clause for the SQL statements.                        *
      *-------------------------------------------------------------------------*/
     whereClause += "WHERE A.tablename = '";
     whereClause += escaped_tablename;
-    whereClause += "' AND A.owner = '";
+    if(ownerIsSchema)
+        whereClause += "' AND A.schema = '";
+    else
+        whereClause += "' AND A.owner = '";
+
     whereClause += escaped_owner;
     whereClause += "' ";
 
@@ -642,18 +603,33 @@ StudioTD_TableDef::readTableDefinition (StudioTD_String tablename, StudioTD_Stri
      *-------------------------------------------------------------------------*/
     sql  = "SELECT A.columnname, A.datatype, A.len, A.dec, A.codetype, "
          "A.nullable,A.mode,A.keypos,A.default, A.comment, B.type "
-         "FROM domain.columns A, "
-         "(select owner, tablename, columnname, type,indexname from domain.indexcolumns outertab where outertab.tablename = '";
-    sql += escaped_tablename;
-    sql += "'AND outertab.owner = '";
-    sql += escaped_owner;
-    sql += "' AND type = 'UNIQUE' and indexname <> 'SYSPRIMARYKEYINDEX' and 1 = ( select count(indexname) from domain.indexcolumns innertab where innertab.tablename = '";
-    sql += escaped_tablename;
-    sql += "'AND innertab.owner = '";
-    sql += escaped_owner;
-    sql += "' AND innertab.indexname = outertab.indexname)) B ";
-    sql += whereClause;
-    sql += "AND A.owner = B.owner (+) AND A.tablename = B.tablename (+) AND A.columnname = B.columnname (+) ORDER BY A.pos";
+         "FROM domain.columns A, ";
+    if(ownerIsSchema){
+        sql +=  "(select schema, tablename, columnname, type,indexname from domain.indexcolumns outertab where outertab.tablename = '";
+        sql += escaped_tablename;
+        sql += "'AND outertab.schema = '";
+        sql += escaped_owner;
+        sql += "' AND type = 'UNIQUE' and indexname <> 'SYSPRIMARYKEYINDEX' and 1 = ( select count(indexname) from domain.indexcolumns innertab where innertab.tablename = '";
+        sql += escaped_tablename;
+        sql += "'AND innertab.schema = '";
+        sql += escaped_owner;
+        sql += "' AND innertab.indexname = outertab.indexname)) B ";
+        sql += whereClause;
+        sql += "AND A.schema = B.schema (+) AND A.tablename = B.tablename (+) AND A.columnname = B.columnname (+) ORDER BY A.pos";
+    }else{        
+        sql +=  "(select owner, tablename, columnname, type,indexname from domain.indexcolumns outertab where outertab.tablename = '";
+        sql += escaped_tablename;
+        sql += "'AND outertab.owner = '";
+        sql += escaped_owner;
+        sql += "' AND type = 'UNIQUE' and indexname <> 'SYSPRIMARYKEYINDEX' and 1 = ( select count(indexname) from domain.indexcolumns innertab where innertab.tablename = '";
+        sql += escaped_tablename;
+        sql += "'AND innertab.owner = '";
+        sql += escaped_owner;
+        sql += "' AND innertab.indexname = outertab.indexname)) B ";
+        sql += whereClause;
+        sql += "AND A.owner = B.owner (+) AND A.tablename = B.tablename (+) AND A.columnname = B.columnname (+) ORDER BY A.pos";
+    }
+
 
     if (!executeSQL (sql))
     {
@@ -828,7 +804,7 @@ StudioTD_TableDef::readTableDefinition (StudioTD_String tablename, StudioTD_Stri
             mSampleUnit  = "Rows";
         }
 
-        if (escaped_owner.ToUpper() == "TEMP")
+        if (escaped_owner == "TEMP")
         {
             mTemporary = true;
         }
@@ -838,7 +814,7 @@ StudioTD_TableDef::readTableDefinition (StudioTD_String tablename, StudioTD_Stri
 
     return StudioTD_OK;
 }
-// StudioTD_TableDef::readTableDefinition()
+// StudioTD_TableDef::readTableDefinition
 
 
 /*===========================================================================*
@@ -1025,7 +1001,8 @@ StudioTD_String StudioTD_TableDef::findIncompleteDefs ()
  *===========================================================================*/
 StudioTD_String StudioTD_TableDef::genCreateStatement ()
 {
-  StudioTD_String          sql,s,k,temp;
+  StudioTD_String          sql,s,k,keyTemp;
+  StudioTD_String          escapedTableName,escapedOwner,escapedColumnName,escapedConstraint,escapedForeignKey;
   SAPDB_Bool               casesensitive = mCaseSensitive;
   SAPDB_Bool               bKey = false;
   SAPDB_Bool               bFirstKey = false;
@@ -1052,11 +1029,16 @@ StudioTD_String StudioTD_TableDef::genCreateStatement ()
   if (mTemporary)
     sql += "TEMP\".\"";
 
-  if(mOwner != ""){
-      sql += CASE(mOwner);
+  escapedOwner = mOwner;
+  escapeQuoteChar(escapedOwner,"\"");
+  if(!mTemporary && escapedOwner != ""){
+      sql += escapedOwner;
       sql += "\".\"";
   }
-  sql += CASE(mName);
+
+  escapedTableName = CASE(mName);
+  escapeQuoteChar(escapedTableName,"\"");
+  sql += escapedTableName;
   sql += "\""StudioTD_NEW_LINE"("StudioTD_NEW_LINE;
 
   /*-------------------------------------------------------------------------*
@@ -1085,14 +1067,18 @@ StudioTD_String StudioTD_TableDef::genCreateStatement ()
             bFirstKey = true;
         }
 
-        temp = (bFirstKey ? "\tPRIMARY KEY (" : "");
-        temp  += "\"";
-        temp  += CASE(col->name());
-        temp  += "\", ";    
+        keyTemp = (bFirstKey ? "\tPRIMARY KEY (" : "");
+        keyTemp  += "\"";
+
+		escapedColumnName = CASE(col->name());
+		escapeQuoteChar(escapedColumnName,"\"");
+
+		keyTemp  += escapedColumnName;
+        keyTemp  += "\", ";    
 
         bFirstKey = false;
     
-        primaryKeys[col->keyPos()-1] = temp;
+        primaryKeys[col->keyPos()-1] = keyTemp;
           
       }             
     }
@@ -1182,7 +1168,9 @@ StudioTD_String StudioTD_TableDef::genCreateStatement ()
 
     sql += SEP;
     sql += "COMMENT ON TABLE \"";
-    sql += CASE(mName);
+
+	sql += escapedTableName;
+
     sql += "\" IS '";
     sql += escaped_Comment;          
     sql += "'";
@@ -1200,9 +1188,15 @@ StudioTD_String StudioTD_TableDef::genCreateStatement ()
 
       sql += SEP;
       sql += "COMMENT ON COLUMN \"";
-      sql += CASE(mName);
+
+	  sql += escapedTableName;
+      
       sql += "\".\"";
-      sql += CASE(col->name());
+
+      escapedColumnName = CASE(col->name());
+	  escapeQuoteChar(escapedColumnName,"\"");
+	  sql += escapedColumnName;
+
       sql += "\" IS '";
       sql += escaped_col_comment;           
       sql += "'";
@@ -1223,9 +1217,11 @@ StudioTD_String StudioTD_TableDef::genCreateStatement ()
 
         sql += SEP;
         sql += "COMMENT ON FOREIGN KEY \"";
-        sql += CASE(mName);
+        sql += escapedTableName;
         sql += "\".\"";
-        sql += CASE(forkey->name());
+		escapedForeignKey = CASE(forkey->name());
+		escapeQuoteChar(escapedForeignKey,"\"");
+        sql += escapedForeignKey;
         sql += "\" IS '";
         sql += escaped_forkey_comment;  
         sql += "'";
@@ -1263,7 +1259,7 @@ StudioTD_String StudioTD_TableDef::genCreateXML()
     return "";
 
   // dbm:DeployedTable
-  deployedTable += "<sapdbm:DeployedTable oim:id=\"_";
+  deployedTable += "<sdbm:DeployedTable oim:id=\"_";
   deployedTable += string2XML(mOwner);
   deployedTable += ".";
   deployedTable += string2XML(mName);
@@ -1273,14 +1269,14 @@ StudioTD_String StudioTD_TableDef::genCreateXML()
   deployedTable += string2XML(mName);
   deployedTable += "\" dbm:comments=\"";
   deployedTable += string2XML(mComment);
-  deployedTable += "\" sapdbm:samplevalue=\"";
+  deployedTable += "\" sdbm:samplevalue=\"";
   if ((mSampleValue != 20000) || (mSampleUnit != "Rows"))
   {
       sprintf (buf, "%d ", mSampleValue );
       StudioTD_String sampleValue( buf );
 
       deployedTable += sampleValue;
-      deployedTable += "\" sapdbm:sampleunit=\"";
+      deployedTable += "\" sdbm:sampleunit=\"";
       deployedTable += UpperCase(mSampleUnit);
   }
   deployedTable += "\">\n";
@@ -1406,7 +1402,7 @@ StudioTD_String StudioTD_TableDef::genCreateXML()
   deployedTable += deployedTableUniqueKeys;
   deployedTable += deployedTableForeignKeys;
   deployedTable += deployedTableColumnConstraints;
-  //deployedTable += "</sapdbm:DeployedTable>\n";       DeployedTableIndices
+  //deployedTable += "</sdbm:DeployedTable>\n";       DeployedTableIndices
   //deployedTable += "</uml:NamespaceOwnedElement>\n";  DeployedTableIndices
 
   return deployedTable;
@@ -1418,10 +1414,11 @@ StudioTD_String StudioTD_TableDef::genCreateXML()
 StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTable)
 {
   StudioTD_String         sql;
+  StudioTD_String         escapedTableName,escapedOwner,escapedColumnName,escapedConstraint,escapedForeignKey;
   SAPDB_Bool              casesensitive = mCaseSensitive;
   SAPDB_Bool              bFirst = true, bKeyChanged = false;
   SAPDB_UInt              nNewKeys, nOldKeys;
-  StudioTD_String         intro, s ,temp;
+  StudioTD_String         intro, s ,keyTemp;
   StudioTD_String*        primaryKeys;
   StudioTD_ColumnDef      *oldCol;
 
@@ -1444,12 +1441,16 @@ StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTabl
    *-------------------------------------------------------------------------*/
   intro  = "ALTER TABLE \"";
 
+  escapedOwner = mOwner;
+  escapeQuoteChar(escapedOwner,"\"");
   if(mOwner != ""){
-      intro += mOwner;
+      intro += escapedOwner;
       intro += "\".\"";
   }
 
-  intro += mName;
+  escapedTableName = mName;
+  escapeQuoteChar(escapedTableName,"\"");
+  intro += escapedTableName;
   intro += "\" ";
 
   /*-------------------------------------------------------------------------*
@@ -1483,7 +1484,9 @@ StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTabl
       }
 
       sql += "\"";
-      sql += CASE(col->name());
+	  escapedColumnName = CASE(col->name());
+	  escapeQuoteChar(escapedColumnName,"\"");
+      sql += escapedColumnName;
       sql += "\", ";
       bFirst = false;
     }
@@ -1511,8 +1514,6 @@ StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTabl
     }
   }
 
-  
-
   nNewKeys = keyCount();
   nOldKeys = oldTable->keyCount();
 
@@ -1541,10 +1542,12 @@ StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTabl
       {
         if (col->key()){
 
-          temp = "\"";
-          temp += CASE(col->name());
-          temp += "\", ";
-          primaryKeys[col->keyPos()-1] = temp;
+          keyTemp = "\"";
+          escapedColumnName = CASE(col->name());
+		  escapeQuoteChar(escapedColumnName,"\"");
+		  keyTemp += escapedColumnName;
+          keyTemp += "\", ";
+          primaryKeys[col->keyPos()-1] = keyTemp;
           
         }               
       }
@@ -1577,7 +1580,9 @@ StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTabl
     {
       sql += intro;
       sql += "ALTER CONSTRAINT \"";
-      sql += constr->name();
+      escapedConstraint = CASE(constr->name());
+	  escapeQuoteChar(escapedConstraint,"\"");
+	  sql += escapedConstraint;
       sql += "\" CHECK ";
       sql += constr->genAlterStatement (mCaseSensitive);
       sql += SEP;
@@ -1598,7 +1603,9 @@ StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTabl
         {
           sql += intro;
           sql += "ADD CONSTRAINT \"";
-          sql += CASE(constr->name());
+		  escapedConstraint = CASE(constr->name());
+	      escapeQuoteChar(escapedConstraint,"\"");
+          sql += escapedConstraint;
           sql += "\" CHECK ";
         }
         else
@@ -1620,7 +1627,10 @@ StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTabl
     {
       sql += intro;
       sql += "DROP CONSTRAINT \"";
-      sql += constr->name();
+	  
+	  escapedConstraint = CASE(constr->name());
+	  escapeQuoteChar(escapedConstraint,"\"");
+      sql += escapedConstraint;
       sql += "\"";
       sql += SEP;
     }
@@ -1646,7 +1656,10 @@ StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTabl
     {
       sql += intro;
       sql += "DROP FOREIGN KEY \"";
-      sql += forkey->name();
+
+	  escapedForeignKey = CASE(forkey->name());
+	  escapeQuoteChar(escapedForeignKey,"\"");
+	  sql += escapedForeignKey;
       sql += "\"";
       sql += SEP;
     }
@@ -1673,11 +1686,11 @@ StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTabl
     escapeQuoteChar(escaped_Comment, "'");
 
     sql += "COMMENT ON TABLE \"";
-    if(mOwner != ""){
-      sql += mOwner;
+    if(escapedOwner != ""){
+      sql += escapedOwner;
       sql += "\".\"";
     }
-    sql += mName;
+    sql += escapedTableName;
     sql += "\" IS '";
     sql += escaped_Comment; 
     sql += "'";
@@ -1701,12 +1714,12 @@ StudioTD_String StudioTD_TableDef::genAlterStatement (StudioTD_TableDef *oldTabl
  *===========================================================================*/
 StudioTD_String StudioTD_TableDef::genDescription ()
 {
-  StudioTD_String         desc;
+  StudioTD_String        temp;
+  StudioTD_String        desc;
   StudioTD_ColumnDef     *col;
   StudioTD_ConstraintDef *constr;
   StudioTD_ForeignKeyDef *forkey;
-  char                    buf[16];
-
+  
   /*-------------------------------------------------------------------------*
    * Add the table attributs to the description.                             *
    *-------------------------------------------------------------------------*/
@@ -1719,8 +1732,8 @@ StudioTD_String StudioTD_TableDef::genDescription ()
   desc += SEP_LEVEL_2;
   desc += mComment;
   desc += SEP_LEVEL_2;
-  sprintf (buf, "%d", mSampleValue);
-  desc += buf;
+  temp.ConvertFromInt(mSampleValue);
+  desc += temp;
   desc += SEP_LEVEL_2;
   desc += mSampleUnit;
   desc += SEP_LEVEL_2;
@@ -1732,6 +1745,8 @@ StudioTD_String StudioTD_TableDef::genDescription ()
   desc += SEP_LEVEL_2;
   desc += (mCaseSensitive  ? "true" : "false");
   desc += SEP_LEVEL_2;  
+  desc += mOwner;
+  desc += SEP_LEVEL_2;
   desc += SEP_LEVEL_1;
 
   /*-------------------------------------------------------------------------*
@@ -1777,8 +1792,8 @@ StudioTD_String StudioTD_TableDef::genDescription ()
   desc += SEP_LEVEL_1;
 
   desc += "mode:::";
-  sprintf (buf, "%d", mMode);
-  desc += buf;
+  temp.ConvertFromInt(mMode);
+  desc += temp;
   desc += SEP_LEVEL_2;
   desc += SEP_LEVEL_1;
 
@@ -1827,6 +1842,7 @@ void StudioTD_TableDef::parseDescription (StudioTD_String description)
       mTemporary      = (NextToken (str, SEP_LEVEL_2, &s) == "true") ? true : false;
       mIgnoreRollback = (NextToken (str, SEP_LEVEL_2, &s) == "true") ? true : false;
       mCaseSensitive  = (NextToken (str, SEP_LEVEL_2, &s) == "true") ? true : false;
+	  mOwner          = NextToken (str, SEP_LEVEL_2, &s);
 
       continue;
     }
