@@ -10,7 +10,7 @@
 
   description:  Definition of the table definition class StudioTD_TableDef.
 
-  version:      7.4
+  version:      7.5
 
   created:      2000-05-09
 
@@ -374,9 +374,9 @@ public:
     function:     findColumn
     description:  Get the index of the element with the given position
     arguments:    pos - position of column in table definition
-    returns:      SAPDB_UInt
+    returns:      SAPDB_Int
   --------------------------------------------------------------------------*/
-  SAPDB_UInt findColumn(SAPDB_UInt2 pos);
+  SAPDB_Int findColumn(SAPDB_UInt2 pos);
 
   /*!-------------------------------------------------------------------------
     function:     columnCount
@@ -468,9 +468,10 @@ public:
     description:  Read the definition of the given table from the database.
     arguments:    tablename - Name of the table to be loaded
                   owner - Owner of the table to be loaded
+                  ownerIsSchema - the owner if the table is a schema
     returns:      SAPDB_UInt
   --------------------------------------------------------------------------*/
-  SAPDB_UInt readTableDefinition (StudioTD_String tablename, StudioTD_String owner);
+  SAPDB_UInt readTableDefinition (StudioTD_String tablename, StudioTD_String owner,SAPDB_Bool ownerIsSchema);
   /*!-------------------------------------------------------------------------
     function:     hasValidColumns
     description:  Check if the table definition contains valid column
