@@ -86,6 +86,43 @@ void __stdcall table_delete (void *_this)
 }
 
 /*===========================================================================*
+*  Wrapper function for StudioTD_TableDef::owner().                          *
+*===========================================================================*/
+SAPDB_Int __stdcall table_owner (void *_this, SAPDB_UCS2 *out)
+{
+ StudioTD_WTableDefODBC  *me = (StudioTD_WTableDefODBC *)_this;
+ SAPDB_UCS2              *end;
+ 
+ assert (me);
+ assert (me->typeOf() == StudioTD_Base::ClassTableDefODBC);
+ assert (out);
+   
+ StudioTD_String sbuf = me->owner();
+ sbuf.ConvertToUCS2 (out, out + wcslen(out), end);
+ 
+ return (sbuf.Size());
+}
+   /*===========================================================================*
+*  Wrapper function for StudioTD_TableDef::setOwner().                       *
+*===========================================================================*/
+void __stdcall table_setOwner (void *_this, SAPDB_UCS2 *name)
+{
+ StudioTD_String         s;
+ StudioTD_WTableDefODBC  *me = (StudioTD_WTableDefODBC *)_this;
+ 
+ assert (me);
+ assert (me->typeOf() == StudioTD_Base::ClassTableDefODBC);
+
+ if (name)
+   s.ConvertFromUCS2 (name, name + wcslen(name));
+ else
+   s = "";
+ 
+ me->setOwner (s);
+}
+
+/*===========================================================================*
+
  *  Wrapper function for StudioTD_TableDef::name().                          *
  *===========================================================================*/
 SAPDB_Int __stdcall table_name (void *_this, SAPDB_UCS2 *out)
@@ -732,7 +769,7 @@ void __stdcall table_reset (void *_this)
 /*===========================================================================*
  *  Wrapper function for StudioTD_TableDef::readTableDefinition().           *
  *===========================================================================*/
-SAPDB_UInt __stdcall table_readTableDefinition (void *_this, SAPDB_UCS2 *tablename, SAPDB_UCS2 *owner)
+SAPDB_UInt __stdcall table_readTableDefinition (void *_this, SAPDB_UCS2 *tablename, SAPDB_UCS2 *owner,SAPDB_Bool ownerIsSchema)
 {
   StudioTD_String         sTable;
   StudioTD_String         sOwner;
@@ -751,7 +788,8 @@ SAPDB_UInt __stdcall table_readTableDefinition (void *_this, SAPDB_UCS2 *tablena
   else
     sOwner = "";
 
-  return me->readTableDefinition (sTable, sOwner);
+  return me->readTableDefinition (sTable, sOwner,ownerIsSchema);
+
 }
 
 /*===========================================================================*
