@@ -52,10 +52,6 @@
 
 #include "SAPDB/SQLStudio/OAL/StudioOAL_WCommon.h"
 
-#include "hsp02.h"
-#include "hsp83.h"
-
-
 //html code fragments
 #include "SAPDB/SQLStudio/WebQuery/StudioWeb_HtmlFragments.h"
 
@@ -65,153 +61,8 @@
 #define SIMPLEQUOTE	 "'"
 
 
-tsp81_UCS2Char hexStringToUcs2(const SAPDB_Char *hexString) {
 
-	 tsp81_UCS2Char ret;
-	 ret.c[0] = 0;
-	 ret.c[1] = 0;
 
-	 //check length
-	 if(strlen(hexString) < 4) return ret;
-
-	 SAPDB_Char c;
-	 SAPDB_Byte temp = 0;
-	 //take the first 2 chars of the hexarray
-	 for (int i = 0; i<4; i ++) {
-		c = hexString[i];
-		if (c > '9') {
-			if (c > 'Z') {
-				c = c - 'a' + 'A';
-			}
-			c = c - 'A' + 10;
-		}
-		else {
-			c = c - '0';
-		}
-		if (c < 16) {
-			temp = (temp*16) + c;
-		}
-		if(i== 1){
-			ret.c[0] = temp;
-			temp = 0;
-		}
-		if(i== 3){
-			ret.c[1] = temp;
-		}
-	 }
-	 return ret;
-}
-
-SAPDB_Int4 hexStringToInt4(const SAPDB_Char *hexString) {
-
-	 //check length
-	 if(strlen(hexString) < 8) return 0;
-
-	 SAPDB_Char c;
-	 SAPDB_Int4 ret = 0;
-	 
-	 //take the first 8 chars of the hexarray
-	 for (int i = 0; i<8; i ++) {
-		c = hexString[i];
-		if (c > '9') {
-			if (c > 'Z') {
-				c = c - 'a' + 'A';
-			}
-			c = c - 'A' + 10;
-		}
-		else {
-			c = c - '0';
-		}
-		if (c < 16) {
-			ret = (ret*16) + c;
-		}
-	 }
-	 return ret;
-}
-
-SAPDB_Bool uncryptPWD(Tools_DynamicUTF8String &sPassWord){
-
-	int i = 0;
-	tsp00_Name  pwClear;		
-	tsp00_CryptPw pwCrypt;
-	const char* buffPointer = (const char* )sPassWord.CharPtr();
-	SAPDB_Int4 dummy[6];
-	//the  source string is a 48 char representation of the crypted password
-	for(i = 0;i<6;i++){
-
-		//use a 4 byte value to store the value of the 8 byte hex string representation
-		dummy[i] = hexStringToInt4(buffPointer + (i*8));
-	}
-
-	memcpy(&pwCrypt,&dummy,sizeof(tsp00_CryptPw));
-	//call decrypt function
-	s02appldecrypt(pwClear,pwCrypt);
-	//ucs2 buffer for clear password
-	SAPDB_Int2 ucs2BuffLength = sizeof(tsp00_Name)/2;
-	tsp81_UCS2Char ucs2Buffer [sizeof(tsp00_Name)/2];
-	memcpy(&ucs2Buffer[0], &pwClear[0], ucs2BuffLength*2);
-
-	//buffer of utf8 password
-	SAPDB_UTF8 utf8Buffer [3*sizeof(tsp00_Name)/2];
-	memset(utf8Buffer,0,sizeof(utf8Buffer));
-
-	tsp81_UCS2Char const    *srcAt = NULL;
-    tsp00_Byte *destAt = NULL;
-	
-	tsp83UTF8_ConversionResult ret = sp83UTF8ConvertFromUCS2(
-													ucs2Buffer,
-													ucs2Buffer + ucs2BuffLength,
-													&srcAt,
-													1,
-													utf8Buffer,
-													utf8Buffer+(ucs2BuffLength*3),
-													&destAt);
-	if(ret != sp83UTF8Convert_Success)
-		return SAPDB_FALSE;
-
-	//reinit password
-	sPassWord = (SAPDB_UTF8*)utf8Buffer;
-	return SAPDB_TRUE;
-	
-}
-
-SAPDB_Bool uncryptUser(Tools_DynamicUTF8String &sUser){
-	
-	int i = 0;
-	const char* buffPointer = (const char* )sUser.StrPtr();
-	//user is maximal 64 ucs2 chars
-	tsp81_UCS2Char  ucs2Buffer[64];
-	
-	SAPDB_Int2 userLength = (int)strlen(buffPointer)/4;
-	for(i = 0;i< userLength; i++){
-
-		ucs2Buffer[i] = hexStringToUcs2(buffPointer + (i*4));			 
-	}		
-		
-	//buffer of utf8 password
-	SAPDB_UTF8 utf8Buffer [sizeof(ucs2Buffer)*3];
-	memset(utf8Buffer,0,sizeof(utf8Buffer));
-
-	tsp81_UCS2Char const    *srcAt = NULL;
-    tsp00_Byte *destAt = NULL;
-	
-	tsp83UTF8_ConversionResult ret = sp83UTF8ConvertFromUCS2(
-													ucs2Buffer,
-													ucs2Buffer + userLength,
-													&srcAt,
-													1,
-													utf8Buffer,
-													utf8Buffer+(userLength*3),
-													&destAt);					
-
-		if(ret != sp83UTF8Convert_Success)
-			return SAPDB_FALSE;
-		
-		//reinit user
-		sUser = (SAPDB_UTF8*)utf8Buffer;
-		return SAPDB_TRUE;
-
-}
 void replaceNoCharWithDot(Tools_DynamicUTF8String &sInAndOut)
 
 {
