@@ -53,51 +53,6 @@
 #include "hwd08dbc.h"
 #include "sapdbwa.hpp" //nocheck
 
-/* Get swap info */
-#define WQ_IS_UCS2_SWAPED   sp77encodingUCS2Native == sp77encodingUCS2Swapped
-
-
-/*!-------------------------------------------------------------------------
-	function:     hexStringToIntUcs2
-	description:  convert a hex string into a ucs2 char
-                  [in] hexstring				  
-				  
-				  
-	return value: tsp81_UCS2Char
---------------------------------------------------------------------------*/
-tsp81_UCS2Char hexStringToUcs2(const SAPDB_Char *hexString);
-
-/*!-------------------------------------------------------------------------
-	function:     hexStringTo4Byte
-	description:  convert a hex string into a SAPDB_Int4
-                  [in] hexstring				  
-				  
-				  
-	return value: SAPDB_Int4
---------------------------------------------------------------------------*/
-SAPDB_Int4 hexStringToInt4(const SAPDB_Char *hexString);
-
-
-/*!-------------------------------------------------------------------------
-	function:     uncryptPWD
-	description:  uncrypt a hex string password (swapped) to a utf8 password
-                  [in] sPassWord				  
-				  
-				  
-	return value: SAPDB_Bool
---------------------------------------------------------------------------*/
-SAPDB_Bool uncryptPWD(Tools_DynamicUTF8String &sPassWord);
-
-/*!-------------------------------------------------------------------------
-	function:     uncryptUser
-	description:  uncrypt a hex string user(swapped) to a utf8 user
-                  [in] sUser				  
-				  
-				  
-	return value: SAPDB_Bool
---------------------------------------------------------------------------*/
-SAPDB_Bool uncryptUser(Tools_DynamicUTF8String &sUser);
-
 /*!-------------------------------------------------------------------------
 	function:     replaceNoCharWithDot
 	description:  replace all chars y ascii 31 with a '.'
