@@ -371,11 +371,6 @@
 //----------------------------------------------------------------------------*/
 #define  WQ_LOGON_TAG "logon"
 
-/*----------------------------------------------------------------------------\
-// WQ_CRYPT_TAG
-//----------------------------------------------------------------------------*/
-#define  WQ_CRYPT_TAG "CRYPT"
-
 /*----------------------------------------------------------------------------\
 // WQ_PREVSTATEMENT_TAG
 //----------------------------------------------------------------------------*/
