@@ -19,35 +19,19 @@
 #    ========== licence end
 #
 
-#
-
-import sys
-
-_pythonVersion = sys.version [:3]
-
-del sys
-
 try:
     from pythondef.dbm import *
     from pythondef.dbm import _buildInfo
 except ImportError:
-    if _pythonVersion == "2.3":
-        from python23.dbm import *
-        from python23.dbm import _buildInfo
-    elif _pythonVersion == "2.2":
-        from python22.dbm import *
-        from python22.dbm import _buildInfo
-    elif _pythonVersion == "2.1":
-        from python21.dbm import *
-        from python21.dbm import _buildInfo
-    elif _pythonVersion == "2.0":
-        from python20.dbm import *
-        from python20.dbm import _buildInfo
-    elif _pythonVersion == "1.5":
-        from python15.dbm import *
-        from python15.dbm import _buildInfo
-    else:
-        raise "Python version not supported", _pythonVersion
-
-del _pythonVersion
+    def importVersionSpecific ():
+        import sys
+        pythonVersion = sys.version [:3]
+        shortVersion = pythonVersion [0] + pythonVersion [2]
+        subpackage = 'python-py%s' % shortVersion
+        if hasattr (sys, 'maxunicode') and (sys.maxunicode > 65535):
+            subpackage = subpackage + 'ucs4'
+        module = __import__ (subpackage + '.dbm', globals (), locals (), ['*'])
+        globals ().update (module.__dict__)
+    importVersionSpecific ()
+    del importVersionSpecific
 
