@@ -3,7 +3,7 @@
    responsible:    DanielD
    description:    Interface to DB Manager
    target:         Python
-   last changed:   04.11.2003
+   last changed:   18.05.2004
 
     ========== licence begin  GPL
     Copyright (c) 2000-2004 SAP AG
@@ -27,7 +27,7 @@
 
 #include <stddef.h>
 
-#if defined (HPUX) || defined (AIX) || defined (SUN) || defined (OSF1) || defined (LINUX)
+#if defined (HPUX) || defined (AIX) || defined (SUN) || defined (OSF1) || defined (LINUX) || defined (FREEBSD)
 #define BAD_STATIC_FORWARD
 #endif
 
@@ -63,6 +63,10 @@
 #define GLUEEXPORT
 #endif
 
+#if PYTHON_API_VERSION < 1009
+#define PyObject_Del(ptr) free (ptr)
+#endif
+
 #define SL_None Py_None
 #define SL_isNone(val) (val == SL_None)
 #define SL_isTrue(val) PyObject_IsTrue (val)
@@ -149,13 +153,14 @@ raiseCommunicationError (
     if (exception != NULL) {
         PyObject_SetAttrString (exception, LIT ("errorCode"), pycode) ;
         PyObject_SetAttrString (exception, LIT ("message"), pymsg);
-        Py_XDECREF (pycode);
-        Py_XDECREF (pymsg);
     }
     else {
         exception = Py_BuildValue (LIT ("NN"), pycode, pymsg);
     }
     PyErr_SetObject (CommunicationErrorType, exception);
+    Py_XDECREF (pycode);
+    Py_XDECREF (pymsg);
+    Py_XDECREF (exception);
 }
 
 /*----------------------------------------*/
@@ -222,17 +227,18 @@ raiseDBMError (
         PyObject_SetAttrString (exception, LIT ("message"), errorMessage);
         PyObject_SetAttrString (exception, LIT ("additionalInfo"), additionalInfo);
         PyObject_SetAttrString (exception, LIT ("cmd"), cmdObject);
-        Py_XDECREF (errorCode);
-        Py_XDECREF (errorSymbol);
-        Py_XDECREF (errorMessage);
-        Py_XDECREF (additionalInfo);
-        Py_XDECREF (cmdObject);
     }
     else {
         exception = Py_BuildValue (LIT ("NNNNN"), errorCode, errorSymbol,
             errorMessage, additionalInfo, cmdObject);
     }
     PyErr_SetObject (DBMServErrorType, exception);
+    Py_XDECREF (errorCode);
+    Py_XDECREF (errorSymbol);
+    Py_XDECREF (errorMessage);
+    Py_XDECREF (additionalInfo);
+    Py_XDECREF (cmdObject);
+    Py_XDECREF (exception);
 }
 
 /*----------------------------------------*/
@@ -394,7 +400,7 @@ destructor_DBM (
     DBMObjectT* self = REINTERPRET_CAST (DBMObjectT*, pyself);
 
     localRelease (self->nself);
-    free (pyself);
+    PyObject_Del (pyself);
 }
 #endif
 
