@@ -3,7 +3,7 @@
    responsible:    DanielD
    description:    Interface to the SAP DB Loader
    target:         Perl
-   last changed:   04.11.2003
+   last changed:   27.07.2004
 
     ========== licence begin  GPL
     Copyright (c) 2000-2004 SAP AG
@@ -168,8 +168,8 @@ static const char * LoaderError_loader = "LoaderError";
 /* snippet Perl_precode */
 #define CN72 1
 #undef na
-#include "hsp100.h"  /*&variant +a,c */
-#include "hcn14.h"   /*&variant +a,c */
+#include "hsp100.h"
+#include "hcn14.h"
 #include "patchlevel.h"/* no check */
 #if PATCHLEVEL >= 5
     #define na PL_na
@@ -231,15 +231,11 @@ loaderErrOccured (
     int rc,
     tsp00_ErrTextc VAR_ARRAY_REF msg,
     const char * cmd,
-    char * croakBuf)
+    char * croakBuf,
+    int raiseSqlErrors)
 {
+    ReplyInfoT    replyInfo;
     int           errOccurred = 0;
-    tsp00_Int4      serverError;
-    const void  * payLoad;
-    tsp00_Int4      payLoadLen;
-    tsp00_Int4      specificError;
-    tsp00_ErrTextc errtext;
-
 
     if (rc == DBMAPI_COMMERR_CN14) {
         sprintf (croakBuf, "Communication Error: %s", msg);
@@ -248,25 +244,21 @@ loaderErrOccured (
     else if (rc == DBMAPI_OK_CN14) {
         return 0;
     }
-    serverError = cn14analyzeDbmAnswer (nself, &payLoad, &payLoadLen,
-            &specificError, errtext);
-    trimPayload (payLoad, &payLoadLen);
-    if (serverError != DBMAPI_OK_CN14) {
+    cn14analyzeRpmAnswer (nself,
+        &replyInfo.errCode, &replyInfo.errText, &replyInfo.errLen,
+        &replyInfo.sqlCode, &replyInfo.sqlErrText, &replyInfo.sqlErrLen);
+    if ((replyInfo.sqlCode != 0) && !raiseSqlErrors) {
+        /* ignore */
+    }
+    else if (replyInfo.sqlCode != 0) {
         errOccurred = 1;
-        if (payLoad == NULL) {
-            tsp00_Int4 dummy;
-            payLoad = cn14rawReadData (nself, &dummy);
-            payLoadLen = cn14bytesAvailable (nself);
-            specificError = -1;
-            if (payLoad == NULL) {
-                payLoad = "Network Structure Parse Error";
-                payLoadLen = strlen (STATIC_CAST (const char*, payLoad));
-            }
-            if (payLoadLen > 100) {
-                payLoadLen = 100;
-            }
-        }
-        sprintf (croakBuf, "Error Loader Server: %*s", payLoadLen, payLoad);
+        sprintf (croakBuf, "Error Loader Server: [SQL %d] %*s",
+            replyInfo.sqlCode, replyInfo.sqlErrLen, replyInfo.sqlErrText);
+    }
+    else if (replyInfo.errCode != 0) {
+        errOccurred = 1;
+        sprintf (croakBuf, "Error Loader Server: [%d] %*s",
+            replyInfo.errCode, replyInfo.errLen, replyInfo.errText);
     }
     return errOccurred;
 }
@@ -374,7 +366,7 @@ destructor_Loader (
     LoaderObjectT* self = REINTERPRET_CAST (LoaderObjectT*, pyself);
 
     localRelease (self->nself);
-    free (pyself);
+    PyObject_Del (pyself);
 }
 #endif
 
@@ -448,20 +440,19 @@ static int cancelCmd (
 static int sql (
    void * nself,
    const char * cmd,
-   tsp00_Int4 * errCode,
    tsp00_ErrTextc VAR_ARRAY_REF errtext)
 {
     int result = 0;
     ReplyInfoT replyInfo;
 
     Glue_BEGIN_ALLOW_THREADS
-    *errCode = cn14ExecuteLoaderCmd (nself, cmd, strlen (cmd), NULL, NULL, errtext);
+    result = cn14ExecuteLoaderCmd (nself, cmd, strlen (cmd), NULL, NULL, errtext);
     Glue_END_ALLOW_THREADS
-    if (*errCode == DBMAPI_OK_CN14) {
+    if (result == DBMAPI_OK_CN14) {
         cn14analyzeRpmAnswer (nself,
             &replyInfo.errCode, &replyInfo.errText, &replyInfo.errLen,
             &replyInfo.sqlCode, &replyInfo.sqlErrText, &replyInfo.sqlErrLen);
-        *errCode = replyInfo.sqlCode;
+        result = replyInfo.sqlCode;
     }
     return result;
 }
@@ -518,7 +509,7 @@ externC XS (Loader_cmd)
     ReadResultT output;
     tsp00_ErrTextc msg;
     tsp00_Int4 result;
-
+    
     cv = cv;
     if (items != 2) {
         croak (invalidArgCount_C);
@@ -537,7 +528,7 @@ externC XS (Loader_cmd)
     }
 /* no code for key Perl_/Generic_nativeCall */
     result = cmdAndRead (nself, cmd, &output, msg);
-    if (loaderErrOccured (nself, result, msg, cmd, croakBuf)) {
+    if (loaderErrOccured (nself, result, msg, cmd, croakBuf, 1)) {
         ok = 0;
         goto cleanup_label;
     }
@@ -568,7 +559,7 @@ externC XS (Loader_rawCmd)
     ReadResultT output;
     tsp00_ErrTextc msg;
     tsp00_Int4 result;
-
+    
     cv = cv;
     if (items != 2) {
         croak (invalidArgCount_C);
@@ -615,10 +606,9 @@ externC XS (Loader_sql)
     LoaderObjectT* self;
     void* nself;
     char * cmd;
-    tsp00_Int4 rc;
     tsp00_ErrTextc msg;
     int result;
-
+    
     cv = cv;
     if (items != 2) {
         croak (invalidArgCount_C);
@@ -636,8 +626,8 @@ externC XS (Loader_sql)
         goto cleanup_label;
     }
 /* no code for key Perl_/Generic_nativeCall */
-    result = sql (nself, cmd, &rc, msg);
-    if (loaderErrOccured (nself, rc, msg, cmd, croakBuf)) {
+    result = sql (nself, cmd, msg);
+    if (loaderErrOccured (nself, result, msg, cmd, croakBuf, 0)) {
         ok = 0;
         goto cleanup_label;
     }
@@ -665,7 +655,7 @@ externC XS (Loader_cancelCmd)
     LoaderObjectT* self;
     void* nself;
     int result;
-
+    
     cv = cv;
     if (items != 1) {
         croak (invalidArgCount_C);
@@ -693,7 +683,7 @@ externC XS (Loader_release)
     dXSARGS;
     LoaderObjectT* self;
     void* nself;
-
+    
     cv = cv;
     if (items != 1) {
         croak (invalidArgCount_C);
@@ -747,15 +737,15 @@ externC XS (loader_Loader)
     int argvi = 0;
     char croakBuf [CROAKBUF_SIZE];
     dXSARGS;
-    char * servernode;
-    char * dbname;
+    char * servernode = LIT( "");
+    char * dbname = LIT( "");
     char * instroot = LIT( "");
     void * session;
     tsp00_ErrTextc msg;
     tsp00_Int4 result;
-
+    
     cv = cv;
-    if ((items < 2) || (items > 3)) {
+    if ((items < 0) || (items > 3)) {
         croak (invalidArgCount_C);
     }
     switch (items) {
@@ -807,7 +797,7 @@ externC XS (loader__buildInfo)
     char croakBuf [CROAKBUF_SIZE];
     dXSARGS;
     char * result;
-
+    
     cv = cv;
     if (items != 0) {
         croak (invalidArgCount_C);
