@@ -3,7 +3,7 @@
    responsible:    DanielD
    description:    Interface to the SAP DB Loader
    target:         Python
-   last changed:   04.11.2003
+   last changed:   27.07.2004
 
     ========== licence begin  GPL
     Copyright (c) 2000-2004 SAP AG
@@ -27,7 +27,7 @@
 
 #include <stddef.h>
 
-#if defined (HPUX) || defined (AIX) || defined (SUN) || defined (OSF1) || defined (LINUX)
+#if defined (HPUX) || defined (AIX) || defined (SUN) || defined (OSF1) || defined (LINUX) || defined (FREEBSD)
 #define BAD_STATIC_FORWARD
 #endif
 
@@ -63,6 +63,10 @@
 #define GLUEEXPORT
 #endif
 
+#if PYTHON_API_VERSION < 1009
+#define PyObject_Del(ptr) free (ptr)
+#endif
+
 #define SL_None Py_None
 #define SL_isNone(val) (val == SL_None)
 #define SL_isTrue(val) PyObject_IsTrue (val)
@@ -94,13 +98,13 @@ createExceptionKind (
 }
 
 static PyObject *CommunicationErrorType;
-static const char * CommunicationErrorCodeC = 
+static const char * CommunicationErrorCodeC =
     "\n"
     "def __str__ (self):\n"
     "    return \"loader.CommunicationError: [%d] %s\" % (self.errorCode, self.message)\n"
     "\n";
 static PyObject *LoaderErrorType;
-static const char * LoaderErrorCodeC = 
+static const char * LoaderErrorCodeC =
     "\n"
     "def __str__ (self):\n"
     "    if self.sqlCode:\n"
@@ -162,13 +166,14 @@ raiseCommunicationError (
     if (exception != NULL) {
         PyObject_SetAttrString (exception, LIT ("errorCode"), pycode) ;
         PyObject_SetAttrString (exception, LIT ("message"), pymsg);
-        Py_XDECREF (pycode);
-        Py_XDECREF (pymsg);
     }
     else {
         exception = Py_BuildValue (LIT ("NN"), pycode, pymsg);
     }
     PyErr_SetObject (CommunicationErrorType, exception);
+    Py_XDECREF (pycode);
+    Py_XDECREF (pymsg);
+    Py_XDECREF (exception);
 }
 
 /*----------------------------------------*/
@@ -187,14 +192,11 @@ raiseLoaderError (
     PyObject * cmdObject;
 
     errorCode = PyInt_FromLong  (replyInfo->errCode);
-    if (replyInfo->errText [replyInfo->errLen - 1] == '\n') {
+    if ((replyInfo->errLen > 0) && (replyInfo->errText [replyInfo->errLen - 1] == '\n')) {
         --replyInfo->errLen;
     }
     message = PyString_FromStringAndSize (replyInfo->errText, replyInfo->errLen);
     sqlCode = PyInt_FromLong  (replyInfo->sqlCode);
-    if (replyInfo->sqlErrText [replyInfo->sqlErrLen - 1] == '\n') {
-        --replyInfo->sqlErrLen;
-    }
     sqlMessage = PyString_FromStringAndSize (replyInfo->sqlErrText, replyInfo->sqlErrLen);
     cmdObject = PyString_FromString (cmd);
     if (exception != NULL) {
@@ -203,17 +205,18 @@ raiseLoaderError (
         PyObject_SetAttrString (exception, LIT ("sqlCode"), sqlCode);
         PyObject_SetAttrString (exception, LIT ("sqlMessage"), sqlMessage);
         PyObject_SetAttrString (exception, LIT ("cmd"), cmdObject);
-        Py_XDECREF (errorCode);
-        Py_XDECREF (message);
-        Py_XDECREF (sqlCode);
-        Py_XDECREF (sqlMessage);
-        Py_XDECREF (cmdObject);
     }
     else {
         exception = Py_BuildValue (LIT ("NNNNN"), errorCode, message,
             sqlCode, sqlMessage, cmdObject);
     }
     PyErr_SetObject (LoaderErrorType, exception);
+    Py_XDECREF (errorCode);
+    Py_XDECREF (message);
+    Py_XDECREF (sqlCode);
+    Py_XDECREF (sqlMessage);
+    Py_XDECREF (cmdObject);
+    Py_XDECREF (exception);
 }
 
 /*----------------------------------------*/
@@ -252,7 +255,8 @@ loaderErrOccured (
     int rc,
     tsp00_ErrTextc msg,
     const char * cmd,
-    char * croakBuf)
+    char * croakBuf,
+    int raiseSqlErrors)
 {
     ReplyInfoT replyInfo;
     int errOccurred = 0;
@@ -267,7 +271,10 @@ loaderErrOccured (
     cn14analyzeRpmAnswer (nself,
         &replyInfo.errCode, &replyInfo.errText, &replyInfo.errLen,
         &replyInfo.sqlCode, &replyInfo.sqlErrText, &replyInfo.sqlErrLen);
-    if ((replyInfo.errCode != 0) || (replyInfo.sqlCode != 0)) {
+    if ((replyInfo.sqlCode != 0) && !raiseSqlErrors) {
+        /* ignore */
+    }
+    else if ((replyInfo.errCode != 0) || (replyInfo.sqlCode != 0)) {
         errOccurred = 1;
         raiseLoaderError (&replyInfo, cmd);
     }
@@ -381,7 +388,7 @@ destructor_Loader (
     LoaderObjectT* self = REINTERPRET_CAST (LoaderObjectT*, pyself);
 
     localRelease (self->nself);
-    free (pyself);
+    PyObject_Del (pyself);
 }
 #endif
 
@@ -455,20 +462,19 @@ static int cancelCmd (
 static int sql (
    void * nself,
    const char * cmd,
-   tsp00_Int4 * errCode,
    tsp00_ErrTextc VAR_ARRAY_REF errtext)
 {
     int result = 0;
     ReplyInfoT replyInfo;
 
     Glue_BEGIN_ALLOW_THREADS
-    *errCode = cn14ExecuteLoaderCmd (nself, cmd, strlen (cmd), NULL, NULL, errtext);
+    result = cn14ExecuteLoaderCmd (nself, cmd, strlen (cmd), NULL, NULL, errtext);
     Glue_END_ALLOW_THREADS
-    if (*errCode == DBMAPI_OK_CN14) {
+    if (result == DBMAPI_OK_CN14) {
         cn14analyzeRpmAnswer (nself,
             &replyInfo.errCode, &replyInfo.errText, &replyInfo.errLen,
             &replyInfo.sqlCode, &replyInfo.sqlErrText, &replyInfo.sqlErrLen);
-        *errCode = replyInfo.sqlCode;
+        result = replyInfo.sqlCode;
     }
     return result;
 }
@@ -533,7 +539,7 @@ cmd_Loader (PyObject *pyself, PyObject* args, PyObject* keywds)
     }
 /* no code for key Python_/Generic_nativeCall */
     result = cmdAndRead (nself, cmd, &output, msg);
-    if (loaderErrOccured (nself, result, msg, cmd, croakBuf)) {
+    if (loaderErrOccured (nself, result, msg, cmd, croakBuf, 1)) {
         ok = 0;
         goto cleanup_label;
     }
@@ -604,7 +610,6 @@ sql_Loader (PyObject *pyself, PyObject* args, PyObject* keywds)
     static const char * kwlist [] = {
         LIT("cmd"), NULL};
     char * cmd;
-    tsp00_Int4 rc;
     tsp00_ErrTextc msg;
     int result;
 
@@ -613,8 +618,8 @@ sql_Loader (PyObject *pyself, PyObject* args, PyObject* keywds)
         goto cleanup_label;
     }
 /* no code for key Python_/Generic_nativeCall */
-    result = sql (nself, cmd, &rc, msg);
-    if (loaderErrOccured (nself, rc, msg, cmd, croakBuf)) {
+    result = sql (nself, cmd, msg);
+    if (loaderErrOccured (nself, result, msg, cmd, croakBuf, 0)) {
         ok = 0;
         goto cleanup_label;
     }
@@ -668,15 +673,15 @@ release_Loader (PyObject *pyself, PyObject* args, PyObject* keywds)
 }
 
 static PyMethodDef LoaderClassMethods [] = {
-    {LIT("cmd"), (PyCFunction)cmd_Loader, METH_VARARGS|METH_KEYWORDS, 
+    {LIT("cmd"), (PyCFunction)cmd_Loader, METH_VARARGS|METH_KEYWORDS,
         "exeute a command and return resulting buffer as string"},
-    {LIT("rawCmd"), (PyCFunction)rawCmd_Loader, METH_VARARGS|METH_KEYWORDS, 
+    {LIT("rawCmd"), (PyCFunction)rawCmd_Loader, METH_VARARGS|METH_KEYWORDS,
         "exeute a command and return resulting buffer as string\n\n            This doesn't scan the buffer for error information."},
-    {LIT("sql"), (PyCFunction)sql_Loader, METH_VARARGS|METH_KEYWORDS, 
+    {LIT("sql"), (PyCFunction)sql_Loader, METH_VARARGS|METH_KEYWORDS,
         "exeute a sql command and return the error code\n\n                This method doesn't raise an exception unless an error\n                unreleated to the SQL command occurs.\n                "},
-    {LIT("cancelCmd"), (PyCFunction)cancelCmd_Loader, METH_VARARGS|METH_KEYWORDS, 
+    {LIT("cancelCmd"), (PyCFunction)cancelCmd_Loader, METH_VARARGS|METH_KEYWORDS,
         "cancels a pending request"},
-    {LIT("release"), (PyCFunction)release_Loader, METH_VARARGS|METH_KEYWORDS, 
+    {LIT("release"), (PyCFunction)release_Loader, METH_VARARGS|METH_KEYWORDS,
         "closes the connection"},
     {NULL, NULL}
 };
@@ -724,8 +729,8 @@ Loader_loader (PyObject *pyself, PyObject* args, PyObject* keywds)
     LoaderObjectT * newObj = NULL;
     static const char * kwlist [] = {
         LIT("servernode"), LIT("dbname"), LIT("instroot"), NULL};
-    char * servernode;
-    char * dbname;
+    char * servernode = LIT( "");
+    char * dbname = LIT( "");
     char * instroot = LIT( "");
     void * session;
     tsp00_ErrTextc msg;
@@ -736,7 +741,7 @@ Loader_loader (PyObject *pyself, PyObject* args, PyObject* keywds)
         ok = 0;
         goto cleanup_label;
     }
-    if (!PyArg_ParseTupleAndKeywords (args, keywds, LIT("ss|s:loader.Loader"), (char **) kwlist, &servernode, &dbname, &instroot)) {
+    if (!PyArg_ParseTupleAndKeywords (args, keywds, LIT("|sss:loader.Loader"), (char **) kwlist, &servernode, &dbname, &instroot)) {
         ok = 0;
         goto cleanup_label;
     }
@@ -779,9 +784,9 @@ _buildInfo_loader (PyObject *pyself, PyObject* args, PyObject* keywds)
 }
 
 static PyMethodDef loaderModuleMethods [] = {
-    {LIT("Loader"), (PyCFunction)Loader_loader, METH_VARARGS|METH_KEYWORDS, 
+    {LIT("Loader"), (PyCFunction)Loader_loader, METH_VARARGS|METH_KEYWORDS,
         "open a connection\n\n                An empty servernode will search for a loader server on\n                the local machine."},
-    {LIT("_buildInfo"), (PyCFunction)_buildInfo_loader, METH_VARARGS|METH_KEYWORDS, 
+    {LIT("_buildInfo"), (PyCFunction)_buildInfo_loader, METH_VARARGS|METH_KEYWORDS,
         "returns a string containing the release and build number"},
     {NULL, NULL}
 };
