@@ -8,7 +8,7 @@
 /*
 
     ========== licence begin  GPL
-    Copyright (c) 2001-2004 SAP AG
+    Copyright (C) 2001-2003 SAP AG
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -57,6 +57,7 @@ extern SAPDBTrace_Topic SysView_Trace;
 
 #define SV_ACCESSCOUNT                      "ACCESSCOUNT"                       /*!<_*/
 #define SV_ACTIVE                           "ACTIVE"                            /*!<_*/
+#define SV_ADDRESS                          "ADDRESS"                           /*!<_*/
 #define SV_ADDRESSINGMODE                   "ADDRESSINGMODE"                    /*!<_*/
 #define SV_ADMINSTATEDATE                   "ADMINSTATEDATE"                    /*!<_*/
 #define SV_ADDNODEDATE                      "ADDNODEDATE"                       /*!<_*/
@@ -69,10 +70,14 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_APPLICATIONPROCESS               "APPLICATIONPROCESS"                /*!<_*/
 #define SV_APPLICATIONTYPE                  "APPLICATIONTYPE"                   /*!<_*/
 #define SV_ARCHIVELOGENABLED                "ARCHIVELOGENABLED"                 /*!<_*/
+#define SV_AVGCOLLISIONCOUNT                "AVGCOLLISIONCOUNT"                 /*!<_*/
 #define SV_AVGEXECUTETIME                   "AVGEXECUTETIME"                    /*!<_*/
 #define SV_AVGHASHTABLECHAINENTRYCOUNT      "AVGHASHTABLECHAINENTRYCOUNT"       /*!<_*/
+#define SV_AVGLOCKCOUNT                     "AVGLOCKCOUNT"                      /*!<_*/
 #define SV_AVGLOCKOBJECTWAITTIME            "AVGLOCKOBJECTWAITTIME"             /*!<_*/
 #define SV_AVGLONGCOLUMNLENGTH              "AVGLONGCOLUMNLENGTH"               /*!<_*/
+#define SV_AVGMAXSPINLOOPCOUNT              "AVGMAXSPINLOOPCOUNT"               /*!<_*/
+#define SV_AVGMAXYIELDLOOPCOUNT             "AVGMAXYIELDLOOPCOUNT"              /*!<_*/
 #define SV_AVGMEMORYDELTA                   "AVGMEMORYDELTA"                    /*!<_*/
 #define SV_AVGREADSTREAMBUFFERCOUNT         "AVGREADSTREAMBUFFERCOUNT"          /*!<_*/
 #define SV_AVGREADSTREAMROWCOUNT            "AVGREADSTREAMROWCOUNT"             /*!<_*/
@@ -83,6 +88,8 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_AVGROWLENGTH                     "AVGROWLENGTH"                      /*!<_*/
 #define SV_AVGSECONDARYKEYLENGTH            "AVGSECONDARYKEYLENGTH"             /*!<_*/
 #define SV_AVGSEPARATORLENGTH               "AVGSEPARATORLENGTH"                /*!<_*/
+#define SV_AVGTOTALSPINLOOPCOUNT            "AVGTOTALSPINLOOPCOUNT"             /*!<_*/
+#define SV_AVGTOTALYIELDLOOPCOUNT           "AVGTOTALYIELDLOOPCOUNT"            /*!<_*/
 #define SV_AVGUSEDENTRYCOUNT                "AVGUSEDENTRYCOUNT"                 /*!<_*/
 #define SV_AVGUSEDENTRYCOUNTPERCENTAGE      "AVGUSEDENTRYCOUNTPERCENTAGE"       /*!<_*/
 #define SV_AVGUSEDSIZEPERCENTAGEINLEAVES    "AVGUSEDSIZEPERCENTAGEINLEAVES"     /*!<_*/
@@ -145,6 +152,8 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_DELETEOBJECTCOUNT                "DELETEOBJECTCOUNT"                 /*!<_*/
 #define SV_DELETEPLANCOUNT                  "DELETEPLANCOUNT"                   /*!<_*/
 #define SV_DESCRIPTION                      "DESCRIPTION"                       /*!<_*/
+#define SV_DESTINATIONMAXTASKRUNNABLETIME   "DESTINATIONMAXTASKRUNNABLETIME"    /*!<_*/
+#define SV_DESTINATIONTASKGROUP             "DESTINATIONTASKGROUP"              /*!<_*/
 #define SV_DEVICESTATE                      "DEVICESTATE"                       /*!<_*/
 #define SV_DISPATCHCOUNT                    "DISPATCHCOUNT"                     /*!<_*/
 #define SV_DISTINCTVALUECOUNT               "DISTINCTVALUECOUNT"                /*!<_*/
@@ -158,6 +167,7 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_ENTRYCOUNT                       "ENTRYCOUNT"                        /*!<_*/
 #define SV_ERRORCODE                        "ERRORCODE"                         /*!<_*/
 #define SV_ERRORTEXT                        "ERRORTEXT"                         /*!<_*/
+#define SV_EXCLUSIVEACCESSCOUNT             "EXCLUSIVEACCESSCOUNT"              /*!<_*/
 #define SV_EXTENDEDTASKSTATE                "EXTENDEDTASKSTATE"                 /*!<_*/
 #define SV_EXECUTECOUNT                     "EXECUTECOUNT"                      /*!<_*/
 #define SV_EXECUTETIME                      "EXECUTETIME"                       /*!<_*/
@@ -184,11 +194,12 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_ID                               "ID"                                /*!<_*/
 #define SV_IDLECOUNT                        "IDLECOUNT"                         /*!<_*/
 #define SV_IDLETIME                         "IDLETIME"                          /*!<_*/
-#define SV_INCREMENTALBACKUPSIZE            "INCREMENTALBACKUPSIZE"            /*!<_*/
+#define SV_INCREMENTALBACKUPSIZE            "INCREMENTALBACKUPSIZE"             /*!<_*/
 #define SV_INDEXNAME                        "INDEXNAME"                         /*!<_*/
 #define SV_INITSTANDBYDATE                  "INITSTANDBYDATE"                   /*!<_*/
 #define SV_INSERTCOMMANDCOUNT               "INSERTCOMMANDCOUNT"                /*!<_*/
 #define SV_INSERTCOUNT                      "INSERTCOUNT"                       /*!<_*/
+#define SV_INTERVALSTARTTIME                "INTERVALSTARTTIME"                 /*!<_*/
 #define SV_ISOLATIONLEVEL                   "ISOLATIONLEVEL"                    /*!<_*/
 #define SV_INSTALLSERVERDBVERSION           "INSTALLSERVERDBVERSION"            /*!<_*/
 #define SV_INTERNAL                         "INTERNAL"                          /*!<_*/
@@ -207,6 +218,7 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_LASTDATABACKUPLOGSEQUENCE        "LASTDATABACKUPLOGSEQUENCE"         /*!<_*/
 #define SV_LASTDATABACKUPVERSION            "LASTDATABACKUPVERSION"             /*!<_*/
 #define SV_LASTEMERGENCYHISTORYCLEANUP      "LASTEMERGENCYHISTORYCLEANUP"       /*!<_*/
+#define SV_LASTENTEREDTASKID                "LASTENTEREDTASKID"              /*!<_*/
 #define SV_LASTFULLDATABACKUPSUCCESSFUL     "LASTFULLDATABACKUPSUCCESSFUL"      /*!<_*/
 #define SV_LASTFULLDATABACKUPVERSION        "LASTFULLDATABACKUPVERSION"         /*!<_*/
 #define SV_LASTHISTORYSEQUENCE              "LASTHISTORYSEQUENCE"               /*!<_*/
@@ -243,10 +255,14 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_MAJORVERSION                     "MAJORVERSION"                      /*!<_*/
 #define SV_MASSCOMMAND                      "MASSCOMMAND"                       /*!<_*/
 #define SV_MASTERLOCALNODE                  "MASTERLOCALNODE"                   /*!<_*/
+#define SV_MAXCOLLISIONCOUNT                "MAXCOLLISIONCOUNT"                 /*!<_*/
 #define SV_MAXCOMMANDCOUNT                  "MAXCOMMANDCOUNT"                   /*!<_*/
 #define SV_MAXEXECUTETIME                   "MAXEXECUTETIME"                    /*!<_*/
 #define SV_MAXHASHTABLECHAINENTRYCOUNT      "MAXHASHTABLECHAINENTRYCOUNT"       /*!<_*/
+#define SV_MAXLOCKCOUNT                     "MAXLOCKCOUNT"                      /*!<_*/
 #define SV_MAXLONGCOLUMNLENGTH              "MAXLONGCOLUMNLENGTH"               /*!<_*/
+#define SV_MAXMAXSPINLOOPCOUNT              "MAXMAXSPINLOOPCOUNT"               /*!<_*/
+#define SV_MAXMAXYIELDLOOPCOUNT             "MAXMAXYIELDLOOPCOUNT"              /*!<_*/
 #define SV_MAXMEMORYDELTA                   "MAXMEMORYDELTA"                    /*!<_*/
 #define SV_MAXPARSEIDCOUNT                  "MAXPARSEIDCOUNT"                   /*!<_*/
 #define SV_MAXPENDINGIOCOUNT                "MAXPENDINGIOCOUNT"                 /*!<_*/
@@ -263,7 +279,10 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_MAXSECONDARYKEYLENGTH            "MAXSECONDARYKEYLENGTH"             /*!<_*/
 #define SV_MAXSEPARATORLENGTH               "MAXSEPARATORLENGTH"                /*!<_*/
 #define SV_MAXSPINLOOPCOUNT                 "MAXSPINLOOPCOUNT"                  /*!<_*/
+#define SV_MAXTASKRUNNABLETIME              "MAXTASKRUNNABLETIME"               /*!<_*/
 #define SV_MAXTEMPORARYDATAPAGENUMBER       "MAXTEMPORARYDATAPAGENUMBER"        /*!<_*/
+#define SV_MAXTOTALSPINLOOPCOUNT            "MAXTOTALSPINLOOPCOUNT"             /*!<_*/
+#define SV_MAXTOTALYIELDLOOPCOUNT           "MAXTOTALYIELDLOOPCOUNT"            /*!<_*/
 #define SV_MAXUSEDENTRYCOUNT                "MAXUSEDENTRYCOUNT"                 /*!<_*/
 #define SV_MAXUSEDENTRYCOUNTPERCENTAGE      "MAXUSEDENTRYCOUNTPERCENTAGE"       /*!<_*/
 #define SV_MAXUSEDHASHTABLESLOTCOUNT        "MAXUSEDHASHTABLESLOTCOUNT"         /*!<_*/
@@ -281,10 +300,17 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_MAXWRITESTREAMBUFFERCOUNT        "MAXWRITESTREAMBUFFERCOUNT"         /*!<_*/
 #define SV_MAXWRITESTREAMROWCOUNT           "MAXWRITESTREAMROWCOUNT"            /*!<_*/
 #define SV_MAXYIELDLOOPCOUNT                "MAXYIELDLOOPCOUNT"                 /*!<_*/
+#define SV_MEASUREMENTINTERVALS             "MEASUREMENTINTERVALS"              /*!<_*/
+#define SV_MEASUREMENTINTERVALTIME          "MEASUREMENTINTERVALTIME"           /*!<_*/
 #define SV_MEMORYRELEASECOUNT               "MEMORYRELEASECOUNT"                /*!<_*/
 #define SV_MEMORYREQUESTCOUNT               "MEMORYREQUESTCOUNT"                /*!<_*/
+#define SV_MINCOLLISIONCOUNT                "MINCOLLISIONCOUNT"                 /*!<_*/
+#define SV_MINDIFFERENCE                    "MINDIFFERENCE"                     /*!<_*/
 #define SV_MINEXECUTETIME                   "MINEXECUTETIME"                    /*!<_*/
+#define SV_MINLOCKCOUNT                     "MINLOCKCOUNT"                      /*!<_*/
 #define SV_MINLONGCOLUMNLENGTH              "MINLONGCOLUMNLENGTH"               /*!<_*/
+#define SV_MINMAXSPINLOOPCOUNT              "MINMAXSPINLOOPCOUNT"               /*!<_*/
+#define SV_MINMAXYIELDLOOPCOUNT             "MINMAXYIELDLOOPCOUNT"              /*!<_*/
 #define SV_MINMEMORYDELTA                   "MINMEMORYDELTA"                    /*!<_*/
 #define SV_MINORVERSION                     "MINORVERSION"                      /*!<_*/
 #define SV_MINREADSTREAMBUFFERCOUNT         "MINREADSTREAMBUFFERCOUNT"          /*!<_*/
@@ -296,17 +322,24 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_MINROWLENGTH                     "MINROWLENGTH"                      /*!<_*/
 #define SV_MINSECONDARYKEYLENGTH            "MINSECONDARYKEYLENGTH"             /*!<_*/
 #define SV_MINSEPARATORLENGTH               "MINSEPARATORLENGTH"                /*!<_*/
+#define SV_MINTOTALSPINLOOPCOUNT            "MINTOTALSPINLOOPCOUNT"             /*!<_*/
+#define SV_MINTOTALYIELDLOOPCOUNT           "MINTOTALYIELDLOOPCOUNT"            /*!<_*/
 #define SV_MINUSEDSIZEPERCENTAGEINLEAVES    "MINUSEDSIZEPERCENTAGEINLEAVES"     /*!<_*/
 #define SV_MINWAITTIMEFORNEWCONSISTENTVIEW  "MINWAITTIMEFORNEWCONSISTENTVIEW"   /*!<_*/
 #define SV_MINWRITESTREAMBUFFERCOUNT        "MINWRITESTREAMBUFFERCOUNT"         /*!<_*/
 #define SV_MINWRITESTREAMROWCOUNT           "MINWRITESTREAMROWCOUNT"            /*!<_*/
 #define SV_MODE                             "MODE"                              /*!<_*/
+#define SV_MOVEABLETASKCOUNT                "MOVEABLETASKCOUNT"                 /*!<_*/
+#define SV_MOVETASKTOTASKGROUPID            "MOVETASKTOTASKGROUPID"             /*!<_*/
+#define SV_MOVETIME                         "MOVETIME"                          /*!<_*/
 #define SV_MULTIPLE                         "MULTIPLE"                          /*!<_*/
 #define SV_NAME                             "NAME"                              /*!<_*/
 #define SV_NOTSAVEDPERCENTAGE               "NOTSAVEDPERCENTAGE"                /*!<_*/
 #define SV_NOTSAVEDSIZE                     "NOTSAVEDSIZE"                      /*!<_*/
 #define SV_NODE                             "NODE"                              /*!<_*/
+#define SV_NONEXCLUSIVEACCESSCOUNT          "NONEXCLUSIVEACCESSCOUNT"           /*!<_*/
 #define SV_NUMERICVALUE                     "NUMERICVALUE"                      /*!<_*/
+#define SV_OBJECTID                         "OBJECTID"                          /*!<_*/
 #define SV_OBJECTSIZE                       "OBJECTSIZE"                        /*!<_*/
 #define SV_OLDESTNOTSAVEDLOGOFFSET          "OLDESTNOTSAVEDLOGOFFSET"           /*!<_*/
 #define SV_OLDESTNOTSAVEDLOGSEQUENCE        "OLDESTNOTSAVEDLOGSEQUENCE"         /*!<_*/
@@ -368,6 +401,7 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_READCACHEDOBJECTCOUNT            "READCACHEDOBJECTCOUNT"             /*!<_*/
 #define SV_READCACHEDOBJECTVIAKEYCOUNT      "READCACHEDOBJECTVIAKEYCOUNT"       /*!<_*/
 #define SV_READCOUNT                        "READCOUNT"                         /*!<_*/
+#define SV_READERWRITERLOCK                 "READERWRITERLOCK"                  /*!<_*/
 #define SV_READOBJECTCOUNT                  "READOBJECTCOUNT"                   /*!<_*/
 #define SV_READOBJECTHISTORYACCESSCOUNT     "READOBJECTHISTORYACCESSCOUNT"      /*!<_*/
 #define SV_READOBJECTVIAKEYCOUNT            "READOBJECTVIAKEYCOUNT"             /*!<_*/
@@ -387,6 +421,7 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_REMOVENODEDATE                   "REMOVENODEDATE"                    /*!<_*/
 #define SV_REQUESTMODE                      "REQUESTMODE"                       /*!<_*/
 #define SV_REQUESTORCOUNT                   "REQUESTORCOUNT"                    /*!<_*/
+#define SV_REQUESTORTASKID                  "REQUESTORTASKID"                   /*!<_*/
 #define SV_REQUESTSTATE                     "REQUESTSTATE"                      /*!<_*/
 #define SV_RESTARTENTRYDATE                 "RESTARTENTRYDATE"                  /*!<_*/
 #define SV_RESTARTENTRYTYPE                 "RESTARTENTRYTYPE"                  /*!<_*/
@@ -422,6 +457,9 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_SLEEPCOUNT                       "SLEEPCOUNT"                        /*!<_*/
 #define SV_SLEEPRELATIVETIME                "SLEEPRELATIVETIME"                 /*!<_*/
 #define SV_SLOT                             "SLOT"                              /*!<_*/
+#define SV_SOURCEMAXTASKRUNNABLETIME        "SOURCEMAXTASKRUNNABLETIME"         /*!<_*/
+#define SV_SOURCETASKGROUP                  "SOURCETASKGROUP"                   /*!<_*/
+#define SV_SPINLOCKPOOLNAME                 "SPINLOCKPOOLNAME"                  /*!<_*/
 #define SV_SPINLOCKCOLLISIONCOUNT           "SPINLOCKCOLLISIONCOUNT"            /*!<_*/
 #define SV_SPINLOOPCOUNT                    "SPINLOOPCOUNT"                     /*!<_*/
 #define SV_SQLCMD                           "SQLCMD"                            /*!<_*/
@@ -444,6 +482,7 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_STATEMENTSIZE                    "STATEMENTSIZE"                     /*!<_*/
 #define SV_STATEMENTWAITABSOLUTETIME        "STATEMENTWAITABSOLUTETIME"         /*!<_*/
 #define SV_STATEMENTWAITRELATIVETIME        "STATEMENTWAITRELATIVETIME"         /*!<_*/
+#define SV_STATISTICSDELTACOUNT             "STATISTICSDELTACOUNT"              /*!<_*/
 #define SV_SUBTRANSACTIONLEVEL              "SUBTRANSACTIONLEVEL"               /*!<_*/
 #define SV_STATUS                           "STATUS"                            /*!<_*/
 #define SV_SUCCESSFULACCESSCOUNT            "SUCCESSFULACCESSCOUNT"             /*!<_*/
@@ -466,6 +505,7 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_TASKCOUNT                        "TASKCOUNT"                         /*!<_*/
 #define SV_TASKGROUPBLOCKED                 "TASKGROUPBLOCKED"                  /*!<_*/
 #define SV_TASKGROUPID                      "TASKGROUPID"                       /*!<_*/
+#define SV_TASKGROUPSLEEPTIME               "TASKGROUPSLEEPTIME"                   /*!<_*/
 #define SV_TASKID                           "TASKID"                            /*!<_*/
 #define SV_TASKRESCEDULECOUNT               "TASKRESCEDULECOUNT"                /*!<_*/
 #define SV_TASKSTATE                        "TASKSTATE"                         /*!<_*/
@@ -475,6 +515,7 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_THREADNAME                       "THREADNAME"                        /*!<_*/
 #define SV_TIMEOUT                          "TIMEOUT"                           /*!<_*/
 #define SV_TIMEOUTEXCEPTIONCOUNT            "TIMEOUTEXCEPTIONCOUNT"             /*!<_*/
+#define SV_TOTALTASKRUNNABLETIME            "TOTALTASKRUNNABLETIME"                /*!<_*/
 #define SV_TRACE                            "TRACE"                             /*!<_*/
 #define SV_TRANSACTIONCOUNT                 "TRANSACTIONCOUNT"                  /*!<_*/
 #define SV_TRANSACTIONID                    "TRANSACTIONID"                     /*!<_*/
@@ -505,6 +546,7 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_USERID                           "USERID"                            /*!<_*/
 #define SV_USERNAME                         "USERNAME"                          /*!<_*/
 #define SV_VALUE                            "VALUE"                             /*!<_*/
+#define SV_VALUEEQUALITY                    "VALUEEQUALITY"                     /*!<_*/
 #define SV_VAROBJECTCONTAINER               "VAROBJECTCONTAINER"                /*!<_*/
 #define SV_VIRTUALREADCOUNT                 "VIRTUALREADCOUNT"                  /*!<_*/
 #define SV_VIRTUALMEMORYSIZE                "VIRTUALMEMORYSIZE"                 /*!<_*/
@@ -568,12 +610,15 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_CC_MEMORYALLOCATORSTATISTICSRESET     8 /*!<_*/
 #define SV_CC_MEMORYHOLDERS                      3 /*!<_*/
 #define SV_CC_MISCELLANEOUS                      5 /*!<_*/
+#define SV_CC_OBJECTLOCKS                        4 /*!<_*/
 #define SV_CC_OMSLOCKS                           5 /*!<_*/
 #define SV_CC_OMSPROCEDURESTATISTICS            54 /*!<_*/
 #define SV_CC_OMSPROCEDURESTATISTICSRESET       54 /*!<_*/
 #define SV_CC_OMSVERSIONS                        7 /*!<_*/
 #define SV_CC_OPTIMIZERINFORMATION               6 /*!<_*/
 #define SV_CC_PAGELOCKS                          4 /*!<_*/
+#define SV_CC_READERWRITERLOCKSTATISTICS         8 /*!<_*/
+#define SV_CC_READERWRITERLOCKSTATISTICSRESET    6/*!<_*/
 #define SV_CC_REGIONLOCKS                        3 /*!<_*/
 #define SV_CC_REGIONSTATISTICS                   9 /*!<_*/
 #define SV_CC_REGIONSTATISTICSRESET              6 /*!<_*/
@@ -581,18 +626,22 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_CC_SCHEMAS                            5 /*!<_*/
 #define SV_CC_SESSIONS                          19 /*!<_*/
 #define SV_CC_SNAPSHOTS                          3 /*!<_*/
+#define SV_CC_SPINLOCKPOOLSTATISTICS            18 /*!<_*/
+#define SV_CC_SPINLOCKPOOLSTATISTICSRESET       18 /*!<_*/
 #define SV_CC_SPINLOCKSTATISTICS                 7 /*!<_*/
 #define SV_CC_SPINLOCKSTATISTICSRESET            7 /*!<_*/
 #define SV_CC_TABLESIZEDETAILS                  21 /*!<_*/
 #define SV_CC_TASKGROUPSTATISTICS               11 /*!<_*/
 #define SV_CC_TASKGROUPSTATISTICSRESET           8 /*!<_*/
-#define SV_CC_TASKSTATISTICS                    54 /*!<_*/
+#define SV_CC_TASKLOADBALANCINGINFORMATION       5 /*!<_*/
+#define SV_CC_TASKLOADBALANCINGTASKGROUPSTATES   7 /*!<_*/
+#define SV_CC_TASKLOADBALANCINGTASKMOVES         7 /*!<_*/
+#define SV_CC_TASKSTATISTICS                    55 /*!<_*/
 #define SV_CC_TASKSTATISTICSRESET               36 /*!<_*/
 #define SV_CC_TRANSACTIONS                      15 /*!<_*/
 #define SV_CC_VERSION                            9 /*!<_*/
 
 // Table estimated rows:
-
 #define SV_ER_ACTIVECONFIGURATION               150 /*!<_*/
 #define SV_ER_BACKUPINFORMATION                   1 /*!<_*/
 #define SV_ER_BACKUPTHREADS                      10 /*!<_*/
@@ -633,12 +682,15 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_ER_MEMORYALLOCATORSTATISTICSRESET    100 /*!<_*/
 #define SV_ER_MEMORYHOLDERS                     500 /*!<_*/
 #define SV_ER_MISCELLANEOUS                      10 /*!<_*/
+#define SV_ER_OBJECTLOCKS                         1 /*!<_*/
 #define SV_ER_OMSLOCKS                          100 /*!<_*/
 #define SV_ER_OMSPROCEDURESTATISTICS            100 /*!<_*/
 #define SV_ER_OMSPROCEDURESTATISTICSRESET       100 /*!<_*/
 #define SV_ER_OMSVERSIONS                       100 /*!<_*/
 #define SV_ER_OPTIMIZERINFORMATION            15000 /*!<_*/
 #define SV_ER_PAGELOCKS                          50 /*!<_*/
+#define SV_ER_READERWRITERLOCKSTATISTICS          1 /*!<_*/
+#define SV_ER_READERWRITERLOCKSTATISTICSRESET     1 /*!<_*/
 #define SV_ER_REGIONLOCKS                       100 /*!<_*/
 #define SV_ER_REGIONSTATISTICS                   25 /*!<_*/
 #define SV_ER_REGIONSTATISTICSRESET              25 /*!<_*/
@@ -646,11 +698,16 @@ extern SAPDBTrace_Topic SysView_Trace;
 #define SV_ER_SCHEMAS                            20 /*!<_*/
 #define SV_ER_SNAPSHOTS                           5 /*!<_*/
 #define SV_ER_SESSIONS                           50 /*!<_*/
+#define SV_ER_SPINLOCKPOOLSTATISTICS             20 /*!<_*/
+#define SV_ER_SPINLOCKPOOLSTATISTICSRESET        20 /*!<_*/
 #define SV_ER_SPINLOCKSTATISTICS                 20 /*!<_*/
 #define SV_ER_SPINLOCKSTATISTICSRESET            20 /*!<_*/
 #define SV_ER_TABLESIZEDETAILS                12000 /*!<_*/
 #define SV_ER_TASKGROUPSTATISTICS                10 /*!<_*/
 #define SV_ER_TASKGROUPSTATISTICSRESET           10 /*!<_*/
+#define SV_ER_TASKLOADBALANCINGINFORMATION        1 /*!<_*/
+#define SV_ER_TASKLOADBALANCINGTASKGROUPSTATES   10 /*!<_*/
+#define SV_ER_TASKLOADBALANCINGTASKMOVES         10 /*!<_*/
 #define SV_ER_TASKSTATISTICS                     50 /*!<_*/
 #define SV_ER_TASKSTATISTICSRESET                50 /*!<_*/
 #define SV_ER_TRANSACTIONS                       50 /*!<_*/
