@@ -1,13 +1,15 @@
+#ifndef TABLE_TEMPHASHTABLE_HPP
+#define TABLE_TEMPHASHTABLE_HPP
 /*!
     @file     Table_TempHashTable.hpp
     @ingroup  Table
     @author   DirkT
-    @brief    Temporary hash table
+    @brief    Temporary hash table OR temporary single record for aggregation
     @see            
 
 \if EMIT_LICENCE
     ========== licence begin  GPL
-    Copyright (c) 2000-2004 SAP AG
+    Copyright (C) 2000-2004 SAP AG
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -47,7 +49,6 @@
 #include "SAPDBCommon/Tracing/SAPDBTrace_Usage.hpp"
 
 #include "RunTime/Configuration/RTEConf_ParameterAccessKernelInterface.hpp"
-//#include "RunTime/RTE_Message.hpp"
 
 #include "Table/Table_Messages.hpp"
 #include "Table/Table_Exceptions.hpp"
@@ -63,10 +64,10 @@ typedef SAPDB_UInt4 TempHashTable_HashValue;
 
 // #############################################################################
 
-class Table_TempHashTable_IDataMerger
+class Table_TempHashTableAggregate_IDataMerger
 {
 public:
-    virtual void MergeData( const SAPDB_Byte* Source, SAPDB_Byte* Destination, SAPDB_UInt2 Length, bool Found ) = 0;
+    virtual void MergeData( const SAPDB_Byte* Source, SAPDB_Byte* Destination, SAPDB_UInt2 Length, SAPDB_Bool Found ) = 0;
 };
 
 // #############################################################################
@@ -91,9 +92,139 @@ typedef Container_Vector<SAPDB_UInt2>               Table_FixSizeEntry_KeyNoList
 
 // #############################################################################
 
-//template <class TMerger=Table_TempHashTable_IDataMerger>  // would be nice, but LINUX can't handle this: internal compiler error
-class Table_FixSizeTempHashTable : public SAPDBAlgo_IMergeSource<SAPDB_Byte*>
-                                 , public SAPDBAlgo_IComparator<SAPDB_Byte*>
+class Table_FixSizeTempAggregate 
+{
+public:
+	virtual SAPDB_Bool CompletelyConstructed( void ) = 0;
+    virtual SAPDB_Bool InsertOrReplace( SAPDB_Byte* Data, SAPDBErr_MessageList& ErrorList ) = 0;
+    virtual SAPDB_Bool First( SAPDBErr_MessageList& ErrorList ) = 0;
+    virtual SAPDB_Bool Next( void ) = 0;
+    virtual SAPDB_Byte* Get( void ) = 0;
+    virtual SAPDB_Bool IsValid( void ) = 0;
+    virtual SAPDB_Bool IsEmpty( void ) = 0;
+    virtual ~Table_FixSizeTempAggregate() {}
+};
+
+// #############################################################################
+
+class Table_FixSizeTempSingleRecordAggregate
+	: public Table_FixSizeTempAggregate
+	, public SAPDBAlgo_IMergeSource<SAPDB_Byte*>
+{
+
+public:
+    Table_FixSizeTempSingleRecordAggregate( tgg00_TransContext&             TransContext, 
+                                Table_TempHashTableAggregate_IDataMerger&	Merger,
+                                SAPDB_UInt4									RecordLength,
+                                SAPDBErr_MessageList&					    ErrorList
+                                )
+		: mSessionAllocator(*( REINTERPRET_CAST(SAPDBMem_IRawAllocator*, TransContext.trAllocator_gg00) ))
+        , mMerger(Merger)
+		, mRecord(0)
+        , mRecordLength(RecordLength)
+		, mFirstInsert(true)
+        , mValid(false)
+	{
+		mRecord = (SAPDB_Byte*) mSessionAllocator.Allocate(RecordLength);
+	}
+
+	virtual ~Table_FixSizeTempSingleRecordAggregate( void )
+	{
+		if ( mRecord )
+			destroy(mRecord, mSessionAllocator);
+	}
+
+	virtual SAPDB_Bool CompletelyConstructed( void )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempSingleRecordAggregate::CompletelyConstructed", Table_Trace, 5);
+        return ( mRecord );
+    }
+
+    virtual SAPDB_Bool InsertOrReplace( SAPDB_Byte* Data, SAPDBErr_MessageList& ErrorList )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempSingleRecordAggregate::InsertOrReplace", Table_Trace, 5);
+        SAPDBERR_ASSERT_STATE( Data );
+
+	    mMerger.MergeData(Data, mRecord, mRecordLength, !mFirstInsert);
+        mFirstInsert = false;
+		mValid = false;	// for Next()
+		return true;
+    }
+
+    SAPDB_Bool First( SAPDBErr_MessageList& ErrorList )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempSingleRecordAggregate::First", Table_Trace, 5);
+		return (mValid=true);
+    }
+
+    SAPDB_Bool Next( void )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempSingleRecordAggregate::Next", Table_Trace, 5);
+        SAPDBERR_ASSERT_STATE( mValid );
+        return (mValid=false);
+    }
+
+    virtual SAPDB_Byte* Get( void )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempSingleRecordAggregate::Get", Table_Trace, 5);
+        SAPDBERR_ASSERT_STATE( mValid );
+        return mRecord;
+    }
+
+    virtual SAPDB_Bool IsValid( void )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempSingleRecordAggregate::IsValid", Table_Trace, 5);
+        return mValid;
+    }
+
+    virtual SAPDB_Bool IsEmpty( void )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempSingleRecordAggregate::IsEmpty", Table_Trace, 5);
+        return mFirstInsert;
+    }
+
+    virtual unsigned int GetListCount( void )
+    {
+        // dummy method
+        return 1;
+    }
+
+    virtual SAPDB_Byte*& GetListElement( unsigned int SourceIndex )
+    {
+        // dummy method
+        static SAPDB_Byte* _do_not_use_this_pointer = 0;
+        return _do_not_use_this_pointer;
+    }
+
+    virtual bool NextListElement( unsigned int SourceIndex )
+    {
+        // dummy method
+        return false;
+    }
+    
+    virtual bool Valid( unsigned int SourceIndex )
+    {
+        // dummy method
+        return false;
+    }
+    
+private:
+    SAPDBMem_IRawAllocator&                         mSessionAllocator;
+    Table_TempHashTableAggregate_IDataMerger&       mMerger;
+
+	SAPDB_Byte*                                     mRecord;
+    SAPDB_UInt2                                     mRecordLength;
+	SAPDB_Bool										mFirstInsert;
+    SAPDB_Bool                                      mValid;
+};
+
+// #############################################################################
+
+//template <class TMerger=Table_TempHashTableAggregate_IDataMerger>  // would be nice, but LINUX can't handle this: internal compiler error
+class Table_FixSizeTempHashTableAggregate 
+	: public Table_FixSizeTempAggregate
+    , public SAPDBAlgo_IMergeSource<SAPDB_Byte*>
+    , public SAPDBAlgo_IComparator<SAPDB_Byte*>
 {
 private:
 
@@ -109,14 +240,13 @@ private:
 
 public:
 
-    Table_FixSizeTempHashTable( tgg00_TransContext&                     TransContext, 
-//                                TMerger&                                Merger, 
-                                Table_TempHashTable_IDataMerger&        Merger,
-                                SAPDB_UInt4                             RecordLength,
-                                Table_FixSizeEntry_FieldDescription&    FieldDescription, 
-                                Table_FixSizeEntry_KeyNoList&           KeyNoList,
-                                SAPDB_UInt8                             ExpectedEntries,
-                                SAPDBErr_MessageList&                   ErrorList
+    Table_FixSizeTempHashTableAggregate( tgg00_TransContext&                TransContext, 
+                                Table_TempHashTableAggregate_IDataMerger&   Merger,
+                                SAPDB_UInt4                                 RecordLength,
+                                Table_FixSizeEntry_FieldDescription&        FieldDescription, 
+                                Table_FixSizeEntry_KeyNoList&               KeyNoList,
+                                SAPDB_UInt8                                 ExpectedEntries,
+                                SAPDBErr_MessageList&                       ErrorList
                                 )
         : mTransContext(TransContext)
         , mSessionAllocator(*( REINTERPRET_CAST(SAPDBMem_IRawAllocator*, TransContext.trAllocator_gg00) ))
@@ -135,8 +265,9 @@ public:
         , mActEntryCnt(0)
         , mFirstFree(0)
         , mValid(false)
+        , mFirstInsert(true)
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::Table_FixSizeTempHashTable", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::Table_FixSizeTempHashTableAggregate", Table_Trace, 8);
 
         RTEConf_Parameter::Integer ParamSize=0;
         SAPDBErr_MessageList       errLst;
@@ -150,12 +281,19 @@ public:
 
         mFieldDescription.Initialize(FieldDescription);
         mKeyNoList.Initialize(KeyNoList);
+        SAPDBERR_ASSERT_STATE( KeyNoList.GetSize() > 0 );
+        SAPDBERR_ASSERT_STATE( FieldDescription.GetSize() >= KeyNoList.GetSize()  );
 
         SAPDB_UInt4  SizePerEntry    = 2*sizeof(void*) + sizeof(TEntry)+RecordLength;  // assume optimal hashing ...
         mMaxEntryCnt                 = 1024L*kBSizeMax / SizePerEntry;
         mMaxEntryCnt                 = (SAPDBAlgo_MAX_QUICKSORT_ELEMENTS < mMaxEntryCnt ? 
                                         SAPDBAlgo_MAX_QUICKSORT_ELEMENTS : mMaxEntryCnt );
 
+		SAPDB_UInt4 RecordPerPageCnt = (SAPDB_Int4)(Data_TempFile::MaxFreeSpacePerPage()/RecordLength);
+
+		if ( (mMaxEntryCnt > ExpectedEntries) && (mMaxEntryCnt > RecordPerPageCnt) )
+			mMaxEntryCnt = (SAPDB_Int4)(ExpectedEntries > RecordPerPageCnt ? ExpectedEntries : RecordPerPageCnt);
+
         mNumberOfSlots               = mMaxEntryCnt;
 
         mKeyCount = mKeyNoList.GetSize();
@@ -191,9 +329,9 @@ public:
         }
     }
 
-    ~Table_FixSizeTempHashTable( void )
+    virtual ~Table_FixSizeTempHashTableAggregate( void )
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::~Table_FixSizeTempHashTable", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::~Table_FixSizeTempHashTableAggregate", Table_Trace, 8);
 
         DestroyHashTableAndAllocator();
 
@@ -206,13 +344,152 @@ public:
         }
     }
 
-    SAPDB_Bool CompletelyConstructed( void )
+    // ----------------------------------------------------------------------------------------------
+
+    virtual SAPDB_Bool CompletelyConstructed( void )
     {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::CompletelyConstructed", Table_Trace, 5);
         return ( mAllocator && mHashTable );
     }
 
+    virtual SAPDB_Bool InsertOrReplace( SAPDB_Byte* Data, SAPDBErr_MessageList& ErrorList )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::InsertOrReplace", Table_Trace, 5);
+        SAPDBERR_ASSERT_STATE( Data );
+
+        mFirstInsert = false;
+        TempHashTable_HashValue Hash = CalcHashValueForKeys(Data);
+        TempHashTable_HashValue Slot = CalcSlot(Hash);
+
+        SAPDBTRACE_WRITELN( Table_Trace, 3, "hash: " << Hash << " slot: " << Slot );
+
+        if ( TEntry* Act = mHashTable[Slot] )
+        {
+            do
+            {
+                SAPDB_Byte* A = (SAPDB_Byte*)(Act+1);
+                if ( IsEqual(Data, A) )
+                {
+                    mMerger.MergeData(Data, A, mRecordLength, true);
+                    mValid = false;
+                    return true;
+                }
+            }while ( Act = Act->mNext );
+        }
+
+        if ( TEntry* NewEntry = GetMemForEntry(ErrorList) )
+        {
+            NewEntry->mNext = mHashTable[Slot];
+            mHashTable[Slot]= NewEntry;
+
+            SAPDB_Byte* N = (SAPDB_Byte*)(NewEntry+1);
+            mMerger.MergeData(Data, N, mRecordLength, false);
+            mValid = false;
+            return true;
+        }
+        else
+        {
+            return false;
+        }
+    }
+
+    virtual SAPDB_Bool First( SAPDBErr_MessageList& ErrorList )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::First", Table_Trace, 5);
+
+        if ( !SortEntries(false, ErrorList) )
+        {
+            ErrorList = ErrorList + Table_Exception(__CONTEXT__, TABLE_TEMPHASH_FIRSTFAILED);
+            return false;
+        }
+        mActSortTableEntry = 0;
+
+        SAPDB_UInt4 FileCnt = mTempFileContainer.GetSize();
+
+        for ( SAPDB_UInt4 i=0; i<FileCnt; i++ )
+        {
+            Data_TempFile::RecordIterator*& Iter = mTempFileContainer[i].mIter;
+            if ( !Iter )
+            {
+                if ( !(Iter = new(mSessionAllocator) Data_TempFile::RecordIterator(mTempFileContainer[i].mFile->PageAccessManager())) )
+                {
+                    ErrorList = Table_Exception(__CONTEXT__, TABLE_TEMPHASH_NOITERATOR);
+                    return 0;
+                }
+            }
+            if(!mTempFileContainer[i].mFile->GetFirst(*Iter))
+                return 0;
+            if ( Iter->IsValid() )
+                mTempFileContainer[i].mData = *(*(Iter));
+            else
+                mTempFileContainer[i].mData = 0;
+        }
+
+        if ( mFunnelMergeIterator = new(mSessionAllocator) SAPDBAlgo_FunnelMergeIteratorIdx<SAPDB_Byte*>(mSessionAllocator, *this, *this, ErrorList) )
+        {
+            if ( mFunnelMergeIterator->CompletelyConstructed() )
+            {
+                mValid = true;
+                Collect();
+                return true;
+            }
+            else
+            {
+                ErrorList = ErrorList + Table_Exception(__CONTEXT__, TABLE_TEMPHASH_FIRSTFAILED);
+                return false;
+            }
+        }
+        else
+        {
+            ErrorList = Table_Exception(__CONTEXT__, TABLE_TEMPHASH_NOITERATOR);
+            ErrorList = ErrorList + Table_Exception(__CONTEXT__, TABLE_TEMPHASH_FIRSTFAILED);
+            return false;
+        }
+    }
+
+    virtual SAPDB_Bool Next( void )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::Next", Table_Trace, 5);
+        SAPDBERR_ASSERT_STATE( mValid );
+
+        Collect();
+        return mValid;
+    }
+
+    virtual SAPDB_Byte* Get( void )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::Get", Table_Trace, 5);
+
+        return mFetchedRecord;
+    }
+
+    virtual SAPDB_Bool IsValid( void )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::IsValid", Table_Trace, 5);
+
+        return mValid;
+    }
+
+    virtual SAPDB_Bool IsEmpty( void )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::IsEmpty", Table_Trace, 5);
+        return mFirstInsert;
+    }
+
+    SAPDB_Bool ForceFlushAndFreeMem( SAPDBErr_MessageList& ErrorList )
+    {
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::ForceFlushAndFreeMem", Table_Trace, 5);
+
+        if ( FlushToPage(ErrorList) )
+        {
+            DestroyHashTableAndAllocator();
+            return true;
+        }
+        return false;
+    }
     // ----------------------------------------------------------------------------------------------
 
+public:
     // Methods derived from SAPDBAlgo_IMergeSource<SAPDB_Byte*>
     virtual unsigned int GetListCount( void )
     {
@@ -225,12 +502,12 @@ public:
 
         if ( SourceIndex==0 )
         {
-            SAPDBERR_ASSERT_STATE( mActSortTableEntry < mSortTableEntryCnt && mSortTableEntryCnt > 0)
+            SAPDBERR_ASSERT_STATE( mActSortTableEntry < mSortTableEntryCnt && mSortTableEntryCnt > 0);
             return mSortTable[mActSortTableEntry];
         }
         else
         {
-            SAPDBERR_ASSERT_STATE( mTempFileContainer[SourceIndex-1].mIter->IsValid() )
+            SAPDBERR_ASSERT_STATE( mTempFileContainer[SourceIndex-1].mIter->IsValid() );
             return mTempFileContainer[SourceIndex-1].mData;  
         }
     }
@@ -275,9 +552,9 @@ public:
 
         for ( unsigned int i=0; i<mKeyCount; i++ )
         {
-            if ( memcmp( A + mFieldDescription[i].Offset, 
-                         B + mFieldDescription[i].Offset,
-                        mFieldDescription[i].Length) != 0)
+            if ( memcmp( A + mFieldDescription[mKeyNoList[i]].Offset, 
+                         B + mFieldDescription[mKeyNoList[i]].Offset,
+                        mFieldDescription[mKeyNoList[i]].Length) != 0)
                 return false;
         }
         return true;
@@ -286,165 +563,30 @@ public:
     {
         SAPDBERR_ASSERT_STATE( A && B);
 
-        int v, s=0;
+        int v;
 
         for ( unsigned int i=0; i<mKeyCount; i++ )
         {
-            v = memcmp( A + mFieldDescription[i].Offset, 
-                        B + mFieldDescription[i].Offset,
-                        mFieldDescription[i].Length);
+            v = memcmp( A + mFieldDescription[mKeyNoList[i]].Offset, 
+                        B + mFieldDescription[mKeyNoList[i]].Offset,
+                        mFieldDescription[mKeyNoList[i]].Length);
 
-            if (v>0)
+            if ( v>0 )
                 return false;
-            else
-                s += v;
+            if ( v<0 )
+                return true;
         }
-        return (s<0);  // s==0 means all keys are equal -> return false
+        return false;  // all keys are equal -> return false
     }
 
     // ----------------------------------------------------------------------------------------------
 
-    SAPDB_Bool InsertOrReplace( SAPDB_Byte* Data, SAPDBErr_MessageList& ErrorList )
-    {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::InsertOrReplace", Table_Trace, 5);
-        SAPDBERR_ASSERT_STATE( Data );
-
-        if ( mAllocator && mHashTable )
-        {
-            TempHashTable_HashValue Hash = CalcHashValueForKeys(Data);
-            TempHashTable_HashValue Slot = CalcSlot(Hash);
-
-            if ( TEntry* Act = mHashTable[Slot] )
-            {
-                do
-                {
-                    SAPDB_Byte* A = (SAPDB_Byte*)(Act+1);
-                    if ( IsEqual(Data, A) )
-                    {
-                        mMerger.MergeData(Data, A, mRecordLength, true);
-                        mValid = false;
-                        return true;
-                    }
-                }while ( Act = Act->mNext );
-            }
-
-            if ( TEntry* NewEntry = GetMemForEntry(ErrorList) )
-            {
-                NewEntry->mNext = mHashTable[Slot];
-                mHashTable[Slot]= NewEntry;
-
-                SAPDB_Byte* N = (SAPDB_Byte*)(NewEntry+1);
-                mMerger.MergeData(Data, N, mRecordLength, false);
-                mValid = false;
-                return true;
-            }
-            else
-            {
-                return false;
-            }
-        }
-        else
-        {
-//            ErrorList = ILLEGAL_OPERATION
-            return false;
-        }
-    }
-
-    SAPDB_Bool First( SAPDBErr_MessageList& ErrorList )
-    {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::First", Table_Trace, 5);
-
-        if ( !SortEntries(false, ErrorList) )
-        {
-            ErrorList = ErrorList + Table_Exception(__CONTEXT__, TABLE_TEMPHASH_FIRSTFAILED);
-            return false;
-        }
-        mActSortTableEntry = 0;
-
-        SAPDB_UInt4 FileCnt = mTempFileContainer.GetSize();
-
-        for ( SAPDB_UInt4 i=0; i<FileCnt; i++ )
-        {
-            Data_TempFile::RecordIterator*& Iter = mTempFileContainer[i].mIter;
-            if ( !Iter )
-            {
-                if ( !(Iter = new(mSessionAllocator) Data_TempFile::RecordIterator(mTempFileContainer[i].mFile->PageAccessManager())) )
-                {
-                    ErrorList = Table_Exception(__CONTEXT__, TABLE_TEMPHASH_NOITERATOR);
-                    return 0;
-                }
-            }
-            mTempFileContainer[i].mFile->GetFirst(*Iter);
-            if ( Iter->IsValid() )
-                mTempFileContainer[i].mData = *(*(Iter));
-            else
-                mTempFileContainer[i].mData = 0;
-        }
-
-        if ( mFunnelMergeIterator = new(mSessionAllocator) SAPDBAlgo_FunnelMergeIteratorIdx<SAPDB_Byte*>(mSessionAllocator, *this, *this, ErrorList) )
-        {
-            if ( mFunnelMergeIterator->CompletelyConstructed() )
-            {
-                mValid = true;
-                Collect();
-                return true;
-            }
-            else
-            {
-                ErrorList = ErrorList + Table_Exception(__CONTEXT__, TABLE_TEMPHASH_FIRSTFAILED);
-                return false;
-            }
-        }
-        else
-        {
-            ErrorList = Table_Exception(__CONTEXT__, TABLE_TEMPHASH_NOITERATOR);
-            ErrorList = ErrorList + Table_Exception(__CONTEXT__, TABLE_TEMPHASH_FIRSTFAILED);
-            return false;
-        }
-    }
-
-    SAPDB_Bool Next( void )
-    {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::Next", Table_Trace, 5);
-        SAPDBERR_ASSERT_STATE( mValid );
-
-        Collect();
-        return mValid;
-    }
-
-    SAPDB_Byte* Get( void )
-    {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::Get", Table_Trace, 5);
-
-        return mFetchedRecord;
-    }
-
-    SAPDB_Bool IsValid( void )
-    {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::IsValid", Table_Trace, 5);
-
-        return mValid;
-    }
-
-    SAPDB_Bool ForceFlushAndFreeMem( SAPDBErr_MessageList& ErrorList )
-    {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::ForceFlushAndFreeMem", Table_Trace, 5);
-
-        if ( FlushToPage(ErrorList) )
-        {
-            DestroyHashTableAndAllocator();
-            return true;
-        }
-        return false;
-    }
-
-    // ----------------------------------------------------------------------------------------------
 
 private:
 
     void DestroyHashTableAndAllocator( void )
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::DestroyHashTableAndAllocator", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::DestroyHashTableAndAllocator", Table_Trace, 8);
 
         if ( mAllocator )
         {
@@ -488,7 +630,7 @@ private:
 
     TEntry* GetMemForEntry( SAPDBErr_MessageList& ErrorList )
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::GetMemForEntry", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::GetMemForEntry", Table_Trace, 8);
 
         TEntry* NewEntry = 0;
 
@@ -519,7 +661,7 @@ private:
 
     TEntry* GetMemOfFirstFree( void )
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::GetMemOfFirstFree", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::GetMemOfFirstFree", Table_Trace, 8);
         SAPDBERR_ASSERT_STATE( mFirstFree );
 
         TEntry* NewEntry = 0;
@@ -531,7 +673,7 @@ private:
 
     void InsertToFreeList( TEntry* Entry )
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::FreeMemForEntry", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::FreeMemForEntry", Table_Trace, 8);
         SAPDBERR_ASSERT_ARGUMENT( Entry );
 
         Entry->mNext = mFirstFree;
@@ -540,7 +682,7 @@ private:
 
     inline TempHashTable_HashValue CalcHashValueForKeys( SAPDB_Byte* Data )
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::CalcHashValueForKeys", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::CalcHashValueForKeys", Table_Trace, 8);
         SAPDBERR_ASSERT_ARGUMENT( Data );
 
         TempHashTable_HashValue Hash = FNV_32_INIT;
@@ -555,13 +697,13 @@ private:
 
     inline TempHashTable_HashValue CalcSlot( const TempHashTable_HashValue Hash )
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::CalcSlot", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::CalcSlot", Table_Trace, 8);
         return Hash%mNumberOfSlots;
     }
 
     inline SAPDB_Bool SortEntries( bool DetachFromHash, SAPDBErr_MessageList& ErrorList )
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::Sortentries", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::Sortentries", Table_Trace, 8);
         SAPDBERR_ASSERT_STATE( mAllocator && mHashTable );
 
         // Create sort table
@@ -609,7 +751,7 @@ private:
 
     inline SAPDB_Bool FlushToPage( SAPDBErr_MessageList& ErrorList )
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::FlushToPage", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::FlushToPage", Table_Trace, 8);
         SAPDBERR_ASSERT_STATE( mAllocator && mHashTable );
 
         // Create new TempFile
@@ -653,7 +795,7 @@ private:
 
     void Collect( void )
     {
-        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTable::Collect", Table_Trace, 8);
+        SAPDBTRACE_METHOD_DEBUG ("Table_FixSizeTempHashTableAggregate::Collect", Table_Trace, 8);
 
         // Merge all Data that have the same key as the current entry 
         // and set the iterator to the first entry having a different key
@@ -695,8 +837,7 @@ private:
     SAPDBMem_RawAllocator*                          mAllocator;
     
     SAPDBAlgo_FunnelMergeIteratorIdx<SAPDB_Byte*>*  mFunnelMergeIterator;
-//    TMerger&                                        mMerger;
-    Table_TempHashTable_IDataMerger&                mMerger;
+    Table_TempHashTableAggregate_IDataMerger&       mMerger;
 
     TEntry**                                        mHashTable;
     TEntry*                                         mFirstFree;
@@ -718,6 +859,8 @@ private:
     SAPDB_Bool                                      mValid;
     SAPDB_Byte*                                     mFetchedRecord;
     SAPDB_UInt2                                     mRecordLength;
+    bool                                            mFirstInsert;
 };
 
 // #############################################################################
+#endif
