@@ -54,6 +54,7 @@
 #include <string.h>
 #include <memory.h>
 #include <assert.h>
+#include "SAPDBCommon/SAPDB_string.h"
 #include "ToolsCommon/Tools_UTF8StringElement.hpp"
 
 
@@ -481,7 +482,7 @@ inline void Tools_DynamicUTF8StringBuffer::Append
      SizeType                       len) 
 {
     ProvideCapacity(m_ElemCount + len);
-    memcpy(End(), src, len * sizeof(BasisType));
+    SAPDB_memmove(End(), src, len * sizeof(BasisType));
     m_ElemCount += len;
 }
 
@@ -518,8 +519,8 @@ inline void Tools_DynamicUTF8StringBuffer::Insert
 
     ProvideCapacity(m_ElemCount + len);
 
-    memmove(PtrTo(pos + len), PtrTo(pos), (m_ElemCount - pos) * sizeof(BasisType));
-    memcpy (PtrTo(pos), src, len * sizeof(BasisType));
+    SAPDB_memmove(PtrTo(pos + len), PtrTo(pos), (m_ElemCount - pos) * sizeof(BasisType));
+    SAPDB_memmove(PtrTo(pos), src, len * sizeof(BasisType));
     m_ElemCount += len;
 }
 
@@ -533,7 +534,7 @@ inline void Tools_DynamicUTF8StringBuffer::Insert
     assert(pos <= m_ElemCount);
 
     ProvideCapacity(m_ElemCount + count);
-    memmove(PtrTo(pos + count), PtrTo(pos), (m_ElemCount - pos) * sizeof(BasisType));
+    SAPDB_memmove(PtrTo(pos + count), PtrTo(pos), (m_ElemCount - pos) * sizeof(BasisType));
     MemSet(PtrTo(pos), count, val);
     m_ElemCount += count;
 }
@@ -550,7 +551,7 @@ inline void Tools_DynamicUTF8StringBuffer::Erase
         m_ElemCount = pos;
     else
     {
-        memmove(PtrTo(pos), PtrTo(pos + len), 
+        SAPDB_memmove(PtrTo(pos), PtrTo(pos + len), 
                 (m_ElemCount - pos - len) * sizeof(BasisType));
         m_ElemCount -= len;
     }
@@ -566,7 +567,7 @@ inline void Tools_DynamicUTF8StringBuffer::Copy
     assert(dest != 0);
     assert(pos <= m_ElemCount);
 
-    memcpy(dest, PtrTo(pos),
+    SAPDB_memmove(dest, PtrTo(pos),
            pos + len > m_ElemCount ? (m_ElemCount - pos - len) * sizeof(BasisType) 
                                    : len * sizeof(BasisType));
 }
@@ -589,16 +590,16 @@ inline void Tools_DynamicUTF8StringBuffer::Replace
 
     if (inslen <= len)
     {
-        memcpy(PtrTo(pos), src, inslen * sizeof(BasisType));
-        memmove(PtrTo(pos + inslen), PtrTo(pos + len), 
+        SAPDB_memmove(PtrTo(pos), src, inslen * sizeof(BasisType));
+        SAPDB_memmove(PtrTo(pos + inslen), PtrTo(pos + len), 
                 (m_ElemCount - pos - len) * sizeof(BasisType));
         m_ElemCount -= len - inslen;
     }
     else
     {
-        memmove(PtrTo(pos + inslen), PtrTo(pos + len), 
+        SAPDB_memmove(PtrTo(pos + inslen), PtrTo(pos + len), 
                 (m_ElemCount - pos - len) * sizeof(BasisType));
-        memcpy(PtrTo(pos), src, inslen * sizeof(BasisType));
+        SAPDB_memmove(PtrTo(pos), src, inslen * sizeof(BasisType));
         m_ElemCount += inslen - len;
     }
 }
@@ -621,13 +622,13 @@ inline void Tools_DynamicUTF8StringBuffer::Replace
     if (count <= len)
     {
         MemSet(PtrTo(pos), count, val);
-        memmove(PtrTo(pos + count), PtrTo(pos + len), 
+        SAPDB_memmove(PtrTo(pos + count), PtrTo(pos + len), 
                 (m_ElemCount - pos - len) * sizeof(BasisType));
         m_ElemCount -= len - count;
     }
     else
     {
-        memmove(PtrTo(pos + count), PtrTo(pos + len), 
+        SAPDB_memmove(PtrTo(pos + count), PtrTo(pos + len), 
                 (m_ElemCount - pos - len) * sizeof(BasisType));
         MemSet(PtrTo(pos), count, val);
         m_ElemCount += count - len;
@@ -670,7 +671,7 @@ inline SAPDB_Bool Tools_DynamicUTF8StringBuffer::ProvideCapacity
     if (IsAssigned())
     {
         Pointer save = m_Data;                                                  // save old buffer contents
-        memcpy(p, save, used * sizeof(BasisType));
+        SAPDB_memcpy(p, save, used * sizeof(BasisType));
         m_Data       = reinterpret_cast<Pointer>(p);
         m_Capacity   = allocsize;
         delete save;
