@@ -49,6 +49,7 @@
   implementation class Tools_ExpressionValue
   -----------------------------------------------------------------------------
  */
+const Tools_ExpressionValue::ExInt Tools_ExpressionValue::UndefPrecision = -1;
 
 /*
   -----------------------------------------------------------------------------
@@ -306,7 +307,8 @@ Tools_ExpressionValue::ExString Tools_ExpressionValue::GetAgeString
   -----------------------------------------------------------------------------
  */
 void Tools_ExpressionValue::Init
-      ( ExType oType )
+      ( ExType oType,
+        ExInt  nPrecision )
 {
   m_oType                   = oType;
   m_oNumericValue           = 0;
@@ -314,7 +316,7 @@ void Tools_ExpressionValue::Init
   m_oLastNumericValue       = 0;
   m_oLastStringValue        = "";
   m_bUndef                  = (m_oType == ExTypeUnknown);
-  m_nPrecision              = -1;
+  m_nPrecision              = nPrecision;
   time(&m_tLastChanged);
 } // end Tools_ExpressionValue::Init
 
@@ -330,13 +332,13 @@ Tools_ExpressionValue::ExString Tools_ExpressionValue::ConvertToString
 {
   char szTmp[100];
 
-  if (nPrecision > -1) {
+  if (nPrecision > Tools_ExpressionValue::UndefPrecision) {
     sprintf(szTmp, "%.*f", (int) nPrecision, nValue);
   } else {
     sprintf(szTmp, "%.*f", (int) 2, nValue);
   } // end if
   ExString szTrimmer(szTmp);
-  if (nPrecision == -1) {
+  if (nPrecision == Tools_ExpressionValue::UndefPrecision) {
     szTrimmer.TrimTrailing("0");
     szTrimmer.TrimTrailing(".");
   } // end if
