@@ -141,7 +141,7 @@ public:
     description:  set a value to undefined
     -------------------------------------------------------------------------
    */
-   void SetUndefined( ) { Init(m_oType); m_bUndef = true; }
+   void SetUndefined( ) { Init(m_oType, m_nPrecision); m_bUndef = true; }
 
   /*!
     -------------------------------------------------------------------------
@@ -207,7 +207,7 @@ private:
 
    void SetNewValue (const Tools_ExpressionValue * pNewValue);
 
-   void Init(ExType oType = ExTypeUnknown);
+   void Init(ExType oType = ExTypeUnknown, ExInt m_nPrecision = -1);
 
    ExString ConvertToString(ExDouble nValue, ExInt nPrecision);
 
@@ -222,6 +222,7 @@ private:
    ExString   m_oLastStringValue;
 
    ExInt      m_nPrecision;
+   static const ExInt UndefPrecision;
 
    time_t     m_tLastChanged;
 
