@@ -32,7 +32,7 @@
   -----------------------------------------------------------------------------
 */
 #include <stdio.h>   // sprintf
-#include <string.h>  // strcpy
+#include <string.h>  // SAPDB_strcpy
 #include <fcntl.h>   // O_TEXT
 #include <stdlib.h>  // atoi
 #include <errno.h>   // errno
@@ -46,9 +46,11 @@
 #endif
 
 #include "heo02.h"
+#include "heo13.h"
 #include "heo06.h"
 #include "hsp02.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
 #include "ToolsCommon/Tools_PipeCall.hpp"
 
 /* 
@@ -65,6 +67,8 @@
 #define OPT_SERVER   "-n"
 #define OPT_BREAK    "-c"
 
+#define OLD_SAPDB_END_OF_PARAMETERS "EOPIPE"
+
 #define OPT_CHAR     '-'
 
 #define TXT_MSG_ENTERUSER   "Enter user name: "
@@ -103,18 +107,29 @@
 
                                         return 0;
                                     }
-  #define SPAWN_PROCESS(prog,pipe)  spawnlp( P_NOWAIT, prog, prog, SAPDB_OPT_PIPE, pipe, NULL )
+  #define SPAWN_PROCESS(prog, arg0, pipe, bWait)  spawnlp( P_NOWAIT, prog, arg0, SAPDB_OPT_PIPE, pipe, NULL )
   #define WAIT_PROCESS(exit,proc)   cwait( exit, proc, WAIT_CHILD );
   #define WEXITSTATUS(a)            a
 #else
   #define HPROCESS                  int
   #define OPEN_PIPE(a)              pipe(a)
-  #define SPAWN_PROCESS(prog,pipe)  mySpawn(prog,pipe)
-                                    int mySpawn (const char * prog, const char * pipe) {
+  #define SPAWN_PROCESS(prog, arg0, pipe, bWait)  mySpawn(prog,pipe, bWait)
+                                    int mySpawn (const char * prog, const char * pipe, bool bWait) {
                                       int hProcess = fork();
                                       if(hProcess==0) {
-                                        if (execlp(prog, prog, SAPDB_OPT_PIPE, pipe, NULL )  == -1 ) {
-                                          exit( 1 );
+                                        if (!bWait) {
+                                          int hProcess2 = fork();
+                                          if(hProcess2==0) {
+                                            if (execlp(prog, prog, SAPDB_OPT_PIPE, pipe, NULL )  == -1 ) {
+                                              exit( 1 );
+                                            } // end if
+                                          } else {
+                                            exit(0);
+                                          } // end if
+                                        } else {
+                                          if (execlp(prog, prog, SAPDB_OPT_PIPE, pipe, NULL )  == -1 ) {
+                                            exit( 1 );
+                                          } // end if
                                         } // end if
                                       } // end if
                                       return hProcess;
@@ -289,13 +304,25 @@ int Tools_PipeCall::ExecuteProgram
 
   HPROCESS  hProzess;
   // start process
-  if( ( hProzess = SPAWN_PROCESS( argv[0], szPipe) ) == -1 ) {
+  char * arg0 = NULL;
+#if defined(WIN32)
+  arg0 = new char[strlen(argv[0]) + 3];
+  if (arg0 == NULL) {
+    SaveError("Memory allcocation error", __FILE__, __LINE__, errno);
+    return -1;
+  } // end if
+  sprintf(arg0, "\"%s\"", argv[0]);
+#endif 
+
+  if( ( hProzess = SPAWN_PROCESS(  argv[0], arg0, szPipe, (oCallType == CallSynchron)) ) == -1 ) {
+
     SaveError("Can not spawn process", __FILE__, __LINE__, errno);
 #if !defined(WIN32)
     close( hPipe[READ] );
     close( hPipe[WRITE] );
 #endif
     RedirectFiles(nStdout, nStderr, hStdout, hStderr);
+	if (arg0 != NULL) delete [] arg0;
     return -1;
   } // end if
 
@@ -318,7 +345,7 @@ int Tools_PipeCall::ExecuteProgram
       while ( argLength >= (SAPDB_PARAMETER_PIPE_SIZE-1) )
       {
           szParameter[0] = SAPDB_PART_OF_LONG_PARAMETER;
-          memcpy(&szParameter[1], argument, SAPDB_PARAMETER_PIPE_SIZE-1);
+          SAPDB_memcpy(&szParameter[1], argument, SAPDB_PARAMETER_PIPE_SIZE-1);
           argLength -= (SAPDB_PARAMETER_PIPE_SIZE-1);
 #if defined(WIN32)
           WriteFile(hPipe, szParameter, SAPDB_PARAMETER_PIPE_SIZE, &bytesWritten, 0);
@@ -329,7 +356,7 @@ int Tools_PipeCall::ExecuteProgram
       }
 
       szParameter[0] = SAPDB_LAST_PART_OF_PARAMETER;
-      memcpy(&szParameter[1], argument, argLength);
+      SAPDB_memcpy(&szParameter[1], argument, argLength);
       memset(&szParameter[argLength+1], 0, SAPDB_PARAMETER_PIPE_SIZE - (argLength+1));
 #if defined(WIN32)
       WriteFile(hPipe, szParameter, SAPDB_PARAMETER_PIPE_SIZE, &bytesWritten, 0);
@@ -355,6 +382,7 @@ int Tools_PipeCall::ExecuteProgram
     WAIT_PROCESS( &nExit, hProzess );
   } // end if
 
+  if (arg0 != NULL) delete [] arg0;
   return WEXITSTATUS(nExit);
 } //  end Tools_PipeCall::ExecuteProgram
 
@@ -408,108 +436,115 @@ int Tools_PipeCall::ReadArguments
         char *completeParameter = 0;
         char *tmpCompleteParameter;
         long completeLength = 0;
-#if defined(WIN32)
-        HANDLE hPipe;
-        hPipe = CreateFile( argvIn[nIndex],
-                            GENERIC_READ,
-                            FILE_SHARE_READ|FILE_SHARE_WRITE,
-                            0,
-                            OPEN_EXISTING,
-                            FILE_ATTRIBUTE_NORMAL,
-                            NULL );
-        if ( INVALID_HANDLE_VALUE != hPipe )
-        {
-            DWORD bytesIn;
-            while ( ReadFile( hPipe, szParameter, SAPDB_PARAMETER_PIPE_SIZE, &bytesIn, 0 ) )
-#else
+
         // Convert passed string handle to integer handle.
         int  hPipe = atoi( argvIn[nIndex] );
-        {
-            while(read(hPipe, szParameter, SAPDB_PARAMETER_PIPE_SIZE) >= 0) 
-#endif
-            {
-                int iParam;
-                for ( iParam = 1; iParam < SAPDB_PARAMETER_PIPE_SIZE; iParam++ )
-                {
-                    if ( szParameter[iParam] == 0 )
-                        break;
-                }
 
-                switch( szParameter[0] )
-                {
-                case SAPDB_PART_OF_LONG_PARAMETER:
-                    if ( iParam != SAPDB_PARAMETER_PIPE_SIZE )
-                    {
-                        SaveError("Invalid long parameter found", __FILE__, __LINE__, iParam);
-                        nRc = -1;
-                        break;
-                    }
-
-                    tmpCompleteParameter = completeParameter;
-                    completeParameter = new char [completeLength+SAPDB_PARAMETER_PIPE_SIZE-1];
-                    if ( !completeParameter )
-                    {
-                        SaveError("Memory allcocation error", __FILE__, __LINE__, errno);
-                        nRc = -1;
-                        break;
-                    }
-                    if ( tmpCompleteParameter )
-                    {
-                        memcpy(completeParameter, tmpCompleteParameter, completeLength);
-                        delete [] tmpCompleteParameter;
-                    }
-                    memcpy(&completeParameter[completeLength], &szParameter[1], SAPDB_PARAMETER_PIPE_SIZE-1);
-                    completeLength += (SAPDB_PARAMETER_PIPE_SIZE - 1);
-                    break;
-
-                case SAPDB_LAST_PART_OF_PARAMETER:
-                    if ( !completeParameter )
-                    {
-                        nRc = AddOption(&szParameter[1], argcPass1, argvPass1);
-                    }
-                    else
-                    {
-                        tmpCompleteParameter = completeParameter;
-                        completeParameter = new char [completeLength+SAPDB_PARAMETER_PIPE_SIZE];
-                        if ( !completeParameter )
-                        {
-                            SaveError("Memory allcocation error", __FILE__, __LINE__, errno);
-                            nRc = -1;
-                            break;
-                        }
-
-                        memcpy(completeParameter, tmpCompleteParameter, completeLength);
-                        delete [] tmpCompleteParameter;
-
-                        if ( iParam > 1 )
-                        {
-                            memcpy(&completeParameter[completeLength], &szParameter[1], iParam-1);
-                        }
-                        completeParameter[completeLength + iParam - 1] = 0;
-
-                        nRc = AddOption(completeParameter, argcPass1, argvPass1);
-                        delete [] completeParameter;
-                        completeParameter = 0;
-                        completeLength = 0;
-                    }
-                    break;
-                case SAPDB_END_OF_PARAMETERS:
-                    break;
-                } // end switch
-                if ( nRc == -1 || SAPDB_END_OF_PARAMETERS == szParameter[0] )
-                {
-                    break;
-                }
-            } // end while
 #if defined(WIN32)
-            CloseHandle(hPipe);
+        HANDLE hNamedPipe = INVALID_HANDLE_VALUE;
+        DWORD  bytesIn;
+        hNamedPipe = CreateFile( argvIn[nIndex],
+                                 GENERIC_READ,
+                                 FILE_SHARE_READ|FILE_SHARE_WRITE,
+                                 0,
+                                 OPEN_EXISTING,
+                                 FILE_ATTRIBUTE_NORMAL,
+                                 NULL );
+
+        while (1) {
+          if ( INVALID_HANDLE_VALUE != hNamedPipe ) {
+            if( !ReadFile( hNamedPipe, szParameter, SAPDB_PARAMETER_PIPE_SIZE, &bytesIn, 0 ) ) {
+              break;
+            } // end if
+          } else {
+            // compatibility mode, maybe it is an unamed pipe
+            if(read(hPipe, szParameter, SAPDB_PARAMETER_PIPE_SIZE) < 0 )  {
+                break;
+            } // end if
+          } // end if
+#else
+        while(read(hPipe, szParameter, SAPDB_PARAMETER_PIPE_SIZE) >= 0) {
 #endif
-        }
+          int iParam;
+          for ( iParam = 1; iParam < SAPDB_PARAMETER_PIPE_SIZE; iParam++ ) {
+            if ( szParameter[iParam] == 0 )
+                break;
+          } // end for
+
+          switch( szParameter[0] )
+          {
+          case SAPDB_PART_OF_LONG_PARAMETER:
+              if ( iParam != SAPDB_PARAMETER_PIPE_SIZE )
+              {
+                  SaveError("Invalid long parameter found", __FILE__, __LINE__, iParam);
+                  nRc = -1;
+                  break;
+              }
+
+              tmpCompleteParameter = completeParameter;
+              completeParameter = new char [completeLength+SAPDB_PARAMETER_PIPE_SIZE-1];
+              if ( !completeParameter )
+              {
+                  SaveError("Memory allcocation error", __FILE__, __LINE__, errno);
+                  nRc = -1;
+                  break;
+              }
+              if ( tmpCompleteParameter )
+              {
+                  SAPDB_memcpy(completeParameter, tmpCompleteParameter, completeLength);
+                  delete [] tmpCompleteParameter;
+              }
+              SAPDB_memcpy(&completeParameter[completeLength], &szParameter[1], SAPDB_PARAMETER_PIPE_SIZE-1);
+              completeLength += (SAPDB_PARAMETER_PIPE_SIZE - 1);
+              break;
+
+          case SAPDB_LAST_PART_OF_PARAMETER:
+              if ( !completeParameter )
+              {
+                  nRc = AddOption(&szParameter[1], argcPass1, argvPass1);
+              }
+              else
+              {
+                  tmpCompleteParameter = completeParameter;
+                  completeParameter = new char [completeLength+SAPDB_PARAMETER_PIPE_SIZE];
+                  if ( !completeParameter )
+                  {
+                      SaveError("Memory allcocation error", __FILE__, __LINE__, errno);
+                      nRc = -1;
+                      break;
+                  }
+
+                  SAPDB_memcpy(completeParameter, tmpCompleteParameter, completeLength);
+                  delete [] tmpCompleteParameter;
+
+                  if ( iParam > 1 )
+                  {
+                      SAPDB_memcpy(&completeParameter[completeLength], &szParameter[1], iParam-1);
+                  }
+                  completeParameter[completeLength + iParam - 1] = 0;
+
+                  nRc = AddOption(completeParameter, argcPass1, argvPass1);
+                  delete [] completeParameter;
+                  completeParameter = 0;
+                  completeLength = 0;
+              }
+              break;
+          case SAPDB_END_OF_PARAMETERS:
+              break;
+          default:
+              if (strcmp(szParameter, OLD_SAPDB_END_OF_PARAMETERS) != 0) {
+                  nRc = AddOption(szParameter, argcPass1, argvPass1);
+              } // end if
+              break;
+
+          } // end switch
+          if ( nRc == -1 || SAPDB_END_OF_PARAMETERS == szParameter[0] || strcmp(szParameter, OLD_SAPDB_END_OF_PARAMETERS) == 0)
+          {
+              break;
+          }
+        } // end while
 #if defined(WIN32)
-        else
-        {
-            SaveError("Open command parameter pipe failed\n", __FILE__, __LINE__, (int)GetLastError() );
-        }
+        if (hNamedPipe != INVALID_HANDLE_VALUE) CloseHandle(hNamedPipe);
 #endif
       } else {
         SaveError("Pipe handle not found", __FILE__, __LINE__, errno);
@@ -629,6 +664,7 @@ int Tools_PipeCall::ReadArguments
 
   // prompt for database name
   if (nRc == 0 && bDBName) {
+
     nRc = AddOption(OPT_DBNAME, argcOut, argvOut);
     if (nRc == 0) {
       pArgument = ReadValues(TXT_MSG_ENTERDBNAME, NULL, 3);
@@ -725,7 +761,7 @@ char * Tools_PipeCall::ReadValues
           (strlen(szData) > 0)   && (strlen(szHidden) > 0)) {
         pData = new char [strlen(szData) + strlen(szHidden) + 2];
         if (pData != NULL) {
-          strcpy(pData, szData);
+          SAPDB_strcpy(pData, szData);
           strcat(pData, ",");
           strcat(pData, szHidden);
         } else {
@@ -735,7 +771,7 @@ char * Tools_PipeCall::ReadValues
       } else if ((szDataPrompt != NULL) && (strlen(szData) > 0)) {
         pData = new char [strlen(szData) + 1];
         if (pData != NULL) {
-          strcpy(pData, szData);
+          SAPDB_strcpy(pData, szData);
         } else {
           SaveError("Memory allcocation error", __FILE__, __LINE__, errno);
         } // end if
@@ -763,7 +799,7 @@ char * Tools_PipeCall::ReadXUserData
  
   memset(recUser.xu_key, ' ', sizeof(recUser.xu_key));
   strncpy(recUser.xu_key, pXUserKey, strlen(pXUserKey));
-  sqlgetuser (&recUser, spError, &bOk);
+  sqlgetuser (&recUser, NULL, spError, &bOk);
 
   if (bOk) {
     tsp00_Name           spClear;
@@ -777,7 +813,7 @@ char * Tools_PipeCall::ReadXUserData
 
     pData = new char [strlen(szUser) + strlen(szPwd) + 2];
     if (pData != NULL) {
-      strcpy(pData, szUser);
+      SAPDB_strcpy(pData, szUser);
       strcat(pData, ",");
       strcat(pData, szPwd);
     } else {
@@ -804,7 +840,7 @@ int Tools_PipeCall::AddOption
   char * pArgument =  new char [strlen(pData) + 1];
 
   if (pArgument != NULL) {
-    strcpy(pArgument, pData);
+    SAPDB_strcpy(pArgument, pData);
     argv[argc++] = pArgument;
   } else {
     SaveError("Memory allcocation error", __FILE__, __LINE__, errno);
@@ -824,7 +860,7 @@ int Tools_PipeCall::AddOption
   char * pArgument =  new char [strlen(pData) + 1];
 
   if (pArgument != NULL) {
-    strcpy(pArgument, pData);
+    SAPDB_strcpy(pArgument, pData);
     argv[argc++] = pArgument;
   } else {
     SaveError("Memory allcocation error", __FILE__, __LINE__, errno);
@@ -935,11 +971,13 @@ void Tools_PipeCall::RedirectFiles
   if (nStdout != 0) {
     fflush( stdout  );
     dup2( nStdout, fileno(stdout));
+    close(nStdout);
   } // end if
 
   if (nStderr != 0) {
     fflush( stderr  );
     dup2( nStderr, fileno(stderr));
+    close(nStderr);
   } // end if
 
   if (hStdout != NULL) {
