@@ -266,7 +266,7 @@ private:
         assert(IsAssigned());
 
         while (count-- > 0)
-            m_Pos += Tools_UTF8Basis::CharacterSize(m_Pos);
+          m_Pos += Tools_UTF8Basis::CharacterSize(m_Pos) == 0 ? 1 : Tools_UTF8Basis::CharacterSize(m_Pos);
 
         return *this;
     }
@@ -525,7 +525,7 @@ private:
         assert(IsAssigned());
 
         while (count-- > 0)
-            m_Pos += Tools_UTF8Basis::CharacterSize(m_Pos);
+            m_Pos += Tools_UTF8Basis::CharacterSize(m_Pos) == 0 ? 1 : Tools_UTF8Basis::CharacterSize(m_Pos);
 
         return *this;
     }
