@@ -395,17 +395,21 @@ void CloseConnection( struct st_tcp_connection *C,
 
 }
 
-static size_t hdllen
-(
- char	*line
-)
-{
- char	*tmp = strstr(line, "\r\n");
+/*========================================================================*/
 
- if (tmp)
-	return(tmp - line);
- else
-	return(0);
+static size_t hdllen( char	*line)
+{
+    char	*tmp = NULL;
+    if (!line) {
+        return 0;
+    }
+
+    tmp = strstr(line, "\r\n");
+
+    if (tmp)
+        return(tmp - line);
+    else
+        return(0);
 }
 
 /*========================================================================*/
@@ -441,96 +445,93 @@ static int ReceiveHttpHeaders( SOCKET    cs,
 
 /*========================================================================*/
 
-static int PrepareHeaders
-(
- struct st_http_req	*HttpReq
-)
+static int PrepareHeaders( struct st_http_req	*HttpReq )
 {
- char			*p = NULL;
- char			*hl = HttpReq->HeadersLine->str;
- size_t			hll = HttpReq->HeadersLine->lg;
- size_t			HeaderNo = 0, pl;
- char			*tmp;
- tsp00_Bool		bOk;
+    char			*p = NULL;
+    char			*hl = NULL;
+    size_t			hll = 0;
+    size_t			HeaderNo = 0, pl;
+    char			*tmp;
+    tsp00_Bool		bOk;
 
- /* because there is an additional entry for HTTP_PORT the HeaderNames- and -ValuesArray 
+    /* because there is an additional entry for HTTP_PORT the HeaderNames- and -ValuesArray 
     has to have one more line */
- char			*Host;
- char			*Port;
- HttpReq->NoOfHeaderLines++;
+    char			*Host;
+    char			*Port;
 
+    if (!HttpReq || !HttpReq->HeadersLine) {
+        return FALSE;
+    }
+    hl = HttpReq->HeadersLine->str;
+    hll = HttpReq->HeadersLine->lg;
 
- wahttp_SQLALLOCAT( HttpReq->NoOfHeaderLines * sizeof(char*),
-		  &HttpReq->HeaderNamesArray,
-		  &bOk);
- if (!bOk)
-	HttpReq->HeaderNamesArray = NULL;
+    HttpReq->NoOfHeaderLines++;
 
- wahttp_SQLALLOCAT( HttpReq->NoOfHeaderLines * sizeof(char*),
-		  &HttpReq->HeaderValuesArray,
-		  &bOk);
- if (!bOk)
-	HttpReq->HeaderValuesArray = NULL;
+    wahttp_SQLALLOCAT( HttpReq->NoOfHeaderLines * sizeof(char*), &HttpReq->HeaderNamesArray, &bOk);
+    if (!bOk) {
+        HttpReq->HeaderNamesArray = NULL;
+        return FALSE;
+    }
 
- if (!HttpReq->HeaderNamesArray || !HttpReq->HeaderValuesArray)
- {
-	if (HttpReq->HeaderNamesArray) wahttp_SQLFREE(HttpReq->HeaderNamesArray);
-	HttpReq->HeaderNamesArray = NULL;
-	if (HttpReq->HeaderValuesArray) wahttp_SQLFREE(HttpReq->HeaderValuesArray);
-	HttpReq->HeaderValuesArray = NULL;
-	return(FALSE);
- }
+    wahttp_SQLALLOCAT( HttpReq->NoOfHeaderLines * sizeof(char*), &HttpReq->HeaderValuesArray, &bOk);
+    if (!bOk) {
+        wahttp_SQLFREE(HttpReq->HeaderNamesArray);
+        HttpReq->HeaderNamesArray = NULL;
+	    HttpReq->HeaderValuesArray = NULL;
 
- for (p=hl;
-      p<hl+hll && HeaderNo < HttpReq->NoOfHeaderLines-1;
-      p+=pl + HTTP_HEADER_SEP_LG, HeaderNo++)
- {
-	pl = strlen(p);
-	HttpReq->HeaderNamesArray[HeaderNo] = p;
-	if (HeaderNo == 0)
-		HttpReq->HeaderValuesArray[HeaderNo] = NULL;
-	else
-	{
-		tmp = strchr(p, ' ');
-		if (!tmp)
-		{
-			if (HttpReq->HeaderNamesArray) wahttp_SQLFREE(HttpReq->HeaderNamesArray);
-			HttpReq->HeaderNamesArray = NULL;
-			if (HttpReq->HeaderValuesArray) wahttp_SQLFREE(HttpReq->HeaderValuesArray);
-			HttpReq->HeaderValuesArray = NULL;
-			return(FALSE);
-		}
-		*tmp = '\0';
-		tmp++;
-		while (*tmp == ' ') tmp++;
-		HttpReq->HeaderValuesArray[HeaderNo] = tmp;
+        return FALSE;
+    }
+
+    for (p=hl; p<hl+hll && HeaderNo < HttpReq->NoOfHeaderLines-1; p+=pl + HTTP_HEADER_SEP_LG, HeaderNo++) {
+	    pl = strlen(p);
+	    HttpReq->HeaderNamesArray[HeaderNo] = p;
+    	if (HeaderNo == 0)
+	    	HttpReq->HeaderValuesArray[HeaderNo] = NULL;
+	    else {
+		    tmp = strchr(p, ' ');
+		    if (!tmp) {
+			    if (HttpReq->HeaderNamesArray) wahttp_SQLFREE(HttpReq->HeaderNamesArray);
+			    HttpReq->HeaderNamesArray = NULL;
+			    if (HttpReq->HeaderValuesArray) wahttp_SQLFREE(HttpReq->HeaderValuesArray);
+			    HttpReq->HeaderValuesArray = NULL;
+			    return(FALSE);
+		    }
+		    *tmp = '\0';
+		    tmp++;
+		    while (*tmp == ' ') tmp++;
+		    HttpReq->HeaderValuesArray[HeaderNo] = tmp;
+
+		    /* remove the trailing colon */
+		    tmp = strchr(p, ':');
+		    if (tmp && *tmp == ':')
+			    *tmp = '\0';
+	    }
+    }
+    /* extract port from host and store it within the last line of the HeaderNames- /
+        -ValuesArray */
+    HttpReq->HeaderNamesArray[HttpReq->NoOfHeaderLines-1] = "Port";
+    HttpReq->HeaderValuesArray[HttpReq->NoOfHeaderLines-1] = "";
+    Host = FindHeader(HttpReq, HTTP_HOST);
+    /* PTS1117671: Host may be not given -> null pointer */
+    if (Host) {
+	    Port = strchr(Host, ':');
+	    if (Port) {
+		    /* PTS1117691 clearly separate Host and Port */
+		    *Port = '\0';
+		    /* PTS1117691 */
+		    (Port)++;
+		    HttpReq->HeaderValuesArray[HttpReq->NoOfHeaderLines-1] = Port;
+	    } else {
+		    HttpReq->HeaderValuesArray[HttpReq->NoOfHeaderLines-1] = "";
+	    }
+    }
+
+    return(TRUE);
 
-		/* remove the trailing colon */
-		tmp = strchr(p, ':');
-		if (*tmp == ':')
-			*tmp = '\0';
-	}
- }
-/* extract port from host and store it within the last line of the HeaderNames- /
-   -ValuesArray */
- HttpReq->HeaderNamesArray[HttpReq->NoOfHeaderLines-1] = "Port";
- Host = FindHeader(HttpReq, HTTP_HOST);
- /* PTS1117671: Host may be not given -> null pointer */
- if (Host) {
-	 Port = strchr(Host, ':');
-	 if (Port) {
-		/* PTS1117691 clearly separate Host and Port */
-		 *Port = '\0';
-		/* PTS1117691 */
-		(Port)++;
-		HttpReq->HeaderValuesArray[HttpReq->NoOfHeaderLines-1] = Port;
-	 } else {
-		HttpReq->HeaderValuesArray[HttpReq->NoOfHeaderLines-1] = "";
-	 }
- }
- return(TRUE);
 }
 
+/*========================================================================*/
+
 struct st_our_socket
 {
  struct st_tcp_connection	*tcp_con;
@@ -549,7 +550,7 @@ static size_t our_recv( struct st_data  *this,
 
     if (os->req->contentChunked) {
         /* Additional way (HTTP 1.1) to read body data with chunks */
-        char    ch[1];
+        char    ch[3];
         char    chunkSizeBuf[64] = "";
         int     i;
 
@@ -599,7 +600,7 @@ static size_t our_recv( struct st_data  *this,
 
         /* If read length == chunk size read end of chunk */
         if (os->req->chunkSize == rlg) {
-            /* Read end of chunk */
+            /* Read end of chunk CRLF */
             if (recv(os->tcp_con->s, chunkSizeBuf, 2, 0) == SOCKET_ERROR) {
                 Log_SyscallError("recv from server failed", rlg);
                 return(0);
@@ -762,16 +763,16 @@ static int ProcessRequest( struct st_http_client    *client,
         (HttpReq->Method == rq_checkout) ||
         (HttpReq->Method == rq_uncheckout) ||
         (HttpReq->Method == rq_report))
-    HttpReq->ContentLengthValid = GetContentLength(HttpReq, &HttpReq->ContentLength);
+    HttpReq->ContentLengthValid = GetContentLength(HttpReq, &HttpReq->ContentLength, &HttpReq->contentChunked);
     /* end insert DanielK */
 
     HttpReq->chunkSize = 0;
-    if (HttpReq->ContentLengthValid && HttpReq->ContentLength == 0) {
+/*    if (HttpReq->ContentLengthValid && HttpReq->ContentLength == 0) {
         HttpReq->contentChunked = TRUE;
     } else {
         HttpReq->contentChunked = FALSE;
     }
-
+*/
     if (HttpReq->Type == hmt_simple_request) {
         HttpReq->HMajorVersion = 0;
         HttpReq->HMinorVersion = 0;
