@@ -151,6 +151,10 @@ WDV_Bool WDVCAPI_AdminTotalSumContentLength( WDVCAPI_WDV        wdv,
 
 	SQLRETURN   sqlRC = 0;
 
+    if(!wdv || !wdv->hAdmin) {
+        return WDV_False;
+    }
+
     /* Statement already parsed? */
     if (!wdv->hAdmin->hStmtSumContentLength) {
 	    /* Allocat statement handle */
@@ -224,6 +228,10 @@ WDV_Bool WDVCAPI_AdminTotalSumCompressedContentLength( WDVCAPI_WDV        wdv,
 
 	SQLRETURN   sqlRC = 0;
 
+    if(!wdv || !wdv->hAdmin) {
+        return WDV_False;
+    }
+
     /* Statement already parsed? */
     if (!wdv->hAdmin->hStmtSumCompressedContentLength) {
 	    /* Allocat statement handle */
@@ -297,6 +305,10 @@ WDV_Bool WDVCAPI_AdminTotalSumPhysicalLength( WDVCAPI_WDV        wdv,
 
 	SQLRETURN   sqlRC = 0;
 
+    if(!wdv || !wdv->hAdmin) {
+        return WDV_False;
+    }
+
     /* Statement already parsed? */
     if (!wdv->hAdmin->hStmtSumPhysicalLength) {
 	    /* Allocat statement handle */
@@ -372,6 +384,10 @@ WDV_Bool WDVCAPI_AdminTotalCompressedRatio( WDVCAPI_WDV        wdv,
 
 	SQLRETURN   sqlRC = 0;
 
+    if(!wdv || !wdv->hAdmin) {
+        return WDV_False;
+    }
+
     /* Statement already parsed? */
     if (!wdv->hAdmin->hStmtSumRatio) {
 	    /* Allocat statement handle */
@@ -478,6 +494,10 @@ WDV_Bool WDVCAPI_AdminTotalSumOfResources( WDVCAPI_WDV        wdv,
 
 	SQLRETURN   sqlRC = 0;
 
+    if(!wdv || !wdv->hAdmin) {
+        return WDV_False;
+    }
+
     /* Statement already parsed? */
     if (!wdv->hAdmin->hStmtSumOfResources) {
 	    /* Allocat statement handle */
@@ -551,6 +571,10 @@ WDV_Bool WDVCAPI_AdminTotalSumOfCollections( WDVCAPI_WDV        wdv,
 
 	SQLRETURN   sqlRC = 0;
 
+    if(!wdv || !wdv->hAdmin) {
+        return WDV_False;
+    }
+
     /* Statement already parsed? */
     if (!wdv->hAdmin->hStmtSumOfCollections) {
 	    /* Allocat statement handle */
