@@ -54,6 +54,7 @@
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_ErrorInternal.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Resource.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Container.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /***********************************************************************
 
@@ -294,8 +295,8 @@ WDV_Bool Container_Copy( WDVCAPI_WDV              wdv,
     }
 
     /* Set input parameter */
-    memcpy(hContainer->copyDestinationId, destinationId, WDVCAPI_MAX_ID_LEN);
-    memcpy(hContainer->resourceId, sourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hContainer->copyDestinationId, destinationId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hContainer->resourceId, sourceId, WDVCAPI_MAX_ID_LEN);
 
     /* Execute statement */
     sqlRC = SQLExecute(hContainer->hStmtCopyContainer);
@@ -610,7 +611,7 @@ WDV_Bool Container_Delete( WDVCAPI_WDV              wdv,
     }
 
     /* Set input parameter */
-    memcpy(hContainer->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hContainer->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
 
     /* Execute delete statement */
     sqlRC = SQLExecute(hContainer->hStmtDeleteContainer);
@@ -680,7 +681,7 @@ WDV_Bool Container_OpenToWrite( WDVCAPI_WDV               wdv,
     }
 
     /* Set input parameters */
-    memcpy(hContainer->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hContainer->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
     
     sqlRC = SQLExecute(hContainer->hStmtWriteContainer);
 	if ( sqlRC != SQL_NEED_DATA ) {
@@ -829,7 +830,7 @@ WDV_Bool Container_OpenToRead( WDVCAPI_WDV              wdv,
     }
 
     /* Init input parameter */
-    memcpy(hContainer->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hContainer->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
     
     sqlRC = SQLExecute(hContainer->hStmtReadContainer);
 	if ( sqlRC != 0 ) {
