@@ -108,6 +108,10 @@ WDV_Bool WDVCAPI_Copy( WDVCAPI_WDV			        wdv,
 
     TRACE_OUT(">> WDVCAPI_Copy()\n");
 
+    if ( !wdv ) {
+		return WDV_False;
+	}
+
     /* Is uri the deleted items folder? */
     if(strncmp(destinationUri, WDVCAPI_DELETED_ITEMS_FOLDER, WDVCAPI_DELETED_ITEMS_FOLDER_LEN) == 0) {
         Error_AddErrorItem(wdv, WDVCAPI_ERR_TYPE_CAPI,
@@ -470,6 +474,10 @@ WDV_Bool Copy_CopyResource( WDVCAPI_WDV                  wdv,
 
     TRACE_OUT(">> Copy_CopyResource()\n");
 
+    if (!destinationCreated || !copyOk) {
+        return WDV_False;
+    }
+
     /* Create new destination resource handle */
     if (!Resource_CreateHandle(wdv, &newDestinationResource)) {
         return WDV_False;
@@ -478,7 +486,7 @@ WDV_Bool Copy_CopyResource( WDVCAPI_WDV                  wdv,
     /* Create new inode, if necessary */
     if (!Resource_CreateInode(wdv, destinationParentResource, destinationName, WDVCAPI_RESOURCE_STATE_DEFAULT,
                               resource->compressedLength, overwrite, newDestinationResource,
-                             &resourceAlreadyExists)) {
+                             &resourceAlreadyExists, WDV_True)) {
         Resource_DestroyHandle(wdv, newDestinationResource);
 
         if (WDVCAPI_IsError(wdv, WDVCAPI_ERR_TYPE_CAPI, WDVCAPI_ERR_CODE_RESOURCE_OR_COLLECTION_ALREADY_EXIST)) {
