@@ -109,7 +109,7 @@ declaration: sapdbwa_{ULong | Long}
 description: basic long data types for fastest execution
              (machine-dependent size)
 --------------------------------------------------------------------------*/
-#if defined(WIN32) && defined(_M_IA64)
+#if defined(WIN32) && defined(BIT64)
 typedef unsigned __int64    WDV_ULong;      /* long (fast, unsigned) */
 typedef signed __int64      WDV_Long;       /* long (fast, signed)   */
 #else
