@@ -54,6 +54,7 @@
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_WDV.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Delete.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Id.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /***********************************************************************
 
@@ -186,6 +187,10 @@ WDV_Bool WDVCAPI_Delete( WDVCAPI_WDV            wdv,
 
     WDV_Bool    uriInDeletedFolder = WDV_False;
 
+    if (!wdv || !uri) {
+        return WDV_False;
+    }
+
     /* Is uri the deleted items folder? */
     if(strcmp(uri, WDVCAPI_DELETED_ITEMS_FOLDER) == 0 ||
        strcmp(uri, WDVCAPI_DELETED_ITEMS_FOLDER"/") == 0) {
@@ -562,7 +567,7 @@ WDV_Bool Delete_Inode( WDVCAPI_WDV      wdv,
         return WDV_False;
     }
 
-    memcpy(hDelete->inodeToDelete, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hDelete->inodeToDelete, resourceId, WDVCAPI_MAX_ID_LEN);
     hDelete->inodeToDeleteIndicator = WDVCAPI_MAX_ID_LEN;
 
     /* Select inodes to delete */
@@ -589,7 +594,7 @@ WDV_Bool Delete_Inode( WDVCAPI_WDV      wdv,
     /* Delete inode itself */
     if (!Resource_IdIsRoot(hDelete->inodeToDelete) && !Resource_IdIsDeletedItems(resource->id)) {
         /* Set inode to delete */
-        memcpy(hDelete->subInodeToDelete, hDelete->inodeToDelete, WDVCAPI_MAX_ID_LEN);
+        SAPDB_memcpy(hDelete->subInodeToDelete, hDelete->inodeToDelete, WDVCAPI_MAX_ID_LEN);
 
         /* Delete inode */
         if (!Delete_DeleteInode(wdv, hDelete)) {
@@ -1172,7 +1177,7 @@ WDV_Bool Delete_SetParentDeletedItems( WDVCAPI_WDV      wdv,
     }
 
     /* Set input parameters */
-    memcpy(wdv->hDelete->inodeToDelete, resource->id, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(wdv->hDelete->inodeToDelete, resource->id, WDVCAPI_MAX_ID_LEN);
 
     sqlRC = SQLExecute(wdv->hDelete->hStmtInsertIntoDeletedItems);
 	if ( sqlRC != 0 ) {
