@@ -56,6 +56,7 @@
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Get.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Id.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_ZLibIO.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /***********************************************************************
 
@@ -587,8 +588,15 @@ WDV_Bool WDVCAPI_GetContentLength( WDVCAPI_WDV            wdv,
 
     WDVCAPI_GetHandle   hGet = NULL;
 
+    if (!wdv || !contentLength) {
+        return WDV_False;
+    }
+
     /* Get get handle */
     GetGetHandle(wdv, &hGet);
+    if (!hGet) {
+        return WDV_False;
+    }
 
     *contentLength = hGet->contentLength;
 
@@ -603,8 +611,15 @@ WDV_Bool WDVCAPI_GetContentType( WDVCAPI_WDV          wdv,
 
     WDVCAPI_GetHandle   hGet = NULL;
 
+    if (!wdv) {
+        return WDV_False;
+    }
+
     /* Get get handle */
     GetGetHandle(wdv, &hGet);
+    if (!hGet) {
+        return WDV_False;
+    }
 
     Common_StrMaxCopy(contentType, hGet->contentTypeString, WDVCAPI_MAX_CONTENT_TYPE_LEN);
 
@@ -620,8 +635,15 @@ WDV_Bool WDVCAPI_GetLastModified( WDVCAPI_WDV                   wdv,
 
     WDVCAPI_GetHandle   hGet = NULL;
 
+    if (!wdv) {
+        return WDV_False;
+    }
+
     /* Get get handle */
     GetGetHandle(wdv, &hGet);
+    if (!hGet) {
+        return WDV_False;
+    }
 
     Common_StrMaxCopy(lastModified, hGet->lastModified, WDV_MAX_PROPERTY_SHORT_VALUE_LEN);
 
@@ -1046,9 +1068,9 @@ WDV_Bool Get_GetProperties( WDVCAPI_WDV          wdv,
         if (WDVCAPI_IdsAreIdentical(hGet->propertyId, ID_PROPERTY_GET_CONTENT_LENGTH)) {
             hGet->contentLength = atol(hGet->propertyShortValue);
         } else if (WDVCAPI_IdsAreIdentical(hGet->propertyId, ID_PROPERTY_GET_CONTENT_TYPE)) {
-            memcpy(hGet->contentTypeString, hGet->propertyShortValue, WDV_MAX_PROPERTY_SHORT_VALUE_LEN);
+            SAPDB_memcpy(hGet->contentTypeString, hGet->propertyShortValue, WDV_MAX_PROPERTY_SHORT_VALUE_LEN);
         } else if (WDVCAPI_IdsAreIdentical(hGet->propertyId, ID_PROPERTY_GET_LAST_MODIFIED)) {
-            memcpy(hGet->lastModified, hGet->propertyShortValue, WDV_MAX_PROPERTY_SHORT_VALUE_LEN);
+            SAPDB_memcpy(hGet->lastModified, hGet->propertyShortValue, WDV_MAX_PROPERTY_SHORT_VALUE_LEN);
         }
     }
 
