@@ -56,6 +56,7 @@
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Resource.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Id.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Property.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /***********************************************************************
 
@@ -240,7 +241,7 @@ WDV_Bool WDVCAPI_LockDestroyHandle( WDVCAPI_WDV         wdv,
 
     WDVCAPI_LockHandle *lockFreeList = NULL;
 
-	if (!hLock) {
+	if (!wdv || !hLock) {
 		return WDV_False;
 	}
 
@@ -679,7 +680,7 @@ WDV_Bool Lock_Copy( WDVCAPI_WDV			wdv,
                     WDVCAPI_LockHandle  hSourceLock )
 {
 
-    memcpy(hDestinationLock, hSourceLock, sizeof(struct st_wdvcapi_lock_description));
+    SAPDB_memcpy(hDestinationLock, hSourceLock, sizeof(struct st_wdvcapi_lock_description));
 
     return WDV_True;
 
