@@ -60,6 +60,7 @@
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_NameSpace.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Id.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_XMLIndexing.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /***********************************************************************
 
@@ -685,7 +686,7 @@ WDV_Bool Property_SetLastModified( WDVCAPI_WDV   wdv,
     /* Set members                                          */
     /* To copy these 24 bytes every time should be better   */
     /* than to bind the resource id every time (performance)*/
-    memcpy(hPropertySetLastModified->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hPropertySetLastModified->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
 
     /* Is statement already prepared? */
     if (!hPropertySetLastModified->hStmt) {
@@ -797,7 +798,7 @@ WDV_Bool Property_SetCreationDate( WDVCAPI_WDV   wdv,
     /* Set members                                          */
     /* To copy these 24 bytes every time should be better   */
     /* than to bind the resource id every time (performance)*/
-    memcpy(hPropertySetCreationDate->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hPropertySetCreationDate->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
 
     /* Is statement already prepared? */
     if (!hPropertySetCreationDate->hStmt) {
@@ -948,7 +949,7 @@ WDV_Bool Property_DeleteAllIndexValues ( WDVCAPI_WDV    wdv,
     }
 
     /* Set input parameter */
-    memcpy(wdv->propertyDeleteAllIndexValuesResourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(wdv->propertyDeleteAllIndexValuesResourceId, resourceId, WDVCAPI_MAX_ID_LEN);
 
     /* Execute delete statement */
     sqlRC = SQLExecute(wdv->hStmtPropertyDeleteAllIndexValues);
@@ -1139,8 +1140,8 @@ WDV_Bool Property_GetShortValue( WDVCAPI_WDV                wdv,
     WDV_GetHandlePropertyGetShortValue(wdv, &hPropertyGetShortValue);
 
 	/* Set input values                         */
-	memcpy(hPropertyGetShortValue->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
-	memcpy(hPropertyGetShortValue->propertyId, propertyId, WDVCAPI_MAX_ID_LEN);
+	SAPDB_memcpy(hPropertyGetShortValue->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+	SAPDB_memcpy(hPropertyGetShortValue->propertyId, propertyId, WDVCAPI_MAX_ID_LEN);
 
     /* Statement already parsed? */
     if (!hPropertyGetShortValue->hStmt) {
@@ -1297,8 +1298,8 @@ WDV_Bool Property_DeleteProperty( WDVCAPI_WDV   wdv,
     }
 
     /* Set members                              */
-    memcpy(wdv->propertyDeleteResourceId, resourceId, WDVCAPI_MAX_ID_LEN);
-    memcpy(wdv->propertyDeletePropertyId, propertyId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(wdv->propertyDeleteResourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(wdv->propertyDeletePropertyId, propertyId, WDVCAPI_MAX_ID_LEN);
 
 	/* Execute SQL */
 	sqlRC = SQLExecute(wdv->propertyDeleteStmt);
@@ -1671,8 +1672,8 @@ WDV_Bool Property_SetShortValue( WDVCAPI_WDV                wdv,
     WDV_GetHandlePropertySetShortValue(wdv, &hPropertySetShortValue);
 
     /* Set members                              */
-    memcpy(hPropertySetShortValue->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
-    memcpy(hPropertySetShortValue->propertyId, propertyId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hPropertySetShortValue->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hPropertySetShortValue->propertyId, propertyId, WDVCAPI_MAX_ID_LEN);
     Common_StrMaxCopy(hPropertySetShortValue->propertyShortValue, propertyShortValue, WDV_MAX_PROPERTY_SHORT_VALUE_LEN);
     hPropertySetShortValue->propertyShortValueIndicator = SQL_NTS;
 
@@ -1767,8 +1768,8 @@ WDV_Bool Property_UpdateShortValue( WDVCAPI_WDV                wdv,
     WDV_GetHandlePropertyUpdateShortValue(wdv, &hPropertyUpdateShortValue);
 
     /* Set members */
-    memcpy(hPropertyUpdateShortValue->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
-    memcpy(hPropertyUpdateShortValue->propertyId, propertyId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hPropertyUpdateShortValue->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hPropertyUpdateShortValue->propertyId, propertyId, WDVCAPI_MAX_ID_LEN);
     Common_StrMaxCopy(hPropertyUpdateShortValue->propertyShortValue, propertyShortValue, WDV_MAX_PROPERTY_SHORT_VALUE_LEN);
 
     /* Is statement already prepared? */
