@@ -59,6 +59,7 @@
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_WDV.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Id.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Property.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /***********************************************************************
 
@@ -977,7 +978,7 @@ WDV_Bool ResourceGetByName( WDVCAPI_WDV                 wdv,
     SQLHDBC           hDBC = 0;
 
     /* Set input values */
-    memcpy(hPut->selectByNameParentId, pid, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(hPut->selectByNameParentId, pid, WDVCAPI_MAX_ID_LEN);
     Common_StrMaxCopy(hPut->selectByNameName, name, WDV_MAX_RESOURCE_NAME_LEN);
 
     /* Statement already prepared */
@@ -1071,7 +1072,7 @@ WDV_Bool ResourceGetByName( WDVCAPI_WDV                 wdv,
 	}
 
     /* Set return value */
-    memcpy(cid, hPut->selectByNameChildId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(cid, hPut->selectByNameChildId, WDVCAPI_MAX_ID_LEN);
 
 	/* Return */
 	return WDV_True;
@@ -1161,13 +1162,13 @@ WDV_Bool WriteContainer( WDVCAPI_WDV            wdv,
     if (hPut->firstBlockDone == WDV_False) {
         /* Set first block */
         if (hPut->firstBlockLength + bufferLength < 8000) {
-            memcpy(hPut->firstBlock + hPut->firstBlockLength, buffer, bufferLength);
+            SAPDB_memcpy(hPut->firstBlock + hPut->firstBlockLength, buffer, bufferLength);
             hPut->firstBlockLength += bufferLength;
 
             return WDV_True;
         } else {
             bufferOffset = 8000 - hPut->firstBlockLength;
-            memcpy(hPut->firstBlock + hPut->firstBlockLength, buffer, bufferOffset);
+            SAPDB_memcpy(hPut->firstBlock + hPut->firstBlockLength, buffer, bufferOffset);
             hPut->firstBlockLength = 8000;
 
             /*(char*)buffer += bufferOffset;*/
