@@ -57,6 +57,7 @@
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Container.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Property.h"
 #include "SAPDB/WebDAV/CAPI/WDVCAPI_Delete.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /***********************************************************************
 
@@ -79,7 +80,7 @@
 
 #define SQL_SELECT_ROOT  \
     "SELECT X'"ID_ROOT_STRING"', 0, Property_Short_Value \
-     FROM WEBDAV_Property WHERE CId = X'"ID_ROOT_STRING"' AND Property_Id = X'"ID_PROPERTY_LOCK_DEPTH_STRING"'"
+     FROM WEBDAV_Property WHERE CId = ? AND Property_Id = X'"ID_PROPERTY_LOCK_DEPTH_STRING"'"
 
 #define SQL_SET_STATE   \
     "UPDATE WEBDAV_Inode SET State = %d WHERE CId = X'%s'"
@@ -629,6 +630,19 @@ WDV_Bool Resource_GetByName( WDVCAPI_WDV         wdv,
 		        return WDV_False;
 	        }
 
+            /* Bind input parameters */
+            sqlRC = SQLBindParameter(resource->hStmtRoot, 1, SQL_PARAM_INPUT, SQL_C_BINARY, SQL_BINARY, 0, 0,
+                                     resource->id, WDVCAPI_MAX_ID_LEN,
+                                    &resource->idIndicator);
+	        if ( sqlRC != 0 ) {
+                Error_AddSQLErrorItem(wdv, resource->hStmtRoot, sqlRC );
+
+		        SQLFreeStmt(resource->hStmtRoot, SQL_DROP);
+		        resource->hStmtRoot = 0;
+
+		        return WDV_False;
+	        }
+
             /* Bind output columns */
             sqlRC = SQLBindCol(resource->hStmtRoot, 1, SQL_C_BINARY,
                                resource->id, WDVCAPI_MAX_ID_LEN,
@@ -671,6 +685,9 @@ WDV_Bool Resource_GetByName( WDVCAPI_WDV         wdv,
         hStmt = resource->hStmtRoot;
 	}
 
+    /* Copy input parameters */
+    SAPDB_memcpy(resource->id, ID_ROOT, WDVCAPI_MAX_ID_LEN);
+
 	/* Execute SQL */
 	sqlRC = SQLExecute(hStmt);
 	if ( sqlRC != 0 ) {
@@ -1057,7 +1074,7 @@ WDV_Bool Resource_GetId( WDVCAPI_Resource  resource,
                          WDVCAPI_Id        id )
 {
 
-    memcpy(id, resource->id, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(id, resource->id, WDVCAPI_MAX_ID_LEN);
 
 	return WDV_True;
 
@@ -1130,7 +1147,8 @@ WDV_Bool Resource_CreateInode( WDVCAPI_WDV            wdv,
                                WDV_UInt4              compressedLength,
                                WDVCAPI_Overwrite      overwrite,
                                WDVCAPI_Resource       newResource,
-                               WDV_Bool              *resourceAlreadyExists )
+                               WDV_Bool              *resourceAlreadyExists,
+                               WDV_Bool               withDisplayname )
 {
 
 	SQLRETURN             sqlRC = 0;
@@ -1298,8 +1316,8 @@ WDV_Bool Resource_CreateInode( WDVCAPI_WDV            wdv,
     }
 
     /* Set current input values */
-    memcpy(wdv->hResourceCreateInode->parentId, parentId, WDVCAPI_MAX_ID_LEN);
-    memcpy(wdv->hResourceCreateInode->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(wdv->hResourceCreateInode->parentId, parentId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(wdv->hResourceCreateInode->resourceId, resourceId, WDVCAPI_MAX_ID_LEN);
     wdv->hResourceCreateInode->resourceState = resourceState;
     wdv->hResourceCreateInode->compressedLength = compressedLength;
     Common_StrMaxCopy(wdv->hResourceCreateInode->resourceName, resourceName, WDV_MAX_RESOURCE_NAME_LEN);
@@ -1317,16 +1335,18 @@ WDV_Bool Resource_CreateInode( WDVCAPI_WDV            wdv,
 	}
 
     /* Insert displayname */
-    if (!Property_SetShortValue(wdv, resourceId, ID_PROPERTY_DISPLAYNAME, resourceName)) {
-		return WDV_False;
-	}
+    if (withDisplayname == WDV_True) {
+        if (!Property_SetShortValue(wdv, resourceId, ID_PROPERTY_DISPLAYNAME, resourceName)) {
+		    return WDV_False;
+	    }
+    }
 
     /* Init new resource */
     if (!Resource_InitHandle(wdv, newResource)) {
         return WDV_False;
     }
 
-    memcpy(newResource->id, resourceId, WDVCAPI_MAX_ID_LEN);
+	SAPDB_memcpy(newResource->id, resourceId, WDVCAPI_MAX_ID_LEN);
 
 	return WDV_True;
 
@@ -1458,7 +1478,7 @@ WDV_Bool Resource_GetUriByChild( WDVCAPI_WDV       wdv,
         if (!Resource_GetParentByCId(wdv, currentCId, parentResource)) {
             /* Is parent root? */
             if (WDVCAPI_IsError(wdv, WDVCAPI_ERR_TYPE_CAPI, WDVCAPI_ERR_CODE_RESOURCE_OR_COLLECTION_DOESNT_EXIST)) {
-                memcpy(currentCId, ID_ROOT, WDVCAPI_MAX_ID_LEN);
+                SAPDB_memcpy(currentCId, ID_ROOT, WDVCAPI_MAX_ID_LEN);
                 strcpy(currentName, WDV_PATH_SEPARATOR_STR);
             } else {
                 Resource_DestroyHandle(wdv, parentResource);
@@ -1839,10 +1859,10 @@ WDV_Bool Resource_CreateNullResource( WDVCAPI_WDV       wdv,
                                       WDVCAPI_Resource  newResource )
 {
 
-    WDVCAPI_URIString       parentUri = "";
-    WDVCAPI_ResourceName    resourceName = "";
-    WDVCAPI_Resource        parentResource = NULL;
-    WDVCAPI_Id              newId;
+    WDVCAPI_URIString           parentUri = "";
+    WDVCAPI_ResourceNameString  resourceName = "";
+    WDVCAPI_Resource            parentResource = NULL;
+    WDVCAPI_Id                  newId;
 
     /* Split uri into parent and name */
 	SplitURI(wdv, uri, parentUri, resourceName);
@@ -1879,7 +1899,7 @@ WDV_Bool Resource_CreateNullResource( WDVCAPI_WDV       wdv,
 
     /* Create inode entry with state null resource */
     if (!Resource_CreateInode(wdv, parentResource, resourceName, WDVCAPI_RESOURCE_STATE_NULL_RESOURCE, 0,
-                              WDVCAPI_NO_OVERWRITE, newResource, NULL)) {
+                              WDVCAPI_NO_OVERWRITE, newResource, NULL, WDV_False)) {
         return WDV_False;
     }
 
@@ -1887,7 +1907,7 @@ WDV_Bool Resource_CreateNullResource( WDVCAPI_WDV       wdv,
     Resource_GetId(newResource, newId);
 
     /* Insert live properties */
-    if (!Property_CreateLiveProperties(wdv, newId, "", "null resource", "sapdbwww/nullresource", "0", WDV_False)) {
+    if (!Property_CreateLiveProperties(wdv, newId, resourceName, "null resource", "sapdbwww/nullresource", "0", WDV_False)) {
 		return WDV_False;
     }
     if (!Property_CreateDefaultProperties(wdv, newId, WDV_False)) {
@@ -2061,7 +2081,7 @@ WDV_Bool Resource_SetCompressedLength( WDVCAPI_WDV          wdv,
 
     /* Set value */
     wdv->compressedLength = compressedLength;
-    memcpy(wdv->compressedParentId, parentId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(wdv->compressedParentId, parentId, WDVCAPI_MAX_ID_LEN);
     Common_StrMaxCopy(wdv->compressedName, name, WDV_MAX_RESOURCE_NAME_LEN);
     wdv->compressedNameIndicator = SQL_NTS;
 
@@ -2152,7 +2172,7 @@ WDV_Bool Resource_GetCompressedLength( WDVCAPI_WDV           wdv,
     }
 
     /* Set value */
-    memcpy(wdv->compressedParentId, parentId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(wdv->compressedParentId, parentId, WDVCAPI_MAX_ID_LEN);
     Common_StrMaxCopy(wdv->compressedName, name, WDV_MAX_RESOURCE_NAME_LEN);
     wdv->compressedNameIndicator = SQL_NTS;
 
@@ -2200,12 +2220,12 @@ WDV_Bool Resource_InitHandle( WDVCAPI_WDV      wdv,
     resource->hStmtCollectChilds = 0;
     resource->hStmtCollectChildsWithProps = 0;
 
-	memcpy(resource->parentId, ID_ROOT, WDVCAPI_MAX_ID_LEN);
+	SAPDB_memcpy(resource->parentId, ID_ROOT, WDVCAPI_MAX_ID_LEN);
     resource->parentIdIndicator = WDVCAPI_MAX_ID_LEN;
 
-	memcpy(resource->id, ID_ROOT, WDVCAPI_MAX_ID_LEN);
+	SAPDB_memcpy(resource->id, ID_ROOT, WDVCAPI_MAX_ID_LEN);
     resource->idIndicator = WDVCAPI_MAX_ID_LEN;
-	memcpy(resource->idWithProps, ID_ROOT, WDVCAPI_MAX_ID_LEN);
+	SAPDB_memcpy(resource->idWithProps, ID_ROOT, WDVCAPI_MAX_ID_LEN);
     resource->idWithPropsIndicator = WDVCAPI_MAX_ID_LEN;
 
 	memset(resource->name, 0, WDV_MAX_RESOURCE_NAME_LEN);
@@ -2272,11 +2292,11 @@ WDV_Bool Resource_ResetHandle( WDVCAPI_WDV      wdv,
         }
     }
 
-	memcpy(resource->parentId, ID_ROOT, WDVCAPI_MAX_ID_LEN);
-	memcpy(resource->id, ID_ROOT, WDVCAPI_MAX_ID_LEN);
-	memcpy(resource->idWithProps, ID_ROOT, WDVCAPI_MAX_ID_LEN);
-	memcpy(resource->name, ID_ROOT, WDV_MAX_RESOURCE_NAME_LEN);
-	memcpy(resource->nameWithProps, ID_ROOT, WDV_MAX_RESOURCE_NAME_LEN);
+	SAPDB_memcpy(resource->parentId, ID_ROOT, WDVCAPI_MAX_ID_LEN);
+	SAPDB_memcpy(resource->id, ID_ROOT, WDVCAPI_MAX_ID_LEN);
+	SAPDB_memcpy(resource->idWithProps, ID_ROOT, WDVCAPI_MAX_ID_LEN);
+	SAPDB_memcpy(resource->name, ID_ROOT, WDV_MAX_RESOURCE_NAME_LEN);
+	SAPDB_memcpy(resource->nameWithProps, ID_ROOT, WDV_MAX_RESOURCE_NAME_LEN);
 
     resource->state = WDVCAPI_RESOURCE_STATE_DEFAULT;
     resource->compressedLength = 0;
@@ -2470,7 +2490,7 @@ WDV_Bool Resource_AddCacheItem( WDVCAPI_WDV                 wdv,
 
     /* Init new cache item */
     Common_StrMaxCopy(newCacheItem->uri, uri, WDV_MAX_URI_LEN);
-    memcpy(newCacheItem->parentId, parentId, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(newCacheItem->parentId, parentId, WDVCAPI_MAX_ID_LEN);
     Common_StrMaxCopy(newCacheItem->resourceName, resourceName, WDV_MAX_RESOURCE_NAME_LEN);
 
     newCacheItem->nextCacheItem = NULL;
@@ -2670,8 +2690,8 @@ WDV_Bool Resource_SetParent( WDVCAPI_WDV            wdv,
     }
 
     /* Set input parameters */
-    memcpy(wdv->setParentParentId, parentResource->id, WDVCAPI_MAX_ID_LEN);
-    memcpy(wdv->setParentResourceId, resource->id, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(wdv->setParentParentId, parentResource->id, WDVCAPI_MAX_ID_LEN);
+    SAPDB_memcpy(wdv->setParentResourceId, resource->id, WDVCAPI_MAX_ID_LEN);
     Common_StrMaxCopy(wdv->setParentResourceName, resourceName, WDV_MAX_RESOURCE_NAME_LEN);
     wdv->setParentResourceNameIndicator = SQL_NTS;
 
