@@ -804,6 +804,10 @@ void SetErrorList( WDVCAPI_WDV        wdv,
                    WDVCAPI_ErrorList  errorList )
 {
 
+    if ( !wdv ) {
+		return;
+	}
+
     wdv->errorList = errorList;
 
 }
@@ -814,6 +818,11 @@ void GetErrorList( WDVCAPI_WDV        wdv,
                    WDVCAPI_ErrorList *errorList )
 {
 
+    if ( !wdv ) {
+        *errorList = NULL;
+		return;
+	}
+
     *errorList = wdv->errorList;
 
 }
@@ -824,6 +833,10 @@ void SetGetHandle( WDVCAPI_WDV        wdv,
                    WDVCAPI_GetHandle  hGet )
 {
 
+    if ( !wdv ) {
+		return;
+	}
+
     wdv->hGet = hGet;
 
 }
@@ -834,6 +847,11 @@ void GetGetHandle( WDVCAPI_WDV        wdv,
                    WDVCAPI_GetHandle *hGet )
 {
 
+    if ( !wdv ) {
+        *hGet = NULL;
+		return;
+	}
+
     *hGet = wdv->hGet;
 
 }
@@ -850,6 +868,10 @@ WDV_Bool InitWDV( WDVCAPI_WDV  wdv,
                   XMLIMAPI_Handle   hXmlModelling )
 {
 
+    if ( !wdv ) {
+		return WDV_False;
+	}
+
 	/* Inits */
 	wdv->hEnv = hEnv;
 	wdv->hDBC = hDBC;
