@@ -38,7 +38,7 @@
 
 	#include "SAPDB/WebDAV/Handler/WDVHandler_CommonUtils.h"
 	#include "hwd21req.h"
-
+	#include "SAPDBCommon/SAPDB_string.h"
 
     /* Set the static values for timestamp conversion */
     static char *Month[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
@@ -229,9 +229,13 @@
 	{
 		WDVH_Char   temp[WDVH_MAX_URI_LEN+1];
 
-		/* set server prefix */
-		strcpy(temp,sapdbwa_GetServiceName(wa));
-		sp77sprintf(prefix,WDVH_MAX_URI_LEN+1,"/%s",temp);
+		if (wa != NULL) {
+			/* set server prefix */
+			strcpy(temp,sapdbwa_GetServiceName(wa));
+			sp77sprintf(prefix,WDVH_MAX_URI_LEN+1,"/%s",temp);
+		} else {
+			sp77sprintf(prefix,WDVH_MAX_URI_LEN+1,"/");
+		}
 	}
 
 	/*------------------------------------------------------------------------
@@ -246,6 +250,9 @@
 		WDVH_Char	*newUri;
 		WDVH_Char   prefix[WDVH_MAX_URI_LEN+1];
 
+		if (host==NULL || port==NULL || wa==NULL || request==NULL || uri==NULL)
+			return;
+
 		*host = (char*)sapdbwa_GetHeader(request, "HTTP_HOST" );
 		*port = (char*)sapdbwa_GetHeader(request, "HTTP_PORT" );
 		/* *port = "85"; */
@@ -272,6 +279,9 @@
 		WDVH_Char	*newUri;
 		WDVH_Char   prefix[WDVH_MAX_URI_LEN+1];
 
+		if (wa==NULL || request==NULL || uri==NULL)
+			return;
+
 		newUri = (char*)sapdbwa_GetRequestURI(request);
 		/* set server prefix */
 		getWebDAVPrefix(wa,prefix);
@@ -297,6 +307,9 @@
 		WDVH_Char	*pNewUri;
 		WDVH_Char	*server=NULL;
 
+		if (wa==NULL || request==NULL || uri==NULL)
+			return;
+
 		strcpy(newUri,uri);
 		/* set server prefix */
 		buildServerString(request,&server);
@@ -323,20 +336,36 @@
 		sapdbwa_Bool			allocatOK;
 		sapdbwa_Int4			length;
 	
+		if (request==NULL || server==NULL)
+			return;
+
 		host = (char*)sapdbwa_GetHeader(request, "HTTP_HOST" );
 		port = (char*)sapdbwa_GetHeader(request, "HTTP_PORT" );
 
-		if (port==NULL) {
-			length = strlen(WDVH_HTTP_PREFIX) + strlen(host) + 3;
-			WDVHandler_Allocat(sizeof(SAPDB_Char)*length,(sapdbwa_UInt1**) &newServer,&allocatOK);
+		if (host==NULL) {
+			WDVHandler_Allocat(sizeof(SAPDB_Char)*2,(sapdbwa_UInt1**) &newServer,&allocatOK);
 			if (allocatOK) {
-				sp77sprintf(newServer,length,"%s%s",WDVH_HTTP_PREFIX,host);
+				sp77sprintf(newServer,2,"");
 			}
 		} else {
-			length = strlen(WDVH_HTTP_PREFIX) + strlen(host) + strlen(port) + 3;
-			WDVHandler_Allocat(sizeof(SAPDB_Char)*length,(sapdbwa_UInt1**) &newServer,&allocatOK);
-			if (allocatOK) {
-				sp77sprintf(newServer,length,"%s%s:%s",WDVH_HTTP_PREFIX,host,port);
+			if (port==NULL) {
+				length = strlen(WDVH_HTTP_PREFIX) + strlen(host) + 3;
+				WDVHandler_Allocat(sizeof(SAPDB_Char)*length,(sapdbwa_UInt1**) &newServer,&allocatOK);
+				if (allocatOK) {
+					sp77sprintf(newServer,length,"%s%s",WDVH_HTTP_PREFIX,host);
+				}
+			} else if (strcmp(port,"")==0) {
+				length = strlen(WDVH_HTTP_PREFIX) + strlen(host) + 3;
+				WDVHandler_Allocat(sizeof(SAPDB_Char)*length,(sapdbwa_UInt1**) &newServer,&allocatOK);
+				if (allocatOK) {
+					sp77sprintf(newServer,length,"%s%s",WDVH_HTTP_PREFIX,host);
+				}
+			} else {
+				length = strlen(WDVH_HTTP_PREFIX) + strlen(host) + strlen(port) + 3;
+				WDVHandler_Allocat(sizeof(SAPDB_Char)*length,(sapdbwa_UInt1**) &newServer,&allocatOK);
+				if (allocatOK) {
+					sp77sprintf(newServer,length,"%s%s:%s",WDVH_HTTP_PREFIX,host,port);
+				}
 			}
 		}
 		
@@ -360,26 +389,35 @@
 		sapdbwa_Int4			length;
 		WDVH_Char				prefix[WDVH_MAX_URI_LEN+1];
 
-	
+		if (wa==NULL || request==NULL || server==NULL)
+			return;
+
 		host = (char*)sapdbwa_GetHeader(request, "HTTP_HOST" );
 		port = (char*)sapdbwa_GetHeader(request, "HTTP_PORT" );
 		/* set server prefix */
 		getWebDAVPrefix(wa,prefix);
 
-		if (port==NULL) {
-			length = strlen(WDVH_HTTP_PREFIX) + strlen(host) + strlen(prefix) + 3;
-			WDVHandler_Allocat(sizeof(SAPDB_Char)*length,(sapdbwa_UInt1**) &newServer,&allocatOK);
+		if (host==NULL) {
+			WDVHandler_Allocat(sizeof(SAPDB_Char)*2,(sapdbwa_UInt1**) &newServer,&allocatOK);
 			if (allocatOK) {
-				sp77sprintf(newServer,length,"%s%s%s",WDVH_HTTP_PREFIX,host,prefix);
+				sp77sprintf(newServer,2,"");
 			}
 		} else {
-			length = strlen(WDVH_HTTP_PREFIX) + strlen(host) + strlen(port) + strlen(prefix) + 3;
-			WDVHandler_Allocat(sizeof(SAPDB_Char)*length,(sapdbwa_UInt1**) &newServer,&allocatOK);
-			if (allocatOK) {
-				sp77sprintf(newServer,length,"%s%s:%s%s",WDVH_HTTP_PREFIX,host,port,prefix);
+			if (port==NULL) {
+				length = strlen(WDVH_HTTP_PREFIX) + strlen(host) + strlen(prefix) + 3;
+				WDVHandler_Allocat(sizeof(SAPDB_Char)*length,(sapdbwa_UInt1**) &newServer,&allocatOK);
+				if (allocatOK) {
+					sp77sprintf(newServer,length,"%s%s%s",WDVH_HTTP_PREFIX,host,prefix);
+				}
+			} else {
+				length = strlen(WDVH_HTTP_PREFIX) + strlen(host) + strlen(port) + strlen(prefix) + 3;
+				WDVHandler_Allocat(sizeof(SAPDB_Char)*length,(sapdbwa_UInt1**) &newServer,&allocatOK);
+				if (allocatOK) {
+					sp77sprintf(newServer,length,"%s%s:%s%s",WDVH_HTTP_PREFIX,host,port,prefix);
+				}
 			}
 		}
-		
+
 		if (allocatOK) 
 			*server = newServer;
 		else
@@ -390,12 +428,16 @@
 		Function:	getIfHeader
 	--------------------------------------------------------------------------*/
 	WDVH_Bool getIfHeader(sapdbwa_HttpRequestP request,
-		                  WDVH_Char *sIf)
+		                  WDVH_Char *sIf,
+						  WDVH_Int4 len)
 	{
 		WDVH_Char	*temp1, *temp2, *temp4, *temp5;
 		WDVH_UInt4	length;
 		WDVH_Char	temp3[WDVH_MAX_IF_HEADER_LEN];
 
+		if (request==NULL || sIf==NULL)
+			return WDVH_False;
+
 		strcpy(sIf,"");
 		temp4 = (char*)sapdbwa_GetHeader(request,"If");
 		if (temp4 != NULL) {
@@ -411,7 +453,8 @@
 				if (strncmp(temp4,"saplocktoken:",strlen("saplocktoken:"))==0) {
 					strncpy(temp4,temp3+strlen("saplocktoken:"),length-strlen("saplocktoken:"));
 					temp4[length-strlen("saplocktoken:")] = '\0';
-					strcpy(sIf,temp4);
+					strncpy(sIf,temp4,len);
+					sIf[length-1]='\0';
 				}
 				return WDVH_True;
 			} else {
@@ -436,6 +479,9 @@
 		WDVH_UInt4	length,state;
 		WDVH_Char	*server;
 
+		if (wa==NULL || request==NULL || wdv==NULL || lockTokenList==NULL)
+			return WDVH_False;
+
 		strcpy(resource,"");
 		strcpy(value,"");
 		length = 0;
@@ -446,49 +492,51 @@
 		if (ifheader) {
 			/* get server string */
 			buildServerStringLong(wa,request,&server);
-			c=ifheader;
-			while (*c!='\0') {
-				if (*c=='<' && state==0) {
-					/* begin of resource url */
-					state = 1;
-					resourceP = c+1;
-					length = 0;
-				} else if (*c=='<' && state==2) {
-					state = 2;
-					valueP = c+1;
-					length = 0;
-				} else if (*c=='(' && state == 0) {
-					/* begin of value */
-					state = 2;
-				} else if (*c==')' && state == 2) {
-					state = 0;
-				} else if (*c=='>' && state == 1) {
-					/* end of resource url */
-					state = 0;
-					if (strncmp(resourceP,server,strlen(server))==0) {
-						resourceP = resourceP+strlen(server);
-						length = length-strlen(server);
-						strncpy(resource,resourceP,length);
-						resource[length] = '\0';
-					}
-				} else if (*c=='>' && state == 2) {
-					/* end of value */
-					state = 2;
-					if (strncmp(valueP,"saplocktoken:",strlen("saplocktoken:"))==0) {
-						valueP = valueP+strlen("saplocktoken:");
-						length = length - strlen("saplocktoken:");
-						strncpy(value,valueP,length);
-						value[length] = '\0';
-						if (strcmp(resource,"")==0) {
-							WDVCAPI_LockAddIdToList(wdv,lockTokenList,NULL,value);			/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1120166 */
-						} else {
-							WDVCAPI_LockAddIdToList(wdv,lockTokenList,resource,value);		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1120166 */
+			if (server!=NULL) {
+				c=ifheader;
+				while (*c!='\0') {
+					if (*c=='<' && state==0) {
+						/* begin of resource url */
+						state = 1;
+						resourceP = c+1;
+						length = 0;
+					} else if (*c=='<' && state==2) {
+						state = 2;
+						valueP = c+1;
+						length = 0;
+					} else if (*c=='(' && state == 0) {
+						/* begin of value */
+						state = 2;
+					} else if (*c==')' && state == 2) {
+						state = 0;
+					} else if (*c=='>' && state == 1) {
+						/* end of resource url */
+						state = 0;
+						if (strncmp(resourceP,server,strlen(server))==0) {
+							resourceP = resourceP+strlen(server);
+							length = length-strlen(server);
+							strncpy(resource,resourceP,length);
+							resource[length] = '\0';
 						}
+					} else if (*c=='>' && state == 2) {
+						/* end of value */
+						state = 2;
+						if (strncmp(valueP,"saplocktoken:",strlen("saplocktoken:"))==0) {
+							valueP = valueP+strlen("saplocktoken:");
+							length = length - strlen("saplocktoken:");
+							strncpy(value,valueP,length);
+							value[length] = '\0';
+							if (strcmp(resource,"")==0) {
+								WDVCAPI_LockAddIdToList(wdv,lockTokenList,NULL,value);			/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1120166 */
+							} else {
+								WDVCAPI_LockAddIdToList(wdv,lockTokenList,resource,value);		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1120166 */
+							}
+						}
+					} else {
+						length++;
 					}
-				} else {
-					length++;
+					c++;
 				}
-				c++;
 			}
 			if (server) {
 				WDVHandler_Free((sapdbwa_UInt1*) server);
@@ -508,6 +556,9 @@
 		WDVH_UInt4	length;
 		WDVH_Char	temp3[WDVH_MAX_IF_HEADER_LEN];
 
+		if (request==NULL || from==NULL || to==NULL)
+			return WDVH_False;
+
 		*from = NULL;
 		*to = NULL;
 		crheader = (char*)sapdbwa_GetHeader(request,"Range");
@@ -544,13 +595,45 @@
 			return WDVH_False;
 		}
 	}
+	/*------------------------------------------------------------------------
+		Function:	getUserAgentHeader
+	--------------------------------------------------------------------------*/
+	WDVH_Bool getUserAgentHeader(sapdbwa_HttpRequestP request,
+		                  WDVH_Char *sUA,
+						  WDVH_Int4 len)
+	{
+		WDVH_Char	*uaheader;
+		WDVH_UInt4	length;
+		WDVH_Char	temp3[WDVH_MAX_IF_HEADER_LEN];
+
+		if (request==NULL || sUA==NULL)
+			return WDVH_False;
+
+		strcpy(sUA,"");
+		uaheader = (char*)sapdbwa_GetHeader(request,"User-Agent");
+		if (uaheader != NULL) {
+			/* User Agent Header found */
+			/* return the User-Agent Header Value */
+			strncpy(sUA,uaheader,len);
+			sUA[len] = '\0';
+			return WDVH_True;
+		} else {
+			return WDVH_False;
+		}
+	}
+
 	/*------------------------------------------------------------------------
 	    Function: isNumber
 	--------------------------------------------------------------------------*/
 	WDVH_Bool isNumber(WDVH_Char *str) 
 	{
-		WDVH_Char	c = str[0];
+		WDVH_Char c;
 		WDVH_Long   pos = 0;
+
+		if (str==NULL)
+			return WDVH_False;
+
+		c = str[0];
 		while(c != '\0') {
 			if (!isdigit(c))
 				return WDVH_False;
@@ -571,6 +654,9 @@
 		WDVH_UInt4	length;
 		WDVH_Char	temp3[WDVH_MAX_IF_HEADER_LEN];
 
+		if (request==NULL || sLockToken==NULL || errormsg==NULL)
+			return WDVH_False;
+
 		temp4 = (char*)sapdbwa_GetHeader(request,"Lock-Token");
 		if (temp4 != NULL) {
 			strcpy(temp3,temp4);
@@ -630,6 +716,9 @@
 	{
 		WDVH_Char		*temp;
 
+		if (request==NULL || sDc==NULL)
+			return WDVH_False;
+
 		temp = (char*)sapdbwa_GetHeader(request,"SAPDB_DocumentClass");
 		if (temp != NULL) {
 			strcpy(sDc,temp);
@@ -648,6 +737,9 @@
 	{
 		WDVH_Char		*temp;
 
+		if (request==NULL || async==NULL)
+			return WDVH_False;
+
 		temp = (char*)sapdbwa_GetHeader(request,"SAPDB_AsyncIndexing");
 		if (temp != NULL) {
 			if (strcmp(temp,"F")==0 || strcmp(temp,"f")==0)
@@ -669,6 +761,9 @@
 	{
 		WDVH_Char		*temp;
 
+		if (request==NULL || delitems==NULL)
+			return WDVH_False;
+
 		temp = (char*)sapdbwa_GetHeader(request,"SAPDB_DeletedItems");
 		if (temp != NULL) {
 			if (strcmp(temp,"F")==0 || strcmp(temp,"f")==0)
@@ -741,7 +836,7 @@
 		WDVH_Bool			ok = WDVH_False;
 		WDVH_Handle			newHandle = NULL;
 
-		if ( !handle ) {
+		if ( !handle || wa==NULL || conn==NULL) {
 			return WDVH_False;
 		}
     
@@ -971,7 +1066,7 @@
 		if (ok == WDVH_True) {
 
 			strcpy(newItem->docClassName,docClassName);
-			memcpy(newItem->docClassId,docClassId,XMLIMAPI_MAX_ID_LEN);
+			SAPDB_memcpy(newItem->docClassId,docClassId,XMLIMAPI_MAX_ID_LEN);
 			newItem->nextItem = NULL;
 			if (!list->first) {
 				/* first item */
@@ -1010,7 +1105,7 @@
 			current = next;
 			next = current->nextItem;
 			if (strcmp(current->docClassName,docClassName)==0) {
-				memcpy(*docClassId,current->docClassId,XMLIMAPI_MAX_ID_LEN);
+				SAPDB_memcpy(*docClassId,current->docClassId,XMLIMAPI_MAX_ID_LEN);
 				return WDVH_True;
 			}
 		}
@@ -1030,7 +1125,7 @@
 		SQLHDBC					odbchandle;
 		SQLHENV					henvhandle;
 
-		if(!handle)
+		if(!handle || wa==NULL || conn==NULL)
 			return WDVH_False;
 
 		if (conn) {
@@ -1047,6 +1142,9 @@
 	--------------------------------------------------------------------------*/
 	WDVH_Bool destroyXmlmdlapiHandle(XMLIMAPI_Handle handle)
 	{
+		if (handle==NULL)
+			return WDVH_False;
+
 		XMLIMAPI_destroyHandle(handle);
 		return WDVH_True;
 	}
@@ -1063,6 +1161,9 @@
 		SAPDB_Char   szPath[256] = "";
 		sapdbwa_Bool bRet;
 		
+		if (wa==NULL || rep==NULL || server==NULL || servicename==NULL)
+			return;
+
 		/* send page headers */
 		sapdbwa_InitHeader(rep,sapdbwa_Status_Ok, "text/html", NULL, NULL,NULL, NULL );
 		sapdbwa_SetHeader (rep,"Expires", "Sat, 01 Jan 2000 00:00:00 GMT");
@@ -1156,6 +1257,9 @@
 		SAPDB_Char   szPath[256] = "";
 		sapdbwa_Bool bRet;
 		
+		if (wa==NULL || rep==NULL || querystring==NULL || server==NULL || servicename==NULL)
+			return;
+
 		/* send page headers */
 		sapdbwa_InitHeader(rep,sapdbwa_Status_Ok, "text/html", NULL, NULL,NULL, NULL );
 		sapdbwa_SetHeader (rep,"Expires", "Sat, 01 Jan 2000 00:00:00 GMT");
@@ -1270,7 +1374,12 @@
 				 	 		    sapdbwa_HttpRequestP req,
 				 	 		    WDVH_Char *sValue)
 	{
-		sapdbwa_StringSeqP	__values = sapdbwa_CreateStringSeq();
+		sapdbwa_StringSeqP	__values;
+		
+		if (pParamName==NULL || req==NULL || sValue==NULL)
+			return WDVH_False;
+
+		__values = sapdbwa_CreateStringSeq();
 
 		strcpy(sValue,"");
 		if (sapdbwa_GetParameterValues(req, pParamName, __values) == sapdbwa_True) {
@@ -1287,7 +1396,12 @@
 	const WDVH_Char* getParameterValueWithoutCopy(const WDVH_Char *pParamName,
 				 	 					sapdbwa_HttpRequestP req)
 	{
-		sapdbwa_StringSeqP	__values = sapdbwa_CreateStringSeq();
+		sapdbwa_StringSeqP	__values;
+
+		if (pParamName==NULL || req==NULL)
+			return NULL;
+
+		__values = sapdbwa_CreateStringSeq();
 
 		if (sapdbwa_GetParameterValues(req, pParamName, __values) == sapdbwa_True) {
 			if (sapdbwa_GetNumElem(__values) > 0) {
@@ -1307,6 +1421,10 @@
 		/* WDVH_ULong	size=0; */
 		WDVH_Double size=0;
 		WDVH_Int2	firstpos,i,pos;
+		
+		if (in==NULL || out==NULL)
+			return;
+
 		/* size = atol(in); */
 		size = strtod(in,(char**)NULL);
 		firstpos = strlen(in)%3;
@@ -1345,132 +1463,171 @@
 				sprintf(out,"%.3f GB",size/(1024*1024*1024));
 	}
 
-	void WDVH_EscapeQuotes(WDVH_Char *oldstring, WDVH_Int4 len, WDVH_Char *newstring)
+	void WDVH_EscapeQuotes(WDVH_Char *oldstring, WDVH_Int4 len, WDVH_Char *newstring, WDVH_Int4 len_new)
 	{
 		WDVH_Int4 posnew,i;
+
+		if (oldstring==NULL || newstring==NULL)
+			return;
+
 		posnew = 0;
 		for (i=0;i<=len;i++) {
 			if (oldstring[i]=='"') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 'q';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 'o';
-				newstring[posnew++] = 't';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 'q';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 'o';
+					newstring[posnew++] = 't';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='&') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 'a';
-				newstring[posnew++] = 'm';
-				newstring[posnew++] = 'p';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 6) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 'a';
+					newstring[posnew++] = 'm';
+					newstring[posnew++] = 'p';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = '#';
-				newstring[posnew++] = '1';
-				newstring[posnew++] = '8';
-				newstring[posnew++] = '6';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = '#';
+					newstring[posnew++] = '1';
+					newstring[posnew++] = '8';
+					newstring[posnew++] = '6';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 's';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 'p';
-				newstring[posnew++] = '2';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 's';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 'p';
+					newstring[posnew++] = '2';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 's';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 'p';
-				newstring[posnew++] = '3';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 's';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 'p';
+					newstring[posnew++] = '3';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 'a';
-				newstring[posnew++] = 'c';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 't';
-				newstring[posnew++] = 'e';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 8) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 'a';
+					newstring[posnew++] = 'c';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 't';
+					newstring[posnew++] = 'e';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 'm';
-				newstring[posnew++] = 'i';
-				newstring[posnew++] = 'c';
-				newstring[posnew++] = 'r';
-				newstring[posnew++] = 'o';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 8) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 'm';
+					newstring[posnew++] = 'i';
+					newstring[posnew++] = 'c';
+					newstring[posnew++] = 'r';
+					newstring[posnew++] = 'o';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = '#';
-				newstring[posnew++] = '8';
-				newstring[posnew++] = '3';
-				newstring[posnew++] = '6';
-				newstring[posnew++] = '4';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 8) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = '#';
+					newstring[posnew++] = '8';
+					newstring[posnew++] = '3';
+					newstring[posnew++] = '6';
+					newstring[posnew++] = '4';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = '#';
-				newstring[posnew++] = '1';
-				newstring[posnew++] = '6';
-				newstring[posnew++] = '7';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = '#';
+					newstring[posnew++] = '1';
+					newstring[posnew++] = '6';
+					newstring[posnew++] = '7';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]==' ') {
-				newstring[posnew++] = '%';
-				newstring[posnew++] = '2';
-				newstring[posnew++] = '0';
+				if (posnew < len_new - 4) {
+					newstring[posnew++] = '%';
+					newstring[posnew++] = '2';
+					newstring[posnew++] = '0';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 'a';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 'm';
-				newstring[posnew++] = 'l';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 'a';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 'm';
+					newstring[posnew++] = 'l';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 'A';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 'm';
-				newstring[posnew++] = 'l';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 'A';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 'm';
+					newstring[posnew++] = 'l';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 'o';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 'm';
-				newstring[posnew++] = 'l';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 'o';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 'm';
+					newstring[posnew++] = 'l';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 'O';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 'm';
-				newstring[posnew++] = 'l';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 'O';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 'm';
+					newstring[posnew++] = 'l';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 'm';
-				newstring[posnew++] = 'l';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 'm';
+					newstring[posnew++] = 'l';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 'U';
-				newstring[posnew++] = 'u';
-				newstring[posnew++] = 'm';
-				newstring[posnew++] = 'l';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 7) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 'U';
+					newstring[posnew++] = 'u';
+					newstring[posnew++] = 'm';
+					newstring[posnew++] = 'l';
+					newstring[posnew++] = ';';
+				}
 			} else if (oldstring[i]=='') {
-				newstring[posnew++] = '&';
-				newstring[posnew++] = 's';
-				newstring[posnew++] = 'z';
-				newstring[posnew++] = 'l';
-				newstring[posnew++] = 'i';
-				newstring[posnew++] = 'g';
-				newstring[posnew++] = ';';
+				if (posnew < len_new - 8) {
+					newstring[posnew++] = '&';
+					newstring[posnew++] = 's';
+					newstring[posnew++] = 'z';
+					newstring[posnew++] = 'l';
+					newstring[posnew++] = 'i';
+					newstring[posnew++] = 'g';
+					newstring[posnew++] = ';';
+				}
 			} else {
-				newstring[posnew++] = oldstring[i];
+				if (posnew < len_new)
+					newstring[posnew++] = oldstring[i];
 			}
 		}
 		newstring[posnew] = '\0';
@@ -1481,6 +1638,9 @@
 	--------------------------------------------------------------------------*/
 	void WDVH_UnEscapeUri(WDVH_PChar uri) 
 	{
+		if (uri==NULL)
+			return;
+
 		wd21_UnescapeUrl(uri,strlen(uri));
 		if (strcmp(uri,"")==0) 
 			strcpy(uri,"/");
@@ -1529,7 +1689,7 @@
 				contenttype[i] == ',' || contenttype[i] == ';' || contenttype[i] == ':' || contenttype[i] == '\\' || contenttype[i] == '"' ||
 				contenttype[i] == '/' || contenttype[i] == '[' || contenttype[i] == ']' || contenttype[i] == '?' || contenttype[i] == '=' ||
 				contenttype[i] == '{' || contenttype[i] == '}' || contenttype[i] == ' ' || contenttype[i] == '\t' ||
-				(contenttype[i] >= 0x00 && contenttype[i] <= 0x1F) || contenttype[i] == 0x7F) {
+				(contenttype[i] >= 0x00 && contenttype[i] <= 0x1F) || contenttype[i] >= 0x7F) {
 				return WDVH_False;
 			}
 		}
@@ -1538,7 +1698,7 @@
 				subtype[i] == ',' || subtype[i] == ';' || subtype[i] == ':' || subtype[i] == '\\' || subtype[i] == '"' ||
 				subtype[i] == '/' || subtype[i] == '[' || subtype[i] == ']' || subtype[i] == '?' || subtype[i] == '=' ||
 				subtype[i] == '{' || subtype[i] == '}' || subtype[i] == ' ' || subtype[i] == '\t' ||
-				(subtype[i] >= 0x00 && subtype[i] <= 0x1F) || subtype[i] == 0x7F) {
+				(subtype[i] >= 0x00 && subtype[i] <= 0x1F) || subtype[i] >= 0x7F) {
 				return WDVH_False;
 			}
 		}
@@ -1552,6 +1712,9 @@
 	--------------------------------------------------------------------------*/
 	WDVH_Bool WDVH_CheckPropertySet(WDVH_PChar tag) 
 	{
+		if (tag==NULL)
+			return WDVH_False;
+
 		/* Live Properties that are forbidden to change:
 		<!ELEMENT creationdate (#PCDATA) >
 		<!ELEMENT getcontentlength (#PCDATA) >
@@ -1575,6 +1738,9 @@
 	--------------------------------------------------------------------------*/
 	WDVH_Bool WDVH_CheckPropertyRemove(WDVH_PChar tag) 
 	{
+		if (tag==NULL)
+			return WDVH_False;
+
 		/* Live Properties that are forbidden to change:
 		<!ELEMENT creationdate (#PCDATA) >
 		<!ELEMENT displayname (#PCDATA) >
@@ -1596,3 +1762,115 @@
 		}
 		return WDVH_True;
 	}
+
+	void WDVH_sendBodyChunk(sapdbwa_HttpReplyP reply, WDVH_Char *data)
+	{
+		WDVH_Char	chunk[101];
+
+		if (reply==NULL || data==NULL)
+			return;
+
+		if (strlen(data)>0) {
+			sp77sprintf(chunk,100,"%x%c%c",strlen(data),0x0D,0x0A);
+			sapdbwa_SendBody(reply,chunk,strlen(chunk));
+			sapdbwa_SendBody(reply,data,strlen(data));
+			sp77sprintf(chunk,100,"%c%c",0x0D,0x0A);
+			sapdbwa_SendBody(reply,chunk,strlen(chunk));
+		}
+	}
+
+	void WDVH_sendBodyChunkPropfind(sapdbwa_HttpReplyP reply, WDVH_Char *data, WDVH_Bool isMSWebFolder)
+	{
+		WDVH_Char	chunk[101];
+
+		if (reply==NULL || data==NULL)
+			return;
+
+		if (strlen(data)>0) {
+			if (isMSWebFolder == WDVH_False) {
+				sp77sprintf(chunk,100,"%x%c%c",strlen(data),0x0D,0x0A);
+				sapdbwa_SendBody(reply,chunk,strlen(chunk));
+			}
+			sapdbwa_SendBody(reply,data,strlen(data));
+			if (isMSWebFolder == WDVH_False) {
+				sp77sprintf(chunk,100,"%c%c",0x0D,0x0A);
+				sapdbwa_SendBody(reply,chunk,strlen(chunk));
+			}
+		}
+	}
+
+	void WDVH_sendBodyChunkEnd(sapdbwa_HttpReplyP reply)
+	{
+		WDVH_Char            chunk[20];
+
+		if (reply==NULL)
+			return;
+
+		chunk[0] = '0';
+		chunk[1] = 0x0D;
+		chunk[2] = 0x0A;
+		chunk[3] = 0x0D;
+		chunk[4] = 0x0A;
+		chunk[5] = 0x00;
+		sapdbwa_SendBody(reply,chunk,strlen(chunk));
+
+	}
+	
+	WDVH_Char bin2hex (WDVH_Char ch)         /* return ascii char for hex value of */
+	{  									     /* rightmost 4 bits of input */
+		ch = ch & 0x0f;            /* mask off right nibble  - & bitwise AND*/
+		ch += '0';                 /* make ascii '0' - '9'  */
+		if (ch > '9')
+			ch += 7;               /* account for 7 chars between '9' and 'A' */
+		return (ch);
+	}
+
+	WDVH_Int4 AlphaNumeric (WDVH_Char ch){
+		return ((ch >='a') && (ch <= 'z') ||   /* logical AND &&, OR || */
+				(ch >='A') && (ch <= 'Z') ||
+				(ch >='0') && (ch <= '9') ||
+				(ch =='!') ||
+				(ch =='$') ||
+				(ch =='\'') ||
+				(ch =='(') ||
+				(ch ==')') ||
+				(ch ==',') ||
+				(ch =='-') ||
+				(ch =='.') ||
+				(ch =='=') ||
+				(ch =='@') ||
+				(ch =='_') ||
+				(ch =='~') ||
+				(ch =='/') );
+	}
+
+
+	WDVH_Int4 URLencode (const WDVH_Char * plain, WDVH_Char * encode, WDVH_Int4 maxlen){
+		WDVH_Char ch;            /* each char, use $t2 */
+		WDVH_Char * limit;       /* point to last available location in encode */
+		WDVH_Char * start;       /* save start of encode for length calculation */
+
+		start = encode;
+		limit = encode + maxlen - 4;    /* need to store 3 chars and a zero */
+		ch = *plain ++;    /* get first character */
+		while (ch != 0) {       /* end of string, asciiz */
+			if (ch == ' ') {
+				* encode ++ = '%';
+				* encode ++ = '2';
+				* encode ++ = '0';
+			} else if (AlphaNumeric (ch))
+				* encode ++ = ch;
+			else {
+				* encode ++ = '%';
+				* encode ++ = bin2hex (ch >> 4); /*shift right for left nibble*/
+				* encode ++ = bin2hex (ch);      /* right nibble */
+			}
+			ch = *plain ++;               /* ready for next character */
+			if (encode > limit){
+				*encode = 0;        /* still room to terminate string */
+				return (-1);        /* get out with error indication */
+			}
+		}
+		* encode = 0;               /* store zero byte to terminate string */
+		return (encode - start);    /* done, return count of characters */
+	}   /* URLencode*/
