@@ -60,7 +60,18 @@
 			double     duration;
 		#endif
 
-		
+		if (wa==NULL)
+			return;
+		if (reply==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavMkcolHandler:Uninitialized Pointers\n");
+			return;
+		}
+		if (request==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavMkcolHandler:Uninitialized Pointers\n");
+			sendErrorReply(ERROR_500, reply, WEBDAV_MKCOL,"");
+			return;
+		}
+
 		/* first get a connection from session pool */
 		#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 			time1 = clock();
@@ -74,6 +85,17 @@
 			/* get wdvh handle */
 			wdvhHandle = getWdvhHandle(wa,connection);
 			strcpy(errormsg,"");
+			/* check the handles */
+			if (pCapiHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDV Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_MKCOL,"");
+				return;
+			}
+			if (wdvhHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDVH Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_MKCOL,"");
+				return;
+			}
 
 			/* get request uri */
 			getRequestUri(wa,request,wdvhHandle->requesturi);
@@ -83,9 +105,8 @@
 
 			/* interpret If-Header */
 	/* ===> TODO!!!!! */
-			getIfHeader(request,wdvhHandle->locktoken);
-/*			pIf = sIf;
-*/			
+			getIfHeader(request,wdvhHandle->locktoken,WDVCAPI_MAX_ID_STRING_LEN);
+
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 				time2 = clock();
 				duration = ((time2-time1)*1000) / CLOCKS_PER_SEC;
@@ -154,12 +175,24 @@
 			double     duration;
 		#endif
 
+		if (wa==NULL)
+			return;
+		if (reply==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavDeleteHandler:Uninitialized Pointers\n");
+			return;
+		}
+		if (request==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavDeleteHandler:Uninitialized Pointers\n");
+			sendErrorReply(ERROR_500, reply, WEBDAV_DELETE,"");
+			return;
+		}
+
 		
 		/* extract Parameters (Host, RequestURI)*/
 		getFirstHeaderLine(wa,request,&host,&port,requestURI);
 
 		/* interpret If-Header */
-		getIfHeader(request,sIf);
+		getIfHeader(request,sIf,100);
 		pIf = sIf;
 
 		/* get SAPDB_DeletedItems Header */
@@ -175,6 +208,12 @@
 		if (connection != NULL) {
 			/* get wdv handle */
 			pCapiHandle = getWdvHandle(wa,connection);
+			/* check the handles */
+			if (pCapiHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDV Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_DELETE,"");
+				return;
+			}
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 				time2 = clock();
 				duration = ((time2-time1)*1000) / CLOCKS_PER_SEC;
@@ -247,12 +286,24 @@
 			double     duration;
 		#endif
 		
+		if (wa==NULL)
+			return;
+		if (reply==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavGetHandler:Uninitialized Pointers\n");
+			return;
+		}
+		if (request==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavGetHandler:Uninitialized Pointers\n");
+			sendErrorReply(ERROR_500, reply, WEBDAV_GET,"");
+			return;
+		}
+
 		/* extract Parameters (Host, RequestURI)*/
 		getFirstHeaderLine(wa,request,&host,&port,requestURI);
 
 		/* interpret Header's */
 /* ===> TODO!!!!! */
-		getIfHeader(request,sIf);
+		getIfHeader(request,sIf,100);
 		pIf = sIf;
 		
 		/* Begin Correction  http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1120395 */
@@ -336,6 +387,18 @@
 			/* get wdv handle */
 			pCapiHandle = getWdvHandle(wa,connection);
 			wdvhHandle = getWdvhHandle(wa,connection);
+			/* check the handles */
+			if (pCapiHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDV Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_GET,"");
+				return;
+			}
+			if (wdvhHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDVH Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_GET,"");
+				return;
+			}
+
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 				time2 = clock();
 				duration = ((time2-time1)*1000) / CLOCKS_PER_SEC;
@@ -403,12 +466,24 @@
 			double     duration;
 		#endif
 		
+		if (wa==NULL)
+			return;
+		if (reply==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavHeadHandler:Uninitialized Pointers\n");
+			return;
+		}
+		if (request==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavHeadHandler:Uninitialized Pointers\n");
+			sendErrorReply(ERROR_500, reply, WEBDAV_HEAD,"");
+			return;
+		}
+
 		/* get request uri */
 		getRequestUri(wa,request,requestURI);
 
 		/* interpret Header's */
 /* ===> TODO!!!!!*/
-		getIfHeader(request,sIf);
+		getIfHeader(request,sIf,100);
 		pIf = sIf;
 		
 		/* first get a connection from session pool */
@@ -421,6 +496,12 @@
 		if (connection != NULL) {
 			/* get wdv handle */
 			pCapiHandle = getWdvHandle(wa,connection);
+			/* check the handles */
+			if (pCapiHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDV Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_HEAD,"");
+				return;
+			}
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 				time2 = clock();
 				duration = ((time2-time1)*1000) / CLOCKS_PER_SEC;
@@ -481,12 +562,24 @@
 		WDVH_Char				sIf[100];
 		WDVH_Char				*pIf;
 		
+		if (wa==NULL)
+			return;
+		if (reply==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavOptionsHandler:Uninitialized Pointers\n");
+			return;
+		}
+		if (request==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavOptionsHandler:Uninitialized Pointers\n");
+			sendErrorReply(ERROR_500, reply, WEBDAV_OPTIONS,"");
+			return;
+		}
+
 		/* extract Parameters (Host, RequestURI)*/
 		getFirstHeaderLine(wa,request,&host,&port,requestURI);
 
 		/* interpret Header's */
 /* ===> TODO!!!!! */
-		getIfHeader(request,sIf);
+		getIfHeader(request,sIf,100);
 		pIf = sIf;
 		
 		/* call CAPI function*/
@@ -512,41 +605,63 @@
 		WDVH_Char				*port;
 		WDVH_Int2				statuscode = ERROR_200;
 		WDVCAPI_Depth			depth;
-		WDVH_Char				sDepth[10];
-		WDVH_Char				*pDepth;
+		WDVH_Char				*pDepth=NULL;
 		WDVH_Char				sIf[100];
 		WDVH_Char				*pIf;
+		WDVH_Char				sUserAgent[1001];
+		WDVH_Char				*pUserAgent;
 		WDVH_Bool				isEmptyPropertyList;	/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123332 */
 		WDVH_Char				errormsg[1001];			/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
+		WDVH_Bool				isPropnameRequest;		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1130891 */
 		#if defined(PROFILING_OVERALL) || defined(PROFILING_DETAIL)
 			clock_t    time1;
 			clock_t	   time2;
 			double     duration;
 		#endif
 		
+		if (wa==NULL)
+			return;
+		if (reply==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavPropfindHandler:Uninitialized Pointers\n");
+			return;
+		}
+		if (request==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavPropfindHandler:Uninitialized Pointers\n");
+			sendErrorReply(ERROR_500, reply, WEBDAV_PROPFIND,"");
+			return;
+		}
+
 		/* extract Parameters (Host, RequestURI)*/
 		getFirstHeaderLine(wa,request,&host,&port,requestURI);
 
 		/* interpret Header's */
-		pDepth = sDepth;
 		pDepth = (char*)sapdbwa_GetHeader(request,"Depth");
 		if (pDepth) {
-			strcpy(sDepth,sapdbwa_GetHeader(request, "Depth" ));
-			if (strcmp(sDepth,WDVH_DEPTH_0)==0)
+			if (strcmp(pDepth,WDVH_DEPTH_0)==0)
 				depth = WDVCAPI_DEPTH_0;
-			else if (strcmp(sDepth,WDVH_DEPTH_1)==0)
+			else if (strcmp(pDepth,WDVH_DEPTH_1)==0)
 				depth = WDVCAPI_DEPTH_1;
-			else if (strcmp(sDepth,WDVH_DEPTH_INFINITY)==0)
+			else if (strcmp(pDepth,WDVH_DEPTH_INFINITY)==0)
 				depth = WDVCAPI_DEPTH_INFINITE;
 			else {
 				sendErrorReply(ERROR_400, reply, WEBDAV_PROPFIND,"");
 			}
 		} else
 			depth = WDVCAPI_DEPTH_INFINITE;
+
 		/* - If */
-		getIfHeader(request,sIf);
+		getIfHeader(request,sIf,100);
 		pIf = sIf;
 
+		/* - User-Agent */
+		pUserAgent = (char*)sapdbwa_GetHeader(request,"User-Agent");
+		if (pUserAgent==NULL) {
+			strcpy(sUserAgent,"");
+		} else {
+			strncpy(sUserAgent,pUserAgent,1000);
+			sUserAgent[1000]='\0';
+		}
+
 		/* Build Handles */
 		/* - get a connection from session pool */
 		#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
@@ -558,6 +673,12 @@
 		if (connection != NULL) {
 			/* get wdv handle */
 			pCapiHandle = getWdvHandle(wa,connection);
+			/* check the handles */
+			if (pCapiHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDV Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_PROPFIND,"");
+				return;
+			}
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 				time2 = clock();
 				duration = ((time2-time1)*1000) / CLOCKS_PER_SEC;
@@ -568,7 +689,7 @@
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 				time1 = clock();
 			#endif
-			statuscode = propfindParseRequestBody(wa,pCapiHandle,&propfind,request,depth,&isEmptyPropertyList,errormsg); /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
+			statuscode = propfindParseRequestBody(wa,pCapiHandle,&propfind,request,depth,&isPropnameRequest,&isEmptyPropertyList,errormsg); /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 				time2 = clock();
 				duration = ((time2-time1)*1000) / CLOCKS_PER_SEC;
@@ -594,7 +715,7 @@
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 				time1 = clock();
 			#endif
-			propfindBuildReply(wa,statuscode,reply,host,port,requestURI,pCapiHandle,propfind,isEmptyPropertyList,errormsg); /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
+			propfindBuildReply(wa,statuscode,reply,host,port,requestURI,pCapiHandle,propfind,isPropnameRequest,isEmptyPropertyList,sUserAgent,errormsg); /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 				time2 = clock();
 				duration = ((time2-time1)*1000) / CLOCKS_PER_SEC;
@@ -645,9 +766,7 @@
 		WDVH_Int2				statuscode = ERROR_200;
 		WDVCAPI_Depth			depth;
 		WDVCAPI_Overwrite		overwrite;
-		WDVH_Char				sDepth[10];
-		WDVH_Char				*pDepth;
-		WDVH_Char				sOverwrite[10];
+		WDVH_Char				*pDepth=NULL;
 		WDVH_Char				*pOverwrite;
 		WDVH_CapiUserData		capiUserData;
 		WDVCAPI_LockIdList		lockTokenList;
@@ -660,6 +779,18 @@
 			double     duration;
 		#endif
 		
+		if (wa==NULL)
+			return WDVH_False;
+		if (reply==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavCopyHandler:Uninitialized Pointers\n");
+			return WDVH_False;
+		}
+		if (request==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavCopyHandler:Uninitialized Pointers\n");
+			sendErrorReply(ERROR_500, reply, WEBDAV_COPY,"");
+			return WDVH_False;
+		}
+
 		/* extract Parameters (Host, RequestURI)*/
 		getFirstHeaderLine(wa,request,&host,&port,requestURI);
 		/* get server string for error messages */
@@ -667,13 +798,11 @@
 
 		/* interpret Header's */
 		/* - Depth */
-		pDepth = sDepth;
 		pDepth = (char*)sapdbwa_GetHeader(request,WDVH_DEPTH_HEADER);
 		if (pDepth) {
-			strcpy(sDepth,sapdbwa_GetHeader(request,WDVH_DEPTH_HEADER));
-			if (strcmp(sDepth,WDVH_DEPTH_0)==0)
+			if (strcmp(pDepth,WDVH_DEPTH_0)==0)
 				depth = WDVCAPI_DEPTH_0;
-			else if (strcmp(sDepth,WDVH_DEPTH_INFINITY)==0)
+			else if (strcmp(pDepth,WDVH_DEPTH_INFINITY)==0)
 				depth = WDVCAPI_DEPTH_INFINITE;
 			else {
 				sendErrorReply(ERROR_400, reply, WEBDAV_COPY,"");
@@ -683,13 +812,11 @@
 			depth = WDVCAPI_DEPTH_INFINITE;
 		
 		/* - Overwrite */
-		pOverwrite = sOverwrite;
 		pOverwrite = (char*)sapdbwa_GetHeader(request,WDVH_OVERWRITE_HEADER);
 		if (pOverwrite) {
-			strcpy(sOverwrite,sapdbwa_GetHeader(request,WDVH_OVERWRITE_HEADER));
-			if (strcmp(sOverwrite,WDVH_OVERWRITE_TRUE)==0) {
+			if (strcmp(pOverwrite,WDVH_OVERWRITE_TRUE)==0) {
 				overwrite = WDVCAPI_OVERWRITE;
-			} else if (strcmp(sOverwrite,WDVH_OVERWRITE_FALSE)==0) {
+			} else if (strcmp(pOverwrite,WDVH_OVERWRITE_FALSE)==0) {
 				overwrite = WDVCAPI_NO_OVERWRITE;
 			} else {
 				sendErrorReply(ERROR_400, reply, WEBDAV_COPY,"");
@@ -709,7 +836,8 @@
 			return WDVH_False;
 		} 
 		wd21_UnescapeUrl(destination,strlen(destination));
-		strcpy(destinationURL,destination);
+		strncpy(destinationURL,destination,WDVH_MAX_URI_LEN);
+		destinationURL[WDVH_MAX_URI_LEN]='\0';
 		destinationURI = destination;
 		if (strncmp(server,destinationURI,strlen(server))==0) {
 			destinationURI += strlen(server);
@@ -731,6 +859,12 @@
 		if (connection != NULL) {
 			/* get wdv handle */
 			pCapiHandle = getWdvHandle(wa,connection);
+			/* check the handles */
+			if (pCapiHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDV Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_COPY,"");
+				return WDVH_False;
+			}
 			/* - CapiUserData for call back */
 			createCapiUserData(&capiUserData);
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
@@ -827,9 +961,7 @@
 		WDVH_Int2				statuscode = ERROR_200;
 		WDVCAPI_Depth			depth;
 		WDVCAPI_Overwrite		overwrite;
-		WDVH_Char				sDepth[10];
-		WDVH_Char				*pDepth;
-		WDVH_Char				sOverwrite[10];
+		WDVH_Char				*pDepth=NULL;
 		WDVH_Char				*pOverwrite;
 		WDVH_CapiUserData		capiUserData;
 		WDVCAPI_LockIdList		lockTokenList;
@@ -841,6 +973,18 @@
 			double     duration;
 		#endif
 		
+		if (wa==NULL)
+			return WDVH_False;
+		if (reply==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavMoveHandler:Uninitialized Pointers\n");
+			return WDVH_False;
+		}
+		if (request==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavMoveHandler:Uninitialized Pointers\n");
+			sendErrorReply(ERROR_500, reply, WEBDAV_MOVE,"");
+			return WDVH_False;
+		}
+
 		/* extract Parameters (Host, RequestURI)*/
 		getFirstHeaderLine(wa,request,&host,&port,requestURI);
 		/* get server string for error messages */
@@ -848,11 +992,9 @@
 
 		/* interpret Header's */
 		/* - Depth */
-		pDepth = sDepth;
 		pDepth = (char*)sapdbwa_GetHeader(request,WDVH_DEPTH_HEADER);
 		if (pDepth) {
-			strcpy(sDepth,sapdbwa_GetHeader(request,WDVH_DEPTH_HEADER));
-			if (strcmp(sDepth,WDVH_DEPTH_INFINITY)==0)
+			if (strcmp(pDepth,WDVH_DEPTH_INFINITY)==0)
 				depth = WDVCAPI_DEPTH_INFINITE;
 			else {
 				sendErrorReply(ERROR_400, reply, WEBDAV_MOVE,"");
@@ -862,13 +1004,11 @@
 			depth = WDVCAPI_DEPTH_INFINITE;
 		
 		/* - Overwrite */
-		pOverwrite = sOverwrite;
 		pOverwrite = (char*)sapdbwa_GetHeader(request,WDVH_OVERWRITE_HEADER);
 		if (pOverwrite) {
-			strcpy(sOverwrite,sapdbwa_GetHeader(request,WDVH_OVERWRITE_HEADER));
-			if (strcmp(sOverwrite,WDVH_OVERWRITE_TRUE)==0) {
+			if (strcmp(pOverwrite,WDVH_OVERWRITE_TRUE)==0) {
 				overwrite = WDVCAPI_OVERWRITE;
-			} else if (strcmp(sOverwrite,WDVH_OVERWRITE_FALSE)==0) {
+			} else if (strcmp(pOverwrite,WDVH_OVERWRITE_FALSE)==0) {
 				overwrite = WDVCAPI_NO_OVERWRITE;
 			} else {
 				sendErrorReply(ERROR_400, reply, WEBDAV_MOVE,"");
@@ -888,7 +1028,8 @@
 			return WDVH_False;
 		} 
 		wd21_UnescapeUrl(destination,strlen(destination));
-		strcpy(destinationURL,destination);
+		strncpy(destinationURL,destination,WDVH_MAX_URI_LEN);
+		destinationURL[WDVH_MAX_URI_LEN]='\0';
 		destinationURI = destination;
 		if (strncmp(server,destinationURI,strlen(server))==0) {
 			destinationURI += strlen(server);
@@ -910,6 +1051,12 @@
 		if (connection != NULL) {
 			/* get wdv handle */
 			pCapiHandle = getWdvHandle(wa,connection);
+			/* check the handles */
+			if (pCapiHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDV Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_MOVE,"");
+				return WDVH_False;
+			}
 			/* - CapiUserData for call back */
 			createCapiUserData(&capiUserData);
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
@@ -1005,7 +1152,7 @@
 		WDVCAPI_XmlDocClassId	docClassId;
 		WDVH_Bool				compress;
 		WDVCAPI_ContentType		contentType;				/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1123371 */
-		WDVH_Char				putContentType[256];		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1123371 */
+		WDVCAPI_ContentTypeString	putContentType;			/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1123371 */
 		WDVH_Char				errormsg[1001];				/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1123371 */
 
 		#if defined(PROFILING_OVERALL) || defined(PROFILING_DETAIL)
@@ -1014,12 +1161,24 @@
 			double     duration;
 		#endif
 		
+		if (wa==NULL)
+			return;
+		if (reply==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavPostHandler:Uninitialized Pointers\n");
+			return;
+		}
+		if (request==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavPostHandler:Uninitialized Pointers\n");
+			sendErrorReply(ERROR_500, reply, WEBDAV_PUT,"");
+			return;
+		}
+
 		/* get request uri */
 		getRequestUri(wa,request,requestURI);
 
 		/* interpret Header's */
 /* ===> TODO!!!!! */
-		getIfHeader(request,sIf);
+		getIfHeader(request,sIf,100);
 		pIf = sIf;
 
 		/* build handles */
@@ -1032,6 +1191,17 @@
 			/* get wdv handle */
 			pCapiHandle = getWdvHandle(wa,connection);
 			wdvhHandle = getWdvhHandle(wa,connection);
+			/* check the handles */
+			if (pCapiHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDV Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_PUT,"");
+				return;
+			}
+			if (wdvhHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDVH Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_PUT,"");
+				return;
+			}
 			compress = getCompressFlag(wa,connection);
 			strcpy(errormsg,"");
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
@@ -1052,7 +1222,8 @@
 					strcpy(putContentType,"text/plain");
 				else {
 					/* check ContentType */
-					strcpy(putContentType,contentType);
+					strncpy(putContentType,contentType,WDVCAPI_MAX_CONTENT_TYPE_LEN);
+					putContentType[WDVCAPI_MAX_CONTENT_TYPE_LEN]='\0';
 					if (!WDVH_CheckContentType(putContentType)) {
 						/* something is wrong with it */
 						statuscode = ERROR_400;
@@ -1135,13 +1306,25 @@
 			double     duration;
 		#endif
 		
+		if (wa==NULL)
+			return;
+		if (reply==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavProppatchHandler:Uninitialized Pointers\n");
+			return;
+		}
+		if (request==NULL) {
+			sapdbwa_WriteLogMsg(wa, "webdavProppatchHandler:Uninitialized Pointers\n");
+			sendErrorReply(ERROR_500, reply, WEBDAV_PROPPATCH,"");
+			return;
+		}
+
 		/* extract Parameters (Host, RequestURI)*/
 		getFirstHeaderLine(wa,request,&host,&port,requestURI);
 		/* get server string for error messages */
 		buildServerString(request,&server);
 
 		/* interpret Header's */
-		getIfHeader(request,sIf);
+		getIfHeader(request,sIf,100);
 		pIf = sIf;
 
 		/* Build Handles */
@@ -1155,6 +1338,12 @@
 		if (connection != NULL) {
 			/* get wdv handle */
 			pCapiHandle = getWdvHandle(wa,connection);
+			/* check the handles */
+			if (pCapiHandle==NULL) {
+				sapdbwa_WriteLogMsg(wa, "Got no valid WDV Handle");
+				sendErrorReply(ERROR_500, reply, WEBDAV_PROPPATCH,"");
+				return;
+			}
 			#if defined(PROFILING_OVERALL) && defined(PROFILING_DETAIL)
 				time2 = clock();
 				duration = ((time2-time1)*1000) / CLOCKS_PER_SEC;
