@@ -38,6 +38,7 @@
 
 	
 	#include "SAPDB/WebDAV/Handler/WDVHandler_LockHandler.h"
+	#include "SAPDBCommon/SAPDB_string.h"
 
 
 	/*------------------------------------------------------------------------
@@ -60,6 +61,13 @@
 		WDVH_Char				errorText[1000];
 		WDVH_Char				*server;
 
+		if (wa==NULL)
+			return ERROR_500;
+		if (request==NULL || pCapiHandle==NULL || requesturi==NULL || userData==NULL || errmsg==NULL) {
+			sapdbwa_WriteLogMsg(wa, "lockCallCapiFunc:Uninitialized Pointers\n");
+			return ERROR_500;
+		}
+
 		/* get server string for error messages */
 		buildServerString(request,&server);
 		
@@ -156,6 +164,12 @@
 
 		WDVH_Char		timeout[50];
 		WDVH_Char		locktoken[100];
+		WDVH_Char		contentlength[10];
+
+		if (reply==NULL || requesturi==NULL || userData==NULL || errmsg==NULL) {
+			sendErrorReply(ERROR_500, reply, WEBDAV_LOCK,"");
+			return;
+		}
 
 		/* if there are errors at the subordinated resources a header was asent allready. 
 		   in this case only send the footer of the body.
@@ -178,132 +192,108 @@
 										"Lock-Token",
 										locktoken );
 
+						sapdbwa_SetHeader( reply,
+							"Transfer-Encoding",
+							"chunked" );
+
 						sapdbwa_SendHeader(reply);
 
 						/* send lockdiscovery property as body for the reply */
 						/* applies to successfull LOCK and lock refresh */
 						/* XML Header */
-						sapdbwa_SendBody(reply,
-										 WDVH_XML_HEADER,
-										 strlen(WDVH_XML_HEADER));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_XML_HEADER);
 						/* <D:prop xmlns:D="DAV:"> */
-						sapdbwa_SendBody(reply,
-										 WDVH_PROP_OPEN_XMLNS,
-										 strlen(WDVH_PROP_OPEN_XMLNS));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_PROP_OPEN_XMLNS);
 						/*		<D:lockdiscovery> */
-						sapdbwa_SendBody(reply,
-										 WDVH_LOCKDISCOVERY_OPEN,
-										 strlen(WDVH_LOCKDISCOVERY_OPEN));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_LOCKDISCOVERY_OPEN);
 						/*			<D:activelock> */
-						sapdbwa_SendBody(reply,
-										 WDVH_ACTIVELOCK_OPEN,
-										 strlen(WDVH_ACTIVELOCK_OPEN));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_ACTIVELOCK_OPEN);
 						/*				<D:locktype> */
-						sapdbwa_SendBody(reply,
-										 WDVH_LOCKTYPE_OPEN,
-										 strlen(WDVH_LOCKTYPE_OPEN));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_LOCKTYPE_OPEN);
 						/*					value (get locktype from CAPI) */
 						if (userData->lockType == WDVH_LOCKTYPE_WRITE)
-							sapdbwa_SendBody(reply,
-											 WDVH_WRITE_EMPTY,
-											 strlen(WDVH_WRITE_EMPTY));
+							WDVH_sendBodyChunk(reply,
+											 WDVH_WRITE_EMPTY);
 						/*				</D:locktype> */
-						sapdbwa_SendBody(reply,
-										 WDVH_LOCKTYPE_CLOSE,
-										 strlen(WDVH_LOCKTYPE_CLOSE));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_LOCKTYPE_CLOSE);
 						/*				<D:lockscope> */
-						sapdbwa_SendBody(reply,
-										 WDVH_LOCKSCOPE_OPEN,
-										 strlen(WDVH_LOCKSCOPE_OPEN));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_LOCKSCOPE_OPEN);
 						/*					value (get lockscope from CAPI) */
 						if (userData->lockScope == WDVCAPI_LOCK_SCOPE_EXCLUSIVE)
-							sapdbwa_SendBody(reply,
-											 WDVH_EXCLUSIVE_EMPTY,
-											 strlen(WDVH_EXCLUSIVE_EMPTY));
+							WDVH_sendBodyChunk(reply,
+											 WDVH_EXCLUSIVE_EMPTY);
 						else if (userData->lockScope == WDVH_LOCKSCOPE_SHARED)
-							sapdbwa_SendBody(reply,
-											 WDVH_SHARED_EMPTY,
-											 strlen(WDVH_SHARED_EMPTY));
+							WDVH_sendBodyChunk(reply,
+											 WDVH_SHARED_EMPTY);
 						/*				</D:lockscope> */
-						sapdbwa_SendBody(reply,
-										 WDVH_LOCKSCOPE_CLOSE,
-										 strlen(WDVH_LOCKSCOPE_CLOSE));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_LOCKSCOPE_CLOSE);
 						/*				<D:depth> */
-						sapdbwa_SendBody(reply,
-										 WDVH_DEPTH_OPEN,
-										 strlen(WDVH_DEPTH_OPEN));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_DEPTH_OPEN);
 						/*					value */
 						if (userData->depth == WDVCAPI_DEPTH_0)
-							sapdbwa_SendBody(reply,
-											 WDVH_DEPTH_0,
-											 strlen(WDVH_DEPTH_0));
+							WDVH_sendBodyChunk(reply,
+											 WDVH_DEPTH_0);
 						else if (userData->depth == WDVCAPI_DEPTH_INFINITE)
-							sapdbwa_SendBody(reply,
-											 WDVH_DEPTH_INFINITY,
-											 strlen(WDVH_DEPTH_INFINITY));
+							WDVH_sendBodyChunk(reply,
+											 WDVH_DEPTH_INFINITY);
 						/*				</D:depth> */
-						sapdbwa_SendBody(reply,
-										 WDVH_DEPTH_CLOSE,
-										 strlen(WDVH_DEPTH_CLOSE));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_DEPTH_CLOSE);
 						/*				<D:owner> */
-						sapdbwa_SendBody(reply,
-										 WDVH_OWNER_OPEN,
-										 strlen(WDVH_OWNER_OPEN));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_OWNER_OPEN);
 						/*					value */
-						sapdbwa_SendBody(reply,
-										 userData->lockOwner,
-										 strlen(userData->lockOwner));
+						WDVH_sendBodyChunk(reply,
+										 userData->lockOwner);
 						/*				</D:owner> */
-						sapdbwa_SendBody(reply,
-										 WDVH_OWNER_CLOSE,
-										 strlen(WDVH_OWNER_CLOSE));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_OWNER_CLOSE);
 						/*				<D:timeout> */
-						sapdbwa_SendBody(reply,
-										 WDVH_TIMEOUT_OPEN,
-										 strlen(WDVH_TIMEOUT_OPEN));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_TIMEOUT_OPEN);
 						/*					value */
 						/* if (userData->timeout == */
 						sp77sprintf(timeout,50,"Second-%d",userData->timeout);
-						sapdbwa_SendBody(reply,
-										 timeout,
-										 strlen(timeout));
+						WDVH_sendBodyChunk(reply,
+										 timeout);
 						/*				</D:timeout> */
-						sapdbwa_SendBody(reply,
-										 WDVH_TIMEOUT_CLOSE,
-										 strlen(WDVH_TIMEOUT_CLOSE));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_TIMEOUT_CLOSE);
 						/*				<D:locktoken> */
-						sapdbwa_SendBody(reply,
-										 WDVH_LOCKTOKEN_OPEN,
-										 strlen(WDVH_LOCKTOKEN_OPEN));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_LOCKTOKEN_OPEN);
 						/*					<D:href> */
-						sapdbwa_SendBody(reply,
-										 WDVH_HREF_OPEN,
-										 strlen(WDVH_HREF_OPEN));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_HREF_OPEN);
 						/*						value */
 						sp77sprintf(locktoken,100,"saplocktoken:%s",userData->lockId);
-						sapdbwa_SendBody(reply,
-										 locktoken,
-										 strlen(locktoken));
+						WDVH_sendBodyChunk(reply,
+										 locktoken);
 						/*					</D:href> */
-						sapdbwa_SendBody(reply,
-										 WDVH_HREF_CLOSE,
-										 strlen(WDVH_HREF_CLOSE));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_HREF_CLOSE);
 						/*				</D:locktoken> */
-						sapdbwa_SendBody(reply,
-										 WDVH_LOCKTOKEN_CLOSE,
-										 strlen(WDVH_LOCKTOKEN_CLOSE));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_LOCKTOKEN_CLOSE);
 						/*			</D:activelock> */
-						sapdbwa_SendBody(reply,
-										 WDVH_ACTIVELOCK_CLOSE,
-										 strlen(WDVH_ACTIVELOCK_CLOSE));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_ACTIVELOCK_CLOSE);
 						/*		</D:lockdiscovery> */
-						sapdbwa_SendBody(reply,
-										 WDVH_LOCKDISCOVERY_CLOSE,
-										 strlen(WDVH_LOCKDISCOVERY_CLOSE));
+						WDVH_sendBodyChunk(reply,
+										 WDVH_LOCKDISCOVERY_CLOSE);
 						/*	</D:prop> */
-						sapdbwa_SendBody(reply,
-										 WDVH_PROP_CLOSE,
-										 strlen(WDVH_PROP_CLOSE));
+						WDVH_sendBodyChunk(reply,WDVH_PROP_CLOSE);
+						/* Send empty chunk as eof */
+						WDVH_sendBodyChunkEnd(reply);
 						break;
 					}
 				case ERROR_400: 
@@ -320,9 +310,10 @@
 										   "DAV",
 										   "1" );
 
+						sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_400));
 						sapdbwa_SetHeader( reply,
-										   "Connection",
-										   "Keep-Alive" );
+										"Content-Length",
+										contentlength);
 
 						sapdbwa_SendHeader(reply);
 						
@@ -345,9 +336,10 @@
 										   "DAV",
 										   "1" );
 
+						sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_409));
 						sapdbwa_SetHeader( reply,
-										   "Connection",
-										   "Keep-Alive" );
+										"Content-Length",
+										contentlength);
 
 						sapdbwa_SendHeader(reply);
 						
@@ -370,9 +362,10 @@
 										   "DAV",
 										   "1" );
 
+						sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_409_OWN)+strlen(errmsg)+strlen(ERROR_PAGE_FOOTER));
 						sapdbwa_SetHeader( reply,
-										   "Connection",
-										   "Keep-Alive" );
+										"Content-Length",
+										contentlength);
 
 						sapdbwa_SendHeader(reply);
 						
@@ -401,9 +394,10 @@
 										   "DAV",
 										   "1" );
 
+						sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_403));
 						sapdbwa_SetHeader( reply,
-										   "Connection",
-										   "Keep-Alive" );
+										"Content-Length",
+										contentlength);
 
 						sapdbwa_SendHeader(reply);
 						
@@ -432,9 +426,10 @@
 										   "DAV",
 										   "1" );
 
+						sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_405));
 						sapdbwa_SetHeader( reply,
-										   "Connection",
-										   "Keep-Alive" );
+										"Content-Length",
+										contentlength);
 
 						sapdbwa_SendHeader(reply);
 						
@@ -443,6 +438,32 @@
 										 strlen(ERROR_PAGE_405));
 						break; 
 					}
+				case ERROR_423: 
+					{
+						sapdbwa_InitHeader(reply,
+										   statuscode,
+										   "text/html",
+										   NULL,
+										   NULL,					/* const char    *LastModified,*/
+										   NULL,					/* const char    *Expires,*/
+										   NULL);					/* const char    *Location );*/
+
+						sapdbwa_SetHeader( reply,
+										   "DAV",
+										   "1" );
+ 
+						sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_423));
+						sapdbwa_SetHeader( reply,
+										"Content-Length",
+										contentlength);
+
+						sapdbwa_SendHeader(reply);
+						
+						sapdbwa_SendBody(reply,
+										 ERROR_PAGE_423,
+										 strlen(ERROR_PAGE_423));
+						break; 
+					}
 				case ERROR_500:
 					{
 						sapdbwa_InitHeader(reply,
@@ -457,9 +478,10 @@
 										   "DAV",
 										   "1" );
 
+						sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_500));
 						sapdbwa_SetHeader( reply,
-										   "Connection",
-										   "Keep-Alive" );
+										"Content-Length",
+										contentlength);
 
 						sapdbwa_SendHeader(reply);
 						
@@ -473,9 +495,8 @@
 			/* there is at least one error occured on an resource different from the root 
 			   resource. In this case the error call back function sent the reply. Only the
 			   closing tag for multistatus is missing now */
-			sapdbwa_SendBody(reply,
-							 WDVH_MULTI_STATUS_CLOSE,
-							 strlen(WDVH_MULTI_STATUS_CLOSE));
+			WDVH_sendBodyChunk(reply,WDVH_MULTI_STATUS_CLOSE);
+			WDVH_sendBodyChunkEnd(reply);
 		}
 	}
 
@@ -509,6 +530,13 @@
 		XML_Parser					p;
 		WDVH_Char					*server;
 
+		if (wa==NULL)
+			return ERROR_500;
+		if (wdv==NULL || request==NULL || capiUserData==NULL || errmsg==NULL) {
+			sapdbwa_WriteLogMsg(wa, "lockParseRequestBody:Uninitialized Pointers\n");
+			return ERROR_500;
+		}
+
 		/* get server string for error messages */
 		buildServerString(request,&server);
 		
@@ -601,7 +629,7 @@
 				}
 				if (newUserData->propertyShortValueStarted) {															/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124043 */
 					if (newUserData->propertyValueEndPos != WDVH_MAX_XML_BUFFER_LEN)									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124043 */
-						memcpy(currentBuffer+newUserData->propertyValueEndPos,"\0",1);									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124043 */
+						SAPDB_memcpy(currentBuffer+newUserData->propertyValueEndPos,"\0",1);									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124043 */
 					if (newUserData->propertyValueEndPos+newUserData->bufferStart < newUserData->bufferEnd)				/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124043 */
 						newUserData->propertyShortValueStarted = WDVH_False;											/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124043 */
 					if ((getStrBufferLength(newUserData->propertyShortValue) +
@@ -665,6 +693,9 @@
 		WDVH_XmlUserData				userData;
 		XML_Parser						p;
 
+		if (data==NULL || el==NULL || attr==NULL) 
+			return;
+
 		userData = (WDVH_XmlUserData)data;
 		p = xmlParserUserDataGetParser(userData);
 		parserState = xmlParserUserDataGetLockParserState(userData);
@@ -688,9 +719,9 @@
 				/* here comes a lockscope value tag (<exclusive/> or <shared/>) */
 				if (strcmp(tag,WDVH_XML_EXCLUSIVE_OPEN)==0) {
 					/* Correction Begin http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1122129 */
-					/*if (xmlParserCheckEmptyTag(p,userData,WDVH_EMPTY_TAG_EXCLUSIVE)) {
+					/*if (xmlParserCheckEmptyTag(p,userData,WDVH_EMPTY_TAG_EXCLUSIVE)) {*/
 						/* do something about the exclusive lock scope */
-						/*userData->lockscope = WDVCAPI_LOCK_SCOPE_EXCLUSIVE;
+						/*userData->lockscope = WDVCAPI_LOCK_SCOPE_EXCLUSIVE;*/
 						/* and go to next parser state*/
 						/*parserState->state = WDVH_LOCK_DFA_STATE_7;*/
 						parserState->state = WDVH_LOCK_DFA_STATE_13;
@@ -701,9 +732,9 @@
 					/* Correction End http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1122129 */
 				} else if (strcmp(tag,WDVH_XML_SHARED_OPEN)==0) {
 					/* Correction Begin http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1122129 */
-					/*if (xmlParserCheckEmptyTag(p,userData,WDVH_EMPTY_TAG_SHARED)) {
+					/*if (xmlParserCheckEmptyTag(p,userData,WDVH_EMPTY_TAG_SHARED)) {*/
 						/* do something about the shared lock scope */
-						/*userData->lockscope = WDVCAPI_LOCK_SCOPE_UNDEFINED;
+						/*userData->lockscope = WDVCAPI_LOCK_SCOPE_UNDEFINED;*/
 						/* and go to next parser state*/
 						/*parserState->state = WDVH_LOCK_DFA_STATE_7;*/
 						parserState->state = WDVH_LOCK_DFA_STATE_14;
@@ -725,12 +756,12 @@
 					/*if (xmlParserCheckEmptyTag(p,userData,WDVH_EMPTY_TAG_WRITE)) {
 						userData->locktype = WDVCAPI_LOCK_TYPE_WRITE;
 						parserState->state = WDVH_LOCK_DFA_STATE_4;
-					} else {
+					} else {*/
 						/*parserState->error = WDVH_XML_PARSER_SYNTAX_ERROR;*/
 						userData->locktype = WDVCAPI_LOCK_TYPE_WRITE;
 						parserState->state = WDVH_LOCK_DFA_STATE_12;
 						parserState->depth++;
-					/*}
+					/*}*/
 					/* Correction End http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1122129 */
 				} else {
 					parserState->error = WDVH_XML_PARSER_SYNTAX_ERROR;
@@ -845,6 +876,9 @@
 		WDVH_XmlLockParserState		parserState;
 		WDVH_XmlUserData			userData;
 
+		if (data==NULL || el==NULL)
+			return;
+
 		userData = (WDVH_XmlUserData)data;
 		parserState = xmlParserUserDataGetLockParserState(userData);
 
@@ -1025,21 +1059,21 @@
 /*				if (data->propertyValueStartPos >= startPosPrevBuff) {
 */					/* read til property value start pos */
 /*					if(-pos3+pos1-1 > WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {
-						memcpy(data->lockShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
-						memcpy(data->lockShortValue-pos3+pos1,"\0",1);
+						SAPDB_memcpy(data->lockShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
+						SAPDB_memcpy(data->lockShortValue-pos3+pos1,"\0",1);
 					} else {
-						memcpy(data->lockShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3+pos1-1);
-						memcpy(data->lockShortValue-pos3+pos1,"\0",1);
+						SAPDB_memcpy(data->lockShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3+pos1-1);
+						SAPDB_memcpy(data->lockShortValue-pos3+pos1,"\0",1);
 					}
 				} else {
 */					/* read til buffer start */
 /*					pos2 = pos0 - ((data->bufferChunks - 2)* WDVH_MAX_XML_BUFFER_LEN);
 					if (pos2 > WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {
-						memcpy(data->lockShortValue,data->previousBuff,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
-						memcpy(data->lockShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN-1,"\0",1);
+						SAPDB_memcpy(data->lockShortValue,data->previousBuff,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
+						SAPDB_memcpy(data->lockShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN-1,"\0",1);
 					} else {
-						memcpy(data->lockShortValue,data->previousBuff,pos2);
-						memcpy(data->lockShortValue+pos2,"\0",1);
+						SAPDB_memcpy(data->lockShortValue,data->previousBuff,pos2);
+						SAPDB_memcpy(data->lockShortValue+pos2,"\0",1);
 					}
 				}
 			} else {
@@ -1054,31 +1088,31 @@
 /*				if (data->propertyValueStartPos >= startPosCurrBuff) {
 */					/* read til property value start pos */
 /*					if (pos1-pos3 > WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {
-						memcpy(data->lockShortValue,data->currentBuff+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
-						memcpy(data->lockShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN-1,"\0",1);
+						SAPDB_memcpy(data->lockShortValue,data->currentBuff+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
+						SAPDB_memcpy(data->lockShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN-1,"\0",1);
 					} else {
-						memcpy(data->lockShortValue,data->currentBuff+pos3+1,pos1-pos3);
-						memcpy(data->lockShortValue+pos1-pos3,"\0",1);
+						SAPDB_memcpy(data->lockShortValue,data->currentBuff+pos3+1,pos1-pos3);
+						SAPDB_memcpy(data->lockShortValue+pos1-pos3,"\0",1);
 					}
 				} else {
 */					/* read first part of property value from previous buffer */
 /*					if (-pos3+pos1 > WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {
 						if (-pos3 > WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {
-							memcpy(data->lockShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
-							memcpy(data->lockShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN-1,"\0",1);
+							SAPDB_memcpy(data->lockShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
+							SAPDB_memcpy(data->lockShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN-1,"\0",1);
 						} else {
-							memcpy(data->lockShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3);
+							SAPDB_memcpy(data->lockShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3);
 */							/* read til buffer start */
-/*							memcpy(data->lockShortValue-pos3-1,
+/*							SAPDB_memcpy(data->lockShortValue-pos3-1,
 								   data->currentBuff,
 								   WDVH_MAX_PROPERTY_SHORT_VALUE_LEN+pos3);
-							memcpy(data->lockShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN-1,"\0",1);
+							SAPDB_memcpy(data->lockShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN-1,"\0",1);
 						}
 					} else {
-						memcpy(data->lockShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3);
+						SAPDB_memcpy(data->lockShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3);
 */						/* read til buffer start */
-/*						memcpy(data->lockShortValue-pos3-1,data->currentBuff,pos1);
-						memcpy(data->lockShortValue+pos1-pos3,"\0",1);
+/*						SAPDB_memcpy(data->lockShortValue-pos3-1,data->currentBuff,pos1);
+						SAPDB_memcpy(data->lockShortValue+pos1-pos3,"\0",1);
 					}
 				}
 			}
@@ -1095,6 +1129,9 @@
 		WDVH_Int4			pos0,pos1;
 		XML_Parser			p;
 
+		if (data==NULL || el==NULL)
+			return;
+
 		p = xmlParserUserDataGetParser(data);
 		/* called from start tag handler after discovery of a owner start tag */
 		/* check wether end tag is in current parse buffer */
@@ -1146,6 +1183,9 @@
 		WDVH_Int4			pos0;
 		XML_Parser			p;
 
+		if (data==NULL || el==NULL)
+			return;
+
 		p = xmlParserUserDataGetParser(data);
 		/* called from start tag handler after discovery of a owner start tag */
 		/* check wether end tag is in current parse buffer */
@@ -1172,6 +1212,10 @@
 		WDVCAPI_ErrorText		errorMsg;
 		WDVH_Error				error;
 		WDVH_Char				errorText[1000];
+		WDVH_Char				uri_enc[WDVH_MAX_URI_LEN+1];
+
+		if (clientData==NULL || uri==NULL || errorItem==NULL)
+			return;
 
 		userData = (WDVH_CapiUserData)clientData;
 
@@ -1190,19 +1234,15 @@
 			                   "1" );
 
 			sapdbwa_SetHeader( userData->reply,
-		                       "Connection",
-			                   "Keep-Alive" );
+				   "Transfer-Encoding",
+				   "chunked" );
 
 			sapdbwa_SendHeader(userData->reply);
 			
 			/* and the beginning of the xml body has to be sent too */
 			/* send body header */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_XML_HEADER,
-							 strlen(WDVH_XML_HEADER));
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_MULTI_STATUS_OPEN,
-							 strlen(WDVH_MULTI_STATUS_OPEN));
+			WDVH_sendBodyChunk(userData->reply,WDVH_XML_HEADER);
+			WDVH_sendBodyChunk(userData->reply,WDVH_MULTI_STATUS_OPEN);
 			
 
 			/* and error occured has to be set */
@@ -1213,36 +1253,23 @@
 			/* it's the second, third, ... turn around */
 			/* send the response part for this special uri */
 			/* response */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_RESPONSE_OPEN,
-							 strlen(WDVH_RESPONSE_OPEN));
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_XML_CLOSE_TAG,
-							 strlen(WDVH_XML_CLOSE_TAG));
+			WDVH_sendBodyChunk(userData->reply,WDVH_RESPONSE_OPEN);
+			WDVH_sendBodyChunk(userData->reply,WDVH_XML_CLOSE_TAG);
 			/*		href */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_HREF_OPEN,
-							 strlen(WDVH_HREF_OPEN));
+			WDVH_sendBodyChunk(userData->reply,WDVH_HREF_OPEN);
 			/*			server */
-			sapdbwa_SendBody(userData->reply,
-							 userData->server,
-							 strlen(userData->server));
+			WDVH_sendBodyChunk(userData->reply,userData->server);
 			/*			slash */
-			sapdbwa_SendBody(userData->reply, "/", 0);
+			WDVH_sendBodyChunk(userData->reply, "/");
 			/*			prefix */
-			sapdbwa_SendBody(userData->reply, userData->prefix, 0);
+			WDVH_sendBodyChunk(userData->reply, (WDVH_Char*)userData->prefix);
 			/*          + uri */
-			sapdbwa_SendBody(userData->reply,
-							 uri,
-							 strlen(uri));
+			URLencode(uri,uri_enc,WDVH_MAX_URI_LEN);
+			WDVH_sendBodyChunk(userData->reply,uri_enc);
 			/*		/href */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_HREF_CLOSE,
-							 strlen(WDVH_HREF_CLOSE));
+			WDVH_sendBodyChunk(userData->reply,WDVH_HREF_CLOSE);
 			/*		status */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_STATUS_OPEN,
-							 strlen(WDVH_STATUS_OPEN));
+			WDVH_sendBodyChunk(userData->reply,WDVH_STATUS_OPEN);
 
 			/* get the CAPI error code and translate it to the appropriate http status code */
 			WDVCAPI_GetErrorType(errorItem,&errorType);
@@ -1251,15 +1278,9 @@
 				case WDVCAPI_ERR_TYPE_SQL: 
 					{
 						/* internal server error */
-						sapdbwa_SendBody(userData->reply,
-										 WDVH_HTTP_1_1,
-										 strlen(WDVH_HTTP_1_1));
-						sapdbwa_SendBody(userData->reply,
-										 ERROR_500_STR,
-										 strlen(ERROR_500_STR));
-						sapdbwa_SendBody(userData->reply,
-										 ERROR_500_TEXT,
-										 strlen(ERROR_500_TEXT));
+						WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+						WDVH_sendBodyChunk(userData->reply,ERROR_500_STR);
+						WDVH_sendBodyChunk(userData->reply,ERROR_500_TEXT);
 						break;
 					}
 				case WDVCAPI_ERR_TYPE_CAPI: 
@@ -1268,29 +1289,17 @@
 						switch(errorCode) {
 							case WDVCAPI_ERR_CODE_LOCK_EXISTS:
 								{
-									sapdbwa_SendBody(userData->reply,
-													 WDVH_HTTP_1_1,
-													 strlen(WDVH_HTTP_1_1));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_423_STR,
-													 strlen(ERROR_423_STR));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_423_TEXT,
-													 strlen(ERROR_423_TEXT));
+									WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+									WDVH_sendBodyChunk(userData->reply,ERROR_423_STR);
+									WDVH_sendBodyChunk(userData->reply,ERROR_423_TEXT);
 									break;
 								}
 
 							default:
 								{
-									sapdbwa_SendBody(userData->reply,
-													 WDVH_HTTP_1_1,
-													 strlen(WDVH_HTTP_1_1));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_500_STR,
-													 strlen(ERROR_500_STR));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_500_TEXT,
-													 strlen(ERROR_500_TEXT));
+									WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+									WDVH_sendBodyChunk(userData->reply,ERROR_500_STR);
+									WDVH_sendBodyChunk(userData->reply,ERROR_500_TEXT);
 									break;
 								}
 						}
@@ -1298,26 +1307,16 @@
 					}
 				default:
 					{
-						sapdbwa_SendBody(userData->reply,
-										 WDVH_HTTP_1_1,
-										 strlen(WDVH_HTTP_1_1));
-						sapdbwa_SendBody(userData->reply,
-										 ERROR_500_STR,
-										 strlen(ERROR_500_STR));
-						sapdbwa_SendBody(userData->reply,
-										 ERROR_500_TEXT,
-										 strlen(ERROR_500_TEXT));
+						WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+						WDVH_sendBodyChunk(userData->reply,ERROR_500_STR);
+						WDVH_sendBodyChunk(userData->reply,ERROR_500_TEXT);
 						break;
 					}
 			}
 			/*			/status */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_STATUS_CLOSE,
-							 strlen(WDVH_STATUS_CLOSE));
+			WDVH_sendBodyChunk(userData->reply,WDVH_STATUS_CLOSE);
 			/*	/response */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_RESPONSE_CLOSE,
-							 strlen(WDVH_RESPONSE_CLOSE));
+			WDVH_sendBodyChunk(userData->reply,WDVH_RESPONSE_CLOSE);
 		}
 
 	}
