@@ -65,6 +65,13 @@
 		WDVH_Char				*server;
 		WDVH_Bool				resourceCreated = WDVH_False;
 
+		if (wa==NULL)
+			return ERROR_500;
+		if (request==NULL || pCapiHandle==NULL || sourceUri==NULL || destinationUri==NULL || userData==NULL) {
+			sapdbwa_WriteLogMsg(wa, "moveCallCapiFunc:Uninitialized Pointers\n");
+			return ERROR_500;
+		}
+
 		/* get server string for error messages */
 		buildServerString(request,&server);
 		
@@ -108,7 +115,7 @@
 									sp77sprintf(errorText,1000,ERROR_MOVE_FORBIDDEN_403,server,(char*)sapdbwa_GetRequestURI(request),(char*)sapdbwa_GetHeader(request,WDVH_DESTINATION_HEADER));
 									sapdbwa_WriteLogMsg(wa, errorText);
 									sp77sprintf(errorText,1000,ERROR_MOVE_MESSAGE,errorMsg);
-									sapdbwa_WriteLogMsg(wa, errorText);
+									sapdbwa_WriteLogMsg(wa, errorText);*/
 									/* Correction End http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1121446 */
 									break;
 								}
@@ -121,7 +128,7 @@
 									sp77sprintf(errorText,1000,ERROR_MOVE_NOTFOUND_404,server,(char*)sapdbwa_GetRequestURI(request),(char*)sapdbwa_GetHeader(request,WDVH_DESTINATION_HEADER));
 									sapdbwa_WriteLogMsg(wa, errorText);
 									sp77sprintf(errorText,1000,ERROR_MOVE_MESSAGE,errorMsg);
-									sapdbwa_WriteLogMsg(wa, errorText);
+									sapdbwa_WriteLogMsg(wa, errorText);*/
 									/* Correction End http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1121446 */
 									break;
 								}
@@ -137,7 +144,7 @@
 									sp77sprintf(errorText,1000,ERROR_MOVE_PARENT_409,server,(char*)sapdbwa_GetRequestURI(request),(char*)sapdbwa_GetHeader(request,WDVH_DESTINATION_HEADER));
 									sapdbwa_WriteLogMsg(wa, errorText);
 									sp77sprintf(errorText,1000,ERROR_MOVE_MESSAGE,errorMsg);
-									sapdbwa_WriteLogMsg(wa, errorText);
+									sapdbwa_WriteLogMsg(wa, errorText);*/
 									/* Correction End http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1121446 */
 									break;
 								}
@@ -152,7 +159,7 @@
 									sp77sprintf(errorText,1000,ERROR_MOVE_CHILD_409,server,(char*)sapdbwa_GetRequestURI(request),(char*)sapdbwa_GetHeader(request,WDVH_DESTINATION_HEADER));
 									sapdbwa_WriteLogMsg(wa, errorText);
 									sp77sprintf(errorText,1000,ERROR_MOVE_MESSAGE,errorMsg);
-									sapdbwa_WriteLogMsg(wa, errorText);
+									sapdbwa_WriteLogMsg(wa, errorText);*/
 									/* Correction End http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1121446 */
 									break;
 								}
@@ -180,7 +187,7 @@
 									sp77sprintf(errorText,1000,ERROR_MOVE_PRECONDITION_412,server,(char*)sapdbwa_GetRequestURI(request),(char*)sapdbwa_GetHeader(request,WDVH_DESTINATION_HEADER));
 									sapdbwa_WriteLogMsg(wa, errorText);
 									sp77sprintf(errorText,1000,ERROR_MOVE_MESSAGE,errorMsg);
-									sapdbwa_WriteLogMsg(wa, errorText);
+									sapdbwa_WriteLogMsg(wa, errorText);*/
 									/* Correction End http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1121446 */
 									break;
 								}
@@ -193,7 +200,7 @@
 									sp77sprintf(errorText,1000,ERROR_MOVE_LOCKED_423,server,(char*)sapdbwa_GetRequestURI(request),(char*)sapdbwa_GetHeader(request,WDVH_DESTINATION_HEADER));
 									sapdbwa_WriteLogMsg(wa, errorText);
 									sp77sprintf(errorText,1000,ERROR_MOVE_MESSAGE,errorMsg);
-									sapdbwa_WriteLogMsg(wa, errorText);
+									sapdbwa_WriteLogMsg(wa, errorText);*/
 									/* Correction End http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1121446 */
 									break;
 								}
@@ -281,6 +288,10 @@
 		sapdbwa_Bool allocatOK;
 		sapdbwa_Int4 length;
 
+		if (reply==NULL || pCapiHandle==NULL || resourceURI==NULL || userData==NULL) {
+			return;
+		}
+
 		/* if there are errors at the subordinated resources a header was asent allready. 
 		   in this case only send the footer of the body.
 		   otherwise the error occured at the root resource and no body has to be sent */
@@ -295,15 +306,17 @@
 						       NULL,					/* const char    *LastModified,*/
 							   NULL,					/* const char    *Expires,*/
 							   NULL);					/* const char    *Location );*/
+			sapdbwa_SetHeader( reply,
+							"Content-Length",
+							"0");
 
 			sapdbwa_SendHeader(reply);
 		} else {
 			/* there is at least one error occured on an resource different from the root 
 			   resource. In this case the error call back function sent the reply. Only the
 			   closing tag for multistatus is missing now */
-			sapdbwa_SendBody(reply,
-							 WDVH_MULTI_STATUS_CLOSE,
-							 strlen(WDVH_MULTI_STATUS_CLOSE));
+			WDVH_sendBodyChunk(reply,WDVH_MULTI_STATUS_CLOSE);
+			WDVH_sendBodyChunkEnd(reply);
 		}
 
 	}
@@ -323,6 +336,11 @@
 		WDVCAPI_ErrorText		errorMsg;
 		WDVH_Error				error;
 		WDVH_Char				errorText[1000];
+		WDVH_Char				uri_enc[WDVH_MAX_URI_LEN+1];
+
+		if (clientData==NULL || uri==NULL || errorItem==NULL) {
+			return;
+		}
 
 		userData = (WDVH_CapiUserData)clientData;
 
@@ -336,20 +354,16 @@
 							   NULL,					/* const char    *Expires,*/
 							   NULL);					/* const char    *Location );*/
 
-/*			sapdbwa_SetHeader( userData->reply,
+			sapdbwa_SetHeader( userData->reply,
 				   "Transfer-Encoding",
 				   "chunked" );
-*/
+
 			sapdbwa_SendHeader(userData->reply);
 			
 			/* and the beginning of the xml body has to be sent too */
 			/* send body header */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_XML_HEADER,
-							 strlen(WDVH_XML_HEADER));
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_MULTI_STATUS_OPEN,
-							 strlen(WDVH_MULTI_STATUS_OPEN));
+			WDVH_sendBodyChunk(userData->reply,WDVH_XML_HEADER);
+			WDVH_sendBodyChunk(userData->reply,WDVH_MULTI_STATUS_OPEN);
 			
 
 			/* and error occured has to be set */
@@ -360,32 +374,19 @@
 			/* it's the second, third, ... turn around */
 			/* send the response part for this special uri */
 			/* response */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_RESPONSE_OPEN,
-							 strlen(WDVH_RESPONSE_OPEN));
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_XML_CLOSE_TAG,
-							 strlen(WDVH_XML_CLOSE_TAG));
+			WDVH_sendBodyChunk(userData->reply,WDVH_RESPONSE_OPEN);
+			WDVH_sendBodyChunk(userData->reply,WDVH_XML_CLOSE_TAG);
 			/*		href */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_HREF_OPEN,
-							 strlen(WDVH_HREF_OPEN));
+			WDVH_sendBodyChunk(userData->reply,WDVH_HREF_OPEN);
 			/*			server */
-			sapdbwa_SendBody(userData->reply,
-							 userData->server,
-							 strlen(userData->server));
+			WDVH_sendBodyChunk(userData->reply,userData->server);
 			/*          + uri */
-			sapdbwa_SendBody(userData->reply,
-							 uri,
-							 strlen(uri));
+			URLencode(uri,uri_enc,WDVH_MAX_URI_LEN);
+			WDVH_sendBodyChunk(userData->reply,uri_enc);
 			/*		/href */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_HREF_CLOSE,
-							 strlen(WDVH_HREF_CLOSE));
+			WDVH_sendBodyChunk(userData->reply,WDVH_HREF_CLOSE);
 			/*		status */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_STATUS_OPEN,
-							 strlen(WDVH_STATUS_OPEN));
+			WDVH_sendBodyChunk(userData->reply,WDVH_STATUS_OPEN);
 
 			/* get the CAPI error code and translate it to the appropriate http status code */
 			WDVCAPI_GetErrorType(errorItem,&errorType);
@@ -394,15 +395,9 @@
 				case WDVCAPI_ERR_TYPE_SQL: 
 					{
 						/* internal server error */
-						sapdbwa_SendBody(userData->reply,
-										 WDVH_HTTP_1_1,
-										 strlen(WDVH_HTTP_1_1));
-						sapdbwa_SendBody(userData->reply,
-										 ERROR_500_STR,
-										 strlen(ERROR_500_STR));
-						sapdbwa_SendBody(userData->reply,
-										 ERROR_500_TEXT,
-										 strlen(ERROR_500_TEXT));
+						WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+						WDVH_sendBodyChunk(userData->reply,ERROR_500_STR);
+						WDVH_sendBodyChunk(userData->reply,ERROR_500_TEXT);
 						break;
 					}
 				case WDVCAPI_ERR_TYPE_CAPI: 
@@ -413,15 +408,9 @@
 							   = Resource Uri and Destination Uri MUST NOT be equal */
 							case WDVCAPI_ERR_CODE_EQUAL_RESOURCE_AND_DESTINATION:
 								{
-									sapdbwa_SendBody(userData->reply,
-													 WDVH_HTTP_1_1,
-													 strlen(WDVH_HTTP_1_1));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_403_STR,
-													 strlen(ERROR_403_STR));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_403_TEXT,
-													 strlen(ERROR_403_TEXT));
+									WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+									WDVH_sendBodyChunk(userData->reply,ERROR_403_STR);
+									WDVH_sendBodyChunk(userData->reply,ERROR_403_TEXT);
 									break;
 								}
 								/* WDVCAPI_ERR_CODE_DESTINATION_URI_DOESNT_EXIST 
@@ -429,15 +418,9 @@
 									  that does not exist rigth now */
 							case WDVCAPI_ERR_CODE_DESTINATION_URI_DOESNT_EXIST:
 								{
-									sapdbwa_SendBody(userData->reply,
-													 WDVH_HTTP_1_1,
-													 strlen(WDVH_HTTP_1_1));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_409_STR,
-													 strlen(ERROR_409_STR));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_409_TEXT,
-													 strlen(ERROR_409_TEXT));
+									WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+									WDVH_sendBodyChunk(userData->reply,ERROR_409_STR);
+									WDVH_sendBodyChunk(userData->reply,ERROR_409_TEXT);
 									break;
 								}
 /* ---> CAPI: WDVCAPI_ERR_CODE_PROPERTY_BEHAVIOR_CONFLICT = die Properties konnten nicht gem
@@ -445,15 +428,9 @@
 															werden 
 							case WDVCAPI_ERR_CODE_PROPERTY_BEHAVIOR_CONFLICT:
 								{
-									sapdbwa_SendBody(userData->reply,
-													 WDVH_HTTP_1_1,
-													 strlen(WDVH_HTTP_1_1));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_412_STR,
-													 strlen(ERROR_412_STR));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_412_TEXT,
-													 strlen(ERROR_412_TEXT));
+									WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+									WDVH_sendBodyChunk(userData->reply,ERROR_412_STR);
+									WDVH_sendBodyChunk(userData->reply,ERROR_412_TEXT);
 									break;
 								}
 */
@@ -461,29 +438,17 @@
 								= overwrite was verbidden and destination allready exists */
 							case WDVCAPI_ERR_CODE_OVERWRITE_CONFLICT:
 								{
-									sapdbwa_SendBody(userData->reply,
-													 WDVH_HTTP_1_1,
-													 strlen(WDVH_HTTP_1_1));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_412_STR,
-													 strlen(ERROR_412_STR));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_412_TEXT,
-													 strlen(ERROR_412_TEXT));
+									WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+									WDVH_sendBodyChunk(userData->reply,ERROR_412_STR);
+									WDVH_sendBodyChunk(userData->reply,ERROR_412_TEXT);
 									break;
 								}
 /* ---> CAPI: WDVCAPI_ERR_CODE_RESOURCE_LOCKED = destination ist gesperrt, kein copy
 							case WDVCAPI_ERR_CODE_RESOURCE_LOCKED:
 								{
-									sapdbwa_SendBody(userData->reply,
-													 WDVH_HTTP_1_1,
-													 strlen(WDVH_HTTP_1_1));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_423_STR,
-													 strlen(ERROR_423_STR));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_423_TEXT,
-													 strlen(ERROR_423_TEXT));
+									WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+									WDVH_sendBodyChunk(userData->reply,ERROR_423_STR);
+									WDVH_sendBodyChunk(userData->reply,ERROR_423_TEXT);
 									break;
 								}
 */
@@ -491,15 +456,9 @@
 								             verweigert die Annahme der Resource, kein copy
 							case WDVCAPI_ERR_CODE_BAD_GATEWAY:
 								{
-									sapdbwa_SendBody(userData->reply,
-													 WDVH_HTTP_1_1,
-													 strlen(WDVH_HTTP_1_1));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_502_STR,
-													 strlen(ERROR_502_STR));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_502_TEXT,
-													 strlen(ERROR_502_TEXT));
+									WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+									WDVH_sendBodyChunk(userData->reply,ERROR_502_STR);
+									WDVH_sendBodyChunk(userData->reply,ERROR_502_TEXT);
 									break;
 								}
 */
@@ -507,28 +466,16 @@
 							   = lack of memory */
 							case WDVCAPI_ERR_CODE_NO_MEMORY:
 								{
-									sapdbwa_SendBody(userData->reply,
-													 WDVH_HTTP_1_1,
-													 strlen(WDVH_HTTP_1_1));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_507_STR,
-													 strlen(ERROR_507_STR));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_507_TEXT,
-													 strlen(ERROR_507_TEXT));
+									WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+									WDVH_sendBodyChunk(userData->reply,ERROR_507_STR);
+									WDVH_sendBodyChunk(userData->reply,ERROR_507_TEXT);
 									break;
 								}
 							default:
 								{
-									sapdbwa_SendBody(userData->reply,
-													 WDVH_HTTP_1_1,
-													 strlen(WDVH_HTTP_1_1));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_500_STR,
-													 strlen(ERROR_500_STR));
-									sapdbwa_SendBody(userData->reply,
-													 ERROR_500_TEXT,
-													 strlen(ERROR_500_TEXT));
+									WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+									WDVH_sendBodyChunk(userData->reply,ERROR_500_STR);
+									WDVH_sendBodyChunk(userData->reply,ERROR_500_TEXT);
 									break;
 								}
 						}
@@ -536,26 +483,16 @@
 					}
 				default:
 					{
-						sapdbwa_SendBody(userData->reply,
-										 WDVH_HTTP_1_1,
-										 strlen(WDVH_HTTP_1_1));
-						sapdbwa_SendBody(userData->reply,
-										 ERROR_500_STR,
-										 strlen(ERROR_500_STR));
-						sapdbwa_SendBody(userData->reply,
-										 ERROR_500_TEXT,
-										 strlen(ERROR_500_TEXT));
+						WDVH_sendBodyChunk(userData->reply,WDVH_HTTP_1_1);
+						WDVH_sendBodyChunk(userData->reply,ERROR_500_STR);
+						WDVH_sendBodyChunk(userData->reply,ERROR_500_TEXT);
 						break;
 					}
 			}
 			/*			/status */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_STATUS_CLOSE,
-							 strlen(WDVH_STATUS_CLOSE));
+			WDVH_sendBodyChunk(userData->reply,WDVH_STATUS_CLOSE);
 			/*	/response */
-			sapdbwa_SendBody(userData->reply,
-							 WDVH_RESPONSE_CLOSE,
-							 strlen(WDVH_RESPONSE_CLOSE));
+			WDVH_sendBodyChunk(userData->reply,WDVH_RESPONSE_CLOSE);
 		}
 
 	}
