@@ -58,6 +58,13 @@
 		WDVH_Char				errorText[1000];
 		WDVH_Char				*server;
 
+		if (wa==NULL)
+			return ERROR_500;
+		if (request==NULL || wdv==NULL || propfind==NULL) {
+			sapdbwa_WriteLogMsg(wa, "propfindCallCapiFunc:Uninitialized Pointers\n");
+			return ERROR_500;
+		}
+
 		/* then call CAPI Function */
 		success = WDVCAPI_Propfind(wdv,propfind);
 
@@ -134,7 +141,9 @@
 							WDVH_Char *uri,
 							WDVCAPI_WDV wdv,
 							WDVCAPI_PropfindHandle  propfind,
+							WDVH_Bool isPropnameRequest,		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1130891 */
 							WDVH_Bool isEmptyPropertyList,     /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123332 */
+							WDVH_Char *userAgent,
 							WDVH_Char *errormsg)			   /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
 	{
 		char *buf;
@@ -151,72 +160,151 @@
 		WDVH_Bool									requestedResource = WDVH_True;
 		WDVH_Char									*server=NULL;
 		WDVCAPI_URI									resourceUri;
+		WDVH_Char									uri_enc[WDVH_MAX_URI_LEN+1];
 		WDVCAPI_LockHandle							lock;
 		WDVH_Char									timeout[50];
 		WDVH_Char									locktoken[100];
 		WDVCAPI_PropfindNameSpaceShortcut			davPropertyNamespaceShortcut = NULL;
         WDVH_HTTPDateString                         httpDate;
 		WDVH_Char									prefix[WDVH_MAX_URI_LEN+1];
+		WDVH_Char									contentlength[10];
+		WDVH_Bool									isMSWebFolder = WDVH_False;
 
-		/* initialize Response Header */
-		if (statuscode==ERROR_207) {
-			sapdbwa_InitHeader(reply,
-							   statuscode,
-							   "text/xml",
-							   NULL,
-							   NULL,								/* const char    *LastModified,*/
-							   NULL,								/* const char    *Expires,*/
-							   NULL);								/* const char    *Location );*/
-
-/*			sapdbwa_SetHeader( reply,
-				   "Transfer-Encoding",
-				   "chunked" );
-*/		} else {
-			sapdbwa_InitHeader(reply,
-							   statuscode,
-							   "text/html",
-							   NULL,
-							   NULL,								/* const char    *LastModified,*/
-							   NULL,								/* const char    *Expires,*/
-							   NULL);								/* const char    *Location );*/
+		if (wa==NULL)
+			return WDVH_False;
+		if (reply==NULL || wdv==NULL || host==NULL || port==NULL || uri==NULL || propfind==NULL || userAgent==NULL || errormsg==NULL) {
+			sapdbwa_WriteLogMsg(wa, "propfindBuildReply:Uninitialized Pointers\n");
+			return WDVH_False;
 		}
-		
-		sapdbwa_SendHeader(reply);
 
-		
 		switch (statuscode) {
 			case ERROR_400: {
+				sapdbwa_InitHeader(reply,
+									statuscode,
+									"text/html",
+									NULL,
+									NULL,					/* const char    *LastModified,*/
+									NULL,					/* const char    *Expires,*/
+									NULL);					/* const char    *Location );*/
+
+				sapdbwa_SetHeader( reply,
+									"DAV",
+									"1" );
+
+				sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_400));
+				sapdbwa_SetHeader( reply,
+								"Content-Length",
+								contentlength);
+
+				sapdbwa_SendHeader(reply);
+				
 				sapdbwa_SendBody(reply,
 								 ERROR_PAGE_400,
 								 strlen(ERROR_PAGE_400));
 				break; 
 			}
 			case ERROR_404: {
+				sapdbwa_InitHeader(reply,
+									statuscode,
+									"text/html",
+									NULL,
+									NULL,					/* const char    *LastModified,*/
+									NULL,					/* const char    *Expires,*/
+									NULL);					/* const char    *Location );*/
+
+				sapdbwa_SetHeader( reply,
+									"DAV",
+									"1" );
+
+				sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_404));
+				sapdbwa_SetHeader( reply,
+								"Content-Length",
+								contentlength);
+
+				sapdbwa_SendHeader(reply);
+				
 				sapdbwa_SendBody(reply,
 								 ERROR_PAGE_404,
 								 strlen(ERROR_PAGE_404));
 				break; 
 			}
-			case ERROR_409: {										/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
-				sapdbwa_SendBody(reply,ERROR_PAGE_409_OWN,0);		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
-				sapdbwa_SendBody(reply,errormsg,0);					/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
-				sapdbwa_SendBody(reply,ERROR_PAGE_FOOTER,0);		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
-				break;												/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
-			}														/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */		
+			case ERROR_409: {										
+				sapdbwa_InitHeader(reply,
+									statuscode,
+									"text/html",
+									NULL,
+									NULL,					/* const char    *LastModified,*/
+									NULL,					/* const char    *Expires,*/
+									NULL);					/* const char    *Location );*/
+
+				sapdbwa_SetHeader( reply,
+									"DAV",
+									"1" );
+
+				sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_409_OWN)+strlen(errormsg)+strlen(ERROR_PAGE_FOOTER));
+				sapdbwa_SetHeader( reply,
+								"Content-Length",
+								contentlength);
+
+				sapdbwa_SendHeader(reply);
+
+				sapdbwa_SendBody(reply,ERROR_PAGE_409_OWN,0);		
+				sapdbwa_SendBody(reply,errormsg,0);					
+				sapdbwa_SendBody(reply,ERROR_PAGE_FOOTER,0);		
+				break;												
+			}														
 			case ERROR_500: {
+				sapdbwa_InitHeader(reply,
+									statuscode,
+									"text/html",
+									NULL,
+									NULL,					/* const char    *LastModified,*/
+									NULL,					/* const char    *Expires,*/
+									NULL);					/* const char    *Location );*/
+
+				sapdbwa_SetHeader( reply,
+									"DAV",
+									"1" );
+
+				sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_500));
+				sapdbwa_SetHeader( reply,
+								"Content-Length",
+								contentlength);
+
+				sapdbwa_SendHeader(reply);
 				sapdbwa_SendBody(reply,
 								 ERROR_PAGE_500,
 								 strlen(ERROR_PAGE_500));
 				break; 
 			}
 			case ERROR_207: {
+				if (strstr(userAgent,"Microsoft Data Access Internet Publishing Provider DAV")!=NULL) {
+					/* User-Agent is MS WebFolder -> do not use transfer-encoding chunked! */
+					isMSWebFolder = WDVH_True;
+				} else {
+					isMSWebFolder = WDVH_False;
+				}
+				
+				sapdbwa_InitHeader(reply,
+								statuscode,
+								"text/xml",
+								NULL,
+								NULL,								/* const char    *LastModified,*/
+								NULL,								/* const char    *Expires,*/
+								NULL);								/* const char    *Location );*/
+
+				if (isMSWebFolder == WDVH_False) {
+					sapdbwa_SetHeader( reply,
+						"Transfer-Encoding",
+						"chunked" );
+				}
+
+				sapdbwa_SendHeader(reply);
+
 				/* send body header */
-				sapdbwa_SendBody(reply,
-								 WDVH_XML_HEADER,
-								 strlen(WDVH_XML_HEADER));
-				sapdbwa_SendBody(reply,
-								 WDVH_MULTI_STATUS_OPEN,
-								 strlen(WDVH_MULTI_STATUS_OPEN));
+				WDVH_sendBodyChunkPropfind(reply,WDVH_XML_HEADER,isMSWebFolder);
+
+				WDVH_sendBodyChunkPropfind(reply,WDVH_MULTI_STATUS_OPEN,isMSWebFolder);
 
 				/* set server prefix */
 				getWebDAVPrefix(wa,prefix);
@@ -312,36 +400,22 @@
 				/* loop at the resultlist provided by propfind function */
 				do {
 					/* response */
-					sapdbwa_SendBody(reply,
-									 WDVH_RESPONSE_OPEN,
-									 strlen(WDVH_RESPONSE_OPEN));
-					sapdbwa_SendBody(reply,
-									 getStrBufferContent(namespaceList),
-									 getStrBufferLength(namespaceList));
-					sapdbwa_SendBody(reply,
-									 WDVH_XML_CLOSE_TAG,
-									 strlen(WDVH_XML_CLOSE_TAG));
+					WDVH_sendBodyChunkPropfind(reply,WDVH_RESPONSE_OPEN,isMSWebFolder);
+					WDVH_sendBodyChunkPropfind(reply,getStrBufferContent(namespaceList),isMSWebFolder);
+					WDVH_sendBodyChunkPropfind(reply,WDVH_XML_CLOSE_TAG,isMSWebFolder);
 								
 					/*		href */
-					sapdbwa_SendBody(reply,
-									 WDVH_HREF_OPEN,
-									 strlen(WDVH_HREF_OPEN));
-					sapdbwa_SendBody(reply,
-									 server,
-									 strlen(server));
+					WDVH_sendBodyChunkPropfind(reply,WDVH_HREF_OPEN,isMSWebFolder);
+					WDVH_sendBodyChunkPropfind(reply,server,isMSWebFolder);
 					if (requestedResource) {
-						sapdbwa_SendBody(reply,
-										 uri,
-										 strlen(uri));
+						URLencode(uri,uri_enc,WDVH_MAX_URI_LEN);
+						WDVH_sendBodyChunkPropfind(reply,uri_enc,isMSWebFolder);
 						requestedResource = WDVH_False;
 					} else {
-						sapdbwa_SendBody(reply,
-										 resourceUri,
-										 strlen(resourceUri));
+						URLencode(resourceUri,uri_enc,WDVH_MAX_URI_LEN);
+						WDVH_sendBodyChunkPropfind(reply,uri_enc,isMSWebFolder);
 					}
-					sapdbwa_SendBody(reply,
-									 WDVH_HREF_CLOSE,
-									 strlen(WDVH_HREF_CLOSE));
+					WDVH_sendBodyChunkPropfind(reply,WDVH_HREF_CLOSE,isMSWebFolder);
 
 					/* loop at properties */
 					if (!isEmptyPropertyList) {
@@ -356,27 +430,17 @@
 					}
 					do {
 						/*		propstat */
-						sapdbwa_SendBody(reply,
-										 WDVH_PROPSTAT_OPEN,
-										 strlen(WDVH_PROPSTAT_OPEN));
+						WDVH_sendBodyChunkPropfind(reply,WDVH_PROPSTAT_OPEN,isMSWebFolder);
 						if (!isEmptyPropertyList) {
 							/*			prop */
-							sapdbwa_SendBody(reply,
-											WDVH_PROP_OPEN,
-											strlen(WDVH_PROP_OPEN));
+							WDVH_sendBodyChunkPropfind(reply,WDVH_PROP_OPEN,isMSWebFolder);
 
 							/*				property name */
 							if (WDVCAPI_PropfindGetPropertyName(wdv,propfind,&propertyName )) {
-								sapdbwa_SendBody(reply,
-												WDVH_XML_OPEN_TAG,
-												strlen(WDVH_XML_OPEN_TAG));
-								sapdbwa_SendBody(reply,
-												WDVH_XML_NS_PREFIX,
-												strlen(WDVH_XML_NS_PREFIX));
+								WDVH_sendBodyChunkPropfind(reply,WDVH_XML_OPEN_TAG,isMSWebFolder);
+								WDVH_sendBodyChunkPropfind(reply,WDVH_XML_NS_PREFIX,isMSWebFolder);
 								if (WDVCAPI_PropfindGetPropertyNameSpaceShortcut(wdv,propfind,&propertyNamespaceShortcut)) {
-									sapdbwa_SendBody(reply,
-													propertyNamespaceShortcut,
-													strlen(propertyNamespaceShortcut));
+									WDVH_sendBodyChunkPropfind(reply,propertyNamespaceShortcut,isMSWebFolder);
 								} else {
 									/* clean up */
 									destroyStrBuffer(namespaceList);
@@ -384,278 +448,171 @@
 									sendErrorReply(ERROR_500, reply, WEBDAV_PROPFIND,"");
 									return WDVH_True;
 								}
-								sapdbwa_SendBody(reply,
-												WDVH_XML_COLON,
-												strlen(WDVH_XML_COLON));
+								WDVH_sendBodyChunkPropfind(reply,WDVH_XML_COLON,isMSWebFolder);
 								/* Property getlastmodified has to be handled especially */
 								if (strcmp(propertyName,WDVH_XML_GETLASTMODIFIED) == 0 &&
 									strcmp(propertyNamespaceShortcut, davPropertyNamespaceShortcut) == 0) {
 									/* give a special attribute for the date/time format in use in order for
 									MS WebFolder to recognize the getlastmodified date */
-									sapdbwa_SendBody(reply,
-													propertyName,
-													strlen(propertyName));
-									sapdbwa_SendBody(reply,
-													" ",
-													strlen(" "));
-									sapdbwa_SendBody(reply,
-													WDVH_XML_GETLASTMODIFIED_ATTR,
-													strlen(WDVH_XML_GETLASTMODIFIED_ATTR));
+									WDVH_sendBodyChunkPropfind(reply,propertyName,isMSWebFolder);
+									WDVH_sendBodyChunkPropfind(reply," ",isMSWebFolder);
+									WDVH_sendBodyChunkPropfind(reply,WDVH_XML_GETLASTMODIFIED_ATTR,isMSWebFolder);
 								/* Property creationdate has to be handled especially */
 								} else if (strcmp(propertyName,WDVH_XML_CREATIONDATE) == 0 &&
 									strcmp(propertyNamespaceShortcut, davPropertyNamespaceShortcut) == 0) {
 									/* give a special attribute for the date/time format in use in order for
 									MS WebFolder to recognize the getlastmodified date */
-									sapdbwa_SendBody(reply,
-													propertyName,
-													strlen(propertyName));
-									sapdbwa_SendBody(reply,
-													" ",
-													strlen(" "));
-									sapdbwa_SendBody(reply,
-													WDVH_XML_CREATIONDATE_ATTR,
-													strlen(WDVH_XML_CREATIONDATE_ATTR));
+									WDVH_sendBodyChunkPropfind(reply,propertyName,isMSWebFolder);
+									WDVH_sendBodyChunkPropfind(reply," ",isMSWebFolder);
+									WDVH_sendBodyChunkPropfind(reply,WDVH_XML_CREATIONDATE_ATTR,isMSWebFolder);
 								} else {
-									sapdbwa_SendBody(reply,
-													propertyName,
-													strlen(propertyName));
+									WDVH_sendBodyChunkPropfind(reply,propertyName,isMSWebFolder);
 								}
-								sapdbwa_SendBody(reply,
-												WDVH_XML_CLOSE_TAG,
-												strlen(WDVH_XML_CLOSE_TAG));
-								/*					property value */
-								if (WDVCAPI_PropfindGetPropertyShortValue(wdv,propfind,&propertyShortValue)) {
-									/* Property getlastmodified has to be handled especially */
-									if (strcmp(propertyName,WDVH_XML_GETLASTMODIFIED) == 0 &&
-										strcmp(propertyNamespaceShortcut, davPropertyNamespaceShortcut) == 0) {
+								if (isPropnameRequest == WDVH_False) {					/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1130891 */
+									WDVH_sendBodyChunkPropfind(reply,WDVH_XML_CLOSE_TAG,isMSWebFolder);
+									/*					property value */
+									if (WDVCAPI_PropfindGetPropertyShortValue(wdv,propfind,&propertyShortValue)) {
+										/* Property getlastmodified has to be handled especially */
+										if (strcmp(propertyName,WDVH_XML_GETLASTMODIFIED) == 0 &&
+											strcmp(propertyNamespaceShortcut, davPropertyNamespaceShortcut) == 0) {
 
-										/* Convert timestamp to HTTP date RFC 2616 Ch: 3.3.1 */
-										/* Date format is RFC 822, updated by RFC 1123       */
-										propertyToHTTPDate(propertyShortValue, httpDate);
+											/* Convert timestamp to HTTP date RFC 2616 Ch: 3.3.1 */
+											/* Date format is RFC 822, updated by RFC 1123       */
+											propertyToHTTPDate(propertyShortValue, httpDate);
 
-										/* Send converted property */
-										sapdbwa_SendBody(reply, httpDate, strlen(httpDate));
+											/* Send converted property */
+											WDVH_sendBodyChunkPropfind(reply, httpDate,isMSWebFolder);
 
-									} else if (strcmp(propertyName,WDVH_XML_CREATIONDATE) == 0 &&
-										strcmp(propertyNamespaceShortcut, davPropertyNamespaceShortcut) == 0) {
+										} else if (strcmp(propertyName,WDVH_XML_CREATIONDATE) == 0 &&
+											strcmp(propertyNamespaceShortcut, davPropertyNamespaceShortcut) == 0) {
 
-										/* Convert timestamp to HTTP date RFC 2616 Ch: 3.3.1 */
-										/* Date format is RFC 822, updated by RFC 1123       */
-										propertyToHTTPDate(propertyShortValue, httpDate);
+											/* Convert timestamp to HTTP date RFC 2616 Ch: 3.3.1 */
+											/* Date format is RFC 822, updated by RFC 1123       */
+											propertyToHTTPDate(propertyShortValue, httpDate);
 
-										/* Send converted property */
-										sapdbwa_SendBody(reply, httpDate, strlen(httpDate));
+											/* Send converted property */
+											WDVH_sendBodyChunkPropfind(reply, httpDate,isMSWebFolder);
 
-									/* property resourcetype has to written as xml tag */
-									} else if (strcmp(propertyName,WDVH_XML_RESOURCETYPE)==0) {
-										if (strcmp(propertyShortValue,"collection")==0) {
-											sapdbwa_SendBody(reply,
-															WDVH_XML_OPEN_TAG,
-															strlen(WDVH_XML_OPEN_TAG));
-											sapdbwa_SendBody(reply,
-															WDVH_DEFAULT_NS,
-															strlen(WDVH_DEFAULT_NS));
-											sapdbwa_SendBody(reply,
-															propertyShortValue,
-															strlen(propertyShortValue));
-											sapdbwa_SendBody(reply,
-															WDVH_XML_SLASH,
-															strlen(WDVH_XML_SLASH));
-											sapdbwa_SendBody(reply,
-															WDVH_XML_CLOSE_TAG,
-															strlen(WDVH_XML_CLOSE_TAG));
-										} else if (strcmp(propertyShortValue,"null resource")==0) {
-											sapdbwa_SendBody(reply,
-															propertyShortValue,
-															strlen(propertyShortValue));
-										}
-									/* property lockdiscovery has to collect additional data */
-									} else if (strcmp(propertyName,WDVH_XML_LOCKDISCOVERY)==0) {
-										if (WDVCAPI_IdStringIsInitialValue(propertyShortValue)) {
-	/*										sapdbwa_SendBody(reply,
-															WDVH_LOCKDISCOVERY_EMPTY,
-															strlen(WDVH_LOCKDISCOVERY_EMPTY));
-	*/									} else {
-											WDVCAPI_PropfindGetLockDescription(wdv,
-																			propfind,
-																			propertyShortValue,
-																			&lock);
-											/* there is a lock on this resource -> get info and
-											merge data into the reply */
-											/*			<D:activelock> */
-											sapdbwa_SendBody(reply,
-															WDVH_ACTIVELOCK_OPEN,
-															strlen(WDVH_ACTIVELOCK_OPEN));
+										/* property resourcetype has to written as xml tag */
+										} else if (strcmp(propertyName,WDVH_XML_RESOURCETYPE)==0) {
+											if (strcmp(propertyShortValue,"collection")==0) {
+												WDVH_sendBodyChunkPropfind(reply,WDVH_XML_OPEN_TAG,isMSWebFolder);
+												WDVH_sendBodyChunkPropfind(reply,WDVH_DEFAULT_NS,isMSWebFolder);
+												WDVH_sendBodyChunkPropfind(reply,propertyShortValue,isMSWebFolder);
+												WDVH_sendBodyChunkPropfind(reply,WDVH_XML_SLASH,isMSWebFolder);
+												WDVH_sendBodyChunkPropfind(reply,WDVH_XML_CLOSE_TAG,isMSWebFolder);
+											} else if (strcmp(propertyShortValue,"null resource")==0) {
+												WDVH_sendBodyChunkPropfind(reply,propertyShortValue,isMSWebFolder);
+											}
+										/* property lockdiscovery has to collect additional data */
+										} else if (strcmp(propertyName,WDVH_XML_LOCKDISCOVERY)==0) {
+											if (WDVCAPI_IdStringIsInitialValue(propertyShortValue)) {
+		/*										WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKDISCOVERY_EMPTY);
+		*/									} else {
+												WDVCAPI_PropfindGetLockDescription(wdv,
+																				propfind,
+																				propertyShortValue,
+																				&lock);
+												/* there is a lock on this resource -> get info and
+												merge data into the reply */
+												/*			<D:activelock> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_ACTIVELOCK_OPEN,isMSWebFolder);
+												/*				<D:locktype> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKTYPE_OPEN,isMSWebFolder);
+												/*					value (get locktype from CAPI) */
+												if (lock->type == WDVCAPI_LOCK_TYPE_WRITE) {
+													WDVH_sendBodyChunkPropfind(reply,WDVH_WRITE_EMPTY,isMSWebFolder);
+												}
+												/*				</D:locktype> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKTYPE_CLOSE,isMSWebFolder);
+												/*				<D:lockscope> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKSCOPE_OPEN,isMSWebFolder);
+												/*					value (get lockscope from CAPI) */
+												if (lock->scope == WDVCAPI_LOCK_SCOPE_EXCLUSIVE) {
+													WDVH_sendBodyChunkPropfind(reply,WDVH_EXCLUSIVE_EMPTY,isMSWebFolder);
+												} else if (lock->scope == WDVH_LOCKSCOPE_SHARED) {
+													WDVH_sendBodyChunkPropfind(reply,WDVH_SHARED_EMPTY,isMSWebFolder);
+												}
+												/*				</D:lockscope> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKSCOPE_CLOSE,isMSWebFolder);
+												/*				<D:depth> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_DEPTH_OPEN,isMSWebFolder);
+												/*					value */
+												if (lock->depth == WDVCAPI_DEPTH_0) {
+													WDVH_sendBodyChunkPropfind(reply,WDVH_DEPTH_0,isMSWebFolder);
+												} else if (lock->depth == WDVCAPI_DEPTH_INFINITE) {
+													WDVH_sendBodyChunkPropfind(reply,WDVH_DEPTH_INFINITY,isMSWebFolder);
+												}
+												/*				</D:depth> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_DEPTH_CLOSE,isMSWebFolder);
+												/*				<D:owner> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_OWNER_OPEN,isMSWebFolder);
+												/*					value */
+												if (strlen(lock->owner)>0) 
+													WDVH_sendBodyChunkPropfind(reply,lock->owner,isMSWebFolder);
+												/*				</D:owner> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_OWNER_CLOSE,isMSWebFolder);
+												/*				<D:timeout> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_TIMEOUT_OPEN,isMSWebFolder);
+												/*					value */
+												sp77sprintf(timeout,50,"%ld",lock->timeout);
+												WDVH_sendBodyChunkPropfind(reply,timeout,isMSWebFolder);
+												/*				</D:timeout> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_TIMEOUT_CLOSE,isMSWebFolder);
+												/*				<D:locktoken> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKTOKEN_OPEN,isMSWebFolder);
+												/*					<D:href> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_HREF_OPEN,isMSWebFolder);
+												/*						value */
+												sp77sprintf(locktoken,100,"saplocktoken:%s",propertyShortValue);
+												WDVH_sendBodyChunkPropfind(reply,locktoken,isMSWebFolder);
+												/*					</D:href> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_HREF_CLOSE,isMSWebFolder);
+												/*				</D:locktoken> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKTOKEN_CLOSE,isMSWebFolder);
+												/*			</D:activelock> */
+												WDVH_sendBodyChunkPropfind(reply,WDVH_ACTIVELOCK_CLOSE,isMSWebFolder);
+											}
+										/* property supportedlock has to build a xml structure */
+										} else if (strcmp(propertyName,WDVH_XML_SUPPORTEDLOCK)==0) {
+											/*			<D:lockentry> */
+											WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKENTRY_OPEN,isMSWebFolder);
 											/*				<D:locktype> */
-											sapdbwa_SendBody(reply,
-															WDVH_LOCKTYPE_OPEN,
-															strlen(WDVH_LOCKTYPE_OPEN));
+											WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKTYPE_OPEN,isMSWebFolder);
 											/*					value (get locktype from CAPI) */
-											if (lock->type == WDVCAPI_LOCK_TYPE_WRITE) {
-												sapdbwa_SendBody(reply,
-																WDVH_WRITE_EMPTY,
-																strlen(WDVH_WRITE_EMPTY));
-											}
+											WDVH_sendBodyChunkPropfind(reply,WDVH_WRITE_EMPTY,isMSWebFolder);
 											/*				</D:locktype> */
-											sapdbwa_SendBody(reply,
-															WDVH_LOCKTYPE_CLOSE,
-															strlen(WDVH_LOCKTYPE_CLOSE));
+											WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKTYPE_CLOSE,isMSWebFolder);
 											/*				<D:lockscope> */
-											sapdbwa_SendBody(reply,
-															WDVH_LOCKSCOPE_OPEN,
-															strlen(WDVH_LOCKSCOPE_OPEN));
+											WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKSCOPE_OPEN,isMSWebFolder);
 											/*					value (get lockscope from CAPI) */
-											if (lock->scope == WDVCAPI_LOCK_SCOPE_EXCLUSIVE) {
-												sapdbwa_SendBody(reply,
-																WDVH_EXCLUSIVE_EMPTY,
-																strlen(WDVH_EXCLUSIVE_EMPTY));
-											} else if (lock->scope == WDVH_LOCKSCOPE_SHARED) {
-												sapdbwa_SendBody(reply,
-																WDVH_SHARED_EMPTY,
-																strlen(WDVH_SHARED_EMPTY));
-											}
+											WDVH_sendBodyChunkPropfind(reply,WDVH_EXCLUSIVE_EMPTY,isMSWebFolder);
 											/*				</D:lockscope> */
-											sapdbwa_SendBody(reply,
-															WDVH_LOCKSCOPE_CLOSE,
-															strlen(WDVH_LOCKSCOPE_CLOSE));
-											/*				<D:depth> */
-											sapdbwa_SendBody(reply,
-															WDVH_DEPTH_OPEN,
-															strlen(WDVH_DEPTH_OPEN));
-											/*					value */
-											if (lock->depth == WDVCAPI_DEPTH_0) {
-												sapdbwa_SendBody(reply,
-																WDVH_DEPTH_0,
-																strlen(WDVH_DEPTH_0));
-											} else if (lock->depth == WDVCAPI_DEPTH_INFINITE) {
-												sapdbwa_SendBody(reply,
-																WDVH_DEPTH_INFINITY,
-																strlen(WDVH_DEPTH_INFINITY));
-											}
-											/*				</D:depth> */
-											sapdbwa_SendBody(reply,
-															WDVH_DEPTH_CLOSE,
-															strlen(WDVH_DEPTH_CLOSE));
-											/*				<D:owner> */
-											sapdbwa_SendBody(reply,
-															WDVH_OWNER_OPEN,
-															strlen(WDVH_OWNER_OPEN));
-											/*					value */
-											sapdbwa_SendBody(reply,
-															lock->owner,
-															strlen(lock->owner));
-											/*				</D:owner> */
-											sapdbwa_SendBody(reply,
-															WDVH_OWNER_CLOSE,
-															strlen(WDVH_OWNER_CLOSE));
-											/*				<D:timeout> */
-											sapdbwa_SendBody(reply,
-															WDVH_TIMEOUT_OPEN,
-															strlen(WDVH_TIMEOUT_OPEN));
-											/*					value */
-											sp77sprintf(timeout,50,"%ld",lock->timeout);
-											sapdbwa_SendBody(reply,
-															timeout,
-															strlen(timeout));
-											/*				</D:timeout> */
-											sapdbwa_SendBody(reply,
-															WDVH_TIMEOUT_CLOSE,
-															strlen(WDVH_TIMEOUT_CLOSE));
-											/*				<D:locktoken> */
-											sapdbwa_SendBody(reply,
-															WDVH_LOCKTOKEN_OPEN,
-															strlen(WDVH_LOCKTOKEN_OPEN));
-											/*					<D:href> */
-											sapdbwa_SendBody(reply,
-															WDVH_HREF_OPEN,
-															strlen(WDVH_HREF_OPEN));
-											/*						value */
-											sp77sprintf(locktoken,100,"saplocktoken:%s",propertyShortValue);
-											sapdbwa_SendBody(reply,
-															locktoken,
-															strlen(locktoken));
-											/*					</D:href> */
-											sapdbwa_SendBody(reply,
-															WDVH_HREF_CLOSE,
-															strlen(WDVH_HREF_CLOSE));
-											/*				</D:locktoken> */
-											sapdbwa_SendBody(reply,
-															WDVH_LOCKTOKEN_CLOSE,
-															strlen(WDVH_LOCKTOKEN_CLOSE));
-											/*			</D:activelock> */
-											sapdbwa_SendBody(reply,
-															WDVH_ACTIVELOCK_CLOSE,
-															strlen(WDVH_ACTIVELOCK_CLOSE));
+											WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKSCOPE_CLOSE,isMSWebFolder);
+											/*			</D:lockentry> */
+											WDVH_sendBodyChunkPropfind(reply,WDVH_LOCKENTRY_CLOSE,isMSWebFolder);
+										} else {
+											WDVH_sendBodyChunkPropfind(reply,propertyShortValue,isMSWebFolder);
 										}
-									/* property supportedlock has to build a xml structure */
-									} else if (strcmp(propertyName,WDVH_XML_SUPPORTEDLOCK)==0) {
-										/*			<D:lockentry> */
-										sapdbwa_SendBody(reply,
-														WDVH_LOCKENTRY_OPEN,
-														strlen(WDVH_LOCKENTRY_OPEN));
-										/*				<D:locktype> */
-										sapdbwa_SendBody(reply,
-														WDVH_LOCKTYPE_OPEN,
-														strlen(WDVH_LOCKTYPE_OPEN));
-										/*					value (get locktype from CAPI) */
-										sapdbwa_SendBody(reply,
-														WDVH_WRITE_EMPTY,
-														strlen(WDVH_WRITE_EMPTY));
-										/*				</D:locktype> */
-										sapdbwa_SendBody(reply,
-														WDVH_LOCKTYPE_CLOSE,
-														strlen(WDVH_LOCKTYPE_CLOSE));
-										/*				<D:lockscope> */
-										sapdbwa_SendBody(reply,
-														WDVH_LOCKSCOPE_OPEN,
-														strlen(WDVH_LOCKSCOPE_OPEN));
-										/*					value (get lockscope from CAPI) */
-										sapdbwa_SendBody(reply,
-														WDVH_EXCLUSIVE_EMPTY,
-														strlen(WDVH_EXCLUSIVE_EMPTY));
-										/*				</D:lockscope> */
-										sapdbwa_SendBody(reply,
-														WDVH_LOCKSCOPE_CLOSE,
-														strlen(WDVH_LOCKSCOPE_CLOSE));
-										/*			</D:lockentry> */
-										sapdbwa_SendBody(reply,
-														WDVH_LOCKENTRY_CLOSE,
-														strlen(WDVH_LOCKENTRY_CLOSE));
 									} else {
-										sapdbwa_SendBody(reply,
-														propertyShortValue,
-														strlen(propertyShortValue));
+										/* clean up */
+										destroyStrBuffer(namespaceList);
+										/* error handling */
+										sendErrorReply(ERROR_500, reply, WEBDAV_PROPFIND,"");
+										return WDVH_True;
 									}
-								} else {
-									/* clean up */
-									destroyStrBuffer(namespaceList);
-									/* error handling */
-									sendErrorReply(ERROR_500, reply, WEBDAV_PROPFIND,"");
-									return WDVH_True;
-								}
-
+									WDVH_sendBodyChunkPropfind(reply,WDVH_XML_OPEN_TAG,isMSWebFolder);
+									WDVH_sendBodyChunkPropfind(reply,WDVH_XML_SLASH,isMSWebFolder);
+									WDVH_sendBodyChunkPropfind(reply,WDVH_XML_NS_PREFIX,isMSWebFolder);
+									WDVH_sendBodyChunkPropfind(reply,propertyNamespaceShortcut,isMSWebFolder);
+									WDVH_sendBodyChunkPropfind(reply,WDVH_XML_COLON,isMSWebFolder);
+									WDVH_sendBodyChunkPropfind(reply,propertyName,isMSWebFolder);
+									WDVH_sendBodyChunkPropfind(reply,WDVH_XML_CLOSE_TAG,isMSWebFolder);
+								} else {														/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1130891 */
+									WDVH_sendBodyChunkPropfind(reply,WDVH_XML_SLASH,isMSWebFolder);					/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1130891 */
+									WDVH_sendBodyChunkPropfind(reply,WDVH_XML_CLOSE_TAG,isMSWebFolder);				/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1130891 */
+								}																/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1130891 */
 								/*				/property name */
-								sapdbwa_SendBody(reply,
-												WDVH_XML_OPEN_TAG,
-												strlen(WDVH_XML_OPEN_TAG));
-								sapdbwa_SendBody(reply,
-												WDVH_XML_SLASH,
-												strlen(WDVH_XML_SLASH));
-								sapdbwa_SendBody(reply,
-												WDVH_XML_NS_PREFIX,
-												strlen(WDVH_XML_NS_PREFIX));
-								sapdbwa_SendBody(reply,
-												propertyNamespaceShortcut,
-												strlen(propertyNamespaceShortcut));
-								sapdbwa_SendBody(reply,
-												WDVH_XML_COLON,
-												strlen(WDVH_XML_COLON));
-								sapdbwa_SendBody(reply,
-												propertyName,
-												strlen(propertyName));
-								sapdbwa_SendBody(reply,
-												WDVH_XML_CLOSE_TAG,
-												strlen(WDVH_XML_CLOSE_TAG));
 							} else {
 								/* clean up */
 								destroyStrBuffer(namespaceList);
@@ -664,23 +621,15 @@
 								return WDVH_True;
 							}
 							/*			/prop */
-							sapdbwa_SendBody(reply,
-											WDVH_PROP_CLOSE,
-											strlen(WDVH_PROP_CLOSE));
+							WDVH_sendBodyChunkPropfind(reply,WDVH_PROP_CLOSE,isMSWebFolder);
 
 							/*			status */
-							sapdbwa_SendBody(reply,
-											WDVH_STATUS_OPEN,
-											strlen(WDVH_STATUS_OPEN));
+							WDVH_sendBodyChunkPropfind(reply,WDVH_STATUS_OPEN,isMSWebFolder);
 							if (WDVCAPI_PropfindGetPropertyStatus(wdv,propfind,&propertyStatus)) {
 								if (propertyStatus == WDVCAPI_PROPERTY_STATUS_OK) {
-									sapdbwa_SendBody(reply,
-													WDVH_STATUS_OK,
-													strlen(WDVH_STATUS_OK));
+									WDVH_sendBodyChunkPropfind(reply,WDVH_STATUS_OK,isMSWebFolder);
 								} else if (propertyStatus == WDVCAPI_PROPERTY_STATUS_FAILED) {
-									sapdbwa_SendBody(reply,
-													WDVH_STATUS_FAILED,
-													strlen(WDVH_STATUS_FAILED));
+									WDVH_sendBodyChunkPropfind(reply,WDVH_STATUS_FAILED,isMSWebFolder);
 								}
 							} else {
 								/* error handling */
@@ -689,27 +638,23 @@
 							}
 
 							/*			/status */
-							sapdbwa_SendBody(reply,
-											WDVH_STATUS_CLOSE,
-											strlen(WDVH_STATUS_CLOSE));
+							WDVH_sendBodyChunkPropfind(reply,WDVH_STATUS_CLOSE,isMSWebFolder);
 						}
 						/*		/propstat */
-						sapdbwa_SendBody(reply,
-										 WDVH_PROPSTAT_CLOSE,
-										 strlen(WDVH_PROPSTAT_CLOSE));
+						WDVH_sendBodyChunkPropfind(reply,WDVH_PROPSTAT_CLOSE,isMSWebFolder);
 					} while(WDVCAPI_PropfindGetNextProperty(wdv,propfind));
 
 					/*	/response */
-					sapdbwa_SendBody(reply,
-									 WDVH_RESPONSE_CLOSE,
-									 strlen(WDVH_RESPONSE_CLOSE));
+					WDVH_sendBodyChunkPropfind(reply,WDVH_RESPONSE_CLOSE,isMSWebFolder);
 
 
 				} while(WDVCAPI_PropfindGetNextURI(wdv,propfind,&resourceUri));
 
-				sapdbwa_SendBody(reply,
-								 WDVH_MULTI_STATUS_CLOSE,
-								 strlen(WDVH_MULTI_STATUS_CLOSE));
+				WDVH_sendBodyChunkPropfind(reply,WDVH_MULTI_STATUS_CLOSE,isMSWebFolder);
+                /* Send empty chunk as eof */
+				if (isMSWebFolder == WDVH_False)
+					WDVH_sendBodyChunkEnd(reply);
+
 				/* clean up */
 				destroyStrBuffer(namespaceList);
 			}
@@ -729,6 +674,7 @@
 										  WDVCAPI_PropfindHandle *propfind,
 										  sapdbwa_HttpRequestP request,
 										  WDVCAPI_Depth				depth,
+										  WDVH_Bool *isPropnameRequest,				/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1130891 */
 										  WDVH_Bool *isEmptyPropertyList,			/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123332 */
 										  WDVH_Char					*errormsg)		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */
 	{
@@ -755,6 +701,13 @@
 		WDVH_Char					*server;
 		WDVCAPI_ErrorText			errorMsg;							/* 	http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1124183 */ 	
 
+		if (wa==NULL)
+			return ERROR_500;
+		if (wdv==NULL || propfind==NULL || request==NULL || isPropnameRequest==NULL || isEmptyPropertyList==NULL || errormsg==NULL) {
+			sapdbwa_WriteLogMsg(wa, "propfindParseRequestBody:Uninitialized Pointers\n");
+			return ERROR_500;
+		}
+
 		/* get server string for error messages */
 		buildServerString(request,&server);
 		
@@ -828,12 +781,10 @@
 					strcpy(currentBuffer,"");
 				} else
 					done = 0;
-				printf("1\n");
 			} else {
 				strcpy(currentBuffer,WDVH_XML_ALLPROP_BODY);
 				pos = strlen(WDVH_XML_ALLPROP_BODY);
 				done = 1;
-				printf("2\n");
 			}
 			if (strcmp(currentBuffer,"")!=0) {
 				if (! XML_Parse(p, currentBuffer, pos, done)) {
@@ -863,6 +814,7 @@
 				sp77sprintf(errorText,1000,ERROR_PROPFIND_PARSE_1,server,(char*)sapdbwa_GetRequestURI(request));
 				sapdbwa_WriteLogMsg(wa, errorText);
 			}
+			*isPropnameRequest = newUserData->isPropnameRequest;
 		}
 
 		
@@ -902,7 +854,9 @@
 		XML_Parser						p;
 		WDVH_Char						*server;
 
-	
+		if (data==NULL || el==NULL || attr==NULL)
+			return;
+
 		userData = (WDVH_XmlUserData)data;
 		p = xmlParserUserDataGetParser(userData);
 		parserState = xmlParserUserDataGetPropfindParserState(userData);
@@ -1016,7 +970,7 @@
 						parserState->state == WDVH_PROPFIND_DFA_STATE_3) {
 						/* Correction Begin http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123337 
 						sp77sprintf(cmptag,strlen(tag)+3,"%s/>",tag);
-						if(xmlParserCheckEmptyTag(p,userData,cmptag)) {
+						if(xmlParserCheckEmptyTag(p,userData,cmptag)) {*/
 							/* property begin 
 							strcpy(capiNamespace,ns);
 							strcpy(capiPropertyName,tag);
@@ -1025,7 +979,7 @@
 																	  *(userData->propfindHandle),
 																	  capiNamespace,
 																	  capiPropertyName );
-							userData->propfindEmptyPropertyList = WDVH_False; /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123332 
+							userData->propfindEmptyPropertyList = WDVH_False;  http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123332 
 						} else {
 							parserState->error = WDVH_XML_PARSER_SYNTAX_ERROR;
 						}*/
@@ -1078,7 +1032,9 @@
 		XML_Parser						p;
 		WDVH_Char						*server;
 
-	
+		if (data==NULL || el==NULL)
+			return;
+
 		userData = (WDVH_XmlUserData)data;
 		capiWdv = xmlParserUserDataGetWDV(userData);
 		parserState = xmlParserUserDataGetPropfindParserState(userData);
@@ -1166,6 +1122,7 @@
 					sapdbwa_WriteLogMsg(userData->wa, errorText);
 					parserState->error = ERROR_500;
 				}
+				userData->isPropnameRequest = WDVH_True;
 				parserState->depth--;
 				parserState->state = WDVH_PROPFIND_DFA_STATE_4;
 			}
@@ -1234,6 +1191,8 @@
 		WDVH_XmlUserData				xmlUserData;
 		XML_Parser						p;
 
+		if (data==NULL || prefix==NULL || uri==NULL)
+			return WDVH_False;
 
 		if (prefix)
 			strcpy(xmlNamespace.prefix,prefix);
@@ -1258,6 +1217,9 @@
 		WDVH_XmlUserData				xmlUserData;
 		XML_Parser						p;
 
+		if (data==NULL || prefix==NULL)
+			return;
+
 		if (prefix)
 			strcpy(xmlNamespace.prefix,prefix);
 		xmlUserData = (WDVH_XmlUserData)data;
