@@ -38,6 +38,7 @@
 
 
 	#include "SAPDB/WebDAV/Handler/WDVHandler_ProppatchHandler.h"
+	#include "SAPDBCommon/SAPDB_string.h"
 
 	/*------------------------------------------------------------------------
 		Function: proppatchCallCapiFunc
@@ -56,6 +57,13 @@
 		WDVH_Error				error;
 		WDVH_Char				errorText[1000];
 
+		if (wa==NULL)
+			return ERROR_500;
+		if (request==NULL || wdv==NULL || proppatch==NULL) {
+			sapdbwa_WriteLogMsg(wa, "proppatchCallCapiFunc:Uninitialized Pointers\n");
+			return ERROR_500;
+		}
+
 		/* then call CAPI Function */
 /*		success = WDVCAPI_Proppatch(wdv,proppatch);
 */
@@ -151,70 +159,128 @@
 		WDVH_Char									*server=NULL;
 		WDVCAPI_URI									resourceUri;
 		WDVH_Char									prefix[WDVH_MAX_URI_LEN+1];
-
-		/* initialize Response Header */
-        if (statuscode == ERROR_200) {
-		    sapdbwa_InitHeader(reply,
-                               statuscode,
-                               NULL,
-                               NULL,
-	                           NULL,								/* const char    *LastModified,*/
-                               NULL,								/* const char    *Expires,*/
-                               NULL);								/* const char    *Location );*/
-        } else if (statuscode == ERROR_207) {
-		    sapdbwa_InitHeader(reply,
-                               statuscode,
-                               "text/xml",
-                               NULL,
-	                           NULL,								/* const char    *LastModified,*/
-                               NULL,								/* const char    *Expires,*/
-                               NULL);								/* const char    *Location );*/
-        } else {
-		    sapdbwa_InitHeader(reply,
-                               statuscode,
-                               "text/html",
-                               NULL,
-	                           NULL,								/* const char    *LastModified,*/
-                               NULL,								/* const char    *Expires,*/
-                               NULL);								/* const char    *Location );*/
-        }
-
-		sapdbwa_SendHeader(reply);
-
+		WDVH_Char									contentlength[10];
 		
+		if (wa==NULL)
+			return WDVH_False;
+		if (request==NULL || wdv==NULL || reply==NULL || host==NULL || port==NULL || uri==NULL || proppatch==NULL) {
+			sapdbwa_WriteLogMsg(wa, "proppatchCallCapiFunc:Uninitialized Pointers\n");
+			return WDVH_False;
+		}
+
 		switch (statuscode) {
 			case ERROR_400: {
+				sapdbwa_InitHeader(reply,
+									statuscode,
+									"text/html",
+									NULL,
+									NULL,					/* const char    *LastModified,*/
+									NULL,					/* const char    *Expires,*/
+									NULL);					/* const char    *Location );*/
+
+				sapdbwa_SetHeader( reply,
+									"DAV",
+									"1" );
+
+				sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_400));
+				sapdbwa_SetHeader( reply,
+								"Content-Length",
+								contentlength);
+
+				sapdbwa_SendHeader(reply);
 				sapdbwa_SendBody(reply,
 								 ERROR_PAGE_400,
 								 strlen(ERROR_PAGE_400));
 				break; 
 			}
 			case ERROR_404: {
+				sapdbwa_InitHeader(reply,
+									statuscode,
+									"text/html",
+									NULL,
+									NULL,					/* const char    *LastModified,*/
+									NULL,					/* const char    *Expires,*/
+									NULL);					/* const char    *Location );*/
+
+				sapdbwa_SetHeader( reply,
+									"DAV",
+									"1" );
+
+				sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_404));
+				sapdbwa_SetHeader( reply,
+								"Content-Length",
+								contentlength);
+
+				sapdbwa_SendHeader(reply);
 				sapdbwa_SendBody(reply,
 								 ERROR_PAGE_404,
 								 strlen(ERROR_PAGE_404));
 				break; 
 			}
 			case ERROR_423: {
+				sapdbwa_InitHeader(reply,
+									statuscode,
+									"text/html",
+									NULL,
+									NULL,					/* const char    *LastModified,*/
+									NULL,					/* const char    *Expires,*/
+									NULL);					/* const char    *Location );*/
+
+				sapdbwa_SetHeader( reply,
+									"DAV",
+									"1" );
+
+				sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_423));
+				sapdbwa_SetHeader( reply,
+								"Content-Length",
+								contentlength);
+
+				sapdbwa_SendHeader(reply);
 				sapdbwa_SendBody(reply,
 								 ERROR_PAGE_423,
 								 strlen(ERROR_PAGE_423));
 				break; 
 			}
 			case ERROR_500: {
+				sapdbwa_InitHeader(reply,
+									statuscode,
+									"text/html",
+									NULL,
+									NULL,					/* const char    *LastModified,*/
+									NULL,					/* const char    *Expires,*/
+									NULL);					/* const char    *Location );*/
+
+				sapdbwa_SetHeader( reply,
+									"DAV",
+									"1" );
+
+				sp77sprintf(contentlength,9,"%d",strlen(ERROR_PAGE_500));
+				sapdbwa_SetHeader( reply,
+								"Content-Length",
+								contentlength);
+
+				sapdbwa_SendHeader(reply);
 				sapdbwa_SendBody(reply,
 								 ERROR_PAGE_500,
 								 strlen(ERROR_PAGE_500));
 				break; 
 			}
 			case ERROR_207: {
+				sapdbwa_InitHeader(reply,
+									statuscode,
+									"text/xml",
+									NULL,
+									NULL,								/* const char    *LastModified,*/
+									NULL,								/* const char    *Expires,*/
+									NULL);								/* const char    *Location );*/
+				sapdbwa_SetHeader( reply,
+					"Transfer-Encoding",
+					"chunked" );
+				sapdbwa_SendHeader(reply);
+
 				/* send body header */
-				sapdbwa_SendBody(reply,
-								 WDVH_XML_HEADER,
-								 strlen(WDVH_XML_HEADER));
-				sapdbwa_SendBody(reply,
-								 WDVH_MULTI_STATUS_OPEN,
-								 strlen(WDVH_MULTI_STATUS_OPEN));
+				WDVH_sendBodyChunk(reply,WDVH_XML_HEADER);
+				WDVH_sendBodyChunk(reply,WDVH_MULTI_STATUS_OPEN);
 
 				/* set server prefix */
 				getWebDAVPrefix(wa,prefix);
@@ -282,29 +348,15 @@
 				}
 
 				/* response */
-				sapdbwa_SendBody(reply,
-								 WDVH_RESPONSE_OPEN,
-								 strlen(WDVH_RESPONSE_OPEN));
-				sapdbwa_SendBody(reply,
-								 getStrBufferContent(namespaceList),
-								 getStrBufferLength(namespaceList));
-				sapdbwa_SendBody(reply,
-								 WDVH_XML_CLOSE_TAG,
-								 strlen(WDVH_XML_CLOSE_TAG));
+				WDVH_sendBodyChunk(reply,WDVH_RESPONSE_OPEN);
+				WDVH_sendBodyChunk(reply,getStrBufferContent(namespaceList));
+				WDVH_sendBodyChunk(reply,WDVH_XML_CLOSE_TAG);
 							
 				/*		href */
-				sapdbwa_SendBody(reply,
-								 WDVH_HREF_OPEN,
-								 strlen(WDVH_HREF_OPEN));
-				sapdbwa_SendBody(reply,
-								 server,
-								 strlen(server));
-				sapdbwa_SendBody(reply,
-								 uri,
-								 strlen(uri));
-				sapdbwa_SendBody(reply,
-								 WDVH_HREF_CLOSE,
-								 strlen(WDVH_HREF_CLOSE));
+				WDVH_sendBodyChunk(reply,WDVH_HREF_OPEN);
+				WDVH_sendBodyChunk(reply,server);
+				WDVH_sendBodyChunk(reply,uri);
+				WDVH_sendBodyChunk(reply,WDVH_HREF_CLOSE);
 
 				/* loop at properties */
 				if (WDVCAPI_ProppatchGetFirstProcessedProperty(wdv,
@@ -323,62 +375,37 @@
 				}
 				while(propertyName) {
 					/*		propstat */
-					sapdbwa_SendBody(reply,
-									 WDVH_PROPSTAT_OPEN,
-									 strlen(WDVH_PROPSTAT_OPEN));
+					WDVH_sendBodyChunk(reply,WDVH_PROPSTAT_OPEN);
 
 					/*			prop */
-					sapdbwa_SendBody(reply,
-									 WDVH_PROP_OPEN,
-									 strlen(WDVH_PROP_OPEN));
+					WDVH_sendBodyChunk(reply,WDVH_PROP_OPEN);
 
 					/*				property name */
-					sapdbwa_SendBody(reply,
-									 WDVH_XML_OPEN_TAG,
-									 strlen(WDVH_XML_OPEN_TAG));
+					WDVH_sendBodyChunk(reply,WDVH_XML_OPEN_TAG);
                     /*?? MaO: Name space shortcut prefix? */
-					sapdbwa_SendBody(reply, "s", 1);
-					sapdbwa_SendBody(reply,
-									 propertyNamespaceShortcut,
-									 strlen(propertyNamespaceShortcut));
-					sapdbwa_SendBody(reply,
-									 WDVH_XML_COLON,
-									 strlen(WDVH_XML_COLON));
-					sapdbwa_SendBody(reply,
-									 propertyName,
-									 strlen(propertyName));
-					sapdbwa_SendBody(reply,
-									 WDVH_XML_SLASH,
-									 strlen(WDVH_XML_SLASH));
-					sapdbwa_SendBody(reply,
-										 WDVH_XML_CLOSE_TAG,
-										 strlen(WDVH_XML_CLOSE_TAG));
+					WDVH_sendBodyChunk(reply, "s");
+					WDVH_sendBodyChunk(reply,propertyNamespaceShortcut);
+					WDVH_sendBodyChunk(reply,WDVH_XML_COLON);
+					WDVH_sendBodyChunk(reply,propertyName);
+					WDVH_sendBodyChunk(reply,WDVH_XML_SLASH);
+					WDVH_sendBodyChunk(reply,WDVH_XML_CLOSE_TAG);
 					/*			/prop */
-					sapdbwa_SendBody(reply,
-									 WDVH_PROP_CLOSE,
-									 strlen(WDVH_PROP_CLOSE));
+					WDVH_sendBodyChunk(reply,WDVH_PROP_CLOSE);
 
 					/*			status */
-					sapdbwa_SendBody(reply,
-									 WDVH_STATUS_OPEN,
-									 strlen(WDVH_STATUS_OPEN));
-/* TODO: Status eintragen! */
+					WDVH_sendBodyChunk(reply,WDVH_STATUS_OPEN);
 					/*  if there was an error during proppatch all would be ok property
 						updates are "Failed Dependency" errors. All failed property
 						updates appear with the real error code */
 					if (iserror == WDVH_True) {
 						if (propertyStatus == WDVCAPI_PROPERTY_STATUS_OK) {
-							sapdbwa_SendBody(reply,
-											 WDVH_STATUS_424,
-											 strlen(WDVH_STATUS_424));
+							WDVH_sendBodyChunk(reply,WDVH_STATUS_424);
 							/*			/status */
-							sapdbwa_SendBody(reply,
-											WDVH_STATUS_CLOSE,
-											strlen(WDVH_STATUS_CLOSE));
+							WDVH_sendBodyChunk(reply,WDVH_STATUS_CLOSE);
 							/*			responsedescription */
-							sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_OPEN,0);
-							sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_424,0);
-							sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_CLOSE,0);
+							WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_OPEN);
+							WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_424);
+							WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_CLOSE);
 						} else if (propertyStatus == WDVCAPI_PROPERTY_STATUS_FAILED) {
 							/*  what error occured? */
 							WDVCAPI_GetErrorType(errorItem,&errorType);
@@ -393,17 +420,13 @@
 										sapdbwa_WriteLogMsg(wa, errorText);
 										sp77sprintf(errorText,1000,ERROR_PROPPATCH_500,errorMsg);
 										sapdbwa_WriteLogMsg(wa, errorText);
-										sapdbwa_SendBody(reply,
-														 WDVH_STATUS_500,
-														 strlen(WDVH_STATUS_500));
+										WDVH_sendBodyChunk(reply,WDVH_STATUS_500);
 										/*			/status */
-										sapdbwa_SendBody(reply,
-														WDVH_STATUS_CLOSE,
-														strlen(WDVH_STATUS_CLOSE));
+										WDVH_sendBodyChunk(reply,WDVH_STATUS_CLOSE);
 										/*			responsedescription */
-										sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_OPEN,0);
-										sapdbwa_SendBody(reply,errorMsg,0);
-										sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_CLOSE,0);
+										WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_OPEN);
+										WDVH_sendBodyChunk(reply,errorMsg);
+										WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_CLOSE);
 										break;
 									}
 								case WDVCAPI_ERR_TYPE_CAPI: 
@@ -420,32 +443,24 @@
 													sapdbwa_WriteLogMsg(wa, errorText);
 													sp77sprintf(errorText,1000,ERROR_PROPPATCH_500,errorMsg);
 													sapdbwa_WriteLogMsg(wa, errorText);
-													sapdbwa_SendBody(reply,
-																	 WDVH_STATUS_500,
-																	 strlen(WDVH_STATUS_500));
+													WDVH_sendBodyChunk(reply,WDVH_STATUS_500);
 													/*			/status */
-													sapdbwa_SendBody(reply,
-																	WDVH_STATUS_CLOSE,
-																	strlen(WDVH_STATUS_CLOSE));
+													WDVH_sendBodyChunk(reply,WDVH_STATUS_CLOSE);
 													/*			responsedescription */
-													sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_OPEN,0);
-													sapdbwa_SendBody(reply,errorMsg,0);
-													sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_CLOSE,0);
+													WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_OPEN);
+													WDVH_sendBodyChunk(reply,errorMsg);
+													WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_CLOSE);
 													break;
 												}
 											case WDVCAPI_ERR_CODE_COULD_NOT_REMOVE_PROPERTY:
 												{
-													sapdbwa_SendBody(reply,
-																	 WDVH_STATUS_409,
-																	 strlen(WDVH_STATUS_409));
+													WDVH_sendBodyChunk(reply,WDVH_STATUS_409);
 													/*			/status */
-													sapdbwa_SendBody(reply,
-																	WDVH_STATUS_CLOSE,
-																	strlen(WDVH_STATUS_CLOSE));
+													WDVH_sendBodyChunk(reply,WDVH_STATUS_CLOSE);
 													/*			responsedescription */
-													sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_OPEN,0);
-													sapdbwa_SendBody(reply,errorMsg,0);
-													sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_CLOSE,0);
+													WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_OPEN);
+													WDVH_sendBodyChunk(reply,errorMsg);
+													WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_CLOSE);
 													break;
 												}
 											default:
@@ -455,17 +470,13 @@
 													sapdbwa_WriteLogMsg(wa, errorText);
 													sp77sprintf(errorText,1000,ERROR_PROPPATCH_500,errorMsg);
 													sapdbwa_WriteLogMsg(wa, errorText);
-													sapdbwa_SendBody(reply,
-																	 WDVH_STATUS_500,
-																	 strlen(WDVH_STATUS_500));
+													WDVH_sendBodyChunk(reply,WDVH_STATUS_500);
 													/*			/status */
-													sapdbwa_SendBody(reply,
-																	WDVH_STATUS_CLOSE,
-																	strlen(WDVH_STATUS_CLOSE));
+													WDVH_sendBodyChunk(reply,WDVH_STATUS_CLOSE);
 													/*			responsedescription */
-													sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_OPEN,0);
-													sapdbwa_SendBody(reply,errorMsg,0);
-													sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_CLOSE,0);
+													WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_OPEN);
+													WDVH_sendBodyChunk(reply,errorMsg);
+													WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_CLOSE);
 													break;
 												}
 										}
@@ -485,17 +496,13 @@
 											case WDVH_ERR_CODE_NAMESPACE_NAME_TOO_LONG:			/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1123800 */
 											case WDVH_ERR_CODE_PROPERTY_NS_NAME_TOO_LONG:		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1123800 */
 												{
-													sapdbwa_SendBody(reply,
-																	 WDVH_STATUS_409,
-																	 strlen(WDVH_STATUS_409));
+													WDVH_sendBodyChunk(reply,WDVH_STATUS_409);
 													/*			/status */
-													sapdbwa_SendBody(reply,
-																	WDVH_STATUS_CLOSE,
-																	strlen(WDVH_STATUS_CLOSE));
+													WDVH_sendBodyChunk(reply,WDVH_STATUS_CLOSE);
 													/*			responsedescription */
-													sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_OPEN,0);
-													sapdbwa_SendBody(reply,errorMsg,0);
-													sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_CLOSE,0);
+													WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_OPEN);
+													WDVH_sendBodyChunk(reply,errorMsg);
+													WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_CLOSE);
 													break;
 												}
 											default:
@@ -505,17 +512,13 @@
 													sapdbwa_WriteLogMsg(wa, errorText);
 													sp77sprintf(errorText,1000,ERROR_PROPPATCH_500,errorMsg);
 													sapdbwa_WriteLogMsg(wa, errorText);
-													sapdbwa_SendBody(reply,
-																	 WDVH_STATUS_500,
-																	 strlen(WDVH_STATUS_500));
+													WDVH_sendBodyChunk(reply,WDVH_STATUS_500);
 													/*			/status */
-													sapdbwa_SendBody(reply,
-																	WDVH_STATUS_CLOSE,
-																	strlen(WDVH_STATUS_CLOSE));
+													WDVH_sendBodyChunk(reply,WDVH_STATUS_CLOSE);
 													/*			responsedescription */
-													sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_OPEN,0);
-													sapdbwa_SendBody(reply,errorMsg,0);
-													sapdbwa_SendBody(reply,WDVH_RESPONSEDESC_CLOSE,0);
+													WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_OPEN);
+													WDVH_sendBodyChunk(reply,errorMsg);
+													WDVH_sendBodyChunk(reply,WDVH_RESPONSEDESC_CLOSE);
 													break;
 												}
 										}
@@ -535,19 +538,13 @@
 							}
 						}
 					} else {
-						sapdbwa_SendBody(reply,
-										 WDVH_STATUS_OK,
-										 strlen(WDVH_STATUS_OK));
+						WDVH_sendBodyChunk(reply,WDVH_STATUS_OK);
 						/*			/status */
-						sapdbwa_SendBody(reply,
-										WDVH_STATUS_CLOSE,
-										strlen(WDVH_STATUS_CLOSE));
+						WDVH_sendBodyChunk(reply,WDVH_STATUS_CLOSE);
 					}
 
 					/*		/propstat */
-					sapdbwa_SendBody(reply,
-									 WDVH_PROPSTAT_CLOSE,
-									 strlen(WDVH_PROPSTAT_CLOSE));
+					WDVH_sendBodyChunk(reply,WDVH_PROPSTAT_CLOSE);
 					WDVCAPI_ProppatchGetNextProcessedProperty(wdv,
 															  proppatch,
 															  &propertyNamespaceShortcut,
@@ -558,14 +555,11 @@
 				}
 
 				/*	/response */
-				sapdbwa_SendBody(reply,
-								 WDVH_RESPONSE_CLOSE,
-								 strlen(WDVH_RESPONSE_CLOSE));
+				WDVH_sendBodyChunk(reply,WDVH_RESPONSE_CLOSE);
 
 
-				sapdbwa_SendBody(reply,
-								 WDVH_MULTI_STATUS_CLOSE,
-								 strlen(WDVH_MULTI_STATUS_CLOSE));
+				WDVH_sendBodyChunk(reply,WDVH_MULTI_STATUS_CLOSE);
+				WDVH_sendBodyChunkEnd(reply);
 				/* clean up */
 				destroyStrBuffer(namespaceList);
 			}
@@ -605,6 +599,13 @@
 		XML_Parser						p;
 
 
+		if (wa==NULL)
+			return ERROR_500;
+		if (request==NULL || wdv==NULL || proppatch==NULL || iserror==NULL) {
+			sapdbwa_WriteLogMsg(wa, "proppatchParseRequestBody:Uninitialized Pointers\n");
+			return ERROR_500;
+		}
+
 		/* Initialization */
 		p =  XML_ParserCreateNS("UTF-8",WDVH_NS_SEPARATOR);
 		if (!p) {
@@ -684,13 +685,13 @@
 					}
 					if (newUserData->propertyShortValueStarted) {															/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 						if (newUserData->propertyValueEndPos != WDVH_MAX_XML_BUFFER_LEN)									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
-							memcpy(currentBuffer+newUserData->propertyValueEndPos,"\0",1);									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
+							SAPDB_memcpy(currentBuffer+newUserData->propertyValueEndPos,"\0",1);									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 						if (newUserData->propertyValueEndPos+newUserData->bufferStart < newUserData->bufferEnd)				/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 							newUserData->propertyShortValueStarted = WDVH_False;											/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 						if ((getStrBufferLength(newUserData->propertyShortValue) +
 							newUserData->propertyValueEndPos - newUserData->propertyValueStartPos + 1) >WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 							WDVCAPI_SetErrorItemEx(&newUserData->error,WDVCAPI_ERR_TYPE_WDVHANDLER,WDVH_ERR_CODE_PROPERTY_TOO_LONG,WDVH_ERR_TEXT_PROPERTY_TOO_LONG,NULL,0); /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
-							/*sp77sprintf(errmsg,1000,"Lock owner exceeds	supported length of %d characters.",WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);	/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
+							/*sp77sprintf(errmsg,1000,"Lock owner exceeds	supported length of %d characters.",WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);*/	/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 						} else {																						/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 							appendStrBuffer(newUserData->propertyShortValue,currentBuffer+newUserData->propertyValueStartPos);	/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 						}																								/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
@@ -756,6 +757,9 @@
 		WDVH_XmlUserData				userData;
 		XML_Parser						p;
 
+		if (data==NULL || el==NULL || attr==NULL)
+			return;
+
 		userData = (WDVH_XmlUserData)data;
 		p = xmlParserUserDataGetParser(userData);
 		parserState = xmlParserUserDataGetProppatchParserState(userData);
@@ -794,7 +798,7 @@
 				   property to remove
 				   --> check wether property is correct EMPTY tag otherwise syntax error */ 
 				/* Correction Begin http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123360
-				if (xmlParserCheckEmptyTag(p,userData,tag)) {
+				if (xmlParserCheckEmptyTag(p,userData,tag)) {*/
 					/* add property to list for removal 
 					if(!WDVCAPI_ProppatchRemove(userData->wdv,
 												userData->proppatchHandle,
@@ -802,7 +806,7 @@
 												tag)) {
 						parserState->error = WDVH_XML_PARSER_PROPPATCH_ERROR;
 					}
-					/* stay at this state 
+					*//* stay at this state 
 				} else {
 					parserState->error = WDVH_XML_PARSER_SYNTAX_ERROR;
 				}*/
@@ -869,14 +873,14 @@
 				   property to remove
 				   --> check wether property is correct EMPTY tag otherwise syntax error */ 
 				/* Correction Begin http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123360
-				if (xmlParserCheckEmptyTag(p,userData,tag)) {
+				if (xmlParserCheckEmptyTag(p,userData,tag)) {*/
 					/* add property to list for removal 
 					if(!WDVCAPI_ProppatchRemove(userData->wdv,
 												userData->proppatchHandle,
 												ns,
 												tag)) {
 						parserState->error = WDVH_XML_PARSER_PROPPATCH_ERROR;
-					}
+					}*/
 					/* stay at this state 
 				} else {
 					parserState->error = WDVH_XML_PARSER_SYNTAX_ERROR;
@@ -982,6 +986,9 @@
 		WDVH_Char						errmsg[1001];
 
 
+		if (data==NULL || el==NULL)
+			return;
+
 		userData = (WDVH_XmlUserData)data;
 		parserState = xmlParserUserDataGetProppatchParserState(userData);
 
@@ -1050,7 +1057,7 @@
 			/* proppatchReadShortValue(userData,tag);*/		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 			/* append this last snippet to the string buffer */
 			if (userData->propertyValueEndPos != WDVH_MAX_XML_BUFFER_LEN)									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
-				memcpy(userData->currentBuff+userData->propertyValueEndPos,"\0",1);									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
+				SAPDB_memcpy(userData->currentBuff+userData->propertyValueEndPos,"\0",1);									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 			if (userData->propertyValueEndPos+userData->bufferStart < userData->bufferEnd)					/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 				userData->propertyShortValueStarted = WDVH_False;											/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 			if ((getStrBufferLength(userData->propertyShortValue) +
@@ -1146,7 +1153,7 @@
 			/* proppatchReadShortValue(userData,tag);*/		/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 			/* append this last snippet to the string buffer */
 			if (userData->propertyValueEndPos != WDVH_MAX_XML_BUFFER_LEN)									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
-				memcpy(userData->currentBuff+userData->propertyValueEndPos,"\0",1);									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
+				SAPDB_memcpy(userData->currentBuff+userData->propertyValueEndPos,"\0",1);									/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 			if (userData->propertyValueEndPos+userData->bufferStart < userData->bufferEnd)					/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 				userData->propertyShortValueStarted = WDVH_False;											/* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1123800 */
 			if ((getStrBufferLength(userData->propertyShortValue) +
@@ -1300,6 +1307,9 @@
 		WDVH_XmlUserData	xmlUserData;
 		XML_Parser			p;
 
+		if (data==NULL || prefix==NULL || uri==NULL)
+			return;
+
 		if (prefix)
 			strcpy(xmlNamespace.prefix,prefix);
 		strcpy(xmlNamespace.uri,uri);
@@ -1326,6 +1336,9 @@
 		WDVH_XmlUserData	xmlUserData;
 		XML_Parser			p;
 
+		if (data==NULL || prefix==NULL)
+			return;
+
 		if (prefix)
 			strcpy(xmlNamespace.prefix,prefix);
 		xmlUserData = (WDVH_XmlUserData)data;
@@ -1412,21 +1425,21 @@
 /*				if (data->proppatchValueStartPos >= startPosPrevBuff) {
 */					/* read til property value start pos */
 /*					if(-pos3+pos1-1 > WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {
-						memcpy(data->proppatchShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
-						memcpy(data->proppatchShortValue-pos3+pos1-1,"\0",1);
+						SAPDB_memcpy(data->proppatchShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
+						SAPDB_memcpy(data->proppatchShortValue-pos3+pos1-1,"\0",1);
 					} else {
-						memcpy(data->proppatchShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3+pos1-1);
-						memcpy(data->proppatchShortValue-pos3+pos1-1,"\0",1);
+						SAPDB_memcpy(data->proppatchShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3+pos1-1);
+						SAPDB_memcpy(data->proppatchShortValue-pos3+pos1-1,"\0",1);
 					}
 				} else {
 */					/* read til buffer start */
 /*					pos2 = pos0 - ((data->bufferChunks - 2)* WDVH_MAX_XML_BUFFER_LEN);
 					if (pos2 > WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {
-						memcpy(data->proppatchShortValue,data->previousBuff,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
-						memcpy(data->proppatchShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN,"\0",1);
+						SAPDB_memcpy(data->proppatchShortValue,data->previousBuff,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
+						SAPDB_memcpy(data->proppatchShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN,"\0",1);
 					} else {
-						memcpy(data->proppatchShortValue,data->previousBuff,pos2);
-						memcpy(data->proppatchShortValue+pos2,"\0",1);
+						SAPDB_memcpy(data->proppatchShortValue,data->previousBuff,pos2);
+						SAPDB_memcpy(data->proppatchShortValue+pos2,"\0",1);
 					}
 				}
 			} else {
@@ -1441,31 +1454,31 @@
 /*				if (data->proppatchValueStartPos >= startPosCurrBuff) {
 */					/* read til property value start pos */
 /*					if (pos1-pos3 > WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {
-						memcpy(data->proppatchShortValue,data->currentBuff+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
-						memcpy(data->proppatchShortValue+pos1-pos3,"\0",1);
+						SAPDB_memcpy(data->proppatchShortValue,data->currentBuff+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
+						SAPDB_memcpy(data->proppatchShortValue+pos1-pos3,"\0",1);
 					} else {
-						memcpy(data->proppatchShortValue,data->currentBuff+pos3+1,pos1-pos3);
-						memcpy(data->proppatchShortValue+pos1-pos3,"\0",1);
+						SAPDB_memcpy(data->proppatchShortValue,data->currentBuff+pos3+1,pos1-pos3);
+						SAPDB_memcpy(data->proppatchShortValue+pos1-pos3,"\0",1);
 					}
 				} else {
 */					/* read first part of property value from previous buffer */
 /*					if (-pos3+pos1 > WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {
 						if (-pos3 > WDVH_MAX_PROPERTY_SHORT_VALUE_LEN) {
-							memcpy(data->proppatchShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
-							memcpy(data->proppatchShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN,"\0",1);
+							SAPDB_memcpy(data->proppatchShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,WDVH_MAX_PROPERTY_SHORT_VALUE_LEN);
+							SAPDB_memcpy(data->proppatchShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN,"\0",1);
 						} else {
 							memcpy(data->proppatchShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3);
 */							/* read til buffer start */
-/*							memcpy(data->proppatchShortValue-pos3-1,
+/*							SAPDB_memcpy(data->proppatchShortValue-pos3-1,
 								   data->currentBuff,
 								   WDVH_MAX_PROPERTY_SHORT_VALUE_LEN+pos3);
-							memcpy(data->proppatchShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN,"\0",1);
+							SAPDB_memcpy(data->proppatchShortValue+WDVH_MAX_PROPERTY_SHORT_VALUE_LEN,"\0",1);
 						}
 					} else {
-						memcpy(data->proppatchShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3);
+						SAPDB_memcpy(data->proppatchShortValue,data->previousBuff+WDVH_MAX_XML_BUFFER_LEN+pos3+1,-pos3);
 */						/* read til buffer start */
-/*						memcpy(data->proppatchShortValue-pos3-1,data->currentBuff,pos1);
-						memcpy(data->proppatchShortValue+pos1-pos3,"\0",1);
+/*						SAPDB_memcpy(data->proppatchShortValue-pos3-1,data->currentBuff,pos1);
+						SAPDB_memcpy(data->proppatchShortValue+pos1-pos3,"\0",1);
 					}
 				}
 			}
@@ -1482,6 +1495,9 @@
 		WDVH_Int4			pos0,pos1;
 		XML_Parser			p;
 
+		if (data==NULL || el==NULL)
+			return;
+
 		p = xmlParserUserDataGetParser(data);
 		/* called from start tag handler after discovery of a owner start tag */
 		/* check wether end tag is in current parse buffer */
@@ -1533,6 +1549,9 @@
 		WDVH_Int4			pos0;
 		XML_Parser			p;
 
+		if (data==NULL || el==NULL)
+			return;
+
 		p = xmlParserUserDataGetParser(data);
 		/* called from start tag handler after discovery of a owner start tag */
 		/* check wether end tag is in current parse buffer */
