@@ -53,6 +53,7 @@
 #include "SAPDB/WebDAV/XIS/WDVXIS_ErrorCodes.h"
 
 #include "SAPDB/XML/Memory/XMLMem_Memory.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /****************************************************************************
  ***
@@ -379,7 +380,7 @@ SAPDB_Bool SAPDBXML_SetIndexValue( XMLSP_Session   hSessionIndexStore,
     }
 
 	/* Set current input values */
-    memcpy(hIndexStoreData->resourceId, resourceId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(hIndexStoreData->resourceId, resourceId, XMLID_MAX_ID_LEN);
     Com_StrMaxCopy(hIndexStoreData->indexNameSpace, indexNameSpace, WDVXIS_MAX_NAME_SPACE_LEN);
     Com_StrMaxCopy(hIndexStoreData->indexName, indexName, WDVXIS_MAX_PROPERTY_NAME_PREFIX_LEN);
     Com_StrMaxCopy(hIndexStoreData->indexValue, indexValue, WDVXIS_MAX_PROPERTY_SHORT_VALUE_LEN);
@@ -845,8 +846,8 @@ SAPDB_Bool CreateProperty( XMLSP_Session    hIndexStoreSession,
     }
 
     /* Set input parameters */
-    memcpy(hIndexStoreData->propertyId, propertyId, XMLID_MAX_ID_LEN);
-    memcpy(hIndexStoreData->nameSpaceId, nameSpaceId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(hIndexStoreData->propertyId, propertyId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(hIndexStoreData->nameSpaceId, nameSpaceId, XMLID_MAX_ID_LEN);
     Com_StrMaxCopy(hIndexStoreData->indexName, propertyName, WDVXIS_MAX_PROPERTY_NAME_PREFIX_LEN);
 
     /* Execute statement */
@@ -956,7 +957,7 @@ SAPDB_Bool GetNameSpaceId( XMLSP_Session    hIndexStoreSession,
     }
 
     /* Set return value */
-    memcpy(nameSpaceId, hIndexStoreData->nameSpaceId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(nameSpaceId, hIndexStoreData->nameSpaceId, XMLID_MAX_ID_LEN);
 
 	/* Return */
 	return SAPDB_TRUE;
@@ -1041,7 +1042,7 @@ SAPDB_Bool CreateNameSpace( XMLSP_Session   hIndexStoreSession,
     }
 
     /* Set input parameters */
-    memcpy(hIndexStoreData->nameSpaceId, nameSpaceId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(hIndexStoreData->nameSpaceId, nameSpaceId, XMLID_MAX_ID_LEN);
     Com_StrMaxCopy(hIndexStoreData->indexNameSpace, nameSpace, WDVXIS_MAX_NAME_SPACE_LEN);
 
     /* Execute statement */
@@ -1163,7 +1164,7 @@ SAPDB_Bool GetURIById( SQLHDBC               hDBCDocStore,
 
     /* Get whole uri */
     rootFound = SAPDB_FALSE;
-    memcpy(currentId, hSearchData->docId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(currentId, hSearchData->docId, XMLID_MAX_ID_LEN);
     do {
         /* Get parent of docId */
         if (!GetParentById(hDBCDocStore, hEnvDocStore, hSearchData, currentId, parentId, parentName, hError)) {
@@ -1182,7 +1183,7 @@ SAPDB_Bool GetURIById( SQLHDBC               hDBCDocStore,
             sp77sprintf(tmpUri, WDVXIS_MAX_URI_STRING_LEN, "/%s", newUri);
         } else {
             sp77sprintf(tmpUri, WDVXIS_MAX_URI_STRING_LEN, "%s/%s", parentName, newUri);
-            memcpy(currentId, parentId, XMLID_MAX_ID_LEN);
+            SAPDB_memcpy(currentId, parentId, XMLID_MAX_ID_LEN);
         }
         Com_StrMaxCopy(newUri, tmpUri, WDVXIS_MAX_URI_STRING_LEN);
     } while (rootFound != SAPDB_TRUE);
@@ -1283,7 +1284,7 @@ SAPDB_Bool GetParentById( SQLHDBC               hDBCDocStore,
     }
 
     /* Set input parameters */
-    memcpy(hSearchData->resourceId, resourceId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(hSearchData->resourceId, resourceId, XMLID_MAX_ID_LEN);
 
 	/* Execute SQL */
 	sqlRC = SQLExecute(hSearchData->hStmtGetParent);
@@ -1313,7 +1314,7 @@ SAPDB_Bool GetParentById( SQLHDBC               hDBCDocStore,
 		return SAPDB_FALSE;
 	}
 
-    memcpy(parentId, hSearchData->parentId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(parentId, hSearchData->parentId, XMLID_MAX_ID_LEN);
     Com_StrMaxCopy(parentName, hSearchData->parentName, WDVXIS_MAX_RESOURCE_NAME_LEN);
 
 	/* Return */
