@@ -48,6 +48,7 @@
  ****************************************************************************/
 
 #include "SAPDB/SAPDBCommon/SAPDB_Types.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /*---------------------------------------------------------------------------*/
 
@@ -440,7 +441,7 @@ SAPDB_Bool DocClass_InitDocClass( XMLDC_DocClass            hDocClass,
 
 
     /* Set doc class id */
-    memcpy(hDocClass->id, docClassId, XMLDC_MAX_DOC_CLASS_ID_LEN);
+    SAPDB_memcpy(hDocClass->id, docClassId, XMLDC_MAX_DOC_CLASS_ID_LEN);
 
     /* Initialize XPath handle */
     hDocClass->hXPath = NULL;
