@@ -53,6 +53,7 @@
 
 #include "SAPDB/XML/Memory/XMLMem_Memory.h"
 #include "SAPDB/XML/Common/XMLCom_Common.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /***********************************************************************
 
@@ -221,7 +222,7 @@ SAPDB_Bool DocStore_SetState( XMLSP_Session         hSessionDocStore,
     }
 
 	/* Set current input values */
-    memcpy(hDocStoreData->stateResourceId, docId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(hDocStoreData->stateResourceId, docId, XMLID_MAX_ID_LEN);
 
     switch (state) {
     case XMLIDX_STATE_DOC_PARSING_HAS_BEEN_FINISHED:
@@ -544,10 +545,10 @@ SAPDB_Bool  DocStore_SelectNextDoc( XMLSP_Session     hSessionDocStore,
 
     /* Convert doc class id */
     Id_StringAsId(hDocStoreData->docClassIdString, hDocStoreData->docClassId);
-    memcpy(docClassId, hDocStoreData->docClassId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(docClassId, hDocStoreData->docClassId, XMLID_MAX_ID_LEN);
 
     /* Set doc id */
-    memcpy(docId, hDocStoreData->cId, XMLID_MAX_ID_LEN);
+	SAPDB_memcpy(docId, hDocStoreData->cId, XMLID_MAX_ID_LEN);
     
     /* Set error code to ok */
     Error_Clear(hError);
@@ -594,7 +595,7 @@ SAPDB_Bool DocStore_ReadDoc( XMLIdx_Work      hWork,
     }
 
     /* Set input parameters */
-    memcpy(hDocStoreData->getContentResourceId, hWork->docId, XMLID_MAX_ID_LEN);
+    SAPDB_memcpy(hDocStoreData->getContentResourceId, hWork->docId, XMLID_MAX_ID_LEN);
 
     /* If no statement handle exists, create a new one */
     if (hDocStoreData->hStmtGetContent == 0) {
