@@ -38,6 +38,7 @@
 
 
 	#include "SAPDB/XML/IndexingMetadataAPI/XMLIMAPI_DocClass.h"
+	#include "SAPDBCommon/SAPDB_string.h"
 
 
 	#define SQL_INSERT_DOCCLASS \
@@ -306,7 +307,7 @@
 		}
 
 		/* set values of parameters */
-		memcpy(handle->stmtDocClassID,id,XMLIMAPI_MAX_ID_LEN);
+		SAPDB_memcpy(handle->stmtDocClassID,id,XMLIMAPI_MAX_ID_LEN);
 		strcpy(handle->stmtDocClassParam.description,description);
 
 		/* execute statement */
@@ -397,7 +398,7 @@
 		}
 
 		/* set values of parameters */
-		memcpy(handle->stmtDocClassID,id,XMLIMAPI_MAX_ID_LEN);
+		SAPDB_memcpy(handle->stmtDocClassID,id,XMLIMAPI_MAX_ID_LEN);
 
 		/* execute statement */
 		if ((rc = SQLExecute(handle->stmtDocClassDelete)) != SQL_SUCCESS) {
@@ -507,7 +508,7 @@
 		}
 
 		/* set values of parameters */
-		memcpy(handle->stmtDocClassID,id,XMLIMAPI_MAX_ID_LEN);
+		SAPDB_memcpy(handle->stmtDocClassID,id,XMLIMAPI_MAX_ID_LEN);
 
 		/* select execute statement */
 		if ((rc = SQLExecute(handle->stmtGetDocClass)) != SQL_SUCCESS) {
@@ -704,7 +705,7 @@
 		}
 
 		/* set values of parameters */
-		memcpy(handle->stmtDocClassID,id,XMLIMAPI_MAX_ID_LEN);
+		SAPDB_memcpy(handle->stmtDocClassID,id,XMLIMAPI_MAX_ID_LEN);
 
 		/* select execute statement */
 		if ((rc = SQLExecute(handle->stmtGetDocClassName)) != SQL_SUCCESS) {
@@ -1504,7 +1505,7 @@
 		}
 
 		/* set values of parameters */
-		memcpy(handle->stmtDocClassID,docClassId,XMLIMAPI_MAX_ID_LEN);
+		SAPDB_memcpy(handle->stmtDocClassID,docClassId,XMLIMAPI_MAX_ID_LEN);
 
 		/* execute statement */
 		if ((rc = SQLExecute(handle->stmtDocClassDeleteAssign)) != SQL_SUCCESS) {
@@ -1584,7 +1585,7 @@
 		}
 
 		/* set values of parameters */
-		memcpy(handle->stmtDocClassID,docClassId,XMLIMAPI_MAX_ID_LEN);
+		SAPDB_memcpy(handle->stmtDocClassID,docClassId,XMLIMAPI_MAX_ID_LEN);
 
 		/* loop the list of xml indexes to insert them one by one */
 		if (XMLIMAPI_XmlIndexListGetFirst(list,&__xmlIndex)) {
