@@ -37,6 +37,7 @@
 *****************************************************************************/
 
 	#include "SAPDB/XML/IndexingMetadataAPI/XMLIMAPI_Service.h"
+	#include "SAPDBCommon/SAPDB_string.h"
 
 
 	#define SQL_INSERT_SERVICE \
@@ -604,7 +605,7 @@
 		}
 
 		/* set document class id */
-		memcpy(handle->stmtServiceParam.ServiceID,serviceId,XMLIMAPI_MAX_ID_LEN);
+		SAPDB_memcpy(handle->stmtServiceParam.ServiceID,serviceId,XMLIMAPI_MAX_ID_LEN);
 
 		/* select execute statement */
 		if ((rc = SQLExecute(handle->stmtServiceGet)) != SQL_SUCCESS) {
