@@ -37,6 +37,7 @@
 *****************************************************************************/
 
 	#include "SAPDB/XML/IndexingMetadataAPI/XMLIMAPI_SessionPools.h"
+	#include "SAPDBCommon/SAPDB_string.h"
 
 
 	#define SQL_GET_SESSIONPOOLS \
@@ -781,7 +782,7 @@
 		}
 
 		/* set document class id */
-		memcpy(handle->stmtSessionPoolParam.PoolID,poolId,XMLIMAPI_MAX_ID_LEN);
+		SAPDB_memcpy(handle->stmtSessionPoolParam.PoolID,poolId,XMLIMAPI_MAX_ID_LEN);
 
 		/* select execute statement */
 		if ((rc = SQLExecute(handle->stmtSessionPoolGet)) != SQL_SUCCESS) {
