@@ -38,6 +38,7 @@
 
 
 	#include "SAPDB/XML/IndexingMetadataAPI/XMLIMAPI_XieAdmin.h"
+	#include "SAPDBCommon/SAPDB_string.h"
 
 	#define SQL_INSERT_XIE \
 			"INSERT INTO XML_INDEXENGINE \
@@ -1223,7 +1224,7 @@
 		}
 
 		/* set Xie Id */
-		memcpy(handle->stmtXieAdminParam.ID,xieadminId,XMLIMAPI_MAX_ID_LEN);
+		SAPDB_memcpy(handle->stmtXieAdminParam.ID,xieadminId,XMLIMAPI_MAX_ID_LEN);
 
 		/* execute statement */
 		if ((__rc = SQLExecute(handle->stmtXieGet)) != SQL_SUCCESS) {
