@@ -43,6 +43,7 @@
 
  #include <string.h>
  #include "SAPDB/XML/XML_UTF8.h"
+ #include "SAPDBCommon/SAPDB_string.h"
 
 /*------------------------------------------------------------------------------*/
 static SAPDB_Int UTF8SingleCharSize[256] =
@@ -244,7 +245,7 @@ XML_UnicodeCharCode XML_UTF8CharToCode( const SAPDB_UTF8 *text )
 SAPDB_Int XML_UTF8GetChar(SAPDB_UTF8 *dest, const SAPDB_UTF8 *src)
 {
     SAPDB_Int len = XML_UTF8SingleCharLength( src );
-    memcpy ( dest, src, len );
+    SAPDB_memcpy ( dest, src, len );
     return len;
 }
 
