@@ -41,6 +41,7 @@
 
 *****************************************************************************/
 #include <string.h>
+#include "SAPDBCommon/SAPDB_string.h"
 
 #include "SAPDB/XML/XPathEngine/XMLXPath_ParseExpr.h"
 #include "SAPDB/XML/XPathEngine/XMLXPath_Memory.h"
@@ -220,7 +221,7 @@ isAxisnameSup( XMLXPath_ContextPtr pCtx,
     case XMLXPath_TOK_SLASH:
         {
             SAPDB_UTF8 axisName[32];
-            memcpy( &axisName[0], item->startChar, item->len );
+            SAPDB_memcpy( &axisName[0], item->startChar, item->len );
             axisName[ item->len ] = '\0';
 
             if ( strcmp( "child", (char*)axisName ) &&
@@ -355,7 +356,7 @@ isNodenameSup( XMLXPath_ContextPtr pCtx,
     if ( SAPDB_TRUE == rc )
     {
         SAPDB_UTF8 nodeName[32];
-        memcpy( (char*)&nodeName[0], (char*)item->startChar, item->len );
+        SAPDB_memcpy( (char*)&nodeName[0], (char*)item->startChar, item->len );
         nodeName[ item->len ] = '\0';
         if ( !strcmp( "text", (char*)nodeName) )
             return SAPDB_TRUE;
@@ -482,7 +483,7 @@ incrStepsUsed(XMLXPath_ContextPtr    pCtx,
     {
         XMLXPath_StepPtr firstStepOld = p->firstStep;
         rc = allocExecDesc( p );
-        memcpy( p->firstStep, firstStepOld,
+        SAPDB_memcpy( p->firstStep, firstStepOld,
                 sizeof( XMLXPath_Step ) * p->stepsAllocated );
         XMLXPath_Mem_Free( firstStepOld );
     };
@@ -563,7 +564,7 @@ parseExpr( XMLXPath_ContextPtr   pCtx,
             if ( SAPDB_TRUE == isAxisnameSup( pCtx, item, prevTok ))
             {
                 SAPDB_UTF8 axisName[32];
-                memcpy( &axisName[0], item->startChar, item->len );
+                SAPDB_memcpy( &axisName[0], item->startChar, item->len );
                 axisName[ item->len ] = '\0';
                 if ( isPredicate )
                 {
