@@ -45,6 +45,7 @@
 #include "SAPDB/XML/XPathEngine/XMLXPath_Memory.h"
 #include "SAPDB/XML/XPathEngine/XMLXPath_Error.h"
 #include "SAPDB/XML/XML_UTF8.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /*------------------------------------------------------------------------------*/
 /* prototypes for all only locally defined and used functions                   */
@@ -155,7 +156,7 @@ charIdx( XMLXPath_ContextPtr   pCtx,
         index->idxList->textUsed = SAPDB_TRUE;
         if ( len >= sizeof( pCtx->tmpWorkBuf ) )
           pbuf = XMLXPath_Mem_Malloc( len + 1 );
-        memcpy( pbuf, s, len );
+        SAPDB_memcpy( pbuf, s, len );
         *(pbuf + len) = '\0';
         pwc = XML_UTF8NormalizeValue( pbuf );
         if (( pwc - pbuf ) < len )
@@ -406,7 +407,7 @@ checkAttributeNodes( XMLXPath_ContextPtr  pCtx,
     
     if ( 1 == actualStep->nameLen && '*' == *actualStep->name )
       starLoop = 1;
-    memcpy( &pCtx->tmpWorkBuf[0], actualStep->name, actualStep->nameLen );
+    SAPDB_memcpy( &pCtx->tmpWorkBuf[0], actualStep->name, actualStep->nameLen );
     pCtx->tmpWorkBuf[ actualStep->nameLen ] = '\0';
     for ( i=0; atts[i]; i = i+2 )
     {
@@ -459,7 +460,7 @@ startElemIdxStep( XMLXPath_ContextPtr  pCtx,
   if ( actualStep->predicate )
   {
     int i;
-    memcpy( (char*)&pCtx->tmpWorkBuf[0], (char*)actualStep->predicate->name,
+    SAPDB_memcpy( (char*)&pCtx->tmpWorkBuf[0], (char*)actualStep->predicate->name,
       actualStep->predicate->nameLen );
     pCtx->tmpWorkBuf[ actualStep->predicate->nameLen ] = '\0';
     brc = SAPDB_FALSE;
@@ -472,7 +473,7 @@ startElemIdxStep( XMLXPath_ContextPtr  pCtx,
         '*' == *actualStep->predicate->name ) ||
         !strcmp((char*)&pCtx->tmpWorkBuf[0],(char*)atts[i] ))
       {
-        memcpy( &pCtx->tmpWorkBuf[0], actualStep->predicate->value,
+        SAPDB_memcpy( &pCtx->tmpWorkBuf[0], actualStep->predicate->value,
           actualStep->predicate->valueLen );
         pCtx->tmpWorkBuf[ actualStep->predicate->valueLen ] = '\0';
         if ((( XMLXPath_StepOp_EQ == actualStep->predicate->selectOP ) &&
