@@ -46,6 +46,7 @@
 #include "SAPDB/XML/XPathEngine/XMLXPath_Error.h"
 #include "SAPDB/XML/XPathEngine/XMLXPath_Memory.h"
 #include "SAPDB/XML/XML_UTF8.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /*------------------------------------------------------------------------------*/
 /* prototypes of local defined/used functions                                   */
@@ -524,7 +525,7 @@ getNextItem( XMLXPath_ContextPtr   pCtx,
         if ( !item )
             return NULL;
         itemList->tokenAllocated  = itemList->tokenAllocated + 10;
-        memcpy( (char*)item, (char*)itemList->firstToken,
+        SAPDB_memcpy( (char*)item, (char*)itemList->firstToken,
                 sizeof( XMLXPath_Item ) * itemList->tokenUsed );
         XMLXPath_Mem_Free( itemList->firstToken );
         itemList->firstToken = item;
@@ -578,7 +579,7 @@ isNodeTest( SAPDB_UTF8 *name,
 {
     unsigned int i;
     SAPDB_UTF8 nodeName[32];
-    memcpy( (char*)&nodeName[0], (char*)name, len );
+    SAPDB_memcpy( (char*)&nodeName[0], (char*)name, len );
     nodeName[ len ] = '\0';
     for ( i=0; i < XPathNodeTypeNamesDim; i++ )
     {
@@ -595,7 +596,7 @@ isOperatorName( SAPDB_UTF8 *name,
 {
     unsigned int i;
     SAPDB_UTF8 opName[32];
-    memcpy( (char*)&opName[0], (char*)name, len );
+    SAPDB_memcpy( (char*)&opName[0], (char*)name, len );
     opName[ len ] = '\0';
     for ( i=0; i < XPathOperatorNamesDim; i++ )
     {
