@@ -284,9 +284,6 @@ HRESULT LVC_Dispatcher::dispGetDispAddr(const GUID &guid_coclsid,
   const IID &guid_iid, tsp00_Uint4 session, tsp00_Addr &idispatch,
   tsp00_Addr &ifactory, void *params)
 {
-  // TODO: in ProcServer case (session-specific parameter), return
-  // LVC_ProcServerExtender COM object, create in hash, if it doesn't exist yet
-
   LVC_DispatcherCOMClassHashPtr HashPtr = LVC_Dispatcher::getDCOMMemory().lphead;
   if (NULL == HashPtr) 
     return RC_DISPADDR_FAILED;
@@ -760,10 +757,6 @@ HRESULT LVC_Dispatcher::dispDispatch(IliveCacheSink *sink, IUnknown *disp_addr,
   tak_dbproc_stack_param_debug *param_data, bool monitor, tsp00_Int2 monitor_slot_id,
   void *parms, void *setupParms)
 {
-  // TODO: if procserver case, call special method of LVC_ProcServerExtender (we
-  // know, COM object IS LVC_ProcServerExtender, if it's not, it's an error - use
-  // QueryInterface to check)
-
   HRESULT         hresult = S_OK;
 
 #if defined(LINUX) 
@@ -775,11 +768,8 @@ HRESULT LVC_Dispatcher::dispDispatch(IliveCacheSink *sink, IUnknown *disp_addr,
   IUnknown *lpClassPtr;
   // if dispid < 0, use sysRequest
   t_lpIFunc lp_IFunc = dispid >= 0 ? LVC_DispatcherVTableCompute(dispid, lpClassPtr, disp_addr) : NULL;
-#if !defined	(_WIN64)
-  // Get stack size for all platforms except WIN64
-  tsp00_Uint4 param_stack_size = dispComputeStackSize(param_num, param_data);
-#else   // !_WIN64
-  // Get stack for WIN64
+#if defined	(_WIN64) && defined (_M_IA64)
+  // Get stack for WINDOWS IA64
   //Define verwenden
   tak_dbproc_stack_param	param_stack[cak260max_params];
   //Define verwenden
@@ -787,8 +777,11 @@ HRESULT LVC_Dispatcher::dispDispatch(IliveCacheSink *sink, IUnknown *disp_addr,
   int				len = 0;
   int				flen = 0;
   dispComputeWin64Stack(param_num, param_data, param_stack, fparam_stack, len, flen);
-#endif  // _WIN64
-
+#else
+  // Get stack size for all platforms except WINDOWS IA64
+  tsp00_Uint4 param_stack_size = dispComputeStackSize(param_num, param_data);
+#endif
+ 
   /*--* +++ FF Change Request 1000139  +++ *---------------------------------*/
   IliveCacheSpec  *pILC;
   tsp00_Int4      start_time_sec; 
@@ -852,11 +845,15 @@ HRESULT LVC_Dispatcher::dispDispatch(IliveCacheSink *sink, IUnknown *disp_addr,
         (unsigned char*) param_data[1].dbg_value.ptr());
       pILC->Release();
     } else if (dispCheckStack(method_name_ascii, param_num, param_data)) {
+#if defined(SUN) && defined(BIT64)
+      /* PTS 1109493 */ /* round up to multiple of 16 bytes */
+      param_stack_size = ( param_stack_size + 15 ) & (~15);
+#endif
       /*
-      * following has been coded in Assembler
-      * to push the exact length of needed parameters
-      */
-#if defined (_WIN64)
+       * following has been coded in Assembler
+       * to push the exact length of needed parameters
+       */
+#if defined(_WIN64) && defined(_M_IA64)
       sql38aVdcomCall (
         lpClassPtr,	       /* class this pointer */ 
         (PROC)lp_IFunc,    /* entry-point */
@@ -865,12 +862,15 @@ HRESULT LVC_Dispatcher::dispDispatch(IliveCacheSink *sink, IUnknown *disp_addr,
         fparam_stack,      /* call-stack (floats) */ 
         flen,              /* call-stack length (floats) */
         0);                /* param-stack size, is not needed in this case */
+#elif defined(_WIN64) && defined(_M_AMD64)
+	  sql38aVdcomCall(
+		(void *)lpClassPtr,
+        (void *)lp_IFunc,
+        (void *)param_data,
+        param_num * sizeof(tak_dbproc_stack_param_debug),
+        param_stack_size);
 #else
-#if defined(SUN) && defined(BIT64)
-      /* PTS 1109493 */ /* round up to multiple of 16 bytes */
-      param_stack_size = ( param_stack_size + 15 ) & (~15);
-#endif
-      sql38aVdcomCall(lpClassPtr,  /* class this pointer*/ 
+	  sql38aVdcomCall(lpClassPtr,  /* class this pointer*/ 
         (PROC)lp_IFunc,            /* entry-point       */
         param_data,                /* call-stack        */ 
         param_num * sizeof(tak_dbproc_stack_param_debug),    /* call-stack length */
@@ -888,7 +888,7 @@ HRESULT LVC_Dispatcher::dispDispatch(IliveCacheSink *sink, IUnknown *disp_addr,
       hresult = GEO00D_DCOM_TERMINATED;
     else
       hresult = GEO00D_DCOM_EXCEPTION_RAISED;
-  } catch (DbpError &e) { 
+  } catch (DbpError &e) {
     DbpErrorCallbackInterface *pCBInterface = DbpError::dbpGetCallback();  // PTS 1127871
     if (pCBInterface){
       pCBInterface->dbpCaughtError(e);
@@ -902,7 +902,7 @@ HRESULT LVC_Dispatcher::dispDispatch(IliveCacheSink *sink, IUnknown *disp_addr,
     dispEvalException(hresult, parms);
 
     hresult = GEO00D_DCOM_DBPROC_CRASHED;
-  } catch ( BAD_ALLOC_GEO573&) {                    
+  } catch ( BAD_ALLOC_GEO573&) {
     DbpErrorCallbackInterface *pCBInterface = DbpError::dbpGetCallback();  // PTS 1127871
     if (pCBInterface){
       pCBInterface->dbpCaughtBadAlloc();
