@@ -430,6 +430,12 @@ static t_lpIFunc LVC_DispatcherVTableCompute(int dispid,
 #elif defined(LINUX) && defined(S390X) && __GNUC__ == 3 && __GNUC_MINOR__ >= 2
   /* gcc 3.2 on S390X */
   lp_IFunc = (t_lpIFunc) lpVTable[dispid];
+#elif defined(LINUX) && defined(X86_64) && __GNUC__ == 3 && __GNUC_MINOR__ >= 2
+  /* gcc 3.2 on X86_64 */
+  lp_IFunc = (t_lpIFunc) lpVTable[dispid];
+#elif defined(LINUX) && defined(SDBonPPC64) && __GNUC__ == 3 && __GNUC_MINOR__ >= 2
+  /* gcc 3.2 on POWERPC */
+  lp_IFunc = (t_lpIFunc) lpVTable[dispid];
 #elif defined(_WIN32) 
   lp_IFunc = (t_lpIFunc) lpVTable[dispid];
 #else
