@@ -29,7 +29,6 @@
 #define USE_KGS
 
 #include "liveCache/LVC_InProcDispatcher.hpp"
-#include "liveCache/LVC_ProcServerExtender.hpp"
 #include "liveCache/LVC_LoadComponentLibrary.hpp"   /* PTS 1121178 FF */
 #include "liveCache/MemoryManagement/LVCMem_Allocator.hpp"
 #include "liveCache/LVC_DispatcherParams.hpp"
@@ -841,26 +840,7 @@ void LVC_InProcDispatcher::dispDestroySink(IliveCacheSink *sink)
 
 /*---------------------------------------------------------------------------*/
 
-bool LVC_InProcDispatcher::startInProcServer()
-{
-  // TODO
-  return false;
-}
-
-/*---------------------------------------------------------------------------*/
-
 void LVC_InProcDispatcher::dispCreateSink(void *addr, size_t size, void *acvaddr)
-{
-  if (startInProcServer()) {
-    createProcServerSink(addr, size, acvaddr);
-  } else {
-    createLocalSink(addr, size, acvaddr);
-  }
-}
-
-/*---------------------------------------------------------------------------*/
-
-void LVC_InProcDispatcher::createLocalSink(void *addr, size_t size, void *acvaddr)
 {
   tak_all_command_glob *acv = reinterpret_cast<tak_all_command_glob*>(acvaddr);
 
@@ -879,30 +859,6 @@ void LVC_InProcDispatcher::createLocalSink(void *addr, size_t size, void *acvadd
 
 /*---------------------------------------------------------------------------*/
 
-void LVC_InProcDispatcher::createProcServerSink(void *addr, size_t size, void *acvaddr)
-{
-  tak_all_command_glob *acv = reinterpret_cast<tak_all_command_glob*>(acvaddr);
-
-  if (sizeof(LVC_ProcServerExtender) > size) {
-    MSGD ((ERR_DCOM_DUMP_INFO, "LVC_InProcDispatcher::createProcServerSink: sizeof(LVC_ProcServerExtender) > reserved space for Object"));
-    vabort(false);   
-  } else {
-    SAPDB_UTF8 identifier[40];
-    sp77sprintf((char *)identifier, sizeof(identifier), "OmsProcServerExtender T%03d", 
-      acv->a_transinf.tri_trans.trTaskId_gg00);
-    SAPDBMem_RawAllocator *alloc = ::new (LVCMem_Allocator::Instance()) 
-      SAPDBMem_RawAllocator(&identifier[0],
-      *REINTERPRET_CAST(SAPDBMem_RawAllocator*, acv->a_transinf.tri_trans.trAllocator_gg00),
-      32 * 1024, 32 * 1024, SAPDBMem_RawAllocator::FREE_RAW_EXTENDS);
-    alloc->SetBadAllocHandler(SAPDBMem_RawAllocator::THROW_STD_BAD_ALLOC);
-    //LVC_ProcServerExtender *lpSink = new(addr) LVC_ProcServerExtender(alloc);
-    //lpSink->lpSessionContext = acv;
-    // TODO
-  }
-}
-
-/*---------------------------------------------------------------------------*/
-
 void LVC_InProcDispatcher::dispReleaseInstanceWrapper(tsp_dcom_release_instance_parms *parms)
 {
   dispReleaseInstance(REINTERPRET_CAST(CIliveCacheSink*, 
