@@ -683,7 +683,7 @@ externC void LVCMem_SignalAnalysis()
 
 void *operator new (size_t Size) THROW_NEW_GEO573
 {
-  LVCMem_Allocator& lvcAllocator = TheLiveCacheAllocator;
+  LVCMem_Allocator& lvcAllocator = TheLiveCacheAllocatorWithBarrier;  // PTS 1130496
   return lvcAllocator.AllocateImplementation(Size);
 }
 
@@ -691,7 +691,7 @@ void *operator new (size_t Size) THROW_NEW_GEO573
 
 void operator delete (void * pBuffer) THROW_DELETE_GEO573
 {
-    LVCMem_Allocator& lvcAllocator = TheLiveCacheAllocator;
+    LVCMem_Allocator& lvcAllocator = TheLiveCacheAllocatorWithBarrier;  // PTS 1130496
     lvcAllocator.IncrementDelete(pBuffer);
     lvcAllocator.DeallocateImplementation(pBuffer);
 }
@@ -700,7 +700,7 @@ void operator delete (void * pBuffer) THROW_DELETE_GEO573
 
 void *operator new[] (size_t Size) THROW_NEW_GEO573
 {
-  LVCMem_Allocator& lvcAllocator = TheLiveCacheAllocator;
+  LVCMem_Allocator& lvcAllocator = TheLiveCacheAllocatorWithBarrier;  // PTS 1130496
   return lvcAllocator.AllocateImplementation(Size);
 }
 
@@ -708,7 +708,7 @@ void *operator new[] (size_t Size) THROW_NEW_GEO573
 
 void operator delete[]  (void * pBuffer) THROW_DELETE_GEO573
 {
-    LVCMem_Allocator& lvcAllocator = TheLiveCacheAllocator;
+    LVCMem_Allocator& lvcAllocator = TheLiveCacheAllocatorWithBarrier;  // PTS 1130496
     lvcAllocator.IncrementDelete(pBuffer);
     lvcAllocator.DeallocateImplementation(pBuffer);
 }
@@ -722,7 +722,7 @@ extern "C" void *__7__nw__FUl(size_t Size) {
 #else  //__MODEL_ARM 
 extern "C" void *__nw__XUl(size_t Size) { 
 #endif
-  LVCMem_Allocator& lvcAllocator = TheLiveCacheAllocator;
+  LVCMem_Allocator& lvcAllocator = TheLiveCacheAllocatorWithBarrier;  // PTS 1130496
   return lvcAllocator.AllocateImplementation(Size);
 }
 #endif
\ No newline at end of file
