@@ -45,6 +45,7 @@
 #include "SAPDBCommon/MemoryManagement/SAPDBMem_IRawAllocator.hpp"
 #include "SAPDBCommon/MemoryManagement/SAPDBMem_SynchronizedRawAllocator.hpp"
 #include "SAPDBCommon/MemoryManagement/SAPDBMem_CallDirectory.hpp"
+#include "RunTime/System/RTESys_MemoryBarrier.h"
 #include "Container/Container_Vector.hpp"
 #include "heo670.h"
 
@@ -312,6 +313,13 @@ public:
 
     inline static LVCMem_Allocator& LVCAllocatorInstance();
 
+   /*!
+     @brief       returns a reference to the LVCMem_Allocator instance
+                  needed for implementation of operator new/delete
+     @return      reference to LVCMem_Allocator instance
+   */
+    inline static LVCMem_Allocator& LVCAllocatorInstanceWithBarrier(); // PTS 1130496
+
     /*!
        @brief     inline implementation of LVCMem_Allocator.Deallocate
                   needed for implementation of operator new/delete
@@ -492,6 +500,21 @@ inline LVCMem_Allocator& LVCMem_Allocator::LVCAllocatorInstance()
 
 /*--------------------------------------------------------------------------------*/
 
+inline LVCMem_Allocator& LVCMem_Allocator::LVCAllocatorInstanceWithBarrier()  // PTS 1130496
+{
+  if (0 == m_Instance)
+  {
+    LVCMem_Allocator::CreateInstance();
+  }
+  else
+  {
+      RTESys_ReadMemoryBarrier();
+  }
+  return *m_Instance;
+}
+
+/*--------------------------------------------------------------------------------*/
+
 inline void* LVCMem_Allocator::OmsAllocate(SAPDB_ULong count)
 {
 #ifdef SAPDBMem_USE_SYSTEM_ALLOC
@@ -550,6 +573,7 @@ inline void LVCMem_Allocator::SetMonitoring(bool enable)
 
 #define TheOmsRawAllocator LVCMem_Allocator::Instance()
 #define TheLiveCacheAllocator LVCMem_Allocator::LVCAllocatorInstance() 
+#define TheLiveCacheAllocatorWithBarrier LVCMem_Allocator::LVCAllocatorInstanceWithBarrier() 
 #define TheLiveCacheOmsAllocator LVCMem_Allocator::LVCOmsAllocatorInstance();
 
 #endif  /* LVCMem_Allocator_HPP */
