@@ -58,6 +58,9 @@
 /* ==========                EXPORT FUNCTION DECLARATION           ========== */
 /* ========================================================================== */
 
+externCpp void
+ak341CancelOmsVersion (tgg91_TransNo &ConsistViewNo);
+
 externCpp void
 ak341CreateVersion (tak_all_command_glob    &acv,
                     tgg01_OmsVersionContext &lpVersionContext,
@@ -376,6 +379,7 @@ ak341CreateKeyRangeIterator
  const void*             pUpperKey,           
  void**                  pKeyRangeIterator,
  bool                    bAscendingKeyOrder,  
+ bool                    readBuffered,
  short&                  e)
 {
   ak341SetConsistentView (&acv, &consistentView);
@@ -394,7 +398,8 @@ ak341CreateKeyRangeIterator
                          CONST_CAST(void*, pLowerKey),    
                          CONST_CAST(void*, pUpperKey),   
                          *pKeyRangeIterator, 
-                         bAscendingKeyOrder);             
+                         bAscendingKeyOrder,
+                         readBuffered);             
   e = acv.a_transinf.tri_trans.trError_gg00;
 }
 
@@ -779,12 +784,14 @@ ak341NextObjFromKeyIterator (tak_all_command_glob& acv,
                              tgg01_OmsVersionContext *VersionContext,
                              void                    *pKeyIterator,
                              bool                     bInclusive,   
-                             tsp00_Int4               KeyLen,       
+                             tsp00_Int4               KeyLen,   
+                             tsp00_Int4               objBodySize,
                              void                    *pRestartKey, 
                              tsp00_Int4              &noOfOid,
                              tgg92_KernelOid         *pResultOId, 
                              tgg91_PageRef           *pObjVers,
                              tsp00_Int4              &LogReadAccesses,
+                             tsp00_MoveObj           *pObjBody,
                              tgg00_BasisError        &e)
 {
     ak341SetConsistentView (&acv, const_cast<tgg91_TransNo*>(&ConsistentView));  // PTS 1127520
@@ -798,11 +805,13 @@ ak341NextObjFromKeyIterator (tak_all_command_glob& acv,
             pKeyIterator,
             bInclusive,
             KeyLen,
+            objBodySize,
             pRestartKey,
             noOfOid,
             pResultOId, 
             pObjVers,
-            LogReadAccesses);
+            LogReadAccesses,
+            pObjBody);
         e = acv.a_transinf.tri_trans.trError_gg00;
         if (e_ok == e || e_no_next_object == e){
           if (VersionContext) {
@@ -850,12 +859,14 @@ ak341PrevObjFromKeyIterator (tak_all_command_glob& acv,
                              tgg01_OmsVersionContext *VersionContext,
                              void                    *pKeyIterator,
                              bool                     bInclusive,   
-                             tsp00_Int4               KeyLen,       
+                             tsp00_Int4               KeyLen,     
+                             tsp00_Int4               objBodySize,
                              void                    *pRestartKey, 
                              tsp00_Int4              &noOfOid,
                              tgg92_KernelOid         *pResultOId, 
                              tgg91_PageRef           *pObjVers,
                              tsp00_Int4              &LogReadAccesses,
+                             tsp00_MoveObj           *pObjBody,
                              tgg00_BasisError        &e)
 {
     ak341SetConsistentView (&acv, const_cast<tgg91_TransNo*>(&ConsistentView));  // PTS 1127520
@@ -869,11 +880,13 @@ ak341PrevObjFromKeyIterator (tak_all_command_glob& acv,
             pKeyIterator,
             bInclusive,
             KeyLen,
+            objBodySize,
             pRestartKey,
             noOfOid,
             pResultOId, 
             pObjVers,
-            LogReadAccesses);
+            LogReadAccesses,
+            pObjBody);
         e = acv.a_transinf.tri_trans.trError_gg00;
         if (e_ok == e || e_no_next_object == e){
           if (VersionContext) {
