@@ -650,7 +650,7 @@ void ak263CheckHandle (tak_all_command_glob& acv,
               tsp00_KnlIdentifier errorIdentifier;
               memset(&errorIdentifier[0], ' ', sizeof(errorIdentifier));
               sprintf((char*) &errorIdentifier[0], "(Param %d,Member %d)", paramNo, errIndex);
-              a07_nb_put_error(acv, e, 1, errorIdentifier);
+              a07_const_b_put_error (acv, e, 1, (tsp00_MoveObjPtr) &errorIdentifier[0], (int) strlen(errorIdentifier));
               sprintf(OpMsg, "%d %s::%s(member%d) : %s", acv.a_returncode, 
                 cMethodName, cParamName, errIndex, errorMsg); 
               a34OpMsg (isError, *REINTERPRET_CAST(tsp00_MoveObj*, OpMsg), (int)strlen(OpMsg));
