@@ -194,6 +194,14 @@ ak341CreateObjFile (tgg00_TransContext &Trans,
 
 /*-----------------------------------------------------------------------------------*/
 
+externCpp void
+ak341CancelOmsVersion (tgg91_TransNo &ConsistViewNo)
+{
+    ak341LibOmsInterfacePtr->CancelVersion(ConsistViewNo);
+};
+
+/*-----------------------------------------------------------------------------------*/
+
 externCpp void
 ak341CreateVersion (tak_all_command_glob&     acv,
                      tgg01_OmsVersionContext& versionContext,
@@ -784,10 +792,26 @@ ak341MonitorHeapContent(tak_all_command_glob& acv,
             a40move (acv, a41v, NULL, 0);
         }
         SQLMan_Context& context = SQLMan_Context::AcvToContext(acv);
+        SAPDB_Byte* pBuf    = &Rec.recBuf_gg00()[cgg_rec_key_offset + sizeof(a41v.a4LongKey)]; 
+        int         moveLen = Rec.recLen_gg00() - cgg_rec_key_offset - sizeof(a41v.a4LongKey);
+        if (g01unicode)
+        {
+            if (Rec.recLen_gg00() * 2 > sizeof(Rec.recBuf_gg00()))
+            {
+                Rec.recLen_gg00() = sizeof(Rec.recBuf_gg00()) / 2;
+            }
+            // translate to kernel unicode
+            SAPDB_Byte* pSource = pBuf + moveLen - 1;
+            SAPDB_Byte* pDest   = pBuf + 2 * moveLen - 1; 
+            for (int ix = 0; ix < moveLen; ++ix)
+            {
+                *pDest-- = *pSource--;
+                *pDest-- = 0;
+            }
+            moveLen *= 2;
+        }
         SysView_PseudoLongHandler::LongIdentificator longId;
-        context.GetPseudoLongHandler().InsertLong(context, 
-            &Rec.recBuf_gg00()[cgg_rec_key_offset - sizeof(a41v.a4LongKey)], 
-            Rec.recLen_gg00() - cgg_rec_key_offset - sizeof(a41v.a4LongKey), longId);
+        context.GetPseudoLongHandler().InsertLong(context, pBuf, moveLen, longId);
         tgg00_ShowKind showKind;
         showKind = a41v.a4sh_kind;
         a41v.a4sh_kind.becomes(sh_pseudo_long);
