@@ -94,7 +94,7 @@ public:
 
     virtual tsp00_Int4 bd460NumPrimKeysInKeyScope() = 0;
 
-    
+
     virtual void bd460GetSecKey ( // PTS 1116651 TS 2002-07-15
         tsp00_KeyPtr &pSecKey,
         tsp00_Int2   &SecKeyLen) const = 0;
@@ -107,27 +107,8 @@ public:
 class cbd460_InvListArrayIterator : public cbd460_InvListIterator
 {
 public:
-    cbd460_InvListArrayIterator (tgg00_RecPtr  pRec,
-                                 tsp00_KeyPtr  StartKey,
-                                 tsp00_Int4    StartKeyLen,
-                                 tsp00_KeyPtr  StopKey,
-                                 tsp00_Int4    StopKeyLen,
-                                 bool          bAscendingKeyOrder,
-                                 bool          bIncludeStartKey);
 
-    cbd460_InvListArrayIterator(){};
-
-    cbd460_InvListIterator      & operator ++ ();
-    void                          bd460Deref (tsp00_KeyPtr &pPrimKey, tsp00_Int2 &PrimKeyLen);
-
-    tsp00_Int4                    bd460NumPrimKeysInKeyScope ();
-
-
-    void bd460GetSecKey (
-        tsp00_KeyPtr &pSecKey,
-        tsp00_Int2   &SecKeyLen) const; // PTS 1116651 TS 2002-07-15
-
-    void bd460_Init (
+    cbd460_InvListArrayIterator (
         tgg00_RecPtr  pRec,
         tsp00_KeyPtr  StartKey,
         tsp00_Int4    StartKeyLen,
@@ -136,20 +117,47 @@ public:
         bool          bAscendingKeyOrder,
         bool          bIncludeStartKey);
 
-private:
+    cbd460_InvListArrayIterator( const bool bAscendingKeyOrder )
+    :m_bAscendingKeyOrder( bAscendingKeyOrder )
+    {};
 
-    void bd460_SearchStopKey (tsp00_KeyPtr  StopKey, tsp00_Int4 StopKeyLen);
-    
-    bool bd460_BeyondStopKey ();
+    cbd460_InvListIterator& operator ++ ();
+
+    void bd460Deref(
+        tsp00_KeyPtr    &pPrimKey,
+        tsp00_Int2      &PrimKeyLen);
+
+    tsp00_Int4 bd460NumPrimKeysInKeyScope();
+
+
+    void bd460GetSecKey(
+        tsp00_KeyPtr &pSecKey,
+        tsp00_Int2   &SecKeyLen) const; // PTS 1116651 TS 2002-07-15
+
+    void bd460_Init(
+        tgg00_RecPtr  pRec,
+        tsp00_KeyPtr  StartKey,
+        tsp00_Int4    StartKeyLen,
+        tsp00_KeyPtr  StopKey,
+        tsp00_Int4    StopKeyLen,
+        bool          bIncludeStartKey );
 
 private:
 
+    void bd460_SearchStopKey(
+        tsp00_KeyPtr    StopKey,
+        tsp00_Int4      StopKeyLen);
+
+    bool bd460_BeyondStopKey();
+
+private:
+
+    const bool             m_bAscendingKeyOrder;
+
     cbd450_InvListArray    m_InvListArray;
     tsp00_Int4             m_PKIndex;
     tsp00_Int4             m_StopPKIndex;
     tbd_searchresult       m_SearchResult;
-    bool                   m_bAscendingKeyOrder;
-
 };
 
 /*---------------------------------------------------------------------------*/
@@ -157,14 +165,18 @@ private:
 class cbd460_InvListArrayIteratorSave : public cbd460_InvListArrayIterator
 {
 public:
-    cbd460_InvListArrayIteratorSave (cbd510_RecordIterator & RecordIterator,
-                                     tsp00_KeyPtr            StartKey,
-                                     tsp00_Int4              StartKeyLen,
-                                     tsp00_KeyPtr            StopKey,
-                                     tsp00_Int4              StopKeyLen,
-                                     bool                    bAscendingKeyOrder,
-                                     bool                    bIncludeStartKey);
+
+    cbd460_InvListArrayIteratorSave(
+        cbd510_RecordIterator & RecordIterator,
+        tsp00_KeyPtr            StartKey,
+        tsp00_Int4              StartKeyLen,
+        tsp00_KeyPtr            StopKey,
+        tsp00_Int4              StopKeyLen,
+        bool                    bAscendingKeyOrder,
+        bool                    bIncludeStartKey );
+
 private:
+
     cbd600_Node  m_Node;
 };
 
@@ -172,45 +184,51 @@ private:
 
 class cbd460_InvListSubtreeIteratorNoBreak : public cbd460_InvListIterator
 {
-    friend class cbd460_InvListSubtreeIterator;
+    friend class cbd460_InvListSubtreeIterator; // because of private methods
 
 public:
-    cbd460_InvListSubtreeIteratorNoBreak (cbd500_Tree                   &MotherTree,
-                                          cbd450_InvListRefSubTree      &InvListRefSubTree,
-                                          tsp00_KeyPtr                   pStartPrimKey,
-                                          tsp00_Int4                     StartPrimKeyLen,
-                                          tsp00_KeyPtr                   pStopPrimKey,
-                                          tsp00_Int4                     StopPrimKeyLen,
-                                          bool                           bAscendingKeyOrder,
-                                          bool                           bIncludeFirstPrimKey,
-                                          bool                          &bStartKeyFound_Dummy,
-                                          bool                          *pTreeIsRequested = NULL);
- 
+    cbd460_InvListSubtreeIteratorNoBreak(
+        cbd500_Tree                   &MotherTree,
+        cbd450_InvListRefSubTree      &InvListRefSubTree,
+        tsp00_KeyPtr                   pStartPrimKey,
+        tsp00_Int4                     StartPrimKeyLen,
+        tsp00_KeyPtr                   pStopPrimKey,
+        tsp00_Int4                     StopPrimKeyLen,
+        bool                           bAscendingKeyOrder,
+        bool                           bIncludeFirstPrimKey,
+        bool                          *pTreeIsRequested = NULL );
+
     cbd460_InvListIterator& operator ++ ();
 
-    void bd460Deref (
+    void bd460Deref(
         tsp00_KeyPtr &pPrimKey,
         tsp00_Int2   &PrimKeyLen);
 
-    tsp00_Int4 bd460NumPrimKeysInKeyScope ();
+    tsp00_Int4 bd460NumPrimKeysInKeyScope();
 
-    void bd460GetSecKey (
+    void bd460GetSecKey(
         tsp00_KeyPtr &pSecKey,
-        tsp00_Int2   &SecKeyLen) const; // PTS 1116651 TS 2002-07-15
+        tsp00_Int2   &SecKeyLen ) const; // PTS 1116651 TS 2002-07-15
 private:
-     
-    void bd460Continue (
+
+    void bd460Break(
+        const tsp00_KeyPtr   rePositionPrimKey,
+        tsp00_Int2           &rePositionPrimKeyLen );
+
+    void bd460Continue(
         cbd450_InvListRefSubTree  &InvListRefSubTree,
-        tsp00_KeyPtr               pStartPrimKey,
-        tsp00_Int4                 StartPrimKeyLen);
+        const tsp00_KeyPtr        rePositionPrimKey,
+        const tsp00_Int4          rePositionPrimKeyLen );
+
+    tgg00_BasisError& bd460TrError();
 
 private:
 
     cbd300_InvSubTreeCurrent  m_InvSubTreeCurrent;
-    bool                      m_TreeIsRequested;
     cbd600_Node               m_SubTreeLeaf;
     cbd500_SubTree            m_SubTree;
-    cbd510_RecordIterator     m_RecIter;
+    cbd510_RecordIterator     m_PrimKeyIter;
+    bool                      m_bStartKeyFound_Dummy; // required by m_PrimKeyIter, but not used
 
     /*-------------------------------------------------------------------------
     declaration: m_pSecKey
@@ -236,29 +254,28 @@ private:
 class cbd460_InvListSubtreeIterator : public cbd460_InvListIterator
 {
 public:
-    cbd460_InvListSubtreeIterator (
-        cbd500_Tree                   &MotherTree,
-        cbd450_InvListRefSubTree      &InvListRefSubTree,
-        // tsp00_KeyPtr                   pSecKey, PTS 1116651 TS 2002-07-15
-        // tsp00_Int4                     SecKeyLen, PTS 1116651 TS 2002-07-15
-        tsp00_KeyPtr                   pStartPrimKey,
-        tsp00_Int4                     StartPrimKeyLen,
-        tsp00_KeyPtr                   pStopPrimKey,
-        tsp00_Int4                     StopPrimKeyLen,
-        bool                           bAscendingKeyOrder,
-        bool                           bIncludeFirstPrimKey);
+    cbd460_InvListSubtreeIterator(
+        cbd500_Tree                 &MotherTree,
+        cbd510_RecordIterator       &SecKeyIter,
+        cbd450_InvListRefSubTree    &InvListRefSubTree,
+        tsp00_KeyPtr                pStartPrimKey,
+        tsp00_Int4                  StartPrimKeyLen,
+        tsp00_KeyPtr                pStopPrimKey,
+        tsp00_Int4                  StopPrimKeyLen,
+        bool                        bAscendingKeyOrder,
+        bool                        bIncludeFirstPrimKey );
 
     cbd460_InvListIterator& operator ++ ();
 
-    void bd460Deref (
+    void bd460Deref(
         tsp00_KeyPtr &pPrimKey,
-        tsp00_Int2  &PrimKeyLen);
+        tsp00_Int2  &PrimKeyLen );
 
-    tsp00_Int4 bd460NumPrimKeysInKeyScope ();
+    tsp00_Int4 bd460NumPrimKeysInKeyScope();
 
-    void bd460GetSecKey (
+    void bd460GetSecKey(
         tsp00_KeyPtr &pSecKey,
-        tsp00_Int2   &SecKeyLen) const;  // PTS 1116651 TS 2002-07-15
+        tsp00_Int2   &SecKeyLen ) const;  // PTS 1116651 TS 2002-07-15
 
 private:
 
@@ -266,21 +283,17 @@ private:
 
 private:
 
-    bool    m_bTreeIsRequested;
-    bool    m_bStartKeyFound;
+    const tsp00_KeyPtr                      m_pStopPrimKey;
+    const tsp00_Int4                        m_StopPrimKeyLen;
 
-    cbd460_InvListSubtreeIteratorNoBreak    m_InvListSubtreeIterator;
+    // inversion list iterator of the proper index tree
+    cbd510_RecordIterator                   &m_SecKeyIter; // PTS 1132245 TS 2004-11-09
+
+    bool                                    m_bTreeIsRequested;
+
+    cbd460_InvListSubtreeIteratorNoBreak    m_InvListSubtreeIteratorNoBreak;
     cbd460_InvListArrayIterator             m_InvListArrayIterator;
-    cbd460_InvListIterator                  *m_pInvListIterator;
-
-    tsp00_KeyPtr    m_pStopPrimKey;
-    tsp00_Int4      m_StopPrimKeyLen;
-    bool            m_bAscendingKeyOrder;
-
-    //tsp00_KeyPtr                     m_pSecKey;              PTS 1116651 TS 2002-07-15
-    //tsp00_Int4                       m_SecKeyLen;            PTS 1116651 TS 2002-07-15
-    //tsp00_Key                        m_ReEntrancePrimKey;    PTS 1116651 TS 2002-07-15
-    //tsp00_Int2                       m_ReEntrancePrimKeyLen; PTS 1116651 TS 2002-07-15
+    cbd460_InvListIterator                  *m_pUnivIterator; // pointer to the current inv list member
 };
 
 /*===========================================================================*
@@ -292,8 +305,9 @@ private:
 /*---------------------------------------------------------------------------*/
 
 inline void
-cbd460_InvListArrayIterator::bd460_SearchStopKey (tsp00_KeyPtr  StopKey,
-        tsp00_Int4    StopKeyLen)
+cbd460_InvListArrayIterator::bd460_SearchStopKey (
+    tsp00_KeyPtr  StopKey,
+    tsp00_Int4    StopKeyLen )
 {
     ROUTINE_DBG_MEO00 ("bd460_SearchStopKey");
 
@@ -332,7 +346,9 @@ cbd460_InvListArrayIterator::bd460_BeyondStopKey ()
 /*---------------------------------------------------------------------------*/
 
 inline void
-cbd460_InvListArrayIterator::bd460Deref (tsp00_KeyPtr &pPrimKey, tsp00_Int2 &PrimKeyLen)
+cbd460_InvListArrayIterator::bd460Deref(
+    tsp00_KeyPtr &pPrimKey,
+    tsp00_Int2   &PrimKeyLen )
 {
     if (thisfound == m_SearchResult)
         m_InvListArray.bd450GetPrimKey (m_PKIndex, pPrimKey, PrimKeyLen);
@@ -366,7 +382,7 @@ cbd460_InvListArrayIterator::operator ++ ()
 /*---------------------------------------------------------------------------*/
 
 inline tsp00_Int4
-cbd460_InvListArrayIterator::bd460NumPrimKeysInKeyScope ()
+cbd460_InvListArrayIterator::bd460NumPrimKeysInKeyScope()
 {
     ROUTINE_DBG_MEO00 ("bd460NumPrimKeysInKeyScope_Array");
 
@@ -385,11 +401,11 @@ cbd460_InvListArrayIterator::bd460NumPrimKeysInKeyScope ()
 /*---------------------------------------------------------------------------*/
 
 inline void
-cbd460_InvListArrayIterator::bd460GetSecKey (
+cbd460_InvListArrayIterator::bd460GetSecKey(
     tsp00_KeyPtr &pSecKey,
-    tsp00_Int2   &SecKeyLen) const // PTS 1116651 TS 2002-07-15
+    tsp00_Int2   &SecKeyLen ) const // PTS 1116651 TS 2002-07-15
 {
-    ROUTINE_DBG_MEO00 ("cbd460_InvListArrayIterator::bd460GetSecKey");
+    ROUTINE_DBG_MEO00 ("cbd460_InvListArrayIter::bd460GetSecKey");
 
     m_InvListArray.bd450GetSecKey( pSecKey, SecKeyLen );
 }
@@ -397,17 +413,20 @@ cbd460_InvListArrayIterator::bd460GetSecKey (
 /*---------------------------------------------------------------------------*/
 
 inline
-cbd460_InvListArrayIterator::cbd460_InvListArrayIterator (tgg00_RecPtr  pRec,
-        tsp00_KeyPtr  StartKey,
-        tsp00_Int4    StartKeyLen,
-        tsp00_KeyPtr  StopKey,
-        tsp00_Int4    StopKeyLen,
-        bool          bAscendingKeyOrder,
-        bool          bIncludeStartKey)
+cbd460_InvListArrayIterator::cbd460_InvListArrayIterator(
+    tgg00_RecPtr  pRec,
+    tsp00_KeyPtr  StartKey,
+    tsp00_Int4    StartKeyLen,
+    tsp00_KeyPtr  StopKey,
+    tsp00_Int4    StopKeyLen,
+    bool          bAscendingKeyOrder,
+    bool          bIncludeStartKey )
+        :m_bAscendingKeyOrder( bAscendingKeyOrder )
+
 {
     ROUTINE_DBG_MEO00 ("cbd460_InvListArrayIterator");
 
-    bd460_Init (pRec, StartKey, StartKeyLen,  StopKey, StopKeyLen,  bAscendingKeyOrder, bIncludeStartKey);
+    bd460_Init( pRec, StartKey, StartKeyLen,  StopKey, StopKeyLen,  bIncludeStartKey );
 }
 
 /*---------------------------------------------------------------------------*/
@@ -415,15 +434,19 @@ cbd460_InvListArrayIterator::cbd460_InvListArrayIterator (tgg00_RecPtr  pRec,
 /*---------------------------------------------------------------------------*/
 
 inline
-cbd460_InvListArrayIteratorSave::cbd460_InvListArrayIteratorSave (cbd510_RecordIterator & RecordIterator,
-        tsp00_KeyPtr            StartKey,
-        tsp00_Int4              StartKeyLen,
-        tsp00_KeyPtr            StopKey,
-        tsp00_Int4              StopKeyLen,
-        bool                    bAscendingKeyOrder,
-        bool                    bIncludeStartKey)
+cbd460_InvListArrayIteratorSave::cbd460_InvListArrayIteratorSave(
+    cbd510_RecordIterator & RecordIterator,
+    tsp00_KeyPtr            StartKey,
+    tsp00_Int4              StartKeyLen,
+    tsp00_KeyPtr            StopKey,
+    tsp00_Int4              StopKeyLen,
+    bool                    bAscendingKeyOrder,
+    bool                    bIncludeStartKey )
         :
-        cbd460_InvListArrayIterator (*RecordIterator, StartKey, StartKeyLen, StopKey, StopKeyLen, bAscendingKeyOrder, bIncludeStartKey),
+        cbd460_InvListArrayIterator( *RecordIterator,
+                                     StartKey, StartKeyLen,
+                                     StopKey, StopKeyLen,
+                                     bAscendingKeyOrder, bIncludeStartKey ),
         m_Node (RecordIterator.bd510Node())
 {}
 
@@ -432,7 +455,7 @@ cbd460_InvListArrayIteratorSave::cbd460_InvListArrayIteratorSave (cbd510_RecordI
 /*---------------------------------------------------------------------------*/
 
 inline
-cbd460_InvListSubtreeIteratorNoBreak::cbd460_InvListSubtreeIteratorNoBreak (
+cbd460_InvListSubtreeIteratorNoBreak::cbd460_InvListSubtreeIteratorNoBreak(
     cbd500_Tree                   &MotherTree,
     cbd450_InvListRefSubTree      &InvListRefSubTree,
     tsp00_KeyPtr                   pStartPrimKey,
@@ -441,14 +464,20 @@ cbd460_InvListSubtreeIteratorNoBreak::cbd460_InvListSubtreeIteratorNoBreak (
     tsp00_Int4                     StopPrimKeyLen,
     bool                           bAscendingKeyOrder,
     bool                           bIncludeFirstPrimKey,
-    bool                          &bStartKeyFound_Dummy,
-    bool                          *pTreeIsRequested)
+    bool                          *pTreeIsRequested )
         :
-        m_InvSubTreeCurrent (MotherTree.bd500GetTheCurrent(), tbd_node_request::fromConst(nr_for_read), InvListRefSubTree, false),
-        m_SubTreeLeaf (m_InvSubTreeCurrent, tbd_node_request::fromConst(nr_for_read), tbd00_PageLockMode::fromConst( plmLockWithoutReschedule_ebd00)), // PTS 1115174 TS 2002-04-09),
-        m_TreeIsRequested (false),
-        m_SubTree (MotherTree, m_SubTreeLeaf, pTreeIsRequested),
-        m_RecIter (m_SubTree, pStartPrimKey, StartPrimKeyLen, pStopPrimKey, StopPrimKeyLen, bAscendingKeyOrder, bIncludeFirstPrimKey, bStartKeyFound_Dummy)
+        m_InvSubTreeCurrent( MotherTree.bd500GetTheCurrent(),
+                             tbd_node_request::fromConst( nr_for_read ),
+                             InvListRefSubTree, false ),
+        m_SubTreeLeaf( m_InvSubTreeCurrent,
+                       tbd_node_request::fromConst( nr_for_read ),
+                       tbd00_PageLockMode::fromConst( plmLockWithoutReschedule_ebd00 )), // PTS 1115174 TS 2002-04-09),
+        m_SubTree( MotherTree, m_SubTreeLeaf, pTreeIsRequested ),
+        m_PrimKeyIter( m_SubTree,
+                       pStartPrimKey, StartPrimKeyLen,
+                       pStopPrimKey, StopPrimKeyLen,
+                       bAscendingKeyOrder,
+                       bIncludeFirstPrimKey, m_bStartKeyFound_Dummy )
 {
     InvListRefSubTree.bd450GetSecKey( m_pSecKey, m_SecKeyLen ); // PTS 1116651 TS 2002-07-15
 }
@@ -456,17 +485,29 @@ cbd460_InvListSubtreeIteratorNoBreak::cbd460_InvListSubtreeIteratorNoBreak (
 /*---------------------------------------------------------------------------*/
 
 inline void
-cbd460_InvListSubtreeIteratorNoBreak::bd460Continue (
-    cbd450_InvListRefSubTree      &InvListRefSubTree,
-    tsp00_KeyPtr                   pStartPrimKey,
-    tsp00_Int4                     StartPrimKeyLen)
+cbd460_InvListSubtreeIteratorNoBreak::bd460Break(
+    const tsp00_KeyPtr   rePositionPrimKey,
+    tsp00_Int2           &rePositionPrimKeyLen )
 {
-    ROUTINE_DBG_MEO00 ("bd460Continue");
+    ROUTINE_DBG_MEO00 ("bd460Break_Subtree");
+
+    m_PrimKeyIter.bd510Break( rePositionPrimKey, rePositionPrimKeyLen );
+}
+
+/*---------------------------------------------------------------------------*/
+
+inline void
+cbd460_InvListSubtreeIteratorNoBreak::bd460Continue(
+    cbd450_InvListRefSubTree    &InvListRefSubTree,
+    const tsp00_KeyPtr          rePositionPrimKey,
+    const tsp00_Int4            rePositionPrimKeyLen )
+{
+    ROUTINE_DBG_MEO00 ("bd460Continue_Subtree");
 
-    m_TreeIsRequested = false;
     m_InvSubTreeCurrent.bd300SetSubRoot (InvListRefSubTree, false);
     InvListRefSubTree.bd450GetSecKey( m_pSecKey, m_SecKeyLen ); // PTS 1116651 TS 2002-07-15
-    m_RecIter.bd510Continue (pStartPrimKey, StartPrimKeyLen);
+
+    m_PrimKeyIter.bd510Continue( rePositionPrimKey, rePositionPrimKeyLen );
 }
 
 /*---------------------------------------------------------------------------*/
@@ -474,23 +515,25 @@ cbd460_InvListSubtreeIteratorNoBreak::bd460Continue (
 inline cbd460_InvListIterator &
 cbd460_InvListSubtreeIteratorNoBreak::operator ++ ()
 {
-    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIterator_++");
+    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIter_++");
 
-    ++m_RecIter;
+    ++m_PrimKeyIter;
     return *this;
 }
 
 /*---------------------------------------------------------------------------*/
 
 inline void
-cbd460_InvListSubtreeIteratorNoBreak::bd460Deref (tsp00_KeyPtr &pPrimKey, tsp00_Int2 &PrimKeyLen)
+cbd460_InvListSubtreeIteratorNoBreak::bd460Deref(
+    tsp00_KeyPtr   &pPrimKey,
+    tsp00_Int2     &PrimKeyLen )
 {
     ROUTINE_DBG_MEO00 ("bd460Deref_Subtree");
 
-    if (NULL != *m_RecIter)
+    if (NULL != *m_PrimKeyIter)
     {
-        pPrimKey   = (tsp00_KeyPtr) &(*m_RecIter)->recKey_gg00().keyVal_gg00();
-        PrimKeyLen = (*m_RecIter)->recKeyLen_gg00();
+        pPrimKey   = (tsp00_KeyPtr) &(*m_PrimKeyIter)->recKey_gg00().keyVal_gg00();
+        PrimKeyLen = (*m_PrimKeyIter)->recKeyLen_gg00();
     }
     else
     {
@@ -502,89 +545,103 @@ cbd460_InvListSubtreeIteratorNoBreak::bd460Deref (tsp00_KeyPtr &pPrimKey, tsp00_
 /*---------------------------------------------------------------------------*/
 
 inline tsp00_Int4
-cbd460_InvListSubtreeIteratorNoBreak::bd460NumPrimKeysInKeyScope ()
+cbd460_InvListSubtreeIteratorNoBreak::bd460NumPrimKeysInKeyScope()
 {
     ROUTINE_DBG_MEO00 ("bd460NumPrimKeysInKeyScope_Subtree");
 
-    return m_RecIter.bd510NumberOfRecordsInKeyScope ();
+    return m_PrimKeyIter.bd510NumberOfRecordsInKeyScope();
 }
 
 /*---------------------------------------------------------------------------*/
 
 inline void
-cbd460_InvListSubtreeIteratorNoBreak::bd460GetSecKey (
+cbd460_InvListSubtreeIteratorNoBreak::bd460GetSecKey(
     tsp00_KeyPtr &pSecKey,
     tsp00_Int2   &SecKeyLen) const // PTS 1116651 TS 2002-07-15
 {
-    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIteratorNoBreak::bd460GetSecKey");
+    ROUTINE_DBG_MEO00 ("bd460GetSecKey_Subtree");
 
     pSecKey   = m_pSecKey;
     SecKeyLen = m_SecKeyLen;
 }
 
+/*---------------------------------------------------------------------------*/
+
+inline tgg00_BasisError&
+cbd460_InvListSubtreeIteratorNoBreak::bd460TrError()
+{
+	return m_InvSubTreeCurrent.curr_trans->trError_gg00;
+}
 /*---------------------------------------------------------------------------*/
 /* METHOD IMPLEMENTATION OF cbd460_InvListSubtreeIterator                    */
 /*---------------------------------------------------------------------------*/
 
 inline
-cbd460_InvListSubtreeIterator::cbd460_InvListSubtreeIterator (
+cbd460_InvListSubtreeIterator::cbd460_InvListSubtreeIterator(
     cbd500_Tree               &MotherTree,
+    cbd510_RecordIterator     &SecKeyIter,
     cbd450_InvListRefSubTree  &InvListRefSubTree,
-    // tsp00_KeyPtr               pSecKey, PTS 1116651 TS 2002-07-15
-    // tsp00_Int4                 SecKeyLen, PTS 1116651 TS 2002-07-15
     tsp00_KeyPtr               pStartPrimKey,
     tsp00_Int4                 StartPrimKeyLen,
     tsp00_KeyPtr               pStopPrimKey,
     tsp00_Int4                 StopPrimKeyLen,
     bool                       bAscendingKeyOrder,
-    bool                       bIncludeFirstPrimKey)
+    bool                       bIncludeFirstPrimKey )
         :
-        m_bTreeIsRequested(false),
-        m_bStartKeyFound  (false),
-        m_InvListSubtreeIterator (MotherTree, InvListRefSubTree, pStartPrimKey, StartPrimKeyLen, pStopPrimKey, StopPrimKeyLen,
-                                  bAscendingKeyOrder, bIncludeFirstPrimKey, m_bStartKeyFound, & m_bTreeIsRequested),
-        m_pInvListIterator     (&m_InvListSubtreeIterator),
-        m_pStopPrimKey         (pStopPrimKey),
-        m_StopPrimKeyLen       (StopPrimKeyLen),
-        m_bAscendingKeyOrder   (bAscendingKeyOrder)
+        m_pStopPrimKey( pStopPrimKey ),
+        m_StopPrimKeyLen( StopPrimKeyLen ),
+        m_SecKeyIter( SecKeyIter ),
+        m_bTreeIsRequested( false ),
+        m_InvListSubtreeIteratorNoBreak( MotherTree,
+                                         InvListRefSubTree,
+                                         pStartPrimKey, StartPrimKeyLen,
+                                         pStopPrimKey, StopPrimKeyLen,
+                                         bAscendingKeyOrder,
+                                         bIncludeFirstPrimKey,
+                                         & m_bTreeIsRequested ),
+        m_InvListArrayIterator( bAscendingKeyOrder ),
+        m_pUnivIterator( &m_InvListSubtreeIteratorNoBreak )
 {
-    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIterator");
+    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIter");
 }
 
 /*---------------------------------------------------------------------------*/
 
-inline  cbd460_InvListIterator      &
+inline  cbd460_InvListIterator&
 cbd460_InvListSubtreeIterator::operator ++ ()
 {
-    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIterator::operator ++");
+    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIter::operator ++");
 
-    m_pInvListIterator->operator++();
+    m_pUnivIterator->operator++();
 
-    if (m_bTreeIsRequested)
+    if( m_bTreeIsRequested ) // is set in cbd500SubTree::bd500_CheckStopKeyAndInterrupt in case of
+    {                        // a root exclusive request. Precondition: mother tree is interruptable
         bd460_LeaveAndReEnterInvTree();
-
+        m_bTreeIsRequested = false; // PTS 1132245 TS 2004-11-16
+    }
     return (*this);
 }
 
 /*---------------------------------------------------------------------------*/
 
 inline void
-cbd460_InvListSubtreeIterator::bd460Deref (tsp00_KeyPtr &pPrimKey,
-        tsp00_Int2   &PrimKeyLen)
+cbd460_InvListSubtreeIterator::bd460Deref(
+    tsp00_KeyPtr &pPrimKey,
+    tsp00_Int2   &PrimKeyLen )
 {
-    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIterator::bd460Deref");
+    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIter::bd460Deref");
 
-    m_pInvListIterator->bd460Deref (pPrimKey, PrimKeyLen);
+    m_pUnivIterator->bd460Deref (pPrimKey, PrimKeyLen);
 }
 
 /*---------------------------------------------------------------------------*/
 
 inline tsp00_Int4
-cbd460_InvListSubtreeIterator::bd460NumPrimKeysInKeyScope ()
+cbd460_InvListSubtreeIterator::bd460NumPrimKeysInKeyScope()
 {
-    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIterator::bd460NumPrim");
+    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIter::bd460NumPrim");
 
-    return (m_pInvListIterator->bd460NumPrimKeysInKeyScope());
+    return (m_pUnivIterator->bd460NumPrimKeysInKeyScope());
 }
 
 /*---------------------------------------------------------------------------*/
@@ -593,13 +650,14 @@ inline void
 cbd460_InvListSubtreeIterator::bd460GetSecKey (
     tsp00_KeyPtr &pSecKey,
     tsp00_Int2   &SecKeyLen) const  // PTS 1116651 TS 2002-07-15
-
 {
-    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIterator::bd460GetSecKey");
+    ROUTINE_DBG_MEO00 ("cbd460_InvListSubtreeIter::bd460GetSecKey");
 
-    m_pInvListIterator->bd460GetSecKey( pSecKey, SecKeyLen );
+    m_pUnivIterator->bd460GetSecKey( pSecKey, SecKeyLen );
 }
 
-/*---------------------------------------------------------------------------*/
+/*===========================================================================*
+ *  END OF CODE                                                              *
+ *===========================================================================*/
 
 #endif  /* GBD460_H */
