@@ -76,9 +76,6 @@
  *  FORWARD DECLARATION                                                      *
  *===========================================================================*/
 
-class cbd460_InvListSubtreeIteratorNoBreak;
-class cbd460_InvListSubtreeIterator;
-class cbd460_InvListIterator;
 class cbd500_SubTree;
 class cbd510_RecordIterator;
 class cbd501_IndexOrder;
@@ -112,8 +109,6 @@ typedef tbd500_PathInfoElem tbd500_PathInfo [MAX_LEVEL_BD00+1];
 
 class cbd500_Tree
 {
-    friend class cbd460_InvListSubtreeIteratorNoBreak;
-    friend class cbd460_InvListSubtreeIterator;
     friend class cbd500_SubTree;
     friend class cbd510_RecordIterator;
 public:
@@ -143,24 +138,17 @@ public:
                        tsp00_PageNo   StartId);
     /*!
     function:    bd500Break
-    description: Current scan over nodes is interrupted.
-
-                 There after all nodes of the tree are released.
-                 The caller must know where to bd500Continue().
-    arguments:   bAscending [in] Specify the orientation of current search direction.
-    */
-    void         bd500Break ( bool bAscending ); // PTS 1121589 UH 2003-04-11 added
-    /*!
-    function:    bd500Break
     description: Current scan over nodes is interrupted.
 
                  The first key in orientation of the current node is copied into rePositionKey.
                  There after all nodes of the tree are released.
-    arguments:   bAscending [in] Specify the orientation of current search direction.
+    arguments:   recIndex         [in]  current record position 
+                 rePositionKey    [out] current key at recIndex position
+                 rePositionKeyLen [out] current key length
     */
-    void         bd500Break ( bool          bAscending,
-                              tsp00_KeyPtr  rePositionKey,      // PTS 1121589 UH 2003-04-11 added
-                              tsp00_Int2   &rePositionKeyLen ); // PTS 1121589 UH 2003-04-11 added
+    void         bd500Break ( const tsp00_Int4      recIndex,
+                              const tsp00_KeyPtr    rePositionKey,       // PTS 1121589 UH 2003-04-11 added
+                              tsp00_Int2            &rePositionKeyLen ); // PTS 1121589 UH 2003-04-11 added
     // void         bd500Continue         (); // PTS 1121589 UH 2003-04-11 removed
     /*!
     function:    bd500Continue
@@ -544,11 +532,6 @@ protected:
 
 class cbd500_SubTree : public cbd500_Tree
 {   
-    friend class cbd460_InvListSubtreeIterator;
-    friend class cbd460_InvListSubtreeIteratorB;
-    friend class cbd460_InvListIterator;
-
-
 private:
     cbd500_Tree  &m_MotherTree;
     bool         *m_pTreeIsRequested;
@@ -837,8 +820,8 @@ inline
 cbd500_SubTree::cbd500_SubTree (cbd500_Tree    &MotherTree,
                                 cbd600_Node    &StartNode,
                                 bool           *pTreeIsRequested)
-:cbd500_Tree       (StartNode),
- m_MotherTree      (MotherTree),
+:cbd500_Tree        (StartNode, MotherTree.m_Interruptible), // PTS 1129329 TS 2004-04-26
+ m_MotherTree       (MotherTree),
  m_pTreeIsRequested (pTreeIsRequested)
 {}
 
