@@ -365,8 +365,6 @@ cbd501_IndexOrders::bd501CreateNew   (tbd501_IndexOrderKind Kind,
 	ROUTINE_DBG_MEO00 ("bd501CreateNew");
     
     // PTS ? UH 2001-11-23 completely rewritten
-    /* PTS 1108839 UH 2000-12-27 begin */
-    /* PTS 1109032 UH 2001-1-17  begin */
     
     int                OrderNo = -1;
     cbd501_IndexOrder* NewOrder = NULL;
@@ -431,51 +429,77 @@ cbd501_IndexOrders::bd501CreateNew   (tbd501_IndexOrderKind Kind,
     {
         // There is an existing order found which operates on the same subnode as the current new order.
 
-        if ( NewOrder == NULL )
-            g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: Ord nil2", pSubNode->nd_id());
-
         cbd501_IndexOrder& order = *NewOrder;
-        
-        if ( iokDelete_ebd501 == order.ioKind_bd501 )
-        {
-            // If the old order removed the node there cannot be
-            // any further new order for this non exiting subnode.
-            // bd501Trace (tgg00_TransContext &Trans)
-            g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord1", pSubNode->nd_id());
-        }
-        
-        if ( order.m_ioFromLevel_bd501 != pSubNode->nd_level() )
-        {
-            // This is impossible
-            // bd501Trace (tgg00_TransContext &Trans)
-            g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord2", pSubNode->nd_id());
-        }
-        
-        if ( order.m_ioSubNodeId_bd501 != pSubNode->nd_id() )
+
+        if ( (iokDelete_ebd501 != Kind && order.m_ioSubNode_bd501 != pSubNode)
+             ||
+             order.m_ioSubNodeId_bd501 != pSubNode->nd_id() )
         {
             // This is impossible
             // bd501Trace (tgg00_TransContext &Trans)
             g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord3", pSubNode->nd_id());
         }
-        // PTS 1115899 UH 2002-05-24 begin
-        if ( iokAdd_ebd501    == order.ioKind_bd501
-             &&
-             iokDelete_ebd501 == Kind )
+
+        // PTS 1115899 UH 2002-05-24 included in the next lines
+        // PTS 1131880 UH 2004-10-15 structured with decision matrix
+        
+        switch ( order.ioKind_bd501 )
         {
-            // The current DeleteOrder deletes a previously new added node.
-            // So the found order must be simply removed from the orderlist.
-            bd501RemoveOrder (OrderNo);
+            case iokAdd_ebd501 :
+                switch ( Kind )
+                {
+                    case iokAdd_ebd501 :
+                        g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord1", pSubNode->nd_id());
+                        break;
+                    case iokDelete_ebd501 :
+                        bd501RemoveOrder (OrderNo);
+                        break;
+                    case iokChange_ebd501 :
+                        order.m_ioLeafCount_bd501 = pSubNode->ndLeafCount_bd00();
+                        break;
+                    default:
+                        g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord2", order.ioKind_bd501);
+                }
+                break;
+            case iokDelete_ebd501 :
+                switch ( Kind )
+                {
+                    case iokAdd_ebd501 :
+                        order.ioKind_bd501        = Kind;
+                        order.m_ioLeafCount_bd501 = pSubNode->ndLeafCount_bd00();
+                        break;
+                    case iokDelete_ebd501 :
+                        g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord3", pSubNode->nd_id());
+                        break;
+                    case iokChange_ebd501 :
+                        g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord4", pSubNode->nd_id());
+                        break;
+                    default:
+                        g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord5", order.ioKind_bd501);
+                }
+                break;
+            case iokChange_ebd501 :
+                switch ( Kind )
+                {
+                    case iokAdd_ebd501 :
+                        g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord6", pSubNode->nd_id());
+                        break;
+                    case iokDelete_ebd501 :
+                        order.ioKind_bd501        = Kind;
+                        order.m_ioSubNode_bd501   = 0;
+                        order.m_ioLeafCount_bd501 = 0;
+                        break;
+                    case iokChange_ebd501 :
+                        order.m_ioLeafCount_bd501 = pSubNode->ndLeafCount_bd00();
+                        break;
+                    default:
+                        g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord7", order.ioKind_bd501);
+                }
+                break;
+            default:
+                g01abort (csp3_bd_msg, csp3_n_btree, "bd501CreateNew: inv ord8", order.ioKind_bd501);
         }
-        // PTS 1115899 UH 2002-05-24 end
         
-        // order.ioKind_bd501 is left unchanged
-        order.m_ioLeafCount_bd501 = pSubNode->ndLeafCount_bd00();
-
-        if ( iokDelete_ebd501 != Kind )
-            order.m_ioSubNode_bd501 = pSubNode;
-        else
-            order.m_ioSubNode_bd501 = NULL;
-
         // PTS 1113462 UH 2001-01-17 begin
         if ( SepKeyLen != INVALID_KEY_LENGTH_GBD501 )
         {
