@@ -310,7 +310,8 @@ class  cbd900_ObjChainInfo
     
 public:
     tsp00_PageNo    ociSubRoot_bd900;       /*!< page number of subroot  */
-    tsp00_PageNo    ociFiller2_bd900;       /*!< page number out of freepage  */
+    //PTS 1130998 tsp00_PageNo    ociFiller2_bd900;       /*!< page number out of freepage */              
+    tsp00_Int4      ociFreePageCnt_bd900;   /*!< # of free pages in chain starting at the subroot     PTS 1130998 */ 
     tsp00_Int4      ociAllObjCnt_bd900;     /*!< # of all object frames which are not free */           
     tsp00_Int4      ociAllPageCnt_bd900;    /*!< # of all pages in chain starting at the subroot */   
     tsp00_Int2      ociCurrActivGC_bd900;   /*!< index of the GC currently active on this chain */
@@ -906,11 +907,11 @@ cbd900_ObjChainInfo::bd900ResetChainInfo()
     ociFiller0_bd900            = false;
     ociFiller1_bd900            = false;
     ociSubRoot_bd900            = NIL_PAGE_NO_GG00;
-    ociFiller2_bd900            = NIL_PAGE_NO_GG00;
     ociAllObjCnt_bd900          = 0;
     ociAllPageCnt_bd900         = 0;   
     ociCurrActivGC_bd900        = NIL_GC_INDEX_BD900;
     ociAllObjCntOnSubRoot_bd900 = 0;
+    ociFreePageCnt_bd900        = 0;            /* PTS 1130998 */
 }
 
 /*---------------------------------------------------------------------------*/
@@ -924,6 +925,7 @@ cbd900_ObjChainInfo::bd900SetChainInfo (tbd_node_ptrs   &NptrsSubRoot)
     ociAllObjCnt_bd900          = SubRootPageInfo.bd900AllObjCnt();
     ociAllPageCnt_bd900         = SubRootPageInfo.bd900AllPageCnt();  
     ociAllObjCntOnSubRoot_bd900 = SubRootPageInfo.bd900AllObjCnt();
+    ociFreePageCnt_bd900        = SubRootPageInfo.bd900FreePageCnt();   /* PTS 1130998 */
 }
 
 /*---------------------------------------------------------------------------*/
