@@ -116,6 +116,7 @@ public:
     bool bd910IsEmptyPage();
     bool bd910IsPageInFreeChain();
     bool bd910IsSubRoot();
+	bool bd910LockSubRootBeforeReleasingFrame() const; 
     void bd910FreePage();
     void bd910MoveToNextPage();
     void bd910AssignToNewPage (tsp00_Int4               ObjBodyLen,
@@ -132,8 +133,15 @@ public:
                                  const teo00_Uint8              NewPageSeqNo,
                                  cbd910_ObjNode                &NptrsNewPage);
     
-    void bd910SetTo (const cbd910_ObjNode & ObjNode);
+   	void bd910SetTo (cbd910_ObjNode & ObjNode, const bool movePrimary = false);
     
+    void   bd910_SetFreePageCntInChainInfo (const tgg00_FileId  &FileId,              /* PTS 1130998 */
+                                            tsp00_Int4           ChainNo,
+                                            tsp00_Int4           FreePageCnt);
+
+    void   bd910_SetAllPageCntInChainInfo (const tgg00_FileId  &FileId,               /* PTS 1130998 */
+                                           tsp00_Int4           ChainNo,
+                                           tsp00_Int4           AllPageCnt);
 private:   // methods
     virtual void bd910_CheckNodeType ();
     void         bd910_InitNode (tbd_current_tree      &Current,
@@ -211,9 +219,6 @@ public:
     static tbd900_ObjFrame *bd910ObjFramePtr (tsp00_Byte *pPageBodyBeg, tsp00_Int4 ObjFramePos);
     
     void   bd910_CheckFreePageCnt (cbd910_SubRootNode &NptrsSubRoot);
-    void   bd910_SetAllPageCntInChainInfo (const tgg00_FileId  &FileId,
-                                                 tsp00_Int4           ChainNo,
-                                                 tsp00_Int4           AllPageCnt);
 };
 
 /*---------------------------------------------------------------------------*/
@@ -480,7 +485,7 @@ cbd910_ObjNode::bd910_CheckNodeType ()
 /*---------------------------------------------------------------------------*/
 
 inline	void
-cbd910_ObjNode::bd910SetTo (const cbd910_ObjNode & ObjNode)
+cbd910_ObjNode::bd910SetTo (cbd910_ObjNode & ObjNode, const bool movePrimary /*PTS 1130601*/)
 {
 #   if COMPILEMODE_MSP00 >= QUICK_MSP00 
     if (np_ptr() != NULL)
@@ -490,7 +495,13 @@ cbd910_ObjNode::bd910SetTo (const cbd910_ObjNode & ObjNode)
 #   endif
     
     *this              = ObjNode;
-    m_PrimaryReference = false;
+	if ( movePrimary ) /*PTS 1130601*/
+	{
+		ObjNode.m_PrimaryReference = false;
+		m_PrimaryReference = true;
+	}
+	else
+		m_PrimaryReference = false;
 }
 
 /*---------------------------------------------------------------------------*/
@@ -578,13 +589,21 @@ cbd910_ObjNode::bd910IsPageInFreeChain ()
     return (0 != this->np_ptr()->nd_in_free_chain()); 
 }
 
+/*---------------------------------------------------------------------------*/
+
+inline bool         
+cbd910_ObjNode::bd910LockSubRootBeforeReleasingFrame() const
+{
+	return  ((this->np_ptr()->nd_free_obj_cnt() == (this->np_ptr()->nd_max_obj_cnt()-1)) ||
+		(this->np_ptr()->nd_free_obj_cnt() == 0) || !this->np_ptr()->nd_in_free_chain() );
+}; 
 
 /*---------------------------------------------------------------------------*/
 
 inline void  
-cbd910_ObjDataNode::bd910_SetAllPageCntInChainInfo (const tgg00_FileId  &FileId,
-                                                    tsp00_Int4           ChainNo,
-                                                    tsp00_Int4           AllPageCnt)
+cbd910_ObjNode::bd910_SetAllPageCntInChainInfo (const tgg00_FileId  &FileId,
+                                                tsp00_Int4           ChainNo,
+                                                tsp00_Int4           AllPageCnt)
 {
     ROUTINE_DBG_MEO00 ("bd910_SetAllPageCntInChainInfo");
     
@@ -597,6 +616,24 @@ cbd910_ObjDataNode::bd910_SetAllPageCntInChainInfo (const tgg00_FileId  &FileId,
         bd93ContObjFileDir[FileNo][ContFileNo][ChainNo].ociAllPageCnt_bd900 = AllPageCnt; 
 }
 
+/*---------------------------------------------------------------------------*/
+/* PTS 1130998 */
+inline void  
+cbd910_ObjNode::bd910_SetFreePageCntInChainInfo (const tgg00_FileId  &FileId,
+                                                 tsp00_Int4           ChainNo,
+                                                 tsp00_Int4           FreePageCnt)
+{
+    ROUTINE_DBG_MEO00 ("bd910_SetAllPageCntInChainInfo");
+
+    const tgg00_ObjFileNo &FileNo     = FileId.fileObjFileNo_gg00();
+    const tsp00_Int4       ContFileNo = FileId.fileContObjFileNo_gg00();
+
+    if (PRIM_CONT_OBJ_FILE_NO_BD900 == ContFileNo)
+        bd90ObjFileDir[FileNo][ChainNo].ociFreePageCnt_bd900 = FreePageCnt;
+    else
+        bd93ContObjFileDir[FileNo][ContFileNo][ChainNo].ociFreePageCnt_bd900 = FreePageCnt; 
+}
+
 /*---------------------------------------------------------------------------*/
 
 inline void
@@ -718,6 +755,7 @@ cbd910_SubRootNode::bd910InsertPageIntoFreeChain  (cbd910_ObjDataNode &NptrsPage
         /* increase free page counter on subroot page */ 
         cbd900_SubRootPageInfo  SubRootPageInfo (*this);
         ++SubRootPageInfo.bd900FreePageCnt();
+	    bd910_SetFreePageCntInChainInfo (m_ptrCurrent->curr_tree_id, this->np_ptr()->nd_chain_no(), SubRootPageInfo.bd900FreePageCnt());
 
         /* perform some simple checks */
 #       if COMPILEMODE_MEO00 >= QUICK_MEO00 	
@@ -892,6 +930,7 @@ cbd910_SubRootNode::bd910RemovePageFromFreeChain (cbd910_ObjDataNode   &NptrsPag
             /* decrease free page counter on subroot page */ 
             cbd900_SubRootPageInfo  SubRootPageInfo (*this);
             --SubRootPageInfo.bd900FreePageCnt();
+    	    bd910_SetFreePageCntInChainInfo (m_ptrCurrent->curr_tree_id, this->np_ptr()->nd_chain_no(), SubRootPageInfo.bd900FreePageCnt());
         }
 
         /* PTS 1127493 FF 2004-FEB-02                                            */
