@@ -79,6 +79,17 @@
   Chapter: Interface Routines
  ****************************************************************************/
 
+/*!***************************************************************************
+function   : bd04CancelOmsVersion
+
+description: Tries to release all Resources held by an OmsVersion
+
+arguments  : ConsistViewNo [in] consistent view, which had been assigned to OmsVersion
+******************************************************************************/
+externC void
+bd04CancelOmsVersion (tsp00_TaskId   TaskId,
+                      tgg91_TransNo &ConsistViewNo);
+
 /*!***************************************************************************
 function   : bd04CreateKeyIterator
 description: see kb04CreateKeyIterator
@@ -92,7 +103,8 @@ bd04CreateKeyIterator (tgg00_TransContext &Trans,
 				       void		          *pLowerKey,             
 				       void		          *pUpperKey,             
 				       void            *&pKeyIterator,
-               bool              bAscendingKeyOrder);   
+               bool              bAscendingKeyOrder,
+               bool              readBuffered);   
 
 /*!***************************************************************************
 function   : bd04CreateObjFile
@@ -531,12 +543,14 @@ externCpp void
 bd04NextObjFromKeyIterator (tgg00_TransContext &Trans,
                             void               *pUntypedKeyIterator,    
                             bool                bInclusive,   
-                            tsp00_Int4          KeyLen,        
+                            tsp00_Int4          KeyLen,   
+                            tsp00_Int4          objBodySize,
                             void               *pRestartKey, 
                             tsp00_Int4         &noOfOid,
                             tgg92_KernelOid    *pResultOId, 
                             tgg91_PageRef      *pObjVers,
-                            tsp00_Int4         &LogReadAccesses);
+                            tsp00_Int4         &LogReadAccesses,
+                            tsp00_MoveObj      *pObjBody);
 
 /*!***************************************************************************
 function   : bd04PrevObjFromKeyIterator
@@ -547,12 +561,14 @@ externCpp void
 bd04PrevObjFromKeyIterator (tgg00_TransContext &Trans,
                             void               *pUntypedKeyIterator,    
                             bool                bInclusive,   
-                            tsp00_Int4          KeyLen,        
+                            tsp00_Int4          KeyLen,  
+                            tsp00_Int4          objBodySize,
                             void               *pRestartKey, 
                             tsp00_Int4         &noOfOid,
                             tgg92_KernelOid    *pResultOId, 
                             tgg91_PageRef      *pObjVers,
-                            tsp00_Int4         &LogReadAccesses);
+                            tsp00_Int4         &LogReadAccesses,
+                            tsp00_MoveObj      *pObjBody);
 
 /*!***************************************************************************
 function   : bd04OmsReadAheadParticipant
