@@ -64,17 +64,27 @@
 #include "hbd92.h"
 #include "hbd95.h"
 #include "heo56.h"    // vreschedule_msec
+#include "heo58.h"    // vinit_akdump_ptr
 #include "hgg01_1.h"
 #include "hgg01_3.h"
 #include "hkb50.h"
+#include "hkb53.h"
+#include "hak341.h"
 
 #include "Converter/Converter_ICommon.hpp"
 #include "liveCache/LVC_AbstractObjKeyIterator.hpp"
 #include "liveCache/LVC_SingleFileObjKeyIterator.hpp"
+#include "liveCache/LVC_BufferedSingleFileObjKeyIterator.hpp"
 #include "liveCache/LVC_MultiFileObjKeyIterator.hpp"
+#include "liveCache/LVC_InProcDispatcher.hpp"
+#include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"
 #include "SAPDBCommon/MemoryManagement/SAPDBMem_IRawAllocator.hpp"
 #include "Trace/Trace_Entry.hpp"
 #include "KernelCommon/Kernel_VTrace.hpp"        /* PTS 1120151 */ 
+#include "SAPDBCommon/MemoryManagement/SAPDBMem_RawAllocator.hpp"
+#include "RunTime/MemoryManagement/RTEMem_BlockAllocator.hpp"
+#include "RunTime/MemoryManagement/RTEMem_RteAllocator.hpp"
+#include "SAPDBCommon/SAPDB_sprintf.h"
 
 #if COMPILEMODE_MEO00 >= SLOW_MEO00
 #include "hta99.h"
@@ -142,6 +152,8 @@
 
 extern cbd900_ObjFileDir          bd90ObjFileDir;
 
+externC void vdcom_init_sink ( void *pInstance );
+
 /*===========================================================================*
  *  LOCAL FUNCTIONS (PROTOTYPES)                                             *
  *===========================================================================*/
@@ -227,6 +239,75 @@ public:
  *  GLOBAL FUNCTIONS (CODE)                                                  *
  *===========================================================================*/
 
+externC void
+bd04CancelOmsVersion ( tsp00_TaskId   TaskId
+                      ,tgg91_TransNo &ConsistViewNo)
+{
+    /* function my be call on timer process without any session and transaction */ 
+    if ( NULL == vGetAcvPtrFromCurrentTask())
+    {
+        /* create a new session & transaction context */
+        tgg00_TransContext Trans;
+        tgg00_TransContext AuxTrans;
+        SAPDB_MemFillNoCheck( &Trans, 0x00, sizeof(Trans));
+        Trans.trTaskId_gg00 = TaskId;
+        Trans.trError_gg00  = e_ok;
+        Trans.trState_gg00.clear();
+
+        /* allocator isneeded by local filedirectory */
+        SAPDB_UTF8 identifier[40];
+        SAPDB_sprintf((char *)identifier, sizeof(identifier), "TransContext(intern) T%03d", TaskId);
+        SAPDBMem_RawAllocator *pAllocator = new(RTEMem_RteAllocator::Instance()) SAPDBMem_RawAllocator(
+            identifier,
+            RTEMem_BlockAllocator::Instance(), 
+            1000,
+            1000,
+            SAPDBMem_RawAllocator::FREE_RAW_EXTENDS_EXCEPT_INITIAL); 
+        Trans.trAllocator_gg00 = pAllocator;
+
+        tsp00_RteCommRec rteComRec;
+        rteComRec.to_cancel  = FALSE;
+        rteComRec.virt_reads = 0;
+        rteComRec.file_root  = 0;
+        rteComRec.file_record_cnt = 0;
+        Trans.trRteCommPtr_gg00 = &rteComRec;
+
+        k53StartInternalSession (Trans, AuxTrans);
+
+        /* provide ACV and ILiveCacheSink for CancelOmsVersion request */
+        /* needed to be able to remove OmsVersion TempFiles & to call  */
+        /* vbegexcl & vendexcl                                         */
+        tak_all_command_glob *acv = reinterpret_cast<tak_all_command_glob*>(
+            pAllocator->Allocate(sizeof(tak_all_command_glob))); 
+        if ( NULL != acv)
+        {
+          SAPDB_MemFillNoCheck( acv, 0x00, sizeof(acv));
+          acv->a_transinf.tri_trans = Trans;
+
+          /* anchor ACV within TCB of runtime */
+          vinit_akdump_ptr ( acv, NULL /* no dump */ );
+
+          tsp_dcom_init_sink_parms sink_info;
+          sink_info.acv_addr      = reinterpret_cast<tsp00_Addr>(acv);
+          sink_info.sink_addr     = reinterpret_cast<tsp00_Addr>(&(acv->a_kernel_sink));
+          sink_info.sink_size     = sizeof (acv->a_kernel_sink);
+          vdcom_init_sink (&sink_info);
+
+          ak341CancelOmsVersion(ConsistViewNo);
+
+          vinit_akdump_ptr ( NULL, NULL );
+
+          pAllocator->Deallocate(acv);
+        }
+        k53StopInternalSession (Trans, AuxTrans, m_commit);
+        destroy(pAllocator,RTEMem_RteAllocator::Instance());
+    }
+    else
+        ak341CancelOmsVersion(ConsistViewNo);
+};
+
+/*---------------------------------------------------------------------------*/
+
 externCpp void
 bd04CreateKeyIterator (tgg00_TransContext &Trans,
 					   tgg00_FileId		  &ObjFileId,
@@ -236,7 +317,8 @@ bd04CreateKeyIterator (tgg00_TransContext &Trans,
 					   void		          *pLowerKey,            // PTS 1119480
 					   void		          *pUpperKey,            // PTS 1119480
 					   void            *&pKeyIterator,
-             bool              bAscendingKeyOrder)   // PTS 1119480
+             bool              bAscendingKeyOrder,   // PTS 1119480
+             bool              readBuffered)
 {
 	ROUTINE_DBG_MSP00 ("bd04CreateKeyIterator");
 	
@@ -302,10 +384,18 @@ bd04CreateKeyIterator (tgg00_TransContext &Trans,
 
 		if (NumObjKeyFiles == 1)
 		{
-			pKeyIterator =   new (RawAllocator) 
-				LVC_SingleFileObjKeyIterator (Trans, ObjFileId, KeyLen, 
+      if (!readBuffered){
+        pKeyIterator =   new (RawAllocator) 
+        LVC_SingleFileObjKeyIterator (Trans, ObjFileId, KeyLen, 
                                       pStartKey, pLowerKey, pUpperKey, 
                                       RawAllocator, bAscendingKeyOrder); 
+      }
+      else {
+        pKeyIterator =   new (RawAllocator) 
+        LVC_BufferedSingleFileObjKeyIterator (Trans, ObjFileId, KeyLen, 
+                                      pStartKey, pLowerKey, pUpperKey, 
+                                      RawAllocator, bAscendingKeyOrder); 
+      }
 		}
 		else
 		{
@@ -766,9 +856,7 @@ bd04GetObjWithKeyConsistent (tgg00_TransContext  &Trans,            //[inp/out]
     //if ( e_too_old_oid == Trans.trError_gg00 )
     //    Trans.trError_gg00 = e_object_not_found;
 
-	/* write vtrace entry                                    CRS 1109559 AK 07/03/2001 */
-    /* write vtrace entry in case an unexpected error occurs PTS 1112190 AK 18/10/2001 */
-    if (g01vtrace.vtrOmsGet_gg00 || ((Trans.trError_gg00 != e_ok) && (Trans.trError_gg00 != e_object_not_found)))
+    if (g01vtrace.vtrOmsGet_gg00)
 	{
 		bd04_VtraceReturn (Trans, ObjFileId, ResultOid, ResultObjVers,
 			b04GetObjWithKey, ot2OutputSeq_egg12);
@@ -1308,12 +1396,14 @@ externCpp void
 bd04NextObjFromKeyIterator (tgg00_TransContext &Trans,
                             void               *pUntypedKeyIterator,    
                             bool                bInclusive,    
-                            tsp00_Int4          KeyLen,        
+                            tsp00_Int4          KeyLen,   
+                            tsp00_Int4          objBodySize,
                             void               *pRestartKey,   
                             tsp00_Int4         &noOfOid,
                             tgg92_KernelOid    *pResultOId, 
                             tgg91_PageRef      *pObjVers,
-                            tsp00_Int4         &LogReadAccesses)
+                            tsp00_Int4         &LogReadAccesses,
+                            tsp00_MoveObj      *pObjBody)
 {
     ROUTINE_DBG_MEO00 ("bd04NextObjFromKeyIterator");
 
@@ -1344,8 +1434,9 @@ bd04NextObjFromKeyIterator (tgg00_TransContext &Trans,
   BD04_CHECK_OBJ_FILE_TYPE (Trans, KeyIterator.GetObjFileId(), b04next_consistent);
 			
 	if (Trans.trError_gg00 == e_ok){
-        bd90NextObjFromKeyIterator (Trans, KeyIterator,  bInclusive, KeyLen, pRestartKey,
-                                    noOfOid, pResultOId, pObjVers, LogReadAccesses);
+        bd90NextObjFromKeyIterator (Trans, KeyIterator,  bInclusive, KeyLen, objBodySize,
+                                    pRestartKey, noOfOid, pResultOId, pObjVers, LogReadAccesses,
+                                    pObjBody);
 	}
 	
   /* PTS 1114891 FF 2003-01-07 */
@@ -1367,11 +1458,13 @@ bd04PrevObjFromKeyIterator (tgg00_TransContext &Trans,
                             void               *pUntypedKeyIterator,    
                             bool                bInclusive,    
                             tsp00_Int4          KeyLen,        
+                            tsp00_Int4          objBodySize,
                             void               *pRestartKey,   
                             tsp00_Int4         &noOfOid,
                             tgg92_KernelOid    *pResultOId, 
                             tgg91_PageRef      *pObjVers,
-                            tsp00_Int4         &LogReadAccesses)
+                            tsp00_Int4         &LogReadAccesses,
+                            tsp00_MoveObj      *pObjBody)
 {
     ROUTINE_DBG_MEO00 ("bd04PrevObjFromKeyIterator");
 
@@ -1402,8 +1495,9 @@ bd04PrevObjFromKeyIterator (tgg00_TransContext &Trans,
   BD04_CHECK_OBJ_FILE_TYPE (Trans, KeyIterator.GetObjFileId(), b04next_consistent);
 			
 	if (Trans.trError_gg00 == e_ok){
-        bd90PrevObjFromKeyIterator (Trans, KeyIterator,  bInclusive, KeyLen, pRestartKey,
-                                    noOfOid, pResultOId, pObjVers, LogReadAccesses);
+        bd90PrevObjFromKeyIterator (Trans, KeyIterator,  bInclusive, KeyLen, objBodySize,
+                                    pRestartKey, noOfOid, pResultOId, pObjVers, LogReadAccesses,
+                                    pObjBody);
 	}
 	
   /* PTS 1114891 FF 2003-01-07 */
