@@ -67,6 +67,7 @@
 #include "ggg01.h"    // OMS errors
 #include "gbd00.h"    // BD Types
 #include "gbd02.h"    // Datacache Types
+#include "gbd499.h"   // IndexMemoryManagementUnit
 #include "ggg91.h"    // Fileversion
 
 #include "hbd01.h"
@@ -103,8 +104,6 @@
 #define SUSP_RES  true
 #endif
 
-#define NUMBER_OF_RETRIES 5
-
 #define IS_READ           true
 #define IS_VIRTUAL        true
 #define NEW_NODE          true
@@ -244,12 +243,13 @@ bd13_InitPageState(
 /*---------------------------------------------------------------------------*/
 
 inline void
-bd13_OverflowReaction(
-    tsp00_TaskId         TaskId,
-    tgg00_BasisError     &TrError,
-    tsp00_PageNo         PageNo,
-    tsp00_Int            &RetryCount,
-    tsp00_Int4           SuspendReason );
+bd13_OverflowReaction (
+    const tsp00_TaskId          TaskId,
+    const tsp00_PageNo          PageNo,
+    const tgg00_MessType        actionType,
+    const tsp00_Int4            SuspendReason,
+    tgg00_BasisError            &TrError,
+    tsp00_Int                   &RetryCount );
 
 /*---------------------------------------------------------------------------*/
 
@@ -393,7 +393,7 @@ bd13GetNode (tbd_current_tree         &Current,
             // data cache overflow handling
             if ((e_sysbuffer_overflow == TrError) || (e_no_more_temp_space == TrError))
             {
-                bd13_OverflowReaction (TaskId, TrError, PageNo, RetryCount, 43);
+                bd13_OverflowReaction( TaskId, PageNo, Current.curr_action, 43, TrError, RetryCount );
                 continue;
             }
             break; // fatal error has occured
@@ -615,7 +615,8 @@ bd13GetNewSpecifiedNode (
             // data cache overflow handling
             if(( e_sysbuffer_overflow == TrError ) || ( e_no_more_temp_space == TrError ))
             {
-                bd13_OverflowReaction( TaskId, TrError, WantedPageNo, RetryCount, 43 );
+                bd13_OverflowReaction( TaskId, WantedPageNo,  tgg00_MessType::fromConst( m_nil ),
+                                       43, TrError, RetryCount );
                 continue;
             }
             break; // fatal error has occured
@@ -1405,7 +1406,7 @@ bd13WriteExclusiveLockedRoot( tbd_current_tree &current )
             // root page and therefore we have to wait with the i/o until the older
             // images of the root page are written to to the data area.
 
-            bd13_MsgAndSuspend( trans, pageNo, b_await_w, WAIT_FOR_WRITE, 56 );
+            bd13_MsgAndSuspend( trans, pageNo, b_await_w, WAIT_FOR_WRITE, 40 );
             nodeState.cpsWaitForWrite_bd02 = false;
             continue;
         }
@@ -1417,7 +1418,7 @@ bd13WriteExclusiveLockedRoot( tbd_current_tree &current )
             // otherwise the converter version materialized with the running savepoint
             // will be inconsistent
 
-            bd13_MsgAndSuspend (trans, pageNo, b_await_w, WAIT_FOR_SVP_END, 24 );
+            bd13_MsgAndSuspend (trans, pageNo, b_await_w, WAIT_FOR_SVP_END, 28 );
             nodeState.cpsWaitForSvpEnd_bd02 = false;
             continue;
         }
@@ -2177,20 +2178,52 @@ bd13_WriteOldOccupant (tgg00_TransContext         &Trans,
 /*---------------------------------------------------------------------------*/
 
 inline void
-bd13_OverflowReaction (tsp00_TaskId         TaskId,
-                       tgg00_BasisError     &TrError,
-                       tsp00_PageNo         PageNo,
-                       tsp00_Int            &RetryCount,
-                       tsp00_Int4           SuspendReason)
+bd13_OverflowReaction (
+    const tsp00_TaskId      TaskId,
+    const tsp00_PageNo      PageNo,
+    const tgg00_MessType    actionType,
+    const tsp00_Int4        SuspendReason,
+    tgg00_BasisError        &TrError,
+    tsp00_Int               &RetryCount )
 {
+    const tsp00_Int4 defaultRetries = 11;
+    const tsp00_Int4 maxRetries     = 61;
+
     if( e_sysbuffer_overflow == TrError )
     {
         vsleep (TaskId, 1);
         ++RetryCount;
-        if( NUMBER_OF_RETRIES > RetryCount )
+
+        if( 0 == ( RetryCount % 10 ))
+        {
+            g01opmsg (sp3p_knldiag, sp3m_warning, csp3_b20_lru_overflow_1,
+                      csp3_n_datacache, "DATA CACHE OVERFLOW     ", TaskId);
+        }
+
+        if( RetryCount <= defaultRetries )
+        {
             TrError = e_ok;
+            return;
+        }
+        // be more soft, if parallel index creation is active
+        // and calling task is not the index builder
+        if(
+            ( cbd499_IndexMemoryManagementUnit::bd499IsInUse()) &&
+            ( actionType != m_create_index_parallel )
+        )
+        {
+            if( RetryCount <= maxRetries )
+            {
+                TrError = e_ok;
+                return;
+            }
+        }
+        //  page acces will fail with e_sysbuffer_overflow
+        g01opmsg (sp3p_knldiag, sp3m_warning, csp3_b20_lru_overflow_2,
+                  csp3_n_datacache, "DATA CACHE OVERFLOW     ", TaskId);
         return;
     }
+
     if( e_no_more_temp_space == TrError )
     {
 # ifdef SUSP_RES
@@ -2262,7 +2295,8 @@ bd13_FrameForNewNode (const tsp00_Int2          LevelNo,
             // data cache overflow handling
             if ((e_sysbuffer_overflow == TrError) || (e_no_more_temp_space == TrError))
             {
-                bd13_OverflowReaction (CurrTrans.trTaskId_gg00, TrError, NewPagePno, RetryCount, 46);
+                bd13_OverflowReaction( CurrTrans.trTaskId_gg00, NewPagePno,
+                                       Current.curr_action, 46, TrError, RetryCount );
                 continue;
             }
             break; // fatal error has occured
@@ -2355,7 +2389,8 @@ bd13_FreeNode (
         {
             if(( e_sysbuffer_overflow == trError ) || ( e_no_more_temp_space == trError ))
             {
-                bd13_OverflowReaction( taskId, trError, pageNo, retryCount, 240 );
+                bd13_OverflowReaction( taskId, pageNo, current.curr_action, 240, 
+                    trError, retryCount );
                 continue;
             }
             break; //fatal error has occured
