@@ -63,9 +63,13 @@
 #include "hbd400.h"
 #include "SAPDB/SAPDBCommon/SAPDB_RangeCode.hpp" // Kernel_move_and_fill
 #include "hgg10.h"
+#include "ggg01.h"
 
 #include "Converter/Converter_IVerify.hpp"
 #include "DataAccess/Data_Types.hpp"
+#include "DataAccess/Data_BasePage.hpp"
+#include "KernelCommon/Kernel_FileIO.hpp"
+
 
 #if COMPILEMODE_MEO00 >= SLOW_MEO00
 #include "hta99.h"
@@ -866,6 +870,78 @@ bd400Verify (cbd300_InvCurrent &InvCurrent,
         b06write_filename_and_root (InvCurrent.curr_tree_id);
     }
 }
+/*---------------------------------------------------------------------------*/
+
+externCpp void
+bd400ExtractIndex ( 
+           cbd300_InvCurrent      &invCurrent,
+           const tsp00_VFilename  &hostFileName )
+{
+    ROUTINE_DBG_MEO00 ("bd400Dump");
+    
+    tgg00_BasisError &trError = invCurrent.curr_trans->trError_gg00;
+    
+    cbd600_Node       node( invCurrent, tbd_node_request::fromConst( nr_for_read ));
+    cbd500_Tree       tree( node );
+    
+    if( e_ok != trError ) 
+        return;
+    
+    tree.bd500SetToRoot();
+    
+    Kernel_FileIO hostFile( hostFileName );
+
+    while ( node.bd600IsAccessable() && ( e_ok == trError ))
+    {
+        Data_BasePage   page( node.bd600GetNodePtr());
+
+        if( ! hostFile.Write( page ))
+        {
+            trError = e_hostfile_error;
+            return;
+        }
+
+        if( node.bd600IsLeaf())
+        {
+            const tsp00_Int4 maxRecIndex = node.bd600MaxRecIndex();
+
+            for( tsp00_Int4 recIndex = FIRST_REC_INDEX_BD00; recIndex <= maxRecIndex; ++recIndex )
+            {
+                if( ilkSubTree_egg00 != node.bd600RecPtr( recIndex)->recInvListKind_gg00() )
+                    continue;
+                
+                cbd450_InvListRefSubTree    invListRefSubTree ( node.bd600RecPtr( recIndex ));
+                cbd300_InvSubTreeCurrent    subTreeCurrent( invCurrent, tbd_node_request::fromConst( nr_for_read ),
+                                                            invListRefSubTree, false);
+                
+                cbd600_Node  subTreeNode( subTreeCurrent, tbd_node_request::fromConst( nr_for_read ));
+                cbd500_Tree  subTree( subTreeNode );
+                
+                if( e_ok != trError ) 
+                    return;
+                
+                subTree.bd500SetToRoot();
+
+                if( e_ok != trError ) 
+                    return;
+
+                while( subTreeNode.bd600IsAccessable())
+                {
+                    Data_BasePage   subPage( subTreeNode.bd600GetNodePtr());
+                    
+                    if( hostFile.Write( subPage ))
+                        subTree.bd500NextNode();
+                    else
+                        trError = e_hostfile_error;
+                    
+                    if ( e_ok != trError ) 
+                        return;
+                }
+            }
+        }
+        tree.bd500NextNode();
+    }
+}
 
 /*===========================================================================*
  *  LOCAL FUNCTIONS (CODE)                                                   *
@@ -882,31 +958,52 @@ bd400_AddPrimKeyToSubTree (cbd300_InvCurrent &InvCurrent,
 
     tgg00_BasisError         &TrError = InvCurrent.curr_trans->trError_gg00;
     cbd450_InvListRefSubTree  InvListRefSubTree (LeafNode.bd600RecPtr (RecIndex));
-    if ( LeafNode.bd600IsUpdateable() )
+    const bool                updateStatistics = LeafNode.bd600IsUpdateable();
+    if ( updateStatistics )
         LeafNode.bd600LeafCount() -= InvListRefSubTree.bd450NumLeaves();
     tbd_searchresult          PrimKeySearchResult;
     bool                      bIsRootExclusiveLocked = false;
+
+    /* in the constructor cbd300_InvSubTreeCurrent InvListRefSubTree */
+    /* is updated from the SubTreeRoot                               */
     cbd300_InvSubTreeCurrent  InvSubTreeCurrent (InvCurrent,
             tbd_node_request::fromConst(nr_for_update),
             InvListRefSubTree,
             LeafNode.bd600IsUpdateable());
+
     if (TrError != e_ok) return;
 
-    /* in the constructor cbd300_InvSubTreeCurrent InvListRefSubTree */
-    /* is updated from the SubTreeRoot                               */
-
-    if ( LeafNode.bd600IsUpdateable() )
+    if ( updateStatistics )
+    {
         LeafNode.bd600LeafCount() += InvListRefSubTree.bd450NumLeaves();
-
-    InvListRefSubTree.bd450IncrNumPrimKeys ();
+        InvListRefSubTree.bd450IncrNumPrimKeys();
+    }
+	// PTS 1131150 UH prepare to revert increment of primkey count
+    // save inv base record information in case an error ocurs we can rollback changes made
+    const tsp00_PageNo leafPageNo = LeafNode.bd600GetId();
+          tgg00_Lkey   invKey;
+    if ( updateStatistics )
+    {
+        tsp00_KeyPtr pSecKey   = 0;
+		tsp00_Int2   SecKeyLen = 0;
+        InvListRefSubTree.bd450GetSecKey(pSecKey,SecKeyLen);
+        SAPDB_RangeMove(__FILE__, 1,
+                        SecKeyLen, sizeof(invKey.keyVal_gg00()),
+                        pSecKey, 1, &invKey.keyVal_gg00(), 1, SecKeyLen,
+                        TrError );
+        if (TrError != e_ok)
+            return;
+        invKey.keyLen_gg00() = SecKeyLen;
+    }
 
     /* release all resources of the primary index tree */
     LeafNode.bd600Release (IS_CHANGED_BD600);
 
     /* do not reference LeafNode any more */
-
-    if ( (m_create_index_parallel != InvCurrent.curr_action) &&
-            (m_create_index          != InvCurrent.curr_action)  )
+    const bool treeIsUnlocked = m_create_index_parallel != InvCurrent.curr_action
+                                &&
+                                m_create_index          != InvCurrent.curr_action;
+    if ( treeIsUnlocked )
     {
         InvCurrent.bd300UnlockTree ();
         if (e_ok != TrError) return;
@@ -927,6 +1024,38 @@ bd400_AddPrimKeyToSubTree (cbd300_InvCurrent &InvCurrent,
     /* return with error if prim key exists already */
     if (thisfound == PrimKeySearchResult)
     {
+        if ( updateStatistics ) 	// PTS 1131150 UH revert increment of primkey count
+        {
+            // reset the PrimKeyCount in the invBaseRecord
+            tsp00_Int4       recindex;
+            tbd_searchresult result;
+            if ( ! treeIsUnlocked )
+			{
+                LeafNode.bd600GetNode(leafPageNo);
+                if (e_ok != TrError) return;
+			}
+            else
+            {
+                InvCurrent.bd300LockTree();
+                if (TrError != e_ok) return;
+                cbd500_Tree InvTree (LeafNode);
+                if (e_ok != TrError) return;
+                InvTree.bd500FindNode ((tsp00_KeyPtr)&invKey.keyVal_gg00(), invKey.keyLen_gg00(), LEAF_LEVEL_BD00);
+                if (e_ok != TrError) return;
+            }
+            LeafNode.bd600SearchRecord((tsp00_KeyPtr)&invKey.keyVal_gg00(), invKey.keyLen_gg00(), recindex, result);
+            if ( thisfound == result ) // is a different result possible ?!
+            {
+                cbd450_InvListRefSubTree invBaseRec (LeafNode.bd600RecPtr(recindex));
+                invBaseRec.bd450DecrNumPrimKeys();
+            }
+			LeafNode.bd600Release (IS_CHANGED_BD600);
+            if ( treeIsUnlocked )
+            {
+                InvCurrent.bd300UnlockTree();
+                if (TrError != e_ok) return;
+            }
+        }
         TrError = e_duplicate_key;
         return;
     }
@@ -1307,7 +1436,10 @@ bd400_Verify (cbd300_InvCurrentBasis &InvCurrent,
             return;
         }
 
-        if( bWithExtendedCheck ) // PTS 1125096 TS 2003-11-21
+        if( 
+            ( bWithExtendedCheck        ) && // PTS 1125096 TS 2003-11-21
+            (0 < Node2.bd600NumRecords())    // PTS 1131553 TS 2004-10-06
+            )
         {
             // recPos and recLength were checked before
             tsp00_LcompResult   compResult;
@@ -1812,7 +1944,7 @@ cbd400_InvTree::bd400AppendPrimKey (tsp00_KeyPtr  pSecKey,
             cbd450_InvListRefSubTree InvList (*m_Node.m_Current.curr_trans,
                                               pRec, pSecKey, SecKeyLen,
                                               NIL_PAGE_NO_GG00 /* SubTreeRoot */,
-                                              0                /* NumPrimKeys */,
+                                              1                /* NumPrimKeys */, 	// PTS 1131150 UH initialize with 1 instead of 0
                                               1                /* NumLeaves   */);
 
             ++(m_Node.m_pNode->ndLeafCount_bd00());
@@ -2018,16 +2150,22 @@ bd400AddToOmsInvTree (cbd300_InvCurrent &InvCurrent,
 }
 
 /*---------------------------------------------------------------------------*/
-
+/* conditional delete - PTS 1131961 FF */
 externCpp void
 bd400DelFromOmsInvTree (cbd300_InvCurrent &InvCurrent,
                         tsp00_KeyPtr       pSecKey,
-                        tsp00_Int4         SecKeyLen)
+                        tsp00_Int4         SecKeyLen,
+                        tsp00_KeyPtr       pPrimKey,
+                        tsp00_Int4         PrimKeyLen,
+                        tsp00_Int4         FoundPrimKeySize,
+                        tsp00_KeyPtr       pFoundPrimKey,
+                        tsp00_Int4         FoundPrimKeyToCompareSize,
+                        tsp00_Int4        &FoundPrimKeyLen)
 {
     ROUTINE_DBG_MEO00 ("bd400DelFromOmsInvTree");
 
     tgg00_BasisError     &TrError = InvCurrent.curr_trans->trError_gg00;
-    tsp00_Int4            RecIndex;
+    tsp00_Int4            SecKeyIndex;
     tbd_searchresult      SecKeySearchResult;
     cbd600_Node           LeafNode (InvCurrent, tbd_node_request::fromConst(nr_for_update));
     cbd500_Tree           Tree     (LeafNode);
@@ -2035,7 +2173,7 @@ bd400DelFromOmsInvTree (cbd300_InvCurrent &InvCurrent,
     Tree.bd500FindNode (pSecKey, SecKeyLen, LEAF_LEVEL_BD00);
     if (TrError != e_ok) return;
 
-    LeafNode.bd600SearchRecord (pSecKey, SecKeyLen, RecIndex, SecKeySearchResult);
+    LeafNode.bd600SearchRecord (pSecKey, SecKeyLen, SecKeyIndex, SecKeySearchResult);
 
     if (thisfound != SecKeySearchResult)
     {
@@ -2043,10 +2181,33 @@ bd400DelFromOmsInvTree (cbd300_InvCurrent &InvCurrent,
         return;
     }
 
-    LeafNode.bd600DelRecordSpace (RecIndex);
-    if (e_ok != TrError) return;
+    tsp00_KeyPtr        pAuxPrimKey;
+    cbd450_InvListArray InvListArray (LeafNode.bd600RecPtr (SecKeyIndex));
+    tsp00_Int2          AuxPrimKeyLen;
+    tsp00_LcompResult   CompResult;
 
-    Tree.bd520LeafUnderFlow (pSecKey, SecKeyLen);
+    InvListArray.bd450GetPrimKey (FIRST_PRIM_KEY_INDEX_BD450, pAuxPrimKey, AuxPrimKeyLen);
+    FoundPrimKeyLen = AuxPrimKeyLen;
+
+    s30cmp (pPrimKey, POS_OFF_DIFF_BD00, (FoundPrimKeyToCompareSize < PrimKeyLen ? FoundPrimKeyToCompareSize : PrimKeyLen),
+        pAuxPrimKey, POS_OFF_DIFF_BD00,
+        (FoundPrimKeyToCompareSize < FoundPrimKeyLen ? FoundPrimKeyToCompareSize : FoundPrimKeyLen), CompResult);
+    if ( l_equal == CompResult )
+    {
+        LeafNode.bd600DelRecordSpace (SecKeyIndex);
+        if (e_ok != TrError) return;
+
+        Tree.bd520LeafUnderFlow (pSecKey, SecKeyLen);
+    }
+    else
+    {
+        SAPDB_RangeMove( __FILE__, 4,
+            PrimKeyLen,  FoundPrimKeySize,
+            pAuxPrimKey, POS_OFF_DIFF_BD00,
+            pFoundPrimKey, POS_OFF_DIFF_BD00,
+            FoundPrimKeyLen,  TrError);
+        TrError = e_primkey_does_not_match;
+    }
 }
 
 /*---------------------------------------------------------------------------*/
