@@ -193,7 +193,8 @@ bd401_KeyIsZeroKey (tsp00_KeyPtr      pKey,
 /*---------------------------------------------------------------------------*/
 
 static inline void 
-bd401_MoveToNextSubRoot (cbd600_Node       &Node,
+bd401_MoveToNextSubRoot (tgg00_BasisError   &TrError,
+                         cbd600_Node       &Node,
                          tsp00_Int4         RecIndex,
                          bool               bCountKeys,
                          bool               bForkBetweenLeftAndRightWayFound, 
@@ -689,7 +690,7 @@ bd401CalculatePageCount (cbd300_InvCurrent   &InvCurrent,
                     }                   
                 }
                 /* goto next subroot on the left way down */
-                bd401_MoveToNextSubRoot (Node, StartRecIndex, bCountKeys, 
+                bd401_MoveToNextSubRoot (TrError, Node, StartRecIndex, bCountKeys, 
                     bForkBetweenLeftAndRightWayFound, bStopDescent, StartLeafPno, PageCnt);
             }
         }
@@ -755,7 +756,7 @@ bd401CalculatePageCount (cbd300_InvCurrent   &InvCurrent,
                 PageCnt += Node.bd600LeafCount (FIRST_REC_INDEX_BD00, StopRecIndex-1);          
                 
                 /* goto next subroot on the right way down */
-                bd401_MoveToNextSubRoot (Node, StopRecIndex, bCountKeys, 
+                bd401_MoveToNextSubRoot (TrError, Node, StopRecIndex, bCountKeys, 
                     bForkBetweenLeftAndRightWayFound, bStopDescent, StopLeafPno, PageCnt);
             }
         }
@@ -900,7 +901,7 @@ bd401GetFromInvTree (cbd300_InvCurrent         &InvCurrent,
             {   /* get primkey iterator on subtrees */
                 cbd450_InvListRefSubTree      InvListRefSubTree (pRec);
 
-                cbd460_InvListSubtreeIterator InvListIter (Tree, InvListRefSubTree,
+                cbd460_InvListSubtreeIteratorNoBreak InvListIter (Tree, InvListRefSubTree,
                     pStartPrimKey, StartPrimKeyLen, pStopPrimKey, // PTS 1116651 TS 2002-07-16
                     StopPrimKeyLen, bAscendingKeyOrder, bIncludeStartKey);
                 
@@ -1409,7 +1410,8 @@ bd401_KeyIsZeroKey (tsp00_KeyPtr      pKey,
 /*---------------------------------------------------------------------------*/
 
 static inline void 
-bd401_MoveToNextSubRoot (cbd600_Node       &Node,
+bd401_MoveToNextSubRoot (tgg00_BasisError  &TrError,
+                         cbd600_Node       &Node,
                          tsp00_Int4         RecIndex,
                          bool               bCountKeys,
                          bool               bForkBetweenLeftAndRightWayFound, 
@@ -1447,6 +1449,9 @@ bd401_MoveToNextSubRoot (cbd600_Node       &Node,
             if ((Separator.bd520LeafCount() > 1) || !bForkBetweenLeftAndRightWayFound || bCountKeys )
             {
                 Node.bd600Move (Separator.bd520SubNodeId());
+                
+                if (e_ok != TrError) return; 
+
                 LeafPno = Node.bd600GetId(); 
             }
             else
