@@ -70,15 +70,7 @@
  *  DEFINES                                                                  *
  *===========================================================================*/
 
-#ifdef NO_INLINES
-# define _INLINE
-#else
-# define _INLINE    inline
-#endif
 
-#define CONCURRENT    true
-#define FOR_UPDATE    true
-#define ASC_KEY_ORDER true
 #define FILLCHAR      '\0'
 #define IGNORE_VWAIT true
 
@@ -126,7 +118,8 @@ bd402_PrepareNextLoop (tgg00_BasisError         &TrError,
 /*---------------------------------------------------------------------------*/
 
 static void
-bd402_SearchOneInvListForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
+bd402_SearchOneInvListForQualifiedRecords (
+        cbd300_InvCurrent        &InvCurrent,
         tgg00_FileId             &PrimFileId,
         tsp00_KeyPtr             &pStartPrimKey,
         tsp00_Int2               &StartPrimKeyLen,
@@ -174,12 +167,6 @@ bd402_SelectRecordFromResultFile (tgg00_TransContext       &Trans,
                                   tgg00_LockReqMode        &GrantedLock,
                                   tsp00_Int4               &CurResultCnt);
 
-/*---------------------------------------------------------------------------*/
-
-inline void
-bd402_UpdateRecordAccessStatistics (tgg00_FileId         &FileId,
-                                    const tgg00_MessType &MessageType,
-                                    const tsp00_Bool     &bUnqualified);
 
 /*===========================================================================*
  *  DEFINITION OF METHODS DECLARED IN gbd402.h (CODE)                        * 
@@ -215,12 +202,7 @@ bd402NextFromInvTree (cbd300_InvCurrent   &InvCurrent,
     tgg00_RecPtr  pRec = *RecIter;
 
     if ( NULL == pRec )
-    {
-        if ( bIncludeSecKey )
-            TrError = e_inv_list_not_found;
-        else
-            TrError = e_no_next_invkey;
-    }
+        TrError = bIncludeSecKey ? e_inv_list_not_found : e_no_next_invkey;
     else
         gg05KeyAssign ((tsp00_BytePtr) &pRec->recKey_gg00().keyVal_gg00(),
                        pRec->recKey_gg00().keyLen_gg00(), pSecKey, SecKeyLen, TrError);
@@ -271,7 +253,6 @@ bd402SearchIndexForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
 
     /* get iterator on records in the index tree */
     cbd600_Node   IndexLeaf (InvCurrent, tbd_node_request::fromConst(nr_for_read));
-    // cbd500_Tree   Tree  (IndexLeaf /*, pStartSecKey, &StartSecKeyLen PTS 1120696 UH 2003-02-19*/);
     cbd500_Tree   Tree (IndexLeaf, true ); // PTS 1122982 TS 2003-07-07
 
     if (e_ok != TrError) return;
@@ -289,10 +270,7 @@ bd402SearchIndexForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
         if ( mm_direct == SelectParam.sfp_bd_mess2_type() )
             TrError = e_inv_list_not_found;
         else
-            if (bAscendingKeyOrder)
-                TrError = e_no_next_record;
-            else
-                TrError = e_no_prev_record;
+            TrError = bAscendingKeyOrder ? e_no_next_record : e_no_prev_record;
     }
 
     while (e_ok == TrError)
@@ -305,7 +283,8 @@ bd402SearchIndexForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
                 {
                     /* resume from previous collection */
                     cbd460_InvListArrayIterator  KeyIter (pRec, pFirstPrimKey, FirstPrimKeyLen,
-                                                          pStopPrimKey, StopPrimKeyLen, bAscendingKeyOrder, bIncludeFirstPrimKey);
+                                                          pStopPrimKey, StopPrimKeyLen, 
+                                                          bAscendingKeyOrder, bIncludeFirstPrimKey);
 
                     bd402_SearchOneInvListForQualifiedRecords (InvCurrent, PrimFileId, pFirstPrimKey, FirstPrimKeyLen,
                             KeyIter, RecBufSize, RecBufPos, pRecBuf, SelectParam, StackDesc,
@@ -337,12 +316,14 @@ bd402SearchIndexForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
                 if (bUseFirstPrimKey)
                 {
                     /* resume from previous collection */
-                    cbd460_InvListSubtreeIterator  SubtreeIterator (Tree, InvListRefSubTree, // PTS 1116651 TS 2002-07-15
+                    cbd460_InvListSubtreeIterator  SubtreeIterator (Tree, RecIter, InvListRefSubTree, 
                             pFirstPrimKey,  FirstPrimKeyLen, pStopPrimKey , StopPrimKeyLen,
                             bAscendingKeyOrder, bIncludeFirstPrimKey);
+
                     if (e_ok != TrError) break;
 
-                    bd402_SearchOneInvListForQualifiedRecords (InvCurrent, PrimFileId, pFirstPrimKey, FirstPrimKeyLen,
+                    bd402_SearchOneInvListForQualifiedRecords (InvCurrent, PrimFileId, 
+                            pFirstPrimKey, FirstPrimKeyLen,
                             SubtreeIterator, RecBufSize, RecBufPos, pRecBuf, SelectParam,
                             StackDesc, GrantedLock, CurResultCnt, LastResultLen,
                             InitialResultLen, bPrimaryFileErr);
@@ -353,7 +334,7 @@ bd402SearchIndexForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
                 {
                     bIncludeFirstPrimKey = true; 
 
-                    cbd460_InvListSubtreeIterator SubtreeIterator (Tree, InvListRefSubTree, // PTS 1116651 TS 2002-07-15
+                    cbd460_InvListSubtreeIterator SubtreeIterator (Tree, RecIter, InvListRefSubTree, 
                             pStartPrimKey, StartPrimKeyLen, pStopPrimKey , StopPrimKeyLen,
                             bAscendingKeyOrder, bIncludeFirstPrimKey);
 
@@ -372,12 +353,10 @@ bd402SearchIndexForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
         }
 
         // PTS 1117878 UH 2002-10-17 begin
-        if ( bUnlockCommittedReadLock
-                &&
-                lckRowShare_egg00 == GrantedLock )
+        if ( bUnlockCommittedReadLock && lckRowShare_egg00 == GrantedLock )
         {
-            tgg00_BasisError auxError = TrError;
-            TrError                   = e_ok;
+            const tgg00_BasisError auxError = TrError;
+            TrError                         = e_ok;
 
 #           if COMPILEMODE_MEO00 >= SLOW_MEO00      
             t01sname   (bd_inv, "unlock key1:");
@@ -434,45 +413,38 @@ bd402SearchIndexForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
         {
         case e_wait_for_lock_release:
             TrError = e_ok;
-            pRec    = *RecIter;    // pRec could be invalidated by interruptable iterator PTS 1116651 TS 2002-07-15
+
             {  // PTS 1116651 TS 2002-07-15
                 tsp00_Key           reEntranceSecKey;
-                tsp00_Int2          reEntranceSecKeyLen = pRec->recKeyLen_gg00();
+                tsp00_Int2          reEntranceSecKeyLen = 0;
+                const tsp00_KeyPtr  pReEntranceSecKey   = ( tsp00_KeyPtr ) &reEntranceSecKey;
 
-                gg05KeyAssign (tsp00_KeyPtr(&(pRec->recKey_gg00().keyVal_gg00())), reEntranceSecKeyLen,
-                               (tsp00_KeyPtr) &reEntranceSecKey, reEntranceSecKeyLen, TrError);
-
-                if ( e_ok != TrError )
-                    return;
-
-                RecIter.bd510Break ();
-
-                if (e_ok != TrError) 
-                    break; // PTS 1121589 UH 2003-04-11 added
+                RecIter.bd510Break( pReEntranceSecKey, reEntranceSecKeyLen );
 
+                // PTS 1133803 UH 2005-02-07 k53 must(!) be called
+                
                 k53wait (*InvCurrent.curr_trans, SelectParam.sfp_bd_mess_type(), SelectParam.sfp_bd_mess2_type());
+
                 if (e_ok != TrError)
                     break; //PTS 1108068 A.K. 17/10/2000
 
                 // PTS 1117878 UH 2002-10-17
                 bUnlockCommittedReadLock = PrimFileId.fileHandling_gg00().includes(hsCollisionTest_egg00);
 
-                RecIter.bd510Continue( (tsp00_KeyPtr) &reEntranceSecKey, reEntranceSecKeyLen ); // PTS 1112883 UH 2001-22-29 not pFirstKey
+                RecIter.bd510Continue( pReEntranceSecKey, reEntranceSecKeyLen ); // PTS 1112883 UH 2001-22-29 not pFirstKey
 
                 if (e_ok != TrError) 
                     break; // PTS 1121589 UH 2003-04-11 added
             }  // PTS 1116651
 
             pRec = *RecIter;
+
             if ( NULL == pRec )
             {
                 if ( mm_direct == SelectParam.sfp_bd_mess2_type() )
                     TrError = e_inv_list_not_found;
                 else
-                    if (bAscendingKeyOrder)
-                        TrError = e_no_next_record;
-                    else
-                        TrError = e_no_prev_record;
+                    TrError = bAscendingKeyOrder ? e_no_next_record : e_no_prev_record;
             }
             break;
         case e_ok:
@@ -484,21 +456,16 @@ bd402SearchIndexForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
                 InvCurrent.curr_trans->trWarning_gg00.addElement (warn14_next_invlist);
 
                 pRec = *RecIter;
+
                 if (NULL == pRec)
                 {
                     if ( mm_direct == SelectParam.sfp_bd_mess2_type() )
                         TrError = e_inv_list_not_found;
                     else
-                        if (bAscendingKeyOrder)
-                            TrError = e_no_next_record;
-                        else
-                            TrError = e_no_prev_record;
+                        TrError = bAscendingKeyOrder ? e_no_next_record : e_no_prev_record;
                 }
                 else
-                {
-                    // gg05KeyAssign (pStartPrimKey, StartPrimKeyLen, pFirstPrimKey, FirstPrimKeyLen, TrError);
                     bIncludeFirstPrimKey = true;
-                }
             }
             break;
         default:
@@ -548,8 +515,7 @@ bd402SearchResultFileForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
                                          bAscendingKeyOrder, bIncludeFirstKey);
 
     /* get iterator on records in the index tree */
-    cbd600_Node           TreeLeaf (InvCurrent, tbd_node_request::fromConst(nr_for_read));
-    // cbd500_Tree   Tree (TreeLeaf /*, pStartKey, &StartKeyLen PTS 1120696 UH 2003-02-19*/);
+    cbd600_Node   TreeLeaf (InvCurrent, tbd_node_request::fromConst(nr_for_read));
     cbd500_Tree   Tree (TreeLeaf, true ); // PTS 1122982 TS 2003-07-07
     if (e_ok != TrError) return;
 
@@ -564,12 +530,7 @@ bd402SearchResultFileForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
     if (NULL == pRec)
     {
         if (mm_direct != SelectParam.sfp_bd_mess2_type())
-        {
-            if (bAscendingKeyOrder)
-                TrError = e_no_next_record;
-            else
-                TrError = e_no_prev_record;
-        }
+            TrError = bAscendingKeyOrder ? e_no_next_record : e_no_prev_record;
         else
             TrError = e_key_not_found;
     }
@@ -585,8 +546,8 @@ bd402SearchResultFileForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
         // PTS 1117878 UH 2002-10-17 begin
         if ( bUnlockCommittedReadLock && lckRowShare_egg00 == GrantedLock )
         {
-            tgg00_BasisError auxError = TrError;
-            TrError                   = e_ok;
+            const tgg00_BasisError auxError = TrError;
+            TrError                         = e_ok;
 
 #           if COMPILEMODE_MEO00 >= SLOW_MEO00      
             t01sname   (bd_inv, "unlock key2:");
@@ -628,12 +589,7 @@ bd402SearchResultFileForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
             {
                 pRec = *RecIter;
                 if ( NULL == pRec )
-                {
-                    if (bAscendingKeyOrder)
-                        TrError = e_no_next_record;
-                    else
-                        TrError = e_no_prev_record;
-                }
+                    TrError = bAscendingKeyOrder ? e_no_next_record : e_no_prev_record; 
                 else
                 {
                     LastResultLen = SelectParam.sfp_result_length();
@@ -643,22 +599,16 @@ bd402SearchResultFileForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
             break;
         case e_wait_for_lock_release:
             TrError = e_ok;
-            pRec    = *RecIter;
             {  // PTS 1116651 TS 2002-07-15
-                tsp00_Key           reEntranceSecKey;
-                tsp00_Int2          reEntranceSecKeyLen = pRec->recKeyLen_gg00();
+                tsp00_Key           reEntranceKey;
+                tsp00_Int2          reEntranceKeyLen = 0;
+                const tsp00_KeyPtr  pReEntranceKey   = ( tsp00_KeyPtr ) &reEntranceKey;
 
-                gg05KeyAssign (tsp00_KeyPtr(&(pRec->recKey_gg00().keyVal_gg00())), reEntranceSecKeyLen,
-                               (tsp00_KeyPtr) &reEntranceSecKey, reEntranceSecKeyLen, TrError);
+                RecIter.bd510Break( pReEntranceKey, reEntranceKeyLen );
 
                 if ( e_ok != TrError )
                     return;
 
-                RecIter.bd510Break ();
-
-                if (e_ok != TrError) 
-                    break; // PTS 1121589 UH 2003-04-11 added
-
                 k53wait (*InvCurrent.curr_trans, SelectParam.sfp_bd_mess_type(), SelectParam.sfp_bd_mess2_type());
 
                 if (e_ok != TrError)
@@ -667,25 +617,20 @@ bd402SearchResultFileForQualifiedRecords (cbd300_InvCurrent        &InvCurrent,
                 // PTS 1117878 UH 2002-10-17
                 bUnlockCommittedReadLock = PrimFileId.fileHandling_gg00().includes(hsCollisionTest_egg00);
 
-                RecIter.bd510Continue( (tsp00_KeyPtr) &reEntranceSecKey, reEntranceSecKeyLen ); // PTS 1112883 UH 2001-22-29 not pFirstKey
+                RecIter.bd510Continue( pReEntranceKey, reEntranceKeyLen ); // PTS 1112883 UH 2001-22-29 not pFirstKey
 
                 if (e_ok != TrError) 
                     break; // PTS 1121589 UH 2003-04-11 added
 
-            }  // PTS 1116651
+                pRec = *RecIter;
+                if ( NULL == pRec )
+                        TrError = bAscendingKeyOrder ? e_no_next_record : e_no_prev_record;
 
-            pRec = *RecIter;
-            if ( NULL == pRec )
-            {
-                if (bAscendingKeyOrder)
-                    TrError = e_no_next_record;
-                else
-                    TrError = e_no_prev_record;
-            }
+            }  // PTS 1116651
             break;
         case e_buffer_limit:
-            bd402_PrepareNextLoop (TrError, CurrentKey, CurrentKeyLen, pStartKey, StartKeyLen,
-                                   SelectParam, CurResultCnt);
+            bd402_PrepareNextLoop (TrError, CurrentKey, CurrentKeyLen, 
+                                   pStartKey, StartKeyLen, SelectParam, CurResultCnt);
             break;
         default:
             break;
@@ -1144,47 +1089,6 @@ bd402_PrepareNextLoop (tgg00_BasisError         &TrError,
     }
 }
 
-
-/*---------------------------------------------------------------------------*/
-
-inline void
-bd402_UpdateRecordAccessStatistics (tgg00_FileId         &FileId,
-                                    const tgg00_MessType &MessageType,
-                                    const tsp00_Bool     &bUnqualified)
-{
-
-    switch (MessageType)
-    {
-    case m_update:
-        if (tfnTable_egg00 == FileId.fileTfn_gg00())
-        {
-            b73cmd_count (iupdates_rows_read);
-            if (!bUnqualified)
-                b73cmd_count (iupdates_rows_upd);
-        }
-        break;
-
-    case m_delete:
-        if (tfnTable_egg00 == FileId.fileTfn_gg00())
-        {
-            b73cmd_count (ideletes_rows_read);
-            if (!bUnqualified)
-                b73cmd_count (ideletes_rows_del);
-        }
-        break;
-
-    default:
-        if ((tfnTable_egg00 == FileId.fileTfn_gg00())   ||
-                ((tfnTemp_egg00 == FileId.fileTfn_gg00()) &&
-                 (ttfnTempTable_egg00 >= FileId.fileTfnTemp_gg00())))
-        {
-            b73cmd_count (iselects_rows_read);
-            if (!bUnqualified)
-                b73cmd_count (iselects_rows_qual);
-        }
-    }
-}
-
 /*===========================================================================*
  *  END OF CODE                                                              *
  *===========================================================================*/
