@@ -196,7 +196,6 @@ private: //members
     /* there defined here to avoid that there are created       */
     /* everytime the functions bd403AddInvList is called        */
     cbd450_InvListRefSubTree m_InvListRefSubTree;
-    bool                     m_bStartKeyFound_Dummy;
 
 };
 
@@ -226,20 +225,6 @@ private:
  *  LOCAL FUNCTIONS (PROTOTYPES)                                             *
  *===========================================================================*/
 
-static cbd470_PrimKeyIterator *
-bd403_ConstructPrimKeyIter (cbd300_InvCurrent      &InvCurrent,
-                            char *                  Buffer,
-                            tsp00_KeyPtr            pSecKey,
-                            tsp00_Int4              SecKeyLen,
-                            tsp00_KeyPtr            pStartPrimKey,
-                            tsp00_Int4              StartPrimKeyLen,
-                            tsp00_KeyPtr            pStopPrimKey,
-                            tsp00_Int4              StopPrimKeyLen,
-                            tsp00_PageNo           &ErrorRootPno,
-                            bool                   &bStartPrimKeyFound);
-
-/*---------------------------------------------------------------------------*/
-
 static void
 bd403_MergeInvLists (cbd300_InvCurrent                       &MergeCurrent,
                      cgg200_MergeTarget <cbd403_SortElement> &MergedInvList,
@@ -597,7 +582,7 @@ cbd403_InvListSet::bd403AddInvList  (cbd510_RecordIterator & RecordIterator)
             *(m_ppInvListIterator + m_NumOfAllLists) = new (m_BuffPosPtr)
                 cbd460_InvListSubtreeIteratorNoBreak (m_InvTree, m_InvListRefSubTree,
                 m_pStartPrimKey, m_StartPrimKeyLen,  m_pStopPrimKey, m_StopPrimKeyLen,
-                ASCENDING_KEY_ORDER_BD510, INCLUDE_START_KEY_BD510, m_bStartKeyFound_Dummy);
+                ASCENDING_KEY_ORDER_BD510, INCLUDE_START_KEY_BD510);
         }
         break;
 
@@ -681,7 +666,7 @@ bd403CountPrimKeysInInvLists (cbd300_InvCurrent &InvCurrent,
             {
                 cbd450_InvListRefSubTree InvListRefSubTree (*InvListRecIter);
 
-                cbd460_InvListSubtreeIterator InvListIter (InvTree,  // PTS 1116651 TS 2002-07-17
+                cbd460_InvListSubtreeIterator InvListIter (InvTree, InvListRecIter,  
                     InvListRefSubTree, pStartPrimKey, StartPrimKeyLen,
                     pStopPrimKey, StopPrimKeyLen, ASCENDING_KEY_ORDER_BD510, INCLUDE_START_KEY_BD510);
 
@@ -738,41 +723,6 @@ bd403MergePrimKeysInInvLists (cbd300_InvCurrent  &MergeCurrent,
  *  LOCAL FUNCTIONS (CODE)                                                   *
  *===========================================================================*/
 
-static cbd470_PrimKeyIterator *
-bd403_ConstructPrimKeyIter (cbd300_InvCurrent      &InvCurrent,
-                            char *                  Buffer,
-                            tsp00_KeyPtr            pSecKey,
-                            tsp00_Int4              SecKeyLen,
-                            tsp00_KeyPtr            pStartPrimKey,
-                            tsp00_Int4              StartPrimKeyLen,
-                            tsp00_KeyPtr            pStopPrimKey,
-                            tsp00_Int4              StopPrimKeyLen,
-                            tsp00_PageNo           &ErrorRootPno,
-                            bool                   &bStartPrimKeyFound)
-{
-        ROUTINE_DBG_MEO00 ("bd403_ConstructPrimKeyIter");
-
-        /* Virtual constructor of a cbd470_PrimKeyIterator     */
-        /* Depending on the file a corresponding iterator      */
-        /* is constructed (in the Buffer) and returned         */
-        /* via the abstract basis class cbd470_PrimKeyIterator */
-
-        if (InvCurrent.curr_tree_id.fileType_gg00().includes(ftsPerm_egg00))
-        {
-            return new (ALIGN_BD470 (Buffer))
-                cbd470_PrimKeyIteratorOnInvTree (InvCurrent, pSecKey, SecKeyLen, pStartPrimKey,
-                StartPrimKeyLen, pStopPrimKey, StopPrimKeyLen, ASCENDING_KEY_ORDER_BD510, INCLUDE_START_KEY_BD510, ErrorRootPno);
-        }
-        else
-        {
-            return new (ALIGN_BD470 (Buffer))
-                cbd470_PrimKeyIteratorOnPrimTree (InvCurrent,pStartPrimKey, StartPrimKeyLen,
-                pStopPrimKey, StopPrimKeyLen, ASCENDING_KEY_ORDER_BD510, INCLUDE_START_KEY_BD510, ErrorRootPno, bStartPrimKeyFound);
-        };
-}
-
-/*---------------------------------------------------------------------------*/
-
 static void
 bd403_MergeInvLists (cbd300_InvCurrent                       &InvCurrent,
                      cgg200_MergeTarget <cbd403_SortElement> &MergedInvList,
