@@ -71,7 +71,6 @@
 # define _INLINE    inline
 #endif
 
-#define TRUE                    1
 #define TWO_BYTES_BD404         2
 #define COUNT_DUPLICATES_BD404  true
 
@@ -484,7 +483,7 @@ bd404_CountAllPrimKeysInInvList (cbd300_InvCurrent       &InvCurrent,
     k71inv_col_select (*InvCurrent.curr_trans, SelectParam, StackDesc, 
         pSecKey, SecKeyLen, pNullPrimKey, 0, bUnqualified);
 
-    if (TRUE == bUnqualified)
+    if (bUnqualified)
     {
         TrError        = e_ok;
         NumAllPrimKeys = 0;
@@ -593,7 +592,7 @@ bd404_CountQualifiedPrimKeysInInvList (cbd300_InvCurrent       &InvCurrent,
         {   /* get primkey iterator on subtrees */
             cbd450_InvListRefSubTree InvListRefSubTree (pInvRecord);
             
-            cbd460_InvListSubtreeIterator InvListIter (InvTree, InvListRefSubTree, // PTS 1116651 TS 2002-07-17
+            cbd460_InvListSubtreeIteratorNoBreak InvListIter (InvTree, InvListRefSubTree, // PTS 1116651 TS 2002-07-17
                 pStartPrimKey, StartPrimKeyLen, pStopPrimKey , StopPrimKeyLen, 
                 ASCENDING_KEY_ORDER_BD510, INCLUDE_START_KEY_BD510);
             
