@@ -79,7 +79,6 @@
  *  DEFINES                                                                  *
  *===========================================================================*/
 
-#define TRUE			    	    1
 #define OVERFLOW_MSG_1_BD405        "Empty Temp Page Chain Failed            "
 #define COUNT_DUPLICATES_BD405      true
 
@@ -199,13 +198,11 @@ bd405DiagnoseIndex (
             break;
         case ilkSubTree_egg00 :
             {
-                bool bDummy;
-                /* */
                 cbd450_InvListRefSubTree InvListRefSubTree (pRec);
 
                 cbd460_InvListSubtreeIteratorNoBreak  SubTreeIterator (InvTree, InvListRefSubTree,
                         pStartPrimKey, StartPrimKeyLen, pStopPrimKey , StopPrimKeyLen,
-                        bAscendingKeyOrder, bIncludeFirstPrimKey, bDummy);
+                        bAscendingKeyOrder, bIncludeFirstPrimKey);
                 if (e_ok != TrError) return;
 
                 bd405_CheckInvListForOneSecKey (InvCurrent, PrimCurrent, TempCurrent,
@@ -306,7 +303,7 @@ bd405ParticipantCreateIndex (
     tgg00_BasisError    &TrError           = PrimCurrent.curr_trans->trError_gg00;
     bool                 bMemorySortFailed = false;
 
-    if (TRUE == bUseMemorySort)
+    if (bUseMemorySort)
     {
         cbd491_CreateIndexParticipant   Participant (PrimCurrent, StackDesc, IndexCounter);
         cbd495_DoubleWriteQueue         InvQueue (InvCurrent, PrimQueuePno, SecQueuePno);
@@ -354,7 +351,7 @@ bd405ParticipantCreateIndex (
         }
     }
 
-    if ((bMemorySortFailed) || (TRUE != bUseMemorySort))
+    if ((bMemorySortFailed) || (!bUseMemorySort))
     {
         cbd300_InvCurrent InvTreeCurrent (*PrimCurrent.curr_trans, InvCurrent.curr_tree_id,
                                           tbd_node_request::fromConst (nr_for_update), tgg00_MessType::fromConst (m_create_index), // PTS 1110414 TS 2001-05-29
