@@ -75,7 +75,6 @@
 #define IN_REGION_BD490         true
 
 #define MAX_RETRIES_BD490       10
-#define TRUE			    	1
 
 #define CANCEL_MSG_1_BD490      "Index Coordinator Canceled              "
 #define SEND_MSG_1_BD490        "All ServerTasks Are Busy                "
@@ -278,8 +277,7 @@ cbd490_CreateIndexParallel::bd490BuildIndex()
 
     while 
         (
-        ((e_ok == m_TrError) && (TRUE != bNoMorePrimData)) ||
-        ((e_too_many_net_requests == m_TrError) && (RetryCount < MAX_RETRIES_BD490))
+        ((e_ok == m_TrError) && (! bNoMorePrimData)) || ((e_too_many_net_requests == m_TrError))
         )
 	{
         if ((e_ok == m_TrError) && (create_index == m_IndexMode))
@@ -308,7 +306,7 @@ cbd490_CreateIndexParallel::bd490BuildIndex()
         if  (
             ( 
             (e_ok == m_TrError) &&
-            ((NumSendOrders == m_NumUsableServerTasks) || (TRUE == bNoMorePrimData)) 
+            ((NumSendOrders == m_NumUsableServerTasks) || (bNoMorePrimData)) 
             )
             ||
             ((e_file_limit == m_TrError) && (NumSendOrders > 0))
@@ -329,10 +327,10 @@ cbd490_CreateIndexParallel::bd490BuildIndex()
         }
 
         if (
-            (create_index == m_IndexMode)                      &&
+            (create_index == m_IndexMode)              &&
             (
-            ((e_ok == m_TrError) && (TRUE == bNoMorePrimData)) || 
-            (e_file_limit == m_TrError                       )
+            ((e_ok == m_TrError) && (bNoMorePrimData)) || 
+            (e_file_limit == m_TrError               )
             )
             )
         {
@@ -341,7 +339,7 @@ cbd490_CreateIndexParallel::bd490BuildIndex()
 
         if  (
             (e_ok == m_TrError                   ) && 
-            (TRUE != bNoMorePrimData             ) &&
+            (! bNoMorePrimData                   ) &&
             (m_Trans.trRteCommPtr_gg00->to_cancel)
             )
         {
@@ -402,7 +400,7 @@ cbd490_CreateIndexParallel::bd490_ProcessKeyRangeByCoordinator (
         if (NIL_PAGE_NO_GG00 != FirstLeafPno)   
             m_FileList.bd496ConvertFile (CurrFileIndex, FirstLeafPno);
         
-        if (TRUE != bEntriesExist)
+        if (! bEntriesExist)
             m_FileList.bd496DeleteFile (CurrFileIndex);
         
         if (e_ok == m_TrError) bd490_AddIndexCounter (m_IndexCounter, AuxCounter);
@@ -647,8 +645,8 @@ cbd490_CreateIndexParallel::bd490_WaitForReply (
         (
         (e_ok == m_TrError) &&
         (
-        ((0 < NumSendOrders) && (TRUE == bNoMorePrimData)                      ) || 
-        ((NumSendOrders == m_NumUsableServerTasks) && (TRUE != bNoMorePrimData))
+        ((0 < NumSendOrders) && (bNoMorePrimData)                        ) || 
+        ((NumSendOrders == m_NumUsableServerTasks) && (! bNoMorePrimData))
         )
         )
         ||
@@ -688,7 +686,7 @@ cbd490_CreateIndexParallel::bd490_WaitForReply (
                 m_FileList.bd496ConvertFile (ReceiveTrans.tcdChild_gg00, 
                                              ReceiveDataPart.rdpFirstLeafPno);
             
-            if (TRUE != ReceiveDataPart.rdpEntriesExist)
+            if (! ReceiveDataPart.rdpEntriesExist)
             {
                 // Result file from servertask is empty
                 m_FileList.bd496DeleteFile (ReceiveTrans.tcdChild_gg00);
