@@ -86,7 +86,6 @@
 #define FIRST_COLUMN_BD491          1
 #define MAX_ALLOC_RETRIES_BD491     2
 #define NUM_PRIM_KEY_DESC_BD491     1
-#define TRUE                        1
 
 #define ALLOCATE_MSG_1_BD491        "Allocate Failed: No More Memory         "
 #define ALLOCATE_MSG_2_BD491        "Realloc Of Dynamic Vector Not Possible  "
@@ -435,7 +434,8 @@ bd491_Malloc (tsp00_Int4    AllocSize,
     {
         vmalloc (AllocSize, &pObject, &bOkay);
 
-        if (TRUE == bOkay) return (pObject);
+        if (bOkay) 
+            return (pObject);
 
         ++RetryCnt;
         vsleep (TaskId, 1);
@@ -1364,7 +1364,7 @@ cbd491_CreateIndexParticipant::bd491MemorySort(
     {
         bEntriesExist = (0 < InvRecDescList.bd491_NumRecDescUsed());
 
-        if( TRUE == bEntriesExist )
+        if( bEntriesExist )
         {
             cbd491_CompareInvRecDescItem CompareOperator (m_Trans, m_Stack);
 
