@@ -68,7 +68,6 @@
 #define FIRST_FILE_INDEX_BD496		0
 #define NUM_RESULT_FILES_BD496		62
 #define NUM_OVERFLOW_FILES_BD496	2
-#define TRUE						1
 
 #define MERGE_MSG_1_BD496			"Perm Leaf Pages Read    "
 #define MERGE_MSG_2_BD496			"Perm Records Read       "
@@ -524,7 +523,7 @@ cbd496_FileHandling::bd496MergeFiles (
 
     m_TrError = e_ok; // m_TrError could be only e_file_limit
 
-    if (TRUE == bNoMorePrimData)
+    if (bNoMorePrimData)
     {
         g01opmsg (sp3p_console, sp3m_info, BD496_MERGE_SP03, csp3_n_index,
                   MERGE_MSG_7_BD496, IndexCounter.idc_prim_leafnodes);
@@ -545,7 +544,7 @@ cbd496_FileHandling::bd496MergeFiles (
     {
         ++m_MergeCount;
 
-        if (TRUE == bNoMorePrimData)
+        if (bNoMorePrimData)
         {
             // The decision if the last Funnel Merge step could be started depends
             // on two factors. First: all primary data is scaned. Second: all object
@@ -611,7 +610,7 @@ cbd496_FileHandling::bd496MergeFiles (
             g01opmsg (sp3p_console, sp3m_info, BD496_MERGE_SP03,
                       csp3_n_index, MERGE_MSG_6_BD496, m_MergeCount);
     }
-    while ((TRUE == bNoMorePrimData) && (!bLastMergeStep) && (e_ok == m_TrError));
+    while ((bNoMorePrimData) && (!bLastMergeStep) && (e_ok == m_TrError));
 
     if (e_sysbuffer_overflow == m_TrError)
         g01optextmsg (sp3p_knldiag, sp3m_warning, BD496_OVERFLOW_1_SP03,
