@@ -287,8 +287,9 @@ cbd500_Tree::bd500_CheckStopKeyAndInterrupt (bool bAscending)
                     tsp00_KeyPtr rePositionKey = reinterpret_cast<tsp00_KeyPtr>(&rePositionKeyValue);
                     tsp00_Int2   rePositionKeyLen; 
                     
-                    bd500Break ( bAscending, rePositionKey, rePositionKeyLen);
-                    
+                    const tsp00_Int4 recIndex = bAscending ? FIRST_REC_INDEX_BD00 : m_Node.bd600MaxRecIndex();
+                    bd500Break( recIndex, rePositionKey, rePositionKeyLen );
+
                     // TorstenS 2002-07-17: reduce data cache priority problems by rescheduling the task
                     // tsp00_TaskId taskId;  PTS 1121589 UH 2003-04-11 exchanged with trTaskId
                     // vgetpid( taskId );    PTS 1121589 UH 2003-04-11 exchanged with trTaskId
@@ -653,40 +654,14 @@ cbd500_Tree::bd500LeftNeighborId (cbd600_Node &auxNode) const
 /*---------------------------------------------------------------------------*/
 
 void
-cbd500_Tree::bd500Break ( bool bAscending )
+cbd500_Tree::bd500Break( 
+    const tsp00_Int4    recIndex,
+    const tsp00_KeyPtr  rePositionKey,      
+    tsp00_Int2          &rePositionKeyLen ) 
 {
-    // PTS 1121589 UH 2003-04-11 new
     ROUTINE_DBG_MEO00 ("bd500Break");
 
-    tgg00_RecPtr pRec;
-    if (bAscending)
-        pRec = m_Node.bd600RecPtr(FIRST_REC_INDEX_BD00);
-    else
-        pRec = m_Node.bd600RecPtr(m_Node.bd600MaxRecIndex());
-
-#   if COMPILEMODE_MEO00 >= SLOW_MEO00 
-    g01opmsg (sp3p_knldiag, sp3m_info, csp3_bd_msg, csp3_n_btree,
-          "BD500: Interrupting  Id:" , m_Node.m_pNode->nd_id() );
-#   endif
-    m_Node.bd600Release(IS_CHANGED_BD600);
-    m_Node.m_Current.bd300UnlockTree();
-
-}
-
-/*---------------------------------------------------------------------------*/
-
-void
-cbd500_Tree::bd500Break ( bool          bAscending,
-                          tsp00_KeyPtr  rePositionKey,     // PTS 1121589 UH 2003-04-11 added
-                          tsp00_Int2   &rePositionKeyLen ) // PTS 1121589 UH 2003-04-11 added
-{
-    ROUTINE_DBG_MEO00 ("bd500Break_with_key");
-
-    tgg00_RecPtr pRec;
-    if (bAscending)
-        pRec = m_Node.bd600RecPtr(FIRST_REC_INDEX_BD00);
-    else
-        pRec = m_Node.bd600RecPtr(m_Node.bd600MaxRecIndex());
+    const tgg00_RecPtr pRec = m_Node.bd600RecPtr( recIndex );
 
     gg05KeyAssign (REINTERPRET_CAST(tsp00_KeyPtr,pRec)+cgg_rec_key_offset, pRec->recKeyLen_gg00(),
         rePositionKey, rePositionKeyLen, m_TrError);
@@ -696,9 +671,9 @@ cbd500_Tree::bd500Break ( bool          bAscending,
     g01opmsg (sp3p_knldiag, sp3m_info, csp3_bd_msg, csp3_n_btree,
           "BD500: Interrupting  Id:" , m_Node.m_pNode->nd_id() );
 #   endif
+
     m_Node.bd600Release(IS_CHANGED_BD600);
     m_Node.m_Current.bd300UnlockTree();
-
 }
 
 /*---------------------------------------------------------------------------*/
@@ -822,23 +797,26 @@ cbd500_SubTree::bd500_CheckStopKeyAndInterrupt (bool bAscending)
         }
         else
         {
-            if ( NULL != m_pTreeIsRequested )
+            if( ! m_Interruptible ) // PTS 1129329 TS 2004-04-26
+                return;
+            
+            if ( NULL == m_pTreeIsRequested )
+                return;
+
+            /* check if another task wants to access the tree */
+            *m_pTreeIsRequested =
+                ((nr_for_read == m_Node.m_Current.bd300RootRequestKind())                       &&
+                (bd20IsPageRequested (m_Node.m_Current.bd300RootPtrs().np_cbptr())              ||
+                bd20IsPageRequested (m_MotherTree.m_Node.m_Current.bd300RootPtrs().np_cbptr()) ||
+                m_MotherTree.m_Node.bd600NodeIsRequested()));
+            
+            /* check if this order was canceled */
+            if ( m_Node.m_Current.curr_trans->trRteCommPtr_gg00->to_cancel             && 
+                ! m_Node.m_Current.curr_trans->trState_gg00.includes(tsRollback_egg00))
             {
-                /* check if another task wants to access the tree */
-                *m_pTreeIsRequested =
-                    ((nr_for_read == m_Node.m_Current.bd300RootRequestKind())                       &&
-                    (bd20IsPageRequested (m_Node.m_Current.bd300RootPtrs().np_cbptr())              ||
-                    bd20IsPageRequested (m_MotherTree.m_Node.m_Current.bd300RootPtrs().np_cbptr()) ||
-                    m_MotherTree.m_Node.bd600NodeIsRequested()));
-                
-                /* check if this order was canceled */
-                if ( m_Node.m_Current.curr_trans->trRteCommPtr_gg00->to_cancel             && 
-                    ! m_Node.m_Current.curr_trans->trState_gg00.includes(tsRollback_egg00))
-                {
-                    m_TrError = e_cancelled;
-                    g01opmsg (sp3p_knldiag, sp3m_info, csp3_bd_msg, csp3_n_btree,
-                        "bd500_Inter: canceled   " , m_Node.m_Current.curr_trans->trTaskId_gg00);
-                }
+                m_TrError = e_cancelled;
+                g01opmsg (sp3p_knldiag, sp3m_info, csp3_bd_msg, csp3_n_btree,
+                    "bd500_Inter: canceled   " , m_Node.m_Current.curr_trans->trTaskId_gg00);
             }
         }
     }
