@@ -346,13 +346,22 @@ cbd500_Tree::bd520_Overflow (cbd600_Node        &Neighbor,
         {
             /* append optimization, i.e. introduce a new right neighbour page where the new */
             /* record has to be inserted without balancing the new and the current page     */  
-            if ( Neighbor.bd600IsAccessable() && (m_CurrLevel > LEAF_LEVEL_BD00) )
+            if ( Neighbor.bd600IsAccessable() ) // PTS 1130325 UH 2004-06-25
             {
                 // PTS 1109033 UH 2000-01-19 if-block added
                 NextNeighbor.bd600Exchange(Neighbor);
             }
             Neighbor.bd600New (m_Node.m_pNode->nd_level());
-            if (e_ok != m_TrError) return;
+            if (e_ok != m_TrError)
+            {
+				// PTS 1130325 UH 2004-06-25
+                bd500Print            (     "Neighbor.bd600New"       );
+                m_Node.bd600Dump      (4711,"Neighbor.bd600New...node");
+                Neighbor.bd600Dump    (4711,"Neighbor.bd600New...neig");
+                NextNeighbor.bd600Dump(4711,"Neighbor.bd600New...next");
+                g01abort (csp3_bd_msg, csp3_n_btree,"bd600New failed         ",4711);
+                return;
+            }
             
             ++NumNodeDiff;
             Neighbor.m_pNode->nd_right()         =  RightNeighbor;
@@ -363,7 +372,16 @@ cbd500_Tree::bd520_Overflow (cbd600_Node        &Neighbor,
             /* create separator between new and current page  */
 
             m_Node.bd600BuildSeparatorKey (IS_LEFT_NODE_GBD600, pKey, KeyLen, pSepKey, SepKeyLen);
-            if (e_ok != m_TrError) return;
+            if (e_ok != m_TrError)
+            {
+				// PTS 1130325 UH 2004-06-25
+                bd500Print            (     "m_Node.bd600BuildSeparatorKey");
+                m_Node.bd600Dump      (4712,"m_Node                  "     );
+                Neighbor.bd600Dump    (4712,"Neigbor                 "     );
+                NextNeighbor.bd600Dump(4712,"NextNeighbor            "     );
+                g01abort (csp3_bd_msg, csp3_n_btree,"bd600BuildSep failed    ",4712);
+                return;
+            }
 
             if ( m_Node.bd600IsRoot() )
             {
@@ -371,7 +389,16 @@ cbd500_Tree::bd520_Overflow (cbd600_Node        &Neighbor,
 
                 bd510_InsertNewLevel (Neighbor, pSepKey, SepKeyLen,
                     FIRST_REC_INDEX_BD00+1, RECORD_IN_RIGHT_NODE, NewLeftSubNode, NumNodeDiff);
-                if (e_ok != m_TrError) return;
+                if (e_ok != m_TrError)
+                {
+					// PTS 1130325 UH 2004-06-25
+                    bd500Print            (     "bd510_InsertNewLevel");
+                    m_Node.bd600Dump      (4713,"m_Node                  "     );
+                    Neighbor.bd600Dump    (4713,"Neigbor                 "     );
+                    NextNeighbor.bd600Dump(4713,"NextNeighbor            "     );
+                    g01abort (csp3_bd_msg, csp3_n_btree,"bd510_InsertNew failed  ",4713);
+                    return;
+                }
 
                 if ( g01vtrace.vtrAll_gg00 && g01glob.treecheck )
                     bd520_PrintBalancing (btNewLevel_ebd501, m_Node, Neighbor);
@@ -422,7 +449,7 @@ cbd500_Tree::bd520_Overflow (cbd600_Node        &Neighbor,
     if ( NIL_PAGE_NO_GG00 != RightNeighbor )
     {
         // PTS 1109033 UH 2000-01-19 begin
-        if ( Neighbor.bd600IsAccessable() && (m_CurrLevel > LEAF_LEVEL_BD00) )
+        if ( Neighbor.bd600IsAccessable() ) // PTS 1130325 UH 2004-06-25
         {
             if ( RightNeighbor != Neighbor.bd600GetId() )
             {
@@ -434,7 +461,16 @@ cbd500_Tree::bd520_Overflow (cbd600_Node        &Neighbor,
             Neighbor.bd600GetNode (RightNeighbor);
         // PTS 1109033 UH 2000-01-19 end
 
-        if (e_ok != m_TrError) return;
+        if (e_ok != m_TrError)
+        {
+			// PTS 1130325 UH 2004-06-25
+            bd500Print            (     "RightNeigbor access failed");
+            m_Node.bd600Dump      (4714,"m_Node                  "     );
+            Neighbor.bd600Dump    (4714,"Neigbor                 "     );
+            NextNeighbor.bd600Dump(4714,"NextNeighbor            "     );
+            g01abort (csp3_bd_msg, csp3_n_btree,"right distrib error     ",4714);
+            return;
+        }
 
         if ( (m_Node.bd600Covering() + Neighbor.bd600Covering() + RequiredLen) > (2 * FULLCOVERING_BD00) )
         {
@@ -471,14 +507,23 @@ cbd500_Tree::bd520_Overflow (cbd600_Node        &Neighbor,
         m_TrError = e_ok;
 
         tsp00_PageNo LeftNeighbor = bd500LeftNeighbor();
-        if ( e_ok != m_TrError ) return;
+        if ( e_ok != m_TrError )
+        {
+			// PTS 1130325 UH 2004-06-25
+            bd500Print            (     "bd500LeftNeighbor failed");
+            m_Node.bd600Dump      (4715,"m_Node                  "     );
+            Neighbor.bd600Dump    (4715,"Neigbor                 "     );
+            NextNeighbor.bd600Dump(4715,"NextNeighbor            "     );
+            g01abort (csp3_bd_msg, csp3_n_btree,"left distrib error      ",4715);
+            return;
+        }
 
         if ( NIL_PAGE_NO_GG00 == LeftNeighbor )
             m_TrError = e_no_more_space;
         else
         {
             // PTS 1109033 UH 2001-01-19 begin
-            if ( Neighbor.bd600IsAccessable() && (m_CurrLevel > LEAF_LEVEL_BD00) )
+            if ( Neighbor.bd600IsAccessable() ) // PTS 1130325 UH 2004-06-25
             {
                 if ( LeftNeighbor != Neighbor.bd600GetId() )
                 {
@@ -487,10 +532,34 @@ cbd500_Tree::bd520_Overflow (cbd600_Node        &Neighbor,
                 }
             }
             else
-                Neighbor.bd600GetNode (LeftNeighbor);
+				// PTS 1130325 UH 2004-06-25 in addition check NextNeighbor too
+                if ( NextNeighbor.bd600IsAccessable() )
+                {
+                    if ( LeftNeighbor == NextNeighbor.bd600GetId() )
+                        Neighbor.bd600Exchange(NextNeighbor);
+                    else
+                    {
+                        bd500Print            (     "NextNeighbor unexpected occupied");
+                        m_Node.bd600Dump      (4718,"m_Node                  "     );
+                        Neighbor.bd600Dump    (4718,"Neigbor                 "     );
+                        NextNeighbor.bd600Dump(4718,"NextNeighbor            "     );
+                        g01abort (csp3_bd_msg, csp3_n_btree,"NextNeighbor unexpected ",4718);
+                    }
+                }
+                else
+                    Neighbor.bd600GetNode (LeftNeighbor);
             // PTS 1109033 UH 2001-01-19 end
 
-            if ( e_ok == m_TrError )
+            if ( e_ok != m_TrError )
+            {
+				// PTS 1130325 UH 2004-06-25
+                bd500Print            (     "LeftNeighbor access failed");
+                m_Node.bd600Dump      (4716,"m_Node                  "     );
+                Neighbor.bd600Dump    (4716,"Neigbor                 "     );
+                NextNeighbor.bd600Dump(4716,"NextNeighbor            "     );
+                g01abort (csp3_bd_msg, csp3_n_btree,"LeftNeighbor access fail",4716);
+            }
+            else
             {
                 if (((m_Node.bd600Covering() + Neighbor.bd600Covering() + RequiredLen) >
                     2 * FULLCOVERING_BD00) || 
@@ -527,14 +596,23 @@ cbd500_Tree::bd520_Overflow (cbd600_Node        &Neighbor,
     {
         m_TrError = e_ok;
 
-        if ( Neighbor.bd600IsAccessable() && (m_CurrLevel > LEAF_LEVEL_BD00) )
+        if ( Neighbor.bd600IsAccessable() ) // PTS 1130325 UH 2004-06-25
         {
             // PTS 1109033 UH 2000-01-19 if-block added
             NextNeighbor.bd600Exchange(Neighbor);
         }
 
         Neighbor.bd600New (m_Node.m_pNode->nd_level());
-        if (e_ok != m_TrError) return;
+        if (e_ok != m_TrError)
+        {
+			// PTS 1130325 UH 2004-06-25
+            bd500Print            (     "New right neighbor failed");
+            m_Node.bd600Dump      (4717,"m_Node                  "     );
+            Neighbor.bd600Dump    (4717,"Neigbor                 "     );
+            NextNeighbor.bd600Dump(4717,"NextNeighbor            "     );
+            g01abort (csp3_bd_msg, csp3_n_btree,"New right neighbor faile",4717);
+            return;
+        }
 
         ++NumNodeDiff;
         Neighbor.m_pNode->nd_right() = RightNeighbor;
@@ -1161,7 +1239,7 @@ cbd500_Tree::bd520_UnderFlow (cbd600_Node        &Neighbor,
         else
         {
             /* PTS 1109619 UH 2001-03-13 begin */
-            if ( Neighbor.bd600IsAccessable() && (m_CurrLevel > LEAF_LEVEL_BD00) )
+            if ( Neighbor.bd600IsAccessable() ) // PTS 1130325 UH 2004-06-25
             {
                 if ( LeftNeighbor != Neighbor.bd600GetId() )
                 {
@@ -1205,7 +1283,7 @@ cbd500_Tree::bd520_UnderFlow (cbd600_Node        &Neighbor,
     else
     {
         /* PTS 1109619 UH 2001-03-13 begin */
-        if ( Neighbor.bd600IsAccessable() && (m_CurrLevel > LEAF_LEVEL_BD00) )
+        if ( Neighbor.bd600IsAccessable() ) // PTS 1130325 UH 2004-06-25
         {
             if ( m_Node.m_pNode->nd_right() != Neighbor.bd600GetId() )
             {
@@ -1331,7 +1409,7 @@ cbd500_Tree::bd520_UnderFlow (cbd600_Node        &Neighbor,
         else
         {
             /* PTS 1109619 UH 2001-03-13 begin */
-            if ( Neighbor.bd600IsAccessable() && (m_CurrLevel > LEAF_LEVEL_BD00) )
+            if ( Neighbor.bd600IsAccessable() ) // PTS 1130325 UH 2004-06-25
             {
                 if ( LeftNeighbor != Neighbor.bd600GetId() )
                 {
@@ -1377,7 +1455,7 @@ cbd500_Tree::bd520_UnderFlow (cbd600_Node        &Neighbor,
                         }
                         else
                         {
-                            NewOrders.iosOrderCount_bd501 = 0;
+                            // NewOrders.iosOrderCount_bd501 = 0; PTS 1131880 UH 2004-10-13
                             if ( NeighborLeafCount != Neighbor.bd600LeafCount() ) /* PTS 1104320 UH 27-10-1999 */
                             {
                                 NewOrders.bd501CreateNew ( iokChange_ebd501,
@@ -1645,7 +1723,7 @@ cbd500_Tree::bd520_DistributeFromLeft (cbd600_Node         &Neighbor,
 
     /* update the index nodes above */
 
-    NewOrders.iosOrderCount_bd501 = 0;
+    // NewOrders.iosOrderCount_bd501 = 0;  PTS 1131880 UH 2004-10-13
     if ( NeighborLeafCount != Neighbor.bd600LeafCount() ) /* PTS 1104320 UH 27-10-1999 */
         NewOrders.bd501CreateNew ( iokChange_ebd501, Neighbor.m_pNode,
                                    m_Node.bd600GetId(),   // PTS 1112713 UH 2001-12-07
