@@ -647,7 +647,8 @@ bd91DumpGarbColl (tgg00_VfFileref     &HostFile,
 {
     ROUTINE_DBG_MEO00 ("bd91DumpGarbColl");
 
-    bd91GarbCollController.bd930DumpGCController (HostFile, BufferSize,  Buffer, OutPno,  OutPos, HostError, ErrText);
+	if (g01is_livecache ()) 
+		bd91GarbCollController.bd930DumpGCController (HostFile, BufferSize,  Buffer, OutPno,  OutPos, HostError, ErrText);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1446,9 +1447,7 @@ bd91ReleaseUnusedObject (tgg00_TransContext      &Trans,
   /* get pointer to subroot page which is required if this operation forces an update of the page chain */	
   cbd910_SubRootNode    NptrsSubRoot  (Current, nr_for_update);
   
-  if ((NptrsPage.np_ptr()->nd_free_obj_cnt() == (NptrsPage.np_ptr()->nd_max_obj_cnt()-1)) ||
-      (!NptrsPage.np_ptr()->nd_in_free_chain()))         // do equivalent checking as within bd91RemoveObjFromFile 
-      //(NptrsPage.np_ptr()->nd_free_obj_cnt() == 0))    // for usage of root page      PTS 1119428 FF 2002-DEC-17
+  if ( NptrsPage.bd910LockSubRootBeforeReleasingFrame()) 
   {	
     /* release history page to avoid a deadlock;  PTS 1114691 FF 2002-03-07 */
     historyIterator.Invalidate();   
@@ -1474,13 +1473,11 @@ bd91ReleaseUnusedObject (tgg00_TransContext      &Trans,
     }
 
     /* check if chaining will still be needed ( meahnwhile the page may have changed a lot */
-    if (!((NptrsPage.np_ptr()->nd_free_obj_cnt() == (NptrsPage.np_ptr()->nd_max_obj_cnt()-1)) ||
-        (!NptrsPage.np_ptr()->nd_in_free_chain())))     // do equivalent checking as within bd91RemoveObjFromFile 
-        //(NptrsPage.np_ptr()->nd_free_obj_cnt() == 0)))// for usage of root page      PTS 1119428 FF 2002-DEC-17
-    {	
-        /* no chaining will be needed anymore ==> release subroot */
-        NptrsSubRoot.bd910ReleasePage();
-    }
+	if ( !NptrsPage.bd910LockSubRootBeforeReleasingFrame() ) 
+	{	
+		/* no chaining will be needed anymore ==> release subroot */
+		NptrsSubRoot.bd910ReleasePage();
+	}
   }
   
   /*------------------------------------------------------------------*/
@@ -1756,7 +1753,16 @@ bd91RemoveObjFromFile (tbd_current_tree        &Current,
     /*-------------------------------------------------------*/
     bd91RemoveObjFromPage (Current,  NptrsPage,  pObjFrame,  OId, ObjFileInfo, ErrorTraceObject, 
         bKeyAlreadyRemoved); // new PTS 1128894 FF 31-MAR-2004
-    if (e_ok != TrError) return;
+    if (e_ok != TrError)
+    {
+        if (!bd90ObjFileDir.bd920IsUsed(NptrsPage.np_ptr()->nd_obj_file_no()) || 
+            ObjFileInfo.ofiRelFileIsReq_bd900)
+        {
+            ErrorTraceObject.SetStatus (rosFileJustDeleted_gg12);
+            TrError = e_ok;
+        }
+        return;
+    }
     
     /*--------------------------------------------------------------------*/
     /* if the page becomes an empty page by removing the object it is     */ 
@@ -1847,8 +1853,15 @@ bd91RemoveObjFromPage (tbd_current_tree        &Current,
         if (e_hash_key_not_found == TrError)  TrError = e_ok;
         if (e_ok != TrError) 
         {
-            g01opmsg (sp3p_knldiag, sp3m_error, csp3_bd_msg, csp3_n_obj, "b91RmObjFP:bd22RemoveOid", TrError);
             ErrorTraceObject.SetStatus (rosDelKeyFailed_gg12);   /* PTS 1106760  Alexander Kley 25/06/2000 */
+            if (!bd90ObjFileDir.bd920IsUsed(Current.curr_tree_id.fileObjFileNo_gg00()) || 
+                ObjFileInfo.ofiRelFileIsReq_bd900)
+            {
+                ErrorTraceObject.SetStatus (rosFileJustDeleted_gg12);
+                TrError = e_ok;
+            }
+            else
+                g01opmsg (sp3p_knldiag, sp3m_error, csp3_bd_msg, csp3_n_obj, "b91RmObjFP:bd22RemoveOid", TrError);
             return;
         }
     }
