@@ -446,8 +446,8 @@ bd92VerifyObjPage (tbd_current_tree  &Current,
     
     tsp00_Int4 DummyResObjCnt = 0;
     
-    /* TRUE signals that no before-image-check is needed ) */
-    bd92_VerifyObjPage (Current, NptrsPage, DummyResObjCnt, TRUE );
+    /* true signals that no before-image-check is needed ) */
+    bd92_VerifyObjPage (Current, NptrsPage, DummyResObjCnt, true );
 }
 
 /*---------------------------------------------------------------------------*/
@@ -573,15 +573,14 @@ bd92_CheckBeforeImage (tgg00_TransContext   &Trans,
     /* PTS 1127493 FF 2004-FEB-02       */
     /* check if lock collision occurred */
     if (ObjTransInfo.otrLockTransId_gg00.gg90IsDummy())
-        bIsActive = TRUE;
+        bIsActive = true;
     else
         kb50CheckActiveTrans (Trans.trTaskId_gg00, 
         ObjTransInfo.otrLockTransId_gg00,
         ObjTransInfo.otrLockTransIndex_gg00,
         bIsActive);
-    Trans.trError_gg00 = saveError;                     // PTS 1121056
 
-    if (bIsActive != TRUE)
+    if (! bIsActive)
     {
         if ( ! Log_History::GetInstance().BeforeImageExists ( Trans,
             WantedOid,
@@ -599,6 +598,9 @@ bd92_CheckBeforeImage (tgg00_TransContext   &Trans,
                 Trans.trError_gg00 = e_obj_history_not_found;
         }
     }
+
+    if ( e_ok != saveError )
+        Trans.trError_gg00 = saveError;                 
 }
 
 /*---------------------------------------------------------------------------*/
@@ -962,6 +964,8 @@ bd92_VerifyObjPageChain (tbd_current_tree        &Current,
                 break;
             else
             {
+                if (e_ok == saveError) saveError = TrError;     
+                TrError = e_ok;
                 NptrsPage.bd910AssignToPage (NextPageNo);
                 if (e_ok != TrError) 
                 {
@@ -1216,7 +1220,7 @@ bd92_VerifyObjDataPage (tbd_current_tree &Current,
                     /* PTS 1127493 FF 2004-FEB-02       */
                     /* check if lock collision occurred */
                     if (pObjFrame->objHeader_bd900.ohdTransInfo_gg92.otrLockTransId_gg00.gg90IsDummy())
-                        bIsActive = TRUE;
+                        bIsActive = true;
                     else
                         kb50CheckActiveTrans (Current.curr_trans->trTaskId_gg00, 
                         pObjFrame->objHeader_bd900.ohdTransInfo_gg92.otrLockTransId_gg00,
@@ -1280,6 +1284,10 @@ bd92_VerifyObjDataPage (tbd_current_tree &Current,
                 /* the hash is not tested because of the objects     */
                 /* created temporaly  when an older object is reused */
                 ++FeotObjCnt;
+                /* the following check isn't permitted, because if a GC is working actually on */
+                /* an entry, then this entry may be marked as deleted within the history,      */
+                /* but the object frame remains still unchanged             FF 2004-FEB-02     */
+                break; 
                 if (!bThisTaskIsAGarbColl && // a GC could create a dead lock
                     Kernel_IAdminInfo::Instance().KernelStateIsOnline() && // PTS 1121056 LockList not yet initialized
                     !bChainIsDeleted )     
@@ -1300,19 +1308,20 @@ bd92_VerifyObjDataPage (tbd_current_tree &Current,
                     Kernel_IAdminInfo::Instance().KernelStateIsOnline())      // PTS 1121056
                 {
                     tgg00_BasisError      saveError = TrError;                // PTS 1121056
+                    TrError = e_ok;
                     /* since reserved and commited objects must not exist */ 
                     /* reserved objects must belong to open transactions  */
 
                     /* PTS 1127493 FF 2004-FEB-02       */
                     /* check if lock collision occurred */
                     if (pObjFrame->objHeader_bd900.ohdTransInfo_gg92.otrLockTransId_gg00.gg90IsDummy())
-                        bIsActive = TRUE;
+                        bIsActive = true;
                     else
                         kb50CheckActiveTrans (Current.curr_trans->trTaskId_gg00, 
                         pObjFrame->objHeader_bd900.ohdTransInfo_gg92.otrLockTransId_gg00,
                         pObjFrame->objHeader_bd900.ohdTransInfo_gg92.otrLockTransIndex_gg00,
                         bIsActive);
-                    TrError = saveError;                                      // PTS 1121056
+                    if ( e_ok != saveError ) TrError = saveError;                                      
                     if (!bIsActive)
                     {
                         Kernel_OpError  opErrMsg( csp3_bd_msg, csp3_n_obj );
