@@ -1046,7 +1046,7 @@ bd93_NewContObj (tbd_current_tree        &Current,                 // [inp]
 	/* CRS 1106534 Alexander Kley 2000-05-11                */
 #   ifdef STORE_OBJECTS_ON_SUBROOT_BD900	 
 	if (NptrsSubRoot.np_ptr()->nd_free_obj_cnt() > 0)
-		NptrsBody.bd910SetTo (NptrsSubRoot);
+		NptrsBody.bd910SetTo (NptrsSubRoot, true /*movePrimary PTS 1130598*/);
 	else
 #   endif
 	{
@@ -1164,8 +1164,7 @@ bd93_ReleaseContObjFrame (tbd_current_tree       &Current,
     /* get pointer to subroot page */	
 	cbd910_SubRootNode   NptrsSubRoot (Current, nr_for_update);  
 
-    if ((NptrsContObjPage.np_ptr()->nd_free_obj_cnt() == (NptrsContObjPage.np_ptr()->nd_max_obj_cnt()-1)) ||
-		(NptrsContObjPage.np_ptr()->nd_free_obj_cnt() == 0)) 
+	if ( NptrsContObjPage.bd910LockSubRootBeforeReleasingFrame() ) 
     {
         NptrsSubRoot.bd910AssignToSubroot (NptrsContObjPage);
 		if (e_ok != TrError) 
@@ -1183,12 +1182,11 @@ bd93_ReleaseContObjFrame (tbd_current_tree       &Current,
         }
 
         /* check if chaining will still be needed ( meahnwhile the page may have changed a lot */
-        if (!((NptrsContObjPage.np_ptr()->nd_free_obj_cnt() == (NptrsContObjPage.np_ptr()->nd_max_obj_cnt()-1)) ||
-            (NptrsContObjPage.np_ptr()->nd_free_obj_cnt() == 0)))
-        {	
-            /* no chaining will be needed anymore ==> release subroot */
-            NptrsSubRoot.bd910ReleasePage();
-        }
+		if ( !NptrsContObjPage.bd910LockSubRootBeforeReleasingFrame() ) 
+		{	
+			/* no chaining will be needed anymore ==> release subroot */
+			NptrsSubRoot.bd910ReleasePage();
+		}
     }
 
 
