@@ -60,6 +60,7 @@
 #include "hbd17.h"
 #include "hbd21.h"
 #include "hbd22.h"  //OMS Key Management
+#include "heo00x.h"
 #include "heo55k.h" //vbegexcl
 #include "heo56.h"  //vsuspend
 #include "SAPDB/SAPDBCommon/SAPDB_RangeCode.hpp" // Kernel_move_and_fill
@@ -336,7 +337,7 @@ cbd930_GarbageCollector::cbd930_GarbageCollector (tgg00_TransContext        &Tra
     while (true)
     {
         /*=====================================================*/
-        vsuspend (Trans.trTaskId_gg00, 228);
+        vsuspend (Trans.trTaskId_gg00, TSK_VSUSPEND_REASON_NO_WORK);
         /*=====================================================*/
 
         bd930_WriteMessageIntoVtrace(GC_IS_STARTED);
@@ -785,6 +786,7 @@ cbd930_GarbageCollector::bd930_ReleaseAllPagesOfOneChain (tgg00_FileId        &O
             {
                 --SubRootPageInfo.bd900FreePageCnt();
             }
+            ChainInfo.ociFreePageCnt_bd900 = SubRootPageInfo.bd900FreePageCnt();  /* PTS 1130998 */
             // PTS 1114176 FF#           endif
 
             RightPageNo = NptrsPage.np_ptr()->nd_right();
