@@ -288,10 +288,7 @@ bd95RollforwardObj_DelObj (tgg00_TransContext       &Trans,
     
     /* get pointer to subroot page which is required if this operation forces an update of the page chain */	
     cbd910_SubRootNode    NptrsSubRoot  (Current, nr_for_update);
-    
-    if ((NptrsPage.np_ptr()->nd_free_obj_cnt() == (NptrsPage.np_ptr()->nd_max_obj_cnt()-1)) ||
-        (!NptrsPage.np_ptr()->nd_in_free_chain()))         // do equivalent checking as within bd91RemoveObjFromFile 
-		//(NptrsPage.np_ptr()->nd_free_obj_cnt() == 0))    // for usage of root page      PTS 1119428 FF 2002-DEC-17
+	if ( NptrsPage.bd910LockSubRootBeforeReleasingFrame() ) 
     {
         NptrsSubRoot.bd910AssignToSubroot (NptrsPage);
         if (e_ok != TrError)
@@ -321,9 +318,7 @@ bd95RollforwardObj_DelObj (tgg00_TransContext       &Trans,
         }
         
         /* check if chaining will still be needed ( meahnwhile the page may have changed a lot */
-        if (!((NptrsPage.np_ptr()->nd_free_obj_cnt() == (NptrsPage.np_ptr()->nd_max_obj_cnt()-1)) ||
-            (!NptrsPage.np_ptr()->nd_in_free_chain())))     // do equivalent checking as within bd91RemoveObjFromFile 
-		    //(NptrsPage.np_ptr()->nd_free_obj_cnt() == 0)))// for usage of root page      PTS 1119428 FF 2002-DEC-17
+	    if ( !NptrsPage.bd910LockSubRootBeforeReleasingFrame() )
         {	
             /* no chaining will be needed anymore ==> release subroot */
             NptrsSubRoot.bd910ReleasePage();
@@ -404,7 +399,7 @@ bd95RollforwardObj_DelObj (tgg00_TransContext       &Trans,
     /* ----- after all checks the object can be released now ----- */
     /* =========================================================== */
     SAPDBMem_IRawAllocator &allocator = *(reinterpret_cast<SAPDBMem_IRawAllocator*>(Trans.trAllocator_gg00));
-    Data_PageAccessManager  dummyPam      (Trans, Data_UndoFile, Data_PageNo());
+    Data_PageAccessManager  dummyPam      (Trans, Data_UndoFile, Data_PageRecoveryMode(), Data_PageNo());
     Log_UndoFile::Iterator  dummyIterator (dummyPam, allocator);
     
     const bool bKeyAlreadyRemoved = true; // new PTS 1128894 FF 31-MAR-2004
@@ -542,9 +537,7 @@ bd95RollforwardObj_NewObj (tgg00_TransContext       &Trans,
                     }
                     else
                     {                       
-                        if ((NptrsPage.np_ptr()->nd_free_obj_cnt() == (NptrsPage.np_ptr()->nd_max_obj_cnt()-1)) ||
-                            (!NptrsPage.np_ptr()->nd_in_free_chain()))         // do equivalent checking as within bd91RemoveObjFromFile 
-                            //(NptrsPage.np_ptr()->nd_free_obj_cnt() == 0))    // for usage of root page      PTS 1119428 FF 2002-DEC-17
+					    if ( NptrsPage.bd910LockSubRootBeforeReleasingFrame() )
                         {
                             tgg00_BasisError AuxError = e_ok;    // PTS 1122026 FF
                             NptrsSubRoot.bd910AssignToSubroot (NptrsPage);
@@ -601,9 +594,7 @@ bd95RollforwardObj_NewObj (tgg00_TransContext       &Trans,
                     if (!bCurrOwningOIdIsAlreadyRemoved)
                     {
                         /* check if chaining will still be needed ( meahnwhile the page may have changed a lot */
-                        if (!((NptrsPage.np_ptr()->nd_free_obj_cnt() == (NptrsPage.np_ptr()->nd_max_obj_cnt()-1)) ||
-                            (!NptrsPage.np_ptr()->nd_in_free_chain())))     // do equivalent checking as within bd91RemoveObjFromFile 
-                            //(NptrsPage.np_ptr()->nd_free_obj_cnt() == 0)))// for usage of root page      PTS 1119428 FF 2002-DEC-17
+					    if ( !NptrsPage.bd910LockSubRootBeforeReleasingFrame() )
                         {	
                             /* no chaining will be needed anymore ==> release subroot */
                             NptrsSubRoot.bd910ReleasePage();
@@ -637,7 +628,7 @@ bd95RollforwardObj_NewObj (tgg00_TransContext       &Trans,
                             bKeyedObjectWithoutKey = true;  
                         } else {                            
                             SAPDBMem_IRawAllocator &allocator = *(reinterpret_cast<SAPDBMem_IRawAllocator*>(Trans.trAllocator_gg00));
-                            Data_PageAccessManager  dummyPam      (Trans, Data_UndoFile, Data_PageNo());
+                            Data_PageAccessManager  dummyPam      (Trans, Data_UndoFile, Data_PageRecoveryMode(), Data_PageNo());
                             Log_UndoFile::Iterator  dummyIterator (dummyPam, allocator);
 
                             const bool bKeyAlreadyRemoved = true; // new PTS 1128894 FF 31-MAR-2004
