@@ -35,6 +35,7 @@
 
 #include "heo01.h"
 #include "cn/buildinfo/buildinfo.h"
+#include "SAPDB/SAPDBCommon/SAPDB_string.h"
 
 #ifdef WIN32
 #include <WINDOWS.h>
@@ -1326,7 +1327,12 @@ get_buildinfo_using_res (int fd) {
 				ascii[i] = (char)(((uint16_t *)untrans)[i]);
 		}
 
-		if (ascii == 0 || memcmp (ascii, "APO", 3) != 0) {
+		if (ascii == 0) {
+				free (raw);
+				return (buildinfo);
+		}
+		
+		if (memcmp (ascii, "APO", 3) != 0) {
 			/*
 			 * apo old style,
 			 * no buildnumber
@@ -1964,6 +1970,8 @@ split_build (char *line, build_t *my) {
 			my->correction -= 'A';
 		} else if (my->correction >= '0' && my->correction <= '9') {
 			my->correction -= '0';
+		} else if (my->correction == ' ' || my->correction == '\t') {
+			my->correction = 0;
 		}
 		my->build = strtol (build + strlen (sz_build), &next, 10);
 		break;
@@ -2191,7 +2199,7 @@ static char dllext[] = ".so";
 #endif
 
 static int
-isa_pathname (char *filename) {
+isa_pathname (const char *filename) {
 	if (filename == 0)
 		return 0;
 
@@ -2227,7 +2235,7 @@ isa_pathname (char *filename) {
 }
 
 static char *
-get_filename (char *logical) {
+get_filename (const char *logical) {
 	tsp01_RteError err[1];
 	char *buff;
 
@@ -2236,7 +2244,8 @@ get_filename (char *logical) {
 		return 0;
 
 	if (isa_pathname (logical) != 0) {
-		strcpy (buff, logical);
+		SAPDB_strncpy (buff, logical, PATH_LEN_EO01-1);
+        buff[PATH_LEN_EO01-1]='\0';
 		return (buff);
 	}
 	
@@ -2524,7 +2533,7 @@ read_sapdir (void *pdir, int *pidx) {
  * top level function
  */
 bool
-cn46BuildInfo (char *logical, char *buff, int size) {
+cn46BuildInfo (const char *logical, char *buff, int size) {
 	char *filename;
 	int fd;
 	buildinfo_t *buildinfo, *curr;
@@ -2569,7 +2578,7 @@ cn46BuildInfo (char *logical, char *buff, int size) {
 }
 
 bool
-cn46BuildInfoFull (char *logical, char *buff, int size) {
+cn46BuildInfoFull (const char *logical, char *buff, int size) {
 	int fd;
 	char *filename;
 	buildinfo_t *buildinfo, *curr;
@@ -2736,9 +2745,8 @@ cn46BuildInfoList (char *buff, int size) {
 			logical = (char *) realloc (logical, strlen (szlibdbpinstall) + 1);
 			strcpy (logical, szlibdbpinstall);
 		} else {
-			if (strcmp (logical, szlibdbpinstall) == 0) {
+			if (strcmp (logical, szlibdbpinstall) == 0)
 				dp = 0; idx = 0;
-			}
 
 			free (logical);
 			logical = read_sapdir (&dp, &idx);
@@ -2747,9 +2755,8 @@ cn46BuildInfoList (char *buff, int size) {
 		}
 
 		filename = get_filename (logical);
-		if (filename == 0) {
+		if (filename == 0)
 			continue;
-		}
 
 		fd = open (filename, O_RDONLY | O_BINARY);
 		free (filename);
