@@ -27,8 +27,8 @@
 #ifndef BUILDINFO_H
 #define BUILDINFO_H BUILDINFO_H
 
-bool cn46BuildInfo (char *, char *, int);
-bool cn46BuildInfoFull (char *, char *, int);
+bool cn46BuildInfo (const char *, char *, int);
+bool cn46BuildInfoFull (const char *, char *, int);
 bool cn46BuildInfoList (char *, int);
 
 #endif
