@@ -62,6 +62,7 @@
 
 #include "gcn00_1.h"
 #include "DBM/Srv/DBMSrv_Common.hpp"
+#include "SAPDBCommon/SAPDB_string.h"
 #include "ToolsCommon/Tools_Arguments.hpp"
 
 /***** Memory Debug BEGIN *****/
@@ -118,8 +119,8 @@
 class cn50DBMUser;          //forward declaration to avoid inclusion of  vcn50.h
 class DBMSrvBHist_History;  //forward declaration to avoid inclusion of  DBMSrvBHist_History.hpp
 class DBMSrvBHist_EBIDList; //forward declaration to avoid inclusion of  DBMSrvBHist_EBIDList.hpp
-class DBMSrv_HSSNodes;      //forward declaration to avoid inclusion of  DBMSrv_HSSNodes.hpp
-class DBMSrv_KernelSession; //forward declaration to avoid inclusion of  DBMSrv_KernelSession.hpp
+class DBMSrvHSS_Nodes;      //forward declaration to avoid include
+class DBMSrvKnl_Session;    //forward declaration to avoid include
 
 class VControlDataT {
 public:
@@ -160,12 +161,12 @@ public:
     void                 * pUtilSession;
     void                 * pSQLUserSession;
     void                 * pServiceSession;
-    DBMSrv_KernelSession * pKernelSession;
+    DBMSrvKnl_Session    * pKernelSession;
     void                 * pBackupSession;
     void                 * pBackupStageSession;
     DBMSrvBHist_History  * pHistorySession;
     DBMSrvBHist_EBIDList * pEBIDList;
-    DBMSrv_HSSNodes      * pHSSNodes;
+    DBMSrvHSS_Nodes      * pHSSNodes;
     void                 * pEventSession;
     void                 * pCommandObj;
     void                 * pParamHistory;
