@@ -222,6 +222,16 @@ inline const char   * cn00_1_ErrTxt
 #define  ERR_NEEDADMI_CN00_1            N_CN00_1( 125), TERR_CN00_1, DBM_CN00_1, "ERR_NEEDADMI: The operating system user is not a member of the database administrators group."
 #define  ERR_SERVICEDBNOTFOUND_CN00_1   N_CN00_1( 126), TERR_CN00_1, DBM_CN00_1, "ERR_SERVICEDBNOTFOUND: The service database could not be found."
 
+#define  ERR_INSERVERPROCESSSTARTER_CN00_1 N_CN00_1( 146),TERR_CN00_1,DBM_CN00_1,"ERR_INSERVERPROCESSSTARTER: Could not start server process (used command: \"%s\")."
+#define  ERR_NOSERVERPROCESSRUNNING_CN00_1 N_CN00_1( 147),TERR_CN00_1,DBM_CN00_1,"ERR_NOSERVERPROCESSRUNNING: Could not start server process (used command: \"%s\")."
+/*
+                !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+                !!                                          !!
+                !! do not define any new error numbers here !!
+                !!    use class DBMSrvMsg_Error instead!    !!
+                !!       !!157 is the last one here!!       !!
+                !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+*/
 
 #define  ERR_UNKNOWN_CN00_1             N_CN00_1(1000), TERR_CN00_1, DBM_CN00_1, "ERR_UNKNOWN: unknown error"
 
