@@ -195,54 +195,6 @@ tcn00_Error cn40ExecXPU
         int           * replyLen,
         int             replyLenMax);
 
-/*!
-  -----------------------------------------------------------------------------
-  function:     cn40ExecLoad
-  -----------------------------------------------------------------------------
-
-  description:  executes loadscript (python)
-
-  arguments:    vcontrol    [INOUT]  - DBMServer session data
-                command     [IN]     - DBMServer command
-                replyData   [OUT]    - reply buffer
-                replyLen    [OUT]    - lenght of reply
-                replyLenMax [IN]     - max. length of reply buffer
-
-  return value: error code specified in gcn00c.h
- 
-  -----------------------------------------------------------------------------
- */
-tcn00_Error cn40ExecLoad
-      ( VControlDataT * vcontrol,
-        CommandT      * command,
-        char          * replyData,
-        int           * replyLen,
-        int             replyLenMax);
-
-/*!
-  -----------------------------------------------------------------------------
-  function:     cn40ExecXLoad
-  -----------------------------------------------------------------------------
-
-  description:  executes loadscript (xload)
-
-  arguments:    vcontrol    [INOUT]  - DBMServer session data
-                command     [IN]     - DBMServer command
-                replyData   [OUT]    - reply buffer
-                replyLen    [OUT]    - lenght of reply
-                replyLenMax [IN]     - max. length of reply buffer
-
-  return value: error code specified in gcn00c.h
- 
-  -----------------------------------------------------------------------------
- */
-tcn00_Error cn40ExecXLoad
-      ( VControlDataT * vcontrol,
-        CommandT      * command,
-        char          * replyData,
-        int           * replyLen,
-        int             replyLenMax);
-
 /*!
   -----------------------------------------------------------------------------
   function:     cn40LoadSysTab
@@ -291,6 +243,54 @@ tcn00_Error cn40LoadR3Tab
         int           * replyLen,
         int             replyLenMax);
 
+/*!
+  -----------------------------------------------------------------------------
+  function:     cn40LoadLcApps
+  -----------------------------------------------------------------------------
+
+  description:  loads the com objects
+
+  arguments:    vcontrol    [INOUT]  - DBMServer session data
+                command     [IN]     - DBMServer command
+                replyData   [OUT]    - reply buffer
+                replyLen    [OUT]    - lenght of reply
+                replyLenMax [IN]     - max. length of reply buffer
+
+  return value: error code specified in gcn00c.h
+ 
+  -----------------------------------------------------------------------------
+ */
+tcn00_Error cn40LoadLcApps
+      ( VControlDataT * vcontrol,
+        CommandT      * command,
+        char          * replyData,
+        int           * replyLen,
+        int             replyLenMax);
+
+/*!
+  -----------------------------------------------------------------------------
+  function:     cn40LoadTutorial
+  -----------------------------------------------------------------------------
+
+  description:  loads the com objects
+
+  arguments:    vcontrol    [INOUT]  - DBMServer session data
+                command     [IN]     - DBMServer command
+                replyData   [OUT]    - reply buffer
+                replyLen    [OUT]    - lenght of reply
+                replyLenMax [IN]     - max. length of reply buffer
+
+  return value: error code specified in gcn00c.h
+ 
+  -----------------------------------------------------------------------------
+ */
+tcn00_Error cn40LoadTutorial
+      ( VControlDataT * vcontrol,
+        CommandT      * command,
+        char          * replyData,
+        int           * replyLen,
+        int             replyLenMax);
+
 /*!
   -----------------------------------------------------------------------------
   function:     cn40ExecXKernprot
@@ -387,30 +387,6 @@ tcn00_Error cn40DBMSetPath
         int           * replyLen,
         int             replyLenMax);
 
-/*!
-  -----------------------------------------------------------------------------
-  function:     cn40DBMVersion
-  -----------------------------------------------------------------------------
-
-  description:  shows extended DBMServer version info
-
-  arguments:    vcontrol    [INOUT]  - DBMServer session data
-                command     [IN]     - DBMServer command
-                replyData   [OUT]    - reply buffer
-                replyLen    [OUT]    - lenght of reply
-                replyLenMax [IN]     - max. length of reply buffer
-
-  return value: error code specified in gcn00c.h
- 
-  -----------------------------------------------------------------------------
- */
-tcn00_Error cn40DBMVersion
-      ( VControlDataT * vcontrol,
-        CommandT      * command,
-        char          * replyData,
-        int           * replyLen,
-        int             replyLenMax);
-
 /*!
   -----------------------------------------------------------------------------
   function:     cn40DBRegister
