@@ -84,6 +84,7 @@
 #define FGET_LCINITHIS_CN42        (*(tsp00_Pathc *) "LCINITHIS"  )
 #define FGET_APOCOMTRC_CN42        (*(tsp00_Pathc *) "APOCOMTRC"  )
 #define FGET_LCTRC_CN42            (*(tsp00_Pathc *) "LCTRC"      )
+#define FGET_ATP_CN42              (*(tsp00_Pathc *) "ATP"        )
 #define FGET_LCBIN_CN42            (*(tsp00_Pathc *) "LCBIN"      )
 #define FGET_INSTPRT_CN42          (*(tsp00_Pathc *) "INSTPRT"    )
 #define FGET_USER_CN42             (*(tsp00_Pathc *) "DBMUPC"     )
@@ -107,6 +108,8 @@
 #define FGET_EVTDISPCDEF_CN42      (*(tsp00_Pathc *) "EVTDISPCDEF")
 #define FGET_EVTDISPPRT_CN42       (*(tsp00_Pathc *) "EVTDISPPRT" )
 #define FGET_EXTDBPRT_CN42         (*(tsp00_Pathc *) "EXTDBPRT"   )
+#define FGET_KNLCOR_CN42           (*(tsp00_Pathc *) "KNLCOR"     )
+#define FGET_KNLBAD_CN42           (*(tsp00_Pathc *) "KNLBAD"     )
 
 #define FGET_TEST_CN42             (*(tsp00_Pathc *) "TESTFILE"   )
 
