@@ -391,7 +391,7 @@ bool cn90GetToken
     ( const char      * szBuffer,
       char            * szToken,
       const long         nRequest,
-      const long         nLength = -1);
+      const long         nLength  );
 
 /*!
   -----------------------------------------------------------------------------
