@@ -17,9 +17,9 @@
 
   modifications:
   -------------------------------------------------------------------------
-  01   who:
-       what:
-       when:
+  001   who: RA
+       what: Peculiar in .NET c++ runtime will be ignored
+       when: 20.12.2004
   -------------------------------------------------------------------------
 \if EMIT_LICENCE
 
@@ -65,8 +65,8 @@
 #define TRUE 1
 
 /* GLOBAL SECTION */
-static int SdbinfoVersionMajor = 1;
-static int SdbinfoVersionMinor = 0;
+char *SdbinfoVersionMajor = "1";
+char *SdbinfoVersionMinor = "001";
 
 char *FullPrtFileName = NULL;     /*<IndepDataPath>/wrk/SDBINFO.PRT*/
 char *IndepDataPath = NULL;
@@ -157,7 +157,10 @@ void check_Usage ( int argc, char *argv[], int *all)
         case 'v':
         case 'V':
           if (strlen(argv[i]) == 2)
-            printf("SDBINFO version %i.%i\n", SdbinfoVersionMajor,SdbinfoVersionMinor);
+          {
+            printf("SDBINFO version %s.%s\n", SdbinfoVersionMajor,SdbinfoVersionMinor);
+            printf("(001) Peculiar in .NET c++ runtime will be ignored\n");
+          }
           else
           {
             print_WrongUsage();
@@ -686,6 +689,8 @@ int get_Dll_info(char *dll_name)
   #ifdef WIN32 
   int flag=-1;                    /*function success flag*/
  
+	int length,i,sonderflag;
+
   FILE *WriteHandle;
 
   char *FullDllFileName = NULL;   /*<WinSystemPath>/dll_name*/
@@ -840,10 +845,39 @@ int get_Dll_info(char *dll_name)
                 wCodePage,wLangID);
       VerQueryValue(lpData,acFormat,(void**)&pcData,&wInfoLen);
  
-      /*add product name to version output (variable)*/
-      printf(" %s runtime\n", pcData);
-      fprintf(WriteHandle," %s runtime\n", pcData);
+
+      /* check first on displayable ascii values (R) */
+      length=strlen(pcData);
+      /*printf("Laenge pcData: %i\n",length);*/
+      sonderflag=-1;
+
+      /* output signs (product name) ignoring peculiar to stdout and prt file */
+      printf(" ");
+			fprintf(WriteHandle," ");
+      for (i=0; i<=length; i++)
+      {
+        if ((pcData[i] > 127) || (pcData[i]<0))
+        { 
+          /*no output*/
+          /*
+          printf("Sonderzeichen gefunden an Stelle: %i ",i);
+          printf("  %c ", pcData[i]);
+          printf("  %i\n", (int) pcData[i]);
+          */
+          sonderflag=0;               
+        }
+        else
+        {
+          /*printf("Stelle: %i",i);*/
+          printf("%c", pcData[i]);
+					fprintf(WriteHandle,"%c",pcData[i]);
+          /*printf("  %i\n", (int) pcData[i]);*/
+        }
+      }
+      printf("runtime\n");       
+			fprintf(WriteHandle,"runtime\n");
  
+
       flag=0;
     }
     else
@@ -1343,7 +1377,7 @@ size_t cnt_configured;                /*counter: processors configured*/
 
   /* print tool information to PRT-file*/
   fprintf( WriteHandle, "%s\n", "# S D B I N F O" );
-  fprintf( WriteHandle, "%s%i.%i\n","# Version: ",SdbinfoVersionMajor,SdbinfoVersionMinor);
+  fprintf( WriteHandle, "%s%s.%s\n","# Version: ",SdbinfoVersionMajor,SdbinfoVersionMinor);
   fprintf( WriteHandle, "%s\n", "# Determination of liveCache operating system parameters" );
   fprintf( WriteHandle, "%s\n", "" );
   fclose(WriteHandle);
