@@ -55,6 +55,7 @@
 
 #include "gsp09.h"
 #include "heo02.h"
+#include "heo13.h"
 #include "heo01.h"
 #include "heo06.h"
 #include "geo43.h"
@@ -686,11 +687,11 @@ static char * cn12GetXUser(tcn12Options * options, const char * szKey, bool bWit
     strncpy(recUser.xu_key, szKey, strlen(szKey));
   } /* end if */
 
-  sqlgetuser (&recUser, szError, &bOk);
+  sqlgetuser (&recUser, NULL, szError, &bOk);
 
   if (bOk) {
 
-    memcpy (&pwCrypt[0], &(recUser.xu_password[0]), sizeof(pwCrypt));
+    SAPDB_memcpy (&pwCrypt[0], &(recUser.xu_password[0]), sizeof(pwCrypt));
     cn90CalculateUncrypt(pwCrypt, szPassword, false);
     cn90StringPtoC(szUser, recUser.xu_user, sizeof(recUser.xu_user));
     sprintf(szDBMUsr, "%s,%s", szUser, szPassword);
@@ -699,7 +700,7 @@ static char * cn12GetXUser(tcn12Options * options, const char * szKey, bool bWit
       // terminate program
       cn12FatalError(options, TXT_ERR_MEMORY);
     } // end if
-    strcpy(szReturn, szDBMUsr); 
+    SAPDB_strcpy(szReturn, szDBMUsr); 
     if (bWithDB && strlen(options->dbname) == 0) {
       g_szDbName.p2c(recUser.xu_serverdb);
       options->dbname = g_szDbName.asCharp();
@@ -766,7 +767,7 @@ char * cn12FindDBRoot
            relInfo.version.no [0] == nVMajor &&
            relInfo.version.no [1] == nVMinor &&
            relInfo.version.no [2] == nVCorr     ) {
-        strcpy(locDBRoot, relInfo.dbroot);
+        SAPDB_strcpy(locDBRoot, relInfo.dbroot);
       } /* end if */
     } /* end while */
     sqlxclose_installation_enum (hEnum);
@@ -897,13 +898,13 @@ static void cn12CallRightDBMCli
 #endif
 
       struct stat sBuf;
-      strcpy(szTmp, szPath);
+      SAPDB_strcpy(szTmp, szPath);
       cn90StripQuotes(szTmp);
       if (stat(szTmp, &sBuf) == 0) {
 
 
         // check version of target dbmcli  
-        strcpy(szTmp, szPath);
+        SAPDB_strcpy(szTmp, szPath);
         strcat(szTmp, " -R ");
         strcat(szTmp, pTueddelchen);
         strcat(szTmp, options->dbroot);
@@ -1377,14 +1378,14 @@ static tsp00_Int4 cn12ReadHereArg
     ++linepos;
   }
   stopArgLen = linepos - 1;
-  memcpy (stopArg, data + 1, stopArgLen);
+  SAPDB_memcpy (stopArg, data + 1, stopArgLen);
 
   /* move command */
   while (isspace (data [linepos])) {
     ++linepos;
   }
   cmdLen = linelen - linepos;
-  memcpy (data, data + linepos, cmdLen);
+  SAPDB_memmove (data, data + linepos, cmdLen);
   data [cmdLen] = '\n';
   cmdPos = data + cmdLen + 1;
   packetEnd = data + options->nMaxData;
@@ -1672,7 +1673,7 @@ static void cn12Next
   } // end if
 
   do {
-    strcpy(data, CMD_NEXT);
+    SAPDB_strcpy(data, CMD_NEXT);
     memset(data + strlen(data), 0, options->nMaxData - strlen(data));
     rc = cn12CServerCommand(options, data, &replyData, true, false, bRelease);
     
@@ -1814,7 +1815,7 @@ static int cn12Logon
 
   g_pOptions = &options;
   
-  strcpy(c_component, C_COMPONENT);
+  SAPDB_strcpy(c_component, C_COMPONENT);
   sqlinit (c_component, NULL);
 
   cn12ParseOptions (&options, argc, argv);
@@ -1848,7 +1849,7 @@ static int cn12Logon
 
   // show version
   if (options.showServerVersion) {
-      strcpy (data, CMD_VERSION);
+      SAPDB_strcpy (data, CMD_VERSION);
       rc = cn12CServerCommand (&options, data, &replyData, false, false, bRelease);
       nExit = cn12CheckAnswer(replyData);
   } /* end if */
