@@ -54,6 +54,7 @@
 
 #include "gsp09.h"
 #include "heo02.h"
+#include "heo13.h"
 #include "heo01.h"
 #include "heo06.h"
 #include "hsp100.h"
@@ -677,11 +678,11 @@ static char * cn12GetXUser(tcn12Options * options, const char * szKey, bool bWit
     strncpy(recUser.xu_key, szKey, strlen(szKey));
   } /* end if */
 
-  sqlgetuser (&recUser, szError, &bOk);
+  sqlgetuser (&recUser, NULL, szError, &bOk);
 
   if (bOk) {
 
-    memcpy (&pwCrypt[0], &(recUser.xu_password[0]), sizeof(pwCrypt));
+    SAPDB_memcpy (&pwCrypt[0], &(recUser.xu_password[0]), sizeof(pwCrypt));
     cn90CalculateUncrypt(pwCrypt, szPassword, false);
     cn90StringPtoC(szUser, recUser.xu_user, sizeof(recUser.xu_user));
     sprintf(szDBMUsr, "%s,%s", szUser, szPassword);
@@ -690,7 +691,7 @@ static char * cn12GetXUser(tcn12Options * options, const char * szKey, bool bWit
       // terminate program
       cn12FatalError(options, TXT_ERR_MEMORY);
     } // end if
-    strcpy(szReturn, szDBMUsr); 
+    SAPDB_strcpy(szReturn, szDBMUsr); 
     if (bWithDB && strlen(options->dbname) == 0) {
       g_szDbName.p2c(recUser.xu_serverdb);
       options->dbname = g_szDbName.asCharp();
@@ -757,7 +758,7 @@ char * cn12FindDBRoot
            relInfo.version.no [0] == nVMajor &&
            relInfo.version.no [1] == nVMinor &&
            relInfo.version.no [2] == nVCorr     ) {
-        strcpy(locDBRoot, relInfo.dbroot);
+        SAPDB_strcpy(locDBRoot, relInfo.dbroot);
       } /* end if */
     } /* end while */
     sqlxclose_installation_enum (hEnum);
@@ -973,14 +974,14 @@ static tsp00_Int4 cn12ReadHereArg
     ++linepos;
   }
   stopArgLen = linepos - 1;
-  memcpy (stopArg, data + 1, stopArgLen);
+  SAPDB_memcpy (stopArg, data + 1, stopArgLen);
 
   /* move command */
   while (isspace (data [linepos])) {
     ++linepos;
   }
   cmdLen = linelen - linepos;
-  memcpy (data, data + linepos, cmdLen);
+  SAPDB_memmove (data, data + linepos, cmdLen);
   data [cmdLen] = '\n';
   cmdPos = data + cmdLen + 1;
   packetEnd = data + options->nMaxData;
@@ -1193,7 +1194,7 @@ static void cn12Next
   } // end if
 
   do {
-    strcpy(data, CMD_NEXT);
+    SAPDB_strcpy(data, CMD_NEXT);
     memset(data + strlen(data), 0, options->nMaxData - strlen(data));
     rc = cn12CServerCommand(options, data, &replyData, true, false, bRelease);
     
@@ -1332,7 +1333,7 @@ static int cn12Logon
 
   g_pOptions = &options;
   
-  strcpy(c_component, C_COMPONENT);
+  SAPDB_strcpy(c_component, C_COMPONENT);
   sqlinit (c_component, NULL);
 
   cn12ParseOptions (&options, argc, argv);
@@ -1359,7 +1360,7 @@ static int cn12Logon
   signal(SIGBREAK, cn12Aborthandler);
 #endif
 
-  strcpy(data, "db_online ");
+  SAPDB_strcpy(data, "db_online ");
   rc = cn12CServerCommand (&options, data, &replyData, false, true, bRelease);
   nExit = cn12CheckAnswer(replyData);
 
