@@ -57,7 +57,10 @@
 #include <string.h>
 #include <malloc.h>
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 #include "heo03.h"
+#include "heo13.h"
 #include "vsp004.h"
 #include "gsp09.h"
 #include "hsp77.h"
@@ -70,6 +73,7 @@
   missing extern prototypes
   -----------------------------------------------------------------------------
 */
+/*
 externC void sqlxucloseuser (
     tsp00_ErrText         errtext,
     tsp00_Bool*             ok);
@@ -89,7 +93,7 @@ externC void sqlgetuser (
     tsp00_Bool*             ok);
 
 externC void sqlclearuser ();
-
+*/
 
 /*
   -----------------------------------------------------------------------------
@@ -741,7 +745,7 @@ static tsp00_Int4 cn14_senduser (
             _pTmpDataBuffer[0] = '\x01';
 
             /* Copy parts of command to message buffer */
-            memcpy(_pPtrToRealData, &((char*)pInData)[nInLen - _nRemainderLen], _nDataLen);
+            SAPDB_memcpy(_pPtrToRealData, &((char*)pInData)[nInLen - _nRemainderLen], _nDataLen);
 
             /* move to the last white space in string and adapt length */
             
@@ -767,7 +771,7 @@ static tsp00_Int4 cn14_senduser (
             _pTmpDataBuffer[0] = 0;
 
             /* Copy command to message buffer */
-            memcpy(_pPtrToRealData, &((char*)pInData)[nInLen - _nRemainderLen], _nRemainderLen);
+            SAPDB_memcpy(_pPtrToRealData, &((char*)pInData)[nInLen - _nRemainderLen], _nRemainderLen);
 
             /* Reset reply buffer length that might have been manipulated during while loop */
             if (NULL != nOutLen)
@@ -832,7 +836,7 @@ static tsp00_Int4 cn14_senduser (
         {
             /* Coming here from the client the db name IS empty */
             size_t slen = _tcslen(dbroot);
-            memcpy(_szInstRoot, dbroot, slen);
+            SAPDB_memcpy(_szInstRoot, dbroot, slen);
             _szInstRoot[slen] = 0;
         }
 
@@ -870,7 +874,7 @@ static tsp00_Int4 cn14_senduser (
                     const char *pszMyReply = NULL;
                     cn14analyzeRpmAnswer(*sessionOut, &lErrCode, &pszMyReply, &lDataLen,
                                                       &lSqlCode, NULL,     &lSqlErrLen);
-                    memcpy(pszData, pszMyReply, lDataLen);
+                    SAPDB_memcpy(pszData, pszMyReply, lDataLen);
                     pszData[lDataLen] = 0;
                 }
             }
@@ -1197,7 +1201,7 @@ tsp00_Int4 cn14replyRead (
       if (*len <= 0) {
           nRc = DBMAPI_NOMOREDATA_CN14;
       } else {
-        memcpy (data, session->packetOutData + session->packetPos, *len);
+        SAPDB_memcpy (data, session->packetOutData + session->packetPos, *len);
         session->packetPos = session->packetPos + *len;
       } /* end if */
     } /* end if */
@@ -1309,7 +1313,7 @@ static tsp00_Int4 cn14_dbmLogon (
     _tcsncpy((_TCHAR *) pwClear, pPassword, _tcslen(pPassword));
     s02applencrypt(pwClear, pwCrypt);
 
-    memcpy(&cryptTemp[0], &pwCrypt[0], sizeof(cryptTemp));
+    SAPDB_memcpy(&cryptTemp[0], &pwCrypt[0], sizeof(cryptTemp));
     _stprintf ( pData , _T("%s %s,%08x%08x%08x%08x%08x%08x"), szCommand,
                                                               (_TCHAR *) locUser,
                                                                cryptTemp[0],
@@ -1390,7 +1394,7 @@ static tsp00_Int4 cn14_cmdWrite (
     if (len > (session->packetSize - session->packetPos)) {
        nRc = DBMAPI_TOSMALL_CN14;
     } else {
-      memcpy (session->packetInData + session->packetPos, data, len);
+      SAPDB_memcpy (session->packetInData + session->packetPos, data, len);
       session->packetPos = session->packetPos + len;
     } /* end if */
   } /* end if */
@@ -1488,7 +1492,7 @@ static void cn14_errtextToC (
   int i;
 
   memset(errtextC, _T('\0'), sizeof(tsp00_ErrTextc));
-  memcpy(errtextC, errtextP, sizeof(tsp00_ErrText));
+  SAPDB_memcpy(errtextC, errtextP, sizeof(tsp00_ErrText));
 
   for (i = sizeof (tsp00_ErrText) - 1; i >= 0; --i) {
     if (errtextC [i] == _T(' ')) {
@@ -1589,11 +1593,11 @@ static tsp00_Int4 cn14_senduser (
     _tcsncpy(pUserkey, szServerNode, _tcslen(szServerNode));
 
     /* get user */
-    sqlgetuser (&recUser, szErrP, &bOk);
+    sqlgetuser (&recUser, NULL, szErrP, &bOk);
     memset(recUser.xu_userUCS2, 0, sizeof(recUser.xu_userUCS2));
 
     if (bOk) {
-      memcpy(cryptTemp, &recUser.xu_password, sizeof(cryptTemp));
+      SAPDB_memcpy(cryptTemp, &recUser.xu_password, sizeof(cryptTemp));
       nIndex = sizeof(recUser.xu_user) - 1;
       while (nIndex >= 0 && recUser.xu_user[nIndex] == _T(' ')) {
         recUser.xu_user[nIndex] = _T('\0');
@@ -1695,11 +1699,11 @@ tsp00_Int4 cn14saveUserByKey (
     pUserkey = (char *) &recUser.xu_key[0];
     _tcsncpy(pUserkey, szKey, _tcslen(szKey));
 
-    sqlxuopenuser (szError, &bOk);
+    sqlxuopenuser (NULL, szError, &bOk);
     if (bOk) {
 
       /* get user */
-      sqlgetuser (&recUser, szError, &bOk);
+      sqlgetuser (&recUser, NULL, szError, &bOk);
       memset(recUser.xu_userUCS2, 0, sizeof(recUser.xu_userUCS2));
       bOk = true;
 
@@ -1816,12 +1820,12 @@ tsp00_Int4 cn14saveUserByKey (
       } /* end if */
 
       if (bOk) {
-        sqlputuser (&recUser, szError, &bOk);
+        sqlputuser (&recUser, NULL, szError, &bOk);
       } /* end if */
       if (!bOk) {
         nReturn = DBMAPI_XUSER_CN14;
       } /* end if */
-      sqlxucloseuser (szError, &bOk);
+      sqlxucloseuser (NULL, szError, &bOk);
     } else {
       nReturn = DBMAPI_XUSER_CN14;
     } /* end if */
@@ -1920,7 +1924,7 @@ tsp00_Int4 cn14checkUserByKey (
     _tcsncpy(pUserkey, szKey, _tcslen(szKey));
 
     /* get user */
-    sqlgetuser (&recUserEnv, szError, &bOk);
+    sqlgetuser (&recUserEnv, NULL, szError, &bOk);
     memset(recUserEnv.xu_userUCS2, 0, sizeof(recUserEnv.xu_userUCS2));
 
     if (bOk) {
@@ -1999,12 +2003,12 @@ tsp00_Int4 cn14checkDefaultUser (
   _tcsncpy(pUserkey, XU_KEY_DEFAULT, _tcslen(XU_KEY_DEFAULT));
 
   /* open XUSer-File */
-  sqlxuopenuser (szError, &bOk);
+  sqlxuopenuser (NULL, szError, &bOk);
 
   if (bOk) {
 
     /* get user */
-    sqlgetuser (&recUserEnv, szError, &bOk);
+    sqlgetuser (&recUserEnv, NULL, szError, &bOk);
     memset(recUserEnv.xu_userUCS2, 0, sizeof(recUserEnv.xu_userUCS2));
 
     if (bOk) {
@@ -2022,14 +2026,14 @@ tsp00_Int4 cn14checkDefaultUser (
       _tcsncpy((_TCHAR *) szPassword, XU_SAP_PWD, _tcslen(XU_SAP_PWD));
       s02applencrypt(szPassword, recUserEnv.xu_password);
       /* save */
-      sqlputuser (&recUserEnv, szError, &bOk);
+      sqlputuser (&recUserEnv, NULL, szError, &bOk);
       if (bOk) {
         nReturn = DBMAPI_OK_CN14;
       } /* end if */
     } /* end if */
 
      /* close xuser-file */
-    sqlxucloseuser (szError, &bOk);
+    sqlxucloseuser (NULL, szError, &bOk);
 
   } /* end if */
 
@@ -2096,17 +2100,17 @@ tsp00_Int4 cn14deleteUserByKey (
     pUserkey = (char *) &recUser.xu_key[0];
     _tcsncpy(pUserkey, szKey, _tcslen(szKey));
 
-    sqlxuopenuser (szError, &bOk);
+    sqlxuopenuser (NULL, szError, &bOk);
 
     /* read */
     nLoadIndex = 0;
     while (bOk && (nLoadIndex < XU_MAX_USERS)) {
-      sqlindexuser(nLoadIndex + 1, &recUsers[nLoadIndex], szError, &bOk);
+      sqlindexuser(nLoadIndex + 1, &recUsers[nLoadIndex], NULL, szError, &bOk);
       nLoadIndex++;
     } /* end while */
 
     /* clear the file */
-    sqlclearuser();
+    sqlclearuser(NULL);
 
     /* save */
     nLoadIndex = (bOk) ? nLoadIndex : nLoadIndex - 1;
@@ -2114,14 +2118,14 @@ tsp00_Int4 cn14deleteUserByKey (
     bOk = true;
     while (bOk && (nSaveIndex < nLoadIndex)) {
       if (memcmp(&recUser.xu_key, &recUsers[nSaveIndex].xu_key, sizeof(recUser.xu_key)     ) != 0) {
-        sqlputuser (&recUsers[nSaveIndex], szError, &bOk);
+        sqlputuser (&recUsers[nSaveIndex], NULL, szError, &bOk);
       } else {
         bFound = true;
       } /* end if */
       nSaveIndex++;
     } /* end while */
 
-    sqlxucloseuser (szError, &bOk);
+    sqlxucloseuser (NULL, szError, &bOk);
 
     if (!bOk) {
       nReturn = DBMAPI_XUSER_CN14;
@@ -2154,18 +2158,18 @@ long cn14listUsers (
   long                nUsed = 0;
   _TCHAR            * pCurrent = pBuffer;
 
-  sqlxuopenuser (szError, &bOk);
+  sqlxuopenuser (NULL, szError, &bOk);
 
   /* read */
   nLoadIndex = 0;
   while (bOk && (nLoadIndex < XU_MAX_USERS)) {
-    sqlindexuser(nLoadIndex + 1, &recUsers[nLoadIndex], szError, &bOk);
+    sqlindexuser(nLoadIndex + 1, &recUsers[nLoadIndex], NULL, szError, &bOk);
     nLoadIndex++;
   } /* end while */
   nLoadIndex = (bOk) ? nLoadIndex : nLoadIndex - 1;
 
   /* clear the file */
-  sqlxucloseuser (szError, &bOk);
+  sqlxucloseuser (NULL, szError, &bOk);
 
   if (nLoadIndex > 0) {
     for (nIndex = 0; nIndex < nLoadIndex; ++nIndex) {
@@ -2216,7 +2220,7 @@ tsp00_Int4 cn14GetTPUser (
 
   szTmpName = tmpnam(szDummy);
   if (szTmpName == NULL) {
-    strcpy(szUserLine, "can not create temporary file");
+    SAPDB_strcpy(szUserLine, "can not create temporary file");
   } else {
 
 #ifdef _WIN32
@@ -2228,7 +2232,7 @@ tsp00_Int4 cn14GetTPUser (
       sprintf(szTmpFile, "%s%s.%s", szShortTmp,  szTmpName, "cst");
     } /* end if */
 #else
-    strcpy(szTmpFile, szTmpName);
+    SAPDB_strcpy(szTmpFile, szTmpName);
 #endif
 /*    if (hTrace != NULL) fprintf(hTrace, "tmpfile is %s (%s %d)\n", szTmpFile, __FILE__, __LINE__); */
 
@@ -2242,7 +2246,7 @@ tsp00_Int4 cn14GetTPUser (
       if (hFile != NULL) {
         while (fgets ( szLine, BUFFER_LEN - 1, hFile ) != NULL) {
 /*          if (hTrace != NULL) fprintf(hTrace, "szLine is %s (%s %d)\n", szLine, __FILE__, __LINE__); */
-          strcpy(szUserLine, szLine);
+          SAPDB_strcpy(szUserLine, szLine);
           if ((strncmp(szUserLine, "LC-INFO=", strlen("LC-INFO=")) == 0)) {
             bOk = true;
             break; /* while */
@@ -2253,10 +2257,10 @@ tsp00_Int4 cn14GetTPUser (
         remove(szTmpFile);
       } else {
 /*        if (hTrace != NULL) fprintf(hTrace, "errno is %d (%s %d)\n", errno, __FILE__, __LINE__); */
-        strcpy(szUserLine, "can not open temporary file");
+        SAPDB_strcpy(szUserLine, "can not open temporary file");
       } /* end if */
     } else {
-      strcpy(szUserLine, "can not execute tp");
+      SAPDB_strcpy(szUserLine, "can not execute tp");
     } /* end if */
      remove(szTmpFile);
   } /* end if */
@@ -2269,7 +2273,7 @@ tsp00_Int4 cn14GetTPUser (
       /* delete newline */
       pUser[_tcslen ( pUser ) - 1] = 0;
 
-      strcpy(user, pUser);
+      SAPDB_strcpy(user, pUser);
     } /* end if */
 
 /*    nRc = cn14_connectDBMUsr (servernode, dbname, dbroot, user, sessionOut, errtext ); */
