@@ -262,7 +262,7 @@ static tcn00_Error cn21_ReadNewHelpFileContents (
       /* count of lines */
       nLine++;
       /* get a copy for analysis */
-      strcpy(szTemp, szBuffer);
+      SAPDB_strcpy(szTemp, szBuffer);
       /* remove tabs */
       char * pTab = &szTemp[0];
       while (*pTab != '\0') {
@@ -512,10 +512,10 @@ static tcn00_Error cn21_GetSecondToken (
   tcn00_Error        nFuncStatus = OK_CN00;
   static char        szToken[XP_STRING_LENGTH_MXCN002 + 1];
 
-  if (cn90GetToken(szBuffer, NULL, 3)) {
+  if (cn90GetToken(szBuffer, NULL, 3, -1)) {
     nFuncStatus = ERR_XPSYNTAX_CN00;
     *pSyntaxError = XPERR_TOKEN_CN002;
-  } else if (cn90GetToken(szBuffer, szToken, 2)) {
+  } else if (cn90GetToken(szBuffer, szToken, 2, XP_STRING_LENGTH_MXCN002 + 1)) {
     if (strlen(szToken) == 0) {
       nFuncStatus = ERR_XPSYNTAX_CN00;
       *pSyntaxError = XPERR_INVVALUE_CN002;
@@ -704,7 +704,7 @@ static tcn00_Error cn21_ReadDefaultFor (
   bool               bValid       = false;
 
   // read token
-  if (!cn90GetToken(szBuffer, szInstance, 2) || !cn90GetToken(szBuffer, szValue, 3)) {
+  if (!cn90GetToken(szBuffer, szInstance, 2, XP_STRING_LENGTH_MXCN002 + 1) || !cn90GetToken(szBuffer, szValue, 3, XP_STRING_LENGTH_MXCN002 + 1)) {
     nFuncStatus  = ERR_XPSYNTAX_CN00;
     *pSyntaxError = XPERR_INVDEF_CN002;
   } // end if
@@ -833,7 +833,10 @@ static tcn00_Error cn21_ReadProperty (
 
         if (aValidTable[k].nClass != XPPropClassUnknown) {
           pParameter->xpProperties[aClassTable[i].nClass] = aValueTable[j].nValue;
-          strcpy(pParameter->szProperties[aClassTable[i].nClass], pValue);
+          SAPDB_memmove(
+              pParameter->szProperties[aClassTable[i].nClass],
+              pValue,
+              strlen(pValue)+1);
         } else {
           nFuncStatus = ERR_XPSYNTAX_CN00;
           *pSyntaxError = XPERR_INVPROP_CN002;
@@ -864,7 +867,7 @@ static void cn21_InitProperties (
   /* search class */
   for (i = 0; i < XPPropClassMAX; i++) {
     pParameter->xpProperties[i] = aClassTable[i].nDefault;
-    strcpy(pParameter->szProperties[i], "");
+    SAPDB_strcpy(pParameter->szProperties[i], "");
   } /* end for */
 
 } /* end cn21_InitProperties */
@@ -900,7 +903,7 @@ static tcn00_Error cn21_AddLine (
       pLine->szLine  = pLineText;
       pLine->xpLine  = xpLine;
       pLine->nNumber = nNumber;
-      strcpy(pLine->szLine, szBuffer);
+      SAPDB_strcpy(pLine->szLine, szBuffer);
 
       *pOut = pLine;
 
