@@ -143,8 +143,8 @@ void cn23PrintDependencies (
   tcn002_XParameter * pParameter = pFirst;
 
   while (pParameter != NULL) {
-    strcpy(szIndent, "");
-    strcpy(szDep, "");
+    SAPDB_strcpy(szIndent, "");
+    SAPDB_strcpy(szDep, "");
     cn23_PrintParamDep(pFirst, pParameter, szIndent, szDep);
     printf("\n");
     pParameter = pParameter->pNext;
@@ -354,7 +354,7 @@ static tcn002_XPError cn23_CalculateExpression (
   long                   bToken        = true;
 
   /* start at the second token */
-  bToken = cn90GetToken(szLine, szToken, nToken++);
+  bToken = cn90GetToken(szLine, szToken, nToken++, XP_STRING_LENGTH_MXCN002 + 1);
 
   while (bToken && (nSyntaxError == XPOK_CN002)) {
     /* reset operator */
@@ -418,7 +418,7 @@ static tcn002_XPError cn23_CalculateExpression (
     } /* end if */
 
     /* next token */
-    bToken = cn90GetToken(szLine, szToken, nToken++);
+    bToken = cn90GetToken(szLine, szToken, nToken++, XP_STRING_LENGTH_MXCN002 + 1);
   } /* end while */
 
   if (nSyntaxError == XPOK_CN002) {
@@ -427,7 +427,7 @@ static tcn002_XPError cn23_CalculateExpression (
     } else if (nStackPointer < 0) {
       nSyntaxError = XPERR_FEWOP_CN002;
     } else {
-      memcpy(pOut, &opStack[0], sizeof(tcn002_XPOperand));
+      SAPDB_memcpy(pOut, &opStack[0], sizeof(tcn002_XPOperand));
     } /* end if */
   } /* end if */
 
@@ -635,7 +635,7 @@ static tcn002_XPError cn23_GetParameterOperand (
     pEndParam = &szCurrent[strlen(szCurrent) - strlen(pEndDyn)];
     if ( (strncmp(szCurrent, pId, strlen(pId)) == 0) &&
          (strncmp(pEndDyn, pEndParam , strlen(pEndDyn))  == 0)    ) {
-      strcpy(pId, szCurrent);
+      SAPDB_memmove(pId, szCurrent, strlen(szCurrent)+1);
     } /* end if */
   } // end if
 
@@ -690,7 +690,7 @@ static tcn002_XPError cn23_GetParameterOperand (
         case RTEConf_Parameter::t_String:
           pOperand->nOpType = (pOperand->nOpType == XPOpTypeUnknown) ? XPOpTypeString : pOperand->nOpType;
           aTmpOp.nOpType = XPOpTypeString;
-          strcpy(aTmpOp.opValue.szValue, pParameter->szNewValue);
+          SAPDB_strcpy(aTmpOp.opValue.szValue, pParameter->szNewValue);
           break;
        case RTEConf_Parameter::t_CryptInfo:
           nSyntaxError = XPERR_CRYPTUSE_CN002;
@@ -736,7 +736,10 @@ static tcn002_XPError cn23_AssignValue (
       break;
     case RTEConf_Parameter::t_String:
       if (pOperand->nOpType == XPOpTypeString) {
-        strcpy(pParameter->szNewValue, pOperand->opValue.szValue);
+        SAPDB_memmove(
+            pParameter->szNewValue,
+            pOperand->opValue.szValue,
+            strlen(pOperand->opValue.szValue)+1);
       } else if (pOperand->nOpType == XPOpTypeArith) {
         sprintf(pParameter->szNewValue, "%ld", pOperand->opValue.nValue);
       } else {
@@ -808,7 +811,7 @@ static void cn23_PrintParamDep (
   char                   szCurrent[10000];
 
   printf("%s%s\n", szIndent, pCurrent->szParameterName.asCharp());
-  strcpy(szCurrent, "");
+  SAPDB_strcpy(szCurrent, "");
 
   while (pLine != NULL) {
     switch(pLine->xpLine) {
@@ -818,7 +821,7 @@ static void cn23_PrintParamDep (
       case XPLineCodeCALC:
         /* start at the second token */
         nToken = 2;
-        bToken = cn90GetToken(pLine->szLine, szToken, nToken++);
+        bToken = cn90GetToken(pLine->szLine, szToken, nToken++, XP_STRING_LENGTH_MXCN002 + 1);
 
         while (bToken) {
           /* reset operator */
@@ -868,7 +871,7 @@ static void cn23_PrintParamDep (
           } /* end if */
 
           /* next token */
-          bToken = cn90GetToken(pLine->szLine, szToken, nToken++);
+          bToken = cn90GetToken(pLine->szLine, szToken, nToken++, XP_STRING_LENGTH_MXCN002 + 1);
         } /* end while */
 
         break;
