@@ -86,7 +86,7 @@ static tcn002_XParameter * cn28_FindParamNew
         pParameter->xpProperties[XPPropClassDynamic] = XPPropValueNo;
         pParameter->xpProperties[XPPropClassIntern] = XPPropValueNo;
 
-       strcpy(pParameter->szParameterName, szNewName);
+       SAPDB_strcpy(pParameter->szParameterName, szNewName);
 
         /* list management (insert after dynamic) */
         pParameter->pNext = pDynHelp->pNext;
@@ -192,7 +192,7 @@ static tcn00_Error cn28_CopyParamString
     } else {
       pParameter = cn28_FindParamNew(pXParamData, szNewName);
       if (pParameter != NULL) {
-        strcpy((char *) pParameter->xpRecord.xpValue.xpValueString, (char *) szValue);
+        SAPDB_strcpy((char *) pParameter->xpRecord.xpValue.xpValueString, (char *) szValue);
         if (!oConfigFile.SetValue ((const SAPDB_UTF8 *) pParameter->szParameterName.asCharp(), (RTEConf_Parameter::String) pParameter->xpRecord.xpValue.xpValueString.asCharp(), oMsg)) {
           SAPDBErr_MessageList aDBM(DBMSrv_DBMError(RTE), 0);
           aDBM.AppendNewMessage(oMsg);
@@ -744,7 +744,7 @@ tcn00_Error cn28MigrateParamFile
       // don't migrate but success
       return nFuncReturn;
     } else {
-      strcpy(szVersion, (char *) xpRecord.xpValue.xpValueString);
+      SAPDB_strcpy(szVersion, (char *) xpRecord.xpValue.xpValueString);
     } // end if
   } // end if
 
