@@ -108,7 +108,7 @@ tcn002_XParameter * cn29SearchDynamic
   while ((pDynHelp != NULL) && (bFound == false)) {
     if (pDynHelp->xpProperties[XPPropClassDynamic] == XPPropValueYes) {
       /* special check for dynamics */
-      strcpy(szDynamic, pDynHelp->szParameterName);
+      SAPDB_strcpy(szDynamic, pDynHelp->szParameterName);
       pEndDyn = strstr(szDynamic, XP_KEY_DYNAMIC_CN002);
       pEndDyn[0] = CHAR_STRINGTERM_CN90;
       pEndDyn++;
@@ -181,9 +181,15 @@ tcn00_Error cn29CopyParameter
   tcn00_Error          nFuncStatus  = OK_CN00;
   long                 i            = 0;
 
-  strcpy(pDest-> szParameterName, pSource->szParameterName);
-  strcpy(pDest->szDefValue,       pSource->szDefValue);
-  memcpy(&(pDest->xpRecord), &(pSource->xpRecord), sizeof(tcn002_XPRecord));
+  SAPDB_memmove(
+      pDest-> szParameterName, 
+      pSource->szParameterName,
+      strlen(pSource->szParameterName)+1);
+  SAPDB_memmove(
+      pDest->szDefValue,
+      pSource->szDefValue,
+      strlen(pSource->szDefValue)+1);
+  SAPDB_memmove(&(pDest->xpRecord), &(pSource->xpRecord), sizeof(tcn002_XPRecord));
   pDest->bHelpExist     = pSource->bHelpExist;
   pDest->bParamExist    = pSource->bParamExist;
 
@@ -244,7 +250,7 @@ static tcn00_Error cn29_RequestedProperties
     } /* end for */
   } /* end for */
 
-  while (cn90GetToken(szLine, szProperty, nToken++) && (nFuncStatus == OK_CN00)) {
+  while (cn90GetToken(szLine, szProperty, nToken++, XP_PARAMLINE_LENGTH_MXCN002) && (nFuncStatus == OK_CN00)) {
     pClass = strtok(szProperty, "=");
     pValue = strtok(NULL, "=");
     pOneValue = strtok(pValue, ",");
@@ -290,7 +296,7 @@ void cn29PrintProperties
       ( char               * pBuffer,
         tcn002_XPPropClass   nClass,
         tcn002_XPPropValue   nValue,
-        tsp00_C70c           szValue)
+        tsp00_C512c          szValue)
 {
   struct {
     tcn002_XPPropValue   nValue;
@@ -378,7 +384,7 @@ static tcn00_Error cn29_CopyLines
         pNewLine->szLine  = pLineText;
         pNewLine->xpLine  = pSource->xpLine;
         pNewLine->nNumber = pSource->nNumber;
-        strcpy(pNewLine->szLine, pSource->szLine);
+        SAPDB_strcpy(pNewLine->szLine, pSource->szLine);
 
         if (*pDest == NULL) {
           *pDest = pNewLine;
@@ -447,17 +453,17 @@ tcn002_XPRecord * cn29StringToParamValue
       pRecord->xpValue.xpValueReal = atof ( szValue );
       break;
     case RTEConf_Parameter::t_String:
-      strcpy( pRecord->xpValue.xpValueString, szValue);
+      SAPDB_memmove( pRecord->xpValue.xpValueString, szValue, strlen(szValue)+1);
       break;
     case RTEConf_Parameter::t_CryptInfo:
     case xp_crypt_type  :
-      strcpy(szTemp, szValue);
+      SAPDB_strcpy(szTemp, szValue);
       if (strlen(szTemp) == sizeof(tsp00_CryptName) * 2) {
         cn90Uncrypt(szTemp, false);
       } /* end if */
       cn90StringCtoP(szName, szTemp, sizeof(tsp00_Name));
       s02encrypt(szName, cryptPwTemp);
-      memcpy(&(pRecord->xpValue.xpValueCrypt[0]), &(cryptPwTemp[0]), sizeof(pRecord->xpValue.xpValueCrypt));
+      SAPDB_memcpy(&(pRecord->xpValue.xpValueCrypt[0]), &(cryptPwTemp[0]), sizeof(pRecord->xpValue.xpValueCrypt));
       break;
   } /* end switch */
 
@@ -480,15 +486,18 @@ char * cn29ParamValueToString
       sprintf ( szValue , "%f" , (double) pRecord->xpValue.xpValueReal );
       break;
     case RTEConf_Parameter::t_String:
-      strcpy(szValue, pRecord->xpValue.xpValueString.asCharp());
+      SAPDB_memmove(
+          szValue,
+          pRecord->xpValue.xpValueString.asCharp(),
+          strlen(pRecord->xpValue.xpValueString.asCharp())+1);
       break;
     case RTEConf_Parameter::t_CryptInfo:
       {
       tsp00_Namec    szClearPwd;
       tsp00_CryptName  oTemp;
-      memcpy(&(oTemp[0]), &(pRecord->xpValue.xpValueCrypt[0]), sizeof(oTemp));
+      SAPDB_memcpy(&(oTemp[0]), &(pRecord->xpValue.xpValueCrypt[0]), sizeof(oTemp));
       cn90CalculateUncrypt(oTemp, szClearPwd, true);
-      strcpy(szValue, szClearPwd);
+      SAPDB_strcpy(szValue, szClearPwd);
       }
       break;
   } /* end switch */
@@ -909,10 +918,10 @@ tcn00_Error cn29CheckRange
 
       nFuncReturn = ERR_XPVALUESET_CN00;
       nToken = 1;
-      while (cn90GetToken(pParameter->szProperties[XPPropClassValueset], szToken, nToken)) {
+      while (cn90GetToken(pParameter->szProperties[XPPropClassValueset], szToken, nToken, XP_STRING_LENGTH_MXCN002)) {
         if (pParameter->xpRecord.xpType == RTEConf_Parameter::t_Integer ||
             pParameter->xpRecord.xpType == RTEConf_Parameter::t_Real       ) {
-          strcpy(szBuffer, szToken);
+          SAPDB_strcpy(szBuffer, szToken);
         } else {
           cn90AnalyzeString(szToken, szBuffer, XP_STRING_LENGTH_MXCN002);
         } // end if
@@ -1038,14 +1047,6 @@ tcn00_Error cn29SoftPutIntern
         if ((pParameter == NULL) || (!bMallocOK)) {
           nFuncReturn = ERR_MEM_CN00;
         } else {
-          /* add the parameter in the list of parameters in the XParam section */
-          if (pXParamData->pFirst == NULL) {
-            pXParamData->pFirst = pParameter;
-          } /* end if */
-          if (pXParamData->pLast != NULL) {
-            pXParamData->pLast->pNext = pParameter;
-          } /* end if */
-
           /* copy data from dynamic record */
           nFuncReturn = cn29CopyParameter(pParameter, pDynHelp);
           /* clear the dynamic/intern property */
@@ -1055,9 +1056,14 @@ tcn00_Error cn29SoftPutIntern
           cn90SaveStrcpy(pParameter->szParameterName, szParameter, sizeof(tcn002_XpValueName));
 
           /* list management */
-          pParameter->pNext = NULL;
-          pParameter->pPrev = pXParamData->pLast;
-          pXParamData->pLast = pParameter;
+          pParameter->pNext = pDynHelp->pNext;
+          pParameter->pPrev = pDynHelp;
+          if (pParameter->pNext != NULL) {
+            pParameter->pNext->pPrev = pParameter;
+          } else {
+            pXParamData->pLast = pParameter;
+          } // end if
+          pDynHelp->pNext   = pParameter;
         } /* end if */
       } /* end if */
     } /* end if */
@@ -1067,10 +1073,10 @@ tcn00_Error cn29SoftPutIntern
 
       if (pParameter->xpRecord.xpType == RTEConf_Parameter::t_Integer ||
           pParameter->xpRecord.xpType == RTEConf_Parameter::t_Real      ) {
-        strcpy(szValue, szValueIn);
+        SAPDB_strcpy(szValue, szValueIn);
       } else {
         if (!cn90AnalyzeString(szValueIn, szValue, XP_STRING_LENGTH_MXCN002)) {
-          strcpy(szValue, szValueIn);
+          SAPDB_strcpy(szValue, szValueIn);
         } // end if
       } // end if
 
@@ -1246,11 +1252,14 @@ static tcn00_Error cn29ReadParamFileContent (
 
         if (pParameter != NULL) {
           /* Copy Data */
-          memcpy(&pParameter->xpRecord, &xpRecord, sizeof(xpRecord));
-          strcpy(pParameter->szParameterName, (char *) sName);
+          SAPDB_memcpy(&pParameter->xpRecord, &xpRecord, sizeof(xpRecord));
+          SAPDB_strcpy(pParameter->szParameterName, (char *) sName);
 
           cn29ParamValueToString(pParameter->szNewValue, &pParameter->xpRecord);
-          strcpy(pParameter->szOldValue, pParameter->szNewValue);
+          SAPDB_memmove(
+              pParameter->szOldValue,
+              pParameter->szNewValue,
+              strlen(pParameter->szNewValue)+1);
 
           /* set flags */
           pParameter->bParamExist = true;
@@ -1284,10 +1293,13 @@ static tcn00_Error cn29ReadParamFileContent (
               (strcmp((char *) sName, PAN_MAXSYSDEVSPACES) != 0) &&
               !bReadOnly) {
             // parameter not known in cserv.pcf -> delete it
-            strcpy(aParameter.szParameterName, (char *)  sName);
-            memcpy(&aParameter.xpRecord, &xpRecord, sizeof(xpRecord));
+            SAPDB_strcpy(aParameter.szParameterName, (char *)  sName);
+            SAPDB_memcpy(&aParameter.xpRecord, &xpRecord, sizeof(xpRecord));
             cn29ParamValueToString(aParameter.szNewValue, &aParameter.xpRecord);
-            strcpy(aParameter.szOldValue, aParameter.szNewValue);
+            SAPDB_memmove(
+                aParameter.szOldValue,
+                aParameter.szNewValue,
+                strlen(aParameter.szNewValue)+1);
             oConfig.Delete(sName, oMsg);
             oConfig.Write(oMsg);
             cn29WriteParamHistory(szDbName, &aParameter, PAHI_DELETE_CN002);
@@ -1310,7 +1322,10 @@ static tcn00_Error cn29ReadParamFileContent (
         pParameter->bParamExist = true;
         pParameter->xpRecord.xpValue.xpValueInt = bChanged ? 1 : 0;
         cn29ParamValueToString(pParameter->szNewValue, &pParameter->xpRecord);
-        strcpy(pParameter->szOldValue, pParameter->szNewValue);
+        SAPDB_memmove(
+            pParameter->szOldValue,
+            pParameter->szNewValue,
+            strlen(pParameter->szNewValue)+1);
       } // end if
 
       pParameter = cn29SearchParameter(pXParamData->pFirst, PAN_PARAM_VERSION_TIME);
@@ -1325,7 +1340,10 @@ static tcn00_Error cn29ReadParamFileContent (
                 (int) aTime.Minute,
                 (int) aTime.Second);
         cn29ParamValueToString(pParameter->szNewValue, &pParameter->xpRecord);
-        strcpy(pParameter->szOldValue, pParameter->szNewValue);
+        SAPDB_memmove(
+            pParameter->szOldValue,
+            pParameter->szNewValue,
+            strlen(pParameter->szNewValue)+1);
       } // end if
 
       pParameter = cn29SearchParameter(pXParamData->pFirst, PAN_CRASH_SEM);
@@ -1333,7 +1351,10 @@ static tcn00_Error cn29ReadParamFileContent (
         pParameter->bParamExist = true;
         pParameter->xpRecord.xpValue.xpValueInt = bCrashed ? 1 : 0;
         cn29ParamValueToString(pParameter->szNewValue, &pParameter->xpRecord);
-        strcpy(pParameter->szOldValue, pParameter->szNewValue);
+        SAPDB_memmove(
+            pParameter->szOldValue,
+            pParameter->szNewValue,
+            strlen(pParameter->szNewValue)+1);
       } // end if
 
 
@@ -1552,7 +1573,7 @@ bool cn29_InvalidForInstance
 
   if ((_tcslen(szInstance) > 0) && (_tcslen(szPossible) > 0)) {
     bInvalid = true;
-    while (bInvalid && cn90GetToken(szPossible, szToken, nToken) ) {
+    while (bInvalid && cn90GetToken(szPossible, szToken, nToken, PARAMETER_MAXLEN_CN90) ) {
       cn90StripQuotes(szToken);
       if (_tcsicmp(szToken, szInstance) == 0) {
         bInvalid = false;
@@ -1595,9 +1616,15 @@ tcn00_Error cn29CheckInstance
         for (nIndex = 0; nIndex < XP_DEFAULTFOR_MXCN002; ++nIndex) {
           if (_tcsicmp(szInstance, pParameter->szDefaultFor[nIndex][XP_DEFAULTFORINSTANCE_CN002]) == 0) {
             if (!pParameter->bParamExist) {
-              strcpy(pParameter->szNewValue,   pParameter->szDefaultFor[nIndex][XP_DEFAULTFORVALUE_CN002]);
+              SAPDB_memmove(
+                  pParameter->szNewValue,
+                  pParameter->szDefaultFor[nIndex][XP_DEFAULTFORVALUE_CN002],
+                  strlen(pParameter->szDefaultFor[nIndex][XP_DEFAULTFORVALUE_CN002])+1);
             } // end if
-            strcpy(pParameter->szDefValue, pParameter->szDefaultFor[nIndex][XP_DEFAULTFORVALUE_CN002]);
+            SAPDB_memmove(
+                pParameter->szDefValue,
+                pParameter->szDefaultFor[nIndex][XP_DEFAULTFORVALUE_CN002],
+                strlen(pParameter->szDefaultFor[nIndex][XP_DEFAULTFORVALUE_CN002])+1);
             break;
           } // end if
         } // end for
