@@ -233,9 +233,9 @@ static tcn00_Error cn30_CheckMediaFile (
         *bExists = true;
 
         szPos = szDate;        
-        memcpy(szPos, rteFileInfo.sp5fi_date_modified, sizeof(rteFileInfo.sp5fi_date_modified));
+        SAPDB_memcpy(szPos, rteFileInfo.sp5fi_date_modified, sizeof(rteFileInfo.sp5fi_date_modified));
         szPos += sizeof(rteFileInfo.sp5fi_date_modified);
-        memcpy(szPos, rteFileInfo.sp5fi_time_modified, sizeof(rteFileInfo.sp5fi_time_modified));
+        SAPDB_memcpy(szPos, rteFileInfo.sp5fi_time_modified, sizeof(rteFileInfo.sp5fi_time_modified));
         szPos += sizeof(rteFileInfo.sp5fi_time_modified);
         *szPos = '\0';
 
@@ -416,18 +416,18 @@ void cn30_InitMedia (
 void cn30_InitMedium(
       tcn003_Medium   * pMedium)
 {
-  strcpy(pMedium->szName, ME_DEFAULT_NAME_CN003);
-  strcpy(pMedium->szLocation, ME_DEFAULT_LOCATION_CN003);
-  strcpy(pMedium->szType, ME_DEFAULT_TYPE_CN003);
-  strcpy(pMedium->szSaveType, ME_DEFAULT_SAVETYPE_CN003);
-  strcpy(pMedium->szSize, ME_DEFAULT_SIZE_CN003);
-  strcpy(pMedium->szBlocksize, ME_DEFAULT_BLCCKSIZE_CN003);
-  strcpy(pMedium->szOverwrite, ME_DEFAULT_OVERWRITE_CN003);
-  strcpy(pMedium->szAutoLoader, ME_DEFAULT_AUTOLOADER_CN003);
-  strcpy(pMedium->szOSCommand, ME_DEFAULT_OSCMD_CN003);
-  strcpy(pMedium->szCreated, ME_DEFAULT_DATE_CN003);
-  strcpy(pMedium->szModified, ME_DEFAULT_DATE_CN003);
-  strcpy(pMedium->szExtern, ME_DEFAULT_EXTERN_CN003);
+  SAPDB_strcpy(pMedium->szName, ME_DEFAULT_NAME_CN003);
+  SAPDB_strcpy(pMedium->szLocation, ME_DEFAULT_LOCATION_CN003);
+  SAPDB_strcpy(pMedium->szType, ME_DEFAULT_TYPE_CN003);
+  SAPDB_strcpy(pMedium->szSaveType, ME_DEFAULT_SAVETYPE_CN003);
+  SAPDB_strcpy(pMedium->szSize, ME_DEFAULT_SIZE_CN003);
+  SAPDB_strcpy(pMedium->szBlocksize, ME_DEFAULT_BLCCKSIZE_CN003);
+  SAPDB_strcpy(pMedium->szOverwrite, ME_DEFAULT_OVERWRITE_CN003);
+  SAPDB_strcpy(pMedium->szAutoLoader, ME_DEFAULT_AUTOLOADER_CN003);
+  SAPDB_strcpy(pMedium->szOSCommand, ME_DEFAULT_OSCMD_CN003);
+  SAPDB_strcpy(pMedium->szCreated, ME_DEFAULT_DATE_CN003);
+  SAPDB_strcpy(pMedium->szModified, ME_DEFAULT_DATE_CN003);
+  SAPDB_strcpy(pMedium->szExtern, ME_DEFAULT_EXTERN_CN003);
   pMedium->bOld = false;
   pMedium->pNext = NULL;
   pMedium->pPrev = NULL;
@@ -482,7 +482,7 @@ static tcn00_Error cn30_CheckMedium (
 
   if (nFuncStatus == OK_CN00) {
     if (pMedium->szType[0] == '\0') {
-      strcpy(pMedium->szType, ME_DEFAULT_TYPE_CN003);
+      SAPDB_strcpy(pMedium->szType, ME_DEFAULT_TYPE_CN003);
     } else {
       cn90StrUpperCopy(pMedium->szType, pMedium->szType, false);
       if ( (strcmp(pMedium->szType, ME_TYPE_FILE_CN003)  != 0) &&
@@ -545,7 +545,7 @@ static tcn00_Error cn30_CheckMedium (
 
   if (nFuncStatus == OK_CN00) {
     if (pMedium->szSaveType[0] == '\0') {
-      strcpy(pMedium->szSaveType, ME_DEFAULT_SAVETYPE_CN003);
+      SAPDB_strcpy(pMedium->szSaveType, ME_DEFAULT_SAVETYPE_CN003);
     } else {
       cn90StrUpperCopy(pMedium->szSaveType, pMedium->szSaveType, false);
       
@@ -565,7 +565,7 @@ static tcn00_Error cn30_CheckMedium (
 
   if (nFuncStatus == OK_CN00) {
     if (pMedium->szSize[0] == '\0') {
-      strcpy(pMedium->szSize, "0");
+      SAPDB_strcpy(pMedium->szSize, "0");
     } else {
       if (cn90AnalyzeNumber(pMedium->szSize, &lSize) == false) {
         nFuncStatus = ERR_MENOVALID_CN00;
@@ -578,14 +578,14 @@ static tcn00_Error cn30_CheckMedium (
 
   if (nFuncStatus == OK_CN00) {
     if (pMedium->szBlocksize[0] == '\0') {
-      strcpy(pMedium->szBlocksize, ME_DEFAULT_BLCCKSIZE_CN003);
+      SAPDB_strcpy(pMedium->szBlocksize, ME_DEFAULT_BLCCKSIZE_CN003);
     } else {
       if (cn90AnalyzeNumber(pMedium->szBlocksize, &lBlocksize) == false) {
         nFuncStatus = ERR_MENOVALID_CN00;
         cn90AnswerIError(replyData, replyLen, nFuncStatus);
       } else {
         if (lBlocksize == 0) {
-          strcpy(pMedium->szBlocksize, ME_DEFAULT_BLCCKSIZE_CN003);
+          SAPDB_strcpy(pMedium->szBlocksize, ME_DEFAULT_BLCCKSIZE_CN003);
         } 
       }
     }
@@ -595,7 +595,7 @@ static tcn00_Error cn30_CheckMedium (
 
   if (nFuncStatus == OK_CN00) {
     if (pMedium->szOverwrite[0] == '\0') {
-      strcpy(pMedium->szOverwrite, ME_PROP_NO_CN003);
+      SAPDB_strcpy(pMedium->szOverwrite, ME_PROP_NO_CN003);
     } else {
       cn90StrUpperCopy(pMedium->szOverwrite, pMedium->szOverwrite, false);
       if ( (strcmp(pMedium->szOverwrite, ME_PROP_YES_CN003) != 0) &&
@@ -610,7 +610,7 @@ static tcn00_Error cn30_CheckMedium (
 
   if (nFuncStatus == OK_CN00) {
     if (pMedium->szAutoLoader[0] == '\0') {
-      strcpy(pMedium->szAutoLoader, ME_DEFAULT_AUTOLOADER_CN003);
+      SAPDB_strcpy(pMedium->szAutoLoader, ME_DEFAULT_AUTOLOADER_CN003);
     } else {
       cn90StrUpperCopy(pMedium->szAutoLoader, pMedium->szAutoLoader, false);
       if ( (strcmp(pMedium->szAutoLoader, ME_PROP_YES_CN003) != 0) &&
@@ -627,15 +627,15 @@ static tcn00_Error cn30_CheckMedium (
     if (pMedium->szExtern[0] == '\0') {
       // check medianame
       if (cn90StrNicmp(pMedium->szName, ME_EXT_NSR_CN003, strlen(ME_EXT_NSR_CN003)) == 0) {
-        strcpy(pMedium->szExtern, ME_EXT_NSR_CN003);
+        SAPDB_strcpy(pMedium->szExtern, ME_EXT_NSR_CN003);
       } else if (cn90StrNicmp(pMedium->szName, ME_EXT_ADSM_CN003, strlen(ME_EXT_ADSM_CN003)) == 0) {
-        strcpy(pMedium->szExtern, ME_EXT_ADSM_CN003);
+        SAPDB_strcpy(pMedium->szExtern, ME_EXT_ADSM_CN003);
       } else if (cn90StrNicmp(pMedium->szName, ME_EXT_BACK_CN003, strlen(ME_EXT_BACK_CN003)) == 0) {
-        strcpy(pMedium->szExtern, ME_EXT_BACK_CN003);
+        SAPDB_strcpy(pMedium->szExtern, ME_EXT_BACK_CN003);
       } else if (cn90StrNicmp(pMedium->szName, ME_EXT_OMNI_CN003, strlen(ME_EXT_OMNI_CN003)) == 0) {
-        strcpy(pMedium->szExtern, ME_EXT_OMNI_CN003);
+        SAPDB_strcpy(pMedium->szExtern, ME_EXT_OMNI_CN003);
       } else {
-        strcpy(pMedium->szExtern, ME_DEFAULT_EXTERN_CN003);
+        SAPDB_strcpy(pMedium->szExtern, ME_DEFAULT_EXTERN_CN003);
       } // end if
     } else {
       cn90StrUpperCopy(pMedium->szExtern, pMedium->szExtern, false);
@@ -678,7 +678,7 @@ static tcn00_Error cn30_CheckAutosave (
         while (pMedium != NULL) {
           if (pMedium->szSaveType[0] != '\0') {
             if ( (strcmp(pMedium->szSaveType, ME_SAVETYPE_AUT_CN003) == 0) ) {
-              strcpy(pMedium->szSaveType, ME_SAVETYPE_LOG_CN003);
+              SAPDB_strcpy(pMedium->szSaveType, ME_SAVETYPE_LOG_CN003);
             }
           }
           pMedium = pMedium->pNext;
@@ -709,7 +709,10 @@ static tcn00_Error cn30_CheckMediaGroup (
 
   pMedium = pMediaGroup->pFirst;
   while (pMedium != NULL) {
-    strcpy(pMedium->szSaveType, pMediumCheck->szSaveType);
+    SAPDB_memmove(
+        pMedium->szSaveType,
+        pMediumCheck->szSaveType,
+        strlen(pMediumCheck->szSaveType)+1);
     pMedium = pMedium->pNext;
   }
 
@@ -755,7 +758,7 @@ static tcn00_Error cn30_ReadMediumProperties (
       sqlfreadc (hMediaFile, szBuffer, ME_LINE_LENGTH_MXCN003, &nRead, &rteFileError);
       if (rteFileError.sp5fe_result == vf_ok) {
 
-        strcpy(szLine, szBuffer);
+        SAPDB_strcpy(szLine, szBuffer);
         cn90Strip(szLine);
         nLen = (long)strlen(szLine);
 
@@ -777,31 +780,31 @@ static tcn00_Error cn30_ReadMediumProperties (
               szProperty[nEqualPos] = '\0';
               cn90Strip(szProperty);
 
-              strcpy(szValue, szEqual + 1);
+              SAPDB_strcpy(szValue, szEqual + 1);
               cn90Strip(szValue);
               
               if ( strcmp(szProperty, ME_KEY_LOCATION_CN003) == 0 ) {
-                strcpy(pMedium->szLocation, szValue);
+                SAPDB_strcpy(pMedium->szLocation, szValue);
               } else if ( strcmp(szProperty, ME_KEY_TYPE_CN003) == 0 ) {
-                strcpy(pMedium->szType, szValue);
+                SAPDB_strcpy(pMedium->szType, szValue);
               } else if ( strcmp(szProperty, ME_KEY_SAVETYPE_CN003) == 0 ) {
-                strcpy(pMedium->szSaveType, szValue);
+                SAPDB_strcpy(pMedium->szSaveType, szValue);
               } else if ( strcmp(szProperty, ME_KEY_SIZE_CN003) == 0 ) {
-                strcpy(pMedium->szSize, szValue);
+                SAPDB_strcpy(pMedium->szSize, szValue);
               } else if ( strcmp(szProperty, ME_KEY_BLOCKSIZE_CN003) == 0 ) {
-                strcpy(pMedium->szBlocksize, szValue);
+                SAPDB_strcpy(pMedium->szBlocksize, szValue);
               } else if ( strcmp(szProperty, ME_KEY_OVERWRITE_CN003) == 0 ) {
-                strcpy(pMedium->szOverwrite, szValue);
+                SAPDB_strcpy(pMedium->szOverwrite, szValue);
               } else if ( strcmp(szProperty, ME_KEY_AUTOLOADER_CN003) == 0 ) {
-                strcpy(pMedium->szAutoLoader, szValue);
+                SAPDB_strcpy(pMedium->szAutoLoader, szValue);
               } else if ( strcmp(szProperty, ME_KEY_OSCOMMAND_CN003) == 0 ) {
-                strcpy(pMedium->szOSCommand, szValue);
+                SAPDB_strcpy(pMedium->szOSCommand, szValue);
               } else if ( strcmp(szProperty, ME_KEY_EXTERN_CN003) == 0 ) {
-                strcpy(pMedium->szExtern, szValue);
+                SAPDB_strcpy(pMedium->szExtern, szValue);
               } else if ( strcmp(szProperty, ME_KEY_CREATED_CN003) == 0 ) {
-                strcpy(pMedium->szCreated, szValue);
+                SAPDB_strcpy(pMedium->szCreated, szValue);
               } else if ( strcmp(szProperty, ME_KEY_MODIFIED_CN003) == 0 ) {
-                strcpy(pMedium->szModified, szValue);
+                SAPDB_strcpy(pMedium->szModified, szValue);
               }
             }
           }
@@ -854,7 +857,7 @@ static tcn00_Error cn30_ReadNextMedium (
     sqlfreadc(hMediaFile, szBuffer, ME_LINE_LENGTH_MXCN003, &nRead, &rteFileError);
     if (rteFileError.sp5fe_result == vf_ok) {
 
-      strcpy(szLine, szBuffer);
+      SAPDB_strcpy(szLine, szBuffer);
       cn90Strip(szLine);
       nLen = (long)strlen(szLine);
 
@@ -930,7 +933,7 @@ static tcn00_Error cn30_ReadParallelIDFromOldMediaFile (
       sqlfreadc(hMediaFile, szBuffer, ME_LINE_LENGTH_MXCN003, &nRead, &rteFileError);
       if (rteFileError.sp5fe_result == vf_ok) {
 
-        strcpy(szLine, szBuffer);
+        SAPDB_strcpy(szLine, szBuffer);
         cn90Strip(szLine);
         nLen = (long)strlen(szLine);
 
@@ -944,7 +947,7 @@ static tcn00_Error cn30_ReadParallelIDFromOldMediaFile (
               szProperty[szPos1 - szLine] = '\0';
               cn90Strip(szProperty);
 
-              strcpy(szValue, szPos1 + 1);
+              SAPDB_strcpy(szValue, szPos1 + 1);
               cn90Strip(szValue);
               
               if ( strcmp(szProperty, ME_KEY_MEDIAID_CN003) == 0 ) {
@@ -953,11 +956,11 @@ static tcn00_Error cn30_ReadParallelIDFromOldMediaFile (
                   strncpy(szName, szValue, szPos1 - szValue);
                   szName[szPos1 - szValue] = '\0';
 
-                  strcpy(szLocation, szPos1 + 1);
+                  SAPDB_strcpy(szLocation, szPos1 + 1);
                   cn90Strip(szLocation);
 
                   if (strncmp(szLocation, ME_KEY_PARALLEL_CN003, strlen(ME_KEY_PARALLEL_CN003)) == 0) {
-                    strcpy(szParallelID, szName);
+                    SAPDB_strcpy(szParallelID, szName);
                     bFound = true;
                   }
                 }
@@ -1027,7 +1030,7 @@ static tcn00_Error cn30_ReadNextOldMedium (
     sqlfreadc(hMediaFile, szBuffer, ME_LINE_LENGTH_MXCN003, &nRead, &rteFileError);
     if (rteFileError.sp5fe_result == vf_ok) {
 
-      strcpy(szLine, szBuffer);
+      SAPDB_strcpy(szLine, szBuffer);
       cn90Strip(szLine);
       nLen = (long)strlen(szLine);
 
@@ -1042,7 +1045,7 @@ static tcn00_Error cn30_ReadNextOldMedium (
             szProperty[szPos1 - szLine] = '\0';
             cn90Strip(szProperty);
 
-            strcpy(szValue, szPos1 + 1);
+            SAPDB_strcpy(szValue, szPos1 + 1);
             cn90Strip(szValue);
               
             if ( strcmp(szProperty, ME_KEY_MEDIAID_CN003) == 0 ) {
@@ -1051,7 +1054,7 @@ static tcn00_Error cn30_ReadNextOldMedium (
                 strncpy(pMedium->szName, szValue, szPos1 - szValue);
                 pMedium->szName[szPos1 - szValue] = '\0';
 
-                strcpy(pMedium->szLocation, szPos1 + 1);
+                SAPDB_strcpy(pMedium->szLocation, szPos1 + 1);
                 cn90Strip(pMedium->szLocation);
 
                 if (strncmp(pMedium->szLocation, ME_KEY_PARALLEL_CN003, strlen(ME_KEY_PARALLEL_CN003)) == 0) {
@@ -1070,10 +1073,10 @@ static tcn00_Error cn30_ReadNextOldMedium (
                 strncpy(szValue, szPos1, szPos2 - szPos1);
                 szValue[szPos2 - szPos1] = '\0';
                 if (strcmp(szValue, ME_OLD_PROP_YES_CN003) == 0 ) {
-                  strcpy(pMedium->szOverwrite, ME_PROP_YES_CN003);
+                  SAPDB_strcpy(pMedium->szOverwrite, ME_PROP_YES_CN003);
                 } else {
                   if (strcmp(szValue, ME_OLD_PROP_VERSION_CN003) == 0 ) {
-                    strcpy(pMedium->szOverwrite, ME_PROP_VERSION_CN003);
+                    SAPDB_strcpy(pMedium->szOverwrite, ME_PROP_VERSION_CN003);
                   }
                 }
               }
@@ -1084,25 +1087,25 @@ static tcn00_Error cn30_ReadNextOldMedium (
                 strncpy(szValue, szPos1, szPos2 - szPos1);
                 szValue[szPos2 - szPos1] = '\0';
                 if (strcmp(szValue, ME_OLD_TYPE_TAPE_CN003) == 0 ) {
-                  strcpy(pMedium->szType, ME_TYPE_TAPE_CN003);
+                  SAPDB_strcpy(pMedium->szType, ME_TYPE_TAPE_CN003);
 
                 } else if (strcmp(szValue, ME_OLD_TYPE_FILE_CN003) == 0 ) {
-                  strcpy(pMedium->szType, ME_TYPE_FILE_CN003);
+                  SAPDB_strcpy(pMedium->szType, ME_TYPE_FILE_CN003);
 
                 } else if (strcmp(szValue, ME_OLD_TYPE_NORE_CN003) == 0 ) {
-                  strcpy(pMedium->szType, ME_TYPE_NORE_CN003);
+                  SAPDB_strcpy(pMedium->szType, ME_TYPE_NORE_CN003);
 
                 } else if (strcmp(szValue, ME_OLD_TYPE_PIPE_CN003) == 0 ) {
-                  strcpy(pMedium->szType, ME_TYPE_PIPE_CN003);
+                  SAPDB_strcpy(pMedium->szType, ME_TYPE_PIPE_CN003);
 
                 } else if (strcmp(szValue, ME_OLD_TYPE_AUTO_CN003) == 0 ) {
-                  strcpy(pMedium->szAutoLoader, ME_PROP_YES_CN003);
+                  SAPDB_strcpy(pMedium->szAutoLoader, ME_PROP_YES_CN003);
 
                 } else if (strcmp(szValue, ME_OLD_TYPE_AUTL_CN003) == 0 ) {
-                  strcpy(pMedium->szAutoLoader, ME_PROP_YES_CN003);
+                  SAPDB_strcpy(pMedium->szAutoLoader, ME_PROP_YES_CN003);
 
                 } else if (strcmp(szValue, ME_OLD_TYPE_UNKN_CN003) == 0 ) {
-                  strcpy(pMedium->szType, ME_TYPE_UNKN_CN003);
+                  SAPDB_strcpy(pMedium->szType, ME_TYPE_UNKN_CN003);
                 } 
               }
               szPos1 = szPos2 + 1;
@@ -1111,7 +1114,7 @@ static tcn00_Error cn30_ReadNextOldMedium (
               if (szPos2) {
                 strncpy(szValue, szPos1, szPos2 - szPos1);
                 szValue[szPos2 - szPos1] = '\0';
-                strcpy(pMedium->szSize, szValue);
+                SAPDB_strcpy(pMedium->szSize, szValue);
               }
               szPos1 = szPos2 + 1;
               /* Usage */
@@ -1129,10 +1132,10 @@ static tcn00_Error cn30_ReadNextOldMedium (
                 strncpy(szValue, szPos1, szPos2 - szPos1);
                 szValue[szPos2 - szPos1] = '\0';
                 if (strcmp(szValue, ME_OLD_PROP_YES_CN003) == 0 ) {
-                  strcpy(szValue, szParallelID);
+                  SAPDB_strcpy(szValue, szParallelID);
                   strcat(szValue, BA_KEY_BAKSLASH_CN003);
                   strcat(szValue, pMedium->szName);
-                  strcpy(pMedium->szName, szValue);
+                  SAPDB_strcpy(pMedium->szName, szValue);
                 }
               }
               szPos1 = szPos2 + 1;
@@ -1140,7 +1143,7 @@ static tcn00_Error cn30_ReadNextOldMedium (
               szPos2 = strchr(szPos1, ME_KEY_PROPSEP_CN003);
               szPos1 = szPos2 + 1;
               /* OS Command */
-              strcpy(pMedium->szOSCommand, szPos1);
+              SAPDB_strcpy(pMedium->szOSCommand, szPos1);
               
               pMedium->bOld = true;
 
@@ -1388,9 +1391,15 @@ void cn30_SplitName (
   if (pSep) {
     strncpy(szGroupName, szFullName, pSep - szFullName);
     szGroupName[pSep - szFullName] = '\0';
-    strcpy(szMediumName, pSep + 1);
+    SAPDB_memmove(
+        szMediumName,
+        pSep + 1,
+        strlen(pSep + 1)+1);
   } else {
-    strcpy(szMediumName, szFullName);
+    SAPDB_memmove(
+        szMediumName,
+        szFullName,
+        strlen(szFullName)+1);
   }
 } /* end cn30_SplitName */
 
@@ -1429,7 +1438,7 @@ static tcn00_Error cn30_AddMedium (
     nFuncStatus = cn30_NewMediaGroup(replyData, replyLen, &pMediaGroup);
 
     if (nFuncStatus == OK_CN00) {
-      strcpy(pMediaGroup->szName,szGroupName);
+      SAPDB_strcpy(pMediaGroup->szName,szGroupName);
 
       /* add the mediagroup to the list of media */
       if (pMedia->pFirstGroup == NULL) {
@@ -1530,7 +1539,7 @@ static tcn00_Error cn30_ReadMedia (
       nFuncStatus = cn30_ReadParallelIDFromOldMediaFile(replyData, replyLen, hMediaFile, szParallelID);
       if (nFuncStatus == OK_CN00) {
         if (szParallelID[0] == '\0') {
-          strcpy(szParallelID, ME_KEY_PARALLEL_CN003);
+          SAPDB_strcpy(szParallelID, ME_KEY_PARALLEL_CN003);
         }
       }
     }
@@ -1592,7 +1601,7 @@ static tcn00_Error cn30_WriteMedium (
   /**/
 
   /* [<name>] */
-  strcpy(szLine, "[");
+  SAPDB_strcpy(szLine, "[");
   strcat(szLine, pMedium->szName);
   strcat(szLine, "]");
   strcat(szLine, "\0");
@@ -1600,7 +1609,7 @@ static tcn00_Error cn30_WriteMedium (
 
   /* LOCATION */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_LOCATION_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_LOCATION_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szLocation);
     strcat(szLine, "\0");
@@ -1608,7 +1617,7 @@ static tcn00_Error cn30_WriteMedium (
   }
   /* TYPE */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_TYPE_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_TYPE_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szType);
     strcat(szLine, "\0");
@@ -1616,7 +1625,7 @@ static tcn00_Error cn30_WriteMedium (
   }
   /* SAVETYPE */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_SAVETYPE_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_SAVETYPE_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szSaveType);
     strcat(szLine, "\0");
@@ -1624,7 +1633,7 @@ static tcn00_Error cn30_WriteMedium (
   }
   /* SIZE */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_SIZE_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_SIZE_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szSize);
     strcat(szLine, "\0");
@@ -1632,7 +1641,7 @@ static tcn00_Error cn30_WriteMedium (
   }
   /* BLOCKSIZE */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_BLOCKSIZE_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_BLOCKSIZE_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szBlocksize);
     strcat(szLine, "\0");
@@ -1640,7 +1649,7 @@ static tcn00_Error cn30_WriteMedium (
   }
   /* OVERWRITE */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_OVERWRITE_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_OVERWRITE_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szOverwrite);
     strcat(szLine, "\0");
@@ -1648,7 +1657,7 @@ static tcn00_Error cn30_WriteMedium (
   }
   /* AUTOLOADER */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_AUTOLOADER_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_AUTOLOADER_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szAutoLoader);
     strcat(szLine, "\0");
@@ -1656,7 +1665,7 @@ static tcn00_Error cn30_WriteMedium (
   }
   /* OSCOMMAND */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_OSCOMMAND_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_OSCOMMAND_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szOSCommand);
     strcat(szLine, "\0");
@@ -1664,7 +1673,7 @@ static tcn00_Error cn30_WriteMedium (
   }
   /* EXTERN */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_EXTERN_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_EXTERN_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szExtern);
     strcat(szLine, "\0");
@@ -1672,7 +1681,7 @@ static tcn00_Error cn30_WriteMedium (
   }
   /* CREATED */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_CREATED_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_CREATED_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szCreated);
     strcat(szLine, "\0");
@@ -1680,14 +1689,14 @@ static tcn00_Error cn30_WriteMedium (
   }
   /* MODIFIED */
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, ME_KEY_MODIFIED_CN003);
+    SAPDB_strcpy(szLine, ME_KEY_MODIFIED_CN003);
     strcat(szLine, "=");
     strcat(szLine, pMedium->szModified);
     strcat(szLine, "\0");
     sqlfwritec(hMediaFile, szLine, -1, &rteFileError);
   }
   if (rteFileError.sp5fe_result == vf_ok) {
-    strcpy(szLine, "\0");
+    SAPDB_strcpy(szLine, "\0");
     sqlfwritec(hMediaFile, szLine, -1, &rteFileError);
   }
 
@@ -1917,7 +1926,7 @@ tcn00_Error cn30BackupMediaGet (
   cn30_InitMedia(&tMedia);
 
   if (nFuncStatus == OK_CN00) {
-    if (cn90GetToken(command->args, szFullName, 1)) {
+    if (cn90GetToken(command->args, szFullName, 1, ME_NAME_LENGTH_MXCN003)) {
       cn30_RemoveDoublequotes(szFullName);
     } else {
       nFuncStatus = ERR_PARAM_CN00;
@@ -1956,9 +1965,9 @@ tcn00_Error cn30BackupMediaGet (
               (bFound == false)    ) {
 
         if (pMedium->bOld) {
-          strcpy(szOld ,"OLD");
+          SAPDB_strcpy(szOld ,"OLD");
         } else {
-          strcpy(szOld,"");
+          SAPDB_strcpy(szOld,"");
         }
 
         if(cn90Stricmp(szFullName,pMedium->szName) == 0) {
@@ -2079,9 +2088,9 @@ tcn00_Error cn30BackupMediaGetAll (
       while ((nFuncStatus == OK_CN00) && (pMedium != NULL)) {
 
         if (pMedium->bOld) {
-          strcpy(szOld ,"OLD");
+          SAPDB_strcpy(szOld ,"OLD");
         } else {
-          strcpy(szOld,"");
+          SAPDB_strcpy(szOld,"");
         }
       
         if ((ME_NAME_LENGTH_MXCN003       +
@@ -2207,61 +2216,7 @@ tcn00_Error cn30BackupMediaMigrate (
 
   return nFuncStatus;
 } /* end cn30BackupMediaMigrate */
-
-/* PUBLIC CONSTANT LongHelp_BackupMediaPut_cn30
- * ------------------------------------------------------------------
- * purpose: information used by the explain function for
- *          cn30BackupMediaPut
- *
- */
-
-const char * LongHelp_BackupMediaPut_cn30=
-    "@command medium_put You create a new backup medium or you change a backup medium that already exists."
-    "@preconditions You have the DBM operator authorization Backup."
-    "@syntax medium_put <name> <location> <type> <backup_type> <size> <block_size> <overwrite> <autoloader> <os_command>"
-    "@param <name> "
-        "The backup medium specified in <name> is created or updated. The name of the medium may comprise a group name "
-        "and a member name. These are separated by a slash (<name> ::= [<group_name>/]<member_name>). "
-        "Groups of parallel backup media are used for parallel backups and restores. To create a group of "
-        "parallel backup media, execute the command medium_put for each group member using the same <group_name>."
-    "@param <location> "
-        "The name of the backup device or file with which it can be addressed through operating system functions. "
-        "If a log backup into a file is created a running number is added to the <location>."
-    "@param <type> "
-        "Type of the backup medium. Possible values are:"
-        "@value AUTO Auto The media with the type AUTO has implicitly the type FILE (see below) and is the default medium "
-            "for the command autolog_on. "
-            "There can only be one backup medium of this type. If there is already a backup medium of type AUTO, and you create a backup "
-            "medium of type AUTO again, the system changes the media type of the existing backup medium to FILE."
-        "@value TAPE Tape The specified device is used as non rewinding tape."
-        "@value FILE File The backup will be written to a file. "
-        "@value PIPE Pipe The backup will be written to a pipe. On Microsoft Windows, the name of the pipe must "
-            "have the following form: \\\\.\\pipe\\<pipe_name>. There are no special naming conventions on UNIX."
-    "@param <backup_type> "
-        "Backup type, for which the backup medium is to be used. Possible values are:"
-        "@value DATA  \"complete data backup\" A complete data backup."
-        "@value PAGES \"differential data backup\" A differential data backup containing the differences from the last complete data backup"
-        "@value LOG   \"log backup\" A backup of log information from the log volume."
-    "@param <size> "
-        "Maximum number of pages that can be written to the backup medium. This is "
-        "necessary to change a tape correctly, for example. If the backup media is "
-        "of sufficient size, a zero can be specified."
-    "@param <block_size> "
-        "Number of pages that are transferred when accessing the medium once (default: 8)."
-    "@param <overwrite> "
-        "This option is only relevant for the medium type FILE and describes the behavior if the file is "
-        "already present. Permitted values are:"
-        "@value NO \"\" If the file already exists, the backup will fail."
-        "@value YES \"\" If the file already exists, it will be overwritten with the new backup."
-    "@param <autoloader> "
-        "Specifies whether the tape device changes the tapes automatically. This option has an effect "
-        "only on Microsoft Windows. Possible values are:"
-        "@value YES \"\" The devices changes the tapes automatically."
-        "@value NO  \"\" The devices does not change the tapes automatically."
-    "@param <os_command> "
-        "Operating system command to be executed before backing up to a succeeding medium or restoring from a succeeding medium."
-    "@reply OK";
-  
+ 
 
 /* PUBLIC FUNCTION cn30BackupMediaPut
  * ------------------------------------------------------------------
@@ -2300,7 +2255,7 @@ tcn00_Error cn30BackupMediaPut (
   nFuncStatus = cn30_NewMedium(replyData, replyLen, &pMediumPut);
 
   if (nFuncStatus == OK_CN00) {
-    if (cn90GetToken(command->args, pMediumPut->szName, 1)) {
+    if (cn90GetToken(command->args, pMediumPut->szName, 1, ME_NAME_LENGTH_MXCN003 + 1)) {
       cn30_RemoveDoublequotes(pMediumPut->szName);
     } else {
       nFuncStatus = ERR_PARAM_CN00;
@@ -2308,7 +2263,7 @@ tcn00_Error cn30BackupMediaPut (
     }
   }
   if (nFuncStatus == OK_CN00) {
-    if (cn90GetToken(command->args, pMediumPut->szLocation, 2)) {
+    if (cn90GetToken(command->args, pMediumPut->szLocation, 2, ME_LOCATION_LENGTH_MXCN003 + 1)) {
       cn30_RemoveDoublequotes(pMediumPut->szLocation);
     } else {
       nFuncStatus = ERR_PARAM_CN00;
@@ -2317,16 +2272,16 @@ tcn00_Error cn30BackupMediaPut (
   }  
 
   if (nFuncStatus == OK_CN00) {
-    cn90GetToken(command->args, pMediumPut->szType, 3);
-    cn90GetToken(command->args, pMediumPut->szSaveType, 4);
-    cn90GetToken(command->args, pMediumPut->szSize, 5);
-    cn90GetToken(command->args, pMediumPut->szBlocksize, 6);
-    cn90GetToken(command->args, pMediumPut->szOverwrite, 7);
-    cn90GetToken(command->args, pMediumPut->szAutoLoader, 8);
-    if (cn90GetToken(command->args, pMediumPut->szOSCommand, 9)) {
+    cn90GetToken(command->args, pMediumPut->szType, 3, ME_TYPE_LENGTH_MXCN003 + 1);
+    cn90GetToken(command->args, pMediumPut->szSaveType, 4, ME_SAVETYPE_LENGTH_MXCN003 + 1);
+    cn90GetToken(command->args, pMediumPut->szSize, 5, ME_SIZE_LENGTH_MXCN003 + 1);
+    cn90GetToken(command->args, pMediumPut->szBlocksize, 6, ME_BLOCKSIZE_LENGTH_MXCN003 + 1);
+    cn90GetToken(command->args, pMediumPut->szOverwrite, 7, ME_OVERWRITE_LENGTH_MXCN003 + 1);
+    cn90GetToken(command->args, pMediumPut->szAutoLoader, 8, ME_AUTOLOADER_LENGTH_MXCN003 + 1);
+    if (cn90GetToken(command->args, pMediumPut->szOSCommand, 9, ME_OSCMD_LENGTH_MXCN003 + 1)) {
       cn30_RemoveDoublequotes(pMediumPut->szOSCommand);
     }
-    cn90GetToken(command->args, pMediumPut->szExtern, 10);
+    cn90GetToken(command->args, pMediumPut->szExtern, 10, ME_EXTERNAL_LENGTH_MXCN003 + 1);
   }
 
   /**/
@@ -2397,7 +2352,7 @@ tcn00_Error cn30BackupMediaPut (
           nFuncStatus = cn30_NewMediaGroup(replyData, replyLen, &pMediaGroup);
 
           if (nFuncStatus == OK_CN00) {
-            strcpy(pMediaGroup->szName,szGroupName);
+            SAPDB_strcpy(pMediaGroup->szName,szGroupName);
 
             /* add the mediagroup to the list of media */
             if (tMedia.pFirstGroup == NULL) {
@@ -2425,7 +2380,7 @@ tcn00_Error cn30BackupMediaPut (
 
       cn30_GetDateString(pMediumPut->szModified);
 
-      strcpy(pMediumPut->szCreated, pMedium->szCreated);
+      SAPDB_strcpy(pMediumPut->szCreated, pMedium->szCreated);
  
       if (pMedium->pPrev != NULL) {
         pMedium->pPrev->pNext = pMediumPut;
@@ -2519,7 +2474,7 @@ tcn00_Error cn30BackupMediaDelete (
   cn30_InitMedia(&tMedia);
 
   if (nFuncStatus == OK_CN00) {
-    if (cn90GetToken(command->args, szFullName, 1)) {
+    if (cn90GetToken(command->args, szFullName, 1, ME_NAME_LENGTH_MXCN003)) {
       cn30_RemoveDoublequotes(szFullName);
     } else {
       nFuncStatus = ERR_PARAM_CN00;
