@@ -45,6 +45,8 @@
 #include "hcn353.h"
 #include "hcn36.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -737,7 +739,7 @@ int tcn35_BackupToolConnector::MoveFileToProtocolFile(const char *FileDesignatio
     
     if(cn36_StrAlloc(FileDescription, strlen(FileDesignation)+strlen(" of ")+strlen(GiveToolName())))
     {
-        strcpy(FileDescription, FileDesignation);
+        SAPDB_strcpy(FileDescription, FileDesignation);
         strcat(FileDescription, " of ");
         strcat(FileDescription, GiveToolName());
     }
