@@ -44,6 +44,8 @@
 #include"hcn36.h"
 #include"hni34.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 #include<time.h>
 #include<stdio.h>
 
@@ -218,7 +220,7 @@ int tcn351_ADSMConnector::CheckToolInstallation(int & IsInstalled)
                 rc=0;
             else
             {
-                strcpy(ADSMProgram, TheADSMDirectory.GiveName());
+                SAPDB_strcpy(ADSMProgram, TheADSMDirectory.GiveName());
                 strcat(ADSMProgram, pszADSMProgramDefault);
 
                 if(!cn36_CheckExe(ADSMProgram))
@@ -773,9 +775,9 @@ int tcn351_ADSMConnector::GiveInformationOfLine(const char                   *
                         if(cn36_StrNToInt(Year, StartOfDate, 2))
                         {
                             if(Year<30)
-                                strcpy(TempDateTimeString, "20");               // init missing century with 00
+                                SAPDB_strcpy(TempDateTimeString, "20");               // init missing century with 00
                             else
-                                strcpy(TempDateTimeString, "19");
+                                SAPDB_strcpy(TempDateTimeString, "19");
 
                             cn36_StrNCpy(TempDateTimeString+2, StartOfDate, DateTimeLength);    // +2 for missing century
                             TempDateTimeConstructed=1;
@@ -1136,7 +1138,7 @@ int tcn351_ADSMConnector::ConstructEBIDFromParallelLabel(char       *& NewEBID,
     if(cn36_StrAlloc(NewEBID, LabelLengthWithoutPieces+strlen(PartSuffix)))
     {
         cn36_StrNCpy(NewEBID, TheLabel, LabelLengthWithoutPieces);
-        strcpy(NewEBID+LabelLengthWithoutPieces, PartSuffix);
+        SAPDB_strcpy(NewEBID+LabelLengthWithoutPieces, PartSuffix);
     }
     else
         rc=0;
@@ -1160,7 +1162,7 @@ int tcn351_ADSMConnector::ConstructParallelLabelFromEBID(char       *& ParallelL
     if(rc)
     {
         cn36_StrNCpy(ParallelLabel, EBID, LengthEBID-strlen(PieceSection));
-        strcpy(ParallelLabel+LengthEBID-strlen(PieceSection), PiecesPart);
+        SAPDB_strcpy(ParallelLabel+LengthEBID-strlen(PieceSection), PiecesPart);
     }
 
     return rc;
