@@ -279,15 +279,7 @@ int tcn35a_DBBackupController::BackupOperationIsRunning()
                MaxWaitingTimeOfCurrentWaitingPhase()>=cn36_SecondsBetween(StartOfCurrentWaitingPhase, cn36_GetCurrentTime()))
             {
                 if(DBReplyAvailable(OutputMessagesForNonEvents)) // check db again
-                {
-                    if(OutputMessagesForNonEvents)
-                        LogWriter->setProperty(tcn35d_DBBackupControllerLogWriter::UsePrefix);
-
                     ReceiveReplyFromDB();
-
-                    if(OutputMessagesForNonEvents)
-                        LogWriter->setProperty(tcn35d_DBBackupControllerLogWriter::UsePrefixAndDateTime);
-                }
                 else
                     ChangePhase=0;
             }
@@ -338,7 +330,7 @@ int tcn35a_DBBackupController::BackupOperationIsRunning()
                 if(BackupToolConnector->ToolIsRunning(ToolFinishedCompletely, OutputMessagesForNonEvents) || !ToolFinishedCompletely)
                 {
                     sprintf(MsgBuf_cn36, "Canceling tool after the timeout of %d seconds elapsed.\n", int(MaxWaitingTimeOfCurrentWaitingPhase()));
-                    LogWriter->Write(MsgBuf_cn36);
+                    LogWriter->Write(MsgBuf_cn36, tcn35d_DBBackupControllerLogWriter::UsePrefixAndDateTime);
 
                     BackupToolConnector->CancelTool();
                 }
@@ -361,18 +353,10 @@ int tcn35a_DBBackupController::BackupOperationIsRunning()
 
         if(RequestSend==StatusOfRequest || CancelSendByDBMSrv==StatusOfRequest) // was the DB working on the request?
         {
-            if(MaxWaitingTimeOfCurrentWaitingPhase()>=cn36_SecondsBetween(StartOfCurrentWaitingPhase,cn36_GetCurrentTime()))
+            if(MaxWaitingTimeOfCurrentWaitingPhase()>=cn36_SecondsBetween(StartOfCurrentWaitingPhase, cn36_GetCurrentTime()))
             {
                 if(DBReplyAvailable(OutputMessagesForNonEvents))  // check db again
-                {
-                    if(OutputMessagesForNonEvents)
-                        LogWriter->setProperty(tcn35d_DBBackupControllerLogWriter::UsePrefix);
-
                     ReceiveReplyFromDB();
-                    
-                    if(OutputMessagesForNonEvents)
-                        LogWriter->setProperty(tcn35d_DBBackupControllerLogWriter::UsePrefixAndDateTime);
-                }
             }
             else 
                 TerminateDBAction();
@@ -391,7 +375,7 @@ int tcn35a_DBBackupController::BackupOperationIsRunning()
                     if(BackupToolConnector->ToolIsRunning(ToolFinishedCompletely, OutputMessagesForNonEvents) || !ToolFinishedCompletely)
                     {
                         sprintf(MsgBuf_cn36, "Terminating tool after the timeout of %d seconds elapsed.\n", int(MaxWaitingTimeOfCurrentWaitingPhase()));
-                        LogWriter->Write(MsgBuf_cn36);
+                        LogWriter->Write(MsgBuf_cn36, tcn35d_DBBackupControllerLogWriter::UsePrefixAndDateTime);
 
                         BackupToolConnector->TerminateTool();
                         StatusOfTool=KilledByDBMSrv;    // calling TerminateTool() only once
