@@ -47,6 +47,9 @@
 
 #include <time.h>
 
+#include "SAPDB/DBM/Srv/DBMSrv_Reply.hpp"
+#include "SAPDB/DBM/Srv/HSS/DBMSrvHSS_Nodes.hpp"
+
 
 // -----------------------------------------------------------------------------
 // member functions of class tcn35b_DBMExternalBackupController
@@ -1084,7 +1087,9 @@ void tcn35b_DBMExternalBackupController::WriteHistoryFile()
 
                 if(0!=BackupInfo->PathOfExternalBackupHistory())
                 {
-                    int NumberOfExternalBackupIDsFound=0;
+                    int    NumberOfExternalBackupIDsFound=0;
+                    char * lineForHSSCommand=0; //!< used to hold the backup_history_append command for HSS systems
+                    bool   isHSS=DBMSrvHSS_Nodes::isHSS();
 
                     AllIsOK=1;
 
@@ -1112,6 +1117,12 @@ void tcn35b_DBMExternalBackupController::WriteHistoryFile()
                             NumberOfExternalBackupIDsFound=BackupInfo->NumberOfPipes();
                     }
 
+                    if(AllIsOK && isHSS && !cn36_StrAllocCpy(lineForHSSCommand, "backup_history_append"))
+                    {
+                        ErrorHandler->Report(tcn35d_DBBackupError::OutOfMemory.Set());
+                        AllIsOK=0;
+                    }
+
                     if(AllIsOK)
                     {
                         tni34_ATextFile TheLogFile(BackupInfo->PathOfExternalBackupHistory(), tni34_AFile::ToDiskAppend, 1, 1);
@@ -1169,15 +1180,47 @@ void tcn35b_DBMExternalBackupController::WriteHistoryFile()
                                         AllIsOK=0; //end the loop
                                         ErrorHandler->Report(tcn35e_DBMBackupError(tcn35e_DBMBackupError::CaseCouldNotWriteToToolHistFile, BackupInfo->PathOfExternalBackupHistory()));
                                     }
+
+                                    if(isHSS) //append NewLogFileEntry to the HSS command line, if this is a HSS
+                                    {
+                                        char * newLineForHSSCommand=0;
+
+                                        if(cn36_StrAlloc(newLineForHSSCommand, strlen(lineForHSSCommand)+2+strlen(NewLogFileEntry)+1))
+                                        {
+                                            strcpy(newLineForHSSCommand, lineForHSSCommand);
+                                            strcat(newLineForHSSCommand, " \"");
+                                            strcat(newLineForHSSCommand, NewLogFileEntry);
+                                            strcat(newLineForHSSCommand, "\"");
+
+                                            cn36_StrDealloc(lineForHSSCommand);
+                                            lineForHSSCommand=newLineForHSSCommand;
+                                        }
+                                        else
+                                        {
+                                            ErrorHandler->Report(tcn35d_DBBackupError::OutOfMemory.Set());
+                                            AllIsOK=0;
+                                        }
+                                    }
                                 }
                             }
 
+                            int AllIsOKBeforeClose=AllIsOK;
+
                             if(!TheLogFile.Close())
                                 AllIsOK=0;
+
+                            if(AllIsOKBeforeClose && isHSS)
+                            {
+                                DBMSrv_Reply tempReply; //use a temporary reply buffer, as we are not really interested in the output during update of dbm.ebf
+
+                                cn00DBMServerData::vcontrol()->pHSSNodes->ExecuteOnAllOtherNodes(lineForHSSCommand, tempReply);
+                            }
                         }
                         else
                             AllIsOK=0;
                     }
+
+                    cn36_StrDealloc(lineForHSSCommand); //last chance to free the memory
                 }
                 else
                     ErrorHandler->Report(tcn35d_DBBackupError(tcn35d_DBBackupError::CaseInternalError));  //AllIsOK is already 0
