@@ -40,6 +40,7 @@
 #include "hcn42.h"
 #include "hcn51.h"
 #include "SAPDB/DBM/Srv/BackupHistory/DBMSrvBHist_History.hpp"
+#include "SAPDB/DBM/Srv/HSS/DBMSrvHSS_Nodes.hpp"
 
 
 // -----------------------------------------------------------------------------
@@ -106,7 +107,7 @@ StageFileInfo::StageFileInfo(const char * AbsoluteNameOfStageFile,
 
     if(cn36_StrAlloc(NameOfStageFile, LengthAbsoluteName+VersionLength))   //create a copy of the absolute stage file name including the version number
     {
-        strcpy(NameOfStageFile, AbsoluteNameOfStageFile);
+        SAPDB_strcpy(NameOfStageFile, AbsoluteNameOfStageFile);
         strncpy(NameOfStageFile+LengthAbsoluteName, Version, VersionLength);
         NameOfStageFile[LengthAbsoluteName+VersionLength]='\0';
 
@@ -180,9 +181,9 @@ int StageFileInfo::SetInternalBackupID(const char * Key, const char * Label)
 
     if(cn36_StrAlloc(InternalBackupID, strlen(Key)+1+strlen(Label)))
     {
-        strcpy(InternalBackupID, Key);
+        SAPDB_strcpy(InternalBackupID, Key);
         InternalBackupID[strlen(Key)]='|';
-        strcpy(InternalBackupID+strlen(Key)+1, Label);
+        SAPDB_strcpy(InternalBackupID+strlen(Key)+1, Label);
     }
     else
         rc=0;
@@ -722,8 +723,8 @@ int tcn35c_DBMStageBackupController::Prepare()
                     {
                         char DateTimeString[17];
 
-                        strcpy(DateTimeString, Date);
-                        strcpy(DateTimeString+8, Time);
+                        SAPDB_strcpy(DateTimeString, Date);
+                        SAPDB_strcpy(DateTimeString+8, Time);
 
                         tcn36_DateTimeString DateTime(DateTimeString, "yyyymmddHHMMSS");
 
@@ -1204,11 +1205,13 @@ void tcn35c_DBMStageBackupController::WriteHistoryFile()
 {
     if(OK_CN00==GiveLastError() && RequestReplied==RequestState())
     {
-        int i, rc=0;
+        int i, rc=1;
 
         LogWriter->Write("Updating external backup history.\n", tcn35d_DBBackupControllerLogWriter::AddALevelToPrefix);
 
-        int NumberOfExternalBackupIDsFound=0;
+        int    NumberOfExternalBackupIDsFound=0;
+        char * lineForHSSCommand=0; //!< used to hold the backup_history_append command for HSS systems
+        bool   isHSS=DBMSrvHSS_Nodes::isHSS();
 
         for(i=0; i<MAX_NUMBER_OF_TOOLPROCESSES_CN35; i++)
             cn36_StrDealloc(ExternalBackupID[i]);
@@ -1216,9 +1219,17 @@ void tcn35c_DBMStageBackupController::WriteHistoryFile()
         if(!BackupToolConnector->CollectExternalBackupIDs(ExternalBackupID, NumberOfExternalBackupIDsFound) ||
             NumberOfExternalBackupIDsFound!=BackupInfo->NumberOfPipes())
         {
-            ErrorHandler->Report(tcn35e_DBMBackupError(tcn35e_DBMBackupError::CaseCouldNotGetExternalBackupIDs));    // rc allready 0
+            ErrorHandler->Report(tcn35e_DBMBackupError(tcn35e_DBMBackupError::CaseCouldNotGetExternalBackupIDs));
+            rc=0;
         }
-        else
+
+        if(rc && isHSS && !cn36_StrAllocCpy(lineForHSSCommand, "backup_history_append"))
+        {
+            ErrorHandler->Report(tcn35d_DBBackupError::OutOfMemory.Set());
+            rc=0;
+        }
+
+        if(rc)
         {
             tni34_ATextFile TheLogFile(BackupInfo->PathOfExternalBackupHistory(), tni34_AFile::ToDiskAppend, 1, 1);
 
@@ -1227,8 +1238,6 @@ void tcn35c_DBMStageBackupController::WriteHistoryFile()
                 char NewLogFileEntry[2048];
                 int i;
 
-                rc=1;
-
                 for(i=0; i<NumberOfExternalBackupIDsFound && rc; i++)
                 {
                     char * DateTime=CurrentStageFile->GiveInfo()->GiveDateTimeAsString();
@@ -1273,17 +1282,51 @@ void tcn35c_DBMStageBackupController::WriteHistoryFile()
                             rc=0; //end the loop
                             ErrorHandler->Report(tcn35e_DBMBackupError(tcn35e_DBMBackupError::CaseCouldNotWriteToToolHistFile, BackupInfo->PathOfExternalBackupHistory()));
                         }
+
+                        if(isHSS) //append NewLogFileEntry to the HSS command line, if this is a HSS
+                        {
+                            char * newLineForHSSCommand=0;
+
+                            if(cn36_StrAlloc(newLineForHSSCommand, strlen(lineForHSSCommand)+2+strlen(NewLogFileEntry)+1))
+                            {
+                                strcpy(newLineForHSSCommand, lineForHSSCommand);
+                                strcat(newLineForHSSCommand, " \"");
+                                strcat(newLineForHSSCommand, NewLogFileEntry);
+                                strcat(newLineForHSSCommand, "\"");
+
+                                cn36_StrDealloc(lineForHSSCommand);
+                                lineForHSSCommand=newLineForHSSCommand;
+                            }
+                            else
+                            {
+                                ErrorHandler->Report(tcn35d_DBBackupError::OutOfMemory.Set());
+                                rc=0;
+                            }
+                        }
                     }
 
                     if(0!=DateTime)
                         cn36_StrDealloc(DateTime);
                 }
 
+                int AllIsOKBeforeClose=rc;
+
                 if(!TheLogFile.Close())
                     rc=0;
-            }   // else: rc already 0
+
+                if(AllIsOKBeforeClose && isHSS)
+                {
+                    DBMSrv_Reply tempReply; //use a temporary reply buffer, as we are not really interested in the output during update of dbm.ebf
+
+                    cn00DBMServerData::vcontrol()->pHSSNodes->ExecuteOnAllOtherNodes(lineForHSSCommand, tempReply);
+                }
+            }
+            else
+                rc=0;
         }
 
+        cn36_StrDealloc(lineForHSSCommand); //last chance to free the memory
+
         LogWriter->Write(rc?"Have updated external backup history successfully.\n":"Could not update external backup history.\n", tcn35d_DBBackupControllerLogWriter::SubALevelFromPrefixAndEndParagraph);
     }
 }
