@@ -224,9 +224,9 @@ int tcn35e_DBMBackupControllerLogWriter::Write(const char * Message, MessageModi
                         CharactersToPrint=strlen(CurrentPrefix)+strlen(giveCurrentTimeString())+(isCurrentTimeStringEmpty()?0:1)+strlen(CurrentPos);
 
                         if(isCurrentTimeStringEmpty())
-                            strcpy(FormatString, "%s%s%s");
+                            SAPDB_strcpy(FormatString, "%s%s%s");
                         else
-                            strcpy(FormatString, "%s%s %s");
+                            SAPDB_strcpy(FormatString, "%s%s %s");
 
                         if((int)CharactersToPrint>fprintf(ProtStream, FormatString, CurrentPrefix, giveCurrentTimeString(), CurrentPos))
                             rc=0;
@@ -333,8 +333,8 @@ int tcn35e_DBMBackupControllerLogWriter::Write(tni34_AFile & TheFileToMove, cons
 
                     if(rc)
                     {
-                        strcpy(LinePlusNewLine, Line);
-                        strcpy(LinePlusNewLine+LengthOfLine, "\n"); //LinePlusNewLine is now Line+"\n"
+                        SAPDB_strcpy(LinePlusNewLine, Line);
+                        SAPDB_strcpy(LinePlusNewLine+LengthOfLine, "\n"); //LinePlusNewLine is now Line+"\n"
 
                         if(!Write(LinePlusNewLine))
                             rc=0;// report this error after the LogFile was closed, because it will be reported to the LogFile !
@@ -599,7 +599,7 @@ int tcn35e_DBMBackupControllerLogWriter::ReadNextLine(tsp00_Int4 & File, char *&
                         rc=0;
                     else
                     {
-                        strcpy(BiggerLine, Line);   //copy the part of the line read so far
+                        SAPDB_strcpy(BiggerLine, Line);   //copy the part of the line read so far
                     
                         cn36_StrDealloc(Line);         //forget Line and us BiggerLine as Line
                         Line=BiggerLine;
