@@ -8,7 +8,7 @@
   version:      7.2.5
   see also:
   ------------------------------------------------------------------------------
-                         Copyright (c) 1999-2004 SAP AG-2004
+                         Copyright (c) 1999-2004 SAP AG
 
 
 
@@ -37,6 +37,8 @@
 #include "hni31.h"
 #include "hcn36.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 #include<sys/types.h>
 #include<sys/stat.h>
 #if defined (WIN32)
@@ -59,6 +61,7 @@
 #include<time.h>
 #include<limits.h>
 
+#include "SAPDB/SAPDBCommon/SAPDB_string.h"
 
 // -----------------------------------------------------------------------------
 // class tcn36_CN36Error
@@ -242,7 +245,7 @@ int cn36_StrAllocCpy(char * & Dest, const char *Src)
     int rc=cn36_StrAlloc(Dest, strlen(Src));
 
     if(rc)
-        strcpy(Dest, Src);
+        SAPDB_strcpy(Dest, Src);
 
     return rc;
 }
@@ -750,6 +753,19 @@ char * cn36_StartOfTrailingWhiteSpaces(char * s)
     return s1;
 }
 
+char * cn36_RemoveOuterDoubleQuotes(char * s) // same as private function cn31_RemoveDoublequotes()
+{
+    size_t length=strlen(s);
+
+    if(1<length && ('"'==s[0]) && ('"'==s[length-1]))   //at least two characters and first and last characters are double quotes
+    {
+        SAPDB_memmove(s, s+1, length-2); //copy character for character do not use strncpy, as source and destination overlap
+
+        s[length-2]='\0';
+    }
+
+    return s;
+}
 
 void cn36_GetStringBetweenDoubleQuotesOrWhiteSpaces(const char * & Start, const char * & End)
 {
@@ -973,7 +989,7 @@ int cn36_ConstructCompletePath(char * & CompletePath, const char * Directory, co
 
     if(cn36_StrAlloc(CompletePath, LengthOfDirectory+1+strlen(FileName)))   // resevere space for the directory, \ or / (only inserted if needed) and the file name
     {
-        strcpy(CompletePath, Directory);    // path starts with directory
+        SAPDB_strcpy(CompletePath, Directory);    // path starts with directory
 
         if(0==LengthOfDirectory ||                                  // if the directory is empty or
            (0<LengthOfDirectory &&                                  // (if at least one char in Directory
@@ -983,7 +999,7 @@ int cn36_ConstructCompletePath(char * & CompletePath, const char * Directory, co
             ++LengthOfDirectory;
         }
 
-        strcpy(CompletePath+LengthOfDirectory, FileName);
+        SAPDB_strcpy(CompletePath+LengthOfDirectory, FileName);
 
         rc=1;
     }
@@ -1559,6 +1575,11 @@ char * tcn36_DateTimeFormat::CreateCopyOfString() const
     return rc;
 }
 
+const char * tcn36_DateTimeFormat::giveFormatString() const
+{
+    return (0==FormatString) ? "" : FormatString;
+}
+
 tcn36_DateTimeFormat::AMPMType tcn36_DateTimeFormat::StartsWithAMPM(const char *String, size_t & CharsUsedByAMPM)
 {
     tcn36_DateTimeFormat::AMPMType rc=None;
