@@ -34,6 +34,8 @@
 #include <limits.h>
 #include <time.h>
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 #include "hcn38.h"
 #include "hcn382.h"
 #include "hcn383.h"
@@ -1187,7 +1189,7 @@ static int cn38_ShareStagingAreasForInputEntries(ni31_List<tcn381_InputEntry *>
                 {
                     if(cn36_StrAlloc(IE->RestoreStageName, strlen(StagingAreaName[j])+20+1))
                     {
-                        strcpy(IE->RestoreStageName, StagingAreaName[j]);
+                        SAPDB_strcpy(IE->RestoreStageName, StagingAreaName[j]);
 
                         FoundStageWithSameName=1;
                         StageInUse[j]=1;
@@ -1271,7 +1273,7 @@ static int cn38_ShareStagingAreasForInputEntries(ni31_List<tcn381_InputEntry *>
                 {
                     StageInUse[MinJ]=1;
 
-                    strcpy(MinIE->RestoreStageName, StagingAreaName[MinJ]); //there is enough memory for strcpy, because we did a new char [strlen()+strlen()+...]
+                    SAPDB_strcpy(MinIE->RestoreStageName, StagingAreaName[MinJ]); //there is enough memory for SAPDB_strcpy, because we did a new char [strlen()+strlen()+...]
                     *cn36_BaseNameFrom(MinIE->RestoreStageName)='\0';
                     strcat(MinIE->RestoreStageName, OriginalFileName);
                 }
@@ -1379,20 +1381,20 @@ static int cn38_InitConverterObjects(ni31_List<tcn381_InputEntry *> & TheInputFi
 
             if(rc) // if wo got all the memory we need ...
             {
-                strcpy(Converter[i].CompleteStageName, IE->StageName);
+                SAPDB_strcpy(Converter[i].CompleteStageName, IE->StageName);
                 strcat(Converter[i].CompleteStageName, ".");
                 Converter[i].EndOfStageName=cn36_TerminatingZeroOf(Converter[i].CompleteStageName);
 
                 if(0!=IE->RestoreStageName)
                 {
-                    strcpy(Converter[i].DestinationStageDirectory, IE->RestoreStageName);   //copy the complete stage name and determine the directory in it
+                    SAPDB_strcpy(Converter[i].DestinationStageDirectory, IE->RestoreStageName);   //copy the complete stage name and determine the directory in it
 
                     if(UseFinalDirectoryDelimiterOnRestore)
                         *cn36_BaseNameFrom(Converter[i].DestinationStageDirectory)='\0';    //the last / or \ is included in the directory name
                     else
                         *cn36_EndOfDirectory(Converter[i].DestinationStageDirectory)='\0';  //the last / or \ is not included in the directory name (some Backint for Oracle implementations need that)
 
-                    strcpy(Converter[i].CompleteDestinationStage, IE->RestoreStageName);
+                    SAPDB_strcpy(Converter[i].CompleteDestinationStage, IE->RestoreStageName);
                     strcat(Converter[i].CompleteDestinationStage, ".");
                     Converter[i].EndOfCompleteDestinationStage=cn36_TerminatingZeroOf(Converter[i].CompleteDestinationStage);
                 }
