@@ -39,6 +39,8 @@
 #include "hcn383.h"
 #include "hcn385.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 #include <stdio.h>
 
 int cn383_WriteInformationToHistoryFile(const char                     * HistoryFileName,
@@ -346,7 +348,10 @@ int cn383_GetInformationFromOwnHistory(const char                     * PathOfHi
                 if(0==IE->ListOfPartialBIDs.First())
                     rc=0;
                 else
-                    strcpy(IE->BID, IE->ListOfPartialBIDs.First()->GiveInfo()); //substitute #NULL with the BID of the last save for the name
+                    SAPDB_memmove(
+                        IE->BID,
+                        IE->ListOfPartialBIDs.First()->GiveInfo(),
+                        strlen(IE->ListOfPartialBIDs.First()->GiveInfo())+1); //substitute #NULL with the BID of the last save for the name
             }
         }
     }
