@@ -41,6 +41,8 @@
 #include "hcn384.h"
 #include "hcn385.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 static const int MaxCmdLineLength_cn384=2048;
 
 int cn384_StartPipeToFileConverters(int                            & StartedProcesses,
@@ -102,13 +104,16 @@ int cn384_StartPipeToFileConverters(int                            & StartedProc
             {
                 char * CurrentPosition=CmdLine;
 
-                strcpy(Converter[StartedProcesses].CompleteStageName, StagingAreaName[StartedProcesses]);
+                SAPDB_memmove(
+                    Converter[StartedProcesses].CompleteStageName,
+                    StagingAreaName[StartedProcesses],
+                    strlen(StagingAreaName[StartedProcesses])+1);
                 strcat(Converter[StartedProcesses].CompleteStageName, ".");
 
                 Converter[StartedProcesses].EndOfStageName=Converter[StartedProcesses].CompleteStageName+strlen(Converter[StartedProcesses].CompleteStageName);
                 sprintf(Converter[StartedProcesses].EndOfStageName, "%d", int(h->GiveInfo()->NumberOfParts));
                 
-                strcpy(CmdLine, "pipe2file -d pipe2file -f ");
+                SAPDB_strcpy(CmdLine, "pipe2file -d pipe2file -f ");
                 strcat(CmdLine, StagingAreaName[StartedProcesses]);
                 strcat(CmdLine, " -p ");
                 strcat(CmdLine, h->GiveInfo()->Name);
@@ -151,7 +156,10 @@ int cn384_StartPipeToFileConverters(int                            & StartedProc
                         Converter[StartedProcesses].InputInfo=h->GiveInfo();
 
                         h->GiveInfo()->Converter=&(Converter[StartedProcesses]);
-                        strcpy(h->GiveInfo()->StageName, StagingAreaName[StartedProcesses]);
+                        SAPDB_memmove(
+                            h->GiveInfo()->StageName,
+                            StagingAreaName[StartedProcesses],
+                            strlen(StagingAreaName[StartedProcesses])+1);
                         h->GiveInfo()->StageSize=StagingAreaSize[StartedProcesses];
 
                         Converter[StartedProcesses].State=tcn381_APipeToFileConverter::ConverterIsWorking;
@@ -214,7 +222,7 @@ int cn384_StartNeededPipeToFileConverters(int                            & Start
             {   
                 char * EndOfCmdLine;
 
-                strcpy(CmdLine, "pipe2file -d file2pipe -f ");
+                SAPDB_strcpy(CmdLine, "pipe2file -d file2pipe -f ");
 
                 if(0==IE->RestoreStageName)
                     strcat(CmdLine, IE->StageName);
