@@ -64,7 +64,7 @@
   #include "RunTime/RTE_UNIXAuthenticate.hpp"           // nocheck
 #endif
 
-#include "SAPDB/DBM/Srv/DBMSrv_HSSNodes.hpp"
+#include "SAPDB/DBM/Srv/HSS/DBMSrvHSS_Nodes.hpp"
 
 /*
   -----------------------------------------------------------------------------
@@ -89,13 +89,15 @@ tcn00_Error cn41DBMSetValue
   _TCHAR  szName  [PARAMETER_MAXLEN_CN90];
   boolean bRaw = false;
 
-  cn90GetToken(command->args, szName,     1);
-  cn90GetToken(command->args, szValue,    2);
+  cn90GetToken(command->args, szName, 1, PARAMETER_MAXLEN_CN90);
+  cn90GetToken(command->args, szValue, 2, PARAMETER_MAXLEN_CN90);
+  cn90StripQuotes(szValue);
+
 
   if (_tcsicmp(szName, CFG_RAWDATA) == 0) {
     bRaw = true;
-    cn90GetToken(command->args, szName, 2);
-    if (!cn90GetToken(command->args, szValue, 3)) {
+    cn90GetToken(command->args, szName, 2, PARAMETER_MAXLEN_CN90);
+    if (!cn90GetToken(command->args, szValue, 3, PARAMETER_MAXLEN_CN90)) {
       szValue[0]='\0';  //force the deletion if no value was supplied (for type raw)
     } // end if
   } // end if
@@ -140,10 +142,10 @@ tcn00_Error cn41DBMGetValue
   _TCHAR  szName  [PARAMETER_MAXLEN_CN90];
   boolean bRaw = false;
   
-  cn90GetToken(command->args, szName, 1);
+  cn90GetToken(command->args, szName, 1, PARAMETER_MAXLEN_CN90);
   if (_tcsicmp(szName, CFG_RAWDATA) == 0) {
     bRaw = true;
-    if (!cn90GetToken(command->args, szName, 2)) {
+    if (!cn90GetToken(command->args, szName, 2, PARAMETER_MAXLEN_CN90)) {
       nFuncReturn = cn90AnswerIError(replyData, replyLen, ERR_PARAM_CN00);
     } // end if
   } // end if
@@ -191,7 +193,7 @@ tcn00_Error cn41UserLogon
   tcn002_XpValueString szColdPwd;
   bool                bErrSysLogon = false;
 
-  cn90GetToken(command->args, szToken, 1);
+  cn90GetToken(command->args, szToken, 1, PARAMETER_MAXLEN_CN90);
   pPassword = _tcschr(szToken, PWD_SEPARATOR_CN00);
   if (pPassword != NULL) {
     *pPassword = CHAR_STRINGTERM_CN90;
@@ -351,7 +353,7 @@ tcn00_Error cn41UserSystem
   tcn00_UserNamec     szUser;
   tsp00_Namec         szPassword;
 
-  cn90GetToken(command->args, szToken, 1);
+  cn90GetToken(command->args, szToken, 1, PARAMETER_MAXLEN_CN90);
   pPassword = _tcschr(szToken, PWD_SEPARATOR_CN00);
   if (pPassword != NULL) {
     *pPassword = CHAR_STRINGTERM_CN90;
@@ -600,8 +602,8 @@ tcn00_Error cn41UserCreate
   bool                bTemplate   = false;
     
   // read parameters
-  cn90GetToken(command->args, szToken, 1);
-  if ( cn90GetToken(command->args, szTemplate, 2)) {
+  cn90GetToken(command->args, szToken, 1, PARAMETER_MAXLEN_CN90);
+  if ( cn90GetToken(command->args, szTemplate, 2, PARAMETER_MAXLEN_CN90)) {
     bTemplate = true;
   } // end if
 
@@ -672,7 +674,7 @@ tcn00_Error cn41UserGet
   _TCHAR       * pData;
   _TCHAR         szName      [PARAMETER_MAXLEN_CN90];
 
-  cn90GetToken(command->args, szName,     1);
+  cn90GetToken(command->args, szName, 1, PARAMETER_MAXLEN_CN90);
 
   cn50DBMUser  dbUser( vcontrol->dbname, 
                        *(tcn00_UserNamec *) szName );
@@ -707,8 +709,8 @@ tcn00_Error cn41UserGetRights
   _TCHAR         szName      [PARAMETER_MAXLEN_CN90];
   _TCHAR         szRights    [PARAMETER_MAXLEN_CN90];
 
-  cn90GetToken(command->args, szName, 1);
-  cn90GetToken(command->args, szRights, 2);
+  cn90GetToken(command->args, szName, 1, PARAMETER_MAXLEN_CN90);
+  cn90GetToken(command->args, szRights, 2, PARAMETER_MAXLEN_CN90);
 
   cn50DBMUser  dbUser( vcontrol->dbname, 
                        *(tcn00_UserNamec *) szName );
@@ -802,7 +804,7 @@ tcn00_Error cn41UserPut
   const _TCHAR * pData       = NULL;
   cn50DBMUser    dbUser;
   
-  cn90GetToken(command->args, szName, 1);
+  cn90GetToken(command->args, szName, 1, PARAMETER_MAXLEN_CN90);
 
   dbUser.setDBName  ( vcontrol->dbname )
         .setUserName( *(tcn00_UserNamec *) szName           );
@@ -852,7 +854,7 @@ tcn00_Error cn41UserDelete
   _TCHAR         szName      [PARAMETER_MAXLEN_CN90];
   cn50DBMUser    dbUser;
   
-  cn90GetToken(command->args, szName, 1);
+  cn90GetToken(command->args, szName, 1, PARAMETER_MAXLEN_CN90);
   dbUser.setDBName  ( vcontrol->dbname )
         .setUserName( *(tcn00_UserNamec *) szName           );
 
