@@ -143,6 +143,7 @@
 #define KEY_2PWD                 _T("SECONDPASSWORD" )
 #define KEY_DISABLED             _T("DISABLED"       )
 #define KEY_COMMENT              _T("COMMENT"        )
+#define KEY_USERTYPE             _T("USERTYPE"       )
 #define KEY_YES                  _T("YES"            )
 #define KEY_NO                   _T("NO"             )
 
@@ -251,6 +252,10 @@ cn50DBMUser & cn50DBMUser :: setSecondPwd ( tsp00_Namec const szValue )
 {
 
   if (szValue.length() == 0) {
+    if (Data.bSecondPwd) {
+      bDirty = true;
+      bSecondPwdDirty = true;
+    } // end if
     Data.bSecondPwd = false;
   } else {
     tsp00_Namec tmpName;
@@ -263,11 +268,10 @@ cn50DBMUser & cn50DBMUser :: setSecondPwd ( tsp00_Namec const szValue )
     s02applencrypt( tmpValue, Data.cryptSecondPwd);
     Data.bSecondPwd = true;
 
+    bDirty = true;
+    bSecondPwdDirty = true;
   } // end if
 
-  bDirty = true;
-  bSecondPwdDirty = true;
-
   return *this;
 } // end cn50DBMUser :: setSecondPwd
 
@@ -590,6 +594,12 @@ void cn50DBMUser :: getData ( _TCHAR * pBuffer ) const
                                         getComment().asCharp());
     pCurrent = pCurrent + _tcslen(pCurrent);
 
+    // Usertype
+    _stprintf(pCurrent, _T("%s%c%s\n"), KEY_USERTYPE, 
+                                        CHAR_ASSIGN,
+                                        getType());
+    pCurrent = pCurrent + _tcslen(pCurrent);
+
   } // end if
 
 } // end cn50DBMUser :: getData
@@ -879,7 +889,7 @@ tcn00_Error  cn50DBMUser :: setData ( const _TCHAR * szData )
   if (szData != NULL) {
 
     // analyze tokens
-    while (cn90GetToken(szData, szToken, nToken)) {
+    while (cn90GetToken(szData, szToken, nToken, 1024)) {
       pValue = _tcschr(szToken, CHAR_ASSIGN);
 
       if (pValue != NULL) {
@@ -972,6 +982,29 @@ cn50DBMUser &  cn50DBMUser :: setRights(_TCHAR * szValue, tsp00_Uint4 & nValue,
   return *this;
 } // end cn50DBMUser :: setRights
 
+/*
+  -----------------------------------------------------------------------------
+  protected function cn50DBMUser :: getType
+  -----------------------------------------------------------------------------
+ */
+const char * cn50DBMUser :: getType
+      ( ) const
+{
+  const char * pType = "";
+
+  if (isColdUser()) {
+    pType = USERTYPE_DBM_CN50;
+  } else if (isSDBAUser()) {
+    pType = USERTYPE_DBA_CN50;
+  } else if (isDOMAIN()) {
+    pType = USERTYPE_DOM_CN50;
+  } else if (isSAPUser()) {
+    pType = USERTYPE_SAP_CN50;
+  } // end if
+
+  return pType;
+} // end cn50DBMUser :: getType 
+
 /*
   -----------------------------------------------------------------------------
   protected function cn50DBMUser :: getUser
@@ -1155,11 +1188,11 @@ tcn00_Error cn50DBMUser :: saveSQLUsr()
   } // end if
 
   if (nFuncReturn == OK_CN00) {
-    if (bSecondPwdDirty) {
+    if (bSecondPwdDirty && (getSecondPwd(true).length() != 0)) {
       sprintf(szCommand, "ALTER USER %s %s PASSWORD %s", 
                          getUserName().asCharp(), 
                          isSecondPwd() ? "ADD" : "DROP",
-                         isSecondPwd() ? getSecondPwd(true).asCharp() : "");
+                         getSecondPwd(true).asCharp());
       nFuncReturn = cn80ExecuteSQL(usrDBA, szCommand);
     } // end if
   } // end if
