@@ -543,7 +543,7 @@ bool cn52ParamHistoryRecord :: setSelection(const tcn52_C1024c & aSelection)
   m_nFields = ID_FULL;
 
   // analyze tokens
-  while (cn90GetToken(szSelection, szToken, nToken) && bOk) {
+  while (cn90GetToken(szSelection, szToken, nToken, 1024) && bOk) {
     pValue = _tcschr(szToken, CHAR_ASSIGN);
 
     if (pValue != NULL) {
