@@ -298,7 +298,7 @@ static tcn00_Error cn80_InternConnect (
   /**/
 
   /* read the parameters */
-  if (cn90GetToken(command->args, szUsername, 1)) {
+  if (cn90GetToken(command->args, szUsername, 1, PARAMETER_MAXLEN_CN90)) {
     pPassword = _tcschr(szUsername, PWD_SEPARATOR_CN00);
     if (pPassword != NULL) {
       *pPassword = CHAR_STRINGTERM_CN90;
@@ -307,7 +307,7 @@ static tcn00_Error cn80_InternConnect (
     } else {
 #ifdef CN71
       pPassword   = &szPassword[0];
-      if (!cn90GetToken(command->args, szPassword, 2)) {
+      if (!cn90GetToken(command->args, szPassword, 2, PARAMETER_MAXLEN_CN90)) {
         nFuncReturn = cn90AnswerIError(replyData, replyLen, ERR_PARAM_CN00);
       } else {
         cn90Uncrypt(pPassword, false);
@@ -437,9 +437,9 @@ static bool cn80_CheckCreateInstance (
   bool      bFound = false;
   char      szToken    [PARAMETER_MAXLEN_CN90];
 
-  cn90GetToken(szStatement, szToken, 1);
+  cn90GetToken(szStatement, szToken, 1, PARAMETER_MAXLEN_CN90);
   if (stricmp(szToken, "CREATE") == 0) {
-    cn90GetToken(szStatement, szToken, 2);
+    cn90GetToken(szStatement, szToken, 2, PARAMETER_MAXLEN_CN90);
     if (stricmp(szToken, "INSTANCE") == 0) {
       bFound = true;
     } // end if
@@ -448,24 +448,36 @@ static bool cn80_CheckCreateInstance (
   return bFound;
 } // end cn80_CheckCreateInstance
 
-/* PRIVATE FUNCTION cn80_CheckVerify
+/* PUBLIC FUNCTION cn80CheckVerify
  * ------------------------------------------------------------------
- * purpose: echeck for VERIFY
+ * purpose: check for VERIFY
  *
  */
-static bool cn80_CheckVerify (
-      const _TCHAR    * szStatement) 
+bool cn80CheckVerify (
+      const char * szStatement, const char * & pAction) 
 {
   bool      bVerify = false;
-  _TCHAR    szToken    [PARAMETER_MAXLEN_CN90];
+  char      szToken    [PARAMETER_MAXLEN_CN90];
 
-  cn90GetToken(szStatement, szToken, 1);
-  if (_tcsicmp(szToken, "VERIFY") == 0) {
-    bVerify = true;
+  if (cn90GetToken(szStatement, szToken, 1, PARAMETER_MAXLEN_CN90)) {
+    if (stricmp(szToken, "VERIFY") == 0) {
+      pAction = DBAOP_CHECKDB_CN53;
+      bVerify = true;
+    } else if (stricmp(szToken, "CHECK") == 0) {
+      if (cn90GetToken(szStatement, szToken, 2, PARAMETER_MAXLEN_CN90)) {
+        if (stricmp(szToken, "DATA") == 0) {
+          pAction = DBAOP_CHECKDB_CN53;
+          bVerify = true;
+        } else if (stricmp(szToken, "TABLE") == 0) {
+          pAction = DBAOP_TABLECHECK_CN53;
+          bVerify = true;
+        } // end if
+      } // end if
+    } // end if
   } // end if
 
   return bVerify;
-} // end cn80_CheckVerify
+} // end cn80CheckVerify
 
 /* PUBLIC FUNCTION cn80DBActivate
  * ------------------------------------------------------------------
@@ -673,8 +685,6 @@ tcn00_Error cn80UtilExecute (
   tin01_sql_session  * pSession;
   tin01_sql_session    aSQLSession;
   bool                 bConnect    = false;
-  bool                 bVerify     = false;
-  tsp00_Timestampc     szTimeStamp;
   
   /**/
 
@@ -693,10 +703,13 @@ tcn00_Error cn80UtilExecute (
     if (cn80_CheckCreateInstance(command->args)) {
       nFuncReturn = cn90AnswerIError(replyData, replyLen, ERR_PARAM_CN00, 0, "please use db_activate");
     } else { 
-      if (cn80_CheckVerify(command->args)) {
+      const char *     pAction = "";
+      bool             bVerify     = false;
+      tsp00_Timestampc szTimeStamp;
+      if (cn80CheckVerify(command->args, pAction)) {
         bVerify = true;
         cn90Timestamp(szTimeStamp);
-        tcn53_DBAAction::start(vcontrol->dbname, szTimeStamp, DBAOP_CHECKDB_CN53, DBA_OBJ_DB_CN53);
+        tcn53_DBAAction::start(vcontrol->dbname, szTimeStamp, pAction, DBA_OBJ_DB_CN53);
         tcn53_DBAAction::writeDetail(DBADTL_CDBCMD_CN53, command->args);
       } // end if
       nFuncReturn = cn80ExecuteUtil(pSession, command->args, replyData, replyLen); 
@@ -988,7 +1001,7 @@ tcn00_Error cn80SQLUserInfo (
                        replyData + strlen(replyData), 
                        SQL_CHAR_FIELDSEP);
 
-    strcpy(&replyData[strlen(replyData)], LINE_SEPSTRING_CN00);
+    SAPDB_strcpy(&replyData[strlen(replyData)], LINE_SEPSTRING_CN00);
 
     if (rc != 0) {
       nFuncReturn = ERR_SQL_CN00;
@@ -1033,7 +1046,7 @@ tcn00_Error cn80SQLUserFetch (
                   replyLenMax - (long)strlen(replyData),
                   SQL_CHAR_FIELDSEP,SQL_CHAR_STRINGDEL,SQL_CHAR_RECSEP,bContinue);
 
-    strcpy(&replyData[strlen(replyData)], LINE_SEPSTRING_CN00);
+    SAPDB_strcpy(&replyData[strlen(replyData)], LINE_SEPSTRING_CN00);
 
     if (rc != 0) {
       nFuncReturn = ERR_SQL_CN00;
@@ -1071,7 +1084,7 @@ static tcn00_Error cn80_SQLExecSQL (
       tin01_sql_session  * pSession,
       const char         * szStatement)
 {
-  FUNCTION_DBG_MCN00_1("cn80SQLExecSQL");
+  FUNCTION_DBG_MCN00_1("cn80_SQLExecSQL");
 
   tcn00_Error	        nFuncReturn = OK_CN00;
   int                 rc;
@@ -1081,8 +1094,18 @@ static tcn00_Error cn80_SQLExecSQL (
   bool                bContinue = false;
   /**/
 
+  const char *     pAction = "";
+  bool             bVerify     = false;
+  tsp00_Timestampc szTimeStamp;
+  if (cn80CheckVerify(szStatement, pAction)) {
+    bVerify = true;
+    cn90Timestamp(szTimeStamp);
+    tcn53_DBAAction::start(cn00DBMServerData::vcontrol()->dbname, szTimeStamp, pAction, DBA_OBJ_DB_CN53);
+    tcn53_DBAAction::writeDetail(DBADTL_CDBCMD_CN53, szStatement);
+  } // end if
+
   /* check the command */
-  if (cn90GetToken(szStatement, szCommand, 1)) {
+  if (cn90GetToken(szStatement, szCommand, 1, PARAMETER_MAXLEN_CN90)) {
     bSelect = (cn90Stricmp(szCommand, SQL_SELECT) == 0);
   } /* end if */
 
@@ -1119,7 +1142,7 @@ static tcn00_Error cn80_SQLExecSQL (
                    replyLenMax - (long)strlen(replyData),
                    SQL_CHAR_FIELDSEP,SQL_CHAR_STRINGDEL,SQL_CHAR_RECSEP,bContinue);
 
-    strcpy(&replyData[strlen(replyData)], LINE_SEPSTRING_CN00);
+    SAPDB_strcpy(&replyData[strlen(replyData)], LINE_SEPSTRING_CN00);
 
     if (rc != 0) {
       nFuncReturn = ERR_SQL_CN00;
@@ -1137,6 +1160,11 @@ static tcn00_Error cn80_SQLExecSQL (
  
   /**/
 
+  if (bVerify) {
+    tcn53_DBAAction::writeDetail(DBADTL_CDBRESULT_CN53, replyData);
+    tcn53_DBAAction::finish(cn90Timestamp(szTimeStamp), (nFuncReturn == OK_CN00));
+  } // end if
+
   return nFuncReturn;
 } /* end cn80_SQLExecSQL */
 
