@@ -227,7 +227,7 @@ tcn00_Error cn81_InfoState(
     hostvar[0].len = sizeof(unsigned long);
     hostvar[0].c_type.becomes(cin01_c_ulong); 
 
-    strcpy(szCommand, "SELECT ");           
+    SAPDB_strcpy(szCommand, "SELECT ");           
     strcat(szCommand, "NUM(VALUE) ");
     strcat(szCommand, "FROM SYSDD.XPARAMETERS ");
     strcat(szCommand, "WHERE UPPER(DESCRIPTION) = '_PAGE_SIZE'");
@@ -266,7 +266,7 @@ tcn00_Error cn81_InfoState(
       hostvar[nIndex].c_type.becomes(cin01_c_ulong); 
     } 
 
-    strcpy(szCommand, "SELECT ");           
+    SAPDB_strcpy(szCommand, "SELECT ");           
     strcat(szCommand, "USED_PERM_PAGES, USED_TEMP_PAGES, ");
     strcat(szCommand, "LOG_PAGES, PCT_LOG_NOT_SAVED, LOG_NOT_SAVED, ");
 //    strcat(szCommand, "MAX_PERM_PAGES, RESERVED_REDO_SIZE ");
@@ -315,7 +315,7 @@ tcn00_Error cn81_InfoState(
     hostvar[0].len = sizeof(unsigned long);
     hostvar[0].c_type.becomes(cin01_c_ulong); 
 
-    strcpy(szCommand, "SELECT ");           
+    SAPDB_strcpy(szCommand, "SELECT ");           
     strcat(szCommand, "SUM(NUMERIC_VALUE) ");
     strcat(szCommand, "FROM SYSDD.CONFIGURATION ");
     strcat(szCommand, "WHERE UPPER(DESCRIPTION) LIKE 'DATA DEVSPACE *'");
@@ -389,7 +389,7 @@ tcn00_Error cn81_InfoState(
     hostvar[0].len = sizeof(unsigned long);
     hostvar[0].c_type.becomes(cin01_c_ulong); 
 
-    strcpy(szCommand, "SELECT ");           
+    SAPDB_strcpy(szCommand, "SELECT ");           
     strcat(szCommand, "COUNT(SESSION) ");
     strcat(szCommand, "FROM SYSDD.TRANSACTIONS");
 
@@ -426,7 +426,7 @@ tcn00_Error cn81_InfoState(
     hostvar[0].len = sizeof(unsigned long);
     hostvar[0].c_type.becomes(cin01_c_ulong); 
 
-    strcpy(szCommand, "SELECT ");           
+    SAPDB_strcpy(szCommand, "SELECT ");           
     strcat(szCommand, "NUM(VALUE) ");
     strcat(szCommand, "FROM SYSDD.XPARAMETERS ");
     strcat(szCommand, "WHERE UPPER(DESCRIPTION) = 'MAXUSERTASKS'");
@@ -465,7 +465,7 @@ tcn00_Error cn81_InfoState(
     hostvar[0].len = sizeof(unsigned long);
     hostvar[0].c_type.becomes(cin01_c_ulong); 
 
-    strcpy(szCommand, "SELECT ");           
+    SAPDB_strcpy(szCommand, "SELECT ");           
     strcat(szCommand, "VALUE ");
     strcat(szCommand, "FROM SYSDD.MONITOR ");
     strcat(szCommand, "WHERE UPPER(TYPE) = 'CACHES' ");
@@ -503,7 +503,7 @@ tcn00_Error cn81_InfoState(
     hostvar[0].len = sizeof(unsigned long);
     hostvar[0].c_type.becomes(cin01_c_ulong); 
 
-    strcpy(szCommand, "SELECT ");           
+    SAPDB_strcpy(szCommand, "SELECT ");           
     strcat(szCommand, "VALUE ");
     strcat(szCommand, "FROM SYSDD.MONITOR ");
     strcat(szCommand, "WHERE UPPER(TYPE) = 'CACHES' ");
@@ -541,7 +541,7 @@ tcn00_Error cn81_InfoState(
     hostvar[0].len = sizeof(unsigned long);
     hostvar[0].c_type.becomes(cin01_c_ulong); 
 
-    strcpy(szCommand, "SELECT ");           
+    SAPDB_strcpy(szCommand, "SELECT ");           
     strcat(szCommand, "COUNT(DESCRIPTION) ");
     strcat(szCommand, "FROM SYSDD.SERVER_DB_STATE ");
     strcat(szCommand, "WHERE UPPER(DESCRIPTION) LIKE 'BAD DEV*'");
@@ -578,7 +578,7 @@ tcn00_Error cn81_InfoState(
     hostvar[0].len = sizeof(unsigned long);
     hostvar[0].c_type.becomes(cin01_c_ulong); 
 
-    strcpy(szCommand, "SELECT ");           
+    SAPDB_strcpy(szCommand, "SELECT ");           
     strcat(szCommand, "VALUE ");
     strcat(szCommand, "FROM SYSDD.MONITOR ");
     strcat(szCommand, "WHERE UPPER(TYPE) = 'CACHES' ");
@@ -616,7 +616,7 @@ tcn00_Error cn81_InfoState(
     hostvar[0].len = sizeof(unsigned long);
     hostvar[0].c_type.becomes(cin01_c_ulong); 
 
-    strcpy(szCommand, "SELECT ");           
+    SAPDB_strcpy(szCommand, "SELECT ");           
     strcat(szCommand, "VALUE ");
     strcat(szCommand, "FROM SYSDD.MONITOR ");
     strcat(szCommand, "WHERE UPPER(TYPE) = 'CACHES' ");
@@ -779,7 +779,7 @@ tcn00_Error cn81_InfoView(
   
   //
 
-  strcpy(szCommand, "SELECT * FROM ");           
+  SAPDB_strcpy(szCommand, "SELECT * FROM ");           
   strcat(szCommand, InfoNamesC[iView].szView);
   if (szWhere[0] != '\0') {
     strcat(szCommand, " WHERE ");
@@ -858,7 +858,7 @@ tcn00_Error cn81Info(
   szWhere[0] = '\0';
 
   if (nFuncReturn == OK_CN00) {
-    if (cn90GetToken(command->args, szName, nParam)) {
+    if (cn90GetToken(command->args, szName, nParam, 255)) {
       nParam++;
     } else {
       nFuncReturn = ERR_PARAM_CN00;
@@ -868,7 +868,7 @@ tcn00_Error cn81Info(
 
   if (nFuncReturn == OK_CN00) {
     bMore = true;
-    while (cn90GetToken(command->args, szParam, nParam) && bMore)
+    while (cn90GetToken(command->args, szParam, nParam, 16) && bMore)
     {
       nParam++;
 
@@ -881,7 +881,7 @@ tcn00_Error cn81Info(
           pWhereOpt = (char *)strstr(command->args, INFO_WHERE_OPT);
           if (strlen(pWhereOpt) > (strlen(INFO_WHERE_OPT) + 1)) {
             pWhereOpt += (strlen(INFO_WHERE_OPT) + 1);
-            strcpy(szWhere, pWhereOpt);
+            SAPDB_strcpy(szWhere, pWhereOpt);
           }
 
           cn81_RemoveDoublequotes(szWhere);
