@@ -155,7 +155,9 @@ static tcn00_Error cn82_AnalyzeParameters
         tcn82EventList * pEvent,
         tcn82PrioList  * pPriority,
         _TCHAR         * sz1stValue,
-        _TCHAR         * sz2ndValue );
+        long             size1stValue,
+        _TCHAR         * sz2ndValue,
+        long             size2ndValue);
 
 static tcn00_Error cn82_EventChange
       ( VControlDataT  * vcontrol,
@@ -473,8 +475,10 @@ static tcn00_Error cn82_AnalyzeParameters
       ( const _TCHAR   * szCommand,
         tcn82EventList * pEvent,
         tcn82PrioList  * pPriority,
-              _TCHAR   * sz1stValue,
-              _TCHAR   * sz2ndValue )
+        _TCHAR         * sz1stValue,
+        long             size1stValue,
+        _TCHAR         * sz2ndValue,
+        long             size2ndValue)
 {
   tcn00_Error	         nFuncReturn = OK_CN00;
   long                 nIndex      = 0; 
@@ -487,7 +491,7 @@ static tcn00_Error cn82_AnalyzeParameters
   _TCHAR               szPriority[PARAMETER_MAXLEN_CN90];  
 
   // read and check eventname
-  cn90GetToken(szCommand, szEvent, 1);
+  cn90GetToken(szCommand, szEvent, 1, PARAMETER_MAXLEN_CN90);
   while (lstEvents[nIndex].pName != NULL && _tcsicmp(szEvent, lstEvents[nIndex].pName) != 0) {
     nIndex++;
   } // end if
@@ -497,7 +501,7 @@ static tcn00_Error cn82_AnalyzeParameters
 
   // read and check priority
   if (nFuncReturn == OK_CN00) {
-    if (cn90GetToken(szCommand, szPriority, 2)) {
+    if (cn90GetToken(szCommand, szPriority, 2, PARAMETER_MAXLEN_CN90)) {
       while (lstPrio[nPrioIndex].pName != NULL && _tcsicmp(szPriority, lstPrio[nPrioIndex].pName) != 0) {
         nPrioIndex++;
       } // end if
@@ -515,8 +519,8 @@ static tcn00_Error cn82_AnalyzeParameters
 
   // read and check values
   if (nFuncReturn == OK_CN00) {
-    nValueCount = nValueCount + cn90GetToken(szCommand, sz1stValue, 3)?1:0;
-    nValueCount = nValueCount + cn90GetToken(szCommand, sz2ndValue, 4)?1:0;
+    nValueCount = nValueCount + cn90GetToken(szCommand, sz1stValue, 3, size1stValue)?1:0;
+    nValueCount = nValueCount + cn90GetToken(szCommand, sz2ndValue, 4, size2ndValue)?1:0;
 
     if (_tcslen(szPriority) > 0 && cn90AnalyzeNumber(szPriority, &nValue)) {
        _tcscpy(sz2ndValue, sz1stValue);
@@ -532,8 +536,8 @@ static tcn00_Error cn82_AnalyzeParameters
       
   // set structures
   if (nFuncReturn == OK_CN00) {
-    memcpy(pPriority, &lstPrio[nPrioIndex], sizeof(tcn82PrioList));
-    memcpy(pEvent,    &lstEvents[nIndex],   sizeof(tcn82EventList));
+    SAPDB_memcpy(pPriority, &lstPrio[nPrioIndex], sizeof(tcn82PrioList));
+    SAPDB_memcpy(pEvent,    &lstEvents[nIndex],   sizeof(tcn82EventList));
   } // end if
 
   return nFuncReturn;
@@ -565,8 +569,10 @@ static tcn00_Error cn82_EventChange
   nFuncReturn = cn82_AnalyzeParameters(command->args, 
                                        &aEvent,
                                        &aPrio, 
-                                       sz1stValue, 
-                                       sz2ndValue);
+                                       sz1stValue,
+                                       PARAMETER_MAXLEN_CN90,
+                                       sz2ndValue,
+                                       PARAMETER_MAXLEN_CN90);
   
   // look for utility session
   if (nFuncReturn == OK_CN00) {
