@@ -297,14 +297,14 @@ tcn00_Error cn84TraceShow
     while (oProtList[nPos].szShortDesc != NULL && oProtList[nPos].szLongDesc != NULL) {
 
       if (oProtList[nPos].nLevel > LEVEL_OFF) {
-        strcpy(szState, STR_ON);
+        SAPDB_strcpy(szState, STR_ON);
       } else {
-        strcpy(szState, STR_OFF);
+        SAPDB_strcpy(szState, STR_OFF);
       } // end if
       if (oProtList[nPos].bLevel) {
         sprintf(szLevel, "%d", oProtList[nPos].nLevel);
       } else {
-        strcpy(szLevel, STR_NOLEVEL);
+        SAPDB_strcpy(szLevel, STR_NOLEVEL);
       } // end if
 
       sprintf (pCurrent, "%-*s%s%-*s%s%-*s%s%s%s%s%s", COL_NAME_L,  oProtList[nPos].szName, VALUE_SEPSTRING_CN00,
@@ -546,7 +546,7 @@ tcn00_Error cn84RestoreTrace
   cn51DBMConfigValue::endBuffering(szDbName);
 
   // build command
-  strcpy(szCommand, SQL_DIAGNOSE_VTRACE);
+  SAPDB_strcpy(szCommand, SQL_DIAGNOSE_VTRACE);
   strcat(szCommand, " ");
   nPos = 0;
   while (oProtList[nPos].szSyntax != NULL) {
@@ -624,8 +624,8 @@ tcn00_Error cn84SaveTrace
         // analyze records
         aRecord = aTable.record();
         while (aTable.sqlRc() == cin01_db_ok) {
-          strcpy(szValue, aRecord.fieldAsChar(SQLF_VAL ));
-          strcpy(szDesc,  aRecord.fieldAsChar(SQLF_DESC));
+          SAPDB_strcpy(szValue, aRecord.fieldAsChar(SQLF_VAL ));
+          SAPDB_strcpy(szDesc,  aRecord.fieldAsChar(SQLF_DESC));
           nPos = 0;
           while (oProtList[nPos].szName != NULL) {
             if (oProtList[nPos].szSysDDValue != NULL) {
@@ -736,7 +736,7 @@ static tcn00_Error cn84_TraceSwitch
   // iterate trough tokens and build command
   if (nFuncReturn == OK_CN00) {
 
-    strcpy(szCommand, SQL_DIAGNOSE_VTRACE);
+    SAPDB_strcpy(szCommand, SQL_DIAGNOSE_VTRACE);
     strcat(szCommand, " ");
 
     nToken = 1;
@@ -865,7 +865,7 @@ static tcn00_Error cn84_TraceSwitchAll
   // iterate trough list entries and build command
   if (nFuncReturn == OK_CN00) {
 
-    strcpy(szCommand, SQL_DIAGNOSE_VTRACE);
+    SAPDB_strcpy(szCommand, SQL_DIAGNOSE_VTRACE);
     strcat(szCommand, " ");
 
     nPos = 0;
