@@ -34,9 +34,11 @@
  */
 #include "hcn85.h"
 
-#include "SAPDB/DBM/Srv/DBMSrv_KernelSession.hpp"
+#include "SAPDB/DBM/Srv/KernelAccess/DBMSrvKnl_Session.hpp"
 
 #include "hcn50.h"
+#include "hcn53.h"
+#include "hcn80.h"
 #include "hcn90.h"
 
 #define CONTINUE_TXT   "CONTINUE\n"
@@ -101,7 +103,7 @@ static tcn00_Error cn85_ReadUserByType
 static tcn00_Error cn85_KernelConnect (
       char                 * replyData,
       int                  * replyLen,
-      DBMSrv_KernelSession & oSession,
+      DBMSrvKnl_Session & oSession,
       const char           * szDatabase,
       const char           * szUsername,
       const char           * szPassword,
@@ -150,7 +152,7 @@ tcn00_Error cn85GetKernelSession
 {
   tcn00_Error     nFuncReturn = OK_CN00;
 
-  DBMSrv_KernelSession * pSession = new DBMSrv_KernelSession;
+  DBMSrvKnl_Session * pSession = new DBMSrvKnl_Session;
   if (pSession == NULL) {
     nFuncReturn = cn90AnswerIError(replyData, replyLen, ERR_MEM_CN00);
   } else {
@@ -263,12 +265,24 @@ static tcn00_Error cn85_DBExecute
 {
   tcn00_Error nFuncReturn = OK_CN00;
   bool        bTemporary  = true;
-  DBMSrv_KernelSession::ResultType oResultType = DBMSrv_KernelSession::resultUnknown;
+  DBMSrvKnl_Session::ResultType oResultType = DBMSrvKnl_Session::resultUnknown;
 
   char        szToken   [PARAMETER_MAXLEN_CN90];
   bool        bUserTypeFound = false;
   cn50DBMUser oUser(vcontrol->dbname);
 
+  // DBA history begin
+  const char *     pAction = "";
+  bool             bVerify     = false;
+  tsp00_Timestampc szTimeStamp;
+  if (cn80CheckVerify(command->args, pAction)) {
+    bVerify = true;
+    cn90Timestamp(szTimeStamp);
+    tcn53_DBAAction::start(vcontrol->dbname, szTimeStamp, pAction, DBA_OBJ_DB_CN53);
+    tcn53_DBAAction::writeDetail(DBADTL_CDBCMD_CN53, command->args);
+  } // end if
+  // DBA history end
+
   cn90GetToken(command->args, szToken, 1, PARAMETER_MAXLEN_CN90);
 
   nFuncReturn = cn85_ReadUserByType(command->args, replyData, replyLen,bUserTypeFound, oUser);
@@ -296,24 +310,24 @@ static tcn00_Error cn85_DBExecute
       nFuncReturn = cn90AnswerMessage(replyData, replyLen, vcontrol->pKernelSession->LastMessage());
     } else {
       switch (oResultType) {
-        case DBMSrv_KernelSession::resultNothing:
+        case DBMSrvKnl_Session::resultNothing:
           cn90AnswerOK(replyData, replyLen, NULL);
           break;
-        case DBMSrv_KernelSession::resultMessage:
+        case DBMSrvKnl_Session::resultMessage:
           {
             SAPDBErr_MessageList oDBM(DBMSrv_ExtendedError(OK_CN00), cn00_1_ErrTxt(OK_CN00_1), 0);
             oDBM.AppendNewMessage(vcontrol->pKernelSession->KernelMessage());
             cn90AnswerMessage(replyData, replyLen, oDBM);
           } // end block
           break;
-        case DBMSrv_KernelSession::resultSQL:
+        case DBMSrvKnl_Session::resultSQL:
           if (bNice) {
             nFuncReturn = cn85DBFetchNice(vcontrol, command, replyData, replyLen, replyLenMax);
           } else {
             nFuncReturn = cn85DBFetch(vcontrol, command, replyData, replyLen, replyLenMax);
           } // end if
           break;
-        case DBMSrv_KernelSession::resultUtility:
+        case DBMSrvKnl_Session::resultUtility:
           {
             DBMSrv_SQLRecord oRecord(*(vcontrol->pKernelSession));
             tsp00_Int4 i = 0;
@@ -339,6 +353,13 @@ static tcn00_Error cn85_DBExecute
     cn85FreeKernelSession ( vcontrol );
   } // end if
 
+  // DBA history begin
+  if (bVerify) {
+    tcn53_DBAAction::writeDetail(DBADTL_CDBRESULT_CN53, replyData);
+    tcn53_DBAAction::finish(cn90Timestamp(szTimeStamp), (nFuncReturn == OK_CN00));
+  } // end if
+  // DBA history end
+
   return nFuncReturn;
 } // end cn85_DBExecute
 
@@ -447,7 +468,7 @@ tcn00_Error cn85DBFetch
 
     char * pCurrent  = replyData + strlen(replyData);
     char * pContinue = pCurrent;
-    strcpy(pCurrent, CONTINUE_TXT);
+    SAPDB_strcpy(pCurrent, CONTINUE_TXT);
     pCurrent  = pCurrent + strlen(pCurrent);
     char * pRecord  = pCurrent;
 
@@ -524,7 +545,7 @@ tcn00_Error cn85DBFetchNice
 
     char * pCurrent  = replyData + strlen(replyData);
     char * pContinue = pCurrent;
-    strcpy(pCurrent, CONTINUE_TXT);
+    SAPDB_strcpy(pCurrent, CONTINUE_TXT);
     pCurrent  = pCurrent + strlen(pCurrent);
 
 
@@ -560,10 +581,10 @@ tcn00_Error cn85DBFetchNice
 void cn85FreeKernelSession
     ( VControlDataT * vcontrol )
 {
-  DBMSrv_KernelSession * pSession;
+  DBMSrvKnl_Session * pSession;
 
   if (vcontrol != NULL) {
-    pSession =  (DBMSrv_KernelSession *) vcontrol->pKernelSession;
+    pSession =  (DBMSrvKnl_Session *) vcontrol->pKernelSession;
     vcontrol->pKernelSession = NULL;
   } // end if
 
