@@ -237,7 +237,10 @@ char  * cn90StringPtoC
   for (i = 0; (i < (nLength - 1)) && (pSourceP[i] != CHAR_STRINGTERM_CN90); i++);
 
   if (pSourceP[i] == CHAR_STRINGTERM_CN90) {
-    strcpy(szDestC, pSourceP);
+    SAPDB_memmove(
+        szDestC,
+        pSourceP,
+        strlen(pSourceP)+1);
   } else {
     szDestC[nLength] = CHAR_STRINGTERM_CN90;
     for (i = nLength - 1; i >= 0; i--) {
@@ -437,52 +440,57 @@ bool cn90GetToken
 {
   long               nToken      = 0;
   long               i           = 0;
-  long               j           = 0;
+  long               j           = 0; //used for wired return statement (if somebody can explain it or even better assure, that the wired properties are not used, please call me ... TiloH)
+  long               j1          = 0; //introduced for write access to szToken, we assure 0<=j1<nLength for all write operations
   bool            bOutside    = true;
   bool            bString     = false;
 
+    if(0<nLength)        //if there is space for a terminating zero in szToken, make sure szToken is zero terminated
+        szToken[nLength-1]='\0'; // all further write access is restricted to j1<nLength-1, so this zero is not overwritten
+
   /* is buffer valid */
   if (szBuffer != 0) {
-    /* loop until we've reached the end of buffer or the requested token is complete */
+    /* loop until we've reached the end of buffer or the requested token is complete, make sure terminating zero is not overwritten */
     for (i = 0; (szBuffer[i] != CHAR_STRINGTERM_CN90) && (nToken <= nRequest) && ((j < nLength) || (nLength == -1)); i++) {
       if (bOutside) {
-        /* if we're outside of a token look out for a non whitesapaces */
+        /* if we're outside of a token look out for a non white spaces */
         if (!isspace((unsigned char) szBuffer[i])) {
           bOutside = false;
           nToken++;
           /* copy the first char of the token */
-          if ((szToken != NULL) && (nToken == nRequest)) szToken[j++] = szBuffer[i];
+          if ((szToken != NULL) && (nToken == nRequest)) { j++; if(j1<nLength-1) szToken[j1++] = szBuffer[i]; }
           /* set a flag for string handling */
           bString = szBuffer[i] == CHAR_SYNTAXSTRING_CN90;
         } /* end if */
       } else {
         if (bString) {
           /* copy the char */
-          if ((szToken != NULL) && (nToken == nRequest)) szToken[j++] = szBuffer[i];
+          if ((szToken != NULL) && (nToken == nRequest)) { j++; if(j1<nLength-1) szToken[j1++] = szBuffer[i]; }
           /* look out for string terminator */
           if (szBuffer[i] == CHAR_SYNTAXSTRING_CN90 && isspace(szBuffer[i+1])) {
             bOutside = true;
-            /* attach the null char */
-            if ((szToken != NULL) && (nToken == nRequest)) szToken[j++] = CHAR_STRINGTERM_CN90;
+            /* attach the null char, if there is space */
+            if ((szToken != NULL) && (nToken == nRequest)) { j++; if(j1<nLength-1) szToken[j1++] = CHAR_STRINGTERM_CN90; }
           } /* end if */
         } else {
           /* look out for whitespaces */
           if (isspace((unsigned char) szBuffer[i])) {
             bOutside = true;
             /* attach the null char */
-            if ((szToken != NULL) && (nToken == nRequest)) szToken[j++] = CHAR_STRINGTERM_CN90;
+            if ((szToken != NULL) && (nToken == nRequest)) { j++; if(j1<nLength-1) szToken[j1++] = CHAR_STRINGTERM_CN90; }
           } else {
             /* copy the current char */
-            if ((szToken != NULL) && (nToken == nRequest)) szToken[j++] = szBuffer[i];
+            if ((szToken != NULL) && (nToken == nRequest)) { j++; if(j1<nLength-1) szToken[j1++] = szBuffer[i]; }
             bString = szBuffer[i] == CHAR_SYNTAXSTRING_CN90;
           } /* end if */
         } /* end if */
       } /* end if */
     } /* end for */
-  } /* end for */
 
-  /* if the token was at the end of the buffer attach a null char */
-  if (szToken != NULL && szBuffer[i] == CHAR_STRINGTERM_CN90) szToken[j] = CHAR_STRINGTERM_CN90;
+    /* if the token was at the end of the buffer and there is space, attach a null char */
+    if (szToken != NULL && szBuffer[i] == CHAR_STRINGTERM_CN90) { if(j1<nLength-1) szToken[j1++] = CHAR_STRINGTERM_CN90;}
+
+  } /* end if */
 
   return ((nToken >= nRequest) && ((j <= nLength) ||(nLength == -1)));
 } /* end cn90GetToken */
@@ -591,7 +599,7 @@ bool cn90AnalyzeString
 
   if (pTemp) {
 
-    strcpy(pTemp, szBuffer);
+    SAPDB_strcpy(pTemp, szBuffer);
     cn90Strip(pTemp);
     nLength = (long)strlen(pTemp);
 
@@ -963,7 +971,7 @@ void cn90Trace
   void          * pBuffer = NULL;
 
   cn90GetWrkPath(szDir);
-  strcpy(szFilename, szDir);
+  SAPDB_strcpy(szFilename, szDir);
   strcat(szFilename, TRACE_FILE_CN90);
 
   time(&aTime);
@@ -1054,7 +1062,7 @@ char * cn90CalculateUncrypt
   } // end if
 
   pwClearc.p2c(pwClear);
-  strcpy(passwd, pwClearc);
+  SAPDB_strcpy(passwd, pwClearc);
 
   return passwd;
 } // end cn90CalculateUncrypt
@@ -1070,7 +1078,7 @@ char * cn90CalculateUncrypt
 {
   tsp00_CryptPw pwTemp;
 
-  memcpy(&pwTemp[0], &pwCrypted[0], sizeof(tsp00_CryptPw));
+  SAPDB_memcpy(&pwTemp[0], &pwCrypted[0], sizeof(tsp00_CryptPw));
 
   return cn90CalculateUncrypt(pwTemp, passwd, bForDatabase);
 } // end cn90CalculateUncrypt
@@ -1124,7 +1132,7 @@ char * cn90GetCfgPath
   tsp00_Pathc      szLokDir;
 
   sqlGetIndependentConfigPath(szLokDir, TERM_WITH_DELIMITER_EO01, &aRteError);
-  strcpy(szDir, szLokDir);
+  SAPDB_strcpy(szDir, szLokDir);
 
   return (char *) szDir;
 } /* end cn90GetCfgPath */
@@ -1139,7 +1147,7 @@ char * cn90GetWrkPath
   tsp00_Pathc     szLokDir;
 
   sqlGetIndependentWrkPath(szLokDir, TERM_WITH_DELIMITER_EO01, &aRteError);
-  strcpy(szDir, szLokDir);
+  SAPDB_strcpy(szDir, szLokDir);
 
   return (char *) szDir;
 } /* end cn90GetWrkPath */
@@ -1155,7 +1163,7 @@ char * cn90GetVersionEnvPath
   tsp00_Pathc      szLokDir;
 
   sqlGetDbrootEnvPath(szLokDir, TERM_WITH_DELIMITER_EO01, &aRteError);
-  strcpy(szDir, szLokDir);
+  SAPDB_strcpy(szDir, szLokDir);
 
   return (char *) szDir;
 } /* end cn90GetVersionEnvPath */
@@ -1170,7 +1178,7 @@ char * cn90GetVersionBinPath
   tsp00_Pathc     szLokDir;
 
   sqlGetDbrootBinPath(szLokDir, TERM_WITH_DELIMITER_EO01, &aRteError);
-  strcpy(szDir, szLokDir);
+  SAPDB_strcpy(szDir, szLokDir);
 
   return (char *) szDir;
 } /* end cn90GetVersionBinPath */
@@ -1434,7 +1442,7 @@ void cn90CryptToCryptString
 {
   tsp00_CryptName stTemp;
 
-  memcpy (&stTemp[0], &stCrypt[0], sizeof(stTemp));
+  SAPDB_memcpy (&stTemp[0], &stCrypt[0], sizeof(stTemp));
 
   cn90CryptToCryptString(stTemp, szCryptString);
 } // end cn90CryptToString
