@@ -1,5 +1,5 @@
 /*!
-  @file           gen50.h
+  @file           Copy of gen50.h
   @author         JoergM
   @special area   Kernel LZU UNIX
   @brief          Kernel interne Strukturen
@@ -58,7 +58,7 @@
    --- Kernel threads stack sizes 
    These sizes are used as input for sqlbeginthread() calls
  */
-#if defined (HP_IA64)
+#if defined (HP_IA64) || (defined(LINUX) && defined(IA64))
 #define UKT_BASE_STACK_SIZE             ( 512 * 1024)
 #define UKT_TEMP_STACK_SIZE             ( 512 * 1024)
 #define KT_WORKER_STACK_SIZE            ( 512 * 1024)
@@ -507,7 +507,7 @@ struct TASK_TYPE
 
     SAPDB_Bool                      TimeCollectionEnabled;     /* actual time enabled flag (copy of kgs->fTimeMeasure)*/
     SAPDB_Bool                      MoveTimeCollectionEnabled; /* actual move time enabled flag (copy of   XPARAM(uLoadBalancingCheck))*/
-    SAPDB_Bool                      SuppressTimeCollection;    /* flag to use for fast reschedule to disable time collection */
+    SAPDB_Bool                      YieldTask;                 /* flag to use task yield (reschedule) to disable time collection */
 
     teo00_Longint                   cputim ;
     teo00_Longint                   dirio ;
@@ -592,6 +592,7 @@ struct TASK_TYPE
 
     SAPDB_Bool                      fTaskIsWaitingForExclusiveAccess; /* ... to a RWRegion */
     struct TASK_TYPE               *nextTaskInWaitQueue ;
+    void                           *RWRegion;
     /* no filler */
 }; /* TASK_TYPE */
 
@@ -827,7 +828,7 @@ typedef struct ten50_UKT_Control
     tsp00_PageAddr                      traceBuffer;
     SAPDB_Int4                          traceBufferSizeInPages;
 
-    SAPDB_UInt4                         taskSwitches;  /* PTS 1108715 */
+    SAPDB_UInt8                         taskSwitches;  /* PTS 1108715 */
 
     struct RTE_CoroutineContext *       removeTaskContext; /* used only if coroutines enabled */
 
@@ -984,7 +985,6 @@ struct KERNEL_GLOBAL_SECTION
     struct DEV_PCB0                 dev0 ;
 
     long                            seg_size ;
-    long                            kgs_shm_part_len ;
 
     unsigned long                   ulDBPageShift;
 
