@@ -46,6 +46,7 @@
 #include "geo03.h"
 #include "heo03.h"
 #include "geo40.h"
+#include "geo401.h"
 #include "heo46.h"
 #include "geo001.h"
 #include "geo57.h"
@@ -59,6 +60,8 @@
 #include "gen46.h"
 #endif
 #include "heo07.h"
+#include "RunTime/System/RTESys_MicroTime.h"
+#include "heo420.h"
 
 /*===========================================================================*
  *  DEFINES                                                                  *
@@ -79,13 +82,16 @@
 #define MAXCONNECTIONS_INL      (sql03_connect_pool.connectionCount)
 #endif
 
-
+#define SQL_PACKET_ALIGNMENT 8
 #define CHECK_PID_EN03 1
 
 /*===========================================================================*
  *  MACROS                                                                   *
  *===========================================================================*/
 
+#define  ALIGN_BUFFER(_val,_alignment)                                               \
+           ((((_val) - 1) / (_alignment) + 1 ) * (_alignment))
+
 /*===========================================================================*
  *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
  *===========================================================================*/
@@ -150,6 +156,7 @@ static ten03_connect_pool sql03_connect_pool = {
 
 static            tsp00_Bool fNI_Installed        = FALSE;
 
+
 /*===========================================================================*
  *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
  *===========================================================================*/
@@ -223,10 +230,141 @@ static  tsp01_CommErr_Enum en03_aPing( char *         pszServerNode,
                                        tsp00_Versionc szServerVersion,
                                        tsp00_ErrTextc errText );
 
+extern int sqlIsRunTimeComponent();
+
 /*===========================================================================*
  *  PUBLIC FUNCTIONS (CODE)                                                  *
  *===========================================================================*/
 
+externC tsp01_CommErr_Enum  SqlPingConnect ( char*                  szServerNode,    /* '\0' terminated node or saprouterstring */
+                                             char*                  szServerDB,      /* '\0' terminated dbname                  */
+                                             tsp00_Int4*            reference,      /* connection reference                    */
+                                             tsp00_ErrTextc         pErrText )       /* '\0' terminated error string            */
+  {
+  #undef  MF__
+  #define MF__ MOD__"SqlPingConnect"
+  tsp01_CommErr_Enum        commError;
+  int                       packetSize;
+  void*                     packetList;
+
+  DBGIN;
+
+  if ( !sql03_connect_pool.initialized )
+      sql03_init();
+
+  if (strlen (szServerDB) >= sizeof (SQL_DBNAMEC))
+  {
+      eo46BuildErrorStringRC ( pErrText, ERRMSG_COM_SERVERDB_NAME_TOO_LONG, 0 );
+      DBGOUT;
+      return commErrNotOk_esp01;
+  }
+
+  commError = (tsp01_CommErr_Enum)sql03_connect ( szServerNode,
+                                                  szServerDB,
+                                                  (int) srvPing_esp01,
+                                                  1,
+                                                  reference,
+                                                  &packetSize,
+                                                  &packetList,
+                                                  NULL,
+                                                  NULL,
+                                                  NULL,
+                                                  pErrText );
+
+
+  DBGOUT
+  return (commError) ;
+  }
+
+
+/*------------------------------*/
+
+
+externC tsp01_CommErr_Enum SqlDBPing ( tsp00_Int4                         reference,
+                                       tsp00_Uint4*                       sendBufferSize,
+                                       tsp00_Uint1*                       hops,
+                                       tsp00_DbNamec   VAR_ARRAY_REF      szServer,
+                                       tsp00_Versionc  VAR_ARRAY_REF      szServerVersion,
+                                       tsp00_ErrTextc                     pErrText )
+{
+    tsp01_CommErr_Enum              commState;
+    char*                           server;
+    char*                           serverVersion;
+    connection_info*                cip;
+    tsp00_Uint4                     sendLen;
+
+    /* --- get the pointer to the internal communication structure */
+    commState = (tsp01_CommErr_Enum)en03GetAndCheckConnectionInfo( reference, CHECK_PID_EN03, &cip, "SqlDBPing", pErrText );
+
+    if ( commState != commErrOk_esp01 )
+        return commErrNotOk_esp01;
+
+    if ( *sendBufferSize > cip->ci_max_request_size ) 
+        *sendBufferSize = cip->ci_max_request_size;
+
+    *sendBufferSize = ALIGN_BUFFER( *sendBufferSize, SQL_PACKET_ALIGNMENT );
+
+    sendLen =  eo420FillPingPacket ( (char*)cip->ci_packetData[0],
+                                     *sendBufferSize, *hops, "PingClient", 0, "!!! SEND DATA !!!" );
+
+    if ( 0 != sendLen )
+    {
+        commState = (tsp01_CommErr_Enum)sql03_request ( reference, cip->ci_packetData[0], sendLen, pErrText );
+
+        if ( commState == commErrOk_esp01 )
+        {
+            char*              pResPacket;
+            tsp00_Int4         resDataLen;
+
+            commState = (tsp01_CommErr_Enum)sql03_receive ( reference, (void**)&pResPacket, &resDataLen, pErrText );
+
+            if ( commState == commErrOk_esp01 )
+            {
+                if ( eo420ExtractPingPacket( pResPacket, resDataLen, hops, &server, &serverVersion ) )
+                {
+                    if ( server )
+                    {
+                        szServer[sizeof(tsp00_DbNamec)-1];
+                        strncpy ( szServer, server, sizeof(tsp00_DbNamec));
+                    }
+                    if ( serverVersion )
+                    {
+                        szServerVersion[sizeof(tsp00_Versionc)-1];
+                        strncpy ( szServerVersion, serverVersion, sizeof(tsp00_Versionc));
+                    }
+                }
+                else
+                {
+                    MSGD (( ERR_PROTOCOL_ERROR, "PING REPLY" ));
+                    eo46BuildErrorStringRC ( pErrText, ERRMSG_PROTOCOL_ERROR, 0 );
+                    commState = commErrNotOk_esp01;
+                }
+            }
+        }
+    }
+    else
+    {
+        MSGD (( ERR_PROTOCOL_ERROR, "PING REPLY" ));
+        eo46BuildErrorStringRC ( pErrText, ERRMSG_PROTOCOL_ERROR, 0 );
+        commState = commErrNotOk_esp01;
+    }
+
+    return commState;
+}
+
+/*------------------------------*/
+
+externC void SqlPingRelease (tsp00_Int4 reference)
+  {
+  #undef  MF__
+  #define MF__ MOD__"SqlPingRelease"
+  tsp00_ErrTextc  pErrText;
+
+  sql03_release ( reference, pErrText );
+
+  return;
+  }
+
 
 
 /*===============================================================================*/
@@ -264,7 +402,7 @@ SqlAPing ( char                  *pszServerNode,     /* '\0' terminated node or
                 commError = commErrNotOk_esp01;
             }
             else {
-                commError = eo03NiPing( pszServerNode, szServerVersion, pErrText );
+                commError = eo03NiXserverPing( pszServerNode, szServerVersion, pErrText );
             }
         }
         else
@@ -542,10 +680,9 @@ void    sql03_xconnect (
     #define MF__ MOD__"sql03_init"
 static  void    sql03_init ()
 {
-    int             idx;
+    int                 idx;
     connection_info     *cip;
 
-
     DBGIN;
 
     /*
@@ -678,10 +815,10 @@ static int en03GetAndCheckConnectionInfo(tsp00_Int4     reference,
      *  The application MUST NOT use a connection in a new process.
      *
      * PTS 1109225 MaO
-     * The PID check is decrepitly in particular for Linux
+     * The PID check is decrepitly in particular for Linux with Linuxthread
      *
      */
-#if !defined(LINUX) && !defined(NMP)
+#if !defined(LINUX) || defined(HAS_NPTL)
     if ( CheckPid == CHECK_PID_EN03 && (*ConnectionInfo)->ci_my_pid != getpid() )
     {
         en42FillErrText( pErrText,
@@ -694,7 +831,7 @@ static int en03GetAndCheckConnectionInfo(tsp00_Int4     reference,
                   CallingFktName, ERRMSG_COM_APPLIC_FORKED))
         return ( commErrNotOk_esp01 );
     }
-#endif /* LINUX || NMP */
+#endif /* !LINUX || HAS_NPTL */
     return(commErrOk_esp01);
 }
 
@@ -742,9 +879,9 @@ static  int     sql03_connect (char            * servernode,
     {
         /*
          * PTS 1109761 BV
-         * Don't check PID for Linux/NMP because threads have different pids.
+         * Don't check PID for Linux with Linuxthread because threads have different pids.
          */
-#if !defined(LINUX) && !defined(NMP)
+#if !defined(LINUX) || defined(HAS_NPTL)
         if ( sql03_pid ) /* The first time this is zero. */
         {
             DBG1 (( MF__,"application forked \n" ))
@@ -961,9 +1098,9 @@ static  int     sql03_release (
      *  It might still be in use by the parent process.
      *
      *  PTS 1109761 BV
-     *  Don't check PID for Linux/NMP because threads have different pids.
+     *  Don't check PID for Linux with Linuxthread because threads have different pids.
      */
-#if !defined(LINUX) && !defined(NMP)
+#if !defined(LINUX) || defined(HAS_NPTL)
     pid_changed = ( cip->ci_my_pid != getpid() );
 #endif
 
@@ -1030,9 +1167,9 @@ static  int     sql03_release (
 
     #undef MF__
     #define MF__ MOD__"sql03_request"
-static  int     sql03_request (tsp00_Int4           reference,
+static  int     sql03_request (tsp00_Int4         reference,
                                void             * packet,
-                               tsp00_Int4           length,
+                               tsp00_Int4         length,
                                tsp00_ErrTextc     pErrText )
 {
     int              rc;
@@ -1842,6 +1979,8 @@ static  void    sql03_set_signals ( )
             (void) signal ( SIGPIPE , sql03_oldsigpipe );
         }
 
+  if ( !sqlIsRunTimeComponent() )
+  {
     old_fetched_signal_func = sql03_fetched_sigint;
     sql03_fetched_sigint = signal ( SIGINT  , sql03_catch_signal );
     DBG1 (( MF__,"old SIGINT  0x%08lx \n",
@@ -1880,6 +2019,7 @@ static  void    sql03_set_signals ( )
          */
         sql03_oldsigterm = old_fetched_signal_func;
     }
+  }
 }
 
 /*==========================================================================*/
@@ -1900,6 +2040,8 @@ static  void    sql03_reset_signals ( )
         (void) signal ( SIGPIPE  , sql03_oldsigpipe  );
         sql03_oldsigpipe = SIG_ERR;
     }
+  if ( !sqlIsRunTimeComponent() )
+  {
     if ( sql03_oldsighup  != SIG_ERR )
     {
         (void) signal ( SIGHUP  , sql03_oldsighup  );
@@ -1910,6 +2052,7 @@ static  void    sql03_reset_signals ( )
         (void) signal ( SIGTERM , sql03_oldsigterm );
         sql03_oldsigterm = SIG_ERR;
     }
+  }
 }
 
 /*==========================================================================*/
