@@ -30,6 +30,8 @@
 
 /*PRETTY*/
 
+#include "RunTime/System/RTESys_MemoryBarrier.h"
+
 /* exported functions */
 #include "gen00.h"
 #include "heo00.h"
@@ -431,8 +433,8 @@ int                 count)
                          "connection broken: semop (%d:%s)",
                          errno,
                          sqlerrs() );
-	    MSGD (( -11987, ERR_TYPE, "CONNECT ", "semop (receive %d) error: %s \n",
-						cip->ci_my_semid , sqlerrs() ))
+	    MSGD (( -11987, ERR_TYPE, "CONNECT ", "receive sem %d : %s\n",
+						cip->ci_my_semid , &pErrText[0] ))
 	    cip->ci_packet_cnt = savpktcnt ;
 	    return ( commErrNotOk_esp01 );
     }
@@ -543,6 +545,7 @@ int                 count)
 		    union semun     arg ;
 	        DBG1 (( MF__,"free shm \n" ))
 	        comseg->cs_client_state = commErrNotOk_esp01 ;
+            RTESys_WriteMemoryBarrier();
 	        comseg->cs_client_flag  = 1 ;
 		    arg.val = 1 ;
 		    RTE_save_semctl ( comseg->cs_server_semid, 0 , SETVAL , SEMCTL_SETVAL(arg) );
@@ -605,6 +608,7 @@ int                 count)
 		    union semun     arg ;
 	        DBG1 (( MF__,"free shm \n" ))
 	        comseg->cs_client_state = SP1CE_RELEASED ;
+            RTESys_WriteMemoryBarrier();
 	        comseg->cs_client_flag  = 1 ;
 		    arg.val = 1 ;
 		    RTE_save_semctl ( comseg->cs_server_semid, 0 , SETVAL , SEMCTL_SETVAL(arg) );
@@ -770,6 +774,7 @@ tsp00_ErrTextc VAR_ARRAY_REF pErrText )
     /*
      *  Flag the comseg as 'filled in by application'.
      */
+    RTESys_WriteMemoryBarrier();
     comseg->cs_client_flag = 1 ;
     /*
      *  If connected via big-comseg, set the common flag.
@@ -913,8 +918,8 @@ restart_after_interrupt:
 	    }
 
         en42FillErrText(pErrText, "connection broken semop (%d:%s)", errno, sqlerrs() );
-	    MSGD (( -11987, ERR_TYPE, "COMMUNIC", "semop (receive %d) error: %s \n",
-						cip->ci_my_semid , sqlerrs() ))
+	    MSGD (( -11987, ERR_TYPE, "COMMUNIC", "checked receive sem %d : %s \n",
+						cip->ci_my_semid , &pErrText[0] ))
 	    return ( commErrNotOk_esp01 );
     }
 
