@@ -30,6 +30,8 @@
 
 #define MOD__ "ven53c :"
 
+#include "RunTime/System/RTESys_MemoryBarrier.h"
+
 #include "gen00.h"
 #include "heo00.h"
 #include "hen53.h"
@@ -39,6 +41,8 @@
 #include "gen003.h"
 #include "geo67.h"
 #include "RunTime/System/RTESys_MicroTime.h"
+#include "geo002.h"
+#include "gen45.h"
 
 /* imported functions */
 
@@ -52,6 +56,11 @@ extern  void              sql32_unlock_comseg (connection_info *cip);
 extern  void              sql46_dump_packet (rte_header * header );
 #endif
 
+static void               HandlePing( tsp00_TaskId          pid );
+
+
+extern  tsp00_DbNamec                   en81_dbname; 
+
 /*
  * ===========================================================================
  */
@@ -105,23 +114,36 @@ ten50_UKT_Control *this_ukt = THIS_UKT_CTRL;
         en71AlterTaskCnt( this_ukt->curr_task, ADD_ACTIVE_COUNTER );
     }
 
+    RTESys_ReadMemoryBarrier();
+
     *webuser = (cip->ci_service == 4);
     *max_data_len = cip->ci_max_data_size ;
     *returncode = ( cip->ci_state == CON_ESTABLISHED ? commErrOk_esp01 : commErrNotOk_esp01 );
 
     if ( *returncode == commErrOk_esp01 )
     {
-        if ( tcb->type == TT_EV )
+        if ( cip->ci_service == srvPing_esp01 )
         {
-            if ( !eo67ConnectEventTask(pid) )
-            {
-                vabort(1);
-            }
+            HandlePing ( pid );
+
+            /* --- release connection */
+            *returncode = commErrNotOk_esp01;
+            vrelease ( pid );
         }
         else
         {
+            if ( tcb->type == TT_EV )
+            {
+                if ( !eo67ConnectEventTask(pid) )
+                {
+                    vabort(1);
+                }
+            }
+            else
+            {
                 /* PTS 1004575 */
-            eo67InsertSessionEvent(true, pid);
+                eo67InsertSessionEvent(true, pid);
+            }
         }
     }
     DBG1 (( MF__,"leaving state %d datasize %d\n",
@@ -261,6 +283,8 @@ ten50_UKT_Control *this_ukt = THIS_UKT_CTRL;
      *************************************************
      */
 
+    RTESys_ReadMemoryBarrier();
+
     tcb->wait_time = 0 ;
 
     en71TempMoveLock ( tcb, 1, false );  /* remove temporary move lock */
@@ -532,6 +556,34 @@ tsp01_CommErr		*returncode )
     cip->BytesSend += header->rh_max_send_len;
 
     *returncode = (tsp01_CommErr) e75_wake_peer ( cip );
+    if ( *returncode != SP1CE_OK )
+    {
+        int logLength ;
+        char logbuf[512];
+    	struct tm timeBuffer;
+	    struct tm *pTime = localtime_r( &cip->ci_connect_time, &timeBuffer );
+        if ( pTime )
+        {
+            (void) sp77sprintf ( logbuf , sizeof(logbuf), "Client died to early? (vreply) Connected to T%d since %02d:%02d:%02d",
+                tcb->index, pTime->tm_hour, pTime->tm_min, pTime->tm_sec );
+        }
+        else
+        {
+            (void) sp77sprintf ( logbuf , sizeof(logbuf), "Client died to early? Connected to T%d since ?%ld?",
+                tcb->index, (long)cip->ci_connect_time );
+        }
+        logLength = strlen ( logbuf );
+        if ( cip->ci_peer_node[0] )
+		    (void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " remote %s" , cip->ci_peer_node );
+        else
+		    (void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " local" );
+        logLength = strlen ( logbuf );
+        if ( cip->ci_remote_pid[0] )
+		    (void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " remote_pid %s" , cip->ci_remote_pid );
+        logLength = strlen ( logbuf );
+		(void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " peer_pid %ld" , (long) cip->ci_peer_pid );
+    	MSGD (( ERR_DISTRIB_WAKE_SEMOP, logbuf ));
+    }
 
     if ( KGS->fTimeMeasure )
     {
@@ -565,32 +617,42 @@ tsp01_CommErr		*returncode )
  * ==================================================================
  */
 void	vgetapplpid( PROCESS_ID           tid ,
-                     tsp00_NodeId           appl_node ,
-                     tsp00_Int4             *appl_pid ,
-                     BOOLEAN              *ok )
- 
-                     
+                     tsp00_NodeId         appl_node ,
+                     tsp00_Int4          *appl_pid ,
+                     BOOLEAN             *ok )
 {
 #undef  MF__
 #define MF__ MOD__"vgetapplpid"
 
-char peer_pid_buffer[32];
-extern	int			         sql45_has_remote_pid ();
-extern	char				*sql45_peer_pid ();
-extern	char                *sql45_peer_node ();
+    if ( ( tid > 0 ) && ( ((SAPDB_Int4)tid) <= KGS->ulNumOfTasks ) )
+    {
+        struct TASK_TYPE *tcb = KGS->pFirstTaskCtrl + ( tid - 1 ) ;
 
-if ( (*ok = tid <= KGS->ulNumOfTasks) )
-  { struct TASK_TYPE *tcb = KGS->pFirstTaskCtrl + ( tid - 1 ) ;
-    *appl_pid = atoi ( sql45_peer_pid ( tcb, peer_pid_buffer ) ) ;
-    eo46CtoP ( appl_node, 
-                 sql45_has_remote_pid ( tcb ) 
-               ? sql45_peer_node ( tcb ) : (char *)(KGS->servernode) ,
-                 sizeof(tsp00_NodeId) );
-   }
+        if ( 0 != tcb->connection )
+        {
+            char peer_pid_buffer[32];
 
-  DBG3 (( MF__, "appl_node: %s, appl_pid %d\n", appl_node, appl_pid ));
+            *appl_pid = atoi ( sql45_peer_pid ( tcb, peer_pid_buffer ) ) ;
 
-  return ;
+            eo46CtoP ( appl_node
+                     , sql45_has_remote_pid ( tcb ) ? sql45_peer_node ( tcb ) : (char *)(KGS->servernode)
+                     , sizeof(tsp00_NodeId) );
+            *ok = true;
+        }
+        else
+        {
+            *appl_pid = -1;
+            eo46CtoP ( appl_node
+                     , (char *)(KGS->servernode)
+                     , sizeof(tsp00_NodeId) );
+            *ok = false;
+        }
+    }
+    else
+    {
+        MSGD (( ERR_KERN_PARAM_PID_OUT_OF_RANGE, tid ));
+        vabort(1);
+    }
 }
 
 /*
@@ -602,5 +664,44 @@ externC	 tsp00_Int4  vconnectedusertasks ()
     return KGS->connectedUserTasks;
 }
 
+/*
+ * =====================================================================
+ */
+
+static void HandlePing( tsp00_TaskId          pid )
+{
+    tsp01_CommErr         commState = commErrOk_esp01;
+    void*                 rcvPacket;
+    tsp00_Int4            rcvPacketLen;
+    void*			      replyPacket;
+    tsp00_Int4		      replyPacketLen;
+
+    do
+    {
+        vreceive ( pid, 240, &rcvPacket, &rcvPacketLen, &replyPacket, &replyPacketLen, &commState );
+
+        if  ( commState == commErrOk_esp01 )
+        {
+           if ( eo420ExtractPingPacket( (char*)rcvPacket, rcvPacketLen, 0, 0, 0 ) )
+           {
+                replyPacketLen =  eo420FillPingPacket ( (char*) replyPacket,
+                                                         replyPacketLen,
+                                                         1,   /* hops greater than 0 means that the final destination was reached */
+                                                         en81_dbname,
+                                                         XPARAM(szKernelVersion), 0 );
+
+                if ( replyPacketLen )
+                    vreply( pid, replyPacket, replyPacketLen, &commState );
+                else
+                    commState = commErrNotOk_esp01;
+            }
+            else
+                commState = commErrNotOk_esp01;
+        }
+        else
+            commState = commErrNotOk_esp01;
+    } 
+    while( commState == commErrOk_esp01  );
+}
 
 
