@@ -1,5 +1,5 @@
 /*!
-  @file           ven55.c
+  @file           Copy of ven55.c
   @author         JoergM
   @brief          Kernel RunTime: Mutual Exclusion
   @see            
@@ -158,7 +158,6 @@ static  void    vbegexcl_one_ukp ( RTE_TaskId pid, REGION_ID  sid )
     if ( KGS->traceflag && *(KGS->traceflag) && l_curr_task->type == TT_TW )
         return;
     l_curr_task->state = TSK_VBEGEXCL;
-    l_curr_task->SuppressTimeCollection = true;
     l_curr_task->sid   = sid;
     l_curr_task->begexcl_count++;
     l_curr_task->ulBegExclCalls++;
@@ -189,14 +188,12 @@ static  void    vbegexcl_one_ukp ( RTE_TaskId pid, REGION_ID  sid )
 #endif
 
         DBG1 (( MF__,"before goto_disp T%d \n", l_curr_task->index ));
-        l_curr_task->SuppressTimeCollection = false;
         GOTO_DISP (&this_ukt);
         DBG1 (( MF__,"after  goto_disp T%d \n", l_curr_task->index ));
         scb->owner->reg_coll_cnt [ sid-1 ]++;
         scb->collision_count++;
     }
 
-    l_curr_task->SuppressTimeCollection = false;
     l_curr_task->state = TSK_RUNNING;
 
     DBGOUT_T(( l_curr_task->index ));
@@ -244,7 +241,6 @@ static  void    vbegexcl_n_ukps_loop ( RTE_TaskId pid, REGION_ID sid )
     if ( KGS->traceflag && *(KGS->traceflag) && l_curr_task->type == TT_TW )
         return;
     l_curr_task->state = TSK_VBEGEXCL;
-    l_curr_task->SuppressTimeCollection = true;
     l_curr_task->sid   = sid;
     l_curr_task->begexcl_count++;
     l_curr_task->ulBegExclCalls++;
@@ -274,7 +270,7 @@ static  void    vbegexcl_n_ukps_loop ( RTE_TaskId pid, REGION_ID sid )
                 }
 
             l_curr_task->qa.req_type = REQ_TEST_AND_SET ;
-            e71_FastEnquRex ( this_ukt, (tsp00_Bool)true );
+            e71_YieldTaskEnquRex ( this_ukt, (tsp00_Bool)true );
             DBG1 (( MF__,"before goto_disp T%d \n", l_curr_task->index));
             GOTO_DISP (&this_ukt);
             DBG1 (( MF__,"after  goto_disp T%d \n", l_curr_task->index));
@@ -285,7 +281,7 @@ static  void    vbegexcl_n_ukps_loop ( RTE_TaskId pid, REGION_ID sid )
         {
             CLEARLOCK ( scbs->sem_exclusive ) ;/* free the lock */
             l_curr_task->qa.req_type = REQ_EXCLUSIVE ;                      /* call the dispatcher */
-        	e71_FastEnquRex ( this_ukt, l_curr_task->prio_flag );           /* to give tasks with urgent needs a chance */
+        	e71_YieldTaskEnquRex ( this_ukt, l_curr_task->prio_flag );           /* to give tasks with urgent needs a chance */
             GOTO_DISP (&this_ukt);
         }                 /* else leave the loop */  
     } while ( region_reserved && reserved_cnt > 0 ) ;  /* fh: do-while end */
@@ -313,7 +309,6 @@ static  void    vbegexcl_n_ukps_loop ( RTE_TaskId pid, REGION_ID sid )
         scb->wait_count++;
         this_ukt->NumRegWait++ ;
         DBG1 (( MF__,"before goto_disp T%d", l_curr_task->index ));
-        l_curr_task->SuppressTimeCollection = false;
         GOTO_DISP (&this_ukt);
         DBG1 (( MF__,"after goto_disp T%d", l_curr_task->index ));
     }
@@ -324,7 +319,6 @@ static  void    vbegexcl_n_ukps_loop ( RTE_TaskId pid, REGION_ID sid )
         scb->collision_count += num_coll ;
         l_curr_task->reg_coll_cnt [ sid-1 ] += num_coll ;
     }
-    l_curr_task->SuppressTimeCollection = false;
     l_curr_task->state = TSK_RUNNING;
 
     DBGOUT_T (( l_curr_task->index ));
@@ -367,7 +361,6 @@ static  void    vbegexcl_n_ukps_noloop ( RTE_TaskId pid, REGION_ID sid )
     if ( KGS->traceflag && *(KGS->traceflag) && l_curr_task->type == TT_TW )
         return;
     l_curr_task->state = TSK_VBEGEXCL;
-    l_curr_task->SuppressTimeCollection = true;
     l_curr_task->sid   = sid;
     l_curr_task->begexcl_count++;
     l_curr_task->ulBegExclCalls++;
@@ -394,7 +387,7 @@ static  void    vbegexcl_n_ukps_noloop ( RTE_TaskId pid, REGION_ID sid )
                 vabort ( WRITE_CORE );
             }
         l_curr_task->qa.req_type = REQ_TEST_AND_SET ;
-        e71_FastEnquRex ( this_ukt, (tsp00_Bool)true );
+        e71_YieldTaskEnquRex ( this_ukt, (tsp00_Bool)true );
         DBG1 (( MF__,"before goto_disp T%d \n", l_curr_task->index));
         GOTO_DISP (&this_ukt);
         DBG1 (( MF__,"after  goto_disp T%d \n", l_curr_task->index));
@@ -422,7 +415,6 @@ static  void    vbegexcl_n_ukps_noloop ( RTE_TaskId pid, REGION_ID sid )
         scb->wait_count++ ;
         this_ukt->NumRegWait++ ;
         DBG1 (( MF__,"before goto_disp T%d", l_curr_task->index ));
-        l_curr_task->SuppressTimeCollection = false;
         GOTO_DISP (&this_ukt);
         DBG1 (( MF__,"after goto_disp T%d", l_curr_task->index ));
 
@@ -430,7 +422,6 @@ static  void    vbegexcl_n_ukps_noloop ( RTE_TaskId pid, REGION_ID sid )
         scb->owner->reg_coll_cnt [ sid-1 ]++;
     }
     scb->begexcl_tas_count += MAX_INTERLOCK_LOOPS - tas_count ;
-    l_curr_task->SuppressTimeCollection = false;
     l_curr_task->state = TSK_RUNNING;
 
     DBGOUT_T (( l_curr_task->index ));
@@ -452,7 +443,7 @@ REGION_ID sid )
     struct TASK_TYPE               * l_curr_task;
     long                           tas_count = MAX_INTERLOCK_LOOPS;
     long loop_region_coll = XPARAM(ulRegionCollisionLoop) ;
-    long reserved_cnt = loop_region_coll, num_coll ; /* only needed for calculation of the parameter for e71_FastEnquRex() */
+    long reserved_cnt = loop_region_coll, num_coll ; /* only needed for calculation of the parameter for e71_YieldTaskEnquRex() */
 
 	DBGIN;
 
@@ -477,7 +468,6 @@ REGION_ID sid )
     if ( KGS->traceflag && *(KGS->traceflag) && l_curr_task->type == TT_TW )
         return;
     l_curr_task->state = TSK_VBEGEXCL;
-    l_curr_task->SuppressTimeCollection = true;
     l_curr_task->sid   = sid;
     l_curr_task->begexcl_count++;
     l_curr_task->ulBegExclCalls++;
@@ -494,8 +484,8 @@ REGION_ID sid )
         {
             --reserved_cnt ;
   	        l_curr_task->qa.req_type = REQ_EXCLUSIVE ;  /* call the dispatcher before trying again */
-            e71_FastEnquRex ( this_ukt, reserved_cnt > 0 ||
-                                      l_curr_task->excl_schachtel > 1 ) ;
+            e71_YieldTaskEnquRex ( this_ukt, reserved_cnt > 0 ||
+                                   l_curr_task->excl_schachtel > 1 ) ;
             GOTO_DISP (&this_ukt);
         }                 /* else leave the loop */  
         else
@@ -516,7 +506,7 @@ REGION_ID sid )
                         vabort ( WRITE_CORE );
                     }
                 l_curr_task->qa.req_type = REQ_TEST_AND_SET ;
-                e71_FastEnquRex ( this_ukt, (tsp00_Bool)true );
+                e71_YieldTaskEnquRex ( this_ukt, (tsp00_Bool)true );
                 DBG1 (( MF__,"before goto_disp T%d \n", l_curr_task->index));
                 GOTO_DISP (&this_ukt);
                 DBG1 (( MF__,"after  goto_disp T%d \n", l_curr_task->index));
@@ -538,9 +528,8 @@ REGION_ID sid )
                 CLEARLOCK ( scbs->sem_exclusive );
                 --reserved_cnt ;
       	        l_curr_task->qa.req_type = REQ_EXCLUSIVE ;
-                e71_FastEnquRex ( this_ukt, reserved_cnt > 0 ||
-                                            l_curr_task->excl_schachtel > 1 ) ;
-                l_curr_task->SuppressTimeCollection = false;
+                e71_YieldTaskEnquRex ( this_ukt, reserved_cnt > 0 ||
+                                       l_curr_task->excl_schachtel > 1 ) ;
                 GOTO_DISP (&this_ukt);
             }                 /* else leave the loop */  
         }
@@ -556,7 +545,6 @@ REGION_ID sid )
         l_curr_task->reg_coll_cnt [ sid-1 ] += num_coll ;
     }
     scb->begexcl_tas_count += MAX_INTERLOCK_LOOPS - tas_count ;
-    l_curr_task->SuppressTimeCollection = false;
     l_curr_task->state = TSK_RUNNING;
     DBGOUT_T (( l_curr_task->index ));
 }
@@ -601,7 +589,6 @@ static  void    vbegexcl_dirty_read_loop ( RTE_TaskId pid, REGION_ID sid )
     if ( KGS->traceflag && *(KGS->traceflag) && l_curr_task->type == TT_TW )
         return;
     l_curr_task->state = TSK_VBEGEXCL;
-    l_curr_task->SuppressTimeCollection = true;
     l_curr_task->sid   = sid;
     l_curr_task->begexcl_count++;
     l_curr_task->ulBegExclCalls++;
@@ -621,8 +608,8 @@ static  void    vbegexcl_dirty_read_loop ( RTE_TaskId pid, REGION_ID sid )
         {
             --reserved_cnt ;
             l_curr_task->qa.req_type = REQ_EXCLUSIVE ;
-            e71_FastEnquRex ( this_ukt, reserved_cnt > 0 ||
-                                      l_curr_task->excl_schachtel > 1 ) ;
+            e71_YieldTaskEnquRex ( this_ukt, reserved_cnt > 0 ||
+                                   l_curr_task->excl_schachtel > 1 ) ;
             GOTO_DISP (&this_ukt);
         }                 /* else leave the loop */  
         else
@@ -643,7 +630,7 @@ static  void    vbegexcl_dirty_read_loop ( RTE_TaskId pid, REGION_ID sid )
                         vabort ( WRITE_CORE );
                     }
                 l_curr_task->qa.req_type = REQ_TEST_AND_SET ;
-                e71_FastEnquRex ( this_ukt, (tsp00_Bool)true );
+                e71_YieldTaskEnquRex ( this_ukt, (tsp00_Bool)true );
                 DBG1 (( MF__,"before goto_disp T%d \n", l_curr_task->index));
                 GOTO_DISP (&this_ukt);
                 DBG1 (( MF__,"after  goto_disp T%d \n", l_curr_task->index));
@@ -656,8 +643,8 @@ static  void    vbegexcl_dirty_read_loop ( RTE_TaskId pid, REGION_ID sid )
                     CLEARLOCK ( scbs->sem_exclusive );
                     --reserved_cnt ;
   	                l_curr_task->qa.req_type = REQ_EXCLUSIVE ;
-                    e71_FastEnquRex ( this_ukt, reserved_cnt > 0 ||
-                                            l_curr_task->excl_schachtel > 1 ) ;
+                    e71_YieldTaskEnquRex ( this_ukt, reserved_cnt > 0 ||
+                                           l_curr_task->excl_schachtel > 1 ) ;
                     GOTO_DISP (&this_ukt);
                 }                 /* else leave the loop */  
         }
@@ -685,7 +672,6 @@ static  void    vbegexcl_dirty_read_loop ( RTE_TaskId pid, REGION_ID sid )
         scb->wait_count++ ;
         this_ukt->NumRegWait++ ;
         DBG1 (( MF__,"before goto_disp T%d", l_curr_task->index ));
-        l_curr_task->SuppressTimeCollection = false;
         GOTO_DISP (&this_ukt);
         DBG1 (( MF__,"after goto_disp T%d", l_curr_task->index ));
     }
@@ -696,7 +682,6 @@ static  void    vbegexcl_dirty_read_loop ( RTE_TaskId pid, REGION_ID sid )
         l_curr_task->reg_coll_cnt [ sid-1 ] += num_coll ;
     }
     scb->begexcl_tas_count += MAX_INTERLOCK_LOOPS - tas_count ;
-    l_curr_task->SuppressTimeCollection = false;
     l_curr_task->state = TSK_RUNNING;
     DBGOUT_T (( l_curr_task->index ));
 }
@@ -730,7 +715,6 @@ void    vendexcl ( RTE_TaskId pid, REGION_ID sid )
         return;
 
     l_curr_task->state = TSK_VENDEXCL;
-    l_curr_task->SuppressTimeCollection = true;
 
     scb  = KGS-> semaphore + sid - 1;
     scbs = & scb->semaliste;
@@ -763,7 +747,7 @@ void    vendexcl ( RTE_TaskId pid, REGION_ID sid )
             if ( ! XPARAM(fEndExclBusyWait)  )  /* the parameter MP_RGN_BUSY_WAIT decides whether blocking or non-blocking wait is to be used here (and nothing else!)  */
             {
                 l_curr_task->qa.req_type = REQ_TEST_AND_SET ;
-                e71_FastEnquRex ( this_ukt, (tsp00_Bool)true );
+                e71_YieldTaskEnquRex ( this_ukt, (tsp00_Bool)true );
                 DBG1 (( MF__,"before goto_disp T%d \n", l_curr_task->index));
                 GOTO_DISP (&this_ukt);
                 DBG1 (( MF__,"after  goto_disp T%d \n", l_curr_task->index));
@@ -794,7 +778,7 @@ void    vendexcl ( RTE_TaskId pid, REGION_ID sid )
             {
                 l_curr_task->qa.req_type = REQ_RESCHEDULE ;
                 l_curr_task->state = TSK_RUNNABLE;
-                e71_FastEnquRex ( this_ukt, (tsp00_Bool)false ) ; /* enter myself in the run queue, with a lower priority than the process waiting for access to the task list */
+                e71_YieldTaskEnquRex ( this_ukt, (tsp00_Bool)false ) ; /* enter myself in the run queue, with a lower priority than the process waiting for access to the task list */
                 GOTO_DISP (&this_ukt);   /* and call the dispatcher */
             }
         }
@@ -830,7 +814,7 @@ void    vendexcl ( RTE_TaskId pid, REGION_ID sid )
             l_curr_task->state = TSK_RUNNABLE;
             l_curr_task->self_susp_cnt++ ;  /* why only here??? */
             l_curr_task->qa.req_type = REQ_RESCHEDULE ;
-            e71_FastEnquRex ( this_ukt, false ); /* set my own priority to a low value ???*/
+            e71_YieldTaskEnquRex ( this_ukt, false ); /* set my own priority to a low value ???*/
             GOTO_DISP (&this_ukt);   /* and call the dispacther */
         }
     }
@@ -838,7 +822,6 @@ void    vendexcl ( RTE_TaskId pid, REGION_ID sid )
 /*
     scb->last_owner = l_curr_task ;  
 */
-    l_curr_task->SuppressTimeCollection = false;
     l_curr_task->state = TSK_RUNNING;
     l_curr_task->sid   = 0 ;
 	DBGOUT_T(l_curr_task->index);
