@@ -41,10 +41,6 @@
 #include "gen71.h"
 #include "gen83.h"
 
-#if defined (SAPDB_SLOW) || defined (SAPDB_QUICK)
-# define MULTIPLE_SIGNAL_CHECK     1
-#endif
-
 extern  int                             e60_dbgdbglvl ;
 
 #if ! ( defined(_IBMR2) || defined(OSF1) || defined(NMP) )	/* already declared in include file */
@@ -353,11 +349,9 @@ void    vresume ( tsp00_TaskId pid )
                 if ( this_ukt->curr_task->excl_schachtel == 0 &&
                     !this_ukt->curr_task->prio_flag && tcb->prio_flag )
                 {
-                    e71_FastEnquRex ( this_ukt, FALSE );
+                    e71_YieldTaskEnquRex ( this_ukt, FALSE );
                     this_ukt->curr_task->self_susp_cnt++ ;
-                    this_ukt->curr_task->SuppressTimeCollection = true;
                     GOTO_DISP (&this_ukt);
-                    this_ukt->curr_task->SuppressTimeCollection = false;
                 }
             }
         }
@@ -408,6 +402,26 @@ void    vwait ( tsp00_TaskId pid,
           eo52UpdateTaskStateStatisticRec( &tcb->TaskStateStat.VwaitStat, &tcb->TaskStateStat );
       }
 
+      if ( this_ukt->curr_quu != &tcb->qa )
+      {
+          MSGALL (( ERR_AWOKE_FROM_WRONG_EVENT, "vsignal", this_ukt->curr_task->index, this_ukt->curr_quu, &tcb->qa ));
+          MSGALL (( ERR_VWAIT_WRONG_REQ_ARGS,
+                    this_ukt->curr_task->index,
+                    this_ukt->curr_quu->taskid  ? this_ukt->curr_quu->taskid->index : 0,
+                    this_ukt->curr_quu->req_type, REQ_VSIGNAL_EO00 ));
+
+          vabort ( WRITE_CORE );
+      }
+      if ( this_ukt->curr_quu->req_type != REQ_VSIGNAL_EO00 )
+      {
+          MSGALL (( ERR_VWAIT_WRONG_REQ ));
+          MSGALL (( ERR_VWAIT_WRONG_REQ_ARGS,
+                    this_ukt->curr_task->index,
+                    this_ukt->curr_quu->taskid  ? this_ukt->curr_quu->taskid->index : 0,
+                    this_ukt->curr_quu->req_type, REQ_VSIGNAL_EO00 ));
+          vabort ( WRITE_CORE );
+      }
+
       DBG1 (( MF__,"after  goto_disp T%d \n", pid ));
   }
 
@@ -416,18 +430,11 @@ void    vwait ( tsp00_TaskId pid,
     case  1 : * ok = 1 ; break ;
     case -1 : * ok = 0 ; break ;
     default : MSGALL (( ERR_VWAIT_ILL_BOOL, tcb->vsignal_ok , pid )) ;
-	      vabort ( WRITE_CORE ) ;
+    	      vabort ( WRITE_CORE ) ;
               break;
   }
   
-#if defined (MULTIPLE_SIGNAL_CHECK)
-  WAIT_UNTIL_ALONE( tcb->exclusive );
   tcb->vsignal_ok = 0 ;
-  CLEARLOCK( tcb->exclusive );
-#else
-  tcb->vsignal_ok = 0 ;
-#endif
-
   tcb->lWaitForTask = undef_ceo00  ;
   tcb->state = TSK_RUNNING_EO00 ;
   tcb->wait_time = 0 ;
@@ -459,24 +466,15 @@ void vsignal ( tsp00_TaskId pidToSignal ,
   }
   tcbToSignal = kgs->pFirstTaskCtrl + pidToSignal - 1 ;
 
-# if defined (MULTIPLE_SIGNAL_CHECK)
-  WAIT_UNTIL_ALONE( tcbToSignal->exclusive );
-
-  if ( tcbToSignal->vsignal_ok != 0 )
-  {
-    CLEARLOCK( tcbToSignal->exclusive );
-    MSGALL(( ERR_MULTIPLE_VSIGNAL_CALL, tcbToSignal->index ));
-    vabort(); 
-  }
-  tcbToSignal->vsignal_ok = ( ok ? 1 : -1 );
-  CLEARLOCK( tcbToSignal->exclusive );
-#endif
-
   if ( tcbToSignal->ukt == this_ukt )
   {
-#   if !defined (MULTIPLE_SIGNAL_CHECK)
+    if ( tcbToSignal->vsignal_ok != 0 )
+    {
+      CLEARLOCK( tcbToSignal->exclusive );
+      MSGALL(( ERR_MULTIPLE_VSIGNAL_CALL, tcbToSignal->index ));
+      vabort(); 
+    }
     tcbToSignal->vsignal_ok = ( ok ? 1 : -1 );
-#   endif
 
     DBG1 (( MF__,"signal local task \n" ));
     /*IST     tcbToSignal->signal_count ++ ; */
@@ -494,11 +492,9 @@ void vsignal ( tsp00_TaskId pidToSignal ,
         && !tcbCaller->prio_flag 
         && tcbToSignal->prio_flag )
       {
-        e71_FastEnquRex ( this_ukt, FALSE );
+        e71_YieldTaskEnquRex ( this_ukt, FALSE );
         tcbCaller->self_susp_cnt++ ;
-        tcbCaller->SuppressTimeCollection = true;
         GOTO_DISP (&this_ukt);
-        tcbCaller->SuppressTimeCollection = false;
       }
     }
   }
@@ -541,7 +537,7 @@ void    vsleep ( tsp00_TaskId  pid,
     lquu = & tcb->qa ;
     lquu->req_type = REQ_RESCHEDULE_EO00 ;
     lquu->taskid   = tcb ;
-    e71_FastEnquRex ( this_ukt, tcb->prio_flag );
+    e71_YieldTaskEnquRex ( this_ukt, tcb->prio_flag );
   
     DBG1 (( MF__,"before goto_disp T%d \n", pid ));
     GOTO_DISP (&this_ukt);
@@ -611,11 +607,9 @@ void  vreschedule_msec     ( tsp00_TaskId      pid,
     *StartMsec = ( (time ( (time_t *)0 ) - KGS->start_time) * 1000);
   }
 
-  e71_FastEnquRex ( this_ukt, this_ukt->curr_task->prio_flag );
+  e71_YieldTaskEnquRex ( this_ukt, this_ukt->curr_task->prio_flag );
 
-  tcb->SuppressTimeCollection = true;
   GOTO_DISP (&this_ukt);
-  tcb->SuppressTimeCollection = false;
 
   time_now_in_msec       = ( (time ( (time_t *)0 ) - KGS->start_time) * 1000);
 
@@ -678,9 +672,14 @@ tsp00_Bool                      ok;
 
   ok = (tsp00_Bool)(((char*)paraOK)-(char *)0);
 
-# if !defined (MULTIPLE_SIGNAL_CHECK)
+  if ( this_ukt->curr_task->vsignal_ok != 0 )
+  {
+    CLEARLOCK( this_ukt->curr_task->exclusive );
+    MSGALL(( ERR_MULTIPLE_VSIGNAL_CALL, this_ukt->curr_task->index ));
+    vabort(); 
+  }
+
   this_ukt->curr_task->vsignal_ok = ( ok ? 1 : -1 );
-# endif
 
 	/*IST this_ukt->curr_task->signal_count ++ ; */
 	/*IST if ( this_ukt->curr_task->signal_count == 0 ) */
