@@ -1,5 +1,5 @@
 /*!
-  @file           ven70.c
+  @file           Copy of ven70.c
   @author         JoergM
   @brief          SAPDB Kernel: Initialization of shared memory
   
@@ -54,15 +54,7 @@ first created  2000-10-17  14:10
 #include	<sys/sysmacros.h>
 #endif
 
-#ifndef HP9
 #include        <ctype.h>
-#endif
-#if     HP9
-#undef  	toupper
-#define 	toupper(x)      (x>='a'&&x<='z'?x-'a'+'A':x)
-#define 	tolower(x)      (x>='A'&&x<='Z'?x-'A'+'a':x)
-#include    <sys/vmmac.h>
-#endif
 
 #include <sys/mman.h>
 #ifndef MAP_FAILED
@@ -92,34 +84,15 @@ first created  2000-10-17  14:10
 
 #define DBPAGESIZE ((unsigned long)XParam->ulDBPageSize)
 
-#define	SHMALIGN		SHMLBA
-
 #define MAX_REQ_PER_TASK                2
  
-#if !defined(BIT64) && defined(SUN) && defined(SVR4)  /* -hs- 10.06.96 */
-#  define  SOLARIS_SHM_ADDR               0xe0000000
-#  define  SOLARIS_SHM_STEP                0xa000000
-#endif /* !BIT64 && SUN && SVR4 */
-
-#define SHMID_RESERVE  4
-
-#if defined(_IBMR2)
-#  define DEF_SHMCHUNK 0x100
-#  define SHMCHUNK(_x) ( ( ( (_x * 0x100000L) - 1 ) / SHMLBA + 1 ) * SHMLBA )
-/* PTS 1110469 */
-#if !defined(BIT64)
-#  define AIX_SHM_ADDRESS (char *)0xc0000000
-#endif
-#elif defined(LINUX)
-#  define DEF_SHMCHUNK 0x20
-#  define SHMCHUNK(_x) ( ( ( (_x * 0x100000L) - 1 ) / SHMLBA + 1 ) * SHMLBA )
-#  define LINUX_SHM_ADDRESS (char *)0x50000000
+#if defined(LINUX)
+/* Workaround for corrupted bits/shm.h definition for SHMLBA in SLES9... */
+#define SHARED_MEMORY_ALIGNMENT getpagesize()
 #else
-#  define DEF_SHMCHUNK 0x7f8
-#  define SHMCHUNK(_x) ( ( ( (_x * 0x100000L) - 1 ) / SHMLBA + 1 ) * SHMLBA )
+#define SHARED_MEMORY_ALIGNMENT SHMLBA
 #endif
 
-
 /*===========================================================================*
  *  MACROS                                                                   *
  *===========================================================================*/
@@ -132,20 +105,6 @@ first created  2000-10-17  14:10
  *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
  *===========================================================================*/
 
-static int en70_CreateShmChunks( int *KgsShmids,
-                                int *ActShmidIdx,
-                                key_t *pBaseKey,
-                                int  MaxShmidIdx,
-                                unsigned long ShmSize,
-                                unsigned long ChunkSize,
-                                char *ShmInfo);
-static int en70_AttachShmChunks( int   *KgsShmids,
-                                int    FirstShmidIdx,
-                                int    LastShmidIdx,
-                                char **AddrToAttach,
-                                int    ShmFlag,       /* CR 1000130 */
-                                unsigned long ChunkSize,
-                                char  *AttachInfo );
 static  int     en70_InsertTask ( struct  TASK_TYPE * tcb ,
 				  int                 task_type ,
 				  int                 stack_size ,
@@ -168,7 +127,7 @@ static  void *  en70_AllocSystemPages(size_t SizeInBytes);
  *===========================================================================*/
 /* exported data */
 
-struct  KERNEL_GLOBAL_SECTION   * kgs ;
+extern struct  KERNEL_GLOBAL_SECTION * kgs ;
 
 /* imported data */
 
@@ -192,8 +151,6 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
 {
   int           i;
   int           idx;
-  int           last_kgs_shmid=0 ;
-
   long          kgs_number_of_tasks ;
   long          kgs_ukt_count ;
   tsp00_Int4    kgs_number_of_user_ukts;
@@ -239,9 +196,6 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
 
   int           rc ;
   int	        count ;
-#ifdef SHMLCK /* MWa for locking shared memory */
-  int           lshm_ret ; 
-#endif /* SHMLCK */
   int           ukt_index ;
   int           task_index ;
 
@@ -255,51 +209,22 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
   struct  DOUBLY_LINKED           * quu ;
   struct  DEV_PCB                 * devpcb ;
   unsigned long	kgs_connection ;
-  unsigned long                     LastKgsPartSize ;
-  char                            * addr;
-  int                             uid, num_kgs_shm_parts ;
-  int                             *e70_kgs_shmid ;
+  int                               kgs_shmid ;
   tsp00_Int2                        maxdevspaces = XParam->lMaxDevspaces  ;
   tsp00_Int2                        num_dw ;
   tsp00_Int4                        num_asynio_tasks = XParam->ulMaxServer+1;/*1=ut*/
   tsp00_Int4                        devpsets  = XParam->lDeviPerDevspace + 
                                               XParam->lPrioDeviPerDevspace ;
   tsp00_Int4                        MaxBackupDevs = XParam->lNoOfBackupDevs + 1;
-#if !defined(BIT64) && defined(SUN) && defined(SVR4)  /* -hs- 10.06.96                 */
-  char                            *solaris_addr;
-#endif /* !BIT64 && SUN && SVR4 */
   unsigned long			        kgs_aioc_queue_elem, kgs_mem_for_sys_asyn_struc;
   unsigned long                   SizeOfSysAsynIOStruct = 
                                         e54a_get_aio_struct_size();
-  unsigned long                   shmchunk ;
-  int                             ShmFlag = 0 ; 
   teo70k_ClusterInfo *            pCluInfo = NULL;
 
   DBGIN;
 
-  if ( XParam->lShmChunk <= 0 )
-    XParam->lShmChunk = DEF_SHMCHUNK ;
-
-  shmchunk = SHMCHUNK( XParam->lShmChunk ) ;
-
-  num_kgs_shm_parts = (XParam->lShmKernel / 1024 ) / XParam->lShmChunk + 
-				   SHMID_RESERVE ;
-  if ( ( num_kgs_shm_parts & 1 ) == 1 ) 
-	  num_kgs_shm_parts++ ; /* ALIGN */
-
-  MSGD(( INFO_SHMCHUNK_SIZE, shmchunk))
-
   num_dw = XParam->ulMaxDataWriter ? XParam->ulMaxDataWriter : 1 ;
 
-  if ( ALLOC_MEM_EO57( (void **)&e70_kgs_shmid, (num_kgs_shm_parts+1) * sizeof (int ) ) 
-      != NO_ERROR_EO001 )
-  {
-    MSGCD ((ERR_INITSHM_MALLOC_FAILED, num_kgs_shm_parts, sqlerrs() )) ;
-    return ( 2 );
-  }
-
-  DBG1 (( MF__,"allocated %d shmids successfull\n", num_kgs_shm_parts+1 ));
-
 /*
  * ================================================================
  *  Analyze the task cluster.
@@ -313,6 +238,12 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
     return ( 2 );
   }
 
+  if ( pCluInfo->fOneToOne && XParam->uLoadBalancingCheck == 0 )
+  {
+      /* disable load balancing permanently */
+      XParam->fForbidLoadBalancing = true;
+  }
+
   e84_connect_strategy = pCluInfo->lConnectStrategy ;
 
   kgs_number_of_tasks     = pCluInfo->ulNumOfTasks ;
@@ -346,8 +277,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
   DBG1 (( MF__,"MAX_QUU_PER_TASK  %d \n",
 				  MAX_QUU_PER_TASK ));
   DBG1 (( MF__,"SHM_RND           0x%08x \n", SHM_RND    ));
-  DBG1 (( MF__,"SHMLBA            0x%08x \n", SHMLBA     ));
-  DBG1 (( MF__,"SHMALIGN          0x%08x \n", SHMALIGN   ));
+  DBG1 (( MF__,"SHARED_MEMORY_ALIGNMENT 0x%08x \n", SHARED_MEMORY_ALIGNMENT ));
 
   tot_size = 0 ;
 
@@ -367,7 +297,8 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
   DBG1 (( MF__,"kgs offset      0x%08lx \n", kgs_start ));
 
   kgs_shmids = tot_size ;
-  tot_size   += num_kgs_shm_parts * sizeof (int) ;
+  tot_size   += sizeof (int) ;
+  tot_size   = ( (tot_size - 1) / sizeof(void *) + 1 ) * sizeof(void *) ;
 
 /*
  *  free_list
@@ -509,9 +440,22 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
   {
     int idx, tot_xxx_elem;
     kgs_mem_xxx_queues = tot_size;
-    for ( tot_xxx_elem = 0, idx=0; idx< kgs_ukt_count; idx++ )
-	    tot_xxx_elem += ( kgs_number_of_tasks + 1 ) 
-					    * (kgs_ukt_count -1) ;
+    if ( XParam->fForbidLoadBalancing )
+    {
+        for ( tot_xxx_elem = 0, idx=0; idx< kgs_ukt_count; idx++ )
+        {
+	        tot_xxx_elem += ( pCluInfo->pThrdInfo[idx].ulNumOfTasks + 1 ) 
+					        * (kgs_ukt_count -1) ;
+        }
+    }
+    else
+    {
+        for ( tot_xxx_elem = 0, idx=0; idx< kgs_ukt_count; idx++ )
+        {
+            tot_xxx_elem += ( kgs_number_of_tasks + 1 ) 
+					        * (kgs_ukt_count -1) ;
+        }
+    }
     DBG1 (( MF__,"init %d tot_xxx_elem for all ukps \n", tot_xxx_elem ));
     tot_size += tot_xxx_elem * sizeof ( struct XXX_QUEUE ) ;  
     DBG1 (( MF__,"total_size for xxx_elements %d at offs %x \n", 
@@ -553,118 +497,45 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
  * ==========================
  */
 
-/* alloc chunks of shared memory, soon to be attached contiguously */
-/* tot_size = ( ( tot_size - 1 ) / SHMALIGN + 1 ) * SHMALIGN;*/
-  uid      = getuid (); 
+  tot_size = ( ( tot_size - 1 ) / SHARED_MEMORY_ALIGNMENT + 1 ) * SHARED_MEMORY_ALIGNMENT;
 
-  if ( ( tot_size / shmchunk + 1 ) >= num_kgs_shm_parts )
+  DBG1 (( MF__,"tot_size = %ld\n", tot_size )) ;
+
+  if ( !en41CreateServerKeyAndFirstSegment( en81_dbname,
+                                            tot_size,
+                                            &kgs_ipcBaseKey,
+                                            &kgs_shmid ) )
   {
-    MSGD (( IERR_INITSHM_TOO_MANY_SHMIDS )) 
-    MSGD (( IERR_INITSHM_IDS_AVAIL_WANT, 
-				     num_kgs_shm_parts , ( tot_size / shmchunk + 1 + 1 ) )) ;
-    return ( 2 );
+      MSGD(( ERR_IPC_SHMS_SIZE, tot_size ));
+      MSGD(( ERR_IPC_SHMC_SHMGET, "Check OS Parameter for IPC SHM "));
+      MSGD ((ERR_IPC_SHMC_SHMGET, "and please check your critical database parameters" )) ;
+      MSGD ((ERR_IPC_SHMC_SHMGET, "These are especially MAXUSERTASKS, MAXSERVERTASKS and CACHE_SIZE" ));
+      return(2);
   }
 
-  DBG1 (( MF__,"tot_size = %ld, shmchunk = %u\n", tot_size ,shmchunk )) ;
-
-  if ( en70_CreateShmChunks(e70_kgs_shmid, &last_kgs_shmid, &kgs_ipcBaseKey, 
-		  num_kgs_shm_parts-1, tot_size, shmchunk, "shared section") != 0 )
-  {
-	return(2);
-  }
-
-/*
- *  Attach the shared segment at the requested address.
- */
-
-#if !defined(BIT64) && defined(SUN) && defined(SVR4)  /* -hs- 10.06.96                         */
-/* avoiding shmat error when using x_clear/x_cons with smaller shm segments  */
-/* by supplying an address in the solaris memory hole between stack and heap */
-/**/
-  DBG1 (( MF__,"SOLARIS_SHM_STEP is 0x%08x\n", SOLARIS_SHM_STEP ));
-  if ( tot_size < SOLARIS_SHM_STEP)
-  {
-	      DBG1 (( MF__,"(SUN/SVR4): trying to attach at 0x%08x\n", 
-		      SOLARIS_SHM_ADDR));
-	      solaris_addr = (char *) SOLARIS_SHM_ADDR;
-	      seg_ptr = (struct SHARED_SEGMENT *)
-			       sql41AttachShm ( e70_kgs_shmid[0] , solaris_addr, ShmFlag );
-  }
-
-  if ( seg_ptr == 0 )
-  {
-     DBG1 (( MF__,"(SUN/SVR4): trying to attach at system supplied address\n" ));
-     solaris_addr = (char *) 0;
-     seg_ptr = (struct SHARED_SEGMENT *)
-			  sql41AttachShm ( e70_kgs_shmid[0] , solaris_addr, ShmFlag );
-  }
-
-  if ( seg_ptr == 0 )
-  {
-    DBG1 (( MF__,"(SUN/SVR4): cannot attach at 0x%08x\n", solaris_addr ));
-    en70_RemoveKgs ( e70_kgs_shmid ) ;
-    return ( 2 );
-  }
-#else /* !BIT64 && SUN && SVR4 */
-
   DBG1 (( MF__,"trying to attach at system supplied address\n" ));
-
-#if defined(LINUX)
-  seg_ptr = (struct SHARED_SEGMENT *)
-	  sql41AttachShm ( e70_kgs_shmid[0] , LINUX_SHM_ADDRESS, ShmFlag );
-#elif defined(_IBMR2) && !defined(BIT64)
-  seg_ptr = (struct SHARED_SEGMENT *)
-	  sql41AttachShm ( e70_kgs_shmid[0] , AIX_SHM_ADDRESS, ShmFlag );
+#if defined(LINUX) && defined(I386)
+  seg_ptr = (struct SHARED_SEGMENT *)sql41AttachShm ( kgs_shmid, (char*) 0x50000000, 0 );
 #else
-  seg_ptr = (struct SHARED_SEGMENT *)
-	  sql41AttachShm ( e70_kgs_shmid[0] , (char*) 0, ShmFlag );
+  seg_ptr = (struct SHARED_SEGMENT *)sql41AttachShm ( kgs_shmid, (char*) 0, 0 );
 #endif
   if ( seg_ptr == 0 )
   {
-    DBG1 (( MF__,"cannot attach at system supplied address\n" ));
-    en70_RemoveKgs ( e70_kgs_shmid ) ;
+    MSGD ((ERR_IPC_SHMC_SHMGET, "Cannot attach the main shared memory segment" )) ;
+    MSGD(( ERR_IPC_SHMS_SIZE, tot_size));
+    MSGD(( ERR_IPC_SHMC_SHMGET, "Check OS Parameter for IPC SHM "));
+    MSGD ((ERR_IPC_SHMC_SHMGET, "and please check also your critical database parameters" )) ;
+    MSGD ((ERR_IPC_SHMC_SHMGET, "These are especially MAXUSERTASKS, MAXSERVERTASKS and CACHE_SIZE" ));
+    en70_RemoveKgs ( &kgs_shmid ) ;
     return ( 2 );
   }
 
-#endif /* !BIT64 && SUN && SVR4 */
-
   /* All segments have been attached at the system supplied address */
   shm_start = (unsigned long) seg_ptr ;
 
   MSGD(( INFO_INITSHM_ATTACHED_AT, "shared section", shm_start ));
   MSGD(( INFO_INITSHM_ENDS_AT    , "shared section", shm_start+tot_size ));
 
-  addr = (char *)shm_start + shmchunk ;
-  if ( en70_AttachShmChunks( e70_kgs_shmid, 1, last_kgs_shmid-1 , &addr, 
-					    ShmFlag, shmchunk, NULL ) )
-  {
-    en70_RemoveKgs ( e70_kgs_shmid ) ;
-    return ( 2 );
-  }
-
-#ifdef SHMLCK /* MWa for locking shared memory */
-  if ( ( lshm_ret = mlock ( shm_start , tot_size ) ) == -1 )
-  {
-    MSGCD (( ERR_INITSHM_LOCK_ERR ,lshm_ret ));
-    en70_RemoveKgs ( e70_kgs_shmid ) ;
-    return ( 2 );
-  }
-  else
-  {
-    MSGD (( INFO_INITSHM_LOCKED ,lshm_ret));
-  }
-#endif /* SHMLCK */
-
-/*
- *  Note the attach address in the id-file.
- */
-  if ( sql41_create_idfile ( "db", en81_dbname , 'm' , e70_kgs_shmid [0] ) )
-  {
-	  MSGCD (( ERR_INITSHM_CANT_OPEN_IDFILE ))
-	  en70_RemoveKgs ( e70_kgs_shmid ) ;
-	  return ( 2 );
-  }
-
 /*
  *  set global pointer to kernel global section
  */
@@ -689,11 +560,10 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
   KGS->this_kgs         = KGS;
   KGS->seg              = seg_ptr ;
   KGS->seg_size         = tot_size ;
-  KGS->kgs_shm_part_len = shmchunk ;
   KGS->ipcBaseKey       = kgs_ipcBaseKey ;
   KGS->kgsshmid         = (int *) (kgs_shmids + shm_start) ;
-  COPY ( KGS->kgsshmid , e70_kgs_shmid , num_kgs_shm_parts * sizeof ( int ) ) ;
-  KGS->uid              = uid;
+  *(KGS->kgsshmid)      = kgs_shmid;
+  KGS->uid              = getuid();
   KGS->pid              = getpid();
   KGS->debuglevel       = e60_dbgdbglvl ;
   KGS->ulNumOfTasks     = kgs_number_of_tasks ;
@@ -726,7 +596,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
   if ( (int) strlen((char *) en81_dbname) >= (int) sizeof(tsp00_DbNamec) )
   {
 	  MSGCD (( ERR_INITSHM_DBNAME_TO_LONG, sizeof(KGS->serverdb)-1 ));
-	  en70_RemoveKgs ( e70_kgs_shmid ) ;
+	  en70_RemoveKgs ( &kgs_shmid ) ;
 	  return ( 2 ) ;
   }
   (void) strcpy ( (char *) KGS->serverdb , (char *) en81_dbname );
@@ -857,7 +727,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
     tcb->lWaitForRoot         = undef_ceo00 ;
     tcb->lWaitForTask         = undef_ceo00 ;
     tcb->DevNameWaitingFor    = NULL ;
-    tcb->SuppressTimeCollection = false;
+    tcb->YieldTask            = false;
     
     tcb->pCurrStackPointer    = 0; /* PTS 1110829 */
 
@@ -883,6 +753,13 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
     /* PTS 1108975 Individual trace sizes */
     kgsTotalTraceBufferSizeInPages = XParam->lTraceDevspaceSize;
     memoryForTraceBuffer = en70_AllocSystemPages(DBPAGESIZE * kgsTotalTraceBufferSizeInPages);
+    if ( !memoryForTraceBuffer )
+    {
+        MSGD ((IERR_VMALLOC_MALLOC_SYS_ERR, "Failed to allocate TRACE space. Please check the critical database parameters" )) ;
+        MSGD ((IERR_VMALLOC_MALLOC_SYS_ERR, "These are MAXUSERTASKS, MAXSERVERTASKS, CACHE_SIZE and TRACE_PAGES_..." ));
+  	    en70_RemoveKgs ( &kgs_shmid ) ;
+        return ( 2 ) ;
+    }
 
     nextUktTraceBuffer  = (char *) (memoryForTraceBuffer);
     nextUktTraceBuffer += DBPAGESIZE; /* First page is infopage */
@@ -979,17 +856,20 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
 /*
  *      TIMEOUT
  */
-  INSERT_TASK(TI_Pos, TT_TI, FIX_TI_STACK_SIZE_EO70K, KGS->ti);
+  /* No own entry for task stack . Reuse entry for data writer task... */
+  INSERT_TASK(TI_Pos, TT_TI, pCluInfo->VarTaskCntRec[VAR_TASK_DW_EO70K].lStackSize, KGS->ti);
   tcb ++;
 /*
  *      ARCHIVELOGWRITER
  */
-  INSERT_TASK(AL_Pos, TT_AL, FIX_AL_STACK_SIZE_EO70K, KGS->al);
+  /* No own entry for task stack . Reuse entry for data writer task... */
+  INSERT_TASK(AL_Pos, TT_AL, pCluInfo->VarTaskCntRec[VAR_TASK_DW_EO70K].lStackSize, KGS->al);
   tcb ++;
 /*
  *      TRACEWRITER
  */
-  INSERT_TASK(TW_Pos, TT_TW, FIX_TW_STACK_SIZE_EO70K, KGS->tw);
+  /* No own entry for task stack . Reuse entry for data writer task... */
+  INSERT_TASK(TW_Pos, TT_TW, pCluInfo->VarTaskCntRec[VAR_TASK_DW_EO70K].lStackSize, KGS->tw);
   tcb ++;
 /*
  *      DATAWRITER
@@ -1000,7 +880,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
     count = pCluInfo->pThrdInfo[THREAD_INDEX(lkp)-1].lDWTasks ;
     if ( count > 0 )
     {
-      rc = en70_InsertTask ( tcb , TT_DW, FIX_DW_STACK_SIZE_EO70K, lkp , count );
+      rc = en70_InsertTask ( tcb , TT_DW, pCluInfo->VarTaskCntRec[VAR_TASK_DW_EO70K].lStackSize, lkp , count );
       tcb += count ;
     }
   }
@@ -1014,7 +894,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
     count = pCluInfo->pThrdInfo[THREAD_INDEX(lkp)-1].lEVTasks ;
     if ( count > 0 )
     {
-      rc = en70_InsertTask ( tcb , TT_EV, FIX_EV_STACK_SIZE_EO70K, lkp , count );
+      rc = en70_InsertTask ( tcb , TT_EV, pCluInfo->VarTaskCntRec[VAR_TASK_EV_EO70K].lStackSize, lkp , count );
       tcb += count ;
     }
   }
@@ -1028,7 +908,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
     count = pCluInfo->pThrdInfo[THREAD_INDEX(lkp)-1].lBUPTasks ;
     if ( count > 0 )
     {
-      rc = en70_InsertTask ( tcb , TT_BUP, FIX_BUP_STACK_SIZE_EO70K, lkp , count );
+      rc = en70_InsertTask ( tcb , TT_BUP, pCluInfo->VarTaskCntRec[VAR_TASK_BUP_EO70K].lStackSize, lkp , count );
       tcb += count ;
     }
   }
@@ -1042,7 +922,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
     count = pCluInfo->pThrdInfo[THREAD_INDEX(lkp)-1].lGCTasks ;
     if ( count > 0 )
     {
-      rc = en70_InsertTask ( tcb , TT_GC, FIX_GC_STACK_SIZE_EO70K, lkp , count );
+      rc = en70_InsertTask ( tcb , TT_GC, pCluInfo->VarTaskCntRec[VAR_TASK_GC_EO70K].lStackSize, lkp , count );
       tcb += count ;
     }
   }
@@ -1059,7 +939,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
     count = pCluInfo->pThrdInfo[THREAD_INDEX(lkp)-1].lServerTasks ;
     if ( count > 0 )
     {
-      rc = en70_InsertTask ( tcb , TT_SV, FIX_SV_STACK_SIZE_EO70K, lkp , count );
+      rc = en70_InsertTask ( tcb , TT_SV, pCluInfo->VarTaskCntRec[VAR_TASK_SV_EO70K].lStackSize, lkp , count );
       tcb += count ;
     }
   }
@@ -1115,7 +995,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
   {
     MSGCD (( IERR_INITSHM_TSK_NOT_LSTTSK, tcb  ))
     MSGCD (( IERR_INITSHM_LSTTSK_INFO, KGS->pLastTaskCtrl ))
-    en70_RemoveKgs ( e70_kgs_shmid ) ;
+    en70_RemoveKgs ( &kgs_shmid ) ;
     return ( 2 );
   }
 /*
@@ -1124,7 +1004,15 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
   l_xxx = KGS->first_xxx ;
   for ( lkp = KGS->first_kp ; lkp <= KGS->last_kp ; lkp ++ )
   {
-    int tot_tasks = kgs_number_of_tasks + 1;
+    int tot_tasks;
+    if ( XParam->fForbidLoadBalancing )
+    {
+        tot_tasks = pCluInfo->pThrdInfo[THREAD_INDEX(lkp)-1].ulNumOfTasks + 1;
+    }
+    else
+    {
+        tot_tasks = kgs_number_of_tasks + 1;
+    }
     DBG1 (( MF__,"xxx pcb 0x%08x \n", lkp ));
     DBG1 (( MF__,"xxx ukt P%d \n", THREAD_INDEX(lkp) ));
     for ( count = 0 ; count < KGS->ulNumOfUKTs ; count++ )
@@ -1203,7 +1091,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
       if ( maxdevspaces < 0 ) 
       {
         MSGCD (( IERR_INITSHM_TO_MANY_DEVSP )) ;
-        en70_RemoveKgs ( e70_kgs_shmid ) ;
+        en70_RemoveKgs ( &kgs_shmid ) ;
         return ( 2 );
       }
       XPARAM(DevspDescr)[idx].Devspaces = 
@@ -1217,7 +1105,6 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
     }
   }
 
-  FREE_MEM_EO57 ( e70_kgs_shmid ) ;
   e55_init_vbegexcl_func_ptr( kgs_ukt_count ) ;
 
   if ( XParam->fUseAsyncIO )
@@ -1262,15 +1149,7 @@ int e70_init_kgs ( RTE_XPARAM_REC *XParam )
 #define MF__ MOD__"e70_is_kgsshmid"
 int e70_is_kgsshmid ( int shmid )
 {
-  int i=0 ;
-
-  DBGIN;
-
-  while ( KGS->kgsshmid[i] != -1 && KGS->kgsshmid[i] != shmid ) 
-    i++ ;
-
-  DBGOUT;
-  return ( KGS->kgsshmid[i] == shmid ) ;
+  return ( KGS->kgsshmid[0] == shmid ) ;
 }
 
 /*
@@ -1435,20 +1314,8 @@ static void * en70_AllocSystemPages(size_t SizeInBytes)
 
 #else
 
-#if defined(SUN)||defined(NMP)
-/*SUN||NMP*/  int m_DevZeroFd = open("/dev/zero", O_RDWR, 0);
-/*SUN||NMP*/  if ( m_DevZeroFd != -1 )
-/*SUN||NMP*/  {
-/*SUN||NMP*/    ChunkAddress = (void *)mmap( 0, SystemSize, PROT_READ|PROT_WRITE, 
-/*SUN||NMP*/                                          MAP_SHARED, m_DevZeroFd, 0 ); /* PTS 1108885 */
-/*SUN||NMP*/    close(m_DevZeroFd);
-/*SUN||NMP*/  }
-#endif /*SUN||NMP*/
-
-# if defined(OSF1)||defined(LINUX)||defined(HPUX)||defined(AIX)
-/*OTHER*/  ChunkAddress = (void *)mmap( 0, SystemSize, PROT_READ|PROT_WRITE, 
-/*OTHER*/                                          MAP_SHARED|MAP_ANONYMOUS, -1, 0 ); /* PTS 1108885 */
-# endif /*OSF1||LINUX||HPUX||AIX*/
+    ChunkAddress = (void *)mmap( 0, SystemSize, PROT_READ|PROT_WRITE, 
+                                 MAP_SHARED|MAP_ANONYMOUS, -1, 0 ); /* PTS 1108885 */
 
     if ( ChunkAddress == (void *) MAP_FAILED )
     {
@@ -1457,176 +1324,9 @@ static void * en70_AllocSystemPages(size_t SizeInBytes)
 #endif
   }
 
-  if ( !ChunkAddress )
-  {
-    MSGCD ((IERR_VMALLOC_MALLOC_SYS_ERR, sqlerrs() )) ;
-    vabort(!WRITE_CORE);
-  }
-
   return ( ChunkAddress );
 }
 
-/*---------------------------------------------------------------------------
-  Function: en70_CreateShmChunks
-  Description: Create shared memory chunks
-
-  Arguments: KgsShmids [in] array of shared memory identifier
-             ActShmidIdx [inout] Next free index
-             MaxShmidIdx [in] Maximum number for an index to be used
-             ShmSize [in] Size of segement wanted
-             ChunkSize [in] Size of a single chunk
-             ShmInfo [in] If set identifies who wants the shared memory
-  Return value: 0 if sucessfully added
-                2 if allocation failed
- */
-static int 
-en70_CreateShmChunks( int *KgsShmids,
-                      int *ActShmidIdx,
-                      key_t *pBaseKey,
-                      int  MaxShmidIdx,
-                      unsigned long ShmSize,
-                      unsigned long ChunkSize,
-                      char *ShmInfo )
-{
-#undef MF__
-#define MF__ MOD__"en70_CreateShmChunks"
-  unsigned long ActShmSize;
-  unsigned long RemainSize;
-  int           uid      = getuid (); 
-
-  ActShmSize = 0;
-  for ( RemainSize = ShmSize; 
-            (RemainSize > 0) && (*ActShmidIdx < MaxShmidIdx) ; 
-                (*ActShmidIdx)++ )
-  {
-    ActShmSize = RemainSize >= ChunkSize ? ChunkSize : RemainSize ;
-
-    MSGD(( INFO_INITSHM_CR_SIZE , ShmInfo, ActShmSize))
-
-    if ( (key_t)0 == *pBaseKey )
-    {
-        if ( !en41CreateServerKeyAndFirstSegment( en81_dbname, 
-                                                  ActShmSize,
-                                                  pBaseKey,
-                                                  &KgsShmids [ *ActShmidIdx ] ) )
-        {
-            KgsShmids [ *ActShmidIdx ] = -1;
-        }
-    }
-    else
-    {
-        KgsShmids [ *ActShmidIdx ] = sql41_create_shm ( *pBaseKey ,
-                                                        *ActShmidIdx ,
-                                                        0 ,
-                                                        ActShmSize ,
-                                                        0664 ,
-                                                        uid ,
-                                                        "db" ,
-                                                        en81_dbname );
-    }
-
-    if ( KgsShmids [ *ActShmidIdx ] < 0 )
-    {
-      en70_RemoveKgs ( KgsShmids ) ;
-      MSGD(( ERR_INITSHM_CR_KGS_ERR )) ;
-      break;
-    }
-    else
-      RemainSize -= RemainSize >= ChunkSize ? ChunkSize : RemainSize ;
-
-    DBG1 (( MF__,"shmid [%d] : %d, size: %ld\n", 
-            *ActShmidIdx,
-            KgsShmids [ *ActShmidIdx ] ,
-            ActShmSize ));
-  }
-
-  KgsShmids [ *ActShmidIdx ] = -1 ;
-  DBG1 (( MF__,"shmid [%d] : %d\n",
-          *ActShmidIdx, 
-          KgsShmids [ *ActShmidIdx ] ));
-
-  if ( RemainSize > 0 && *ActShmidIdx == MaxShmidIdx )
-  {
-    MSGD (( IERR_INITSHM_TOO_MANY_SHMIDS ));
-    MSGD (( IERR_INITSHM_IDS_AVAIL_WANT, 
-            MaxShmidIdx, 
-            ( RemainSize / ChunkSize + 1 ) )) ;
-  }
-
-  return( RemainSize == 0 ? 0 : 2 );
-}
-
-/*---------------------------------------------------------------------------
-  Function: en70_AttachShmChunks
-  Description: Attach shared memory chunks
-
-  The chunks have already been created. Now they are attached in such a way, that the memory
-  is contiguous for the caller.
-
-  Arguments: KgsShmids [in] array of shared memory identifier
-             FirstShmidIdx [in] First index
-             LastShmidIdx [in] Last index (Last - First + 1 is number of chunks wanted)
-             AddrToAttach [inout] Address wanted and address received
-             ShmFlag [in] Flags used for allocating shared memory segments
-             ChunkSize [in] Size of a single chunk
-             AttachInfo [in] If set identifies who wants the shared memory
-  Return value: 0 if sucessfully added
-                2 if allocation failed
- */
-#undef MF__
-#define MF__ MOD__"en70_AttachShmChunks"
-static int en70_AttachShmChunks( int   *KgsShmids,
-                                int    FirstShmidIdx,
-                                int    LastShmidIdx,
-                                char **AddrToAttach,
-                                int    ShmFlag,       /* CR 1000130 */
-                                unsigned long ChunkSize,
-                                char  *AttachInfo )
-{
-  int   ActShmidIdx ;
-  char *AddrWanted, *FirstAddr, 
-       *AddrGot = (char *)1 ;
-
-  for ( ActShmidIdx = FirstShmidIdx, AddrWanted = *AddrToAttach;
-        ActShmidIdx <= LastShmidIdx && AddrGot != NULL ; )
-  {
-    DBG1 (( MF__,"attaching %4d at "FMTA" \n", ActShmidIdx, AddrWanted ));
-    AddrGot = (char *) sql41AttachShm ( KgsShmids [ ActShmidIdx ] , 
-                                        AddrWanted, ShmFlag );
-    if ( AddrGot != NULL )
-    {
-      if ( ActShmidIdx == FirstShmidIdx )
-        FirstAddr = AddrGot ;
- 
-      ActShmidIdx++ ;
-      AddrWanted = AddrGot + ChunkSize ; 
-    }
-  }
-
-  if ( ! AddrGot )
-  {
-    DBG1 (( MF__,"cannot attach at "FMTA" \n", (char *) AddrWanted ));
-    for ( ActShmidIdx--, AddrWanted -= ChunkSize; 
-          ActShmidIdx > FirstShmidIdx ; 
-          ActShmidIdx--, AddrWanted -= ChunkSize)
-    {
-      AddrGot = AddrWanted;
-      (void) sql41_detach_shm ( & AddrGot ) ;
-    }
-  }
-  else
-  if( AddrGot != (char *)1 ) /* one or more chunks attached */
-  {
- 
-    if ( AttachInfo )
-      { MSGD(( INFO_INITSHM_ATTACHED_AT, AttachInfo, FirstAddr )); }
-
-    *AddrToAttach = FirstAddr ;
-  }
-
-  return ( AddrGot ? 0 : 2 );
-}
-
 /*===========================================================================*
  *  END OF CODE                                                              *
  *===========================================================================*/
