@@ -1,5 +1,5 @@
 /*!
-  @file           ven71.c
+  @file           Copy of ven71.c
   @author         JoergM
   @brief          Kernel RunTime: Dispatcher and Queue Management
   @see            
@@ -192,13 +192,13 @@ extern  int                             e60_dbgdbglvl ;
 
 /* local data */
 
-static void en71_HandleTaskStatisticReset( ten50_UKT_Control *this_ukt
-                                         , struct TASK_TYPE  *pTaskCtrl );
+static void en71_HandleTaskStatisticReset( ten50_UKT_Control *this_ukt,
+                                           struct TASK_TYPE  *pTaskCtrl );
 
 static void en71_HandleUKTStatisticReset( ten50_UKT_Control *this_ukt );
 
-static void en71_DoEnterDispatcherTiming( ten50_UKT_Control *this_ukt
-                                        , struct TASK_TYPE  *pTaskCtrl );
+static void en71_DoEnterDispatcherTiming( ten50_UKT_Control *this_ukt,
+                                          struct TASK_TYPE  *pTaskCtrl );
 
 static void en71_DoLeaveDispatcherTiming(ten50_UKT_Control *this_ukt);
 
@@ -546,7 +546,35 @@ MSGD(( INFO_TEST_WILDCARD, eBuffer ));
              */
             if (this_ukt->curr_quu->req_type == REQ_NEW_MOVE_TASK_MEAS_INTERVAL_EO00)
             {   
-                struct REQ_PARMS_MOVE_TASK_MESS_INT *reqArg;
+                tsp00_Int4                           idx;
+                struct REQ_PARMS_MOVE_TASK_MESS_INT* reqArg;
+                SAPDB_UInt8                          runnableMicroSeconds;
+                struct TASK_TYPE*                    pTaskCtrl;
+                SAPDB_UInt8                          timeNow = RTESys_MicroSecTimer();
+                tsp00_Int4                           runQueueLen = this_ukt->RunQueueLen;
+
+                for ( idx = 0; idx < runQueueLen; idx++ )
+                {
+                    pTaskCtrl = this_ukt->RunQueue[idx].Task;
+
+                    if ( pTaskCtrl->moveLock.type.longTerm != 0 ) 
+                        continue;
+
+                    runnableMicroSeconds = timeNow - pTaskCtrl->TaskStateStat.runqueueEnqueueTime;
+
+                    /* accumulate the tasks time waititng in the run queue to be scheduled */
+                    pTaskCtrl->runnableMicroSeconds[this_ukt->currTimeVar] += runnableMicroSeconds;
+
+                    /* Has the tasks a new runnable time maximum? */
+                    if ( pTaskCtrl->maxRunnableMicroSeconds[this_ukt->currTimeVar] < runnableMicroSeconds )
+                        pTaskCtrl->maxRunnableMicroSeconds[this_ukt->currTimeVar] = runnableMicroSeconds;
+
+                    /* set the UKTs runnable time values */
+                    this_ukt->runnableMicroSeconds[this_ukt->currTimeVar] += runnableMicroSeconds;
+
+                    if ( this_ukt->maxRunnableMicroSeconds[this_ukt->currTimeVar] < runnableMicroSeconds )
+                        this_ukt->maxRunnableMicroSeconds[this_ukt->currTimeVar] = runnableMicroSeconds;
+                }
 
                 reqArg = &this_ukt->curr_quu->args.mtmi_parms;
                 /* skip old requests */
@@ -741,8 +769,6 @@ MSGD(( INFO_TEST_WILDCARD, eBuffer ));
 
                 if ( tempTaskCtrl == moveTask )
                 {
-                    if ( tempTaskCtrl->prio_flag ) this_ukt->SelfDispatch--;
-
                     tempTaskCtrl->is_in_queue = TASK_IS_IN_NON_QUEUE;
                     tempTaskCtrl->ulDynPrio   = 0; 
 
@@ -792,14 +818,13 @@ MSGD(( INFO_TEST_WILDCARD, eBuffer ));
 
                 if ( pTaskCtrl 
                   && pTaskCtrl != highestPrioTask
-                  && pTaskCtrl->SuppressTimeCollection )
+                  && pTaskCtrl->YieldTask )
                 {
                     /* we will spend some unexpected time in the run queue as runnable... */
                     /* to allow dispatcher to move us later we reenable timing collection temporary */
                     /* We also have to repeat the dispatcherEnterTime measurement which we skipped before... */
-                    pTaskCtrl->SuppressTimeCollection = false;
+                    pTaskCtrl->YieldTask = false;
                     en71_DoEnterDispatcherTiming(this_ukt, pTaskCtrl);
-                    pTaskCtrl->SuppressTimeCollection = true;
                 }
 
                 this_ukt->curr_task = highestPrioTask ;
@@ -812,21 +837,9 @@ MSGD(( INFO_TEST_WILDCARD, eBuffer ));
                 PRINT_RUN_TASK ;
                 this_ukt->state = KT_RUNNING ;
 
-                if ( pTaskCtrl 
-                  && pTaskCtrl != highestPrioTask
-                  && highestPrioTask->SuppressTimeCollection )
-                {
-                    /* we awoke after spending some unexpected time in the run queue as runnable... */
-                    /* to allow dispatcher to move us later we reenable timing collection temporary */
-                    pTaskCtrl->SuppressTimeCollection = false;
-                    en71_DoLeaveDispatcherTiming(this_ukt);
-                    pTaskCtrl->SuppressTimeCollection = true;
-                }
-                else
-                {
-                    /* if we rescheduled ourself, time collection may be suppressed */
-                    en71_DoLeaveDispatcherTiming(this_ukt);
-                }
+                /* if we rescheduled ourself(task yield), time collection may be suppressed */
+                en71_DoLeaveDispatcherTiming(this_ukt);
+                this_ukt->curr_task->YieldTask = false;
 #ifdef DEBUG_RTE
                 sp77sprintf(eBuffer, sizeof(eBuffer),
                             "LEAVE en71Dispatcher UKT%d curr_task T%d", 
@@ -862,15 +875,10 @@ WAIT_AGAIN :
 
             this_ukt->SleepingStateEnterMicroSeconds = RTESys_MicroSecTimer();
 
-#ifndef SAPDB_ALWAYS_MEASURE_UKT_SEMAPHORE_SLEEP
-            if ( KGS->fTimeMeasure )
-#endif
+            if ( this_ukt->RunningStateEnterMicroSeconds )      /* if it is set, it marks the time after wakeup below... */
             {
-                if ( this_ukt->RunningStateEnterMicroSeconds )      /* if it is set, it marks the time after wakeup below... */
-                {
-                    this_ukt->UKTRunningStat.ulCount++;
-                    this_ukt->UKTRunningStat.ulAbsMicroSeconds += (this_ukt->SleepingStateEnterMicroSeconds - this_ukt->RunningStateEnterMicroSeconds) ;
-                }
+                this_ukt->UKTRunningStat.ulCount++;
+                this_ukt->UKTRunningStat.ulAbsMicroSeconds += (this_ukt->SleepingStateEnterMicroSeconds - this_ukt->RunningStateEnterMicroSeconds) ;
             }
 
             rc = semop ( this_ukt->semid , &semrcv , 1 );
@@ -879,16 +887,11 @@ WAIT_AGAIN :
             this_ukt->UKTSleepTimeMicroSeconds[this_ukt->currTimeVar] =
                 this_ukt->RunningStateEnterMicroSeconds - this_ukt->SleepingStateEnterMicroSeconds;
 
-#ifndef SAPDB_ALWAYS_MEASURE_UKT_SEMAPHORE_SLEEP
-            if ( KGS->fTimeMeasure )
-#endif
+            /* UKTIOWaitStat.ulAbsMicroSeconds only counts the micro seconds spended waiting for DeviIO */
+            if ( this_ukt->wait_for_ioproc_io )
             {
-                /* UKTIOWaitStat.ulAbsMicroSeconds only counts the micro seconds spended waiting for DeviIO */
-                if ( this_ukt->wait_for_ioproc_io )
-                {
-                    this_ukt->UKTIOWaitStat.ulCount++;
-                    this_ukt->UKTIOWaitStat.ulAbsMicroSeconds += (this_ukt->RunningStateEnterMicroSeconds - this_ukt->SleepingStateEnterMicroSeconds) ;
-                }
+                this_ukt->UKTIOWaitStat.ulCount++;
+                this_ukt->UKTIOWaitStat.ulAbsMicroSeconds += (this_ukt->RunningStateEnterMicroSeconds - this_ukt->SleepingStateEnterMicroSeconds) ;
             }
 
             if ( rc )
@@ -1407,15 +1410,16 @@ void    e71_com ( struct TASK_TYPE * tcb )
  * ===========================================================================
  */
 
-void e71_FastEnquRex ( ten50_UKT_Control * this_ukt,
-                        tsp00_Bool          Prio )
+void e71_YieldTaskEnquRex ( ten50_UKT_Control * this_ukt,
+                            tsp00_Bool          Prio )
 {
 #undef  MF__
-#define MF__ MOD__"e71_FastEnquRex"
+#define MF__ MOD__"e71_YieldTaskEnquRex"
   struct TASK_TYPE *tcb = this_ukt->curr_task;
 
   DBGIN;
 
+  tcb->YieldTask                                     = true;
   this_ukt->RunQueue[this_ukt->RunQueueLen].Task     = tcb;
   this_ukt->RunQueue[this_ukt->RunQueueLen].FromTask = tcb;
   this_ukt->RunQueue[this_ukt->RunQueueLen].Req      = &tcb->qa;
@@ -1582,7 +1586,7 @@ void vdelay_lw ( RTE_TaskId pid )
                     ? 0 : this_ukt->delay_prio ;
         }
         
-        e71_FastEnquRex ( this_ukt, this_ukt->curr_task->prio_flag );
+        e71_YieldTaskEnquRex ( this_ukt, this_ukt->curr_task->prio_flag );
         this_ukt->curr_task->prio_flag = 0 ;
         this_ukt->delay_active++ ;
         GOTO_DISP(&this_ukt);
@@ -2098,13 +2102,13 @@ static void en71_HandleTaskStatisticReset( ten50_UKT_Control *this_ukt
 /* 
   >>>>>  Enter dispatcher timing >>>>>>>>
  */
-static void en71_DoEnterDispatcherTiming( ten50_UKT_Control *this_ukt
-                                        , struct TASK_TYPE  *pTaskCtrl )
+static void en71_DoEnterDispatcherTiming( ten50_UKT_Control *this_ukt,
+                                          struct TASK_TYPE  *pTaskCtrl )
 {
     pTaskCtrl->TimeCollectionEnabled     = KGS->fTimeMeasure;
     pTaskCtrl->MoveTimeCollectionEnabled = XPARAM(uLoadBalancingCheck);
 
-    if ( !pTaskCtrl->SuppressTimeCollection 
+    if ( !pTaskCtrl->YieldTask 
      && ( pTaskCtrl->MoveTimeCollectionEnabled || pTaskCtrl->TimeCollectionEnabled ) )
     {
         SAPDB_UInt8 timeNow = RTESys_MicroSecTimer();
@@ -2112,7 +2116,7 @@ static void en71_DoEnterDispatcherTiming( ten50_UKT_Control *this_ukt
         /* 
           <<<<<<  Enter dispatcher timing <<<<<<<<
          */
-        if ( TASK_IS_IN_RUN_QUEUE == pTaskCtrl->is_in_queue )
+        if ( TASK_IS_IN_RUN_QUEUE == pTaskCtrl->is_in_queue ) 
         {
             /* if self dispatching, reset runqueueEnqueueTime */
             /* to prevent negaitiv differenz to dispatcherEnterTime */
@@ -2130,7 +2134,7 @@ static void en71_DoLeaveDispatcherTiming(ten50_UKT_Control *this_ukt)
     /* leaving is possible for current task only... */
     struct TASK_TYPE  *pTaskCtrl = this_ukt->curr_task;
 
-    if ( !pTaskCtrl->SuppressTimeCollection 
+    if ( !pTaskCtrl->YieldTask 
      && ( pTaskCtrl->MoveTimeCollectionEnabled || pTaskCtrl->TimeCollectionEnabled ) )
     {
         pTaskCtrl->TaskStateStat.dispatcherLeaveTime = RTESys_MicroSecTimer();
