@@ -48,20 +48,6 @@
 #include "hen40.h"
 #include "hsp77.h"
 
-#if FIXED_COMSEG
-#define		SHMALIGN		SHMLBA
-#if defined(I386) || defined(_IBMR2) || defined(HP9) || defined(SUN) || defined(ALPHA)
-#define         COMSEGSIZ       XParam->ulPacketSize
-#define         MINCOMSHMSIZ    (((COMSEGSIZ+SHMALIGN-1)/SHMALIGN)*SHMALIGN)
-#define         UCBSHMSIZE      ((MINCOMSHMSIZ<SHM_RND)?SHM_RND:MINCOMSHMSIZ)
-#endif
-#if	defined(NMP)
-#define         UCBSHMSIZE              SHMALIGN
-int                                     kgsaddr = 0;
-int                                     kgssize = 0;
-#endif	/*NMP*/
-#endif	/*FIXED_COMSEG*/
-
 /* exported functions */
 
 unsigned long	e74_comdata_size ( 
@@ -132,11 +118,6 @@ void	e74_init_comdata ( RTE_XPARAM_REC		*XParam,
     struct TASK_TYPE		*tcb ;
     ten50_UKT_Control           *kp ;
     connection_info		*cip ;
-#if FIXED_COMSEG
-    big_comseg_header		*big_comseg ;
-    unsigned long		attach_addr ;
-#endif
-
 
     DBGIN;
     DBG1(( MF__, "packet size                  %10ld",
@@ -240,52 +221,6 @@ void	e74_init_comdata ( RTE_XPARAM_REC		*XParam,
 	    	THREAD_INDEX(kp), kp->connection->ci_big_size ));
     }
 
-#if FIXED_COMSEG
-    /*
-     *  Each UKT and each NET attach the big_comseg at this address.
-     */
-    big_comseg = (big_comseg_header*) ((char*) KGS->seg +
-		    ((KGS->seg_size / UCBSHMSIZE) + 1) * UCBSHMSIZE);
-    DBG1(( MF__, "common big_comseg_addr       0x%lx",
-					    (long) big_comseg ));
-    for ( kp = KGS->first_kp ; kp <= KGS->last_kp ; kp ++ )
-    {
-		kp->connection->ci_big_comseg = big_comseg ;
-    }
-
-    /*
-     *  Get the attach addresses for small comseg.
-     */
-    for ( kp = KGS->first_kp ; kp <= KGS->last_kp ; kp ++ )
-	{
-		/*
-		 *  Start behind big comseg.
-		 */
-		attach_addr = (unsigned long) big_comseg ;
-		attach_addr += (unsigned long) kp->connection->ci_big_size ;
-		attach_addr = ((attach_addr / UCBSHMSIZE) + 1) * UCBSHMSIZE ;
-
-		/*
-		 *  Reserve one for reject of connection request.
-		 */
-		kp->connection->ci_loccomseg = (comseg_header*) attach_addr ;
-		attach_addr += XParam->ulPacketSize ;
-		attach_addr = ((attach_addr / UCBSHMSIZE) + 1) * UCBSHMSIZE ;
-
-		for ( tcb = kp->pFirstTaskCtrl ; tcb ; tcb = tcb->next_task )
-		{
-			if ( tcb->connection )
-			{
-			    DBG1 (( MF__,"T%d loc_comseg 0x%lx \n",
-				    tcb->index, (long)attach_addr ));
-			    tcb->connection->ci_loccomseg = (comseg_header*) attach_addr ;
-			    attach_addr += XParam->ulPacketSize ;
-			    attach_addr = ((attach_addr / UCBSHMSIZE) + 1) * UCBSHMSIZE ;
-			}
-		}
-	}
-#endif
-
     DBGOUT;
 }
 
@@ -308,7 +243,7 @@ int     e74_create_big_comsegs ( )
 
 	DBGIN;
 
-    ref = KGS->seg_size / KGS->kgs_shm_part_len + 1 ;
+    ref = 1 ;
 
     for ( kp = KGS->first_kp ; kp <= KGS->last_kp ; kp ++ )
 	{
@@ -363,17 +298,7 @@ void	e74_init_big_comseg (ten50_UKT_Control *this_ukt )
 		return ;
     }
 
-    /*  F.H.
-     *  The address of the KGS depends on MAXUSER task-stacks,
-     *  so we should be sure that the address of the big_comseg is BEHIND the
-     *  KGS. Otherwise it is possible to get memory problems if using the
-     *  dynamic-task-stack allocation!
-     *  IST 1995-11-01
-     *  ci_big_comseg is initialized in 'init_comdata'.
-     *  It either has the right value (FIXED_COMSEG) or is NULL.
-     */
-    addr = sql41_attach_shm ( this_ukt->connection->ci_shmid ,
-				(char*) this_ukt->connection->ci_big_comseg );
+    addr = sql41_attach_shm ( this_ukt->connection->ci_shmid , (char *)0 );
     if ( ! addr )
     {
 		MSGD(( ERR_INITCOM_ATTACH_BIGCOM, sqlerrs() )) ;
