@@ -218,6 +218,7 @@ void e75_release_connection ( int reason ,
     connection_info *cip ;
     char            *errmsg ;
     tsp00_ErrTextc   errtext ;
+    int wait_for_client = waitflag;
 
 	DBGIN;
     cip = tcb->connection;
@@ -316,22 +317,47 @@ void e75_release_connection ( int reason ,
 			comseg->cs_server_ref   = 0 ;
 			comseg->cs_server_semid = 0 ;
 			comseg->cs_server_flag  = 1 ;
+            if ( SP1CE_RELEASED == comseg->cs_client_state )
+            {
+                wait_for_client = false;
+            }
 			sql32_unlock_comseg ( cip );
 
-			/*
-			 *  A locally connected application should be awakened
-			 *  by removing its semaphore.
-			 *  If called directly from vreply, NETWORK would already know
-			 *  of the released connection and complain about the
-			 *  unexpected event.
-			 */
-			if ( (cip->ci_peer_semid == -2) && (oldstate != CON_REPLIED) )
+            if ( oldstate != CON_REPLIED && wait_for_client )
 			{
-				(void) e75_wake_peer ( cip );
+                int rc = e75_wake_peer ( cip );
+                if ( rc != SP1CE_OK )
+                {
+                    int logLength ;
+                    char logbuf[512];
+    				struct tm timeBuffer;
+	    			struct tm *pTime = localtime_r( &cip->ci_connect_time, &timeBuffer );
+                    if ( pTime )
+                    {
+                        (void) sp77sprintf ( logbuf , sizeof(logbuf), "Client died to early? (e75_release_connection) Connected to T%d since %02d:%02d:%02d",
+                            tcb->index, pTime->tm_hour, pTime->tm_min, pTime->tm_sec );
+                    }
+                    else
+                    {
+                        (void) sp77sprintf ( logbuf , sizeof(logbuf), "Client died to early? Connected to T%d since ?%ld?",
+                            tcb->index, (long)cip->ci_connect_time );
+                    }
+                    logLength = strlen ( logbuf );
+                    if ( cip->ci_peer_node[0] )
+		                (void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " remote %s" , cip->ci_peer_node );
+                    else
+		                (void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " local" );
+                    logLength = strlen ( logbuf );
+                    if ( cip->ci_remote_pid[0] )
+		                (void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " remote_pid %s" , cip->ci_remote_pid );
+                    logLength = strlen ( logbuf );
+		            (void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " peer_pid %ld" , (long) cip->ci_peer_pid );
+    				MSGD (( ERR_DISTRIB_WAKE_SEMOP, logbuf ));
+                }
 			}
 		}
 
-		if ( waitflag )
+		if ( wait_for_client )
 		{
 			tcb->task_timeout = LOCL_PKT_TMO + time( (time_t *)0 ) + MAXSLEEPTIME ;
 			/*
@@ -641,6 +667,34 @@ int e75_connect_request ( struct DOUBLY_LINKED	*lquu )
      */
     DBG1 (( MF__,"waking application" ));
     rc = e75_wake_peer ( cip );
+    if ( rc != SP1CE_OK )
+    {
+        int logLength ;
+        char logbuf[512];
+    	struct tm timeBuffer;
+	    struct tm *pTime = localtime_r( &cip->ci_connect_time, &timeBuffer );
+        if ( pTime )
+        {
+            (void) sp77sprintf ( logbuf , sizeof(logbuf), "Client died to early? (e75_connect_request) Connected to T%d since %02d:%02d:%02d",
+                tcb->index, pTime->tm_hour, pTime->tm_min, pTime->tm_sec );
+        }
+        else
+        {
+            (void) sp77sprintf ( logbuf , sizeof(logbuf), "Client died to early? Connected to T%d since ?%ld?",
+                tcb->index, (long)cip->ci_connect_time );
+        }
+        logLength = strlen ( logbuf );
+        if ( cip->ci_peer_node[0] )
+		    (void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " remote %s" , cip->ci_peer_node );
+        else
+		    (void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " local" );
+        logLength = strlen ( logbuf );
+        if ( cip->ci_remote_pid[0] )
+		    (void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " remote_pid %s" , cip->ci_remote_pid );
+        logLength = strlen ( logbuf );
+		(void) sp77sprintf ( logbuf + logLength , sizeof(logbuf) - logLength, " peer_pid %ld" , (long) cip->ci_peer_pid );
+    	MSGD (( ERR_DISTRIB_WAKE_SEMOP, logbuf ));
+    }
     if ( (rc != SP1CE_OK) || (result != SP1CE_OK) )
     {
 		e74_release_comseg ( cip, tcb );
