@@ -62,6 +62,7 @@
 #include "gen84.h"
 #include "gen85.h"
 #include "gen88.h"
+#include "heo92.h"
 #include "hsp100.h"
 #include "geo57.h"
 #include "heo15.h"
@@ -91,7 +92,7 @@ extern char *getenv(const char *);
 /*===========================================================================*
  *  DEFINES                                                                  *
  *===========================================================================*/
-#if defined(LINUX)
+#if defined(LINUX) 
 #define HAS_CLONE_PROCESSES_INSTEAD_OF_THREADS
 #endif /* OSF1 */
 
@@ -249,7 +250,7 @@ static void en81_StopDatabase( int sig, siginfo_t *sigInfo, void *context );
 static void en81_CrashSignalHandler( int sig, siginfo_t *sigInfo, void *context );
 
 static  void                            en81_InitFirstDescriptors( /* void */ );
-static  void                            en81_SetSignalHandler( /* void */ );
+static  void                            en81_SetSignalHandler( tsp00_Bool suppressCore );
 static  int                             en81_FillServerDbName( char *dbname, const char *arg0 );
 /*
  * Wrapper for catching C++ exceptions not handled
@@ -401,11 +402,6 @@ externC int en81_kernel_main ( int argc , char *argv[] , char *envp[] )
      */
     en81_MemPageSize = sysconf(_SC_PAGESIZE);
 
-    /*
-     * Setup SignalHandler for common signals
-     */
-    en81_SetSignalHandler();
-
     /*
      * Start in debugger if wanted and possible...
      */
@@ -517,11 +513,13 @@ externC int en81_kernel_main ( int argc , char *argv[] , char *envp[] )
 
     if ( rc == 0 )
     {
+        int waitpidErrno = 0;
+
         runKernelPid = fork();
         switch ( runKernelPid )
         {
         case 0:
-            en81_SetSignalHandler();
+            en81_SetSignalHandler( XPARAM(suppressCore) );
 		    /* on OSF1 the kernel with overwritten operator new crashes because
              * in the startup phase the LIBC seems to allocate memory from
              * the not overwritten operator new but deletes it with overwritten
@@ -561,6 +559,7 @@ externC int en81_kernel_main ( int argc , char *argv[] , char *envp[] )
 			if ( 0 == rc )
 			{
 				if ( 0 != KGS 
+                  && XPARAM(suppressCore)
 			      && SERVER_KILL == KGS->state )
 				{
 					static SAPDB_Int4 deadCheck = 0;
@@ -576,10 +575,37 @@ externC int en81_kernel_main ( int argc , char *argv[] , char *envp[] )
 				sleep(1); /* sleep for a second */
 				continue;
 			}
+            else if ( -1 == rc )
+            {
+                waitpidErrno = errno;
+            }
         } while ( ((rc == -1) && (errno == EINTR)) || (0 == rc) );
 
-        if ( rc != runKernelPid ) /* check for valid runKernelExitCode */
+        if ( KGS )
         {
+            en60_ReopenKnldiagFile( KGS->XParam );
+        }
+
+        if ( -1 == rc )
+        {
+            if ( ECHILD == waitpidErrno )
+            {
+                MSGD(( WRN_WAITPID_FOR_NO_CHILD ));
+                runKernelExitCode = 0; /* be friendly to linux users... */
+            }
+            else
+            {
+                errno = waitpidErrno; /* for sqlerrs()... */
+                MSGD(( ERR_WAITPID_ERROR, waitpidErrno, sqlerrs() ));
+            }
+        }
+        else if ( 0 == rc )
+        {
+            MSGD(( ERR_WAITPID_TIMEOUT, MAX_DEADCHECK_SLEEP_SECONDS ));
+        }
+        else if ( rc != runKernelPid ) /* check for valid runKernelExitCode */
+        {
+            MSGD(( ERR_WAITPID_WRONG_PID, (long)rc, (long)runKernelPid ));
             runKernelExitCode = KERNEL_TERMINATED_BY_HUP_SIGNAL; /* mark as terminated by HUP signal ... */
         }
 
@@ -726,17 +752,16 @@ externC int en81_kill_database ( )
 #define MF__ MOD__"en81NotifyStarter"
 void en81NotifyStarter()
 {
-    if ( KGS->ulStartupOptions & FORCE_AUTORESTART )
+    if ( KGS->ulStartupOptions & FORCE_AUTORESTART
+      && RTE_SpawnRestartShutdown((void *)1) )
     {
-        RTE_SpawnRestartShutdown((void *)1);
+        return;
     }
-    else
+
+    if(RTE_DiagPipeIsOpen())
     {
-        if(RTE_DiagPipeIsOpen())
-        {
-            RTE_CloseDiagPipe();
-            (void)en41RemovePipeFDFile(en81_dbname);
-        }
+        RTE_CloseDiagPipe();
+        (void)en41RemovePipeFDFile(en81_dbname);
     }
 }
 
@@ -780,6 +805,79 @@ externC void en81TimerThreadWakeup()
 /*===========================================================================*
  *  LOCAL FUNCTIONS (CODE)                                                   *
  *===========================================================================*/
+/*
+   XParam independend startup information
+ */
+static void ShowXParamIndependendStatupInformation(tsp00_Versionc currentVersion, tsp00_NodeIdc dbnode)
+{
+    MSGALL(( INFO_DB_START_SERVERDB, en81_dbname ));
+    {
+        time_t     time_now;
+        struct tm  tm_now;
+        struct tm *now;
+
+        time_now = time((time_t *)0);
+        now = en52Localtime_r(&time_now, &tm_now);
+        if ( now )
+        {
+            MSGALL(( INFO_DB_START_DATE, now->tm_year + 1900, now->tm_mon + 1, now->tm_mday ));
+        }
+        else
+        {
+            MSGALL(( INFO_DB_START_DATE, 0, 0, 0 ));
+        }
+    }
+    MSGALL(( INFO_DB_START_PID, getpid() ));
+
+    {
+        struct passwd *pw;
+        pw = getpwuid(geteuid());
+        if ( pw )
+        {
+            MSGALL(( INFO_DB_START_OWNER, pw->pw_name ));
+        }
+    }
+
+    {
+        struct group *grp;
+        grp = getgrgid(getegid());
+        if ( grp )
+        {
+            MSGALL(( INFO_DB_START_GROUP, grp->gr_name ));
+        }
+    }
+   
+#ifdef BIT64
+    MSGALL(( INFO_DB_START_VERSION, "64BIT Kernel" ));
+#else
+    MSGALL(( INFO_DB_START_VERSION, "32BIT Kernel" ));
+#endif
+    MSGALL(( INFO_DB_START_VERSION, &currentVersion[0] ));
+    MSGALL(( INFO_DB_START_VERSION, RTE_GetSpeed() ));
+
+    /* 
+        http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1127694
+    */
+    {
+        SAPDB_Char cpuType[64];
+        RTESys_GetCPUType(cpuType,64);
+        MSGALL(( INFO_DB_START_MACHINE, cpuType ));
+    }
+
+    {
+        tsp00_Pathc currentDirectory;
+        if ( getcwd((char *)currentDirectory, sizeof(tsp00_Path)) != 0 )
+        {
+            MSGALL(( INFO_DB_START_DBROOT, &currentDirectory[0] ));
+        }
+    }
+
+    MSGALL(( INFO_DB_START_LIBPATH, LDLIBPATH_IP00,
+            getenv(LDLIBPATH_IP00) ? getenv(LDLIBPATH_IP00) : ""));
+}
+
+/*----*/
+
 #undef  MF__
 #define MF__ MOD__"en81_Startup"
 static  int     en81_Startup ( tsp00_Versionc currentVersion  , char *envp[] )
@@ -802,7 +900,9 @@ static  int     en81_Startup ( tsp00_Versionc currentVersion  , char *envp[] )
 		MSGCD (( ERR_MISSING_HOSTNAME )) ;
 		return ( 2 );
     }
-    DBG1 (( MF__,"dbnode  '%s' \n", dbnode ));
+
+    ShowXParamIndependendStatupInformation( currentVersion, dbnode);
+
 #ifdef SOCKETS
     rc = sql43_get_official_node_name ( dbnode , dbnode ,
 					sizeof(tsp00_NodeId) );
@@ -864,10 +964,6 @@ static  int     en81_Startup ( tsp00_Versionc currentVersion  , char *envp[] )
 		MSGCD (( ERR_CANT_OPEN_DIAGFILES ));
 		return ( 2 );
     }
-    /*
-     *  Issue start message
-     */
-    MSGALL(( INFO_DB_START_SERVERDB, en81_dbname ));
     /* If found, setup OfficialNode from parameter file */
     if ( XparamParameters.szOfficialNode[0] != '\0' )
     {
@@ -880,60 +976,6 @@ static  int     en81_Startup ( tsp00_Versionc currentVersion  , char *envp[] )
         MSGALL(( INFO_DB_START_SERVERNODE, dbnode ));
     }
 
-    {
-        time_t     time_now;
-        struct tm  tm_now;
-        struct tm *now;
-
-        time_now = time((time_t *)0);
-        now = en52Localtime_r(&time_now, &tm_now);
-        if ( now )
-        {
-            MSGALL(( INFO_DB_START_DATE, now->tm_year + 1900, now->tm_mon + 1, now->tm_mday ));
-        }
-        else
-        {
-            MSGALL(( INFO_DB_START_DATE, 0, 0, 0 ));
-        }
-    }
-    MSGALL(( INFO_DB_START_PID, getpid() ));
-
-    {
-        struct passwd *pw;
-        pw = getpwuid(geteuid());
-        if ( pw )
-        {
-            MSGALL(( INFO_DB_START_OWNER, pw->pw_name ));
-        }
-    }
-
-    {
-        struct group *grp;
-        grp = getgrgid(getegid());
-        if ( grp )
-        {
-            MSGALL(( INFO_DB_START_GROUP, grp->gr_name ));
-        }
-    }
-   
-#ifdef BIT64
-    MSGALL(( INFO_DB_START_VERSION, "64BIT Kernel" ));
-#else
-    MSGALL(( INFO_DB_START_VERSION, "32BIT Kernel" ));
-#endif
-    MSGALL(( INFO_DB_START_VERSION, &currentVersion[0] ));
-    /* 
-        http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1127694
-    */
-    {
-        SAPDB_Char cpuType[64];
-        RTESys_GetCPUType(cpuType,64);
-        MSGALL(( INFO_DB_START_MACHINE, cpuType ));
-    }
-    MSGALL(( INFO_DB_START_DBROOT, XparamParameters.szRunDirectory ));
-    MSGALL(( INFO_DB_START_LIBPATH, LDLIBPATH_IP00, getenv(LDLIBPATH_IP00) ? getenv(LDLIBPATH_IP00) : ""));
-
-
     /*
      * PTS 1116686
      */
@@ -950,6 +992,11 @@ static  int     en81_Startup ( tsp00_Versionc currentVersion  , char *envp[] )
         en81_AllowCoreDump();
     }
 
+    /*
+     * Setup SignalHandler for common signals
+     */
+    en81_SetSignalHandler( XparamParameters.suppressCore );
+
     /* Memory allocation limit etc... */
     RTEConf_HandleSpecialParametersWithMessageOutput();
 
@@ -1605,9 +1652,9 @@ static void en81_StopAction()
     if ( KGS != NULL )
 	{
         if ( KGS->state == SERVER_ONLINE_EO00 
-          && KGS->gracefully )
+          && KGS->gracefully
+          && RTE_SpawnRestartShutdown((void *)0) )
         {
-            RTE_SpawnRestartShutdown((void *)0);
             return;
         }
 
@@ -1867,7 +1914,9 @@ static void en81_CrashSignalHandler( int sig, siginfo_t *sigInfo, void *context
     eo670_CTraceStackInit();
     eo670_CTraceContextStack(context);
 
-    LVCMem_SignalAnalysis();
+    if ( KGS ) LVCMem_SignalAnalysis();
+
+    if ( KGS ) en81ShowMaxStackUse();
 
     if ( en50NoCoreDumpWanted == false )
     {
@@ -1967,7 +2016,7 @@ static void en81_HupSignaled( int sig, siginfo_t *sigInfo, void *context )
 
 #undef MF__
 #define MF__ MOD__"en81_SetSignalHandler"
-static void en81_SetSignalHandler()
+static void en81_SetSignalHandler( tsp00_Bool suppressCore )
 {
 #ifdef USE_SIGALTSTACK
     stack_t newStack;
@@ -2009,16 +2058,18 @@ static void en81_SetSignalHandler()
     (void) en81_SetSigAction ( SIG_FORK_AND_EXIT , en81_ForkAndExit );
 
 #ifndef CRASH_IMMEDIATLY
-    /*
-     *  These should crash the kernel.
-     */
-    (void) en81_SetSigAction ( SIGILL  , en81_CrashSignalHandler );
-    (void) en81_SetSigAction ( SIGTRAP , en81_CrashSignalHandler );
-    (void) en81_SetSigAction ( SIGABRT , en81_CrashSignalHandler );
-    (void) en81_SetSigAction ( SIGFPE  , en81_CrashSignalHandler );
-    (void) en81_SetSigAction ( SIGBUS  , en81_CrashSignalHandler );
-    (void) en81_SetSigAction ( SIGSEGV , en81_CrashSignalHandler );
-    (void) en81_SetSigAction ( SIGXCPU , en81_CrashSignalHandler );
+    if ( suppressCore )
+    {
+        /*
+         *  These should crash the kernel.
+         */
+        (void) en81_SetSigAction ( SIGILL  , en81_CrashSignalHandler );
+        (void) en81_SetSigAction ( SIGTRAP , en81_CrashSignalHandler );
+        (void) en81_SetSigAction ( SIGABRT , en81_CrashSignalHandler );
+        (void) en81_SetSigAction ( SIGFPE  , en81_CrashSignalHandler );
+        (void) en81_SetSigAction ( SIGBUS  , en81_CrashSignalHandler );
+        (void) en81_SetSigAction ( SIGSEGV , en81_CrashSignalHandler );
+        (void) en81_SetSigAction ( SIGXCPU , en81_CrashSignalHandler );
 #if defined(AIX)
 /*AIX*/   (void) en81_SetSigAction ( SIGIOT  , en81_CrashSignalHandler );
 #endif
@@ -2026,7 +2077,7 @@ static void en81_SetSignalHandler()
 /*!LINUX*/ (void) en81_SetSigAction ( SIGEMT  , en81_CrashSignalHandler );
 /*!LINUX*/ (void) en81_SetSigAction ( SIGSYS  , en81_CrashSignalHandler );
 #endif
-
+    }
 #endif /* !CRASH_IMMEDIATLY */
 
 }
@@ -2043,6 +2094,23 @@ static void en81_InitFirstDescriptors(void)
     if ( devNullFd > 2 ) close(devNullFd);
 }
 
+/*---*/
+
+void en81ShowMaxStackUse()
+{
+    if ( XPARAM(showMaxStackUse) )
+    {
+        struct TASK_TYPE *tcb;
+
+        for ( tcb = KGS->pFirstTaskCtrl ; tcb <= KGS->pLastTaskCtrl; tcb++ )
+        {
+            vsShowMaxStackUse( tcb->index, eo92GetTaskTypeName(tcb->type) );
+        }
+    }
+}
+
+/*---*/
+
 #undef MF__
 #define MF__ MOD__"en81RunKernel"
 externC int en81RunKernel(struct ten50_KT_Control *pMainThread)
@@ -2051,6 +2119,12 @@ externC int en81RunKernel(struct ten50_KT_Control *pMainThread)
 
     DBGIN;
 
+    MSGALL(( INFO_DB_START_SERVERDB, "+++++++++++++++++++++++++++++++++++++++++++++++++" ));
+    MSGALL(( INFO_DB_START_SERVERDB, "+++++++++++++++++++ Kernel Forked +++++++++++++++" ));
+    MSGALL(( INFO_DB_START_SERVERDB, en81_dbname ));
+    MSGALL(( INFO_DB_START_PID, getpid() ));
+    MSGALL(( INFO_DB_START_SERVERDB, "+++++++++++++++++++++++++++++++++++++++++++++++++" ));
+
     /*
      * Start in debugger if wanted and possible...
      */
@@ -2060,8 +2134,8 @@ externC int en81RunKernel(struct ten50_KT_Control *pMainThread)
     }
 
     KGS->traceWriterGone = 0;
-
     KGS->pid = getpid();
+
     if ( en41SetPIDInFile( en81_dbname, KGS->pid ) != 0 )
     {
         return 1;
@@ -2122,6 +2196,9 @@ externC int en81RunKernel(struct ten50_KT_Control *pMainThread)
         MSGALL (( ERR_RTE_ABORT_EXEPT ));
     }
 
+    /* PTS 1110953 */
+    en81ShowMaxStackUse();
+
     /*
      *  Issue stop message
      */
@@ -2134,17 +2211,6 @@ externC int en81RunKernel(struct ten50_KT_Control *pMainThread)
     {
         KGS->traceNeedsSaving = 0;
 
-        /* PTS 1110953 */
-        if ( XPARAM(showMaxStackUse) )
-        {
-            struct TASK_TYPE *tcb;
-
-            for ( tcb = KGS->pFirstTaskCtrl ; tcb <= KGS->pLastTaskCtrl; tcb++ )
-            {
-                vsShowMaxStackUse( tcb->index, eo92GetTaskTypeName(tcb->type) );
-            }
-        }
-
         MSGALL (( INFO_DB_STOPPED ));
 
 		rc = 0 ;
@@ -2330,7 +2396,6 @@ static void en81_LastWords(int runKernelExitCode)
 {
     int lastFd;
 
-    en60_ReopenKnldiagFile( KGS->XParam );
 
     /* Reopen message file for Kernel ExitCode Information */
     if ( 0 == runKernelExitCode )
@@ -2343,13 +2408,18 @@ static void en81_LastWords(int runKernelExitCode)
     }
     else
     {
+        /* If Kernel failed, look for an existing rtedump. 
+         * If nothing or an old version found, dump it from shared memory segment 
+         * In case of crash the dump may not have been written... */
+        en54DumpNewRteDumpOnly();
+
 #ifndef WCOREDUMP
 #define WCOREDUMP(e_) (((e_) & 0200) == 0200)
 #endif
         /* Show exit code with/without core */
         if ( WCOREDUMP(runKernelExitCode) )
         {
-            chmod("core", 0660); /* make core readable for sdba group */
+            system( "chmod g+r core*" ); /* make core readable for sdba group */
             MSGD(( INFO_KERNEL_EXIT_WITH_CORE, runKernelExitCode ));
         }
         else
@@ -2439,14 +2509,6 @@ static void en81_LastWords(int runKernelExitCode)
     eo15CompleteMessageFiles( XPARAM(ulMaxMessageFiles),
                                     (teo15MessageBufferSpace *)KGS->MessageBufferSpace );
 
-    /* If Kernel failed, look for an existing rtedump. 
-     * If nothing or an old version found, dump it from shared memory segment 
-     * In case of crash the dump may not have been written... */
-    if ( runKernelExitCode != 0 )
-    {
-        en54DumpNewRteDumpOnly();
-    }
-
 	/* PTS 1113008
 	  Since kernel does no longer inform the clients, they are informed now....
 	 */
@@ -2904,11 +2966,11 @@ static void en81OutputResourceLimits(RTE_XPARAM_REC *XparamParameters  , char *e
     char **pEnv;
 
     getrlimit(RLIMIT_CPU,&limit);
-    en81WriteResourceLimitToString((SAPDB_UInt8)limit.rlim_cur,value,true,RLIM_INFINITY);
+    en81WriteResourceLimitToString((SAPDB_UInt8)limit.rlim_cur,value,false,RLIM_INFINITY);
     MSGD ((INFO_MAX_CPU_TIME,value));
 #ifdef RLIMIT_NPROC
     getrlimit(RLIMIT_NPROC,&limit);
-    en81WriteResourceLimitToString((SAPDB_UInt8)limit.rlim_cur,value,true,RLIM_INFINITY);
+    en81WriteResourceLimitToString((SAPDB_UInt8)limit.rlim_cur,value,false,RLIM_INFINITY);
     MSGD ((INFO_MAX_NUMBER_OF_PROCESSES,value));
 #endif
     getrlimit(RLIMIT_CORE,&limit);
@@ -2977,7 +3039,7 @@ http://pts:1080/webpts?wptsdetail=yes&ErrorType=1&ErrorID=1125631
     MSGD(( INFO_ENVIRONMENT_DUMP_START ));
     for ( pEnv = envp; pEnv && *pEnv; ++pEnv )
     {
-#define MAX_ENV_PART 40
+#define MAX_ENV_PART 114
         char  EnvBuffer[MAX_ENV_PART+2];
         char  *pEnvPart;
         int   len;
