@@ -551,7 +551,7 @@ struct TASK_TYPE **pTcb )
          *  are still alive.
          */
         DBG1 (( MF__,"testing applications \n" ));
-        for ( tcb = KGS-> first_event_task ; tcb <= KGS-> first_backup_task ; tcb ++ )
+        for ( tcb = KGS-> first_event_task ; tcb < KGS-> first_backup_task ; tcb ++ )
         {
 			e84_test_application ( tcb );
             sqlyieldthread();
