@@ -128,9 +128,16 @@ tsp00_TaskId &taskId )
 {
 #undef  MF__
 #define MF__ MOD__"vgetpid"
-    struct TASK_TYPE *tcb = RTE_ITaskStackControl::Instance().GetTaskControl();
+    if ( !KGS || !KGS->XParam )
+    {
+        taskId = 0;
+    }
+    else
+    {
+        struct TASK_TYPE *tcb = RTE_ITaskStackControl::Instance().GetTaskControl();
 
-    taskId = ( 0 == tcb ? 0 : tcb->index );
+        taskId = ( 0 == tcb ? 0 : tcb->index );
+    }
 }
 
 /*---------------------------------------------------------------------------
@@ -141,6 +148,9 @@ tsp00_TaskId &taskId )
  */
 RTE_TaskId RTE_GetOwnTaskId()
 {
+    if ( !KGS || !KGS->XParam )
+        return 0;
+
     struct TASK_TYPE *tcb = RTE_ITaskStackControl::Instance().GetTaskControl();
     return (RTE_TaskId)( 0 == tcb ? 0 : tcb->index );
 }
@@ -151,6 +161,9 @@ RTE_TaskId RTE_GetOwnTaskId()
  */
 RTE_CoroutineContext *RTE_GetTaskContext(RTE_TaskId id)
 {
+    if ( !KGS || !KGS->XParam )
+        return 0;
+
     return (KGS->pFirstTaskCtrl + (id-1))->taskContext;
 }
 
@@ -165,6 +178,9 @@ extern "C" ten50_UKT_Control *en88GetThisUkt()
 {
 #undef MF__
 #define MF__ MOD__ "en88GetThisUkt"
+    if ( !KGS || !KGS->XParam )
+        return 0;
+
     struct TASK_TYPE *tcb = RTE_ITaskStackControl::Instance().GetTaskControl();
 
     return ( 0 == tcb ? (ten50_UKT_Control *)0 : tcb->ukt );
@@ -629,3 +645,7 @@ MSGD(( INFO_TEST_WILDCARD, eBuffer ));
 }
 
 
+extern "C" void en88memcpy(void *a,void *b,SAPDB_UInt s)
+{
+    memcpy(a,b,s);
+}
