@@ -258,6 +258,18 @@
  *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
  *===========================================================================*/
 
+#if !defined(_WIN32)
+typedef enum _RTE_SimulateVectorIo
+{
+    RTE_SimVioUnknown = 0,
+    RTE_SimVioNever   = 1,
+    RTE_SimVioIfOpenDirect = 2,
+    RTE_SimVioIfRawDevice = 3,
+    RTE_SimVioIfOpenDirectOrRawDevice = 4,
+    RTE_SimVioAlways = 5
+} RTE_SimulateVectorIo;
+#endif
+
 /* --- RTE_XPARAM_REC ------------------------------------------------------ */
 
 typedef struct  dev_descr
@@ -377,13 +389,14 @@ typedef struct  rte_xparam_record
 /***************************************************************/
   tsp00_VFilenamec            szOpMsg1;
   tsp00_VFilenamec            szOpMsg2;
-  tsp00_Int                   lShmChunk;
   tsp00_Int                   lShmKernel;
   tsp00_Bool                  fUseProcessScope;          /* PTS 1109046 */
   tsp00_Bool                  suppressCore;
   tsp00_Bool                  fUseCoroutines;
   tsp00_Bool                  fUseStackOnStack;
   tsp00_Bool                  fUseUcontext;
+  tsp00_C32c                  szVectorIoMode;            /* PTS 1132877 */
+  RTE_SimulateVectorIo        simulateVectorIo;          /* see RTE_SimulateVectorIo above PTS 1132876 */
 /***************************************************************/
 #endif
 /***************************************************************/
@@ -413,6 +426,7 @@ typedef struct  rte_xparam_record
   tsp00_Bool                  allowDemangle;
   tsp00_Bool                  fLogMirrored;
   tsp00_Bool                  fSetVolumeLock;
+  tsp00_Bool                  fForbidLoadBalancing;
 } RTE_XPARAM_REC;
 
 /*===========================================================================*
