@@ -90,7 +90,7 @@ static SUSPEND_STATE_REASON SuspendReasonText[MAX_SUSPEND_REASONS] =
     {  "Vsuspend (037)" , "                                        " } ,
     {  "Vsuspend (038)" , "                                        " } ,
     {  "Vsuspend (039)" , "                                        " } ,
-    {  "Vsuspend (040)" , "                                        " } ,
+    {  "IOWait(W)(040)" , "bd13WriteExclusiveLockedRoot            " } ,
     {  "IOWait(R)(041)" , "b13get_node: await read              :-(" } ,
     {  "IOWait(W)(042)" , "b13get_node                          :-(" } ,
     {  "DC OvFlow(043)" , "b13get_node: sysbuf_overflow (SVP)   :-(" } ,
@@ -99,11 +99,11 @@ static SUSPEND_STATE_REASON SuspendReasonText[MAX_SUSPEND_REASONS] =
     {  "DC OvFlow(046)" , "bd13frame_for: sysbuf_overflow       :-(" } ,
     {  "Vsuspend (047)" , "                                        " } ,
     {  "Vsuspend (048)" , "                                        " } ,
-    {  "No-Work  (049)" , "Trace_Manager::WaitForWork              " } ,
+    {  "Vsuspend (049)" , "                                        " } ,
     {  "Vsuspend (050)" , "                                        " } ,
     {  "VTRACE IO(051)" , "Trace_Manager::PrepareAndWaitForFlush   " } ,
     {  "InitState(052)" , "                                        " } ,
-    {  "No-Work  (053)" , "Pager_Writer::Run                       " } ,
+    {  "Vsuspend (053)" , "                                        " } ,
     {  "Vsuspend (054)" , "Pager_Controller::WaitForDwReply        " } ,
     {  "Vsuspend (055)" , "                                        " } ,
     {  "FreeWait (056)" , "bd13frame_for_new_node                  " } ,
@@ -261,7 +261,7 @@ static SUSPEND_STATE_REASON SuspendReasonText[MAX_SUSPEND_REASONS] =
     {  "Vsuspend (208)" , "                                        " } ,
     {  "Vsuspend (209)" , "k90receiver_process                     " } ,
     {  "Vsuspend (210)" , "k90sender_process                       " } ,
-    {  "Vsuspend (211)" , "k90sreceiver_server                     " } ,
+    {  "Vsuspend (211)" , "                                        " } ,
     {  "Vsuspend (212)" , "kb90parent_wait                         " } ,
     {  "Vsuspend (213)" , "k91integrate_prot                       " } ,
     {  "Vsuspend (214)" , "k96client_wait                          " } ,
@@ -278,14 +278,14 @@ static SUSPEND_STATE_REASON SuspendReasonText[MAX_SUSPEND_REASONS] =
     {  "Vsuspend (225)" , "kb57wait_for_stamp_flush                " } ,
     {  "Vsuspend (226)" , "kb39io_remote                           " } ,
     {  "Vsuspend (227)" , "                                        " } ,
-    {  "No-Work  (228)" , "bd91ObjGarbageCollector                 " } ,
+    {  "Vsuspend (228)" , "RedoReader waits for end of all RedoTask" } ,
     {  "Vsuspend (229)" , "bd91StopAllGarbageCollectors            " } ,
     {  "Prep-End (230)" , "bd13GetNode                             " } ,
     {  "NoRedoJob(231)" , "Rst_RedoTrafficControl::ExecuteJobs()   " } ,
     {  "SyncRedo (232)" , "Rst_RedoTrafficControl::WaitForIoSeq()  " } ,
     {  "LogQ FULL(233)" , "Log_Queue::ReserveSpace              :-(" } ,
     {  "LogIOwait(234)" , "Log_Queue::UserTaskEOTReady             " } ,
-    {  "No-Work  (235)" , "Log_Writer                              " } ,
+    {  "Vsuspend (235)" , "                                        " } ,
     {  "Vsuspend (236)" , "Conv_FreePageNoWaitDuringSavepoint      " } ,
     {  "Vsuspend (237)" , "Conv_ExpandWaitDuringSavepoint          " } ,
     {  "Vsuspend (238)" , "Conv_WaitForConvPageIOToGetSpecPageNo   " } ,
@@ -305,10 +305,11 @@ static SUSPEND_STATE_REASON SuspendReasonText[MAX_SUSPEND_REASONS] =
     {  "Vsuspend (252)" , "HS::Admin: sync susp. until reader init " } ,
     {  "Vsuspend (253)" , "                                        " } ,
     {  "Vsuspend (254)" , "                                        " } ,
-    {  "Vsuspend (255)" , "                                        " }
+    {  "No-Work  (255)" , "Task is waiting for work                " }
   } ;
 
-  
+#define TSK_VSUSPEND_REASON_NO_WORK                         255  
+
 #define SUSPEND_REASON_INV_SEL_SERVER_REQUEST            70
 #define SUSPEND_REASON_INV_SEL_SERVER_RESERVE_ROW_BUFFER 71
 #define SUSPEND_REASON_INV_SEL_ITER_SUSPEND              72
