@@ -1,3 +1,4 @@
+
 /*!
   @file           heo01.h
   @author         JoergM
@@ -336,6 +337,27 @@ sqlGetIndependentPgmPath      ( tsp00_Pathc VAR_ARRAY_REF    PgmPath,
                                 int                          TerminateWithDelimiter,
                                 tsp01_RteError              *RteError ) ;
 
+/* ------------------------------------------------------------------
+ * PUBLIC FUNCTION sqlGetIndependentPgmPath
+ * ------------------------------------------------------------------
+ * purpose:   gets the release independent "lib" path
+ *
+ * parameters:
+ *   LIBPath                  - path to the "lib" directory
+ *                              Path is a C ( null terminated ) string
+ *   TerminateWithDelimiter   - TERM_WITHOUT_DELIMITER_EO01
+ *                            - TERM_WITH_DELIMITER_EO01
+ *   RteError                 - RTE / OS errorcode and errortext
+ *                              ( if function returned FALSE )
+ *
+ * return - TRUE : success
+ *          FALSE: failed, for more information see "RteError"
+ */
+externC tsp00_Bool
+sqlGetIndependentLibPath	   ( tsp00_Pathc VAR_ARRAY_REF  LIBPath,
+                                 int						TerminateWithDelimiter,
+                                 tsp01_RteError			   *pRteError );
+
 /* ------------------------------------------------------------------
  * PUBLIC FUNCTION sqlGetIndependentBinPath
  * ------------------------------------------------------------------
@@ -504,6 +526,24 @@ sqlGetIndependentConfigPath   ( tsp00_Pathc  VAR_ARRAY_REF   ConfigPath,
 externC char *sqlGetConfigPath ( tsp00_Pathc VAR_ARRAY_REF ConfigPath,
                                  int                       TerminateWithDelimiter ) ;
 
+
+/*!
+   @brief          Update LIBPATH environment for NI
+   @param          RteError [out] the error return
+   @return         false if update failed (RteError is filled), true if successfull
+
+
+   Used for starting the niserver so that it is guarantied that the shared objects needed for 
+   NI comunication are found (e.g. libsapu16.so).
+
+   It updates the shared library paths environment by prefix our libary path (<IndepPgmPath>/lib).
+ */
+
+externC tsp00_Bool
+sqlUpdateNiLibPathEnvironment(tsp01_RteError *pRteError);
+
+
+
 /* ------------------------------------------------------------------
  * PUBLIC FUNCTION sqlGetDbrootPath (use within vos78.cpp PTS 1107355)
  * ------------------------------------------------------------------
