@@ -142,7 +142,7 @@ sqlfopenp (
 externC void
 sqlfopenEncodedc (
     const char *filename,
-    const tsp77encoding * fileEncoding,
+    const tsp77encoding ** fileEncoding,
     const tsp77encoding * clientEncoding,
     tsp05_RteFileMode_Param fileMode,
     tsp05_RteBufferingKind_Param buffering,
