@@ -174,10 +174,12 @@ enum ProgramExecPathType
 #define SQLDB_CONS   "x_cons"  , "xcons" , DbrootPGM, "cons.exe\0"       , NO_TERM
 #define SQLDB_SHOW   "x_show"  , "xshow" , DbrootPGM, "show.exe\0"       , NO_TERM
 #define SQLDB_SERV   "x_server","xserver", IndependentPGM, "serv.exe\0"  , NO_TERM
+#define SQLDB_PING   "x_ping" , NO_ALTERN, DbrootPGM, "sqlping.exe\0"       , NO_TERM
 #define SQLDB_DIAG   "x_diag" , "x_diagnose", DbrootPGM, "diagnose.exe\0", INIT_TERM
 #define SQLDB_QUERY  "xquery" , NO_ALTERN, DbrootPGM, "query.exe\0"      , INIT_TERM
 #define SQLDB_LOAD   "xload"  , NO_ALTERN, DbrootPGM, "load.exe\0"       , INIT_TERM
 #define SQLDB_PYTHON "x_python",NO_ALTERN, DbrootPGM, "pysapdb.exe\0"    , NO_TERM
+#define SQLDB_PYTHON23 "x_python23",NO_ALTERN, DbrootPGM, "py23sapdb.exe\0"    , NO_TERM
 #define SQLDB_EASY   "xeasy"  , NO_ALTERN, DbrootPGM, "easy.exe\0"       , INIT_TERM
 #define SQLDB_PL     "xpl"    , NO_ALTERN, DbrootPGM, "sqlpl.exe\0"      , INIT_TERM
 #define SQLDB_PLRUN  "xplrun" , NO_ALTERN, DbrootPGM, "plrun.exe\0"      , INIT_TERM
@@ -219,7 +221,9 @@ int e952_call_stop(struct program_id*,int, char**);
 int e952_call_console(struct program_id*,int, char**);
 int e952_call_show(struct program_id*,int, char**);
 int e952_call_vserver(struct program_id*,int, char**);
+int eo952CallPing(struct program_id*,int, char**);
 int e953_call_python(struct program_id*,int, char**);
+int e953_call_python23(struct program_id*,int, char**);
 int e950_call_component(struct program_id*,int, char**);
 int e952_call_sqlfilter(struct program_id*,int, char**);
 int e952_call_backup(struct program_id*,int, char**);
@@ -266,10 +270,12 @@ int eo954PrintSysRC(struct program_id*,int, char**);
 #define SQLDB_CLEAR  "x_clear", NO_ALTERN, DbrootPGM, "clr_kernel\0clr_ps_ipc\0",NO_TERM
 #define SQLDB_CONS   "x_cons" , NO_ALTERN, DbrootPGM, "cons\0console\0"     , NO_TERM
 #define SQLDB_SERV   "x_server",NO_ALTERN, IndependentPGM, "vserver\0"     , NO_TERM
+#define SQLDB_PING   "x_ping" , NO_ALTERN, DbrootPGM, "sqlping\0"        , NO_TERM
 #define SQLDB_NISERV "x_niserver",NO_ALTERN, IndependentPGM, "niserver\0"    , NO_TERM
 #define SQLDB_QUERY  "xquery" , NO_ALTERN, DbrootPGM, "query\0"       , INIT_TERM
 #define SQLDB_LOAD   "xload"  , "xdbload", DbrootPGM, "load\0"        , INIT_TERM
 #define SQLDB_PYTHON "x_python",NO_ALTERN, DbrootPGM, "python\0"      , NO_TERM
+#define SQLDB_PYTHON23 "x_python23",NO_ALTERN, DbrootPGM, "python23\0"      , NO_TERM
 #define SQLDB_EASY   "xeasy"  , NO_ALTERN, DbrootPGM, "easy\0"        , INIT_TERM
 #define SQLDB_PL     "xpl"    , NO_ALTERN, DbrootPGM, "sqlpl\0"       , INIT_TERM
 #define SQLDB_PLRUN  "xplrun" , NO_ALTERN, DbrootPGM, "plrun\0"       , INIT_TERM
@@ -311,7 +317,9 @@ int e952_call_show() ;
 int e952_call_regcomp(); /* PTS 1108471 */
 int e952_call_vserver() ;
 int eo952CallNiServer() ;
+int eo952CallPing() ;
 int e953_call_python() ;
+int e953_call_python23() ;
 int e950_call_component() ;
 int e952_call_sqlfilter() ;
 int e952_call_clear() ;
@@ -396,6 +404,7 @@ static PROG_ID  proglist[] = {
                 { SQLDB_QUERY , e950_call_component , query_hlpfiles } ,
                 { SQLDB_LOAD  , e950_call_component , load_hlpfiles } ,
                 { SQLDB_PYTHON, e953_call_python    , NO_HELP } ,
+                { SQLDB_PYTHON23, e953_call_python23    , NO_HELP } ,
                 { SQLDB_EASY  , e950_call_component , easy_hlpfiles } ,
                 { SQLDB_PL    , e950_call_component , pl_hlpfiles } ,
                 { SQLDB_PLRUN , e950_call_component , plrun_hlpfiles } ,
@@ -415,6 +424,7 @@ static PROG_ID  proglist[] = {
                 { SQLDB_GWSTART, e952_call_gwstart,  gwstart_hlpfiles} ,
                 { SQLDB_STOP  , e952_call_stop ,      stop_hlpfiles } ,
                 { SQLDB_SERV  , e952_call_vserver   , server_hlpfiles } ,
+                { SQLDB_PING  , eo952CallPing , NO_HELP } ,
                 { SQLDB_FILTER, e952_call_sqlfilter , NO_HELP } ,
                 { SQLDB_FILTER_RO, e952_call_sqlfilter , NO_HELP } ,
 #ifdef WIN32
