@@ -63,6 +63,7 @@
 #define INSTROOT_ENV_VAR_EO01 "INSTROOT"
 
 #define SUBPATH_INDEP_PGM_EO01    "pgm"
+#define SUBPATH_INDEP_LIB_EO01    "lib"
 #define SUBPATH_INDEP_BIN_EO01    "bin"
 #define SUBPATH_INDEP_ENV_EO01    "env"
 #define SUBPATH_INDEP_PROT_EO01   "prot"
@@ -77,23 +78,26 @@
 
 #define SAPDB_NO_INDEPPATH_EO01 "SAPDB_NO_INDEPPATH"
 
-/* PTS 1104841 */
-/*********************/
-#if defined(BIT64) && !defined(OSF1) && !(defined(LINUX) && (defined(IA64) || defined(S390X))) && !defined(HP_IA64)
-/*********************/
+#if defined(OSF1) || \
+    defined(LINUX) && (defined(IA64) || defined(S390X) || defined(X86_64) || \
+    defined(SDBonPPC64)) \
+    || defined(HP_IA64)
+
+#define DBROOT_LIB_EO01     "lib"
+
+#elif defined(BIT64) && defined(WIN32)
 
-#ifdef WIN32
 #define DBROOT_LIB_EO01     "lib\\lib64"
-#else
+
+#elif defined(BIT64) && !defined(WIN32)
+
 #define DBROOT_LIB_EO01     "lib/lib64"
-#endif
 
 #else
 
 #define DBROOT_LIB_EO01     "lib"
-/*********************/
-#endif  /* BIT64 */
-/*********************/
+
+#endif
 
 #define DBROOT_MISC_EO01    "misc"
 #define DBROOT_PGM_EO01     "pgm"
@@ -122,6 +126,10 @@
 #endif
 /************/
 
+#if defined(SUN) && defined(BIT64)
+#define LDLIBPATH64_SUN "LD_LIBRARY_PATH64"
+#endif
+
 /*===========================================================================*
  *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
  *===========================================================================*/
@@ -215,6 +223,7 @@ eo01_GetDbrootSubPath          ( tsp00_Pathc        Path,
 
 static int en01_SearchSubPath(const char *GivenPath, const char *SubPath);
 
+static SAPDB_Char *eo01_TrimLDLIBPATH(SAPDB_Char *currentLDLIBPATH);
 /*===========================================================================*
  *  GLOBAL FUNCTIONS (CODE)                                                  *
  *===========================================================================*/
@@ -296,6 +305,16 @@ tsp00_Bool sqlGetIndependentPgmPath ( tsp00_Pathc     PGMPath,
 
 /*------------------------------*/
 
+tsp00_Bool sqlGetIndependentLibPath ( tsp00_Pathc     LIBPath,
+                                      int             TerminateWithDelimiter,
+                                      tsp01_RteError  *pRteError )
+{
+  return ( eo01_GetIndependentSubPath ( LIBPath, SUBPATH_INDEP_LIB_EO01, IndependentPrograms,
+                                        TerminateWithDelimiter, pRteError ) ) ;
+}
+
+/*------------------------------*/
+
 tsp00_Bool sqlGetIndependentBinPath ( tsp00_Pathc     BINPath,
                                       int             TerminateWithDelimiter,
                                       tsp01_RteError  *pRteError )
@@ -451,6 +470,216 @@ tsp00_Bool sqlGetIndependentWrkPath ( tsp00_Pathc     WrkPath,
                                         TerminateWithDelimiter, pRteError ) ) ;
 }
 
+
+/*------------------------------*/
+
+externC tsp00_Bool
+sqlUpdateNiLibPathEnvironment(tsp01_RteError *pRteError)
+{
+/*
+*  It is necessary to add '<IndepProgPath>/lib' to the LD_LIBRARY_PATH
+*  to guaranty that the UNIX SAPNI shared object finds the right SAP objects (e.g. libsapu16.so)
+    */
+#ifdef WIN32
+    eo46_rte_error_init ( pRteError ) ;
+    return 1;
+#else
+    static char *envLDLIB = NULL;
+    char *oldenvLDLIB = NULL;
+    char *pureLDLIBPATH;
+    char *currentLDLIBPATH;
+    tsp00_Pathc LibPath;
+    int         addLibPath = 0;
+    tsp00_Bool  result = 0;
+
+    eo46_rte_error_init ( pRteError ) ;
+
+    pureLDLIBPATH = getenv(LDLIBPATH_IP00);
+
+    currentLDLIBPATH = eo01_TrimLDLIBPATH(pureLDLIBPATH);
+
+    if (sqlGetIndependentLibPath (LibPath, 0, pRteError))
+    {
+        /* If path is found set add...Path to 0 otherwise to 1! This allows to use it for calculation */
+        addLibPath = ( en01_SearchSubPath(currentLDLIBPATH, LibPath) ? 0 : 1 );
+
+        if ( addLibPath == 0
+          && currentLDLIBPATH == pureLDLIBPATH )
+        {
+            /* Nothing to do. Environment already corrected */
+            result = 1;
+        }
+        else
+        {
+            oldenvLDLIB = envLDLIB;
+
+            if ( currentLDLIBPATH != NULL )
+            {
+                /* Lib path is added if not already found */
+                /* If either one or two path are added, for each path a colon is needed */
+                /* This is handled by 'addLibPath'! */
+                envLDLIB = (char *)malloc( strlen(LDLIBPATH_IP00 "=")
+                    + addLibPath * ( strlen(LibPath) + strlen(":") )
+                    + strlen(currentLDLIBPATH) + 1 );
+            }
+            else
+            {
+                /* Lib path is added if not already found */
+                envLDLIB = (char *)malloc( strlen(LDLIBPATH_IP00 "=")
+                    + strlen(LibPath) + strlen(":")
+                    + 1 );
+            }
+
+            if ( envLDLIB == NULL )
+            {
+                eo46_set_rte_error (pRteError, 0, "Failed to allocate memory for ", LDLIBPATH_IP00);
+            }
+            else
+            {
+                strcpy(envLDLIB, LDLIBPATH_IP00 "=");
+
+                /* Add LIB path always first */
+                if ( addLibPath != 0 )
+                {
+                    strcat(envLDLIB, LibPath);
+                }
+
+                if ( currentLDLIBPATH != NULL ) /* Append former LDLIBPATH */
+                {
+                    if (addLibPath != 0)
+                    {
+                        strcat(envLDLIB, ":");
+                    }
+
+                    strcat(envLDLIB, currentLDLIBPATH);
+                }
+
+                if ( putenv(envLDLIB) < 0 )
+                {
+                    free(envLDLIB);
+                    envLDLIB = oldenvLDLIB;
+                    eo46_set_rte_error (pRteError, 0, "Failed to putenv ", LDLIBPATH_IP00);
+                }
+                else
+                {
+                    if ( oldenvLDLIB != NULL )
+                    {
+                        free(oldenvLDLIB); /* prevent some memory leak, accept that last malloc is not freed... */
+                    }
+                    result = 1; /* everything fine */
+                }
+            }
+        }
+    }
+    else
+    {
+        eo46_set_rte_error (pRteError, 0, "Failed to get lib and sap path for", LDLIBPATH_IP00);
+    }
+
+#if defined(SUN) && defined(BIT64)
+
+    if ( result )
+    {
+        char *pureLDLIB64PATH;
+        pureLDLIB64PATH = getenv(LDLIBPATH64_SUN);
+
+        /* If LD_LIBRARY_PATH64 is not set, we do not override it */
+        if ( 0 != pureLDLIB64PATH )
+        {
+            static char *envLDLIB64 = NULL;
+            char *oldenvLDLIB64 = NULL;
+            char *currentLDLIB64PATH = pureLDLIB64PATH;
+            tsp00_Pathc LibPath;
+            int         addLibPath = 0;
+            
+            result = 0;
+
+            if (sqlGetIndependentLibPath (LibPath, 0, pRteError))
+            {
+                /* If path is found set add...Path to 0 otherwise to 1! This allows to use it for calculation */
+                addLibPath = ( en01_SearchSubPath(currentLDLIB64PATH, LibPath) ? 0 : 1 );
+
+                if ( addLibPath == 0
+                  && currentLDLIB64PATH == pureLDLIB64PATH )
+                {
+                    /* Nothing to do. Environment already corrected */
+                    result = 1;
+                }
+                else
+                {
+                    oldenvLDLIB64 = envLDLIB64;
+
+                    if ( currentLDLIB64PATH != NULL )
+                    {
+                        /* Lib path is added if not already found */
+                        /* If either one or two path are added, for each path a colon is needed */
+                        /* This is handled by 'addLibPath'! */
+                        envLDLIB64 = (char *)malloc( strlen(LDLIBPATH64_SUN "=")
+                            + addLibPath * ( strlen(LibPath) + strlen(":") )
+                            + strlen(currentLDLIB64PATH) + 1 );
+                    }
+                    else
+                    {
+                        /* Lib path is added if not already found */
+                        envLDLIB64 = (char *)malloc( strlen(LDLIBPATH64_SUN "=")
+                            + strlen(LibPath) + strlen(":")
+                            + 1 );
+                    }
+
+                    if ( envLDLIB64 == NULL )
+                    {
+                        eo46_set_rte_error (pRteError, 0, "Failed to allocate memory for ", LDLIBPATH64_SUN);
+                    }
+                    else
+                    {
+                        strcpy(envLDLIB64, LDLIBPATH64_SUN "=");
+
+                        /* Add LIB path always first */
+                        if ( addLibPath != 0 )
+                        {
+                            strcat(envLDLIB64, LibPath);
+                        }
+
+                        if ( currentLDLIB64PATH != NULL ) /* Append former LDLIBPATH */
+                        {
+                            if (addLibPath != 0)
+                            {
+                                strcat(envLDLIB64, ":");
+                            }
+
+                            strcat(envLDLIB64, currentLDLIB64PATH);
+                        }
+
+                        if ( putenv(envLDLIB64) < 0 )
+                        {
+                            free(envLDLIB64);
+                            envLDLIB64 = oldenvLDLIB64;
+                            eo46_set_rte_error (pRteError, 0, "Failed to putenv ", LDLIBPATH64_SUN);
+                        }
+                        else
+                        {
+                            if ( oldenvLDLIB64 != NULL )
+                            {
+                                free(oldenvLDLIB64); /* prevent some memory leak, accept that last malloc is not freed... */
+                            }
+                            result = 1; /* everything fine */
+                        }
+                    }
+                }
+            }
+            else
+            {
+                eo46_set_rte_error (pRteError, 0, "Failed to get lib and sap path for", LDLIBPATH64_SUN);
+            }
+        }
+    }
+#endif
+
+    return result;
+#endif /* else !WIN32 */
+}
+
+
 /*###########################*/
 #endif /* DBROOT_PATHES_ONLY */
 /*###########################*/
@@ -689,10 +918,130 @@ tsp00_Bool sqlUpdateLibPathEnvironment(tsp01_RteError *pRteError)
         eo46_set_rte_error (pRteError, 0, "Failed to get lib and sap path for", LDLIBPATH_IP00);
     }
 
+#if defined(SUN) && defined(BIT64)
+
+    if ( result )
+    {
+        char *pureLDLIB64PATH = getenv(LDLIBPATH64_SUN);
+
+        /* If LD_LIBRARY_PATH64 is not set, we do not override it */
+        if ( 0 != pureLDLIB64PATH )
+        {
+            static char *envLDLIB64 = NULL;
+            char *oldenvLDLIB64 = NULL;
+            char *currentLDLIB64PATH = pureLDLIB64PATH;
+            tsp00_Pathc LibPath;
+            tsp00_Pathc SapPath;
+            int         addLibPath = 0;
+            int         addSapPath = 0;
+
+            result = 0;
+
+            if ( sqlGetDbrootLibPath(LibPath, 0, pRteError )
+             &&  sqlGetDbrootSapPath(SapPath, 0, pRteError ) )
+            {
+                /* If path is found set add...Path to 0 otherwise to 1! This allows to use it for calculation */
+                addLibPath = ( en01_SearchSubPath(currentLDLIB64PATH, LibPath) ? 0 : 1 );
+                addSapPath = ( en01_SearchSubPath(currentLDLIB64PATH, SapPath) ? 0 : 1 );
+
+                if ( addLibPath == 0
+                  && addSapPath == 0
+                  && currentLDLIB64PATH == pureLDLIB64PATH )
+                {
+                    /* Nothing to do. Environment already corrected */
+                    result = 1;
+                }
+                else
+                {
+                    oldenvLDLIB64 = envLDLIB64;
+
+                    if ( currentLDLIB64PATH != NULL )
+                    {
+                        /* Lib or Sap path are added if not already found (addLibSapPath == 0) */
+                        /* If either one or two path are added, for each path a colon is needed */
+                        /* This is handled by 'addLibPath' and 'addSapPath' ! */
+                        envLDLIB64 = (char *)malloc( strlen(LDLIBPATH64_SUN "=")
+                            + addLibPath * ( strlen(LibPath) + strlen(":") )
+                            + addSapPath * ( strlen(SapPath) + strlen(":") )
+                            + strlen(currentLDLIB64PATH) + 1 );
+                    }
+                    else
+                    {
+                        /* Lib or Sap path are added if not already found (addLibSapPath == 0) */
+                        /* +1 is needed for zero termination */
+                        envLDLIB64 = (char *)malloc( strlen(LDLIBPATH64_SUN "=")
+                            + strlen(LibPath) + strlen(":")
+                            + strlen(SapPath)
+                            + 1 );
+                    }
+
+                    if ( envLDLIB64 == NULL )
+                    {
+                        eo46_set_rte_error (pRteError, 0, "Failed to allocate memory for ", LDLIBPATH64_SUN);
+                    }
+                    else
+                    {
+                        strcpy(envLDLIB64, LDLIBPATH64_SUN "=");
+
+                        /* Add LIB path always first */
+                        if ( addLibPath != 0 )
+                        {
+                            strcat(envLDLIB64, LibPath);
+                            if ( addSapPath != 0 )
+                            {
+                                /* Add colon to separate SAP path from LIB path */
+                                strcat(envLDLIB64, ":");
+                                strcat(envLDLIB64, SapPath);
+                            }
+                        }
+                        else
+                        {
+                            if ( addSapPath != 0 )
+                            {
+                                strcat(envLDLIB64, SapPath);
+                            }
+                        }
+
+                        if ( currentLDLIB64PATH != NULL ) /* Append former LDLIBPATH */
+                        {
+                            if ( addLibPath != 0 || addSapPath != 0 )
+                            {
+                                strcat(envLDLIB64, ":");
+                            }
+                            strcat(envLDLIB64, currentLDLIB64PATH);
+                        }
+
+                        if ( putenv(envLDLIB64) < 0 )
+                        {
+                            free(envLDLIB64);
+                            envLDLIB64 = oldenvLDLIB64;
+                            eo46_set_rte_error (pRteError, 0, "Failed to putenv ", LDLIBPATH64_SUN);
+                        }
+                        else
+                        {
+                            if ( oldenvLDLIB64 != NULL )
+                            {
+                                free(oldenvLDLIB64); /* prevent some memory leak, accept that last malloc is not freed... */
+                            }
+                            result = 1; /* everything fine */
+                        }
+                    }
+                }
+            }
+            else
+            {
+                eo46_set_rte_error (pRteError, 0, "Failed to get lib and sap path for", LDLIBPATH64_SUN);
+            }
+
+        }
+    }
+#endif
+
     return result;
 #endif /* else !WIN32 */
 }
 
+
 /*------------------------------*/
 
 externC char *sqlGetDBrootFromExecPath ( tsp00_Pathc ExecPath,
@@ -733,7 +1082,7 @@ externC char *sqlGetInstallationPathFromExecPath ( tsp00_Pathc ExecPath,
 /*********************/
 /* PTS 1105226 / 1108841 */
 /*-----------------------------------------------------------*/
-static int eo01CheckFilenameForRelativePath ( char        *RelFilename,
+static int eo01CheckFilenameForRelativePath ( char const  *RelFilename,
                                               char        *RelPath,
                                               char        *AbsPath,
            				    		          int          MaxAbsPathLen )
@@ -743,6 +1092,7 @@ static int eo01CheckFilenameForRelativePath ( char        *RelFilename,
     tsp00_Pathc HelpPath ;
     int  Found = false ;
     char *Ptr ;
+    char const *FilePtr ;
 
     AbsPath[0] = '\0' ;
 
@@ -757,9 +1107,10 @@ static int eo01CheckFilenameForRelativePath ( char        *RelFilename,
       *Ptr = '\0' ;
       strcat ( AbsPath, HelpPath ) ;
       *Ptr++ ;
+      FilePtr = Ptr;
     }
     else
-      Ptr  = RelFilename;
+      FilePtr  = RelFilename;
 
 
     if ( AbsPath[0] != '\0' )
@@ -769,7 +1120,7 @@ static int eo01CheckFilenameForRelativePath ( char        *RelFilename,
       if ( getcwd ( AbsPath, MaxAbsPathLen ) != NULL )
       {
         int AbsPathLen = strlen(AbsPath);
-		if ( AbsPathLen + 1 + strlen(Ptr) < MaxAbsPathLen )
+		if ( AbsPathLen + 1 + strlen(FilePtr) < MaxAbsPathLen )
 		{
           if ( AbsPath [ AbsPathLen -1 ] != PATH_DELIMITER_EO01 )
 		  {
@@ -777,7 +1128,7 @@ static int eo01CheckFilenameForRelativePath ( char        *RelFilename,
             AbsPath [ AbsPathLen+1] = '\0' ;
 		  }
           /* I hope relative Path is AbsPath */
-          strcat ( AbsPath, Ptr ) ;
+          strcat ( AbsPath, FilePtr ) ;
           Found = access(AbsPath, X_OK) == 0 ;
 		}
       }
@@ -787,7 +1138,7 @@ static int eo01CheckFilenameForRelativePath ( char        *RelFilename,
 
 #define ENV_PATH_DELIMITER_EO01 ':'
 /*-----------------------------------------------------------*/
-static char *eo01GetModuleFilename ( char    *Filename,
+static char *eo01GetModuleFilename ( char const   *Filename,
                                      char    *FullPath,
    				    		         int      MaxPathLen )
 /*-----------------------------------------------------------*/
