@@ -31,6 +31,9 @@
 /*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/
+#if defined(__GNUC__) || defined(__INTEL_COMPILER)
+#  define HAS_RTLD_LOCAL_UNDERMINES_EXCEPTION_HANDLING
+#endif
 
 #include "heo02x.h"
 #include "gip00.h"
@@ -188,12 +191,16 @@ HANDLE sqlLoadLibraryEx(char *szPath, int Mode, char *ErrText, int cbLenErrText)
 #else
 #if defined SUN || defined AIX || defined OSF1 || defined NMP || defined LINUX || ( defined HPUX && defined BIT64 )
   dlMode = RTLD_LAZY;
-#if !defined OSF1
+#  ifdef HAS_RTLD_LOCAL_UNDERMINES_EXCEPTION_HANDLING
+    dlMode = dlMode | RTLD_GLOBAL;
+#  else
+#    if !defined OSF1
   if ( RTLD_GLOBAL_EO02 == ( Mode & RTLD_GLOBAL_EO02 )) 
     dlMode = dlMode | RTLD_GLOBAL;
   if ( RTLD_LOCAL_EO02 == ( Mode & RTLD_LOCAL_EO02 )) 
     dlMode = dlMode | RTLD_LOCAL;
-#endif
+#    endif
+#  endif
   h = (HANDLE) dlopen (szFullName, dlMode);
   if (h == 0) 
   {
