@@ -305,12 +305,13 @@ eo03NiReceive  ( teo40_NiConnectInfoRecord    *pNIConnInfo,
 }
 /*===========================================================================*/
 
+/*===========================================================================*/
 
 /* PTS 1105295 */
 externC tsp00_Uint     
-eo03NiPing( char                         *szSapRouterString,
-            tsp00_Versionc VAR_ARRAY_REF  szServerVersion,
-            tsp00_ErrTextc VAR_ARRAY_REF  pErrText )
+eo03NiXserverPing( char                         *szSapRouterString,
+                   tsp00_Versionc VAR_ARRAY_REF  szServerVersion,
+                   tsp00_ErrTextc VAR_ARRAY_REF  pErrText )
 {
 
   teo40_NiConnectInfoRecord       TmpNIConnInfo;
@@ -625,26 +626,19 @@ eo03InitConnectParamRec ( teo40_NiConnectInfoRecord *pNIConnInfo,
                           teo003_ConPktParamRecord  *pConPktParamRec,
                           tsp00_Uint                 ulMessClass)
 {
+  eo420InitConPktParamRec ( pConPktParamRec );
   pConPktParamRec->ulMessClass         = ulMessClass;
   pConPktParamRec->ulSenderRef         = pNIConnInfo->ulClientRef;
-  pConPktParamRec->ulReceiverRef       = (tsp00_Uint    )undef_ceo00 ;
   pConPktParamRec->ulServiceType       = pNIConnInfo->ulServiceType;
   pConPktParamRec->ulMaxSegmentSize    = pNIConnInfo->ulMaxSegmentSize;
   pConPktParamRec->ulPacketSize        = pNIConnInfo->ulPacketSize;
   pConPktParamRec->ulMaxDataLen        = pNIConnInfo->ulMaxDataLen;
   pConPktParamRec->ulMinReplySize      = pNIConnInfo->ulMinReplySize;
-  pConPktParamRec->ulSwapType          = (tsp00_Uint    )undef_ceo00 ;
-  pConPktParamRec->szSenderServerDB[0] = '\0';
   pConPktParamRec->pidSenderPID        = pNIConnInfo->pidClientPID;
   pConPktParamRec->usServicePort       = pNIConnInfo->usServicePort;
-  pConPktParamRec->ulCommState         = commErrOk_esp01;
   strcpy ( pConPktParamRec->szReceiverServerDB, pNIConnInfo->szServerDB );
-  pConPktParamRec->szServerDBRoot [0]  = '\0';
-  pConPktParamRec->szServerPgm [0]     = '\0';
 
-  if ( ulMessClass == RSQL_INFO_REQUEST_EO003 )
-    pConPktParamRec->fAcknowledge        = false;
-  else
+  if ( ulMessClass != RSQL_INFO_REQUEST_EO003 )
     pConPktParamRec->fAcknowledge        = pNIConnInfo->fAcknowledge;
 
 }
