@@ -89,28 +89,41 @@ static bool            fReadDataIsNewer  = false;
  *===========================================================================*/
 
 /* Used by XUser API and sqlarg3 */
-static tsp00_Int4   eo13_putUser       ( tsp4_xuser_record  *prcUserParams,
-                                            tsp00_ErrText       acErrorText );
-static bool   eo13_xuserKeyIsBlank   ( tsp4_xuserkey       acXUserKey );
-static bool   eo13_findXuserKey       ( tsp4_xuserkey       acXUserKey,
-                                               tsp00_Uint4      *pulPage );
-static tsp00_Int4   eo13_readXuserEntries   ( tsp00_ErrText         acErrorText );
-static tsp00_Int4   eo13_checkUserId        ( PXUSER_INFO_PAGE    pInfoPage,
-                                            tsp00_ErrText         acErrorText );
-static tsp00_Int4   eo13_createInfoPage     ( tsp00_ErrText         acErrorText );
+static tsp00_Int4   eo13_putUser        (tsp4_xuser_record  *   prcUserParams,
+                                         SAPDB_Char *           accountName,
+                                         tsp00_ErrText          acErrorText );
 
-static void   eo13_cryptNameToC20    ( tsp00_CryptName      alCryptedName,
-                                            tsp00_C20            acC20Array[csp_cryptname] );
-static void   eo13_recryptName         ( tsp00_C20            acReCryptName[csp_cryptname],
-                                             tsp00_CryptName      alCryptedName );
-static void cn90CalculateUncrypt
-      ( const tsp00_CryptPw pwCrypted,
-        char              * passwd,
-        bool                bForDatabase );
+static bool   eo13_xuserKeyIsBlank      (tsp4_xuserkey          acXUserKey );
 
-static void eo13_CopyAsciiToUCS2(tsp00_Uint2 *destUCS2, const tsp00_Byte *srcAscii, int srcLen);
+static bool   eo13_findXuserKey         (tsp4_xuserkey          acXUserKey,
+                                         tsp00_Uint4 *          pulPage );
 
-static bool eo13_CheckPureAscii(const tsp00_Uint2 *srcUCS2, int srcLenInBytes);
+static tsp00_Int4 eo13_readXuserEntries (SAPDB_Char *           accountName,
+                                         tsp00_ErrText          acErrorText );
+
+static tsp00_Int4   eo13_checkUserId    (SAPDB_Char *           accountName,
+                                         PXUSER_INFO_PAGE       pInfoPage,
+                                         tsp00_ErrText          acErrorText );
+
+static tsp00_Int4   eo13_createInfoPage (SAPDB_Char *           accountName, 
+                                         tsp00_ErrText          acErrorText);
+
+static void   eo13_cryptNameToC20       (tsp00_CryptName        alCryptedName,
+                                         tsp00_C20              acC20Array[csp_cryptname] );
+
+static void   eo13_recryptName          (tsp00_C20              acReCryptName[csp_cryptname],
+                                         tsp00_CryptName        alCryptedName );
+
+static void cn90CalculateUncrypt        (const tsp00_CryptPw    pwCrypted,
+                                         char *                 passwd,
+                                         bool                   bForDatabase);
+
+static void eo13_CopyAsciiToUCS2        (tsp00_Uint2 *          destUCS2, 
+                                         const tsp00_Byte *     srcAscii, 
+                                         int srcLen);
+
+static bool eo13_CheckPureAscii         (const tsp00_Uint2 *    srcUCS2, 
+                                         int                    srcLenInBytes);
 
 /* PTS 1109583 */
 static void eo13_SetupUCS2Entries( tsp00_KnlIdentifier userUCS2, 
@@ -125,8 +138,9 @@ static void eo13_SetupUCS2Entries( tsp00_KnlIdentifier userUCS2,
  *  EXPORTED FUNCTIONS, EXPORTED CLASS METHODS (IMPLEMENTATION)              *
  *===========================================================================*/
 
-void sqlxuopenuser ( tsp00_ErrText acErrorText,
-                     bool     *pbOk)
+void sqlxuopenuser (SAPDB_Char *   accountName,
+                    tsp00_ErrText  acErrorText,
+                    bool *         pbOk)
 {
   #undef  MF__
   #define MF__ MOD__"sqlxuopenuser"
@@ -137,7 +151,7 @@ void sqlxuopenuser ( tsp00_ErrText acErrorText,
   memset (( char* ) acErrorText, ' ', sizeof ( tsp00_ErrText ));
   *pbOk = FALSE;
 
-  rc = eo13_readXuserEntries ( acErrorText );
+  rc = eo13_readXuserEntries (accountName, acErrorText);
 
   if ( rc != NO_ERROR )
     {
@@ -153,8 +167,9 @@ void sqlxuopenuser ( tsp00_ErrText acErrorText,
 
 /*------------------------------*/
 
-void sqlxucloseuser ( tsp00_ErrText acErrorText,
-                      bool     *pbOk )
+void sqlxucloseuser (SAPDB_Char *   accountName,
+                     tsp00_ErrText  acErrorText,
+                     bool     *     pbOk )
 {
   #undef  MF__
   #define MF__ MOD__"sqlxucloseuser"
@@ -175,8 +190,10 @@ void sqlxucloseuser ( tsp00_ErrText acErrorText,
 
   if ( fXuserBufEmpty == FALSE )
     {
-    rc = sql13u_write_xuser_entries ( (char *)&cDataBuffer,
-                                      pXUserInfoPage, acErrorText );
+    rc = sql13u_write_xuser_entries (accountName, 
+                                     (char *)&cDataBuffer,
+                                     pXUserInfoPage, 
+                                     acErrorText );
 
     if ( rc != NO_ERROR )
       {
@@ -195,9 +212,10 @@ void sqlxucloseuser ( tsp00_ErrText acErrorText,
 
 /*------------------------------*/
 
-void sqlputuser ( tsp4_xuser_record  *prcUserParams,
-                  tsp00_ErrText        acErrorText,
-                  bool             *pbOk )
+void sqlputuser (tsp4_xuser_record *    prcUserParams,
+                 SAPDB_Char *           accountName,
+                 tsp00_ErrText          acErrorText,
+                 bool             *     pbOk )
 {
   #undef  MF__
   #define MF__ MOD__"sqlputuser"
@@ -211,7 +229,7 @@ void sqlputuser ( tsp4_xuser_record  *prcUserParams,
 
   if ( fXuserBufEmpty )
     {
-    rc = eo13_readXuserEntries ( acErrorText );
+    rc = eo13_readXuserEntries (accountName, acErrorText);
 
     if ( rc != NO_ERROR )
       {
@@ -220,7 +238,7 @@ void sqlputuser ( tsp4_xuser_record  *prcUserParams,
       }
     }
 
-  rc = eo13_putUser ( prcUserParams, acErrorText );
+  rc = eo13_putUser (prcUserParams, accountName, acErrorText);
 
   if ( rc == NO_ERROR )
    *pbOk = TRUE;
@@ -232,9 +250,10 @@ void sqlputuser ( tsp4_xuser_record  *prcUserParams,
 
 /*------------------------------*/
 
-void sqlgetuser ( tsp4_xuser_record  *prcUserParams,
-                  tsp00_ErrText        acErrorText,
-                  bool            *pbOk )
+void sqlgetuser (tsp4_xuser_record *    prcUserParams,
+                 SAPDB_Char *           accountName,
+                 tsp00_ErrText          acErrorText,
+                 bool            *      pbOk)
 {
   #undef  MF__
   #define MF__ MOD__"sqlgetuser"
@@ -243,7 +262,7 @@ void sqlgetuser ( tsp4_xuser_record  *prcUserParams,
 
   DBGIN;
 
-  rc = sql13u_getuser ( prcUserParams, acErrorText );
+  rc = sql13u_getuser ( prcUserParams, accountName, acErrorText );
 
   if ( rc != NO_ERROR )
     {
@@ -260,10 +279,11 @@ void sqlgetuser ( tsp4_xuser_record  *prcUserParams,
 
 /*------------------------------*/
 
-void sqlindexuser ( tsp00_Int2            kUserIndex,
+void sqlindexuser ( tsp00_Int2          kUserIndex,
                     tsp4_xuser_record  *prcUserParams,
-                    tsp00_ErrText        acErrorText,
-                    bool            *pbOk )
+                    SAPDB_Char *        accountName,
+                    tsp00_ErrText       acErrorText,
+                    bool            *   pbOk )
 {
   #undef  MF__
   #define MF__ MOD__"sqlindexuser"
@@ -290,7 +310,7 @@ void sqlindexuser ( tsp00_Int2            kUserIndex,
 
   if ( fXuserBufEmpty )
     {
-    rc = eo13_readXuserEntries ( acErrorText );
+    rc = eo13_readXuserEntries (accountName, acErrorText);
 
     if ( rc != NO_ERROR )
       {
@@ -313,7 +333,7 @@ void sqlindexuser ( tsp00_Int2            kUserIndex,
 
 /*------------------------------*/
 
-int sqlclearuser ( void )
+int sqlclearuser (SAPDB_Char *   accountName)
 {
   #undef  MF__
   #define MF__ MOD__"sqlclearuser"
@@ -333,12 +353,13 @@ int sqlclearuser ( void )
    return ( ERROR_ACCESS_DENIED );
    }
 
-  if ( sql13u_remove_xuser_entries( acErrorText ) != NO_ERROR )
+  if ( sql13u_remove_xuser_entries( accountName, acErrorText ) != NO_ERROR )
     {
     DBGOUT;
     return ( FALSE );
     }
 
+
   DBGOUT;
 
   return ( TRUE );
@@ -400,8 +421,9 @@ void sql13u_init_user_params ( tsp4_xuser_record  *prcUserParams )
 
 /*---------------------------------------------------------------------------*/
 
-tsp00_Int4 sql13u_getuser ( tsp4_xuser_record  *prcUserParams,
-                            tsp00_ErrText        acErrorText )
+tsp00_Int4 sql13u_getuser (tsp4_xuser_record  *prcUserParams,
+                           SAPDB_Char *        accountName,
+                           tsp00_ErrText       acErrorText )
 {
   #undef  MF__
   #define MF__ MOD__"sql13u_getuser"
@@ -430,7 +452,7 @@ tsp00_Int4 sql13u_getuser ( tsp4_xuser_record  *prcUserParams,
 
   if ( fXuserBufEmpty )
     {
-    rc = eo13_readXuserEntries ( acErrorText );
+    rc = eo13_readXuserEntries (accountName, acErrorText );
 
     if ( rc != NO_ERROR )
       {
@@ -463,8 +485,9 @@ tsp00_Int4 sql13u_getuser ( tsp4_xuser_record  *prcUserParams,
  *  LOCAL/INLINE FUNCTIONS, LOCAL CLASS METHODS (IMPLEMENTATION)             *
  *===========================================================================*/
 
-static tsp00_Int4 eo13_putUser ( tsp4_xuser_record  *prcUserParams,
-                                       tsp00_ErrText        acErrorText )
+static tsp00_Int4 eo13_putUser (tsp4_xuser_record  *prcUserParams,
+                                SAPDB_Char *        accountName,
+                                tsp00_ErrText       acErrorText )
 {
   #undef  MF__
   #define MF__ MOD__"eo13_putUser"
@@ -559,7 +582,7 @@ static tsp00_Int4 eo13_putUser ( tsp4_xuser_record  *prcUserParams,
     memset ( &cDataBuffer, 0, sizeof(cDataBuffer) );
 
     /* --- create info page */
-    rc = eo13_createInfoPage ( acErrorText );
+    rc = eo13_createInfoPage (accountName, acErrorText);
 
     if ( rc != NO_ERROR )
       {
@@ -655,7 +678,8 @@ static bool eo13_findXuserKey ( tsp4_xuserkey       acXUserKey,
 
 /*------------------------------*/
 
-static tsp00_Int4 eo13_createInfoPage ( tsp00_ErrText         acErrorText )
+static tsp00_Int4 eo13_createInfoPage (SAPDB_Char *       accountName,
+                                       tsp00_ErrText      acErrorText )
 {
   #undef  MF__
   #define MF__ MOD__"eo13_createInfoPage"
@@ -681,7 +705,7 @@ static tsp00_Int4 eo13_createInfoPage ( tsp00_ErrText         acErrorText )
   #if defined (_WIN32)
    if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
      {
-     rc = sql49c_get_own_textual_SID ( &pszTextualSID );
+     rc = sql49c_get_user_textual_SID (accountName, &pszTextualSID);
 
      if ( rc != NO_ERROR )
        {
@@ -762,7 +786,8 @@ static tsp00_Int4 eo13_createInfoPage ( tsp00_ErrText         acErrorText )
 
 /*------------------------------*/
 
-static tsp00_Int4 eo13_readXuserEntries ( tsp00_ErrText   acErrorText )
+static tsp00_Int4 eo13_readXuserEntries (SAPDB_Char *       accountName, 
+                                         tsp00_ErrText      acErrorText )
 {
    #undef  MF__
    #define MF__ MOD__"eo13_readXuserEntries"
@@ -780,7 +805,7 @@ static tsp00_Int4 eo13_readXuserEntries ( tsp00_ErrText   acErrorText )
    fXuserBufEmpty = TRUE;
    memset ( &cDataBuffer, 0, sizeof(cDataBuffer) );
 
-   rc = eo13_createInfoPage ( acErrorText );
+   rc = eo13_createInfoPage (accountName, acErrorText );
 
    if ( rc != NO_ERROR )
      {
@@ -788,8 +813,8 @@ static tsp00_Int4 eo13_readXuserEntries ( tsp00_ErrText   acErrorText )
      return ( rc );
      }
 
-   rc = sql13u_get_all_entries ( &pcDataBuffer, (tsp00_Int4*)&ulDataLen, &pRel611XUserPages,
-                                 acErrorText );
+   rc = sql13u_get_all_entries (accountName, &pcDataBuffer, (tsp00_Int4*)&ulDataLen, &pRel611XUserPages,
+                                acErrorText );
 
    if ( rc != NO_ERROR )
      {
@@ -830,7 +855,8 @@ static tsp00_Int4 eo13_readXuserEntries ( tsp00_ErrText   acErrorText )
 
      /* --- check user-id wether you have the permissions to modify
             the XUSER content? */
-     rc = eo13_checkUserId ( pInfoPage, acErrorText );
+
+     rc = eo13_checkUserId (accountName, pInfoPage, acErrorText );
 
      if ( rc != NO_ERROR )
        {
@@ -920,8 +946,9 @@ static tsp00_Int4 eo13_readXuserEntries ( tsp00_ErrText   acErrorText )
 
 /*------------------------------*/
 
-static tsp00_Int4 eo13_checkUserId ( PXUSER_INFO_PAGE  pInfoPage,
-                                         tsp00_ErrText     acErrorText )
+static tsp00_Int4 eo13_checkUserId (SAPDB_Char *        accountName,
+                                    PXUSER_INFO_PAGE    pInfoPage,
+                                    tsp00_ErrText       acErrorText )
 {
   #undef  MF__
   #define MF__ MOD__"eo13_checkUserId"
@@ -947,7 +974,7 @@ static tsp00_Int4 eo13_checkUserId ( PXUSER_INFO_PAGE  pInfoPage,
 #ifdef WIN32
   if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
     {
-    rc = sql49c_get_own_textual_SID ( &pszTextualSID );
+    rc = sql49c_get_user_textual_SID (accountName, &pszTextualSID );
 
     if ( rc != NO_ERROR )
       {
