@@ -137,7 +137,25 @@ static void parseString( const char *explain,
              maxNumber [in] maximum number value allowed
  */
 static int parseNumber( const char *explain,
-                        unsigned int maxNumber ); 
+                        unsigned int maxNumber );
+
+/*---------------------------------------
+  Function: parseStringC
+  Description: Parse a 7bit ASCII string and copy it to dest. If NULL is given for dest the
+               length of the parameter to be parsed is determined.
+               Usually this function is called twice:
+               First call: Determine length
+               Alloc memory
+               Second Call: Copy paramter to memory location 
+  Arguments: explain [in] Name of parameter for error output
+             dest [out] destination or NULL, if the length of the pararmter should be determined
+
+  Return: Length of the parsed String
+ */
+
+static SAPDB_UInt4 parseStringC(SAPDB_Char const * explain,
+                                SAPDB_Char *       dest);
+
 /*---------------------------------------
   Function: parseUsernamePassword
   Description: Parse a username/password combination
@@ -167,14 +185,15 @@ static void updatePasswordRecord( tsp00_KnlIdentifier passwordUCS2,
   
   Arguments: key [in] Key of entry to be cleared
  */
-static void listSingleEntry( tsp4_xuserkey key );
+static void listSingleEntry (tsp4_xuserkey   key,
+                             SAPDB_Char*     accountName);
 /*---------------------------------------
   Function: listAllEntries
   Description: List all entries
   
   All entries are read in first. Then each entry is shown.
  */
-static void listAllEntries();
+static void listAllEntries  (SAPDB_Char*     accountName);
 /*---------------------------------------
   Function: clearSingleEntry
   Description: Clear only one entry, leaving all other entries 
@@ -185,14 +204,15 @@ static void listAllEntries();
 
   Arguments: key [in] Key of entry to be cleared
  */
-static void clearSingleEntry( tsp4_xuserkey key );
+static void clearSingleEntry(tsp4_xuserkey  key,
+                             SAPDB_Char*    accountName );
 /*---------------------------------------
   Function: readAllEntries
   Description: Read all XUSER entries into own memory buffer
 
   The global variables entriesFound and allEntries are filled.
  */
-static void readAllEntries();
+static void readAllEntries  (SAPDB_Char*    accountName);
 /*---------------------------------------
   Function: printEntry
   Description: Print a XUSER entry in readable form (except password...)
@@ -209,7 +229,8 @@ static void printEntry( int which, tsp4_xuser_record *pEntry );
 
   Arguments: filename [in] The file with the entry information
  */
-static void parseBatchFile( char const * filename );
+static void parseBatchFile(char const *    filename,
+                           SAPDB_Char *    accountName);
 /*---------------------------------------
   Function: readString
   Description: Read a 7bit ASCII string and copy it padded with blanks
@@ -272,9 +293,11 @@ static void readUsernamePassword( bool isUCS2,
   The entry is verified (isolation level and sqlmodname) are checked.
   If everything want ok so far, the update of XUSER information is done.
 
-  Arguments: paramUserRecord [in] The XUSER record to use
+  Arguments: paramUserRecord [in]   The XUSER record to use
+             accountName [in]       Account of the user to be updated
  */
-static void updateUserRecord( struct tsp4_xuser_record *paramUserRecord );
+static void updateUserRecord (struct tsp4_xuser_record *    paramUserRecord,
+                              SAPDB_Char *                  accountName);
 /*===========================================================================*
  *=====================  END OF DECLARATIONS ================================*
  *===========================================================================*/
@@ -335,6 +358,8 @@ static int c_main(int argc, char *argv[])
   int                      gotUserKey = 0;
   int                      gotAnythingElse = 0;
   struct tsp4_xuser_record paramUserRecord;
+  SAPDB_Char *             pAccountName = NULL;
+  SAPDB_UInt4              lenAccountName = 0;
 
   /* Initialize paramUserRecord */
   memset(paramPassword, ' ', sizeof(tsp00_KnlIdentifier));
@@ -348,7 +373,7 @@ static int c_main(int argc, char *argv[])
 
   do
   {
-    nextOption = sqlgetopt(argc, argv, "zVAab:U:u:d:n:S:y:t:I:x:?h");
+    nextOption = sqlgetopt(argc, argv, "zVAab:c:U:u:d:n:S:y:t:I:x:?h");
     switch( nextOption )
     {
     case '?':
@@ -371,6 +396,23 @@ static int c_main(int argc, char *argv[])
           gotAnythingElse = 1;
 	  paramBatchFile = optarg;
 	  break;
+    case 'c':
+        gotAnythingElse = 1;
+        lenAccountName = parseStringC("Account Name",
+                                      pAccountName);
+        if (0 != lenAccountName)
+        {
+            pAccountName = (SAPDB_Char *)alloca (lenAccountName + 1);
+            if (NULL == pAccountName)
+            {
+                printf("FATAL: Could not allocate memory to hold account name\n");
+                return -1;
+            }
+
+            parseStringC ("Account Name", pAccountName);
+        }
+
+        break;
     case 'U':
           gotAnythingElse = 1;
 	  parseString( "UserKey", 
@@ -485,12 +527,12 @@ static int c_main(int argc, char *argv[])
     if ( gotUserKey )
     {
       if ( beVerbose ) printf("Action ListSingleEntry (%8.8s)\n", paramUserRecord.xu_key );
-      listSingleEntry( paramUserRecord.xu_key );
+      listSingleEntry (paramUserRecord.xu_key, pAccountName);
     }
     else
     {
       if ( beVerbose ) printf("Action ListAllEntries\n" );
-      listAllEntries();
+      listAllEntries (pAccountName);
     }
     break;
 
@@ -498,12 +540,12 @@ static int c_main(int argc, char *argv[])
     if ( gotUserKey )
     {
       if ( beVerbose ) printf("Action ClearSingleEntry (%8.8s)\n", paramUserRecord.xu_key );
-      clearSingleEntry( paramUserRecord.xu_key );
+      clearSingleEntry (paramUserRecord.xu_key, pAccountName);
     }
     else
     {
       if ( beVerbose ) printf("Action ClearAllEntries\n" );
-      sqlclearuser();
+      sqlclearuser (pAccountName);
     }
     break;
 
@@ -513,13 +555,13 @@ static int c_main(int argc, char *argv[])
       if ( paramBatchFile )
       {
         if ( beVerbose ) printf("Parsing Batchfile %s\n", paramBatchFile );
-        parseBatchFile( paramBatchFile );
+        parseBatchFile( paramBatchFile, pAccountName);
       }
       else
       {
         if ( beVerbose ) printf("Update user record\n");
         /* Fill password and check also sqlmode,isolation */ 
-        updateUserRecord( &paramUserRecord );
+        updateUserRecord( &paramUserRecord, pAccountName);
       }
     }
     break;
@@ -556,7 +598,7 @@ static void Usage(int longHelp)
   printf("    -h show long helptext\n");
   if ( !longHelp )
   {
-    printf("    -V -A -b file -U key -u name,password -d dbname -n nodename\n");
+    printf("    -V -A -b file -c account -U key -u name,password -d dbname -n nodename\n");
     printf("    -S sqlmode -t timeout -I isolation\n");
   }
   else
@@ -572,6 +614,9 @@ static void Usage(int longHelp)
     printf("       Entries are NOT separated by a separation line.\n");
     printf("       Since version 7.4 entries 7 and 10 are obsolete and\n");
     printf("       should contain empty lines.\n");
+    printf("    -c account name       user account <domain>\\<user> for which the xuser command\n");
+    printf("                          should be executed (only available for administrators)\n");
+    printf("       string (no restriction to the length), default '' = current user\n");
     printf("    -U user_key           user key\n");
     printf("       string (max. 18 characters), default 'DEFAULT')\n");
     printf("    -u user_name,password user name/password combination\n");
@@ -715,6 +760,54 @@ static int parseNumber( const char *explain,
   return ( negativ ? -1 : (int)number );
 } 
 
+/*---------------------------------------
+  Function: parseStringC
+  Description: Parse a 7bit ASCII string and copy it to dest. If NULL is given for dest the
+               length of the parameter to be parsed is determined.
+               Usually this function is called twice:
+               First call: Determine length
+               Alloc memory
+               Second Call: Copy paramter to memory location 
+
+               !!! C a u t i o n !!!
+               If dest is unequal to NULL it is assumed that sufficient memory
+               is allocated to hold the whole string including the terminating null byte.
+
+  Arguments: explain [in] Name of parameter for error output
+             dest [out] destination or NULL, if the length of the pararmter should be determined
+
+  Return: Length of the parsed String
+ */
+
+static SAPDB_UInt4 parseStringC(SAPDB_Char const * explain,
+                                SAPDB_Char *       dest)
+{
+  SAPDB_UInt4   idx;
+
+  for (idx = 0; (optarg[idx] != 0); idx++ )
+  {
+     if ( (optarg[idx] & 0x7f) != optarg[idx] )
+     {
+	    printf("Warning: Non Ascii7 parameter '0x%x' for '%s' (parameter ignored)\n",
+                optarg[idx], explain);
+        idx= 0;
+        break;
+     }
+
+     if (NULL != dest)
+     {
+        dest[idx] = optarg[idx];
+     }
+  }
+
+  if (NULL != dest)
+  {
+     dest[idx] = '\0';
+  }
+
+  return idx;
+}
+
 /* PTS 1111170 */
 /*---------------------------------------
   Function: GetQuotedArgumentUppercase
@@ -1015,7 +1108,8 @@ static void parseUsernamePassword( tsp00_KnlIdentifier username,
 
   Arguments: key [in] Key of entry to be cleared
  */
-static void clearSingleEntry( tsp4_xuserkey key )
+static void clearSingleEntry(tsp4_xuserkey  key,
+                             SAPDB_Char*    accountName )
 {
   int errorCount = 0;
   int i;
@@ -1025,7 +1119,7 @@ static void clearSingleEntry( tsp4_xuserkey key )
 
   memset( errText, 0, sizeof(tsp00_ErrText));
 
-  readAllEntries();
+  readAllEntries(accountName);
 
   for ( i = 0; i < entriesFound; i++ )
   {
@@ -1042,45 +1136,45 @@ static void clearSingleEntry( tsp4_xuserkey key )
   }
   else
   {
-    sqlclearuser();
+      sqlclearuser(accountName);/*Lscht den Key mit*/
 
-    sqlxuopenuser( errText, &ok );
-    if ( !ok )
-    {
-      printf("FATAL: Reopen xuser entry file failed: %40.40s\n", errText);
-      printf("xuser entry file lost\n");
-      exit(2);
-    }
-
-    for ( i = 0; i < entriesFound; i++ )
-    {
-      if ( i != entryToDelete )
+      sqlxuopenuser( accountName, errText, &ok );
+      if ( !ok )
       {
-        sqlputuser( &allEntries[i], errText, &ok );
-        if ( !ok )
-	{
-	  printf("FATAL: Put old xuser entry failed: %40.40s\n", errText);
-	  printf("FATAL: xuser entry for key %18.18s may be lost\n", 
-			allEntries[i].xu_key);
-          ok = 1;
-          errorCount++;
-	}
+          printf("FATAL: Reopen xuser entry file failed: %40.40s\n", errText);
+          printf("xuser entry file lost\n");
+          exit(2);
       }
-    }
 
-    sqlxucloseuser(errText, &ok);
-    if ( !ok )
-    {
-      printf("FATAL: Closing xuser entry file failed: %40.40s\n", errText);
-      printf("xuser entry file may be lost\n");
-      exit(2);
-    }
+      for ( i = 0; i < entriesFound; i++ )
+      {
+          if ( i != entryToDelete )
+          {
+              sqlputuser( &allEntries[i], accountName, errText, &ok );
+              if ( !ok )
+              {
+                  printf("FATAL: Put old xuser entry failed: %40.40s\n", errText);
+                  printf("FATAL: xuser entry for key %18.18s may be lost\n",
+                      allEntries[i].xu_key);
+                  ok = 1;
+                  errorCount++;
+              }
+          }
+      }
 
-    if ( errorCount > 0 )
-    {
-      printf("FATAL: %d errors while putting old entries\n", errorCount);
-      exit(2);
-    }
+      sqlxucloseuser(accountName, errText, &ok);
+      if ( !ok )
+      {
+          printf("FATAL: Closing xuser entry file failed: %40.40s\n", errText);
+          printf("xuser entry file may be lost\n");
+          exit(2);
+      }
+
+      if ( errorCount > 0 )
+      {
+          printf("FATAL: %d errors while putting old entries\n", errorCount);
+          exit(2);
+      }
   }
 }
 
@@ -1092,12 +1186,13 @@ static void clearSingleEntry( tsp4_xuserkey key )
   
   Arguments: key [in] Key of entry to be cleared
  */
-static void listSingleEntry( tsp4_xuserkey key )
+static void listSingleEntry(tsp4_xuserkey   key,
+                             SAPDB_Char*     accountName)
 {
   int entryFound = -1; 
   int i;
 
-  readAllEntries();
+  readAllEntries(accountName);
 
   for ( i = 0; i < entriesFound; i++ )
   {
@@ -1124,11 +1219,11 @@ static void listSingleEntry( tsp4_xuserkey key )
   
   All entries are read in first. Then each entry is shown.
  */
-static void listAllEntries()
+static void listAllEntries(SAPDB_Char*     accountName)
 {
   int i;
 
-  readAllEntries();
+  readAllEntries(accountName);
 
   for ( i = 0; i < entriesFound; i++ )
   {
@@ -1209,7 +1304,7 @@ static void printEntry(int i, tsp4_xuser_record *pEntry)
 
   The global variables entriesFound and allEntries are filled.
  */
-static void readAllEntries()
+static void readAllEntries(SAPDB_Char *    accountName)
 {
   int i;
   tsp00_Bool ok;
@@ -1217,7 +1312,7 @@ static void readAllEntries()
 
   memset( errText, 0, sizeof(tsp00_ErrText));
 
-  sqlxuopenuser( errText, &ok );
+  sqlxuopenuser(accountName, errText, &ok );
   if ( !ok ) 
   {
     if ( errText[0] != ' ' )
@@ -1232,7 +1327,7 @@ static void readAllEntries()
   entriesFound = REL611_MX_XUSER_ENTRIES;
   for ( i = 0; i < REL611_MX_XUSER_ENTRIES; i++ )
   {
-    sqlindexuser((tsp00_Int2)(i+1), &allEntries[i], errText, &ok);
+    sqlindexuser((tsp00_Int2)(i+1), &allEntries[i], accountName, errText, &ok);
     if ( !ok )
     {
       if ( errText[0] != ' ' )
@@ -1247,7 +1342,7 @@ static void readAllEntries()
     }
   }
 
-  sqlxucloseuser(errText, &ok);
+  sqlxucloseuser(accountName, errText, &ok);
   if ( !ok )
   {
     printf("Warning: Error closing xuser file: %40.40s\n", errText);
@@ -1314,18 +1409,21 @@ static bool skipUCS2intro(FILE *fp, bool *isUCS2)
 
 /*---------------------------------------
   Function: parseBatchFile
-  Description: Read entries in from given file
+  Description:  Read entries in from given file. Does not support the '-c' option for giving an
+                account name.
 
   All entries are read in first and verified. Only valid entries are put 
   into XUSER information. If an invalid entry was found a warning is given.
 
   Arguments: filename [in] The file with the entry information
  */
-static void parseBatchFile( char const *filename )
+static void parseBatchFile(char const *     filename,
+                           SAPDB_Char *     accountName)
 {
-  FILE *fp;
-  tsp4_xuser_record paramUserRecord;
-  tsp00_KnlIdentifier paramPassword;
+  FILE *                    fp;
+  tsp4_xuser_record         paramUserRecord;
+  tsp00_KnlIdentifier       paramPassword;
+
 
   /* readResult (reset on first read failure. '8' forces non error on first read check... */
   wchar_t *readResult = (wchar_t *)8;
@@ -1366,6 +1464,7 @@ static void parseBatchFile( char const *filename )
     memset(&paramUserRecord, 0, sizeof(tsp4_xuser_record));
     sql13u_init_user_params( &paramUserRecord );
 
+
     /* 1-UserKey */
     readString( isUCS2, fp, &readResult, &insertEntry, "UserKey", 
                  paramUserRecord.xu_key, sizeof(tsp4_xuserkey) );
@@ -1399,7 +1498,7 @@ static void parseBatchFile( char const *filename )
     {
         printf("Updating entry %d for Userkey '%18.18s'\n",
             entryIndex, &paramUserRecord.xu_key[0] );
-        updateUserRecord( &paramUserRecord );
+        updateUserRecord(&paramUserRecord, accountName);
     }
     else
     {
@@ -1826,7 +1925,8 @@ static void updatePasswordRecord( tsp00_KnlIdentifier passwordUCS2,
   Arguments: passwordUCS2 [in] UCS2 encoded password
              paramUserRecord [in] The XUSER record to use
  */
-static void updateUserRecord( struct tsp4_xuser_record *paramUserRecord )
+static void updateUserRecord( struct tsp4_xuser_record *    paramUserRecord,
+                              SAPDB_Char *                  accountName)
 {
   tsp00_Bool    ok;
   tsp00_ErrText errText;
@@ -1880,22 +1980,22 @@ static void updateUserRecord( struct tsp4_xuser_record *paramUserRecord )
     break;
   }
 
-  sqlxuopenuser ( errText, &ok );
+  sqlxuopenuser (accountName, errText, &ok );
   if ( !ok ) 
   {
     printf("FATAL: Open xuser entry file failed: %40.40s\n", errText);
     return;
   }
 
-  sqlputuser ( paramUserRecord, errText, &ok);
+  sqlputuser (paramUserRecord, accountName, errText, &ok);
   if ( !ok ) 
   {
     printf("FATAL: Put xuser entry failed: %40.40s\n", errText);
-    sqlxucloseuser ( errText, &ok );
+    sqlxucloseuser ( accountName, errText, &ok );
     return;
   }
 
-  sqlxucloseuser ( errText, &ok );
+  sqlxucloseuser (accountName, errText, &ok );
   if ( !ok ) 
   {
     printf("FATAL: Close xuser entry failed: %40.40s\n", errText);
