@@ -837,7 +837,8 @@ void ceo15_MessageFile::eo15OpenAgain(const teo15_Position MaxFileSize,
 *===========================================================================*/
 extern "C"
 {
-  void *eo15DumpMessageFile( void * );
+  void *eo15DumpMessageFile();
+  void *eo15DumpMessageFileThread( void * );
 }
 /*---------------------------------------------------------------------------*/
 //
@@ -916,7 +917,7 @@ externC void eo15InitMessageBufferSpace(const tsp00_Int4 NumberOfBuffers,
   teo07_ThreadErr ThreadOk = THR_OK_EO07;
 
   sqlbeginthread( WORKER_STACK_EO15,
-    eo15DumpMessageFile,
+    eo15DumpMessageFileThread,
     NULL,
     THR_CREATE_SUSPENDED_EO07,
     &eo15_WorkerThread,
@@ -1675,8 +1676,31 @@ void ceo15_MessageFile::eo15Unlock()
 /*---------------------------------------------------------------------------*/
 extern "C"
 {
+  void *eo15DumpMessageFileThread( void * )
+  {
+      void *ret = 0;
+
+#if defined (_WIN32) && defined (KERNEL)
+    __try
+    {
+#       if defined(_FASTCAP)
+         CAPNameThread ("eo15DumpMessageFile");
+#       endif
+#endif
+         ret = eo15DumpMessageFile();
+
+#if defined (_WIN32) && defined (KERNEL)
+    }
+    __except( sql50k_excepthandler(GetExceptionInformation()) )
+    {
+        ;
+    }
+#endif
+    return ret;
+  }
+
   // static       is not allowed for extern "C" functions...
-  void *eo15DumpMessageFile( void * )
+  void *eo15DumpMessageFile()
   {
 #ifndef HPUX
     // aCC compiler complains:
@@ -1689,7 +1713,6 @@ extern "C"
     teo07_ThreadErr ThreadOk;
     tsp00_VfReturn_Param ReturnStatus;
 
-
     while ( eo15_NumberOfBuffers > 0 )
     {
       for ( FileIndex = 1; FileIndex <= NUMBER_OF_MESSAGE_BUFFERS; FileIndex++ )
@@ -1720,7 +1743,6 @@ extern "C"
 //  TEST_PRINT("Worker suspends itself\n")
       sqlsuspendthread( eo15_WorkerThread, ThreadErrText, &ThreadOk );
     }
-
     return (void *)0;
   }
 };
