@@ -162,6 +162,10 @@ public:
 
 static tsp00_Int4                                                    DummyBadAllocCountDown = -1;
 
+/* PTS 1128888, PG.
+   See CIliveCacheSink::GetSessionAllocator for information about this global variable. */
+static RTESync_InterlockedCounter<tsp00_Uint4> OMS_Session_Allocator_Cntr;
+
 RTESync_Spinlock            CIliveCacheSink::m_allocatorPoolLock;
 CIliveCacheSink::AllocPool  *CIliveCacheSink::m_allocatorPool = NULL;
 CIliveCacheSink::AllocPool  *CIliveCacheSink::m_allocatorPoolFreelist = NULL;
@@ -845,6 +849,7 @@ STDMETHODIMP CIliveCacheSink::CreateKeyRangeIterator(
            tgg00_FileId             &objFile,
            tgg01_OmsVersionContext  *lpVersionContext,
            void                    **pKeyRangeIterator,
+           tsp00_Int4                bodySize,
            tsp00_Int4                keyLen,
            const void               *pStartKey,              
            const void               *pLowerKey,              
@@ -852,12 +857,15 @@ STDMETHODIMP CIliveCacheSink::CreateKeyRangeIterator(
            tsp00_Int4               &noOfOid,              
            OmsTypeOid               *pResultOId,  
            tgg91_PageRef            *pObjVers,
+           unsigned char            *pBody,
            tsp00_Int4               &LogReadAccesses,
            tgg00_BasisError         &e,
            OmsIterDirection          direction)               
 {
   LogReadAccesses = 0;
 
+  bool readBuffered = (noOfOid == 1);
+
   ENTER_SINK_METHOD(CreateKeyIterator_method_idx);
   ak341CreateKeyRangeIterator (*this->lpSessionContext,
     consistentView, 
@@ -869,6 +877,7 @@ STDMETHODIMP CIliveCacheSink::CreateKeyRangeIterator(
     pUpperKey,
     pKeyRangeIterator,
     direction == OMS_ASCENDING ? true : false,
+    readBuffered,
     e);
 
   if (e == 0){
@@ -880,11 +889,13 @@ STDMETHODIMP CIliveCacheSink::CreateKeyRangeIterator(
         *pKeyRangeIterator,  
         true,              // bInclusive
         keyLen,
+        bodySize,
         CONST_CAST(void*,pStartKey),
         noOfOid,
         REINTERPRET_CAST(tgg92_KernelOid*, pResultOId), 
         pObjVers,
         LogReadAccesses,
+        REINTERPRET_CAST(tsp00_MoveObj*, pBody),
         e);
     }
     else {
@@ -895,11 +906,13 @@ STDMETHODIMP CIliveCacheSink::CreateKeyRangeIterator(
         *pKeyRangeIterator,  
         true,              // bInclusive
         keyLen,
+        bodySize,
         CONST_CAST(void*,pStartKey),
         noOfOid,
         REINTERPRET_CAST(tgg92_KernelOid*, pResultOId), 
         pObjVers,
         LogReadAccesses,
+        REINTERPRET_CAST(tsp00_MoveObj*, pBody),
         e);
     }
   }
@@ -1138,11 +1151,18 @@ STDMETHODIMP CIliveCacheSink::SetError(short error,
                                        tsp00_CodeType codeType)
 {
   ENTER_SINK_METHOD(SetError_method_idx);
-  a34SetError (*this->lpSessionContext,
-    error,
-    *REINTERPRET_CAST(tsp00_MoveObj*, CONST_CAST(unsigned char*, lpv)),
-    buflen,
-    codeType);
+  if (!memcmp (lpv, "-28999", 6)){
+    // indicate, that the exception was provoked by the testkernel
+    lpSessionContext->a_provoked_exception =  true;
+  }
+  else {
+    a34SetError (*this->lpSessionContext,
+      error,
+      *REINTERPRET_CAST(tsp00_MoveObj*, CONST_CAST(unsigned char*, lpv)),
+      buflen,
+      codeType);
+  }
+
   this->CallbackIsActive = false;
   sqlErrorFlag = true;    /* PTS 1110814 */
   
@@ -1380,6 +1400,7 @@ STDMETHODIMP CIliveCacheSink::SQLRequest(tsp00_Int4 buflen,
   LEAVE_SINK_METHOD(S_OK);
 }
 
+
 // PTS 1120478                                        
 STDMETHODIMP CIliveCacheSink::NextOids(tgg91_TransNo           &ConsistentView,
                                        tgg00_FileId            &FileId,
@@ -1460,11 +1481,13 @@ STDMETHODIMP CIliveCacheSink::PrevObjFromKeyIterator (
                             tgg01_ContainerId       &ObjFile,
                             tgg01_OmsVersionContext *VersionContext,
                             void                    *pKeyIterator,
+                            tsp00_Int4               bodySize,
                             tsp00_Int4               KeyLen,
                             void                    *pRestartKey, 
                             tsp00_Int4              &noOfOid,
                             OmsTypeOid              *pResultOId, 
                             tgg91_PageRef           *pObjVers,
+                            unsigned char           *pBody,
                             tsp00_Int4              &LogReadAccesses,
                             tgg00_BasisError        &e)
 {
@@ -1478,11 +1501,13 @@ STDMETHODIMP CIliveCacheSink::PrevObjFromKeyIterator (
     pKeyIterator,  
     false,             // bInclusive
     KeyLen,
+    bodySize,
     pRestartKey,
     noOfOid,
     REINTERPRET_CAST(tgg92_KernelOid*, pResultOId), 
     pObjVers,
     LogReadAccesses,
+    REINTERPRET_CAST(tsp00_MoveObj*, pBody),
     e);
   LEAVE_SINK_METHOD(S_OK);
 }
@@ -1493,11 +1518,13 @@ STDMETHODIMP CIliveCacheSink::NextObjFromKeyIterator (
                             tgg01_ContainerId       &ObjFile,
                             tgg01_OmsVersionContext *VersionContext,
                             void                    *pKeyIterator,
+                            tsp00_Int4               bodySize,
                             tsp00_Int4               KeyLen,
                             void                    *pRestartKey, 
                             tsp00_Int4              &noOfOid,
                             OmsTypeOid              *pResultOId, 
                             tgg91_PageRef           *pObjVers,
+                            unsigned char           *pBody,
                             tsp00_Int4              &LogReadAccesses,
                             tgg00_BasisError        &e)
 {	
@@ -1511,11 +1538,13 @@ STDMETHODIMP CIliveCacheSink::NextObjFromKeyIterator (
     pKeyIterator,  
     false,             // bInclusive
     KeyLen,
+    bodySize,
     pRestartKey,
     noOfOid,
     REINTERPRET_CAST(tgg92_KernelOid*, pResultOId), 
     pObjVers,
     LogReadAccesses,
+    REINTERPRET_CAST(tsp00_MoveObj*, pBody),
     e);
   LEAVE_SINK_METHOD(S_OK);
 }
@@ -2079,11 +2108,24 @@ STDMETHODIMP CIliveCacheSink::SimCtlDestroyHandleCB(
 
 /*---------------------------------------------------------------------------*/
 
-
-
 STDMETHODIMP CIliveCacheSink::GetSessionAllocator(SAPDBMem_IRawAllocator* &allocator)
 {
+  /* PTS 1128888, PG
+     An allocator returned by this method serves as base for the three user allocators 
+     - OmsSessionUser T<taskId>
+     - OmsSessionCall T<taskId>
+     - OmsSessionTrans T<taskId>
+     of session <taskId>. So far, such an allocator was just named 'OMS Session'.
+     For analysis purpose this name isn't significant enough as several allocators
+     with this name exist. As such an allocator isn't destructed at the end of a
+     session but added to a pool and reused in a later session it doesn't make
+     sense to extend the name 'OMS Session' by 'T<taskId>' when constructing a
+     new allocator of this kind. Therefore as identificatior a unique number
+     provided by OMS_Session_Allocator_Cntr is appended to 'OMS Session'.
+  */
+
   if (!m_pSessionRawAllocator) {
+
     {
       RTESync_LockedScope lck(m_allocatorPoolLock);
       if (m_allocatorPool) {
@@ -2098,12 +2140,19 @@ STDMETHODIMP CIliveCacheSink::GetSessionAllocator(SAPDBMem_IRawAllocator* &alloc
       }
     }
 
-    m_pSessionRawAllocator = ::new (LVCMem_Allocator::Instance()) 
-      LVCMem_UserAllocator(reinterpret_cast<const unsigned char*>("OMS Session"),
+    SAPDB_UTF8 identifier[40];
+    OMS_Session_Allocator_Cntr.Increment();
+    sp77sprintf( (char*)identifier, sizeof(identifier), "OMS Session  %03d", 
+                 OMS_Session_Allocator_Cntr.GetValue() );
+    m_pSessionRawAllocator = 
+        ::new (LVCMem_Allocator::Instance()) 
+        /* LVCMem_UserAllocator(reinterpret_cast<const unsigned char*>("OMS Session"),          */
+        LVCMem_UserAllocator(reinterpret_cast<const unsigned char*>(identifier), /* PTS 1128888 */
         LVCMem_Allocator::Instance(),
         32 * 1024, 32 * 1024, SAPDBMem_RawAllocator::FREE_RAW_EXTENDS);
     m_pSessionRawAllocator->SetBadAllocHandler(SAPDBMem_RawAllocator::THROW_STD_BAD_ALLOC);
   }
+
   allocator = m_pSessionRawAllocator;
   return S_OK;
 }
@@ -2344,94 +2393,6 @@ STDMETHODIMP CIliveCacheSink::GetContainerId(
 
 /*---------------------------------------------------------------------------*/
 
-STDMETHODIMP CIliveCacheSink::VersionDictInsert(
-    const OmsVersionId  &versionId,
-    const OMS_Context   *pVersionContext,
-    tgg00_BasisError    &error) 
-{
-  return S_OK;
-}
-
-/*---------------------------------------------------------------------------*/
-
-STDMETHODIMP CIliveCacheSink::VersionDictFind(
-    const OmsVersionId   &versionId,
-    OMS_Context         **ppVersionContext) 
-{
-  return S_OK;
-}
-
-/*---------------------------------------------------------------------------*/
-
-STDMETHODIMP CIliveCacheSink::VersionDictDrop(
-    const OmsVersionId  &versionId,
-    tgg00_BasisError    &error) 
-{
-  return S_OK;
-}
-
-/*---------------------------------------------------------------------------*/
-
-STDMETHODIMP CIliveCacheSink::VersionDictShutdown() 
-{
-  return S_OK;
-}
-
-/*---------------------------------------------------------------------------*/
-
-STDMETHODIMP CIliveCacheSink::VersionDictUnloadOldestVersion(
-  bool &unloaded) 
-{
-  return S_OK;
-}
-
-/*---------------------------------------------------------------------------*/
-
-STDMETHODIMP CIliveCacheSink::VersionDictMarkNotUnloadable(
-    OMS_Context *pVersionContext)
-{
-  return S_OK;
-}
-
-/*---------------------------------------------------------------------------*/
-
-STDMETHODIMP CIliveCacheSink::VersionDictMarkUnloadable(
-    OMS_Context *pVersionContext,
-    bool         callFromDestructor)
-{
-  return S_OK;
-}
-
-/*---------------------------------------------------------------------------*/
-
-STDMETHODIMP CIliveCacheSink::VersionDictCreateIter(
-  OMS_LockMode   lockMode, 
-  void         **ppIter, 
-  OMS_Context  **ppContext)
-{
-  return S_OK;
-}
-
-/*---------------------------------------------------------------------------*/
-
-STDMETHODIMP CIliveCacheSink::VersionDictGetNext(
-  void         *pIter, 
-  bool          dropCurr,
-  OMS_Context **ppContext)
-{
-  return S_OK;
-}
-
-/*---------------------------------------------------------------------------*/
-
-STDMETHODIMP CIliveCacheSink::VersionDictDestructIter(
-  void  *pIter)
-{
-  return S_OK;
-}
-
-/*---------------------------------------------------------------------------*/
-
 inline void eo38_EvalTaskSelfIOInfo(const tsp92_RteTaskSelfIOInfo& currInfo, tsp92_RteTaskSelfIOInfo& resInfo)
 {
   resInfo.Cnt         = currInfo.Cnt        - resInfo.Cnt;
