@@ -39,11 +39,16 @@
 /*--------------------------------------------------------------------*/
 /* SAP includes                                                       */
 /*--------------------------------------------------------------------*/
+/*The following platforms already use the 6.20 Unicode NI libraries */
+#if !defined (WIN32) || (defined (WIN32) && defined (AMD64))
+#define SAPwithUNICODE 
+#endif
+
 #ifdef WIN32
   #define SAPonNT
 #endif
 
-#ifdef _IBMR2 
+#ifdef _IBMR2
   #define SAPonRS6000
 #endif
 
@@ -68,16 +73,35 @@
 #define SAPonLIN
 #endif
 
-#if defined(SVR4) && defined(NMP)
-  #define SAPonRM600
+#if defined SAPwithUNICODE
+#if defined (LINUX) && defined (X86_64)
+#include "sap/700/saptype.h"	/* nocheck */
+#include "sap/700/saptypeb.h"	/* nocheck */
+#include "sap/700/saptypec.h"	/* nocheck */
+#include "sap/700/sapuc.h"		/* nocheck */
+#include "sap/700/err.h"		/* nocheck */
+#include "sap/700/nixx.h"		/* nocheck */
+#include "sap/700/dptrace.h"	/* nocheck */
+#include "sap/700/sapuc2.h"		/* nocheck */
+#else
+#include "sap/640/saptype.h"	/* nocheck */
+#include "sap/640/saptypeb.h"	/* nocheck */
+#include "sap/640/saptypec.h"	/* nocheck */
+#include "sap/640/sapuc.h"		/* nocheck */
+#include "sap/640/err.h"		/* nocheck */
+#include "sap/640/nixx.h"		/* nocheck */
+#include "sap/640/dptrace.h"	/* nocheck */
+#include "sap/640/sapuc2.h"		/* nocheck */
+#endif
+#else /* Windows uses header from 46D (no unicode)! */
+#include "sap/saptype.h"	/* nocheck */
+#include "sap/saptypeb.h"	/* nocheck */
+#include "sap/saptypec.h"	/* nocheck */
+#include "sap/sapuc.h"		/* nocheck */
+#include "sap/err.h"		/* nocheck */
+#include "sap/nixx.h"		/* nocheck */
+#include "sap/dptrace.h"	/* nocheck */
 #endif
-
-#include "sap/saptype.h"
-#include "sap/saptypeb.h"
-#include "sap/saptypec.h"
-#include "sap/sapuc.h"
-#include "sap/err.h"
-#include "sap/nixx.h"
 /*===========================================================================*/
 #endif /* IMPLEMENTATION_MODULE || STUB_MODULE */
 /*===========================================================================*/
@@ -120,6 +144,10 @@
 #endif
 
 #define NI_DEFAULT_SERVICE 7269
+
+
+#define MAX_LENGTH_OF_UNICODE_CHARACTER		4
+#define MAX_PORT_LENGTH                     6
 /*===========================================================================*
  *  MACROS                                                                   *
  *===========================================================================*/
@@ -148,18 +176,23 @@ extern FILE *tf ; /* SAP global trace/error file descriptor */
 /*===========================================================================*
  *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
  *===========================================================================*/
-static void 
+static void
 eo40NiBuildErrorString  ( tsp00_ErrTextc VAR_ARRAY_REF  pErrText,
                           char                         *ErrorMsgHeader,
                           SAPRETURN                     SapRc );
 
-static void * 
+static void *
 eo40NiDupHandle         ( void                         *NiHandle );
 
-static void 
+
+static void
 eo40GetServiceAsString  ( unsigned short               *Service,
-                          char                         *szService,
-                          int                           szServiceLen) ;
+                          int                           szServiceLen,
+#if defined SAPwithUNICODE
+                          SAP_UC                       *szService);
+#else
+                          char                         *szService);
+#endif
 
 /*===========================================================================*
  *  PUBLIC FUNCTIONS (CODE)                                                  *
@@ -167,102 +200,12 @@ eo40GetServiceAsString  ( unsigned short               *Service,
 
 #endif /* IMPLEMENTATION_MODULE */
 
-#if defined IMPLEMENTATION_MODULE || defined STUB_MODULE 
-
-/*===========================================================================*/
-
-externC int   
-eo40NiIsSaprouterSring     (  char                   *StrToCheck )
-{
-  return( memcmp (StrToCheck, NI_HOSTSEP, NI_SEPSIZ) == 0 );
-}
-
-/*===========================================================================*/
-
-externC int   
-eo40NiExtractDBNodeFromSaprouterSring ( char                        *SaprouterSring,
-                                        tsp00_NodeIdc VAR_ARRAY_REF  ServerNode  ) 
-{
-  int IsSaprouterSring = eo40NiIsSaprouterSring(SaprouterSring);
-
-  if ( IsSaprouterSring )
-  {
-    int   NodeLen ;
-    char *NodeBeg = SaprouterSring + NI_SEPSIZ ,
-         *Ptr = NodeBeg ;
-
-    /* search last host in routerstring ==> server node */
-    while ( (Ptr = strstr ( Ptr, NI_HOSTSEP )) != NULL )
-    {
-      NodeBeg = Ptr + NI_SEPSIZ;
-      Ptr     = NodeBeg ;
-    }
-
-    Ptr = strchr(NodeBeg, *NI_HOSTSEP ) ;/* end of host */
-
-    NodeLen = Ptr ? (int)(Ptr - NodeBeg) : (int)strlen(NodeBeg);
-    
-    if ( NodeLen >= sizeof(tsp00_NodeIdc) )  
-      NodeLen = sizeof(tsp00_NodeIdc) - 1 ;
-
-    memcpy( ServerNode, NodeBeg, NodeLen );
-    ServerNode [ NodeLen+1 ] = '\0' ;
-    }
-
-  return ( IsSaprouterSring ) ;
-}
-
-/*===========================================================================*/
-
-externC int   
-eo40NiExtractServiceFromSaprouterSring ( char                        *SaprouterSring,
-                                         unsigned short              *Service )
-{
-  int IsSaprouterSring = eo40NiIsSaprouterSring(SaprouterSring);
-
-  if ( IsSaprouterSring )
-  {
-    char *LastHost = SaprouterSring ;
-    char *Ptr      = LastHost;
-
-    /* search last host in routerstring ==> server node */
-
-    while ( (Ptr = strstr ( LastHost+1, NI_HOSTSEP ) ) )
-      LastHost = Ptr ;
- 
-    if ( (Ptr = strstr ( LastHost, NI_SERVSEP )) == NULL ) 
-    {
-      *Service = 0 ;
-    }
-    else
-    {
-      /* There is only one NI_HOSTSEP AND a NI_SERVSEP */
-      int i;
-      char szService[6] ;
-
-      for ( i=0, Ptr += NI_SEPSIZ ;
-            i < sizeof(szService)-1 && isdigit(*Ptr) ; 
-            i++, Ptr++ )
-        szService[i] = *Ptr ;
-
-       szService[i] = '\0' ;
-
-      *Service = (unsigned short) atoi(szService);
-    }
-  }
-  return ( IsSaprouterSring ) ;
-}
-
-/*===========================================================================*/
-
-#endif /* IMPLEMENTATION_MODULE || STUB_MODULE */
-
 #if defined IMPLEMENTATION_MODULE
 
 /*===========================================================================*/
 
 externC int
-eo40NiIsAvailable() 
+eo40NiIsAvailable()
 {
   return(TRUE);
 }
@@ -271,19 +214,26 @@ eo40NiIsAvailable()
 
 #define CTRC_TRUNC      1
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiInit        ( tsp00_Pathc     VAR_ARRAY_REF  TraceFile,
                     int                            TraceLevel,
                     tsp00_ErrTextc  VAR_ARRAY_REF  ErrText )
 {
-  SAPRETURN               SapRc ;
-  tsp00_Uint              ulCommState;
-  extern FILE *CTrcInit ( const char   *  file,
-                          SAP_BYTE     mode,
-                          int          tracelevel);
+  SAPRETURN             SapRc ;
+  tsp00_Uint            ulCommState;
+#if defined SAPwithUNICODE
+  SAP_UC *				pTraceFileNameUC;
+  SAP_UC				nullByte = {'\0'};
 
-  tf = (FILE *)CTrcInit (TraceFile[0] != '\0' ? (char *)TraceFile : NULL , 
+  pTraceFileNameUC = (SAP_UC *) alloca ((strlen (TraceFile) + 1) * MAX_LENGTH_OF_UNICODE_CHARACTER);
+  A7sToUcs (pTraceFileNameUC, TraceFile);
+  tf = (FILE *)CTrcInit (pTraceFileNameUC[0] != nullByte ? pTraceFileNameUC : NULL ,
                          CTRC_TRUNC, TraceLevel);
+#else
+  tf = (FILE *)CTrcInit (TraceFile[0] != '\0' ? (char *)TraceFile : NULL , 
+						 CTRC_TRUNC, TraceLevel);
+
+#endif
 
   SapRc = NiInit();
 
@@ -300,7 +250,7 @@ eo40NiInit        ( tsp00_Pathc     VAR_ARRAY_REF  TraceFile,
 /*===========================================================================*/
 #define ERRMSG_NICOM_GET_SD_ERROR "Can't get sd from NiHdl:"
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiHandleToSocket        ( void                         *NiHandle ,
                               int                          *Sd,
                               tsp00_ErrTextc  VAR_ARRAY_REF  ErrText )
@@ -314,7 +264,7 @@ eo40NiHandleToSocket        ( void                         *NiHandle ,
     ulCommState = commErrOk_esp01 ;
   else
   {
-    ulCommState = commErrNotOk_esp01 ;  
+    ulCommState = commErrNotOk_esp01 ;
     eo40NiBuildErrorString ( ErrText, ERRMSG_NICOM_GET_SD_ERROR, SapRc );
     ErrTrace(tf);
   }
@@ -326,7 +276,7 @@ eo40NiHandleToSocket        ( void                         *NiHandle ,
 
 /*===========================================================================*/
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiDataAvailable   ( void                         *NiHandle ,
                         tsp00_ErrTextc VAR_ARRAY_REF  pErrText )
 {
@@ -341,7 +291,7 @@ eo40NiDataAvailable   ( void                         *NiHandle ,
   {
     case SAP_O_K    : ulCommState = commErrOk_esp01 ;        break ;
     case NIETIMEOUT : ulCommState = commErrWouldBlock_esp01; break ;
-    default         : ulCommState = commErrNotOk_esp01;      
+    default         : ulCommState = commErrNotOk_esp01;
                       eo40NiBuildErrorString ( pErrText,
                                                ERRMSG_NICOM_PEEK_ERROR, SapRc );
                       ErrTrace(tf);
@@ -360,14 +310,14 @@ int eo40NiIsValidHandle      ( void                     *NiHandle )
 
 /*===========================================================================*/
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiClose     ( void                         **NiHandle)
 {
   tsp00_Uint              ulCommState = commErrOk_esp01;
   SAPRETURN               SapRc ;
 
   if ( eo40NiIsValidHandle ( *NiHandle) )
-  { 
+  {
     SapRc = NiCloseHandle (**(NI_HDL **)NiHandle) ;
     free ( *NiHandle ) ;
     *NiHandle = NULL ;
@@ -377,12 +327,12 @@ eo40NiClose     ( void                         **NiHandle)
 
 /*===========================================================================*/
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiFinish(tsp00_ErrTextc VAR_ARRAY_REF   pErrText)
 {
   SAPRETURN               SapRc = NiExit() ;
   tsp00_Uint              ulCommState ;
-  
+
   if ( SapRc  == SAP_O_K )
     ulCommState = commErrOk_esp01 ;
   else
@@ -396,7 +346,7 @@ eo40NiFinish(tsp00_ErrTextc VAR_ARRAY_REF   pErrText)
 
 /*===========================================================================*/
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiListen    ( void                        **NiHandle,
                   unsigned short               *Service,
                   tsp00_ErrTextc VAR_ARRAY_REF  pErrText )
@@ -404,11 +354,16 @@ eo40NiListen    ( void                        **NiHandle,
   tsp00_Uint              CommState = commErrOk_esp01;
   SAPRETURN               SapRc ;
   NI_HDL                  ListenHandle ;
-  char                    szService[6] ;
-
-  eo40GetServiceAsString ( Service, szService, sizeof(szService) );
-
+  SAPDB_Char			  szService[MAX_PORT_LENGTH * MAX_LENGTH_OF_UNICODE_CHARACTER];	/* Maximal 6 unicode character   */
+																			            /* (we suppose that an charakter */
+																			            /* needs maximal 4 bytes)        */
+#if defined SAPwithUNICODE
+  eo40GetServiceAsString (Service, MAX_PORT_LENGTH, (SAP_UC *)szService);
+  SapRc = NiListen ((SAP_UC *)szService, &ListenHandle) ;
+#else
+  eo40GetServiceAsString (Service, MAX_PORT_LENGTH, szService);
   SapRc = NiListen (szService, &ListenHandle) ;
+#endif
 
   if ( SapRc == SAP_O_K)
   {
@@ -426,10 +381,10 @@ eo40NiListen    ( void                        **NiHandle,
 
 /*===========================================================================*/
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiAccept    ( void                        *NiListenHandle,
                   void                       **NiCommHandle,
-                  char                        *ServerNode, 
+                  char                        *ServerNode,
                   int                          MaxServerNodeLen,
                   tsp00_ErrTextc VAR_ARRAY_REF ErrText )
 {
@@ -442,14 +397,27 @@ eo40NiAccept    ( void                        *NiListenHandle,
 
   if ( SapRc == SAP_O_K)
   {
-    char *HostName = NiAddrToHost (&hostaddr) ;
+#if defined SAPwithUNICODE
+    SAP_UC *pHostNameUC = NiAddrToHost (&hostaddr);
+	if ( pHostNameUC == NULL )
+      pHostNameUC = NiAdrToStr (&hostaddr);
 
-    if ( HostName == NULL )
-      HostName = NiAdrToStr (&hostaddr);
+	if ( pHostNameUC != NULL )
+#else
+    char *pHostName = NiAddrToHost (&hostaddr) ;
+    if ( pHostName == NULL )
+        pHostName = NiAdrToStr (&hostaddr);
 
-    if ( HostName != NULL )
+	if ( pHostName != NULL )
+#endif
     {
-      strncpy ( ServerNode, HostName, MaxServerNodeLen ) ;
+#if defined SAPwithUNICODE
+      SAP_A7	* pHostName;
+
+	  pHostName = (SAP_A7 *) alloca (strlenU (pHostNameUC) + 1);
+	  UcnToA7n(pHostName, pHostNameUC, strlenU (pHostNameUC));
+#endif
+      strncpy ( ServerNode, (SAPDB_Char *)pHostName, MaxServerNodeLen ) ;
       ServerNode [ MaxServerNodeLen -1 ] = '\0' ;
     }
     *NiCommHandle = eo40NiDupHandle( &NiHandle ) ;
@@ -469,7 +437,7 @@ eo40NiAccept    ( void                        *NiListenHandle,
 
 /*===========================================================================*/
 
-tsp00_Uint     
+tsp00_Uint
 eo40NiSend ( void                         *SendHandle,
              char                         *pData,
              int                           Len,
@@ -480,8 +448,8 @@ eo40NiSend ( void                         *SendHandle,
   NI_HDL                  NiHandle = *((NI_HDL *)SendHandle ) ;
   int                     BytesSend;
 
-  for (BytesSend = 0; 
-       Len > 0 && CommState == commErrOk_esp01 ; 
+  for (BytesSend = 0;
+       Len > 0 && CommState == commErrOk_esp01 ;
        pData += BytesSend, Len -= BytesSend )
   {
     SapRc = NiRawWrite (NiHandle, (SAP_BYTE *)pData, Len, NI_BLOCK, &BytesSend);
@@ -496,7 +464,7 @@ eo40NiSend ( void                         *SendHandle,
 }
 
 /*===========================================================================*/
-tsp00_Uint     
+tsp00_Uint
 eo40NiReceive ( void                         *RecvHandle,
                 char                         *pData,
                 tsp00_Uint                    ulMaxDataLen,
@@ -505,7 +473,7 @@ eo40NiReceive ( void                         *RecvHandle,
 {
   #undef  MF__
   #define MF__ MOD__"eo40NiReceive"
- 
+
   tsp00_Uint              CommState = commErrOk_esp01;
   SAPRETURN               SapRc ;
   SAP_INT                 readLen;
@@ -531,25 +499,37 @@ eo40NiReceive ( void                         *RecvHandle,
 }
 
 /*===========================================================================*/
-tsp00_Uint     
+tsp00_Uint
 eo40NiConnectToServer( teo40_NiConnectInfoRecord    *pNIConnInfo,
                        tsp00_ErrTextc VAR_ARRAY_REF  ErrText )
 {
   SAPRETURN      SapRc ;
   tsp00_Uint     CommState;
   NI_HDL         NiHandle ;
-  char           szService[6] ;
+  SAPDB_Char	 szService[MAX_PORT_LENGTH * MAX_LENGTH_OF_UNICODE_CHARACTER];  /* Maximal 6 unicode character   */
+															                    /* (we suppose that an charakter */
+															                    /* needs maximal 4 bytes)        */
+#if defined SAPwithUNICODE
+  SAP_UC *		 pSapRouterString;		
 
-  eo40GetServiceAsString ( &pNIConnInfo->usServicePort, szService, sizeof(szService) );
 
+  eo40GetServiceAsString (&pNIConnInfo->usServicePort, MAX_PORT_LENGTH, (SAP_UC *)szService);
+
+  pSapRouterString = (SAP_UC *) alloca ((strlen (pNIConnInfo->szSapRouterString) + 1) * MAX_LENGTH_OF_UNICODE_CHARACTER);
+  A7sToUcs (pSapRouterString, pNIConnInfo->szSapRouterString);
+
+  SapRc = NiRawConnect (pSapRouterString, (SAP_UC *)szService, NI_BLOCK, &NiHandle);
+#else
+  eo40GetServiceAsString ( &pNIConnInfo->usServicePort, MAX_PORT_LENGTH, szService);
   SapRc = NiRawConnect (pNIConnInfo->szSapRouterString, szService, 
                         NI_BLOCK, &NiHandle);
+#endif
 
   if ( SapRc == SAP_O_K )
   {
     pNIConnInfo->NiHandle = eo40NiDupHandle( &NiHandle ) ;
     CommState = pNIConnInfo->NiHandle ? commErrOk_esp01 : commErrNotOk_esp01 ;
-  } 
+  }
   else
   {
     eo40NiBuildErrorString ( ErrText, ERRMSG_NICOM_CONNECT_ERROR, SapRc );
@@ -579,7 +559,7 @@ eo40NiConnectToServer( teo40_NiConnectInfoRecord    *pNIConnInfo,
 #define ERR_NIEPING             "ping received"
 #define ERR_NIECONN_PENDING     "NIECONN_PENDING"
 #define ERR_NIEVERSION          "bad version"
- 
+
 #define ERR_NIEROUT_HOST_UNKNOWN "ROUT: unknown host"
 #define ERR_NIEROUT_SERV_UNKNOWN "ROUT: unknown service"
 #define ERR_NIEROUT_PERM_DENIED  "ROUT: perm. denied"
@@ -600,7 +580,7 @@ eo40NiConnectToServer( teo40_NiConnectInfoRecord    *pNIConnInfo,
 #define ERR_NI_UNKNOWN_RC        "NI: RC="
 /*===========================================================================*/
 
-static void 
+static void
 eo40NiBuildErrorString ( tsp00_ErrTextc VAR_ARRAY_REF ErrTextC,
                          char                        *ErrorMsgHeader,
                          SAPRETURN                    SapRc )
@@ -610,36 +590,36 @@ eo40NiBuildErrorString ( tsp00_ErrTextc VAR_ARRAY_REF ErrTextC,
   int  ErrorHeaderLen  = (int)strlen(ErrorMsgHeader) ;
   int  ErrorTrailerLen = sizeof(tsp00_ErrTextc) - ErrorHeaderLen - 1 ;
 
-  switch ( SapRc ) 
+  switch ( SapRc )
   {
-    case NIEHOST_UNKNOWN     : NiErrorString = ERR_NIEHOST_UNKNOWN     ; break ; 
-    case NIESERV_UNKNOWN     : NiErrorString = ERR_NIESERV_UNKNOWN     ; break ; 
-    case NIESERV_USED        : NiErrorString = ERR_NIESERV_USED        ; break ; 
-    case NIETIMEOUT          : NiErrorString = ERR_NIETIMEOUT          ; break ; 
-    case NIECONN_BROKEN      : NiErrorString = ERR_NIECONN_BROKEN      ; break ; 
-    case NIEINVAL            : NiErrorString = ERR_NIEINVAL            ; break ; 
+    case NIEHOST_UNKNOWN     : NiErrorString = ERR_NIEHOST_UNKNOWN     ; break ;
+    case NIESERV_UNKNOWN     : NiErrorString = ERR_NIESERV_UNKNOWN     ; break ;
+    case NIESERV_USED        : NiErrorString = ERR_NIESERV_USED        ; break ;
+    case NIETIMEOUT          : NiErrorString = ERR_NIETIMEOUT          ; break ;
+    case NIECONN_BROKEN      : NiErrorString = ERR_NIECONN_BROKEN      ; break ;
+    case NIEINVAL            : NiErrorString = ERR_NIEINVAL            ; break ;
     case NIECONN_REFUSED     : NiErrorString = ERR_NIECONN_REFUSED     ; break ;
     case NIEPING             : NiErrorString = ERR_NIEPING             ; break ;
     case NIECONN_PENDING     : NiErrorString = ERR_NIECONN_PENDING     ; break ;
     case NIEVERSION          : NiErrorString = ERR_NIEVERSION          ; break ;
-    case NIEINTERN           : NiErrorString = ERR_NIEINTERN           ; break ; 
+    case NIEINTERN           : NiErrorString = ERR_NIEINTERN           ; break ;
 
     case NIEROUT_HOST_UNKNOWN: NiErrorString = ERR_NIEROUT_HOST_UNKNOWN; break ;
     case NIEROUT_SERV_UNKNOWN: NiErrorString = ERR_NIEROUT_SERV_UNKNOWN; break ;
     case NIEROUT_PERM_DENIED : NiErrorString = ERR_NIEROUT_PERM_DENIED ; break ;
     case NIEROUT_CONN_REFUSED: NiErrorString = ERR_NIEROUT_CONN_REFUSED; break ;
     case NIEROUT_CONN_BROKEN : NiErrorString = ERR_NIEROUT_CONN_BROKEN ; break ;
-    case NIEROUT_INTERN      : NiErrorString = ERR_NIEROUT_INTERN      ; break ; 
-    case NIEROUT_VERSION     : NiErrorString = ERR_NIEROUT_VERSION     ; break ; 
-    case NIEROUT_CANCELED    : NiErrorString = ERR_NIEROUT_CANCELED    ; break ; 
+    case NIEROUT_INTERN      : NiErrorString = ERR_NIEROUT_INTERN      ; break ;
+    case NIEROUT_VERSION     : NiErrorString = ERR_NIEROUT_VERSION     ; break ;
+    case NIEROUT_CANCELED    : NiErrorString = ERR_NIEROUT_CANCELED    ; break ;
 
-    case NIEROUT_SHUTDOWN    : NiErrorString = ERR_NIEROUT_SHUTDOWN    ; break ; 
-    case NIEROUT_INFO_DENIED : NiErrorString = ERR_NIEROUT_INFO_DENIED ; break ; 
-    case NIEROUT_OVERFLOW    : NiErrorString = ERR_NIEROUT_OVERFLOW    ; break ; 
-    case NIEROUT_MODE_DENIED : NiErrorString = ERR_NIEROUT_MODE_DENIED ; break ; 
-    case NIEROUT_NOCLIENT    : NiErrorString = ERR_NIEROUT_NOCLIENT    ; break ; 
-    case NIEROUT_EXTERN      : NiErrorString = ERR_NIEROUT_EXTERN      ; break ; 
-    case NIEROUT_SNC_FAILURE : NiErrorString = ERR_NIEROUT_SNC_FAILURE ; break ; 
+    case NIEROUT_SHUTDOWN    : NiErrorString = ERR_NIEROUT_SHUTDOWN    ; break ;
+    case NIEROUT_INFO_DENIED : NiErrorString = ERR_NIEROUT_INFO_DENIED ; break ;
+    case NIEROUT_OVERFLOW    : NiErrorString = ERR_NIEROUT_OVERFLOW    ; break ;
+    case NIEROUT_MODE_DENIED : NiErrorString = ERR_NIEROUT_MODE_DENIED ; break ;
+    case NIEROUT_NOCLIENT    : NiErrorString = ERR_NIEROUT_NOCLIENT    ; break ;
+    case NIEROUT_EXTERN      : NiErrorString = ERR_NIEROUT_EXTERN      ; break ;
+    case NIEROUT_SNC_FAILURE : NiErrorString = ERR_NIEROUT_SNC_FAILURE ; break ;
 
     default                  : strcpy(UnknownRC, ERR_NI_UNKNOWN_RC) ;
                                sql47_itoa(SapRc, UnknownRC  + sizeof(ERR_NI_UNKNOWN_RC),
@@ -649,7 +629,7 @@ eo40NiBuildErrorString ( tsp00_ErrTextc VAR_ARRAY_REF ErrTextC,
 
   strcpy( ErrTextC, ErrorMsgHeader ) ;
 
-  if ( ErrorTrailerLen > 0 ) 
+  if ( ErrorTrailerLen > 0 )
   {
     ErrTextC [ ErrorHeaderLen ] = ':';
     strncpy( &ErrTextC [ ErrorHeaderLen + 1] , NiErrorString, ErrorTrailerLen ) ;
@@ -659,7 +639,7 @@ eo40NiBuildErrorString ( tsp00_ErrTextc VAR_ARRAY_REF ErrTextC,
 
 /*===========================================================================*/
 
-static void * 
+static void *
 eo40NiDupHandle ( void *NiHandle )
 {
   void *NiDupHandle = malloc ( sizeof(NI_HDL) ) ;
@@ -671,12 +651,19 @@ eo40NiDupHandle ( void *NiHandle )
 }
 
 /*===========================================================================*/
-static void 
+static void
 eo40GetServiceAsString ( unsigned short *Service,
-                         char           *szService,
-                         int             szServiceLen)
+                         int             szServiceLen,
+#if defined SAPwithUNICODE
+                         SAP_UC         *szService)
+#else
+                         char           *szService)
+#endif
 {
   unsigned short NiServiceNo ;
+  SAPDB_Char *   szServiceAsChar;
+
+  szServiceAsChar = (SAPDB_Char *)alloca (szServiceLen);
 
   if ( !Service || *Service == 0 )
   {
@@ -687,7 +674,13 @@ eo40GetServiceAsString ( unsigned short *Service,
   else
     NiServiceNo = *Service ;
 
-  sql47_itoa( ( int )NiServiceNo, szService, szServiceLen);
+  sql47_itoa( ( int )NiServiceNo, szServiceAsChar, szServiceLen);
+
+#if defined SAPwithUNICODE
+  A7sToUcs (szService, szServiceAsChar);
+#else
+  strcpy (szService, szServiceAsChar);
+#endif
 }
 
 /*===========================================================================*/
@@ -709,7 +702,7 @@ typedef struct
 
 /* Fill in *INFO with the following information about ADDRESS.
    Returns 0 iff no shared object's segments contain that address.  */
-int dladdr (const void *__address, Dl_info *__info) 
+int dladdr (const void *__address, Dl_info *__info)
 {
     return 0;
 }
@@ -724,12 +717,12 @@ int dladdr (const void *__address, Dl_info *__info)
 /*===========================================================================*/
 
 externC int
-eo40NiIsAvailable() 
+eo40NiIsAvailable()
 {
   return( 0 );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiInit        ( tsp00_Pathc     VAR_ARRAY_REF  TraceFile,
                     int                            TraceLevel,
                     tsp00_ErrTextc  VAR_ARRAY_REF  ErrText )
@@ -737,7 +730,7 @@ eo40NiInit        ( tsp00_Pathc     VAR_ARRAY_REF  TraceFile,
   return ( commErrNotOk_esp01 );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiHandleToSocket        ( void                         *NiHandle ,
                               int                          *Sd,
                               tsp00_ErrTextc  VAR_ARRAY_REF  ErrText )
@@ -745,14 +738,14 @@ eo40NiHandleToSocket        ( void                         *NiHandle ,
   return ( commErrNotOk_esp01 );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiDataAvailable   ( void                         *NiHandle ,
                         tsp00_ErrTextc VAR_ARRAY_REF  pErrText )
 {
   return ( commErrNotOk_esp01 );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiSqlCancelDump       ( teo40_NiConnectInfoRecord    *pNIConnInfo,
                             tsp00_Uint                    ulReqType,
                             tsp00_ErrTextc VAR_ARRAY_REF  ErrText )
@@ -761,46 +754,46 @@ eo40NiSqlCancelDump       ( teo40_NiConnectInfoRecord    *pNIConnInfo,
 }
 
 
-externC int   
-eo40NiExtractDBNodeFromSaprouterSring ( char                         *SaprouterSring,
+externC int
+eo40NiExtractDBNodeFromSaprouterSring ( char                         *SaprouterString,
                                         tsp00_NodeIdc  VAR_ARRAY_REF  ServerNode  )
 {
   return ( commErrNotOk_esp01 );
 }
 
-externC int   
-eo40NiExtractServiceFromSaprouterSring ( char                        *SaprouterSring,
+externC int
+eo40NiExtractServiceFromSaprouterSring ( char                        *SaprouterString,
                                          unsigned short              *Service )
 {
   *Service = 0 ;
   return ( commErrNotOk_esp01 );
 }
 
-externC int   
+externC int
 eo40NiIsSaprouterSring   (  char                  *StrToCheck )
 {
   return ( 0);
 }
 
-externC int   
+externC int
 eo40NiIsValidHandle      ( void                   *NiHandle )
 {
   return ( 0 );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiClose     ( void                         **NiHandle)
 {
   return ( commErrNotOk_esp01 );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiFinish(tsp00_ErrTextc VAR_ARRAY_REF   pErrText)
 {
   return ( commErrNotOk_esp01 );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiListen    ( void                         **NiHandle,
                   unsigned short                *Service,
                   tsp00_ErrTextc VAR_ARRAY_REF   pErrText )
@@ -808,17 +801,17 @@ eo40NiListen    ( void                         **NiHandle,
   return ( commErrNotOk_esp01 );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiAccept    ( void                          *NiListenHandle,
                   void                         **NiCommHandle,
-                  char                          *ServerNode, 
+                  char                          *ServerNode,
                   int                            MaxServerNodeLen,
                   tsp00_ErrTextc VAR_ARRAY_REF   pErrText )
 {
   return ( commErrNotOk_esp01 );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiSend              ( void                         *SendHandle,
                           char                         *pData,
                           int                           Len,
@@ -827,7 +820,7 @@ eo40NiSend              ( void                         *SendHandle,
   return ( commErrNotOk_esp01 );
 }
 
-tsp00_Uint     
+tsp00_Uint
 eo40NiReceive ( void                         *RecvHandle,
                 char                         *pData,
                 tsp00_Uint                    ulMaxDataLen,
@@ -837,7 +830,7 @@ eo40NiReceive ( void                         *RecvHandle,
   return ( commErrNotOk_esp01 );
 }
 
-tsp00_Uint     
+tsp00_Uint
 eo40NiConnectToServer   ( teo40_NiConnectInfoRecord    *pNIConnInfo,
                           tsp00_ErrTextc VAR_ARRAY_REF  ErrText )
 {
@@ -855,7 +848,7 @@ eo40NiConnectToServer   ( teo40_NiConnectInfoRecord    *pNIConnInfo,
  *===========================================================================*/
 
 #include <string.h>
- 
+
 /*===========================================================================*
  *  DEFINES                                                                  *
  *===========================================================================*/
@@ -996,19 +989,19 @@ eo40NiLoadLib ()
     FreeLibrary ( hSAPNI );
 	return 0;
   }
-  
+
   fpeo40NiInit = (eo40NiInit_t) GetProcAddress (hSAPNI, TEXT ("eo40NiInit"));
   if ( fpeo40NiInit == 0 ) {
     FreeLibrary ( hSAPNI );
 	return 0;
   }
-  
+
   fpeo40NiHandleToSocket = (eo40NiHandleToSocket_t) GetProcAddress (hSAPNI, TEXT ("eo40NiHandleToSocket"));
   if ( fpeo40NiHandleToSocket == 0 ) {
     FreeLibrary ( hSAPNI );
 	return 0;
   }
-  
+
   fpeo40NiDataAvailable = (eo40NiDataAvailable_t) GetProcAddress (hSAPNI, TEXT ("eo40NiDataAvailable"));
   if ( fpeo40NiDataAvailable == 0 ) {
     FreeLibrary ( hSAPNI );
@@ -1070,29 +1063,29 @@ eo40NiLoadLib ()
  *  PUBLIC FUNCTIONS (CODE)                                                  *
  *===========================================================================*/
 #if 0
-externC int   
+externC int
 eo40NiIsSaprouterSring   (  char                  *StrToCheck )
 {
   return ( fpeo40NiIsSaprouterSring ( StrToCheck ) );
 }
 
-externC int   
-eo40NiExtractDBNodeFromSaprouterSring ( char                         *SaprouterSring,
+externC int
+eo40NiExtractDBNodeFromSaprouterSring ( char                         *SaprouterString,
                                         tsp00_NodeIdc  VAR_ARRAY_REF  ServerNode  )
 {
-  return ( fpeo40NiExtractDBNodeFromSaprouterSring ( SaprouterSring, ServerNode ) );
+  return ( fpeo40NiExtractDBNodeFromSaprouterSring ( SaprouterString, ServerNode ) );
 }
 
-externC int   
-eo40NiExtractServiceFromSaprouterSring ( char                        *SaprouterSring,
+externC int
+eo40NiExtractServiceFromSaprouterSring ( char                        *SaprouterString,
                                          unsigned short              *Service )
 {
-  return ( fpeo40NiExtractServiceFromSaprouterSring ( SaprouterSring, Service ) );
+  return ( fpeo40NiExtractServiceFromSaprouterSring ( SaprouterString, Service ) );
 }
 #endif
 
 externC int
-eo40NiIsAvailable() 
+eo40NiIsAvailable()
 {
   HANDLE hLib;
 
@@ -1103,7 +1096,7 @@ eo40NiIsAvailable()
   return( fpeo40NiIsAvailable () );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiInit        ( tsp00_Pathc     VAR_ARRAY_REF  TraceFile,
                     int                            TraceLevel,
                     tsp00_ErrTextc  VAR_ARRAY_REF  ErrText )
@@ -1119,7 +1112,7 @@ eo40NiInit        ( tsp00_Pathc     VAR_ARRAY_REF  TraceFile,
   return ( fpeo40NiInit ( TraceFile, TraceLevel, ErrText ) );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiHandleToSocket        ( void                         *NiHandle ,
                               int                          *Sd,
                               tsp00_ErrTextc  VAR_ARRAY_REF  ErrText )
@@ -1127,32 +1120,32 @@ eo40NiHandleToSocket        ( void                         *NiHandle ,
   return ( fpeo40NiHandleToSocket ( NiHandle, Sd, ErrText ));
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiDataAvailable   ( void                         *NiHandle ,
                         tsp00_ErrTextc VAR_ARRAY_REF  pErrText )
 {
   return ( fpeo40NiDataAvailable ( NiHandle, pErrText ) );
 }
 
-externC int   
+externC int
 eo40NiIsValidHandle      ( void                   *NiHandle )
 {
   return ( fpeo40NiIsValidHandle ( NiHandle ) );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiClose     ( void                         **NiHandle)
 {
   return ( fpeo40NiClose (NiHandle) );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiFinish(tsp00_ErrTextc VAR_ARRAY_REF   pErrText)
 {
   return ( fpeo40NiFinish ( pErrText ) );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiListen    ( void                         **NiHandle,
                   unsigned short                *Service,
                   tsp00_ErrTextc VAR_ARRAY_REF   pErrText )
@@ -1160,17 +1153,17 @@ eo40NiListen    ( void                         **NiHandle,
   return ( fpeo40NiListen ( NiHandle, Service, pErrText ) );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiAccept    ( void                          *NiListenHandle,
                   void                         **NiCommHandle,
-                  char                          *ServerNode, 
+                  char                          *ServerNode,
                   int                            MaxServerNodeLen,
                   tsp00_ErrTextc VAR_ARRAY_REF   pErrText )
 {
   return ( fpeo40NiAccept ( NiListenHandle, NiCommHandle, ServerNode, MaxServerNodeLen, pErrText ) );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiSend              ( void                         *SendHandle,
                           char                         *pData,
                           int                           Len,
@@ -1179,7 +1172,7 @@ eo40NiSend              ( void                         *SendHandle,
   return ( fpeo40NiSend ( SendHandle, pData, Len, pErrText ) );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiReceive ( void                         *RecvHandle,
                 char                         *pData,
                 tsp00_Uint                    ulMaxDataLen,
@@ -1189,7 +1182,7 @@ eo40NiReceive ( void                         *RecvHandle,
   return ( fpeo40NiReceive ( RecvHandle, pData, ulMaxDataLen,  BytesReceived, pErrText) );
 }
 
-externC tsp00_Uint     
+externC tsp00_Uint
 eo40NiConnectToServer   ( teo40_NiConnectInfoRecord    *pNIConnInfo,
                           tsp00_ErrTextc VAR_ARRAY_REF  ErrText )
 {
