@@ -174,10 +174,163 @@ eo420ExtractRteConnectPacket( teo003_ConPktParamRecord    *pConPktParamRec,
                               teo003_RteConnectPacketRec  *pConnectPacket,
                               tsp00_ErrTextc VAR_ARRAY_REF pErrText );
 
+
+static tsp00_Uint1 LoadUint1  ( tsp00_Uint1 const *   pIn );
+static tsp00_Uint2 LoadUint2  ( tsp00_Uint2 const *   pIn );
+static tsp00_Uint4 LoadUint4  ( tsp00_Uint4 const *   pIn );
+static tsp00_Uint8 LoadUint8  ( tsp00_Uint8 const *   pIn );
+
+static void        StoreUint1 ( tsp00_Uint1 * const   pOut,
+                                tsp00_Uint1 const     value );
+static void        StoreUint2 ( tsp00_Uint2 * const   pOut,
+                                tsp00_Uint2 const     value );
+static void        StoreUint4 ( tsp00_Uint4 * const   pOut,
+                                tsp00_Uint4 const     value );
+static void        StoreUint8 ( tsp00_Uint8 * const   pOut,
+                                tsp00_Uint8 const     value );
+
 /*===========================================================================*
  *  PUBLIC FUNCTIONS (CODE)                                                  *
  *===========================================================================*/
 
+externC tsp00_Uint eo420FillPingPacket ( char*       pData,
+                                         tsp00_Uint  maxLength,
+                                         tsp00_Uint1 hops,
+                                         char*       peerName,
+                                         char*       versionString,
+                                         char*       fillPattern )
+{
+    teo003_PingRecord*  pPingRecord = (teo003_PingRecord*)pData;
+    tsp00_Uint2         len;
+    tsp00_Uint2         dataLen = 0;
+
+    if ( sizeof(teo003_PingRecord) + 3 <= maxLength )
+    {
+      pData = (char*)pPingRecord + sizeof(teo003_PingRecord);
+
+      pPingRecord->RecordVersion = 0;
+      StoreUint1 ( &pPingRecord->Hops, hops);
+      StoreUint2 ( &pPingRecord->DataOffset, sizeof(teo003_PingRecord) );
+
+      maxLength -= sizeof(teo003_PingRecord) + 3; /* subtract record and end marker */
+
+      if ( peerName )
+      {
+          len = 3 + strlen(peerName) + 1;
+          if  ( len <= maxLength )
+          {
+              pData[0] = 's'; /* server name identifier */
+              StoreUint2 ( (tsp00_Uint2*)(pData + 1), len - 3 );
+              strcpy ( pData + 3,  peerName );
+              pData                += len;
+              maxLength            -= len;
+              dataLen              += len;
+          }
+      }
+
+
+      if ( versionString ) 
+      {
+          len = 3 + strlen(versionString) + 1;
+          if ( len <= maxLength )
+          {
+              pData[0] = 'v'; /* verion string identifier */
+              StoreUint2 ( (tsp00_Uint2*)(pData + 1), len - 3 );
+              strcpy ( pData + 3, versionString );
+              pData                += len;
+              maxLength            -= len;
+              dataLen              += len;
+          }
+      }
+
+      if ( fillPattern )
+      {
+          len = 3 + strlen(fillPattern) + 1;
+
+          while ( len <= maxLength )
+          {
+              pData[0] = '#'; /* fill pattern string identifier */
+              StoreUint2 ( (tsp00_Uint2*)(pData + 1), len - 3 );
+              strcpy ( pData + 3, fillPattern );
+              pData                += len;
+              maxLength            -= len;
+              dataLen              += len;
+          }
+      }
+
+      pData[0] = 0;  /* add end marker, maxlength correction already considered! */
+      StoreUint2 ( (tsp00_Uint2*)(pData + 1), 0 );
+      pData   += 3;
+      dataLen += 3;
+
+      if ( fillPattern )
+      {
+          /* fill up with blanks */
+          memset ( pData, 0, maxLength );
+          dataLen += maxLength;
+      }
+
+      /* now we know the data length, store it! */
+      StoreUint4 ( &pPingRecord->DataLen, dataLen );
+
+      return dataLen + sizeof(teo003_PingRecord);
+    }
+
+    return 0;   /* nothing inserted! */
+}
+
+/*------------------------------*/
+
+externC tsp00_Bool eo420ExtractPingPacket ( char*         pData,
+                                            tsp00_Uint    length,
+                                            tsp00_Uint1*  hops,
+                                            char**        peerName,
+                                            char**        versionString )
+{
+    tsp00_Bool          ok          = true;
+    teo003_PingRecord*  pPingRecord = (teo003_PingRecord*)pData;
+    tsp00_Uint2         dataLen     = LoadUint4( &pPingRecord->DataLen );
+    tsp00_Uint2         valueLength;
+
+    if ( peerName )     *peerName      = 0;
+    if ( versionString )*versionString = 0;
+    if (hops)           *hops          = 0;
+
+    if ( length >= sizeof(teo003_PingRecord) + dataLen )
+    {
+        if (hops) 
+            *hops  = LoadUint1( &pPingRecord->Hops);
+
+        pData  = (char*)pPingRecord + LoadUint2( &pPingRecord->DataOffset );
+        length = dataLen;
+
+        valueLength = LoadUint2( (tsp00_Uint2*)(pData + 1) );
+
+        while ( length && valueLength )
+        {
+            switch (pData[0])
+            {
+            case 's' :
+                if (peerName) *peerName = pData + 3;
+                break;
+            case 'v' :
+                if (versionString) *versionString = pData + 3;
+                break;
+            }
+
+            length      -= valueLength + 3;
+            pData       += valueLength + 3;
+            valueLength  = LoadUint2( (tsp00_Uint2*)(pData + 1) );
+        }
+    }
+    else
+        ok = false;
+
+    return ok;
+}
+
+/*------------------------------*/
+
 externC tsp00_Uint1  eo420NewSwapType ()
 { 
   static  tsp00_Uint1               SwapType = (tsp00_Uint1)UNDEF_SP00;
@@ -201,6 +354,10 @@ externC tsp00_Uint1  eo420NewSwapType ()
 
 /*------------------------------*/
 
+#if !defined (KERNEL_LZU)
+
+/*------------------------------*/
+
 externC tsp00_Uint eo420ServiceTypeToMessClass ( tsp00_Uint     ulServiceType )
 {
   #undef  MF__
@@ -813,10 +970,100 @@ externC tsp00_Uint eo420SetNiSocketBufferSize ( void                         *Ni
   return ( ulCommState );
 }
 
+#endif
+
 /*===========================================================================*
  *  LOCAL/PRIVATE/PROTECTED FUNCTIONS (CODE)                                 *
  *===========================================================================*/
 
+static tsp00_Uint1 LoadUint1 ( tsp00_Uint1 const * pIn )
+{
+    return pIn[0];  /* This function does not make much sense,
+                       but it helps to keep in mind the value 
+                       swapping if the value type changes */
+}
+
+/*---------------------------------------------------------------------------*/
+
+static void StoreUint1 ( tsp00_Uint1 * const   pOut,
+                         tsp00_Uint1 const     value )
+{
+    pOut[0] = value; /* This function does not make much sense,
+                        but it helps to keep in mind the value 
+                        swapping if the value type changes */
+}
+
+/*---------------------------------------------------------------------------*/
+
+static tsp00_Uint2 LoadUint2 ( tsp00_Uint2 const * pIn )
+{
+    return ( (tsp00_Uint2)(((char*)pIn)[ 0 ]) << 8 ) | 
+           ( (tsp00_Uint2)(((char*)pIn)[ 1 ]) );
+}
+
+/*---------------------------------------------------------------------------*/
+
+static void StoreUint2 ( tsp00_Uint2 * const   pOut,
+                         tsp00_Uint2 const     value )
+{
+    ((unsigned char*)pOut)[0] = (unsigned char)(value >> 8);
+    ((unsigned char*)pOut)[1] = (unsigned char) value;
+}
+
+/*---------------------------------------------------------------------------*/
+
+static tsp00_Uint4 LoadUint4 ( tsp00_Uint4 const * pIn )
+{
+    return ( (tsp00_Int4)(((unsigned char*)pIn)[ 0 ]) << 24 ) |
+           ( (tsp00_Int4)(((unsigned char*)pIn)[ 1 ]) << 16 ) |
+           ( (tsp00_Int4)(((unsigned char*)pIn)[ 2 ]) <<  8 ) |
+           ( (tsp00_Int4) ((unsigned char*)pIn)[ 3 ]);
+}
+
+/*---------------------------------------------------------------------------*/
+
+static void StoreUint4 ( tsp00_Uint4 * const   pOut,
+                         tsp00_Uint4 const     value )
+{
+    ((unsigned char*)pOut)[0] = (unsigned char)(value >> 24);
+    ((unsigned char*)pOut)[1] = (unsigned char)(value >> 16);
+    ((unsigned char*)pOut)[2] = (unsigned char)(value >>  8);
+    ((unsigned char*)pOut)[3] = (unsigned char) value;
+}
+
+/*---------------------------------------------------------------------------*/
+
+static tsp00_Uint8 LoadUint8 ( tsp00_Uint8 const * pIn )
+{
+    return ( (tsp00_Int8)((unsigned char*)pIn[ 0 ]) << 56 ) |
+           ( (tsp00_Int8)((unsigned char*)pIn[ 1 ]) << 48 ) |
+           ( (tsp00_Int8)((unsigned char*)pIn[ 2 ]) << 40 ) |
+           ( (tsp00_Int8)((unsigned char*)pIn[ 3 ]) << 32 ) |
+           ( (tsp00_Int8)((unsigned char*)pIn[ 4 ]) << 24 ) |
+           ( (tsp00_Int8)((unsigned char*)pIn[ 5 ]) << 16 ) |
+           ( (tsp00_Int8)((unsigned char*)pIn[ 6 ]) <<  8 ) |
+           ( (tsp00_Int8) (unsigned char*)pIn[ 7 ]);
+}
+
+/*---------------------------------------------------------------------------*/
+
+static void StoreUint8 ( tsp00_Uint8 * const      pOut,
+                         tsp00_Uint8 const        value )
+{
+    ((unsigned char*)pOut)[0] = (unsigned char)(value >> 56);
+    ((unsigned char*)pOut)[1] = (unsigned char)(value >> 48);
+    ((unsigned char*)pOut)[2] = (unsigned char)(value >> 40);
+    ((unsigned char*)pOut)[3] = (unsigned char)(value >> 32);
+    ((unsigned char*)pOut)[4] = (unsigned char)(value >> 24);
+    ((unsigned char*)pOut)[5] = (unsigned char)(value >> 16);
+    ((unsigned char*)pOut)[6] = (unsigned char)(value >>  8);
+    ((unsigned char*)pOut)[7] = (unsigned char) value;
+}
+
+/*---------------------------------------------------------------------------*/
+
+#if !defined (KERNEL_LZU)
+
 static tsp00_Uint eo420ReceivePacket ( void                        *RecvHandle,
                                        RECV_PKT_FUNC_PTR_EO003      RecvFunc,
                                        tsp00_Uint                   ulSwapType,
@@ -1538,7 +1785,6 @@ static tsp00_Uint eo420UnpackInt2 ( tsp00_Uint                    ulSwapTyp,
   {
   #undef  MF__
   #define MF__ MOD__"eo420UnpackInt2"
-
   register unsigned char *puc;
 
   DBGPAS;
@@ -1614,6 +1860,7 @@ static tsp00_Uint eo420UnpackInt4 ( tsp00_Uint                    ulSwapTyp,
 
   return ( commErrOk_esp01 );
   }
+#endif
 
 /*===========================================================================*
  *  END OF CODE                                                              *
