@@ -51,6 +51,7 @@
 #include "heo69.h"
 
 #include "RunTime/Configuration/RTEConf_ParameterAccessKernelInterface.h" 
+#include "SAPDBCommon/SAPDB_sprintf.h"
 /*===========================================================================*
  *  DEFINES                                                                  *
  *===========================================================================*/
@@ -144,9 +145,10 @@ static tsp00_Int4  eo70_PrepOneToOne     ( teo70k_ClusterInfo     *pCluInfo,
                                      tsp00_Int4                    ulMaxUKTs);
 
 static tsp00_Int4  eo70_CreateVarTaskUKTs( teo70k_ClusterInfo     *pCluInfo,
-                                     teo70k_VarTaskCntInfo  *pVarTasks,
-                                     tsp00_Int4                   *plThrdIdx,
-                                     tsp00_Int4                    lMaxUKTs);
+                                           teo70k_VarTaskCntInfo  *pVarTasks,
+                                           tsp00_Int4             *plThrdIdx,
+                                           tsp00_Int4              lMaxUKTs,
+                                           tsp00_Uint              ulMaxCPU);
 
 static void  eo70_WrnActNotEqMax   ( teo70k_VarTaskCntInfo  *pVarTasks );
 
@@ -159,7 +161,8 @@ static void  eo70_AddExpliciteVarTasksToUKT ( teo70k_ThrdInfo       *pThrdInfo,
 
 static void  eo70_AddImpliciteVarTaskToUKT  ( teo70k_ThrdInfo       *pThrdInfo,
                                               teo70k_VarTaskCntInfo *pVarTask,
-                                              tsp00_Longuint                  ulUKTBaseStack);
+                                              tsp00_Longuint         ulUKTBaseStack,
+                                              tsp00_Uint             ulMaxCPU );
 
 static tsp00_Int4  eo70_create_taskcluster        ( RTE_XPARAM_REC        *XParam,
                                               char                 **TaskCluster );
@@ -233,12 +236,12 @@ externC tsp00_Int4  eo70k_AnalyzeTaskcluster    ( RTE_XPARAM_REC      *XParam,
   ulTotTasks = eo70_GetNumberOfTasks (XParam ) ;
   DBG3 ((MF__, "Number of Tasks: %d", ulTotTasks ));
 
-  fConStratDefined = FALSE;
+  fConStratDefined = false;
   ulTasksFound     = 0;
   lThrdIdx         = 1;
   lCount           = 0;
-  fAllInOne        = FALSE;
-  fOneToOne        = FALSE;
+  fAllInOne        = false;
+  fOneToOne        = false;
   lThrdPrio        = DEFAULT_UKT_PRIO;
 
   ulMaxUKTs = ulTotTasks ;
@@ -437,14 +440,14 @@ externC tsp00_Int4  eo70k_AnalyzeTaskcluster    ( RTE_XPARAM_REC      *XParam,
             return ( -1 );
             }
           pCluInfo->pThrdInfo[lThrdIdx-1].lSpecialTasks ++;
-          pCluInfo->pThrdInfo[lThrdIdx-1].fUtilityTask  = TRUE;
+          pCluInfo->pThrdInfo[lThrdIdx-1].fUtilityTask  = true;
           ulTasksFound ++;
           break;
           }
 
         if ( ! strcmp ( szToken , TOKEN_COMPRESS_EO70K ) )
           {
-          if ( fConStratDefined == TRUE )
+          if ( fConStratDefined == true )
             {
             MSGALL(( ERR_TSK_CLST_CON_ALRDY_SPEC ));
             DBG1  (( MF__, "Connect strategy already specified" ));
@@ -454,14 +457,14 @@ externC tsp00_Int4  eo70k_AnalyzeTaskcluster    ( RTE_XPARAM_REC      *XParam,
           else
             {
             pCluInfo->lConnectStrategy = STRAT_COMPRESS;
-            fConStratDefined          = TRUE;
+            fConStratDefined          = true;
             }
           break;
           }
 
         if ( ! strcmp ( szToken , TOKEN_EQUALIZE_EO70K ) )
           {
-          if ( fConStratDefined == TRUE )
+          if ( fConStratDefined == true )
             {
             MSGALL(( ERR_TSK_CLST_CON_ALRDY_SPEC ));
             DBG1  (( MF__, "Connect strategy already specified" ));
@@ -471,7 +474,7 @@ externC tsp00_Int4  eo70k_AnalyzeTaskcluster    ( RTE_XPARAM_REC      *XParam,
           else
             {
             pCluInfo->lConnectStrategy = STRAT_EQUALIZE;
-            fConStratDefined          = TRUE;
+            fConStratDefined          = true;
             }
           break;
           }
@@ -486,7 +489,7 @@ externC tsp00_Int4  eo70k_AnalyzeTaskcluster    ( RTE_XPARAM_REC      *XParam,
             DBGOUT;
             return ( -1 );
             }
-            fAllInOne = TRUE;
+            fAllInOne = true;
             break;
           }
 
@@ -500,7 +503,7 @@ externC tsp00_Int4  eo70k_AnalyzeTaskcluster    ( RTE_XPARAM_REC      *XParam,
             DBGOUT;
             return ( -1 );
             }
-          fOneToOne = TRUE;
+          fOneToOne = true;
           break;
           }
 
@@ -538,25 +541,33 @@ externC tsp00_Int4  eo70k_AnalyzeTaskcluster    ( RTE_XPARAM_REC      *XParam,
    * ----------------------------------------------------------------------- */
   
   if ( fAllInOne )
-    eo70_PrepAllInOne ( pCluInfo, pVarTasks ) ; 
-  else
   {
-    if ( fOneToOne )
-    {
+      eo70_PrepAllInOne ( pCluInfo, pVarTasks ) ; 
+      pCluInfo->ulNumOfUserUKTs = 1;
+  }
+  else if ( fOneToOne )
+  {
       if ( eo70_PrepOneToOne ( pCluInfo, pVarTasks, ulMaxUKTs ) != NO_ERROR_EO001 )
       {
         DBGOUT;
         return ( -1 );
       }
-    }
-    else
-    {
+      pCluInfo->ulNumOfUserUKTs = XParam->ulMaxUser;
+  }
+  else
+  {
       if (eo70_CheckTaskTypes(pCluInfo, pVarTasks) != NO_ERROR_EO001)
       {
         DBGOUT;
         return ( -1 );
       }
-    }
+      pCluInfo->ulNumOfUserUKTs = XParam->ulMaxCPU;
+      if ( pVarTasks[VAR_TASK_US_EO70K].lLastFoundCnt == 1 )
+      {
+          pCluInfo->ulNumOfUserUKTs = ( 1 == pVarTasks[VAR_TASK_US_EO70K].lLastFoundCnt 
+                                            ? XParam->ulMaxUser 
+                                            : XParam->ulMaxCPU );
+      }
   }
 
   /* -----------------------------------------------------------------------
@@ -617,9 +628,6 @@ externC tsp00_Int4  eo70k_AnalyzeTaskcluster    ( RTE_XPARAM_REC      *XParam,
     /* --- Init the number of tasks */
     pPrevThrdInfo->ulNumOfTasks = pPrevThrdInfo->lSpecialTasks;
 
-    if ( pPrevThrdInfo->lUserTasks > 0 )
-      pCluInfo->ulNumOfUserUKTs++;
-
   /* -----------------------------------------------------------------------
    *     Compute this thread stack size.
    *     Round each task value up to a multiple of SystemPageSize. It is needed
@@ -658,7 +666,7 @@ externC tsp00_Int4  eo70k_AnalyzeTaskcluster    ( RTE_XPARAM_REC      *XParam,
 
   lThrdIdx-- ;
 
-  if ( eo70_CreateVarTaskUKTs ( pCluInfo, pVarTasks, &lThrdIdx, (tsp00_Int4 )ulMaxUKTs ) != NO_ERROR_EO001 )
+  if ( eo70_CreateVarTaskUKTs ( pCluInfo, pVarTasks, &lThrdIdx, (tsp00_Int4 )ulMaxUKTs, XParam->ulMaxCPU ) != NO_ERROR_EO001 )
   {
     DBGOUT;
     return ( -1 );
@@ -674,6 +682,20 @@ externC tsp00_Int4  eo70k_AnalyzeTaskcluster    ( RTE_XPARAM_REC      *XParam,
 
   RTEConf_FreeTaskClusterArrayInXParamStructure( XParam );
 
+  {
+    SAPDB_Char msgBuffer[256];
+    SAPDB_sprintf(msgBuffer, sizeof(msgBuffer), "Total Number of UKT %d", pCluInfo->ulNumOfUKTs);
+    MSGD (( INFO_TSK_CLST, msgBuffer ));
+    SAPDB_sprintf(msgBuffer, sizeof(msgBuffer), "Number of UKT with User Tasks %d", pCluInfo->ulNumOfUserUKTs);
+    MSGD (( INFO_TSK_CLST, msgBuffer ));
+  }
+
+  /* WARNING! This is correct but not 'clean'... */
+  /* Add regions for log history directory here, since dynamically dependend from task cluster */
+  /* This MUST be done here, since parameter analysis can only setup the 'constant' number of regions! */
+  /* A clean solution would mean to do task cluster analysis during configuration parameter reading... ffs... */
+  XParam->ulNoOfRegions += pCluInfo->ulNumOfUserUKTs;
+
   DBGOUT;
   return ( NO_ERROR_EO001 );
   }
@@ -942,6 +964,7 @@ static void  eo70_InitVarTaskInfo ( teo70k_VarTaskCntInfo  *pVarTasks,
   for ( iVarTask = 0; iVarTask < MX_VAR_TASK_TYPES_EO70K ; iVarTask ++, pVarTasks++  ) 
   { 
     pVarTasks->lVarTaskType = iVarTask ;
+    pVarTasks->lUKTsFilled  = 0;
     switch ( iVarTask )
     {
       case VAR_TASK_DW_EO70K :
@@ -1049,7 +1072,7 @@ static tsp00_Int4  eo70_CheckVarTaskCnt (char                   *pszToken,
   #define MF__ MOD__"eo70k_CheckVarTaskCnt"
 
   int     iVarTask ;
-  bool    fFound = FALSE ;
+  bool    fFound = false ;
   tsp00_Int4    rc     = NO_ERROR_EO001;
 
   DBGPAS;
@@ -1060,7 +1083,7 @@ static tsp00_Int4  eo70_CheckVarTaskCnt (char                   *pszToken,
   {
     if ( ! strcmp ( pszToken , pVarTasks[iVarTask].TaskTypeToken ) )
     { 
-      fFound = TRUE ;
+      fFound = true ;
       if ( lCount == 0 )
         lCount = pVarTasks[iVarTask].lLastFoundCnt;
       
@@ -1101,7 +1124,7 @@ static void  eo70_PrepAllInOne ( teo70k_ClusterInfo    *pCluInfo,
 
   DBG3 ((MF__, "All tasks go into one UKT" ));
 
-  pCluInfo->pThrdInfo[0].fUtilityTask   = TRUE;
+  pCluInfo->pThrdInfo[0].fUtilityTask   = true;
 
   pCluInfo->TI_Pos                      = 1;
   pCluInfo->AL_Pos                      = 1;
@@ -1140,8 +1163,8 @@ static tsp00_Int4  eo70_PrepOneToOne ( teo70k_ClusterInfo    *pCluInfo,
   pCluInfo->TW_Pos     = 3;
   pCluInfo->UT_Pos     = 4;
   lTotTask             = pCluInfo->UT_Pos ;
-
-  pCluInfo->pThrdInfo[pCluInfo->UT_Pos - 1].fUtilityTask   = TRUE;
+  pCluInfo->pThrdInfo[pCluInfo->UT_Pos - 1].fUtilityTask   = true;
+  pCluInfo->fOneToOne  = true;
 
   for ( lThrdIdx = 1; lThrdIdx <= pCluInfo->UT_Pos; lThrdIdx ++ )
      pCluInfo->pThrdInfo[lThrdIdx-1].lSpecialTasks = 1;
@@ -1187,6 +1210,7 @@ static void  eo70_AddExpliciteVarTasksToUKT ( teo70k_ThrdInfo       *pThrdInfo,
       pThrdInfo->ulNumOfTasks  += lTasksInUkt ;
       pVarTask->lActTaskCnt    += lTasksInUkt ;
       pVarTask->pLastFoundUKT   = pThrdInfo;
+      pVarTask->lUKTsFilled     = 1;
     }
   }
 
@@ -1197,18 +1221,34 @@ static void  eo70_AddExpliciteVarTasksToUKT ( teo70k_ThrdInfo       *pThrdInfo,
 
 static void eo70_AddImpliciteVarTaskToUKT ( teo70k_ThrdInfo       *pThrdInfo,
                                             teo70k_VarTaskCntInfo *pVarTask,
-                                            tsp00_Longuint                  ulUKTBaseStackSize ) 
+                                            tsp00_Longuint         ulUKTBaseStackSize,
+                                            tsp00_Uint             ulMaxCPU )
 {
   #undef  MF__
   #define MF__ MOD__"eo70k_AddImpliciteVarTaskToUKT"
 
-  tsp00_Int4 lTasksInUkt = min (pVarTask->lLastFoundCnt, 
-                          pVarTask->lMaxTaskCnt - pVarTask->lActTaskCnt );
-  
+  tsp00_Int4 lTasksInUkt = 0;
+
+  if ( pVarTask->lVarTaskType == VAR_TASK_US_EO70K
+    || pVarTask->lVarTaskType == VAR_TASK_SV_EO70K )
+  {
+      if ( 1 < pVarTask->lLastFoundCnt )
+      {
+          lTasksInUkt = ( pVarTask->lMaxTaskCnt + ulMaxCPU - 1 - pVarTask->lUKTsFilled ) / ulMaxCPU;
+      }
+  }
+
+  if ( 0 == lTasksInUkt )
+  {
+      lTasksInUkt = min (pVarTask->lLastFoundCnt, pVarTask->lMaxTaskCnt - pVarTask->lActTaskCnt );
+  }
+
   if ( lTasksInUkt > 0 )
   {
     DBG3 (( MF__, "adding task '%d*%s' to UKT",lTasksInUkt, pVarTask->TaskTypeToken ));
     
+    ++(pVarTask->lUKTsFilled);
+
     eo70_SetUKTVarTaskCnt( pThrdInfo, pVarTask, lTasksInUkt ) ;
  
     if ( pThrdInfo->ulStackSize == 0 ) /* initialize Thread once */
@@ -1229,10 +1269,11 @@ static void eo70_AddImpliciteVarTaskToUKT ( teo70k_ThrdInfo       *pThrdInfo,
 
 /*------------------------------*/
 
-static tsp00_Int4  eo70_CreateVarTaskUKTs ( teo70k_ClusterInfo    *pCluInfo,
-                                       teo70k_VarTaskCntInfo *pVarTasks,
-                                       tsp00_Int4                *plThrdIdx,
-                                       tsp00_Int4                 lMaxUKTs) 
+static tsp00_Int4  eo70_CreateVarTaskUKTs ( teo70k_ClusterInfo     *pCluInfo,
+                                            teo70k_VarTaskCntInfo  *pVarTasks,
+                                            tsp00_Int4             *plThrdIdx,
+                                            tsp00_Int4              lMaxUKTs,
+                                            tsp00_Uint              ulMaxCPU )
   {
   #undef  MF__
   #define MF__ MOD__"eo70k_CreateVarTaskUKTs"
@@ -1245,25 +1286,26 @@ static tsp00_Int4  eo70_CreateVarTaskUKTs ( teo70k_ClusterInfo    *pCluInfo,
 
   DBGIN;
 
-  for ( iActVarTask = 0; 
+  for ( iActVarTask = 0;
         iActVarTask < MX_VAR_TASK_TYPES_EO70K && rc == NO_ERROR_EO001 ; 
         iActVarTask++ ) 
   {
     pActVarTask = pVarTasks + iActVarTask ;
+
     if (pActVarTask->lActTaskCnt < pActVarTask->lMaxTaskCnt )
     { 
       do
       {
         pThrdInfo = &pCluInfo->pThrdInfo[lThrdIdx] ;
         DBG3 (( MF__, "Filling UKT %d with additional (var) tasks", lThrdIdx));
-        eo70_AddImpliciteVarTaskToUKT ( pThrdInfo, pActVarTask, pCluInfo->ulUKTBaseStackSize ) ; 
+        eo70_AddImpliciteVarTaskToUKT ( pThrdInfo, pActVarTask, pCluInfo->ulUKTBaseStackSize, ulMaxCPU ) ; 
         for ( iNxtVarTask = iActVarTask+1; iNxtVarTask < MX_VAR_TASK_TYPES_EO70K; iNxtVarTask++ )
         {
           pNxtVarTask = pVarTasks + iNxtVarTask ;
           if ( pNxtVarTask->pLastFoundUKT == pActVarTask->pLastFoundUKT && /* add other types ?? */
                pNxtVarTask->lActTaskCnt    < pNxtVarTask->lMaxTaskCnt )
           {
-            eo70_AddImpliciteVarTaskToUKT ( pThrdInfo, pNxtVarTask, pCluInfo->ulUKTBaseStackSize ) ; 
+            eo70_AddImpliciteVarTaskToUKT ( pThrdInfo, pNxtVarTask, pCluInfo->ulUKTBaseStackSize, ulMaxCPU ) ; 
           }
         }
         lThrdIdx++ ;
