@@ -215,6 +215,8 @@ char *eo92GetTaskStateName ( tsp00_Int lState,
     case TSK_VVECTORIO_EO00: return XCONS_TSK_STATE_VVECTORIO;
     case TSK_VDUALVECTORIO_EO00: return XCONS_TSK_STATE_VDUALVECTORIO;
     case TSK_VCOPYVOLUME_EO00: return XCONS_TSK_STATE_VCOPYVOLUME;
+    case TSK_ENTERRWREGION_EO00:return ( XCONS_TSK_STATE_VENTERRWREGION );
+    case TSK_LEAVERWREGION_EO00:return ( XCONS_TSK_STATE_VLEAVERWREGION );
 
     default: break;
   }
@@ -410,7 +412,7 @@ void eo92PrintTaskCounts( tsp00_Int4                   TimeMeasurementActive,
         printf ( XCONS_SHOW_TASK_COUNT_ASYN_CNT_PG ,
                        XCONS_SHOW_TASK_COUNT_KT_ASYN_W_CNT ,
                        pTaskStateStat->AsynIOWriteStat.IOStat.ulCount   ,
-                       XCONS_SHOW_TASK_COUNT_KT_ASYN_W_CNT  ,
+                       XCONS_SHOW_TASK_COUNT_KT_ASYN_W_PG  ,
                        pTaskStateStat->AsynIOWriteStat.ulPageCount);
 
       if ( pTaskStateStat->SelfIOReadStat.IOStat.ulCount ||
