@@ -293,7 +293,7 @@ static void eo922GetRteOsSpecificInfoRegion ( tsp92_RteRegionInfo *RteRegionInfo
        for ( pSemQueElem = pSemCtrl->semaliste.last, RteRegionInfo->NumWaiter = 0 ;
              pSemQueElem && (pWaitTaskCtrl = pSemQueElem->taskid) != NULL &&
              RteRegionInfo->NumWaiter < (tsp00_Int4 )KGS->ulNumOfTasks ;
-             pSemQueElem = pSemQueElem->next )
+             pSemQueElem = pSemQueElem->next, (RteRegionInfo->NumWaiter)++ )
       {
         pWaitTasks[RteRegionInfo->NumWaiter] = pWaitTaskCtrl->index;
       }
@@ -888,6 +888,7 @@ externC tsp00_Bool vGetRteInfoUKTNext ( tsp92_RteUKTInfo *RteUKTInfo  )
 //=======================================================================
 #endif
 //=======================================================================
+    RteUKTInfo->UKTTaskSwitchCnt = (tsp00_8ByteCounter) (SAPDB_Int8)pActUKTCtrl->taskSwitches;
   }
   return ( Found ) ;
 }
@@ -1090,6 +1091,7 @@ externC tsp00_Bool vGetRteInfoTaskNext ( tsp92_RteTaskInfo *RteTaskInfo )
     {
       Found = IS_TASK_TO_SHOW_EO92(TASK_TYPE(pTaskCtrl),
                                    TASK_STATE(pTaskCtrl),
+                                   TASK_STATE_REASON(pTaskCtrl),
                                    pTaskCtrl->TaskOMSStat.OMSCallActive,    /* PTS 1116933 */
                                    TASK_QUQUE_LOCATION(pTaskCtrl),
                                    RteTaskInfo->TaskTypeToShow,
