@@ -46,6 +46,7 @@
 #include "RunTime/RTE_GetDBRootOfDB.h"
 
 #include "SAPDBCommon/SAPDB_Names.h"
+#include "SAPDBCommon/SAPDB_string.h"
 
 /*===========================================================================*
  *  DEFINES                                                                  *
@@ -93,7 +94,8 @@ static int ClearCalledFromStop = FALSE ;
 #define START_GATEWAY      "-g"
 #define START_OPT_LOW_PRIO "-l"
 #define START_OPT_LOWER_PRIORITY "-lowprio"
-#define STOP_OPT_IGN_SHUT  "-i"
+#define STOP_OPT_IGN_SHUT  "-force"
+#define STOP_OPT_DUMP      "-dump"
 #define END_OF_PARSABLE_OPTIONS "--"
 
 #define TYPE_FAST_KERNEL   0
@@ -101,6 +103,7 @@ static int ClearCalledFromStop = FALSE ;
 #define TYPE_SLOW_KERNEL   2
 #define TYPE_TEST_KERNEL   3
 
+#define OPT_VERBOSE   "-v"
 #define OPT_NODE     "-n"
 #define OPT_CHECK_INTERVAL "-c"
 #define OPT_DEBUG_LEVEL "-D"
@@ -125,9 +128,9 @@ static int ClearCalledFromStop = FALSE ;
  *===========================================================================*/
 typedef struct          xstop_args
                         { char *dbname ;
-                          char *dbnode ;
                           char *ign_shut ;
                           char *dumpflag;
+                          char *verbose;
                         } XSTOP_ARGS ;
 
 /*===========================================================================*
@@ -151,18 +154,6 @@ static int check_db_to_start         ( char      *dbname,
 							           char      *full_knl_path, 
  		   					           char      *knl_prog, 
                                        int        type_flag );
-
-static int e952_analyze_start_params ( int        num_args, 
-									   char     **args,
-                                       int       *num_params, 
-									   char     **params,
-                                       int       *kernel_type, 
-									   char     **dbname ) ;
-
-static int remove_knl_link           ( PROG_ID    *stop, 
-							           char       *dbname, 
-							           int        *ret ) ;
-
 #endif
 
 static int fill_stop_rec             ( XSTOP_ARGS *stop_rec, 
@@ -214,10 +205,6 @@ int e952_call_stop ( PROG_ID *stop,
           loc_params[num_params] = "-d"            ; ++num_params ; 
 #endif
           loc_params[num_params] = args_rec.dbname ; ++num_params ;
-          if ( args_rec.dbnode ) 
-            { loc_params[num_params] = OPT_NODE        ; ++num_params ; 
-              loc_params[num_params] = args_rec.dbnode ; ++num_params ; 
-            }
 
           if ( args_rec.ign_shut ) 
             { loc_params[num_params] = args_rec.ign_shut; ++num_params ; }
@@ -225,11 +212,11 @@ int e952_call_stop ( PROG_ID *stop,
           if ( args_rec.dumpflag ) 
             { loc_params[num_params] = args_rec.dumpflag ; ++num_params ; }
 
+          if ( args_rec.verbose ) 
+            { loc_params[num_params] = args_rec.verbose ; ++num_params ; }
+
           call_ret = e950_call_component (stop, num_params, loc_params);
           free ( loc_params ) ;
-#ifndef WIN32
-          remove_knl_link ( stop, args_rec.dbname, &call_ret ) ;
-#endif
         }
     }
   return ( call_ret ) ;
@@ -495,6 +482,32 @@ int e952_call_backup( PROG_ID *backup,
   return ( call_ret ) ;
 }
 
+/*-----------------------------------------------------------*/
+int eo952CallPing     ( PROG_ID *Ping,
+  					    int      num_args, 
+					    char   **args )
+/*-----------------------------------------------------------*/
+{
+    int call_ret = TRUE ;
+	tsp01_RteError RteError;
+
+
+    DBG0 (( "\ncall_ping called\n" ));
+
+
+	if (sqlUpdateNiLibPathEnvironment(&RteError))
+	{
+		call_ret = e950_call_component ( Ping, num_args, args ) ;
+	}
+	else
+	{
+		printf("\nUpdate environment failed:%s\nPing not executed!\n", RteError.RteErrText ) ;
+		call_ret = NOTOK;
+	}
+    
+    return ( call_ret ) ;
+}
+
 /* ###################################################### */
 #ifndef WIN32
 /* ###################################################### */
@@ -514,129 +527,6 @@ int en952_GetDbrootPgmPath ( char *DbrootPgmPath )
   return (rc) ;
 }
 
-#define MSG_O(x)   { sqloutwrite x ; }
-/*-----------------------------------------------------------*/
-int e952_call_start ( PROG_ID *start, 
-					  int      num_args, 
-					  char   **args )
-/*-----------------------------------------------------------*/
-{
-  int call_ret=TRUE, num_params ;
-  int kernel_type ;
-  tsp00_Pathc knl_prog_path ;
-  char *params[MAX_START_PARAMS+1],
-       *dbname ;
-
-  DBG0 (( "\ncall_start called with %d parameters\n", num_args ));
-
-  num_params = 2;
-  call_ret = e952_analyze_start_params ( num_args, args,
-                                         &num_params, params,
-                                         &kernel_type, &dbname ) ;
-  if ( call_ret == NOTOK )
-    e950_call_help (start->progid, start->prog_helpfiles ); 
-  else
-    { int check_db_to_start ();
-      char *adabas_kernels[] = ADABAS_KERNELS ;
-      char *knl_prog = adabas_kernels [kernel_type] ;  
-    {
-        tsp00_Pathc     DBRootOfDB;
-
-        if ( RTE_GetDBRootOfDB(dbname, DBRootOfDB, sizeof(DBRootOfDB)) )
-        {
-            e950_PutEnvDBRoot(DBRootOfDB) ;
-        }
-        else
-        {
-            printf("Installationpath of serverdb <%s> not found!", dbname);
-            return NOTOK;
-        }
-    }
-      params[1] = knl_prog ;
-      if ( check_db_to_start ( dbname, knl_prog_path, knl_prog, 
-      			       TFLG_KERNEL ) )
-        { params[0] = knl_prog_path ;
-	      MSG_O (( INFO_STARTING , dbname ));
-          if ( ! sqlPutDbNameToEnv ( dbname ) )
-          {
-            printf("\ncan't expand environment with <%s>\n", dbname ) ;
-            call_ret = NOTOK ;
-          }
-          else
-            if ( (call_ret = e950_call_component (start, num_params, params) ) )
-              { char *rundir = CNULL ;
-                e951_getparam ( dbname, PAN_RUNDIR, &rundir) ;
-
-                if ( rundir != CNULL )
-                  printf("\n%s: Unsuccessful startup, see\n%s%c%s\nfor errors",
-                                start->progid, rundir, PATH_DELIMITER_EO01,SAPDB_DIAG_FILE_NAME);
-                else
-                  printf("\n%s: Unsuccessful startup, see %s for errors",
-                                start->progid, SAPDB_DIAG_FILE_NAME );
-              }
-        }
-      else
-        call_ret = NOTOK ;
-    }
-  return ( call_ret ) ;
-}
-
-/*-----------------------------------------------------------*/
-int e952_call_gwstart ( PROG_ID *start, 
-					    int      num_args,
-						char   **args )
-/*-----------------------------------------------------------*/
-{
-  int  call_ret=TRUE, num_params ;
-  int kernel_type ;
-  char *dbname ;
-  tsp00_Pathc knl_prog_path;
-  char *params[MAX_START_PARAMS+1];
-  DBG0 (( "\ncall_gwstart called with %d parameters\n", num_args ));
-  params[1]="gateway";
-  num_params = 2;
-  call_ret = e952_analyze_start_params ( num_args, args,
-                                         &num_params, params,
-                                         &kernel_type, &dbname ) ;
-  if ( call_ret == NOTOK )
-    e950_call_help (start->progid, start->prog_helpfiles ); 
-  else
-    { int check_db_to_start ();
-      char *adabas_gatways[] = ADABAS_GATEWAYS ;
-      char *oracle_gatways[] = ORACLE_GATEWAYS ;
-      char *gwtype = CNULL;
-      char *knl_prog = "unknown" ;
-      e951_getparam ( dbname, "GATEWAYTYPE", &gwtype) ;
-      if (strcmp(gwtype,"ORACLE") == 0)
-         knl_prog = oracle_gatways [kernel_type] ;  
-      else
-        if (strcmp(gwtype,"ADABAS_C") == 0)
-           knl_prog = adabas_gatways [kernel_type] ;  
-
-      if ( check_db_to_start ( dbname , knl_prog_path, knl_prog, 
-                               TFLG_GATEWAY ) )
-        { params[0] = knl_prog_path ;
-	      MSG_O (( INFO_GWSTARTING , dbname  ));
-          if ( ! sqlPutDbNameToEnv ( dbname  ) )
-          {
-            printf("\ncan't expand environment with <%s>\n", dbname ) ;
-            call_ret = NOTOK ;
-          }
-          else
-            if ( (call_ret = e950_call_component (start, num_params, params) ) )
-              { printf("\n%s: Unsuccessful startup, see diag file for errors",
-                              start->progid );
-                printf("\nClearing %s\n", dbname  ) ;
-                e950_search_and_call_sqlprog ( "x_clear",  1, &dbname ) ;
-              }
-        }
-      else
-        call_ret = NOTOK ;
-    }
-  return ( call_ret ) ;
-}
-
-
 /*-----------------------------------------------------------*/
 int e952_call_clear ( PROG_ID *clear, 
 					  int      num_args, 
@@ -698,6 +588,7 @@ int eo952CallNiServer ( PROG_ID *NiServer,
 {
     char **params;
     int i, num_params=0, call_ret = TRUE ;
+	tsp01_RteError RteError;
 
     DBG0 (( "\ncall_niserver called\n" ));
 
@@ -780,12 +671,21 @@ int eo952CallNiServer ( PROG_ID *NiServer,
 
     if ( call_ret != NOTOK )
     {
-        call_ret = e950_call_component ( NiServer, num_params, params ) ;
+		if (sqlUpdateNiLibPathEnvironment(&RteError))
+		{
+			call_ret = e950_call_component ( NiServer, num_params, params ) ;
+		}
+		else
+		{
+	        printf("\nUpdate environment failed:%s\nNiServer not executed!\n", RteError.RteErrText ) ;
+			call_ret = NOTOK;
+		}
     }
 
     return ( call_ret ) ;
 }
 
+
 /*-----------------------------------------------------------*/
 /* PTS 1108781 / 1108471 / 1108798 */
 int e952_call_regcomp ( PROG_ID *regcomp,
@@ -809,7 +709,7 @@ int e952_call_regcomp ( PROG_ID *regcomp,
     return ( call_ret ) ;
 }
 /* ###################################################### */
-#else
+#endif
 /* ###################################################### */
 
 /*-----------------------------------------------------------*/
@@ -819,7 +719,7 @@ int e952_call_start ( PROG_ID *start,
 /*-----------------------------------------------------------*/
 {
   char *params[MAX_START_PARAMS+3], *serverdb = NULL; 
-  int i, num_params=0, call_ret = TRUE, rem_start = FALSE ;
+  int i, num_params=0, call_ret = TRUE;
   DBG0 (( "\ncall_start called with %d parameters\n", num_args ));
 
   for ( i=0 ; i<num_args && 
@@ -835,11 +735,14 @@ int e952_call_start ( PROG_ID *start,
           { params[num_params] = args[i] ; num_params++ ; }
         else
         if ( !strcmp ( &args[i][1], &START_OPT_LOWER_PRIORITY[1]) )
-          { params[num_params] = args[i] ; num_params++ ; }
+          { params[num_params] = START_OPT_LOW_PRIO ; num_params++ ; }
         else
         if ( !strcmp ( &args[i][1], &START_OPT_ADMIN_MODE[1]) )
           { params[num_params] = args[i] ; num_params++ ; }
         else
+        if ( !strcmp ( &args[i][1], &OPT_VERBOSE[1]) )
+        { params[num_params] = args[i] ; num_params++ ; }
+        else
         if ( !strcmp ( &args[i][1], &START_OPT_IGN_REST[1]) )
           { params[num_params] = args[i] ; num_params++ ; }
         else
@@ -869,7 +772,7 @@ int e952_call_start ( PROG_ID *start,
       e950_call_help (start->progid, start->prog_helpfiles ); 
     }
   else
-    { char *rundir ;
+  { char *rundir ;
     {
         tsp00_Pathc     DBRootOfDB;
 
@@ -883,12 +786,9 @@ int e952_call_start ( PROG_ID *start,
             return NOTOK;
         }
     }
-      if ( !rem_start )
-        call_ret = !rundir_exists ( serverdb , &rundir ) ? NOTOK :
-                  e950_call_component ( start, num_params, params ) ; 
-      else
-        call_ret = e950_call_component ( start, num_params, params ) ;
-    }
+    call_ret = !rundir_exists ( serverdb , &rundir ) ? NOTOK :
+              e950_call_component ( start, num_params, params ) ; 
+  }
   return ( call_ret ) ;
 }
 
@@ -910,11 +810,6 @@ int e952_call_gwstart ( PROG_ID *start,
   return(TRUE);
 }
 
-
-/* ###################################################### */
-#endif
-/* ###################################################### */
-
 /*===========================================================================*
  *  LOCAL/PRIVATE/PROTECTED FUNCTIONS (CODE)                                 *
  *===========================================================================*/
@@ -927,44 +822,46 @@ static int fill_stop_rec ( XSTOP_ARGS *stop_rec,
 /*-----------------------------------------------------------*/
 {
   int fill_ok = num_args > 0 , act_arg = 0 ;
-  memset ( stop_rec, '\0', sizeof ( XSTOP_ARGS ) ) ;
+  SAPDB_memset ( stop_rec, '\0', sizeof ( XSTOP_ARGS ) ) ;
   while ( num_args && fill_ok )
     if ( IS_OPT_CHR ( *(args[act_arg]) ) )
-      {
-#if WIN32
-        if ( (fill_ok = !strcmp ( &args[act_arg][1], &OPT_NODE[1] ) ) )
-          { if ( (fill_ok = num_args > 1) )
-              { stop_rec->dbnode = args[act_arg+1] ;
-                num_args -= 2, act_arg+=2 ;
-                DBG0(("dbnode: %s\n", stop_rec->dbnode ));
-              }
+    {
+          if ( !SAPDB_strcasecmp(args[act_arg], STOP_OPT_IGN_SHUT) )
+          {
+              stop_rec->ign_shut = STOP_OPT_IGN_SHUT; act_arg++; --num_args ;
           }
-        else
-#endif
-          if ( (fill_ok = !strcmp ( &args[act_arg][1], &STOP_OPT_IGN_SHUT[1])))
-            { 
-#if WIN32
-              stop_rec->ign_shut = args[act_arg] ;
-#endif
-              num_args --, act_arg++ ;
-              DBG0(("ignore: %s\n", stop_rec->ign_shut ));
-            }
-      }
+          else if ( !SAPDB_strcasecmp(args[act_arg], STOP_OPT_DUMP) )
+          {
+              stop_rec->dumpflag = STOP_OPT_DUMP; act_arg++; --num_args ;
+          }
+          else if ( !SAPDB_strcasecmp(args[act_arg], OPT_VERBOSE) )
+          {
+              stop_rec->verbose = OPT_VERBOSE; act_arg++; --num_args ;
+          }
+          else
+          {
+              fill_ok = false;
+          }
+    }
     else
+    {
       if ( !stop_rec->dbname ) 
-        { stop_rec->dbname = args[act_arg] ; act_arg++; --num_args ;}
+      {
+          stop_rec->dbname = args[act_arg] ; act_arg++; --num_args ;
+      }
+      else if ( !SAPDB_strcasecmp( args[act_arg], &STOP_OPT_IGN_SHUT[1] ) )
+      {
+          stop_rec->ign_shut = STOP_OPT_IGN_SHUT; act_arg++; --num_args ;
+      }
+      else if ( !SAPDB_strcasecmp( args[act_arg], &STOP_OPT_DUMP[1] ) )
+      {
+          stop_rec->dumpflag = STOP_OPT_DUMP; act_arg++; --num_args ;
+      }
       else
-        if ( !strcmp ( e951_toupper_str (args[act_arg]) , "DUMP" ) )
-          { act_arg++; --num_args; 
-#ifdef WIN32
-            stop_rec->dumpflag = "-p" ;
-#else
-            stop_rec->dumpflag = "dump" ;
-#endif
-          }
-        else
-            fill_ok = FALSE ;
-
+      {
+          fill_ok = false ;
+      }
+    }
   return ( fill_ok ) ;
 }
 
@@ -1005,7 +902,6 @@ static int rundir_exists ( char  *dbname,
 /* ###################################################### */
 #ifndef WIN32
 /* ###################################################### */
-
 /*-----------------------------------------------------------*/
 static int check_db_to_start ( char *dbname, 
 							   char *full_knl_path, 
@@ -1019,6 +915,10 @@ static int check_db_to_start ( char *dbname,
 
   if ( ( check_ok = en952_GetDbrootPgmPath ( dbrootPgm ) ) )
   {
+      char *rundir ;
+      /* ignore current umask setting for directory creation */
+      unsigned short saved_umask = umask(0) ;
+
       strcpy ( full_knl_path, dbrootPgm ) ;
       strcat ( full_knl_path, knl_prog ) ;
       {
@@ -1029,41 +929,18 @@ static int check_db_to_start ( char *dbname,
           sprintf ( ipcus, "%s%s/%s%s", &spoolPath[0], SAPDB_SPOOL_IPC_DIRECTORY, SAPDB_USER_DIRECTORY_PREFIX, dbname ) ;
       }
 
-      check_ok = !e951_f_access ( full_knl_path, F_EX, FALSE )    &&
-                 !e951_f_access ( ipcdb, S_IFDIR, FALSE ) &&
-                 !e951_f_access ( ipcus, S_IFDIR, FALSE ) ;
-      if ( !check_ok )
-      { 
-          if (type_flag == TFLG_KERNEL) 
+      if ( ( check_ok = rundir_exists ( dbname, &rundir ) ) )
+      {
+          if ( ( check_ok = e951_f_access ( rundir, F_RWX, FALSE ) ) )
           {
-              printf("\nDatabase %s already started or not stopped correctly",
-                    dbname ) ;
+              check_ok = e951_create_path ( ipcdb, 0770 ) && /* database tag files only for owner and group */
+                     e951_create_path ( ipcus, 0777 ) && /* user tag files for everybody... */
+                     umask ( saved_umask ) == 0 ;
           }
           else
           {
-              printf("\nGateway %s already started or not stopped correctly",
-                    dbname ) ;
-              printf("\nTo clean up after a crash call 'x_clear %s'\n", dbname);
-          }
-      }
-      else
-      {
-          char *rundir ;
-          /* ignore current umask setting for directory creation */
-          unsigned short saved_umask = umask(0) ;
-          if ( ( check_ok = rundir_exists ( dbname, &rundir ) ) )
-          {
-              if ( ( check_ok = e951_f_access ( rundir, F_RWX, FALSE ) ) )
-              {
-                  check_ok = e951_create_path ( ipcdb, 0770 ) && /* database tag files only for owner and group */
-                         e951_create_path ( ipcus, 0777 ) && /* user tag files for everybody... */
-                         umask ( saved_umask ) == 0 ;
-              }
-              else
-              {
-                  printf("\nYou don't have read+write+execute permission on the run-directory\n%s", rundir) ;
-                  printf("\nPlease check the mode and start again\n");
-              }
+              printf("\nYou don't have read+write+execute permission on the run-directory\n%s", rundir) ;
+              printf("\nPlease check the mode and start again\n");
           }
       }
   }
@@ -1073,114 +950,6 @@ static int check_db_to_start ( char *dbname,
   }
   return ( check_ok ) ;
 }
-
-
-/*-----------------------------------------------------------*/
-static int e952_analyze_start_params ( int  num_args, 
-									   char **args,
-                                       int  *num_params, 
-									   char **params,
-                                       int  *kernel_type, 
-									   char **dbname ) 
-/*-----------------------------------------------------------*/
-{
-  int i , call_ret = TRUE;
-
-  *kernel_type = TYPE_FAST_KERNEL ;
-  for ( i=0 ; i<num_args && IS_OPT_CHR(*args[i]) &&
-              call_ret != NOTOK && *num_params < MAX_START_PARAMS ; i++ )
-  { 
-    DBG0 (( "start: analyze parameters %s\n", args[i] ));
-    if ( !strcmp ( &args[i][1], START_OPT_SLOW ) )
-      *kernel_type = TYPE_SLOW_KERNEL ;
-    else
-      if ( !strcmp ( &args[i][1], START_OPT_TEST ) )
-        *kernel_type = TYPE_TEST_KERNEL;
-      else if ( !strcmp ( &args[i][1], START_OPT_QUICK ) )
-        *kernel_type = TYPE_QUICK_KERNEL ;
-      else if ( !strcmp ( &args[i][1], START_OPT_FAST ) )
-        *kernel_type = TYPE_FAST_KERNEL ;
-      else if ( !strcmp ( &args[i][1], &START_OPT_IGN_REST[1]) )
-          ;
-      else if ( !strcmp ( &args[i][1], &END_OF_PARSABLE_OPTIONS[1]) )
-      {
-          while ( i < num_args && *num_params < MAX_START_PARAMS )
-          {
-              params[*num_params] = args[i] ; *num_params++ ; 
-              ++i;
-          }
-          i = num_args - 1;
-          break;
-      }
-      else
-          call_ret = NOTOK ;
-    }
-
-  if ( call_ret == NOTOK || ( num_args - i ) != 1 )
-    call_ret = NOTOK;
-  else
-    *dbname = args[i];
-
-  return ( call_ret ) ;
-}
-
-
-/*-----------------------------------------------------------*/
-static int remove_knl_link ( PROG_ID *stop, 
-							 char    *dbname, 
-							 int     *ret )
-/*-----------------------------------------------------------*/
-{
-  struct stat	statbuf;
-  if ( *ret == 0 )
-    {   
-      tsp00_Pathc      pgm_link  ;
-
-      if ( en952_GetDbrootPgmPath(pgm_link) )
-        { 
-          strcat (pgm_link, SAPDB_DATABASE_DIRECTORY_PREFIX ) ; 
-          strcat (pgm_link, dbname ) ; 
-
-          if ( stat ( pgm_link, &statbuf ) == -1 )
-            {
-              en950_GetProgramExecPath( stop->ProgramLocation, pgm_link ); 
-              sprintf ( pgm_link, "%s%c%s%s", pgm_link, PATH_DELIMITER_EO01,
-                                          "gw:", dbname ) ;
-#ifdef HPUX
-              if ( stat ( pgm_link, &statbuf ) != -1 )
-                {
-                  printf ( "\ngateway %s stopped\n", dbname ) ;
-                  sleep (3) ;
-                }  
-#endif
-            } 
-          else
-            {
-#ifdef HPUX
-              printf ( "\ndatabase %s stopped\n", dbname ) ;
-              sleep (3) ;
-#endif
-            }
-          DBG0 (( "\ntry to remove link <%s>\n", pgm_link )) ;
-          if ( (*ret = unlink ( pgm_link )) != 0  )
-            perror ( "can not remove link" ) ;
-        }
-      else
-          *ret = NOTOK ;
-     }
-
-  if ( *ret )
-    { MSG_O (( INFO_STOP_CLEARING, dbname ));
-      ClearCalledFromStop = TRUE ;
-      e950_search_and_call_sqlprog ( "x_clear", 1, &dbname ) ;
-    }
-  else
-    { MSG_O (( INFO_STOP_STOPPED, dbname ));
-    }
-
-  return ( *ret ) ;
-}
-
 /* ###################################################### */
 #endif
 /* ###################################################### */
