@@ -765,6 +765,7 @@ externPascal pasbool gg06buildCompressedKey (
     // move parts of original key to compressed key
 
     const tsp00_Int4 moveKeyLengthFirstPart  = moveKeyLength/2;
+    const tsp00_Int4 moveKeyLengthSecondPart = moveKeyLength-moveKeyLengthFirstPart; // PTS 1129353 UH 2004-04-27
 
     SAPDB_RangeMove ( __FILE__, 2,
              keySize,       compressedKeySize,
@@ -777,9 +778,9 @@ externPascal pasbool gg06buildCompressedKey (
 
     SAPDB_RangeMove ( __FILE__, 3,
              keySize,       compressedKeySize,
-             key,           keyStartPosition + moveKeyLengthFirstPart,
+             key,           keyLength - moveKeyLengthSecondPart + 1, // PTS 1129353 UH 2004-04-27 was not set to end
              compressedKey, 1 + moveKeyLengthFirstPart,
-             moveKeyLength-moveKeyLengthFirstPart, moveError);
+             moveKeyLengthSecondPart, moveError); // PTS 1129353 UH 2004-04-27
 
     if ( moveError != e_ok )
         return false;
