@@ -412,57 +412,105 @@ externPascal pasbool gg999KernelStateIsRestart ()
 }
 /* --------------------------------------------------------------------------- */
 externPascal void gg999GetBlockAddress (
-    tsp00_TaskId                         taskid,
-    tsp00_PageNo                         pageno,
-    pasbool                              isDynamic,
-    tsp00_Int4            VAR_VALUE_REF  deviceno,
-    tsp00_Int4            VAR_VALUE_REF  blockno)
+    tsp00_TaskId                         taskId,
+    tsp00_PageNo                         pageNo,
+    tgg00_RecoveryMode_Param             recoveryMode,
+    tsp00_Int4            VAR_VALUE_REF  deviceNo,
+    tsp00_Int4            VAR_VALUE_REF  blockNo)
 {
-    Data_PageNo         auxpageno   = pageno;
-    Data_AddressingMode addressmode = isDynamic ? Data_Dynamic : Data_Static;
-    IOMan_BlockAddress  blockaddress;
-    
-    Kernel_IAdminDiagnose::Instance().GetBlockAddress (taskid, auxpageno, addressmode, blockaddress);
-    
-    if (blockaddress.IsValid())
+     
+    deviceNo = 0;
+    blockNo  = 0;
+
+    Data_PageRecoveryMode   recMode;
+
+    switch( recoveryMode )
     {
-        deviceno = blockaddress.GetDeviceNo();
-        blockno  = blockaddress.GetBlockNo();
+        case rmPerm_egg00:
+                recMode.SetMode( Data_Dynamic, Data_Recoverable );
+            break;
+        case rmStatic_egg00:
+                recMode.SetMode( Data_Static, Data_Recoverable );
+            break;
+        case rmTemp_egg00:
+                recMode.SetMode( Data_Dynamic, Data_NotRecoverable );
+            // break; TODO
+        default:
+            return;
     }
-    else
+
+    Data_PageNo         dataPageNo = pageNo;
+    IOMan_BlockAddress  block;
+
+    Kernel_IAdminDiagnose::Instance().GetBlockAddress( taskId, dataPageNo, recMode, block );
+    
+    if( block.IsValid())
     {
-        deviceno = 0;
-        blockno  = 0;
+        deviceNo = block.GetDeviceNo();
+        blockNo  = block.GetBlockNo();
     }
 }
 /* --------------------------------------------------------------------------- */
 externPascal void gg999SetNewBlockAddress (
-    tsp00_TaskId                         taskid,
+    tsp00_TaskId                         taskId,
     tgg00_BasisError      VAR_VALUE_REF  error,
-    tsp00_PageNo                         pageno,
-    pasbool                              isDynamic,
-    tsp00_Int4                           deviceno,
-    tsp00_Int4                           blockno)
+    tsp00_PageNo                         pageNo,
+    tgg00_RecoveryMode_Param             recoveryMode,
+    tsp00_Int4                           deviceNo,
+    tsp00_Int4                           blockNo)
 {
-    Data_PageNo         auxpageno = pageno;
-    Data_AddressingMode addressmode = isDynamic ? Data_Dynamic : Data_Static;
-    IOMan_BlockAddress  blockaddress (deviceno, blockno);
+
+    Data_PageRecoveryMode   recMode;
     
-    if ( ! Kernel_IAdminDiagnose::Instance().SetNewBlockAddress(taskid, auxpageno, addressmode, blockaddress) )
+    switch( recoveryMode )
+    {
+        case rmPerm_egg00:
+            recMode.SetMode( Data_Dynamic, Data_Recoverable );
+            break;
+        case rmStatic_egg00:
+            recMode.SetMode( Data_Static, Data_Recoverable );
+            break;
+        default: // rmTemp_egg00 and rmNone_egg00
+            return;
+    }
+
+    Data_PageNo         dataPageNo = pageNo;
+    IOMan_BlockAddress  block( deviceNo, blockNo );
+    
+    if ( ! Kernel_IAdminDiagnose::Instance().SetNewBlockAddress( 
+        taskId, dataPageNo, recMode, block) )
+    {
         error = e_disk_not_accessible;
+    }
 }
 /* --------------------------------------------------------------------------- */
 externPascal void gg999DeleteBlockAddress (
-    tsp00_TaskId                         taskid,
+    tsp00_TaskId                         taskId,
     tgg00_BasisError      VAR_VALUE_REF  error,
-    tsp00_PageNo                         pageno,
-    pasbool                              isDynamic)
+    tsp00_PageNo                         pageNo,
+    tgg00_RecoveryMode_Param             recoveryMode)
 {
-    Data_PageNo         auxpageno = pageno;
-    Data_AddressingMode addressmode = isDynamic ? Data_Dynamic : Data_Static;
+    Data_PageRecoveryMode   recMode;
+
+    switch( recoveryMode )
+    {
+        case rmPerm_egg00:
+            recMode.SetMode( Data_Dynamic, Data_Recoverable );
+            break;
+        case rmStatic_egg00:
+            recMode.SetMode( Data_Static, Data_Recoverable );
+            break;
+        default: // rmTemp_egg00 and rmNone_egg00
+            return;
+    }
+
+    Data_PageNo dataPageNo = pageNo;
     
-    if ( ! Kernel_IAdminDiagnose::Instance().DeleteBlockAddress(taskid, auxpageno, addressmode) )
+    if ( ! Kernel_IAdminDiagnose::Instance().DeleteBlockAddress( 
+        taskId, dataPageNo, recMode ))
+    {
         error = e_disk_not_accessible;
+    }
 }
 /* --------------------------------------------------------------------------- */
 externPascal tgg00_BasisError gg999SuspendLogWriter (
@@ -868,6 +916,34 @@ externPascal void gg999SynchronizeServer (
     SAPDBMem_IRawAllocator &allocator = *(reinterpret_cast <SAPDBMem_IRawAllocator*>(trans.trAllocator_gg00));
     Kernel_IAdminHotStandby::Instance().SynchronizeServer(trans.trTaskId_gg00, allocator);
 }
+/* --------------------------------------------------------------------------- */
+
+/* Don't edit this file by hand, it is generated by ptocProto.py */
+
+externPascal void gg999BroadcastBackupHistoryEntry (
+    tsp00_TaskId                         taskid,
+    char *                               backup_info,
+    tsp00_Int4                           info_len)
+{
+    Kernel_IAdminHotStandby::Instance().BroadcastBackupHistoryEntry(
+                    taskid,
+                    backup_info,
+                    info_len);
+}
+
+/* --------------------------------------------------------------------------- */
+externPascal void gg999BroadcastBackupMediumDefEntry (
+    tsp00_TaskId                         taskid,
+    char *                               backup_info,
+    tsp00_Int4                           info_len)
+{
+    Kernel_IAdminHotStandby::Instance().BroadcastBackupMediumDefEntry(
+                    taskid, 
+                    backup_info,
+                    info_len);
+}
+
+
 /* --------------------------------------------------------------------------- */
 externPascal void gg999CreateSnapShot (
     tgg00_TransContext    VAR_VALUE_REF  trans)
@@ -913,26 +989,32 @@ externPascal void ggg999WriteUserForcedDump(
 }
 /* --------------------------------------------------------------------------- */
 externPascal void ggg999CheckData( 
-                                  tgg00_TransContext    VAR_VALUE_REF  trans,
-                                  pasbool                              bWithExtendedCheck,
-                                  pasbool                              bWithIndexes,
-                                  tsp00_Int4            VAR_VALUE_REF  badIndexCount,
-                                  tsp00_Int4            VAR_VALUE_REF  totalBadCount)
+    tgg00_TransContext    VAR_VALUE_REF  trans,
+    pasbool                              bWithExtendedCheck,
+    pasbool                              bWithIndexes,
+    tsp00_Int4            VAR_VALUE_REF  badIndexCount,
+    tsp00_Int4            VAR_VALUE_REF  totalBadCount)
 {
     Kernel_IAdminDiagnose::Instance().CheckData( trans, bWithExtendedCheck, bWithIndexes, 
         badIndexCount,totalBadCount );
 }
 /* --------------------------------------------------------------------------- */
 externPascal void ggg999CheckDataWithUpdate( 
-                                      tgg00_TransContext    VAR_VALUE_REF  trans,
-                                      pasbool                              bWithExtendedCheck,
-                                      tsp00_Int4            VAR_VALUE_REF  pagesUsed,
-                                      tsp00_Int4            VAR_VALUE_REF  blocksReleased,
-                                      tsp00_Int4            VAR_VALUE_REF  badIndexCount,
-                                      tsp00_Int4            VAR_VALUE_REF  totalBadCount)
+    tgg00_TransContext    VAR_VALUE_REF  trans,
+    pasbool                              bWithExtendedCheck,
+    tsp00_Int4            VAR_VALUE_REF  pagesUsed,
+    tsp00_Int4            VAR_VALUE_REF  blocksReleased,
+    tsp00_Int4            VAR_VALUE_REF  badIndexCount,
+    tsp00_Int4            VAR_VALUE_REF  totalBadCount)
 {
     Kernel_IAdminDiagnose::Instance().CheckDataWithUpdate( trans, bWithExtendedCheck,
         pagesUsed, blocksReleased, badIndexCount,totalBadCount );
 }
 /* --------------------------------------------------------------------------- */
+externPascal void ggg999Diagnose( tgg00_MessBlock VAR_VALUE_REF  messBlock )
+{
+    Kernel_IAdminDiagnose::Instance().Diagnose( messBlock );
+}
+
+/* --------------------------------------------------------------------------- */
 
