@@ -81,6 +81,8 @@ arguments : Trans              [in/out] TransContext
             pKeyIterator       [out]  pointer to the key range iterator
             bAscendingKeyOrder [in]   Should the iterator run from the smallest entry to the largest (=true)
                                       or vice versa (=false)
+            readBuffered       [in]   If true, then the kernel iterator buffers oids otherwise
+                                      the buffering has to occur in the liboms.
 ******************************************************************************/ 
 externCpp void
 kb04CreateKeyIterator (tgg00_TransContext &Trans,
@@ -90,7 +92,8 @@ kb04CreateKeyIterator (tgg00_TransContext &Trans,
 				       void		          *pLowerKey,             // PTS 1119480
 				       void		          *pUpperKey,             // PTS 1119480
 				       void            *&pKeyIterator,
-               bool              bAscendingKeyOrder);   // PTS 1119480
+               bool              bAscendingKeyOrder,    // PTS 1119480
+               bool              readBuffered);
 
 /*---------------------------------------------------------------------------*/
 
@@ -282,6 +285,7 @@ arguments : Trans            [in/out] TransContext
                                     the oid of this object is included in the return list. 
                                     Otherwise such an object in excluded
             KeyLen           [in]   length of the keys 
+            objBodySize      [in]   Size of the object
             pRestartKey      [in]   key which is used to find the position the iterator should start reading from
             noOfOid          [in/out] in: maximal number of oids which can be returned (=size of array pResultOId)
                                     out: number of oids which have been read (buckets [0..noOfOid-1] of the array
@@ -290,6 +294,9 @@ arguments : Trans            [in/out] TransContext
             pObjVers         [out]  array with 'Log-OIds' of the objects. Later these oids can be used to read the 
                                     objects without the need to access the history chain 
             LogReadAccesses  [out]  total number of read accesses to the log to find the next consisten object
+            pObjBody         [out]  pointer to a memory area of size 'objBodySize' into which the object is
+                                    copied after the call. The memory must be allocated in advance by the calling
+                                    routine. If the pointer equals NULL, then only the oid(s) is returned.
 
 Interface changed: PTS 1119480 
 
@@ -298,12 +305,15 @@ externCpp void
 kb04NextObjFromKeyIterator (tgg00_TransContext &Trans,
                             void               *pUntypedKeyIterator,    
                             bool                bInclusive,       
-                            tsp00_Int4          KeyLen,          
+                            tsp00_Int4          KeyLen,
+                            tsp00_Int4          objBodySize,
                             void               *pRestartKey,  
                             tsp00_Int4         &noOfOid,
                             tgg92_KernelOid    *pResultOId, 
                             tgg91_PageRef      *pObjVers,
-                            tsp00_Int4         &LogReadAccesses);
+                            tsp00_Int4         &LogReadAccesses,
+                            tsp00_MoveObj      *pObjBody);
+
 
 /*!***************************************************************************
 function   : kb04PrevObjFromKeyIterator
@@ -322,6 +332,7 @@ arguments : Trans            [in/out] TransContext
                                     the oid of this object is included in the return list. 
                                     Otherwise such an object in excluded
             KeyLen           [in]   length of the keys 
+            objBodySize      [in]   Size of the object
             pRestartKey      [in]   key which is used to find the position the iterator should start reading from
             noOfOid          [in/out] in: maximal number of oids which can be returned (=size of array pResultOId)
                                     out: number of oids which have been read (buckets [0..noOfOid-1] of the array
@@ -330,6 +341,9 @@ arguments : Trans            [in/out] TransContext
             pObjVers         [out]  array with 'Log-OIds' of the objects. Later these oids can be used to read the 
                                     objects without the need to access the history chain 
             LogReadAccesses  [out]  total number of read accesses to the log to find the next consisten object
+            pObjBody         [out]  pointer to a memory area of size 'objBodySize' into which the object is
+                                    copied after the call. The memory must be allocated in advance by the calling
+                                    routine. If the pointer equals NULL, then only the oid(s) is returned.
 
 New function since: PTS 1119480 
 
@@ -338,12 +352,14 @@ externCpp void
 kb04PrevObjFromKeyIterator (tgg00_TransContext &Trans,
                             void               *pUntypedKeyIterator,    
                             bool                bInclusive,       
-                            tsp00_Int4          KeyLen,          
+                            tsp00_Int4          KeyLen,  
+                            tsp00_Int4          objBodySize,
                             void               *pRestartKey,  
                             tsp00_Int4         &noOfOid,
                             tgg92_KernelOid    *pResultOId, 
                             tgg91_PageRef      *pObjVers,
-                            tsp00_Int4         &LogReadAccesses);
+                            tsp00_Int4         &LogReadAccesses,
+                            tsp00_MoveObj      *pObjBody);
 
 /*---------------------------------------------------------------------------*/
 
