@@ -121,7 +121,7 @@ inline static void kb04_EnterKb (tgg00_TransContext	   &Trans,
     {
         Trans.trState_gg00.delElement (tsFunctRollbSet_egg00);
         REINTERPRET_CAST(Log_Transaction*,Trans.trLogContext_gg00)->BeginSubtrans();
-        if ((Trans.trError_gg00 = e_ok) &&                                      // PTS XYZXYZ FF 2003-DEC-05
+        if ((Trans.trError_gg00 != e_ok) &&       // PTS XYZXYZ FF 2003-DEC-05
             ((g01vtrace.vtrAll_gg00 || g01vtrace.vtrBdObject_gg00)))
             Kernel_VTrace() << __FILE__ << ":" << __LINE__ << " BeginSubtrans error:" << Trans.trError_gg00; 
     }
@@ -230,7 +230,8 @@ kb04CreateKeyIterator (tgg00_TransContext &Trans,
 				       void		          *pLowerKey,   
 				       void		          *pUpperKey,   
 				       void              *&pKeyIterator,
-               bool              bAscendingKeyOrder)   
+               bool              bAscendingKeyOrder,
+               bool              readBuffered)   
 {
     ROUTINE_DBG_MSP00 ("kb04CreateKeyIterator");
     
@@ -242,7 +243,8 @@ kb04CreateKeyIterator (tgg00_TransContext &Trans,
     if (Trans.trError_gg00 != e_ok) return;
     
     bd04CreateKeyIterator (Trans,  ObjFile, KeyLen,  pStartKey, 
-        pLowerKey, pUpperKey, pKeyIterator, bAscendingKeyOrder);
+        pLowerKey, pUpperKey, pKeyIterator, bAscendingKeyOrder,
+        readBuffered);
     
     kb04_ErrorHandling (Trans, ThisMessType, WriteVtrace, ! BEGIN_SUBTRANS_SET);      
 }
@@ -671,20 +673,16 @@ kb04NewObj (tgg00_TransContext  &Trans,
         }
         else
         {
-            // Any error occured during registering the key.
+            // PTS 1132118 UH 2004-10-25 ResultOid is always assigned
+            /* return OId of object which already belongs to the   */
+            /* current key, PTS 1105726 Alexander Kley 29/02/2000  */     
 
-            if (e_duplicate_hash_key == Trans.trError_gg00) 
-            {
-                /* return OId of object which already belongs to the   */
-                /* current key, PTS 1105726 Alexander Kley 29/02/2000  */     
-    
-                ResultOid     = ReUsedOId;
-                ResultObjVers = ReUsedObjVers;
-    
-                // PTS 1116896 mb 2002-07-25 write rollback-information in order to perform
-                // a rollback of the newObj at restart-time
-                // PTS 1117603 UH 2002-09-12 removed again
-            }
+            ResultOid     = ReUsedOId;
+            ResultObjVers = ReUsedObjVers;
+
+            // PTS 1116896 mb 2002-07-25 write rollback-information in order to perform
+            // a rollback of the newObj at restart-time
+            // PTS 1117603 UH 2002-09-12 removed again
         }
     }
 
@@ -754,12 +752,14 @@ externCpp void
 kb04NextObjFromKeyIterator (tgg00_TransContext &Trans,
                             void               *pUntypedKeyIterator,    
                             bool                bInclusive,  
-                            tsp00_Int4          KeyLen,      
+                            tsp00_Int4          KeyLen,    
+                            tsp00_Int4          objBodySize,
                             void               *pRestartKey, 
                             tsp00_Int4         &noOfOid,
                             tgg92_KernelOid    *pResultOId, 
                             tgg91_PageRef      *pObjVers,
-                            tsp00_Int4         &LogReadAccesses)
+                            tsp00_Int4         &LogReadAccesses,
+                            tsp00_MoveObj      *pObjBody)
 { 
     ROUTINE_DBG_MEO00 ("kb04NextObjConsistent");
     
@@ -770,8 +770,9 @@ kb04NextObjFromKeyIterator (tgg00_TransContext &Trans,
     kb04_EnterKb (Trans, ThisMessType, ThisMessType2, WriteVtrace, ! BEGIN_SUBTRANS_SET);
     if (e_ok != Trans.trError_gg00) return;
 	
-	bd04NextObjFromKeyIterator (Trans, pUntypedKeyIterator, bInclusive, KeyLen, pRestartKey, 
-                                noOfOid, pResultOId, pObjVers, LogReadAccesses);
+	  bd04NextObjFromKeyIterator (Trans, pUntypedKeyIterator, bInclusive, KeyLen, objBodySize,
+                                pRestartKey, noOfOid, pResultOId, pObjVers, LogReadAccesses,
+                                pObjBody);
 
     /* PTS 1119480
     #   if COMPILEMODE_MEO00 >= SLOW_MEO00
@@ -789,11 +790,13 @@ kb04PrevObjFromKeyIterator (tgg00_TransContext &Trans,
                             void               *pUntypedKeyIterator,    
                             bool                bInclusive,  
                             tsp00_Int4          KeyLen,      
+                            tsp00_Int4          objBodySize,
                             void               *pRestartKey, 
                             tsp00_Int4         &noOfOid,
                             tgg92_KernelOid    *pResultOId, 
                             tgg91_PageRef      *pObjVers,
-                            tsp00_Int4         &LogReadAccesses)
+                            tsp00_Int4         &LogReadAccesses,
+                            tsp00_MoveObj      *pObjBody)
 { 
     ROUTINE_DBG_MEO00 ("kb04PrevObjConsistent");
     
@@ -804,8 +807,9 @@ kb04PrevObjFromKeyIterator (tgg00_TransContext &Trans,
     kb04_EnterKb (Trans, ThisMessType, ThisMessType2, WriteVtrace, ! BEGIN_SUBTRANS_SET);
     if (e_ok != Trans.trError_gg00) return;
     
-    bd04PrevObjFromKeyIterator (Trans, pUntypedKeyIterator, bInclusive, KeyLen, pRestartKey, 
-                                noOfOid, pResultOId, pObjVers, LogReadAccesses);
+    bd04PrevObjFromKeyIterator (Trans, pUntypedKeyIterator, bInclusive, KeyLen, objBodySize,
+                                pRestartKey, noOfOid, pResultOId, pObjVers, LogReadAccesses,
+                                pObjBody);
 
     /* PTS 1119480
 #   if COMPILEMODE_MEO00 >= SLOW_MEO00
