@@ -48,7 +48,8 @@ externPascal void kb391_InitLogSaveIterator (
     pasbool                              dbIsCold,
     pasbool                              forAutosave,
     pasbool                              completeLog,
-    Log_SaveIterator*&                   iterator)
+    pasbool                              repeatable)
+
 {
     SAPDBTRACE_ROUTINE_DEBUG ("kb391_InitLogSaveIterator", LogVolume_Trace, 5);
 
@@ -65,168 +66,158 @@ externPascal void kb391_InitLogSaveIterator (
               *(reinterpret_cast <SAPDBMem_IRawAllocator*>(trans.trAllocator_gg00));
 
     Log_Volume           &log = Log_Volume::Instance();
-    Log_Volume::SaveInfo  saveInfo;
-    
-    log.GetSaveLogIterator(  trans.trTaskId_gg00,
-                             allocator,
-                             1==dbIsCold,
-                             1==forAutosave,
-                             1==completeLog,
-                             maxToBeRead,
-                             iterator,
-                             saveInfo);
+    Log_Volume::SaveRange  saveRange;
 
-    if ( iterator == NULL )
+    if (forAutosave)                                  //PTS 1128703 mb 2004-04-23
     {
-        switch (saveInfo)
-        {
-            case Log_Volume::okay:
-                    trans.trError_gg00 = e_ok;
-                    break;
-            case Log_Volume::noLogToSave:
-					if ( forAutosave )
-						trans.trError_gg00 = e_incomplete_logsegm;
-					else
-						trans.trError_gg00 = e_no_log_to_save;
-                    break;
-            case Log_Volume::historyIsLost:
-                    trans.trError_gg00 = e_backup_history_lost; 
-                    break;
-            case Log_Volume::incompleteSegment:
+        saveRange = Log_Volume::OnlyCompleteLogSegment;
+    }
+    else if (completeLog)
+    {
+        saveRange = Log_Volume::AllAtOnce;
+    }
+    else
+    {
+        saveRange = Log_Volume::AllIncompleteSegmentsAllowed;
+    }
+
+    Log_Volume::InitLogSaveResult  result = log.InitLogSaveIterator( trans.trTaskId_gg00,
+                                                              allocator,
+                                                              saveRange,
+                                                              repeatable,
+                                                              maxToBeRead);
+    switch (result)
+    {
+        case Log_Volume::Okay:
+                trans.trError_gg00 = e_ok;
+                break;
+        case Log_Volume::NoLogToSave:
+                if ( forAutosave )
                     trans.trError_gg00 = e_incomplete_logsegm;
-                    break;
-            default:
-                    trans.trError_gg00 = e_ok;
-                    break;
-        }
+                else
+                    trans.trError_gg00 = e_no_log_to_save;
+                break;
+        case Log_Volume::HistoryIsLost:
+                trans.trError_gg00 = e_backup_history_lost; 
+                break;
+        case Log_Volume::IncompleteSegment:
+                trans.trError_gg00 = e_incomplete_logsegm;
+                break;
+        case Log_Volume::InitializeFailed:
+        default:
+                trans.trError_gg00 = e_log_error;
+                break;
     }
 }
 /*---------------------------------------------------------------------------*/
-externPascal void kb391_ReadNextLogpages (
-    tgg00_TransContext    VAR_VALUE_REF  trans,
-    Log_SaveIterator&                    iterator,
-    tkb3_block_ptr                       queue_buffer,
-    tsp00_Int2            VAR_VALUE_REF  num_read)
+externPascal void kb391_InvalidateSaveIterator ( )
 {
-    SAPDBTRACE_ROUTINE_DEBUG ("kb391_ReadNextLogpages", LogVolume_Trace, 5);
-    Log_DeviceBlockCount unsigned_numPagesRead;
-    
-    iterator.GetNextLogPages (trans.trTaskId_gg00, queue_buffer, unsigned_numPagesRead);
-    num_read = unsigned_numPagesRead;
+    Log_Volume::Instance().GetLogSaveIter().Invalidate();
 }
-/*---------------------------------------------------------------------------*/
-externPascal void kb391_PrepareFreeLogForPipe (
-    Log_SaveIterator&                    iterator,
+/*-----------------------------------------------------------------------------------*/
+tgg00_BasisError MapFreeLogResult(const Log_Volume::FreeLogResult result)
+{
+    
+    switch (result)
+    {
+        case Log_Volume::FreeLogOk:
+        case Log_Volume::RepeatedLogSaveNeeded:
+            return e_ok;
+            break;
+        case Log_Volume::IOSequenceDoesNotMatch:
+            return e_invalid;
+            break;
+        case Log_Volume::LogVolumeNotAccessible:
+        case Log_Volume::NoSaveLogExecutedBefore:
+        default:
+            return e_log_error;
+    }
+}
+/*-----------------------------------------------------------------------------------*/
+externPascal void kb391_FreeLog ( tsp00_TaskId                         taskId,
+                                 tgg00_BasisError      VAR_VALUE_REF  trError)
+{
+    SAPDBTRACE_ROUTINE_DEBUG ("kb391_FreeLog", LogVolume_Trace, 5);
+    Log_Volume& log = Log_Volume::Instance();
+
+    trError = MapFreeLogResult(log.FreeLog (taskId));    
+}
+/*-----------------------------------------------------------------------------------*/
+externPascal void kb391_FreeLogForPipe (
+    tsp00_TaskId                         taskId,
+    const Log_IOSequenceNo&              firstSavedIOsequence,
+    const Log_IOSequenceNo&              lastSavedIOsequence,
     tgg00_BasisError      VAR_VALUE_REF  trError)
 {
     // PTS 1114791 mb 2002-08-22 new
     Log_Volume &log = Log_Volume::Instance();
-    Log_Volume::PipeHandlingResult result;
-    result = log.PrepareFreeLogForPipe(iterator.GetStartIOSequence(),
-                                       iterator.GetEndIOSequence(),
-                                       iterator.GetStartPageOffset(),
-                                       iterator.GetEndPageOffset());
-    switch (result)
+
+    trError = MapFreeLogResult(log.FreeLogForPipe(taskId, firstSavedIOsequence, lastSavedIOsequence));
+}
+
+
+/*---------------------------------------------------------------------------*/
+externPascal void kb391_ReadNextLogpages (
+    tgg00_TransContext    VAR_VALUE_REF  trans,
+    tkb3_block_ptr                       queue_buffer,
+    tsp00_Int2            VAR_VALUE_REF  numRead)
+{
+    SAPDBTRACE_ROUTINE_DEBUG ("kb391_ReadNextLogpages", LogVolume_Trace, 5);
+
+    Log_SaveIterator&  iterator=Log_Volume::Instance().GetLogSaveIter();
+
+    Log_DeviceBlockCount unsignedNumPagesRead;
+    
+    if (iterator.IsValid())
     {
-        case Log_Volume::pipeOk:
-            trError = e_ok; 
-            break;
-        case Log_Volume::noFreeLogCalledInbetween:
-            trError = e_invalid; // same error as in V7.3
-            break;
-        default:
-            trError = e_log_error;
+        iterator.GetNextLogPages ( trans.trTaskId_gg00, 
+                                   queue_buffer, 
+                                   unsignedNumPagesRead);
+        numRead = unsignedNumPagesRead;
+    }
+    else
+    {
+        numRead = 0;
     }
 }
 /*---------------------------------------------------------------------------*/
-externPascal void kb391_DeleteIterator (
-    tgg00_TransContext    VAR_VALUE_REF  trans,
-    Log_SaveIterator**                   iterator)
-{
-    SAPDBTRACE_ROUTINE_DEBUG ("kb391_DeleteIterator", LogVolume_Trace, 5);
-    SAPDBERR_ASSERT_STATE( iterator != NULL );
-
-    // deallocate memory */
-    SAPDBMem_IRawAllocator& Allocator =
-        *(REINTERPRET_CAST (SAPDBMem_IRawAllocator*,
-                            trans.trAllocator_gg00));
-
-    (*iterator)->Delete();
-    destroy(*iterator, Allocator);
-    *iterator = NULL;
-}
-/*---------------------------------------------------------------------------*/
-externPascal void kb391_GetStartTime (
-    Log_SaveIterator&                    iterator,
+externPascal void kb391_GetSaveIterInfo (
     Kernel_Time&                         startTime,
-    Kernel_Date&                         startDate)
-{
-    iterator.GetStartTime(startDate, startTime);
-}
-/*---------------------------------------------------------------------------*/
-externPascal void kb391_GetEndTime (
-    Log_SaveIterator&                    iterator,
+    Kernel_Date&                         startDate,
     Kernel_Time&                         endTime,
-    Kernel_Date&                         endDate)
+    Kernel_Date&                         endDate,
+    Log_IOSequenceNo&                    startIOSeq,
+    Log_IOSequenceNo&                    endIOSeq,
+    tsp00_Uint4           VAR_VALUE_REF  backupCount)
 {
-    iterator.GetEndTime(endDate, endTime);
-}
-/*---------------------------------------------------------------------------*/
-externPascal void kb391_GetStartIOSeq (
-    Log_SaveIterator&                    iterator,
-    Log_IOSequenceNo&                    startIOSeq)
-{
-    startIOSeq = iterator.GetStartIOSequence();
-}
-/*---------------------------------------------------------------------------*/
-externPascal void kb391_GetEndIOSeq (
-    Log_SaveIterator&                    iterator,
-    Log_IOSequenceNo&                    endIOSeq)
-{
-    endIOSeq = iterator.GetEndIOSequence();
-}
-/*---------------------------------------------------------------------------*/
-externPascal void kb391_GetStartPageOffset (
-    Log_SaveIterator&                    iterator,
-    Log_RawDeviceOffset&                 startOffset)
-{
-    startOffset = iterator.GetStartPageOffset();
-}
-/*---------------------------------------------------------------------------*/
-externPascal void kb391_GetLastSavedIOSeq (
-    Log_SaveIterator&                    iterator,
-    Log_IOSequenceNo&                    endIOSeq)
-{
-    SAPDBTRACE_ROUTINE_DEBUG ("kb391_GetLastSavedIOSeq", LogVolume_Trace, 5);
-    endIOSeq = iterator.GetEndIOSequence();
-}
-/*---------------------------------------------------------------------------*/
-externPascal void kb391_GetLastSavedPageOffset (
-    Log_SaveIterator&                    iterator,
-    tsp00_Uint4           VAR_VALUE_REF  endOffset)
-{
-    SAPDBTRACE_ROUTINE_DEBUG ("kb391_GetLastSavedPageOffset", LogVolume_Trace, 5);
-    endOffset = iterator.GetEndPageOffset();
+    Log_SaveIterator & saveIter = Log_Volume::Instance().GetLogSaveIter();
+    
+    Log_Volume::Instance().GetLogSaveIter().GetStartTime(startDate, startTime);
+    Log_Volume::Instance().GetLogSaveIter().GetEndTime(endDate, endTime);
+    startIOSeq = Log_Volume::Instance().GetLogSaveIter().GetStartIOSequence();
+    endIOSeq = Log_Volume::Instance().GetLogSaveIter().GetEndIOSequence();
+    backupCount = Log_Volume::Instance().GetLogSaveIter().GetLogBackupCount();
 }
+
 /*---------------------------------------------------------------------------*/
 externPascal void kb391_GetNumOfPagesLeftForSegment (
-    Log_SaveIterator*                    iterator,
     tsp00_Uint4           VAR_VALUE_REF  numOfUnsavedPages)
 {
-    if ( 0 == iterator )
+    if ( Log_Volume::Instance().GetLogSaveIter().IsValid() )
         numOfUnsavedPages = 0;
 	else
-        numOfUnsavedPages = iterator->GetNumPagesLeft();
+        numOfUnsavedPages = Log_Volume::Instance().GetLogSaveIter().GetNumPagesLeft();
 }
 /*---------------------------------------------------------------------------*/
 externPascal void kb391_GetNumOfPagesLeftForLogDev (
-    Log_SaveIterator*                    iterator,
     tsp00_Uint4           VAR_VALUE_REF  numOfUnsavedPages)
 {
     SAPDBTRACE_ROUTINE_DEBUG ("kb391_GetNumOfPagesLeftForLogDev", LogVolume_Trace, 5);
 
     Log_Volume &log = Log_Volume::Instance();
-    if ( 0 == iterator )
+    Log_SaveIterator&  iterator=Log_Volume::Instance().GetLogSaveIter();
+
+    if ( iterator.IsInvalid() )
     {
         numOfUnsavedPages = log.GetNumberOfUnsavedPages();
         return;
@@ -236,39 +227,34 @@ externPascal void kb391_GetNumOfPagesLeftForLogDev (
 
     if ( log.GetNumberOfUnsavedPages() >= autosaveDistance )
         numOfUnsavedPages = log.GetNumberOfUnsavedPages()
-                            - autosaveDistance + iterator->GetNumPagesLeft();
+                            - autosaveDistance + iterator.GetNumPagesLeft();
     else
         // the last segment to be saved
-        numOfUnsavedPages = iterator->GetNumPagesLeft();
+        numOfUnsavedPages = iterator.GetNumPagesLeft();
 }
-/*---------------------------------------------------------------------------*/
-externPascal void kb391_GetBackupCount (
-    Log_SaveIterator&                    iterator,
-    tsp00_Uint4           VAR_VALUE_REF  backupCount)
+//----------------------------------------------------------------------------
+externPascal pasbool kb391_IOSeqIsInPage (
+    const Log_IOSequenceNo               &   ioSequence,
+          tkb00_Page         VAR_VALUE_REF   page)
 {
-    backupCount = iterator.GetLogBackupCount();
+    Kernel_IPage::PageFrame frame(&page, g01page_size);
+    Log_Page logPage(frame);
+
+    return ioSequence.IsInRange(logPage.ReadFirstWriterIOSequenceNo(),
+                                logPage.ReadLastWriterIOSequenceNo());
 }
-/*---------------------------------------------------------------------------*/
-externPascal tsp00_Uint4 kb391_ReadFirstWriterIOSequenceNo (
+//----------------------------------------------------------------------------
+externPascal pasbool kb391_IOSeqIsYoungerThanPage (
+    const Log_IOSequenceNo&                 ioSequence,
+    const Log_IOSequenceNo&                 oldestKnownIOSeq,
     tkb00_Page            VAR_VALUE_REF  page)
 {
     Kernel_IPage::PageFrame frame(&page, g01page_size);
-    Log_Page log_page(frame);
-    return log_page.ReadFirstWriterIOSequenceNo().RawValue();
-}
-/*---------------------------------------------------------------------------*/
-externPascal tsp00_Uint4 kb391_ReadLastWriterIOSequenceNo (
-    tkb00_Page            VAR_VALUE_REF  page)
-{
-    Kernel_IPage::PageFrame frame(&page, g01page_size);
-    Log_Page log_page(frame);
-    return log_page.ReadLastWriterIOSequenceNo().RawValue();
-}
-/*---------------------------------------------------------------------------*/
-externPascal pasbool kb391_IsCompleteSegment (
-    Log_SaveIterator&                    iterator)
-{
-    return iterator.IsCompleteSegment();
+    Log_Page logPage(frame);
+
+    return Log_IOSequenceNo::LeftIsYounger( ioSequence,  
+                                            logPage.ReadLastWriterIOSequenceNo(),
+                                            oldestKnownIOSeq);
 }
 //----------------------------------------------------------------------------
 Kernel_TaskWaitList kb391WaitListAutosaveRead;
@@ -326,7 +312,7 @@ externPascal void kb391InsertWaitingTaskAtLast (
     pasbool                              forRead,
     pasbool                              forAutosave)
 {
-    SAPDBTRACE_ROUTINE_DEBUG ("kb391ResumeFirstWaitingTask", LogVolume_Trace, 5);
+    SAPDBTRACE_ROUTINE_DEBUG ("kb391InsertWaitingTaskAtLast", LogVolume_Trace, 5);
     GetWaitList(forRead,forAutosave).InsertAsLast (taskid, waitcontext);
 }
 /*---------------------------------------------------------------------------*/
@@ -462,13 +448,35 @@ externPascal void kb391CheckLogTapeAndDataCompatibility (
         return;
     }
 
-    /* PTS 1113550 mb 2002-22-03 */
-    if (memcmp(dbident, &(restartRecord.rstDbIdent_kb00()), sizeof(tsp00_Line)))
+    /* PTS 1113550 mb 2002-22-03 && PTS 1133214 mb 2005-01-11 */
+    if (restartRecord.rstDbIdent_kb00()[0] != '\0')
     {
-        if (!kb391SupressDBIdentCheckForMigration(backupVersion))
+        if (dbident[0] != '\0')
         {
-            trans.trError_gg00 = e_invalid_label;
-            return;
+            if (memcmp(dbident, &(restartRecord.rstDbIdent_kb00()), sizeof(tsp00_Line)))
+            {
+                if (!kb391SupressDBIdentCheckForMigration(backupVersion))
+                {
+                    trans.trError_gg00 = e_invalid_label;
+                    return;
+                }
+            }
+        }
+
+        // check logvolume and logbackup
+        Log_Volume & log = Log_Volume::Instance();
+        tsp00_Line   logVolumeDBIdent;
+        log.GetLogDBIdent ( logVolumeDBIdent );
+        if ((logVolumeDBIdent[0] != '\0') && !log.LogIsEmpty())
+        {
+            if (memcmp(logVolumeDBIdent, &(restartRecord.rstDbIdent_kb00()), sizeof(tsp00_Line)))
+            {
+                if (!kb391SupressDBIdentCheckForMigration(backupVersion))
+                {
+                    trans.trError_gg00 = e_invalid_label;
+                    return;
+                }
+            }
         }
     }
 
@@ -482,3 +490,9 @@ externPascal void kb391CheckLogTapeAndDataCompatibility (
     }
 }
 
+/*-----------------------------------------------------------------------------------*/
+
+externPascal pasbool kb391_ValidLogSaveIterator ()
+{
+    return Log_Volume::Instance().GetLogSaveIter().IsValid();    
+}
