@@ -299,53 +299,6 @@ externPascal tsp00_Int4 kb560GetNumOfUserUKTs ()
     return RTEInfo.NumOfUserUKTs;
 }
 
-/*-----------------------------------------------------------------------------------*/
-externPascal void kb560FreeLog (
-    tsp00_TaskId                         taskId,
-    const Log_IOSequenceNo              &oldLastSavedIOsequence,
-    tsp00_Uint4                          oldLastSavedOffset,
-    const Log_IOSequenceNo              &newLastSavedIOsequence,
-    tsp00_Uint4                          newLastSavedOffset)
-{
-    SAPDBTRACE_ROUTINE_DEBUG ("kb560FreeLog", LogVolume_Trace, 5);
-    Log_Volume& log = Log_Volume::Instance();
-    if ( ! log.FreeLog (taskId, oldLastSavedIOsequence, oldLastSavedOffset,
-                                  newLastSavedIOsequence, newLastSavedOffset) )
-    {
-        log.WriteDeviceParametersToTrace();
-        RTE_Crash( Log_Exception
-                   (__FILE__, __LINE__,
-                    LOG_FREE, SAPDB_ToString(oldLastSavedIOsequence,_T_d),
-                              SAPDB_ToString(newLastSavedIOsequence,_T_d),
-                              SAPDB_ToString(oldLastSavedOffset,_T_d),
-                              SAPDB_ToString(newLastSavedOffset,_T_d)) );
-    }
-}
-/*-----------------------------------------------------------------------------------*/
-externPascal void kb560ExecuteFreeLogForPipe (
-    tsp00_TaskId                         taskId,
-    const Log_IOSequenceNo&                 firstSavedIOsequence,
-    const Log_IOSequenceNo&                 lastSavedIOsequence,
-    tgg00_BasisError      VAR_VALUE_REF  trError)
-{
-    // PTS 1114791 mb 2002-08-22 new
-    Log_Volume &log = Log_Volume::Instance();
-
-    Log_Volume::PipeHandlingResult result;
-    result = log.ExecuteFreeLogForPipe(taskId, firstSavedIOsequence, lastSavedIOsequence);
-    switch (result)
-    {
-        case Log_Volume::pipeOk:
-            trError = e_ok;
-            break;
-        case Log_Volume::IOSequenceDoesNotMatch:
-            trError = e_invalid;
-            break;
-        default:
-            trError = e_log_error;
-    }
-}
-
 /*-----------------------------------------------------------------------------------*/
 externPascal pasbool kb560TransactionIsOpen (void* TransPtr)
 {
@@ -589,6 +542,8 @@ externPascal void kb560GetFirstKnownIOSeqBeforeRestoreLog (
     Log_IOSequenceNo&                    FirstIOSeq,
     pasbool               VAR_VALUE_REF  firstIOSeqIsValid)
 {
+    SAPDBTRACE_ROUTINE_DEBUG ("kb560GetFirstKnownIOSeqBeforeRestoreLog", LogVolume_Trace, 5);
+
     Log_Volume &log  = Log_Volume::Instance();
     
     Log_RawDeviceOffset     offset;
@@ -692,10 +647,10 @@ externPascal pasbool kb560GetLogInfo (
 }
 
 /*-----------------------------------------------------------------------------------*/
-externPascal pasbool kb560LogIsFull (tsp00_TaskId TaskId)
+externPascal pasbool kb560LogIsFull ()
 {
     SAPDBTRACE_ROUTINE_DEBUG ("kb560LogIsFull", LogVolume_Trace, 5);
-    return Log_Volume::Instance().LogIsFull(TaskId);
+    return Log_Volume::Instance().LogIsFull();
 }
 
 
@@ -1004,3 +959,9 @@ externPascal void kb560GetEvents (
 
     Log_Volume::Instance().GetLogFillingEvents(*pEventDescription);
 }
+/*-----------------------------------------------------------------------------------*/
+externPascal void kb560DetermineLocalRedoBufferSizeForTransactions ()
+{
+    SAPDBTRACE_ROUTINE_DEBUG ("kb560DetermineLocalRedoBufferSizeForTransactions", LogVolume_Trace, 5);
+	Log_Transaction::DetermineLocalRedoBufferSize();
+}
\ No newline at end of file
