@@ -733,16 +733,18 @@ kb67GetBeforeImage (tgg00_TransContext &Trans,
             ++LogReadAccesses;
         }
         
-        if ( (e_ok == Trans.trError_gg00) && (ConsistentView == Trans.trConsistView_gg00) )
-        {
-            /* UH 14-03-2000 */
-            /* It is forbidden to update an object within an explicite view.     */
-            /* In other words, if a transaction has updated an object by itself, */
-            /* then always the objects itself is consistent it.                  */
-            /* The current ConsistentView cannot be found in an history entry !! */
+		/* 1129577 FF 2004-05-11  following test isn't correct anymore
+		if ( (e_ok == Trans.trError_gg00) && (ConsistentView == Trans.trConsistView_gg00) )
+		{
+		/ * UH 14-03-2000 * /
+		/ * It is forbidden to update an object within an explicite view.     * /
+		/ * In other words, if a transaction has updated an object by itself, * /
+		/ * then always the objects itself is consistent it.                  * /
+		/ * The current ConsistentView cannot be found in an history entry !! * /
 
-            g01abort (kbMsg_csp03, csp3_n_history, "Invalid Upd in ExpliView", CurrEntryRef.gg91RefPno());
-        }
+		g01abort (kbMsg_csp03, csp3_n_history, "Invalid Upd in ExpliView", CurrEntryRef.gg91RefPno());
+		}
+		*/
         
         if ((e_ok == Trans.trError_gg00) && ! bObjFound)     
         {
