@@ -7,15 +7,15 @@
  
   responsible:  SteffenS and d033893
 
-  special area: SAP DB LOADER
+  special area: Loader
  
   description:  Packets, segments and parts
                 
-  version:      7.4.
+  version:      7.5.0
 
   -----------------------------------------------------------------------------
  
-  copyright:    (c) 1997-2004 SAP AG-2002
+  Copyright (c) 1997-2004 SAP AG-2002
  
   -----------------------------------------------------------------------------
 
