@@ -106,9 +106,6 @@ lLatin1_ToUpperMap_ls05[256] =
  0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xFF
 };
 
-
-
-
 /*!
     \brief  Converts ASCII 8 bit Latin-1 string in place to UPPER CASE
     \param  pszString      [IN] - the string to convert
@@ -397,6 +394,38 @@ ls05ConvertToUCS2Simple(unsigned char  *pszDestination,
                         tsp81_CodePage *pASCII_CP = NULL);
 
 
+/*!
+    \brief Converts UTF8 or ASCII string to given UCS2 encoding
+
+    \param  pszDestination  [OUT]  - destination string (ucs2)
+    \param  lDestinationLen [IN]   - length of destination string in byte
+    \param  DestEncoding    [IN]   - encoding of destination string
+    \param  pszSource       [IN]   - source string (ascii or utf8)
+    \param  lSourceLen      [IN]   - length of source string
+    \param  SrcEncoding     [IN]   - encoding of source string
+    \param  pszErrText      [OUT]  - error text
+    \param  pASCII_CP       [IN]   - code page structure used for conversion;
+                                      defaults to NULL pointer;
+                                      if NULL given code page ISO8859-1 = Latin1
+                                      is used
+
+    \return  errOK_els00                 - successful conversion<br>
+             errInternal_els98           - when partial character in source,
+                                            but hit end or insufficient<br>
+                                            space in target for conversion
+             errConvertingEncoding_els98 - source contains invalid chars or
+                                            char sequences
+*/
+tsp00_Longint
+ls05ConvertToUCS2Simple(unsigned char  *pszDestination,
+                        tsp00_Longuint &lDestinationLen,
+                        tls00_CodeType  DestEncoding,
+                        unsigned char  *pszSource,
+                        tsp00_Longint   lSourceLen,
+                        tls00_CodeType  SrcEncoding,
+                        tsp00_Addr      pszErrText,
+                        tsp81_CodePage *pASCII_CP = NULL);
+
 /*!
     \brief Converts a UTF8 or ASCII string to UCS2 encoding not swapped
 
@@ -480,7 +509,7 @@ ls05ConvertFromUCS2Simple(unsigned char  *pszDestination,
                           tsp00_Longint  &lDestinationLen,
                           tsp00_Longint   lSourceLen,
                           tls00_CodeType  DestEncoding,
-                          bool            bSourceSwapped,
+                          tls00_CodeType  SrcEncoding,
                           tsp00_Addr      pszErrText);
 
 
