@@ -1,25 +1,17 @@
 /*!
   -----------------------------------------------------------------------------
-
   module: hls08_long.h
-
   -----------------------------------------------------------------------------
 
   responsible:  SteffenS
 
-  special area: Replication Manager
+  special area: Loader
 
   description:  Manipulation of LONG values - loading and extracting
 
-  version:      7.4.
-
+  version:      7.5.
   -----------------------------------------------------------------------------
 
-  copyright:    (c) 1999-2004 SAP AG-2002
-
-  -----------------------------------------------------------------------------
-
-
     ========== licence begin  GPL
     Copyright (c) 1999-2004 SAP AG
 
@@ -37,10 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
-
 */
 #ifndef HLS08_LONG
 #define HLS08_LONG
@@ -148,9 +136,10 @@ ls08InitLongFileInfo(tls00_MultColSpec  *pMultColSpec,
   -----------------------------------------------------------------------------
 */
 int
-ls08ReadLongFileInfo(tls00_Column *myColumn,
-                     tls00_String *Value,
-                     tsp00_Addr    ErrText);
+ls08ReadLongFileInfo(tls00_Column    *pField,
+                     tls00_ParamInfo *pColumnParameters,
+                     tls00_String    *pValue,
+                     tsp00_Addr       pszErrText);
 
 
 /*!
@@ -426,4 +415,4 @@ ls08CloseGetVal(tls00_DBInfo         *pDBInfo,
   EndChapter:   Functions used to EXTRACT long values
   -----------------------------------------------------------------------------
 */
-#endif //HLS08_LONG
\ No newline at end of file
+#endif //HLS08_LONG
