@@ -1,26 +1,15 @@
 /*! 
   -----------------------------------------------------------------------------
- 
   module: hls13.h
- 
   -----------------------------------------------------------------------------
- 
   responsible:  SteffenS and d033893 
 
-  special area: SAP DB LOADER
+  special area: Loader
  
   description:  Functions to get all necessary information on a table from the database
                 
-  version:      7.4.
-
+  version:      7.5.
   -----------------------------------------------------------------------------
- 
-  copyright:    (c) 1999-2004 SAP AG-2002
- 
-  -----------------------------------------------------------------------------
-
-
-
     ========== licence begin  GPL
     Copyright (c) 1999-2004 SAP AG
 
@@ -38,9 +27,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 #ifndef HLS13
 #define HLS13
@@ -96,6 +82,30 @@ int
 ls13ExecExtSQLStmt(tls00_DBInfo *DBInfo, tls00_DBSqlCmd *SqlCmd, tsp00_Addr ErrText);
 
 
+/*!
+  -----------------------------------------------------------------------------
+   function:     ls13ExecExtSQLStmt (overloaded)
+  -----------------------------------------------------------------------------
+  description:  Executes an SQL statement thats sent by the user.
+
+  arguments:    DBInfo          [IN]    - structure with all info on the running
+                                           session and the database connected to
+                pszSqlCmd       [IN]    - Stmt to be executed
+                ErrText         [OUT]   - Error text
+
+  returnvalue:  errOK_els00                 - no errors
+                errFinishPart_els00         - could not finish part to send - fatal error
+                error code for sql error    - otherwise
+
+                Error messages will be returned in ErrText in the following form:
+                "SQL-Error <error number> = <error text> (error position: <position>)".
+                Error text is logged to the log file, too.
+  -----------------------------------------------------------------------------
+*/
+int
+ls13ExecExtSQLStmt(tls00_DBInfo *pDBInfo, const char* pszSqlCmd, tsp00_Addr ErrText);
+
+
 /*!
   -----------------------------------------------------------------------------
   function:     ls13ExecIntSQLStmt
@@ -133,84 +143,6 @@ ls13ExecIntSQLStmt(tls00_DBInfo            *DBInfo,
                    tsp1_cmd_mess_type_Enum  MessType  = sp1m_dbs,
                    bool                     bMassCmd  = false);
 
-/*!
-  -----------------------------------------------------------------------------
- 
-  function:     ls13ExecIntSQLPrepared
-
-  -----------------------------------------------------------------------------
-
-  description:  Executes an prepared SQL statement that was built by an repserver 
-                internal function.
-                All commands are send as internal (producer = sp1pr_internal_cmd)
-
-  arguments:    DBInfo          [IN]    - structure with all info on the running
-                                           session and the database connected to
-                lPcktNo         [IN]    - packet to use
-                PartStruct      [IN]    - structure that contains all info to process 
-                                          the command
-                ErrText         [OUT]   - error text
-                ErrPos          [OUT]   - position of erroneous part in packet
-                bWithInfo       [IN]    - with info flag (defaults to false)
-                MessType        [IN]    - message type (defaults to sp1m_dbs)
-                bMassCmd        [IN]    - mass command flag (defaults to false)
-
-  returnvalue:  errOK_els00                 - no errors
-                errFinishPart_els00         - could not finish part to send - fatal error
-                error code for sql error    - otherwise
-
-                Error messages will be returned in ErrText in the following form:
-                "SQL-Error: <error number> = <error text> (error position: <position>)".
-  -----------------------------------------------------------------------------
-*/
-tsp00_Int4
-ls13ExecIntSQLPrepared(tls00_DBInfo            *DBInfo,
-                       tsp00_Int4               lPcktNo,
-                       tls00_Parts4InternalSql &PartStruct,
-                       tsp00_Addr               pszErrText,
-                       tsp00_Int4              &lErrPos,
-                       bool                     bWithInfo = false ,  
-                       tsp1_cmd_mess_type_Enum  MessType  = sp1m_execute,  
-                       bool                     bMassCmd  = false ) ;
-
-
-/*!
-  -----------------------------------------------------------------------------
-  function:     ls13ExecIntSQLStmtUTF8
-  -----------------------------------------------------------------------------
-
-  description:  Executes an SQL statement thats build by an repserver internal
-                function. The build command is an UTF8 String.
-                All commands are send as internal (producer = sp1pr_internal_cmd)
-
-  arguments:    DBInfo          [IN]    - structure with all info on the running
-                                           session and the database connected to
-                lPcktNo         [IN]    - packet to use
-                pszSqlStmt      [IN]    - Statement to execute (must be null terminated
-                                           and UTF8 encoded)
-                ErrText         [OUT]   - error text
-                ErrPos          [OUT]   - position of erroneous part in packet
-                bWithInfo       [IN]    - with info flag (defaults to false)
-                MessType        [IN]    - message type (defaults to sp1m_dbs)
-                bMassCmd        [IN]    - mass command flag (defaults to false)
-
-  returnvalue:  errOK_els00                 - no errors
-                errFinishPart_els00         - could not finish part to send - fatal error
-                error code for sql error    - otherwise
-
-                Error messages will be returned in ErrText in the following form:
-                "SQL-Error: <error number> = <error text> (error position: <position>)".
-  -----------------------------------------------------------------------------
-*/
-int
-ls13ExecIntSQLStmtUTF8(tls00_DBInfo            *DBInfo,
-                       tsp00_Int4               lPcktNo,
-                       tsp00_Addr               pszSqlStmt,
-                       tsp00_Addr               pszErrText,
-                       tsp00_Int4              &lErrPos,
-                       bool                     bWithInfo = false,
-                       tsp1_cmd_mess_type_Enum  MessType  = sp1m_dbs,
-                       bool                     bMassCmd  = false);
 
 /*!
   -----------------------------------------------------------------------------
@@ -259,55 +191,6 @@ ls13ExecIntSQLStmtEx(tls00_DBInfo            *DBInfo,
                      bool                     bMassCmd  = false);
 
 
-/*!
-  -----------------------------------------------------------------------------
-  function:     ls13ExecIntSQLStmtExUTF8
-  -----------------------------------------------------------------------------
-
-  description:  Executes an SQL statement thats built by a repserver internal
-                function. The build command is an UTF8 String.
-                The packet contains an additional data part necessary
-                for command execution.
-                In case the length of data (lDataLength) is set to 0 no data
-                part is added.
-                All commands are send as internal (producer = sp1pr_internal_cmd)
-
-  arguments:    DBInfo       [IN]    - structure with all info on the running
-                                        session and the database connected to
-                lPcktNo      [IN]    - packet to use
-                pszSqlStmt   [IN]    - Statement to execute (must be null terminated
-                                        and UTF8 encoded)
-                pszData      [IN]    - Data to send in data part, not null terminated
-                lDataLength  [IN]    - length of data
-                pszErrText   [OUT]   - error text
-                lErrorPos    [OUT]   - error position in sent packet in case of error
-                bWithInfo    [IN]    - with info flag (defaults to false)
-                MessType     [IN]    - message type (defaults to sp1m_dbs)
-                bMassCmd     [IN]    - mass command flag (defaults to false)
-
-  returnvalue:  errOK_els00              - no errors
-                errFinishPart_els00      - could not finish part to send - fatal error
-                error code for sql error - otherwise
-
-                Error messages will be returned in ErrText in the following form:
-                "SQL-Error <error number> = <error text> (error position: <position>)"
-                No error text is logged by this function!
-  -----------------------------------------------------------------------------
-*/
-tsp00_Int4
-ls13ExecIntSQLStmtExUTF8(tls00_DBInfo            *DBInfo,
-                         tsp00_Int4               lPcktNo,
-                         tsp00_Addr               pszSqlStmt,
-                         tsp00_Addr               pszData,
-                         tsp00_Int4               lDataLength,
-                         tsp00_Addr               pszErrText,
-                         tsp00_Int4              &ErrPos,
-                         tsp1_part_kind_Enum      PartKind  = sp1pk_data,
-                         bool                     bWithInfo = false,
-                         tsp1_cmd_mess_type_Enum  MessType  = sp1m_dbs,
-                         bool                     bMassCmd  = false);
-
-
 /*!
   -----------------------------------------------------------------------------
   function:     ls13ExecIntSQLMultParts
@@ -359,57 +242,6 @@ ls13ExecIntSQLMultParts(tls00_DBInfo            *DBInfo,
                         bool                     bMassCmd  = false);
 
 
-/*!
-  -----------------------------------------------------------------------------
-  function:     ls13ExecIntSQLMultPartsUTF8
-  -----------------------------------------------------------------------------
-
-  description:  Executes an SQL statement thats build by an repserver internal
-                function. The built command must be UTF8 encoded.
-                The PartStruct parameter contains all necessary data - the command
-                itself, the data, that has to be sent - for kernel execution. This
-                structure has to be filled by the caller.
-                All commands are send as internal (producer = sp1pr_internal_cmd).
-
-                The first member of PartStruct MUST be a command (part kind
-                sp1pk_command) which is checked. If not the function fails with
-                an assertion.
-  
-  attention:    Useful function only if command part is not immediately followed by
-                a data part or if more than 2 parts need to be send because the
-                filling of the needed structure is expensive.
-
-  arguments:    DBInfo       [IN]    - structure with all info on the running
-                                        session and the database connected to
-                lPcktNo      [IN]    - packet to use
-                PartStruct   [IN]    - structure that contains all info to process 
-                                        the command
-                pszErrText   [OUT]   - error text
-                lErrorPos    [OUT]   - error position in sent packet in case of error
-                bWithInfo    [IN]    - with info flag (defaults to false)
-                MessType     [IN]    - message type (defaults to sp1m_dbs)
-                bMassCmd     [IN]    - mass command flag (defaults to false)
-
-  returnvalue:  errOK_els00                 - no errors
-                errFinishPart_els00         - could not finish part to send - fatal error
-                error code for sql error    - otherwise
-
-                Error messages will be returned in ErrText in the following form:
-                "SQL-Error <error number> = <error text> (error position: <position>)"
-                No error text is logged by this function!
-  -----------------------------------------------------------------------------
-*/
-tsp00_Int4
-ls13ExecIntSQLMultPartsUTF8(tls00_DBInfo            *DBInfo,
-                            tsp00_Int4               lPcktNo,
-                            tls00_Parts4InternalSql &PartStruct,
-                            tsp00_Addr               pszErrText,
-                            tsp00_Int4              &lErrPos,
-                            bool                     bWithInfo = false,
-                            tsp1_cmd_mess_type_Enum  MessType  = sp1m_dbs,
-                            bool                     bMassCmd  = false);
-
-
 /*!
   -----------------------------------------------------------------------------
   function:     ls13Parse
