@@ -1,9 +1,7 @@
 /*! 
   -----------------------------------------------------------------------------
- 
-  module: hls18.h
- 
-  -----------------------------------------------------------------------------
+   module: hls18.h
+   -----------------------------------------------------------------------------
  
   responsible:  SteffenS and d033893
 
@@ -12,15 +10,8 @@
   description:  Data file row and value functionality               
 
   version:      7.5.
-
   -----------------------------------------------------------------------------
 
-  copyright:    (c) 1999-2004 SAP AG-2004
-
-  -----------------------------------------------------------------------------
-
-
-
     ========== licence begin  GPL
     Copyright (c) 1999-2004 SAP AG
 
@@ -38,9 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 
 #ifndef HLS18
@@ -290,14 +278,13 @@ ls18GetUnformattedValue(tsp00_Addr    pszLine,
 
 /*!
   -----------------------------------------------------------------------------
-  function:     ls18GetUnformattedValueEx
+  function:     ls18GetCSVData
   -----------------------------------------------------------------------------
 
   description:  Parses a data line and assigns single data to special structure
                 members. The data is separated by separators and may be enclosed
                 in delimiters.
-                The function uses a special pattern searching routine to find
-                the data.
+                This is true CSV parsing without a head record.
 
   arguments:    pString       [IN]  - char string to parse
                 lStrLen       [IN]  - string length of pString
@@ -316,14 +303,14 @@ ls18GetUnformattedValue(tsp00_Addr    pszLine,
   -----------------------------------------------------------------------------
 */
 tsp00_Int4
-ls18GetUnformattedValueEx(char          *pString,
-                          tsp00_Uint4    lStrLen,
-                          tls00_Pattern *pPattern,
-                          int            lCount,
-                          tsp00_Uint4    lPosToRead[],
-                          tls00_CodeType CodeType,
-                          tls00_String   pValue[],
-                          int           &lMissingIndex);
+ls18GetCSVData(char          *pString,
+               tsp00_Uint4    lStrLen,
+               tls00_Pattern *pPattern,
+               int            lCount,
+               tsp00_Uint4    lPosToRead[],
+               tls00_CodeType CodeType,
+               tls00_String   pValue[],
+               int           &lMissingIndex);
 
 
 /*!
