@@ -7,15 +7,15 @@
  
   responsible:  SteffenS and d033893
 
-  special area: SAP DB LOADER
+  special area: Loader
  
   description:  dataload
                 
-  version:      7.5.
+  version:      7.6.
 
   -----------------------------------------------------------------------------
  
-  copyright:    (c) 1997-2004 SAP AG-2004
+  Copyright (c) 1997-2004 SAP AG-2004
  
   -----------------------------------------------------------------------------
 
@@ -80,7 +80,7 @@ static char const *szDuplicateString[4] = {" ",
 
 /*!
   -----------------------------------------------------------------------------
-  function:     ls20ExecDataload
+  function:     ls20ExecColumnLoad
   -----------------------------------------------------------------------------
   description:  dataload main function
 
@@ -95,22 +95,21 @@ static char const *szDuplicateString[4] = {" ",
   -----------------------------------------------------------------------------
 */
 int
-ls20ExecDataload(tls00_DatLoadCmd *DloadCmd, tls00_DBInfo * DBInfo, tsp00_Addr ErrText);
+ls20ExecColumnLoad( MetaDataDef *pMetaDataDef, tls00_DBInfo * DBInfo, tsp00_Addr ErrText);
 
 
 /*
   -----------------------------------------------------------------------------
-  function:     ls20LoadTableRows
+  function:     ls20LoadTableColumns
   -----------------------------------------------------------------------------
   description:  Executes  TABLELOAD TABLE RECORDS = 
                           DATALOAD  TABLE RECORDS .
 
   arguments:    pDBInfo     [IN]  - structure that contains all info on the
                                     running session and the database connected to.
-                pDatLoadCmd [IN]  - dataload command structure
                 pDatLoadRec [IN]  - structure, holding information for executing
                                     DATALOAD for a single table
-                pszErrText [OUT]  -  error text
+                ErrText    [OUT]  -  error text
 
   returnvalue:  errOK_els00       - no error
                 errors from loading single table
@@ -120,10 +119,9 @@ ls20ExecDataload(tls00_DatLoadCmd *DloadCmd, tls00_DBInfo * DBInfo, tsp00_Addr E
   -----------------------------------------------------------------------------
 */
 tsp00_Int4
-ls20LoadTableRows(MetaDataDef      *&pMetaDataDef,
-                  tls00_DatLoadCmd *pDatLoadCmd,
-                  tls00_DloadRec   *pDatLoadRec,
-                  tsp00_Addr        ErrText) ;
+ls20LoadTableColumns(MetaDataDef       *&pMetaDataDef,
+                     tls00_DloadRec    *pDatLoadRec,
+                     tsp00_Addr        ErrText) ;
 
 /*
   -----------------------------------------------------------------------------
@@ -131,11 +129,11 @@ ls20LoadTableRows(MetaDataDef      *&pMetaDataDef,
   -----------------------------------------------------------------------------
 */
 tsp00_Int4
-ls20ReadMetaHeader( tls00_DBInfo      *pDBInfo,
-                    tls00_VFile       &VFile,
-                    tls00_DatLoadCmd  *pDatLoadCmd,
-                    tls00_DloadRec    *pDatLoadRec,
-                    tsp00_Addr         ErrText);
+ls20ReadMetaHeader( tls00_DBInfo             *pDBInfo,
+                    tls00_VFile              &VFile,
+                    tls00_TransformationCmd  *pTransformationCmd,
+                    tls00_DloadRec           *pDatLoadRec,
+                    tsp00_Addr               ErrText);
 
 /*
   -----------------------------------------------------------------------------
@@ -144,9 +142,9 @@ ls20ReadMetaHeader( tls00_DBInfo      *pDBInfo,
 */
 tsp00_Int4
 ls20ReadMetaTrailer( tls00_DBInfo              *pDBInfo,
-                    TransformationDef      *&pTransformationDef,
+                     TransformationDef         *&pTransformationDef,
                      tls00_VFile               &VFile,
-                     tls00_DatLoadCmd          *pDatLoadCmd,
+                     tls00_TransformationCmd   *pTransformationCmd,
                      tls00_DloadRec            *pDatLoadRec,
                      Tools_DynamicUTF8String   &command,
                      tsp00_Addr                ErrText );
@@ -351,7 +349,7 @@ ls20ReadData(tls00_VFile*      pInfile,
              tls00_MetaData*   pMeta,
              tls00_Buffer2*    pBuffer,
              SAPDB_Bool*       pbTabLoaded,
-             SAPDB_Bool        bFirstRecord,
+			 SAPDB_Bool        bFirstRecord,
              tsp00_Addr        pszErrText);
 
 
@@ -392,4 +390,4 @@ ls20BuildRecord(tls00_DataSource       *pDataSource,
   -----------------------------------------------------------------------------
 */
 
-#endif /*HLS20*/
\ No newline at end of file
+#endif /*HLS20*/
