@@ -1,25 +1,16 @@
 /*! 
   -----------------------------------------------------------------------------
- 
   module: hls30.h
- 
   -----------------------------------------------------------------------------
  
   responsible:  SteffenS and d033893
 
-  special area: SAP DB LOADER
+  special area: Loader
  
   description:  Functions for Virtual File access
                 
-  version:      7.4.
-
+  version:      7.6.
   -----------------------------------------------------------------------------
- 
-  copyright:    (c) 1997-2004 SAP AG-2003
- 
-  -----------------------------------------------------------------------------
-
-
 
     ========== licence begin  GPL
     Copyright (c) 1997-2004 SAP AG
@@ -38,9 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 
 #ifndef HLS30
@@ -539,6 +527,106 @@ tsp00_Int4
 ls30VFMoveToStartPosition(tls00_VFile             &VFile,
                           const tls00_LargeNumber *plStartPos,
                           tsp00_Addr               pszErrText);
+
+/*!
+  -----------------------------------------------------------------------------
+   function:     ls30Detect_BinaryFile_Encoding
+  -----------------------------------------------------------------------------
+
+  description:  Detects encoding in binary files. Mainly used for LONG data files.
+                If the file encoding is set upon calling it is compared to
+                the real encoding of the file. In case of a mismatch the function
+                returns an error.
+
+  arguments:    Vfile      [IN]   - virtual file structure
+                pszErrText [OUT]  - error text
+
+  returnvalue:  errOK_els00                - in case of NO errors
+                errFileWrongEncoding_els98 - specified and 'real' encoding don't match
+  -----------------------------------------------------------------------------
+*/
+tsp00_Int4
+ls30Detect_BinaryFile_Encoding(tls00_VFile& VFile, tsp00_Addr pszErrText);
+
+/*!
+  -----------------------------------------------------------------------------
+   function:     ls30Mark_BinaryFile_Encoding
+  -----------------------------------------------------------------------------
+
+  description:  Marks a binary file as encoded by writing magic bytes for UTF8/UCS2
+                encoding.
+
+  arguments:    Vfile      [IN]   - virtual file structure
+                pszErrText [OUT]  - error text
+
+  returnvalue:  errOK_els00         - in case of NO errors
+                errFileWrite_els98  - error writing magic bytes
+  -----------------------------------------------------------------------------
+*/
+tsp00_Int4
+ls30Mark_BinaryFile_Encoding(tls00_VFile& VFile, tsp00_Addr pszErrText);
+
+
+/*!
+  -----------------------------------------------------------------------------
+   function:     ls30VFEmptyMagicBytesCache
+  -----------------------------------------------------------------------------
+
+  description:  Sets Bytes in magic bytes 'cache' in structure tls00_VFile
+                to binary 0.
+                (Removes UNICODE marker)
+
+  arguments:    Vfile      [IN]   - virtual file structure
+
+  returnvalue:  None
+  -----------------------------------------------------------------------------
+*/
+inline void
+ls30VFEmptyMagicBytesCache(tls00_VFile* pFile, bool bAll = false)
+{
+    if (true == bAll)
+    {
+        pFile->vfMagicBytesCache_ls00.lBufPos_ls00 = pFile->vfMagicBytesCache_ls00.lBufLen_ls00;
+    }
+    else
+    {
+        pFile->vfMagicBytesCache_ls00.lBufPos_ls00 =
+            pEncodings_ls00[pFile->vfFileEncoding_ls00]->charTable->indicator.byteCount;
+    }
+}
+
+/*!
+  -----------------------------------------------------------------------------
+   function:     ls30VFIsMagicBytesCacheEmpty
+  -----------------------------------------------------------------------------
+
+  description:  Detects if magic bytes 'cache' in structure tls00_VFile is filled.
+
+  arguments:    Vfile      [IN]   - virtual file structure
+
+  returnvalue:  None
+  -----------------------------------------------------------------------------
+*/
+inline bool
+ls30VFIsMagicBytesCacheEmpty(tls00_VFile* pFile)
+{
+    return (pFile->vfMagicBytesCache_ls00.lBufPos_ls00 == pFile->vfMagicBytesCache_ls00.lBufLen_ls00);
+}
+
+inline void
+ls30VFGetMagicBytes(unsigned char* pszBuffer, tls00_VFile *pFile, tsp00_Int4 &lCount)
+{
+    tsp00_Int4 _avail = pFile->vfMagicBytesCache_ls00.lBufLen_ls00 - pFile->vfMagicBytesCache_ls00.lBufPos_ls00;
+    if (lCount > _avail)
+    {
+        lCount = _avail;
+    }
+    for (tsp00_Int4 i = 0; i < lCount; ++i, ++pFile->vfMagicBytesCache_ls00.lBufPos_ls00)
+    {
+        pszBuffer[i] = REINTERPRET_CAST(unsigned char*, pFile->vfMagicBytesCache_ls00.pBufPtr_ls00)[pFile->vfMagicBytesCache_ls00.lBufPos_ls00];
+    }
+}
+
 /*!  
   -----------------------------------------------------------------------------
   EndChapter: Global functions
