@@ -10,9 +10,9 @@
 
   responsible:  d033893 and SteffenS
 
-  special area: odbc SAP DB LOADER
+  special area: Loader
   description:  
-  version:      7.5.
+  version:      7.6.
 
   see also:
 
@@ -51,21 +51,21 @@
 #include "SAPDB/SQLStudio/OAL/StudioOAL_WResult.hpp"  // Unicode Version using W Version of ODBC calls
 // Table
 #include "SQLStudio/TableDefinition/StudioTD_TableDef.h"
-// GUID
+// GUID_
 #include "SAPDB/RunTime/RTE_UniqueId.h"
 
 // forward declarations
 
 class ResultSet          ;
 
-class GUIDGenerator           ;
+class GUID_           ;
 
 // classes derived from ResultSet
 class MetaDataItem        ;
 class TransformationDef   ;
+class RelationalDef       ;
 class CatalogLoadDef      ;
-class UserModeDef         ;
-class CURUserDef          ;
+class UserConnectionDef          ;
 class UserDef             ;
 class RoleDef             ;
 class TableDef            ;
@@ -80,7 +80,7 @@ class IndexColumnDef      ;
 class ColumnTypeDef       ;
 class SequenceValueDef    ;
 class SequenceDef         ;
-class sapdb_Error             ;
+class ErrorHndl             ;
 
 typedef enum 
 {
@@ -105,6 +105,8 @@ typedef struct definfo_ls31
 static RTE_UniqueId  hIdManagement;
 static SAPDB_Bool    idManagementInitialized = SAPDB_FALSE;
 
+#define FILE_EXTENSION_SIZE_LS31     4
+
 #define ID_INITIAL_STRING            "000000000000000000000000000000000000000000000000"
 #define ID_INITIAL                   "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
 
@@ -118,16 +120,16 @@ typedef char                         GUID_Id[GUID_MAX_ID_LEN];
 typedef char                         GUID_IdString[GUID_MAX_ID_STRING_LEN+1];
 
 /*===========================================================================*
- *    class GUIDGenerator
+ *    class GUID_
  *===========================================================================*/
-class GUIDGenerator 
+class GUID_ 
 {
 protected:
-    GUIDGenerator();
+    GUID_();
 public:
-    static GUIDGenerator *Instance();
+    static GUID_ *Instance();
 
-    ~GUIDGenerator();
+    ~GUID_();
     SAPDB_Bool Id_Init               ( );
     SAPDB_Bool IdGetNext             ( GUID_Id id );
     void       IdAsString            ( GUID_Id  id , GUID_IdString idString );
@@ -139,7 +141,7 @@ public:
     SAPDB_Bool IdStringsAreIdentical ( GUID_IdString idString1, GUID_IdString idString2 );
 
 private:
-    static GUIDGenerator  *_guidinstance;
+    static GUID_  *_guidinstance;
 
 };
 /*===========================================================================*
@@ -148,107 +150,34 @@ private:
 class MetaDataDef 
 {
 public:
-     MetaDataDef( tls00_DBInfo * pDBInfo );
+	 MetaDataDef( tls00_TransformationCmd *pTransformationCmd , tls00_DBInfo * pDBInfo  );
     ~MetaDataDef();
 
+	SAPDB_Bool       initResultSets      ( tls00_TransformationCmd *pTransformationCmd , tls00_DBInfo * pDBInfo );
 
-    // Catalog
-    void              setSqlMode         ( tpr00_SQLModeEnum catSqlMode ){ m_catSqlMode = catSqlMode;}
-    tpr00_SQLModeEnum getSqlMode         (){ return m_catSqlMode;}
-
-    void              setExtrMode        ( tls00_ExtrMode   catextrMode ){ m_catExtrMode = catextrMode;}
-    SAPDB_Int4        getExtrMode        (){ return m_catExtrMode;}
-
-    void              setPartSpec        ( tls00_PartSpec & catpartSpec ){m_catPartSpec = catpartSpec;}
-    tls00_PartSpec    getPartSpec        (){ return m_catPartSpec ;}
-
-    void              setMaxErrorCount   (SAPDB_Int4 lMaxErrorCount){m_catMaxAdmissibleErrors = lMaxErrorCount;}
-    SAPDB_Int4        getMaxErrorCount   (){return m_catMaxAdmissibleErrors;}
-
-    void              setTransMode       ( tls31_TransMode transmode ){m_catTransMode = transmode;}
-    tls31_TransMode   getTransMode       (){ return m_catTransMode ;}
+    // Transformation and Relational Model
+    TransformationDef * getTransformationModel () { return m_pTransformationDef;}
+	RelationalDef     * getRelationalModel     () { return m_pRelationalDef;}
 
     // ConnectionSet
-    SAPDB_Bool       initResultSets       ( tls00_DBInfo *pDBInfo );
-    
     void             setConnection        ( tls00_DBInfo *pDBInfo ) { m_pDBInfo   = pDBInfo; }
     tls00_DBInfo*    getConnection        (){ return m_pDBInfo; }
-
     SAPDB_Bool       setODBCDriver        ( tls00_DBInfo *pDBSession     );
     StudioOAL_String getOdbcDriver        (){ return m_odbcDriver    ; }
     // Authorization
     void             setCurrentUser       ( StudioOAL_String& user );
     StudioOAL_String getCurrentUser       ();
-    
-    sapdb_Error *    getError             () { return m_pError; }
-
+    // Error Handling
+	void             setMaxErrorCount     (SAPDB_Int4 lMaxErrorCount){m_catMaxAdmissibleErrors = lMaxErrorCount;}
+    SAPDB_Int4       getMaxErrorCount     (){return m_catMaxAdmissibleErrors;}
+    ErrorHndl *      getError             () { return m_pError; }
+	ErrorHndl      * m_pError;
     // Medium
     void            setMedium             ( tls00_FileInfo  &FileSession ){ m_FileSession = FileSession ;}
     tls00_FileInfo  getMedium             (){ return m_FileSession ;}
     SAPDB_Bool      readLine              ( StudioOAL_String &record  );
     SAPDB_Bool      writeLine             ( StudioOAL_String&  record  );
 
-
-    // Transformation Model
-    TransformationDef    * getTransformationModel  () { return m_pTransformationDef;}
-
-    CatalogLoadDef       * getCatalogLoadDef       () { return m_pCatalogLoadDef;}
-    UserModeDef          * getUserModeDef          () { return m_pUserModeDef;}
-    CURUserDef           * getCURUserDef           () { return m_pCURUserDef;}
-
-    // Relational Model
-    UserDef              * getUserDef              () { return m_pUserDef;}
-    RoleDef              * getRoleDef              () { return m_pRoleDef;}
-    TableDef             * getTableDef             () { return m_pTableDef;}
-    TriggerDef           * getTriggerDef           () { return m_pTriggerDef;}
-    ViewDef              * getViewDef              () { return m_pViewDef;}
-    SynonymDef           * getSynonymDef           () { return m_pSynonymDef;}
-    ColumnTypeDef        * getColumnTypeDef        () { return m_pColumnTypeDef;}
-    IndexDef             * getIndexDef             () { return m_pIndexDef;}
-    SequenceDef          * getSequenceDef          () { return m_pSequenceDef;}
-    DBProcedureDef       * getDBProcedureDef       () { return m_pDBProcedureDef;}
-    GrantDBProcedureDef  * getGrantDBProcedureDef  () { return m_pGrantDBProcedureDef;}
-    
-    // Extract Catalog Commands
-    SAPDB_Int4 ExtrCatalogCmd           ( StudioOAL_String user );
-    SAPDB_Int4 ExtrSchemaCmd            ( StudioOAL_String user );
-    SAPDB_Int4 ExtrTableCmd             ( tls00_PartSpec  catpartSpec );
-
-    // Load Catalog Commands
-    SAPDB_Int4 LoadCatalogCmd           ( StudioOAL_String user );
-    SAPDB_Int4 LoadSchemaCmd            ( StudioOAL_String user );
-    SAPDB_Int4 LoadTableCmd             ( tls00_PartSpec  catpartSpec );
-    
-    SAPDB_Bool writeCatalogHeader       ( );    
-
-    // Extract Authorization Objects
-    SAPDB_Int4 schemaExtrCmd            ( StudioOAL_String user  );
-    SAPDB_Int4 userExtrCmd              ( tls00_PartSpec catpartSpec );
-    SAPDB_Int4 roleExtrCmd              ( tls00_PartSpec catpartSpec );
-        
-    // Extract Catalog Objects 
-    SAPDB_Int4 connectionExtrCmd        ( );
-    SAPDB_Int4 columntypesetExtrCmd     ( );
-    
-     // Relational Model
-    SAPDB_Int4 tabletypeExtrCmdDirect   ( tls00_PartSpec  catpartSpec );
-    
-    SAPDB_Int4 tableExtrCmdExecDirect   ( StudioOAL_String& owner, StudioOAL_String& tablename );
-    SAPDB_Int4 tableExtrCmdExec         ( StudioOAL_String& owner, StudioOAL_String& tablename );
-    SAPDB_Int4 viewExtrCmdExecDirect    ( StudioOAL_String& owner, StudioOAL_String& viewname  );
-    SAPDB_Int4 viewExtrCmdExec          ( StudioOAL_String  owner, StudioOAL_String  viewname  );
-    SAPDB_Int4 synonymExtrCmdExecDirect ( StudioOAL_String  owner, StudioOAL_String  tablename );
-    SAPDB_Int4 synonymExtrCmdExec       ( StudioOAL_String  owner, StudioOAL_String  tablename );
-    SAPDB_Int4 indexExtrCmdExecDirect   ( StudioOAL_String& owner, StudioOAL_String& tablename );
-    SAPDB_Int4 indexExtrCmdExec         ( StudioOAL_String  owner, StudioOAL_String  tablename );
-    SAPDB_Int4 triggerExtrCmdExecDirect ( StudioOAL_String& owner, StudioOAL_String& tablename );
-    SAPDB_Int4 triggerExtrCmdExec       ( StudioOAL_String  owner, StudioOAL_String  tablename );
-
-    SAPDB_Int4 sequenceExtrCmdDirect    ( tls00_PartSpec  catpartSpec );
-    SAPDB_Int4 dbprocedureExtrCmdDirect ( tls00_PartSpec  catpartSpec );
-
-    SAPDB_Bool writeCatalogFooter       ( );
-    
 private:
     // database connectionset for metadata
     StudioOAL_String      m_odbcDriver;
@@ -256,32 +185,12 @@ private:
     // database connection for data
     tls00_DBInfo         *m_pDBInfo;
 
-    // transformation model 
+    // Transformation and Relational Model
     TransformationDef    *m_pTransformationDef;
+	RelationalDef        *m_pRelationalDef;
+    UserConnectionDef    *m_pUserConnectionDef;
+	StudioOAL_String      m_currentUser;
 
-    // relational model
-    CatalogLoadDef       *m_pCatalogLoadDef;
-    UserModeDef          *m_pUserModeDef;
-    CURUserDef           *m_pCURUserDef;
-
-    UserDef              *m_pUserDef;
-    RoleDef              *m_pRoleDef;
-    TableDef             *m_pTableDef;
-    TriggerDef           *m_pTriggerDef;
-    ViewDef              *m_pViewDef;
-    SynonymDef           *m_pSynonymDef;
-    ColumnTypeDef        *m_pColumnTypeDef;
-    IndexDef             *m_pIndexDef;
-    SequenceDef          *m_pSequenceDef;
-    DBProcedureDef       *m_pDBProcedureDef;
-    GrantDBProcedureDef  *m_pGrantDBProcedureDef;
-        
-    SAPDB_Int4            m_nSchemaCount;
-    SAPDB_Int4            m_nTableCount;
-    SAPDB_Int4            m_nViewCount;
-    SAPDB_Int4            m_nSynonymCount;
-    
-    sapdb_Error          *m_pError;
     SAPDB_Char            m_ReadWriteErrText[MAX_REPLY_LENGTH_LS00];
 
     SAPDB_Char            m_ReadBuffer[LINELENGTH_MXSP00];
@@ -289,13 +198,21 @@ private:
     // stream connection
     tls00_FileInfo        m_FileSession;
 
-    tls31_TransMode       m_catTransMode;
-    tls00_ExtrMode        m_catExtrMode;
-    tls00_PartSpec        m_catPartSpec;
-    tpr00_SQLModeEnum     m_catSqlMode;
-    SAPDB_Int4            m_catMaxAdmissibleErrors;
+public: // ToDo move to Transformation and Relational Model
+    void              setExtrMode        ( tls00_ExtrMode   catextrMode ){ m_catExtrMode = catextrMode;}
+    SAPDB_Int4        getExtrMode        (){ return m_catExtrMode;}
+    void              setPartSpec        ( tls00_PartSpec & catpartSpec ){m_catPartSpec = catpartSpec;}
+    tls00_PartSpec    getPartSpec        (){ return m_catPartSpec ;}
+    void              setTransMode       ( tls31_TransMode transmode ){m_catTransMode = transmode;}
+    tls31_TransMode   getTransMode       (){ return m_catTransMode ;}
+    void              setSqlMode         ( tpr00_SQLModeEnum catSqlMode ){ m_catSqlMode = catSqlMode;}
+    tpr00_SQLModeEnum getSqlMode         (){ return m_catSqlMode;}
+    tls31_TransMode   m_catTransMode;
+    tls00_ExtrMode    m_catExtrMode;
+    tls00_PartSpec    m_catPartSpec;
+    tpr00_SQLModeEnum m_catSqlMode;
+    SAPDB_Int4        m_catMaxAdmissibleErrors;
 
-    StudioOAL_String      m_currentUser;
 };
 
 /*===========================================================================*
@@ -346,53 +263,57 @@ private:
 class TransformationDef : public MetaDataItem
 {
 public:
-   TransformationDef( sapdbwa_DBC & odbcConnection , MetaDataDef * pMetaData );
+   TransformationDef( sapdbwa_DBC & odbcConnection , MetaDataDef * pMetaData,  tls00_TransformationCmd *pTransformationCmd );
    TransformationDef( sapdbwa_DBC & odbcConnection  );
   ~TransformationDef();
 
    // 1.Direct SQL Statement
-   SAPDB_Bool initResultSet ( sapdbwa_DBC & odbcConnection );
+   SAPDB_Int4 LoadTransformationPackage  ( MetaDataDef *&pMetaDataDef, tsp00_Addr ErrText);
+   SAPDB_Bool initResultSet              ( sapdbwa_DBC & odbcConnection );
+   SAPDB_Bool LoadTransformationTables   ();
+   SAPDB_Bool ExistPackage               ( StudioOAL_String packageguid );
+   SAPDB_Bool LoadPackage                ( StudioOAL_String& packagestmt );
+   SAPDB_Int4 UnloadTransformationPackage( MetaDataDef *&pMetaDataDef, tsp00_Addr ErrText);
 
-   SAPDB_Bool readResult             ( StudioOAL_String *record );
-   SAPDB_Bool writeResult            ( StudioOAL_String  record );
+   SAPDB_Bool readResult                 ( StudioOAL_String *record );
+   SAPDB_Bool writeResult                ( StudioOAL_String  record );
 
-   SAPDB_Bool initTransformationStep ( StudioOAL_String *record );
-   SAPDB_Int4 initTransformation     ();
-   void       freeTransformation     ( tls00_ScanResult Type );
-   SAPDB_Int4 switchUser             ( StudioOAL_String oldUser , StudioOAL_String newUser, tsp00_Addr ErrText );
+   tls00_TransformationCmd *           getTransformationCmd() ;
+   SAPDB_Bool initTransformationStep   ( StudioOAL_String *record );
+   void       setTransformationStmt    ( StudioOAL_String  record ) { m_statement = record;}
+   SAPDB_Int4 initTransformationCmd    ();
+   void       freeTransformationCmd    ( tls00_ScanResult Type );
+   void       freeTransformationCmd    ( );
+   void       genLoaderStatement       ();
+   void       genInverseLoaderStatement( tls00_TransformationCmd *& pTransformationCmd, tls00_DextractRec  *pDatExtrRec );
+   SAPDB_Int4 switchUser               ( StudioOAL_String oldUser , StudioOAL_String newUser, tsp00_Addr ErrText );
    
-   SAPDB_Int4 getObjectCount()      { return m_nObjectCount; }
+   SAPDB_Int4 getObjectCount()         { return  m_nObjectCount; }
 
+   SAPDB_Bool update                ( StudioOAL_String statement );
    SAPDB_Bool updateTaskStatistic   ();
    SAPDB_Bool updateDatTask         ( StudioOAL_String *parameters );
    SAPDB_Bool updateBegin           ( StudioOAL_String  owner, StudioOAL_String tablename );
-   SAPDB_Bool updatePages           ( StudioOAL_String  owner, StudioOAL_String tablename );
-   SAPDB_Bool updateRecords         ( StudioOAL_String  owner, StudioOAL_String tablename );
    SAPDB_Bool updateExclude         ( StudioOAL_String  owner, StudioOAL_String tablename );
    SAPDB_Bool updateRestart         ( StudioOAL_String  owner, StudioOAL_String tablename );
    SAPDB_Bool updateDatStream       ( StudioOAL_String *parameters );
    SAPDB_Bool updateHeader          ( StudioOAL_String *parameters );
    SAPDB_Bool updateTrailer         ( StudioOAL_String *parameters );
-
-   SAPDB_Bool updateEnd             ( StudioOAL_String&  owner, StudioOAL_String& tablename );
-   SAPDB_Bool updatePerformance     ( StudioOAL_String   owner, StudioOAL_String tablename );
-   
+   SAPDB_Bool updateEnd             ( StudioOAL_String& owner, StudioOAL_String& tablename );
+   SAPDB_Bool updatePerformance     ( StudioOAL_String  owner, StudioOAL_String tablename );
    SAPDB_Bool updateExtrData        ( StudioOAL_String *parameters );
    SAPDB_Bool updateLoadData        ( StudioOAL_String *parameters );
-   
    SAPDB_Bool updateCatTask         ( StudioOAL_String *parameters );
    SAPDB_Bool updateCat             ( StudioOAL_String *parameters );
    SAPDB_Bool updateCatStream       ( StudioOAL_String *parameters );
-
-   SAPDB_Bool unload         ( sapdbwa_DBC & odbcConnection );
-
-   void              genLoaderStatement       ();
-   void              genInverseLoaderStatement( tls00_DatExtrCmd *& pDatExtrCmd, tls00_DextractRec  *pDatExtrRec );
-
-   // transformation statement and command of loader
-   StudioOAL_String  m_statement;
-   StudioOAL_String  m_inverseStatement;
-   tls00_Command    *m_pCommand;
+   SAPDB_Bool unload                ( sapdbwa_DBC & odbcConnection );
+   
+   // transformation statement and transformation command
+   tls00_TransformationCmd *m_pTransformationCmd;
+   tls00_Command           *m_pLoaderCmd;
+   StudioOAL_String         m_statement;
+   StudioOAL_String         m_inverseStatement;
+   
 
    // transformationpackage table columns for catalog
    SAPDB_Int4        m_nTaskID;
@@ -454,9 +375,88 @@ public:
    ResultSet        *m_pUpdateEnd;
 #endif // ODBC_PREPARED
 
+};
+
+/*===========================================================================*
+ *    class RelationalDef 
+ *===========================================================================*/
+class RelationalDef : public MetaDataItem
+{
+public:
+   RelationalDef( sapdbwa_DBC & odbcConnection , MetaDataDef * pMetaData );
+   RelationalDef( sapdbwa_DBC & odbcConnection  );
+  ~RelationalDef();
+
+   // 1.Direct SQL Statement
+   SAPDB_Bool initResultSet ( sapdbwa_DBC & odbcConnection );
+
+   UserDef         * getUserDef              () { return m_pUserDef;}
+   RoleDef         * getRoleDef              () { return m_pRoleDef;}
+   TableDef        * getTableDef             () { return m_pTableDef;}
+   TriggerDef      * getTriggerDef           () { return m_pTriggerDef;}
+   ViewDef         * getViewDef              () { return m_pViewDef;}
+   SynonymDef      * getSynonymDef           () { return m_pSynonymDef;}
+   ColumnTypeDef   * getColumnTypeDef        () { return m_pColumnTypeDef;}
+   IndexDef        * getIndexDef             () { return m_pIndexDef;}
+   SequenceDef     * getSequenceDef          () { return m_pSequenceDef;}
+   DBProcedureDef  * getDBProcedureDef       () { return m_pDBProcedureDef;}
+
+   // Extract Catalog Commands
+   SAPDB_Int4 ExtrCatalogCmd           ( StudioOAL_String user, tsp00_Addr ErrText );
+   SAPDB_Int4 ExtrSchemaCmd            ( StudioOAL_String user );
+   SAPDB_Int4 ExtrTableCmd             ( tls00_PartSpec  catpartSpec );
+
+   // Load Catalog Commands
+   SAPDB_Int4 LoadCatalogCmd           ( StudioOAL_String user );
+   SAPDB_Int4 LoadSchemaCmd            ( StudioOAL_String user );
+   SAPDB_Int4 LoadTableCmd             ( tls00_PartSpec   catpartSpec, tsp00_Addr ErrText );
+    
+   // Extract Authorization Objects
+   SAPDB_Int4 schemaExtrCmd            ( StudioOAL_String user  );
+   SAPDB_Int4 userExtrCmd              ( tls00_PartSpec catpartSpec );
+   SAPDB_Int4 roleExtrCmd              ( tls00_PartSpec catpartSpec );
+        
+   // Extract Catalog Objects 
+   SAPDB_Int4 connectionExtrCmd        ( );
+   SAPDB_Int4 columntypesetExtrCmd     ( );
+    
+    // Relational Model
+   SAPDB_Int4 tabletypeExtrCmdDirect   ( tls00_PartSpec  catpartSpec );
+    
+   SAPDB_Int4 tableExtrCmdExecDirect   ( StudioOAL_String& owner, StudioOAL_String& tablename );
+   SAPDB_Int4 tableExtrCmdExec         ( StudioOAL_String& owner, StudioOAL_String& tablename );
+   SAPDB_Int4 viewExtrCmdExecDirect    ( StudioOAL_String& owner, StudioOAL_String& viewname  );
+   SAPDB_Int4 viewExtrCmdExec          ( StudioOAL_String  owner, StudioOAL_String  viewname  );
+   SAPDB_Int4 synonymExtrCmdExecDirect ( StudioOAL_String  owner, StudioOAL_String  tablename );
+   SAPDB_Int4 synonymExtrCmdExec       ( StudioOAL_String  owner, StudioOAL_String  tablename );
+   SAPDB_Int4 indexExtrCmdExecDirect   ( StudioOAL_String& owner, StudioOAL_String& tablename );
+   SAPDB_Int4 indexExtrCmdExec         ( StudioOAL_String  owner, StudioOAL_String  tablename );
+   SAPDB_Int4 triggerExtrCmdExecDirect ( StudioOAL_String& owner, StudioOAL_String& tablename );
+   SAPDB_Int4 triggerExtrCmdExec       ( StudioOAL_String  owner, StudioOAL_String  tablename );
+
+   SAPDB_Int4 sequenceExtrCmdDirect    ( tls00_PartSpec  catpartSpec );
+   SAPDB_Int4 dbprocedureExtrCmdDirect ( tls00_PartSpec  catpartSpec );
+
+   SAPDB_Bool writeCatalogHeader       ( );    
+   SAPDB_Bool writeCatalogFooter       ( );
+
 private:
-    SAPDB_Bool CreateLoaderSystemObjects();
-    SAPDB_Bool LoadTransformationPackage(StudioOAL_String& SaveCatalogStmt);
+   CatalogLoadDef       *m_pCatalogLoadDef ;
+   UserDef              *m_pUserDef  ;
+   RoleDef              *m_pRoleDef  ;
+   TableDef             *m_pTableDef;
+   TriggerDef           *m_pTriggerDef;
+   ViewDef              *m_pViewDef;
+   SynonymDef           *m_pSynonymDef;
+   ColumnTypeDef        *m_pColumnTypeDef;
+   IndexDef             *m_pIndexDef;
+   SequenceDef          *m_pSequenceDef;
+   DBProcedureDef       *m_pDBProcedureDef;
+
+   SAPDB_Int4            m_nSchemaCount;
+   SAPDB_Int4            m_nTableCount;
+   SAPDB_Int4            m_nViewCount;
+   SAPDB_Int4            m_nSynonymCount;
 };
 
 /*===========================================================================*
@@ -473,6 +473,7 @@ public:
    StudioOAL_String getSQLMode ( );
 
    SAPDB_Bool initResultSet    ( sapdbwa_DBC & odbcConnection );
+   SAPDB_Bool execute_SQL      ( StudioOAL_String  statement, tsp00_Addr ErrText);
    SAPDB_Bool readResult       ( StudioOAL_String *record );
    SAPDB_Bool writeResult      ( StudioOAL_String  record );
 
@@ -530,14 +531,14 @@ private:
 
 };
 /*===========================================================================*
- *    class CURUserDef 
+ *    class UserConnectionDef 
  *===========================================================================*/
-class CURUserDef :  public MetaDataItem
+class UserConnectionDef :  public MetaDataItem
 {
 public:
-   CURUserDef( sapdbwa_DBC & odbcConnection, MetaDataDef * pMetaData );
-   CURUserDef( sapdbwa_DBC & odbcConnection );
-  ~CURUserDef();
+   UserConnectionDef( sapdbwa_DBC & odbcConnection, MetaDataDef * pMetaData );
+   UserConnectionDef( sapdbwa_DBC & odbcConnection );
+  ~UserConnectionDef();
 
    SAPDB_Bool initResultSet ( sapdbwa_DBC & odbcConnection );
    SAPDB_Bool writeResult   ( StudioOAL_String  record      );
@@ -827,7 +828,7 @@ public:
    StudioOAL_String        genCreateXML();
 
    SAPDB_Int4 grantDBProcedureExtrCmd  ( StudioOAL_String dbprocedureName );
-   GrantDBProcedureDef * m_pGrantDBProcedureDef;
+   GrantDBProcedureDef  * m_pGrantDBProcedureDef;
  
 private:
    StudioOAL_String       m_dbprocedureName;
@@ -938,13 +939,13 @@ private:
 };
 
 /*===========================================================================*
- *    class sapdb_Error
+ *    class ErrorHndl
  *===========================================================================*/
-class sapdb_Error
+class ErrorHndl
 {
 public:
-     sapdb_Error( );
-    ~sapdb_Error( );
+     ErrorHndl( );
+    ~ErrorHndl( );
 
     inline void             setErrorText ( StudioOAL_String errorText ) { m_ErrorText = errorText; }
     inline StudioOAL_String getErrorText ()                             { return m_ErrorText ; }
