@@ -1,8 +1,6 @@
 /*! 
   -----------------------------------------------------------------------------
- 
   module: hls98msg.h
- 
   -----------------------------------------------------------------------------
  
   responsible:  SteffenS and d033893
@@ -11,14 +9,11 @@
  
   description:  Server protocol class
                 
-  version:      7.5.
-
+  version:      7.6.
   -----------------------------------------------------------------------------
 
-
-
     ========== licence begin  GPL
-    Copyright (c) 2000-2004 SAP AG
+    Copyright (c) 1997-2004 SAP AG
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -34,9 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 
 #ifndef HLS98MSG_H
@@ -139,7 +131,8 @@ typedef enum
     
     
     // File handling errors
-    errFileTruncate_els98                       = -25507,
+    errFileWrongEncoding_els98                  = -25508,
+    errFileTruncate_els98,                      //= -25507,
                                                 //= -25506
     errFileErase_els98                          = -25505,
     errFileWrite_els98,                         //= -25504
@@ -260,7 +253,12 @@ typedef enum
     //*
     //*     fehler waehrend des parsens
     //*
-    errLoaderConstraintViolation_els98          = -25244,
+    errWrongIsolationLevel_els98                = -25249,
+    errDatastreamFormatnotSupported,            //= -25248
+    errCatalogOrDatastreamNeeded,               //= -25247
+    errCatalogAndDatastreamNeeded,              //= -25246
+    errPackagestreamNeeded,                     //= -25245
+	errLoaderConstraintViolation_els98,         //= -25244
     errErrorParsingDateTimeMask_els98,          //= -25243
     errDBIdentifierTooLong_els98,               //= -25242
     errWrongFullQualTableNameSpec_els98,        //= -25241
@@ -345,16 +343,18 @@ typedef enum
     //*
     //*     WARNUNGEN_WARNUNGEN_WARNUNGEN_WARNUNGEN_WARNUNGEN_WARNUNGEN_WARNUNGEN_WARNUNGEN
     //*
-    errNILValueTruncated_els98                  = 25601,
+    wrnConfigurationStreamNotSupported_els98    = 25600,
+    errNILValueTruncated_els98,                 //= 25601
     errIntValueOverflow_els98,                  //= 25602
     errLiteralTruncated_els98,                  //= 25603
     errSurplusLongInfo_els98,                   //= 25604
     errNoDatafileHeader_ls98,                   //= 25605
     wrnTL_LongsButDifferentSwap_ls98,           //= 25606
-    wrnUsageIgnored_els98,                      //= 25607
+    wrnUsageIgnoredPercent_els98,               //= 25607
     wrnNoRestartFileFound_els98,                //= 25608
     wrnLoaderConstraintViolation_els98,         //= 25609
     wrnDEforFLCreatesDLStmt_els98,              //= 25610
+    wrnUsageIgnoredRows_els98,                  //= 25611
 
 
     //*
@@ -383,6 +383,9 @@ typedef enum
     msgLastCommInputLine2_els98,                //= 25919     this message is necessary for different handling of paramters
     msgInsertedRejectedRows2_els98,             //= 25920     this message is necessary for different handling of paramters
     msgSequenceRangeExhausted_els98,            //= 25921 
+    msgExecuteSystemCommand_els98,              //= 25922 
+	msgModuleFileName_els98 ,                   //= 25923
+	msgEnvironmentPath_els98,                   //= 25924
 
     
 
@@ -445,24 +448,28 @@ const char * const pszWarningLineHeader = "// W ";
 
 #define MAX_OI_SIZE_LS98 128*1024                // size corresponds to max size of message buffer of DB
 
-/***************** KLASSE RepServMessages **************************************/
+/***************** KLASSE LoaderMessages **************************************/
 
 
-class RepServMessages
+class LoaderMessages
 {
 // Construction/Deconstruction
 protected:
-    RepServMessages(); // Constructor
+    LoaderMessages(); // Constructor
 
 public:
-    ~RepServMessages();
-
+    ~LoaderMessages();
 
 // Attributes
 private:
-    static RepServMessages  *_msgsinstance;
+    static LoaderMessages  *_msgsinstance;
 
-    static _TCHAR           *m_LogFileName;
+	_TCHAR                   m_LoaderDataPath_c[PATH_MXSP00];
+	_TCHAR                   m_LoaderApplPath_c[PATH_MXSP00];
+	_TCHAR                   m_ConfigPath_c[PATH_MXSP00];
+	_TCHAR                   m_LogPath_c[PATH_MXSP00];
+    _TCHAR                 	 m_TransportPath_c[PATH_MXSP00];
+    static _TCHAR           *m_LogFileName_c;
     static tsp00_Int4        m_LogFile;          // File handle
     
     tsp00_ErrTextc           m_ErrorText;        // Interner fehlertext, wenn beim dateibearbeiten etwas schiefgeht
@@ -476,15 +483,16 @@ private:
 
 
 public:
-    static RepServMessages  *Instance();    // This is the instance controlling function !!!
-    static void              ls98SetLogFileName(tsp00_VFilenamec pszFileName);
-    static void              ls98SetLogFileName(tsp00_Pathc pszPath, tsp00_VFilenamec pszFileName);
-
-    
-    int                      ls98InitLogFile(const char* pszDBName = NULL);
-    void                     ls98GetLogFileName    (char *pszLogFileName);
-    void                     ls98GetErrorText      (char *pszErrorText);
-    int                      ls98ChangeLogFileOwner(tsp00_Addr pszUsername, tsp00_C256c & szErrorText);
+    static LoaderMessages  *Instance();   
+    int                     ls98InitLogFile         (const char* pszDBName);
+	void                    ls98InitLoaderPath      ();
+	void                    ls98GetLoaderPath       (char *pszLoaderPath);
+	void                    ls98GetLoaderLogPath    (char *pszLoaderPath);
+	void                    ls98GetLoaderConfigPath (char *pszLoaderPath);
+	static void             ls98SetLogFileName      (tsp00_VFilenamec pszFileName);
+    void                    ls98GetLogFileName      (char *pszLogFileName);
+    void                    ls98GetErrorText        (char *pszErrorText);
+    int                     ls98ChangeLogFileOwner  (tsp00_Addr pszUsername, tsp00_C256c & szErrorText);
         
     /*!
     -----------------------------------------------------------------------------
@@ -612,12 +620,7 @@ private:
     void ls98_GetLayerText(const LayerNumbers Layer);
     int ls98_Write();
     void ls98_Init();
-       
-#if defined(LINUX) && defined(S390X)
-    void ls98_FormatMsg(const ErrorNumbers lMsgNr, bool  bMsgToFileOnly, va_list xarglist);
-#else
-    void ls98_FormatMsg(const ErrorNumbers lMsgNr, bool  bMsgToFileOnly, const va_list arglist);
-#endif
+    void ls98_FormatMsg(const ErrorNumbers lMsgNr, bool  bMsgToFileOnly, va_list arglist);
 };
 
 
