@@ -7,15 +7,15 @@
 
   responsible:  SteffenS and d033893
 
-  special area: SAP DB LOADER
+  special area: Loader
 
   description:  communication with client
 
-  version:      7.5.
+  version:      7.6.
 
   -----------------------------------------------------------------------------
 
-  copyright:    (c) 1997-2004 SAP AG-2003
+  Copyright (c) 1997-2004 SAP AG-2003
 
   -----------------------------------------------------------------------------
 
@@ -107,6 +107,7 @@ void ls01_InitDBInfo(tls00_DBInfo *DBInfo, const char* DBName)
 
     DBInfo->dbiSqlRc_ls00.becomes(commErrOk_esp01);
     DBInfo->dbiEncoding_ls00    = ctAscii_els00;
+    DBInfo->dbiOI_Encoding_ls00 = ctAscii_els00;
     DBInfo->dbiPageSize_ls00    = 0;
     DBInfo->dbiPktCount_ls00    = 2;
     DBInfo->dbiPktSize_ls00     = 0;
@@ -120,24 +121,28 @@ void ls01_InitDBInfo(tls00_DBInfo *DBInfo, const char* DBName)
     DBInfo->dbiPktRcvList_ls00[0]   = NULL;
     DBInfo->dbiPktRcvList_ls00[1]   = NULL;
 
+	DBInfo->dbiLoaderSessionIsoLevel_ls00 = 3;		// This is the predefined default that can be set with SET ISOLATION LEVEL
+
     // This is the kernel parameter DATE_TIME_FORMAT and set during user connect
     DBInfo->dbiDateTimeFormat_ls00  = dtf_normal;
 
     memset(DBInfo->dbiSqlTerm_ls00, BLANK_LS00, sizeof (tsp00_TermId));
     memset(DBInfo->dbiSqlErr_ls00,  BLANK_LS00, sizeof (tsp00_ErrText));
 
-    DBInfo->dbiSession_ls00.siUserKey_ls00      = NULL;
-    DBInfo->dbiSession_ls00.siSQLMode_ls00      = sp1sm_internal;
-    DBInfo->dbiSession_ls00.siIsoLevel_ls00     = 3;
-    DBInfo->dbiSession_ls00.siBackupTool_ls00   = btTypeUnknown_els00;
+    DBInfo->dbiSession_ls00.siUserKey_ls00           = NULL;
+    DBInfo->dbiSession_ls00.siSQLMode_ls00           = sp1sm_internal;
+    DBInfo->dbiSession_ls00.siDBSessionIsoLevel_ls00 = DBInfo->dbiLoaderSessionIsoLevel_ls00;
+    DBInfo->dbiSession_ls00.siBackupTool_ls00        = btTypeUnknown_els00;
+    DBInfo->dbiSession_ls00.siConfigurationType_ls00 = ctExclude_els00;
 
-    DBInfo->dbiSession_ls00.siDBConnection_ls00 = false;
-    DBInfo->dbiSession_ls00.siSqlStat_ls00      = false;
-
-    DBInfo->dbiSession_ls00.siAutoCommit_ls00   = true;
-    DBInfo->dbiSession_ls00.siMessageOn_ls00    = false;
-    DBInfo->dbiSession_ls00.siInstallOn_ls00    = false;
-    DBInfo->dbiSession_ls00.siNolog_ls00        = false;
+    DBInfo->dbiSession_ls00.siDBConnection_ls00    = false;
+    DBInfo->dbiSession_ls00.siSqlStat_ls00         = false;
+    DBInfo->dbiSession_ls00.siAutoCommit_ls00      = true;
+    DBInfo->dbiSession_ls00.siMessageOn_ls00       = false;
+    DBInfo->dbiSession_ls00.siInstallOn_ls00       = false;
+    DBInfo->dbiSession_ls00.siODBCTrace_ls00       = false;
+    DBInfo->dbiSession_ls00.siSystemAttribute_ls00 = false;
+    DBInfo->dbiSession_ls00.siNolog_ls00           = false;
 
     DBInfo->dbiSession_ls00.siDateType_ls00     = dtInternal_els00;
     DBInfo->dbiSession_ls00.siTimeType_ls00     = dtInternal_els00;
@@ -154,6 +159,8 @@ void ls01_InitDBInfo(tls00_DBInfo *DBInfo, const char* DBName)
 
     DBInfo->dbiSession_ls00.siNullSpec_ls00.rawAssign("?                   ");
 
+    DBInfo->dbiSession_ls00.siConfigurationType_ls00 = ctExclude_els00;
+
     // Boolwert fuer TRUE
     DBInfo->dbiSession_ls00.siTrueVal_ls00[0] = '\0';
     strcpy(DBInfo->dbiSession_ls00.siTrueVal_ls00,  "TRUE");
@@ -173,7 +180,7 @@ void ls01_InitDBInfo(tls00_DBInfo *DBInfo, const char* DBName)
     memset(DBInfo->dbiSession_ls00.siUserPwd_ls00.asCharp(), BLANK_LS00, sizeof(tsp00_Namec));
 
     // Codetypes, termcharsets and conversion tables
-    DBInfo->dbiSession_ls00.siCodeType_ls00     = ctAscii_els00;
+    DBInfo->dbiSession_ls00.siCodeType_ls00     = ctUnknown_els00;
 
     // Set default code page for ASCII to UCS2 conversion
     DBInfo->dbiSession_ls00.siCodePage_ls00.name.Init();
@@ -249,7 +256,7 @@ void ctrlservopen(void       **vcontrolData,
                   BOOLEAN     *cancelAddr,
                   BOOLEAN      loggedOn,
                   BOOLEAN      couldLogOn,
-                  int          callerId)
+                  int          callerId )
 {
     ROUTINE_DBG_MEO00 ("ls01CtrlServOpen");
 
@@ -314,9 +321,9 @@ void ctrlservopen(void       **vcontrolData,
     ls01_InitDBInfo(DBInfo, dbname);        // Initializes dbinfo struct and adjusts parameter
 
     // Initialize log file
-    RepServMessages*  MsgInstance = RepServMessages::Instance();
+    LoaderMessages* MsgInstance = LoaderMessages::Instance();
 
-    tsp00_Int4 rc = MsgInstance->ls98InitLogFile(dbname);
+	tsp00_Int4 rc = MsgInstance->ls98InitLogFile(dbname);
 
     memcpy(cname, COMP_NAME_LS00, 64);
     sqlinit(cname, NULL);
