@@ -1,25 +1,16 @@
 /* 
   -----------------------------------------------------------------------------
- 
   module: vls04.cpp
- 
   -----------------------------------------------------------------------------
  
   responsible:  SteffenS
 
-  special area: Replication Manager
+  special area: Loader
  
   description:  SQL-Packet handling
                 
-  version:      7.4.
-
+  version:      7.5.
   -----------------------------------------------------------------------------
- 
-  copyright:    (c) 1997-2004 SAP AG-2001
- 
-  -----------------------------------------------------------------------------
-
-
 
     ========== licence begin  GPL
     Copyright (c) 1997-2004 SAP AG
@@ -38,9 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 
 #include <assert.h>
@@ -341,7 +329,7 @@ int ls04FastInit(tsp1_packet         *SqlPacket,
     *SqlPart = &SqlPacket->sp1_segm().sp1p_part();
 
 
-    if (true == ls00IsDBUnicode(DBInfo))
+    if (true == ls00UseUnicodeOI(DBInfo))
     {
         if ( (sw_full_swapped == DBInfo->dbiOwnSwap_ls00) ||
              (sw_part_swapped == DBInfo->dbiOwnSwap_ls00) )
@@ -600,7 +588,7 @@ ls04RcFromPacket(tsp1_packet *SqlPacket, tsp00_Addr ErrText, tsp00_Int4 &ErrorPo
         if (true == ls04GetPart(SqlPacket, sp1pk_errortext, SqlPart) )
         {
             tsp00_Int4 len = ls04GetPartLength(SqlPart);
-			tsp00_Int4 pos = 0;
+            tsp00_Int4 pos = 0;
 
             // We write everything in utf8
             if ( (csp_unicode_swap == SqlPacket->sp1_header.sp1h_mess_code) || 
@@ -609,21 +597,21 @@ ls04RcFromPacket(tsp1_packet *SqlPacket, tsp00_Addr ErrText, tsp00_Int4 &ErrorPo
                 pos = sp77sprintfUnicode(sp77encodingUTF8, ErrText, lMaxBufLen, SQL_ERRTEXTU_LS04,
                                                                                 STATIC_CAST(int, ErrCode),
                                                                                 len / 2,
-																				SqlPart->sp1p_buf().asCharp());
-				ls05StrTrimRight(ErrText, pos);
+                                                                                SqlPart->sp1p_buf().asCharp());
+                ls05StrTrimRight(ErrText, pos);
 
-				pos = sp77sprintfUnicode(sp77encodingUTF8, ErrText + pos, lMaxBufLen, SQL_ERRPOS_TEXT_LS04,
+                pos = sp77sprintfUnicode(sp77encodingUTF8, ErrText + pos, lMaxBufLen, SQL_ERRPOS_TEXT_LS04,
                                                                                       ErrorPos);
             }
             else if (csp_ascii == SqlPacket->sp1_header.sp1h_mess_code)
             {
                 pos = sp77sprintfUnicode(sp77encodingUTF8, ErrText + pos, lMaxBufLen, SQL_ERRTEXTA_LS04,
                                                                                       STATIC_CAST(int, ErrCode),
-																					  len,
-																					  SqlPart->sp1p_buf().asCharp());
-				ls05StrTrimRight(ErrText, pos);
+                                                                                      len,
+                                                                                      SqlPart->sp1p_buf().asCharp());
+                ls05StrTrimRight(ErrText, pos);
 
-				pos = sp77sprintfUnicode(sp77encodingUTF8, ErrText + pos, lMaxBufLen, SQL_ERRPOS_TEXT_LS04,
+                pos = sp77sprintfUnicode(sp77encodingUTF8, ErrText + pos, lMaxBufLen, SQL_ERRPOS_TEXT_LS04,
                                                                                       ErrorPos);
             }
 
