@@ -1,25 +1,16 @@
 /*! 
   -----------------------------------------------------------------------------
- 
   module: vls05.cpp
- 
   -----------------------------------------------------------------------------
  
-  responsible:  SteffenS and d033893
+  responsible:  SteffenS, d033893
 
-  special area: SAP DB LOADER
+  special area: Loader
  
   description:  String functions
                 
   version:      7.5.
-
   -----------------------------------------------------------------------------
- 
-  copyright:    (c) 1997-2004 SAP AG-2003
- 
-  -----------------------------------------------------------------------------
-
-
 
     ========== licence begin  GPL
     Copyright (c) 1997-2004 SAP AG
@@ -38,9 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 
 #include <ctype.h>
@@ -371,14 +359,16 @@ ls05_StrCmpPaddedUCS2(const SAPDB_Byte*  pszShorterVal,
   function:     ls05StrToUpr
   -----------------------------------------------------------------------------
 */
+// Caller have to make sure the strings coming here are already UTF8!!
 void ls05StrToUpr(tsp00_Addr pszString, tsp00_Int4 lStringLength)
 {
     ROUTINE_DBG_MEO00 ("ls05StrToUpr");
 
-    for (int i = 0; i < lStringLength; ++i)
-    {
-        pszString[i] = lLatin1_ToUpperMap_ls05[STATIC_CAST(unsigned char, pszString[i])];
-    }
+    // The function ToUpper() assures inplace replacement; so we can
+    // safely copy the new value to the old place; but the performance ...
+    Tools_DynamicUTF8String s(REINTERPRET_CAST(Tools_DynamicUTF8String::ConstPointer, pszString));
+    s.ToUpper();
+    SAPDB_memcpy(pszString, s.StrPtr(), lStringLength);
 }
 //  ls05StrToUpr()
 
@@ -452,6 +442,13 @@ ls05StrTrimRight(tsp00_Addr      pszString,
                                                      cTrimVal);
             break;
         }
+        case ctUCS2Native_els00:
+        {
+            lCount = sp77encodingUCS2Native->countPadChars(STATIC_CAST(void*, pszString),
+                                                           STATIC_CAST(tsp00_Uint4, lStrLength),
+                                                           cTrimVal);
+            break;
+        }
         default: // covers ctAscii_els00 and ctUTF8_els00 (trim char may only be US7ASCII)
         {
             ls05StrTrimRight(pszString, lStrLength, cTrimVal);
@@ -479,7 +476,41 @@ ls05StrTrimLeft(tsp00_Addr       pBuffer,
     bool          bdone = false;
     tsp00_Longint i     = 0;
 
-    switch (CodeType)
+    // Hmm, we suppose that trim char is ALWAYS blank in case of UCS2 !
+    if (true == ls00IsUCS2(&CodeType))
+    {
+        register char c1 = 0;
+        register char c2 = 0;
+        if (0 == pEncodings_ls00[CodeType]->charTable->blank.bytes[0])
+        {
+            // UCS2 normal
+            c2 = BLANK_LS00;
+        }
+        else
+        {
+            // UCS2 swapped
+            c1 = BLANK_LS00;
+        }
+        for (; ( (i < lBufferLength) && (pBuffer[i] == c1) && (pBuffer[i+1] == c2) ); i += 2)
+            ;
+
+        if ( (0 != i) && (lBufferLength != i) )
+        {
+            lBufferLength -= i;
+            memmove(pBuffer, &pBuffer[i], lBufferLength*sizeof(char));              
+        }
+        else if (lBufferLength == i)
+        {
+            lBufferLength = 0;
+        }
+    }
+    else
+    {
+        // covers ctAscii_els00 and ctUTF8_els00 (trim char may only be US7ASCII)
+        ls05StrTrimLeftEx(pBuffer, lBufferLength);
+    }
+
+/*    switch (CodeType)
     {
         case ctUCS2Swapped_els00:
         {
@@ -512,7 +543,7 @@ ls05StrTrimLeft(tsp00_Addr       pBuffer,
             ls05StrTrimLeftEx(pBuffer, lBufferLength);
         }
     }
-
+*/
     return;
 }
 // ls05StrTrimLeft()
@@ -833,7 +864,7 @@ ls05ConvertToUCS2Simple(unsigned char  *pszDestination,
     {
         if ( errConvertingEncoding_els98 != rc )
         {
-            RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
                                                                               _INTERNAL_FILE_ID_, __LINE__, rc);
             rc = errInternal_els98;
         }
@@ -841,12 +872,12 @@ ls05ConvertToUCS2Simple(unsigned char  *pszDestination,
         {
             if (ctAscii_els00== SrcEncoding)
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "ASCII", "UCS2");
             }
             else
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "UTF8", "UCS2");
             }
 
@@ -858,6 +889,83 @@ ls05ConvertToUCS2Simple(unsigned char  *pszDestination,
 // ls05ConvertToUCS2Simple()
 
 
+/*
+  -----------------------------------------------------------------------------
+  function:     ls05ConvertToUCS2Simple - overloaded
+  -----------------------------------------------------------------------------
+*/
+// This function converts to a given destination UCS2 (swapped or not) and does
+// not define the native swap of the running server
+tsp00_Longint
+ls05ConvertToUCS2Simple(unsigned char  *pszDestination,
+                        tsp00_Longuint &lDestinationLen,
+                        tls00_CodeType  DestEncoding,
+                        unsigned char  *pszSource,
+                        tsp00_Longint   lSourceLen,
+                        tls00_CodeType  SrcEncoding,
+                        tsp00_Addr      pszErrText,
+                        tsp81_CodePage *pASCII_CP)
+{
+    ROUTINE_DBG_MEO00 ("ls05ConvertToUCS2Simple");
+
+    tsp00_Longint rc = errOK_els00;
+
+    if (false == ls00IsUCS2(&SrcEncoding))        // source encoding may not be already ucs2
+    {
+        if (ctUTF8_els00 == SrcEncoding)
+        {
+            rc = ls05_ConvertUTF8ToUCS2(pszDestination, lDestinationLen, pszSource, lSourceLen, DestEncoding);
+        }
+        else
+        {
+            // Coming here code page structure might be NULL-pointer or filled.
+            //int _Swapped = STATIC_CAST(int, ls07Swapped(NULL));
+            int _Swapped = 0;
+            if (ctUCS2Swapped_els00 == DestEncoding)
+            {
+                _Swapped = 1;
+            }
+            else if (ctUCS2Native_els00 == DestEncoding)
+            {
+                _Swapped = STATIC_CAST(int, ls07Swapped(NULL));
+            }
+            rc = ls05_ConvertASCIIAnyCPToUCS2(pszDestination, lDestinationLen, pszSource, lSourceLen, _Swapped, pASCII_CP);
+        }
+    }
+
+    // Both conversion functions return identical values for
+    //  errOK_els00                      : conversion successful; length of converted string is returned in lDestinationLen
+    //  errEncodingSourceExhausted_els98 : partial character in source, but hit end
+    //  errConvertingEncoding_els98      : source contains invalid chars or char sequences)
+    //  errEncodingTargetExhausted_els98 : insuff. space in target for conversion)
+    if (errOK_els00 != rc)
+    {
+        if ( errConvertingEncoding_els98 != rc )
+        {
+            LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
+                                                                             _INTERNAL_FILE_ID_, __LINE__, rc);
+            rc = errInternal_els98;
+        }
+        else
+        {
+            if (ctAscii_els00== SrcEncoding)
+            {
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                                                                     errConvertingEncoding_els98, "ASCII", "UCS2");
+            }
+            else
+            {
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                                                                     errConvertingEncoding_els98, "UTF8", "UCS2");
+            }
+
+            rc = errConvertingEncoding_els98;
+        }
+    }
+    return rc;
+}
+// ls05ConvertToUCS2Simple()
+
 /*
   -----------------------------------------------------------------------------
   function:     ls05ConvertToUCS2NotSwapped
@@ -896,7 +1004,7 @@ ls05ConvertToUCS2NotSwapped(unsigned char  *pszDestination,
     {
         if ( errConvertingEncoding_els98 != rc )
         {
-            RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
                                                                               _INTERNAL_FILE_ID_, __LINE__, rc);
             rc = errInternal_els98;
         }
@@ -904,12 +1012,12 @@ ls05ConvertToUCS2NotSwapped(unsigned char  *pszDestination,
         {
             if (ctAscii_els00== SrcEncoding)
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "ASCII", "UCS2");
             }
             else
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "UTF8", "UCS2");
             }
 
@@ -943,14 +1051,12 @@ ls05ConvertFromUCS2(unsigned char  *pszDestination,
     // external code type UCS2 is checked by called func
     if ( (dstrb != DBDataType) && (dstrdb != DBDataType) )
     {
-        short swapTest = 1;
-        
         rc = ls05ConvertFromUCS2Simple(pszDestination,
                                        pszSource,
                                        lDestinationLen,
                                        lSourceLen,
                                        DestEncoding,
-                                       ((*(char *)&swapTest) ? true : false),
+                                       ctUCS2Native_els00,
                                        pszErrText);
     }
     
@@ -970,23 +1076,33 @@ ls05ConvertFromUCS2Simple(unsigned char  *pszDestination,
                           tsp00_Longint  &lDestinationLen,
                           tsp00_Longint   lSourceLen,
                           tls00_CodeType  DestEncoding,
-                          bool            bSourceSwapped,
+                          tls00_CodeType  SrcEncoding,
                           tsp00_Addr      pszErrText)
 {
     ROUTINE_DBG_MEO00 ("ls05ConvertFromUCS2Simple I");
 
     tsp00_Longint rc = errOK_els00;
 
-    const tsp77encoding *pSrcEncoding_sp77 = (true == bSourceSwapped) ? sp77encodingUCS2Swapped : sp77encodingUCS2;
-    tls00_CodeType       pSrcEncoding      = (true == bSourceSwapped) ? ctUCS2Swapped_els00 : ctUCS2_els00;
-
-    if ( (ctUCS2_els00 != DestEncoding) && (ctUCS2Swapped_els00 != DestEncoding) )        // destination encoding may not be already ucs2
+    if (false == ls00IsUCS2(&DestEncoding))        // destination encoding may not be already ucs2
     {
+        tls00_CodeType _mySrcEncoding = SrcEncoding;
+        if (ctUCS2Native_els00 == _mySrcEncoding)
+        {
+            if ('\xFF' == sp77encodingUCS2Native->charTable->indicator.bytes[0])    // swapped
+            {
+                _mySrcEncoding = ctUCS2Swapped_els00;
+            }
+            else
+            {
+                _mySrcEncoding = ctUCS2_els00;
+            }
+        }
+
         if (ctUTF8_els00 == DestEncoding)
         {
             Tools_DynamicUTF8String s;
 
-            rc = ls05_ConvertUCS2ToUTF8(s, pszSource, lSourceLen, pSrcEncoding);
+            rc = ls05_ConvertUCS2ToUTF8(s, pszSource, lSourceLen, _mySrcEncoding);
             if (errOK_els00 == rc)
             {
                 lDestinationLen = s.BasisSize();
@@ -996,7 +1112,7 @@ ls05ConvertFromUCS2Simple(unsigned char  *pszDestination,
         }
         else
         {
-            rc = ls05_ConvertUCS2ToASCII(pszDestination, lDestinationLen, pszSource, lSourceLen, pSrcEncoding_sp77);
+            rc = ls05_ConvertUCS2ToASCII(pszDestination, lDestinationLen, pszSource, lSourceLen, pEncodings_ls00[_mySrcEncoding]);
         }
     }
 
@@ -1010,7 +1126,7 @@ ls05ConvertFromUCS2Simple(unsigned char  *pszDestination,
     {
         if (errConvertingEncoding_els98 != rc)
         {
-            RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
                                                                               _INTERNAL_FILE_ID_, __LINE__, rc);
             rc = errInternal_els98;
         }
@@ -1018,12 +1134,12 @@ ls05ConvertFromUCS2Simple(unsigned char  *pszDestination,
         {
             if (ctAscii_els00 == DestEncoding)
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "UCS2", "ASCII");
             }
             else
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "UCS2", "UTF8");
             }
 
@@ -1051,14 +1167,12 @@ ls05ConvertFromUCS2Simple(unsigned char  *pszDestination,
 {
     ROUTINE_DBG_MEO00 ("ls05ConvertFromUCS2Simple II");
 
-    short swapTest = 1;
-    
     tsp00_Longint rc = ls05ConvertFromUCS2Simple(pszDestination,
                                                  pszSource,
                                                  lDestinationLen,
                                                  lSourceLen,
                                                  DestEncoding,
-                                                 ((*(char *)&swapTest) ? true : false),
+                                                 ctUCS2Native_els00,
                                                  pszErrText);
     return rc;
 }
@@ -1102,7 +1216,7 @@ ls05ConvertToUTF8(Tools_DynamicUTF8String &Destination,
     {
         if ( errConvertingEncoding_els98 != rc )
         {
-            RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
                                                                               _INTERNAL_FILE_ID_, __LINE__, rc);
             rc = errInternal_els98;
         }
@@ -1110,12 +1224,12 @@ ls05ConvertToUTF8(Tools_DynamicUTF8String &Destination,
         {
             if (ctAscii_els00 == SrcEncoding)
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "ASCII", "UTF8");
             }
             else
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "UCS2", "UTF8");
             }
             rc = errConvertingEncoding_els98;
@@ -1162,7 +1276,7 @@ ls05ConvertFromUTF8(unsigned char           *pszDestination,
     {
         if (errConvertingEncoding_els98 != rc)
         {
-            RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
                                                                               _INTERNAL_FILE_ID_, __LINE__, rc);
             rc = errInternal_els98;
         }
@@ -1170,12 +1284,12 @@ ls05ConvertFromUTF8(unsigned char           *pszDestination,
         {
             if (ctAscii_els00 == DestEncoding)
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "UTF8", "ASCII");
             }
             else
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "UTF8", "UCS2");
             }
             rc = errConvertingEncoding_els98;
@@ -1224,7 +1338,7 @@ ls05ConvertFromUTF8(unsigned char           *pszDestination,
     {
         if (errConvertingEncoding_els98 != rc)
         {
-            RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText, errInternal_els98,
                                                                               _INTERNAL_FILE_ID_, __LINE__, rc);
             rc = errInternal_els98;
         }
@@ -1232,12 +1346,12 @@ ls05ConvertFromUTF8(unsigned char           *pszDestination,
         {
             if (ctAscii_els00 == DestEncoding)
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "UTF8", "ASCII");
             }
             else
             {
-                RepServMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCursor_els98, pszErrText,
                                                                       errConvertingEncoding_els98, "UTF8", "UCS2");
             }
             rc = errConvertingEncoding_els98;
