@@ -5,9 +5,9 @@
  
   -----------------------------------------------------------------------------
  
-  responsible:  SteffenS
+  responsible:  SteffenS, d033893
 
-  special area: Replication Manager
+  special area: Loader
  
   description:  Numerical and code conversion functions
                 
@@ -15,7 +15,7 @@
 
   -----------------------------------------------------------------------------
  
-  copyright:    (c) 1999-2004 SAP AG-2003
+  Copyright (c) 1999-2004 SAP AG-2003
  
   -----------------------------------------------------------------------------
 
