@@ -1,17 +1,15 @@
 /*
   -----------------------------------------------------------------------------
-
   module: vls08_long.cpp
-
   -----------------------------------------------------------------------------
 
-  responsible:  SteffenS
+  responsible:  SteffenS, d033893
 
-  special area: Replication Manager
+  special area: Loader
 
   description:  Long data type support
 
-  version:      7.5.
+  version:      7.6.
   -----------------------------------------------------------------------------
 
     ========== licence begin  GPL
@@ -51,13 +49,13 @@
 
 /* Possible db data types; includes long data types which are marked UPPERCASE
 
- 0
+ 0                                                                 6            7           8             9
  dfixed,   dfloat,    dcha,      dche,       dchb,     drowid,     DSTRA,       DSTRE,      DSTRB,        DSTRDB,
- 10
+ 10                                                                                                       19
  ddate,    dtime,     dvfloat,   dtimestamp, dunknown, dnumber,    dnonumber,   dduration,  ddbyteebcdic, DLONGA,
- 20
+ 20        21         22
  DLONGE,   DLONGB,    DLONGDB,   dboolean,   dunicode, ddtfiller1, ddtfiller2,  ddtfiller3, ddtfiller4,   dsmallint,
- 30
+ 30                                          34        35
  dinteger, dvarchara, dvarchare, dvarcharb,  DSTRUNI,  DLONGUNI,   dvarcharuni, dudt,       dabaptabhandle
 */
 
@@ -74,14 +72,15 @@
 // 3 - utf8   -> ucs2
 // 4 - ebcdic -> ucs2 (actually not longer supported)
 static char
-szLOAD_LONG_Compatibility_ls08[8][39] =
+szLOAD_LONG_Compatibility_ls08[9][39] =
              {  {0},
                 {0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0},    //ctAscii_els00
                 {0,0,0,0,0,0,4,4,1,1,0,0,0,0,0,0,0,0,0,4,4,1,1,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0},    //ctEbcdic_els00
-                {0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0},    //ctUTF8_els00
+                {0,0,0,0,0,0,3,0,1,1,0,0,0,0,0,0,0,0,0,3,0,1,1,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0},    //ctUTF8_els00
                 {0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},    //ctIgnore_els00
-                {0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2_els00
-                {0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2Swapped_els00
+                {0,0,0,0,0,0,1,0,1,1,0,0,0,0,0,0,0,0,0,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2_els00
+                {0,0,0,0,0,0,1,0,1,1,0,0,0,0,0,0,0,0,0,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2Swapped_els00
+                {0,0,0,0,0,0,1,0,1,1,0,0,0,0,0,0,0,0,0,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2Native_els00
                 {0}};
 
 
@@ -99,7 +98,7 @@ szLOAD_LONG_Compatibility_ls08[8][39] =
 // 3 - ucs2   -> utf8
 // 4 - ucs2   -> ebcdic (actually not longer supported)
 static char
-szEXTRACT_LONG_UNICODE_ls08[8][39] =
+szEXTRACT_LONG_UNICODE_ls08[9][39] =
              {  {0},
                 {0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},    //ctAscii_els00
                 {0,0,0,0,0,0,4,4,1,1,0,0,0,0,0,0,0,0,0,4,4,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},    //ctEbcdic_els00
@@ -107,11 +106,12 @@ szEXTRACT_LONG_UNICODE_ls08[8][39] =
                 {0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctIgnore_els00
                 {0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2_els00
                 {0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2Swapped_els00
+                {0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2Native_els00
                 {0}};
 
 // Datatype compatibility/conversion matrix for the process of extracting LONG values
 // (mapping db LONG types to external code types) in case
-// of ASCII database when using dataextract with ASCII order interface
+// of ASCII/UNICODE database when using dataextract with ASCII order interface
 
 // 0 - conversion forbidden - results in error message
 // 1 - no conversion needed - internal binary LONGs (e.g. datatype LONG BYTE) will be extracted
@@ -122,14 +122,15 @@ szEXTRACT_LONG_UNICODE_ls08[8][39] =
 // 3 - ascii  -> utf8
 // 4 - ascii  -> ebcdic (actually not longer supported)
 static char
-szEXTRACT_LONG_ASCII_ls08[8][39] =
+szEXTRACT_LONG_ASCII_ls08[9][39] =
              {  {0},
                 {0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctAscii_els00
                 {0,0,0,0,0,0,4,4,1,1,0,0,0,0,0,0,0,0,0,4,4,1,1,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0},    //ctEbcdic_els00
                 {0,0,0,0,0,0,3,3,1,1,0,0,0,0,0,0,0,0,0,3,3,1,1,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0},    //ctUTF8_els00
                 {0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctIgnore_els00
-                {0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0},    //ctUCS2_els00
-                {0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0},    //ctUCS2Swapped_els00
+                {0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2_els00
+                {0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2Swapped_els00
+                {0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,2,2,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0},    //ctUCS2Native_els00
                 {0}};
 
 // Maps external encoding types of long files to db long data types
@@ -168,9 +169,6 @@ ls08_CheckPositionSpecification(tsp00_Addr         pszString,
                                 tls00_LargeNumber &lFromValue,
                                 tsp00_Longint     &lLength);
 
-
-
-
 //*******  P U B L I C   F U N C T I O N S   I M P L E M E N T A T I O N  ********
 
 /*
@@ -239,7 +237,7 @@ ls08InitLongFileInfo(tls00_MultColSpec *pMultColSpec, tls00_MultFileSpec *pMultL
                 }
             }
         }
-    }       // for (i = 0; i < DloadCmd->dloLongSpec_ls00.mfsCount_ls00; ++i)
+    }//  for (i = 0; i < pMultLongFileSpec->mfsCount_ls00; ++i)
     
     return;
 }
@@ -252,25 +250,22 @@ ls08InitLongFileInfo(tls00_MultColSpec *pMultColSpec, tls00_MultFileSpec *pMultL
   -----------------------------------------------------------------------------
 */
 int
-ls08ReadLongFileInfo(tls00_Column *myColumn, tls00_String *Value, tsp00_Addr ErrText)
+ls08ReadLongFileInfo(tls00_Column    *pField,
+                     tls00_ParamInfo *pColumnParameters,
+                     tls00_String    *pValue,
+                     tsp00_Addr       pszErrText)
 {
     ROUTINE_DBG_MEO00 ("ls08ReadLongFileInfo");
 
     // We don't get in here if the value represents the NULL value
-
     tsp00_Int4 rc = errOK_els00;
 
-    tsp00_Longint lFrom = 0;
-    
-    tls00_ParamInfo_LONG &pLCI = myColumn->colLongInfo;
+    tls00_ParamInfo_LONG &pLCI  = pField->colLongInfo;
 
-
-
-
-    if (NULL == Value->strAddr_ls00)        // indicates compact format -> better using fileformat as a parameter?
+    if (NULL == pValue->strAddr_ls00)        // indicates compact format -> better using fileformat as a parameter?
     {
         // Initialize some values
-        pLCI.loiLength_ls00          = Value->strLeng_ls00;
+        pLCI.loiLength_ls00          = pValue->strLeng_ls00;
         pLCI.loiPart_ls00            = true;   // Long value is part of the data file
         pLCI.loiFileOffset_ls00      = pLCI.loiFileStruct_ls00->vfFilePos_ls00;     // File Offset in input file
         pLCI.loiCountOfSendData_ls00 = 0L;     // count of already send data (to kernel)
@@ -287,24 +282,37 @@ ls08ReadLongFileInfo(tls00_Column *myColumn, tls00_String *Value, tsp00_Addr Err
         pLCI.loiCountOfSendData_ls00 = 0L;     // count of already send data (to kernel)
 
         // Because we use c-func atol we need to make the string null terminated
-        tsp00_Longint lLongSpecLen = Value->strLeng_ls00;
+        tsp00_Longint lLongSpecLen = pValue->strLeng_ls00;
         _TCHAR       *pszLongSpec  = new _TCHAR[lLongSpecLen + 1];
-        memcpy(pszLongSpec, Value->strAddr_ls00, lLongSpecLen);
+        memcpy(pszLongSpec, pValue->strAddr_ls00, lLongSpecLen);
         pszLongSpec[lLongSpecLen] = 0;
 
-        if (true == myColumn->colLongInfo.loiWithLongInfo_ls00)
+        if (true == pField->colLongInfo.loiWithLongInfo_ls00)
         {   // LONGFILE Info assigned to column already in command
             pLCI.loiPart_ls00 = true;    // Long value is only a part of the whole file
 
             // Analyze LONGINFO parameter in data
             rc = ls08_CheckPositionSpecification(pszLongSpec, pLCI.loiFileOffset_ls00, pLCI.loiLength_ls00);
+
+            if ( (false == ls30VFIsMagicBytesCacheEmpty(pLCI.loiFileStruct_ls00))
+                 && 
+                 ( (0 != pLCI.loiFileOffset_ls00.lnu2GB_Blocks_ls00)
+                   ||
+                   ( (0 == pLCI.loiFileOffset_ls00.lnu2GB_Blocks_ls00)
+                     &&
+                     (pLCI.loiFileOffset_ls00.lnuRemainder_ls00 > pLCI.loiFileStruct_ls00->vfMagicBytesCache_ls00.lBufLen_ls00) )
+                 )
+               )
+            {
+                ls30VFEmptyMagicBytesCache(pLCI.loiFileStruct_ls00);
+            }
         }
         else
         {
             // checking for specs like
             // '<file name>' <from position>-<to position>
             //  where the single quotes and the positions are optional
-            rc = ls30VFClose(*pLCI.loiFileStruct_ls00, ErrText);   // closing LONGFILE
+            rc = ls30VFClose(*pLCI.loiFileStruct_ls00, pszErrText);   // closing LONGFILE
             if (errOK_els00 != rc)
             {
                 return rc;
@@ -317,7 +325,6 @@ ls08ReadLongFileInfo(tls00_Column *myColumn, tls00_String *Value, tsp00_Addr Err
             }
 
             ls05StrTrimRightEx(pszLongSpec);                    // trim string from left and right
-
             ls05StrTrimLeftEx(pszLongSpec, lLongSpecLen);
 
             _TCHAR *pszWorkPtr = pszLongSpec;
@@ -346,7 +353,7 @@ ls08ReadLongFileInfo(tls00_Column *myColumn, tls00_String *Value, tsp00_Addr Err
                     }
                 }
             }
-            else
+            /*else
             {
                 int i = 0;
                 // No single quoted string as file name --> find first white space to check for positions
@@ -366,7 +373,7 @@ ls08ReadLongFileInfo(tls00_Column *myColumn, tls00_String *Value, tsp00_Addr Err
                         pLCI.loiPart_ls00 = true;    // Long value is only a part of the whole file
                     }
                 }
-            }
+            }*/
 
             if (errOK_els00 == rc)
             {
@@ -376,31 +383,96 @@ ls08ReadLongFileInfo(tls00_Column *myColumn, tls00_String *Value, tsp00_Addr Err
             }
 
             pszWorkPtr = NULL;
-        }   // end if (true == myColumn->colLongInfo.loiWithLongInfo_ls00)
+        }   // end if (true == pField->colLongInfo.loiWithLongInfo_ls00)
 
         // If everything is analyzed we could open the file and seek to the right position
         if (errOK_els00 == rc)
         {
-            if (0 == pLCI.loiFileStruct_ls00->vfFileHndl_ls00)         // LONGFILE already open ?
-            {
-                if (-1 != pLCI.loiFileOffset_ls00.lnuRemainder_ls00)   // insert empty string --> no file to open
-                {
-                    ls30VFInitialize(*pLCI.loiFileStruct_ls00,         // Init file as binary
-                                     pLCI.loiLongFileName_ls00);
+            bool         _bEmptyLongFile = false;
+            tls00_VFile* pLongFile       = pLCI.loiFileStruct_ls00;
 
-                    rc = ls30VFOpen(*pLCI.loiFileStruct_ls00, ErrText);    // open file
+            if ( (0  == pLongFile->vfFileHndl_ls00)  &&       // LONGFILE already open ?
+                 (-1 != pLCI.loiFileOffset_ls00.lnuRemainder_ls00)  )       // insert empty string --> no file to open
+            {
+                ls30VFInitialize(*pLongFile,         // Init file as binary
+                                 pLCI.loiLongFileName_ls00,
+                                 sp5vf_binary, sp5vf_read, pLCI.loiFileEncoding_ls00);
+
+                rc = ls30VFOpen(*pLongFile, pszErrText);    // open file
+                if (errOK_els00 == rc)
+                {
+                    if ( (0 == pLongFile->vfFileLen_ls00.lnu2GB_Blocks_ls00) &&
+                         (0 == pLongFile->vfFileLen_ls00.lnuRemainder_ls00) )
+                    {
+                        // File is empty --> special treatment for LONG values: this indicates to insert empty string
+                        pLCI.loiFileOffset_ls00.lnuRemainder_ls00 = 0L;
+                        _bEmptyLongFile = true;
+                    }
+                    else
+                    {
+                        rc = ls30Detect_BinaryFile_Encoding(*pLongFile, pszErrText);
+                        if (errOK_els00 == rc)
+                        {
+                            // Check if file contains anything else than the magic bytes; those are
+                            // only found in UNICODE files
+                            if ( (true         == ls00IsUCS2(&pLongFile->vfFileEncoding_ls00)) ||
+                                 (ctUTF8_els00 == pLongFile->vfFileEncoding_ls00) )
+                            {
+                                if (0 == pLongFile->vfFileLen_ls00.lnu2GB_Blocks_ls00)
+                                {
+                                    if (pLongFile->vfFileLen_ls00.lnuRemainder_ls00 ==
+                                        pLongFile->vfMagicBytesCache_ls00.lBufPos_ls00)
+                                    {
+                                        // File is empty --> special treatment for LONG values: this
+                                        //                   indicates to insert empty string
+                                        pLCI.loiFileOffset_ls00.lnuRemainder_ls00 = 0L;
+                                        _bEmptyLongFile = true;
+                                    }
+                                }
+                            }
+                            if ( (false == _bEmptyLongFile) &&
+                                 (0 == pLCI.loiFileOffset_ls00.lnu2GB_Blocks_ls00) &&
+                                 (0 == pLCI.loiFileOffset_ls00.lnuRemainder_ls00)  &&
+                                 (0 < pLongFile->vfFilePos_ls00.lnuRemainder_ls00) )
+                            {
+                                pLCI.loiFileOffset_ls00.lnuRemainder_ls00 =
+                                    pLongFile->vfFilePos_ls00.lnuRemainder_ls00;
+                            }
+                        }
+                    }   // end else of if ( (0 == pLongFile->vfFileLen_ls00.lnu2GB_Blocks_ls00) && ...
+                }
+                if ( (errOK_els00 == rc) && (false == _bEmptyLongFile) )
+                {
+                    rc = ls08CheckLONG_DT_Compatibility(pLongFile->vfFileEncoding_ls00, 
+                                                        pColumnParameters->piBasicInfo_ls00.fbDataType_ls00);
                     if (errOK_els00 == rc)
                     {
-                        ls30VFSeekLong(*pLCI.loiFileStruct_ls00, &pLCI.loiFileOffset_ls00, ErrText);
+                        ls30VFSeekLong(*pLongFile, &pLCI.loiFileOffset_ls00, pszErrText);
                     }
+                    else
+                    {
+                        // error: data types do not match (function returns only ok or errIncompatibleDataTypes_els98)
+                        LoaderMessages::Instance()->ls98Msg(layDataLoad_els98, pszErrText,
+                                                                               errIncompatibleDataTypes_els98,
+                                                                               pField->colName_ls00.asCharp());
+                    }
+                }
+            }   // end if ( (0  == pLCI.loiFileStruct_ls00->vfFileHndl_ls00)  && ...
+            else
+            {
+                if ( (pLongFile->vfFilePos_ls00.lnuRemainder_ls00 == pLongFile->vfMagicBytesCache_ls00.lBufLen_ls00) &&
+                     (0 == pLCI.loiFileOffset_ls00.lnu2GB_Blocks_ls00) &&
+                     (pLCI.loiFileOffset_ls00.lnuRemainder_ls00 < pLongFile->vfMagicBytesCache_ls00.lBufLen_ls00) )
+                {
+                    pLCI.loiFileOffset_ls00.lnuRemainder_ls00 = pLongFile->vfFilePos_ls00.lnuRemainder_ls00;
                 }
             }
 
-            if (false == pLCI.loiPart_ls00)    // LONG VALUE = whole content of data file?
+            if ( (false == pLCI.loiPart_ls00) && (false == _bEmptyLongFile) )    // LONG VALUE = whole content of data file?
             {
-                if (0 == pLCI.loiFileStruct_ls00->vfFileLen_ls00.lnu2GB_Blocks_ls00)
+                if (0 == pLongFile->vfFileLen_ls00.lnu2GB_Blocks_ls00)
                 {
-                    pLCI.loiLength_ls00 = pLCI.loiFileStruct_ls00->vfFileLen_ls00.lnuRemainder_ls00;
+                    pLCI.loiLength_ls00 = pLongFile->vfFileLen_ls00.lnuRemainder_ls00;
                 }
                 else
                 {
@@ -415,9 +487,7 @@ ls08ReadLongFileInfo(tls00_Column *myColumn, tls00_String *Value, tsp00_Addr Err
             delete [] pszLongSpec;
             pszLongSpec = NULL;
         }
-
-    }   // end if (NULL == Value->strAddr_ls00)
-
+    }   // end if (NULL == pValue->strAddr_ls00)
 
     return rc;
 }
@@ -569,7 +639,7 @@ ls08ExecPutVal(tls00_Buffer           &LongDataBuffer,
 {
     ROUTINE_DBG_MEO00 ("ls08ExecPutVal");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance();    // error msg obj
+    LoaderMessages *pMsgObj = LoaderMessages::Instance();    // error msg obj
     tsp00_Longint    rc      = errOK_els00;
     
     tsp1_segment *pSqlSegm = NULL;
@@ -918,7 +988,7 @@ ls08ExtractLongs(tls00_DBInfo           *pDBInfo,
 
 
     // Parameter used to process unicode data
-    bool                    bUnicodeOI     = ls00IsDBUnicode(pDBInfo);  // inline function
+    bool                    bUnicodeOI     = ls00UseUnicodeOI(pDBInfo);  // inline function
 
     //*
     //*     Start processing LONGs
@@ -938,15 +1008,19 @@ ls08ExtractLongs(tls00_DBInfo           *pDBInfo,
         }
         else
         {
-            if (vm_lastdata == pLDesc->ld_valmode())
+            if ( (vm_lastdata == pLDesc->ld_valmode()) || (vm_startpos_invalid == pLDesc->ld_valmode()) )
             {
                 // Seems the whole data is delivered when requesting the length of the LONG;
                 // In this case
-                // - the kernel has set valmode to vm_lastdata that is not modified by the Loader
+                //  either
+                // - the kernel has set valmode to vm_lastdata that is not modified by the Loader if it
+                //   is the only long in a table row or the first one
+                //  or
+                // - upon requesting the LONGs over GETVAL in the while loop 
                 // - the max length and vallen is lower than 3 as the Loader requests 2 Bytes when
                 //   defining the maximal length of that LONG in ls27_GetLongLength
                 if ( (pLDesc->ld_maxlen() > 0) && (pLDesc->ld_maxlen() < 3) && 
-                     (pLDesc->ld_vallen() > 0) && (pLDesc->ld_vallen() < 3) )
+                     (pLDesc->ld_vallen() >= 0) && (pLDesc->ld_vallen() < 3) )
                 {
                     pLDesc->ld_valmode().becomes(vm_datapart);
                     pLDesc->ld_intern_pos() = 1;
@@ -979,9 +1053,16 @@ ls08ExtractLongs(tls00_DBInfo           *pDBInfo,
                                                               "%.*d", pLParam->loiNumberSize_ls00, lNumber);
 
                 // ATTENTION: Name of the file will be adapted here
-                ls30VFInitialize(*pLParam->loiFileStruct_ls00, pLParam->loiFileName_ls00, sp5vf_binary, sp5vf_write);
+                ls30VFInitialize(*pLParam->loiFileStruct_ls00, pLParam->loiFileName_ls00,
+                                 sp5vf_binary, sp5vf_write,
+                                 pData->cdExtLONGEncoding_ls00);
 
                 rc = ls30VFOpen(*pLParam->loiFileStruct_ls00, pszErrText);    // Open LONG data file; func write msg in case of error
+                if (errOK_els00 == rc)
+                {
+                    rc = ls30Mark_BinaryFile_Encoding(*pLParam->loiFileStruct_ls00, pszErrText);
+                }
+
             }
         }
 
@@ -1115,9 +1196,7 @@ ls08ExtractLongs(tls00_DBInfo           *pDBInfo,
                             //* In case of external encoding type UTF8 no further conversion needed,
                             //* readable hex representation may only be US7ASCII which is already UTF8
 
-                            if ( (ctUCS2_els00        == pData->cdExtLONGEncoding_ls00) ||
-                                 (ctUCS2Swapped_els00 == pData->cdExtLONGEncoding_ls00) ||
-                                 (ctUCS2Native_els00  == pData->cdExtLONGEncoding_ls00)  )
+                            if ( true == ls00IsUCS2(&pData->cdExtLONGEncoding_ls00) )
                             {
                                 lDestLen = pConvertBuffer->lBufLen_ls00;
                                 rc = ls05ConvertToUCS2Simple(REINTERPRET_CAST(unsigned char*, pConvertBuffer->pBufPtr_ls00),
@@ -1273,7 +1352,7 @@ ls08ExecGetVal(tls00_DBInfo         *DBInfo,
                                        pLongInfo->dliFinishedNNLongs_ls00);
             if (errOK_els00 != rc)
             {
-                RepServMessages::Instance()->ls98Msg(layDataExtract_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layDataExtract_els98, pszErrText,
                                                                            STATIC_CAST(ErrorNumbers, rc));
             }
         }
@@ -1301,9 +1380,11 @@ ls08CloseGetVal(tls00_DBInfo         *pDBInfo,
     tsp1_packet  *pSqlReqPacket = NULL;
     tsp1_segment *pSqlSegm      = NULL;
     tsp1_part    *pSqlPart      = NULL;
+    tsp00_Int4    lPcktNo       = 1;
     
-    // Generating new GETVAL-command
-    pSqlReqPacket = pDBInfo->dbiPktSndList_ls00[0];
+    // Generating new GETVAL-command - we use packet 1 here because we use this also when
+    // executing the GETVALs
+    pSqlReqPacket = pDBInfo->dbiPktSndList_ls00[lPcktNo];
 
     rc = ls04FastInit(pSqlReqPacket, &pSqlSegm, &pSqlPart, pDBInfo);   // fast packet initialization
     ls04SetMsgType(pSqlSegm, sp1m_getval);                          // message type 'PUTVAL'
@@ -1339,7 +1420,7 @@ ls08CloseGetVal(tls00_DBInfo         *pDBInfo,
         if (errOK_els00 == rc)
         {
             tsp00_Int4 lErrPos = 0;
-            rc = ls03ProcessSQLPacket(pDBInfo, 0, lErrPos, pszErrText); // Send, receive and check SQL packet
+            rc = ls03ProcessSQLPacket(pDBInfo, lPcktNo, lErrPos, pszErrText); // Send, receive and check SQL packet
             if (errOK_els00 != rc)
             {
                 rc = errPutVal_els98;
@@ -1428,11 +1509,11 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
 {
     ROUTINE_DBG_MEO00 ("ls08_FillLong");
 
-    tsp00_Longint      rc           = errOK_els00;
-    tls00_FieldBasics *pFB          = &ParamInfo.piBasicInfo_ls00;
+    tsp00_Longint      rc       = errOK_els00;
+    tls00_FieldBasics *pFB      = &ParamInfo.piBasicInfo_ls00;
 
     // Compute input length of data to read
-    tsp00_Longint      ReadLen      = LCI.loiLength_ls00 - LCI.loiCountOfSendData_ls00;
+    tsp00_Longint      ReadLen  = LCI.loiLength_ls00 - LCI.loiCountOfSendData_ls00;
 
     //*
     //*     Emtpy values
@@ -1488,11 +1569,11 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
 
             lBytesToRead = (0 == (lBytesToRead % 2)) ? lBytesToRead : (lBytesToRead - 1);
             
-            if (1 == (szLOAD_LONG_Compatibility_ls08[LCI.loiFileEncoding_ls00][pFB->fbDataType_ls00]) )
+            if (1 == (szLOAD_LONG_Compatibility_ls08[LCI.loiFileStruct_ls00->vfFileEncoding_ls00][pFB->fbDataType_ls00]) )
             {
                 lBytesToRead *= 2;
             }
-            if ( (ctUCS2_els00 == LCI.loiFileEncoding_ls00) || (ctUCS2Swapped_els00 == LCI.loiFileEncoding_ls00) )
+            if (true == ls00IsUCS2(&LCI.loiFileStruct_ls00->vfFileEncoding_ls00))
             {
                 lBytesToRead *=2;
             }
@@ -1520,7 +1601,7 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
 
             lBytesToRead *=2;
 
-            if ( (ctUCS2_els00 == LCI.loiFileEncoding_ls00) || (ctUCS2Swapped_els00 == LCI.loiFileEncoding_ls00) )
+            if (true == ls00IsUCS2(&LCI.loiFileStruct_ls00->vfFileEncoding_ls00))
             {
                 lBytesToRead *=2;
             }
@@ -1549,13 +1630,13 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
             // char data unicode:   we can read exactly as much data from file as fits into a single packet
             //                      --> no conversion step
             // binary data:         same as for char data unicode
-            if (1 < (szLOAD_LONG_Compatibility_ls08[LCI.loiFileEncoding_ls00][pFB->fbDataType_ls00]) )
+            if (1 < (szLOAD_LONG_Compatibility_ls08[LCI.loiFileStruct_ls00->vfFileEncoding_ls00][pFB->fbDataType_ls00]) )
             {
                 lBytesToRead /= 2;
             }
 
             // 2 Byte alignment of read buffer for UCS2 chars
-            if ( (ctUCS2_els00 == LCI.loiFileEncoding_ls00) || (ctUCS2Swapped_els00 == LCI.loiFileEncoding_ls00) )
+            if (true == ls00IsUCS2(&LCI.loiFileStruct_ls00->vfFileEncoding_ls00))
             {
                 lBytesToRead = (0 == (lBytesToRead % 2)) ? lBytesToRead : (lBytesToRead - 1);
             }
@@ -1576,27 +1657,43 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
     //*     Read data from data file (taking necessary subsequent conversion to UCS2/from HEX into account) 
     //* We need to move the offset for reading by the amount of already read data
     //*
+    tsp00_Int4 mbcount = 0;
     if (0 != ReadLen)
     {
-        tls00_LargeNumber foffset = ls07AddLargeNumbersEx(LCI.loiFileOffset_ls00, LCI.loiCountOfSendData_ls00);
-        
         if ( (true == ExternalValueIsHex)
              ||
              ( (true == bUnicodeOI) &&
-               (1 < szLOAD_LONG_Compatibility_ls08[LCI.loiFileEncoding_ls00][pFB->fbDataType_ls00]) ) )
+               (1 < szLOAD_LONG_Compatibility_ls08[LCI.loiFileStruct_ls00->vfFileEncoding_ls00][pFB->fbDataType_ls00]) ) )
         {
             pszBuffer = pszConvertBuffer;
         }
 
-        ReadLen = ls30VFReadLongBin(REINTERPRET_CAST(char*, pszBuffer),
-                                    *LCI.loiFileStruct_ls00,
-                                    ReadLen,
-                                    &foffset,
-                                    pszErrText);
-        if (ReadLen < 0)
+
+        // MAGIC BYTEs handling
+        if (false == ls30VFIsMagicBytesCacheEmpty(LCI.loiFileStruct_ls00))
         {
-            // the negative value ReadLen IS the error code for reading file
-            rc = ReadLen;
+            mbcount = ReadLen;
+            ls30VFGetMagicBytes(pszBuffer, LCI.loiFileStruct_ls00, mbcount);
+        }
+
+        tls00_LargeNumber foffset = ls07AddLargeNumbersEx(LCI.loiFileOffset_ls00, LCI.loiCountOfSendData_ls00);
+
+        if (ReadLen > mbcount)      // Read only if the whole LONG content is not yet in the magic bytes cache
+        {
+            ReadLen = ls30VFReadLongBin(REINTERPRET_CAST(char*, pszBuffer + mbcount),
+                                        *LCI.loiFileStruct_ls00,
+                                        ReadLen - mbcount,
+                                        &foffset,
+                                        pszErrText);
+            if (ReadLen < 0)
+            {
+                // the negative value ReadLen IS the error code for reading file
+                rc = ReadLen;
+            }
+            else
+            {
+                ReadLen += mbcount;
+            }
         }
     }   // end if (0 != ReadLen)
 
@@ -1621,6 +1718,15 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
 
         // save LONG Data length; it is the right length in unicode case, too
         LCI.loiCountOfSendData_ls00 += ReadLen;
+
+        // adjust the file offset again if mbcount is gt 0; otherwise exactly this count
+        // of data (mbcount) is not loaded if the data length exceeds the size of a single
+        // order interface packet;
+        // mbcount can only be gt 0 if the first bytes of a files are loaded
+        if (mbcount > 0)
+        {
+            LCI.loiFileOffset_ls00.lnuRemainder_ls00 -= mbcount;
+        }
     }
 
 
@@ -1638,8 +1744,7 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
         //*
         //*     In case it is UCS2 and hex - first conversion from UCS2 to ASCII hex readable
         //*
-        if ( (ctUCS2_els00 == LCI.loiFileEncoding_ls00) ||
-             (ctUCS2Swapped_els00 == LCI.loiFileEncoding_ls00) )
+        if (true == ls00IsUCS2(&LCI.loiFileStruct_ls00->vfFileEncoding_ls00))
         {
             len = pConversionBuffer->lBufLen_ls00;
     
@@ -1655,7 +1760,7 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
         //*
         len = LongDataBuffer.lBufSize_ls00;
         if ( (false == bUnicodeOI) ||
-             (1 == szLOAD_LONG_Compatibility_ls08[LCI.loiFileEncoding_ls00][pFB->fbDataType_ls00]) )
+             (1 == szLOAD_LONG_Compatibility_ls08[LCI.loiFileStruct_ls00->vfFileEncoding_ls00][pFB->fbDataType_ls00]) )
         {
             pszBuffer = &(STATIC_CAST(unsigned char*, LongDataBuffer.pBufPtr_ls00)[LongDataBuffer.lBufLen_ls00]);
         }
@@ -1667,7 +1772,7 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
         }
         else
         {
-            RepServMessages::Instance()->ls98Msg(layDataLoad_els98, pszErrText, STATIC_CAST(ErrorNumbers,rc),
+            LoaderMessages::Instance()->ls98Msg(layDataLoad_els98, pszErrText, STATIC_CAST(ErrorNumbers,rc),
                                                                     ReadLen,
                                                                     pszConvertBuffer);
         }
@@ -1679,7 +1784,7 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
     //*
     if ( (errOK_els00 == rc) && (true == bUnicodeOI) )
     {
-        if (1 < szLOAD_LONG_Compatibility_ls08[LCI.loiFileEncoding_ls00][pFB->fbDataType_ls00])
+        if (1 < szLOAD_LONG_Compatibility_ls08[LCI.loiFileStruct_ls00->vfFileEncoding_ls00][pFB->fbDataType_ls00])
         {
             pszBuffer = &(STATIC_CAST(unsigned char*, LongDataBuffer.pBufPtr_ls00)[LongDataBuffer.lBufLen_ls00]);
 
@@ -1690,7 +1795,7 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
                                    lDestLen,
                                    pszConvertBuffer,
                                    STATIC_CAST(tsp00_Int4, ReadLen),
-                                   LCI.loiFileEncoding_ls00,
+                                   LCI.loiFileStruct_ls00->vfFileEncoding_ls00,
                                    pFB->fbDataType_ls00,
                                    pszErrText,
                                    pCodePage);
@@ -1699,6 +1804,16 @@ ls08_FillLong(tls00_Buffer         &LongDataBuffer,
                 ReadLen = STATIC_CAST(tsp00_Longint, lDestLen);
             }
         }
+        else if (true == ls00IsUCS2(&LCI.loiFileStruct_ls00->vfFileEncoding_ls00) )
+        {
+            tsp00_SwapKind_Enum Swap = ls07DefineSwap(NULL);
+            if ( ( (sw_full_swapped == Swap) && (ctUCS2_els00 == LCI.loiFileStruct_ls00->vfFileEncoding_ls00) )
+                 ||
+                 ( (sw_normal == Swap)       && (ctUCS2Swapped_els00 == LCI.loiFileStruct_ls00->vfFileEncoding_ls00) ) )
+            {
+                ls07SwapUCS2StringInPlace(REINTERPRET_CAST(char*, pszBuffer), ReadLen);
+            }
+        }
     }
 
     // Check for errors and adapt buffer length and size
