@@ -1,25 +1,16 @@
 /* 
   -----------------------------------------------------------------------------
- 
   module: vls13.cpp
- 
   -----------------------------------------------------------------------------
  
-  responsible:  SteffenS and d033893
+  responsible:  SteffenS, d033893
 
-  special area: SAP DB LOADER
+  special area: Loader
  
   description:  Implementation of support functions
                 
-  version:      7.4.
-
+  version:      7.5.
   -----------------------------------------------------------------------------
- 
-  copyright:    (c) 1997-2004 SAP AG-2004
- 
-  -----------------------------------------------------------------------------
-
-
 
     ========== licence begin  GPL
     Copyright (c) 1997-2004 SAP AG
@@ -38,9 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 #include <ctype.h>
 #include <assert.h>
@@ -104,42 +92,6 @@ ls13_GetRawDescription(tls00_DBInfo *DBInfo,
   function:     ls13_PutCmd
   -----------------------------------------------------------------------------
 
-  description:  Initializes a message packet and puts the command (first part)
-                in.
-                Converts command to UNICODE if db is unicode.
-
-  arguments:    DBInfo       [IN]  - structure with all info on the running
-                                      session and the database connected to
-                lPcktNo      [IN]  - packet number to use for init
-                pszSqlStmt   [IN]  - command to put into packet
-                pszErrText   [OUT] - error text
-                CmdProducer  [IN]  - type of producer (defaults to sp1pr_internal_cmd)
-                bWithInfo    [IN]  - with info flag (defaults to false)
-                MessType     [IN]  - message type (defaults to sp1m_dbs)
-                bMassCmd     [IN]  - mass command flag (defaults to false)
-
-  returnvalue:  errOK_els00                  - no error
-                errConvertingEncoding_els98  - error converting stmt text from
-                                                UTF8 to UCS2
-                errFinishPart_els98          - error finishing command part
-                Logs the error to log file and to return string.
-  -----------------------------------------------------------------------------
-*/
-static tsp00_Int4
-ls13_PutCmd(tls00_DBInfo            *pDBInfo,
-            tsp00_Int4               lPcktNo,
-            tsp00_Addr               pszSqlStmt,
-            tsp00_Addr               pszErrText,
-            tsp1_producer_Enum       CmdProducer = sp1pr_internal_cmd,
-            bool                     bWithInfo = false,
-            tsp1_cmd_mess_type_Enum  MessType = sp1m_dbs,
-            bool                     bMassCmd = false);
-
-/*
-  -----------------------------------------------------------------------------
-  function:     ls13_PutCmdUTF8
-  -----------------------------------------------------------------------------
-
   description:  Initializes a message packet and puts the command (first part)
                 in. The command must be UTF8 encoded.
                 Converts command to UNICODE if db is unicode.
@@ -162,15 +114,14 @@ ls13_PutCmd(tls00_DBInfo            *pDBInfo,
   -----------------------------------------------------------------------------
 */
 static tsp00_Int4
-ls13_PutCmdUTF8(tls00_DBInfo            *pDBInfo,
-                tsp00_Int4               lPcktNo,
-                tsp00_Addr               pszSqlStmt,
-                tsp00_Addr               pszErrText,
-                tsp1_producer_Enum       CmdProducer = sp1pr_internal_cmd,
-                bool                     bWithInfo   = false,
-                tsp1_cmd_mess_type_Enum  MessType    = sp1m_dbs,
-                bool                     bMassCmd    = false);
-
+ls13_PutCmd(tls00_DBInfo            *pDBInfo,
+            tsp00_Int4               lPcktNo,
+            const char*              pszSqlStmt,
+            tsp00_Addr               pszErrText,
+            tsp1_producer_Enum       CmdProducer = sp1pr_internal_cmd,
+            bool                     bWithInfo   = false,
+            tsp1_cmd_mess_type_Enum  MessType    = sp1m_dbs,
+            bool                     bMassCmd    = false);
 
 /*
   -----------------------------------------------------------------------------
@@ -189,10 +140,10 @@ ls13_PutCmdUTF8(tls00_DBInfo            *pDBInfo,
   -----------------------------------------------------------------------------
 */
 static void
-ls13_GetCorrectErrorPos(tls00_DBSqlCmd* pSqlCmd,
-                        tsp00_Int4      lErrPos,
-                        tsp00_Int4     &lRow,
-                        tsp00_Int4     &lCol);
+ls13_GetCorrectErrorPos(const char*   pszSqlCmd,
+                        tsp00_Int4    lErrPos,
+                        tsp00_Int4   &lRow,
+                        tsp00_Int4   &lCol);
 
 /*  
   -----------------------------------------------------------------------------
@@ -218,7 +169,7 @@ int ls13ExecExtSQLStmt(tls00_DBInfo *DBInfo, tls00_DBSqlCmd *SqlCmd, tsp00_Addr
     TRACE_PRNF_MLS99(("ls13ExecExtSQLStmt", "Executing external SQL statement '%s'",
                                             SqlCmd->dscSqlStmt_ls00));
 
-    RepServMessages *MsgInstance = RepServMessages::Instance(); // error reporting obj
+    LoaderMessages *MsgInstance = LoaderMessages::Instance(); // error reporting obj
 
     tsp00_Int4 rc = errOK_els00;
     
@@ -253,7 +204,7 @@ int ls13ExecExtSQLStmt(tls00_DBInfo *DBInfo, tls00_DBSqlCmd *SqlCmd, tsp00_Addr
             {
                 tsp00_Int4 lRow = 0;
                 tsp00_Int4 lCol = 0;
-                ls13_GetCorrectErrorPos(SqlCmd, lErrorPos, lRow, lCol);
+                ls13_GetCorrectErrorPos(SqlCmd->dscSqlStmt_ls00, lErrorPos, lRow, lCol);
                 // this comes always in form "error text ... (error position: <pos>)\n"
                 // add some more accurate info about error location
                 char*      _pbracket = strrchr(szErr, ')');
@@ -273,6 +224,68 @@ int ls13ExecExtSQLStmt(tls00_DBInfo *DBInfo, tls00_DBSqlCmd *SqlCmd, tsp00_Addr
 // ls13ExecExtSQLStmt()
 
 
+/*
+  -----------------------------------------------------------------------------
+  function:     ls13ExecExtSQLStmt
+  -----------------------------------------------------------------------------
+*/
+// Incoming sql command has to be UTF8 encoded
+int ls13ExecExtSQLStmt(tls00_DBInfo *pDBInfo, const char* pszSqlCmd, tsp00_Addr ErrText)
+{
+    ROUTINE_DBG_MEO00 ("ls13ExecExtSQLStmt");
+
+    TRACE_PRNF_MLS99(("ls13ExecExtSQLStmt", "Executing external SQL statement '%s'", pszSqlCmd));
+
+    LoaderMessages *MsgInstance = LoaderMessages::Instance(); // error reporting obj
+
+    tsp00_Int4 rc = errOK_els00;
+    
+    //*
+    //*     Initialize/fill packet for statement
+    //*
+    //* message type = sp1m_dbs (= default);  with info = false (= default);
+    //* Mass command = false    (= default)
+    //*
+    //* sp1pr_user_cmd is the fundamental difference to executing internal commands
+    rc = ls13_PutCmd(pDBInfo, 0, pszSqlCmd, ErrText, sp1pr_user_cmd);
+    if (errOK_els00 == rc)
+    {
+        //*
+        //*     send SqlPacket, receive answer packet and analyse latter
+        //*
+        tsp00_Int4  lErrorPos = 0;
+        _TCHAR     szErr[MAX_REPLY_LENGTH_LS00];
+        szErr[0] = 0;
+
+        rc = ls03ProcessSQLPacket(pDBInfo, 0, lErrorPos, szErr);
+        if (errOK_els00 != rc)
+        {
+            if (1 == lErrorPos)
+            {
+                MsgInstance->ls98Msg(laySQLInterface_els98, ErrText, errSimpleSQL_els98, szErr);
+            }
+            else
+            {
+                tsp00_Int4 lRow = 0;
+                tsp00_Int4 lCol = 0;
+                ls13_GetCorrectErrorPos(pszSqlCmd, lErrorPos, lRow, lCol);
+                // this comes always in form "error text ... (error position: <pos>)\n"
+                // add some more accurate info about error location
+                char*      _pbracket = strrchr(szErr, ')');
+                tsp00_Int4 _len      = _pbracket - szErr;
+                sp77sprintf(szErr + _len, MAX_REPLY_LENGTH_LS00 - _len,
+                                           "[line: %d, col: %d])\n",
+                                           lRow, lCol);
+                MsgInstance->ls98Msg(laySQLInterface_els98, ErrText, errSimpleSQL_els98, szErr);
+            }
+        }
+    }
+
+    return rc;
+}
+// ls13ExecExtSQLStmt()
+
+
 /*
   -----------------------------------------------------------------------------
   function:     ls13ExecIntSQLStmt      
@@ -298,139 +311,6 @@ ls13ExecIntSQLStmt(tls00_DBInfo            *DBInfo,
 }
 // ls13ExecIntSQLStmt()
 
-/*
-  -----------------------------------------------------------------------------
-  function:     ls13ExecIntSQLStmtUTF8
-  -----------------------------------------------------------------------------
-*/
-int
-ls13ExecIntSQLStmtUTF8(tls00_DBInfo            *DBInfo,
-                       tsp00_Int4               lPcktNo,
-                       tsp00_Addr               pszSqlStmt,
-                       tsp00_Addr               pszErrText,
-                       tsp00_Int4              &lErrPos,
-                       bool                     bWithInfo,  // = false
-                       tsp1_cmd_mess_type_Enum  MessType,   // = sp1m_dbs
-                       bool                     bMassCmd)   // = false
-{
-    ROUTINE_DBG_MEO00 ("ls13ExecIntSQLStmtUTF8");
-
-    tsp00_Int4 rc = errOK_els00;
-
-    rc = ls13ExecIntSQLStmtExUTF8(DBInfo, lPcktNo, pszSqlStmt, NULL, 0, pszErrText, lErrPos,
-                                  sp1pk_command, bWithInfo, MessType, bMassCmd);
-    return rc;
-}
-// ls13ExecIntSQLStmtUTF8()
-
-
-/*
-  -----------------------------------------------------------------------------
-  function:     ls13ExecIntSQLPrepared
-  -----------------------------------------------------------------------------
-*/
-tsp00_Int4
-ls13ExecIntSQLPrepared(tls00_DBInfo            *DBInfo,
-                       tsp00_Int4               lPcktNo,
-                       tls00_Parts4InternalSql &PartStruct,
-                       tsp00_Addr               pszErrText,
-                       tsp00_Int4              &lErrPos,
-                       bool                     bWithInfo,  // = false
-                       tsp1_cmd_mess_type_Enum  MessType,   // = sp1m_execute
-                       bool                     bMassCmd)   // = false
-{
-    ROUTINE_DBG_MEO00 ("ls13ExecIntSQLPrepared");
-
-    TRACE_PRNF_MLS99(("ls13ExecIntSQLPrepared", "Executing internal prepared SQL statement"));
-
-    RepServMessages *pMsgObj = RepServMessages::Instance(); // error reporting obj
-
-    tsp00_Int4 rc = errOK_els00;
-
-    // Check if first data is really a parse id
-    assert(sp1pk_parsid == PartStruct.pfiPartKind_ls00[0]);
-
-    //*
-    //*     Initialize packet
-    //*
-    tsp1_packet  *pPacket = DBInfo->dbiPktSndList_ls00[lPcktNo];
-    tsp1_segment *pSegm   = NULL;
-    tsp1_part    *pPart   = NULL;
-    
-    //*
-    //*     Fill packet with parse id (1. part)
-    //*
-    ls04FastInit(pPacket, &pSegm, &pPart, DBInfo, sp1pr_internal_cmd);
-
-    if (sp1m_dbs != MessType)
-    {
-        ls04SetMsgType(pSegm, MessType);
-    }
-
-    if (false != bWithInfo)
-    {
-        ls04SetWInfoFlag(pSegm, bWithInfo);
-    }
-
-    if (false != bMassCmd)
-    {
-        ls04SetMCFlag(pSegm, bMassCmd);
-    }
-
-    int i = 0;
-    ls04BufToPart(pPart, PartStruct.pfiData_ls00[i].strAddr_ls00, PartStruct.pfiData_ls00[i].strLeng_ls00);
-
-    rc = ls04FinishPart(pPacket, pPart);
-    if (errOK_els00 != rc)
-    {
-        pMsgObj->ls98Msg(laySQLInterface_els98, pszErrText, errInternal_els98,
-                                                            _INTERNAL_FILE_ID_,
-                                                            __LINE__,
-                                                            errFinishPart_els98);
-        rc = errInternal_els98;
-    }
-
-
-    //*
-    //*     Fill remaining parts
-    //*
-    if (errOK_els00 == rc)
-    {
-        if (PartStruct.pfiCount_ls00 > 1)
-        {
-            for (i=1; i < PartStruct.pfiCount_ls00; ++i)
-            {
-                pPart = NULL;             // reinitialize
-
-                rc = ls04InitPart(pPacket, pSegm, &pPart, PartStruct.pfiPartKind_ls00[i]);
-                if (errOK_els00 != rc)
-                {
-                    break;
-                }
-                ls04BufToPart(pPart, PartStruct.pfiData_ls00[i].strAddr_ls00, PartStruct.pfiData_ls00[i].strLeng_ls00);
-
-                rc = ls04FinishPart(pPacket, pPart);
-                if (errOK_els00 != rc)
-                {
-                    break;
-                }
-            }
-            
-            if (errOK_els00 != rc)
-            {
-                pMsgObj->ls98Msg(laySQLInterface_els98, pszErrText, STATIC_CAST(ErrorNumbers, rc));
-            }
-        }
-
-        //*
-        //*     Send und receive sql packet
-        //*
-        rc = ls03ProcessSQLPacket(DBInfo, lPcktNo, lErrPos, pszErrText);
-    }
-
-    return rc;
-}
-// ls13ExecIntSQLPrepared()
 
 /*
   -----------------------------------------------------------------------------
@@ -454,7 +334,7 @@ ls13ExecIntSQLStmtEx(tls00_DBInfo            *DBInfo,
 
     TRACE_PRNF_MLS99(("ls13ExecIntSQLStmt", "Executing internal SQL statement '%s'", pszSqlStmt));
 
-    RepServMessages *MsgInstance = RepServMessages::Instance(); // error reporting obj
+    LoaderMessages *MsgInstance = LoaderMessages::Instance(); // error reporting obj
 
     tsp00_Int4 rc = errOK_els00;
     
@@ -497,71 +377,6 @@ ls13ExecIntSQLStmtEx(tls00_DBInfo            *DBInfo,
 }
 // ls13ExecIntSQLStmtEx()
 
-/*
-  -----------------------------------------------------------------------------
-  function:     ls13ExecIntSQLStmtExUTF8
-  -----------------------------------------------------------------------------
-*/
-tsp00_Int4
-ls13ExecIntSQLStmtExUTF8(tls00_DBInfo            *DBInfo,
-                         tsp00_Int4               lPcktNo,
-                         tsp00_Addr               pszSqlStmt,
-                         tsp00_Addr               pszData,    // always of type char*
-                         tsp00_Int4               lDataLength,
-                         tsp00_Addr               pszErrText,
-                         tsp00_Int4              &lErrPos,
-                         tsp1_part_kind_Enum      PartKind,
-                         bool                     bWithInfo,  // = false
-                         tsp1_cmd_mess_type_Enum  MessType,   // = sp1m_dbs
-                         bool                     bMassCmd)   // = false
-{
-    ROUTINE_DBG_MEO00 ("ls13ExecIntSQLStmtExUTF8");
-
-    TRACE_PRNF_MLS99(("ls13ExecIntSQLStmtExUTF8", "Executing internal SQL statement '%s'", pszSqlStmt));
-
-    RepServMessages *MsgInstance = RepServMessages::Instance(); // error reporting obj
-
-    tsp00_Int4 rc = errOK_els00;
-    
-    rc = ls13_PutCmdUTF8(DBInfo, lPcktNo, pszSqlStmt, pszErrText, sp1pr_internal_cmd, bWithInfo, MessType, bMassCmd);
-    if (errOK_els00 == rc)
-    {
-        //*
-        //*     2.part - data part
-        //*
-        //* In case lDataLength = 0 there is no 2nd part to fill
-        if (0 != lDataLength)
-        {
-            tsp1_packet  *SqlPacket = DBInfo->dbiPktSndList_ls00[lPcktNo];
-            tsp1_segment *SqlSegm   = ls04GetSegment(SqlPacket);
-            tsp1_part    *SqlPart   = NULL;
-
-            rc = ls04InitPart(SqlPacket, SqlSegm, &SqlPart, PartKind);
-            if (errOK_els00 == rc)
-            {
-                ls04BufToPart(SqlPart, pszData, lDataLength);
-    
-                rc = ls04FinishPart(SqlPacket, SqlPart);
-            }
-        }
-
-        //*
-        //*     Send und receive sql packet
-        //*
-        if (errOK_els00 != rc)
-        {
-            MsgInstance->ls98Msg(laySQLInterface_els98, pszErrText, STATIC_CAST(ErrorNumbers,rc));
-        }
-        else
-        {
-            rc = ls03ProcessSQLPacket(DBInfo, lPcktNo, lErrPos, pszErrText);
-        }
-    }
-
-    return rc;
-}
-// ls13ExecIntSQLStmtExUTF8()
-
 
 /*
   -----------------------------------------------------------------------------
@@ -582,7 +397,7 @@ ls13ExecIntSQLMultParts(tls00_DBInfo            *DBInfo,
 
     TRACE_PRNF_MLS99(("ls13ExecIntSQLStmt", "Executing internal SQL statement"));
 
-    RepServMessages *MsgInstance = RepServMessages::Instance(); // error reporting obj
+    LoaderMessages *MsgInstance = LoaderMessages::Instance(); // error reporting obj
 
     tsp00_Int4 rc = errOK_els00;
 
@@ -647,88 +462,6 @@ ls13ExecIntSQLMultParts(tls00_DBInfo            *DBInfo,
 // ls13ExecIntSQLMultParts()
 
 
-/*
-  -----------------------------------------------------------------------------
-  function:     ls13ExecIntSQLMultPartsUTF8
-  -----------------------------------------------------------------------------
-*/
-tsp00_Int4
-ls13ExecIntSQLMultPartsUTF8(tls00_DBInfo            *DBInfo,
-                            tsp00_Int4               lPcktNo,
-                            tls00_Parts4InternalSql &PartStruct,
-                            tsp00_Addr               pszErrText,
-                            tsp00_Int4              &lErrPos,
-                            bool                     bWithInfo,  // = false
-                            tsp1_cmd_mess_type_Enum  MessType,   // = sp1m_dbs
-                            bool                     bMassCmd)   // = false
-{
-    ROUTINE_DBG_MEO00 ("ls13ExecIntSQLMultPartsUTF8");
-
-    TRACE_PRNF_MLS99(("ls13ExecIntSQLMultPartsUTF8", "Executing internal SQL statement"));
-
-    tsp00_Int4 rc = errOK_els00;
-
-    // Check if first data is really a statement
-    assert(sp1pk_command == PartStruct.pfiPartKind_ls00[0]);
-
-    //*
-    //*     Fill parts - first one MUST be a statement
-    //*
-    rc = ls13_PutCmdUTF8(DBInfo,
-                         lPcktNo,
-                         PartStruct.pfiData_ls00[0].strAddr_ls00,
-                         pszErrText,
-                         sp1pr_internal_cmd,
-                         bWithInfo,
-                         MessType,
-                         bMassCmd);
-    if (errOK_els00 == rc)
-    {
-        //*
-        //*     Fill remaining parts
-        //*
-        if (PartStruct.pfiCount_ls00 > 1)
-        {
-            tsp1_packet  *SqlPacket = DBInfo->dbiPktSndList_ls00[lPcktNo];
-            tsp1_segment *SqlSegm   = ls04GetSegment(SqlPacket);
-            tsp1_part    *SqlPart   = NULL;
-
-            int i = 1;
-            for (i; i < PartStruct.pfiCount_ls00; ++i)
-            {
-                SqlPart = NULL;             // reinitialize
-
-                rc = ls04InitPart(SqlPacket, SqlSegm, &SqlPart, PartStruct.pfiPartKind_ls00[i]);
-                if (errOK_els00 != rc)
-                {
-                    break;
-                }
-                ls04BufToPart(SqlPart, PartStruct.pfiData_ls00[i].strAddr_ls00, PartStruct.pfiData_ls00[i].strLeng_ls00);
-
-                rc = ls04FinishPart(SqlPacket, SqlPart);
-                if (errOK_els00 != rc)
-                {
-                    break;
-                }
-            }
-            
-            if (errOK_els00 != rc)
-            {
-                RepServMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText, STATIC_CAST(ErrorNumbers, rc));
-            }
-        }
-
-        //*
-        //*     Send und receive sql packet
-        //*
-        rc = ls03ProcessSQLPacket(DBInfo, lPcktNo, lErrPos, pszErrText);
-    }
-
-    return rc;
-}
-// ls13ExecIntSQLMultPartsUTF8()
-
-
 /*
   -----------------------------------------------------------------------------
   function:     ls13Parse
@@ -745,7 +478,7 @@ ls13Parse(tsp00_Addr              pszCmd,
 {
     ROUTINE_DBG_MEO00 ("ls13GetTableDescription");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance(); // error reporting obj
+    LoaderMessages *pMsgObj = LoaderMessages::Instance(); // error reporting obj
     tsp00_Longint    rc      = errOK_els00;
 
     tsp00_Int4 ErrPos = 0;
@@ -797,7 +530,7 @@ ls13GetTableDescription(tls00_DBInfo            *pDBInfo,
 {
     ROUTINE_DBG_MEO00 ("ls13GetTableDescription");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance(); // error reporting obj
+    LoaderMessages *pMsgObj = LoaderMessages::Instance(); // error reporting obj
     tsp00_Int4       rc      = errOK_els00;
 
     char   szErr[MAX_REPLY_LENGTH_LS00];
@@ -823,6 +556,7 @@ ls13GetTableDescription(tls00_DBInfo            *pDBInfo,
 }
 // ls13GetTableDescription()
 
+
 /*
   -----------------------------------------------------------------------------
   function:     ls13GetIndexDescription
@@ -838,7 +572,7 @@ ls13GetIndexDescription(tls00_DBInfo            *pDBInfo,
 {
     ROUTINE_DBG_MEO00 ("ls13GetIndexDescription");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance(); // error reporting obj
+    LoaderMessages *pMsgObj = LoaderMessages::Instance(); // error reporting obj
     tsp00_Int4       rc      = errOK_els00;
 
     char   szErr[MAX_REPLY_LENGTH_LS00];
@@ -1041,7 +775,7 @@ ls13RetrieveIndexNames(tls00_DBInfo            *pDBInfo,
         // of column names because source and target database MUST be of the same
         // encoding.
         tls00_CodeType SrcEncoding = ctAscii_els00;
-        if (true == ls00IsDBUnicode(pDBInfo))
+        if (true == ls00UseUnicodeOI(pDBInfo))
         {
             SrcEncoding = ctUCS2_els00;     // NOT SWAPPED !!!
         }
@@ -1173,10 +907,10 @@ ls13DropAllIndexesOnTable(tls00_DBInfo            *pDBInfo,
             // genereated statement is already UTF8!
 
             // execute stmt
-            rc = ls13ExecIntSQLStmtUTF8(pDBInfo, 0, szStmt, szErr, lErrPos);
+            rc = ls13ExecIntSQLStmt(pDBInfo, 0, szStmt, szErr, lErrPos);
             if (errOK_els00 != rc)
             {
-                RepServMessages::Instance()->ls98Msg(layTableLoad_els98, pszErrText, errInternSQL_els98,
+                LoaderMessages::Instance()->ls98Msg(layTableLoad_els98, pszErrText, errInternSQL_els98,
                                                                                      szStmt, szErr);
                 rc = errInternSQL_els98;
             }
@@ -1230,7 +964,7 @@ ls13CreateIndexes(tls00_DBInfo            *pDBInfo,
         //* of column names because source and target database MUST be of the same encoding.
         //*
         tls00_CodeType SrcEncoding = ctAscii_els00;
-        if (true == ls00IsDBUnicode(pDBInfo))
+        if (true == ls00UseUnicodeOI(pDBInfo))
         {
             SrcEncoding = ctUCS2_els00;
         }
@@ -1318,10 +1052,10 @@ ls13CreateIndexes(tls00_DBInfo            *pDBInfo,
             //*
             //*     Run CREATE INDEX statement
             //*
-            rc = ls13ExecIntSQLStmtUTF8(pDBInfo, 0, szStmt, szErr, lErrPos);
+            rc = ls13ExecIntSQLStmt(pDBInfo, 0, szStmt, szErr, lErrPos);
             if (0 != rc)
             {
-                RepServMessages::Instance()->ls98Msg(layTableLoad_els98, pszErrText, errInternSQL_els98,
+                LoaderMessages::Instance()->ls98Msg(layTableLoad_els98, pszErrText, errInternSQL_els98,
                                                                                      szStmt, szErr);
                 rc = errInternSQL_els98;
             }
@@ -1376,15 +1110,15 @@ ls13GetDefaultValue(tls00_DBInfo             *pDBInfo,
 
         // For defining the default we send "DESCRIBE DEFAULT <table name>" plus column no
         // of column for which the default is defined
-        rc = ls13ExecIntSQLStmtExUTF8(pDBInfo, 0, szCmd, REINTERPRET_CAST(_TCHAR*, &lColumnNo),
-                                      sizeof(tsp00_Int2), pszErrText, lErrorPos);
+        rc = ls13ExecIntSQLStmtEx(pDBInfo, 0, szCmd, REINTERPRET_CAST(_TCHAR*, &lColumnNo),
+                                  sizeof(tsp00_Int2), pszErrText, lErrorPos);
         if (errOK_els00 == rc)
         {
             tsp1_packet *pSqlPacket = pDBInfo->dbiPktRcvList_ls00[0];
             tsp1_part   *pSqlPart   = NULL;
             if ( false == ls04GetPart(pSqlPacket, sp1pk_data, pSqlPart) )
             {
-                RepServMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText, errInternal_els98,
+                LoaderMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText, errInternal_els98,
                                                                                         __FILE__,
                                                                                         __LINE__,
                                                                                         rc);
@@ -1431,7 +1165,7 @@ ls13GetDefaultValue(tls00_DBInfo             *pDBInfo,
         }   // if (errOK_els00 == rc)
         else
         {
-            RepServMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText, errInternal_els98,
                                                                                     __FILE__,
                                                                                     __LINE__,
                                                                                     rc);
@@ -1474,7 +1208,7 @@ ls13CheckTableExistence(tls00_DBInfo            *pDBInfo,
     tsp00_Int4 lErrPos;
     _TCHAR     szErr[MAX_REPLY_LENGTH_LS00];
 
-    rc = ls13ExecIntSQLStmtUTF8(pDBInfo, 0, szCmd, szErr, lErrPos);
+    rc = ls13ExecIntSQLStmt(pDBInfo, 0, szCmd, szErr, lErrPos);
     if (errOK_els00 != rc)
     {
         if (dbUnknownTableName_els00 == rc)
@@ -1483,7 +1217,7 @@ ls13CheckTableExistence(tls00_DBInfo            *pDBInfo,
         }
         else
         {
-            RepServMessages::Instance()->ls98Msg(layTableLoad_els98, pszErrText, errCheckTableExistence_els98,
+            LoaderMessages::Instance()->ls98Msg(layTableLoad_els98, pszErrText, errCheckTableExistence_els98,
                                                                                  TableName.StrPtr(), szErr);
             rc = errCheckTableExistence_els98;
         }
@@ -1639,7 +1373,7 @@ ls13_GetRawDescription(tls00_DBInfo *DBInfo,
     tsp00_Int4 rc       = errOK_els00;
     tsp00_Int4 ErrorPos = 0;
 
-    rc = ls13ExecIntSQLStmtUTF8(DBInfo, lPacketNo, pszCmd, pszErrText, ErrorPos);
+    rc = ls13ExecIntSQLStmt(DBInfo, lPacketNo, pszCmd, pszErrText, ErrorPos);
     if (errOK_els00 == rc)
     {
         rc = ls04GetRawDataPtr(DBInfo->dbiPktRcvList_ls00[lPacketNo], pszDescription, lDescriptionLen);
@@ -1662,7 +1396,7 @@ ls13_GetRawDescription(tls00_DBInfo *DBInfo,
 static tsp00_Int4
 ls13_PutCmd(tls00_DBInfo            *pDBInfo,
             tsp00_Int4               lPcktNo,
-            tsp00_Addr               pszSqlStmt,
+            const char*              pszSqlStmt,
             tsp00_Addr               pszErrText,
             tsp1_producer_Enum       CmdProducer,
             bool                     bWithInfo,  // = false
@@ -1670,115 +1404,18 @@ ls13_PutCmd(tls00_DBInfo            *pDBInfo,
             bool                     bMassCmd)   // = false
 {
     tsp00_Longint rc = errOK_els00;
-    
-    size_t  lStmtLen    = _tcslen(pszSqlStmt);
-    _TCHAR *pszSqlStmtU = NULL;
-    
-    if (true == ls00IsDBUnicode(pDBInfo))       // Check for unicode and convert to UCS2 if necessary
-    {
-        tsp00_Longuint lStmtLenU = 2*lStmtLen;
-        pszSqlStmtU = new _TCHAR[lStmtLenU];
-        
-        rc = ls05ConvertToUCS2Simple(REINTERPRET_CAST(unsigned char*, pszSqlStmtU),
-                                     lStmtLenU,
-                                     REINTERPRET_CAST(unsigned char*, pszSqlStmt),
-                                     STATIC_CAST(tsp00_Longint, lStmtLen),
-                                     ctAscii_els00,
-                                     pszErrText);
-        if (errOK_els00 == rc)
-        {
-            lStmtLen = lStmtLenU;
-        }
-    }
-    else
-    {
-        pszSqlStmtU = pszSqlStmt;
-    }
-
-    if (errOK_els00 == rc)
-    {
-        tsp1_packet  *pPacket = pDBInfo->dbiPktSndList_ls00[lPcktNo];
-        tsp1_segment *pSegm   = NULL;
-        tsp1_part    *pPart   = NULL;
-    
-        //*
-        //*     Fill packet with command (1. part)
-        //*
-        ls04FastInit(pPacket, &pSegm, &pPart, pDBInfo, CmdProducer);
-
-        if (sp1m_dbs != MessType)
-        {
-            ls04SetMsgType(pSegm, MessType);
-        }
-    
-        if (false != bWithInfo)
-        {
-            ls04SetWInfoFlag(pSegm, bWithInfo);
-        }
-
-        if (false != bMassCmd)
-        {
-            ls04SetMCFlag(pSegm, bMassCmd);
-        }
-
-        rc = ls04BufToPart(pPart, pszSqlStmtU, STATIC_CAST(tsp00_Int4, lStmtLen));
-        if (errOK_els00 != rc)
-        {
-            RepServMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText, errSQLStmtTooLong_els98);
-            rc = errSQLStmtTooLong_els98;
-        }
-        else
-        {
-            rc = ls04FinishPart(pPacket, pPart);
-            if (errOK_els00 != rc)
-            {
-                RepServMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText,
-                                                                            errInternal_els98,
-                                                                            _INTERNAL_FILE_ID_,
-                                                                            __LINE__,
-                                                                            errFinishPart_els98);
-                rc = errInternal_els98;
-            }
-        }
-    }
-
-    if ( (NULL != pszSqlStmtU) && (pszSqlStmtU != pszSqlStmt) )      // some cleaning work
-    {
-        delete [] pszSqlStmtU;
-        pszSqlStmtU = NULL;
-    }
-
-    return STATIC_CAST(tsp00_Int4, rc);
-}
-// ls13_PutCmd()
-
-/*
-  -----------------------------------------------------------------------------
-  function:     ls13_PutCmdUTF8
-  -----------------------------------------------------------------------------
-*/
-static tsp00_Int4
-ls13_PutCmdUTF8(tls00_DBInfo            *pDBInfo,
-                tsp00_Int4               lPcktNo,
-                tsp00_Addr               pszSqlStmt,
-                tsp00_Addr               pszErrText,
-                tsp1_producer_Enum       CmdProducer,
-                bool                     bWithInfo,  // = false
-                tsp1_cmd_mess_type_Enum  MessType,   // = sp1m_dbs
-                bool                     bMassCmd)   // = false
-{
-    tsp00_Longint rc = errOK_els00;
 
     Tools_DynamicUTF8String SqlStmtUTF8(REINTERPRET_CAST(Tools_DynamicUTF8String::ConstPointer, pszSqlStmt));
 
 
-    tsp00_Longuint  lStmtLenUTF8 = SqlStmtUTF8.Length();
+    //tsp00_Longuint  lStmtLenUTF8 = SqlStmtUTF8.Length();
+    tsp00_Longuint  lStmtLenUTF8 = SqlStmtUTF8.BasisSize();
     _TCHAR *pszMySqlStmt = NULL;
 
     tsp00_Longuint lStmtLen = 0;
     tls00_CodeType DestCode = ctAscii_els00;
 
-    if (true == ls00IsDBUnicode(pDBInfo))       // Check for unicode and convert to UCS2 if necessary
+    if (true == ls00UseUnicodeOI(pDBInfo))       // Check for encoding of order interface to use
     {
         lStmtLen = 4*lStmtLenUTF8;
         DestCode = ctUCS2Native_els00;          // in case of commands the UCS2 encoded string must be in native swap
@@ -1824,7 +1461,7 @@ ls13_PutCmdUTF8(tls00_DBInfo            *pDBInfo,
         rc = ls04BufToPart(pPart, pszMySqlStmt, STATIC_CAST(tsp00_Int4, lStmtLen));
         if (errOK_els00 != rc)
         {
-            RepServMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText, errSQLStmtTooLong_els98);
+            LoaderMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText, errSQLStmtTooLong_els98);
             rc = errSQLStmtTooLong_els98;
         }
         else
@@ -1832,7 +1469,7 @@ ls13_PutCmdUTF8(tls00_DBInfo            *pDBInfo,
             rc = ls04FinishPart(pPacket, pPart);
             if (errOK_els00 != rc)
             {
-                RepServMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(laySQLInterface_els98, pszErrText,
                                                                             errInternal_els98,
                                                                             _INTERNAL_FILE_ID_,
                                                                             __LINE__,
@@ -1851,7 +1488,7 @@ ls13_PutCmdUTF8(tls00_DBInfo            *pDBInfo,
     return STATIC_CAST(tsp00_Int4, rc);
 
 }
-// ls13_PutCmdUTF8()
+// ls13_PutCmd()
 
 
 /*
@@ -1860,17 +1497,17 @@ ls13_PutCmdUTF8(tls00_DBInfo            *pDBInfo,
   -----------------------------------------------------------------------------
 */
 static void
-ls13_GetCorrectErrorPos(tls00_DBSqlCmd* pSqlCmd,
-                        tsp00_Int4      lErrPos,
-                        tsp00_Int4     &lRow,
-                        tsp00_Int4     &lCol)
+ls13_GetCorrectErrorPos(const char*   pszSqlCmd,  
+                        tsp00_Int4    lErrPos,
+                        tsp00_Int4   &lRow,
+                        tsp00_Int4   &lCol)
 {
     ROUTINE_DBG_MEO00 ("ls13_GetCorrectErrorPos");
 
     lCol = lErrPos;
     lRow = 1;
 
-    char*      pszSqlStmt = pSqlCmd->dscSqlStmt_ls00;
+    char*      pszSqlStmt = CONST_CAST(char*, pszSqlCmd);
     tsp00_Int4 lPos       = 0;
     char*      pdest      = pszSqlStmt;
 
