@@ -5,9 +5,9 @@
  
   -----------------------------------------------------------------------------
  
-  responsible:  SteffenS
+  responsible:  SteffenS, d033893
 
-  special area: Replication Server
+  special area: Loader
  
   description:  Implementation of condition checking and converting functions
                 
@@ -15,7 +15,7 @@
 
   -----------------------------------------------------------------------------
  
-  copyright:    (c) 1999-2004 SAP AG-2003
+  Copyright (c) 1999-2004 SAP AG-2003
  
   -----------------------------------------------------------------------------
 
@@ -63,9 +63,9 @@
 
   description:  Evaluates a simple condition
                 (for instance: checking POS 1-3 <> '===').
-                Therefore the value is extracted from the data line and compared
-                to the constant which has to be converted to the data type of the
-                data.
+                Therefore the value is extracted from the DATA line and compared
+                to the constant which has to be converted to the DATA type of the
+                DATA.
                 Data of type CHAR, ZONED and DECIMAL is compared lexicographically
                 using memcmp. The strings have not to be of the same length! If two
                 strings of different length are equal in the
@@ -74,10 +74,10 @@
                 Data of type INT and REAL is directly compared (i.e. 2 == 4 ).
 
   arguments:    pCondition   [IN]    - condition structure
-                DataLine     [IN]    - data line where the data to be compared
+                DataLine     [IN]    - DATA line where the DATA to be compared
                                         has to be extracted from.
                 FileFormat   [IN]    - FileFormat structure to know
-                                        how to extract the data
+                                        how to extract the DATA
 
   returnvalue:  true        - condition is fullfilled
                 false       - the opposite of the one when true is delivered
@@ -117,7 +117,7 @@ ls16_EvaluateCondition(tls00_Condition  *pCondition,
     }
 
     //*
-    //*     Compare values depending on there data type
+    //*     Compare values depending on there DATA type
     //*
     switch (pCondition->conFormat_ls00)
     {
@@ -159,7 +159,7 @@ ls16_EvaluateCondition(tls00_Condition  *pCondition,
         case ioInteger_els00:
         {
             // We assume that the comparison constant has been converted to the type of 
-            // values from data file before calling this function
+            // values from DATA file before calling this function
             
             // actually we need to check if the length of Value.strAddr_ls00 is really 4 (or 2 or 1)
 
@@ -328,13 +328,13 @@ ls16CheckCondition(tls00_CondSpec&   ConditionSpec,
   -----------------------------------------------------------------------------
 
   description:  Recursive function to convert comparison constants in a complex
-                condition to the data type of the data.
-                The possible data types are CHAR (no conversion), DECIMAL, INTEGER,
+                condition to the DATA type of the DATA.
+                The possible DATA types are CHAR (no conversion), DECIMAL, INTEGER,
                 ZONED and REAL.
                 Conversion takes place for numerical values!
                 To convert the comparison constant first the constant is
-                converted to float and then REconverted to the data type of
-                the data.
+                converted to float and then REconverted to the DATA type of
+                the DATA.
 
   arguments:    ConditionSpec   [IN]    - structure of condition specification 
                                            (actually a tree built of simple conditions)
@@ -383,7 +383,7 @@ ls16ConvertCondition(tls00_CondSpec      &ConditionSpec,
         {
             // cosDefNull_ls00 is then and only than true if the ConditionSpec is not NULL;
             // additionally cosDefNull_ls00 may only be defined in the first condition spec
-            // of every column data structure
+            // of every column DATA structure
             tls00_Condition* pNC = ConditionSpec.cosCond_ls00;
 
             //*
@@ -402,7 +402,7 @@ ls16ConvertCondition(tls00_CondSpec      &ConditionSpec,
             }   // end if (true == pNC->conIsHex_ls00)
 
 
-            // In case data file encoding is UCS2 in any form we need to
+            // In case DATA file encoding is UCS2 in any form we need to
             // transform the DEFAULT NULL value representation to UCS2, too.
             if ( (ctUCS2_els00        == FileFormat.ffoCodeType_ls00) ||
                  (ctUCS2Swapped_els00 == FileFormat.ffoCodeType_ls00) ||
@@ -502,7 +502,7 @@ ls16ConvertCondition(tls00_CondSpec      &ConditionSpec,
                                                 (ConditionSpec.cosCond_ls00->conStart_ls00 - 1);
                     }
 
-                    switch (ConditionSpec.cosCond_ls00->conFormat_ls00)     // convert internal value to requested data type
+                    switch (ConditionSpec.cosCond_ls00->conFormat_ls00)     // convert internal value to requested DATA type
                     {
                         case ioInteger_els00:
                         {
@@ -613,13 +613,13 @@ ls16ConvertCondition(tls00_CondSpec      &ConditionSpec,
         {
             if (errInvalidNumCondition_els98 == lResult)
             {
-                RepServMessages::Instance()->ls98Msg(layCondition_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCondition_els98, pszErrText,
                                                      errInvalidNumCondition_els98,
                                                      ConditionSpec.cosCond_ls00->conCompVal_ls00);
             }
             else
             {
-                RepServMessages::Instance()->ls98Msg(layCondition_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layCondition_els98, pszErrText,
                                                      errTruncatedConditionValue_els98,
                                                      ConditionSpec.cosCond_ls00->conCompVal_ls00);
             }
