@@ -1,25 +1,16 @@
 /*
   -----------------------------------------------------------------------------
- 
-  module: vls19_check.cpp
- 
+   module: vls19_check.cpp
   -----------------------------------------------------------------------------
  
-  responsible:  SteffenS und d033893
+  responsible:  SteffenS, d033893
 
-  special area: SAP DB Loader
+  special area: Loader
  
   description:  Some check functionality
                 
-  version:      7.4.
-
+  version:      7.5.
   -----------------------------------------------------------------------------
- 
-  copyright:    (c) 2000-2004 SAP AG-2002
- 
-  -----------------------------------------------------------------------------
-
-
 
     ========== licence begin  GPL
     Copyright (c) 2000-2004 SAP AG
@@ -38,9 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 
 #include "gls00.h"
@@ -77,7 +65,7 @@
                 fraction the value ExtLenFrac may be adapted.
 
                 ExtLenFrac defaults to NULL because this function is also used
-                to compute length of buffer at compressed data format, and at time
+                to compute length of buffer at compressed DATA format, and at time
                 of computing this variable has no useful value!
 
   arguments:    ValueLength   [IN]  - column length
@@ -94,7 +82,7 @@
                 errIntValueOverflow_els98      - warning
                 errWrongDecimalLength_els98    - admissible length of numbers in 
                                                   BCD representation exceeded
-                errIncompatibleDataTypes_els98 - incompatible data types (db values
+                errIncompatibleDataTypes_els98 - incompatible DATA types (db values
                                                   with fraction cannot be written as
                                                   integers)
                 errWrongZonedLength_els98      - too less fraction digits for BCD or 
@@ -115,9 +103,10 @@ ls19_CheckFixLength(const tsp00_Int4   ValueLength,
     tsp00_Int4          rc        = errOK_els00;
     tls00_FieldOption & FOption   = Column->colOption_ls00;
 
-    tsp00_Int4          lFracPart = FieldInfo->fbFraction_ls00;
-    tsp00_Int4          lIntPart  = FieldInfo->fbLength_ls00 - lFracPart;
-
+    tsp00_Int4 lThsdCharNum = 0;
+    tsp00_Int4 lFracPart = FieldInfo->fbFraction_ls00;
+    tsp00_Int4 lIntPart  = FieldInfo->fbLength_ls00 - lFracPart;
+    
     if (NULL != ExtLenFrac)
     {
         lIntPart  = ExtLenFrac[0] - ExtLenFrac[1];      // ExtLenFrac contains complete length!
@@ -127,7 +116,6 @@ ls19_CheckFixLength(const tsp00_Int4   ValueLength,
         }
         lFracPart = ExtLenFrac[1];
     }
-
     
     switch (Column->colFormat_ls00.ffoFormat_ls00)
     {
@@ -135,7 +123,6 @@ ls19_CheckFixLength(const tsp00_Int4   ValueLength,
         {
             if (dfloat != FieldInfo->fbDataType_ls00)
             {
-                tsp00_Int4 lThsdCharNum = 0;
                 if (NO_THSD_GROUPS_LS00 != FileFormat->ffoThsdChar_ls00)
                 {
                     lThsdCharNum = (lIntPart - 1) / 3;      // should result in integer again
@@ -160,7 +147,7 @@ ls19_CheckFixLength(const tsp00_Int4   ValueLength,
         }
         case ioInteger_els00:
         {
-            // if fraction > 0 reject, because no fraction possible: incompatible data types
+            // if fraction > 0 reject, because no fraction possible: incompatible DATA types
             if ( (lFracPart > 0) || (dfloat == FieldInfo->fbDataType_ls00) )
             {
                 rc = errIncompatibleDataTypes_els98;
@@ -684,8 +671,7 @@ ls19CheckLengthForExtract(const tsp00_Int4   ValueLength,
         bool       bHex   = ( (true == Column->colFormat_ls00.ffoIsHex_ls00) &&
                               (1 != szMap_BlobDataType_ls00[pFieldInfo->fbDataType_ls00]) );
 
-        if ( (ctUCS2_els00        == pFileFormat->ffoCodeType_ls00)     ||
-             (ctUCS2Swapped_els00 == pFileFormat->ffoCodeType_ls00) )
+        if ( true == ls00IsUCS2(&pFileFormat->ffoCodeType_ls00) )
         {
             if (true == bHex)
             {
@@ -696,8 +682,7 @@ ls19CheckLengthForExtract(const tsp00_Int4   ValueLength,
                 factor = 2;
             }
         }
-        else if ( (ctAscii_els00 == pFileFormat->ffoCodeType_ls00) ||
-                  (ctUTF8_els00 == pFileFormat->ffoCodeType_ls00)   )
+        else if ( (ctAscii_els00 == pFileFormat->ffoCodeType_ls00) || (ctUTF8_els00 == pFileFormat->ffoCodeType_ls00) )
         {
             if (true == bHex)
             {
@@ -783,7 +768,7 @@ ls19CheckDataTypeCompatibility(tls00_Column      *Column,
     {
         case ioChar_els00:
         {
-            break;      // any internal data type may be written as char
+            break;      // any internal DATA type may be written as char
         }
         case ioInteger_els00:
         {
