@@ -1,8 +1,6 @@
 /* 
   -----------------------------------------------------------------------------
- 
   module: vls21.cpp
- 
   -----------------------------------------------------------------------------
  
   responsible:  SteffenS
@@ -11,15 +9,8 @@
  
   description:  FASTLOAD execution
                 
-  version:      7.5.
-
+  version:      7.6.
   -----------------------------------------------------------------------------
- 
-  copyright:    (c) 1997-2004 SAP AG-2003
- 
-  -----------------------------------------------------------------------------
-
-
 
     ========== licence begin  GPL
     Copyright (c) 1997-2004 SAP AG
@@ -38,9 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 #include <assert.h>
 
@@ -56,6 +44,7 @@
 #include "hls18.h"
 #include "hls21.h"
 #include "hls22.h"          // fastload-laden der seiten
+#include "hls30.h"
 #include "hls99.h"          // trace-ausgaben
 #include "hls98msg.h"       // fehlermeldungen/logdatei
 
@@ -200,7 +189,7 @@ ls21_GetDefaultValue(tls00_DBInfo   *pDBInfo,
                      tsp00_Addr      pszTableName,
                      tsp00_Int4      ColNum[],
                      tsp00_Int2      lDefaultCount,
-                     tsp00_Addr      pszErrText);
+                     tsp00_Addr      ErrText);
 
 /*
   -----------------------------------------------------------------------------
@@ -244,8 +233,10 @@ ls21_CreateFloadRec(tls00_FloadRec *&pFLRec,
         pFLRec->flrLRejected_ls00 = pDBInfo->dbiSession_ls00.siMaxAdmissibleErrors;
 
         pFLRec->flrSwapKind_ls00 = pDBInfo->dbiOwnSwap_ls00;   //  Swap behavior
-    }
 
+        pFLRec->flrpInfile_ls00 = new tls00_VFile;
+        memset(pFLRec->flrpInfile_ls00, 0, sizeof(tls00_VFile));
+    }
     return rc;
 }
 //  ls21_CreateFloadRec()
@@ -321,6 +312,12 @@ ls21_Destroy(tls00_FloadRec *& FloadRec)
 
     ls18DestructPattern(&FloadRec->flrPattern_ls00);
 
+    if (NULL != FloadRec->flrpInfile_ls00)
+    {
+        delete FloadRec->flrpInfile_ls00;
+        FloadRec->flrpInfile_ls00 = NULL;
+    }
+
     delete FloadRec;
     FloadRec = NULL;
 }
@@ -335,7 +332,7 @@ ls21_Destroy(tls00_FloadRec *& FloadRec)
 static int
 ls21_ExtractFieldInfos(tsp1_packet              *SqlPacket,
                        tls00_TableDescription & TableInfo,
-                       tsp00_Addr               pszErrText)
+                       tsp00_Addr               ErrText)
 {
     ROUTINE_DBG_MEO00 ("ls21_ExtractFieldInfos");
 
@@ -349,7 +346,7 @@ ls21_ExtractFieldInfos(tsp1_packet              *SqlPacket,
     
     if ( false == ls04GetPart(SqlPacket, sp1pk_data, pPart) )
     {
-        RepServMessages::Instance()->ls98Msg(layFastload_els98, pszErrText,
+        LoaderMessages::Instance()->ls98Msg(layFastload_els98, ErrText,
                                                                 errUnknownPart_els98);
         rc = errUnknownPart_els98;
     }
@@ -517,7 +514,7 @@ ls21_GetDefaultValue(tls00_DBInfo   *pDBInfo,
                      tsp00_Addr      pszTableName,
                      tsp00_Int4      ColNum[],
                      tsp00_Int2      lDefaultCount,
-                     tsp00_Addr      pszErrText)
+                     tsp00_Addr      ErrText)
 {
     ROUTINE_DBG_MEO00 ("ls21_GetDefaultValue");
 
@@ -532,7 +529,7 @@ ls21_GetDefaultValue(tls00_DBInfo   *pDBInfo,
     Tools_DynamicUTF8String TableNameUTF8;
     rc = ls05ConvertToUTF8(TableNameUTF8,
                            REINTERPRET_CAST(unsigned char*, pszTableName), _tcslen(pszTableName),
-                           ctAscii_els00, pszErrText);
+                           ctAscii_els00, ErrText);
 
     int i = 0;
     for (i; i < lDefaultCount; ++i) 
@@ -540,7 +537,7 @@ ls21_GetDefaultValue(tls00_DBInfo   *pDBInfo,
         // Define number of column to get the default for
         lColNo = pTD->tdFieldsArray_ls00[ColNum[i]]->fiDefColNo_ls00;
 
-        rc = ls13GetDefaultValue(pDBInfo, TableNameUTF8, lColNo, pDefault, lDefLen, pszErrText);
+        rc = ls13GetDefaultValue(pDBInfo, TableNameUTF8, lColNo, pDefault, lDefLen, ErrText);
         if (errOK_els00 == rc)
         {
             FloadRec.flrDefValue_ls00[ColNum[i]] = new tls00_String;
@@ -589,11 +586,11 @@ ls21_GetDefaultValue(tls00_DBInfo   *pDBInfo,
 int ls21_ExecBeginLoad(tls00_DBInfo   *pDBInfo,
                        tls00_FloadRec *pFLRec,
                        tsp00_Addr      pszTableName,
-                       tsp00_Addr      pszErrText)
+                       tsp00_Addr      ErrText)
 {
     ROUTINE_DBG_MEO00 ("ls21_ExecBeginLoad");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance(); // Fr error reports
+    LoaderMessages *pMsgObj = LoaderMessages::Instance(); // Fr error reports
     tsp00_Int4       rc      = errOK_els00;
     
     //*
@@ -611,10 +608,10 @@ int ls21_ExecBeginLoad(tls00_DBInfo   *pDBInfo,
     }
 
     tsp00_Int4 ErrPos = 0;
-    rc = ls13ExecIntSQLStmt(pDBInfo, 0, szCmd, pszErrText, ErrPos, false, sp1m_dbs, false);
+    rc = ls13ExecIntSQLStmt(pDBInfo, 0, szCmd, ErrText, ErrPos, false, sp1m_dbs, false);
     if (errOK_els00 != rc)
     {
-        pMsgObj->ls98PrintMessage(layFastload_els98, errSimpleSQL_els98, pszErrText);
+        pMsgObj->ls98PrintMessage(layFastload_els98, errSimpleSQL_els98, ErrText);
     }
     else
     {
@@ -714,7 +711,7 @@ int ls21_ExecBeginLoad(tls00_DBInfo   *pDBInfo,
 
     if (errUnknownPart_els98 == rc)
     {
-        pMsgObj->ls98Msg(layFastload_els98, pszErrText, errInternal_els98, _INTERNAL_FILE_ID_,
+        pMsgObj->ls98Msg(layFastload_els98, ErrText, errInternal_els98, _INTERNAL_FILE_ID_,
                                                                            __LINE__,
                                                                            errUnknownPart_els98);
         rc = errInternal_els98;
@@ -757,7 +754,7 @@ ls21_ExecEndLoad(tls00_DBInfo   *DBInfo,
     tsp00_Int4 rc        = errOK_els00;
     tsp00_Addr pszBuffer = NULL;
 
-    RepServMessages *pMsgObj = RepServMessages::Instance(); // Fr error reports
+    LoaderMessages *pMsgObj = LoaderMessages::Instance(); // Fr error reports
     
     tls00_Parts4InternalSql S;
     
@@ -888,34 +885,24 @@ ls21_ExecEndLoad(tls00_DBInfo   *DBInfo,
   -----------------------------------------------------------------------------
   description:  Processes a fastload command.
 
-  arguments:    pFLCmd     [IN]  - fastload command structure with user info
+  arguments:    pTransformationCmd     [IN]  - fastload command structure with user info
                 pDBInfo    [IN]  - structure with info on db and running session
-                pszErrText [OUT] - error text
+                ErrText [OUT] - error text
 
   returnvalue:  errOK_els00 - no error
                 errors from processing 
   -----------------------------------------------------------------------------
 */
 int
-ls21ExecFastload(tls00_FloadCmd *pFLCmd, tls00_DBInfo *pDBInfo, tsp00_Addr pszErrText)
+ls21ExecFastload(MetaDataDef *pMetaDataDef, tls00_DBInfo *pDBInfo, tsp00_Addr ErrText)
 {
     ROUTINE_DBG_MEO00 ("ls21ExecFastload");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance();    // Fr error reports
-    tsp00_Int4       rc      = errOK_els00;
+    LoaderMessages *pMsgObj = LoaderMessages::Instance();    // Fr error reports
+    tsp00_Int4       rc     = errOK_els00;
 
-    //*
-    //*     Before doing anything else we check external encoding and
-    //*     database encoding.
-    //*     Not supported is loading UNICODE data into ASCII databases.
-    //*
-    rc = STATIC_CAST(tsp00_Int4, ls05AreEncodingsCompatible(pFLCmd->flcInfile_ls00.fsFormat_ls00.ffoCodeType_ls00,
-                                                            ls00IsDBUnicode(pDBInfo) ? ctUCS2_els00 : ctAscii_els00));
-    if (errOK_els00 != rc)
-    {
-        pMsgObj->ls98Msg(layFastload_els98, pszErrText, errIncompatibleEncodings_els98);
-        return rc;
-    }
+    TransformationDef       *pTransformationDef = pMetaDataDef->getTransformationModel();
+    tls00_TransformationCmd *pTransformationCmd = pTransformationDef->getTransformationCmd();
 
     //*
     //*     Initialize fastload-record structure
@@ -927,28 +914,73 @@ ls21ExecFastload(tls00_FloadCmd *pFLCmd, tls00_DBInfo *pDBInfo, tsp00_Addr pszEr
         return rc;
     }
 
-    tls00_MultColSpec      *pMultCol  = &pFLCmd->flcColSpec_ls00;
+    tls00_MultColSpec      *pMultCol  = &pTransformationCmd->trColSpec_ls00;
     tls00_TableDescription &TableInfo = pFLRec->flrTableInfo_ls00;
-    tls00_TableSpec        &TableSpec = pFLCmd->flcTable_ls00;
+    tls00_TableSpec        &TableSpec = pTransformationCmd->trTable_ls00;
     tls00_FieldInfo        *pFI       = NULL;
     tls00_Column           *pCol      = NULL;
 
     int i = 0;
     int j = 0;
     int k = 0;
-    
+
+    // Initialize and open data file
+    if (false == pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoBinary_ls00)
+    {
+        ls30VFInitialize(*pFLRec->flrpInfile_ls00,
+                         pTransformationCmd->trDataFile_ls00.fsFileName_ls00,
+                         sp5vf_text,
+                         sp5vf_read,
+                         pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00);
+    }
+    else
+    {
+        ls30VFInitialize(*pFLRec->flrpInfile_ls00, pTransformationCmd->trDataFile_ls00.fsFileName_ls00);
+    }
+
+    // Open file and set encoding type if necessary
+    rc = ls30VFOpen(*pFLRec->flrpInfile_ls00, ErrText);
+    if (errOK_els00 != rc)
+    {
+        ls21_Destroy(pFLRec);
+        return rc;
+    }
+    else
+    {
+        if (ctUnknown_els00 == pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00)
+        {
+            pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00 = pFLRec->flrpInfile_ls00->vfFileEncoding_ls00;
+        }
+    }
+
+    //*
+    //*     Check file encoding and database encoding.
+    //*     Not supported is loading UNICODE data into ASCII databases.
+    //*
+    rc = STATIC_CAST(tsp00_Int4, ls05AreEncodingsCompatible(pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00,
+                                                            ls00IsDBUnicode(pDBInfo) ? ctUCS2_els00 : ctAscii_els00));
+    if (errOK_els00 != rc)
+    {
+        pMsgObj->ls98Msg(layFastload_els98, ErrText, errIncompatibleEncodings_els98);
+        ls21_Destroy(pFLRec);
+        return rc;
+    }
+
     //*
     //*     Get detailed info on the table to process from kernel
     //*
-    rc = ls21_GetFieldInfo(pDBInfo, TableSpec.tsFullTabName_ls00, pMultCol, TableInfo, pszErrText);
-    if (errOK_els00 != rc)
+    if (errOK_els00 == rc)
     {
-        // Here we get already an error text that is only to be printed. We'll use
-        // the errSimpleSQL_els98-message to do it.
-        pMsgObj->ls98PrintMessage(layFastload_els98, errSimpleSQL_els98, pszErrText);
+        rc = ls21_GetFieldInfo(pDBInfo, TableSpec.tsFullTabName_ls00, pMultCol, TableInfo, ErrText);
+        if (errOK_els00 != rc)
+        {
+            // Here we get already an error text that is only to be printed. We'll use
+            // the errSimpleSQL_els98-message to do it.
+            pMsgObj->ls98PrintMessage(layFastload_els98, errSimpleSQL_els98, ErrText);
 
-        ls21_Destroy(pFLRec);
-        return rc;
+            ls21_Destroy(pFLRec);
+            return rc;
+        }
     }
 
     //*
@@ -982,14 +1014,14 @@ ls21ExecFastload(tls00_FloadCmd *pFLCmd, tls00_DBInfo *pDBInfo, tsp00_Addr pszEr
         {
             ls21_Destroy(pFLRec);
 
-            pMsgObj->ls98Msg(layFastload_els98, pszErrText, errLongColIllegal_els98);
+            pMsgObj->ls98Msg(layFastload_els98, ErrText, errLongColIllegal_els98);
             return errLongColIllegal_els98;
         }
     }
     //*     Gabs benutzerdefinierte defaults ? Dann hole diese.
     if (j > 0)
     {
-        rc = ls21_GetDefaultValue (pDBInfo, *pFLRec, TableSpec.tsTabName_ls00, lDefaultFieldNo, j, pszErrText);
+        rc = ls21_GetDefaultValue (pDBInfo, *pFLRec, TableSpec.tsTabName_ls00, lDefaultFieldNo, j, ErrText);
         if (errOK_els00 != rc)
         {
             ls21_Destroy(pFLRec);
@@ -1076,8 +1108,8 @@ ls21ExecFastload(tls00_FloadCmd *pFLCmd, tls00_DBInfo *pDBInfo, tsp00_Addr pszEr
             {
                 rc = ls16ConvertCondition(*pCol->colNullCond_ls00,
                                           pDBInfo->dbiOwnSwap_ls00,
-                                          pFLCmd->flcInfile_ls00.fsFormat_ls00,
-                                          pszErrText);
+                                          pTransformationCmd->trDataFile_ls00.fsFormat_ls00,
+                                          ErrText);
                 if (errOK_els00 != rc)
                 {
                     ls21_Destroy(pFLRec);
@@ -1268,15 +1300,15 @@ ls21ExecFastload(tls00_FloadCmd *pFLCmd, tls00_DBInfo *pDBInfo, tsp00_Addr pszEr
             }
         }
     }
-
+    
     //*
     //*     Fill pattern structure
     //*
     rc = ls18ConstructPattern(&pFLRec->flrPattern_ls00,
-                              pFLCmd->flcInfile_ls00.fsFormat_ls00.ffoSeperat_ls00,
-                              pFLCmd->flcInfile_ls00.fsFormat_ls00.ffoDelimit_ls00,
-                              pFLCmd->flcInfile_ls00.fsFormat_ls00.ffoCodeType_ls00,
-                              pszErrText);
+                              pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoSeperat_ls00,
+                              pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoDelimit_ls00,
+                              pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00,
+                              ErrText);
 
     //*     Some traces
     TRACE_PRNF_MLS99 (("ls21ExecFastload", "Computed internal length of fix long columns: %d",
@@ -1294,24 +1326,22 @@ ls21ExecFastload(tls00_FloadCmd *pFLCmd, tls00_DBInfo *pDBInfo, tsp00_Addr pszEr
 
     TRACE_PRNF_MLS99 (("ls21ExecFastload", "Available page space at full usage: %d", tmpSpace));
 
-    if (true == pFLCmd->flcUsage_ls00.uUsingRows_ls00)
+    if (true == pTransformationCmd->trUsage_ls00.uUsingRows_ls00)
     {
         // fuer jeden einzufuegenden datensatz wird auf der page ein pointer angelegt, der auf den
         // anfang dieses datensatzes verweist; der platzbedarf fuer diesen pointer (POINTERSIZE_BD00)
         // muss in die berechnung mit eingehen
-        PGI->pgiDataSpace =
-            pFLCmd->flcUsage_ls00.uUsageValue_ls00 * (pFLRec->flrRecordLen_ls00 + POINTERSIZE_BD00);
+        PGI->pgiDataSpace = pTransformationCmd->trUsage_ls00.uUsageValue_ls00 * (pFLRec->flrRecordLen_ls00 + POINTERSIZE_BD00);
 
         TRACE_PRNF_MLS99 (("ls21ExecFastload", "Available page space if %d rows per page: %d.",
-                                               pFLCmd->flcUsage_ls00.uUsageValue_ls00,
+                                               pTransformationCmd->trUsage_ls00.uUsageValue_ls00,
                                                PGI->pgiDataSpace));
-
         if (PGI->pgiDataSpace > tmpSpace)
         {
-            pMsgObj->ls98Msg(layFastload_els98, pszErrText, errUsageNotApplicable_els98,
-                                                            pFLCmd->flcUsage_ls00.uUsageValue_ls00,
-                                                            (tmpSpace/(pFLRec->flrRecordLen_ls00 + POINTERSIZE_BD00)));
-            return errUsageNotApplicable_els98;
+            // Issue a warning and adapt space value
+            pMsgObj->ls98PrintMessage(layFastload_els98, wrnUsageIgnoredRows_els98, pTransformationCmd->trUsage_ls00.uUsageValue_ls00);
+            pTransformationCmd->trUsage_ls00.uUsageValue_ls00 = tmpSpace/(pFLRec->flrRecordLen_ls00 + POINTERSIZE_BD00);
+            PGI->pgiDataSpace = tmpSpace;
         }
     }
     else
@@ -1319,7 +1349,7 @@ ls21ExecFastload(tls00_FloadCmd *pFLCmd, tls00_DBInfo *pDBInfo, tsp00_Addr pszEr
         // der platzbedarf fuer den pointer auf den ersten datensatz muss an dieser stelle bereits
         // abgezogen werden, um den wirklich vorhandenen platz zu bestimmen
 
-        PGI->pgiDataSpace = ( (tmpSpace * pFLCmd->flcUsage_ls00.uUsageValue_ls00) / 100 ) -  POINTERSIZE_BD00;
+        PGI->pgiDataSpace = ( (tmpSpace * pTransformationCmd->trUsage_ls00.uUsageValue_ls00) / 100 ) -  POINTERSIZE_BD00;
 
         // zusaetzlich muessen wir pruefen, ob bereits der erste satz diese grenze ueberschreiten wuerde;
         // fuer 100% usage wird das eigentlich vom kern abgefangen, da ein record nicht laenger gewaehlt
@@ -1328,37 +1358,34 @@ ls21ExecFastload(tls00_FloadCmd *pFLCmd, tls00_DBInfo *pDBInfo, tsp00_Addr pszEr
         // ueberschreiten. In diesem fall wird eine warnung ausgegeben und der fuellgrad schweigend ignoriert.
 
         TRACE_PRNF_MLS99 (("ls21ExecFastload", "Available page space at %d%% usage: %d",
-                                               pFLCmd->flcUsage_ls00.uUsageValue_ls00,
+                                               pTransformationCmd->trUsage_ls00.uUsageValue_ls00,
                                                PGI->pgiDataSpace));
 
         if (pFLRec->flrRecordLen_ls00 > PGI->pgiDataSpace)
         {
             // Issue a warning and adapt space value
-            pMsgObj->ls98PrintMessage(layFastload_els98, wrnUsageIgnored_els98,
-                                                         pFLCmd->flcUsage_ls00.uUsageValue_ls00);
+            pMsgObj->ls98PrintMessage(layFastload_els98, wrnUsageIgnoredPercent_els98, pTransformationCmd->trUsage_ls00.uUsageValue_ls00);
             PGI->pgiDataSpace = tmpSpace -  POINTERSIZE_BD00;
         }
     }
     // recordlength is implicitely checked while building the page in ls22-module: every value
     // must fit into the length given by the database
 
-
-
     //*
     //*     Ausfuehren von fastload ********************************************
     //*
-    if (errOK_els00 == (rc = ls21_ExecBeginLoad(pDBInfo, pFLRec, TableSpec.tsFullTabName_ls00, pszErrText)))
+    if (errOK_els00 == (rc = ls21_ExecBeginLoad(pDBInfo, pFLRec, TableSpec.tsFullTabName_ls00, ErrText)))
     {
         SAPDB_Bool _bRollback = SAPDB_TRUE;
 
-        rc = ls22LoadPages(pFLCmd, pFLRec, pDBInfo, &_bRollback, pszErrText);
+        rc = ls22LoadPages(pTransformationCmd, pFLRec, pDBInfo, &_bRollback, ErrText);
         if ( (errOK_els00 == rc) ||
              ( (errOK_els00 != rc) && (SAPDB_FALSE == _bRollback) ) )
         {
             //*
             //*     This is the final statement that signals the kernel that all data was sent
             //*
-            tsp00_Int4 rc2 = ls21_ExecEndLoad(pDBInfo, pFLRec, TableSpec.tsFullTabName_ls00, pszErrText);
+            tsp00_Int4 rc2 = ls21_ExecEndLoad(pDBInfo, pFLRec, TableSpec.tsFullTabName_ls00, ErrText);
             if (errOK_els00 == rc2)
             {
                 pMsgObj->ls98PrintMessage(layFastload_els98, msgLastCommInputLine2_els98, pFLRec->flrLastCommLine_ls00);
@@ -1382,7 +1409,7 @@ ls21ExecFastload(tls00_FloadCmd *pFLCmd, tls00_DBInfo *pDBInfo, tsp00_Addr pszEr
             //*
             //*     Error processing FASTLOAD command; all submitted data will be rolled back
             //*
-            tsp00_Int4 rc2 = ls15Rollback(pDBInfo, pszErrText);
+            tsp00_Int4 rc2 = ls15Rollback(pDBInfo, ErrText);
 
             pMsgObj->ls98PrintMessage(layFastload_els98, msgLastCommInputLine2_els98, pFLRec->flrLastCommLine_ls00);
 
@@ -1392,6 +1419,11 @@ ls21ExecFastload(tls00_FloadCmd *pFLCmd, tls00_DBInfo *pDBInfo, tsp00_Addr pszEr
         }
     }
 
+
+
+    // Close file
+    tsp00_Int4 rc2 = ls30VFClose(*pFLRec->flrpInfile_ls00, ErrText);
+
     ls21_Destroy(pFLRec);
 
     return rc;
