@@ -1,18 +1,15 @@
 /*! 
   -----------------------------------------------------------------------------
- 
   module: vls22.cpp
- 
   -----------------------------------------------------------------------------
  
-  responsible:  SteffenS and d033893
+  responsible:  SteffenS, d033893
 
-  special area: SAP DB LOADER
+  special area: Loader
  
   description:  Fastload - filling db pages
                 
-  version:      7.5.
- 
+  version:      7.5. 
   -----------------------------------------------------------------------------
 
     ========== licence begin  GPL
@@ -50,7 +47,7 @@
 #include "gbd00.h"      // database bd layer (data page layout)
 #include "ggg00.h"      // database bd layer (data page layout)
 
-#define _INTERNAL_FILE_ID_   _T("vls22.cpp")
+#define _INTERNAL_FILE_ID_   _T("ls22.cpp")
 
 /*!
   -----------------------------------------------------------------------------
@@ -65,7 +62,7 @@
   description:  Main loop of fastload.
                 Fills data pages, sends those pages and receives answers from db.
   
-  arguments:    FloadCmd   [IN]   - fastload command structure
+  arguments:    pTransformationCmd   [IN]   - fastload command structure
                 FloadRec   [IN]   - fastload record structure
                 DBInfo     [IN]   - dbinfo structure
                 VFile      [IN]   - vfile structure
@@ -80,20 +77,22 @@
   -----------------------------------------------------------------------------
 */
 static int
-ls22_PageLoadLoop(tls00_FloadCmd*  pFloadCmd,
+ls22_PageLoadLoop(tls00_TransformationCmd*  pTransformationCmd,
                   tls00_FloadRec*  pFloadRec,
                   tls00_DBInfo*    pDBInfo,
                   tls00_VFile*     pVFile,
                   SAPDB_Bool*      pbRollback,
                   tsp00_Addr       pszErrText);
 
+
 /*!
   -----------------------------------------------------------------------------
   function:     ls22_FillPacket
   -----------------------------------------------------------------------------
+
   description:  Fills a packet with data from infile.
   
-  arguments:    FloadCmd   [IN]   - fastload structure built by the parser from
+  arguments:    pTransformationCmd   [IN]   - fastload structure built by the parser from
                                      given command
                 FloadRec   [IN]   - fastload structure built by the server
                 DBInfo     [IN]   - dbinfo structure
@@ -108,13 +107,13 @@ ls22_PageLoadLoop(tls00_FloadCmd*  pFloadCmd,
                 Logs the error in log file.
   -----------------------------------------------------------------------------
 */
-int ls22_FillPacket(tls00_FloadCmd*    FloadCmd,
-                    tls00_FloadRec*    FloadRec,
-                    tls00_DBInfo*      DBInfo,
-                    tsp00_Int4         lPcktNo,
-                    tls00_VFile*       VFile,
-                    tls22_Statistics*  pPcktStats,
-                    tsp00_Addr         ErrText);
+int ls22_FillPacket(tls00_TransformationCmd*  pTransformationCmd,
+                    tls00_FloadRec*           FloadRec,
+                    tls00_DBInfo*             DBInfo,
+                    tsp00_Int4                lPcktNo,
+                    tls00_VFile*              VFile,
+                    tls22_Statistics*         pPcktStats,
+                    tsp00_Addr                ErrText);
 
 
 /*!
@@ -239,7 +238,7 @@ ls22_RepairPage(tbd_node*                 pDataPage,
   function:     ls22LoadPages
   -----------------------------------------------------------------------------
 */
-int ls22LoadPages(tls00_FloadCmd*  FloadCmd,
+int ls22LoadPages(tls00_TransformationCmd*  pTransformationCmd,
                   tls00_FloadRec*  FloadRec,
                   tls00_DBInfo*    DBInfo,
                   SAPDB_Bool*      pbRollback,
@@ -249,111 +248,82 @@ int ls22LoadPages(tls00_FloadCmd*  FloadCmd,
 
     tsp00_Int4 rc = errOK_els00;
     
-    tls00_FileSpec *pInFileSpec = &FloadCmd->flcInfile_ls00;
-    
-    
-    // Initialize file structure
-    tls00_VFile VFile;
-    memset(&VFile,0,sizeof(tls00_VFile));
+    tls00_FileSpec *pInFileSpec = &pTransformationCmd->trDataFile_ls00;
+    tls00_VFile    *pVFile      = FloadRec->flrpInfile_ls00;
 
-    if (false == pInFileSpec->fsFormat_ls00.ffoBinary_ls00)
+    tsp00_Int4 lByteCount = ls18GetRowLength(pTransformationCmd->trColSpec_ls00,
+                                             FloadRec->flrTableInfo_ls00,
+                                             pInFileSpec->fsFormat_ls00.ffoFormatted_ls00);
+
+    if (true == pInFileSpec->fsFormat_ls00.ffoBinary_ls00)
     {
-        ls30VFInitialize(VFile,
-                         pInFileSpec->fsFileName_ls00,
-                         sp5vf_text,
-                         sp5vf_read,
-                         FloadCmd->flcInfile_ls00.fsFormat_ls00.ffoCodeType_ls00);
+        if (0 == pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00)       // NOHEADER not found in command
+        {
+            rc = ls30VFReadHeader(*pVFile, pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00, ErrText);
+            if (errOK_els00 != rc)
+            {
+                if (errNoDatafileHeader_ls98 == rc)
+                {
+                    pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00 = lByteCount;
+                    LoaderMessages::Instance()->ls98PrintMessage(layFastload_els98,
+                                                                 errNoDatafileHeader_ls98,
+                                                                 pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00);
+                    rc = errOK_els00;
+                }
+                // In case of file read error we stop at this point;
+                // if only no header log it and calculate length
+            }
+        }
+        // else is done because value is already set
     }
     else
     {
-        ls30VFInitialize(VFile, pInFileSpec->fsFileName_ls00);
+        pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00 = lByteCount;
     }
 
-    // Open file and calculate record length
-    rc = ls30VFOpen(VFile, ErrText);
+    //*
+    //*     Check for loading a part of the file only
+    //*
     if (errOK_els00 == rc)
     {
-        tsp00_Int4 lByteCount = ls18GetRowLength(FloadCmd->flcColSpec_ls00,
-                                                 FloadRec->flrTableInfo_ls00,
-                                                 //pInFileSpec->fsFormat_ls00.ffoBinary_ls00);
-                                                 pInFileSpec->fsFormat_ls00.ffoFormatted_ls00);
+        if ( (UNDEFINED_LS00 != pInFileSpec->fsExtract_ls00.feStart_ls00.lnuRemainder_ls00) &&
+             (0              != pInFileSpec->fsExtract_ls00.feStart_ls00.lnuRemainder_ls00) )   // in case of 0 we're already on the right pos
+        {
+            tls00_LargeNumber lFilePos;
+            lFilePos.lnu2GB_Blocks_ls00 = 0;
 
-        if (true == pInFileSpec->fsFormat_ls00.ffoBinary_ls00)
-        {
-            if (0 == pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00)       // NOHEADER not found in command
+            if (false == pInFileSpec->fsFormat_ls00.ffoBinary_ls00)  // text file
             {
-                rc = ls30VFReadHeader(VFile, pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00, ErrText);
-                if (errOK_els00 != rc)
-                {
-                    if (errNoDatafileHeader_ls98 == rc)
-                    {
-                        pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00 = lByteCount;
-                        RepServMessages::Instance()->ls98PrintMessage(layFastload_els98,
-                                                                      errNoDatafileHeader_ls98,
-                                                                      pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00);
-                        rc = errOK_els00;
-                    }
-                    // In case of file read error we stop at this point;
-                    // if only no header log it and calculate length
-                }
+                lFilePos.lnuRemainder_ls00 = pInFileSpec->fsExtract_ls00.feStart_ls00.lnuRemainder_ls00;
             }
-            // else is done because value is already set
-        }
-        else
-        {
-            pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00 = lByteCount;
+            else
+            {
+                lFilePos.lnuRemainder_ls00 = pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00 * (pInFileSpec->fsExtract_ls00.feStart_ls00.lnuRemainder_ls00 - 1);
+            }
+            rc = ls30VFMoveToStartPosition(*pVFile, &lFilePos, ErrText);
         }
     }
 
+
+    //*
+    //*     Main working routine for loading fast
+    //*
+    //* We need to catch the case when the user specified a START position in infile and
+    //* a count of rows to load but this count is 0.
+    //*
     if (errOK_els00 == rc)
     {
-        //*
-        //*     Check for loading a part of the file only
-        //*
-        if (errOK_els00 == rc)
+        if ( (UNDEFINED_LS00 == pInFileSpec->fsExtract_ls00.feEnd_ls00.lnuRemainder_ls00) ||
+             (0               < pInFileSpec->fsExtract_ls00.feEnd_ls00.lnuRemainder_ls00) )
         {
-            if ( (UNDEFINED_LS00 != pInFileSpec->fsExtract_ls00.feStart_ls00.lnuRemainder_ls00) &&
-                 (0              != pInFileSpec->fsExtract_ls00.feStart_ls00.lnuRemainder_ls00) )   // in case of 0 we're already on the right pos
-            {
-                tls00_LargeNumber lFilePos;
-                lFilePos.lnu2GB_Blocks_ls00 = 0;
-
-                if (false == pInFileSpec->fsFormat_ls00.ffoBinary_ls00)  // text file
-                {
-                    lFilePos.lnuRemainder_ls00 = pInFileSpec->fsExtract_ls00.feStart_ls00.lnuRemainder_ls00;
-                }
-                else
-                {
-                    lFilePos.lnuRemainder_ls00 = pInFileSpec->fsFormat_ls00.ffoBytesPerLine_ls00 * (pInFileSpec->fsExtract_ls00.feStart_ls00.lnuRemainder_ls00 - 1);
-                }
-                rc = ls30VFMoveToStartPosition(VFile, &lFilePos, ErrText);
-            }
+            rc = ls22_PageLoadLoop(pTransformationCmd, FloadRec, DBInfo, pVFile, pbRollback, ErrText);
         }
-
-    
-        //*
-        //*     Main working routine for loading fast
-        //*
-        //* We need to catch the case when the user specified a START position in infile and
-        //* a count of rows to load but this count is 0.
-        //*
-        if (errOK_els00 == rc)
+        else if (0 == pInFileSpec->fsExtract_ls00.feEnd_ls00.lnuRemainder_ls00)
         {
-            if ( (UNDEFINED_LS00 == pInFileSpec->fsExtract_ls00.feEnd_ls00.lnuRemainder_ls00) ||
-                 (0               < pInFileSpec->fsExtract_ls00.feEnd_ls00.lnuRemainder_ls00) )
-            {
-                rc = ls22_PageLoadLoop(FloadCmd, FloadRec, DBInfo, &VFile, pbRollback, ErrText);
-            }
-            else if (0 == pInFileSpec->fsExtract_ls00.feEnd_ls00.lnuRemainder_ls00)
-            {
-                FloadRec->flrLastCommLine_ls00 = 0;
-            }
+            FloadRec->flrLastCommLine_ls00 = 0;
         }
     }
 
-//TODOTODO  catch error from closing file (not that important)
-    tsp00_Int4 rc2 = ls30VFClose(VFile, ErrText);
-
     return rc;
 }
 //  ls22LoadPages()
@@ -380,7 +350,7 @@ int ls22LoadPages(tls00_FloadCmd*  FloadCmd,
   -----------------------------------------------------------------------------
 */
 static int
-ls22_PageLoadLoop(tls00_FloadCmd*  pFloadCmd,
+ls22_PageLoadLoop(tls00_TransformationCmd*  pTransformationCmd,
                   tls00_FloadRec*  pFloadRec,
                   tls00_DBInfo*    pDBInfo,
                   tls00_VFile*     pVFile,
@@ -388,7 +358,7 @@ ls22_PageLoadLoop(tls00_FloadCmd*  pFloadCmd,
                   tsp00_Addr       pszErrText)
 {
     ROUTINE_DBG_MEO00 ("ls22_PageLoadLoop");
-    RepServMessages* pMsgObj = RepServMessages::Instance();
+    LoaderMessages* pMsgObj = LoaderMessages::Instance();
 
     tsp00_Int4  rc                 = errOK_els00;
     tsp00_Int4  rcFill             = errOK_els00;
@@ -418,7 +388,7 @@ ls22_PageLoadLoop(tls00_FloadCmd*  pFloadCmd,
         //*
         //*     Fill SQL-Packet (by building pages and filling them into the packet)
         //*
-        rcFill = ls22_FillPacket(pFloadCmd, pFloadRec, pDBInfo, lSndPcktNo, pVFile, &PcktStats[lSndPcktNo], pszErrText);
+        rcFill = ls22_FillPacket(pTransformationCmd, pFloadRec, pDBInfo, lSndPcktNo, pVFile, &PcktStats[lSndPcktNo], pszErrText);
         if (lSndPcktNo != lRcvPcktNo)
         {
             //*
@@ -538,8 +508,10 @@ ls22_PageLoadLoop(tls00_FloadCmd*  pFloadCmd,
         pFloadRec->flrLastCommLine_ls00  = SentValues.lRecordCount_ls22;
         pFloadRec->flrRecordCount_ls00   = SentValues.lRecordCount_ls22;
         pFloadRec->flrPageCount_ls00     = SentValues.lPageCount_ls22;
-        pFloadRec->flrLenKeyAcp_ls00     = SentValues.lLastKeyLength_ls22;
-        memcpy(pFloadRec->flrLstKeyAcp_ls00, SentValues.LastKey_ls22, pFloadRec->flrLenKeyAcp_ls00);
+        if (pFloadRec->flrRecordCount_ls00 > 0) {
+            pFloadRec->flrLenKeyAcp_ls00     = SentValues.lLastKeyLength_ls22;
+            memcpy(pFloadRec->flrLstKeyAcp_ls00, SentValues.LastKey_ls22, pFloadRec->flrLenKeyAcp_ls00);
+        }
     }
 
     return rc;
@@ -552,15 +524,15 @@ ls22_PageLoadLoop(tls00_FloadCmd*  pFloadCmd,
    function:     ls22_FillPacket
   -----------------------------------------------------------------------------
 */
-int ls22_FillPacket(tls00_FloadCmd*  pFloadCmd,
-                    tls00_FloadRec*  pFloadRec,
-                    tls00_DBInfo*    pDBInfo,
-                    tsp00_Int4       lPcktNo,
-                    tls00_VFile*     pVFile,
-                    tls22_Statistics *pPcktStats,
-                    tsp00_Addr       ErrText)
+int ls22_FillPacket(tls00_TransformationCmd*  pTransformationCmd,
+                    tls00_FloadRec*    pFloadRec,
+                    tls00_DBInfo*      pDBInfo,
+                    tsp00_Int4         lPcktNo,
+                    tls00_VFile*       pVFile,
+                    tls22_Statistics   *pPcktStats,
+                    tsp00_Addr         ErrText)
 {
-    ROUTINE_DBG_MEO00 ("ls22_FillPacket");
+   ROUTINE_DBG_MEO00 ("ls22_FillPacket");
 
     tsp00_Int4   ErrCode       = errOK_els00;
     tsp00_Int4   rcREAD        = errOK_els00;
@@ -578,7 +550,7 @@ int ls22_FillPacket(tls00_FloadCmd*  pFloadCmd,
     tsp00_SwapKind_Enum SSwap   = pFloadRec->flrSwapKind_ls00;
     tsp00_SwapKind_Enum DSwap   = pFloadRec->flrTableInfo_ls00.tdKernelSwap_ls00;
 
-    tls00_FileSpec *pInFileSpec = &pFloadCmd->flcInfile_ls00;
+    tls00_FileSpec *pInFileSpec = &pTransformationCmd->trDataFile_ls00;
     tls00_String   DataRecord;
     DataRecord.strAddr_ls00 = STATIC_CAST(char*, pFloadRec->flrDataSource_ls00.dasInputBuffer.pBufPtr_ls00);
 
@@ -595,7 +567,7 @@ int ls22_FillPacket(tls00_FloadCmd*  pFloadCmd,
     ls13Prepare_FL_TL_Packet(pDBInfo, lPcktNo, SqlPacket, SqlPart_page);
 
 
-    SAPDB_Bool bOfNoInterest = SAPDB_TRUE;
+    SAPDB_Bool bOfNoInterest = SAPDB_FALSE;
 
     //*
     //*     2 while loops to fill the data pages
@@ -620,12 +592,12 @@ int ls22_FillPacket(tls00_FloadCmd*  pFloadCmd,
         while ( (Space > pFloadRec->flrRecordLen_ls00) && (errOK_els00 == ErrCode) && (SAPDB_FALSE == bLimitReached) )
         {
             rcREAD = ls20ReadData(pVFile,
-                                  &pFloadCmd->flcInfile_ls00.fsFormat_ls00,
+                                  &pTransformationCmd->trDataFile_ls00.fsFormat_ls00,
                                   &pFloadRec->flrPattern_ls00,
                                   NULL,
                                   &pFloadRec->flrDataSource_ls00.dasInputBuffer,
                                   &bOfNoInterest,
-                                  (0 == pFloadRec->flrRecordCount_ls00),
+                                  (0== pFloadRec->flrRecordCount_ls00),
                                   ErrText);
             if (rcREAD < 0)
             {
@@ -655,9 +627,9 @@ int ls22_FillPacket(tls00_FloadCmd*  pFloadCmd,
             //*
             //*     Check for condition in command
             //*
-            if (NULL != pFloadCmd->flcTable_ls00.tsCondSpec_ls00)
+            if (NULL != pTransformationCmd->trTable_ls00.tsCondSpec_ls00)
             {
-                if ( false == ls16CheckCondition(*pFloadCmd->flcTable_ls00.tsCondSpec_ls00,
+                if ( false == ls16CheckCondition(*pTransformationCmd->trTable_ls00.tsCondSpec_ls00,
                                                  DataRecord,
                                                  pInFileSpec->fsFormat_ls00) )
                 {
@@ -668,7 +640,7 @@ int ls22_FillPacket(tls00_FloadCmd*  pFloadCmd,
             //*
             //*     Insert record into data page
             //*
-            RecordLen = ls23BuildRecord(Buffer, pFloadCmd, pFloadRec, pDBInfo->dbiSession_ls00.siSQLMode_ls00, ErrText);
+            RecordLen = ls23BuildRecord(Buffer, pTransformationCmd, pFloadRec, pDBInfo->dbiSession_ls00.siSQLMode_ls00, ErrText);
             
             ls18RecordInit(pFloadRec->flrDataSource_ls00.lReadFieldsCnt_ls00,
                            RawData,
@@ -682,13 +654,13 @@ int ls22_FillPacket(tls00_FloadCmd*  pFloadCmd,
                     tsp00_Int8 ErrorLine = (UNDEFINED_LS00 == pInFileSpec->fsExtract_ls00.feStart_ls00.lnuRemainder_ls00) ?
                                             pVFile->vfFileLineNum_ls00 :
                                             pVFile->vfFileLineNum_ls00 + pInFileSpec->fsExtract_ls00.feStart_ls00.lnuRemainder_ls00 - 1;
-                    RepServMessages::Instance()->ls98PrintMessage(layGRec_els98, errAtRow2_els98, ErrorLine);
+                    LoaderMessages::Instance()->ls98PrintMessage(layGRec_els98, errAtRow2_els98, ErrorLine);
                 }
                 else
                 {
                     tsp00_Int8 ErrorPos = pVFile->vfFilePos_ls00.lnu2GB_Blocks_ls00*MAX_INT4_SP00 +
                                            pVFile->vfFilePos_ls00.lnuRemainder_ls00;
-                    RepServMessages::Instance()->ls98PrintMessage(layGRec_els98, errAtPos2_els98, ErrorPos);
+                    LoaderMessages::Instance()->ls98PrintMessage(layGRec_els98, errAtPos2_els98, ErrorPos);
                 }
 
                 pFloadRec->flrLRejected_ls00 -= 1;
@@ -723,9 +695,9 @@ int ls22_FillPacket(tls00_FloadCmd*  pFloadCmd,
             //*
             //*     Check for user specified filling level for a single database page (in rows)
             //*
-            if (true == pFloadCmd->flcUsage_ls00.uUsingRows_ls00)
+            if (true == pTransformationCmd->trUsage_ls00.uUsingRows_ls00)
             {
-                if (nPageRecCount == pFloadCmd->flcUsage_ls00.uUsageValue_ls00)
+                if (nPageRecCount == pTransformationCmd->trUsage_ls00.uUsageValue_ls00)
                 {
                     bLimitReached = SAPDB_TRUE;
                 }
@@ -739,7 +711,8 @@ int ls22_FillPacket(tls00_FloadCmd*  pFloadCmd,
             //*
             if (UNDEFINED_LS00 != pInFileSpec->fsExtract_ls00.feEnd_ls00.lnuRemainder_ls00)
             {
-                if ( (pFloadRec->flrRecordCount_ls00 + pPcktStats->lRecordCount_ls22 + nPageRecCount) == pInFileSpec->fsExtract_ls00.feEnd_ls00.lnuRemainder_ls00 )
+                if (pInFileSpec->fsExtract_ls00.feEnd_ls00.lnuRemainder_ls00 == 
+                            (pFloadRec->flrRecordCount_ls00 + nPageRecCount + pPcktStats->lRecordCount_ls22) )
                 {
                     ErrCode       = STOP_LS22;
                     bLimitReached = SAPDB_TRUE;
@@ -848,7 +821,7 @@ ls22_ReceiveBdinfoPckt(tls00_DBInfo*      pDBInfo,
             rcCheck = ls22_RepairPacket(pDBInfo, pFloadRec, lRcvPcktNo, pPacketStatistics, _szErrText_UTF8);
 
             // Log sql error to log file
-            RepServMessages::Instance()->ls98PrintMessage(layFastload_els98, errFL_IntegrityViolation_els98,
+            LoaderMessages::Instance()->ls98PrintMessage(layFastload_els98, errFL_IntegrityViolation_els98,
                                                                              rc, _szErrText_UTF8.StrPtr());
 
             // Valid return values for rcCheck are:
@@ -894,7 +867,7 @@ ls22_ReceiveBdinfoPckt(tls00_DBInfo*      pDBInfo,
             if (pFloadRec->flrLRejected_ls00 <= 0)
             {
                 // report this error now as we exceed the maximum number of admissible errors
-                RepServMessages::Instance()->ls98MsgToString(layFastload_els98, pszErrText, errFL_IntegrityViolation_els98, rc, "");
+                LoaderMessages::Instance()->ls98MsgToString(layFastload_els98, pszErrText, errFL_IntegrityViolation_els98, rc, "");
                 rc = errFL_IntegrityViolation_els98;
             }
             else
@@ -1219,8 +1192,8 @@ ls22_RepairPage(tbd_node*                 pDataPage,
                 // _lPtrListEntry is the position of the first invalid record in a number of invalid records
                 //  and thus defines the distance to copy
                 memcpy(&_pPageBody[_lBottom - BODY_BEG_BD00],
-                       &_pPageBody[_lPosOf1stValidRecord - BODY_BEG_BD00],
-                       _lPtrListEntry - _lPosOf1stValidRecord);
+                    &_pPageBody[_lPosOf1stValidRecord - BODY_BEG_BD00],
+                    _lPtrListEntry - _lPosOf1stValidRecord);
 
                 // Adapt pointer list values for copied valid records
                 _lPtrListEntry_Retain = _plMyPtrList[_lValidRecordsOnPage - _lValidRecords];
@@ -1249,13 +1222,13 @@ ls22_RepairPage(tbd_node*                 pDataPage,
             
             // key length and key
             memcpy(&pPacketStatistics->lLastKeyLength_ls22,
-                   &_pPageBody[_lPtrListEntry - BODY_BEG_BD00] + KEYLEN_OFFSET_LS23,
-                   sizeof(tsp00_Int2));
+                &_pPageBody[_lPtrListEntry - BODY_BEG_BD00] + KEYLEN_OFFSET_LS23,
+                sizeof(tsp00_Int2));
             ls07Swap2BCharInPlace(REINTERPRET_CAST(char*, &pPacketStatistics->lLastKeyLength_ls22), SSwap, DSwap);
 
             memcpy(pPacketStatistics->LastKey_ls22,
-                   &_pPageBody[_lPtrListEntry - BODY_BEG_BD00] + HEADER_OFFSET_LS23,
-                   pPacketStatistics->lLastKeyLength_ls22);
+                &_pPageBody[_lPtrListEntry - BODY_BEG_BD00] + HEADER_OFFSET_LS23,
+                pPacketStatistics->lLastKeyLength_ls22);
 
             // Set bottom value
             // The used algorithm assures that there is always one more pointer list entry written than
