@@ -1,25 +1,16 @@
 /*! 
   -----------------------------------------------------------------------------
- 
   module: vls23.cpp
- 
   -----------------------------------------------------------------------------
  
-  responsible:  SteffenS and d033893
+  responsible:  SteffenS, d033893
 
-  special area: Replication Manager
+  special area: Loader
  
   description:  Build single database record
                 
   version:      7.5.
-
   -----------------------------------------------------------------------------
- 
-  copyright:    (c) 1997-2004 SAP AG-2004
- 
-  -----------------------------------------------------------------------------
-
-
 
     ========== licence begin  GPL
     Copyright (c) 1997-2004 SAP AG
@@ -38,9 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 #include <assert.h>
 
@@ -71,6 +59,7 @@
   -----------------------------------------------------------------------------
   function:     ls23_BuildDefault - local function
   -----------------------------------------------------------------------------
+
   description:  Assigns default or NULL values to a buffer depending on the
                 data type and column type.
   
@@ -101,6 +90,7 @@ ls23_BuildDefault(tsp00_Addr           pszDestination,
   -----------------------------------------------------------------------------
   function:     ls23_FillValue - local function
   -----------------------------------------------------------------------------
+
   description:  Fills prepared values into page buffer. Converts to UCS2 if
                 necessary and adjusts the length of the values depending on their
                 length type in the db.
@@ -157,22 +147,22 @@ ls23_FillValue(tsp00_Addr      pszDestination,
    function:     ls23BuildRecord
   -----------------------------------------------------------------------------
 */
-int ls23BuildRecord(tsp00_Addr         Buffer, 
-                    tls00_FloadCmd    *pFLCmd,
-                    tls00_FloadRec    *pFLRec,
-                    tsp1_sqlmode_Enum  SQLMode,
-                    tsp00_Addr         ErrText)
+int ls23BuildRecord(tsp00_Addr               Buffer, 
+                    tls00_TransformationCmd  *pTransformationCmd,
+                    tls00_FloadRec           *pFLRec,
+                    tsp1_sqlmode_Enum        SQLMode,
+                    tsp00_Addr               ErrText)
 {
-    ROUTINE_DBG_MEO00 ("ls23BuildRecord");
+   ROUTINE_DBG_MEO00 ("ls23BuildRecord");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance();    // error reporting obj
+    LoaderMessages *pMsgObj = LoaderMessages::Instance();    // error reporting obj
     tsp00_Longint    rc      = errOK_els00;
     
     tls00_TableDescription *pTI = &pFLRec->flrTableInfo_ls00;
     tls00_FieldInfo        *pFI = NULL;
-    tls00_FileFormat       &Format      = pFLCmd->flcInfile_ls00.fsFormat_ls00;
+    tls00_FileFormat       &Format      = pTransformationCmd->trDataFile_ls00.fsFormat_ls00;
     tls00_CodeType          ExtEncoding = Format.ffoCodeType_ls00;
-    tls00_MultColSpec      *MultColumns = &pFLCmd->flcColSpec_ls00;
+    tls00_MultColSpec      *MultColumns = &pTransformationCmd->trColSpec_ls00;
     tls00_String           *RawData     = pFLRec->flrDataSource_ls00.dasConvertedData;
     tls00_Column           *pCol        = NULL;
 
@@ -194,7 +184,7 @@ int ls23BuildRecord(tsp00_Addr         Buffer,
     {
         rc = ls18GetFormattedValueEx(InputLine.strAddr_ls00,//pFLRec->flrDataSource_ls00.dasInputBuffer.strAddr_ls00,
                                      InputLine.strLeng_ls00,
-                                     &pFLCmd->flcColSpec_ls00,
+                                     &pTransformationCmd->trColSpec_ls00,
                                      pFLRec->flrDataSource_ls00.lReadFieldsCnt_ls00,
                                      pFLRec->flrDataSource_ls00.plDataToColIndex_ls00,
                                      RawData,
@@ -202,14 +192,14 @@ int ls23BuildRecord(tsp00_Addr         Buffer,
     }
     else
     {
-        rc = ls18GetUnformattedValueEx(InputLine.strAddr_ls00,//pFLRec->flrDataSource_ls00.dasInputBuffer.strAddr_ls00,
-                                       InputLine.strLeng_ls00,
-                                       &pFLRec->flrPattern_ls00,
-                                       pFLRec->flrDataSource_ls00.lReadFieldsCnt_ls00,
-                                       pFLRec->flrDataSource_ls00.lReadPos_ls00,
-                                       pFLCmd->flcInfile_ls00.fsFormat_ls00.ffoCodeType_ls00,
-                                       RawData,
-                                       lFailIndex);
+        rc = ls18GetCSVData(InputLine.strAddr_ls00,//pFLRec->flrDataSource_ls00.dasInputBuffer.strAddr_ls00,
+                            InputLine.strLeng_ls00,
+                            &pFLRec->flrPattern_ls00,
+                            pFLRec->flrDataSource_ls00.lReadFieldsCnt_ls00,
+                            pFLRec->flrDataSource_ls00.lReadPos_ls00,
+                            pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00,
+                            RawData,
+                            lFailIndex);
     }
     if (errOK_els00 != rc)
     {
@@ -317,12 +307,12 @@ int ls23BuildRecord(tsp00_Addr         Buffer,
             //*
             //*     Convert values to internal encoding (ASCII) for further processing
             //*
-            if ( (ctAscii_els00 != pFLCmd->flcInfile_ls00.fsFormat_ls00.ffoCodeType_ls00) &&
+            if ( (ctAscii_els00 != pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00) &&
                  (ioChar_els00 == InOutType) )
             {
                 rc = ls20ConvertValue2InternEncoding(RawData[i],
                                                      pFI->fiBasics_ls00.fbDataType_ls00,
-                                                     pFLCmd->flcInfile_ls00.fsFormat_ls00.ffoCodeType_ls00,
+                                                     pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00,
                                                      ErrText);
                 if (errOK_els00 != rc)
                 {
@@ -565,7 +555,7 @@ int ls23BuildRecord(tsp00_Addr         Buffer,
             return -STATIC_CAST(tsp00_Int4, rc);
         }
 
-        memcpy(&Buffer[plBufLen], pFLRec->flrLstKeyAcp_ls00, pFLRec->flrLenKeyAcp_ls00);
+        SAPDB_memcpy(&Buffer[plBufLen], pFLRec->flrLstKeyAcp_ls00, pFLRec->flrLenKeyAcp_ls00);
         plBufLen += pFLRec->flrLenKeyAcp_ls00;    // mv bufpointer behind copied value
 
         lKeyLen += pFLRec->flrLenKeyAcp_ls00;
@@ -642,7 +632,7 @@ int ls23BuildRecord(tsp00_Addr         Buffer,
             if (l_greater == CompVal)
             {
                 // key value starts BEHIND record header (BUF_START_POS_LS23 + HEADER_OFFSET_LS23)
-                memcpy(pFLRec->flrLstKeyAcp_ls00, &Buffer[HEADER_OFFSET_LS23], lKeyLen);
+                SAPDB_memcpy(pFLRec->flrLstKeyAcp_ls00, &Buffer[HEADER_OFFSET_LS23], lKeyLen);
                 pFLRec->flrLenKeyAcp_ls00 = lKeyLen;
             }
             else    // new key is less or result is undefined
@@ -882,7 +872,8 @@ ls23_FillValue(tsp00_Addr      pszDestination,
                tsp00_Int4      lValIndex,
                tsp00_Addr      pszErrText)
 {
-    RepServMessages *pMsgObj = RepServMessages::Instance();    // error reporting obj
+
+    LoaderMessages *pMsgObj = LoaderMessages::Instance();    // error reporting obj
     tsp00_Longint    rc      = errOK_els00;
     
     tls00_TableDescription &TD  = pFLRec->flrTableInfo_ls00;
@@ -963,7 +954,7 @@ ls23_FillValue(tsp00_Addr      pszDestination,
             //*
             if (0 != pSrcData->strLeng_ls00)        // char values might be empty (consist entirely of blanks)
             {
-                memcpy(&pszDestination[sizeof(char)], pSrcData->strAddr_ls00, pSrcData->strLeng_ls00);
+                SAPDB_memcpy(&pszDestination[sizeof(char)], pSrcData->strAddr_ls00, pSrcData->strLeng_ls00);
                 lDestLen += pSrcData->strLeng_ls00 + sizeof(char);
             }
             else
@@ -985,11 +976,11 @@ ls23_FillValue(tsp00_Addr      pszDestination,
     //* In case of external encoding type UCS2 we have only to check the swap type and
     //* swap the char data to the right type.
     //*
-    if ( (rc == errOK_els00 )      &&
-         (lDestLen > sizeof(char)) &&       // http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1127787
+    if ( (rc == errOK_els00 ) &&
+         (lDestLen > sizeof(char)) &&       // http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1127784
          ( (1 == szMap_UnicodeDataType_ls00[pFI->fiBasics_ls00.fbDataType_ls00]) && (false == bDefaultIsUnicode) ) )
     {
-        if ( (ctUCS2_els00 == SrcDataEncoding) || (ctUCS2Swapped_els00 == SrcDataEncoding)  )
+        if ( true == ls00IsUCS2(&SrcDataEncoding) )
         {
             // This step assumes running the load server on the same machine that stores the data
             if (sw_full_swapped == pFLRec->flrSwapKind_ls00)
@@ -1001,7 +992,7 @@ ls23_FillValue(tsp00_Addr      pszDestination,
         {
             unsigned char szTempBuffer[PAGE_MXSP00]; // a single value may not be bigger than a whole db page
 
-            memcpy(szTempBuffer, pszDestination + sizeof(char), lDestLen - sizeof(char));   // skip defined byte
+            SAPDB_memcpy(szTempBuffer, pszDestination + sizeof(char), lDestLen - sizeof(char));   // skip defined byte
 
             tsp00_Longuint lTmpSrcLen  = pFI->fiBasics_ls00.fbInOutLength_ls00;
 
