@@ -5,17 +5,17 @@
  
   -----------------------------------------------------------------------------
  
-  responsible:  SteffenS
+  responsible:  SteffenS, d033893
 
-  special area: Replication Server
+  special area: Loader
  
   description:  Implementation of transformation routines for numeric values
                 
-  version:      7.4.
+  version:      7.5.
 
   -----------------------------------------------------------------------------
  
-  copyright:    (c) 1998-2004 SAP AG-2002
+  Copyright (c) 1998-2004 SAP AG-2002
  
   -----------------------------------------------------------------------------
 
