@@ -5,9 +5,9 @@
 
   -----------------------------------------------------------------------------
 
-  responsible:  SteffenS and d033893
+  responsible:  SteffenS, d033893
 
-  special area: SAP DB LOADER
+  special area: Loader
 
   description:  Implementation of TABLEEXTRACT for a single table
 
@@ -15,12 +15,6 @@
 
   -----------------------------------------------------------------------------
 
-  copyright:    (c) 2000-2004 SAP AG-2003
-
-  -----------------------------------------------------------------------------
-
-
-
     ========== licence begin  GPL
     Copyright (c) 2000-2004 SAP AG
 
@@ -45,6 +39,7 @@
 #include <time.h>
 
 #include "gls00.h"
+#include "hls01.h"
 #include "hls04.h"
 #include "hls03.h"
 #include "hls05.h"
@@ -74,10 +69,10 @@
   -----------------------------------------------------------------------------
 */
 tsp00_Int4
-ls26WriteMetaHeader(tls00_DBInfo      *pDBInfo,
-                    tls00_TabExtrCmd  *pTabExtrCmd,
-                    tls00_TextractRec *pTabExtrRec,
-                    tsp00_Addr         ErrText);
+ls26WriteMetaHeader(tls00_DBInfo             *pDBInfo,
+                    tls00_TransformationCmd  *pTransformationCmd,
+                    tls00_TextractRec        *pTabExtrRec,
+                    tsp00_Addr               ErrText);
 
 /*
   -----------------------------------------------------------------------------
@@ -511,25 +506,24 @@ ls26_WLongDataHeaderPage(tls00_TextractRec *pTabExtrRec,
   -----------------------------------------------------------------------------
 */
 tsp00_Int4
-ls26ExtractTablePages(MetaDataDef         *&pMetaDataDef ,
-                      tls00_TabExtrCmd    *pTabExtrCmd,
-                      tls00_TextractRec   *pTabExtrRec,
-                      tsp00_Addr           ErrText)
+ls26ExtractTablePages(MetaDataDef       *&pMetaDataDef ,
+                      tls00_TextractRec *pTabExtrRec,
+                      tsp00_Addr        ErrText)
 {
     ROUTINE_DBG_MEO00 ("ls26ExtractTablePages");
 
-    tsp00_Int4       rc           = errOK_els00;
-    RepServMessages *pMsgObj = RepServMessages::Instance();
-
-	TransformationDef* pTransformationDef = pMetaDataDef->getTransformationModel();
-	tls00_DBInfo     * pDBInfo            = pMetaDataDef->getConnection();
+	LoaderMessages *pMsgObj = LoaderMessages::Instance();
+    tsp00_Int4       rc     = errOK_els00;
 
+	TransformationDef       * pTransformationDef = pMetaDataDef->getTransformationModel();
+    tls00_TransformationCmd * pTransformationCmd = pTransformationDef->getTransformationCmd();
+	tls00_DBInfo            * pDBInfo            = pMetaDataDef->getConnection();
     //*
     //*  update transformation table and write metadata header
     //*
     if ( errOK_els00 == rc )
     {
-         rc = ls26UpdateTransformationPackage(pDBInfo, pMetaDataDef , pTabExtrCmd, pTabExtrRec, ErrText);
+         rc = ls26UpdateTransformationPackage(pDBInfo, pMetaDataDef , pTransformationCmd, pTabExtrRec, ErrText);
 
          if (errOK_els00 != rc )
          {
@@ -538,17 +532,15 @@ ls26ExtractTablePages(MetaDataDef         *&pMetaDataDef ,
          }
          rc = ls15RSCommit(pDBInfo, ErrText);
 
-         ls26WriteMetaHeader(pDBInfo, pTabExtrCmd, pTabExtrRec, ErrText );
+         ls26WriteMetaHeader(pDBInfo, pTransformationCmd, pTabExtrRec, ErrText );
     }
-
-
     //*
     //*     Convert ownername and tablename (if given) to UTF8 strings
     //*
     Tools_DynamicUTF8String  ownername;
     Tools_DynamicUTF8String  tablename;
 
-    if (ptUser_els00 == pTabExtrCmd->texPartSpec_ls00.psPartType_ls00)
+    if (ptUser_els00 == pTransformationCmd->trPartSpec_ls00.psPartType_ls00)
     {
         //*
         //*     In case of TABLEEXTRACT USER the connected user IS the owner of the extracted tables
@@ -560,24 +552,23 @@ ls26ExtractTablePages(MetaDataDef         *&pMetaDataDef ,
     }
     else
     {
-        if (0 != pTabExtrCmd->texPartSpec_ls00.psTabOwnr_ls00.length())
+        if (0 != pTransformationCmd->trPartSpec_ls00.psTabOwnr_ls00.length())
         {
             rc = ls05AssignUTF8(ownername,
-                                REINTERPRET_CAST(unsigned char*, pTabExtrCmd->texPartSpec_ls00.psTabOwnr_ls00.asCharp()),
-                                pTabExtrCmd->texPartSpec_ls00.psTabOwnr_ls00.length(),
+                                REINTERPRET_CAST(unsigned char*, pTransformationCmd->trPartSpec_ls00.psTabOwnr_ls00.asCharp()),
+                                pTransformationCmd->trPartSpec_ls00.psTabOwnr_ls00.length(),
                                 ErrText);
         }
     }
     rc = ls05AssignUTF8(tablename,
-                        REINTERPRET_CAST(unsigned char*, pTabExtrCmd->texPartSpec_ls00.psTabName_ls00.asCharp()),
-                        pTabExtrCmd->texPartSpec_ls00.psTabName_ls00.length(),
+                        REINTERPRET_CAST(unsigned char*, pTransformationCmd->trPartSpec_ls00.psTabName_ls00.asCharp()),
+                        pTransformationCmd->trPartSpec_ls00.psTabName_ls00.length(),
                         ErrText);
 
-
     //*
     //*     Extract table data and necessary meta data
     //*
-    if ( (errOK_els00 == rc) && (false == pTabExtrCmd->texExclude_ls00) )
+    if ( (errOK_els00 == rc) && (false == pTransformationCmd->trExclude_ls00) )
     {
         //*
         //*     Get list of contraint and default definitons
@@ -585,7 +576,7 @@ ls26ExtractTablePages(MetaDataDef         *&pMetaDataDef ,
         //* 'DESCRIBE TABLE'; this command locks the table definition until a commit has been
         //* issued and no information can be retrieved from the db
         //*
-        TableDef *pTableDef = pMetaDataDef ->getTableDef();
+		TableDef *pTableDef = pMetaDataDef->getRelationalModel()->getTableDef();
         if (errOK_els00 == rc)
         {
             rc = pTableDef->Init(ownername, tablename);
@@ -598,7 +589,6 @@ ls26ExtractTablePages(MetaDataDef         *&pMetaDataDef ,
 
         }
 
-
         // Needed to decide whether to save short LONG column pages or not
         bool bShortColumnFileAvailable = false;
 
@@ -736,7 +726,7 @@ ls26ExtractTablePages(MetaDataDef         *&pMetaDataDef ,
             pTabExtrRec->terTabExtracted_ls00  = true;
         }
 
-    } // if ( false == pTabExtrCmd->texExclude_ls00 )
+    } // if ( false == pTransformationCmd->trExclude_ls00 )
 
     //*
     //*  update transformation table and write metadata trailer
@@ -745,7 +735,7 @@ ls26ExtractTablePages(MetaDataDef         *&pMetaDataDef ,
     {
         ls26_WriteMetaTrailer(pDBInfo, ownername, tablename, pTabExtrRec, ErrText);
 
-        rc = ls26UpdateTransformationPackage(pDBInfo, pMetaDataDef , pTabExtrCmd, pTabExtrRec, ErrText);
+        rc = ls26UpdateTransformationPackage(pDBInfo, pMetaDataDef , pTransformationCmd, pTabExtrRec, ErrText);
 
         if (errOK_els00 != rc )
         {
@@ -754,7 +744,7 @@ ls26ExtractTablePages(MetaDataDef         *&pMetaDataDef ,
         }
         rc = ls15RSCommit(pDBInfo, ErrText);
     }
-	if ( false == pTabExtrCmd->texInternal_ls00 && errOK_els00 == rc )
+	if ( false == pTransformationCmd->trInternal_ls00 && errOK_els00 == rc )
 		 pMsgObj->ls98PrintMessage(layTableExtract_els98, msgTESuccessful_els98);
 
     return rc;
@@ -769,7 +759,7 @@ ls26ExtractTablePages(MetaDataDef         *&pMetaDataDef ,
 */
 tsp00_Int4
 ls26WriteMetaHeader(tls00_DBInfo      *pDBInfo,
-                    tls00_TabExtrCmd  *pTabExtrCmd,
+                    tls00_TransformationCmd  *pTransformationCmd,
                     tls00_TextractRec *pTabExtrRec,
                     tsp00_Addr         ErrText )
 {
@@ -777,7 +767,7 @@ ls26WriteMetaHeader(tls00_DBInfo      *pDBInfo,
 
     tsp00_Int4 rc = errOK_els00;
 
-    RepServMessages  *MsgInstance = RepServMessages::Instance();
+    LoaderMessages  *MsgInstance = LoaderMessages::Instance();
 
     assert(pTabExtrRec->terOutFile_ls00->vfDataType_ls00 == sp5vf_binary);
 
@@ -852,8 +842,8 @@ ls26WriteMetaHeader(tls00_DBInfo      *pDBInfo,
         lBufPos += sizeof(bMultipleTables);
 
 		// write command to buffer
-        memcpy(&pszBuffer[lBufPos], pTabExtrCmd->texFullUserInput_ls00, _tcslen(pTabExtrCmd->texFullUserInput_ls00));
-        lBufPos += _tcslen(pTabExtrCmd->texFullUserInput_ls00);
+        memcpy(&pszBuffer[lBufPos], pTransformationCmd->trFullUserInput_ls00, _tcslen(pTransformationCmd->trFullUserInput_ls00));
+        lBufPos += _tcslen(pTransformationCmd->trFullUserInput_ls00);
 
         // write structure to file; updates file pos
         rc = ls30VFWriteLn(*pTabExtrRec->terOutFile_ls00, pszBuffer, len, ErrText); // error is already logged by ls30VFWriteLn
@@ -1312,7 +1302,6 @@ ls26_WriteShortStringPages(tls00_DBInfo            *pDBInfo,
 
     tsp00_Int4 rc = errOK_els00;
 
-
     _TCHAR szCmd[lStmtLength_ls26];
     szCmd[0] = 0;
     sp77sprintf(szCmd, lStmtLength_ls26, SAVE_LONG_STMT_LS26, TableName.StrPtr());
@@ -1320,10 +1309,10 @@ ls26_WriteShortStringPages(tls00_DBInfo            *pDBInfo,
     tsp00_Int4 lErrPos;
     _TCHAR     szErr[MAX_REPLY_LENGTH_LS00];
 
-    rc = ls13ExecIntSQLStmtUTF8(pDBInfo, 0, szCmd, szErr, lErrPos);
+    rc = ls13ExecIntSQLStmt(pDBInfo, 0, szCmd, szErr, lErrPos);
     if (errOK_els00 != rc)
     {
-        RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternSQL_els98,
+        LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternSQL_els98,
                                                                                 szCmd, szErr);
         rc = errInternSQL_els98;
     }
@@ -1334,7 +1323,7 @@ ls26_WriteShortStringPages(tls00_DBInfo            *pDBInfo,
 
         if ( false == ls04GetPart(SqlPacket, sp1pk_bdinfo, SqlPart) )
         {
-            RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
                                                                                     __FILE__,
                                                                                     __LINE__,
                                                                                     errUnknownPart_els98);
@@ -1386,10 +1375,10 @@ ls26_WritePrimaryDataPages(tls00_DBInfo            *pDBInfo,
     tsp00_Int4 lErrPos;
     _TCHAR     szErr[MAX_REPLY_LENGTH_LS00];
 
-    rc = ls13ExecIntSQLStmtUTF8(pDBInfo, 0, szCmd, szErr, lErrPos);
+    rc = ls13ExecIntSQLStmt(pDBInfo, 0, szCmd, szErr, lErrPos);
     if (errOK_els00 != rc)
     {
-        RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternSQL_els98,
+        LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternSQL_els98,
                                                                                 szCmd, szErr);
         rc = errInternSQL_els98;
     }
@@ -1400,7 +1389,7 @@ ls26_WritePrimaryDataPages(tls00_DBInfo            *pDBInfo,
 
         if ( false == ls04GetPart(SqlPacket, sp1pk_bdinfo, SqlPart) )
         {
-            RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
                                                                                     __FILE__,
                                                                                     __LINE__,
                                                                                     errUnknownPart_els98);
@@ -1504,7 +1493,7 @@ ls26_WriteLongStringPages(tls00_DBInfo      *pDBInfo,
 
     while (1)
     {
-        rc = ls13ExecIntSQLMultPartsUTF8(pDBInfo, 0, myStruct, szErr, lErrorPos);
+        rc = ls13ExecIntSQLMultParts(pDBInfo, 0, myStruct, szErr, lErrorPos);
         if (errOK_els00 != rc)
         {
             if (100 == rc)      // no more LONGs to extract found by kernel
@@ -1513,7 +1502,7 @@ ls26_WriteLongStringPages(tls00_DBInfo      *pDBInfo,
             }
             else
             {
-                RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternSQL_els98,
+                LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternSQL_els98,
                                                                                         szCmd, szErr);
                 rc = errInternSQL_els98;
             }
@@ -1525,7 +1514,7 @@ ls26_WriteLongStringPages(tls00_DBInfo      *pDBInfo,
 
         if ( false == ls04GetPart(SqlPacket, sp1pk_bdinfo, SqlPart) )
         {
-            RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
                                                                                     __FILE__,
                                                                                     __LINE__,
                                                                                     errUnknownPart_els98);
@@ -1539,7 +1528,7 @@ ls26_WriteLongStringPages(tls00_DBInfo      *pDBInfo,
 
         if ( false == ls04GetPart(SqlPacket, sp1pk_key, SqlPart) )
         {
-            RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
                                                                                     __FILE__,
                                                                                     __LINE__,
                                                                                     errUnknownPart_els98);
@@ -1739,9 +1728,8 @@ ls26_WriteMetaTrailer(tls00_DBInfo             *pDBInfo,
         // datafile
         transformation += " DATA INSTREAM ";
 
-        infile.ConvertFromASCII_Latin1(pTabExtrRec->terOutFile_ls00->vfFileName_ls00,
-                                       pTabExtrRec->terOutFile_ls00->vfFileName_ls00 +
-                               _tcslen(pTabExtrRec->terOutFile_ls00->vfFileName_ls00));
+        infile.Assign(REINTERPRET_CAST(Tools_DynamicUTF8String::ConstPointer,
+                                       pTabExtrRec->terOutFile_ls00->vfFileName_ls00));
 
         transformation += "'";
         transformation += infile;
@@ -1813,6 +1801,8 @@ ls26_SendBdinfoPckt(tls00_DBInfo *pDBInfo,
 
     ls04SetMsgType(SqlSegm, sp1m_unload);
 
+    ls04SetPartKind(SqlPart, sp1pk_bdinfo);
+
     // Eigentlich muesste fuer die folgende funktion der fehlercode abgefragt werden. Der kann
     // allerdings nur beinhalten, dass der puffer zu klein ist. Da das hier aber das einzige
     // ist, was wir eintragen, reicht der puffer in jedem fall aus.
@@ -1830,7 +1820,7 @@ ls26_SendBdinfoPckt(tls00_DBInfo *pDBInfo,
     }
     else
     {
-        RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, STATIC_CAST(ErrorNumbers, rc));
+        LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, STATIC_CAST(ErrorNumbers, rc));
         rc = errFinishPart_els98;
     }
 
@@ -1879,7 +1869,7 @@ ls26_PageExtractLoop(tls00_DBInfo      *pDBInfo,
                 rc = ls13ReceiveBdinfoPckt(pDBInfo, lRcvPcktNo, pTabExtrRec->terBdInfo_ls00, pszErrText);
                 if (errOK_els00 != rc)
                 {
-                    RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
+                    LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
                                                                                             __FILE__,
                                                                                             __LINE__,
                                                                                             STATIC_CAST(ErrorNumbers, rc));
@@ -1887,13 +1877,19 @@ ls26_PageExtractLoop(tls00_DBInfo      *pDBInfo,
                     break;
                 }
             }
+            // Write message in case of cancel request
+            if (errCancelRequest_els98 == (rc = ls01CheckForCancel(pDBInfo->dbiSession_ls00.sipCancelByte_ls00) ))
+			{
+                LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errCancelRequest_els98);
+				break;
+			}
 
             //*     SQL-Packet senden
             //  Wenn erfolgreich passe die nummer des zu sendenden pakets an
             rc = ls26_SendBdinfoPckt(pDBInfo, lSndPcktNo, pTabExtrRec->terBdInfo_ls00, BDINFO_SIZE_LS00, pszErrText);
             if (errOK_els00 != rc)
             {
-                RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
+                LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
                                                                                         __FILE__,
                                                                                         __LINE__,
                                                                                         STATIC_CAST(ErrorNumbers, rc));
@@ -1933,13 +1929,16 @@ ls26_PageExtractLoop(tls00_DBInfo      *pDBInfo,
         tsp00_Int4 rc2 = ls13ReceiveBdinfoPckt(pDBInfo, lRcvPcktNo, pTabExtrRec->terBdInfo_ls00, pszErrText);
         if (errOK_els00 != rc2)
         {
-            RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
+            LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
                                                                                     __FILE__,
                                                                                     __LINE__,
                                                                                     STATIC_CAST(ErrorNumbers, rc2));
             rc = errInternal_els98;
         }
     }
+	// Write message in case of cancel request
+    if (errCancelRequest_els98 == (rc = ls01CheckForCancel(pDBInfo->dbiSession_ls00.sipCancelByte_ls00) ))
+        LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errCancelRequest_els98);
 
     return rc;
 }
@@ -1964,7 +1963,7 @@ ls26_Analyse(tsp1_packet_ptr    pSqlPacket,
     tsp1_part *pSqlPart = NULL;
     if ( false == ls04GetPart(pSqlPacket, sp1pk_page, pSqlPart) )
     {
-        RepServMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
+        LoaderMessages::Instance()->ls98Msg(layTableExtract_els98, pszErrText, errInternal_els98,
                                                                                 __FILE__,
                                                                                 __LINE__,
                                                                                 errUnknownPart_els98);
