@@ -1,25 +1,17 @@
 /*!
   -----------------------------------------------------------------------------
-
   module: vls29_dataupdate.cpp
-
   -----------------------------------------------------------------------------
 
   responsible:  d033893
 
-  special area: SAP DB LOADER
+  special area: Loader
 
   description:  dataupdate
 
   version:      7.5.
-
   -----------------------------------------------------------------------------
 
-  copyright:    (c) 1997-2004 SAP AG-2004
-
-  -----------------------------------------------------------------------------
-
-
     ========== licence begin  GPL
     Copyright (c) 1997-2004 SAP AG
 
@@ -58,16 +50,16 @@
 
 
 static tsp00_Int4
-ls29_InitDatUpdCmd(tls00_DBInfo    *pDBInfo,
-                   tls00_DatUpdCmd *pDatUpdCmd,
-                   tls00_DatUpdRec *pDatUpdRec,
-                   tsp00_Addr       errText);
+ls29_InitDatUpdCmd(tls00_DBInfo            *pDBInfo,
+                   tls00_TransformationCmd *pTransformationCmd,
+                   tls00_DatUpdRec         *pDatUpdRec,
+                   tsp00_Addr              errText);
 
 static tsp00_Longint
-ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
-                   tls00_DatUpdCmd  *pDatUpdCmd,
-                   tls00_DatUpdRec *&pDatUpdRec,
-                   tsp00_Addr        errText);
+ls29_InitDatUpdRec(tls00_DBInfo             *pDBInfo,
+                   tls00_TransformationCmd  *pTransformationCmd,
+                   tls00_DatUpdRec          *&pDatUpdRec,
+                   tsp00_Addr               errText);
 
 
 /*
@@ -82,7 +74,7 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
   arguments:    pDBInfo     [IN]    - A pointer to a structure that contains all
                                       information on the running session and the
                                       database connected to.
-                pDatUpdCmd  [IN]    - A pointer to a structure that contains all
+                pTransformationCmd  [IN]    - A pointer to a structure that contains all
                                       information on the dataupdate command given
                                       by the user
                 pDatUpdRec  [IN]    - A pointer to a structure that contains some
@@ -94,18 +86,18 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
   -----------------------------------------------------------------------------
 */
 static tsp00_Int4
-ls29_ExecDatUpdCmd(tls00_DBInfo*     pDBInfo,
-                   tls00_DatUpdCmd*  pDatUpdCmd,
-                   tls00_DatUpdRec *&pDatUpdRec,
-                   tsp00_Addr        errText);
+ls29_ExecDatUpdCmd(tls00_DBInfo           *  pDBInfo,
+                   tls00_TransformationCmd*  pTransformationCmd,
+                   tls00_DatUpdRec        *& pDatUpdRec,
+                   tsp00_Addr             errText);
 
 
 // record fields
 static tsp00_Longint
-ls29_GetRecFields(tls00_DatUpdCmd   *pDatUpdCmd,
-                  tls00_DatUpdRec   *pDatUpdRec,
-                  tsp1_sqlmode_Enum  SQLMode,
-                  tsp00_Addr         errText);
+ls29_GetRecFields(tls00_TransformationCmd   *pTransformationCmd,
+                  tls00_DatUpdRec           *pDatUpdRec,
+                  tsp1_sqlmode_Enum         SQLMode,
+                  tsp00_Addr                errText);
 
 
 // Recovery
@@ -148,7 +140,7 @@ static void
 ls29_FreeDatUpdRec(tls00_DatUpdRec *&pDatUpdRec, tsp00_Addr errText);
 
 static void
-ls29_FreeDatUpdCmd( tls00_DatUpdCmd* pDatUpdCmd);
+ls29_FreeDatUpdCmd( tls00_TransformationCmd* pTransformationCmd);
 
 
 /*
@@ -162,7 +154,7 @@ ls29_FreeDatUpdCmd( tls00_DatUpdCmd* pDatUpdCmd);
   arguments:    pDBInfo     [IN]    - A pointer to a structure that contains all
                                       information on the running session and the
                                       database connected to.
-                pDatUpdCmd  [IN]    - A pointer to a structure that contains all
+                pTransformationCmd  [IN]    - A pointer to a structure that contains all
                                       information on the dataupdate command given
                                       by the user
                 pDatUpdRec  [IN]    - A pointer to a structure that contains some
@@ -181,63 +173,67 @@ ls29_FreeDatUpdCmd( tls00_DatUpdCmd* pDatUpdCmd);
   -----------------------------------------------------------------------------
 */
 static tsp00_Longint
-ls29_GetNewDatPartBuf(tls00_DBInfo    *pDBInfo,
-                      tls00_DatUpdCmd *pDatUpdCmd,
-                      tls00_DatUpdRec *pDatUpdRec,
-                      tsp00_Int4      &iRecCount ,
-                      bool            &bPutVal,
-                      tsp00_Addr       errText);
+ls29_GetNewDatPartBuf(tls00_DBInfo            *pDBInfo,
+                      tls00_TransformationCmd *pTransformationCmd,
+                      tls00_DatUpdRec         *pDatUpdRec,
+                      tsp00_Int4              &iRecCount ,
+                      bool                    &bPutVal,
+                      tsp00_Addr              ErrText);
 
 
 // Definition - Globale functions
 
 /* -----------------------------------------------------------------------------
-   ls29ExecDataUpdate
+   ls29ExecColumnUpdate
 /-------------------------------------------------------------------------------*/
 tsp00_Int4
-ls29ExecDataUpdate(tls00_DatUpdCmd* pDatUpdCmd, tls00_DBInfo *pDBInfo, tsp00_Addr errText)
+ls29ExecColumnUpdate(MetaDataDef *pMetaDataDef, tls00_DBInfo *pDBInfo, tsp00_Addr errText)
 {
-    ROUTINE_DBG_MEO00 ("ls29ExecDataUpdate");
+    ROUTINE_DBG_MEO00 ("ls29ExecColumnUpdate");
+
+    tsp00_Longint    rc   = errOK_els00;
+
+    TransformationDef       * pTransformationDef  = pMetaDataDef->getTransformationModel();
+    tls00_TransformationCmd * pTransformationCmd  = pTransformationDef->getTransformationCmd();
 
-    tsp00_Longint    rc         = errOK_els00;
     tls00_DatUpdRec *pDatUpdRec = NULL;
 
     // interpret <dataupdate statement>::=
     // DATAUPDATE TABLE <table_spec> <acc_column_spec_mlt> <set_column_spec_mlt> <infile_spec> <longfile_spec_mlt>;
 
-    rc = ls29_InitDatUpdRec(pDBInfo,pDatUpdCmd,pDatUpdRec,errText);
+    rc = ls29_InitDatUpdRec(pDBInfo,pTransformationCmd,pDatUpdRec,errText);
     if (errOK_els00 == rc)
     {
-        rc = ls29_InitDatUpdCmd(pDBInfo,pDatUpdCmd,pDatUpdRec,errText);
+        rc = ls29_InitDatUpdCmd(pDBInfo,pTransformationCmd,pDatUpdRec,errText);
         if (errOK_els00 == rc)
         {
-            rc = ls29_ExecDatUpdCmd(pDBInfo,pDatUpdCmd,pDatUpdRec,errText);
+            rc = ls29_ExecDatUpdCmd(pDBInfo,pTransformationCmd,pDatUpdRec,errText);
         }
     }
 
-    if ( false == pDatUpdCmd->dupInternal_ls00 )
+    if ( false == pTransformationCmd->trInternal_ls00 )
          ls29_FreeDBInfo(pDBInfo,pDatUpdRec);
 
-    ls29_FreeDatUpdCmd(pDatUpdCmd);
+    ls29_FreeDatUpdCmd(pTransformationCmd);
     ls29_FreeDatUpdRec(pDatUpdRec,errText);
 
     return STATIC_CAST(tsp00_Int4, rc);
 }
-//ls29ExecDataUpdate()
+//ls29ExecColumnUpdate()
 
 
 /*-----------------------------------------------------------------------------
     ls29_ExecDatUpdCmd
 /-----------------------------------------------------------------------------*/
 static tsp00_Int4
-ls29_ExecDatUpdCmd(tls00_DBInfo     *pDBInfo,
-                   tls00_DatUpdCmd  *pDatUpdCmd,
-                   tls00_DatUpdRec *&pDatUpdRec,
-                   tsp00_Addr        errText)
+ls29_ExecDatUpdCmd(tls00_DBInfo             *pDBInfo,
+                   tls00_TransformationCmd  *pTransformationCmd,
+                   tls00_DatUpdRec          *&pDatUpdRec,
+                   tsp00_Addr               errText)
 {
     ROUTINE_DBG_MEO00 ("ls29_ExecuteDatUpdCmd");
 
-    RepServMessages *pMsgInstance = RepServMessages::Instance();
+    LoaderMessages *pMsgInstance = LoaderMessages::Instance();
     tsp00_Longint    rc           = errOK_els00;
 
     // msgtype : execute
@@ -275,7 +271,7 @@ ls29_ExecDatUpdCmd(tls00_DBInfo     *pDBInfo,
         // message Type: execute
         if (errOK_els00 == rc)
         {
-            rc = ls29_GetNewDatPartBuf(pDBInfo, pDatUpdCmd, pDatUpdRec, iRecCount, bPutVal, errText);
+            rc = ls29_GetNewDatPartBuf(pDBInfo, pTransformationCmd, pDatUpdRec, iRecCount, bPutVal, errText);
             if (errOK_els00 == rc)
             {
                 // message Type: putval
@@ -286,7 +282,7 @@ ls29_ExecDatUpdCmd(tls00_DBInfo     *pDBInfo,
                                         pDBInfo,
                                         pDatUpdRec->duprLongExecInfo_ls00,
                                         pDatUpdRec->duprpVecLongDescriptor_ls00,
-                                        pDatUpdCmd->dupColSpec_ls00,
+                                        pTransformationCmd->trColSpec_ls00,
                                         pDatUpdRec->duprPartAttr_ls00,
                                         pDatUpdRec->duprTableDescription_ls00,
                                         pDatUpdRec->duprDataSource_ls00.pdasCodePage_ls00,
@@ -330,7 +326,7 @@ ls29_ExecDatUpdCmd(tls00_DBInfo     *pDBInfo,
             // request next or last SQLPacket
             pSqlSndPkt = pDBInfo->dbiPktSndList_ls00[0];
             memset(pSqlSndPkt, '\0', pDBInfo->dbiPktSize_ls00);
-            memcpy((char*)&pSqlSndPkt->sp1_header, pDatUpdRec->duprpPktDatPartHeader.strAddr_ls00, pDatUpdRec->duprpPktDatPartHeader.strLeng_ls00);
+            SAPDB_memcpy((char*)&pSqlSndPkt->sp1_header, pDatUpdRec->duprpPktDatPartHeader.strAddr_ls00, pDatUpdRec->duprpPktDatPartHeader.strLeng_ls00);
 
             // partattr =  nextpacket or lastpacket
             ls04SetPartAttribute(pDataPart, pDatUpdRec->duprPartAttr_ls00);
@@ -356,16 +352,16 @@ ls29_ExecDatUpdCmd(tls00_DBInfo     *pDBInfo,
    ls29_GetNewDatPartBuf
 /------------------------------------------------------------------------------*/
 tsp00_Longint
-ls29_GetNewDatPartBuf(tls00_DBInfo     *pDBInfo   ,
-                      tls00_DatUpdCmd  *pDatUpdCmd ,
-                      tls00_DatUpdRec  *pDatUpdRec,
-                      tsp00_Int4       &iRecCount ,
-                      bool             &bPutVal,
-                      tsp00_Addr        errText )
+ls29_GetNewDatPartBuf(tls00_DBInfo             *pDBInfo   ,
+                      tls00_TransformationCmd  *pTransformationCmd ,
+                      tls00_DatUpdRec          *pDatUpdRec,
+                      tsp00_Int4               &iRecCount ,
+                      bool                     &bPutVal,
+                      tsp00_Addr               errText )
 {
     ROUTINE_DBG_MEO00 ("ls29_GetNewDatPartBuf");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance();
+    LoaderMessages *pMsgObj = LoaderMessages::Instance();
     
     tsp00_Longint    rc     = errOK_els00;
     tsp00_Int4       rcREAD = errOK_els00;
@@ -375,7 +371,7 @@ ls29_GetNewDatPartBuf(tls00_DBInfo     *pDBInfo   ,
     SAPDB_Bool      bOfNoInterest  = SAPDB_TRUE;
 
     tls00_String    DataRecord;
-    DataRecord.strAddr_ls00               = STATIC_CAST(char*, pDatUpdRec->duprDataSource_ls00.dasInputBuffer.pBufPtr_ls00);
+    DataRecord.strAddr_ls00                 = STATIC_CAST(char*, pDatUpdRec->duprDataSource_ls00.dasInputBuffer.pBufPtr_ls00);
 
     tls00_String           *pRecFields      = pDatUpdRec->duprDataSource_ls00.dasConvertedData;
 
@@ -394,7 +390,7 @@ ls29_GetNewDatPartBuf(tls00_DBInfo     *pDBInfo   ,
     {
         // get record from datasource
         rcREAD = ls20ReadData(pDatUpdRec->duprpvFile,
-                              &pDatUpdCmd->dupInfile_ls00.fsFormat_ls00,
+                              &pTransformationCmd->trDataFile_ls00.fsFormat_ls00,
                               &pDatUpdRec->duprPattern_ls00,
                               NULL,
                               &pDatUpdRec->duprDataSource_ls00.dasInputBuffer,
@@ -415,12 +411,12 @@ ls29_GetNewDatPartBuf(tls00_DBInfo     *pDBInfo   ,
             pDatUpdRec->duprPartAttr_ls00.becomes(sp1pa_last_packet);  // Last data in any case
             break;
         }
-        if ( NULL != pDatUpdCmd->dupTable_ls00.tsCondSpec_ls00 )
+        if ( NULL != pTransformationCmd->trTable_ls00.tsCondSpec_ls00 )
         {
             DataRecord.strLeng_ls00 = pDatUpdRec->duprDataSource_ls00.dasInputBuffer.lBufPos_ls00;
-            if ( false == ls16CheckCondition(*pDatUpdCmd->dupTable_ls00.tsCondSpec_ls00,
+            if ( false == ls16CheckCondition(*pTransformationCmd->trTable_ls00.tsCondSpec_ls00,
                                              DataRecord,
-                                             pDatUpdCmd->dupInfile_ls00.fsFormat_ls00) )
+                                             pTransformationCmd->trDataFile_ls00.fsFormat_ls00) )
             {
                 --iRecord;
                 continue;
@@ -429,7 +425,7 @@ ls29_GetNewDatPartBuf(tls00_DBInfo     *pDBInfo   ,
         ++pDatUpdRec->duprRecordCount_ls00;
 
         // get fields from record
-        rc = ls29_GetRecFields(pDatUpdCmd, pDatUpdRec, pDBInfo->dbiSession_ls00.siSQLMode_ls00, errText);
+        rc = ls29_GetRecFields(pTransformationCmd, pDatUpdRec, pDBInfo->dbiSession_ls00.siSQLMode_ls00, errText);
         if ( errOK_els00 == rc )
         {
             pPartHandling->dphRecordNumbers_ls00[iRecord] = pDatUpdRec->duprRecordCount_ls00;
@@ -444,9 +440,9 @@ ls29_GetNewDatPartBuf(tls00_DBInfo     *pDBInfo   ,
             rc = ls20BuildRecord(&pDatUpdRec->duprDataSource_ls00,
                                  pDataPartBuffer,
                                  &pDatUpdRec->duprTableDescription_ls00,
-                                 &pDatUpdCmd->dupColSpec_ls00,
+                                 &pTransformationCmd->trColSpec_ls00,
                                  &pDatUpdRec->duprLongExecInfo_ls00,
-                                 ls00IsDBUnicode(pDBInfo),
+                                 ls00UseUnicodeOI(pDBInfo),
                                  pDatUpdRec->duprRecordCount_ls00,
                                  errText);
             if ( errOK_els00 == rc )
@@ -464,11 +460,11 @@ ls29_GetNewDatPartBuf(tls00_DBInfo     *pDBInfo   ,
                     // which we have to import the LONG
                     rc = ls08InsertLongs(pDatUpdRec->duprDataPartHandling_ls00,
                                          pDatUpdRec->duprLongExecInfo_ls00,
-                                         pDatUpdCmd->dupColSpec_ls00,
+                                         pTransformationCmd->trColSpec_ls00,
                                          pDatUpdRec->duprTableDescription_ls00,
                                          pDatUpdRec->duprDataSource_ls00,
                                          bPutVal,
-                                         ls00IsDBUnicode(pDBInfo),
+                                         ls00UseUnicodeOI(pDBInfo),
                                          errText);
                     if ( rc == errOK_els00 )
                     {
@@ -574,11 +570,11 @@ ls29_GetNewDatPartBuf(tls00_DBInfo     *pDBInfo   ,
                 // request next or last SQLPacket
                 pSqlSndPkt = pDBInfo->dbiPktSndList_ls00[0];
                 memset(pSqlSndPkt, '\0', pDBInfo->dbiPktSize_ls00);
-                memcpy((char*) &pSqlSndPkt->sp1_header, pDatUpdRec->duprpPktDatPartHeader.strAddr_ls00, pDatUpdRec->duprpPktDatPartHeader.strLeng_ls00);
+                SAPDB_memcpy((char*) &pSqlSndPkt->sp1_header, pDatUpdRec->duprpPktDatPartHeader.strAddr_ls00, pDatUpdRec->duprpPktDatPartHeader.strLeng_ls00);
                 // partattr =  nextpacket or lastpacket
                 ls04SetPartAttribute(pDataPart, pDatUpdRec->duprPartAttr_ls00);
 
-                rc = ls29_GetNewDatPartBuf(pDBInfo, pDatUpdCmd , pDatUpdRec, iRecCount, bPutVal, errText);
+                rc = ls29_GetNewDatPartBuf(pDBInfo, pTransformationCmd , pDatUpdRec, iRecCount, bPutVal, errText);
             }
         }   // end if ( rc != errOK_els00 )
     }   // end if (errOK_els00 == rc)
@@ -600,7 +596,7 @@ ls29_LoggingDatPartBuf(tsp1_packet     *pSqlRcvPkt,
 {
     ROUTINE_DBG_MEO00 ("ls29_LoggingDatPartBuf");
 
-    RepServMessages *pMsgInstance = RepServMessages::Instance();
+    LoaderMessages *pMsgInstance = LoaderMessages::Instance();
     tsp00_Int4       rc           = errOK_els00;
 
     tls00_DataPartHandling *pHandling = &pDatUpdRec->duprDataPartHandling_ls00;
@@ -665,7 +661,7 @@ static void
 ls29_FreeDBInfo(tls00_DBInfo* pDBInfo, tls00_DatUpdRec* pDatUpdRec)
 {
     ROUTINE_DBG_MEO00 ("ls29_FreeDBInfo");
-    RepServMessages *pMsgInstance = RepServMessages::Instance();
+    LoaderMessages *pMsgInstance = LoaderMessages::Instance();
 
     // Print Statistic
     if ( pDBInfo->dbiSession_ls00.siAutoCommit_ls00 == true )
@@ -682,10 +678,10 @@ ls29_FreeDBInfo(tls00_DBInfo* pDBInfo, tls00_DatUpdRec* pDatUpdRec)
   ls29_InitDatUpdRec
 /-----------------------------------------------------------------------------*/
 static tsp00_Longint
-ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
-                   tls00_DatUpdCmd  *pDataUpdCmd,
-                   tls00_DatUpdRec *&pDatUpdRec,
-                   tsp00_Addr        errText)
+ls29_InitDatUpdRec(tls00_DBInfo             *pDBInfo,
+                   tls00_TransformationCmd  *pTransformationCmd,
+                   tls00_DatUpdRec          *&pDatUpdRec,
+                   tsp00_Addr               errText)
 {
     ROUTINE_DBG_MEO00 ("ls29_InitDataUpdateRec");
 
@@ -706,7 +702,7 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
     
     // Prepares the DataUpdate command as a update mass cmd
     tls00_Column*       pColumn      = NULL;
-    tls00_MultColSpec*  pMultColSpec = &pDataUpdCmd->dupColSpec_ls00;   
+    tls00_MultColSpec*  pMultColSpec = &pTransformationCmd->trColSpec_ls00;   
 
     tsp00_Int4 k                = 0;
     tsp00_Int4 iKeyColCnt       = 0;
@@ -731,7 +727,7 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
             iCmdLen += pColumn->colAssign_ls00.fasAsgnLen_ls00;
         }
     }
-    iWhereColCnt = pDataUpdCmd->dupSetColIndex_ls00 - iKeyColCnt;
+    iWhereColCnt = pTransformationCmd->trSetColIndex_ls00 - iKeyColCnt;
 
 
     iCmdLen += (pMultColSpec->mcsCount_ls00 + 24)*sizeof(tsp00_KnlIdentifierc);
@@ -744,21 +740,21 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
     //*
     //*  There must be at least one SET column!
     //*
-    if (0 != pDataUpdCmd->dupTable_ls00.tsTabOwnr_ls00.length())
+    if (0 != pTransformationCmd->trTable_ls00.tsTabOwnr_ls00.length())
     {
-        k = sp77sprintf(pszCmd, iCmdLen, "UPDATE \"%s\".\"%s\" SET", pDataUpdCmd->dupTable_ls00.tsTabOwnr_ls00.asCharp(),
-                                                                     pDataUpdCmd->dupTable_ls00.tsTabName_ls00.asCharp());
+        k = sp77sprintf(pszCmd, iCmdLen, "UPDATE \"%s\".\"%s\" SET", pTransformationCmd->trTable_ls00.tsTabOwnr_ls00.asCharp(),
+                                                                     pTransformationCmd->trTable_ls00.tsTabName_ls00.asCharp());
     }
     else
     {
-        k = sp77sprintf(pszCmd, iCmdLen, "UPDATE \"%s\" SET", pDataUpdCmd->dupTable_ls00.tsTabName_ls00.asCharp());
+        k = sp77sprintf(pszCmd, iCmdLen, "UPDATE \"%s\" SET", pTransformationCmd->trTable_ls00.tsTabName_ls00.asCharp());
     }
 
     tsp00_Uint4 _lValueStartPos = k;
 
 
     //Set columns start behind access/key columns in structure MultColSpec
-    for (i = pDataUpdCmd->dupSetColIndex_ls00; i < pMultColSpec->mcsCount_ls00; ++i)
+    for (i = pTransformationCmd->trSetColIndex_ls00; i < pMultColSpec->mcsCount_ls00; ++i)
     {
         k += sp77sprintf(pszCmd+k, iCmdLen - k, " \"%s\" = ?,", pMultColSpec->mcsColumn_ls00[i]->colName_ls00.asCharp() );
     }
@@ -770,7 +766,7 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
     {
         k += sp77sprintf(pszCmd+k, iCmdLen - k, " KEY "); 
 
-        for (i = 0; i < pDataUpdCmd->dupSetColIndex_ls00; ++i)
+        for (i = 0; i < pTransformationCmd->trSetColIndex_ls00; ++i)
         {
             if ( true == pMultColSpec->mcsColumn_ls00[i]->colIsKey_ls00 )
             {
@@ -786,10 +782,11 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
     {
         k += sp77sprintf(pszCmd+k, iCmdLen - k, " WHERE "); 
 
-        for (i = 0; i < pDataUpdCmd->dupSetColIndex_ls00; ++i)
+        for (i = 0; i < pTransformationCmd->trSetColIndex_ls00; ++i)
         {
-            if ( ( false == pMultColSpec->mcsColumn_ls00[i]->colIsKey_ls00 ) &&
-                 ( false == pMultColSpec->mcsColumn_ls00[i]->colIsSet_ls00) )
+             if ( ( false == pMultColSpec->mcsColumn_ls00[i]->colIsKey_ls00 ) &&
+                  ( false == pMultColSpec->mcsColumn_ls00[i]->colIsSet_ls00) )
+
             {
                  k += sp77sprintf(pszCmd+k, iCmdLen - k, " \"%s\" = ? AND",
                                                          pMultColSpec->mcsColumn_ls00[i]->colName_ls00.asCharp());
@@ -810,7 +807,7 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
 
     if (errOK_els00 == rc)
     {
-        _lTmpSetColIndex = pDataUpdCmd->dupSetColIndex_ls00;    // In case of assignments this value will be decreased
+        _lTmpSetColIndex = pTransformationCmd->trSetColIndex_ls00;    // In case of assignments this value will be decreased
                                                                 // by 1 for every assignment in qualification columns
                                                                 // for later use in ls20InitDataSource
         if (true == _bAssignment)
@@ -829,7 +826,7 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
 
             //Set columns start behind access/key columns in structure MultColSpec
             tls00_FieldAsgn *pAssignment = NULL;
-            for (i = pDataUpdCmd->dupSetColIndex_ls00; i < pMultColSpec->mcsCount_ls00; ++i)
+            for (i = pTransformationCmd->trSetColIndex_ls00; i < pMultColSpec->mcsCount_ls00; ++i)
             {
                 pColumn     = pMultColSpec->mcsColumn_ls00[i];
                 pAssignment = &pColumn->colAssign_ls00;
@@ -843,7 +840,7 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
                     {
                         case asLiteral_els00:
                         {
-                            if (1 == szMap_NumberDataType_ls00[pDatUpdRec->duprTableDescription_ls00.tdParamArray_ls00[i-pDataUpdCmd->dupSetColIndex_ls00]->piBasicInfo_ls00.fbDataType_ls00])
+                            if (1 == szMap_NumberDataType_ls00[pDatUpdRec->duprTableDescription_ls00.tdParamArray_ls00[i-pTransformationCmd->trSetColIndex_ls00]->piBasicInfo_ls00.fbDataType_ls00])
                             {
                                 k += sp77sprintf(pszCmd+k, iCmdLen-k, "%s,", pAssignment->fasAsgnVal_ls00);
                             }
@@ -860,14 +857,14 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
                         }
                     }   // end switch (pAssignment->fasAsgnTyp_ls00)
                 }   // end if (asUnknown_els00 != pAssignment->fasAsgnTyp_ls00)
-            }   // end for (i = pDataUpdCmd->dupSetColIndex_ls00; i < pMultColSpec->mcsCount_ls00; ++i)
+            }   // end for (i = pTransformationCmd->trSetColIndex_ls00; i < pMultColSpec->mcsCount_ls00; ++i)
             --k;        // Remove last comma
 
             // [ KEY <key_spec>,...]
             if ( 0 != iKeyColCnt )
             {
                 k += sp77sprintf(pszCmd+k, iCmdLen - k, " KEY "); 
-                for (i = 0; i < pDataUpdCmd->dupSetColIndex_ls00; ++i)
+                for (i = 0; i < pTransformationCmd->trSetColIndex_ls00; ++i)
                 {
                     if ( true == pMultColSpec->mcsColumn_ls00[i]->colIsKey_ls00 )
                     {
@@ -883,7 +880,7 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
                 tsp00_Uint4 _lSetColCnt = pMultColSpec->mcsCount_ls00 - iWhereColCnt;
 
                 k += sp77sprintf(pszCmd+k, iCmdLen - k, " WHERE "); 
-                for (i = 0; i < pDataUpdCmd->dupSetColIndex_ls00; ++i)
+                for (i = 0; i < pTransformationCmd->trSetColIndex_ls00; ++i)
                 {
                     pColumn     = pMultColSpec->mcsColumn_ls00[i];
                     pAssignment = &pColumn->colAssign_ls00;
@@ -939,7 +936,7 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
             {
                 if (NULL != pDatUpdRec->duprTableDescription_ls00.tdParamArray_ls00[j])
                 {
-                    delete [] pDatUpdRec->duprTableDescription_ls00.tdParamArray_ls00[j];
+                    delete pDatUpdRec->duprTableDescription_ls00.tdParamArray_ls00[j];
                     pDatUpdRec->duprTableDescription_ls00.tdParamArray_ls00[j] = NULL;
                 }
             }
@@ -958,36 +955,17 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
     {
         tls00_TableDescription *pTD = &pDatUpdRec->duprTableDescription_ls00;
         pTD->tdTablename_ls00.Init();
-        pTD->tdTablename_ls00.rawAssign(pDataUpdCmd->dupTable_ls00.tsTabName_ls00.asCharp());
+        pTD->tdTablename_ls00.rawAssign(pTransformationCmd->trTable_ls00.tsTabName_ls00.asCharp());
 
-        // Init dataSource
         pDatUpdRec->duprpvFile       = new tls00_VFile;
-        tls00_FileSpec  *pInFileSpec = &pDataUpdCmd->dupInfile_ls00;
-
-        pDatUpdRec->duprDataSource_ls00.dasInputBuffer.lBufLen_ls00 = 6*pDBInfo->dbiPageSize_ls00;
-        ls20InitDataSourceStruct(&pDatUpdRec->duprDataSource_ls00,
-                                 &pInFileSpec->fsFormat_ls00,
-                                 pTD->tdFieldCount_ls00,
-                                 pMultColSpec,
-                                 _lTmpSetColIndex);
-    
-        // Fill pattern structure only if the file format is COMPRESSED
-        if ( (errOK_els00 == rc) && (true == pDataUpdCmd->dupInfile_ls00.fsFormat_ls00.ffoCompress_ls00) )
-        {
-            rc = ls18ConstructPattern(&pDatUpdRec->duprPattern_ls00,
-                                      pInFileSpec->fsFormat_ls00.ffoSeperat_ls00,
-                                      pInFileSpec->fsFormat_ls00.ffoDelimit_ls00,
-                                      pInFileSpec->fsFormat_ls00.ffoCodeType_ls00,
-                                      errText);
-        }
+        tls00_FileSpec  *pInFileSpec = &pTransformationCmd->trDataFile_ls00;
 
         pDatUpdRec->duprpvFile->vfFileType_ls00 = pInFileSpec->fsDeviceType_ls00;
         if ( dtypeTape_ls00 == pDatUpdRec->duprpvFile->vfFileType_ls00 )
         {
             pDatUpdRec->duprpvFile->vfBackupTool_ls00 = pDBInfo->dbiSession_ls00.siBackupTool_ls00;
-            pDatUpdRec->duprpvFile->vfBackupId_ls00.ConvertFromASCII_Latin1(pInFileSpec->fsBackupId_ls00.bBackupIdName_ls00,
-                                                                            pInFileSpec->fsBackupId_ls00.bBackupIdName_ls00 +
-                                                                             _tcslen(pInFileSpec->fsBackupId_ls00.bBackupIdName_ls00));
+            pDatUpdRec->duprpvFile->vfBackupId_ls00.Assign(REINTERPRET_CAST(Tools_DynamicUTF8String::ConstPointer,
+                                                                            pInFileSpec->fsBackupId_ls00.bBackupIdName_ls00));
         }
 
 
@@ -1039,7 +1017,7 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
                                pDataBuffer);
         if (errOK_els00 != rc)
         {
-            RepServMessages::Instance()->ls98Msg(layDataUpdate_els98, errText, STATIC_CAST(ErrorNumbers, rc));
+            LoaderMessages::Instance()->ls98Msg(layDataUpdate_els98, errText, STATIC_CAST(ErrorNumbers, rc));
         }
         else
         {
@@ -1050,7 +1028,6 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
                                  sp5vf_text,
                                  sp5vf_read,
                                  pInFileSpec->fsFormat_ls00.ffoCodeType_ls00);
-
             }
             else
             {
@@ -1058,7 +1035,31 @@ ls29_InitDatUpdRec(tls00_DBInfo     *pDBInfo,
             }
 
             rc = ls30VFOpen(*pDatUpdRec->duprpvFile, errText);
+
+            if (ctUnknown_els00 == pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00)
+            {
+                pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00 = pDatUpdRec->duprpvFile->vfFileEncoding_ls00;
+            }
         }
+
+
+        // Fill pattern structure only if the file format is COMPRESSED
+        if ( (errOK_els00 == rc) && (true == pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCompress_ls00) )
+        {
+            rc = ls18ConstructPattern(&pDatUpdRec->duprPattern_ls00,
+                                      pInFileSpec->fsFormat_ls00.ffoSeperat_ls00,
+                                      pInFileSpec->fsFormat_ls00.ffoDelimit_ls00,
+                                      pInFileSpec->fsFormat_ls00.ffoCodeType_ls00,
+                                      errText);
+        }
+
+        // Init dataSource
+        pDatUpdRec->duprDataSource_ls00.dasInputBuffer.lBufLen_ls00 = 6*pDBInfo->dbiPageSize_ls00;
+        ls20InitDataSourceStruct(&pDatUpdRec->duprDataSource_ls00,
+                                 &pInFileSpec->fsFormat_ls00,
+                                 pTD->tdFieldCount_ls00,
+                                 pMultColSpec,
+                                 _lTmpSetColIndex);
     }
 
     return rc;
@@ -1081,7 +1082,7 @@ ls29_FreeDatUpdRec(tls00_DatUpdRec *&pDatUpdRec, tsp00_Addr errText)
     {
         if (pTD->tdParamArray_ls00[iColumn] != NULL )
         {
-            delete [] pTD->tdParamArray_ls00[iColumn];
+            delete pTD->tdParamArray_ls00[iColumn];
             pTD->tdParamArray_ls00[iColumn] = NULL;
         }
     }
@@ -1164,23 +1165,23 @@ ls29_FreeDatUpdRec(tls00_DatUpdRec *&pDatUpdRec, tsp00_Addr errText)
     ls29_InitDatUpdCmd
 /-----------------------------------------------------------------------------*/
 static tsp00_Int4
-ls29_InitDatUpdCmd(tls00_DBInfo    *pDBInfo,
-                   tls00_DatUpdCmd *pDatUpdCmd,
-                   tls00_DatUpdRec *pDatUpdRec,
-                   tsp00_Addr       errText)
+ls29_InitDatUpdCmd(tls00_DBInfo            *pDBInfo,
+                   tls00_TransformationCmd *pTransformationCmd,
+                   tls00_DatUpdRec         *pDatUpdRec,
+                   tsp00_Addr              errText)
 {
     ROUTINE_DBG_MEO00 ("ls29_InitDatUpdCmd");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance();
+    LoaderMessages *pMsgObj = LoaderMessages::Instance();
     tsp00_Int4       rc      = errOK_els00;
 
     //  1. Init tls00_TableSpec
-    tls00_FileFormat *pInFileFormat = &pDatUpdCmd->dupInfile_ls00.fsFormat_ls00;
+    tls00_FileFormat *pInFileFormat = &pTransformationCmd->trDataFile_ls00.fsFormat_ls00;
 
     // <table_spec> ::= <table_name> <if_condition>
-    if ( pDatUpdCmd->dupTable_ls00.tsCondSpec_ls00 )
+    if ( pTransformationCmd->trTable_ls00.tsCondSpec_ls00 )
     {
-        rc = ls16ConvertCondition(*pDatUpdCmd->dupTable_ls00.tsCondSpec_ls00,
+        rc = ls16ConvertCondition(*pTransformationCmd->trTable_ls00.tsCondSpec_ls00,
                                   pDBInfo->dbiOwnSwap_ls00,
                                   *pInFileFormat,
                                   errText );
@@ -1190,7 +1191,7 @@ ls29_InitDatUpdCmd(tls00_DBInfo    *pDBInfo,
 
     // 2. Init tls00_MultColSpec
     tls00_TableDescription *pTD          = &pDatUpdRec->duprTableDescription_ls00;
-    tls00_MultColSpec      *pMultColSpec = &pDatUpdCmd->dupColSpec_ls00;
+    tls00_MultColSpec      *pMultColSpec = &pTransformationCmd->trColSpec_ls00;
     tls00_Column           *pColumn      = NULL;
     tls00_FieldBasics      *pFieldBasic  = NULL;
 
@@ -1246,14 +1247,14 @@ ls29_InitDatUpdCmd(tls00_DBInfo    *pDBInfo,
     //<longfile_spec_mlt> ::= [<longfile_spec>][<longfile_spec_mlt>]
     if (errOK_els00 == rc)
     {
-       ls08InitLongFileInfo(pMultColSpec, &pDatUpdCmd->dupLongSpec_ls00);
+       ls08InitLongFileInfo(pMultColSpec, &pTransformationCmd->trColLongSpec_ls00);
     }
 
 
     // <infile_spec>      ::= INFILE <valFILE_NAME> <file_format>
     if (errOK_els00 == rc)
     {
-        tsp00_Int4 iBytesPerLine = ls18GetRowLength(pDatUpdCmd->dupColSpec_ls00,
+        tsp00_Int4 iBytesPerLine = ls18GetRowLength(pTransformationCmd->trColSpec_ls00,
                                                     *pTD,
                                                     //pInFileFormat->ffoBinary_ls00);
                                                     pInFileFormat->ffoFormatted_ls00);
@@ -1264,7 +1265,7 @@ ls29_InitDatUpdCmd(tls00_DBInfo    *pDBInfo,
         else
         {
             tls00_VFile vFile;
-            ls30VFInitialize(vFile, pDatUpdCmd->dupInfile_ls00.fsFileName_ls00);
+            ls30VFInitialize(vFile, pTransformationCmd->trDataFile_ls00.fsFileName_ls00);
 
             rc = ls30VFOpen(vFile,errText);
             if ( (errOK_els00 == rc) && (0 == pInFileFormat->ffoBytesPerLine_ls00) )
@@ -1294,11 +1295,11 @@ ls29_InitDatUpdCmd(tls00_DBInfo    *pDBInfo,
   ls29_FreeDatUpdCmd
 /-----------------------------------------------------------------------------*/
 void
-ls29_FreeDatUpdCmd(tls00_DatUpdCmd * pDatUpdCmd)
+ls29_FreeDatUpdCmd(tls00_TransformationCmd * pTransformationCmd)
 {
     ROUTINE_DBG_MEO00 ("ls29_FreeDatUpdateCmd");
 
-    tls00_MultColSpec *pMultColSpec = &pDatUpdCmd->dupColSpec_ls00;
+    tls00_MultColSpec *pMultColSpec = &pTransformationCmd->trColSpec_ls00;
 
     if (NULL != pMultColSpec)
     {
@@ -1327,7 +1328,7 @@ ls29_FreeDatUpdCmd(tls00_DatUpdCmd * pDatUpdCmd)
         }
     }
     // <infile_spec>
-//    tls00_FileSpec    InFileSpec   = pDatUpdCmd->dupInfile_ls00;
+//    tls00_FileSpec    InFileSpec   = pTransformationCmd->trDataFile_ls00;
 //    tls00_FileFormat  InFileFormat = InFileSpec.fsFormat_ls00;
 }
 // ls29_FreeDatUpdCmd()
@@ -1337,19 +1338,19 @@ ls29_FreeDatUpdCmd(tls00_DatUpdCmd * pDatUpdCmd)
    ls29_GetRecFields
 /------------------------------------------------------------------------------*/
 static tsp00_Longint
-ls29_GetRecFields(tls00_DatUpdCmd   *pDatUpdCmd,
-                  tls00_DatUpdRec   *pDatUpdRec,
-                  tsp1_sqlmode_Enum  SQLMode,
-                  tsp00_Addr         errText)
+ls29_GetRecFields(tls00_TransformationCmd   *pTransformationCmd,
+                  tls00_DatUpdRec           *pDatUpdRec,
+                  tsp1_sqlmode_Enum         SQLMode,
+                  tsp00_Addr                errText)
 {
     ROUTINE_DBG_MEO00 ("ls29_GetRecFields");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance();
+    LoaderMessages *pMsgObj = LoaderMessages::Instance();
     tsp00_Longint    rc = errOK_els00;
 
     tls00_TableDescription *pTD          = &pDatUpdRec->duprTableDescription_ls00;
-    tls00_FileFormat       *pFileFormat  = &pDatUpdCmd->dupInfile_ls00.fsFormat_ls00;
-    tls00_MultColSpec      *pMultColSpec = &pDatUpdCmd->dupColSpec_ls00;
+    tls00_FileFormat       *pFileFormat  = &pTransformationCmd->trDataFile_ls00.fsFormat_ls00;
+    tls00_MultColSpec      *pMultColSpec = &pTransformationCmd->trColSpec_ls00;
     tsp00_Addr              pInputLine   = STATIC_CAST(char*, pDatUpdRec->duprDataSource_ls00.dasInputBuffer.pBufPtr_ls00);
     tsp00_Int4              iInputLen    = pDatUpdRec->duprDataSource_ls00.dasInputBuffer.lBufPos_ls00;
     tls00_String           *pRecFields   = pDatUpdRec->duprDataSource_ls00.dasConvertedData;
@@ -1362,7 +1363,7 @@ ls29_GetRecFields(tls00_DatUpdCmd   *pDatUpdCmd,
     {
         rc = ls18GetFormattedValueEx(pInputLine,
                                      iInputLen,
-                                     &pDatUpdCmd->dupColSpec_ls00,
+                                     &pTransformationCmd->trColSpec_ls00,
                                      pDatUpdRec->duprDataSource_ls00.lReadFieldsCnt_ls00,
                                      pDatUpdRec->duprDataSource_ls00.plDataToColIndex_ls00,
                                      pRecFields,
@@ -1370,14 +1371,14 @@ ls29_GetRecFields(tls00_DatUpdCmd   *pDatUpdCmd,
     }
     else if (true == pFileFormat->ffoCompress_ls00)
     {
-        rc = ls18GetUnformattedValueEx(pInputLine,
-                                       iInputLen,
-                                       &pDatUpdRec->duprPattern_ls00,
-                                       pDatUpdRec->duprDataSource_ls00.lReadFieldsCnt_ls00,
-                                       pDatUpdRec->duprDataSource_ls00.lReadPos_ls00,
-                                       pDatUpdCmd->dupInfile_ls00.fsFormat_ls00.ffoCodeType_ls00,
-                                       pRecFields,
-                                       lFailIndex);
+        rc = ls18GetCSVData(pInputLine,
+                            iInputLen,
+                            &pDatUpdRec->duprPattern_ls00,
+                            pDatUpdRec->duprDataSource_ls00.lReadFieldsCnt_ls00,
+                            pDatUpdRec->duprDataSource_ls00.lReadPos_ls00,
+                            pTransformationCmd->trDataFile_ls00.fsFormat_ls00.ffoCodeType_ls00,
+                            pRecFields,
+                            lFailIndex);
     }
 
     if (errOK_els00 != rc)
@@ -1411,7 +1412,7 @@ ls29_GetRecFields(tls00_DatUpdCmd   *pDatUpdCmd,
                                       pColumn,
                                       &pDatUpdRec->duprDataSource_ls00.dasInputBuffer,
                                       pParamInfo,
-                                      &pDatUpdCmd->dupInfile_ls00,
+                                      &pTransformationCmd->trDataFile_ls00,
                                       pDatUpdRec->duprSwapKind_ls00,
                                       SQLMode,
                                       &pDatUpdRec->duprPattern_ls00,
