@@ -1,26 +1,17 @@
 /*!
   -----------------------------------------------------------------------------
-
   module: vls30.cpp
-
   -----------------------------------------------------------------------------
 
-  responsible:  SteffenS and d033893
+  responsible:  SteffenS, d033893
 
-  special area: SAP DB LOADER
+  special area: Loader
 
   description:  Implementation of functions for Virtual File access
 
   version:      7.5.
-
   -----------------------------------------------------------------------------
 
-  copyright:    (c) 1997-2004 SAP AG-2004
-
-  -----------------------------------------------------------------------------
-
-
-
     ========== licence begin  GPL
     Copyright (c) 1997-2004 SAP AG
 
@@ -38,9 +29,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     ========== licence end
-
-
-
 */
 #include <assert.h>
 #include <time.h>
@@ -54,6 +42,7 @@
 
 #include "hsp100.h"     // versionsbestimmung
 
+
 /* Lokale Funktionen - DEFINITIONEN: ********************************************/
 
 
@@ -141,6 +130,8 @@ void ls30VFInitialize(tls00_VFile&                   NewFileStruct,
 {
     ROUTINE_DBG_MEO00 ("ls30VFInitialize");
 
+    LoaderMessages* pMsgInstance = LoaderMessages::Instance();
+
     NewFileStruct.vfFileHndl_ls00                   = 0;
     NewFileStruct.vfFileName_ls00                   = pszFileName;
     NewFileStruct.vfDataType_ls00.becomes (DataType);
@@ -154,10 +145,15 @@ void ls30VFInitialize(tls00_VFile&                   NewFileStruct,
     NewFileStruct.vfFileLineNum_ls00                = 0;
     NewFileStruct.vfFileEncoding_ls00               = Encoding;
 
+    // Initialize magic byte array
+    NewFileStruct.vfMagicBytesCache_ls00.lBufLen_ls00 = 3;
+    NewFileStruct.vfMagicBytesCache_ls00.lBufPos_ls00 = 0;
+    NewFileStruct.vfMagicBytesCache_ls00.pBufPtr_ls00 = new unsigned char[NewFileStruct.vfMagicBytesCache_ls00.lBufLen_ls00];
+    memset(NewFileStruct.vfMagicBytesCache_ls00.pBufPtr_ls00, 0, NewFileStruct.vfMagicBytesCache_ls00.lBufLen_ls00);
+
     if ( NewFileStruct.vfFileType_ls00   == dtypeTape_ls00 &&
          NewFileStruct.vfBackupTool_ls00 != btTypeUnknown_els00 )
     {
-         tsp01_RteError RteError;
          tcn35d_BackupInfo::ADataDirection  TheDataDirection;
          tcn35d_BackupInfo::ADataType       TheDataType;
          Tools_DynamicUTF8String            TheMediaName;
@@ -180,7 +176,10 @@ void ls30VFInitialize(tls00_VFile&                   NewFileStruct,
          TheDataType             = tcn35d_BackupInfo::DataMigration;
          TheNumberOfPipes        = 1;
 
-         sqlGetIndependentWrkPath(TheDefaultDirectory, TERM_WITH_DELIMITER_EO01, &RteError);
+         _TCHAR   szDefaultDirectory[PATH_MXSP00];
+         szDefaultDirectory[0] = '\0';
+		 pMsgInstance->ls98GetLoaderConfigPath(szDefaultDirectory);
+		 TheDefaultDirectory.rawAssign(szDefaultDirectory);
 
          if ( FileMode == sp5vf_write )
          {
@@ -248,64 +247,89 @@ ls30VFOpen(tls00_VFile &VFile, tsp00_Addr ErrText)
              VFile.vfpBackupController_ls00->BeginBackupOperation();
         else
         {
-             Tools_DynamicUTF8String errmsg = "Backuptool not specified";
-             RepServMessages::Instance()->ls98Msg(layVFile_els98, ErrText, errFileOpen_els98,
-                                                                  VFile.vfFileName_ls00, errmsg.StrPtr());
+             LoaderMessages::Instance()->ls98Msg(layVFile_els98, ErrText, errFileOpen_els98,
+                                                                  VFile.vfFileName_ls00, "Backuptool not specified");
              rc = errFileOpen_els98;
              return rc;
         }
     }
 
-    
+    //*
+    //* check for UTF8
+    //*
+    char* _pszTmpFileName = VFile.vfFileName_ls00;
+    int   i               = 0;
+
+    tsp00_Int4 _len = _tcslen(VFile.vfFileName_ls00);
+    for (i; i < _len; ++i)
+    {
+        if (0 == sp77encodingUTF8->isAscii7(STATIC_CAST(const void*, &VFile.vfFileName_ls00[i])))
+        {
+            break;
+        }
+    }
+    if (i != _len)
+    {
+        char* _pEndPtr = NULL;
+        _pszTmpFileName = new char[_len + sizeof(char)];
+
+        Tools_DynamicUTF8String _filename;
+        _filename.Assign(REINTERPRET_CAST(Tools_DynamicUTF8String::ConstPointer, VFile.vfFileName_ls00));
+        _filename.ConvertToASCII_Latin1(_pszTmpFileName, _pszTmpFileName + _len, _pEndPtr);
+        *_pEndPtr = 0;
+    }
+
     //*
     //*  open stream
     //*
-    if (ctAscii_els00 == VFile.vfFileEncoding_ls00)
+    if ( (sp5vf_binary == VFile.vfDataType_ls00) || (ctAscii_els00 == VFile.vfFileEncoding_ls00) )
     {
-        sqlfopenc(VFile.vfFileName_ls00, VFile.vfDataType_ls00, VFile.vfFileMode_ls00,
+//        sqlfopenc(VFile.vfFileName_ls00, VFile.vfDataType_ls00, VFile.vfFileMode_ls00,
+        sqlfopenc(_pszTmpFileName, VFile.vfDataType_ls00, VFile.vfFileMode_ls00,
                   VFile.vfFileBuf_ls00, &VFile.vfFileHndl_ls00, &RteError);
     }
     else
     {
         // UNICODE files are always treated as text files; opening a file binary will
         // always result in opening it not encoded
+        const tsp77encoding* pEncoding = pEncodings_ls00[VFile.vfFileEncoding_ls00];
 
-        const tsp77encoding *encoding;
-        switch (VFile.vfFileEncoding_ls00)
-        {
-            case ctUTF8_els00:
-            {
-                encoding = sp77encodingUTF8;
-                break;
-            }
-            case ctUCS2_els00:
-            {
-                encoding = sp77encodingUCS2;
-                break;
-            }
-            case ctUCS2Swapped_els00:
-            {
-                encoding = sp77encodingUCS2Swapped;
-                break;
-            }
-            case ctUCS2Native_els00:
-            {
-                encoding = sp77encodingUCS2Native;
-                break;
-            }
-            default:
-            {
-                encoding = sp77encodingAscii;
-            }
-        }   // end switch (VFile.vfFileEncoding_ls00)
-
-        sqlfopenEncodedc(VFile.vfFileName_ls00,
-                         encoding,
-                         encoding,
+//        sqlfopenEncodedc(VFile.vfFileName_ls00,
+        sqlfopenEncodedc(_pszTmpFileName,
+                         &pEncoding,
+                         pEncoding,
                          VFile.vfFileMode_ls00,
                          VFile.vfFileBuf_ls00,
                          &VFile.vfFileHndl_ls00,
                          &RteError);
+        if ( (vf_ok == RteError.sp5fe_result) && (ctUnknown_els00 == VFile.vfFileEncoding_ls00) )
+        {
+            // Set the encoding of the file
+            switch (pEncoding->EncodingType)
+            {
+                case csp_unicode:
+                {
+                    VFile.vfFileEncoding_ls00 = ctUCS2_els00;
+                    break;
+                }
+                case csp_unicode_swap:
+                {
+                    VFile.vfFileEncoding_ls00 = ctUCS2Swapped_els00;
+                    break;
+                }
+                case csp_utf8:
+                {
+                    VFile.vfFileEncoding_ls00 = ctUTF8_els00;
+                    break;
+                }
+                default:
+                {
+                    VFile.vfFileEncoding_ls00 = ctAscii_els00;
+                }
+            }
+        }   // end if ( (vf_ok == RteError.sp5fe_result) && ...
+        
+        // In case there is a real error it is handled in the code snippet that follows
     }
 
     if (vf_ok == RteError.sp5fe_result)             
@@ -313,7 +337,8 @@ ls30VFOpen(tls00_VFile &VFile, tsp00_Addr ErrText)
         VFile.vfFilePos_ls00.lnu2GB_Blocks_ls00 = 0;
         VFile.vfFilePos_ls00.lnuRemainder_ls00  = 0;
 
-        sqlfinfoc (VFile.vfFileName_ls00, &RteInfo, &RteError); // Get some file information
+//        sqlfinfoc (VFile.vfFileName_ls00, &RteInfo, &RteError); // Get some file information
+        sqlfinfoc(_pszTmpFileName, &RteInfo, &RteError); // Get some file information
         if ( ( vf_ok == RteError.sp5fe_result ) && (sp5vf_write != VFile.vfFileMode_ls00) )
         {
             // in case of WRITE the initial values for file size are valid; so we get
@@ -332,13 +357,18 @@ ls30VFOpen(tls00_VFile &VFile, tsp00_Addr ErrText)
 
         tsp00_C40c pszTmpLZUText;
         pszTmpLZUText.p2c(RteError.sp5fe_text);
-        RepServMessages::Instance()->ls98Msg(layVFile_els98, ErrText, errFileOpen_els98,
+        LoaderMessages::Instance()->ls98Msg(layVFile_els98, ErrText, errFileOpen_els98,
                                                                       VFile.vfFileName_ls00,
                                                                       pszTmpLZUText.asCharp());
         VFile.vfFilePos_ls00.lnuRemainder_ls00 = EOF_LS00;    // Indicates error
         rc = errFileOpen_els98;
     }
 
+    // delete temp file name pointer
+    if (_pszTmpFileName != VFile.vfFileName_ls00)
+    {
+        delete [] _pszTmpFileName;
+    }
     return rc;
 }
 // ls30VFOpen()
@@ -362,7 +392,7 @@ int ls30VFClose(tls00_VFile &VFile, tsp00_Addr ErrText)
         {
             tsp00_C40c pszTmpLZUText;
             pszTmpLZUText.p2c(RteError.sp5fe_text);
-            RepServMessages::Instance()->ls98Msg(layVFile_els98, ErrText,
+            LoaderMessages::Instance()->ls98Msg(layVFile_els98, ErrText,
                                                                  errFileClose_els98,
                                                                  VFile.vfFileName_ls00,
                                                                  pszTmpLZUText.asCharp());
@@ -390,7 +420,7 @@ int ls30VFClose(tls00_VFile &VFile, tsp00_Addr ErrText)
          else
          {
              Tools_DynamicUTF8String errmsg = "Backuptool not specified";
-             RepServMessages::Instance()->ls98Msg(layVFile_els98, ErrText, errFileClose_els98,
+             LoaderMessages::Instance()->ls98Msg(layVFile_els98, ErrText, errFileClose_els98,
                                                   VFile.vfFileName_ls00,errmsg.StrPtr());
              rc = errFileClose_els98;
              return rc;
@@ -417,7 +447,7 @@ int ls30VFReadLn(tls00_VFile   &VFile,
 {
     ROUTINE_DBG_MEO00 ("ls30VFReadLn");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance(); // error reporting obj
+    LoaderMessages *pMsgObj = LoaderMessages::Instance(); // error reporting obj
 
     tsp00_Longint ReadLen   = 0;
     tsp00_Addr    pszBuffer = NULL;
@@ -469,7 +499,7 @@ ls30VFReadLnEx(tls00_VFile   &VFile,
 
     if (lCount > MaxReadLen)
     {
-        RepServMessages::Instance()->ls98Msg(layVFile_els98, pszErrText,
+        LoaderMessages::Instance()->ls98Msg(layVFile_els98, pszErrText,
                                                              errFileRead_els98,
                                                              VFile.vfFileName_ls00,
                                                              "Maximum read length exceeded");
@@ -543,7 +573,7 @@ ls30VFReadLongBin(tsp00_Addr                pszBuffer,
             {
                 tsp00_C40c pszTmpLZUText;
                 pszTmpLZUText.p2c(RteError.sp5fe_text);
-                RepServMessages::Instance()->ls98Msg(layVFile_els98, pszErrText,
+                LoaderMessages::Instance()->ls98Msg(layVFile_els98, pszErrText,
                                                                      errFileRead_els98,
                                                                      VFile.vfFileName_ls00,
                                                                      pszTmpLZUText.asCharp());
@@ -602,7 +632,7 @@ ls30VFWriteLn(tls00_VFile   &VFile,
     {
         tsp00_C40c pszTmpLZUText;
         pszTmpLZUText.p2c(RteError.sp5fe_text);
-        RepServMessages::Instance()->ls98Msg(layVFile_els98, pszErrText,
+        LoaderMessages::Instance()->ls98Msg(layVFile_els98, pszErrText,
                                                              errFileWrite_els98,
                                                              VFile.vfFileName_ls00,
                                                              pszTmpLZUText.asCharp());
@@ -673,6 +703,7 @@ ls30VFWriteHeader(tls00_VFile & VFile,
 }
 // ls30VFWriteHeader()
 
+
 /*
   -----------------------------------------------------------------------------
   function:     ls30VFReadHeader
@@ -766,7 +797,7 @@ int ls30VFSeek(tls00_VFile &VFile, tsp00_Longint FilePos, tsp00_Addr ErrText)
         {
             tsp00_C40c pszTmpLZUText;
             pszTmpLZUText.p2c(RteError.sp5fe_text);
-            RepServMessages::Instance()->ls98Msg(layVFile_els98, ErrText,
+            LoaderMessages::Instance()->ls98Msg(layVFile_els98, ErrText,
                                                                  errFileRead_els98,
                                                                  VFile.vfFileName_ls00,
                                                                  pszTmpLZUText.asCharp());
@@ -851,7 +882,7 @@ ls30VFSeekLong(tls00_VFile             &VFile,
     {
         tsp00_C40c pszTmpLZUText;
         pszTmpLZUText.p2c(RteError.sp5fe_text);
-        RepServMessages::Instance()->ls98Msg(layVFile_els98, pszErrText, errFileRead_els98,
+        LoaderMessages::Instance()->ls98Msg(layVFile_els98, pszErrText, errFileRead_els98,
                                                                          VFile.vfFileName_ls00,
                                                                          pszTmpLZUText.asCharp());
         rc = errFileRead_els98;
@@ -892,7 +923,7 @@ ls30VFSeekLong(tls00_VFile             &VFile,
         {
             tsp00_C40c pszTmpLZUText;
             pszTmpLZUText.p2c(RteError.sp5fe_text);
-            RepServMessages::Instance()->ls98Msg(layVFile_els98, pszErrText, errFileRead_els98,
+            LoaderMessages::Instance()->ls98Msg(layVFile_els98, pszErrText, errFileRead_els98,
                                                                              VFile.vfFileName_ls00,
                                                                              pszTmpLZUText.asCharp());
             rc = errFileRead_els98;
@@ -929,7 +960,7 @@ ls30VFTruncate(tls00_VFile &VFile, tsp00_Addr ErrText, tsp00_Longint lNewSize)
         {
             tsp00_C40c pszTmpLZUText;
             pszTmpLZUText.p2c(RteError.sp5fe_text);
-            RepServMessages::Instance()->ls98MsgToString(layVFile_els98, ErrText,
+            LoaderMessages::Instance()->ls98MsgToString(layVFile_els98, ErrText,
                                                                          errFileTruncate_els98,
                                                                          VFile.vfFileName_ls00,
                                                                          pszTmpLZUText.asCharp());
@@ -1027,6 +1058,127 @@ ls30VFMoveToStartPosition(tls00_VFile              &VFile,
 }
 // ls30VFMoveToStartPosition()
 
+
+/*
+  -----------------------------------------------------------------------------
+  function:     ls30Detect_BinaryFile_Encoding
+  -----------------------------------------------------------------------------
+*/
+tsp00_Int4
+ls30Detect_BinaryFile_Encoding(tls00_VFile& VFile, tsp00_Addr pszErrText)
+{
+    ROUTINE_DBG_MEO00 ("ls30Detect_BinaryFile_Encoding");
+
+    tsp00_Int4 rc     = errOK_els00;
+    tsp00_Int4 rcRead = errOK_els00;
+
+    tls00_Buffer2 *_pMB_Cache = &VFile.vfMagicBytesCache_ls00;
+    tls00_LargeNumber lReadPos;
+    lReadPos.lnu2GB_Blocks_ls00 = 0;
+    lReadPos.lnuRemainder_ls00  = 0;
+
+    tls00_CodeType FileEncoding = ctUnknown_els00;
+
+    rcRead = ls30VFReadLongBin(STATIC_CAST(char*, _pMB_Cache->pBufPtr_ls00),
+                               VFile,
+                               _pMB_Cache->lBufLen_ls00,
+                               &lReadPos,
+                               pszErrText);
+    if (rcRead < 0)
+    {
+        rc = rcRead;
+    }
+    else
+    {
+        int                   i          = ctCodeSet_els00;
+        const tsp77charConst* pIndicator = NULL;
+
+        if (ctUnknown_els00 == VFile.vfFileEncoding_ls00)       // we have to check them all
+        {
+            do
+            {
+                if (NULL != pEncodings_ls00[i])
+                {
+                    pIndicator = &pEncodings_ls00[i]->charTable->indicator;
+                    if (sp77charIsEqual(pIndicator, _pMB_Cache->pBufPtr_ls00, pIndicator->byteCount))
+                    {
+                        VFile.vfFileEncoding_ls00 = STATIC_CAST(tls00_CodeType, i);
+                    }
+                }
+                --i;
+            }
+            while ((ctUnknown_els00 == VFile.vfFileEncoding_ls00) && (i > 0));
+                // At the end the ASCII encoding fits in any case
+        }
+        else
+        {
+            // Wir koennen gezielt ueberpruefen, ob das richtige encoding angegeben ist
+            if (NULL != pEncodings_ls00[i])
+            {
+                pIndicator = &pEncodings_ls00[VFile.vfFileEncoding_ls00]->charTable->indicator;
+                if (SAPDB_FALSE == sp77charIsEqual(pIndicator, _pMB_Cache->pBufPtr_ls00, pIndicator->byteCount))
+                {
+                    LoaderMessages::Instance()->ls98Msg(layDataLoad_els98, pszErrText, errFileWrongEncoding_els98,
+                                                                                        VFile.vfFileName_ls00,
+                                                                                        pszCodeTypeName_ls00[FileEncoding]);
+                    rc = errFileWrongEncoding_els98;
+                }
+            }
+        }
+    }
+
+    if (errOK_els00 == rc)
+    {
+        // In case of ASCII encoding we have to use the read magic bytes as data later when
+        // loading the real data; in all other cases we can ignore the magic bytes in further processing
+        if ( (ctAscii_els00 != VFile.vfFileEncoding_ls00) && (ctIgnore_els00 != VFile.vfFileEncoding_ls00) )
+        {
+            ls30VFEmptyMagicBytesCache(&VFile);
+        }
+// TODOTODO altlasten unterstuetzen? wenn die dateien noch ohne magic bytes geschrieben wurden, ist ein vergleich
+//           hier sinnlos
+    }
+
+    return rc;
+}
+// ls30Detect_BinaryFile_Encoding()
+
+
+/*
+  -----------------------------------------------------------------------------
+  function:     ls30Mark_BinaryFile_Encoding
+  -----------------------------------------------------------------------------
+*/
+tsp00_Int4
+ls30Mark_BinaryFile_Encoding(tls00_VFile& VFile, tsp00_Addr pszErrText)
+{
+    ROUTINE_DBG_MEO00 ("ls30Mark_BinaryFile_Encoding");
+
+    tsp00_Int4 rc     = errOK_els00;
+
+    if ( (ctAscii_els00  != VFile.vfFileEncoding_ls00) &&
+         (ctIgnore_els00 != VFile.vfFileEncoding_ls00) )
+    {
+        if (ctUTF8_els00 == VFile.vfFileEncoding_ls00)
+        {
+            rc = ls30VFWriteLn(VFile, 
+                               CONST_CAST(char*, sp77encodingUTF8->charTable->indicator.bytes),
+                               sp77encodingUTF8->charTable->indicator.byteCount,
+                               pszErrText);
+        }
+        else
+        {
+            rc = ls30VFWriteLn(VFile, 
+                               CONST_CAST(char*, sp77encodingUCS2Native->charTable->indicator.bytes),
+                               sp77encodingUCS2Native->charTable->indicator.byteCount,
+                               pszErrText);
+        }
+    }
+    return rc;
+}
+// ls30Mark_BinaryFile_Encoding()
+
+
 /*
   -----------------------------------------------------------------------------
   Local functions: implementation
@@ -1047,7 +1199,7 @@ ls30_ReadLnTxt(tsp00_Addr     pszBuffer,
 {
     ROUTINE_DBG_MEO00 ("ls30_ReadLnTxt");
 
-    RepServMessages *pMsgObj = RepServMessages::Instance();    // error reporting obj
+    LoaderMessages *pMsgObj = LoaderMessages::Instance();    // error reporting obj
 
     tsp05_RteFileError RteError;
     tsp00_Longint      ReadLen = 0;
@@ -1118,7 +1270,7 @@ ls30_ReadLnBin(tsp00_Addr    &pszBuffer,
         {
             tsp00_C40c pszTmpLZUText;
             pszTmpLZUText.p2c(RteError.sp5fe_text);
-            RepServMessages::Instance()->ls98Msg(layVFile_els98, pszErrorText,
+            LoaderMessages::Instance()->ls98Msg(layVFile_els98, pszErrorText,
                                                                  errFileRead_els98,
                                                                  VFile.vfFileName_ls00,
                                                                  pszTmpLZUText.asCharp());
