@@ -7,15 +7,15 @@
  
   responsible:  SteffenS
 
-  special area: SAPDB Loader
+  special area: Loader
  
   description:  trace
 
-  version:      7.4.
+  version:      7.5.
 
   -----------------------------------------------------------------------------
  
-  copyright:    (c) 1997-2004 SAP AG-2004
+  Copyright (c) 1997-2004 SAP AG-2004
  
   -----------------------------------------------------------------------------
 
@@ -49,6 +49,7 @@
 #include <assert.h>
 
 #include "gls00.h"
+#include "hls98msg.h"
 #include "hls99.h"
 #include "heo01.h"
 
@@ -775,17 +776,17 @@ void tls99_TraceClass::ls99Tab2(const _TCHAR *Layer, tls00_TableDescription *Tab
                            STATIC_CAST(int, Table->tdKernelLoc_ls00),
                            STATIC_CAST(int, Table->tdConstraintCount_ls00));
         ls99_Write();
-        len = 0;
-        m_szTraceText[0] = 0;
-
+        
         tsp00_DataType DBDataType;
 
-        for (int i=0; i < Table->tdFieldCount_ls00; i++)
+        for (int i=0; i < Table->tdFieldCount_ls00; ++i)
         {
-            DBDataType = Table->tdFieldsArray_ls00[i]->fiBasics_ls00.fbDataType_ls00;
+            len              = 0;
+            m_szTraceText[0] = 0;
+            DBDataType       = Table->tdFieldsArray_ls00[i]->fiBasics_ls00.fbDataType_ls00;
 
             len += sp77sprintf(m_szTraceText + len, BUF_SIZE_LS99 - len,
-                                                    "%i\t%i\t%i\t%i\t%i\t%i\t",
+                                                    "%i\t%i\t\t%i\t\t%i\t\t%i\t%i\t\t",
                                                     i,
                                                     STATIC_CAST(int, Table->tdFieldsArray_ls00[i]->fiStmtColNo_ls00),
                                                     STATIC_CAST(int, Table->tdFieldsArray_ls00[i]->fiDefColNo_ls00),
@@ -843,11 +844,10 @@ void tls99_TraceClass::ls99Tab2(const _TCHAR *Layer, tls00_TableDescription *Tab
                 len += sp77sprintf(m_szTraceText + len, BUF_SIZE_LS99 - len, pszDefaultType[Table->tdFieldsArray_ls00[i]->fiDefault_ls00]);
                 
             }
-            len += sp77sprintf(m_szTraceText + len, BUF_SIZE_LS99 - len, "%s\n", Table->tdColName_ls00[i]);
+            len += sp77sprintf(m_szTraceText + len, BUF_SIZE_LS99 - len, "%s", Table->tdColName_ls00[i]);
 
             ls99_Write();  // End of for-loop reached -> flush the stuff
-        }
-
+        }   // end for (int i=0; i < Table->tdFieldCount_ls00; i++)
     }
 }
 // ls99Tab2()
@@ -973,6 +973,8 @@ void tls99_TraceClass::ls99TabParam(const _TCHAR *Layer, tls00_TableDescription
 */
 void tls99_TraceClass::ls99Init(const _TCHAR *Layer, const _TCHAR *Text, const _TCHAR *FileName)
 {
+	LoaderMessages* pMsgInstance = LoaderMessages::Instance();
+
     bTraceFileInit_ls99 = false;        // Initialisiere die statische variable
     
     tsp00_Date Date;
@@ -986,19 +988,16 @@ void tls99_TraceClass::ls99Init(const _TCHAR *Layer, const _TCHAR *Text, const _
     
     // Hole speicher fuer den dateinamen. Dieser muss in der ende-funktion wieder
     // freigegeben werden!!! Dann uebernimm den gelieferten dateinamen in die interne variable.
-    tsp01_RteError  Rte1Error;
-    tsp00_Pathc     tracefilepath;
+	_TCHAR   szLoaderPath[PATH_MXSP00];
+    szLoaderPath[0] = '\0';
+    pMsgInstance->ls98GetLoaderLogPath(szLoaderPath);
         
-    sqlGetIndependentWrkPath(tracefilepath,
-                             TERM_WITH_DELIMITER_EO01,
-                             &Rte1Error);
-        
-    size_t len = tracefilepath.length() + strlen(FileName);
+    size_t len = _tcslen(szLoaderPath) + _tcslen(FileName);
         
     m_pszTraceFileName = new char[len + 1];      // Initialisiere die member variablen
     m_pszTraceFileName[0] = '\0';
 
-    sp77sprintf(m_pszTraceFileName, len+1, "%s%s", tracefilepath.asCharp(), FileName);
+    sp77sprintf(m_pszTraceFileName, len+1, "%s%s", szLoaderPath, FileName);
     
     m_RteError.sp5fe_result.becomes(vf_ok);
 
@@ -1014,15 +1013,14 @@ void tls99_TraceClass::ls99Init(const _TCHAR *Layer, const _TCHAR *Text, const _
         len += szDate.size() + szTime.size() + sizeof(char);
         m_pszTraceFileName = new char [len + 1];
 
-        sp77sprintf(m_pszTraceFileName, len+1, "%sloader_%s%s.trc", tracefilepath.asCharp(), szDate.asCharp(),
-                                                                                             szTime.asCharp());
+        sp77sprintf(m_pszTraceFileName, len+1, "%sloader_%s%s.trc", szLoaderPath, szDate.asCharp(), szTime.asCharp());
 
         sqlfopenc(m_pszTraceFileName, sp5vf_text, sp5vf_write, sp5bk_unbuffered, &m_TraceFileHandle, &m_RteError);
     }
     
     m_szTraceText[0] = 0;
 
-    sp77sprintf(m_szTraceText, BUF_SIZE_LS99, "%s: %s	%s%s", Layer, Text, szDate.asCharp(), szTime.asCharp());
+    sp77sprintf(m_szTraceText, BUF_SIZE_LS99, "%s: %s   %s%s", Layer, Text, szDate.asCharp(), szTime.asCharp());
     ls99_Write();
 
     bTraceFileInit_ls99 = true;
