@@ -861,8 +861,11 @@ typedef struct access_rights_record
   {
   BOOL                   fDescOwnerIsAdmin;
   BOOL                   fAddDefaultDACL;
+  SAPDB_Bool             ignoreOwnUser;
+  SAPDB_Char *           pUseOtherUser;
 
   USER_ACCESS_RIGHTS_REC User;
+  USER_ACCESS_RIGHTS_REC OtherUser;
   USER_ACCESS_RIGHTS_REC Creator;
   USER_ACCESS_RIGHTS_REC Admin;
   USER_ACCESS_RIGHTS_REC System;
@@ -1104,7 +1107,8 @@ INT             sql40c_get_serviceport      ( teo00_Socket    sd,
  LONG            sql49c_get_user_info            ( char *                *ppszLogonName,
                                                    char *                *ppszRefDomain );
  LONG            sql49c_user_is_admin            ( BOOL                  *pfIsAdmin );
- LONG            sql49c_get_own_textual_SID      ( char *                *ppszSIDStringName );
+ LONG            sql49c_get_user_textual_SID     ( SAPDB_Char *          accountName,
+                                                   PSZ *                 ppszTextualSID);
  LONG            sql49c_set_file_security        ( char *                pszFileName,
                                                    PSECURITY_DESCRIPTOR  pSD );
  LONG            sql49c_set_service_logon_right  ( char *                pszNodeName,
@@ -1134,6 +1138,8 @@ INT             sql40c_get_serviceport      ( teo00_Socket    sd,
  LONG sql50_reg_del_applic_key     ( char *                    pszNodeName,
                                      HKEY                      hKey,
                                      char *                    pszSubKey );
+ LONG sql50_RegistryDeleteApplSupKey (HKEY                     hKey,
+                                     SAPDB_Char *              pszSubKey);
  LONG sql50_reg_get_applic_values  ( char *                    pszNodeName,
                                      HKEY                      hKey,
                                      char *                    pszSubKey,
@@ -1145,6 +1151,7 @@ INT             sql40c_get_serviceport      ( teo00_Socket    sd,
                                      ULONG                     ulNumOfEntries,
                                      PREG_ENTRY_REC            pEntyArray );
  LONG sql50_reg_put_applic_values  ( char *                    pszNodeName,
+                                     SAPDB_Char *              accountName,
                                      HKEY                      hKey,
                                      char *                    pszSubKey,
                                      char *                    pszPrivateSubKey,
@@ -1323,10 +1330,11 @@ APIRET           sql97_xserver_state       ( char *       pszNodeName,
 
 // --- VOS97CC ---------------------------------------------------------------
 
-BOOL             sql97cc_findControlServer ( char *    pszExecutable,   // OUT
-                                             char *    pszServerDB,     // IN
-                                             char *    pszServerPgm,    // IN
-                                             char *    pszDBRoot);      // IN/OUT
+BOOL             sql97cc_findControlServer ( char *                         pszExecutable,// OUT
+                                             char *                         pszServerDB,  // IN
+                                             char *                         pszServerPgm, // IN
+                                             char *                         pszDBRoot,    // IN/OUT
+                                             tsp00_ErrText VAR_ARRAY_REF    errText);     // OUT
 #if defined (USER)
 // ===================== USER PROTOTYPES ======================================
 
