@@ -129,6 +129,7 @@ typedef struct user_task_comm_ctrl
   struct teo003_CommPacketRecord  *pReplyCommPacket;
   ULONG                           ulPacketCnt;
   struct teo003_CommPacketRecord  *pCommPacketList[MAX_SQL_PACKETS];
+  ULONG                           ulServiceType;
   } USER_TASK_COMM_CTRL, tos00k_UserTaskCommCtrl;
 
 typedef USER_TASK_COMM_CTRL       *PUSER_TASK_COMM_CTRL;
@@ -500,7 +501,7 @@ VOID                   vnclose           ( PROCESS_ID       pid,
                                            tsp00_ErrText      errtext );
 
 externC VOID	       vgetapplpid       ( PROCESS_ID                   tid ,
-                                           tsp00_NodeId VAR_VALUE_REF   appl_node ,
+                                           tsp00_NodeId VAR_ARRAY_REF   appl_node ,
                                            tsp00_Int4   VAR_VALUE_REF   appl_pid ,
                                            BOOLEAN      VAR_VALUE_REF   ok );
 
