@@ -833,6 +833,7 @@ typedef struct task_ctrl_record
    BOOL                           fDebugTask;
    SAPDB_Bool                     fTaskIsWaitingForExclusiveAccess; // ... to a RWRegion
    struct task_ctrl_record       *nextTaskInWaitQueue ;
+   void                          *RWRegion;
 #if defined(_ALPHA_)
    double                         StructureAlignment; // - don't insert any element behind this line!
 #endif
@@ -999,7 +1000,7 @@ typedef struct ukt_ctrl_record
   SAPDB_UInt8                     RunnableMicroSeconds[TIME_VAL_ARRAY_SIZE];
   SAPDB_UInt8                     MaxRunnableMicroSeconds[TIME_VAL_ARRAY_SIZE];
 
-  tsp00_Uint4                     taskSwitches;       // PTS 1108715
+  tsp00_Uint8                     taskSwitches;       // PTS 1108715
   #if defined(_ALPHA_)
    double                         StructureAlignment; // - don't insert any element behind this line!
   #endif
@@ -1374,6 +1375,7 @@ typedef struct kernel_global_sec_record
   SECURITY_ATTRIBUTES             WorldSA;               // - NT only
   SECURITY_ATTRIBUTES             LockSA;                // - NT only
   SECURITY_ATTRIBUTES             AdminSA;               // - NT only
+  SECURITY_ATTRIBUTES             FileSA;                // - NT only
 
   HANDLE                          hCompletionPort;
   ULONG                           ulNumOfDevWorker;
