@@ -388,7 +388,7 @@ ULONG sqlSetLocMachRegValue ( PSZ    pszRelativePath,
     }
 
 
-  rc = sql50_reg_put_applic_values ( NULL, HKEY_LOCAL_MACHINE,
+  rc = sql50_reg_put_applic_values ( NULL, NULL, HKEY_LOCAL_MACHINE,
                                      pszRelativePath, NULL, 1, RegistryEntries );
 
   DBGOUT;
