@@ -47,8 +47,11 @@
 #include "hos23.h"
 #include "hos33.h"
 #include "geo40.h"
+#include "geo401.h"
 #include "geo03.h"
 #include "geo43_0.h"
+#include "RunTime/System/RTESys_MicroTime.h"
+#include "heo420.h"
 
 
 /*
@@ -203,14 +206,149 @@ sql03u_procserver_constructor (
 /*===============================================================================*/
 /* This function is used for calls from C / C++ functions only                   */
 /*===============================================================================*/
-externC tsp01_CommErr_Enum  SqlAConnect (	char                  *szServerNode,	/* '\0' terminated node or saprouterstring */
-                                          tsp00_DbNamec          szServerDB,    /* '\0' terminated dbname                  */
-                                          tsp01_Service_Enum     service,
-                                          tsp00_Int4             packet_cnt,			
-                                          tsp00_Int4             *reference,
-                                          tsp00_Int4             *sql_packet_size,
-                                          void                   *sql_packet_list[],  /* tsp00_Array<tsp1_packet_ptr, ...> */
-                                          tsp00_ErrTextc         pErrText )          /* /* '\0' terminated error string   */
+
+
+externC tsp01_CommErr_Enum  SqlPingConnect ( char*                  szServerNode,    /* '\0' terminated node or saprouterstring */
+                                             char*                  szServerDB,      /* '\0' terminated dbname                  */
+                                             tsp00_Int4*            reference,      /* connection reference                    */
+                                             tsp00_ErrTextc         pErrText )       /* '\0' terminated error string            */
+  {
+  #undef  MF__
+  #define MF__ MOD__"SqlPingConnect"
+  tsp01_CommErr_Enum        commError;
+  ULONG                     packetSize;
+  void*                     packetList;
+
+  DBGIN;
+
+  sql03u_init();
+
+  if (strlen (szServerDB) >= sizeof (SQL_DBNAMEC))
+  {
+      eo46BuildErrorStringRC ( pErrText, ERRMSG_COM_SERVERDB_NAME_TOO_LONG, 0 );
+      DBGOUT;
+      return commErrNotOk_esp01;
+  }
+
+  commError = (tsp01_CommErr_Enum)sql03u_connect ( szServerNode,
+                                                   szServerDB,
+                                                   (ULONG) srvPing_esp01,
+                                                   1,
+                                                   &packetSize,
+                                                   &packetList,
+                                                   reference,
+                                                   NULL,
+                                                   NULL,
+                                                   NULL,
+                                                   pErrText );
+
+  DBGOUT
+  return (commError) ;
+  }
+
+/*------------------------------*/
+
+
+externC tsp01_CommErr_Enum SqlDBPing ( tsp00_Int4                         reference,
+                                       tsp00_Uint4*                       sendBufferSize,
+                                       tsp00_Uint1*                       hops,
+                                       tsp00_DbNamec   VAR_ARRAY_REF      szServer,
+                                       tsp00_Versionc  VAR_ARRAY_REF      szServerVersion,
+                                       tsp00_ErrTextc                     pErrText )
+{
+    tsp01_CommErr_Enum              commState;
+    char*                           server;
+    char*                           serverVersion;
+    PUSER_CONNECT_INFO_REC          pConnInfo;
+    tsp00_Uint4                     sendLen;
+    ULONG                           tmp;
+
+    // --- get the pointer to the internal communication structure
+    if ( !sql09c_get_handle_data  ( pHandles, reference, (PVOID*)&pConnInfo, &tmp, &tmp ))
+    {
+        MSG1 (( INFO_CL_COMM_REQUEST_ERR, pConnInfo->ulTraceConID, commErrNotOk_esp01, ERRMSG_COM_ILL_REFERENCE ));
+        eo46BuildErrorStringRC ( pErrText, ERRMSG_COM_ILL_REFERENCE, 0 );
+        return commErrNotOk_esp01;
+    }
+
+    if ( *sendBufferSize > pConnInfo->ulMaxCmdDataLen ) 
+        *sendBufferSize = pConnInfo->ulMaxCmdDataLen;
+
+    *sendBufferSize = ALIGN( *sendBufferSize, SQL_PACKET_ALIGNMENT );
+
+    sendLen =  eo420FillPingPacket ( (char*)pConnInfo->pCommPacketList[0]->pDataPart,
+                                     *sendBufferSize, *hops, "PingClient", 0, "!!! SEND DATA !!!" );
+
+    if ( 0 != sendLen )
+    {
+        commState = sql03u_request ( reference, pConnInfo->pCommPacketList[0]->pDataPart, sendLen, pErrText );
+
+        if ( commState == commErrOk_esp01 )
+        {
+            char*              pResPacket;
+            ULONG              ulResDataLen;
+
+            commState = sql03u_receive ( reference, &pResPacket, (PULONG)&ulResDataLen, pErrText );
+
+            if ( commState == commErrOk_esp01 )
+            {
+                if ( eo420ExtractPingPacket( pResPacket, ulResDataLen, hops, &server, &serverVersion ) )
+                {
+                    if ( server )
+                    {
+                        szServer[sizeof(tsp00_DbNamec)-1];
+                        strncpy ( szServer, server, sizeof(tsp00_DbNamec));
+                    }
+                    if ( serverVersion )
+                    {
+                        szServerVersion[sizeof(tsp00_Versionc)-1];
+                        strncpy ( szServerVersion, serverVersion, sizeof(tsp00_Versionc));
+                    }
+                }
+                else
+                {
+                    MSGD (( ERR_PROTOCOL_ERROR, "PING REPLY" ));
+                    eo46BuildErrorStringRC ( pErrText, ERRMSG_PROTOCOL_ERROR, 0 );
+                    commState = commErrNotOk_esp01;
+                }
+            }
+        }
+    }
+    else
+    {
+        MSGD (( ERR_PROTOCOL_ERROR, "PING REPLY" ));
+        eo46BuildErrorStringRC ( pErrText, ERRMSG_PROTOCOL_ERROR, 0 );
+        commState = commErrNotOk_esp01;
+    }
+
+    return commState;
+}
+
+
+/*------------------------------*/
+
+externC void SqlPingRelease (tsp00_Int4 reference)
+  {
+  #undef  MF__
+  #define MF__ MOD__"SqlPingRelease"
+  tsp00_ErrTextc  pErrText;
+
+  sql03u_release ( reference, pErrText );
+
+  return;
+  }
+
+/*------------------------------*/
+
+
+externC tsp01_CommErr_Enum  SqlAConnect (   char                  *szServerNode,        /* '\0' terminated node or saprouterstring */
+                                            tsp00_DbNamec          szServerDB,          /* '\0' terminated dbname                  */
+                                            tsp01_Service_Enum     service,
+                                            tsp00_Int4             packet_cnt,          
+                                            tsp00_Int4             *reference,
+                                            tsp00_Int4             *sql_packet_size,
+                                            void                   *sql_packet_list[],  /* tsp00_Array<tsp1_packet_ptr, ...> */
+                                            tsp00_ErrTextc         pErrText )           /* '\0' terminated error string      */
   {
   #undef  MF__
   #define MF__ MOD__"SqlAConnect"
@@ -251,16 +389,16 @@ externC tsp01_CommErr_Enum  SqlAConnect (	char                  *szServerNode,	/
  * ========================== GLOBAL FUNCTIONS ================================
  */
 
-void sqlaconnect (	tsp00_TaskId           pid,
-                    tsp00_NodeId           servernode,			/* tsp00_Array<char, mxsp_c64> */
-                    tsp00_DbName           serverdb,			  /* tsp00_Array<char, mxsp_c18> */
+void sqlaconnect (  tsp00_TaskId           pid,
+                    tsp00_NodeId           servernode,          /* tsp00_Array<char, mxsp_c64> */
+                    tsp00_DbName           serverdb,              /* tsp00_Array<char, mxsp_c18> */
                     tsp01_Service_Enum     service,
-                    tsp00_Int4          	 packet_cnt,			
+                    tsp00_Int4               packet_cnt,            
                     tsp00_Int4             *reference,
                     tsp00_Int4             *sql_packet_size,
                     void                   *sql_packet_list[],  /* tsp00_Array<tsp1_packet_ptr, ...> */
-                    tsp00_ErrText          errtext,			        /* tsp00_Array<char, mxsp_c40> */
-                    tsp01_CommErr          *returncode) 		    /* tsp00_Enum<tsp1_comm_error_Enum, ...> */
+                    tsp00_ErrText          errtext,                 /* tsp00_Array<char, mxsp_c40> */
+                    tsp01_CommErr          *returncode)             /* tsp00_Enum<tsp1_comm_error_Enum, ...> */
   {
   #undef  MF__
   #define MF__ MOD__"sqlaconnect"
@@ -309,7 +447,7 @@ void sqlaconnect (	tsp00_TaskId           pid,
 
 /*------------------------------*/
 
-void sqlarelease (tsp00_Int4				reference)
+void sqlarelease (tsp00_Int4                reference)
   {
   #undef  MF__
   #define MF__ MOD__"sqlarelease"
@@ -330,8 +468,8 @@ void sqlarelease (tsp00_Int4				reference)
 void sqlarequest (  tsp00_Int4        reference,
                     void              *sql_packet_addr,
                     tsp00_Int4        sql_packet_length,
-                    tsp00_ErrText     errtext,			
-                    tsp01_CommErr     *returncode) 		
+                    tsp00_ErrText     errtext,          
+                    tsp01_CommErr     *returncode)      
   {
   #undef  MF__
   #define MF__ MOD__"sqlarequest"
@@ -359,8 +497,8 @@ void sqlarequest (  tsp00_Int4        reference,
 
 /*------------------------------*/
 
-void sqlareplyavailable (  tsp00_Int4				reference,
-                           tsp00_ErrText			errtext,			
+void sqlareplyavailable (  tsp00_Int4               reference,
+                           tsp00_ErrText            errtext,            
                            tsp01_CommErr          *returncode)
 
   {
@@ -384,10 +522,10 @@ void sqlareplyavailable (  tsp00_Int4				reference,
 /*------------------------------*/
 
 
-void sqlareceive (	tsp00_Int4        reference,
+void sqlareceive (  tsp00_Int4        reference,
                     void              **res_packet_ptr,
                     tsp00_Int4        *res_packet_length,
-                    tsp00_ErrText     errtext,			
+                    tsp00_ErrText     errtext,          
                     tsp01_CommErr     *returncode)
 
   {
@@ -432,9 +570,9 @@ void sqlacancel ( tsp00_Int4              reference)
 
 /*------------------------------*/
 
-externC tsp01_CommErr_Enum  SqlAPing ( char                  *pszServerNode,     /* '\0' terminated node or saprouterstring */
-                                       tsp00_Versionc         szServerVersion,   /* '\0' terminated version from answering server */
-                                       tsp00_ErrTextc         pErrText)          /* '\0' terminated error string   */
+externC tsp01_CommErr_Enum  SqlAPing (   char*                          pszServerNode,   /* '\0' terminated node or saprouterstring */
+                                         tsp00_Versionc  VAR_ARRAY_REF  szServerVersion, /* '\0' terminated version from answering server */
+                                         tsp00_ErrTextc  VAR_ARRAY_REF  pErrText)        /* '\0' terminated error string   */
 {
   #undef  MF__
   #define MF__ MOD__"SqlAPing"
@@ -467,11 +605,13 @@ externC tsp01_CommErr_Enum  SqlAPing ( char                  *pszServerNode,
         if ( !fNI_Installed )
           commError = commErrNotOk_esp01 ;
         else
-          commError = eo03NiPing( pszServerNode, szServerVersion, pErrText ) ;
+        {
+            commError = eo03NiXserverPing( pszServerNode, szServerVersion, pErrText ) ;
+        }
       }
       else
       {
-        commError = os23NiPing( pszServerNode, szServerVersion, pErrText ) ;
+          commError = os23XServerPing( pszServerNode, szServerVersion, pErrText ) ;
       }
     }
   }
@@ -1475,8 +1615,8 @@ static LONG sql03u_get_protocol ( PSZ                    pszServerNode,
         if ( rc != NO_ERROR )
           strcpy ( szLocalNodeName, szNodeName );
         }
-	  else
-	    strcpy ( szLocalNodeName, szNodeName );
+      else
+        strcpy ( szLocalNodeName, szNodeName );
       DBG3 (( MF__, "Local node: '%s'", szLocalNodeName ));
       }
 
