@@ -243,8 +243,12 @@ VOID    sqlexec (   tsp00_ExecArgLine   pszCommand,
      ulProgNameLen = ulCmdLineLen - ulArgsLen;
 
      // --- WORK AROUND (Windows 95 is modifying the console title)
-     if ( !strcmpi ( szProgName, TITLE_COMMAND ))
-       {
+#if _MSC_VER >= 1400
+     if ( !stricmp ( szProgName, TITLE_COMMAND ))
+#else
+	 if ( !strcmpi ( szProgName, TITLE_COMMAND ))
+#endif
+	 {
        SetConsoleTitle ( pszArgs );
 
        *pfProgResult = 0;
