@@ -79,12 +79,14 @@
                                              C20                acC20Array[MAX_CRYPT] );
  static VOID   sql13u_recrypt_name         ( C20                acReCryptName[MAX_CRYPT],
                                              CRYPTNAME          alCryptedName );
- static LONG   sql13u_read_data_from_reg   ( PCHAR              *ppcDataBuffer,
-                                             ULONG              *pulDataLen,
-                                             tsp00_ErrText        acErrorText );
- static LONG   sql13u_write_data_to_reg    ( PCHAR              pcDataBuffer,
+ static LONG   sql13u_read_data_from_reg   ( SAPDB_Char *       accountName,
+                                             PCHAR *            ppcDataBuffer,
+                                             ULONG *            pulDataLen,
+                                             tsp00_ErrText      acErrorText );
+ static LONG   sql13u_write_data_to_reg    ( SAPDB_Char *       accountName,
+                                             PCHAR              pcDataBuffer,
                                              ULONG              ulDataLen,
-                                             tsp00_ErrText        acErrorText );
+                                             tsp00_ErrText      acErrorText );
 
  static BOOL sql13u_use_hkey_local_machine ( void ) ;
 
@@ -93,10 +95,11 @@
 // ========================== GLOBAL FUNCTIONS ================================
 //
 
-tsp00_Int sql13u_get_all_entries ( char                      **pcDataBuffer,
-                                   tsp00_Int4                *DataLen,
-                                   PREL611_XUSER_PAGE_UNION  *pRel611XUserPages,
-                                   tsp00_ErrText               acErrorText )
+tsp00_Int sql13u_get_all_entries (SAPDB_Char *              accountName, 
+                                  char                      **pcDataBuffer,
+                                  tsp00_Int4                *DataLen,
+                                  PREL611_XUSER_PAGE_UNION  *pRel611XUserPages,
+                                  tsp00_ErrText               acErrorText )
   {
   #undef  MF__
   #define MF__ MOD__"sql13u_get_all_entries"
@@ -105,8 +108,34 @@ tsp00_Int sql13u_get_all_entries ( char                      **pcDataBuffer,
 
   *pRel611XUserPages = NULL;
 
-  rc = sql13u_read_data_from_reg( (PCHAR *)pcDataBuffer,
-                                   DataLen, acErrorText );
+
+  /* Only Administrators are allowed to read XUSER entries for another user */
+  if (NULL != accountName)
+  {
+      BOOL     isAdmin = false;
+
+      rc = sql49c_user_is_admin (&isAdmin);
+      if (NO_ERROR != rc)
+      {
+          DBGOUT;
+          return ( rc );
+      }
+
+      if (!isAdmin)
+      {
+          DBG1  (( MF__, ERRMSG_XUSER_NO_PERMISSION));
+          MSGD  ((ERR_XUSER_NO_PERMISSION));
+          sql46c_build_error_string ( acErrorText, ERRMSG_XUSER_NO_PERMISSION, 0);
+          DBGOUT;
+          return ( -1 );
+      }
+  }
+
+
+  rc = sql13u_read_data_from_reg(accountName, 
+                                 (PCHAR *)pcDataBuffer,
+                                 DataLen, 
+                                 acErrorText );
 
   if ( rc != NO_ERROR )
      *pcDataBuffer = (char *)NULL;
@@ -116,7 +145,8 @@ tsp00_Int sql13u_get_all_entries ( char                      **pcDataBuffer,
 
  /*------------------------------*/
 
-tsp00_Int sql13u_remove_xuser_entries ( tsp00_ErrText  acErrorText )
+tsp00_Int sql13u_remove_xuser_entries (SAPDB_Char *     accountName,
+                                       tsp00_ErrText    acErrorText )
    {
    #undef  MF__
    #define MF__ MOD__"sql13u_remove_xuser_entries"
@@ -127,41 +157,71 @@ tsp00_Int sql13u_remove_xuser_entries ( tsp00_ErrText  acErrorText )
 
    if (( sql02_get_platform_id()         == VER_PLATFORM_WIN32_NT ) &&
        ( sql13u_use_hkey_local_machine() == TRUE ))
-     {
-     rc = sql49c_get_own_textual_SID ( &pszTextualSID );
+   {
 
-     if ( rc != NO_ERROR )
+	   /* Only Administrators are allowed to remove XUSER entries for another user */
+	   if (NULL != accountName)
+	   {
+		   BOOL     isAdmin = false;
+
+		   rc = sql49c_user_is_admin (&isAdmin);
+		   if (NO_ERROR != rc)
+		   {
+			   DBGOUT;
+			   return ( rc );
+		   }
+
+		   if (!isAdmin)
+		   {
+			   DBG1  (( MF__, ERRMSG_XUSER_NO_PERMISSION));
+			   MSGD  ((ERR_XUSER_NO_PERMISSION));
+			   sql46c_build_error_string ( acErrorText, ERRMSG_XUSER_NO_PERMISSION, 0);
+			   DBGOUT;
+			   return ( -1 );
+		   }
+	   }
+
+       rc = sql49c_get_user_textual_SID (accountName, &pszTextualSID);
+
+       if ( rc != NO_ERROR )
        {
-       DBGOUT;
-       return ( rc );
+           DBGOUT;
+           return ( rc );
        }
 
-     strcpy ( szSubKey, REG_VN_USER );
-     strcat ( szSubKey, "\\" );
-     strcat ( szSubKey, pszTextualSID );
+       strcpy ( szSubKey, REG_VN_USER );
+       strcat ( szSubKey, "\\" );
+       strcat ( szSubKey, pszTextualSID );
 
-     FREE_MEM ( pszTextualSID );
 
-     rc = sql50_reg_del_applic_value ( NULL, HKEY_LOCAL_MACHINE,
-                                       szSubKey, REG_VN_XUSER );
-     }
+       /* Remove Registry value */
+       rc = sql50_reg_del_applic_value ( NULL, HKEY_LOCAL_MACHINE,
+           szSubKey, REG_VN_XUSER );
+
+       if (NO_ERROR == rc || ERROR_FILE_NOT_FOUND == rc)
+       {
+           rc = sql50_RegistryDeleteApplSupKey  (HKEY_LOCAL_MACHINE, szSubKey);
+       }
+
+       FREE_MEM ( pszTextualSID );
+   }
    else
-     {
-     rc = sql50_reg_del_applic_value ( NULL, HKEY_CURRENT_USER,
-                                       "", REG_VN_XUSER );
-     }
+   {
+       rc = sql50_reg_del_applic_value ( NULL, HKEY_CURRENT_USER,
+           "", REG_VN_XUSER );
+   }
 
 
    if (( rc != NO_ERROR ) && ( rc !=  ERROR_FILE_NOT_FOUND ))
-     {
-     DBG1  (( MF__, ERRMSG_REMOVE_XUSER_REG_DATA, rc ));
-     MSGD  (( ERR_REMOVE_XUSER_REG_DATA, rc ));
-     sql46c_build_error_string ( acErrorText, ERRMSG_REMOVE_XUSER_REG_DATA,
-                                 rc );
+   {
+       DBG1  (( MF__, ERRMSG_REMOVE_XUSER_REG_DATA, rc ));
+       MSGD  (( ERR_REMOVE_XUSER_REG_DATA, rc ));
+       sql46c_build_error_string ( acErrorText, ERRMSG_REMOVE_XUSER_REG_DATA,
+           rc );
 
-     DBGOUT;
-     return ( rc );
-     }
+       DBGOUT;
+       return ( rc );
+   }
 
 
    DBGOUT;
@@ -171,9 +231,10 @@ tsp00_Int sql13u_remove_xuser_entries ( tsp00_ErrText  acErrorText )
 
  /*------------------------------*/
 
-tsp00_Int sql13u_write_xuser_entries ( char              *cDataBuffer,
-                                       PXUSER_INFO_PAGE   pXUserInfoPage,
-                                       tsp00_ErrText        acErrorText )
+tsp00_Int sql13u_write_xuser_entries (SAPDB_Char *      accountName,
+                                      char              *cDataBuffer,
+                                      PXUSER_INFO_PAGE   pXUserInfoPage,
+                                      tsp00_ErrText        acErrorText )
    {
    #undef  MF__
    #define MF__ MOD__"sql13u_write_xuser_entries"
@@ -183,7 +244,7 @@ tsp00_Int sql13u_write_xuser_entries ( char              *cDataBuffer,
    ulDataLen = pXUserInfoPage->ulInfoPageSize +
                (pXUserInfoPage->ulPages * pXUserInfoPage->ulPageSize);
 
-   rc = sql13u_write_data_to_reg( cDataBuffer, ulDataLen, acErrorText );
+   rc = sql13u_write_data_to_reg(accountName, cDataBuffer, ulDataLen, acErrorText);
 
    DBGOUT;
    return ( rc );
@@ -301,9 +362,10 @@ static void sql13u_recrypt_name ( tsp00_C20         acReCryptName[MAX_CRYPT],
 
  /*------------------------------*/
 
-static LONG sql13u_read_data_from_reg( PCHAR       *ppcDataBuffer,
-                                       ULONG       *pulDataLen,
-                                       tsp00_ErrText acErrorText )
+static LONG sql13u_read_data_from_reg (SAPDB_Char *     accountName, 
+                                       PCHAR *          ppcDataBuffer,
+                                       ULONG *          pulDataLen,
+                                       tsp00_ErrText    acErrorText )
    {
    #undef  MF__
    #define MF__ MOD__"sql13u_read_data_from_reg"
@@ -322,9 +384,8 @@ static LONG sql13u_read_data_from_reg( PCHAR       *ppcDataBuffer,
 
    if (( sql02_get_platform_id()         == VER_PLATFORM_WIN32_NT ) &&
        ( sql13u_use_hkey_local_machine() == TRUE ))
-     {
-     rc = sql49c_get_own_textual_SID ( &pszTextualSID );
-
+   {
+     rc = sql49c_get_user_textual_SID (accountName, &pszTextualSID);
      if ( rc != NO_ERROR )
        {
        sql46c_build_error_string ( acErrorText, ERRMSG_GET_SID_STRING, rc );
@@ -397,64 +458,95 @@ static LONG sql13u_read_data_from_reg( PCHAR       *ppcDataBuffer,
 
  /*------------------------------*/
 
- static LONG sql13u_write_data_to_reg( PCHAR       pcDataBuffer,
-                                       ULONG       ulDataLen,
-                                       tsp00_ErrText acErrorText )
-   {
-   #undef  MF__
-   #define MF__ MOD__"sql13u_write_data_to_reg"
-   LONG                rc               = NO_ERROR;
-   REG_ENTRY_REC       RegistryEntries[1];
-   PSZ                 pszTextualSID;
-   PATHNAME            szPrivateSubKey;
+static LONG sql13u_write_data_to_reg    (SAPDB_Char *      accountName, 
+                                         PCHAR             pcDataBuffer,
+                                         ULONG             ulDataLen,
+                                         tsp00_ErrText     acErrorText )
+{
+#undef  MF__
+#define MF__ MOD__"sql13u_write_data_to_reg"
+    LONG                rc               = NO_ERROR;
+    REG_ENTRY_REC       RegistryEntries[1];
+    PSZ                 pszTextualSID;
+    PATHNAME            szPrivateSubKey;
+    PATHNAME            szSubKey;
 
-   RegistryEntries[0].pszValueName = REG_VN_XUSER;
-   RegistryEntries[0].pValue       = pcDataBuffer;
-   RegistryEntries[0].ulValueSize  = ulDataLen;
-   RegistryEntries[0].ulValueType  = REG_BINARY;
 
-   if (( sql02_get_platform_id()         == VER_PLATFORM_WIN32_NT ) &&
-       ( sql13u_use_hkey_local_machine() == TRUE ))
-     {
-     rc = sql49c_get_own_textual_SID ( &pszTextualSID );
+    RegistryEntries[0].pszValueName = REG_VN_XUSER;
+    RegistryEntries[0].pValue       = pcDataBuffer;
+    RegistryEntries[0].ulValueSize  = ulDataLen;
+    RegistryEntries[0].ulValueType  = REG_BINARY;
 
-     if ( rc != NO_ERROR )
+    if (( sql02_get_platform_id()         == VER_PLATFORM_WIN32_NT ) &&
+        ( sql13u_use_hkey_local_machine() == TRUE ))
+    {
+        rc = sql49c_get_user_textual_SID (accountName, &pszTextualSID);
+
+        if ( rc != NO_ERROR )
+        {
+            DBGOUT;
+            return ( rc );
+        }
+
+        strcpy ( szPrivateSubKey, pszTextualSID );
+        FREE_MEM ( pszTextualSID );
+
+        /* Registry subkey and value are first removed */ 
+        /* Remove Registry value */
+
+        strcpy ( szSubKey, REG_VN_USER );
+        strcat ( szSubKey, "\\" );
+        strcat ( szSubKey, szPrivateSubKey );
+
+        rc = sql50_reg_del_applic_value ( NULL, HKEY_LOCAL_MACHINE,
+            szSubKey, REG_VN_XUSER );
+
+       if (NO_ERROR == rc || ERROR_FILE_NOT_FOUND == rc)
        {
-       DBGOUT;
-       return ( rc );
+           rc = sql50_RegistryDeleteApplSupKey  (HKEY_LOCAL_MACHINE, szSubKey);
        }
 
-     strcpy ( szPrivateSubKey, pszTextualSID );
-     FREE_MEM ( pszTextualSID );
+        if (( rc != NO_ERROR ) && ( rc !=  ERROR_FILE_NOT_FOUND ))
+        {
+            DBG1  (( MF__, ERRMSG_REMOVE_XUSER_REG_DATA, rc ));
+            MSGD  (( ERR_REMOVE_XUSER_REG_DATA, rc ));
+            sql46c_build_error_string ( acErrorText, ERRMSG_REMOVE_XUSER_REG_DATA,
+                rc );
 
-     rc = sql50_reg_put_applic_values ( NULL, HKEY_LOCAL_MACHINE,
-                                        REG_VN_USER, szPrivateSubKey,
-                                        1, RegistryEntries );
-     }
-   else
-     {
-     rc = sql50_reg_put_applic_values ( NULL, HKEY_CURRENT_USER,
-                                        "", "", 1, RegistryEntries );
-     }
+            DBGOUT;
+            return ( rc );
+        }
 
-   if ( rc != NO_ERROR )
-     {
-     DBG1  (( MF__, ERRMSG_WRITE_XUSER_REG_DATA, rc ));
-     MSGD  (( ERR_WRITE_XUSER_REG_DATA, rc ));
-     sql46c_build_error_string ( acErrorText, ERRMSG_WRITE_XUSER_REG_DATA,
-                                 rc );
-     DBGOUT;
-     return ( rc );
-     }
 
-   DBGOUT;
-   return ( NO_ERROR );
-   }
+        rc = sql50_reg_put_applic_values ( NULL, accountName, HKEY_LOCAL_MACHINE,
+            REG_VN_USER, szPrivateSubKey,
+            1, RegistryEntries );
+    }
+    else
+    {
+        rc = sql50_reg_put_applic_values ( NULL, accountName, HKEY_CURRENT_USER,
+            "", "", 1, RegistryEntries );
+    }
+
+    if ( rc != NO_ERROR )
+    {
+        DBG1  (( MF__, ERRMSG_WRITE_XUSER_REG_DATA, rc ));
+        MSGD  (( ERR_WRITE_XUSER_REG_DATA, rc ));
+        sql46c_build_error_string ( acErrorText, ERRMSG_WRITE_XUSER_REG_DATA,
+            rc );
+        DBGOUT;
+        return ( rc );
+    }
+
+    DBGOUT;
+    return ( NO_ERROR );
+}
 
  /*------------------------------*/
-
-static LONG sql13u_check_user_id ( PXUSER_INFO_PAGE  pInfoPage,
-                                   tsp00_ErrText       acErrorText )
+/*
+static LONG sql13u_check_user_id (SAPDB_Char *      accountName,
+                                  PXUSER_INFO_PAGE  pInfoPage,
+                                  tsp00_ErrText       acErrorText )
   {
   #undef  MF__
   #define MF__ MOD__"sql13u_check_user_id"
@@ -479,7 +571,7 @@ static LONG sql13u_check_user_id ( PXUSER_INFO_PAGE  pInfoPage,
 
   if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
     {
-    rc = sql49c_get_own_textual_SID ( &pszTextualSID );
+    rc = sql49c_get_user_textual_SID (accountName, &pszTextualSID);
 
     if ( rc != NO_ERROR )
       {
@@ -501,7 +593,7 @@ static LONG sql13u_check_user_id ( PXUSER_INFO_PAGE  pInfoPage,
     eo46CtoP ( acTextualSID, pszTextualSID, sizeof (acTextualSID));
     FREE_MEM ( pszTextualSID );
 
-    /* --- change textual SID to type "tsp00_CryptName" */
+    /* --- change textual SID to type "tsp00_CryptName" *
     sql21put_name ( acTextualSID, alCryptedName );
     }
   else
@@ -522,7 +614,7 @@ static LONG sql13u_check_user_id ( PXUSER_INFO_PAGE  pInfoPage,
      eo46CtoP ( acUserName, pszUserName, sizeof (acUserName));
      FREE_MEM ( pszUserName );
 
-     /* --- change eff. user  from type "tsp00_C20" to type "tsp00_CryptName" */
+     /* --- change eff. user  from type "tsp00_C20" to type "tsp00_CryptName" *
      sql21put_name ( acUserName, alCryptedName );
     #else
      DBGOUT;
@@ -530,7 +622,7 @@ static LONG sql13u_check_user_id ( PXUSER_INFO_PAGE  pInfoPage,
     #endif
     }
 
-  /* --- test of equality */
+  /* --- test of equality *
   for ( nIndex = 0; ( rc == NO_ERROR ) && ( nIndex < MAX_CRYPT ); nIndex++ )
     {
     if ( alCryptedName [ nIndex ] != alXUserCryptedName [ nIndex ] )
@@ -547,7 +639,7 @@ static LONG sql13u_check_user_id ( PXUSER_INFO_PAGE  pInfoPage,
   DBGOUT;
   return ( rc );
   }
-
+*/
  /*------------------------------*/
 
  APIRET sql13u_get_options_char_set_map ( BOOL *pfEnabled,
@@ -621,20 +713,22 @@ static LONG sql13u_check_user_id ( PXUSER_INFO_PAGE  pInfoPage,
 
 /*------------------------------*/
 
-void _stdcall sc_sqlxuopenuser ( tsp00_ErrText acErrorText,
-                                 BOOLEAN     *pbOk)
+void _stdcall sc_sqlxuopenuser ( SAPDB_Char *   accountName,
+                                 tsp00_ErrText  acErrorText,
+                                 BOOLEAN     *  pbOk)
   {
   #undef  MF__
   #define MF__ MOD__"sc_sqlxuopenuser"
 
   DBGPAS;
 
-  sqlxuopenuser ( acErrorText, pbOk);
+  sqlxuopenuser (accountName, acErrorText, pbOk);
   }
 
 /*------------------------------*/
 
-void _stdcall sc_sqlxucloseuser ( tsp00_ErrText acErrorText,
+void _stdcall sc_sqlxucloseuser ( SAPDB_Char *   accountName,
+                                  tsp00_ErrText acErrorText,
                                   BOOLEAN     *pbOk )
   {
   #undef  MF__
@@ -642,41 +736,44 @@ void _stdcall sc_sqlxucloseuser ( tsp00_ErrText acErrorText,
 
   DBGPAS;
 
-  sqlxucloseuser ( acErrorText, pbOk );
+  sqlxucloseuser (accountName, acErrorText, pbOk );
   }
 
 /*------------------------------*/
 
-void _stdcall sc_sqlputuser ( tsp4_xuser_record  *prcUserParams,
-                              tsp00_ErrText        acErrorText,
-                              BOOLEAN            *pbOk )
+void _stdcall sc_sqlputuser ( tsp4_xuser_record  *  prcUserParams,
+                              SAPDB_Char *          accountName,
+                              tsp00_ErrText         acErrorText,
+                              BOOLEAN            *  pbOk )
   {
   #undef  MF__
   #define MF__ MOD__"sc_sqlputuser"
 
   DBGPAS;
 
-  sqlputuser ( prcUserParams, acErrorText, pbOk );
+  sqlputuser ( prcUserParams, accountName, acErrorText, pbOk );
   }
 
 /*------------------------------*/
 
-void _stdcall sc_sqlgetuser ( tsp4_xuser_record  *prcUserParams,
-                              tsp00_ErrText        acErrorText,
-                              BOOLEAN            *pbOk )
+void _stdcall sc_sqlgetuser ( tsp4_xuser_record  *  prcUserParams,
+                              SAPDB_Char *          accountName,
+                              tsp00_ErrText         acErrorText,
+                              BOOLEAN            *  pbOk )
   {
   #undef  MF__
   #define MF__ MOD__"sc_sqlgetuser"
 
   DBGPAS;
 
-  sqlgetuser ( prcUserParams, acErrorText, pbOk );
+  sqlgetuser ( prcUserParams, accountName, acErrorText, pbOk );
   }
 
 /*------------------------------*/
 
 void _stdcall sc_sqlindexuser ( tsp00_Int2            kUserIndex,
                                 tsp4_xuser_record  *prcUserParams,
+                                SAPDB_Char *          accountName,
                                 tsp00_ErrText        acErrorText,
                                 BOOLEAN          *pbOk )
   {
@@ -685,19 +782,19 @@ void _stdcall sc_sqlindexuser ( tsp00_Int2            kUserIndex,
 
   DBGPAS;
 
-  sqlindexuser ( kUserIndex, prcUserParams, acErrorText, pbOk );
+  sqlindexuser ( kUserIndex, prcUserParams, accountName, acErrorText, pbOk );
   }
 
 /*------------------------------*/
 
-int _stdcall sc_sqlclearuser ( void )
+int _stdcall sc_sqlclearuser (SAPDB_Char *          accountName)
   {
   #undef  MF__
   #define MF__ MOD__"sc_sqlclearuser"
 
   DBGPAS;
 
-  return ( sqlclearuser () );
+  return ( sqlclearuser (accountName) );
   }
 
 //
