@@ -212,8 +212,9 @@
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, os202_EvtOptions);
      SetLastError(ERROR_NOT_ENOUGH_MEMORY);
@@ -263,8 +264,9 @@
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      if ( XSERVER.os204_ResourceTypeCreated())
        XSERVER.os204_Remove();
 
@@ -310,8 +312,9 @@
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, os202_EvtOptions);
      SetLastError(ERROR_NOT_ENOUGH_MEMORY);
@@ -431,8 +434,9 @@
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      // --- remove objects if created
      RemoveCreatedObjects = true;
 
@@ -562,8 +566,9 @@
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, os202_EvtOptions);
      SetLastError(ERROR_NOT_ENOUGH_MEMORY);
@@ -690,8 +695,9 @@
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, os202_EvtOptions);
      SetLastError(ERROR_NOT_ENOUGH_MEMORY);
@@ -732,8 +738,9 @@
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, os202_EvtOptions);
      SetLastError(ERROR_NOT_ENOUGH_MEMORY);
@@ -774,8 +781,9 @@
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, os202_EvtOptions);
      SetLastError(ERROR_NOT_ENOUGH_MEMORY);
@@ -807,8 +815,9 @@
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, os202_EvtOptions);
      SetLastError(ERROR_NOT_ENOUGH_MEMORY);
@@ -884,7 +893,7 @@
        
        // --- set DLL print options
        LOAD_FUNC_OS203( os202_SetEvtOptions );
-       os202_SetEvtOptions( EvtOptions & ~EVTOPT_DLL_LOAD_ERROR_OFF_OS202 );
+       os202_SetEvtOptions( EvtOptions & ~EVTOPT_DLL_LOAD_ERROR_OFF_OS202, false ); // PTS 1129537
      }
    }
    catch ( teo200_EventList  EventList )
@@ -893,8 +902,9 @@
      SetLastError(EventList.eo200_rc()); 
      os203_UnloadDynDLL();
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, EvtOptions);
 
@@ -963,8 +973,9 @@ static const bool os202_ClusterServerInsalled ()
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, os202_EvtOptions);
      SetLastError(ERROR_NOT_ENOUGH_MEMORY);
@@ -1014,8 +1025,9 @@ static const bool os202_ClusterServerInsalled ()
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, os202_EvtOptions);
      SetLastError(ERROR_NOT_ENOUGH_MEMORY);
@@ -1077,8 +1089,9 @@ static const bool os202_ClusterServerInsalled ()
      SetLastError(EventList.eo200_rc());
      Ok = false;
    }
-   catch (CMemoryException)
+   catch (CMemoryException *e) //   PTS 1129537
    {  
+	 e->Delete();
      teo200_EventList EventList ( ERROR_NOT_ENOUGH_MEMORY, FUNCTION_NAME_MEO60, ERR_OUT_OF_MEMORY ); 
      os202_HandleEvt(EventList, os202_EvtOptions);
      SetLastError(ERROR_NOT_ENOUGH_MEMORY);
