@@ -61,6 +61,7 @@
 #define MF__   MOD__"UNDEFINED"
 
 #define LINGER_TIME  15
+
 /*
  *  MACROS
  */
@@ -110,10 +111,11 @@ static VOID os23InitRteConnectParam ( PTCPIP_CONNECT_INFO_REC   pTCPIPConnInfo,
 
 /* PTS 1105295 */
 /*------------------------------*/
-tsp01_CommErr_Enum 
-os23NiPing ( char                  *pszServerNode,     /* '\0' terminated node or saprouterstring */
-             tsp00_Versionc         szServerVersion,   /* '\0' terminated version from answering server */
-             tsp00_ErrTextc         pErrText)          /* '\0' terminated error string   */
+
+externC tsp01_CommErr_Enum 
+os23XServerPing ( char                  *pszServerNode,     /* '\0' terminated node or saprouterstring */
+                  tsp00_Versionc         szServerVersion,   /* '\0' terminated version from answering server */
+                  tsp00_ErrTextc         pErrText)          /* '\0' terminated error string   */
 {
   #undef  MF__
   #define MF__ MOD__"os23NiPing"
@@ -189,6 +191,7 @@ os23NiPing ( char                  *pszServerNode,     /* '\0' terminated node o
   return ( ulCommState );
 }
 
+#ifdef SERVER
 /* PTS 1111208 */
 /*------------------------------*/
 tsp01_CommErr_Enum 
@@ -230,7 +233,28 @@ os23NewDebugLevel ( char                  *pszServerNode,     /* '\0' terminated
   memset(&ConPktParamRec, 0, sizeof(teo003_ConPktParamRecord) ) ;
   ConPktParamRec.ulSenderRef = newDebugLevel;
 
-  /* send PING request */
+  // to prevent a 'denial of service attack' by setting debug level.... it is only settable via registry
+  {
+      DWORD         dwDebugLevel = (DWORD)newDebugLevel;
+      REG_ENTRY_REC RegistryEntries[1];
+      PATHNAME      szKey;
+      LONG          rc;
+
+      RegistryEntries[0].pszValueName = REG_VN_DEBUG_LEVEL;
+      RegistryEntries[0].pValue       = &dwDebugLevel;
+      RegistryEntries[0].ulValueSize  = sizeof(dwDebugLevel);
+      RegistryEntries[0].ulValueType  = REG_DWORD;
+
+      strcpy ( szKey, XSERV_TITLE "\\" REG_SK_SERVICE_PARAM );
+
+      rc = sql50_reg_put_service_values ( NULL, szKey, 1, RegistryEntries );
+      if ( rc != NO_ERROR )
+      {
+          strcpy(pErrText, "Cant update registry");
+          return ( commErrNotOk_esp01 );
+      }
+  }
+  /* send NEW_DEBUG_LEVEL request */
   ConPktParamRec.ulMessClass         = RSQL_NEW_DEBUG_LEVEL_REQUEST_EO003;
 
   ulCommState = eo420SendConnectPacket( (void *) &TmpTCPIPConnInfo.sd,
@@ -265,6 +289,7 @@ os23NewDebugLevel ( char                  *pszServerNode,     /* '\0' terminated
   DBGOUT;
   return ( ulCommState );
 }
+#endif
 
 /*------------------------------*/
 
@@ -309,7 +334,7 @@ ULONG sql23c_connect ( PCONNECT_PARAM_REC       pConnParam,
     {
     // packet sizes are fixed, no info request necessary
     pTCPIPConnInfo->ulPacketSize   = csp9_ctrl_packet_size;
-    pTCPIPConnInfo->ulMinReplySize = ALIGN( 100, ALIGNMENT_VALUE );;
+    pTCPIPConnInfo->ulMinReplySize = ALIGN( 100, ALIGNMENT_VALUE );
     pTCPIPConnInfo->ulMaxDataLen   = pTCPIPConnInfo->ulPacketSize
                                      - (2 * RTE_HEADER_SIZE);
     }
@@ -841,10 +866,9 @@ static ULONG sql23c_get_server_addr ( PTCPIP_CONNECT_INFO_REC  pTCPIPConnInfo,
       {
           strcpy  ( pErrText, ERRMSG_MISSING_TCP_SERVICE_ENTRY );
           strncat ( pErrText, ServiceString,
-                    sizeof(pErrText) -
-                    strlen(pErrText) -
-                    sizeof(ServiceString) - 1 );
-          pErrText[sizeof(pErrText)] = '\0';
+                    sizeof(tsp00_ErrTextc) -
+                    strlen(pErrText) - 1 );
+          pErrText[sizeof(tsp00_ErrTextc) - 1] = '\0';
 
           MSGD (( ERR_SERV_BY_NAME_ERROR, ServiceString ));
 
