@@ -28,8 +28,7 @@
 \endif
 */
 
-
-
+#include "RunTime/System/RTESys_MemoryBarrier.h"
 
 /*
  * INCLUDE FILES
@@ -478,8 +477,9 @@ ULONG  sql33c_connect ( PCONNECT_PARAM_REC      pConnParam,
       {
       // --- signal to the server that we lost the connection!
       pShmConnInfo->pCSHeader->ulClientCommState  = commErrCrash_esp01;
-      pShmConnInfo->pCSHeader->ulClientFlag       = 1;
       pShmConnInfo->pCSHeader->ulServerFlag       = 2;
+      RTESys_WriteMemoryBarrier();
+      pShmConnInfo->pCSHeader->ulClientFlag       = 1;
       }
 
     sql33c_connect_cleanup  ( pShmConnInfo );
@@ -677,8 +677,9 @@ ULONG  sql33c_connect ( PCONNECT_PARAM_REC      pConnParam,
        {
        // --- signal to the server that we lost the connection!
        pShmConnInfo->pCSHeader->ulClientCommState  = commErrCrash_esp01;
-       pShmConnInfo->pCSHeader->ulClientFlag       = 1;
        pShmConnInfo->pCSHeader->ulServerFlag       = 2;
+       RTESys_WriteMemoryBarrier();
+       pShmConnInfo->pCSHeader->ulClientFlag       = 1;
        }
 
      sql33c_connect_cleanup  ( pShmConnInfo );
@@ -789,10 +790,18 @@ ULONG  sql33c_request ( PSHM_CONNECT_INFO_REC   pShmConnInfo,
   //
   pCSHeader->ulClientCommState   = commErrOk_esp01;
   pCSHeader->ulCurrentPacket     = pShmConnInfo->ulCurrentPacket;
-  pCSHeader->ulClientFlag        = 1;
 
   if ( pShmConnInfo->pulCommFlag != NULL )
+  {
+    pCSHeader->ulClientFlag      = 1;
+    RTESys_WriteMemoryBarrier();
     *pShmConnInfo->pulCommFlag   = 1;
+  }
+  else
+  {
+    RTESys_WriteMemoryBarrier();
+    pCSHeader->ulClientFlag      = 1;
+  }
 
   // ---  Wake up the UKT?
   /* Kernel may have corrupted the Communication Segment Header. Therefore we do a range check here. */
@@ -868,6 +877,8 @@ ULONG  sql33c_request ( PSHM_CONNECT_INFO_REC   pShmConnInfo,
      }
    while ( pCSHeader->ulServerFlag != 1 );
 
+   RTESys_ReadMemoryBarrier();
+
    // --- signal that we received the data
    pCSHeader->ulClientFlag = 0;
    pCSHeader->ulServerFlag = 2;
@@ -906,10 +917,18 @@ ULONG  sql33c_request ( PSHM_CONNECT_INFO_REC   pShmConnInfo,
       //
       pCSHeader->pidClientOrPeerPID = (PID)UNDEF;
       pCSHeader->ulClientCommState  = commErrSendLineDown_esp01;
-      pCSHeader->ulClientFlag       = 1;
 
       if ( pShmConnInfo->pulCommFlag != NULL )
+      {
+        pCSHeader->ulClientFlag    = 1;
+        RTESys_WriteMemoryBarrier();
         *pShmConnInfo->pulCommFlag = 1;
+      }
+      else
+      {
+        RTESys_WriteMemoryBarrier();
+        pCSHeader->ulClientFlag    = 1;
+      }
 
       if ( pShmConnInfo->phServerSems[pCSHeader->ulSemaNo] != INVALID_HANDLE_VALUE )
         {
@@ -1022,6 +1041,8 @@ ULONG  sql33c_request ( PSHM_CONNECT_INFO_REC   pShmConnInfo,
        }
      }
 
+   RTESys_ReadMemoryBarrier();
+
    // --- signal that we have received
    pCSHeader->ulServerFlag = 2;
 
@@ -1191,7 +1212,10 @@ ULONG sql33c_release ( PSHM_CONNECT_INFO_REC   pShmConnInfo,
      pCSHeader->pidClientOrPeerPID = (PID)UNDEF;
 
      if ( pShmConnInfo->pulCommFlag != NULL )
+     {
+       RTESys_WriteMemoryBarrier();
        *pShmConnInfo->pulCommFlag  = 1;
+     }
 
 	 /* Kernel may have corrupted the Communication Segment Header. Therefore we do a range check here. */
 	 ulSemaNo = pCSHeader->ulSemaNo;
@@ -1283,6 +1307,8 @@ static ULONG sql33c_connect_wait ( HANDLE                hClientSem,
     pTaskCtrl->ulCOMTimeOut = ulTimeOut + kgs.ulCurrTime;
     pTaskCtrl->fTimeOut     = FALSE;
 
+    {
+        int gotCommuniction = 0;
     do
       {
       sql73k_COM_enqu ( pUKT, pTaskCtrl );
@@ -1305,10 +1331,16 @@ static ULONG sql33c_connect_wait ( HANDLE                hClientSem,
         DBGOUT;
         return ( commErrTimeout_esp01 );
         }
+        if ( pCSHeader->ulServerFlag == 1 )
+        {
+            gotCommuniction = 1;
+            RTESys_ReadMemoryBarrier();
+        }
       }
-    while (( pCSHeader->ulServerFlag       != 1 )          &&
+    while (( !gotConnection ) &&
            ( pCSHeader->pidClientOrPeerPID != (PID)UNDEF ) &&
            ( pCSHeader->ulServerCommState  == commErrOk_esp01 ));
+    }
   #else
    if ( ulTimeOut != (ULONG)UNDEF )
      ulTimeOut *= 1000;
