@@ -551,7 +551,7 @@ LONG sql40c_set_tcpip_dll_path ( PSZ  pszTCPIPDLLName )
   RegistryEntries[0].ulValueSize  = (ULONG)strlen(pszTCPIPDLLName);
   RegistryEntries[0].ulValueType  = REG_EXPAND_SZ;
 
-  rc = sql50_reg_put_applic_values ( NULL, HKEY_LOCAL_MACHINE,
+  rc = sql50_reg_put_applic_values ( NULL, NULL, HKEY_LOCAL_MACHINE,
                                      REG_SK_DLLS, NULL, 1, RegistryEntries );
 
   DBGOUT;
