@@ -159,6 +159,7 @@ typedef struct  { LSA_HANDLE   Hdl;
 /*
  * LOCAL FUNCTION PROTOTYPES
  */
+static LONG   sql49c_get_own_textual_SID (PSZ *                   ppszTextualSID);
 
 static LONG   sql49c_lookup_account_name( PSZ                     pszNodeName,
                                           PSZ                     pszAccountName,
@@ -881,6 +882,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
                              if( pCreatorSid)       FreeSid  ( pCreatorSid);       \
                              if( pTDacl )           FREE_MEM ( pTDacl );           \
                              if( pTU )              FREE_MEM ( pTU );              \
+                             if (pOtherUserSid)     FREE_MEM (pOtherUserSid);      \
                              if( *ppSD )            FREE_MEM ( *ppSD );            \
                              if( plOperSid )        FREE_MEM ( plOperSid );        \
                              if( pDomOperSid )      FREE_MEM ( pDomOperSid );      \
@@ -902,6 +904,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
   SID_IDENTIFIER_AUTHORITY  BackupSidAuthority  = SECURITY_NT_AUTHORITY;
   SID_IDENTIFIER_AUTHORITY  WorldSidAuthority   = SECURITY_WORLD_SID_AUTHORITY;
   SID_IDENTIFIER_AUTHORITY  CreatorSidAuthority = SECURITY_CREATOR_SID_AUTHORITY;
+  PSID                      pOtherUserSid       = NULL;
   PSID                      pAdminsSid          = NULL;
   PSID                      plOperSid           = NULL;
   PSID                      pDomOperSid         = NULL;
@@ -936,8 +939,8 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
     }
 
   // --- no access right?
-  if ( !pAccess->User.ulAccess     && !pAccess->Creator.ulAccess &&
-       !pAccess->Admin.ulAccess    && !pAccess->System.ulAccess  &&
+  if ( !pAccess->User.ulAccess     && !pAccess->OtherUser.ulAccess  && !pAccess->Creator.ulAccess &&
+       !pAccess->Admin.ulAccess    && !pAccess->System.ulAccess     &&
        !pAccess->SAPDBOp.ulAccess  && !pAccess->BackupOp.ulAccess )
     {
     // --- check world access?
@@ -993,14 +996,53 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
   //     (SID) associated with the process/thread access token.
   //
   //
-  rc = sql49c_get_token_information ( &pTU, NULL, &pTDacl);
 
-  if ( rc != NO_ERROR )
+    if (pAccess->ignoreOwnUser && NULL == pAccess->pUseOtherUser)
     {
-    DBGOUT;
-    return ( rc );
+        DBG1  (( MF__, ERRMSG_SEC_NO_USER));
+        MSGCD  (( ERR_SEC_NO_USER ));
+        CRASH_CLEANUP( 1, 0 );
+        DBGOUT;
+        return -1;
     }
 
+    if (!pAccess->ignoreOwnUser)
+    {
+        rc = sql49c_get_token_information ( &pTU, NULL, &pTDacl);
+    }
+    else
+    {
+        rc = sql49c_get_token_information ( NULL, NULL, &pTDacl);
+    }
+
+    if ( rc != NO_ERROR )
+    {
+        DBGOUT;
+        CRASH_CLEANUP( 2, rc );
+        return ( rc );
+    }
+
+
+    if (NULL != pAccess->pUseOtherUser)
+    {
+       SID_NAME_USE        snuType;
+
+       rc = sql49c_lookup_account_name (NULL, pAccess->pUseOtherUser,
+                                        &pOtherUserSid, NULL, &snuType );
+  
+       if ((rc == NO_ERROR) && (snuType != SidTypeUser))
+       {
+           rc = ERROR_NONE_MAPPED;
+       }
+
+       if (NO_ERROR != rc)
+       {
+            DBGOUT;
+            CRASH_CLEANUP( 3, rc );
+            return ( rc );
+       }
+    }
+    
   //
   //
   // --- get the sid of the database administrator groups
@@ -1010,7 +1052,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
 
   if (( rc != NO_ERROR ) && ( rc != ERROR_NONE_MAPPED ))
     {
-    CRASH_CLEANUP( 1, rc );
+    CRASH_CLEANUP( 4, rc );
 
     DBGOUT;
     return ( rc );
@@ -1032,7 +1074,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
                            sizeof(AclSizeStruct), AclSizeInformation))
       {
       rc = GetLastError ();
-      CRASH_CLEANUP( 2, rc );
+      CRASH_CLEANUP( 5, rc );
 
       DBGOUT;
       return ( rc );
@@ -1064,7 +1106,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
     {
     rc = GetLastError();
 
-    CRASH_CLEANUP( 3, rc );
+    CRASH_CLEANUP( 6, rc );
     DBGOUT;
     return( rc );
     }
@@ -1076,10 +1118,15 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
   //     the current users SID and the new DACL.
   //
   //
-  if ( pAccess->fDescOwnerIsAdmin )
+
+  if ( pAccess->fDescOwnerIsAdmin || pAccess->ignoreOwnUser)
+  {
     pTmpDescOwnerSid = pAdminsSid;
+  }
   else
+  {
     pTmpDescOwnerSid = pTU->User.Sid;
+  }
 
   ulDescOwnerSidSize  = ALIGN(GetLengthSid(pTmpDescOwnerSid), ALIGNMENT_VALUE);
   ulMaxDaclSize       = sizeof(ACL);
@@ -1087,7 +1134,16 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
   if ( pAccess->fAddDefaultDACL )
    ulMaxDaclSize     += AclSizeStruct.AclBytesInUse;
 
-  ulMaxDaclSize += __sql49c_get_ace_sizes(&pAccess->User,          pTU->User.Sid );
+  if (NULL != pAccess->pUseOtherUser)
+  {
+      ulMaxDaclSize += __sql49c_get_ace_sizes (&pAccess->OtherUser, pOtherUserSid );
+  }
+
+  if (!pAccess->ignoreOwnUser)
+  {
+      ulMaxDaclSize += __sql49c_get_ace_sizes (&pAccess->User, pTU->User.Sid );
+  }
+
   ulMaxDaclSize += __sql49c_get_ace_sizes(&pAccess->Creator,       pCreatorSid );
   ulMaxDaclSize += __sql49c_get_ace_sizes(&pAccess->System,        pSystemSid );
   ulMaxDaclSize += __sql49c_get_ace_sizes(&pAccess->Admin,         pAdminsSid );
@@ -1101,7 +1157,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
 
   if ( rc != NO_ERROR )
     {
-    CRASH_CLEANUP( 4, rc );
+    CRASH_CLEANUP( 7, rc );
     DBGOUT;
     return ( rc );
     }
@@ -1119,7 +1175,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
   if ( !InitializeSecurityDescriptor( *ppSD, SECURITY_DESCRIPTOR_REVISION1 ))
     {
     rc = GetLastError();
-    CRASH_CLEANUP( 5, rc );
+    CRASH_CLEANUP( 8, rc );
 
     DBGOUT;
     return( rc );
@@ -1128,16 +1184,16 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
   if (!CopySid ( ulDescOwnerSidSize, pDescOwnerSid, pTmpDescOwnerSid ))
     {
     rc = GetLastError();
-    CRASH_CLEANUP( 6, rc );
+    CRASH_CLEANUP( 9, rc );
 
     DBGOUT;
     return( rc );
     }
 
-  if (!SetSecurityDescriptorOwner(*ppSD, pDescOwnerSid, 0))
+    if (!SetSecurityDescriptorOwner(*ppSD, pDescOwnerSid, 0))
     {
     rc = GetLastError();
-    CRASH_CLEANUP( 7, rc );
+    CRASH_CLEANUP( 10, rc );
 
     DBGOUT;
     return( rc );
@@ -1151,7 +1207,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
   if (!InitializeAcl ( pNewDacl, ulMaxDaclSize, ACL_REVISION2) )
     {
     rc = GetLastError ();
-    CRASH_CLEANUP( 8, rc );
+    CRASH_CLEANUP( 11, rc );
 
     DBGOUT;
     return ( rc );
@@ -1164,21 +1220,30 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
                  AclSizeStruct.AclBytesInUse - sizeof(ACL))))
       {
       rc = GetLastError ();
-      CRASH_CLEANUP( 9, rc );
+      CRASH_CLEANUP( 12, rc );
 
       DBGOUT;
       return ( rc );
       }
     }
 
-  ADD_ACES( pNewDacl, pTU->User.Sid, &pAccess->User,             101 );
-  ADD_ACES( pNewDacl, pCreatorSid,   &pAccess->Creator,          102 );
-  ADD_ACES( pNewDacl, pAdminsSid,    &pAccess->Admin,            103 );
-  ADD_ACES( pNewDacl, pSystemSid,    &pAccess->System,           104 );
-  ADD_ACES( pNewDacl, pWorldSid,     &pAccess->World,            105 );
-  ADD_ACES( pNewDacl, pBackupSid,    &pAccess->BackupOp,         105 );
-  ADD_ACES( pNewDacl, plOperSid,     &pAccess->SAPDBOp,          106 );
-  ADD_ACES( pNewDacl, pDomOperSid,   &pAccess->DomSAPDBOp,       107 );
+  if (NULL != pAccess->pUseOtherUser)
+  {
+        ADD_ACES( pNewDacl, pOtherUserSid, &pAccess->OtherUser, 101);
+  }
+
+  if (!pAccess->ignoreOwnUser)
+  {
+        ADD_ACES( pNewDacl, pTU->User.Sid, &pAccess->User, 102);
+  }
+
+  ADD_ACES( pNewDacl, pCreatorSid,   &pAccess->Creator,          103 );
+  ADD_ACES( pNewDacl, pAdminsSid,    &pAccess->Admin,            104 );
+  ADD_ACES( pNewDacl, pSystemSid,    &pAccess->System,           105 );
+  ADD_ACES( pNewDacl, pWorldSid,     &pAccess->World,            106 );
+  ADD_ACES( pNewDacl, pBackupSid,    &pAccess->BackupOp,         107 );
+  ADD_ACES( pNewDacl, plOperSid,     &pAccess->SAPDBOp,          108 );
+  ADD_ACES( pNewDacl, pDomOperSid,   &pAccess->DomSAPDBOp,       109 );
 
   //
   //
@@ -1188,7 +1253,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
   if ( !SetSecurityDescriptorDacl ( *ppSD, TRUE, pNewDacl, FALSE ) )
     {
     rc = GetLastError ();
-    CRASH_CLEANUP( 10, rc );
+    CRASH_CLEANUP( 13, rc );
 
     DBGOUT;
     return ( rc );
@@ -1198,7 +1263,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
   if ( !IsValidSecurityDescriptor ( *ppSD ))
     {
     rc = GetLastError ();
-    CRASH_CLEANUP( 11, rc );
+    CRASH_CLEANUP( 14, rc );
 
     DBGOUT;
     return ( rc );
@@ -1212,6 +1277,7 @@ LONG sql49c_alloc_and_init_SD ( PACCESS_RIGHTS_REC     pAccess,
   if ( plOperSid )        FREE_MEM ( plOperSid );
   if ( pDomOperSid )      FREE_MEM ( pDomOperSid );
   if ( pTU )              FREE_MEM ( pTU );
+  if ( pOtherUserSid )    FREE_MEM ( pOtherUserSid );
   if ( pTDacl )           FREE_MEM ( pTDacl );
 
   DBGOUT;
@@ -1948,56 +2014,51 @@ LONG sql49c_user_privilege_check ( BOOL *pfIsOk )
    }
 
 /*------------------------------*/
-
-LONG sql49c_get_own_textual_SID ( PSZ *ppszTextualSID )
-   {
+LONG sql49c_get_user_textual_SID (SAPDB_Char *   accountName,
+                                  PSZ *          ppszTextualSID)
+ {
    #undef  MF__
-   #define MF__ MOD__"sql49c_get_own_textual_SID"
-   PTOKEN_USER               pTU = NULL;
-   LONG                      rc  = NO_ERROR;
+   #define MF__ MOD__"sql49c_get_user_textual_SID"
+   LONG                rc  = NO_ERROR;
 
    DBGIN;
 
-   //
-   //
-   // --- check the platform id first
-   //
-   //
-   if ( sql02_get_platform_id() != VER_PLATFORM_WIN32_NT )
-     {
-     *ppszTextualSID = NULL;
-     DBGOUT;
-     return ( NO_ERROR );
-     }
+   if (NULL == accountName || '\0' == accountName[0])
+   {
+        rc = sql49c_get_own_textual_SID (ppszTextualSID);
+   }
+   else
+   {
 
-   //
-   // --- get the default DACL and and the current users security identifier
-   //     (SID) associated with the process/thread access token.
-   //
-   rc = sql49c_get_token_information ( &pTU, NULL, NULL);
+       PSID                pAccountSid       = NULL;
+       SID_NAME_USE        snuType;
 
-   if ( rc != NO_ERROR )
-     {
-     *ppszTextualSID = NULL;
-     DBGOUT;
-     return ( rc );
-     }
+       rc = sql49c_lookup_account_name (NULL, accountName,
+                                        &pAccountSid, NULL, &snuType );
+  
+       if ((rc == NO_ERROR) && (snuType != SidTypeUser))
+       {
+           rc = ERROR_NONE_MAPPED;
+       }
+       else if (NO_ERROR == rc)
+       {
+           rc =  sql49c_get_textual_SID(pAccountSid, ppszTextualSID );
 
-   rc =  sql49c_get_textual_SID( pTU->User.Sid, ppszTextualSID );
-
-   if ( pTU ) FREE_MEM ( pTU );
-
-
-   if ( rc != NO_ERROR )
-     {
-     *ppszTextualSID = NULL;
-     DBGOUT;
-     return( rc );
-     }
+           if ( rc != NO_ERROR )
+           {
+               *ppszTextualSID = NULL;
+           }
+       }
+  
+       if ( pAccountSid != NULL )
+           FREE_MEM ( pAccountSid );
+   }
 
    DBGOUT;
    return( rc );
-   }
+
+ }
+
 
 #endif
 
@@ -2008,6 +2069,47 @@ LONG sql49c_get_own_textual_SID ( PSZ *ppszTextualSID )
 
 #if defined(_WIN32)
 
+ static LONG sql49c_get_own_textual_SID (PSZ *ppszTextualSID )
+ {
+#undef  MF__
+#define MF__ MOD__"sql49c_get_own_textual_SID"
+     PTOKEN_USER               pTU = NULL;
+     LONG                      rc  = NO_ERROR;
+
+     DBGIN;
+
+
+     //
+     // --- get the default DACL and and the current users security identifier
+     //     (SID) associated with the process/thread access token.
+     //
+     rc = sql49c_get_token_information ( &pTU, NULL, NULL);
+
+     if ( rc != NO_ERROR )
+     {
+         *ppszTextualSID = NULL;
+         DBGOUT;
+         return ( rc );
+     }
+
+     rc =  sql49c_get_textual_SID( pTU->User.Sid, ppszTextualSID );
+
+     if ( pTU ) FREE_MEM ( pTU );
+
+
+     if ( rc != NO_ERROR )
+     {
+         *ppszTextualSID = NULL;
+         DBGOUT;
+         return( rc );
+     }
+
+     DBGOUT;
+     return( rc );
+ }
+
+/*------------------------------*/
+
 static LONG sql49c_lookup_account_name ( PSZ           pszNodeName,
                                          PSZ           pszAccountName,
                                          PSID          *ppSid,
