@@ -308,6 +308,27 @@ LONG sql50_reg_del_applic_key ( PSZ    pszNodeName,
   DBGOUT;
   return ( rc );
   }
+/*------------------------------*/
+
+LONG sql50_RegistryDeleteApplSupKey (HKEY            hKey,
+                                     SAPDB_Char *    pszSubKey)
+{
+#undef  MF__
+#define MF__ MOD__"sql50_RegistryDeleteApplSupKey"
+    APIRET    rc  = NO_ERROR;
+    PATHNAME  szKey;
+
+    DBGIN;
+
+    BUILD_APPLIC_SUBKEY (szKey);
+    ADD_SUBKEY (szKey, pszSubKey);
+
+    /* Remove Registry Subkey */
+    rc = RegDeleteKey (hKey, szKey);
+
+    DBGOUT;
+    return ( rc );
+}
 
 /*------------------------------*/
 #endif  /* ENUMINST_ONLY         PTS 1107355 FF 18-AUG-2000 */
@@ -372,12 +393,13 @@ LONG sql50_reg_get_oldapplic_values ( PSZ             pszNodeName,
 
 /*#if !defined(LIBOMS)   PTS 1107355 FF 18-Aug-2000 */
 
-LONG sql50_reg_put_applic_values ( PSZ             pszNodeName,
-                                   HKEY            hKey,
-                                   PSZ             pszSubKey,
-                                   PSZ             pszPrivateSubKey,
-                                   ULONG           ulNumOfEntries,
-                                   PREG_ENTRY_REC  pEntyArray )
+LONG sql50_reg_put_applic_values ( PSZ              pszNodeName,
+                                   SAPDB_Char *     accountName,
+                                   HKEY             hKey,
+                                   PSZ              pszSubKey,
+                                   PSZ              pszPrivateSubKey,
+                                   ULONG            ulNumOfEntries,
+                                   PREG_ENTRY_REC   pEntyArray )
   {
   #undef  MF__
   #define MF__ MOD__"sql50_reg_put_applic_values"
@@ -452,13 +474,25 @@ LONG sql50_reg_put_applic_values ( PSZ             pszNodeName,
     memset ( &Access, 0, sizeof (Access) );  // - this must be done first!
     Access.fDescOwnerIsAdmin  = FALSE;
     Access.fAddDefaultDACL    = FALSE;
-    Access.User.ulAccess      = KEY_ALL_ACCESS;
-    Access.User.ulAceFlags    = CONTAINER_INHERIT_ACE;
+
+    if (NULL != accountName)
+    {
+        Access.ignoreOwnUser  = TRUE;
+        Access.pUseOtherUser  = accountName; 
+        Access.OtherUser.ulAccess      = KEY_ALL_ACCESS;
+        Access.OtherUser.ulAceFlags    = CONTAINER_INHERIT_ACE;
+    }
+    else
+    {
+        Access.User.ulAccess      = KEY_ALL_ACCESS;
+        Access.User.ulAceFlags    = CONTAINER_INHERIT_ACE;
+    }
+
     Access.Admin.ulAccess     = KEY_ENUMERATE_SUB_KEYS |
-                                DELETE                 |
-                                READ_CONTROL           |
                                 KEY_QUERY_VALUE        |
-                                KEY_SET_VALUE;
+                                KEY_NOTIFY             | 
+                                KEY_SET_VALUE          |
+                                DELETE;                 
     Access.Admin.ulAceFlags   = CONTAINER_INHERIT_ACE;
     Access.System.ulAccess    = KEY_ALL_ACCESS;
     Access.System.ulAceFlags  = CONTAINER_INHERIT_ACE;
