@@ -201,26 +201,70 @@ VOID  sqlabort (VOID)
 
 /*------------------------------*/
 
+
 VOID vdump_rte ( VOID )
 {
   char *RTEDumpName = XPARAM(szRTEDumpFilename) ;
 
   if ( *RTEDumpName != '\0' )
   {
-    int             rc ;
-    tsp01_RteError  RteError ;
-    tsp00_Pathc     szPgmPath;
-    char            szCmdBuf[sizeof(tsp00_Pathc) + sizeof(SQL_DBNAMEC) + 
-                             sizeof(VF_FILENAMEC) + 50 ];
+      STARTUPINFO         StartupInfo;
+      PROCESS_INFORMATION ProcessInfo;
+      tsp01_RteError      RteError;
+      tsp00_Pathc         szPgmPath;
+      DWORD               rc = NO_ERROR;
+      HANDLE              hFile   = INVALID_HANDLE_VALUE;
+      char                szCmdBuf[sizeof(tsp00_Pathc) + sizeof(SQL_DBNAMEC) + 
+                                   sizeof(VF_FILENAMEC) + 30 ];
 
+      if ( sqlGetDbrootPgmPath (szPgmPath, TERM_WITH_DELIMITER_EO01, &RteError) )
+      {
+          kgs.ulDebugLevel = 3;
+
+          hFile = CreateFile( RTEDumpName,
+                              GENERIC_WRITE | GENERIC_READ, 
+                              FILE_SHARE_READ | FILE_SHARE_WRITE, &kgs.FileSA,
+                              CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL , NULL );
+
+          if ( hFile == INVALID_HANDLE_VALUE )
+              rc = GetLastError();
+          else
+          {
+              GetStartupInfo (&StartupInfo);
+              memset ( &ProcessInfo, 0, sizeof(ProcessInfo) );
+
+              sprintf(szCmdBuf, "\"%scons.exe\" \"%s\" sh all", szPgmPath, kgs.szServerDB );
+
+              SetHandleInformation(hFile, HANDLE_FLAG_INHERIT, TRUE);
+
+              StartupInfo.lpTitle    = szCmdBuf;
+              StartupInfo.dwFlags    = STARTF_USESTDHANDLES;
+              StartupInfo.hStdOutput = hFile;
+              StartupInfo.hStdError  = hFile;
+
+              if ( false ==  CreateProcess ( 0, szCmdBuf, 0, 0, TRUE, CREATE_NO_WINDOW,
+                                             0, 0, &StartupInfo, &ProcessInfo) )
+              {
+                  rc = GetLastError();
+              }
+              else
+              {
+                  WaitForSingleObject(ProcessInfo.hProcess, 60000);
+
+                  CloseHandle( ProcessInfo.hProcess );
+                  CloseHandle( ProcessInfo.hThread );
+              }
+              CloseHandle( hFile );
+          }
+      }
+
+      if ( NO_ERROR != rc )
+      {
+          sprintf( szCmdBuf, "%scons.exe %s sh all > %s",
+                   szPgmPath, kgs.szServerDB, RTEDumpName);
+          system (szCmdBuf);
+      }
 
-    if ( sqlGetDbrootPgmPath (szPgmPath, TERM_WITH_DELIMITER_EO01, &RteError) )
-    {
-      kgs.ulDebugLevel=3 ;
-      sprintf(szCmdBuf, "%scons.exe %s sh all > %s", 
-                         szPgmPath, kgs.szServerDB, RTEDumpName);
-      rc = system(szCmdBuf) ; 
-    }
   }
 }
 
@@ -988,7 +1032,6 @@ static VOID  sql50k_dump_ak ( VOID )
  }  
 #endif
 
-
 /*===========================================================================*
  *  END OF CODE                                                              *
  *===========================================================================*/
