@@ -694,6 +694,8 @@ BOOL  sql51k_auto_shutdown ( VOID )
     PDLQ_REC      pRequest;
     static  BOOL  fAutoShutdownInProgress = FALSE;
 
+    BOOLEAN volatile *pGracefully;
+
     DBGPAS;
 
     if (  pkss 
@@ -701,8 +703,10 @@ BOOL  sql51k_auto_shutdown ( VOID )
        && ( fAutoShutdownInProgress == FALSE ) 
        && ( *kgs.pDBState           == SERVER_WARM ) )
     {
+        pGracefully = (BOOLEAN volatile *)&(pkss->gracefully);
         fAutoShutdownInProgress = TRUE;
-        if ( kgs.fWinShutdownPending )
+        if ( kgs.fWinShutdownPending 
+          || !RTE_SpawnRestartShutdown((void *)0) )
         {
             kgs.ulTerminationTimeOut    = sql50k_get_termination_timeout() / 3;
 
@@ -710,31 +714,37 @@ BOOL  sql51k_auto_shutdown ( VOID )
             {
                 while (( kgs.pUT->pUserCommCtrl->ulConnected != 0 )            &&
                        ( kgs.pUT->TaskState                  != TSK_INACTIVE ) &&
-                       --kgs.ulTerminationTimeOut )
+                       --kgs.ulTerminationTimeOut &&
+                       *pGracefully )
                     SLEEP (1000);
             }
 
-            kgs.pUT->TaskState  = TSK_RUNNING;            
-
-            if ( sql88k_create_task_or_fiber ( kgs.pUT ) == NO_ERROR )
+            if (  *pGracefully )
             {
-                pRequest            = RESERVE_FREELIST_ELEM();
-                pRequest->ulReqType = REQ_INITOK;
-                sql74k_UKT_enqu_and_wake ( kgs.pUT->pUKT, kgs.pUT, pRequest );
+                kgs.pUT->TaskState  = TSK_RUNNING;            
 
-                kgs.ulTerminationTimeOut += (sql50k_get_termination_timeout() / 3) * 2;
+                if ( sql88k_create_task_or_fiber ( kgs.pUT ) == NO_ERROR )
+                {
+                    pRequest            = RESERVE_FREELIST_ELEM();
+                    pRequest->ulReqType = REQ_INITOK;
+                    sql74k_UKT_enqu_and_wake ( kgs.pUT->pUKT, kgs.pUT, pRequest );
+
+                    kgs.ulTerminationTimeOut += (sql50k_get_termination_timeout() / 3) * 2;
   
-                while ( !kgs.fWinShutdownCompleted &&  --kgs.ulTerminationTimeOut )
-                    SLEEP (1000);
+                    while ( !kgs.fWinShutdownCompleted
+                        &&  --kgs.ulTerminationTimeOut
+                        &&  *pGracefully )
+                        SLEEP (1000);
+                }
             }
         }
         else
         {
-            RTE_SpawnRestartShutdown((void *)0);
-
             kgs.ulTerminationTimeOut    = sql50k_get_termination_timeout();
 
-            while ( ( *kgs.pDBState == SERVER_WARM ) &&  --kgs.ulTerminationTimeOut )
+            while ( ( *kgs.pDBState == SERVER_WARM ) 
+                    &&  --kgs.ulTerminationTimeOut
+                    &&  *pGracefully )
                 SLEEP (1000);
         }
     }
