@@ -26,7 +26,7 @@
 */
 
 
-
+#include "RunTime/System/RTESys_MemoryBarrier.h"
 
 /*
  * INCLUDE FILES
@@ -44,7 +44,7 @@
 #include "geo67.h"
 #include "heo56.h"
 #include "gos74.h"
-
+#include "heo420.h"
 
 /*
  *  DEFINES
@@ -97,6 +97,9 @@ static ULONG sql53k_reply           ( PTASK_CTRL_REC        pTaskCtrl,
 
 static VOID sql53k_release          ( PTASK_CTRL_REC        pTaskCtrl );
 
+static VOID sql53k_handle_ping      ( PTASK_CTRL_REC        pTaskCtrl,
+                                      PUSER_TASK_COMM_CTRL  pUserCommCtrl );
+
 static VOID sql53k_remove_unrel_conn( PTASK_CTRL_REC        pTaskCtrl );
 
 static tsp00_Uint1 sql53k_new_swap_type ();
@@ -235,6 +238,19 @@ VOID  vconnect ( PROCESS_ID      pid,
       pUserCommCtrl->ulCommState = commErrCrash_esp01;
       *returncode                = commErrCrash_esp01;
 
+      sql53k_release ( pTaskCtrl );
+      }
+    else if ( pUserCommCtrl->ulServiceType == srvPing_esp01 )
+      {
+      MSGD (( INFO_CONN_TASK_WITH_APP, pTaskCtrl->ulTaskIndex,
+              pCSHeader->szClientOrPeerNode,
+              pCSHeader->pidClientOrPeerPID ));
+
+      sql53k_handle_ping ( pTaskCtrl, pUserCommCtrl );
+
+      // --- release connection
+      pUserCommCtrl->ulCommState = commErrNotOk_esp01;
+      *returncode                = (tsp01_CommErr)pUserCommCtrl->ulCommState;
       sql53k_release ( pTaskCtrl );
       }
     else
@@ -327,7 +343,6 @@ VOID  vreceive ( PROCESS_ID     pid,
                                                  pCSHeader, timeout,
                                                  &pcRcvPacket, &ulRcvPacketLen);
 
-
   if  ( *returncode != commErrOk_esp01 )
     {
     pTaskCtrl->TaskState = TSK_RUNNING;
@@ -336,7 +351,6 @@ VOID  vreceive ( PROCESS_ID     pid,
     return;
     }
 
-
   *rcv_packet_ptr    = pcRcvPacket;
   *rcv_packet_length = (INT4) ulRcvPacketLen;
   *reply_packet_ptr  = (PCHAR)pUserCommCtrl->pReplyCommPacket->pDataPart;
@@ -561,36 +575,32 @@ VOID	vgetapplpid( PROCESS_ID         tid ,
 #undef  MF__
 #define MF__ MOD__"vgetapplpid"
 
-#ifdef DEVELOP_CHECKS
-	PUKT_CTRL_REC               pUKT      = THIS_UKT_CTRL;
-	PTASK_CTRL_REC              pTaskCtrl = pUKT->pCTask;
-#else
-	PTASK_CTRL_REC				pTaskCtrl = &kgs.pFirstTaskCtrl[tid - 1];
-#endif
+    if ( ( tid > 0 ) && ( ((ULONG)tid) <= kgs.ulNumOfTasks ) )
+    {
+        PTASK_CTRL_REC				pTaskCtrl = &kgs.pFirstTaskCtrl[tid - 1];
 
-	*ok = ((ULONG)tid <= kgs.ulNumOfTasks);
+        if ( pTaskCtrl->pUserCommCtrl )
+        {
+	 	    *appl_pid = pTaskCtrl->pUserCommCtrl->pidClientPID ;
 
-	if (*ok)
-	{
-		if ( pTaskCtrl->pUserCommCtrl )
-		{
-			*appl_pid = pTaskCtrl->pUserCommCtrl->pidClientPID ;
-
-			eo46CtoP (appl_node, 
-				pTaskCtrl->pUserCommCtrl->szClientNode[0] == '\0' 
-				? kgs.szServerNode  
-				: pTaskCtrl->pUserCommCtrl->szClientNode, sizeof(tsp00_NodeId) );
-		} 
-		else 
-		{
-			*appl_pid = -1;
-			eo46CtoP ( appl_node, kgs.szServerNode, sizeof(tsp00_NodeId) ); 
-		}
-
-		DBG3 (( MF__, "appl_node: %s, appl_pid %d\n", appl_node, *appl_pid ));
-	}
-
-	return;
+            eo46CtoP ( appl_node, 
+		               pTaskCtrl->pUserCommCtrl->szClientNode[0] == '\0' 
+ 				     ? kgs.szServerNode  
+				     : pTaskCtrl->pUserCommCtrl->szClientNode, sizeof(tsp00_NodeId) );
+            *ok = true;
+        }
+	    else 
+        {
+		    *appl_pid = -1;
+		    eo46CtoP ( appl_node, kgs.szServerNode, sizeof(tsp00_NodeId) ); 
+            *ok = false;
+        }
+    }
+    else
+    {
+        MSGD (( ERR_KERN_PARAM_PID_OUT_OF_RANGE, tid ));
+        ABORT();
+    }
 }
 
 /*------------------------------*/
@@ -663,7 +673,8 @@ BOOL sql53k_comm_cancel ( PTASK_CTRL_REC  pTaskCtrl,
       if ( ulCommState != commErrOk_esp01 )
       {
         // --- on error kill connection
-        pUserCommCtrl->ulGlobCommState            = ulCommState;
+        pUserCommCtrl->ulGlobCommState = ulCommState;
+        RTESys_WriteMemoryBarrier();
         kgs.pulCommFlags[pTaskCtrl->ulCommFlagNo] = 1;
       }
 
@@ -731,6 +742,7 @@ BOOL sql53k_comm_termination_cleanup ( PTASK_CTRL_REC  pTaskCtrl )
               
               pUserCommCtrl->ulGlobCommState            = ulCommState;
               pTaskCtrl->KernelCtrlRecord.to_cancel     = TRUE;
+              RTESys_WriteMemoryBarrier();
               kgs.pulCommFlags[pTaskCtrl->ulCommFlagNo] = 1;
               
               hClientSem                                = pUserCommCtrl->hClientSem;
@@ -1036,12 +1048,12 @@ static ULONG sql53k_receive ( PTASK_CTRL_REC        pTaskCtrl,
     // --- insert task into COM queue, where it waits for a packet
     sql73k_COM_enqu (pTaskCtrl->pUKT, pTaskCtrl);
 
-    
     GOTO_DISP( &pTaskCtrl->pUKT );
+    RTESys_ReadMemoryBarrier();
     }
-  while (( pCSHeader->ulClientFlag        != 1 )          &&
-         ( pUserCommCtrl->ulGlobCommState == commErrOk_esp01 )      &&
-         ( pCSHeader->pidClientOrPeerPID  != (PID)UNDEF ) &&
+  while (( pCSHeader->ulClientFlag        != 1 )               &&
+         ( pUserCommCtrl->ulGlobCommState == commErrOk_esp01 ) &&
+         ( pCSHeader->pidClientOrPeerPID  != (PID)UNDEF )      &&
          ( pTaskCtrl->fTimeOut            == FALSE )) ;
 
   sql74k_temp_move_lock ( pTaskCtrl, 1, false );
@@ -1241,6 +1253,7 @@ static ULONG sql53k_reply ( PTASK_CTRL_REC        pTaskCtrl,
   sql74k_temp_move_lock ( pTaskCtrl, 1, true );                  // so that, we should not move to other UKTs
 
   pCSHeader->ulServerCommState     = pUserCommCtrl->ulCommState;
+  RTESys_WriteMemoryBarrier();
   pCSHeader->ulServerFlag          = 1;
 
   // --- wake up client
@@ -1274,6 +1287,7 @@ static VOID sql53k_release ( PTASK_CTRL_REC    pTaskCtrl )
     {
     pCSHeader->ulServerCommState = pUserCommCtrl->ulCommState;
     pCSHeader->ulServerRef       = (ULONG)UNDEF;
+    RTESys_WriteMemoryBarrier();
     pCSHeader->ulServerFlag      = 1;
     }
 
@@ -1343,6 +1357,53 @@ static VOID sql53k_release ( PTASK_CTRL_REC    pTaskCtrl )
 
 /*------------------------------*/
 
+static VOID sql53k_handle_ping ( PTASK_CTRL_REC        pTaskCtrl,
+                                 PUSER_TASK_COMM_CTRL  pUserCommCtrl )
+{
+#   undef  MF__
+#   define MF__ MOD__"sql53k_handle_ping"
+    ULONG     ulCommState = commErrOk_esp01;
+
+    PCOMM_SEG_HEADER_REC  pCSHeader   = pUserCommCtrl->pCSHeader;
+    PCHAR                 pcRcvPacket;
+    ULONG                 ulRcvPacketLen;
+
+    do
+    {
+        ulCommState = sql53k_receive (pTaskCtrl, pUserCommCtrl, pCSHeader, 240, &pcRcvPacket, &ulRcvPacketLen);
+
+        if  ( ulCommState == commErrOk_esp01 )
+        {
+           if ( eo420ExtractPingPacket( (char*)pcRcvPacket, ulRcvPacketLen, 0, 0, 0 ) )
+           {
+               ULONG ulReplyLen;
+               if ( pCSHeader->ulMinReplySize )
+                   ulReplyLen = pUserCommCtrl->ulMaxDataLen - ALIGN ( ulRcvPacketLen, SQL_PACKET_ALIGNMENT);
+               else
+                   ulReplyLen = pUserCommCtrl->ulMaxDataLen;
+
+                ulReplyLen =  eo420FillPingPacket ( (char*) pUserCommCtrl->pReplyCommPacket->pDataPart,
+                                                    ulReplyLen,
+                                                    1,   // hops greater than 0 means that the final destination was reached
+                                                    kgs.szServerDB,
+                                                    XPARAM(szKernelVersion), 0 );
+
+                if ( ulReplyLen )
+                    ulCommState = sql53k_reply ( pTaskCtrl, pUserCommCtrl, (PCHAR)pUserCommCtrl->pReplyCommPacket->pDataPart, ulReplyLen );
+                else
+                    ulCommState = commErrNotOk_esp01;
+            }
+            else
+                ulCommState = commErrNotOk_esp01;
+        }
+        else
+            ulCommState = commErrNotOk_esp01;
+    } 
+    while( ulCommState == commErrOk_esp01  );
+}
+
+/*------------------------------*/
+
 static VOID sql53k_remove_unrel_conn ( PTASK_CTRL_REC  pTaskCtrl )
   {
   #undef  MF__
@@ -1371,7 +1432,6 @@ static VOID sql53k_remove_unrel_conn ( PTASK_CTRL_REC  pTaskCtrl )
     return;
     }
 
-  #if defined(_WIN32)
    if( WaitForSingleObject( pUserCommCtrl->hLocalClientProcess, 0) != WAIT_TIMEOUT )
      {
      WAIT_UNTIL_ALONE( &pTaskCtrl->lExclusive );
@@ -1379,6 +1439,7 @@ static VOID sql53k_remove_unrel_conn ( PTASK_CTRL_REC  pTaskCtrl )
      if ( ulConnected == pUserCommCtrl->ulConnected )
        {
        pUserCommCtrl->ulGlobCommState            = commErrCrash_esp01;
+       RTESys_WriteMemoryBarrier();
        kgs.pulCommFlags[pTaskCtrl->ulCommFlagNo] = 1;
        pTaskCtrl->KernelCtrlRecord.to_cancel     = TRUE;
        fWake                                     = TRUE;
@@ -1391,7 +1452,6 @@ static VOID sql53k_remove_unrel_conn ( PTASK_CTRL_REC  pTaskCtrl )
      MSGD (( INFO_UNREL_CONN_FOUND, pTaskCtrl->ulTaskIndex ));
      sql72k_wake_ukt ( pTaskCtrl->pUKT );
      }
-  #endif
 
   // --- wake task if the client has not done this correctly!
   WAIT_UNTIL_ALONE( &pTaskCtrl->lExclusive );
