@@ -49,10 +49,6 @@
 #define MOD__  "vos56kc: "
 #define MF__   MOD__"UNDEFINED"
 
-#if defined (SAPDB_SLOW) || defined (SAPDB_QUICK)
-# define MULTIPLE_SINGNAL_CHECK     1
-#endif
-
 /*
  *  MACROS
  */
@@ -495,13 +491,7 @@ VOID  vwait   ( tsp00_TaskId      pid,
               break;
     }
 
-#if defined (MULTIPLE_SINGNAL_CHECK)
-  WAIT_UNTIL_ALONE( &pTaskCtrl->lExclusive );
-  pTaskCtrl->lVSignalOk = 0;
-  CLEARLOCK( &pTaskCtrl->lExclusive );
-#else
-  pTaskCtrl->lVSignalOk = 0;
-#endif
+  pTaskCtrl->lVSignalOk   = 0;
   pTaskCtrl->lWaitForTask = 0 ;
 
   #ifdef DEVELOPMENT
@@ -535,39 +525,21 @@ VOID  vsignal    ( tsp00_TaskId      pid,
     ABORT();
     }
 
-#if defined (MULTIPLE_SINGNAL_CHECK)
-
-  WAIT_UNTIL_ALONE( &pReqTaskCtrl->lExclusive );
-
-  if ( pReqTaskCtrl->lVSignalOk )
-    {
-    CLEARLOCK( &pReqTaskCtrl->lExclusive );
-    MSGD (( ERR_MULTIPLE_VSIGNAL_CALL, pReqTaskCtrl->ulTaskIndex ));
-    ABORT(); 
-    }
-
-  if ( ok )
-    pReqTaskCtrl->lVSignalOk = 1;      // - means TRUE
-  else
-    pReqTaskCtrl->lVSignalOk = -1;     // - means FALSE
-
-  CLEARLOCK( &pReqTaskCtrl->lExclusive );
-
-#endif
-
-
   // ---  is the requested UKT eq. to the local UKT
   if ( pReqTaskCtrl->pUKT == pUKT )
     {
-#   if !defined (MULTIPLE_SINGNAL_CHECK)
+    if ( pReqTaskCtrl->lVSignalOk )
+        {
+        CLEARLOCK( &pReqTaskCtrl->lExclusive );
+        MSGD (( ERR_MULTIPLE_VSIGNAL_CALL, pReqTaskCtrl->ulTaskIndex ));
+        ABORT(); 
+        }
 
     if ( ok )
       pReqTaskCtrl->lVSignalOk = 1;      // - means TRUE
     else
       pReqTaskCtrl->lVSignalOk = -1;     // - means FALSE
 
-#   endif
-
     DBG4 (( MF__, "[T:0x%03u] vsignal request for local task [T:0x%03u]",
             pTaskCtrl->ulTaskIndex, pReqTaskCtrl->ulTaskIndex ));
 
@@ -817,15 +789,18 @@ static void sql56k_remote_vsignal ( PPTASK_CTRL_REC ppCTask,
 
   DBGPAS_T ((*ppCTask)->ulTaskIndex);
 
-# if !defined (MULTIPLE_SINGNAL_CHECK)
+  if ( (*ppCTask)->lVSignalOk )
+    {
+    CLEARLOCK( &(*ppCTask)->lVSignalOk );
+    MSGD (( ERR_MULTIPLE_VSIGNAL_CALL, (*ppCTask)->ulTaskIndex ));
+    ABORT(); 
+    }
 
   if ( ulOk != 0 )
     (*ppCTask)->lVSignalOk = 1;      // - means TRUE
   else
     (*ppCTask)->lVSignalOk = -1;     // - means FALSE
 
-# endif
-
   if ( (*ppCTask)->TaskState == TSK_VWAIT_EO00 )
     {
     DBG4 (( MF__, "[T:0x%03u] signal ok", (*ppCTask)->ulTaskIndex));
