@@ -2752,7 +2752,7 @@ STATIC LONG sql60_user_diag_opt ( BOOL *pfEnabled,
          RegistryEntries[0].pValue       = pszUserDiagName;
          RegistryEntries[0].ulValueSize  = (ULONG)strlen(pszUserDiagName);
          RegistryEntries[0].ulValueType  = REG_SZ;
-         rc = sql50_reg_put_applic_values ( NULL, HKEY_CURRENT_USER, "", "", 1, RegistryEntries );
+         rc = sql50_reg_put_applic_values ( NULL, NULL, HKEY_CURRENT_USER, "", "", 1, RegistryEntries );
      }
    }
 
