@@ -158,6 +158,11 @@ LONG sql70k_init_control_records (VOID)
 
   if (rc == NO_ERROR)
     {
+      if ( pCluInfo->fOneToOne && XPARAM(uLoadBalancingCheck) == 0 )
+      {
+          // disable load balancing permanently
+          XPARAM(fForbidLoadBalancing) = true;
+      }
     kgs.ulUKTBaseStackSize = pCluInfo->ulUKTBaseStackSize; // PTS 1106437
     kgs.ulNumOfUKTs        = pCluInfo->ulNumOfUKTs;
     kgs.NumOfUserUKTs      = pCluInfo->ulNumOfUserUKTs;
@@ -634,7 +639,15 @@ static LONG  sql70k_init_UKT_ctrls (teo70k_ClusterInfo  *pCluInfo)
                                   (XPARAM(lNoOfBackupDevs) + XPARAM(lMaxDevspaces));
     ulNumUToUQueueHeads         = kgs.ulNumOfUKTs;
    
-	  ulNumUToUQueueElemForActUKT = kgs.ulNumOfTasks + 1;
+    if ( XPARAM(fForbidLoadBalancing) )    
+    {
+
+    	ulNumUToUQueueElemForActUKT = pCluInfo->pThrdInfo[ulIdx].ulNumOfTasks + 1;
+    }
+    else
+    {
+    	ulNumUToUQueueElemForActUKT = kgs.ulNumOfTasks + 1;
+    }
     ulNumUToUQueueElem          = ulNumUToUQueueElemForActUKT * (kgs.ulNumOfUKTs - 1);
 
     //
@@ -781,8 +794,9 @@ static LONG  sql70k_init_task_ctrls (teo70k_ClusterInfo  *pCluInfo)
   kgs.pTI      = pCurrTaskCtrl++;
   pCurrUKTCtrl = kgs.pFirstUKTCtrl + pCluInfo->TI_Pos - 1;
   DBG3 ((MF__, " --- TI ------------------------------------"));
+  /* No own entry for task stack . Reuse entry for data writer task... */
   rc = sql70k_insert_tasks          (kgs.pTI, pCurrUKTCtrl, 1, &ulTaskIndex, TT_TI,
-                                     FIX_TI_STACK_SIZE_EO70K, NULL, NULL );
+                                     pCluInfo->VarTaskCntRec[VAR_TASK_DW_EO70K].lStackSize, NULL, NULL );
   //
   // --- ARCHIVELOGWRITER
   //
@@ -791,8 +805,9 @@ static LONG  sql70k_init_task_ctrls (teo70k_ClusterInfo  *pCluInfo)
     kgs.pAL      = pCurrTaskCtrl++;
     pCurrUKTCtrl = kgs.pFirstUKTCtrl + pCluInfo->AL_Pos - 1;
     DBG3 ((MF__, " --- AL ------------------------------------"));
+  /* No own entry for task stack . Reuse entry for data writer task... */
     rc = sql70k_insert_tasks          (kgs.pAL, pCurrUKTCtrl, 1, &ulTaskIndex, TT_AL,
-                                       FIX_AL_STACK_SIZE_EO70K, NULL, NULL );
+                                       pCluInfo->VarTaskCntRec[VAR_TASK_DW_EO70K].lStackSize, NULL, NULL );
     }
 
   //
@@ -803,8 +818,9 @@ static LONG  sql70k_init_task_ctrls (teo70k_ClusterInfo  *pCluInfo)
     kgs.pTW      = pCurrTaskCtrl++;
     pCurrUKTCtrl = kgs.pFirstUKTCtrl + pCluInfo->TW_Pos - 1;
     DBG3 ((MF__, " --- TW ------------------------------------"));
+  /* No own entry for task stack . Reuse entry for data writer task... */
     rc = sql70k_insert_tasks          (kgs.pTW, pCurrUKTCtrl, 1, &ulTaskIndex, TT_TW,
-                                       FIX_TW_STACK_SIZE_EO70K, NULL, NULL );
+                                       pCluInfo->VarTaskCntRec[VAR_TASK_DW_EO70K].lStackSize, NULL, NULL );
     }
 
   //
