@@ -762,8 +762,6 @@ _INLINE VOID __os74SearchRUN ( PUKT_CTRL_REC    pUKT,
 
       if ( pTempTaskCtrl == pMoveTask )
         {
-        if ( pTempTaskCtrl->PrioFlag ) pUKT->ulImmediateDispatch--;
-
         pTempTaskCtrl->QueLocation    = TASK_IS_IN_NON_QUEUE;
         pTempTaskCtrl->ulDynPrio      = 0;
 
@@ -771,19 +769,19 @@ _INLINE VOID __os74SearchRUN ( PUKT_CTRL_REC    pUKT,
 
         if ( (pUKT->ulRUNQueueLen = --ulRunQueueLen) )
           {
-          if ( ulRunQueueLen > uldx )
-          {
-            if ( uldx == 0 )
-              HighestPrioTask = pRUNQue[ulRunQueueLen].pTaskCtrl;
+            if ( ulRunQueueLen > uldx )
+            {
+              if ( uldx == 0 )
+                HighestPrioTask = pRUNQue[ulRunQueueLen].pTaskCtrl;
 
-            pRUNQue[uldx--] = pRUNQue [ulRunQueueLen];
-          }
-          else
-          {
-            pTempTaskCtrl = pRUNQue[--uldx].pTaskCtrl;
-            pTempRequest  = pRUNQue[uldx].pRequest;
-          }
-          continue;
+              pRUNQue[uldx--] = pRUNQue [ulRunQueueLen];
+            }
+            else
+            {
+              pTempTaskCtrl = pRUNQue[--uldx].pTaskCtrl;
+              pTempRequest  = pRUNQue[uldx].pRequest;
+            }
+            continue;
           }
         else
           {
@@ -1080,12 +1078,38 @@ _INLINE VOID __os74SearchUKT ( PUKT_CTRL_REC    pUKT,
           break;
 
         case REQ_NEW_MOVE_TASK_MEAS_INTERVAL_EO00:
-
           // --- skip old requests
           if ( pCurrReq->ReqArgs.NewTaskMoveMeasRequest.TaskMoveIntervalStartTime ==
                kgs.TaskMoveIntervalStartTimes[kgs.CurrTimeVar] )
           {
             tos00k_DlqNewTaskMoveMeasReqArgs *pReqArg;
+            ULONG                             uldx, ulRunQueueLen = pUKT->ulRUNQueueLen;
+            SAPDB_UInt8                       timeNow = RTESys_MicroSecTimer();
+            SAPDB_UInt8                       RunnableMicroSeconds;
+
+            for ( uldx = 0; uldx < ulRunQueueLen; uldx++ )
+            {
+                PTASK_CTRL_REC  pTempTaskCtrl = pUKT->RUNQue[uldx].pTaskCtrl;
+
+                // long term move lock set?
+                if ( pTempTaskCtrl->MoveLock.Type.LongTerm == 0 ) 
+                    continue;
+
+                RunnableMicroSeconds = timeNow - pTempTaskCtrl->TaskStateStat.runqueueEnqueueTime;
+
+                // accumulate the tasks time waititng in the run queue to be scheduled
+                pTempTaskCtrl->RunnableMicroSeconds[pUKT->CurrTimeVar] += RunnableMicroSeconds;
+
+                // Has the tasks a new runnable time maximum?
+                if ( pTempTaskCtrl->MaxRunnableMicroSeconds[pUKT->CurrTimeVar] < RunnableMicroSeconds )
+                    pTempTaskCtrl->MaxRunnableMicroSeconds[pUKT->CurrTimeVar] = RunnableMicroSeconds;
+
+                // set the UKTs runnable time values
+                pUKT->RunnableMicroSeconds[pUKT->CurrTimeVar] += RunnableMicroSeconds;
+
+                if ( pUKT->MaxRunnableMicroSeconds[pUKT->CurrTimeVar] < RunnableMicroSeconds )
+                    pUKT->MaxRunnableMicroSeconds[pUKT->CurrTimeVar] = RunnableMicroSeconds;
+            }
 
             pReqArg = &pCurrReq->ReqArgs.NewTaskMoveMeasRequest;
 
