@@ -110,10 +110,10 @@ FLAG_COMM_SEG                *pfcs = NULL;
  * LOCAL FUNCTION PROTOTYPES
  */
 LONG  sql81k_get_serverdb_dbroot            ( PSZ   pszServerDB );
-LONG  sql81k_startup                        ( PSZ   pszServerDB ,   SECURITY_ATTRIBUTES    *pDiagfileSA);
+LONG  sql81k_startup                        ( PSZ   pszServerDB );
 LONG  sql81k_create_kernel_threads          ( VOID );
 LONG  sql81k_create_UKTs                    ( VOID );
-LONG  sql81k_create_SAs                     ( PSECURITY_ATTRIBUTES pDiagfileSA );
+LONG  sql81k_create_SAs                     (  );
 LONG  sql81k_init_kernel_shared_segments    ( VOID );
 LONG  sql81k_set_max_num_of_file_handles    ( VOID );
 LONG  sql81k_set_rundir                     ( PSZ pszServerDB );
@@ -125,9 +125,9 @@ BOOL sql81k_continue_search_exception       ( VOID );
 BOOL sql81k_get_debug_break_entry           ( PSZ   pszServerDB );
 LONG sql81k_set_path_env                    ( VOID );
 
-LONG sql81_InitEventing                     ( SECURITY_ATTRIBUTES *pDiagfileSA );
+LONG sql81_InitEventing                     ( );
 bool sql81k_always_check_conn               ( VOID );
-LONG sql81k_open_diag_files                 ( PSZ pszServerDB, PSECURITY_ATTRIBUTES pDiagfileSA);
+LONG sql81k_open_diag_files                 ( PSZ pszServerDB );
 
 /*
  * ========================== GLOBAL FUNCTIONS ================================
@@ -147,7 +147,6 @@ LONG sql81k_kernel_main ( PSZ              pszServerDB,
   teo07_Thread          mainThread;
   tsp00_ErrTextc        threadErrtext;
   teo07_ThreadErr       threadOk;
-  SECURITY_ATTRIBUTES   DiagfileSA;
   DBGIN;
 
   __try   // --- add exit handler to a thread's chain of ex. handlers
@@ -196,10 +195,10 @@ LONG sql81k_kernel_main ( PSZ              pszServerDB,
           return(rc);
 
       // --- create security descriptors like 'kgs.AdminSA'
-      sql81k_create_SAs(&DiagfileSA);
+      sql81k_create_SAs();
         
       // --- Open diag files
-      sql81k_open_diag_files(pszServerDB,&DiagfileSA);
+      sql81k_open_diag_files(pszServerDB);
 
       if ( RTE_KernelMain(argc, argv) )
       {
@@ -255,7 +254,7 @@ LONG sql81k_kernel_main ( PSZ              pszServerDB,
           }
       }
       // --- Initalize function
-      if ((rc = sql81k_startup ( pszServerDB,   &DiagfileSA )) != NO_ERROR)
+      if ((rc = sql81k_startup ( pszServerDB )) != NO_ERROR)
       {
         *kgs.pDBState = SERVER_ABORT;
 
@@ -452,7 +451,7 @@ static void sql81PrintStartUpInformation ( SYSTEM_INFO           *SystemInfo)
 
 /*------------------------------*/
 
-static LONG sql81k_startup ( PSZ   pszServerDB ,   SECURITY_ATTRIBUTES    *pDiagfileSA)
+static LONG sql81k_startup ( PSZ   pszServerDB )
   {
   #undef  MF__
   #define MF__ MOD__"sql81k_startup"
@@ -589,7 +588,7 @@ static LONG sql81k_startup ( PSZ   pszServerDB ,   SECURITY_ATTRIBUTES    *pDiag
 
   // --- Open util diag files
   rc = sql60_open_util_diag_file ( XPARAM(szUtilityProtFile),
-                                   XPARAM(ulUtilityProtSize), pDiagfileSA );
+                                   XPARAM(ulUtilityProtSize), &kgs.FileSA );
   if (rc != NO_ERROR)
     return( rc );
 
@@ -610,20 +609,11 @@ static LONG sql81k_startup ( PSZ   pszServerDB ,   SECURITY_ATTRIBUTES    *pDiag
   if ((rc = sql81k_init_kernel_shared_segments ()) != NO_ERROR)
     return(rc);
 
-  rc = sql81_InitEventing( pDiagfileSA );
+  rc = sql81_InitEventing();
 
   if (rc != NO_ERROR)
     return( rc );
 
-  // --- pDiagfileSA no longer needed
-  if ( sql02_get_platform_id () == VER_PLATFORM_WIN32_NT )
-  {
-     rc = FREE_MEM( pDiagfileSA->lpSecurityDescriptor );
-
-     if (rc != NO_ERROR)
-       return( rc );
-  }
-
   // --- After initializing the control records we are able to
   //     store COORDINATOR's semaphore
   kgs.Coord.ThrdCtrlHeader.hevSem = hevTmpCoord;
@@ -1651,7 +1641,7 @@ static LONG sql81k_init_kernel_shared_segments ( VOID )
 
  /*------------------------------*/
 
- static LONG  sql81k_create_SAs ( PSECURITY_ATTRIBUTES pDiagfileSA )
+ static LONG  sql81k_create_SAs ()
    {
    #undef  MF__
    #define MF__ MOD__"sql81k_create_SAs"
@@ -1710,9 +1700,9 @@ static LONG sql81k_init_kernel_shared_segments ( VOID )
      }
 
 
-   pDiagfileSA->nLength              = sizeof(SECURITY_ATTRIBUTES);
-   pDiagfileSA->bInheritHandle       = FALSE;
-   pDiagfileSA->lpSecurityDescriptor = NULL;
+   kgs.FileSA.nLength              = sizeof(SECURITY_ATTRIBUTES);
+   kgs.FileSA.bInheritHandle       = FALSE;
+   kgs.FileSA.lpSecurityDescriptor = NULL;
 
    memset ( &Access, 0, sizeof (Access) );  // - this must be done first!
    Access.fDescOwnerIsAdmin       = FALSE;
@@ -1726,7 +1716,7 @@ static LONG sql81k_init_kernel_shared_segments ( VOID )
    Access.World.ulAccess          = 0;
    Access.BackupOp.ulAccess       = GENERIC_READ;
 
-   rc = sql49c_alloc_and_init_SD(&Access, &pDiagfileSA->lpSecurityDescriptor);
+   rc = sql49c_alloc_and_init_SD(&Access, &kgs.FileSA.lpSecurityDescriptor);
 
    if ( rc != NO_ERROR )
      {
@@ -1834,7 +1824,7 @@ static VOID _System sql81_InsertKeepAlive ( DWORD CycleTime )
 
 /*------------------------------*/
 
-static LONG sql81_InitEventing( SECURITY_ATTRIBUTES *pDiagfileSA)
+static LONG sql81_InitEventing()
 {
     LONG rc = NO_ERROR;
     tsp00_Int4 spaceNeeded = 0;
@@ -1844,7 +1834,7 @@ static LONG sql81_InitEventing( SECURITY_ATTRIBUTES *pDiagfileSA)
         MSGD (( INFO_EVENT_FILE, XPARAM(szEventFile), XPARAM(ulEventSize) ));
         rc = sql60_open_event_file( XPARAM(szEventFile),
                                     XPARAM(ulEventSize),
-                                    pDiagfileSA );
+                                    &kgs.FileSA );
         if (rc != NO_ERROR)
         {
             MSGD (( ERR_CANT_OPEN_FILE, XPARAM(szEventFile), rc ));
@@ -1928,7 +1918,7 @@ static LONG sql81_InitEventing( SECURITY_ATTRIBUTES *pDiagfileSA)
 
 /*------------------------------*/
 
-static LONG sql81k_open_diag_files(PSZ pszServerDB, PSECURITY_ATTRIBUTES pDiagfileSA)
+static LONG sql81k_open_diag_files(PSZ pszServerDB)
 {
   ULONG           ulDiagSize;
   tsp00_C256c     szDiagSize;
@@ -1939,11 +1929,11 @@ static LONG sql81k_open_diag_files(PSZ pszServerDB, PSECURITY_ATTRIBUTES pDiagfi
     return -1;
   ulDiagSize = atoi(szDiagSize);
 
-  rc = sql60_open_diag_file ( SAPDB_DIAG_FILE_NAME, ulDiagSize, pDiagfileSA );
+  rc = sql60_open_diag_file ( SAPDB_DIAG_FILE_NAME, ulDiagSize, &kgs.FileSA );
   if(0 != rc)
     return rc;
 
-  rc = sql60_open_error_diag_file( SAPDB_ERROR_FILE_NAME, pDiagfileSA );
+  rc = sql60_open_error_diag_file( SAPDB_ERROR_FILE_NAME, &kgs.FileSA );
 
   return rc;
 }
