@@ -286,9 +286,16 @@ APIRET sql88k_create_task_or_fiber ( PTASK_CTRL_REC pTaskCtrl )
                                             sql88k_kernel_fiber, pTaskCtrl );
 	  if( pTaskCtrl->pFiberAddress == NULL) 
       {
-        rc = GetLastError();
-        //PTS 1116512
-        MSGCD (( ERR_CREATING_FIBER, rc ));
+          rc = GetLastError();
+
+          if ( ERROR_NOT_ENOUGH_MEMORY == rc )
+          {
+              MSGCD (( ERR_CREATING_FIBER_OUT_OF_MEMORY ));
+          }
+          else
+          {
+              MSGCD (( ERR_CREATING_FIBER, rc ));
+          }
       }
     }
   else
