@@ -343,11 +343,9 @@ sql905c_dllcomm_connect (
 
     // find dbroot
     ok = sql97cc_findControlServer (dllpath, pszServerDB, 
-                                    pszServerPgm, pszDBRoot);
+                                    pszServerPgm, pszDBRoot, errtext);
     if (!ok)
     {
-        strcpy (errtext, pszServerPgm);
-        strcat (errtext, " not found");
         return commErrNotOk_esp01;
     }
 
@@ -696,17 +694,14 @@ sql905c_shmclient_spawnServer (
   DWORD         bytesRead;
   PATHNAME      executable;
   SECURITY_ATTRIBUTES secAttr;
-
   secAttr.nLength = sizeof (secAttr);
   secAttr.lpSecurityDescriptor = NULL;
   secAttr.bInheritHandle = TRUE;
   // look for server
   ok = sql97cc_findControlServer (executable, pszServerDB, 
-                                  pszServerPgm, pszDBRoot);
+                                  pszServerPgm, pszDBRoot, errtext);
   if (!ok)
     {
-    strcpy (errtext, pszServerPgm);
-    strcat (errtext, " not found");
     return commErrNotOk_esp01;
     }
   // create low level pipes
