@@ -39,12 +39,12 @@
 #include          <winerror.h>
 
 #include         "heo46.h"
-#include	       "geo007_1.h"
+#include	     "geo007_1.h"
 #include         "hos42.h"
 #include         "heo420.h"
-#include	  	   "gos96.h"
-#include		     "gos960.h"
-#include		     "geo908.h"
+#include	  	 "gos96.h"
+#include		 "gos960.h"
+#include		 "geo908.h"
 
 /*===========================================================================*
  *  DEFINES                                                                  *
@@ -110,6 +110,8 @@ static ULONG os960Request         ( PTCPIP_SERV_CONNECT_INFO_REC pConnInfo );
 static VOID _System os960TCPIP_ServerThread ( ULONG ulConnInfoHandle ) ;
 
 
+extern VOID os968UpdateDebugLevelFromRegistry( VOID );
+
 /*===========================================================================*
  *  PUBLIC FUNCTIONS (CODE)                                                  *
  *===========================================================================*/
@@ -495,6 +497,7 @@ static ULONG os960Request ( PTCPIP_SERV_CONNECT_INFO_REC pConnInfo )
         if ( ulCommState == commErrOk_esp01 )
           {
           pConnInfo->ulServerRef      = LocConnParam.ulServerRef;
+          pConnInfo->ulMinReplySize   = LocConnParam.ulMinReplySize;
           pConnInfo->ulMaxCmdDataLen  = LocConnParam.ulMaxDataLen -
                                         LocConnParam.ulMinReplySize;
           pConnInfo->pCommPacket      = LocConnParam.pCommPacketList[0];
@@ -659,7 +662,7 @@ static ULONG os960Request ( PTCPIP_SERV_CONNECT_INFO_REC pConnInfo )
       break;
 
     case RSQL_NEW_DEBUG_LEVEL_REQUEST_EO003:  /* PTS 1111208 */
-	  gxr.usDebugLevel = ConPktParamRec.ulSenderRef;
+      os968UpdateDebugLevelFromRegistry();
 	  MSGD(( INFO_XSERVER_NEW_DEBUG_LEVEL, gxr.usDebugLevel ));
       ConPktParamRec.ulMessClass           = RSQL_NEW_DEBUG_LEVEL_REPLY_EO003;
       ConPktParamRec.ulCommState           = commErrOk_esp01;
@@ -752,6 +755,58 @@ static ULONG os960Transfer ( PTCPIP_SERV_CONNECT_INFO_REC pConnInfo )
       if ( ulCommState != commErrOk_esp01 )
         break;
     }
+    else if ( pConnInfo->ulServiceType == srvPing_esp01 )
+    {
+        SAPDB_UInt1     hops;
+
+        if ( eo420ExtractPingPacket( (char*)pConnInfo->pCommPacket->pDataPart, ulDataLen, &hops, 0, 0 ) )
+        {
+            if ( hops-- == 0 )
+            {   // No more hops!
+                pResCommPacket  = (PCOMM_PACKET_REC)(PCHAR)pConnInfo->pCommPacket;
+
+                ulResDataLen =  eo420FillPingPacket ( (char*) pResCommPacket->pDataPart,
+                                                      pConnInfo->ulMaxCmdDataLen - ulDataLen + 
+                                                      pConnInfo->ulMinReplySize,
+                                                      0, // hops equal to 0 means that the final destination was not reached
+                                                      XSERV_TITLE,
+                                                      gxr.RTEVersionString, 0 );
+
+                pConnInfo->ThrdCtrlHeader.ThrdState = THRD_DATA_SEND;
+
+                ulCommState = eo420SendCommPacket ( (void *)&pConnInfo->sd, os42SocketSend,
+                                                    pConnInfo->ulMaxSegmentSize,
+                                                    pResCommPacket,
+                                                    ulResDataLen,
+                                                    ulSenderRef,
+                                                    ulReceiverRef,
+                                                    RSQL_USER_DATA_REPLY_EO003,
+                                                    commErrOk_esp01,
+                                                    pErrText );
+
+                if ( ulCommState != commErrOk_esp01 )
+                {
+                    MSGD (( ERR_XSER_SEND_PACKET, pConnInfo->ulServerRef, pErrText ));
+                    break;
+                }
+                MSG8 (( INFO_XSER_SEND_REPLY_PACKET, ulResDataLen, pConnInfo->ulServerRef ));
+                continue;
+            }
+    
+            ulCommState = sql33c_request( &pConnInfo->Shm, 0, ulDataLen, pErrText );
+
+            if ( ulCommState != commErrOk_esp01 )
+            {
+                MSGD (( ERR_XSER_REQUEST, pConnInfo->ulServerRef, pErrText ));
+                break;
+            }
+        }
+        else
+        {
+            ulCommState = commErrNotOk_esp01;
+            break;
+        }
+    }
     else
     {
       ulCommState = sql33c_request( &pConnInfo->Shm, 0, ulDataLen, pErrText );
