@@ -780,6 +780,7 @@ static ULONG os962NiGetRequest ( PTCPIP_SERV_CONNECT_INFO_REC pConnInfo )
         if ( ulCommState == commErrOk_esp01 )
           {
           pConnInfo->ulServerRef      = LocConnParam.ulServerRef;
+          pConnInfo->ulMinReplySize   = LocConnParam.ulMinReplySize;
           pConnInfo->ulMaxCmdDataLen  = LocConnParam.ulMaxDataLen -
                                         LocConnParam.ulMinReplySize;
           pConnInfo->pCommPacket      = LocConnParam.pCommPacketList[0];
@@ -1029,6 +1030,60 @@ static ULONG os962NiTransfer ( PTCPIP_SERV_CONNECT_INFO_REC pConnInfo )
       else if ( ! ulDataLen )
         break;
     }
+    else if ( pConnInfo->ulServiceType == srvPing_esp01 )
+    {
+        SAPDB_UInt1     hops;
+
+        if ( eo420ExtractPingPacket( (char*)pConnInfo->pCommPacket->pDataPart, ulDataLen, &hops, 0, 0 ) )
+        {
+            if ( hops-- == 0 )
+            {   // No more hops!
+                pResCommPacket  = (PCOMM_PACKET_REC)(PCHAR)pConnInfo->pCommPacket;
+
+                ulResDataLen =  eo420FillPingPacket ( (char*) pResCommPacket->pDataPart,
+                                                      pConnInfo->ulMaxCmdDataLen - ulDataLen + 
+                                                      pConnInfo->ulMinReplySize,
+                                                      0, // hops equal to 0 means that the final destination was not reached
+                                                      XSERV_TITLE,
+                                                      gxr.RTEVersionString, 0 );
+
+                pConnInfo->ThrdCtrlHeader.ThrdState = THRD_DATA_SEND;
+
+                ulCommState = eo420SendCommPacket ( pConnInfo->NiHandle, eo40NiSend,
+                                                    pConnInfo->ulMaxSegmentSize,
+                                                    pResCommPacket,
+                                                    ulResDataLen,
+                                                    ulSenderRef,
+                                                    ulReceiverRef,
+                                                    RSQL_USER_DATA_REPLY_EO003,
+                                                    commErrOk_esp01,
+                                                    pErrText );
+
+
+                if ( ulCommState != commErrOk_esp01 )
+                {
+                    MSGD (( ERR_XSER_SEND_PACKET, pConnInfo->ulServerRef, pErrText ));
+                    break;
+                }
+                MSG8 (( INFO_XSER_SEND_REPLY_PACKET, ulResDataLen, pConnInfo->ulServerRef ));
+
+                continue;
+            }
+
+            ulCommState = sql33c_request( &pConnInfo->Shm, 0, ulDataLen, pErrText );
+
+            if ( ulCommState != commErrOk_esp01 )
+            {
+                MSGD (( ERR_XSER_REQUEST, pConnInfo->ulServerRef, pErrText ));
+                break;
+            }
+        }
+        else
+        {
+            ulCommState = commErrNotOk_esp01;
+            break;
+        }
+    }
     else
     {
       ulCommState = sql33c_request( &pConnInfo->Shm, 0, ulDataLen, pErrText );
