@@ -121,6 +121,7 @@ static INT          os968Server              ( VOID );
 
 static BOOL WINAPI  os968ConsoleCtrlHandler  ( DWORD                  dwCtrlType );
 static VOID WINAPI  os968ServiceCtrlHandler  ( DWORD                  dwCtrlCode );
+       VOID         os968UpdateDebugLevelFromRegistry ( VOID );
 static VOID         os968Service             ( VOID );
 static BOOL         os968DebuggerIsDrWatson  ( VOID );
 static void         os968UpdateVersion       ( VOID );
@@ -480,15 +481,7 @@ static int os968Main ( int argc, char  *argv[] )
 
                break;
        case 'F' :
-               if (( ulStrtOpt      == OPT_INSTALL_SERVICE ) ||
-                   ( ulStrtOpt      == OPT_UPDATE_SERVICE  ) ||
-                   ( gxr.fNTService == FALSE )) 
-               {
-                 gxr.fNoDNSReverseLookup = TRUE;
-			   }
-			   else
-                 fCommandError = TRUE;
-
+               gxr.fNoDNSReverseLookup = TRUE;
                break;
        case 'n' :
                if ( gxr.fNTService )
@@ -522,6 +515,13 @@ static int os968Main ( int argc, char  *argv[] )
       }
     }
 
+    if (( gxr.fNoDNSReverseLookup   == TRUE  ) &&
+        (( ulStrtOpt                 != OPT_INSTALL_SERVICE ) &&
+         ( ulStrtOpt                 != OPT_UPDATE_SERVICE  ))) 
+    {
+        fCommandError = TRUE;
+    }
+
   if ( (argc - sql80_OptInd) || (fCommandError == TRUE) )
     {
     printf ( XSERVER_NO_VALID_OPTION );
@@ -725,6 +725,27 @@ static int os968Main ( int argc, char  *argv[] )
 
  /*------------------------------*/
 
+VOID os968UpdateDebugLevelFromRegistry( VOID )
+{
+    PATHNAME                szKey;
+    REG_ENTRY_REC           RegistryEntries[1];
+    DWORD                   dwDebugLevel;
+
+    strcpy ( szKey, gxr.ServerSpecification.ServerName );
+    strcat ( szKey, "\\"REG_SK_SERVICE_PARAM );
+    RegistryEntries[0].pszValueName        = REG_VN_DEBUG_LEVEL;
+    RegistryEntries[0].pValue              = &dwDebugLevel;
+    RegistryEntries[0].ulValueSize         = sizeof(DWORD);
+    RegistryEntries[0].ulValueType         = REG_DWORD;
+
+    if (sql50_reg_get_service_values ( NULL, szKey, 1, RegistryEntries ) == NO_ERROR)
+    {
+  	    gxr.usDebugLevel = (SHORT)dwDebugLevel;
+    }
+}
+
+ /*------------------------------*/
+
  static VOID os968Service ( VOID )
    {
    #undef  MF__
@@ -737,7 +758,6 @@ static int os968Main ( int argc, char  *argv[] )
    REG_ENTRY_REC           RegistryEntries[1];
    DWORD                   dwValUseIndepPath = 0;
    PATHNAME                szKey;
-   DWORD                   dwDebugLevel;
    DWORD                   dwDiagFileSize       = DIAGSIZE;
    DWORD                   dwNoDNSReverseLookup = 0;
 
@@ -794,17 +814,7 @@ static int os968Main ( int argc, char  *argv[] )
         gxr.fNoDNSReverseLookup = dwNoDNSReverseLookup == 1;
         }
 
-      strcpy ( szKey, gxr.ServerSpecification.ServerName );
-      strcat ( szKey, "\\"REG_SK_SERVICE_PARAM );
-      RegistryEntries[0].pszValueName        = REG_VN_DEBUG_LEVEL;
-      RegistryEntries[0].pValue              = &dwDebugLevel;
-      RegistryEntries[0].ulValueSize         = sizeof(DWORD);
-      RegistryEntries[0].ulValueType         = REG_DWORD;
-
-      if (sql50_reg_get_service_values ( NULL, szKey, 1, RegistryEntries ) == NO_ERROR)
-        {
-  	    gxr.usDebugLevel = (SHORT)dwDebugLevel;
-        }
+      os968UpdateDebugLevelFromRegistry();
 
       strcpy ( szKey, gxr.ServerSpecification.ServerName );
       strcat ( szKey, "\\"REG_SK_SERVICE_PARAM );
