@@ -41,6 +41,8 @@
 #include "geo007_2.h"
 
 #include "RunTime/RTE_GetDBRootOfDB.h"
+#include "SAPDBCommon/SAPDB_string.h"
+#include "SAPDBCommon/SAPDB_sprintf.h"
 
 #if defined (CTRLCOMP)
 # include "heo05.h"       /* nocheck */
@@ -223,10 +225,11 @@ int main ( int argc, char  *argv[] )
 
 #if !defined (ENUMINST_ONLY)
 
-BOOL sql97cc_findControlServer ( PSZ    pszExecutable,   // OUT
-                                 PSZ    pszServerDB,     // IN
-                                 PSZ    pszServerPgm,    // IN
-                                 PSZ    pszDBRoot)       // IN/OUT
+BOOL sql97cc_findControlServer ( PSZ            pszExecutable,  // OUT
+                                 PSZ            pszServerDB,    // IN
+                                 PSZ            pszServerPgm,   // IN
+                                 PSZ            pszDBRoot,      // IN/OUT
+                                 tsp00_ErrText  errText)        // OUT
 {
     #undef MF__
     #define MF__ MOD__"sql97cc_findControlServer"
@@ -271,10 +274,14 @@ BOOL sql97cc_findControlServer ( PSZ    pszExecutable,   // OUT
         /* check existence of file */
         if ( GetFileAttributes (pszExecutable) == 0xffffffff )
         {   
+            SAPDB_sprintf(errText , sizeof(tsp00_ErrText), "%s executable not found",pszServerPgm);
             found = false;
         }
     }
-
+    else
+    {
+        SAPDB_strncpy_zero_terminated(errText , "database instance not found", sizeof(tsp00_ErrText));
+    }
     return found;
 }
 
