@@ -474,6 +474,9 @@ extern "C" {                         /* Assume C declarations for C++ */
 #define SQL_TYPE_DRIVER_END                     SQL_UNICODE_LONGVARCHAR
 #endif  /* ODBCVER < 0x0300 */
 
+#define SQL_ABAPHANDLE                     47
+#define SQL_C_ABAPHANDLE                   SQL_ABAPHANDLE
+
 /* C datatype to SQL datatype mapping      SQL types
                                            ------------------- */
 #define SQL_C_CHAR    SQL_CHAR             /* CHAR, VARCHAR, DECIMAL, NUMERIC */
