@@ -370,7 +370,7 @@ ODBC_PROC(SQLGetDiagField, ( SQLSMALLINT      handleType,
         case SQL_DIAG_CONNECTION_NAME:
         case SQL_DIAG_SERVER_NAME:
             parentDBC = pa10GetParentDBC( handleType, handle );
-            if (parentDBC!=NULL) {
+            if (parentDBC != NULL  &&  parentDBC->dsname != NULL) {
                 retcode = pa80CopyTpr05String( diagInfoPtr,
                                                bufferLen*charSize,
                                                &strLen,
