@@ -522,6 +522,16 @@ API_RETCODE  apeprep ( tpa41ESQBlock *api_cb_ptr,
             API_TRACE(API_TR_UWORD,"sqln",&sqlda_ptr -> sqln);
             API_TRACE(API_TR_UWORD,"sqld",&sqlda_ptr -> sqld);
 
+            /* if reconnect failed (e.g. kernel was offline)
+               http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1128976
+               http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1130529
+               http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1131374 */
+            if (api_cb_ptr->sqlca.sqlcode == PA_ERR_CONNECTION_DOWN  ||
+                api_cb_ptr->sqlca.sqlcode == PA_ERR_SERVERDB_NOT_AVAILABLE) {
+              retcode = API_NOT_OK;
+              goto exit;
+            }
+
             pa30apcruntime(api_dbc_ptr, api_stmt_ptr);
 
             aperetg (api_cb_ptr);
@@ -568,6 +578,7 @@ API_RETCODE  apeprep ( tpa41ESQBlock *api_cb_ptr,
                 api_stmt_ptr->stmtopt.sqlmode;
         }
     }
+ exit:
     API_TRACE(API_TR_EXIT,"apeprep",0);
     API_TRACE(API_TR_PARSID, "parsid", parsid_ptr);
     API_TRACE(API_TR_API_RETCODE,"retcode",&retcode);
@@ -830,6 +841,12 @@ API_RETCODE apegetl ( tpa41ESQBlock  *api_cb_ptr,
                                 parsid_ptr );
                 API_TRACE(API_TR_SWORD, "varno", &varno);
                 *col = varno;
+
+                if (api_cb_ptr->sqlca.sqlcode != PA_OK) {
+                  aperetg (api_cb_ptr);	
+                  retcode = API_NOT_OK;
+                }
+
             }; /* if */
         }; /* if */
     if (varno <= 0) {
@@ -1420,7 +1437,10 @@ SWORD apereconn( tpa41ESQBlock *api_cb_ptr,
     
     ret = FALSE;    
     API_TRACE(API_TR_ENTRY,"apereconn",0);
-    if ( api_cb_ptr->sqlca.sqlcode == 700 || api_cb_ptr->sqlca.sqlcode == 70) {
+    /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1131374 */
+    if ( api_cb_ptr->sqlca.sqlcode == 700  ||  api_cb_ptr->sqlca.sqlcode == 70  ||
+         api_cb_ptr->sqlca.sqlcode == PA_ERR_CONNECTION_DOWN  ||
+         api_cb_ptr->sqlca.sqlcode == PA_ERR_SERVERDB_NOT_AVAILABLE) {
         API_TRACE(API_TR_SWORD, "dbno", &api_dbc_ptr->dbno);
         API_TRACE(API_TR_STRING, "userid", api_dbc_ptr->userid);
         API_TRACE(API_TR_STRING, "passw", api_dbc_ptr->passw);
