@@ -90,6 +90,8 @@ UDWORD pa60_ComputePosition (UDWORD           long_pos,
 tsp00_Int pa60_ComputeKernelPosition (UDWORD           long_pos,
                                       tpa21DescRecord *impl_rec_ptr);
 
+int pa60_isConnected (tpa40DBC *dbc_block_ptr);
+
 /*!**********************************************************************
 
   Chapter: ODBC functions
@@ -840,6 +842,11 @@ VOID pa60DropFetchParseIds( tpa60Stmt *stmt_block_ptr )
 
     API_ASSERT( pa40VerifyDBC( dbc_block_ptr ) == API_OK );
     
+    /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1129839 */
+    /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1131844 */
+    if (!pa60_isConnected (dbc_block_ptr))
+      return;
+
     apedrprs( &dbc_block_ptr->esqblk,
               stmt_block_ptr -> fetch_first_parsid,
               dbc_block_ptr,
@@ -878,7 +885,10 @@ VOID pa60DropParseIds( tpa60Stmt *stmt_block_ptr )
     API_ASSERT( pa40VerifyDBC( dbc_block_ptr ) == API_OK );
     
     pa60DropFetchParseIds( stmt_block_ptr );
-    
+
+    if (!pa60_isConnected (dbc_block_ptr))
+      return;
+
     apedrprs( &dbc_block_ptr->esqblk,
               stmt_block_ptr -> close_parsid,
               dbc_block_ptr,
@@ -2308,7 +2318,17 @@ tsp00_Int pa60_ComputeKernelPosition (UDWORD           long_pos,
   return (tsp00_Int4) long_pos;
 }
 
+int pa60_isConnected (tpa40DBC *dbc_block_ptr)
+{
+  if (dbc_block_ptr->esqblk.esq_sql_code == -807 || 
+      dbc_block_ptr->esqblk.esq_sql_code == 700  ||
+      dbc_block_ptr->esqblk.esq_sql_code == -813 ||
+      dbc_block_ptr->esqblk.esq_sql_code == 800  ||
+      dbc_block_ptr->esqblk.esq_sql_code == -708)
+    return FALSE;
 
+  return TRUE;
+}
 
 #endif  /* of _UNICODE_ODBC */
 
