@@ -75,6 +75,8 @@
  *  EXTERNAL PROTOTYPES                                             *
  *==================================================================*/
 
+extern char *s98CPCDrvBuildNumber(void);
+
 /*==================================================================*
  *  EXTERNAL VARIABLES                                              *
  *==================================================================*/
@@ -162,6 +164,7 @@ typedef struct {
   sqlPROC(sqlblnk);
   sqlPROC(sqlglm);
   sqlPROC(sqlgetver);
+  sqlPROC(sqlgetbuild);
   pi01SQLDAPROC lpsqlald;
   pi01intPROC   lpsqlcsize;
   pi01voidPROC  lpsqlcdynp;
@@ -348,6 +351,7 @@ void pi01ErrorInstallation(char *szVersion)
 }
 
 void sqlgetver(char *szVersion, char* szMinVersion);
+void sqlgetbuild(char *szBuild);
 
 boolean pi01IsGoodPCRLib(HANDLE hLib, char *szVersion, char* szMinVersion)
 {
@@ -604,14 +608,21 @@ static void pi01TraceAppStarted(char *szVersion)
 static void pi01TraceLoadLibSucceed(char *szPath, char *szCompactVersion)
 {
   char szVersion[64];
+  char szBuild[16] = "";
 
   if ((pi01TraceEnabled == false) ||
       (pi01TraceFile == NULL))
     return;
 
   pi01TraceLineHeader (ok_epi01);
-  fprintf (pi01TraceFile, " %s %s load succeed.\n", szPath, 
-	   pi01UnpackVersion(szVersion, szCompactVersion));
+  sqlgetbuild(szBuild);
+  if (szBuild[0] == '\0') {
+    fprintf (pi01TraceFile, " %s %s Build <unknown> load succeed.\n", szPath, 
+             pi01UnpackVersion(szVersion, szCompactVersion));
+  } else {
+    fprintf (pi01TraceFile, " %s %s Build %s load succeed.\n", szPath, 
+             pi01UnpackVersion(szVersion, szCompactVersion), szBuild);
+  }
   LoadErr = FALSE;
 }
 
@@ -680,13 +691,14 @@ int pi01OpenVersionInfo(tpi01VersionDesc *VerDesc)
       InstVer = pr09AddItem(VerDesc->pVersionList);
       ret = pr09ConfigNextRuntimeEntry(hConfig, &location, szPath, sizeof(szPath), szVersion, sizeof(szVersion), ErrText);
       if (ERR_OK_epr09 == ret) {
-	pi01VersionS2B(szVersion, &InstVer->version);
-	strncpy(InstVer->szPath, szPath, sizeof(InstVer->szPath));
-      }
-      if (ERR_NO_MORE_DATA_epr09 == ret)
-	pr09CloseItem(VerDesc->pVersionList);
-      else
-	count++;
+        pi01VersionS2B(szVersion, &InstVer->version);
+        strncpy(InstVer->szPath, szPath, sizeof(InstVer->szPath));
+        count++;
+      } else if (ret == ERR_READ_VALUE_epr09 ){
+        continue;
+      } else if (ERR_NO_MORE_DATA_epr09 == ret) {
+          pr09CloseItem(VerDesc->pVersionList);
+      } 
     }      
     pr09SetIndex(VerDesc->pVersionList, Start_epr09);
     pr09ConfigCloseRuntimeSection(hConfig, ErrText);
@@ -1227,6 +1239,11 @@ void sqlgetver(char *szVersion, char* szMinVersion)
   PI01PCRCALL(sqlgetver, (szVersion, szMinVersion));
 }
 
+void sqlgetbuild(char *szBuild)
+{
+  PI01PCRCALL(sqlgetbuild, (szBuild));
+}
+
 void sqlTCompGetConnection (
 			    void        *rawca,
 			    int          sessionNumber,
@@ -1606,25 +1623,28 @@ externC void sqlferasep (const tsp00_VFilename rawFName,
 }
 
 externC void sqlgetuser (tsp4_xuser_record  *prcUserParams,
+             SAPDB_Char *         accountName,
 			 tsp00_ErrText        acErrorText,
-			 tsp00_BoolAddr pbOk)
+			 tsp00_BoolAddr       pbOk)
 {
-  PI01LZUCALL(sqlgetuser, (prcUserParams, acErrorText, pbOk));
+  PI01LZUCALL(sqlgetuser, (prcUserParams, accountName, acErrorText, pbOk));
 }
 
 externC void sqlputuser (tsp4_xuser_record  *prcUserParams,
+             SAPDB_Char *         accountName,
 			 tsp00_ErrText        acErrorText,
-			 tsp00_BoolAddr pbOk)
+			 tsp00_BoolAddr       pbOk)
 {
-  PI01LZUCALL(sqlputuser, (prcUserParams, acErrorText, pbOk));
+  PI01LZUCALL(sqlputuser, (prcUserParams, accountName, acErrorText, pbOk));
 }
 
 externC void sqlindexuser (tsp00_Int2          kUserIndex,
 			   tsp4_xuser_record  *prcUserParams,
+               SAPDB_Char *        accountName,
 			   tsp00_ErrText       acErrorText,
-			   tsp00_BoolAddr pbOk)
+			   tsp00_BoolAddr      pbOk)
 {
-  PI01LZUCALL(sqlindexuser, (kUserIndex, prcUserParams, acErrorText, pbOk));
+  PI01LZUCALL(sqlindexuser, (kUserIndex, prcUserParams, accountName, acErrorText, pbOk));
 }
 
 externC void sqlinfouser (tsp00_KnlIdentifier acUser,
@@ -1635,12 +1655,12 @@ externC void sqlinfouser (tsp00_KnlIdentifier acUser,
   PI01LZUCALL(sqlinfouser, (acUser, acPassword, acComponent, pfResult));
 }
 
-externC tsp00_Int4 sqlclearuser (void)
+externC tsp00_Int4 sqlclearuser (SAPDB_Char *        accountName)
 {
   if (!pi01LZUProc.lpsqlclearuser)
     pi01LZUProc.lpsqlclearuser = (pi01intPROC)pi01DynamicLoad("sqlclearuser");
   if (pi01LZUProc.lpsqlclearuser)
-    return(pi01LZUProc.lpsqlclearuser());
+    return(pi01LZUProc.lpsqlclearuser(accountName));
   return 0;
 }
 
