@@ -88,10 +88,49 @@ static pi30SQLAreasStruct pi30SQLAreas[]={
 
 tpr01_ModuleContainer *sqlModuleCont=NULL;
 extern tpr01_EnvContainer *sqlEnvCont;
-
+tpr_pcVersion sqlPCVersion;
+extern tpr_pcVersion *p07pcversion;
+char sqlPCRelStr[200];
 void sqlCPCEndInit (sqlcatype *sqlca, sqlxatype *sqlxa, char *szPCVersion )
 {
   M90TRACE(M90_TR_ENTRY, "sqlCPCEndInit", 0);
+  if (!p07pcversion) {
+    char *c;
+    sqlPCRelStr[0] = '\0';
+    sqlPCVersion.cbBuild = 0;
+    for(c=szPCVersion; *c != '\0'; c++) {
+      if(*c >= '0' && *c <= '9') {
+        char *p = sqlPCRelStr;
+        for(;*c != '\0'; c++,p++) {
+          *p = *c;
+          if (*c == ' ') {
+            *p='\0';
+            break;
+          }
+        }
+        for(;*(c+6) != '\0'; c++) {
+#ifdef WIN32
+#define STRNICMP strnicmp
+#else
+#define STRNICMP strncasecmp
+#endif
+          if (!STRNICMP(c, "BUILD ", 6)) {
+            int x =
+            sscanf(c+6, "%d-", &sqlPCVersion.cbBuild);
+            if (x != 1) {
+              sqlPCVersion.cbBuild = 0;
+            }
+            break;
+          }
+        }        
+        break;
+      }
+    }
+    sqlPCVersion.szVersion = sqlPCRelStr;    
+    if (sqlPCRelStr[0] != '\0') {
+      p07pcversion = &sqlPCVersion;
+    }    
+  }
   M90TRACE(M90_TR_EXIT, "sqlCPCEndInit", 0);
 }
 
@@ -585,7 +624,11 @@ void sqlCPCOpen (sqlcatype *sqlca, sqlxatype *sqlxa, long stno, long sess)
 void sqlCPCExecDirect (sqlcatype *sqlca, sqlxatype *sqlxa, long stno, long sess)
 {  
   M90TRACE(M90_TR_ENTRY, "sqlCPCExecDirect", 0);
-  {
+  if (!pi10TryEnterSession(sqlca->sqlgap)) {
+    p08runtimeerror (sqlca, sqlxa, cpr_connection_busy);
+    p03traceerror(sqlca->sqlrap, sqlca->sqlemp); 
+  }
+  else {
     tpr01_ModuleDesc *ModuleDesc = sqlxa->xaModDesc;
     tpr01_ModuleContainer *Module = ModuleDesc->Module;
     tpr01_PrecomDesc *PrecomDesc = Module->GetPrecomDesc(ModuleDesc, sqlxa, stno);
@@ -594,9 +637,11 @@ void sqlCPCExecDirect (sqlcatype *sqlca, sqlxatype *sqlxa, long stno, long sess)
     Precom->PrepareStmtName(PrecomDesc, ModuleDesc, ConDesc);
     Precom->DeclareCursor(PrecomDesc, ModuleDesc, ConDesc);
     Precom->Open(PrecomDesc, ModuleDesc, ConDesc);
+    pi10LeaveSession(sqlca->sqlgap);
   }
   M90TRACE(M90_TR_EXIT, "sqlCPCExecDirect", 0);
 }
+
 void sqlCPCDescribe (sqlcatype *sqlca, sqlxatype *sqlxa, long stno, long sess)
 {
   M90TRACE(M90_TR_ENTRY, "sqlCPCDescribe", 0);
@@ -690,19 +735,6 @@ void sqlCPCTryExecute (sqlcatype *sqlca, sqlxatype *sqlxa, long kano, long sess)
   M90TRACE(M90_TR_ENTRY, "sqlCPCTryExecute", 0);
   M90TRACE(M90_TR_DWORD, "kano", &kano);
   M90TRACE(M90_TR_DWORD, "sess", &sess);
-  if (sqlxa->sqlkap[kano-1].katyp == cpr_com_abort_session) {
-    sqlca->sqlrap->raactsession = (sqlint2) sess;
-    {
-      tpr01_ModuleDesc *ModuleDesc = sqlxa->xaModDesc;
-      tpr01_ModuleContainer *Module = ModuleDesc->Module;
-      tpr01_PrecomDesc *PrecomDesc = Module->GetPrecomDesc(ModuleDesc, sqlxa, kano);
-      tpr01_PrecomContainer *Precom = PrecomDesc->Precom;
-      tpr01_ConDesc *ConDesc = Precom->GetConnection(PrecomDesc, ModuleDesc, sess);
-      Precom->AbortSession(PrecomDesc, ModuleDesc, ConDesc);
-    }
-    pi10LeaveSession(sqlca->sqlgap);
-  }
-  else
     if (!pi10TryEnterSession(sqlca->sqlgap)) {
       p08runtimeerror (sqlca, sqlxa, cpr_connection_busy);
       p03traceerror(sqlca->sqlrap, sqlca->sqlemp); 
@@ -710,7 +742,18 @@ void sqlCPCTryExecute (sqlcatype *sqlca, sqlxatype *sqlxa, long kano, long sess)
     else {
       sqlxa->xakano = (sqlint2) kano;
       sqlca->sqlrap->raactsession = (sqlint2) sess;
-      {
+      if (sqlxa->sqlkap[kano-1].katyp == cpr_com_abort_session) {
+        sqlca->sqlrap->raactsession = (sqlint2) sess;
+        {
+          tpr01_ModuleDesc *ModuleDesc = sqlxa->xaModDesc;
+          tpr01_ModuleContainer *Module = ModuleDesc->Module;
+          tpr01_PrecomDesc *PrecomDesc = Module->GetPrecomDesc(ModuleDesc, sqlxa, kano);
+          tpr01_PrecomContainer *Precom = PrecomDesc->Precom;
+          tpr01_ConDesc *ConDesc = Precom->GetConnection(PrecomDesc, ModuleDesc, sess);
+          Precom->AbortSession(PrecomDesc, ModuleDesc, ConDesc);
+        }
+      }
+      else {
         tpr01_ModuleDesc *ModuleDesc = sqlxa->xaModDesc;
         tpr01_ModuleContainer *Module = ModuleDesc->Module;
         tpr01_PrecomDesc *PrecomDesc = Module->GetPrecomDesc(ModuleDesc, sqlxa, kano);
@@ -1052,6 +1095,13 @@ void sqlgetver(char *szVersion, char* szMinVersion)
   M90TRACE(M90_TR_EXIT, "sqlgetver", 0);
 }
 
+void sqlgetbuild(char *szBuild)
+{
+  M90TRACE(M90_TR_ENTRY, "sqlgetbuild", 0);
+  strcpy(szBuild, pr07BuildNr());
+  M90TRACE(M90_TR_EXIT, "sqlgetbuild", 0);
+}
+
 void sqlCPCDeclare (sqlcatype *sqlca, sqlxatype *sqlxa, long kano, long sess)
 {
   M90TRACE(M90_TR_ENTRY, "sqlCPCDeclare", 0);
