@@ -522,6 +522,15 @@ typedef enum tpr00_Version_Enum {
 
 #define BlobSupport_CPCVersion_cpr00     Version722_cpr00
 
+
+#define FeatureSet743_cpr00 { \
+                              {sp1f_nil_feature,           0}, \
+                              {sp1f_multiple_drop_parseid, 0}, \
+                              {sp1f_space_option,          0}, \
+                              {sp1f_variable_input,        0}, \
+                              {sp1f_optimized_streams,     1}\
+                            }
+
 #define mxpr_macroline   (126)
 
 typedef tsp00_KnlIdentifierc tpr00_DatabaseName;
