@@ -65,6 +65,15 @@
  *  DECLARATIONS                                                    *
  *==================================================================*/
 
+const tpr01Con_FeatureSet pr01ConFeatureSetWanted = FeatureSet743_cpr00;
+
+const tpr01Con_FeatureSet pr01ConFeatureSetEmpty  = { {sp1f_nil_feature,           0},
+                                                      {sp1f_multiple_drop_parseid, 0},
+                                                      {sp1f_space_option,          0},
+                                                      {sp1f_variable_input,        0},
+                                                      {sp1f_optimized_streams,     0}};
+
+
 /*==================================================================*
  *  EXTERNAL PROTOTYPES                                             *
  *==================================================================*/
@@ -123,7 +132,8 @@ static void pr01ConInitCont(tpr01_ConContainer *Connection, tpr01_EnvDesc *EnvDe
   Connection->sqlca      = EnvDesc->sqlca;
   Connection->NoLog      = cpr_is_false;
   Connection->SessionCnt = 0;
-  Connection->ParseInfoParams.CacheSize = 0; /*<= Cache is disabled DEFAULT_HASHTABLE_SIZE; */ 
+  Connection->OptimizedStreams = pr01ConFeatureSetWanted[sp1f_optimized_streams].value;
+  Connection->ParseInfoParams.CacheSize = 0; /*<= Cache is disabled DEFAULT_HASHTABLE_SIZE; */
   Connection->ParseInfoParams.HashTableLoadFactor = DEFAULT_HASHTABLE_LOADFACTOR;
   Connection->OldStyle   = FALSE;
   Connection->InitCont   = pr01ConInitCont;
@@ -219,6 +229,9 @@ static tpr01_ConDesc *pr01ConAddDesc(tpr01_ConContainer *Connection, tpr00_Datab
     ConDesc->sqlca = Connection->sqlca;
     ConDesc->sqlxa = sqlxa;
     ConDesc->ka = pr01cDupKa(NULL);
+    ConDesc->SessionType = SessionEmpty_epr01Con;
+    memcpy(ConDesc->DBFeatures, pr01ConFeatureSetWanted, sizeof(ConDesc->DBFeatures));
+    ConDesc->Connection->SetAttr(ConDesc, OptimizedStreams_epr01Con, (void*)&ConDesc->Connection->OptimizedStreams, sizeof(ConDesc->Connection->OptimizedStreams));    
     if (!ConDesc->ka)
       pr07CheckAssert(false);
 
@@ -296,19 +309,31 @@ static void pr01ConDeleteDesc(tpr01_ConDesc *ConDesc)
   int cbIndex = pr09ItemFindIndex(ConDesc->Connection->Desc, ConDesc);
   if (cbIndex >= 0) {
     pr01DeleteErrorDesc(ConDesc->ErrorDesc);
-    if (ConDesc->ga)
+    ConDesc->ErrorDesc = NULL;
+    if (ConDesc->ga){
       pr03mFree(ConDesc->ga, "ConDesc->ga");
-    if(ConDesc->ka)
+      ConDesc->ga = NULL;
+    }  
+    if(ConDesc->ka){
       pr01cFreeKa(ConDesc->ka);
+      ConDesc->ka = NULL;
+    }  
+
     pr09CloseItemEx(ConDesc->Connection->Desc, cbIndex);
+
     if (ConDesc->SegmDesc)
       pr01ConSegmentDelete(ConDesc);
+
     /* delete Parseinfo Cache */
-    if (ConDesc->ParseInfoCache)
+    if (ConDesc->ParseInfoCache){
       pr01ParseInfoDeleteCont(ConDesc->ParseInfoCache);
+      ConDesc->ParseInfoCache = NULL;
+    }  
     /* http://pts:1080/webpts?wptsdetail=yes&ErrorType=0&ErrorID=1127862 */
-    if (ConDesc->ParseIdCache)
+    if (ConDesc->ParseIdCache){
       pr06ParseIDDeleteCache(ConDesc->ParseIdCache);
+      ConDesc->ParseIdCache = NULL;
+    }  
   }
 }
 
@@ -509,6 +534,8 @@ static void pr01ConAutoConnect (tpr01_ConDesc *ConDesc)
       ConDesc->ga->gareference = 9999;
       ga->gareqptr = pr03PacketNewPacket(1000);
       Connection->InitPacket(ConDesc, sp77encodingAscii, sp1m_dbs);
+      sqlemp.ereturncode = 0;
+      sqlemp.elzu = sp1ce_ok;
       pr03ConMakeConnectPart(ConDesc, &sqlemp);
       p03connectcmdtrace (sqlra, ga);
       p01pparsidtrace (sqlca, sqlxa, ConDesc->ka, NULL, TraceParseIDSQLStmt_epr00);
@@ -600,6 +627,10 @@ static BOOLEAN pr01ConSetAttr (tpr01_ConDesc *ConDesc, epr01Con_Attr Attr, void
       ConDesc->Connection->Actual = (tpr01_ConDesc*)pAttr;
       break;
     }
+    case (OptimizedStreams_epr01Con) : {
+      ConDesc->DBFeatures[sp1f_optimized_streams].value = (*(BOOLEAN*)pAttr) ? 1 : 0;
+      break;
+    }
     default: {
       ret = FALSE;
       break;
