@@ -66,7 +66,7 @@ static void pr01CursorInitCont(tpr01_CursorContainer *Cont);
 
 static tsp00_Int4 pr01CursorOpenCont(tpr01_CursorContainer *Cursor);
 
-static tsp00_Int4 pr01CursorGetCount(tpr01_CursorContainer *Cursor);
+/*static tsp00_Int4 pr01CursorGetCount(tpr01_CursorContainer *Cursor);*/
 
 static void pr01CursorCloseCont(tpr01_CursorContainer *Cursor);
 
@@ -78,7 +78,7 @@ static void pr01CursorDeleteDesc(tpr01_CursorDesc *CursorDesc);
 
 static tpr01_CursorDesc *pr01CursorFindDesc(tpr01_CursorContainer *Cursor, tpr05_String* CursorName, tpr01_UniqueID UniqueID);
 
-static tsp00_Int4 pr01CursorSetIndex(tpr01_CursorContainer *Cursor, tsp00_Int4 cbIndex);
+static void pr01CursorResetIterator(tpr01_CursorContainer *Cursor);
 
 static void pr01CursorExecute(tpr01_CursorDesc *CursorDesc);
 
@@ -100,6 +100,25 @@ static void pr01CursorGetAttr (tpr01_CursorDesc *CursorDesc, epr01Cursor_Attr At
 			       void *pAttr, tsp00_Int4 *cbLen);
 
 static void pr01Cursor_InitCursorName (tpr00_CursorNameDesc *CursorNameDesc);
+#ifdef DYNAHASH  
+static tpr09HashValue pr01Cursor_GetKey (const tpr09HashValue data_p, tsp00_Uint4* HashKeyLen);
+
+static tsp00_Int4 pr01Cursor_CompareKey (const tpr09HashKey Item1,tsp00_Uint4 Item1Len, const tpr09HashKey Item2, tsp00_Uint4 Item2Len);
+
+static void pr01Cursor_printKey (const tpr09HashValue data_p, FILE* datei);
+
+static tpr09HashIndex pr01Cursor_hashFunc (const tpr09HashKey HashKey, tsp00_Uint4 cbHashKeyLen, tsp00_Uint4 cbHashTabLen);
+/*
+static void dumpKey ( tpr09HashValue data_p, char* praefix){
+  tpr01_CursorDesc_CursorID *cursorID = (struct tpr01_CursorDesc_CursorID *)data_p;
+  if ( cursorID->UniqueID == EmptyUniqueID_cpr01) {
+    printDump("%s \"%.*s\"\n", praefix, cursorID->cursorName->cbLen, cursorID->cursorName->rawString);
+  } else {
+    printDump("%s \":%#08lx\"\n", praefix, cursorID->UniqueID);
+  }
+}
+*/
+#endif 
 
 /*==================================================================*
  *  CODE                                                            *
@@ -107,16 +126,28 @@ static void pr01Cursor_InitCursorName (tpr00_CursorNameDesc *CursorNameDesc);
 
 static void pr01CursorInitCont(tpr01_CursorContainer *Cursor)
 {
+#ifdef DYNAHASH  
+  Cursor->Desc = pr09NewHashDescriptor(20, 
+                                       sizeof(tpr01_CursorDesc), 
+                                       pr01Cursor_CompareKey, 
+                                       pr01Cursor_hashFunc, 
+                                       pr01Cursor_GetKey,
+                                       pr01Cursor_printKey);
+#else
   Cursor->Desc = pr09NewDescriptor(3, sizeof(tpr01_CursorDesc));
+#endif 
+
+  Cursor->cachedCursorID.UniqueID = EmptyUniqueID_cpr01;
+  Cursor->cachedCursorID.cursorName = NULL;     
   Cursor->ErrorDesc = pr01NewErrorDesc();
   Cursor->InitCont  = pr01CursorInitCont;
   Cursor->OpenCont  = pr01CursorOpenCont;
   Cursor->CloseCont = pr01CursorCloseCont;
-  Cursor->GetCount  = pr01CursorGetCount;
+/*  Cursor->GetCount  = NULL; */
   Cursor->EnumDesc  = pr01CursorEnumDesc;
   Cursor->AddDesc   = pr01CursorAddDesc;
   Cursor->DeleteDesc = pr01CursorDeleteDesc;
-  Cursor->SetIndex  = pr01CursorSetIndex;
+  Cursor->ResetIterator = pr01CursorResetIterator;
   Cursor->FindDesc  = pr01CursorFindDesc;
   Cursor->Execute   = pr01CursorExecute;
   Cursor->Declare   = pr01CursorDeclare;
@@ -138,7 +169,11 @@ void pr01CursorDeleteCont(tpr01_CursorContainer *Cursor)
 	Cursor->DeleteDesc(CursorDesc);
     }
     pr01DeleteErrorDesc(Cursor->ErrorDesc);    
+#ifdef DYNAHASH  
+    pr09DeleteHashDescriptor(Cursor->Desc);
+#else
     pr09DeleteDescriptor(Cursor->Desc);
+#endif 
     Cursor->Desc = NULL;
     pr03mFree((tsp00_Uint1*)Cursor, "Cursor");
   }
@@ -156,28 +191,46 @@ tpr01_CursorContainer *pr01CursorNewCont()
 
 static tsp00_Int4 pr01CursorOpenCont(tpr01_CursorContainer *Cursor)
 {
-  if (!Cursor->Desc) {
+/*  if (!Cursor->Desc) {
     Cursor->Desc = pr09NewDescriptor(10, sizeof(tpr01_CursorDesc));
   }
-  pr09SetIndex(Cursor->Desc, -1);
-  return(Cursor->GetCount(Cursor));
+*/
+    if (Cursor->Desc) {
+#ifdef DYNAHASH  
+    pr09ResetHashItemIterator(Cursor->Desc);
+#else
+    pr09SetIndex(Cursor->Desc, -1);
+#endif 
+    return 1;
+  } else {
+    return 0;
+  }
 }
 
+/*
 static tsp00_Int4 pr01CursorGetCount(tpr01_CursorContainer *Cursor)
 {
   return(pr09GetItemCount(Cursor->Desc));
 }
-
+*/
 static tpr01_CursorDesc *pr01CursorEnumDesc(tpr01_CursorContainer *Cursor)
 {
+#ifdef DYNAHASH  
+  tpr01_CursorDesc *CursorDesc=pr09HashItemIteratorNextItem(Cursor->Desc);
+#else
   tpr01_CursorDesc *CursorDesc=pr09GetNextItem(Cursor->Desc);
+#endif 
   return(CursorDesc);
 }
 
 static void pr01CursorCloseCont(tpr01_CursorContainer *Cursor)
 {
   M90TRACE(M90_TR_HANDLE, "pr01CursorCloseCont", Cursor);
+#ifdef DYNAHASH  
+  pr09DeleteAllHashItems(Cursor->Desc);
+#else
   pr09CloseAllItems(Cursor->Desc);
+#endif 
 }
 
 void pr01CursorInitAttr (tpr01_CursorDesc *CursorDesc)
@@ -194,7 +247,12 @@ static tpr01_CursorDesc *pr01CursorAddDesc(tpr01_CursorContainer *Cursor, tpr05_
   M90TRACE(M90_TR_ENTRY, "pr01CursorAddDesc", 0);
   {
     tpr01_CursorDesc *CursorDesc;      
+#ifdef DYNAHASH  
+    tpr09_HashTableError hTabErr; 
+    CursorDesc = pr09GetNewHashItem(Cursor->Desc);
+#else
     CursorDesc = pr09AddItem(Cursor->Desc);
+#endif 
     CursorDesc->DescType = CursorDesc_epr01;
     CursorDesc->Cursor=Cursor;
     CursorDesc->ConDesc      = ConDesc;
@@ -206,26 +264,37 @@ static tpr01_CursorDesc *pr01CursorAddDesc(tpr01_CursorContainer *Cursor, tpr05_
     CursorDesc->ErrorDesc    = pr01NewErrorDesc();
     CursorDesc->cue = pr03mAllocat(sizeof(sqlcuentry), "CursorDesc->cu");
     pr03cCuInit(CursorDesc->cue);
-    CursorDesc->UniqueID = UniqueID;
     Cursor->InitCursorName(&(CursorDesc->CursorNameDesc));
     if (UniqueID != EmptyUniqueID_cpr01) {
       /*generate an implizit cursor name*/
+      CursorDesc->CursorNameDesc.CursorName.cbLen = 
       sp77sprintfUnicode (CursorDesc->CursorNameDesc.CursorName.encodingType,
                           CursorDesc->CursorNameDesc.CursorName.rawString,
-                          CursorDesc->CursorNameDesc.CursorName.cbLen,
+                          CursorDesc->CursorNameDesc.CursorName.cbMaxLen,
                           ":%#08lx", 
                           UniqueID);
     }
     else {
       /*take the given cursor name*/  
-      tpr05_StringError erg = pr05IfCom_String_Convert (&(CursorDesc->CursorNameDesc.CursorName),
-                                                        CursorName );
+      CursorDesc->CursorNameDesc.CursorName.cbLen = CursorName->cbLen;
+      CursorDesc->CursorNameDesc.CursorName.encodingType = CursorName->encodingType;
+      memcpy(CursorDesc->CursorNameDesc.CursorName.rawString, CursorName->rawString, CursorName->cbLen);
     }
+    CursorDesc->CursorID.UniqueID = UniqueID;
+    CursorDesc->CursorID.cursorName = &(CursorDesc->CursorNameDesc.CursorName);
+
+#ifdef DYNAHASH  
+    /*add the descriptor to the cursor container hash index*/
+    hTabErr = pr09AddHashItemToHash(Cursor->Desc, 
+                                    &(CursorDesc->CursorID), 
+                                    sizeof(tpr01_CursorDesc_CursorID),
+                                    (tpr09HashValue*)CursorDesc);
+#endif 
+
     CursorDesc->SetAttr = pr01CursorSetAttr;
     CursorDesc->GetAttr = pr01CursorGetAttr;
     pr01CursorInitAttr (CursorDesc);
-    /* grep shows that CursorDesc->cue->cuname is no longer required
-      pr07C2P(CursorDesc->cue->cuname, szCursorName, sizeof(CursorDesc->cue->cuname));*/
+         
     M90TRACE(M90_TR_EXIT, "pr01CursorAddDesc", 0);
     return(CursorDesc);
   }
@@ -233,31 +302,65 @@ static tpr01_CursorDesc *pr01CursorAddDesc(tpr01_CursorContainer *Cursor, tpr05_
 
 static void pr01CursorDeleteDesc(tpr01_CursorDesc *CursorDesc)
 {
+#ifdef DYNAHASH  
+  if (NULL != pr09FindHashItem(CursorDesc->Cursor->Desc, 
+                               &(CursorDesc->CursorID), 
+                               sizeof(tpr01_CursorDesc_CursorID))) {  
+
+#else
   int cbIndex = pr09ItemFindIndex(CursorDesc->Cursor->Desc, CursorDesc);
   if (cbIndex >= 0) {
+#endif 
+
+    tpr05_String* cName = &(CursorDesc->CursorNameDesc.CursorName);                            
     pr01DeleteErrorDesc(CursorDesc->ErrorDesc);    
     if (CursorDesc->cue) {
       pr03mFree(CursorDesc->cue, "CursorDesc->cue");
     }
     /*delete cursor name and release memory*/
-    pr05IfCom_String_DeleterawString(&(CursorDesc->CursorNameDesc.CursorName));
+#ifdef DYNAHASH  
+    pr09DeleteHashItem(CursorDesc->Cursor->Desc, 
+                       &(CursorDesc->CursorID), 
+                       sizeof(tpr01_CursorDesc_CursorID));
+#else
     pr09CloseItemEx(CursorDesc->Cursor->Desc, cbIndex);
-  }
+#endif 
+    if (cName && cName->rawString) pr05IfCom_String_DeleterawString(cName);
+  } 
 }
 
-static tsp00_Int4 pr01CursorSetIndex(tpr01_CursorContainer *Cursor, tsp00_Int4 cbIndex)
-{
-  return(pr09SetIndex(Cursor->Desc, cbIndex));
+static void pr01CursorResetIterator(tpr01_CursorContainer *Cont){
+#ifdef DYNAHASH  
+  if (Cont && Cont->Desc) {pr09ResetHashItemIterator(Cont->Desc);}    
+#else
+  if (Cont && Cont->Desc) {pr09SetIndex(Cont->Desc, -1);}
+#endif 
 }
-
+#ifdef DYNAHASH  
 static tpr01_CursorDesc *pr01CursorFindDesc(tpr01_CursorContainer *Cursor, tpr05_String *CursorName, tpr01_UniqueID UniqueID)
 {
   M90TRACE(M90_TR_ENTRY, "pr01CursorFindDesc", 0);
   {
-    tsp00_Int4 cbOldIndex = Cursor->SetIndex(Cursor, -1);
+    tpr01_CursorDesc *CursorDesc = NULL;
+    pr01CursorResetIterator(Cursor);
+    Cursor->cachedCursorID.cursorName = CursorName;
+    Cursor->cachedCursorID.UniqueID   = UniqueID;
+    CursorDesc = (tpr01_CursorDesc *)pr09FindHashItem(  Cursor->Desc, 
+                                                        &(Cursor->cachedCursorID), 
+                                                        sizeof(tpr01_CursorDesc_CursorID));    
+    M90TRACE(M90_TR_EXIT, "pr01CursorFindDesc", 0);
+    M90TRACE(M90_TR_HANDLE, "CursorDesc", &CursorDesc);
+    return(CursorDesc);    
+  }
+}
+#else
+static tpr01_CursorDesc *pr01CursorFindDesc(tpr01_CursorContainer *Cursor, tpr05_String *CursorName, tpr01_UniqueID UniqueID)
+{
+  M90TRACE(M90_TR_ENTRY, "pr01CursorFindDesc", 0);
+  {
+    tsp00_Int4 cbOldIndex = pr09SetIndex(Cursor->Desc, -1);
     tpr01_CursorDesc *CursorDesc = NULL;
     if ( UniqueID == EmptyUniqueID_cpr01) {
-    /*no implicit cursor name*/    
       while(CursorDesc=Cursor->EnumDesc(Cursor)) {
 	    if (! pr05IfCom_String_strcmp(CursorName,&(CursorDesc->CursorNameDesc.CursorName)))
 	      break;
@@ -265,17 +368,18 @@ static tpr01_CursorDesc *pr01CursorFindDesc(tpr01_CursorContainer *Cursor, tpr05
     }
     else {
       while(CursorDesc=Cursor->EnumDesc(Cursor)) {
-	    if (UniqueID == CursorDesc->UniqueID)
+	    if (UniqueID == CursorDesc->CursorID.UniqueID)
 	      break;
       }
     }
     if(!CursorDesc)
-      Cursor->SetIndex(Cursor, cbOldIndex);    
+      pr09SetIndex(Cursor->Desc, cbOldIndex);    
     M90TRACE(M90_TR_EXIT, "pr01CursorFindDesc", 0);
     M90TRACE(M90_TR_HANDLE, "CursorDesc", &CursorDesc);
     return(CursorDesc);    
   }
 }
+#endif 
 
 static void pr01CursorExecute(tpr01_CursorDesc *CursorDesc)
 {
@@ -440,8 +544,21 @@ static void pr01CursorAlterCursorName (tpr01_CursorDesc *CursorDesc, tpr05_Strin
   PR07_CHECKDESC(CursorDesc, CursorDesc_epr01);
   {
     tpr05_StringError erg;
+#ifdef DYNAHASH  
+    tpr09HashIndex  hashIndex;
+    pr09HTRemoveHashItemByKey ( &(CursorDesc->Cursor->Desc->hTab), 
+                                &(CursorDesc->CursorID), 
+                                sizeof(tpr01_CursorDesc_CursorID));                   
+#endif 
     erg = pr05IfCom_String_Convert (&(CursorDesc->CursorNameDesc.CursorName),CursorName);
-    CursorDesc->UniqueID = EmptyUniqueID_cpr01;
+    CursorDesc->CursorID.UniqueID = EmptyUniqueID_cpr01;
+#ifdef DYNAHASH  
+    pr09HTInsertHashItemByValue (&(CursorDesc->Cursor->Desc->hTab), 
+                                 &(CursorDesc->CursorID), 
+                                 sizeof(tpr01_CursorDesc_CursorID),
+                                 CursorDesc, 
+                                 &hashIndex);
+#endif 
   }
 }
 
@@ -589,4 +706,54 @@ static void pr01Cursor_InitCursorName (tpr00_CursorNameDesc *CursorNameDesc)
                               sizeof(CursorNameDesc->private_StrBuf), 
                               sp77encodingAscii, 
                               constant_epr05);
+  CursorNameDesc->CursorName.cbLen=0;
 }
+
+#ifdef DYNAHASH  
+static tpr09HashValue pr01Cursor_GetKey (const tpr09HashValue data_p, tsp00_Uint4* HashKeyLen){
+  tpr01_CursorDesc_CursorID* cursorID=&((tpr01_CursorDesc *)data_p)->CursorID;
+  *HashKeyLen = sizeof(tpr01_CursorDesc_CursorID);
+  return cursorID;
+}
+
+static tsp00_Int4 pr01Cursor_CompareKey (const tpr09HashKey Item1,tsp00_Uint4 Item1Len, const tpr09HashKey Item2, tsp00_Uint4 Item2Len){
+  tpr01_CursorDesc_CursorID *cursorName1 = (tpr01_CursorDesc_CursorID*) Item1;
+  tpr01_CursorDesc_CursorID *cursorName2 = (tpr01_CursorDesc_CursorID*) Item2;
+
+  if (cursorName1->UniqueID == EmptyUniqueID_cpr01){ 
+    if (cursorName2->UniqueID == EmptyUniqueID_cpr01){
+       return  (! pr05IfCom_String_strcmp(cursorName1->cursorName, cursorName2->cursorName));
+    } else {
+      return 0;
+    }       
+  } else if (cursorName2->UniqueID != EmptyUniqueID_cpr01){
+    return (cursorName1->UniqueID == cursorName2->UniqueID)?1:0; 
+  }
+  return 0;
+}
+
+static void pr01Cursor_printKey (const tpr09HashValue data_p, FILE* datei){
+  tpr01_CursorDesc_CursorID *cursorID = &((struct tpr01_CursorDesc *)data_p)->CursorID;
+  if ( cursorID->UniqueID == EmptyUniqueID_cpr01) {
+    fprintf(datei, "<CursorDesc addr=0x%p name=\"%.*s\" />", data_p, cursorID->cursorName->cbLen, cursorID->cursorName->rawString);
+  } else {
+    fprintf(datei, "<CursorDesc addr=0x%p uniqueID=\"%d\" />", data_p, cursorID->UniqueID);
+  }
+}
+
+static tpr09HashIndex pr01Cursor_hashFunc (const tpr09HashKey HashKey, tsp00_Uint4 cbHashKeyLen, tsp00_Uint4 cbHashTabLen)
+{
+  tpr01_CursorDesc_CursorID *cursorID = (tpr01_CursorDesc_CursorID*) HashKey; 
+  if ( cursorID->UniqueID == EmptyUniqueID_cpr01) {
+    tsp00_Uint4 i;
+    tpr09HashIndex HashVal=0;
+    const char*	myKey = (char*) cursorID->cursorName->rawString;
+    cbHashKeyLen = cursorID->cursorName->cbLen;
+    for(i=0; i < cbHashKeyLen; i++)
+        if (myKey[i]) HashVal = (HashVal<<5) - HashVal + myKey[i];
+    return ((tpr09HashIndex)HashVal%cbHashTabLen);
+  } else {
+    return ((tpr09HashIndex)cursorID->UniqueID%cbHashTabLen);
+  }  
+}
+#endif 
