@@ -94,8 +94,6 @@ static void pr01ParseInfoHashTableInsert(tpr01_ParseInfoDesc *ParseInfoDesc);
 
 static void pr01ParseInfoAccessLRUItem(tpr01_ParseInfoDesc *SQLStmt);
 
-static tsp00_Int4 CompareSQLStatements (const tpr09HashValue Item1, const tpr09HashValue Item2);
-
 static void pr01ParseInfo_ParseInfosInit(tpr01_ParseInfoDesc *ParseInfoDesc);
 
 static void pr01ParseInfo_ParseInfosDelete(tpr01_ParseInfoDesc *ParseInfoDesc);
@@ -104,6 +102,12 @@ static void pr01ParseInfoPutParseInfos(tpr01_ParseInfoDesc *ParseInfoDesc, sqlor
 
 static void pr01ParseInfoGetParseInfos(tpr01_ParseInfoDesc *ParseInfoDesc, sqlorentry *orentry, sqlkaentry *kaentry);
 
+static tpr09HashValue pr01ParseInfo_GetKey (const tpr09HashValue data_p, tsp00_Uint4* HashKeyLen);
+
+static tsp00_Int4 pr01ParseInfo_CompareKey (const tpr09HashKey Item1,tsp00_Uint4 Item1Len, const tpr09HashKey Item2, tsp00_Uint4 Item2Len);
+
+static void pr01ParseInfo_printKey (const tpr09HashValue data_p, FILE* datei);
+
 /*==================================================================*
  *  CODE                                                            *
  *==================================================================*/
@@ -127,8 +131,11 @@ static void pr01ParseInfoInitCont(tpr01_ParseInfoContainer *Cont, tpr01_ConDesc
   myError = pr09HTCreateHashTable(&(Cont->HashTable),
                                   HT_capacity,
                                   HT_LoadFactor,
+                                  rehash_disable,
+                                  pr01ParseInfo_CompareKey,
                                   NULL,
-                                  NULL);
+                                  pr01ParseInfo_GetKey,
+                                  pr01ParseInfo_printKey);
   pr09LRUListInit(&(Cont->LRUList));
   Cont->CacheStat.requests        = 0;
   Cont->CacheStat.founds          = 0;
@@ -309,9 +316,13 @@ static tsp00_Int4 pr01ParseInfoDeleteDesc(tpr01_ParseInfoDesc *ParseInfoDesc, ts
          if (   ParseInfoDesc->State != uncached_epr01ParseInfo
              && ParseInfoDesc->State != created_epr01ParseInfo) {
          /*remove from hashtable and LRU List*/
+           tsp00_Uint4  hKeyLen;
+           tpr09HashKey hKey = pr01ParseInfo_GetKey((tpr09HashValue) ParseInfoDesc, &hKeyLen);
            pr09HTRemoveHashItemByIndex (&(ParseInfoDesc->ParseInfo->HashTable),
                                         ParseInfoDesc->HashIndex,
-                                        ParseInfoDesc);
+                                        hKey,
+                                        hKeyLen
+                                       );
            pr09LRUListRemoveItem(&(ParseInfoDesc->ParseInfo->LRUList), &(ParseInfoDesc->LRUListItem));
          }
          pr05IfCom_String_DeleteString(ParseInfoDesc->SQLStatement);
@@ -442,45 +453,38 @@ static tpr01_ParseInfoDesc *pr01ParseInfoFindDesc(tpr01_ParseInfoContainer *Pars
 {
   M90TRACE(M90_TR_ENTRY, "pr01ParseInfoFindDesc", 0);
   {
-    tpr09HashItem* tmpHashItem=NULL;
+    tpr09HashValue* tmpHashItem=NULL;
     tpr01_ParseInfoDesc *ParseInfoDesc=NULL;
-    tpr01_ParseInfoDesc tempParseInfoDesc;   /*only for comparsion of sql stmt in hash routine*/
-    tpr09CompareMethod oldCompareMethod = ParseInfo->HashTable.CompareMethod; /*storing current compare function*/
 
     ParseInfo->CacheStat.requests++;
 
     if (!SQLStatement && HashIndex < 0)
       return(NULL);
 
-    /*overwrite default compare function with a special function, that compares the SQL statements*/
-    ParseInfo->HashTable.CompareMethod = CompareSQLStatements;
-
-    tempParseInfoDesc.SQLStatement = SQLStatement;
-
     if (HashIndex < 0) {
         /* searching for SQL statement in hashtable. The SQL string will used as key */
         tmpHashItem = pr09HTGetHashItemByKey (&(ParseInfo->HashTable),
                                                (tpr09HashKey) SQLStatement->rawString,
-                                                SQLStatement->cbLen,
-                                               (tpr09HashValue)&tempParseInfoDesc);
+                                                SQLStatement->cbLen
+                                             );
     }
     else {
         /* searching for SQL statement in hashtable. The HashIndex will used as key */
         tmpHashItem = pr09HTGetHashItemByIndex (&(ParseInfo->HashTable),
                                                (tpr09HashIndex) HashIndex,
-                                               (tpr09HashValue)&tempParseInfoDesc);
+                                               (tpr09HashKey) SQLStatement->rawString,
+                                                SQLStatement->cbLen
+                                                );
     }
 
     if (tmpHashItem) {
-       ParseInfoDesc = (tpr01_ParseInfoDesc*) tmpHashItem->data_p;
+       ParseInfoDesc = (tpr01_ParseInfoDesc*) tmpHashItem;
        ParseInfo->CacheStat.founds++;
        if (ParseInfoDesc->State != created_epr01ParseInfo) {
          /* When ParseInfoDesc is hashed, put founded descriptor to end of LRU list */
          pr01ParseInfoAccessLRUItem(ParseInfoDesc);
        }
     }
-    /*reset hash table compare function to the default compare method*/
-    ParseInfo->HashTable.CompareMethod = oldCompareMethod;
     M90TRACE(M90_TR_EXIT, "pr01ParseInfoFindDesc", 0);
     M90TRACE(M90_TR_HANDLE, "ParseInfoDesc", &ParseInfoDesc);
     return(ParseInfoDesc);
@@ -580,8 +584,8 @@ tsp00_Int4 pr01ParseInfoParse(tpr01_SQLDesc *SQLDesc, tsp00_Int2 *cmdfetch, tsp0
       if (sqlca->sqlrap->raparse_again){
       /*after -8 parse again, update parseinfos in StmtNameDescs which use this cache entry*/
         tpr01_StmtNameContainer *StmtName = StmtNameDesc->StmtName;
-        tsp00_Int4 cbOldIndex = StmtName->SetIndex(StmtName, -1);
         tpr01_StmtNameDesc *tmpStmtNameDesc=NULL;
+        StmtName->ResetIterator(StmtName);
         while(tmpStmtNameDesc=StmtName->EnumDesc(StmtName)) {
           /*loop over all StmtNameDesc*/
           if (tmpStmtNameDesc != StmtNameDesc
@@ -597,7 +601,6 @@ tsp00_Int4 pr01ParseInfoParse(tpr01_SQLDesc *SQLDesc, tsp00_Int2 *cmdfetch, tsp0
             }  
           }
 	}
-        StmtName->SetIndex(StmtName, cbOldIndex);
       }  
     }  
     M90TRACE(M90_TR_EXIT, "pr01ParseInfoParse", 0);
@@ -925,18 +928,24 @@ static void pr01ParseInfoHashTableInsert(tpr01_ParseInfoDesc *ParseInfoDesc)
                                                   (tpr09HashIndex*)&(ParseInfoDesc->HashIndex));
         }
         else {
+            tsp00_Uint4  hKeyLen; 
+            tpr09HashKey hKey = pr01ParseInfo_GetKey ( (tpr09HashValue)ParseInfoDesc, &hKeyLen);
             myError= pr09HTInsertHashItemByIndex (&(ParseInfo->HashTable),
                                                   (tpr09HashIndex*)&(ParseInfoDesc->HashIndex),
+                                                  hKey,
+                                                  hKeyLen,
                                                   ParseInfoDesc);
-
         }
 
         if (myError==HashTableFull_epr09) {
-           /* hash table is full, remove the least recently used item from hash table */
            tpr01_ParseInfoDesc *displaceParseInfoDesc = (tpr01_ParseInfoDesc*) (pr09LRUListRemove(&(ParseInfo->LRUList))->data_p);
+           tsp00_Uint4  hKeyLen; 
+           tpr09HashKey hKey = pr01ParseInfo_GetKey ( (tpr09HashValue)displaceParseInfoDesc, &hKeyLen);
+           /* hash table is full, remove the least recently used item from hash table */
            pr09HTRemoveHashItemByIndex (&(ParseInfo->HashTable),
                                         displaceParseInfoDesc->HashIndex,
-                                        displaceParseInfoDesc);
+                                        hKey,
+                                        hKeyLen);
            displaceParseInfoDesc->State = uncached_epr01ParseInfo;
            ParseInfo->DeleteDesc(displaceParseInfoDesc, DROP_PARSEID);
         }
@@ -963,34 +972,21 @@ static void pr01ParseInfoAccessLRUItem(tpr01_ParseInfoDesc *ParseInfoDesc)
     return;
 }
 
-/*!
-  Function: CompareSQLStatements
+static tpr09HashValue pr01ParseInfo_GetKey (const tpr09HashValue data_p, tsp00_Uint4* HashKeyLen){
+  tpr05_String* string1=(tpr05_String*)((tpr01_ParseInfoDesc*)data_p)->SQLStatement;
+  *HashKeyLen = string1->cbLen;
+  return string1->rawString;
+}
 
-  see also:
+static tsp00_Int4 pr01ParseInfo_CompareKey (const tpr09HashKey Item1,tsp00_Uint4 Item1Len, const tpr09HashKey Item2, tsp00_Uint4 Item2Len){
+   if (Item1Len != Item2Len)
+     return 0;
+   return (! memcmp(Item1, Item2, Item1Len));
+}
 
-  Description: compares the SQL-String of two SQLStmtDesc
-
-  Arguments:
-    Item1   [in] hash value
-    Item2   [in] hash value
-
-  returns: 1 if Item1 and Item2 are the same
- */
-static tsp00_Int4 CompareSQLStatements (const tpr09HashValue Item1, const tpr09HashValue Item2)
-{
-    tpr05_String* string1=(tpr05_String*)((tpr01_ParseInfoDesc*)Item1)->SQLStatement;
-    tpr05_String* string2=(tpr05_String*)((tpr01_ParseInfoDesc*)Item2)->SQLStatement;
-
-    if (string1->encodingType == string2->encodingType)
-    {
-        if (string1->cbLen != string2->cbLen)
-        {
-            return 0;
-        }
-        return (! memcmp(string1->rawString, string2->rawString, string1->cbLen));
-    }
-    else
-      return (0);
+static void pr01ParseInfo_printKey (const tpr09HashValue data_p, FILE* datei){
+  tpr05_String* string1=(tpr05_String*)((tpr01_ParseInfoDesc*)data_p)->SQLStatement;
+  fprintf(datei, "<ParseInfoDesc addr=0x%p name=\"%.*s\" />", data_p, string1->cbLen, string1->rawString);
 }
 /*===========================================================================*
  *  END OF CODE                                                              *
