@@ -68,6 +68,7 @@
 #include "vpr01EXECSQL.h"
 #include "vpr01Trace.h"
 #include <stdio.h>
+#include <string.h>
 /*==================================================================*
  *  DECLARATIONS                                                    *
  *==================================================================*/
@@ -90,13 +91,11 @@ extern tpr01_EnvContainer *sqlEnvCont;
 
 static tsp00_Int4 pr01PrecomOpenCont(tpr01_PrecomContainer *Cont);
 
-static tsp00_Int4 pr01PrecomGetCount(tpr01_PrecomContainer *Cont);
-
 static void pr01PrecomCloseCont(tpr01_PrecomContainer *Cont);
 
-static tpr01_PrecomDesc *pr01PrecomEnumDesc(tpr01_PrecomContainer *Cont);
+static void pr01PrecomResetIterator(tpr01_PrecomContainer *Cont);
 
-static tsp00_Int4 pr01PrecomSetIndex(tpr01_PrecomContainer *Cont, tsp00_Int4 cbIndex);
+static tpr01_PrecomDesc *pr01PrecomEnumDesc(tpr01_PrecomContainer *Cont);
 
 static tpr01_PrecomDesc *pr01PrecomAddDesc(tpr01_PrecomContainer *Cont, tsp00_Int4 CmdNo);
 
@@ -166,6 +165,20 @@ static void pr01PrecomAbortSession(tpr01_PrecomDesc *PrecomDesc, tpr01_ModuleDes
 
 static pr01PrecomRuntimeError(tpr01_PrecomDesc *PrecomDesc, tpr01_ConDesc *ConDesc, enum tpr_runtime_errors_Enum error);
 
+#ifdef DYNAHASH  
+static tpr09HashValue pr01Precom_GetKey (const tpr09HashValue data_p, tsp00_Uint4* HashKeyLen);
+
+static tsp00_Int4 pr01Precom_CompareKey (const tpr09HashKey Item1,tsp00_Uint4 Item1Len, const tpr09HashKey Item2, tsp00_Uint4 Item2Len);
+
+static void pr01Precom_printKey (const tpr09HashValue data_p, FILE* datei);
+
+static tpr09HashIndex pr01Precom_HashFunc (const tpr09HashKey HashKey, tsp00_Uint4 cbHashKeyLen, tsp00_Uint4 cbHashTabLen);
+/*
+static void dumpKey ( tpr09HashValue data_p, char* praefix){
+  printDump("%s \"CmdNo=%04d\"\n", praefix, *((tsp00_Int4*)data_p));
+}
+*/
+#endif 
 /*==================================================================*
  *  CODE                                                            *
  *==================================================================*/
@@ -180,7 +193,11 @@ void pr01PrecomDeleteCont(tpr01_PrecomContainer *Precom)
     }
     pr01SQLDeleteCont(Precom->SQL);
     pr01DeleteErrorDesc(Precom->ErrorDesc);
+#ifdef DYNAHASH  
+    pr09DeleteHashDescriptor(Precom->Desc);
+#else
     pr09DeleteDescriptor(Precom->Desc);
+#endif 
     Precom->Desc = NULL;
     pr03mFree((tsp00_Uint1*)Precom, "Precom");
   }
@@ -191,7 +208,17 @@ void pr01PrecomDeleteCont(tpr01_PrecomContainer *Precom)
 static void pr01PrecomInitCont(tpr01_PrecomContainer *Precom, tpr01_ModuleDesc *ModuleDesc, sqlcatype *sqlca, sqlxatype *sqlxa)
 {
   M90TRACE(M90_TR_ENTRY, "pr01PrecomInitCont", 0);
+#ifdef DYNAHASH  
+  Precom->Desc = pr09NewHashDescriptor(20, 
+                                       sizeof(tpr01_PrecomDesc), 
+                                       pr01Precom_CompareKey, 
+                                       pr01Precom_HashFunc, 
+                                       pr01Precom_GetKey,
+                                       pr01Precom_printKey);
+#else
   Precom->Desc = pr09NewDescriptor(3, sizeof(tpr01_PrecomDesc));
+#endif 
+
   Precom->ModuleDesc = ModuleDesc;
   Precom->ErrorDesc  = pr01NewErrorDesc();
   Precom->SQL        = pr01SQLNewCont(ModuleDesc->Module->EnvDesc);
@@ -199,8 +226,7 @@ static void pr01PrecomInitCont(tpr01_PrecomContainer *Precom, tpr01_ModuleDesc *
   Precom->sqlca = sqlca;
   Precom->OpenCont        = pr01PrecomOpenCont;
   Precom->CloseCont       = pr01PrecomCloseCont;
-  Precom->GetCount        = pr01PrecomGetCount;
-  Precom->SetIndex        = pr01PrecomSetIndex;
+  Precom->ResetIterator   = pr01PrecomResetIterator;
   Precom->EnumDesc        = pr01PrecomEnumDesc;
   Precom->AddDesc         = pr01PrecomAddDesc;
   Precom->DeleteDesc      = pr01PrecomDeleteDesc;
@@ -245,13 +271,16 @@ tpr01_PrecomContainer *pr01PrecomNewCont(struct tpr01_ModuleDesc *ModuleDesc)
 
 static tsp00_Int4 pr01PrecomOpenCont(tpr01_PrecomContainer *Precom)
 {
-  M90TRACE(M90_TR_ENTRY, "pr01PrecomOpenCont", 0);
-  if (!Precom->Desc) {
-    Precom->Desc = pr09NewDescriptor(3, sizeof(tpr01_PrecomDesc));
+  if (Precom->Desc) {
+#ifdef DYNAHASH  
+    pr09ResetHashItemIterator(Precom->Desc);
+#else
+    pr09SetIndex(Precom->Desc, -1);
+#endif 
+    return 1;
+  } else {
+    return 0;
   }
-  pr09SetIndex(Precom->Desc, -1);
-  M90TRACE(M90_TR_EXIT, "pr01PrecomOpenCont", 0);
-  return(Precom->GetCount(Precom));
 }
 
 /*---------------------------------------------------------------------------*/
@@ -259,8 +288,13 @@ static tsp00_Int4 pr01PrecomOpenCont(tpr01_PrecomContainer *Precom)
 static tpr01_PrecomDesc *pr01PrecomAddDesc(tpr01_PrecomContainer *Precom, tsp00_Int4 CmdNo)
 {
   tpr01_PrecomDesc *PrecomDesc;
-  M90TRACE(M90_TR_ENTRY, "pr01PrecomAddDesc", 0);
+#ifdef DYNAHASH  
+  tpr09_HashTableError hTabErr; 
+  PrecomDesc = pr09GetNewHashItem(Precom->Desc);
+#else
   PrecomDesc = pr09AddItem(Precom->Desc);
+#endif 
+  M90TRACE(M90_TR_ENTRY, "pr01PrecomAddDesc", 0);
   PrecomDesc->DescType = PrecomDesc_epr01;
   PrecomDesc->Precom   = Precom;
   PrecomDesc->SQLDesc  = NULL;
@@ -274,6 +308,14 @@ static tpr01_PrecomDesc *pr01PrecomAddDesc(tpr01_PrecomContainer *Precom, tsp00_
     PrecomDesc->pr = NULL;
   }
   PrecomDesc->CmdNo            = CmdNo;
+#ifdef DYNAHASH  
+   /*add the descriptor to the statement container hash index*/
+  hTabErr = pr09AddHashItemToHash(Precom->Desc, 
+                                  &CmdNo, 
+                                  sizeof(tsp00_Int4),
+                                  (tpr09HashValue*)PrecomDesc);
+#endif 
+
   PrecomDesc->ErrorDesc        = pr01NewErrorDesc();
   PrecomDesc->StmtNameDesc     = NULL;
   PrecomDesc->ComType = PrecomDesc->ka->katyp;
@@ -285,28 +327,44 @@ static tpr01_PrecomDesc *pr01PrecomAddDesc(tpr01_PrecomContainer *Precom, tsp00_
 
 static void pr01PrecomDeleteDesc(tpr01_PrecomDesc *PrecomDesc)
 {
+#ifdef DYNAHASH  
+  if (NULL != pr09FindHashItem(PrecomDesc->Precom->Desc, 
+                               &PrecomDesc->CmdNo, 
+                               sizeof(tsp00_Int4)) ){
+#else
   int cbIndex = pr09ItemFindIndex(PrecomDesc->Precom->Desc, PrecomDesc);
   if (cbIndex >= 0) {
+#endif 
     pr01DeleteErrorDesc(PrecomDesc->ErrorDesc);
+#ifdef DYNAHASH  
+    pr09DeleteHashItem(PrecomDesc->Precom->Desc, 
+                               &PrecomDesc->CmdNo, 
+                               sizeof(tsp00_Int4));
+#else
     pr09CloseItemEx(PrecomDesc->Precom->Desc, cbIndex);
+#endif 
   }
 }
 
 /*---------------------------------------------------------------------------*/
 
-static tsp00_Int4 pr01PrecomGetCount(tpr01_PrecomContainer *Precom)
-{
-  M90TRACE(M90_TR_ENTRY, "pr01PrecomGetCount", 0);
-  M90TRACE(M90_TR_EXIT, "pr01PrecomGetCount", 0);
-  return(pr09GetItemCount(Precom->Desc));
+static void pr01PrecomResetIterator(tpr01_PrecomContainer *Cont){
+#ifdef DYNAHASH  
+  if (Cont && Cont->Desc) pr09ResetHashItemIterator(Cont->Desc);
+#else
+  if (Cont && Cont->Desc) pr09SetIndex(Cont->Desc, -1);
+#endif 
 }
-
 /*---------------------------------------------------------------------------*/
 
 static tpr01_PrecomDesc *pr01PrecomEnumDesc(tpr01_PrecomContainer *Precom)
 {
   {
-    tpr01_PrecomDesc *NextDesc=pr09GetNextItem(Precom->Desc);
+#ifdef DYNAHASH  
+    tpr01_PrecomDesc *NextDesc = pr09HashItemIteratorNextItem(Precom->Desc);
+#else
+    tpr01_PrecomDesc *NextDesc = pr09GetNextItem(Precom->Desc);
+#endif 
     M90TRACEIF(M90_TR_DWORD, "pr01PrecomEnumDesc", (NextDesc), &NextDesc->CmdNo);
     return(NextDesc);
   }
@@ -317,26 +375,29 @@ static tpr01_PrecomDesc *pr01PrecomEnumDesc(tpr01_PrecomContainer *Precom)
 static void pr01PrecomCloseCont(tpr01_PrecomContainer *Cont)
 {
   M90TRACE(M90_TR_ENTRY, "pr01PrecomCloseCont", 0);
+#ifdef DYNAHASH  
+  pr09DeleteAllHashItems(Cont->Desc);
+#else
   pr09CloseAllItems(Cont->Desc);
+#endif 
   M90TRACE(M90_TR_EXIT, "pr01PrecomCloseCont", 0);
 }
 
 /*---------------------------------------------------------------------------*/
-
-static tsp00_Int4 pr01PrecomSetIndex(tpr01_PrecomContainer *Cont, tsp00_Int4 cbIndex)
-{
-  tsp00_Int4 index
-  M90TRACE(M90_TR_ENTRY, "pr01PrecomSetIndex", 0);
-  index = pr09SetIndex(Cont->Desc, cbIndex);
-  M90TRACE(M90_TR_EXIT, "pr01PrecomSetIndex", 0);
-  return(index);
-}
-
-/*---------------------------------------------------------------------------*/
-
+#ifdef DYNAHASH  
 static tpr01_PrecomDesc *pr01PrecomFindDesc(tpr01_PrecomContainer *Cont, tsp00_Int4 CmdNo)
 {
-  tsp00_Int4 cbOldIndex = Cont->SetIndex(Cont, -1);
+  tpr01_PrecomDesc *PrecomDesc = (tpr01_PrecomDesc *)pr09FindHashItem(  Cont->Desc, 
+                                                            &CmdNo, 
+                                                            sizeof(tsp00_Int4)
+                                                          );
+  M90TRACE(M90_TR_HANDLE, "PrecomDesc", &PrecomDesc);
+  return(PrecomDesc);
+}
+#else
+static tpr01_PrecomDesc *pr01PrecomFindDesc(tpr01_PrecomContainer *Cont, tsp00_Int4 CmdNo)
+{
+  tsp00_Int4 cbOldIndex = pr09SetIndex(Cont->Desc, -1);
   tpr01_PrecomDesc *PrecomDesc=NULL;
   M90TRACE(M90_TR_ENTRY, "pr01PrecomFindDesc", 0);
   while(PrecomDesc=Cont->EnumDesc(Cont)) {
@@ -346,11 +407,13 @@ static tpr01_PrecomDesc *pr01PrecomFindDesc(tpr01_PrecomContainer *Cont, tsp00_I
     }
   }
   if (!PrecomDesc)
-    Cont->SetIndex(Cont, cbOldIndex);
+    pr09SetIndex(Cont->Desc, cbOldIndex);
   M90TRACE(M90_TR_EXIT, "pr01PrecomFindDesc", 0);
   M90TRACE(M90_TR_HANDLE, "PrecomDesc", &PrecomDesc);
   return(PrecomDesc);
 }
+#endif 
+
 
 /*---------------------------------------------------------------------------*/
 
@@ -441,6 +504,7 @@ static tpr01_UniqueID pr01PrecomGetCursorName(tpr01_PrecomDesc *PrecomDesc, tpr0
   PR07_CHECKDESC(PrecomDesc, PrecomDesc_epr01);
   M90TRACE(M90_TR_DWORD, "CmdNo", &PrecomDesc->CmdNo);
   {
+    tpr01_StmtNameDesc *StmtNameDesc=PrecomDesc->StmtNameDesc; 
     tpr01_PrecomContainer *Cont = PrecomDesc->Precom;
     sqlxatype *sqlxa = Cont->sqlxa;
     sqlprpointer pr = PrecomDesc->pr;
@@ -458,7 +522,7 @@ static tpr01_UniqueID pr01PrecomGetCursorName(tpr01_PrecomDesc *PrecomDesc, tpr0
       if (pr) 
       {
 	    len = (tsp00_Uint4) s30klen ((char*)pr->prCursorName, bsp_c1, sizeof(tsp00_KnlIdentifier));
-	    len = PR07MIN(len, CursorNameDesc->CursorName.cbLen);
+	    len = PR07MIN(len, CursorNameDesc->CursorName.cbMaxLen);
 	    M90TRACE(M90_TR_DWORD, "len", &len);
 	    if (len) 
 	    {
@@ -466,11 +530,11 @@ static tpr01_UniqueID pr01PrecomGetCursorName(tpr01_PrecomDesc *PrecomDesc, tpr0
                                                                    (char*)pr->prCursorName, 
                                                                    len,
                                                                    sp77encodingAscii);
-	    }
+	    } 
       }
       if (len == 0) 
       {
-	    if (PrecomDesc->StmtNameDesc) 
+	    if (StmtNameDesc) 
 	    {
 	        UniqueID = pr01PrecomUniqueID(PrecomDesc);
 	    }
@@ -1003,6 +1067,7 @@ static tpr01_CursorDesc *pr01PrecomGetCursorDesc(tpr01_PrecomDesc *PrecomDesc, t
     tpr01_PrecomContainer *Precom = PrecomDesc->Precom;
     tpr01_CursorContainer *Cursor = ModuleDesc->Cursor;
     tpr01_UniqueID UniqueID;
+    memset (CursorNameDesc.private_StrBuf, 0, sizeof(CursorNameDesc.private_StrBuf));
     Cursor->InitCursorName(&CursorNameDesc);
     UniqueID = pr01PrecomGetCursorName(PrecomDesc, &CursorNameDesc);
     M90TRACE(M90_TR_DWORD, "UniqueID", &UniqueID);
@@ -1201,7 +1266,7 @@ pr01PrecomDescribe(tpr01_PrecomDesc *PrecomDesc, tpr01_ModuleDesc *ModuleDesc, t
           pr05IfCom_String_InitString(pCursorName,
                                       &SQLDesc->StmtNameDesc->SQLStatement->rawString[SQLDesc->StmtNameDesc->StmtTypeDesc.CursorPos-1],
                                       SQLDesc->StmtNameDesc->StmtTypeDesc.CursorLength,
-                                      sp77encodingAscii,
+                                      SQLDesc->StmtNameDesc->SQLStatement->encodingType,
                                       constant_epr05);
           SQLDesc->CursorDesc = Cursor->FindDesc(Cursor, pCursorName, 0);
         }
@@ -2106,11 +2171,13 @@ static void pr01PrecomAfterExecute(tpr01_PrecomDesc *PrecomDesc, tpr01_ModuleDes
                 NOTE that the following statements assumes (and only work if) encoding type UTF8 
                 for Precom->sqlca->sqlresn*/ 
 	      pr07P2C(CursorNameDesc.CursorName.rawString, Precom->sqlca->sqlresn, sizeof(Precom->sqlca->sqlresn));
-	      pr07StripChar(CursorNameDesc.CursorName.rawString, " ");
-	      CursorNameDesc.CursorName.cbLen = (tsp00_Uint4)strlen(CursorNameDesc.CursorName.rawString);
+              pr07StripChar(CursorNameDesc.CursorName.rawString, " ");
+              CursorNameDesc.CursorName.cbLen = (tsp00_Uint4)strlen(CursorNameDesc.CursorName.rawString);
+              CursorNameDesc.CursorName.cbLen = (tsp00_Uint4)strlen(CursorNameDesc.CursorName.rawString);
 	      CursorDesc = Cursor->Declare(Cursor, SQLDesc->StmtNameDesc, ConDesc, &(CursorNameDesc.CursorName), EmptyUniqueID_cpr01);
 	      Cursor->PrepareOpen(CursorDesc, SQLDesc->ka);
-	    }
+	    } 
+
 	  }
 	  break;
 	}
@@ -2386,4 +2453,28 @@ static void pr01PrecomAbortSession(tpr01_PrecomDesc *PrecomDesc, tpr01_ModuleDes
   }
   M90TRACE(M90_TR_EXIT, "pr01PrecomAbortSession", 0);
 }
+#ifdef DYNAHASH  
+static tpr09HashValue pr01Precom_GetKey (const tpr09HashValue data_p, tsp00_Uint4* HashKeyLen){
+  *HashKeyLen = sizeof(tsp00_Int4);
+  return &(((struct tpr01_PrecomDesc *)data_p)->CmdNo);
+}
+
+static tsp00_Int4 pr01Precom_CompareKey (const tpr09HashKey Item1,tsp00_Uint4 Item1Len, const tpr09HashKey Item2, tsp00_Uint4 Item2Len){
+   tsp00_Int4 val1 = *((tsp00_Int4*)Item1);
+   tsp00_Int4 val2 = *((tsp00_Int4*)Item2);
+  
+   return (val1 == val2)?1:0;
+}
+
+static void pr01Precom_printKey (const tpr09HashValue data_p, FILE* datei){
+  tsp00_Int4 val1 = ((struct tpr01_PrecomDesc *)data_p)->CmdNo;
+  fprintf(datei, "<PrecomDesc addr=0x%p CmdNo=\"%d\" />", data_p, val1);
+}
+
+static tpr09HashIndex pr01Precom_HashFunc (const tpr09HashKey HashKey, tsp00_Uint4 cbHashKeyLen, tsp00_Uint4 cbHashTabLen)
+{
+  tsp00_Int4 val1 = *((tsp00_Int4*)HashKey);
+  return ((tpr09HashIndex)val1%cbHashTabLen);
+}
+#endif 
 
