@@ -623,7 +623,7 @@ void pr01SQLgetAllAttributes(struct tpr01_SQLDesc *SQLDesc)
             SQLDesc->Attribute.parspre  = TRUE;
             if ( SQLDesc->StmtNameDesc->StmtTypeDesc.StmtOption != KeywordNOT_FOUND_epr01
                 || (   SQLDesc->StmtNameDesc->StmtTypeDesc.UsingPos > 0         /*SELECT INTO*/
-                    && cpr_kind_oracle > pr03cMapSqlMode(sqlca->sqlrap->rakamode, sqlca->sqlrap->rasqlansi)
+                    && sp1sm_oracle > pr03cMapSqlMode(sqlca->sqlrap->rakamode, sqlca->sqlrap->rasqlansi)
                     )
                )
             /*a SELECT ... INTO can handled by database kernel in oracle mode only*/
