@@ -76,13 +76,11 @@ static void pr01StmtNameInitCont(tpr01_StmtNameContainer *Cont);
 
 static tsp00_Int4 pr01StmtNameOpenCont(tpr01_StmtNameContainer *Cont);
 
-static tsp00_Int4 pr01StmtNameGetCount(tpr01_StmtNameContainer *Cont);
-
 static void pr01StmtNameCloseCont(tpr01_StmtNameContainer *Cont);
 
 static tpr01_StmtNameDesc *pr01StmtNameEnumDesc(tpr01_StmtNameContainer *Cont);
 
-static tsp00_Int4 pr01StmtNameSetIndex(tpr01_StmtNameContainer *Cont, tsp00_Int4 cbIndex);
+static void pr01StmtNameResetIterator(tpr01_StmtNameContainer *Cont);
 
 static tpr01_StmtNameDesc *pr01StmtNameAddDesc(tpr01_StmtNameContainer *StmtName, tpr05_String *stStmtName, tpr01_UniqueID UniqueID, sqlkaentry *ka);
 
@@ -90,6 +88,8 @@ static void pr01StmtNameDeleteDesc(tpr01_StmtNameDesc *StmtNameDesc);
 
 static tpr01_StmtNameDesc *pr01StmtNameFindDesc(tpr01_StmtNameContainer *StmtName, tpr05_String *stStmtName, tpr01_UniqueID UniqueID);
 
+static tsp00_Int4 pr01StmtNameSetIndex(tpr01_StmtNameContainer *Cont, tsp00_Int4 cbIndex);
+
 static int pr01StmtNameGetSQL(tpr01_StmtNameDesc *StmtNameDesc, tpr05_String **pSQLStatement, tpr01_SQLStatementOptionEnum fOption);
 
 static int pr01StmtNamePutSQL(tpr01_StmtNameDesc *StmtNameDesc, tpr05_String *pSQLStatement, tpr01_SQLStatementOptionEnum fOption);
@@ -104,6 +104,25 @@ static void pr01StmtNameDropParsId(tpr01_StmtNameDesc *StmtNameDesc, tpr01_SQLDe
 
 static void pr01StmtName_InitStmtNameStruct (tpr00_StmtNameStruct *StmtNameStruct);
 
+#ifdef DYNAHASH  
+static tpr09HashValue pr01StmtName_GetKey (const tpr09HashValue data_p, tsp00_Uint4* HashKeyLen);
+
+static tsp00_Int4 pr01StmtName_CompareKey (const tpr09HashKey Item1,tsp00_Uint4 Item1Len, const tpr09HashKey Item2, tsp00_Uint4 Item2Len);
+
+static void pr01StmtName_printKey (const tpr09HashValue data_p, FILE* datei);
+
+static tpr09HashIndex pr01StmtName_hashFunc (const tpr09HashKey HashKey, tsp00_Uint4 cbHashKeyLen, tsp00_Uint4 cbHashTabLen);
+/*
+static void dumpKey ( tpr09HashValue data_p, char* praefix){
+  tpr01_StmtNameDesc_StmtID *stmtID = (struct tpr01_StmtNameDesc_StmtID *)data_p;
+  if ( stmtID->UniqueID == EmptyUniqueID_cpr01) {
+    printDump("%s \"%.*s\"\n", praefix, stmtID->stmtName->cbLen, stmtID->stmtName->rawString);
+  } else {
+    printDump("%s \":%#08lx\"\n", praefix, stmtID->UniqueID);
+  }
+}
+*/
+#endif 
 /*==================================================================*
  *  CODE                                                            *
  *==================================================================*/
@@ -121,12 +140,25 @@ static void pr01StmtName_InitStmtNameStruct (tpr00_StmtNameStruct *StmtNameStruc
  */
 static void pr01StmtNameInitCont(tpr01_StmtNameContainer *Cont)
 {
+#ifdef DYNAHASH  
+  Cont->Desc = pr09NewHashDescriptor(20, 
+                                     sizeof(tpr01_StmtNameDesc), 
+                                     pr01StmtName_CompareKey, 
+                                     pr01StmtName_hashFunc, 
+                                     pr01StmtName_GetKey,
+                                     pr01StmtName_printKey);
+#else
   Cont->Desc = pr09NewDescriptor(3, sizeof(tpr01_StmtNameDesc));
+#endif 
+                                     
+  Cont->cachedStmtID.UniqueID = EmptyUniqueID_cpr01;
+  Cont->cachedStmtID.stmtName = NULL;     
   Cont->InitCont           = pr01StmtNameInitCont;
   Cont->OpenCont           = pr01StmtNameOpenCont;
   Cont->CloseCont          = pr01StmtNameCloseCont;
-  Cont->GetCount           = pr01StmtNameGetCount;
+/*  Cont->GetCount           = NULL;*/
   Cont->SetIndex           = pr01StmtNameSetIndex;
+  Cont->ResetIterator      = pr01StmtNameResetIterator;
   Cont->EnumDesc           = pr01StmtNameEnumDesc;
   Cont->DeleteDesc         = pr01StmtNameDeleteDesc;
   Cont->AddDesc            = pr01StmtNameAddDesc;
@@ -160,8 +192,13 @@ void pr01StmtNameDeleteCont(tpr01_StmtNameContainer *StmtName)
       while(StmtNameDesc = pr01StmtNameEnumDesc(StmtName))
 	pr01StmtNameDeleteDesc(StmtNameDesc);
     }
-    if (StmtName->Desc)
+    if (StmtName->Desc){
+#ifdef DYNAHASH  
+      pr09DeleteHashDescriptor(StmtName->Desc);
+#else
       pr09DeleteDescriptor(StmtName->Desc);
+#endif 
+    }  
     pr03mFree((tsp00_Uint1*)StmtName, "StmtName");
   }
 }
@@ -179,9 +216,15 @@ tpr01_StmtNameContainer *pr01StmtNameNewCont()
 static tsp00_Int4 pr01StmtNameOpenCont(tpr01_StmtNameContainer *StmtName)
 {
   if (StmtName->Desc) {
+#ifdef DYNAHASH  
+    pr09ResetHashItemIterator(StmtName->Desc);
+#else
     pr09SetIndex(StmtName->Desc, -1);
+#endif 
+    return 1;
+  } else {
+    return 0;
   }
-  return(StmtName->GetCount(StmtName));
 }
 
 /*!
@@ -214,29 +257,44 @@ static tpr01_StmtNameDesc *pr01StmtNameAddDesc(tpr01_StmtNameContainer *StmtName
   M90TRACE(M90_TR_WORD, "kalineno", &ka->kalineno);
   {
     tpr01_StmtNameDesc *StmtNameDesc;  
+#ifdef DYNAHASH  
+    tpr09_HashTableError hTabErr; 
+    StmtNameDesc = pr09GetNewHashItem(StmtName->Desc);
+#else
     StmtNameDesc = pr09AddItem(StmtName->Desc);
+#endif 
     StmtNameDesc->DescType = StmtNameDesc_epr01;
     StmtNameDesc->StmtName = StmtName;
     StmtNameDesc->sn = pr03mAllocat(sizeof(sqlsnentry), "StmtNameDesc->sn");
     pr01StmtNameSnEntryInit(StmtNameDesc->sn);
-    StmtNameDesc->UniqueID = UniqueID;
     StmtNameDesc->StmtName->InitStmtNameStruct(&(StmtNameDesc->StmtNameStruct));
     if (UniqueID != EmptyUniqueID_cpr01) {
       /*generate an implizit statement name*/
+      StmtNameDesc->StmtNameStruct.stStmtName.cbLen = 
       sp77sprintfUnicode (StmtNameDesc->StmtNameStruct.stStmtName.encodingType,
                           StmtNameDesc->StmtNameStruct.stStmtName.rawString,
-                          StmtNameDesc->StmtNameStruct.stStmtName.cbLen,
+                          StmtNameDesc->StmtNameStruct.stStmtName.cbMaxLen,
                           ":%#08lx", 
                           UniqueID);
     }
     else {
       /*take the given statement name*/  
-      tpr05_StringError erg = pr05IfCom_String_Convert (&(StmtNameDesc->StmtNameStruct.stStmtName),
-                                                        stStmtName );
-
+            /*take the given cursor name*/  
+      StmtNameDesc->StmtNameStruct.stStmtName.cbLen = stStmtName->cbLen;
+      StmtNameDesc->StmtNameStruct.stStmtName.encodingType = stStmtName->encodingType;
+      memcpy(StmtNameDesc->StmtNameStruct.stStmtName.rawString, stStmtName->rawString, stStmtName->cbLen);
     }
-    /* grep shows that CursorDesc->cue->cuname is no longer required
-    pr07C2P(StmtNameDesc->sn->snname, StmtNameDesc->szStmtName, sizeof(StmtNameDesc->sn->snname));*/
+    StmtNameDesc->StmtID.UniqueID = UniqueID;
+    StmtNameDesc->StmtID.stmtName = &(StmtNameDesc->StmtNameStruct.stStmtName);
+    /*add the descriptor to the statement container hash index*/
+    
+#ifdef DYNAHASH  
+    hTabErr = pr09AddHashItemToHash(StmtName->Desc, 
+                                    &(StmtNameDesc->StmtID), 
+                                    sizeof(tpr01_StmtNameDesc_StmtID),
+                                    (tpr09HashValue*)StmtNameDesc);
+#endif
+    
     /* alloc & init new SQLStatetement */
     StmtNameDesc->SQLStatement = pr05IfCom_String_NewDynString(0, NULL);
     StmtNameDesc->fOption      = First_epr01;   /* type of SQLStatement is new Statement */
@@ -259,7 +317,8 @@ static tpr01_StmtNameDesc *pr01StmtNameAddDesc(tpr01_StmtNameContainer *StmtName
     StmtNameDesc->ore = pr03mAllocat(sizeof(sqlorentry), "StmtNameDesc->ore");
     pr03cOrInit(StmtNameDesc->ore);    
     StmtNameDesc->cue = pr03mAllocat(sizeof(sqlcuentry), "StmtNameDesc->cue");
-    pr03cCuInit(StmtNameDesc->cue);    
+    pr03cCuInit(StmtNameDesc->cue);
+    
     M90TRACE(M90_TR_EXIT, "pr01StmtNameAddDesc", 0);    
     M90TRACE(M90_TR_HANDLE, "StmtNameDesc", &StmtNameDesc);    
     return(StmtNameDesc);
@@ -268,8 +327,15 @@ static tpr01_StmtNameDesc *pr01StmtNameAddDesc(tpr01_StmtNameContainer *StmtName
 
 static void pr01StmtNameDeleteDesc(tpr01_StmtNameDesc *StmtNameDesc)
 {
+#ifdef DYNAHASH  
+  if (NULL != pr09FindHashItem(  StmtNameDesc->StmtName->Desc, 
+                                 &(StmtNameDesc->StmtID), 
+                                 sizeof(tpr01_StmtNameDesc_StmtID)
+                               )){
+#else
   int cbIndex = pr09ItemFindIndex(StmtNameDesc->StmtName->Desc, StmtNameDesc);
   if (cbIndex >= 0) {
+#endif
     PR07_HEAPCHECK(NULL);
     if (StmtNameDesc->sn)
       pr03mFree(StmtNameDesc->sn, "StmtNameDesc->sn");
@@ -297,14 +363,17 @@ static void pr01StmtNameDeleteDesc(tpr01_StmtNameDesc *StmtNameDesc)
       StmtNameDesc->ParseInfoDesc->ParseInfo->DeleteDesc(StmtNameDesc->ParseInfoDesc, DROP_PARSEID);
     PR07_HEAPCHECK(NULL);
 
+#ifdef DYNAHASH  
+    pr09DeleteHashItem(  StmtNameDesc->StmtName->Desc, 
+                         &(StmtNameDesc->StmtID), 
+                         sizeof(tpr01_StmtNameDesc_StmtID)
+                       );
+#else
     pr09CloseItemEx(StmtNameDesc->StmtName->Desc, cbIndex);
+#endif 
   }
 }
 
-static tsp00_Int4 pr01StmtNameGetCount(tpr01_StmtNameContainer *StmtName)
-{
-  return(pr09GetItemCount(StmtName->Desc));
-}
 
 /*!
   Function:     pr01StmtNameEnumDesc
@@ -326,20 +395,62 @@ static tsp00_Int4 pr01StmtNameGetCount(tpr01_StmtNameContainer *StmtName)
  */
 static tpr01_StmtNameDesc *pr01StmtNameEnumDesc(tpr01_StmtNameContainer *StmtName)
 {
+#ifdef DYNAHASH  
+  tpr01_StmtNameDesc *StmtNameDesc= pr09HashItemIteratorNextItem(StmtName->Desc);
+#else
   tpr01_StmtNameDesc *StmtNameDesc=pr09GetNextItem(StmtName->Desc);
+#endif 
   return(StmtNameDesc);
 }
 
 static void pr01StmtNameCloseCont(tpr01_StmtNameContainer *StmtName)
 {
+#ifdef DYNAHASH  
+  pr09DeleteAllHashItems(StmtName->Desc);
+#else
   pr09CloseAllItems(StmtName->Desc);
+#endif 
 }
 
 static tsp00_Int4 pr01StmtNameSetIndex(tpr01_StmtNameContainer *StmtName, tsp00_Int4 cbIndex)
 {
+#ifdef DYNAHASH  
+  return 0;
+#else
   return(pr09SetIndex(StmtName->Desc, cbIndex));
+#endif 
 }
 
+static void pr01StmtNameResetIterator(tpr01_StmtNameContainer *Cont){
+  if (Cont && Cont->Desc) {
+#ifdef DYNAHASH  
+    pr09ResetHashItemIterator(Cont->Desc);
+#else
+    pr09SetIndex(Cont->Desc, -1);
+#endif 
+  }
+}
+
+#ifdef DYNAHASH  
+static tpr01_StmtNameDesc *pr01StmtNameFindDesc(tpr01_StmtNameContainer *StmtName, tpr05_String *stStmtName, tpr01_UniqueID UniqueID)
+{
+  M90TRACE(M90_TR_ENTRY, "pr01StmtNameFindDesc", 0);
+  {
+    tpr01_StmtNameDesc *StmtNameDesc=NULL; 
+    
+    StmtName->cachedStmtID.stmtName = stStmtName;
+    StmtName->cachedStmtID.UniqueID = UniqueID;
+  
+    StmtNameDesc = (tpr01_StmtNameDesc *)pr09FindHashItem(  StmtName->Desc, 
+                                                            &(StmtName->cachedStmtID), 
+                                                            sizeof(tpr01_StmtNameDesc_StmtID)
+                                                          );    
+    M90TRACE(M90_TR_EXIT, "pr01StmtNameFindDesc", 0);
+    M90TRACE(M90_TR_HANDLE, "StmtNameDesc", &StmtNameDesc);
+    return(StmtNameDesc);    
+  }
+}
+#else
 static tpr01_StmtNameDesc *pr01StmtNameFindDesc(tpr01_StmtNameContainer *StmtName, tpr05_String *stStmtName, tpr01_UniqueID UniqueID)
 {
   M90TRACE(M90_TR_ENTRY, "pr01StmtNameFindDesc", 0);
@@ -355,11 +466,12 @@ static tpr01_StmtNameDesc *pr01StmtNameFindDesc(tpr01_StmtNameContainer *StmtNam
     }
     else {
       while(StmtNameDesc=StmtName->EnumDesc(StmtName)) {
-	if (UniqueID == StmtNameDesc->UniqueID) {
+	if (UniqueID == StmtNameDesc->StmtID.UniqueID) {
 	  break;
 	}
       }
     }
+
     if (!StmtNameDesc)
       StmtName->SetIndex(StmtName, cbOldIndex);
     M90TRACE(M90_TR_EXIT, "pr01StmtNameFindDesc", 0);
@@ -367,6 +479,7 @@ static tpr01_StmtNameDesc *pr01StmtNameFindDesc(tpr01_StmtNameContainer *StmtNam
     return(StmtNameDesc);    
   }
 }
+#endif 
 
 
 void pr01StmtNameSnEntryInit(sqlsnentry *sn)
@@ -627,6 +740,54 @@ static void pr01StmtName_InitStmtNameStruct (tpr00_StmtNameStruct *StmtNameStruc
                               sp77encodingAscii, 
                               constant_epr05);
 }
+#ifdef DYNAHASH  
+static tpr09HashValue pr01StmtName_GetKey (const tpr09HashValue data_p, tsp00_Uint4* HashKeyLen){
+  tpr01_StmtNameDesc_StmtID* stmtID=&((tpr01_StmtNameDesc *)data_p)->StmtID;
+  *HashKeyLen = sizeof(tpr01_StmtNameDesc_StmtID);
+  return stmtID;
+}
+
+static tsp00_Int4 pr01StmtName_CompareKey (const tpr09HashKey Item1,tsp00_Uint4 Item1Len, const tpr09HashKey Item2, tsp00_Uint4 Item2Len){
+  tpr01_StmtNameDesc_StmtID *stmtName1 = (tpr01_StmtNameDesc_StmtID*) Item1;
+  tpr01_StmtNameDesc_StmtID *stmtName2 = (tpr01_StmtNameDesc_StmtID*) Item2;
+
+  if (stmtName1->UniqueID == EmptyUniqueID_cpr01){ 
+    if (stmtName2->UniqueID == EmptyUniqueID_cpr01){
+       return  (! pr05IfCom_String_strcmp(stmtName1->stmtName, stmtName2->stmtName));
+    } else {
+      return 0;
+    }       
+  } else if (stmtName2->UniqueID != EmptyUniqueID_cpr01){
+    return (stmtName1->UniqueID == stmtName2->UniqueID)?1:0; 
+  }
+  return 0;
+}
+
+static void pr01StmtName_printKey (const tpr09HashValue data_p, FILE* datei){
+  tpr01_StmtNameDesc_StmtID *stmtID = &((struct tpr01_StmtNameDesc *)data_p)->StmtID;
+  if ( stmtID->UniqueID == EmptyUniqueID_cpr01) {
+    fprintf(datei, "\"%.*s\"\n", data_p, stmtID->stmtName->cbLen, stmtID->stmtName->rawString);
+  } else {
+    fprintf(datei,"\":%#08lx\"\n",stmtID->UniqueID);
+  }
+}
+
+static tpr09HashIndex pr01StmtName_hashFunc (const tpr09HashKey HashKey, tsp00_Uint4 cbHashKeyLen, tsp00_Uint4 cbHashTabLen)
+{
+  tpr01_StmtNameDesc_StmtID *stmtID = (tpr01_StmtNameDesc_StmtID*) HashKey; 
+  if ( stmtID->UniqueID == EmptyUniqueID_cpr01) {
+    tsp00_Uint4 i;
+    tpr09HashIndex HashVal=0;
+    const char*	myKey = (char*) stmtID->stmtName->rawString;
+    cbHashKeyLen = stmtID->stmtName->cbLen;
+    for(i=0; i < cbHashKeyLen; i++)
+        if (myKey[i]) HashVal = (HashVal<<5) - HashVal + myKey[i];
+    return ((tpr09HashIndex)HashVal%cbHashTabLen);
+  } else {
+    return ((tpr09HashIndex)stmtID->UniqueID%cbHashTabLen);
+  }  
+}
+#endif 
 
 /*===========================================================================*
  *  END OF CODE                                                              *
