@@ -658,7 +658,11 @@ void pr01cEnd(tpr01_SQLDesc *SQLDesc)
     	/* rette kommando fur sysprofile */
     	p03putsysprofbuf (sqlra);
       }
-    if (cu && cu->cuerrd3 == -100)
+    if (SQLDesc->StmtNameDesc &&
+	((SQLDesc->StmtNameDesc->StmtTypeDesc.StmtType == KeywordFETCH_epr01) ||
+	 (SQLDesc->StmtNameDesc->StmtTypeDesc.StmtType == KeywordFFETCH_epr01) ||
+	 (SQLDesc->StmtNameDesc->StmtTypeDesc.StmtType == KeywordMFETCH_epr01)) &&
+	(cu && cu->cuerrd3 == -100))
       {
     	sqlxa->xacmdinit = cpr_is_false;
     	sqlca->sqlerrd[2] = 0;
@@ -806,7 +810,7 @@ void pr01cOpenCursor(tpr01_SQLDesc *SQLDesc)
     sqlkaentry *ka   = SQLDesc->ka;
     sqlprentry *pr   = SQLDesc->pr;
     sqlgaentry *gae   = SQLDesc->ConDesc->ga;
-    tpr_runtime_errors_Enum erg;
+    tpr_runtime_errors_Enum erg = false;
     tsp00_KnlIdentifier pnam;
     tsp00_Int2 len= 0;
     char massopen;
@@ -1122,7 +1126,7 @@ tsp00_Int4 pr01cParse(tpr01_SQLDesc *SQLDesc, tsp00_Int2 *cmdfetch, tsp00_Int4 *
   sqlorentry *ore  = SQLDesc->ore;
   sqlcuentry *cue  = SQLDesc->cu;
   struct SQLERROR *sqlemp = sqlca->sqlemp;
-  tsp00_Int4 mass_cmd, datafound = *datafd;
+  tsp00_Int4 mass_cmd = false, datafound = *datafd;
   boolean IsParsed = false;
   tpr00_ParseId *selparsid = NULL;  
 #ifndef SQLODBC  
@@ -1194,6 +1198,7 @@ tsp00_Int4 pr01cParse(tpr01_SQLDesc *SQLDesc, tsp00_Int2 *cmdfetch, tsp00_Int4 *
       /*checking if open command has received result data and updateting faentry*/
       p01copendata(sqlca, sqlxa, ka, cue, gae, mass_cmd, &datafound, selparsid);
     }
+    if (SQLDesc->ComType != cpr_com_sql_close)
     if (!datafound) {
       p04sfi (sqlca, sqlxa, Describe_Params_epr00);
       if ((sqlca->sqldbmode != cpr_kind_internal) && SQLDesc->CursorDesc)
@@ -1358,14 +1363,15 @@ void pr01cUseNewParseInfo(tpr01_SQLDesc *SQLDesc)
 
     if (SQLDesc->StmtNameDesc) {
       tpr01_StmtNameContainer *StmtName = SQLDesc->StmtNameDesc->StmtName;
-      tsp00_Int4 cbOldIndex = StmtName->SetIndex(StmtName, -1);
       tpr01_StmtNameDesc *StmtNameDesc=NULL;
+      
+      StmtName->ResetIterator(StmtName);
       while(StmtNameDesc=StmtName->EnumDesc(StmtName)) {
         /*loop over all StmtNameDesc*/
         tpr01_ParseInfoContainer *ParseInfoCache = SQLDesc->ConDesc->ParseInfoCache;
         tpr00_ParseInfo *ParseInfo = &StmtNameDesc->ka->kaParseInfo;
 	sqlsnentry *sn  = StmtNameDesc->sn;
-        
+
         if (pr06ParseIdIsEqual(&OldParseInfo.ParseId, &ParseInfo->ParseId)) {
           /*StmtName with old parseid found -> update*/      
           pr06ParseInfoCopy(ParseInfo, &NewParseInfo);
@@ -1389,7 +1395,6 @@ void pr01cUseNewParseInfo(tpr01_SQLDesc *SQLDesc)
           }
 	}
       }
-      StmtName->SetIndex(StmtName, cbOldIndex);
     }
 
     if (SQLDesc->CursorDesc) {
