@@ -169,7 +169,7 @@ void pr02ConGetConParameter (sqlcatype *sqlca, sqlxatype *sqlxa,
         if (sqlv2.va2size == cpr_pointerlen)
           len = mxsp_buf;
         len = s30len ((unsigned char *) vaaddr, '\0', len);
-      } else if (sqlv2.va2typ == cpr_vucs2) {
+      } else if (sqlv2.va2typ == cpr_vucs2 || sqlv2.va2typ == cpr_vunicodec) {
         void *userpasswbuf;
         tsp00_Uint4 userlen;
 
@@ -197,7 +197,8 @@ void pr02ConGetConParameter (sqlcatype *sqlca, sqlxatype *sqlxa,
           (sqlv2.va2typ == cpr_vbchar) ||
           (sqlv2.va2typ == cpr_vcharc) ||
           (sqlv2.va2typ == cpr_vbuf) ||
-          (sqlv2.va2typ == cpr_vucs2)) {
+          (sqlv2.va2typ == cpr_vucs2)||
+          (sqlv2.va2typ == cpr_vunicodec)) {
         memcpy (userpassw, vaaddr, len);
       } else {
         if ((sqlv2.va2typ == cpr_vstring) ||
@@ -394,6 +395,7 @@ static void pr02fillxuserset(tsp4_xuser_record *xuserrec, tsp4_xuserset xuserset
   tsp00_Uint4 fillLength;
 
   memset (&zeroBytesC[0],'\0',sizeof(tsp00_KnlIdentifier));
+  memset (xuserset, '\0', sizeof(tsp4_xuserset));
   fillLength  = sizeof(tsp00_KnlIdentifier);
   fillPointer = &UniblankC[0];
   sp77encodingUCS2Native->fillString( &fillPointer, &fillLength, fillLength/2, ' ');
@@ -474,7 +476,7 @@ static pr02ConGetFromXUser(sqlcatype *sqlca, sqlgaentry *gae, BOOLEAN OldStyle)
   if (keyDefault) {
     /*KEY DEFAULT found*/
     tsp4_xuser_record  tmpxuserrec;
-    sqlindexuser(1, &tmpxuserrec, sqlemp->etext, &ok);
+    sqlindexuser(1, &tmpxuserrec, NULL, sqlemp->etext, &ok);
     if (ok) {
       tsp4_xuserset xuserset;
       pr02fillxuserset(&tmpxuserrec, xuserset);
@@ -514,7 +516,7 @@ static pr02ConGetFromXUser(sqlcatype *sqlca, sqlgaentry *gae, BOOLEAN OldStyle)
   }
   else {
     /*any key other than DEFAULT and SQLOPT*/    
-    sqlgetuser(gaxu, sqlemp->etext, &ok);
+    sqlgetuser(gaxu, NULL, sqlemp->etext, &ok);
     if (!ok) {
       sqlemp->elzu = sp1ce_notok;
       p03cseterror(sqlemp, cpr_xuser_file_open_error);
