@@ -71,6 +71,7 @@ static void pr03ConSessionInfoGet(tpr01_ConDesc *ConDesc);
 static void pr03ConMakeSQLStmt(tpr01_ConDesc *ConDesc, tpr05_String *pConString, struct SQLERROR *sqlemp);
 
 static void pr03ConMakePasswordPart(tpr01_ConDesc *ConDesc);
+static void pr03ConMakeFeaturePart(tpr01_ConDesc *ConDesc);
 
 
 /*==================================================================*
@@ -134,6 +135,7 @@ pr03ConSendConnect (struct tpr01_ConDesc *ConDesc)
 	Connection->InitPacket(ConDesc, PacketEncoding, sp1m_dbs);
 	pr03ConMakeConnectPart(ConDesc, sqlemp);
 	pr03ConMakePasswordPart(ConDesc);
+	pr03ConMakeFeaturePart(ConDesc);
 	/* p03putconnect(ConDesc->sqlxa->xaSQLDesc, sqlca->sqlrap, gaen, sqlemp); */
 	p03connectcmdtrace(sqlca->sqlrap, gaen);
 #ifdef SQLODBC
@@ -211,6 +213,44 @@ static void pr03ConSessionInfoGet(tpr01_ConDesc *ConDesc)
       pr03ConClientSessionID++;
       memcpy (&ga->gaKnlSessionID.ClientSession, &pr03ConClientSessionID, sizeof(pr03ConClientSessionID));      
     }
+    /* Database FeatureSet */
+    memcpy(ConDesc->DBFeatures, pr01ConFeatureSetEmpty, sizeof(ConDesc->DBFeatures));
+    if (!pr03PacketGetKernelParameters(segm, KernelFeatures_epr03, ConDesc->DBFeatures)) {
+      memcpy(ConDesc->DBFeatures, pr01ConFeatureSetEmpty, sizeof(ConDesc->DBFeatures));
+    }
+    if (pr01TraceIsTrace(sqlca->sqlrap))  {
+      boolean first = true; 
+      int i;
+      for (i=0;i < sizeof(tpr01Con_FeatureSet)/2;i++) {
+        if (!ConDesc->DBFeatures[i].value) 
+          continue;
+        if (first)
+          pr01TracePrintf(sqlca->sqlrap, "KERNEL FEATURES :");
+        first = false;
+        switch(ConDesc->DBFeatures[i].feature) {
+        case (sp1f_multiple_drop_parseid) : {
+          pr01TracePrintf(sqlca->sqlrap, "MULTI DROP PARSEID");
+          break;
+        }
+        case (sp1f_space_option) : {
+          pr01TracePrintf(sqlca->sqlrap, "SPACE OPTION");
+          break;
+        }
+        case (sp1f_variable_input) : {
+          pr01TracePrintf(sqlca->sqlrap, "VARIABLE INPUT");
+          break;
+        }
+        case (sp1f_optimized_streams) : {
+          pr01TracePrintf(sqlca->sqlrap, "OPTIMIZED STREAMS");
+          break;
+        }
+        default: {
+          pr01TracePrintf(sqlca->sqlrap, "UNKNOWN(%d)", ConDesc->DBFeatures[i].feature);
+          break;
+        }
+        }
+      }
+    }
   }
   M90TRACE (M90_TR_EXIT, "pr03ConSessionInfoGet", 0);
 }
@@ -237,6 +277,17 @@ static void pr03ConMakePasswordPart(tpr01_ConDesc *ConDesc)
 }
 
 
+static void pr03ConMakeFeaturePart(tpr01_ConDesc *ConDesc)
+{
+  tsp00_TermId tid;
+  tpr03_SegmDesc *SegmDesc = ConDesc->SegmDesc;
+  tsp1_part *partPtr = pr03SegmentAddPart(SegmDesc, sp1pk_feature);
+  pr03PartCopy(partPtr, (void*)ConDesc->DBFeatures, sizeof(ConDesc->DBFeatures));
+  partPtr->sp1p_part_header.sp1p_arg_count = sizeof(ConDesc->DBFeatures)/2;
+  pr03SegmentFinishPart(SegmDesc);
+}
+
+
 void pr03ConMakeConnectPart(tpr01_ConDesc *ConDesc, struct SQLERROR *sqlemp)
 {
   tpr03_SegmDesc *SegmDesc = ConDesc->SegmDesc;
