@@ -651,7 +651,7 @@ p04oradin (sqlcatype * sqlca, sqlxatype * sqlxa, sqlgaentry *gae, sqlcuentry *cu
 	  datapart->sp1p_part_header.sp1p_attributes[0] |= 1 << sp1pa_last_packet;
 	}
 	pr04LongILvcOneRecord (sqlca, sqlxa, gae, &parsid, datapart, cntpart);
-	if (sqlca->sqlemp->ereturncode < 0)
+	if ((sqlca->sqlemp->ereturncode != 0) && (sqlca->sqlemp->ereturncode != 100))
 	  return;
 	/* PTS 1102362 */
 	/* wenn Longspalten an den Kerngeschickt wurden konnen sich
@@ -1048,13 +1048,13 @@ p04sftoora (struct SQLROW *row, struct SQLCOL *col,
     break;
   }
   case dunicode: {
-    *l /= 2;
+    *l = (col->coliolen-1)/2;
     *t = SQL_ORA_UCS2;
     break;
   }
   case dstruni: {}
   case dlonguni: {
-    *l /= 2;
+    *l = (col->coliolen-1)/2;
     *t = SQL_ORA_LONG_UNICODE;
     break;
   }
