@@ -294,6 +294,7 @@ int pr05cAnalyseSQLStatement (tpr05_String *SQLStatement, tpr05_StmtAnalysisDesc
                 {
                     /*INTO was found - set using pos */
                     aStmtAnalysisDesc->UsingPos = SymbolPos;
+		    aStmtAnalysisDesc->StmtOption = myKeyword;
                 }    
                 
             } while (   Symbol    != CPR_S_EOF
@@ -355,13 +356,14 @@ int pr05cAnalyseSQLStatement (tpr05_String *SQLStatement, tpr05_StmtAnalysisDesc
                             while (1)
                             {
                                 pr05cNextSymbol(pStatement, StmtLength, pAnalysePos, &SymbolPos, &SymbolLength, &Symbol, &actPos, encodingType);
-                                pAnalysePos = actPos;
                                 if(Symbol == CPR_S_RIGHTPAR)
                                 {
-                                    parcnt--;
+                                  pAnalysePos = actPos;
+                                  parcnt--;
                                 }
                                 else 
                                 {
+                                  pAnalysePos = SymbolPos;
                                   break;
                                 }
                             }
