@@ -454,6 +454,9 @@ tpr05_StringError pr05IfCom_String_ConvertP (tpr05_String *dest,
 tpr05_StringError pr05IfCom_String_strcpy (      tpr05_String *dest,
                                            const tpr05_String *source)
 {
+    if (dest == source)
+        return (ok_epr05);
+
     switch(source->allocationType)
     { 
         case(allocated_epr05):
