@@ -705,21 +705,40 @@ int pr08cputStatementtoPart(tpr01_SQLDesc *SQLDesc)
                 }    
 
                 /*put first part (up to cursorname) of sqlStatement to part*/
-                if (StmtNameDesc->StmtTypeDesc.CursorType == CPR_S_PARAMETER_NAME)
-                /* cursor name comes from hostvariable */
-                {
-                    buflen = StmtNameDesc->StmtTypeDesc.CursorPos-1;  /*NOTE that CursorPos is pascal counting, starting with 1 not 0*/
+                /* Three cases - parameter name, identifier, unnamed result set. */
+                switch(StmtNameDesc->StmtTypeDesc.CursorType) {
+                case (CPR_S_PARAMETER_NAME): {
+                    buflen = StmtNameDesc->StmtTypeDesc.CursorPos-1;  
+                    /*NOTE that CursorPos is pascal counting, starting with 1 not 0*/
                     actPos = buflen + StmtNameDesc->StmtTypeDesc.CursorLength;
+                    break;
                 }
-                else
-                {
-                  if (StmtNameDesc->StmtTypeDesc.CursorType == CPR_S_IDENTIFIER) {
+                case (CPR_S_IDENTIFIER): {
                     pr05IfCom_String_toupperBuf(&StmtNameDesc->SQLStatement->rawString[StmtNameDesc->StmtTypeDesc.CursorPos-1], 
                                                 StmtNameDesc->StmtTypeDesc.CursorLength, 
                                                 StmtNameDesc->SQLStatement->encodingType);
-                  }
-                    buflen = StmtNameDesc->StmtTypeDesc.CursorPos+StmtNameDesc->StmtTypeDesc.CursorLength;
+                    buflen = StmtNameDesc->StmtTypeDesc.CursorPos + StmtNameDesc->StmtTypeDesc.CursorLength;
+                    /* We must use not only the cursor name, but also the space that 
+                       follows in the statement. Therefore we must add the size of a
+                       single char in the resp. encoding. */
+                    buflen += StmtNameDesc->SQLStatement->encodingType->fixedCharacterSize > 1 ? 1 : 0;
                     actPos = buflen;  
+                    break;
+                }
+                case (CPR_S_STRING_LITERAL): {
+                    buflen = StmtNameDesc->StmtTypeDesc.CursorPos + StmtNameDesc->StmtTypeDesc.CursorLength;
+                    /* We must use not only the cursor name, but also the space that 
+                       follows in the statement. Therefore we must add the size of a
+                       single char in the resp. encoding. */
+                    buflen += StmtNameDesc->SQLStatement->encodingType->fixedCharacterSize > 1 ? 1 : 0;
+                    actPos = buflen;  
+                    break;
+                }
+                default:
+                case (CPR_S_UNKNOWN): {
+                      buflen = StmtNameDesc->StmtTypeDesc.CursorPos;
+                      actPos = buflen;
+                }
                 }
                 erg = pr03PartConverttoPart(part_Ptr, 
                                             &offset,
@@ -954,7 +973,7 @@ int pr08cputStatementtoPart(tpr01_SQLDesc *SQLDesc)
      M90TRACE(M90_TR_EXIT, "pr08cPutStatementtoPart", 0);
   } /*M90Trace*/    
   return 0;
-}    
+}        
 
 void p08analyzeconnectstatm (sqlcatype *sqlca, sqlxatype *sqlxa, sqlgatype *sqlga, sqlgaentry *ga, sqlkaentry *ka, tsp00_Int4 buflen, char *buf)
 {
