@@ -94,10 +94,17 @@
  *===========================================================================*/
 /* array of prime numbers */
 static const tsp00_Uint4	PrimeNumbers[] = {
-  7,53, 97, 193, 389, 769, 1543, 3079, 6151, 12289, 24593, 49157, 98317,
-  196613, 393241, 786433, 1572869, 3145739, 6291469, 12582917, 25165843,
-  50331653, 100663319, 201326611, 402653189, 805306457, 1610612741,
-  3221225473, 4294967291, 0 };
+  7UL,53UL, 97UL, 193UL, 389UL, 769UL, 1543UL, 3079UL, 6151UL, 12289UL, 24593UL, 49157UL, 98317UL,
+  196613UL, 393241UL, 786433UL, 1572869UL, 3145739UL, 6291469UL, 12582917UL, 25165843UL,
+  50331653UL, 100663319UL, 201326611UL, 402653189UL, 805306457UL, 1610612741UL,
+  3221225473UL, 4294967291UL, 0 };
+
+
+static tpr09_HashTableError pr09HTRehash(tpr09HashTable*    HashTable);
+  
+static void defaultPrintHashItem (const tpr09HashValue data_p, FILE* datei){
+  fprintf(datei, "<hashitem addr=0x%p />", data_p);
+}
 
 /*!
   Function: defaultCompareMethod
@@ -112,7 +119,26 @@ static const tsp00_Uint4	PrimeNumbers[] = {
 
   returns: 1 if Item1 and Item2 are the same
  */
-static tsp00_Int4 defaultCompareMethod (const tpr09HashValue Item1, const tpr09HashValue Item2)
+static tpr09HashValue defaultKeyFunction (const tpr09HashValue data_p, tsp00_Uint4* HashKeyLen)
+{
+    *HashKeyLen = (tsp00_Uint4)strlen ((char*)data_p);
+    return data_p;
+}
+
+/*!
+  Function: defaultCompareMethod
+
+  see also:
+
+  Description: default compare method for comparsion of two hash values
+
+  Arguments:
+    Item1   [in] hash value
+    Item2   [in] hash value
+
+  returns: 1 if Item1 and Item2 are the same
+ */
+static tsp00_Int4 defaultCompareMethod (const tpr09HashKey Item1,tsp00_Uint4 Item1Len, const tpr09HashKey Item2, tsp00_Uint4 Item2Len)
 {
     return (Item1==Item2?1:0);
 }
@@ -131,13 +157,20 @@ static tsp00_Int4 defaultCompareMethod (const tpr09HashValue Item1, const tpr09H
 
   returns: hash index
  */
-static tpr09HashIndex defaultHashFunction (const tpr09HashKey * HashKey, tsp00_Uint4 cbHashKeyLen, tsp00_Uint4 cbHashTabLen)
+static tpr09HashIndex defaultHashFunction (const tpr09HashKey HashKey, tsp00_Uint4 cbHashKeyLen, tsp00_Uint4 cbHashTabLen)
 {
     tsp00_Uint4 i;
     tpr09HashIndex HashVal=0;
     const char*	myKey = (char*) HashKey;
+/*    printf("Hashfunc: TabLen->%u<\n", cbHashTabLen);
+    printf("Hashfunc: Length->%u<\n", cbHashKeyLen);
+    printf("Hashfunc: Value->%.*s<\n", cbHashKeyLen, HashKey);
+*/
     for(i=0; i < cbHashKeyLen; i++)
-        HashVal += (tpr09HashIndex) *(myKey++);
+        if (myKey[i]) HashVal = (HashVal<<5) - HashVal + myKey[i];
+/*      HashVal += (tpr09HashIndex) myKey[i];
+      printf("Hashfunc: %u\n", (tpr09HashIndex)HashVal%cbHashTabLen);
+*/
     return ((tpr09HashIndex)HashVal%cbHashTabLen);
 }
 
@@ -156,6 +189,7 @@ static tpr09HashIndex defaultHashFunction (const tpr09HashKey * HashKey, tsp00_U
  */
 static void pr09addtoFreeLst (tpr09HashItem * const HItem, tpr09HashTable * HashTable)
 {
+   HItem->data_p = NULL;
    HItem->next_p = HashTable->HashTableFreeLst.first_p;
    HashTable->HashTableFreeLst.first_p = HItem;
 }
@@ -187,25 +221,34 @@ static tpr09_HashTableError pr09getnewCollLstItem (tpr09HashItem ** const newIte
          from the heap and assign them to the free-list  */
       tsp00_Uint4	  i;
       tpr09HashItem * item_p;
-      tsp00_Int4 cbChunk = HashTable->cbMaxElem / HASH_FL_CHUNK_CNT + 1;
+
+      /* init */
+      if (! HashTableFreeLst->cbChunkCnt){
+        HashTableFreeLst->cbChunkCnt = HASH_FL_CHUNK_INC_CNT;
+        HashTableFreeLst->chunks = (tpr09HashItem**)pr03mAllocatT(HashTableFreeLst->cbChunkCnt*sizeof(void*), "HashTableChunkLst"); 
+        memset (HashTableFreeLst->chunks,0,HashTableFreeLst->cbChunkCnt*sizeof(void*));
+        if (! HashTableFreeLst->chunks) return AllocMemErr_epr09;  
+      }  
 
       /* search for the first "unused" chunk */
-      for ( i = 0; i < HASH_FL_CHUNK_CNT && HashTableFreeLst->chunks[i] != NULL; i++ )
+      for ( i = 0; i < HashTableFreeLst->cbChunkCnt && HashTableFreeLst->chunks[i] != NULL; i++ )
       {}
-      if ( i == HASH_FL_CHUNK_CNT )
+      if ( i == HashTableFreeLst->cbChunkCnt &&  HashTableFreeLst->chunks[i] != NULL)
       {
-         /* this must not ocur because it would indicate that more collision
-            list items are used than hashed objects are there */
-         return ( CollisionLstFull_epr09 );
+           HashTableFreeLst->chunks = (tpr09HashItem**)pr03mReallocat(HashTableFreeLst->chunks, (HashTableFreeLst->cbChunkCnt+HASH_FL_CHUNK_INC_CNT)*sizeof(void*), "HashTableChunkLst");
+           memset (&HashTableFreeLst->chunks[HashTableFreeLst->cbChunkCnt],0,HASH_FL_CHUNK_INC_CNT*sizeof(void*));
+           HashTableFreeLst->cbChunkCnt += HASH_FL_CHUNK_INC_CNT;
+           if (! HashTableFreeLst->chunks) return AllocMemErr_epr09;
+           i++; 
       }
-      HashTableFreeLst->chunks[i] = (tpr09HashItem*)pr03mCallocT(cbChunk, sizeof(tpr09HashItem), "HashTableFreeLst");
+      HashTableFreeLst->chunks[i] = (tpr09HashItem*)pr03mAllocatT(HASH_FL_CHUNK_SIZE, "HashTableFreeLst");
 
       /* set the 'next_p' of the last free-list element to NULL and
          and all the others to the respective next element in the
          allocated array of items; this is done from "right" to
         "left";  */
       HashTableFreeLst->first_p = HashTableFreeLst->chunks[i];
-      item_p = HashTableFreeLst->first_p + (cbChunk - 1);
+      item_p = (HashTableFreeLst->first_p) + ((HASH_FL_CHUNK_SIZE/sizeof(tpr09HashItem)) - 1);
       item_p->next_p = NULL;
       while ( item_p != HashTableFreeLst->first_p )
       {
@@ -226,11 +269,14 @@ static tpr09_HashTableError pr09getnewCollLstItem (tpr09HashItem ** const newIte
 /*===========================================================================*
  *  METHODS                                                                  *
  *===========================================================================*/
-tpr09_HashTableError pr09HTCreateHashTable(tpr09HashTable * HashTable,
-                                           tsp00_Int4 cbMaxElem,
-                                           tsp00_Int4 cbLoadFactor,
-                                           tpr09CompareMethod CompareMethod,
-                                           tpr09HashFunc HashFunc)
+tpr09_HashTableError pr09HTCreateHashTable(tpr09HashTable*    HashTable, 
+                                           tsp00_Int4         cbMaxElem, 
+                                           tsp00_Int4         cbLoadFactor, 
+                                           tpr09_HTRehash     rehash,
+                                           tpr09CompareMethod CompFunc, 
+                                           tpr09HashFunc      HashFunc, 
+                                           tpr09ExtractKey    KeyFunc,
+                                           tpr09PrintHashItem PrintFunc)
 {
    /* determine size of hash vectors used by the hash indexes with
       respect to the number of cbMaxElem */
@@ -248,8 +294,11 @@ tpr09_HashTableError pr09HTCreateHashTable(tpr09HashTable * HashTable,
    HashTable->cbMaxElem     = HashTable->cbMaxCapacity * LoadFactor / 100; /*number of maximal elements*/
    HashTable->cbLoadFactor  = LoadFactor;  /*measure of how full the hash table is allowed to get
                                              before its capacity is reached value between 1 and 100*/
-   HashTable->CompareMethod = (CompareMethod==NULL)?defaultCompareMethod:CompareMethod; /*method for comparsion of two Hashvalues*/
-   HashTable->HashFunc      = (HashFunc==NULL)?defaultHashFunction:HashFunc; /*method for comparsion of two Hashvalues*/
+   HashTable->rehash        = rehash;
+   HashTable->CompareMethod = (CompFunc  ==NULL)?defaultCompareMethod:CompFunc; /*method for comparsion of two Hashvalues*/
+   HashTable->HashFunc      = (HashFunc  ==NULL)?defaultHashFunction:HashFunc;  /*method for comparsion of two Hashvalues*/
+   HashTable->KeyFunc       = (KeyFunc   ==NULL)?defaultKeyFunction:KeyFunc;    /*method for comparsion of two Hashvalues*/
+   HashTable->PrintFunc     = (PrintFunc ==NULL)?defaultPrintHashItem:PrintFunc;
    HashTable->statistic.requests   = 0;  /* number of requests against hashtable */
    HashTable->statistic.founds     = 0;  /* number of hashvalues found in hashtable */
    HashTable->statistic.inserts    = 0;  /* number of inserted values */
@@ -261,15 +310,19 @@ tpr09_HashTableError pr09HTCreateHashTable(tpr09HashTable * HashTable,
    HashTable->pHashArray = (tpr09HashItem*) pr03mCallocT(HashTable->cbMaxCapacity, sizeof(tpr09HashItem), "pr09HashTable");
 
    /*initinal free list*/
-   memset (HashTable->HashTableFreeLst.chunks,0, sizeof(HashTable->HashTableFreeLst.chunks));
+   HashTable->HashTableFreeLst.chunks  = NULL;
    HashTable->HashTableFreeLst.first_p = NULL;
+   HashTable->HashTableFreeLst.cbChunkCnt = 0;
+
+   pr09HTResetIterator(HashTable);
 
    return (HashTable->pHashArray==NULL?AllocMemErr_epr09:ok_epr09);
 }
 
 void pr09HTDeleteHashTable(tpr09HashTable * HashTable)
 {
-   int i;
+   tsp00_Uint4 i;
+  
    /*reset HashTable record*/
    HashTable->cbMaxCapacity = 0;
    HashTable->cbCLIncCount  = 0;
@@ -278,6 +331,7 @@ void pr09HTDeleteHashTable(tpr09HashTable * HashTable)
    HashTable->cbLoadFactor  = 100;  /*measure of how full the hash table is allowed to get
                                       before its capacity is reached value between 1 and 100*/
    HashTable->CompareMethod = NULL; /*method for comparsion of two Hashvalues*/
+   HashTable->KeyFunc       = NULL; 
    HashTable->statistic.requests   = 0;  /* number of requests against hashtable */
    HashTable->statistic.founds     = 0;  /* number of hashvalues found in hashtable */
    HashTable->statistic.inserts    = 0;  /* number of inserted values */
@@ -286,21 +340,27 @@ void pr09HTDeleteHashTable(tpr09HashTable * HashTable)
    HashTable->statistic.weightedHitrate = 0;  /* weighted hitrate */
 
    /*free HashTableFreeLst*/
-   for ( i = 0; i < HASH_FL_CHUNK_CNT && HashTable->HashTableFreeLst.chunks[i] != NULL; i++ ) {
+   for ( i = 0; i < HashTable->HashTableFreeLst.cbChunkCnt && HashTable->HashTableFreeLst.chunks[i] != NULL; i++ ) {
      pr03mFreeT(HashTable->HashTableFreeLst.chunks[i], "HashTableFreeLst");
      HashTable->HashTableFreeLst.chunks[i]=NULL;
    }
+   pr03mFreeT(HashTable->HashTableFreeLst.chunks, "HashTableChunkLst");
+   HashTable->HashTableFreeLst.chunks = NULL;
    HashTable->HashTableFreeLst.first_p = NULL;
+   HashTable->HashTableFreeLst.cbChunkCnt = 0;
+
    /*free memory*/
    pr03mFreeT(HashTable->pHashArray, "pr09HashTable");
 }
 
 void pr09HTClearHashTable (tpr09HashTable * HashTable)
 {
-   int i;
+   tsp00_Uint i,j;
+   tpr09HashItem* pElem;
+   
    /*reset HashTable record*/
    HashTable->cbActElem     = 0; /*number of current used elements */
-
+   
    /*reset statistics*/
    HashTable->statistic.requests   = 0;  /* number of requests against hashtable */
    HashTable->statistic.founds     = 0;  /* number of hashvalues found in hashtable */
@@ -309,44 +369,60 @@ void pr09HTClearHashTable (tpr09HashTable * HashTable)
    HashTable->statistic.collisions = 0;  /* number of collisopns */
    HashTable->statistic.weightedHitrate = 0;  /* weighted hitrate */
 
-    /*free HashTableFreeLst*/
-    for ( i = 0; i < HASH_FL_CHUNK_CNT && HashTable->HashTableFreeLst.chunks[i] != NULL; i++ ) {
-      pr03mFreeT(HashTable->HashTableFreeLst.chunks[i], "HashTableFreeLst");
-      HashTable->HashTableFreeLst.chunks[i]=NULL;
+    HashTable->HashTableFreeLst.first_p = NULL;    
+    for ( i = 0; i < HashTable->HashTableFreeLst.cbChunkCnt && HashTable->HashTableFreeLst.chunks[i] != NULL; i++ ) {
+      for (j=0; j < HASH_FL_CHUNK_SIZE/sizeof(tpr09HashItem); j++){
+/*        printf ("FL[%08d] CHUNK[%08d] ITEM[%p]\n", i, j, (HashTable->HashTableFreeLst.chunks[i]+j));*/
+        pElem = HashTable->HashTableFreeLst.chunks[i]+j;
+        pElem->data_p = NULL;
+        pElem->next_p = HashTable->HashTableFreeLst.first_p;
+        HashTable->HashTableFreeLst.first_p = pElem;
+      } 
     }
-    HashTable->HashTableFreeLst.first_p = NULL;
 
     /*clear hash array*/
     memset(HashTable->pHashArray, 0, HashTable->cbMaxCapacity*sizeof(tpr09HashItem));
+    pr09HTResetIterator(HashTable);
 }
 
-tsp00_Int4 pr09HTContainsByKey (tpr09HashTable * HashTable, tpr09HashKey HashKey, tsp00_Int4 cbHashKeyLen, tpr09HashValue HashValue)
+tsp00_Int4 pr09HTContainsByKey (tpr09HashTable * HashTable, 
+                                tpr09HashKey HashKey, 
+                                tsp00_Uint4 cbHashKeyLen)
 {
-    if (NULL != pr09HTGetHashItemByKey (HashTable, HashKey, cbHashKeyLen, HashValue))
+    if (NULL != pr09HTGetHashItemByKey (HashTable, HashKey, cbHashKeyLen))
       return 1;
     else
       return 0;
 }
 
-tsp00_Int4 pr09HTContainsByIndex (tpr09HashTable * HashTable, tpr09HashIndex HashIndex, tpr09HashValue HashValue)
+tsp00_Int4 pr09HTContainsByIndex (tpr09HashTable * HashTable, 
+                                  tpr09HashIndex HashIndex, 
+                                  tpr09HashKey   HashValue,
+                                  tsp00_Uint4    cbHashKeyLen)
 {
-    if (NULL != pr09HTGetHashItemByIndex (HashTable, HashIndex, HashValue))
+    if (NULL != pr09HTGetHashItemByIndex (HashTable, HashIndex, HashValue, cbHashKeyLen))
       return 1;
     else
       return 0;
 }
 
-tpr09HashItem* pr09HTGetHashItemByKey (tpr09HashTable * HashTable, tpr09HashKey HashKey, tsp00_Int4 cbHashKeyLen, tpr09HashValue HashValue)
+tpr09HashValue pr09HTGetHashItemByKey (tpr09HashTable * HashTable, 
+                                        tpr09HashKey   HashKey, 
+                                        tsp00_Uint4    cbHashKeyLen)
 {
     tpr09HashIndex myHashIndex = HashTable->HashFunc(HashKey, cbHashKeyLen, HashTable->cbMaxCapacity);
-    return pr09HTGetHashItemByIndex (HashTable, myHashIndex, HashValue);
+    return pr09HTGetHashItemByIndex (HashTable, myHashIndex, HashKey, cbHashKeyLen);
 }
 
 
-tpr09HashItem* pr09HTGetHashItemByIndex (tpr09HashTable * HashTable, tpr09HashIndex HashIndex, tpr09HashValue HashValue)
+tpr09HashValue pr09HTGetHashItemByIndex (tpr09HashTable * HashTable, 
+                                         tpr09HashIndex HashIndex, 
+                                         tpr09HashKey   HashValue,
+                                         tsp00_Uint4    cbHashKeyLen)
 {
+    void* tmpElem;
+    tsp00_Uint4 tmpElemLen;
     tpr09HashItem* myHashItem = (HashIndex >= 0 && HashIndex <= HashTable->cbMaxCapacity)?HashTable->pHashArray + HashIndex : NULL;
-    tsp00_Int4 collision=0;
     HashTable->statistic.requests++;
 
     if (myHashItem->data_p == NULL) {
@@ -355,34 +431,54 @@ tpr09HashItem* pr09HTGetHashItemByIndex (tpr09HashTable * HashTable, tpr09HashIn
     }
 
     do {
-      if (HashTable->CompareMethod(HashValue, myHashItem->data_p)) {
+      tmpElem = HashTable->KeyFunc(myHashItem->data_p, &tmpElemLen);
+      if (HashTable->CompareMethod(HashValue, 
+                                   cbHashKeyLen, 
+                                   tmpElem, 
+                                   tmpElemLen)) {
         HashTable->statistic.founds++;
-        if (collision)
-          HashTable->statistic.collisions++;
-        return myHashItem;
+        return myHashItem->data_p;
       }
-      collision++;
       myHashItem = myHashItem->next_p;
     } while (myHashItem != NULL);
 
     return NULL;
 }
 
-tpr09_HashTableError pr09HTInsertHashItemByValue (tpr09HashTable * HashTable, tpr09HashKey HashKey, tsp00_Int4 cbHashKeyLen, tpr09HashValue HashValue, tpr09HashIndex *HashIndex) {
+tpr09_HashTableError pr09HTInsertHashItemByValue (tpr09HashTable * HashTable, 
+                                                  tpr09HashKey   HashKey, 
+                                                  tsp00_Uint4    cbHashKeyLen, 
+                                                  tpr09HashValue HashValue, 
+                                                  tpr09HashIndex *HashIndex)
+{
     *HashIndex = HashTable->HashFunc(HashKey, cbHashKeyLen, HashTable->cbMaxCapacity);
-    return pr09HTInsertHashItemByIndex (HashTable, HashIndex, HashValue);
+    return pr09HTInsertHashItemByIndex (HashTable, HashIndex, HashKey, cbHashKeyLen, HashValue);
 }
 
-tpr09_HashTableError pr09HTInsertHashItemByIndex (tpr09HashTable * HashTable, tpr09HashIndex *HashIndex, tpr09HashValue HashValue)
+tpr09_HashTableError pr09HTInsertHashItemByIndex (tpr09HashTable * HashTable, 
+                                                  tpr09HashIndex *HashIndex,
+                                                  tpr09HashKey   HashKey, 
+                                                  tsp00_Uint4    cbHashKeyLen, 
+                                                  tpr09HashValue HashValue)
 {
    tpr09HashItem * hashItem;
 
-   if (HashTable->cbActElem >= HashTable->cbMaxElem) /*hash table full*/
-     return HashTableFull_epr09;
+   if (HashTable->cbActElem >= HashTable->cbMaxElem){ /*hash table full*/
+     if (HashTable->rehash == rehash_enable) {
+       tpr09_HashTableError erg = pr09HTRehash(HashTable);
+       if (erg != ok_epr09) return erg;
+       *HashIndex = HashTable->HashFunc(HashKey, cbHashKeyLen, HashTable->cbMaxCapacity);
+     }    
+     else
+       return HashTableFull_epr09;
+   }  
 
    hashItem = HashTable->pHashArray + *HashIndex;
 
-   if ( NULL != pr09HTGetHashItemByIndex (HashTable, *HashIndex, HashValue)) /*Hash item already exists*/
+   if ( NULL != pr09HTGetHashItemByIndex (HashTable, 
+                                          *HashIndex, 
+                                          HashKey,
+                                          cbHashKeyLen)) /*Hash item already exists*/
      return ok_epr09;
 
    if ( hashItem->data_p == NULL )
@@ -406,20 +502,28 @@ tpr09_HashTableError pr09HTInsertHashItemByIndex (tpr09HashTable * HashTable, tp
       new_p->next_p = hashItem->next_p;
       new_p->data_p = HashValue;
       hashItem->next_p = new_p;
+      HashTable->statistic.collisions++;
    }
    HashTable->cbActElem++;
    HashTable->statistic.inserts++;
    return ok_epr09;
 }
 
-void pr09HTRemoveHashItemByKey (tpr09HashTable * HashTable, tpr09HashKey HashKey, tsp00_Int4 cbHashKeyLen, tpr09HashValue HashValue)
+void pr09HTRemoveHashItemByKey (tpr09HashTable * HashTable, 
+                                tpr09HashKey   HashKey, 
+                                tsp00_Uint4    cbHashKeyLen)
 {
     tpr09HashIndex myHashIndex = HashTable->HashFunc(HashKey, cbHashKeyLen, HashTable->cbMaxCapacity);
-    pr09HTRemoveHashItemByIndex (HashTable, myHashIndex, HashValue);
+    pr09HTRemoveHashItemByIndex (HashTable, myHashIndex, HashKey, cbHashKeyLen);
 }
 
-void pr09HTRemoveHashItemByIndex (tpr09HashTable * HashTable, tpr09HashIndex HashIndex, tpr09HashValue HashValue)
+void pr09HTRemoveHashItemByIndex (tpr09HashTable * HashTable, 
+                                  tpr09HashIndex HashIndex, 
+                                  tpr09HashKey HashKey, 
+                                  tsp00_Uint4 cbHashKeyLen)
 {
+    void* tmpElem;
+    tsp00_Uint4 tmpElemLen;
     tpr09HashItem* myHashItem = HashTable->pHashArray + HashIndex;
     tpr09HashItem* prevHashItem = myHashItem;
 
@@ -429,7 +533,13 @@ void pr09HTRemoveHashItemByIndex (tpr09HashTable * HashTable, tpr09HashIndex Has
     }
 
     do {
-      if (HashTable->CompareMethod(HashValue, myHashItem->data_p)) {
+      tmpElem = HashTable->KeyFunc(myHashItem->data_p, &tmpElemLen);
+      if (HashTable->CompareMethod(HashKey, cbHashKeyLen, tmpElem, tmpElemLen)) {
+        if (HashTable->iterator.pElem == myHashItem){
+        /*move iterator to the next item*/
+          pr09HTIteratorNextItem (HashTable);
+        }  
+        /*remove hash item*/
         if (prevHashItem == myHashItem) {
           /*first item*/
           myHashItem = myHashItem->next_p;
@@ -453,6 +563,99 @@ void pr09HTRemoveHashItemByIndex (tpr09HashTable * HashTable, tpr09HashIndex Has
    return;
 }
 
+static tpr09_HashTableError pr09HTRehash(tpr09HashTable*    HashTable) {
+
+   tpr09_HashTableError erg = ok_epr09; 
+   tsp00_Uint4 i, hashIndex, rehashKeyLen,oldMaxCapacity = HashTable->cbMaxCapacity;
+   tpr09HashItem *oldElem, *tmpElem,      *oldHashArray  = HashTable->pHashArray;
+   tpr09HashKey   rehashKey;
+   tpr09HashValue rehashVal;
+   
+   HashTable->cbMaxCapacity = (200 * HashTable->cbMaxCapacity) / HashTable->cbLoadFactor;
+   for ( i = 0; (tsp00_Uint4) HashTable->cbMaxCapacity > PrimeNumbers[i] && PrimeNumbers[i] != 0; i++ ){}
+   HashTable->cbMaxCapacity = PrimeNumbers[i];
+   HashTable->cbMaxElem     = HashTable->cbMaxCapacity * HashTable->cbLoadFactor / 100; /*number of maximal elements*/
+   HashTable->statistic.collisions = 0;
+   
+   /*alloc new bucket array*/
+   HashTable->pHashArray = (tpr09HashItem*) pr03mCallocT(HashTable->cbMaxCapacity, sizeof(tpr09HashItem), "pr09HashTable");
+   if (! HashTable->pHashArray){
+     HashTable->cbMaxCapacity = 0;
+     HashTable->cbMaxElem     = 0;
+     return AllocMemErr_epr09;
+   }
+   
+   for (i=0; i<oldMaxCapacity; i++){
+     oldElem = oldHashArray+i;
+     if (oldElem->data_p == NULL) {
+       continue;
+     }
+     while (oldElem != NULL && erg == ok_epr09){
+       rehashVal = oldElem->data_p;
+       rehashKey = HashTable->KeyFunc(rehashVal, &rehashKeyLen);
+       hashIndex = HashTable->HashFunc(rehashKey, rehashKeyLen, HashTable->cbMaxCapacity);
+       tmpElem   = HashTable->pHashArray+hashIndex;
+
+       if ( tmpElem->data_p == NULL ){
+         tmpElem->data_p = rehashVal;
+       }
+       else {
+         tpr09HashItem		* new_p;
+         if (ok_epr09!=(erg = pr09getnewCollLstItem (&new_p, HashTable))) continue; 
+         new_p->next_p = tmpElem->next_p;
+         new_p->data_p = rehashVal;
+         tmpElem->next_p = new_p;
+         HashTable->statistic.collisions++;
+       }
+       
+       tmpElem = oldElem->next_p;
+       if (tmpElem){
+         oldElem->data_p = tmpElem->data_p;
+         oldElem->next_p = tmpElem->next_p;
+         pr09addtoFreeLst(tmpElem, HashTable);
+       } else {
+         oldElem = NULL;
+       }
+     }
+   }
+   
+   pr03mFreeT (oldHashArray,"pr09HashTable");
+   pr09HTResetIterator(HashTable);
+   return erg;  
+} 
+
+void pr09HTDump (tpr09HashTable * HashTable, FILE * datei ){
+  tsp00_Uint4 i, maxDeep=0, tmpMaxDeep, sumDeep=0, usedItems=0;
+  tpr09HashItem *pElem;
+  
+  
+  pr09HTPrintStatHeader (HashTable, datei);
+  pr09HTPrintStatValues (HashTable, datei);
+  
+  fprintf (datei, "\nHASHTABLE DUMP\n");
+  fprintf (datei, "==============\n\n");
+  for (i=0; i<HashTable->cbMaxCapacity; i++){
+    pElem = HashTable->pHashArray+i;
+    if (pElem->data_p != NULL) {
+      fprintf (datei, "Bucket[%08d]",i);
+    } else {
+      continue;
+    }
+    tmpMaxDeep = 0;
+    usedItems ++;
+    while (pElem != NULL){
+      fprintf (datei, "->",pElem->data_p);
+      HashTable->PrintFunc(pElem->data_p,datei);
+      pElem = pElem->next_p;
+      tmpMaxDeep++;
+      sumDeep++;
+    }
+    if (maxDeep < tmpMaxDeep) maxDeep = tmpMaxDeep;
+    fprintf (datei, "\n");
+  }
+  fprintf (datei, "\nMax deep of collision list: %d\n", maxDeep);
+  fprintf (datei, "\nAvg deep of collision list: %f\n", ((double)sumDeep)/usedItems);
+}
 
 void pr09HTPrintStatHeader (tpr09HashTable * HashTable, FILE * outputdev )
 {
@@ -529,8 +732,41 @@ const char * pr09HTerrString (tpr09_HashTableError procResult)
     }
 }
 
+void pr09HTResetIterator (tpr09HashTable * HashTable){
+  HashTable->iterator.cbCurElem = 0;
+  HashTable->iterator.pElem = HashTable->pHashArray;
+  if (HashTable->iterator.pElem->data_p == NULL)
+    pr09HTIteratorNextItem (HashTable);
+}
 
+static void getNextBucket (tpr09HashTable * HashTable){
+  tpr09HashTableIterator *iter = &(HashTable->iterator);
+  do {
+    iter->cbCurElem++;
+    iter->pElem = HashTable->pHashArray+iter->cbCurElem;
+  } while ( iter->cbCurElem < HashTable->cbMaxCapacity-1 
+            && (iter->pElem == NULL || iter->pElem->data_p == NULL));
+  if (iter->cbCurElem >= HashTable->cbMaxCapacity){
+    iter->pElem = NULL;
+  }
+}
 
+tpr09HashValue pr09HTIteratorNextItem (tpr09HashTable * HashTable){
+  tpr09HashTableIterator *iter = &(HashTable->iterator);
+  tpr09HashValue hVal;
+  
+  if (iter->pElem) 
+    hVal = iter->pElem->data_p;
+  else
+    return NULL;
+  
+  if(iter->pElem->next_p == NULL){
+    getNextBucket(HashTable);
+  } else {
+    iter->pElem = iter->pElem->next_p;
+  }
+  return hVal;
+}
 
 /*===========================================================================*
  *  END OF CODE                                                              *
