@@ -107,6 +107,16 @@ typedef enum tpr09_HashTableError
   InvalidHashIndex_epr09    /* hash index is invalid*/
 } tpr09_HashTableError;
 
+/*!
+  Decleration: tpr09_HTRehash
+  Description: rehash flags
+ */
+typedef enum tpr09_HTRehash
+{
+  rehash_enable  = 1,
+  rehash_disable = 0
+} tpr09_HTRehash;
+
 /*!
   Decleration: tpr09HashItem
   Description: hashtable element
@@ -144,6 +154,23 @@ typedef tsp00_Uint4 tpr09HashIndex;
 */
 typedef void* tpr09HashValue;
 
+
+/*!
+  Decleration: tpr09PrintHashItem
+  Description: callback to print out the hash value
+*/
+typedef void (*tpr09PrintHashItem) (const tpr09HashValue data_p, FILE* datei);
+
+
+/*!
+  Decleration: tpr09ExtractKey
+  Description: pointer to extract key function
+  
+  Retrieves the key member from the data object used for compare
+  
+*/
+typedef tpr09HashKey (*tpr09ExtractKey) (const tpr09HashValue data_p, tsp00_Uint4* HashKeyLen);
+
 /*!
   Decleration: tpr09CompareMethod
   Description: pointer to callback function for comparsion of two Hashvalues
@@ -153,13 +180,13 @@ typedef void* tpr09HashValue;
   function gets two hashvalues as input. They must return zero, if hashvalues are
   not equal und a value greater zero if hashvalues are equal.
 */
-typedef tsp00_Int4 (*tpr09CompareMethod) (const tpr09HashValue Item1, const tpr09HashValue Item2);
+typedef tsp00_Int4 (*tpr09CompareMethod) (const tpr09HashKey Item1,tsp00_Uint4 Item1Len, const tpr09HashKey Item2, tsp00_Uint4 Item2Len);
 
 /*!
   Decleration: tpr09HashFunc
   Description: pointer to hash function
 */
-typedef tpr09HashIndex (*tpr09HashFunc) (const tpr09HashKey * HashKey, tsp00_Uint4 cbHashKeyLen, tsp00_Uint4 cbHashTabLen);
+typedef tpr09HashIndex (*tpr09HashFunc) (const tpr09HashKey HashKey, tsp00_Uint4 cbHashKeyLen, tsp00_Uint4 cbHashTabLen);
 
 /*!
   Decleration: tpr09CacheStat
@@ -186,17 +213,20 @@ typedef struct tpr09CacheStat
   lists in case if necessary. It is organized as a set of HASH_ITEM chunks
   which will be allocated dynamically if needed.
 */
-#define HASH_FL_CHUNK_CNT	6
+#define HASH_FL_CHUNK_SIZE	(4096/sizeof(tpr09HashItem))*sizeof(tpr09HashItem)
+#define HASH_FL_CHUNK_INC_CNT	6
 typedef struct tpr09HashTableFreeLst
 {
-   tpr09HashItem	      * chunks[HASH_FL_CHUNK_CNT];
-                                /* pointers to HASH_ITEM arrays which */
-                                /* will be allocated dynamically      */
-   tpr09HashItem          * first_p;
-                                /* pointer to the first element of    */
-                                /* the hash free list                 */
+   tpr09HashItem ** chunks; /* pointers to HASH_ITEM arrays which will be allocated dynamically      */
+   tpr09HashItem * first_p; /* pointer to the first element of the hash free list */
+   tsp00_Uint4	 cbChunkCnt;
 } tpr09HashTableFreeLst;
 
+typedef struct tpr09HashTableIterator
+{
+   tsp00_Uint4	   cbCurElem; 
+   tpr09HashItem * pElem;
+} tpr09HashTableIterator;
 /*!
   Decleration: tpr09HashTable
   Description: basis structure for hashtable
@@ -210,11 +240,15 @@ typedef struct {
   tsp00_Uint4 cbLoadFactor;   /*measure of how full the hash table is allowed to get
                                before its capacity is reached value between 0 and 100*/
   tsp00_Uint4 cbMaxCapacity;  /*maximum capacity of hashtable*/
+  tpr09_HTRehash rehash;      /*signals whether rehashing is enabled or disabled*/
   tpr09CompareMethod CompareMethod; /*method for comparsion of two Hashvalues*/
-  tpr09HashFunc HashFunc;    /*pointer to hash function*/
+  tpr09HashFunc      HashFunc;    /*pointer to hash function*/
+  tpr09ExtractKey    KeyFunc;     /*pointer to key extract function*/
+  tpr09PrintHashItem PrintFunc;   /*pointer to dump function*/
   tpr09CacheStat statistic;  /*statistic data of hash table*/
   tpr09HashItem *pHashArray; /*pointer to array with hashed key value pairs */
   tpr09HashTableFreeLst HashTableFreeLst; /**/
+  tpr09HashTableIterator iterator;
 } tpr09HashTable;
 
 /*!
@@ -234,7 +268,14 @@ typedef struct {
 
   returns: tpr09_HashTableError, if an error occurred
  */
-tpr09_HashTableError pr09HTCreateHashTable(tpr09HashTable * HashTable, tsp00_Int4 capacity, tsp00_Int4 LoadFactor, tpr09CompareMethod, tpr09HashFunc HashFunc);
+tpr09_HashTableError pr09HTCreateHashTable(tpr09HashTable*    HashTable, 
+                                           tsp00_Int4         capacity, 
+                                           tsp00_Int4         LoadFactor, 
+                                           tpr09_HTRehash     rehash,
+                                           tpr09CompareMethod CompFunc, 
+                                           tpr09HashFunc      HashFunc, 
+                                           tpr09ExtractKey    KeyFunc,
+                                           tpr09PrintHashItem PrintFunc);
 
 /*!
   Function: pr09HTDeleteHashTable
@@ -276,12 +317,13 @@ void pr09HTClearHashTable (tpr09HashTable * HashTable);
        tpr09HashTable [in|out] pointer to a HashTable
        HashKey        [in]     key you are looking for
        cbHashKeyLen   [in]     length of hash Key
-       HashValue      [in]     hash value you are looking for
 
   returns: 0 if hash table don`t contains the key/value pair
            1 if key/value pair founds in the table
  */
-tsp00_Int4 pr09HTContainsByKey (tpr09HashTable * HashTable, tpr09HashKey HashKey, tsp00_Int4 cbHashKeyLen, tpr09HashValue HashValue);
+tsp00_Int4 pr09HTContainsByKey (tpr09HashTable* HashTable, 
+                                tpr09HashKey    HashKey, 
+                                tsp00_Uint4      cbHashKeyLen);
 
 
 /*!
@@ -294,12 +336,16 @@ tsp00_Int4 pr09HTContainsByKey (tpr09HashTable * HashTable, tpr09HashKey HashKey
   Arguments:
        tpr09HashTable [in|out] pointer to a HashTable
        HashIndex      [in]     index into hashtable
-       HashValue      [in]     hash value you are looking for
+       HashKey        [in]     key you are looking for
+       cbHashKeyLen   [in]     length of hash Key
 
   returns: 0 if hash table don`t contains the key/value pair
            1 if key/value pair founds in the table
  */
-tsp00_Int4 pr09HTContainsByIndex (tpr09HashTable * HashTable, tpr09HashIndex HashIndex, tpr09HashValue HashValue);
+tsp00_Int4 pr09HTContainsByIndex (tpr09HashTable * HashTable, 
+                                  tpr09HashIndex HashIndex, 
+                                  tpr09HashKey   HashValue,
+                                  tsp00_Uint4    cbHashKeyLen);
 
 /*!
   Function: pr09HTGetHashItemByKey
@@ -312,12 +358,13 @@ tsp00_Int4 pr09HTContainsByIndex (tpr09HashTable * HashTable, tpr09HashIndex Has
        tpr09HashTable [in|out] pointer to a HashTable
        HashKey        [in]     key you are looking for
        cbHashKeyLen   [in]     length of hash Key
-       HashValue      [in]     hash value you are looking for
 
   returns: NULL      - if nothing found
            HashItem  - the specified hashitem
  */
-tpr09HashItem * pr09HTGetHashItemByKey (tpr09HashTable * HashTable, tpr09HashKey HashKey, tsp00_Int4 cbHashKeyLen, tpr09HashValue HashValue);
+tpr09HashValue  pr09HTGetHashItemByKey (tpr09HashTable * HashTable, 
+                                        tpr09HashKey   HashKey, 
+                                        tsp00_Uint4    cbHashKeyLen);
 
 /*!
   Function: pr09HTGetHashItemByIndex
@@ -329,12 +376,16 @@ tpr09HashItem * pr09HTGetHashItemByKey (tpr09HashTable * HashTable, tpr09HashKey
   Arguments:
        tpr09HashTable [in|out] pointer to a HashTable
        HashIndex      [in]     index into hashtable for searching the value
-       HashValue      [in]     hash value you are looking for
+       HashKey        [in]     key you are looking for
+       cbHashKeyLen   [in]     length of hash Key
 
   returns: NULL      - if nothing found
            HashItem  - the specified hashitem
  */
-tpr09HashItem * pr09HTGetHashItemByIndex (tpr09HashTable * HashTable, tpr09HashIndex HashIndex, tpr09HashValue HashValue);
+tpr09HashValue  pr09HTGetHashItemByIndex (tpr09HashTable * HashTable, 
+                                          tpr09HashIndex HashIndex, 
+                                          tpr09HashKey   HashValue,
+                                          tsp00_Uint4    cbHashKeyLen);
 
 /*!
   Function: pr09HTInsertHashItemByValue
@@ -357,7 +408,11 @@ tpr09HashItem * pr09HTGetHashItemByIndex (tpr09HashTable * HashTable, tpr09HashI
 
   returns: tpr09_HashTableError
  */
-tpr09_HashTableError pr09HTInsertHashItemByValue (tpr09HashTable * HashTable, tpr09HashKey HashKey, tsp00_Int4 cbHashKeyLen, tpr09HashValue HashValue, tpr09HashIndex *HashIndex);
+tpr09_HashTableError pr09HTInsertHashItemByValue (tpr09HashTable * HashTable, 
+                                                  tpr09HashKey   HashKey, 
+                                                  tsp00_Uint4    cbHashKeyLen, 
+                                                  tpr09HashValue HashValue, 
+                                                  tpr09HashIndex *HashIndex);
 
 
 /*!
@@ -373,11 +428,17 @@ tpr09_HashTableError pr09HTInsertHashItemByValue (tpr09HashTable * HashTable, tp
   Arguments:
        tpr09HashTable [in|out] pointer to a HashTable
        HashIndex      [in|out] hash index computed by hashfunction
+       HashKey        [in]     key you are looking for
+       cbHashKeyLen   [in]     length of hash Key
        HashValue      [in]     hash value you are looking for
 
   returns: tpr09_HashTableError
  */
-tpr09_HashTableError pr09HTInsertHashItemByIndex (tpr09HashTable * HashTable, tpr09HashIndex *HashIndex, tpr09HashValue HashValue);
+tpr09_HashTableError pr09HTInsertHashItemByIndex (tpr09HashTable * HashTable, 
+                                                  tpr09HashIndex *HashIndex,
+                                                  tpr09HashKey   HashKey, 
+                                                  tsp00_Uint4    cbHashKeyLen, 
+                                                  tpr09HashValue HashValue);
 
 
 /*!
@@ -391,11 +452,12 @@ tpr09_HashTableError pr09HTInsertHashItemByIndex (tpr09HashTable * HashTable, tp
        tpr09HashTable [in|out] pointer to a HashTable
        HashKey        [in]     key you are looking for
        cbHashKeyLen   [in]     length of hash Key
-       HashValue      [in]     hash value you are looking for
 
   returns: void
  */
-void pr09HTRemoveHashItemByKey (tpr09HashTable * HashTable, tpr09HashKey HashKey, tsp00_Int4 cbHashKeyLen, tpr09HashValue HashValue);
+void pr09HTRemoveHashItemByKey (tpr09HashTable * HashTable, 
+                                tpr09HashKey   HashKey, 
+                                tsp00_Uint4    cbHashKeyLen);
 
 /*!
   Function: pr09HTRemoveHashItemByIndex
@@ -407,11 +469,41 @@ void pr09HTRemoveHashItemByKey (tpr09HashTable * HashTable, tpr09HashKey HashKey
   Arguments:
        tpr09HashTable [in|out] pointer to a HashTable
        HashIndex      [in]     index you are looking for
-       HashValue      [in]     hash value you are looking for
+       HashKey        [in]     key you are looking for
+       cbHashKeyLen   [in]     length of hash Key
 
   returns: void
  */
-void pr09HTRemoveHashItemByIndex (tpr09HashTable * HashTable, tpr09HashIndex HashIndex, tpr09HashValue HashValue);
+void pr09HTRemoveHashItemByIndex (tpr09HashTable * HashTable, 
+                                  tpr09HashIndex HashIndex, 
+                                  tpr09HashKey   HashKey, 
+                                  tsp00_Uint4    cbHashKeyLen);
+
+/*!
+  Function: pr09HTResetIterator
+
+  see also:
+
+  Description: Resets the iterator to the first item in the hashtable
+
+  Arguments:
+
+  returns: void
+ */
+void pr09HTResetIterator (tpr09HashTable * HashTable);
+
+/*!
+  Function: pr09HTResetIterator
+
+  see also:
+
+  Description: Returns the next item retrieved while iterating thru the hashtable
+
+  Arguments:
+
+  returns: void* 
+ */
+tpr09HashValue pr09HTIteratorNextItem (tpr09HashTable * HashTable);
 
 /*!
   Function: pr09HTPrintStatHeader
@@ -460,4 +552,7 @@ void pr09HTPrintStatValues (tpr09HashTable * HashTable, FILE * datei );
  */
 const char * pr09HTerrString (tpr09_HashTableError procResult);
 
+
+void pr09HTDump (tpr09HashTable * HashTable, FILE * datei );
+
 #endif  /*VPR09HashTable_H */
