@@ -414,7 +414,7 @@ externCGlobal const tsp77encoding * const sp77encodingUCS2Swapped;
   Native (machine dependend UCS2): Machine byte order
  */
 /* PTS 1110303 */
-#if defined(WIN32) || (defined(LINUX) && !defined(S390X)) || defined(OSF1)
+#if defined(WIN32) || (defined(LINUX) && !(defined(S390X)||defined(SDBonPPC64))) || defined(OSF1)
 #define sp77encodingUCS2Native sp77encodingUCS2Swapped
 #else
 #define sp77encodingUCS2Native sp77encodingUCS2
