@@ -1882,7 +1882,12 @@ static tjk_convFunctionStruct sp77_convUpper[26] =
 
 /*----------------------------------------*/
 
-#if defined(LINUX) && defined(S390X)
+/*
+ * 04/23/2004 cba:
+ * gcc 3.2.2 needs a real va_list array to work properly, not only a pointer
+ */
+
+#if defined(LINUX) && (defined(S390X) || defined(X86_64) || defined(SDBonPPC64))
 static int
 sp77_vsprintfCommon (
     const tsp77encoding *encoding,
@@ -1909,8 +1914,9 @@ sp77_vsprintfCommon (
     t_len               charsWritten;
     cbool               error = FALSE;
     t_len               literalLen;
-#if defined(LINUX) && defined(S390X)
+#if defined(LINUX) && (defined(S390X) || defined(X86_64) || defined(SDBonPPC64))
     va_list args;
+
     __va_copy(args,xargs);
 #endif
 
