@@ -52,6 +52,8 @@
 #include "hwd104fsdesc.h"
 #include "hwd111getinode.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 /*!**********************************************************************
 
   Chapter: Prototypes and constants for internal functions
@@ -250,7 +252,7 @@ sapdbwa_Bool wd111DbGetInodeByName( twd111GetInodeDescP  desc,
             };
         } else {
             if (!strcmp(Name, ".")) {
-                memcpy( desc->nameParam, Directory, MAX_SYSKEY_LEN_WD103 );
+                SAPDB_memcpy( desc->nameParam, Directory, MAX_SYSKEY_LEN_WD103 );
                 desc->nameIndi = MAX_SYSKEY_LEN_WD103;
 
                 desc->dirIndi = SQL_NULL_DATA;
@@ -280,7 +282,7 @@ sapdbwa_Bool wd111DbGetInodeByName( twd111GetInodeDescP  desc,
 					                  (tsp81_UCS2Char*)(desc->nameParamUCS2+2*(DB_MAX_NAME_LEN_WD103+1)), 
 					                  &destAt, WD_IS_UCS2_SWAPED);
 
-                memcpy( desc->dirParam, Directory, MAX_SYSKEY_LEN_WD103 );
+                SAPDB_memcpy( desc->dirParam, Directory, MAX_SYSKEY_LEN_WD103 );
                 desc->dirIndi = MAX_SYSKEY_LEN_WD103;
 
                 selectHstmt = desc->selectHstmt;
@@ -355,7 +357,7 @@ sapdbwa_Bool wd111GetInodeById( twd111GetInodeDescP  desc,
             return sapdbwa_False;
         }; /* if */
     };
-    memcpy( &desc->dirParam, inodeId, MAX_SYSKEY_LEN_WD103 );
+    SAPDB_memcpy( &desc->dirParam, inodeId, MAX_SYSKEY_LEN_WD103 );
     desc->dirIndi = MAX_SYSKEY_LEN_WD103;
 
     /* execute stmt and fetch result */
