@@ -52,6 +52,8 @@
 #include "hwd104fsdesc.h"
 #include "hwd115container.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 #define COMPRESSED   '1'
 #define UNCOMPRESSED '0'
 #define MAX_COMPR_BLOCK_SIZE (1024*64)
@@ -339,7 +341,7 @@ sapdbwa_Bool wd115DeleteContainer( twd115ContainerDescP  desc,
     RETCODE	sqlRetcode;
     
     /* set values for input parameters */
-    memcpy( &desc->inodeId, inodeId, sizeof(twd103InodeId) );
+    SAPDB_memcpy( &desc->inodeId, inodeId, sizeof(twd103InodeId) );
     desc->cbNode = MAX_SYSKEY_LEN_WD103;
     desc->cbViewOwner = SQL_NTS;
     
@@ -878,7 +880,7 @@ sapdbwa_Bool wd115_PrepareInsertStmt( twd115ContainerDescP desc,
                                                1,
                                                &desc->cbCompr );
             }; /* if */
-        /*}; /* if */
+        /*};  */
     }; /* if */
     if (!wd00Success( sqlRetcode )) {        
         wd26SetOdbcError( err, sqlRetcode,
