@@ -171,11 +171,11 @@ sapdbwa_Bool wd119Copy( twd119CopyDescP  desc,
         if (withLinks && newInodeType == inode_file) {
             /* generate link. Dont copy file. */
             newInodeType = inode_link;
-            memcpy( &linkInodeId, &sourceInode->InodeId, sizeof( linkInodeId ));
+            SAPDB_memcpy( &linkInodeId, &sourceInode->InodeId, sizeof( linkInodeId ));
         } else {
             /* make copy copy of file dir or link */
             newInodeType = sourceInode->Type;
-            memcpy( &linkInodeId, &sourceInode->LinkInodeId, sizeof( linkInodeId ));
+            SAPDB_memcpy( &linkInodeId, &sourceInode->LinkInodeId, sizeof( linkInodeId ));
         };
         desc->inodeIdIndi = MAX_SYSKEY_LEN_WD103;
         copyOk = wd119_InsertInode( desc->fsDesc,
@@ -267,7 +267,7 @@ sapdbwa_Bool wd119_Push( twd119InodeStack *stack, twd103Inode *inode )
     }; /* if */
 
     if (allocatOK) {
-        memcpy( &stack->stackArray[stack->stackPos++],
+        SAPDB_memcpy( &stack->stackArray[stack->stackPos++],
                 inode, sizeof(twd103Inode));
     }; /* if */
 
@@ -279,7 +279,7 @@ sapdbwa_Bool wd119_Pop( twd119InodeStack *stack, twd103Inode *inode )
 {
     if (stack->stackPos > 0) {
         stack->stackPos--;
-        memcpy( inode,
+        SAPDB_memcpy( inode,
                 &stack->stackArray[stack->stackPos], sizeof(twd103Inode));
         return sapdbwa_True;
     } else {
@@ -485,13 +485,13 @@ sapdbwa_Bool wd119_CopyInode( twd119CopyDescP  desc,
                         /* create new inode */
                         if (withLinks && (desc->inode.Type == inode_file)) {
                             /* create link to original file */
-                            memcpy( &linkInodeId,
+                            SAPDB_memcpy( &linkInodeId,
                                     &desc->inode.InodeId,
                                     sizeof( linkInodeId ));
                             desc->inode.Type = inode_link;
                         } else {
                             /* copy file */
-                            memcpy( &linkInodeId,
+                            SAPDB_memcpy( &linkInodeId,
                                     &desc->inode.LinkInodeId,
                                     sizeof( linkInodeId ));
                         };
