@@ -54,6 +54,8 @@
 #include "hwd32hses.h"
 #include "hwd40user.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 /****************************************************************************
  ***
  *** Defines
@@ -220,10 +222,10 @@ sapdbwa_Bool sapdbwa_GetParam( sapdbwa_Handle    wa,
 } /* sapdbwa_GetParam */
 
 /***************************************************************************
-/***
-/***    sapdbwa_GetDocumentRoot
-/***
-/***************************************************************************/
+ ***
+ ***    sapdbwa_GetDocumentRoot
+ ***
+ ***************************************************************************/
 
 sapdbwa_Bool sapdbwa_GetDocumentRoot( sapdbwa_Handle  wa,
                                       char           *valueBuf,
@@ -235,10 +237,10 @@ sapdbwa_Bool sapdbwa_GetDocumentRoot( sapdbwa_Handle  wa,
 }
 
 /***************************************************************************
-/***
-/***
-/***
-/***************************************************************************/
+ ***
+ ***
+ ***
+ ***************************************************************************/
 
 sapdbwa_SessionPoolP WINAPI sapdbwa_GetSessionPool( sapdbwa_Handle wa )
 {
@@ -515,7 +517,7 @@ void wd23_WAHandleInit( sapdbwa_Handle              wa,
     wa->ownedWebSession = false;
 
     if (sessionId != NULL) {
-        memcpy( &wa->sessionId[0],
+        SAPDB_memcpy( &wa->sessionId[0],
                 &sessionId[0], sizeof( twd37SessionId ));
     } else {
         wa->sessionId[0] = '\0';
