@@ -54,6 +54,8 @@
 #include "hsp81.h"
 #include "hsp83.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 /*!**********************************************************************
 
   prototypes for internal functions
@@ -301,17 +303,27 @@ twd26ErrP wd26CreateErr()
 
 void wd26DestroyErr( twd26ErrP err )
 {
+    
+    if (!err) {
+        return;
+    };
+
     if (err->msg != NULL) {
         sapdbwa_SQLFREE( (sapdbwa_UInt1*) err->msg );
     }; /* if */
     
     sapdbwa_SQLFREE( (sapdbwa_UInt1*) err );
+
 } /* wd26DestroyErr */
 
 
 void wd26CopyErr( twd26ErrP destErr, twd26ErrP sourceErr )
 {
-    memcpy( destErr, sourceErr, sizeof( struct st_wa_err ) );
+    if (!destErr || !sourceErr) {
+        return;
+    };
+
+    SAPDB_memcpy( destErr, sourceErr, sizeof( struct st_wa_err ) );
     /* dont share errMsg buffer */
     destErr->msg = NULL;
 } /* wd26CopyErr */
@@ -319,8 +331,15 @@ void wd26CopyErr( twd26ErrP destErr, twd26ErrP sourceErr )
 
 void wd26CopyMsg( twd26ErrP err, char *buf, sapdbwa_UInt4 bufLen )
 {
-    const char *msg = wd26GetMsg( err );
-    sapdbwa_UInt4 copyLen = sapdbwa_MIN( bufLen-1, strlen( msg )-1 );
+    const char *msg = NULL;
+    sapdbwa_UInt4 copyLen = 0;
+
+    if (!err || !buf) {
+        return;
+    }
+
+    msg = wd26GetMsg( err );
+    copyLen = sapdbwa_MIN( bufLen-1, strlen( msg )-1 );
 
     strncpy( buf, msg, copyLen );
     buf[copyLen] = '\0';    
@@ -372,6 +391,10 @@ sapdbwa_UInt4 wd26GetId( twd26ErrP err )
 
 sapdbwa_Int4 wd26GetNativeOdbcErr( twd26ErrP err )
 {
+    if (!err) {
+        return 0;
+    };
+
     return (sapdbwa_Int4) err->nativeOdbcErr;
 } /* wd26GetNativeOdbcErr */
 
@@ -381,6 +404,10 @@ void wd26SetErr( twd26ErrP   err,
                  const char *arg1,
                  const char *arg2 )
 {
+    if (!err) {
+        return;
+    };
+
     wd26ResetErr( err );
     err->errId = errId;
     if (errId == ERR_COMPRESS_WD26 || errId == ERR_UNCOMPRESS_WD26) {
@@ -415,6 +442,10 @@ void wd26SetOdbcError( twd26ErrP  err,
                        SQLHDBC    hdbc,
                        SQLHSTMT   hstmt )
 {
+    if (!err) {
+        return;
+    };
+
     err->odbcReturnCode = sqlRetcode;
     switch (sqlRetcode) {
     case SQL_ERROR:
@@ -450,10 +481,12 @@ void wd26SetOdbcError( twd26ErrP  err,
 
 void wd26ResetErr( twd26ErrP err )
 {
-    if (err->msg != NULL) {
-        sapdbwa_SQLFREE( (sapdbwa_UInt1*) err->msg );
-        err->msg = NULL;
-    }; /* if */
+    if (!err) {
+        return;
+    };
+
+    sapdbwa_SQLFREE( (sapdbwa_UInt1*) err->msg );
+    err->msg = NULL;
     
     err->errId      = ERR_NO_ERROR_WD26;
     err->errArg1[0] = '\0';
@@ -468,6 +501,10 @@ void wd26LogErr( twd26ErrP err, twd25LogP log )
 {
     char msgBuff[2*MAX_ERRORTEXT_LEN_WD00+2*ERR_ARG_LEN];
 
+    if (!err||!log) {
+        return;
+    };
+
 	/* Is there an error? */
 	if ( err->errId == ERR_NO_ERROR_WD26 )
 		return;
@@ -489,6 +526,10 @@ void wd26PrintErr( twd26ErrP err )
 {
     char msgBuff[2*MAX_ERRORTEXT_LEN_WD00+2*ERR_ARG_LEN];
 
+    if (!err) {
+        return;
+    };
+
     sprintf( msgBuff,
              wd26_GetErrMsg( err->errId ),
              err->errArg1,
@@ -751,6 +792,10 @@ void wd26_SetOdbcErrorMessage( twd26ErrP  err,
     tsp00_Byte             *destAtUCS2 = NULL;
     SWORD                   errLen;
 
+    if (!err) {
+        return;
+    };
+
     retcode = SQLErrorW( henv, hdbc, hstmt,
                          sqlState,
                         &err->nativeOdbcErr,
