@@ -118,6 +118,10 @@ twd30SessionPoolP wd30CreateSessionPool( const char *name, twd26ErrP   err )
     twd30SessionPoolP  newPool;
     sapdbwa_UInt2        nameLen;
     
+    if (!name) {
+        return NULL;
+    }
+
     sapdbwa_SQLALLOCAT( sizeof(struct st_wa_sessionpool),
                 (sapdbwa_UInt1**) &newPool,
                 &allocatOK );
@@ -148,6 +152,10 @@ twd30SessionPoolP wd30CreateSessionPool( const char *name, twd26ErrP   err )
 
 void wd30DestroySessionPool( twd30SessionPoolP pool )
 {
+    if (!pool) {
+        return;
+    }
+
     wd30_FreeMembers( pool );
     
     sapdbwa_SQLFREE( (sapdbwa_UInt1*) pool );
@@ -158,6 +166,10 @@ twd31DbConnP wd30DbConnect( twd30SessionPoolP pool, twd26ErrP err )
 {
     twd35_ResourceId  connId;
     twd31DbConnP      conn = NULL;
+
+    if (!pool) {
+        return NULL;
+    }
     
     if (wd35LockElem( pool->connPool, (void**) &conn, &connId )) {
         if (conn == NULL) {
@@ -201,7 +213,13 @@ twd31DbConnP wd30DbConnect( twd30SessionPoolP pool, twd26ErrP err )
 
 sapdbwa_Bool wd30DbDisconnect( twd30SessionPoolP pool, twd31DbConnP conn )
 {
-    sapdbwa_Int4 connId = wd31GetConnId( conn );
+    sapdbwa_Int4 connId;
+
+    if (!conn) {
+        return sapdbwa_False;
+    }
+
+    connId = wd31GetConnId( conn );
 
     /* we don't want to keep broken connections */
     if (!wd31IsAlive( conn )) {
@@ -214,7 +232,7 @@ sapdbwa_Bool wd30DbDisconnect( twd30SessionPoolP pool, twd31DbConnP conn )
     
     wd35UnlockElem( pool->connPool, connId );
     
-    return true;
+    return sapdbwa_True;
 } /* wd30DbDisconnect */
 
 
@@ -226,6 +244,10 @@ sapdbwa_Bool wd30GetConf( twd30SessionPoolP      pool,
     char sizeStr[20];
     const char *autocommitStr;
     
+    if (!pool || !textBuffer || !bufferLen ) {
+        return sapdbwa_False;
+    }
+
     sp77sprintf(sizeStr, 20, "%d", pool->PoolSize );
     if (pool->autocommit ) {
         autocommitStr = "SQL_ATTR_AUTOCOMMIT_ON";
@@ -284,36 +306,60 @@ sapdbwa_Bool wd30GetConf( twd30SessionPoolP      pool,
 
 SQLHENV wd30GetHenv( twd30SessionPoolP pool )
 {
+    if (!pool) {
+        return 0;
+    }
+
     return pool->henv;
 } /* wd30GetHenv */
 
 
 char* wd30GetName( twd30SessionPoolP pool )
 {
+    if (!pool) {
+        return NULL;
+    }
+
     return pool->Name;
 } /* wd30GetName */
 
 
 char* wd30GetServerNode( twd30SessionPoolP pool )
 {
+    if (!pool) {
+        return NULL;
+    }
+
     return pool->Host;
 } /* wd30GetServerNode */
 
 
 char* wd30GetServerDb( twd30SessionPoolP pool )
 {
+    if (!pool) {
+        return NULL;
+    }
+
     return pool->ServerDb;
 } /* wd30GetServerDb */
 
 
 char* wd30GetUser( twd30SessionPoolP pool )
 {
+    if (!pool) {
+        return NULL;
+    }
+
     return pool->User;
 } /* wd30GetUser */
 
 
 char* wd30GetPassword( twd30SessionPoolP pool )
 {
+    if (!pool) {
+        return NULL;
+    }
+
     return pool->Password;
 } /* wd30GetUser */
 
@@ -334,6 +380,10 @@ sapdbwa_Bool wd30InitSessionPool( twd30SessionPoolP  pool,
     sapdbwa_Bool    initOK = true;
     RETCODE       retcode; /* retcode for ODBC-functions */
 
+    if (!pool || !datasource || !driver || !serverNode || !serverDb || !user || !password || !sqltrace) {
+        return sapdbwa_False;
+    }
+
     pool->poolType = ODBCPoolType;
     if ( strlen(serverNode) >= sizeof( pool->Host )
          || strlen(datasource) >= sizeof( pool->Datasource )
@@ -394,6 +444,10 @@ sapdbwa_Bool wd30InitSqlSessionPool( twd30SessionPoolP  pool,
 {
     sapdbwa_Bool    initOK = true;
     
+    if (!pool || !serverNode || !serverDb || !user || !password) {
+        return sapdbwa_False;
+    }
+
     pool->poolType = SQLPoolType;
     if ( strlen(serverNode) >= sizeof( pool->Host )
          || strlen(serverDb) >= sizeof( pool->ServerDb )
@@ -439,6 +493,10 @@ sapdbwa_Bool wd30Reconnect( twd30SessionPoolP pool,
 {
     sapdbwa_Bool reconnectOk = true;
     
+    if (!pool) {
+        return sapdbwa_False;
+    }
+
     switch (pool->poolType) {
     case ODBCPoolType:
         wd31OdbcDisconnect( conn, err );
@@ -471,6 +529,10 @@ sapdbwa_Bool wd30_Connect( twd30SessionPoolP pool,
 {
     sapdbwa_Bool connOK = true;
     
+    if (!pool) {
+        return sapdbwa_False;
+    }
+    
     switch (pool->poolType) {
     case ODBCPoolType:
         connOK = wd31Connect( conn,
@@ -512,6 +574,10 @@ sapdbwa_Bool wd30_Disconnect( twd30SessionPoolP pool,
 {
     sapdbwa_Bool disconnOK = true;
     
+    if (!pool) {
+        return sapdbwa_False;
+    }
+    
     switch (pool->poolType) {
     case ODBCPoolType:
         disconnOK = wd31OdbcDisconnect( conn, err );
@@ -536,6 +602,10 @@ void wd30_FreeConnections( twd30SessionPoolP pool )
     sapdbwa_UInt2   iConn;
     twd31DbConnP  conn = NULL;
     
+    if (!pool) {
+        return;
+    }
+    
     for (iConn=0; iConn < wd35GetNumAllocElems( pool->connPool ); iConn++ ) {
         if (wd35GetElemByIndex( pool->connPool, iConn, (void**) &conn )) {
 			if ( conn ) {
@@ -550,6 +620,11 @@ void wd30_FreeConnections( twd30SessionPoolP pool )
 
 void wd30_InitAttributes( twd30SessionPoolP pool )
 {
+    
+    if (!pool) {
+        return;
+    }
+
     pool->henv        = SQL_NULL_HENV;
     pool->PoolSize    = 0;
     pool->connPool    = NULL;
@@ -569,6 +644,10 @@ void wd30_InitAttributes( twd30SessionPoolP pool )
 
 void wd30_FreeMembers( twd30SessionPoolP pool )
 {
+    if (!pool) {
+        return;
+    }
+
     if (pool->connPool != NULL) {
         /* close and destroy connections */
         wd30_FreeConnections( pool );
