@@ -58,6 +58,8 @@
 #include "hsp81.h"
 #include "hsp83.h"
 
+#include "SAPDBCommon/SAPDB_string.h"
+
 sapdbwa_Bool wd34_CreateString( char **newStr, const char *sourceStr );
 
 sapdbwa_Bool wd34_SetConnectFromConnStr( twd34ConnectParam *connectParam,
@@ -869,7 +871,7 @@ twd34PoolElemP wd34CreatePoolElem( const twd34DBCAttr *attr )
         newPoolElem->userDataList	= NULL;
         newPoolElem->henv = SQL_NULL_HENV;
         newPoolElem->hdbc = SQL_NULL_HDBC;
-        memcpy( &newPoolElem->attr,
+        SAPDB_memcpy( &newPoolElem->attr,
                 attr,
                 sizeof(struct st_wa_dbc_attr) );
         wd34InitConnectParam( &newPoolElem->connectParam );
