@@ -30,6 +30,7 @@ import java.sql.Clob;
 import java.sql.Ref;
 import java.sql.ResultSetMetaData;
 import java.sql.Types;
+import java.sql.ResultSet;
 import java.util.*;
 import java.math.BigDecimal;
 import java.net.URL;
@@ -52,7 +53,8 @@ public class CallableStatementSapDB
 {
         private Vector inputOMSStreams;
         private Vector outputOMSStreams;
-
+        private Vector inputProcedureLongs;
+        
         final static int maxParseAgainCnt = 10;
     Parseinfo                   parseinfo;
     boolean                     lastWasNull;
@@ -77,7 +79,12 @@ public class CallableStatementSapDB
         String sql)
     throws SQLException
     {
-        super (connection);
+        super(connection,
+             (connection.isKernelFeaturesupported(Feature.sp1f_check_scrollableoption))?
+             ResultSet.TYPE_FORWARD_ONLY:StatementSapDB.defaultResultSetType_C,
+             defaultConcurrency_C,
+             connection.getHoldability());
+
         this.constructor(sql);
     }
     /**
@@ -200,7 +207,7 @@ public class CallableStatementSapDB
 
         RequestPacket requestPacket;
         ReplyPacket replyPacket;
-        PartEnumeration enum;
+        PartEnumeration enume;
         Parseinfo result = null;
         ParseinfoCache cache = this.connection.parseCache;
         String tableName;
@@ -224,7 +231,7 @@ public class CallableStatementSapDB
             // boolean backupWithInfo=setWithInfo;
             // !!! withinfo flag is not set as it may cause a crash !!!
             try {
-                setWithInfo=false;
+                setWithInfo=true;
                 replyPacket = this.sendSQL (sql, this.resultSetType, this.resultSetConcurrency, parseAgain);
             } catch(com.sap.dbtech.jdbc.exceptions.TimeoutException exc){
                 replyPacket = this.sendSQL (sql, this.resultSetType, this.resultSetConcurrency, parseAgain);
@@ -238,11 +245,13 @@ public class CallableStatementSapDB
             if (!parseAgain) {
                 result = new Parseinfo (this.connection, sql, replyPacket.functionCode ());
             }
-            enum = replyPacket.partEnumeration ();
+          
+            
+            enume = replyPacket.partEnumeration ();
             DBTechTranslator shortInfos[] = null;
-            while (enum.hasMoreElements ()) {
-                enum.nextElement ();
-                switch (enum.partKind ()) {
+            while (enume.hasMoreElements ()) {
+                enume.nextElement ();
+                switch (enume.partKind ()) {
                 case PartKind.Parsid_C:
                     int parseidPos = replyPacket.getPartDataPos ();
                     result.setParseIdAndSession(replyPacket.getBytes (parseidPos, 12),
@@ -250,7 +259,8 @@ public class CallableStatementSapDB
                     break;
                 case PartKind.Shortinfo_C:
                     shortInfos=replyPacket.parseShortFields(this.connection.isSpaceoptionSet,
-                                                                                                result.procParamInfos);
+                                                            result.isDBProc, 
+                                                            result.procParamInfos);
                     break;
                 case PartKind.Resulttablename_C:
                     result.setSelect(true);
@@ -269,7 +279,7 @@ public class CallableStatementSapDB
 //                    this.addWarning(new SQLWarning
 //                                    (MessageTranslator.translate
 //                                     (MessageKey.WARNING_PART_NOTHANDLED,
-//                                      PartKind.names[enum.partKind()]))
+//                                      PartKind.names[enume.partKind()]))
 //                                    );
                     break;
                 }
@@ -355,60 +365,76 @@ public class CallableStatementSapDB
                         this.outputOMSStreams = null;
 
             requestPacket = this.connection.getRequestPacket ();
-            requestPacket.initExecute (this.parseinfo.getParseId(), this.connection.autocommit);
+            requestPacket.initExecute (this.parseinfo.getParseId(),
+                                       this.connection.autocommit,
+                                       this.resultSetType);
             if (this.parseinfo.isSelect) {
                 requestPacket.addCursorPart (this.cursorName);
             }
             // We must add a data part if we have input parameters or even if
             // we have output streams.
             if (this.parseinfo.inputCount > 0 || this.parseinfo.hasStreams) {
-                dataPart = requestPacket.newDataPart ();
-                if(this.parseinfo.inputCount > 0) {
-                        for (int i = 0; i < this.parseinfo.paramInfos.length; ++i) {
+                dataPart = requestPacket.newDataPart();
+                if (this.parseinfo.inputCount > 0) {
+                    for (int i = 0; i < this.parseinfo.paramInfos.length; ++i) {
                         if (this.parseinfo.paramInfos[i].isInput()
-                                && initialParamValue == this.inputArgs [i]) {
-                                                if(this.parseinfo.paramInfos[i].isStreamKind() && this.outPutTypes != null && this.outPutTypes[i+1] != 0) {
-                                                        if(this.outputOMSStreams == null) {
-                                                                this.outputOMSStreams = new Vector();
-                                                        }
-                                                                ABAPStreamTranslator t = (ABAPStreamTranslator) this.parseinfo.paramInfos[i];
-                                                                AbstractABAPStreamGetval getval = t.createGetval();
-                                                                outputOMSStreams.add(getval);
-                                                                t.put(dataPart, getval);
-                                                } else {
-                                        throw new SQLException(MessageTranslator.translate
-                                                        (MessageKey.ERROR_MISSINGINOUT,
-                                                             Integer.toString(i+1)), "02000");
-                                                }
+                                && initialParamValue == this.inputArgs[i]) {
+                            if (this.parseinfo.paramInfos[i].isStreamKind()
+                                    && this.outPutTypes != null
+                                    && this.outPutTypes[i + 1] != 0) {
+                                if (this.outputOMSStreams == null) {
+                                    this.outputOMSStreams = new Vector();
+                                }
+                                ABAPStreamTranslator t = (ABAPStreamTranslator) this.parseinfo.paramInfos[i];
+                                AbstractABAPStreamGetval getval = t
+                                        .createGetval();
+                                outputOMSStreams.add(getval);
+                                t.put(dataPart, getval);
+                            } else {
+                                throw new SQLException(MessageTranslator
+                                        .translate(
+                                                MessageKey.ERROR_MISSINGINOUT,
+                                                Integer.toString(i + 1)),
+                                        "02000");
+                            }
                         } else {
-                                this.parseinfo.paramInfos [i].put (dataPart, this.inputArgs [i]);
+                            this.parseinfo.paramInfos[i].put(dataPart,
+                                    this.inputArgs[i]);
                         }
+                    }
+                    this.inputProcedureLongs = null;
+                    if (this.parseinfo.hasLongs) {
+                        if (this.parseinfo.isDBProc) {
+                            this.handleProcedureStreamsForExecute(dataPart,
+                                    this.inputArgs);
+                        } else {
+                            this.handleStreamsForExecute(dataPart,
+                                    this.inputArgs);
                         }
-                        if (this.parseinfo.hasLongs) {
-                        this.handleStreamsForExecute (dataPart, this.inputArgs);
-                        }
-                        if(this.parseinfo.hasStreams) {
-                                this.handleOMSStreamsForExecute(dataPart);
-                        }
-                } else {
-                        // we have only streams
+                    }
+                    if (this.parseinfo.hasStreams) {
                         this.handleOMSStreamsForExecute(dataPart);
+                    }
+                } else {
+                    // we have only streams
+                    this.handleOMSStreamsForExecute(dataPart);
                 }
-                dataPart.close ();
+                dataPart.close();
             }
             /*add a decribe order if command rturns a resultset*/
             if (  this.parseinfo.isSelect
                 &&this.parseinfo.columnInfos == null
                 &&this.parseinfo.functionCode != FunctionCode.DBProcWithResultSetExecute_FC ){
-                  requestPacket.initDbsCommand("DESCRIBE ",false, false);
+                  requestPacket.initDbsCommand("DESCRIBE ",false, false, ResultSet.TYPE_FORWARD_ONLY);
                   requestPacket.addParseidPart(this.parseinfo.getParseId());
             }
             try {
-                replyPacket = this.connection.execute (requestPacket, this,
-                        !this.parseinfo.hasStreams?ConnectionSapDB.GC_ALLOWED:ConnectionSapDB.GC_NONE);
+                replyPacket = this.connection.execute (requestPacket, this,                                              
+                        (!this.parseinfo.hasStreams && this.inputProcedureLongs==null)?ConnectionSapDB.GC_ALLOWED:ConnectionSapDB.GC_NONE);
                 // Recycling of parse infos and cursor names is not allowed
                 // if streams are in the command. Even sending it just behind
-                // as next packet is harmful.
+                // as next packet is harmful. Same with INPUT LONG parameters of
+                // DB Procedures.
             }
             catch (SQLException dbExc) {
                 if ((dbExc.getErrorCode() == -8) && afterParseAgain > 0) {
@@ -426,24 +452,26 @@ public class CallableStatementSapDB
             }
 
             // --- now it becomes difficult ...
-            if(this.parseinfo.isSelect) {
-                isQuery = this.parseResult(replyPacket, null, this.parseinfo.getColumnInfos(), this.parseinfo.columnNames);
+            if (this.parseinfo.isSelect) {
+                isQuery = this.parseResult(replyPacket, null, this.parseinfo
+                        .getColumnInfos(), this.parseinfo.columnNames);
             } else {
-                                if(this.parseinfo.hasStreams) {
-                                        replyPacket = this.processOMSStreams(replyPacket);
-                                }
-                                isQuery = this.parseResult (replyPacket,
-                                                                    null,
-                                                                                        this.parseinfo.getColumnInfos(),
-                                                                                        this.parseinfo.columnNames);
-                                int returnCode = replyPacket.returnCode();
-                                if (replyPacket.existsPart (PartKind.Data_C)) {
-                                        this.replyMem = replyPacket.getPointer (replyPacket.getPartDataPos ());
-                                }
-                                if (this.parseinfo.hasLongs && (returnCode == 0)) {
-                                        this.handleStreamsForPutval (replyPacket);
-                                }
+                if(inputProcedureLongs != null) {
+                    replyPacket = this.processProcedureStreams(replyPacket);
+                } else if (this.parseinfo.hasStreams) {
+                    replyPacket = this.processOMSStreams(replyPacket);
                 }
+                isQuery = this.parseResult(replyPacket, null, this.parseinfo
+                        .getColumnInfos(), this.parseinfo.columnNames);
+                int returnCode = replyPacket.returnCode();
+                if (replyPacket.existsPart(PartKind.Data_C)) {
+                    this.replyMem = replyPacket.getPointer(replyPacket
+                            .getPartDataPos());
+                }
+                if ((this.parseinfo.hasLongs && !this.parseinfo.isDBProc) && (returnCode == 0)) {
+                    this.handleStreamsForPutval(replyPacket);
+                }
+            }
 
             return isQuery;
         }
@@ -459,7 +487,8 @@ public class CallableStatementSapDB
         }
     }
 
-        /**
+
+    /**
      * This method is inherited from <code>java.sql.Statement</code>. <b>This method
      * executes the current statement, which is in contrast to the standard. According to
      * the standard, it has
@@ -565,7 +594,9 @@ public class CallableStatementSapDB
                 streamVec  = null;
                 int firstRecordNo = inputCursor;
                 requestPacket = this.connection.getRequestPacket();
-                requestPacket.initExecute (this.parseinfo.getMassParseid(), this.connection.autocommit);
+                requestPacket.initExecute (this.parseinfo.getMassParseid(),
+                                           this.connection.autocommit,
+                                           this.resultSetType);
                 if (executeCount == -1) {
                     requestPacket.addUndefResultCount ();
                 }
@@ -1301,13 +1332,13 @@ public class CallableStatementSapDB
         {
             Putval p1 = (Putval) o1;
             Putval p2 = (Putval) o2;
-            
+
             int p1_bufpos = p1.getBufpos();
             int p2_bufpos = p2.getBufpos();
 
             return p1_bufpos - p2_bufpos;
         }
-        
+
         public boolean equals(Object o)
         {
             return o==this;
@@ -1357,87 +1388,153 @@ public class CallableStatementSapDB
                         putval.transferStream (dataPart, i);
         }
     }
+    
+    private void handleProcedureStreamsForExecute(DataPart dataPart, Object[] objects)
+    {
+        this.inputProcedureLongs = new Vector();
+        for(int i=0; i<this.parseinfo.paramInfos.length; ++i) {
+            Object arg = this.inputArgs[i];
+            if(arg == null) {
+                continue;
+            } else {
+                try {
+                    AbstractProcedurePutval pv = (AbstractProcedurePutval) arg;
+                    inputProcedureLongs.addElement(pv);
+                    pv.updateIndex(inputProcedureLongs.size() - 1);
+                } catch(ClassCastException ignored) {
+                    continue;
+                }
+            }
+        }
+    }
+    
 
         /**
-         *
+         * 
          * @param dataPart
          * @param objects
          */
-        private void handleOMSStreamsForExecute(DataPart dataPart) {
-                //
-                this.inputOMSStreams = new Vector();
-                for (int i = 0; i < this.parseinfo.paramInfos.length; ++i) {
-                        Object inarg = this.inputArgs [i];
-                        if (inarg == null) {
-                                continue;
-                        }
-                        try {
-                                this.inputOMSStreams.addElement((AbstractABAPStreamPutval)
-                                        inputArgs[i]);
-                        }
-                        catch (ClassCastException exc) {
-                                // not a stream
-                        }
-                }
-                int inputoms_size=inputOMSStreams.size();
-                for(int k=0; k<inputoms_size; ++k) {
-                        ABAPStreamDescriptor t = (ABAPStreamDescriptor) inputOMSStreams.elementAt(k);
-                        t.updateIndex(k);
-                }
-                if(this.outputOMSStreams != null) {
-                        // update the index for the output streams
-                        for(int k=0; k<outputOMSStreams.size(); ++k) {
-                                ABAPStreamDescriptor t = (ABAPStreamDescriptor) outputOMSStreams.elementAt(k);
-                                t.updateIndex(k);
-                        }
-                }
+    private void handleOMSStreamsForExecute(DataPart dataPart)
+    {
+        //
+        this.inputOMSStreams = new Vector();
+        for (int i = 0; i < this.parseinfo.paramInfos.length; ++i) {
+            Object inarg = this.inputArgs[i];
+            if (inarg == null) {
+                continue;
+            }
+            try {
+                this.inputOMSStreams
+                        .addElement((AbstractABAPStreamPutval) inputArgs[i]);
+            } catch (ClassCastException exc) {
+                // not a stream
+            }
         }
-
-        private ReplyPacket processOMSStreams(ReplyPacket replyPacket)
-                throws SQLException {
-
-                do {
-                        if(replyPacket.existsPart(PartKind.AbapIStream_C)) {
-                                int tabid = replyPacket.parseABAPTabIDForInput();
-                                AbstractABAPStreamPutval pv=(AbstractABAPStreamPutval) this.inputOMSStreams.elementAt(tabid);
-                                if(pv.atEnd()) {
-                                        throw new SQLExceptionSapDB
-                                                (MessageTranslator.translate(MessageKey.ERROR_STREAM_EOF, Integer.toString(tabid)));
-                                }
-                                RequestPacket requestPacket
-                                        = this.connection.getRequestPacket ();
-                                DataPart dataPart = requestPacket.initStreamCommand(this.connection.autocommit);
-                                pv.transferStream(dataPart);
-                                dataPart.close ();
-                                replyPacket = connection.execute (requestPacket, this, ConnectionSapDB.GC_NONE);
-                        } else if(replyPacket.existsPart(PartKind.AbapOStream_C)) {
-                                int tabid = replyPacket.parseABAPTabIDForOutput();
-                                AbstractABAPStreamGetval gv = (AbstractABAPStreamGetval) this.outputOMSStreams.elementAt(tabid);
-                                boolean eos=gv.addReplyData(replyPacket);
-                                RequestPacket requestPacket = this.connection.getRequestPacket();
-                                DataPart datapart = requestPacket.initStreamCommand(this.connection.autocommit);
-                                if(eos) {
-                                        datapart.fillWithOMSReturnCode(100);
-                                } else {
-                                        datapart.fillWithOMSReturnCode(0);
-                                }
-                                datapart.close();
-                                replyPacket = connection.execute (requestPacket, this, ConnectionSapDB.GC_NONE);
-                        } else {
-                                if(this.outputOMSStreams != null) {
-                                        for(int i=0; i<this.outputOMSStreams.size(); ++i) {
-                                                AbstractABAPStreamGetval gv = (AbstractABAPStreamGetval) this.outputOMSStreams.elementAt(i);
-                                                if(gv != null) {
-                                                        gv.coalesceReply();
-                                                }
-                                        }
-                                }
-                                return replyPacket;
-                        }
-                } while(true);
+        int inputoms_size = inputOMSStreams.size();
+        for (int k = 0; k < inputoms_size; ++k) {
+            ABAPStreamDescriptor t = (ABAPStreamDescriptor) inputOMSStreams
+                    .elementAt(k);
+            t.updateIndex(k);
         }
+        if (this.outputOMSStreams != null) {
+            // update the index for the output streams
+            for (int k = 0; k < outputOMSStreams.size(); ++k) {
+                ABAPStreamDescriptor t = (ABAPStreamDescriptor) outputOMSStreams
+                        .elementAt(k);
+                t.updateIndex(k);
+            }
+        }
+    }
+
+    private ReplyPacket processProcedureStreams(ReplyPacket packet)
+    	throws SQLException
+    {
+        do {
+            try {
+                if (packet.existsPart(PartKind.AbapIStream_C)) {
+                    StreamInfo[] streaminfos = packet.parseStreamInfos();
+                    StreamInfo info = streaminfos[0];
+                    int tabid = info.getTabId();
+                if (tabid < 0 || tabid > inputProcedureLongs.size()) {
+                        throw new SQLExceptionSapDB(MessageTranslator
+                                .translate(MessageKey.ERROR_UNKNOWN_PROCEDURELONG, Integer
+                                        .toString(tabid)));
+                 }
+                 AbstractProcedurePutval pv = (AbstractProcedurePutval) inputProcedureLongs.elementAt(tabid);
+                 RequestPacket requestPacket = connection.getRequestPacket();
+                 DataPart dataPart = requestPacket.initStreamCommand(connection.autocommit);
+                 pv.transferStream(dataPart, info.getRowCount());
+                 dataPart.close();
+                 packet = connection.execute(requestPacket, this, ConnectionSapDB.GC_NONE);
+                } else {
+                    return packet;
+                }
+            } catch(SQLException sqlEx) {
+                ReplyPacket tmppacket = connection.sendStreamErrorPacket(sqlEx);
+                if(tmppacket == null) {
+                    throw sqlEx;
+                }
+                return tmppacket;
+            }
+        } while (true);
+    }
+
+        
+    private ReplyPacket processOMSStreams(ReplyPacket replyPacket)
+            throws SQLException
+    {
+
+        do {
+            if (replyPacket.existsPart(PartKind.AbapIStream_C)) {
+                int tabid = replyPacket.parseABAPTabIDForInput();
+                AbstractABAPStreamPutval pv = (AbstractABAPStreamPutval) this.inputOMSStreams
+                        .elementAt(tabid);
+                if (pv.atEnd()) {
+                    throw new SQLExceptionSapDB(MessageTranslator.translate(
+                            MessageKey.ERROR_STREAM_EOF, Integer
+                                    .toString(tabid)));
+                }
+                RequestPacket requestPacket = this.connection
+                        .getRequestPacket();
+                DataPart dataPart = requestPacket
+                        .initStreamCommand(this.connection.autocommit);
+                pv.transferStream(dataPart);
+                dataPart.close();
+                replyPacket = connection.execute(requestPacket, this,
+                        ConnectionSapDB.GC_NONE);
+            } else if (replyPacket.existsPart(PartKind.AbapOStream_C)) {
+                int tabid = replyPacket.parseABAPTabIDForOutput();
+                AbstractABAPStreamGetval gv = (AbstractABAPStreamGetval) this.outputOMSStreams
+                        .elementAt(tabid);
+                boolean eos = gv.addReplyData(replyPacket);
+                RequestPacket requestPacket = this.connection
+                        .getRequestPacket();
+                DataPart datapart = requestPacket
+                        .initStreamCommand(this.connection.autocommit);
+                if (eos) {
+                    datapart.fillWithOMSReturnCode(100);
+                } else {
+                    datapart.fillWithOMSReturnCode(0);
+                }
+                datapart.close();
+                replyPacket = connection.execute(requestPacket, this,
+                        ConnectionSapDB.GC_NONE);
+            } else {
+                if (this.outputOMSStreams != null) {
+                    for (int i = 0; i < this.outputOMSStreams.size(); ++i) {
+                        AbstractABAPStreamGetval gv = (AbstractABAPStreamGetval) this.outputOMSStreams
+                                .elementAt(i);
+                        if (gv != null) {
+                            gv.coalesceReply();
+                        }
+                    }
+                }
+                return replyPacket;
+            }
+        } while (true);
+    }
     /**
-     *
+     *  
      */
     private void getChangedPutvalDescriptors (
         ReplyPacket replyPacket)
