@@ -52,10 +52,10 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
     private static final int MODE_READ   = 0;
     private static final int MODE_INSERT = 1;
     private static final int MODE_UPDATE = 2;
-    
+
 
     private int currentMode = MODE_READ;
-    
+
     /**
      * constant that signals that no current-cached record was updated
      */
@@ -65,17 +65,18 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
      */
     UpdatableResultSetSapDB(ConnectionSapDB connection,
                             FetchInfo fetchInfo,
-                            java.sql.Statement producer,
+                            StatementSapDB producer,
                             int fetchSize,
                             int maxRow,
                             String tableName,
-                            boolean doNotDrop)
+                            int cursorType,
+                            com.sap.dbtech.jdbc.packet.ReplyPacket reply)
         throws java.sql.SQLException
     {
-        super(connection, fetchInfo, producer, fetchSize, maxRow, doNotDrop);
+        super(connection, fetchInfo, producer, fetchSize, maxRow, cursorType, reply);
         this.tableName=tableName;
     }
-    
+
     /**
      * UpdatableResultSetSapDBTech constructor comment.
      * @param connection com.sap.dbtech.jdbc.ConnectionSapDBTech
@@ -89,14 +90,15 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
             String cursorName,
             com.sap.dbtech.jdbc.translators.DBTechTranslator[] infos,
             java.lang.String[] colNames,
-            java.sql.Statement producer,
+            StatementSapDB producer,
             int fetchSize,
             int maxRow,
             String tableName,
-            boolean doNotDrop)
+            int  cursorType,
+            com.sap.dbtech.jdbc.packet.ReplyPacket reply)
         throws java.sql.SQLException
     {
-        super(connection, cursorName, infos, colNames, producer, fetchSize, maxRow, doNotDrop);
+        super(connection, cursorName, infos, colNames, producer, fetchSize, maxRow, cursorType, reply);
         this.tableName = tableName;
     }
     /**
@@ -151,7 +153,7 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
     * called when on the insert row
     *
     */
-    public void cancelRowUpdates() throws java.sql.SQLException 
+    public void cancelRowUpdates() throws java.sql.SQLException
     {
         if(this.currentMode == MODE_INSERT) {
             throw new SQLExceptionSapDB(MessageTranslator.translate(MessageKey.ERROR_CANCELUPDATES_INSERTROW));
@@ -167,24 +169,24 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
         }
         this.currentMode = MODE_READ;
     }
-    
-    
-    public void deleteRow () 
-        throws SQLException 
+
+
+    public void deleteRow ()
+        throws SQLException
     {
         assertNotClosed();
-        
+
         if(currentMode==MODE_INSERT) {
             throw new SQLExceptionSapDB(MessageTranslator.translate(MessageKey.ERROR_DELETEROW_INSERTROW));
         }
-        
+
         int i=getRow();
-        
+
         if(i==0) {
             throw new SQLExceptionSapDB(MessageTranslator.translate(MessageKey.ERROR_DELETEROW_NOROW));
         }
-        
-        
+
+
         if (this.deleteCmd == null) {
             String cmdString = "DELETE FROM " + this.tableName
                 + " WHERE POS OF \"" + this.getCursorName ()
@@ -193,14 +195,14 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
                 (CallableStatementSapDB) this.connection.prepareStatement(cmdString);
         }
 
-        
+
         /*set delete position to current row */
         this.deleteCmd.setInt(1,i);
         this.deleteCmd.execute ();
 
         // repositioning
         this.initializeFields();
-        this.absolute(i); 
+        this.absolute(i);
     }
 
     /**
@@ -210,7 +212,7 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
         int colIndex)
             throws java.sql.SQLException
     {
-        
+
         this.assertNotClosed();
         this.prepareUpdate ();
         int result = 0;
@@ -267,10 +269,10 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
     * if called when not on the insert row, or if not all of non-nullable columns in
     * the insert row have been given a value
     */
-    public void insertRow() throws java.sql.SQLException 
+    public void insertRow() throws java.sql.SQLException
     {
-        // TBD: 
-        // - reflect the change in the database 
+        // TBD:
+        // - reflect the change in the database
         // - reposition back to the row where it was before the insert
 
         if (this.currentMode == MODE_INSERT) {
@@ -289,7 +291,7 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
         this.internalCancelRowUpdates ();
         return super.last ();
     }
-    
+
     /**
      *
      * Moves the cursor to the insert row.  The current cursor position is
@@ -326,11 +328,11 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
             }
             savedPositionState=this.positionState; // before 1st, after last ?
             savedCurrentRow = this.getRow(); // this wrecks the mode, and the position.
-            
+
             this.currentChangeCmd = this.insertCmd;
             this.currentMode = MODE_INSERT;
-            
-            
+
+
         }
     }
 
@@ -343,7 +345,7 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
     {
         // check that result set is open
         this.assertNotClosed();
-        
+
         if(this.currentMode == MODE_INSERT) {
             if (this.currentChangeCmd != null) {
                 this.currentChangeCmd.clearParameters ();
@@ -359,7 +361,7 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
                 break;
             default:
                 if(savedPositionState!=0) {
-                    absolute(savedCurrentRow); 
+                    absolute(savedCurrentRow);
                 }
             }
         } else {
@@ -396,12 +398,12 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
         DBTechTranslator[] colInfo=this.getColInfo();
         int colinfo_length=colInfo.length;
         java.util.Vector validColumns = new java.util.Vector (colinfo_length);
-        
+
 
         for (int i = 0; i < colinfo_length; ++i) {
             validColumns.addElement(colInfo [i].getColumnName());
         }
-        int colcount=colinfo_length;        
+        int colcount=colinfo_length;
         while ((result == null) && (validColumns.size() > 0) && colcount>0) {
             colcount--;
             String columns = com.sap.dbtech.util.StringUtil.join (validColumns.elements (), "\" = ?, \"");
@@ -422,7 +424,7 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
             }
         }
         if (result == null) {
-            throw new com.sap.dbtech.jdbc.exceptions.JDBCDriverException 
+            throw new com.sap.dbtech.jdbc.exceptions.JDBCDriverException
                 (MessageTranslator.translate(MessageKey.ERROR_NOCOLUMNS_UPDATABLE));
         }
         int count = validColumns.size ();
@@ -435,7 +437,7 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
         }
         return result;
     }
-    
+
     /**
      * @exception java.sql.SQLException The exception description.
      */
@@ -459,7 +461,7 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
         }
     }
 
-    
+
 
     /**
     * JDBC 2.0
@@ -1063,7 +1065,7 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
         // ... therfore we keep them at a safe place meanwhile
         this.currentMode=safeCurrentMode;
         this.currentChangeCmd=saveCurrentChangeCmd;
-        
+
         if(internal==0) {
             throw new SQLExceptionSapDB(MessageTranslator.translate(MessageKey.ERROR_UPDATEROW_NOROW));
         }
@@ -1077,10 +1079,10 @@ public class UpdatableResultSetSapDB extends ResultSetSapDB {
             modifiedKernelPos=internal; // the kernel position has changed to the current row.
         } else if(this.currentMode == MODE_INSERT) {
             throw new SQLExceptionSapDB(MessageTranslator.translate(MessageKey.ERROR_UPDATEROW_INSERTROW));
-        } 
+        }
         // otherwise some called updateRow without updating something ..., this will not harm
     }
-    
+
     /**
      * JDBC 2.0
      *
